Package: RnBeads
Title: RnBeads
Description: RnBeads facilitates comprehensive analysis of various types of DNA
    methylation data at the genome scale.
Authors@R: c(
    person("Yassen", "Assenov", email = "assenov@gmail.com",
    role = c("aut")),
    person("Christoph", "Bock", email = "cbock@cemm.at",
    role = c("aut")),
    person("Pavlo", "Lutsik", email = "p.lutsik@dkfz.de",
    role = c("aut")),
    person("Michael", "Scherer", email = "mscherer@mpi-inf.mpg.de",
    role = c("aut")),
    person("Fabian", "Mueller", email = "team@rnbeads.org",
    role = c("aut", "cre")))
Date: 2026-07-16
Version: 2.31.2
Suggests: Category, GOstats, Gviz,
        IlluminaHumanMethylation450kmanifest, RPMM, RnBeads.hg19,
        RnBeads.mm9, RnBeads.hg38, XML, annotate, biomaRt, foreach,
        doParallel, ggbio, isva, mclust, mgcv, minfi, nlme,
        org.Hs.eg.db, org.Mm.eg.db, org.Rn.eg.db, quadprog,
        rtracklayer, qvalue, sva, wateRmelon, wordcloud, qvalue,
        argparse, glmnet, IlluminaHumanMethylation450kanno.ilmn12.hg19,
        scales, missMethyl, impute, shiny, shinyjs, plotrix, hexbin,
        RUnit, MethylSeekR, sesame, dplyr
Depends: R (>= 4.1.0), BiocGenerics, S4Vectors (>= 0.9.25),
        GenomicRanges, MASS, cluster, ff, fields, ggplot2 (>= 0.9.2),
        gplots, grid, gridExtra, limma, matrixStats, methods,
        illuminaio, methylumi, plyr, reshape2
Imports: IRanges, ComplexHeatmap
License: GPL-3
biocViews: DNAMethylation, MethylationArray, MethylSeq, Epigenetics,
        QualityControl, Preprocessing, BatchEffect,
        DifferentialMethylation, Sequencing, CpGIsland, ImmunoOncology,
        TwoChannel, DataImport
Collate: 'CNV.R' 'Report-class.R' 'Report-methods.R'
        'ReportPlot-class.R' 'ReportPlot-methods.R'
        'RnBDiffMeth-class.R' 'bigFf.R' 'RnBSet-class.R'
        'RnBeadSet-class.R' 'RnBeadRawSet-class.R' 'RnBeads-package.R'
        'RnBiseqSet-class.R' 'agePrediction.R' 'annotations.R'
        'batch.R' 'batch.quality.R' 'bmiq.R' 'cellTypeAdjustment.R'
        'clusterArchitecture.R' 'clusterArchitectureLSF.R'
        'clusterArchitectureSGE.R' 'clusterArchitectureSLURM.R'
        'clustering.R' 'combining.R' 'computeCluster.R'
        'controlPlots.R' 'controlPlotsBiSeq.R' 'dataExport.R'
        'dataImport.R' 'differentialMethylation.R'
        'differentialVariability.R' 'enmix.R' 'enrichment.R'
        'exportGEO.R' 'filtering.R' 'filteringSummary.R' 'genomewide.R'
        'greedycut.R' 'immune.R' 'loading.R' 'logger.R' 'lolaUtils.R'
        'main.R' 'normalization.R' 'normalizationMethods.R' 'options.R'
        'parallelProcessing.R' 'plottingUtils.R' 'profiles.R'
        'qualityControl.R' 'readGEO.R' 'regionDescription.R'
        'regionProfiles.R' 'segmentation.R' 'sesame.R' 'sex.R'
        'subSegments.R' 'sva.R' 'utilities.R' 'wbcInference.R'
        'xenomix.R'
RoxygenNote: 7.3.3
git_url: https://git.bioconductor.org/packages/RnBeads
git_branch: devel
git_last_commit: 1c6f9f8
git_last_commit_date: 2026-07-23
Repository: Bioconductor 3.24
Date/Publication: 2026-07-23
NeedsCompilation: no
Packaged: 2026-07-23 23:48:45 UTC; biocbuild
Author: Yassen Assenov [aut],
  Christoph Bock [aut],
  Pavlo Lutsik [aut],
  Michael Scherer [aut],
  Fabian Mueller [aut, cre]
Maintainer: Fabian Mueller <team@rnbeads.org>

# Generated by roxygen2: do not edit by hand

export(BMIQ)
export(RnBeadRawSet)
export(RnBeadSet)
export(RnBiseqSet)
export(addRegionSubsegments)
export(apply.iEVORA)
export(combineTestPvalsMeth)
export(computeDiffTab.default.region)
export(computeDiffTab.default.site)
export(computeDiffTab.extended.site)
export(create.densityScatter)
export(create.scatter.dens.points)
export(createReport)
export(createReportGgPlot)
export(createReportPlot)
export(data.frame2GRanges)
export(deviation.plot.beta)
export(diffVar)
export(downloadLolaDbs)
export(estimateProportionsCP)
export(exportDMRs2regionFile)
export(get.adjustment.variables)
export(get.comparison.info)
export(get.covariates.ct)
export(get.covariates.sva)
export(get.cpg.stats)
export(get.files)
export(getCellTypesFromLolaDb)
export(getNamesFromLolaDb)
export(getTargetFromLolaDb)
export(greedycut.filter.matrix)
export(greedycut.get.statistics)
export(greedycut.get.submatrix)
export(has.covariates.ct)
export(has.covariates.sva)
export(limmaP)
export(load.region.subsegment.annotation)
export(load.rnb.diffmeth)
export(load.rnb.set)
export(loadLolaDbs)
export(logger.argument)
export(logger.close)
export(logger.completed)
export(logger.error)
export(logger.getfiles)
export(logger.info)
export(logger.isinitialized)
export(logger.machine.name)
export(logger.start)
export(logger.status)
export(logger.validate.file)
export(logger.warning)
export(lolaBarPlot)
export(lolaBoxPlotPerTarget)
export(lolaVolcanoPlot)
export(off)
export(parallel.getNumWorkers)
export(parallel.isEnabled)
export(parallel.setup)
export(parallel.teardown)
export(performGOEnrichment.diffVar)
export(performGOenrichment.diffMeth.entrez)
export(performGoEnrichment.diffMeth)
export(performLolaEnrichment.diffMeth)
export(performLolaEnrichment.diffVar)
export(prepareSOFTfileForGEO)
export(read.GS.report)
export(read.bed.files)
export(read.data.dir)
export(read.idat.files)
export(read.sample.annotation)
export(read.single.bed)
export(refFreeEWASP)
export(rnb.RnBSet.to.GRangesList)
export(rnb.RnBSet.to.bed)
export(rnb.RnBSet.to.bedGraph)
export(rnb.add.figure)
export(rnb.add.list)
export(rnb.add.paragraph)
export(rnb.add.reference)
export(rnb.add.section)
export(rnb.add.table)
export(rnb.add.tables)
export(rnb.annotation.size)
export(rnb.annotation2data.frame)
export(rnb.bed.from.segmentation)
export(rnb.beta2mval)
export(rnb.boxplot.from.segmentation)
export(rnb.build.index)
export(rnb.color.legends)
export(rnb.combine.arrays)
export(rnb.combine.seq)
export(rnb.execute.age.prediction)
export(rnb.execute.batch.qc)
export(rnb.execute.batcheffects)
export(rnb.execute.clustering)
export(rnb.execute.clustering.all)
export(rnb.execute.computeDiffMeth)
export(rnb.execute.context.removal)
export(rnb.execute.cross.reactive.removal)
export(rnb.execute.ct.estimation)
export(rnb.execute.diffVar)
export(rnb.execute.dreduction)
export(rnb.execute.export.csv)
export(rnb.execute.filter.summary)
export(rnb.execute.genomewide)
export(rnb.execute.greedycut)
export(rnb.execute.high.coverage.removal)
export(rnb.execute.high.dpval.masking)
export(rnb.execute.import)
export(rnb.execute.imputation)
export(rnb.execute.low.coverage.masking)
export(rnb.execute.lump)
export(rnb.execute.na.removal)
export(rnb.execute.normalization)
export(rnb.execute.pOOBAH)
export(rnb.execute.quality)
export(rnb.execute.segmentation)
export(rnb.execute.sex.prediction)
export(rnb.execute.sex.removal)
export(rnb.execute.snp.removal)
export(rnb.execute.sva)
export(rnb.execute.tnt)
export(rnb.execute.training)
export(rnb.execute.variability.removal)
export(rnb.export.all.annotation)
export(rnb.export.annotation)
export(rnb.export.to.ewasher)
export(rnb.export.to.trackhub)
export(rnb.find.relative.site.coord)
export(rnb.get.annotation)
export(rnb.get.assemblies)
export(rnb.get.chromosomes)
export(rnb.get.directory)
export(rnb.get.mapping)
export(rnb.get.reference)
export(rnb.get.reliability.matrix)
export(rnb.getOption)
export(rnb.infinium.control.targets)
export(rnb.initialize.reports)
export(rnb.is.option)
export(rnb.load.annotation)
export(rnb.load.annotation.from.db)
export(rnb.load.sitelist)
export(rnb.message.plot)
export(rnb.mval2beta)
export(rnb.options)
export(rnb.options2xml)
export(rnb.performance.profile)
export(rnb.plot.beta.comparison)
export(rnb.plot.betadistribution.probeCategories)
export(rnb.plot.betadistribution.sampleGroups)
export(rnb.plot.biseq.coverage)
export(rnb.plot.biseq.coverage.hist)
export(rnb.plot.biseq.coverage.violin)
export(rnb.plot.control.barplot)
export(rnb.plot.control.boxplot)
export(rnb.plot.coverage.thresholds)
export(rnb.plot.ct.heatmap)
export(rnb.plot.dreduction)
export(rnb.plot.locus.profile)
export(rnb.plot.marker.fstat)
export(rnb.plot.negative.boxplot)
export(rnb.plot.num.sites.covg)
export(rnb.plot.nv.heatmap)
export(rnb.plot.pheno.categories)
export(rnb.plot.region.profile.density)
export(rnb.plot.region.profiles)
export(rnb.plot.region.site.density)
export(rnb.plot.sentrix.distribution)
export(rnb.plot.sentrix.distributions)
export(rnb.plot.snp.barplot)
export(rnb.plot.snp.boxplot)
export(rnb.plot.snp.heatmap)
export(rnb.read.geo)
export(rnb.region.types)
export(rnb.region.types.for.analysis)
export(rnb.remove.annotation)
export(rnb.run.analysis)
export(rnb.run.differential)
export(rnb.run.dj)
export(rnb.run.example)
export(rnb.run.exploratory)
export(rnb.run.import)
export(rnb.run.inference)
export(rnb.run.preprocessing)
export(rnb.run.qc)
export(rnb.run.tnt)
export(rnb.run.xml)
export(rnb.run_xeno)
export(rnb.sample.groups)
export(rnb.sample.replicates)
export(rnb.sample.summary.table)
export(rnb.save.annotation)
export(rnb.set.annotation)
export(rnb.set.annotation.and.cpg.stats)
export(rnb.show.report)
export(rnb.step.betadistribution)
export(rnb.step.cnv)
export(rnb.write.table)
export(rnb.xml2options)
export(rowOneSampleTP)
export(rowPairedTP)
export(rowWelchP)
export(run.cross.validation)
export(save.rnb.diffmeth)
export(save.rnb.set)
export(set.covariates.ct)
export(set.covariates.sva)
exportClasses(ClusterArchitecture)
exportClasses(Report)
exportClasses(ReportGgPlot)
exportClasses(ReportPlot)
exportClasses(RnBClusterRun)
exportClasses(RnBDiffMeth)
exportClasses(RnBSet)
exportClasses(RnBeadClustering)
exportClasses(RnBeadRawSet)
exportClasses(RnBeadSet)
exportClasses(RnBiseqSet)
exportMethods(M)
exportMethods(U)
exportMethods(addDiffMethTable)
exportMethods(addPheno)
exportMethods(annotation)
exportMethods(assembly)
exportMethods(combine.rnb.sets)
exportMethods(covg)
exportMethods(destroy)
exportMethods(dpval)
exportMethods(geno)
exportMethods(get.comparison.grouplabels)
exportMethods(get.comparison.groupsizes)
exportMethods(get.comparisons)
exportMethods(get.covg.thres)
exportMethods(get.region.types)
exportMethods(get.site.test.method)
exportMethods(get.table)
exportMethods(get.variability.method)
exportMethods(getExecutable)
exportMethods(getModuleNumCores)
exportMethods(getSubCmdStr)
exportMethods(getSubCmdTokens)
exportMethods(hasCovg)
exportMethods(includes.sites)
exportMethods(initialize)
exportMethods(is.valid)
exportMethods(isImputed)
exportMethods(join.diffMeth)
exportMethods(mergeSamples)
exportMethods(meth)
exportMethods(mval)
exportMethods(nsites)
exportMethods(off)
exportMethods(pheno)
exportMethods(qc)
exportMethods(regionMapping)
exportMethods(regions)
exportMethods(reload)
exportMethods(remove.regions)
exportMethods(remove.samples)
exportMethods(remove.sites)
exportMethods(run)
exportMethods(samples)
exportMethods(save.tables)
exportMethods(setExecutable)
exportMethods(setModuleNumCores)
exportMethods(setModuleResourceRequirements)
exportMethods(sites)
exportMethods(summarize.regions)
exportMethods(summarized.regions)
exportMethods(updateMethylationSites)
exportMethods(updateRegionSummaries)
import(GenomicRanges)
import(IRanges)
import(MASS)
import(S4Vectors)
import(cluster)
import(ff)
import(fields)
import(ggplot2)
import(limma)
import(methods)
import(methylumi)
importFrom(BiocGenerics,"annotation<-")
importFrom(BiocGenerics,annotation)
importFrom(gplots,colorpanel)
importFrom(gplots,heatmap.2)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,densCols)
importFrom(grDevices,dev.control)
importFrom(grDevices,dev.off)
importFrom(grDevices,dev2bitmap)
importFrom(grDevices,pdf)
importFrom(grDevices,png)
importFrom(grDevices,rainbow)
importFrom(grDevices,rgb)
importFrom(grDevices,xy.coords)
importFrom(graphics,abline)
importFrom(graphics,close.screen)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,plot.new)
importFrom(graphics,polygon)
importFrom(graphics,screen)
importFrom(graphics,split.screen)
importFrom(gridExtra,arrangeGrob)
importFrom(illuminaio,readIDAT)
importFrom(matrixStats,colMaxs)
importFrom(matrixStats,colMedians)
importFrom(matrixStats,colMins)
importFrom(matrixStats,colQuantiles)
importFrom(matrixStats,colSds)
importFrom(matrixStats,colVars)
importFrom(matrixStats,rowMaxs)
importFrom(matrixStats,rowMedians)
importFrom(matrixStats,rowMins)
importFrom(matrixStats,rowQuantiles)
importFrom(matrixStats,rowSds)
importFrom(matrixStats,rowVars)
importFrom(plyr,rbind.fill)
importFrom(stats,as.dendrogram)
importFrom(stats,as.dist)
importFrom(stats,as.formula)
importFrom(stats,bartlett.test)
importFrom(stats,coef)
importFrom(stats,cutree)
importFrom(stats,dbeta)
importFrom(stats,density)
importFrom(stats,dexp)
importFrom(stats,dist)
importFrom(stats,dnorm)
importFrom(stats,ecdf)
importFrom(stats,fisher.test)
importFrom(stats,hclust)
importFrom(stats,knots)
importFrom(stats,kruskal.test)
importFrom(stats,lm)
importFrom(stats,model.matrix)
importFrom(stats,optim)
importFrom(stats,p.adjust)
importFrom(stats,pbeta)
importFrom(stats,pchisq)
importFrom(stats,pf)
importFrom(stats,pnorm)
importFrom(stats,prcomp)
importFrom(stats,predict)
importFrom(stats,pt)
importFrom(stats,qbeta)
importFrom(stats,residuals)
importFrom(stats,rnorm)
importFrom(stats,rt)
importFrom(stats,t.test)
importFrom(stats,vcov)
importFrom(stats,wilcox.test)
importFrom(utils,browseURL)
importFrom(utils,capture.output)
importFrom(utils,combn)
importFrom(utils,data)
importFrom(utils,download.file)
importFrom(utils,installed.packages)
importFrom(utils,memory.size)
importFrom(utils,read.csv)
importFrom(utils,read.delim)
importFrom(utils,read.table)
importFrom(utils,untar)
importFrom(utils,unzip)
importFrom(utils,write.table)
importFrom(utils,zip)

RnBeads is an R package for comprehensive analysis of DNA methylation data
obtained with any experimental protocol that provides single-CpG resolution,
including Infinium 450K microarray and bisulfite sequencing protocols, but also
MeDIP-seq and MBD-seq once the data have been preprocessed with
DNA methylation inference software. RnBeads implements an analysis workflow
that is significantly more comprehensive than those of existing tools. 
It documents its results in a highly annotated and readable hypertext report,
and it scales to the large sample sizes that are becoming the norm for
DNA methylation analysis in human cohorts.

# RnBeadsDJ - RnBeads Data Juggler
* __Package name:__ RnBeadsDJ
* __Title:__ RnBeads Data Juggler
* __Description:__ A Shiny app for configuring and running RnBeads analyses 
* __Author:__ Fabian Mueller
* __Maintainer:__ Fabian Mueller <fmueller@mpi-inf.mpg.de>
* __Date:__ 2017-10-04

################################################################################

#' rnb.execute.pOOBAH
#' 
#' Probe signal intensities are masked based on their out-of-band signal intensities to counter hybridization failure.
#' 
#' rnb.execute.pOOBAH is used to apply the method \emph{pOOBAH} (P-value with OOB probes for Array Hybridization), 
#' which was conceived by Zhou, Triche, Laird and Shen to mask probes associated with hybridization failures. 
#' pOOBAH has been implemented in the R-package \emph{"sesame"}, a dependency needed for this function (see Zhou et al, 2018 and the respective Bioconductor/github pages). 
#' \emph{pOOBAH} computes the detection p-values by constructing 2 empirical cumulative density functions (eCDFs) based 
#' on the out-of-band signal intensities of the red and the green channel, respectively, to detect hybridization failures. 
#' The (in-band) green and red channel signal intensities of the probes are passed to the eCDFs and the probes with a 
#' p-value higher than the given threshold (\code{pval.thresh}) are masked, as they are considered background. 
#' \emph{pOOBAH} is applied \emph{separately to each sample}. 
#' Hybridization failures might occur due to somatic or germline deletions. 
#' In addition, unreliable low-intensity probes might also be masked.
#'  
#' @param raw.set Methylation dataset as an instance of \code{RnBeadRawSet}.
#' @param anno.table Annotation for \code{raw.set}.
#' @param pval.thresh Computed detection p-values above this threshold are masked. Default value is 0.05.
#' @param verbose If set to true, a short information is printed on how many probes are masked by the method. 
#' @return Returns a modified \code{RnBeadRawSet}, in which signal intensities are masked, if their computed p-value  
#'         was greater than \code{pval.thresh}. Note, in datasets with several samples, signal intensities of a specific probe 
#'         might be masked in sample A, but not in sample B, as \emph{pOOBAH} is applied separately to each sample.
#'         For example: the signal intensities of probe cg24488772 might be masked in sample 1, but not in sample 12. 
#' @author \emph{pOOBAH} method: Wanding Zhou. Adapted by Nathan Steenbuck. 
#' @examples
#' library(RnBeads.hg19) 
#' data(small.example.object)
#' 
#'
#' @export 

rnb.execute.pOOBAH <- function(raw.set, anno.table = NULL, pval.thresh = 0.05, verbose = FALSE){
  rnb.require("sesame")
  
  if(!(is.numeric(pval.thresh) && pval.thresh <= 1 && pval.thresh >= 0)){
    stop("Invalid value for pval.thresh. Please specify a numeric in the range of [0, 1].")
  }
  if(is.numeric(anno.table)){
    stop("Invalid value for anno.table. Wanted to specify the p-value threshold?")
  }
  if(!inherits(raw.set, "RnBeadRawSet")){
    stop("Please provide input inhereting from RnBeadRawSet.")
  }
  else{
    if(raw.set@target == "probes450"){
      platform = "HM450"
    }else if(raw.set@target == "probesEPIC"){
      platform = "EPIC"
    }else if(raw.set@target == "probesEPICv2"){
      platform = "EPICv2"
    }else{
      stop("Invalid value for platform")
    }

    if(is.null(anno.table) || !("ID" %in% colnames(anno.table))){
      anno.table <- annotation(raw.set, add.names=TRUE) 
    }else if(length(anno.table[["ID"]]) != nrow(raw.set@sites)){
      stop("The annotation and dataset are not compatible.")
    }
    probeIDs <- anno.table[["ID"]]
    
    intensities.by.channel <- intensities.by.color(raw.set, address.rownames = FALSE, add.oob = TRUE, 
                                                  add.controls =  FALSE, add.missing = FALSE, re.separate = TRUE)
    
    grn <- intensities.by.channel$Cy3.I
    red <- intensities.by.channel$Cy5.I
    grn.oob <- intensities.by.channel$Cy3.I.oob
    red.oob <- intensities.by.channel$Cy5.I.oob
    tII <- intensities.by.channel$II
    rm(intensities.by.channel, anno.table)
    
    if(sum((rownames(grn$M) %in% rownames(grn$U)) == FALSE) > 0 ||
       sum((rownames(red$M) %in% rownames(red$U)) == FALSE) > 0 ||
       sum((rownames(grn.oob$M) %in% rownames(grn.oob$U)) == FALSE) > 0 ||
       sum((rownames(red.oob$M) %in% rownames(red.oob$U)) == FALSE) > 0 ||
       sum((rownames(tII$M) %in% rownames(tII$U)) == FALSE) > 0 ||
       length(grn$M) != length(red.oob$M) ||
       length(red$M) != length(grn.oob$M)){
      stop("Equal dimensions and IDs are expected.")
    }
    
    nsamples = length(samples(raw.set)) 
    if(nsamples == 0){
      stop("Dataset contains no samples.")
    }
    
    if(is.null(raw.set@pval.sites) || nrow(raw.set@pval.sites) != length(probeIDs) 
       || ncol(raw.set@pval.sites) != nsamples){
      raw.set@pval.sites <- matrix(data = NA, nrow = length(probeIDs), ncol = nsamples)
    }
    
    nmasked = 0
    
    for (i in 1:nsamples){
      sdf = SigDF(rbind(
          data.frame(
              Probe_ID = rownames(grn$M),
              MG = grn$M[,i], MR = red.oob$M[,i],
              UG = grn$U[,i], UR = red.oob$U[,i], col="G", mask=FALSE),
          data.frame(
              Probe_ID = rownames(red$M),
              MG = grn.oob$M[,i], MR = red$M[,i],
              UG = grn.oob$U[,i], UR = red$U[,i], col="R", mask=FALSE),
          data.frame(
              Probe_ID = rownames(tII$M),
              MG = NA, MR = NA,
              UG = tII$M[,i], UR = tII$U[,i], col="2", mask=FALSE)), platform)
      
      pvalues = pOOBAH(sdf, return.pval = TRUE)

      mask <- names(pvalues)[pvalues > pval.thresh]
      mask <- mask[!is.na(mask)]
      raw.set@pval.sites[, i] <- pvalues[match(probeIDs, names(pvalues))] 
      
      if(!(length(mask) == 0)){
        nmasked = nmasked + length(mask)
        maskedIDs <- match(mask, probeIDs) 
        raw.set@pval.sites[maskedIDs, i] <- NA
        raw.set@M[maskedIDs, i] <- NA
        raw.set@U[maskedIDs, i] <- NA
        raw.set@M0[maskedIDs, i]<- NA
        raw.set@U0[maskedIDs, i]<- NA
      }
    }
    rm(sigset.l, maskedIDs, mask, pvalues, grn, grn.oob, red, red.oob, tII, pIG, pII, pIR, platform, i)
    if (verbose){
      nprobes = length(probeIDs)    
      ntotal = nsamples*nprobes
      
      logger.info(paste('=======================',
        '=    pOOBAH           =',
        '=======================',
        paste0('No. probes: ', nprobes),
        paste0('No. samples: ', nsamples),
        paste0('No. probes times samples: ', ntotal),
        paste0('No. of masked probes: ', nmasked),
        paste0('Fraction of masked probes: ', round(nmasked/ntotal, digits = 3)),sep='\n'))
    }
    
    return(raw.set)
  }
}

################################################################################

########################################################################################################################
## differentialvariability.R
## created: 2017-07-28
## creator: Michael Scherer
## ---------------------------------------------------------------------------------------------------------------------
## The differential variability analysis methods between sample groups.
########################################################################################################################

#' rnb.execute.diffVar
#' 
#' This routine computes sites that are differentially variable between two sample groups specified as the column name 
#' in the phenotypic table.
#' 
#' @author Michael Scherer
#' @export
#' @param rnb.set Object of type \code{\linkS4class{RnBSet}} on which differential variability analysis should be conducted
#' @param pheno.cols Column names used to define the classes, whose methylation variability should be compared with each other
#' @param region.types Regions types to be used for the analysis. Defaults to the results given by rnb.region.types.for.analysis of the given RnBSet.
#' @param columns.adj Column names or indices in the table of phenotypic information to be used for confounder adjustment in the differential variability analysis.
#' @param adjust.celltype Flag indicating whether the resulting table should also contain estimated celltype contributions. See \code{\link{rnb.execute.ct.estimation}} for details.
#' @param disk.dump Flag indicating whether the resulting differential methylation object should be file backed, ie.e the matrices dumped to disk
#' @param disk.dump.dir disk location for file backing of the resulting differential methylation object. Only meaningful if \code{disk.dump=TRUE}.
#' @return Object of type \code{\linkS4class{RnBDiffMeth}} containing information about the differential variability analysis.
rnb.execute.diffVar <- function(rnb.set,pheno.cols=rnb.getOption("differential.comparison.columns"),
                                region.types=rnb.region.types.for.analysis(rnb.set),
                                columns.adj=rnb.getOption("covariate.adjustment.columns"),
                                adjust.celltype=rnb.getOption("differential.adjustment.celltype"),
                                disk.dump=rnb.getOption("disk.dump.big.matrices"),
                                disk.dump.dir=tempfile(pattern="diffMethTables_")){
  logger.start("Differential Variability")
  logger.start("Retrieving comparison info")
  cmp.info <- get.comparison.info(rnb.set, pheno.cols=pheno.cols,region.types = region.types, 
                                  columns.adj=columns.adj,
                                  adjust.celltype=adjust.celltype)
  logger.completed()
  if (is.null(cmp.info)) {
    return(NULL)
  }
  
  variability.method <- rnb.getOption("differential.variability.method")
  diff.meth <- new("RnBDiffMeth",variability.method=variability.method,disk.dump=disk.dump,disk.path=disk.dump.dir)
  if(!isImputed(rnb.set)) rnb.set <- rnb.execute.imputation(rnb.set,update.ff=FALSE)

  for(i in 1:length(cmp.info)){
    cmp.info.cur <- cmp.info[[i]]
    logger.start(c("Comparing ",cmp.info.cur$comparison))
    if(cmp.info.cur$paired){
      logger.status("Conducting PAIRED analysis")
    }
    meth.matrix <- meth(rnb.set)
    diffVar <- computeDiffVar.bin.site(X=meth.matrix,inds.g1=cmp.info.cur$group.inds$group1,
                                       inds.g2=cmp.info.cur$group.inds$group2, 
                                       paired=cmp.info.cur$paired,adjustment.table=cmp.info.cur$adjustment.table)

    diff.meth <- addDiffMethTable(diff.meth,diffVar,comparison=cmp.info.cur$comparison,region.type="sites",grp.labs=cmp.info.cur$group.names)
    rnb.cleanMem()
    if (length(cmp.info.cur$region.types)>0){
      diffVar.region <- computeDiffVar.bin.region(rnb.set,diffVar,
                                                  inds.g1=cmp.info.cur$group.inds$group1,inds.g2=cmp.info.cur$group.inds$group2,
                                                  region.types=cmp.info.cur$region.types,
                                                  paired = cmp.info.cur$paired
      )
      for(rt in region.types){
        diff.meth <- addDiffMethTable(diff.meth,diffVar.region[[rt]],comparison=cmp.info.cur$comparison, 
                                      region.type=rt, grp.labs=cmp.info.cur$group.names
        )
      }
    }
    logger.completed()
  }
  if(!all(get.comparisons(diff.meth)%in%names(cmp.info)))
    diff.meth <- addComparisonInfo(diff.meth,cmp.info)
  logger.completed()
  return(diff.meth)
}

#' diffVar
#' 
#' This routine applies the diffVar method from the \code{missMethyl} package that determines sites exhibiting
#' differential variability between two sample groups
#' 
#' @author Michael Scherer
#' @param meth.matrix Matrix containing the methylation information used to calculate differentially variable sites 
#'                    between the two groups
#' @param inds.g1 Indices in the phenotypic table corresponding to the first group.
#' @param inds.g2 Indices in the phenotypic table corresponding to the second group.
#' @param adjustment.table A \code{data.frame} containing variables to adjust for in the testing
#' @param paired Should the analysis be performed in a paired fashion. If yes, the first index in \code{inds.g1} must 
#'           correspond to the first in \code{inds.g2} and so on.
#' @return P-values as the result of the diffVar method not adjusted for multiple hypothesis testing.
#' 
#' @export
#' @references Phipson, Belinda, Oshlack, Alicia (2014)
#'             DiffVar: a new method for detecting differential variability with application to methylation in cancer and aging
#'             Genome Biology 15(9):465.
diffVar <- function(meth.matrix,inds.g1,inds.g2,adjustment.table=NULL,paired=FALSE){
  logger.start("diffVar method")
  rnb.require('missMethyl')
  if(is.logical(inds.g1)) inds.g1 <- which(inds.g1)
  if(is.logical(inds.g2)) inds.g2 <- which(inds.g2)
  n.g1 <- length(inds.g1)
  n.g2 <- length(inds.g2)
  n <- n.g1 + n.g2
  if (!is.null(adjustment.table)){
    if (!(is.data.frame(adjustment.table) && nrow(adjustment.table)==n && (!any(is.na(adjustment.table))))) {
      stop("invalid value for adjustment.table")
    }
    m <- ncol(adjustment.table)
    if (m == 0) {
      adjustment.table <- NULL
    } else {
      colnames(adjustment.table) <- paste0("x",1:m,"x")
    }
  }
  
  ind.vec <- c(inds.g1,inds.g2)
  if (length(ind.vec) < 2) stop("need at least two samples indices to compare")
  meth.matrix <- rnb.beta2mval(meth.matrix[,ind.vec,drop=FALSE])
  df <- data.frame(xg = factor(rep(c("group1","group2"), c(n.g1,n.g2)), levels=c("group1","group2")))
  if (!is.null(adjustment.table)){
    df <- cbind(df,adjustment.table)
  }
  if(paired){
    if(n.g1 != n.g2){
      stop("Could not conduct paired diffVar analysis: unequal groupsizes")
    }
    df$xp <- as.factor(rep(1:n.g1,2))
  }
  formula.text <- paste0(c("~0",colnames(df)),collapse="+")
  design.m <- model.matrix(as.formula(formula.text),data=df)
  colnames(design.m) <- make.names(colnames(design.m),unique=TRUE)
  colnames(design.m)[1:2] <- c("group1","group2")
  var.fit <- tryCatch(varFit(data=meth.matrix,design=design.m),error=function(e){
    logger.warning("Could not compute p-values with diffVar, returning NA")
    return(NULL)
  })
  if(is.null(var.fit)){
    return(rep(NA,nrow(meth.matrix)))
  }
  contrasts.m <- makeContrasts(group1vs2=group1-group2,levels=design.m)
  var.fit <- contrasts.fit(var.fit,contrasts.m)
  var.fit <- eBayes(var.fit)
  logger.completed()
  return(var.fit$p.value[,"group1vs2"])
}

#' apply.iEVORA
#' 
#' This routine applies the iEVORA method created by Teschendorff et.al. to the supplied methylation matrix in a similar way
#' as the diffVar method.
#' 
#' @author Michael Scherer
#' @param meth.matrix Matrix containing the methylation information used to calculate differentially variable sites 
#'                    between the two groups
#' @param inds.g1 Indices in the phenotypic table corresponding to the first group.
#' @param inds.g2 Indices in the phenotypic table corresponding to the second group.
#' @return Q-values as the result of applying the iEVORA method and then correct for multiple testing.
#' @export
apply.iEVORA <- function(meth.matrix,inds.g1,inds.g2){
  if(is.logical(inds.g1)) inds.g1 <- which(inds.g1)
  if(is.logical(inds.g2)) inds.g2 <- which(inds.g2)
  n <- length(inds.g1)+length(inds.g2)
  pheno.binary <- rep(0,n)
  pheno.binary[inds.g2] <- 1
  result <- suppressWarnings(iEVORA(data.m=meth.matrix,pheno.v=pheno.binary))
  if(is.null(result)){
    p.values <- rep(1,dim(meth.matrix)[1])
    return(p.values)
  }
  p.values <- rep(1,dim(meth.matrix)[1])
  p.values[result[,"index"]] <- result[,"P(BT)"]
  return(p.values)
}

#' iEVORA
#' 
#' This method performs the iEVORA algorithm introduced by Teschendorff et.al. to detect sites showing differential
#' variability between two groups. The routine has been modified from the R script associated with the publication.
#' 
#' @author Andrew E. Teschendorff, with minor modifications by Michael Scherer
#' @param data.m Methylation matrix with rows labeling features, and columns labeling samples. 
#'               Rownames should be feature/probe IDs and should be provided.
#' @param pheno.v Phenotype vector with entries either 0 or 1 correpsonding to the group assignment.
#' @param thDV q-value threshold for the differntial variability test. Default is 0.001.
#' @param thDM p-value threshold for differntial methylation means. Default is 0.05.
#' 
#' @return A matrix of ranked differentially variable (DV) and differentially methylated CpGs (DVMCs), 
#'          ranked according to the t-statistic p-value, but selected using the Bartlett's DV test. 
#'          Columns labels are the t-statistic, its P-value, the mean of phenotype-1, the mean of phenotype-0, 
#'          the log-ratio of the variances of phenotype-1 to phenotype-0,as well as the Bartlett's test P-value 
#'          and q-value.
#' @noRd
#' @references Teschendorff, Andrew E, Gao, Yang, Jones, Allison, Ruebner, Matthias, Beckmann, Matthias W., Wachter, David L.
#'              ,Fasching, Peter A., Widschwendter, Martin
#'             DNA methylation outliers in normal breast tissue identify field defects that are enriched in cancer
#'             Nature Communications 7:10478.
iEVORA <- function(data.m,pheno.v,thDV=0.001,thDM=0.05){
  logger.start("iEVORA method")
  rnb.require("qvalue")
  rnb.require("stats")
  statDVC.m <- t(apply(data.m,1,doDV,pheno.v));
  qvDVC.v <- qvalue(statDVC.m[,2])$qval;
  dvc.idx <- which(qvDVC.v < thDV);
  nDVC <- length(dvc.idx);
  if( nDVC > 0 ){
    statDMC.m <- t(apply(data.m[dvc.idx,],1,doTT,pheno.v))
    tmp.s <- sort(statDMC.m[,2],decreasing=FALSE,index.return=TRUE);
    pvDMC.v <- tmp.s$x;
    ntop <- length(which(pvDMC.v < 0.05));
    if(ntop > 0){
      topDVMC.m <- cbind(dvc.idx[tmp.s$ix[1:ntop]],statDMC.m[tmp.s$ix[1:ntop],],statDVC.m[dvc.idx[tmp.s$ix[1:ntop]],c(3:4,1:2)],qvDVC.v[dvc.idx[tmp.s$ix[1:ntop]]]);
      colnames(topDVMC.m) <- c("index","t","P(TT)","Av1","Av0","log[V1/V0]","P(BT)","q(BT)");
      rownames(topDVMC.m) <- rownames(statDMC.m)[tmp.s$ix[1:ntop]];
    }
    else {
      logger.info("No DVCs found.")
      return(NULL)
    }
  }
  else {
    logger.info("No DVCs detected. All p-values set to 1.");
    return(NULL)
  }
  logger.completed()
  return(topDVMC.m);
}

#' doDV
#' 
#' Helper function for the iEVORA method also supplied with the publication used to determine differentially variable
#' sites.
#' 
#' @author Andrew Teschendorff, minor modifications by Michael Scherer
#' @param tmp.v A row from the methylation matrix corresponding to a single site.
#' @param pheno.v Vector containing the splitting into the two groups for which differential variability should be detected.
#' 
#' @return Vector containing the log odds of variances, the p-value from the Bartlett's test and the mean methylation 
#'          levels for the two groups.
#' @noRd
doDV <- function(tmp.v,pheno.v){
  rnb.require('stats')
  co.idx <- which(pheno.v==0);
  ca.idx <- which(pheno.v==1);
  bt.o <- bartlett.test(x=tmp.v,g=pheno.v);
  pv <- bt.o$p.value;
  logR <- log2(var(tmp.v[ca.idx])/var(tmp.v[co.idx]));
  avCA <- mean(tmp.v[ca.idx]);
  avCO <- mean(tmp.v[co.idx]);
  out.v <- c(logR,pv,avCA,avCO);
  names(out.v) <- c("log(V1/V0)","P(BT)","Av1","Av0");
  return(out.v);
}

#' doTT
#' Helper function for the iEVORA method also supplied with the publication used to calculate t-statistics of differential
#' methylation between two sample groups.
#' 
#' 
#' @author Andrew Teschendorff, with minor modifications by Michael Scherer
#' @param tmp.v A row from the methylation matrix corresponding to a single site.
#' @param pheno.v Vector containing the splitting into the two groups for which differential variability should be detected.
#' 
#' @return Vector containing the t-statistics and the t-statistics p-value.
#' @noRd
doTT <- function(tmp.v,pheno.v){
  tt.o <- t.test(tmp.v ~ pheno.v);
  out.v <- c(-tt.o$stat,tt.o$p.val);
  names(out.v) <- c("t","P");
  return(out.v);
}

#' Returns the colum names of the differential variability table.
#' @param includeCovg Flag indicating if dataset contains coverage information
#' @return Column names of the differential variability table
get.table.ids <- function(includeCovg=FALSE){
  ret <- c("var.g1","var.g2","var.diff","var.log.ratio","diffVar.p.val")
  if(includeCovg){
    ret <- c(ret,"mean.covg.g1","covg.thresh.nsamples.g1","mean.covg.g2","covg.thresh.nsamples.g2")
  }
  return(ret)
}

#' addReportPlot.diffVar.scatter.site
#' 
#' This function creates a scatterplot for the given comparison comparing the difference in variances with the
#' usted) p-value as the result of the differentially variability analysis.
#' 
#' @param report Object of class \code{\linkS4class{Report}} to which the plot should be added
#' @param var.table Variability table containing the relevant information to be put into the scatterplot
#' @param comparison.name Comparison for which the scatterplot should be creaed
#' @param rank.cutoffs.numbers Numbers to be used for plotting the best ranked sites
#' @param auto.cutoff Auto selected cutoffs for the given comparison
#' @param group.name1 Name of the first group of the comparison
#' @param group.name2 Name of the second group of the comparison
#' @return list of scatterplots
#' @noRd
addReportPlot.diffVar.scatter.site <- function (report, var.table, comparison.name,
                              rank.cutoffs.numbers, auto.cutoff, rerank=TRUE,
                              group.name1="Group1",group.name2="Group2"){
  ret <- list()
  sparse.points <- DENS.SCATTER.SPARSE.POINTS.PERC
  if (DENS.SCATTER.SPARSE.POINTS.MAX < sparse.points*nrow(var.table)){
    sparse.points <- DENS.SCATTER.SPARSE.POINTS.MAX
  }
  #dens.subsample <- FALSE
  dens.subsample <- FALSE
  if(nrow(var.table)>10000){
    dens.subsample <- 10000
  }
  #if (nrow(var.table) > DENS.SCATTER.SUBSAMPLE.THRES){
  #  dens.subsample <- DENS.SCATTER.SUBSAMPLE.THRES
  #}
  
  if("diffVar.p.adj.fdr" %in% colnames(var.table)){
    var.sites <- var.table[,"diffVar.p.adj.fdr"] < P.VAL.CUT
    plot <- create.densityScatter(var.table[,c("var.g1","var.g2")],is.special=var.sites,
                                  dens.subsample=dens.subsample, add.text.cor=TRUE) +
        xlab(paste("Variance",group.name1)) + ylab(paste("Variance",group.name2)) + coord_fixed(xlim = c(0,max(var.table[,c("var.g1","var.g2")])),ylim = c(0,max(var.table[,c("var.g1","var.g2")])))
    comp.type <- "fdrAdjPval"
    fig.name <- paste("diffVar_site",comparison.name,comp.type,sep = "_")
    plot <- createReportGgPlot(plot,fig.name,report=report,create.pdf = FALSE, high.png = 200)
    plot <- off(plot,handle.errors=TRUE)
    ret <- c(ret,list(plot))
  }
  ranks <- var.table[,"combinedRank.var"]
  if (rerank)	ranks <- rank(ranks,na.last="keep",ties.method="min")
  for(i in 1:length(rank.cutoffs.numbers)){
    number <- rank.cutoffs.numbers[i]
    cutoff.name <- paste0("rc",i)
    var.table$isDVC <- ranks < number
    plot <- create.densityScatter(var.table[,c("var.g1","var.g2")], is.special = var.table$isDVC,
                                  dens.subsample=dens.subsample, add.text.cor=TRUE) +
      xlab(paste("Variance",group.name1)) + ylab(paste("Variance",group.name2)) + coord_fixed(xlim = c(0,max(var.table[,c("var.g1","var.g2")])),ylim = c(0,max(var.table[,c("var.g1","var.g2")])))
    fig.name <- paste("diffVar_site",comparison.name,cutoff.name,sep="_")
    plot <- createReportGgPlot(plot,fig.name,report=report,create.pdf = FALSE, high.png = 200)
    plot <- off(plot,handle.errors=TRUE)
    ret <- c(ret,list(plot))
  }
  if(is.integer(auto.cutoff)){
    var.table$isDVC <- ranks < auto.cutoff
    plot <- create.densityScatter(var.table[,c("var.g1","var.g2")], is.special = var.table$isDVC,
                                  dens.subsample=dens.subsample, add.text.cor=TRUE) +
      xlab(paste("Variance",group.name1)) + ylab(paste("Variance",group.name2))  + coord_fixed(xlim = c(0,max(var.table[,c("var.g1","var.g2")])),ylim = c(0,max(var.table[,c("var.g1","var.g2")])))
    fig.name <- paste("diffVar_site",comparison.name,"rcAuto",sep="_")
    plot <- createReportGgPlot(plot,fig.name,report=report,create.pdf = FALSE, high.png = 200)
    plot <- off(plot,handle.errors=TRUE)
    ret <- c(ret,list(plot))
  }
  return(ret)
}
#' @noRd
addReportPlot.diffVar.scatter.region <- function (report, var.table, comparison.name, region.name, rerank=TRUE,
                                           ranking.cutoffs, auto.cutoff=NULL, group.name1="Group1",group.name2="Group2",useSiteCols=FALSE){
  ret <- list()
  
  cn.x <- "mean.var.g1"
  cn.y <- "mean.var.g2"
  cn.pa <- "comb.p.adj.var.fdr"
  if (useSiteCols){
    cn.x <- "var.g1"
    cn.y <- "var.g2"
    cn.pa <- "diffVar.p.adj.fdr"
  }
  al.x <- paste("Mean Variance",group.name1,sep=".")
  al.y <- paste("Mean Variance",group.name2,sep=".")
  
  dens.subsample <- FALSE
  if(nrow(var.table) > DENS.SCATTER.SUBSAMPLE.THRES){
    dens.subsample <- DENS.SCATTER.SUBSAMPLE.THRES
  }
  
  if(cn.pa %in% colnames(var.table)){
    var.sites <- var.table[,cn.pa] < P.VAL.CUT
    plot <- create.densityScatter(var.table[,c(cn.x,cn.y)],is.special=var.sites,
                                  dens.subsample = dens.subsample, add.text.cor=TRUE) +
      xlab(al.x) + ylab(al.y) + coord_fixed(xlim = c(0,max(var.table[,c("mean.var.g1","mean.var.g2")])),ylim = c(0,max(var.table[,c("mean.var.g1","mean.var.g2")])))
    comp.type <- "fdrAdjPval"
    fig.name <- paste("diffVar_region",comparison.name,region.name,comp.type,sep = "_")
    plot <- createReportGgPlot(plot,fig.name,report=report,create.pdf = FALSE)
    plot <- off(plot,handle.errors=TRUE)
    ret <- c(ret,list(plot))
  }
  
  rrs <- var.table[,"combinedRank.var"]
  if (rerank)	rrs <- rank(rrs,na.last="keep",ties.method="min")
  for (i in 1:length(ranking.cutoffs)){
    rc <- ranking.cutoffs[i]
    cur.cut.name <- paste("rc",i,sep="")
    var.table$isDVR <- rrs < rc
    
    pp <- create.densityScatter(var.table[,c(cn.x,cn.y)],is.special=var.table$isDVR,
                                dens.subsample = dens.subsample, add.text.cor=TRUE) +
      labs(x=al.x, y=al.y) + coord_fixed(xlim = c(0,max(var.table[,c("mean.var.g1","mean.var.g2")])),ylim = c(0,max(var.table[,c("mean.var.g1","mean.var.g2")])))
    
    figName <- paste("diffVar_region",comparison.name,region.name,cur.cut.name,sep="_")
    report.plot <- createReportGgPlot(pp,figName, report,create.pdf=FALSE,high.png=200)
    report.plot <- off(report.plot,handle.errors=TRUE)
    ret <- c(ret,list(report.plot))
  }
  
  if (is.integer(auto.cutoff)){
    var.table$isDVR <- var.table[,"combinedRank.var"] <= auto.cutoff
    pp <- create.densityScatter(var.table[,c(cn.x,cn.y)],is.special=var.table$isDVR,
                                dens.subsample = dens.subsample, add.text.cor=TRUE) +
      labs(x=al.x, y=al.y) + coord_fixed(xlim = c(0,max(var.table[,c("mean.var.g1","mean.var.g2")])),ylim = c(0,max(var.table[,c("mean.var.g1","mean.var.g2")])))
    figName <- paste("diffVar_region",comparison.name,region.name,"rcAuto",sep="_")
    report.plot <- createReportGgPlot(pp,figName, report,create.pdf=FALSE,high.png=200)
    report.plot <- off(report.plot,handle.errors=TRUE)
    ret <- c(ret,list(report.plot))
  }

  return(ret)
}

#' addReportPlot.diffVar.volcano
#' 
#' This function creates volcano plots for the given comparison comparing the difference in variances with the
#' negative decadic logarithm of the (adjusted) p-value as the result of the differentially variability analysis.
#' 
#' @param report Object of class \code{\linkS4class{Report}} to which the plot should be added
#' @param var.table Variability table containing the relevant information to be put into the scatterplot
#' @param comparison.name Comparison for which the scatterplot should be creaed
#' @param group.name1 Name of the first group of the comparison
#' @param group.name2 Name of the second group of the comparison
#' @aliases addReportPlot.diffVar.volcano
#' @aliases addReportPlot.diffVar.volcano.region
#' @return list of volcano plots
#' @noRd
addReportPlot.diffVar.volcano <- function (report, var.table, comparison.name,
                                           group.name1="Group1",group.name2="Group2"){
  ret <- list()
  dont.plot.p.val <- all(is.na(var.table[,"diffVar.p.val"]))
  
  fig.name <- paste("diffVar_volcano_site",comparison.name,"pVal",sep="_")
  if (!dont.plot.p.val){
    pp <- ggplot(var.table) + aes(x=var.diff,y=log10P,color=log10(combinedRank.var)) + 
      scale_color_gradientn(colours=rev(rnb.getOption("colors.gradient"))) +
      geom_point()
  } else {
    pp <- rnb.message.plot("No p-value available")
  }
  report.plot <- createReportGgPlot(pp,fig.name, report,create.pdf=FALSE,high.png=200)
  report.plot <- off(report.plot,handle.errors=TRUE)
  ret <- c(ret,list(report.plot))
  
  fig.name <- paste("diffVar_volcano_site",comparison.name,"pValAdj",sep="_")
  pp <- ggplot(var.table) + aes(x=var.diff,y=log10FDR,color=log10(combinedRank.var)) +
    scale_color_gradientn(colours=rev(rnb.getOption("colors.gradient")))+
    geom_point()
  report.plot <- createReportGgPlot(pp,fig.name, report,create.pdf=FALSE,high.png=200)
  report.plot <- off(report.plot,handle.errors=TRUE)
  ret <- c(ret,list(report.plot))
  
  return(ret)
}
#' addReportPlot.diffVar.volcano.region
#' @noRd
addReportPlot.diffVar.volcano.region <- function(report, var.table,comparison.name,region.type,
                                                  group.name1,group.name2, useSiteCols=FALSE){
  cn.d <- "mean.var.diff"
  cn.q <- "mean.var.log.ratio"
  cn.p <- "comb.p.val.var"
  cn.pa <- "comb.p.adj.var.fdr"
  if (useSiteCols){
    cn.d <- "var.diff"
    cn.q <- "var.log.ratio"
    cn.p <- "diffVar.p.val"
    cn.pa <- "diffVar.p.adj.fdr"
  }

  figPlots <- list()
  dont.plot.p.val <- all(is.na(var.table[,cn.p]))
  
  figName <- paste("diffVar_region_volcano",comparison.name,region.type,"diff","pVal",sep="_")
  if (!dont.plot.p.val){
    pp <- ggplot(var.table) + aes_string(cn.d, paste0("-log10(",cn.p,")"), color="log10(combinedRank.var)") +
      scale_color_gradientn(colours=rev(rnb.getOption("colors.gradient"))) +
      geom_point()#(alpha=0.3)
  } else {
    pp <- rnb.message.plot("No p-value available")
  }
  report.plot <- createReportGgPlot(pp,figName, report,create.pdf=FALSE,high.png=200)
  report.plot <- off(report.plot,handle.errors=TRUE)
  figPlots <- c(figPlots,list(report.plot))
  
  figName <- paste("diffVar_region_volcano",comparison.name,region.type,"diff","pValAdj",sep="_")
  pp <- ggplot(var.table) + aes_string(cn.d, paste0("-log10(",cn.pa,")"), color="log10(combinedRank.var)") +
    scale_color_gradientn(colours=rev(rnb.getOption("colors.gradient"))) +
    geom_point()#(alpha=0.3)
  report.plot <- createReportGgPlot(pp,figName, report,create.pdf=FALSE,high.png=200)
  report.plot <- off(report.plot,handle.errors=TRUE)
  figPlots <- c(figPlots,list(report.plot))
  
  figName <- paste("diffVar_region_volcano",comparison.name,region.type,"quot","pVal",sep="_")
  if (!dont.plot.p.val){
    pp <- ggplot(var.table) + aes_string(cn.q, paste0("-log10(",cn.p,")"), color="log10(combinedRank.var)") +
      scale_color_gradientn(colours=rev(rnb.getOption("colors.gradient"))) +
      geom_point()#(alpha=0.3)
  } else {
    pp <- rnb.message.plot("No p-value available")
  }
  report.plot <- createReportGgPlot(pp,figName, report,create.pdf=FALSE,high.png=200)
  report.plot <- off(report.plot,handle.errors=TRUE)
  figPlots <- c(figPlots,list(report.plot))
  
  figName <- paste("diffVar_region_volcano",comparison.name,region.type,"quot","pValAdj",sep="_")
  pp <- ggplot(var.table) + aes_string(cn.q, paste0("-log10(",cn.pa,")"), color="log10(combinedRank.var)") +
    scale_color_gradientn(colours=rev(rnb.getOption("colors.gradient"))) +
    geom_point()#(alpha=0.3)
  report.plot <- createReportGgPlot(pp,figName, report,create.pdf=FALSE,high.png=200)
  report.plot <- off(report.plot,handle.errors=TRUE)
  figPlots <- c(figPlots,list(report.plot))
  
  figName <- paste("diffVar_region_volcano",comparison.name,region.type,"diff","quotSig",sep="_")
  pp <- ggplot(var.table) + aes_string(cn.d, cn.q, color="log10(combinedRank.var)") +
    scale_color_gradientn(colours=rev(rnb.getOption("colors.gradient"))) +
    geom_point()#(alpha=0.3)
  report.plot <- createReportGgPlot(pp, figName, report,create.pdf=FALSE,high.png=200)
  report.plot <- off(report.plot,handle.errors=TRUE)
  figPlots <- c(figPlots,list(report.plot))
  
  figName <- paste("diffVar_region_volcano",comparison.name,region.type,"quot","quotSig",sep="_")
  pp <- rnb.message.plot("Quotient--Quotient scatterplot not available")
  report.plot <- createReportGgPlot(pp,figName, report,create.pdf=FALSE,high.png=200)
  report.plot <- off(report.plot,handle.errors=TRUE)
  figPlots <- c(figPlots,list(report.plot))
  
  return(figPlots)
}

#' addReportPlot.diffVar.meth
#' 
#' This function compares differentially methylated sites (DMCs) with differentially variable sites (DVCs) in a scatterplot
#' and colors significantly differential sites with different colors.
#' 
#' @param report Object of class \code{\linkS4class{Report}} to which the plot should be added.
#' @param var.table Table containing the differentially variable sites.
#' @param comparison.name Name of the comparison to be conducted.
#' @param group.name1 Name of the first group in the comparison
#' @param group.name2 Name of the second group in the comparison
#' @param auto.diffVar Automatically selected rank cutoff for differential varibility
#' @param auto.diffMeth Automatically selected rank cutoff for differential methylation
#' @param rank.cuts Cutoffs used for differentiality detection
#' @return The modified report
#' @noRd
addReportPlot.diffVar.meth <- function(report, var.table, comparison.name,
                           group.name1,group.name2,auto.diffVar=NULL,auto.diffMeth=NULL,rank.cuts=c(100,1000,10000,100000),rerank=TRUE){
  ret <- list()
  dens.subsample <- FALSE
  if (nrow(var.table) > 10000){
    dens.subsample <- 10000
  }

  ranks.diffVar <- var.table[,"combinedRank.var"]
  if(rerank) ranks.diffVar <- rank(ranks.diffVar,na.last = "keep",ties.method = "min")
  ranks.diffMeth <- var.table[,"combinedRank"]
  if(rerank) ranks.diffMeth <- rank(ranks.diffMeth,na.last = "keep",ties.method = "min")
  
  fig.name <- paste("diffVarMeth_site",comparison.name,sep="_")
  for(i in 1:length(rank.cuts)){
    cutoff <- rank.cuts[i]
    cut.id <- paste0("rc",i)
    toPlot <- data.frame(Rank.meth=var.table[,"combinedRank"],Rank.var=var.table[,"combinedRank.var"])
    is.diff.meth <- ranks.diffMeth < cutoff
    is.diff.var <- ranks.diffVar < cutoff
    rank.cut.diffMeth <- max(toPlot[is.diff.meth,1])
    rank.cut.diffVar <- max(toPlot[is.diff.var,2])
    color.diff <- rep('Not Significant',dim(toPlot)[1])
    color.diff[is.diff.meth] <- "DMC"
    color.diff[is.diff.var] <- "DVC"
    color.diff[is.diff.meth&is.diff.var] <- "Both"
    toPlot <- data.frame(toPlot,Color.diff=color.diff)
    is.special <- is.diff.meth|is.diff.var
    pp <- create.diffMeth.diffVar.subsample(toPlot,dens.subsample, is.special=is.special, rank.cut.diffMeth = rank.cut.diffMeth, rank.cut.diffVar = rank.cut.diffVar)
    fig.name <- paste("diffMethVar",comparison.name,cut.id,sep="_")
    report.plot <- createReportGgPlot(pp,fig.name,report,create.pdf = FALSE,high.png = 200)
    report.plot <- off(report.plot,handle.errors=TRUE)
    ret <- c(ret,report.plot)
  }
  if(is.integer(auto.diffVar)&&is.integer(auto.diffMeth)){
    toPlot <- data.frame(Rank.meth=var.table[,"combinedRank"],Rank.var=var.table[,"combinedRank.var"])
    is.diff.meth <- toPlot$Rank.meth < auto.diffMeth
    is.diff.var <- toPlot$Rank.var < auto.diffVar
    color.diff <- rep('Not Significant',dim(toPlot)[1])
    color.diff[is.diff.meth] <- "DMC"
    color.diff[is.diff.var] <- "DVC"
    color.diff[is.diff.meth&is.diff.var] <- "Both"
    toPlot <- data.frame(toPlot,Color.diff=color.diff)
    is.special <- is.diff.meth|is.diff.var
    pp <- create.diffMeth.diffVar.subsample(toPlot,dens.subsample,is.special=is.special, rank.cut.diffMeth = auto.diffMeth, rank.cut.diffVar = auto.diffVar)
    fig.name <- paste("diffMethVar",comparison.name,"rcAuto",sep="_")
    report.plot <- createReportGgPlot(pp,fig.name,report,create.pdf = FALSE,high.png = 200)
    report.plot <- off(report.plot,handle.errors=TRUE)
    ret <- c(ret,report.plot)
  }
  return(ret)
}

#' addReportPlot.diffVar.meth.region
#' 
#' This function compares differentially methylated sites (DMCs) with differentially variable sites (DVCs) in a scatterplot
#' and colors significantly differential sites with different colors.
#' 
#' @param report Object of class \code{\linkS4class{Report}} to which the plot should be added.
#' @param var.table Table containing the differentially variable sites.
#' @param comparison.name Name of the comparison to be conducted.
#' @param region.type Region type used
#' @param group.name1 Name of the first group in the comparison
#' @param group.name2 Name of the second group in the comparison
#' @param auto.diffVar Automatically selected rank cutoff for differential varibility
#' @param auto.diffMeth Automatically selected rank cutoff for differential methylation
#' @param rank.cuts Cutoffs used for differentiality detection
#' @return The modified report
#' @noRd
addReportPlot.diffVar.meth.region <- function(report, var.table, comparison.name, region.type,
                                       group.name1,group.name2,auto.diffVar=NULL,auto.diffMeth=NULL,rank.cuts=c(100,500,1000),rerank=TRUE){
  ret <- list()
  dens.subsample <- FALSE
  if (nrow(var.table) > 10000){
    dens.subsample <- 10000
  }
  
  ranks.diffVar <- var.table[,"combinedRank.var"]
  if(rerank) ranks.diffVar <- rank(ranks.diffVar,na.last = "keep",ties.method = "min")
  ranks.diffMeth <- var.table[,"combinedRank"]
  if(rerank) ranks.diffMeth <- rank(ranks.diffMeth,na.last = "keep",ties.method = "min")
  
  fig.name <- paste("diffVarMeth_region",comparison.name,region.type,sep="_")
  for(i in 1:length(rank.cuts)){
    cutoff <- rank.cuts[i]
    cut.id <- paste0("rc",i)
    toPlot <- data.frame(Rank.meth=var.table[,"combinedRank"],Rank.var=var.table[,"combinedRank.var"])
    is.diff.meth <- ranks.diffMeth < cutoff
    is.diff.var <- ranks.diffVar < cutoff
    rank.cut.diffMeth <- max(toPlot[is.diff.meth,1])
    rank.cut.diffVar <- max(toPlot[is.diff.var,2])
    color.diff <- rep('Not Significant',dim(toPlot)[1])
    color.diff[is.diff.meth] <- "DMR"
    color.diff[is.diff.var] <- "DVR"
    color.diff[is.diff.meth&is.diff.var] <- "Both"
    toPlot <- data.frame(toPlot,Color.diff=color.diff)
    is.special <- is.diff.meth|is.diff.var
    pp <- create.diffMeth.diffVar.subsample(toPlot,dens.subsample, is.special=is.special, rank.cut.diffMeth = rank.cut.diffMeth, rank.cut.diffVar = rank.cut.diffVar)
    fig.name <- paste("diffMethVar",comparison.name,region.type,cut.id,sep="_")
    report.plot <- createReportGgPlot(pp,fig.name,report,create.pdf = FALSE,high.png = 200)
    report.plot <- off(report.plot,handle.errors=TRUE)
    ret <- c(ret,report.plot)
  }
  if(is.integer(auto.diffVar)&&is.integer(auto.diffMeth)){
    toPlot <- data.frame(Rank.meth=var.table[,"combinedRank"],Rank.var=var.table[,"combinedRank.var"])
    is.diff.meth <- toPlot$Rank.meth < auto.diffMeth
    is.diff.var <- toPlot$Rank.var < auto.diffVar
    color.diff <- rep('Not Significant',dim(toPlot)[1])
    color.diff[is.diff.meth] <- "DMR"
    color.diff[is.diff.var] <- "DVR"
    color.diff[is.diff.meth&is.diff.var] <- "Both"
    toPlot <- data.frame(toPlot,Color.diff=color.diff)
    is.special <- is.diff.meth|is.diff.var
    pp <- create.diffMeth.diffVar.subsample(toPlot,dens.subsample, is.special=is.special, rank.cut.diffMeth = auto.diffMeth, rank.cut.diffVar = auto.diffVar)
    fig.name <- paste("diffMethVar",comparison.name,region.type,"rcAuto",sep="_")
    report.plot <- createReportGgPlot(pp,fig.name,report,create.pdf = FALSE,high.png = 200)
    report.plot <- off(report.plot,handle.errors=TRUE)
    ret <- c(ret,report.plot)
  }
  return(ret)
}

#' create.diffMeth.diffVar.subsample
#' 
#' This routine creates a plot showing both differentially methylated and differentially variable sites for the given rank cutoffs,
#' where the x-axis is the combined rank of the differential methylation and the y-axis the combined rank of the differential
#' variability analysis.
#' @param df2p Data frame containing the ranks and color of the sites
#' @param dens.subsample Number specifiying the subsample that should be taken from the whole population of sites
#' @param rank.cut.diffMeth Rank cutoff for differential methylation (absolute rank)
#' @param rank.cut.diffVar Rank cutoff for differential variability (absolute rank)
#' @return Plot object with the corresponding plot
#' @noRd
create.diffMeth.diffVar.subsample <- function(df2p,dens.subsample,is.special=NULL,rank.cut.diffMeth, rank.cut.diffVar,sparse.points=0.01,dens.n=100){
  if (!(is.numeric(sparse.points) && sparse.points>=0)) {
    stop("Invalid parameter value: sparse.points")
  }
  if (is.null(df2p) || nrow(df2p)<1){
    logger.warning(c("Could not create density scatterplot"))
    pp <- rnb.message.plot("Could not create plot")
    return(pp)
  }
  df2p <- na.omit(df2p)
  if (is.null(df2p) || nrow(df2p)<1){
    logger.warning(c("Could not create density scatterplot (NA omission removed all entries)"))
    pp <- rnb.message.plot("Could not create plot")
    return(pp)
  }
  df2p.sub <- df2p
  dens.ranks <- NULL
  tryCatch(
    dens.ranks <- densRanks(x=df2p[,1],y=df2p[,2]),
    error=function(ee){
      logger.warning(c("Could not assess density ranking:",ee$message))
    }
  )
  if (is.numeric(dens.subsample) && dens.subsample>0){
    ss <- as.integer(dens.subsample)
    if (nrow(df2p) > ss) {
      df2p.sub <- df2p[sample(nrow(df2p),ss),]
    }
  }
  
  #the standard bandwith function of MASS::kde2d is unstable when looking at
  #distributions with very low variance. Here's a more stable version
  stable.bandwidth.fun <- function(x,eps=1e-4){
    r <- quantile(x, c(0.25, 0.75))
    h <- (r[2] - r[1])/1.34
    if (h==0) h <- eps
    4 * 1.06 * min(sqrt(var(x)), h) * length(x)^(-1/5)
  }
  stable.h <- c(stable.bandwidth.fun(df2p.sub[,1]),stable.bandwidth.fun(df2p.sub[,2]))
  
  if (is.null(dens.ranks)){
    pp <- rnb.message.plot("Could not assess density")
  } else {
    pp <- ggplot(df2p.sub) + aes_string(x=colnames(df2p)[1],y=colnames(df2p)[2]) + 
      stat_density2d(geom="tile", fill=DENS.COLORS.LOW[1], aes(alpha=after_stat(density)^0.25), contour=FALSE, n=dens.n, h=stable.h) +
      scale_alpha(range = c(0.0, 1),guide=FALSE) +
      geom_vline(xintercept = rank.cut.diffMeth,linetype='dotted') + geom_hline(yintercept = rank.cut.diffVar,linetype='dotted')+
      scale_color_manual(values=c(rnb.getOption("colors.category")[c(1,2,4)],DENS.COLORS.LOW[1]))
    if (sparse.points > 0){
      if (sparse.points <= 1){
        thres <- ceiling(nrow(df2p)*sparse.points)
      } else {
        thres <- sparse.points
      }
      df2p.loose <- df2p[dens.ranks<=thres,]#the sub data.frame in of the least dens points
      pp <- pp + geom_point(data=df2p.loose,aes_string(x=colnames(df2p)[1],y=colnames(df2p)[2],colour=colnames(df2p)[3]),size=0.4)
    }
    if(!is.null(is.special)){
      df2p.special <- df2p[is.special,]
      if(is.numeric(dens.subsample) && dens.subsample>0){
        ss <- as.numeric(dens.subsample)
        if(nrow(df2p.special)>ss){
          pp <- pp + geom_point(data=df2p.special,aes_string(x=colnames(df2p)[1],y=colnames(df2p)[2],colour=colnames(df2p)[3]),size=0.4) + guides(fill=FALSE)
        }else{
          pp <- pp + geom_point(data=df2p.special,aes_string(x=colnames(df2p)[1],y=colnames(df2p)[2],colour=colnames(df2p)[3]),size=1) + guides(fill=FALSE)
        }
      }else{
        pp <- pp + geom_point(data=df2p.special,aes_string(x=colnames(df2p)[1],y=colnames(df2p)[2],colour=colnames(df2p)[3]),size=1) + guides(fill=FALSE)
      }
    }  
  }
  return(pp)
}

#' rnb.section.diffVar
#'
#' This function adds information about the differential variability analysis to the specified report.
#' 
#' @param rnb.set Object of class \code{\linkS4class{RnBSet}} on which the analysis was conducted.
#' @param diff.meth Object of class \code{\linkS4class{RnBDiffMeth}} as the result of applying \code{rnb.execute.diffVar} 
#'                  containing the differentially variable sites
#' @param report Report object (\code{\linkS4class{Report}}) to which information should be added.
#' @param gzTable Flag indicating if the tables should ne gzipped
#' @param level Which level of section should be created. See \code{rnb.add.section}.
#' @return Modified report with section about result of differential variability analysis
#' @author Michael Scherer
#' @noRd
rnb.section.diffVar <- function(rnb.set,diff.meth,report,gzTable=FALSE,differentiality.method=rnb.getOption("differential.variability.method"),level=1){
  if (length(get.comparisons(diff.meth))<1){
    stop("no valid comparisons")
  }
  diff.var.cutoff <- 0.01
  rank.cutoffs.numbers <- c(100,1000,10000,100000)
  logger.start("Adding Differential Variability Information")
  txt <- c("Differentially variable sites were computed with <code>", differentiality.method, "</code>.",
           " For more information about the method, have a look at the <a href=")
  if(differentiality.method=='diffVar'){
    refText.missMethyl <- c("Phipson, B., & Oshlack, A. (2014). DiffVar: a new method for detecting differential variability with application to methylation in cancer and aging. ",
                      "<i>Genome Biology</i>, <b>15</b>(9), 465")
    report <- rnb.add.reference(report, refText.missMethyl)
    txt <- c(txt,"https://bioconductor.org/packages/release/bioc/html/missMethyl.html>missMethyl</a> Bioconductor package.",
             rnb.get.reference(report,refText.missMethyl))
  }else if (differentiality.method=='iEVORA'){
    refText.iEVORA <- c("Teschendorff, A.E. et.al., DNA methylation outliers in normal breast tissue identify field defects that are enriched in cancer",
                        "<i>Nature Communications</i>, <b>7</b>, 10478")
    txt <- c(txt,"https://www.nature.com/articles/ncomms10478>iEVORA</a> method published in Nature.",
             "For the iEVORA method, adjustment columns are not supported and therefore ignored.",
             rnb.get.reference(report,refText.iEVORA))
  }
  txt <- c(txt," This section contains plots and tables describing the results of this test and further analyses ",
            "of the sites that were selected as differentially variable. Please note that missing methylation values have ",
           "been imputed with ",rnb.getOption("imputation.method"),".")
  report <- rnb.add.section(report, 'Differential Variability', txt,level = level)
  
  diffSitesRankCut <- c(100,1000,10000,100000)
  comps <- get.comparisons(diff.meth)

  logger.start("Selection of rank cutoffs")
  rank.cuts.auto <- lapply(1:length(comps),FUN=function(i){
    comp.name <- names(comps)[i]
    comp <- comps[[i]]
    var.table <- get.table(diff.meth,comp,region.type="sites",return.data.frame=TRUE)
    res <- auto.select.rank.cut(var.table$diffVar.p.adj.fdr,var.table$combinedRank.var,alpha=0.1)
    return(as.integer(res))
  })
  rank.cuts.auto.meth <- lapply(1:length(comps),FUN=function(i){
    comp.name <- names(comps)[i]
    comp <- comps[[i]]
    var.table <- get.table(diff.meth,comp,region.type="sites",return.data.frame=TRUE)
    res <- auto.select.rank.cut(var.table$diffmeth.p.adj.fdr,var.table$combinedRank,alpha=0.1)
    return(as.integer(res))
  })
  txt <- paste("The following rank cutoffs have been automatically selected for the analysis of differentially",
               "variable sites:")
  rnb.add.paragraph(report, txt)
  
  tt <- data.frame(matrix(unlist(rank.cuts.auto),ncol=1,nrow=length(comps),byrow=TRUE))
  colnames(tt) <- "Rank Cutoff"
  rownames(tt) <- comps
  rnb.add.table(report,tt)
  logger.completed()
  
  #scatterplots
  logger.start("Adding scatterplots")
  rnb.cleanMem()
  plots <- list()
  if(parallel.isEnabled()){
    plots <- foreach(i=1:length(get.comparisons(diff.meth)),.combine="c") %dopar% {
      comp.name <- names(get.comparisons(diff.meth))[i]
      comp <- get.comparisons(diff.meth)[comp.name]
      var.table <- get.table(diff.meth,comp,region.type="sites",return.data.frame=TRUE)
      comp.name.allowed <- ifelse(is.valid.fname(comp.name),comp.name,paste("cmp",i,sep=""))
      grp.names <- get.comparison.grouplabels(diff.meth)[comp,]
      selected.cutoff <- rank.cuts.auto[[i]]
      res <- addReportPlot.diffVar.scatter.site(report,var.table,comp.name.allowed,
                                                      rank.cutoffs.numbers=rank.cutoffs.numbers,
                                                      auto.cutoff=selected.cutoff, rerank = TRUE,
                                                      group.name1=grp.names[1],group.name2=grp.names[2])
      rnb.cleanMem()
      res
    }
  } else {
    for (i in 1:length(get.comparisons(diff.meth))){
      comp.name <- names(get.comparisons(diff.meth))[i]
      comp <- get.comparisons(diff.meth)[comp.name]
      var.table <- get.table(diff.meth,comp,region.type="sites",return.data.frame=TRUE)
      comp.name.allowed <- ifelse(is.valid.fname(comp.name),comp.name,paste("cmp",i,sep=""))
      grp.names <- get.comparison.grouplabels(diff.meth)[comp,]
      selected.cutoff <- rank.cuts.auto[[i]]
      res <- addReportPlot.diffVar.scatter.site(report,var.table,comp.name.allowed,
                                                rank.cutoffs.numbers=rank.cutoffs.numbers,
                                                auto.cutoff=selected.cutoff,
                                           group.name1=grp.names[1],group.name2=grp.names[2])
      rnb.cleanMem()
      plots <- c(plots,res)
    }
  }
  
  comps <- get.comparisons(diff.meth)
  diff.var.type <- c(paste("FDR adjusted p-value &lt;",P.VAL.CUT),
                     paste("combined rank among the",rank.cutoffs.numbers,"best ranking sites"),
                     "automatically selected rank cutoff")
  names(diff.var.type) <-  c("fdrAdjPval",paste0("rc",1:length(rank.cutoffs.numbers)),"rcAuto")
  setting.names <- list(
    'comparison' = comps,
    'differential variability measure' = diff.var.type)
  description <- c('Scatterplot for differential variable sites.',
                   ' The transparency corresponds to point density. If the number of points exceeds ',DENS.SCATTER.SUBSAMPLE.THRES,
                   ' then the number of points for density estimation is reduced to that number by random sampling.',
                   ' The',round(DENS.SCATTER.SPARSE.POINTS.PERC*100),
                   '% of the points in the sparsest populated plot regions are drawn explicitly (up to a maximum of ',DENS.SCATTER.SPARSE.POINTS.MAX,
                   " points).",
                   ' Additionally, the colored points represent differentially variable sites (according to the selected criterion).')
  report <- rnb.add.figure(report, description, plots, setting.names)
  logger.completed()
  
  #volcano plots
  logger.start("Adding volcano plots")
  rnb.cleanMem()
  plots <- list()
  if(parallel.isEnabled()){
    plots <- foreach(i=1:length(get.comparisons(diff.meth)),.combine="c") %dopar% {
      comp.name <- names(get.comparisons(diff.meth))[i]
      comp <- get.comparisons(diff.meth)[comp.name]
      var.table <- get.table(diff.meth,comp,region.type="sites",return.data.frame=TRUE)
      comp.name.allowed <- ifelse(is.valid.fname(comp.name),comp.name,paste("cmp",i,sep=""))
      grp.names <- get.comparison.grouplabels(diff.meth)[comp,]
      res <- addReportPlot.diffVar.volcano(report, var.table, comp.name.allowed,
                                                      group.name1=grp.names[1],group.name2=grp.names[2])
      rnb.cleanMem()
      res
    }
  } else {
    for (i in 1:length(get.comparisons(diff.meth))){
      comp.name <- names(get.comparisons(diff.meth))[i]
      comp <- get.comparisons(diff.meth)[comp.name]
      var.table <- get.table(diff.meth,comp,region.type="sites",return.data.frame=TRUE)
      comp.name.allowed <- comp.name
      grp.names <- get.comparison.grouplabels(diff.meth)[comp,]
      res <- addReportPlot.diffVar.volcano(report, var.table, comp.name.allowed,
                                           group.name1=grp.names[1],group.name2=grp.names[2])
      rnb.cleanMem()
      plots <- c(plots,res)
    }
  }
  
  signif.measure <- c("pVal"="p-value","pValAdj"="adjusted p-value")
  setting.names <- list(
    'comparison' = comps,
    'significance metric' = signif.measure)
  description <- 'Volcano plot for differential variable sites.'
  report <- rnb.add.figure(report, description, plots, setting.names)
  logger.completed()
  
  #Comparing differentially methylated sites and differentially variable
  logger.start("Comparing DMCs and DVCs")
  rnb.cleanMem()
  plots <- list()
  if(parallel.isEnabled()){
    plots <- foreach(i=1:length(get.comparisons(diff.meth)),.combine="c") %dopar% {
      comp.name <- names(get.comparisons(diff.meth))[i]
      comp <- get.comparisons(diff.meth)[comp.name]
      var.table <- get.table(diff.meth,comp,region.type="sites",return.data.frame=TRUE)
      comp.name.allowed <- ifelse(is.valid.fname(comp.name),comp.name,paste("cmp",i,sep=""))
      grp.names <- get.comparison.grouplabels(diff.meth)[comp,]
      auto.cutoff <- rank.cuts.auto[[i]]
      auto.cutoff.meth <- rank.cuts.auto.meth[[i]]
      res <- addReportPlot.diffVar.meth(report, var.table, comp.name.allowed,auto.diffVar = auto.cutoff, auto.diffMeth = auto.cutoff.meth,
                                           group.name1=grp.names[1],group.name2=grp.names[2])
      rnb.cleanMem()
      res
    }
  } else {
    for (i in 1:length(get.comparisons(diff.meth))){
      comp.name <- names(get.comparisons(diff.meth))[i]
      comp <- get.comparisons(diff.meth)[comp.name]
      var.table <- get.table(diff.meth,comp,region.type="sites",return.data.frame=TRUE)
      comp.name.allowed <- ifelse(is.valid.fname(comp.name),comp.name,paste("cmp",i,sep=""))
      grp.names <- get.comparison.grouplabels(diff.meth)[comp,]
      auto.cutoff <- rank.cuts.auto[[i]]
      auto.cutoff.meth <- rank.cuts.auto.meth[[i]]
      res <- addReportPlot.diffVar.meth(report, var.table, comp.name.allowed,auto.diffVar = auto.cutoff, auto.diffMeth = auto.cutoff.meth,
                                        group.name1=grp.names[1],group.name2=grp.names[2])
      rnb.cleanMem()
      plots <- c(plots,res)
    }
  }
  diffVarType = c(paste("combined rank among the ",diffSitesRankCut," best ranking sites",sep=""),
                  "automatically selected rank cutoff")
  names(diffVarType) = c(paste("rc",1:length(diffSitesRankCut),sep=""),"rcAuto")
  setting.names <- list(
    'comparison' = comps,
    'rankCutoff' = diffVarType)
  description <- c('Scatterplot comparing differentially methylated (DMCs) and variable sites (DVCs), as well as sites ',
  'that are both differentially methylated and variable. The dotted lines corrspond to the respective rank cutoffs used ',
  'to call a site differentially methylated/variable.')
  report <- rnb.add.figure(report, description, plots, setting.names)
  logger.completed()
  
  logger.completed()
  return(report)
}

#' rnb.section.diffVar.region
#'
#' Adds information for differentially variable regions to the report.
#' @author Michael Scherer
#' @param rnb.set Object of type \code{\linkS4class{RnBSet}} containing methylation information
#' @param diff.meth RnBDiffMeth object. See \code{\link{RnBDiffMeth-class}} for details.
#' @param report Report object to which the content is added
#' @param gzTable Flag indicating if tables should be gzipped
#' @param level Which level of section should be created. See \code{rnb.add.section}.
#' @return The modified report object
rnb.section.diffVar.region <- function(rnb.set,diff.meth,report,gzTable=FALSE,level=1){
  if (length(get.comparisons(diff.meth))<1){
    stop("no valid comparisons")
  }
  if (length(get.region.types(diff.meth))<1){
    stop("no valid region types")
  }

  skipSites <- !includes.sites(diff.meth)
  diffRegionRankCut <- c(100,500,1000)
  logger.start("Adding Region Level Information (Differential Variability)")

  sectionText <- c("Differential variability on the region level was computed similar to differential methylation, ", 
                   "but the mean of variances, ",
                   "the log-ratio of the quotient of variances as well as the p-values from the differentiality test were ",
                   "employed. Ranking was performed in line with the ranking of differential methylation.")
  report <- rnb.add.section(report, "Differential Variability", sectionText,level = level)
  
  comps <- get.comparisons(diff.meth)
  reg.types <- get.region.types(diff.meth)
  
  logger.start("Selection of rank cutoffs")
  rank.cuts.auto <- lapply(1:length(comps),FUN=function(i){
    lapply(1:length(reg.types),FUN=function(j){
      var.table <- get.table(diff.meth,comps[i],region.type=reg.types[j],return.data.frame=TRUE)
      res <- auto.select.rank.cut(var.table$comb.p.adj.var.fdr,var.table$combinedRank.var,alpha=0.1)
      return(as.integer(res))
    })
  })
  rank.cuts.auto.meth <- lapply(1:length(comps),FUN=function(i){
    lapply(1:length(reg.types),FUN=function(j){
      var.table <- get.table(diff.meth,comps[i],region.type=reg.types[j],return.data.frame=TRUE)
      res <- auto.select.rank.cut(var.table$comb.p.adj.fdr,var.table$combinedRank,alpha=0.1)
      return(as.integer(res))
    })
  })
  txt <- paste("The following rank cutoffs have been automatically selected for the analysis of differentially",
               "variable regions:")
  rnb.add.paragraph(report, txt)
  
  tt <- data.frame(matrix(unlist(rank.cuts.auto),ncol=length(reg.types),nrow=length(comps),byrow=TRUE))
  colnames(tt) <- reg.types
  rownames(tt) <- comps
  rnb.add.table(report,tt)
  logger.completed()
  
  #scatterplots
  logger.start("Adding scatterplots")
  rnb.cleanMem()
  addedPlots <- list()
  grp.labels <- get.comparison.grouplabels(diff.meth)
  if(parallel.isEnabled()){
    iis <- 1:length(comps)
    jjs <- 1:length(reg.types)
    pps <- expand.grid(iis,jjs)
    
    addedPlots <- foreach(k=1:nrow(pps),.combine="c") %dopar% {
      i <- pps[k,1]
      j <- pps[k,2]
      cc <- names(comps)[i]
      ccc <- comps[cc]
      ccn <- ifelse(is.valid.fname(cc),cc,paste("cmp",i,sep=""))
      rr <- reg.types[j]
      rrn <- ifelse(is.valid.fname(rr),rr,paste("reg",j,sep=""))
      auto.rank.cut <- rank.cuts.auto[[i]][[j]]
      var.table <- get.table(diff.meth,ccc,rr,return.data.frame=TRUE)
      res <- addReportPlot.diffVar.scatter.region(report,var.table,comparison.name=ccn,region.name=rrn,
                                                  ranking.cutoffs = diffRegionRankCut, rerank=TRUE,
                                                  auto.cutoff = auto.rank.cut,group.name1 = grp.labels[ccc,1],group.name2 = grp.labels[ccc,2], useSiteCols=skipSites)
      rnb.cleanMem()
      res
    }
    
  } else {
    for (i in 1:length(comps)){
      cc <- names(comps)[i]
      ccc <- comps[cc]
      ccn <- ifelse(is.valid.fname(cc),cc,paste("cmp",i,sep=""))
      for (j in 1:length(reg.types)){
        rr <- reg.types[j]
        rrn <- ifelse(is.valid.fname(rr),rr,paste("reg",j,sep=""))
        auto.rank.cut <- rank.cuts.auto[[i]][[j]]
        var.table <- get.table(diff.meth,ccc,rr,return.data.frame=TRUE)
        addedPlots <- c(addedPlots,addReportPlot.diffVar.scatter.region(report,var.table,comparison.name=ccn,region.name=rrn,
                                                    ranking.cutoffs = diffRegionRankCut,
                                                    auto.cutoff = auto.rank.cut,group.name1 = grp.labels[ccc,1],group.name2 = grp.labels[ccc,2], useSiteCols=skipSites)
        )
       rnb.cleanMem()
      }
    }
  }
  
  names(reg.types) <- ifelse(is.valid.fname(reg.types),reg.types,paste("reg",1:length(reg.types),sep=""))
  diffVarType = c(paste("FDR adjusted p-value <",P.VAL.CUT),
                   paste("combined rank among the ",diffRegionRankCut," best ranking regions",sep=""),
                   "automatically selected rank cutoff")
  names(diffVarType) = c("fdrAdjPval",paste("rc",1:length(diffRegionRankCut),sep=""),"rcAuto")
  setting.names <- list(
    'comparison' = comps,
    'regions' = reg.types ,
    'differential variability measure' = diffVarType)
  description <- 'Scatterplot for differential variable regions. The transparency corresponds to point density. The 1% of the points in the sparsest populated plot regions are drawn explicitly.
  Additionally, the colored points represent differentially methylated regions (according to the selected criterion).'
  report <- rnb.add.figure(report, description, addedPlots, setting.names)
  logger.completed()
  
  #volcano plots
  logger.start("Adding volcano plots")
  rnb.cleanMem()
  addedPlots <- list()
  if(parallel.isEnabled()){
    #generate pairs of comparison, region combinations with indices
    iis <- 1:length(comps)
    jjs <- 1:length(reg.types)
    pps <- expand.grid(iis,jjs)

    addedPlots <- foreach(k=1:nrow(pps),.combine="c") %dopar% {
      i <- pps[k,1]
      j <- pps[k,2]
      cc <- names(comps)[i]
      ccc <- comps[cc]
      ccn <- ifelse(is.valid.fname(cc),cc,paste("cmp",i,sep=""))
      rr <- reg.types[j]
      rrn <- ifelse(is.valid.fname(rr),rr,paste("reg",j,sep=""))
      var.table <- get.table(diff.meth,ccc,region.type=rr,return.data.frame=TRUE)
      res <- addReportPlot.diffVar.volcano.region(report,var.table,comparison.name=ccn,region.type=rrn,
                                                        group.name1=grp.labels[ccc,1],group.name2=grp.labels[ccc,2], useSiteCols=skipSites)
      rnb.cleanMem()
      res
    }
  } else {
    for (i in 1:length(comps)){
      cc <- names(comps)[i]
      ccc <- comps[cc]
      ccn <- ifelse(is.valid.fname(cc),cc,paste("cmp",i,sep=""))
      for (j in 1:length(reg.types)){
        rr <- reg.types[j]
        rrn <- ifelse(is.valid.fname(rr),rr,paste("reg",j,sep=""))
      var.table <- get.table(diff.meth,ccc,region.type=rr,return.data.frame=TRUE)
      addedPlots <- c(addedPlots,addReportPlot.diffVar.volcano.region(report,var.table,comparison.name=ccn,region.type=rrn,
                                                        group.name1=grp.labels[ccc,1],group.name2=grp.labels[ccc,2], useSiteCols=skipSites)
      )
      rnb.cleanMem()
      }
    }
  }
  diff.measure <- c("diff"="Difference","quot"="Quotient")
  signif.measure <- c("pVal"="combined p-value","pValAdj"="adjusted combined p-value","quotSig"="Quotient (only meaningful if 'Difference' is selected above)")
  setting.names <- list(
    'comparison' = comps,
    'regions' = reg.types ,
    'difference metric' = diff.measure,
    'significance metric' = signif.measure)
  description <- 'Volcano plot for differential variability quantified by various metrics. Color scale according to
  combined ranking.'
  report <- rnb.add.figure(report, description, addedPlots, setting.names)
  logger.completed()
  
  #Comparing differentially methylated regions and differentially variable
  logger.start("Comparing DMRs and DVRs")
  rnb.cleanMem()
  plots <- list()
  if(parallel.isEnabled()){
    plots <- foreach(k=1:nrow(pps),.combine="c") %dopar% {
      i <- pps[k,1]
      j <- pps[k,2]
      comp.name <- names(comps)[i]
      comp <- comps[comp.name]
      ccn <- ifelse(is.valid.fname(comp.name),comp.name,paste("cmp",i,sep=""))
      rr <- reg.types[j]
      rrn <- ifelse(is.valid.fname(rr),rr,paste("reg",j,sep=""))
      var.table <- get.table(diff.meth,comp,region.type=rr,return.data.frame=TRUE)
      grp.names <- get.comparison.grouplabels(diff.meth)[comp,]
      auto.cutoff <- rank.cuts.auto[[i]][[j]]
      auto.cutoff.meth <- rank.cuts.auto.meth[[i]][[j]]
      res <- addReportPlot.diffVar.meth.region(report, var.table, ccn, region.type=rrn, auto.diffVar = auto.cutoff, auto.diffMeth = auto.cutoff.meth,
                                        group.name1=grp.names[1],group.name2=grp.names[2])
      rnb.cleanMem()
      res
    }
  } else {
    for (i in 1:length(comps)){
      cc <- names(comps)[i]
      ccc <- comps[cc]
      ccn <- ifelse(is.valid.fname(cc),cc,paste("cmp",i,sep=""))
      for (j in 1:length(reg.types)){
        rr <- reg.types[j]
        rrn <- ifelse(is.valid.fname(rr),rr,paste("reg",j,sep=""))
        var.table <- get.table(diff.meth,ccc,region.type=rr,return.data.frame=TRUE)
        grp.names <- get.comparison.grouplabels(diff.meth)[ccc,]
        auto.cutoff <- rank.cuts.auto[[i]][[j]]
        auto.cutoff.meth <- rank.cuts.auto.meth[[i]][[j]]
        plots <- c(plots,addReportPlot.diffVar.meth.region(report,var.table,ccn,region.type=rrn,auto.diffVar = auto.cutoff, auto.diffMeth = auto.cutoff.meth,
                                                                        group.name1=grp.labels[ccc,1],group.name2=grp.labels[ccc,2])
        )
        rnb.cleanMem()
      }
    }
  }
  diffVarType = c(paste("combined rank among the ",diffRegionRankCut," best ranking regions",sep=""),
                  "automatically selected rank cutoff")
  names(diffVarType) = c(paste("rc",1:length(diffRegionRankCut),sep=""),"rcAuto")
  setting.names <- list(
    'comparison' = comps,
    'regions' = reg.types,
    'rankCutoff' = diffVarType)
  description <- c('Scatterplot comparing differentially methylated (DMRs) and variable regions (DVRs), as well as regions ',
  'that are both differentially methylated and variable. The dotted lines corrspond to the respective rank cutoffs used ',
  'to call a region differentially methylated/variable.')
  report <- rnb.add.figure(report, description, plots, setting.names)
  

  logger.completed()
  return(report)
}

#' cols.to.rank.site
#' 
#' Return a matrix containing the negative absolute values of the information used to rank the sites. Those are currently:
#' the variance difference, the log ratio in variances and the p-value from the statistical test.
#' 
#' @param diff.var A differential variability table.
#' @return A matrix with the absolute values of the relevant columns
#' @author Michael Scherer
#' @rdname cols.to.rank
#' @aliases cols.to.rank.site
#' @aliases cols.to.rank.region
cols.to.rank.site <- function(diff.var){
  return(cbind(-abs(diff.var$var.diff), -abs(diff.var$var.log.ratio), diff.var$diffVar.p.val))
}
#' @rdname cols.to.rank
cols.to.rank.region <- function(diff.var){
  return(cbind(-abs(diff.var$mean.var.diff), -abs(diff.var$mean.var.log.ratio), diff.var$comb.p.adj.var.fdr))
}

#' computeDiffVar.bin.site
#' 
#' This function computes the table of differentially variable sites with the corresponding statistics.
#' 
#' @param X methylation matrix with sites as rows and samples as columns
#' @param inds.g1 the column indicies of the first group
#' @param inds.g2 the column indicies of the second group
#' @param variability.method method to use for calculating p-values. One of `diffVar` or `iEVORA`
#' @param paired Should paired analysis be conducted? If yes, the first entry in \code{inds.g1} should correspond to the first
#'        in \code{indg.g2} and so on.
#' @param eps Epsilon for computing quotients (avoid division by 0 by adding this value to denominator and enumerator before calculating the quotient)
#' @param adjustment.table Table specifying the variables to be adjusted for in the analysis.
#' @noRd
computeDiffVar.bin.site <- function(X,inds.g1,inds.g2,
                                    variability.method=rnb.getOption("differential.variability.method"),
                                    paired=FALSE, adjustment.table=NULL, eps=0.01){
  p.vals.var <- rep(as.double(NA),nrow(X))
  do.p.vals <- ncol(X[,inds.g1]) > 1 || ncol(X[inds.g2]) > 1
  if (do.p.vals) {
    if (variability.method == "diffVar"){
      logger.info("Conducting differential variability using diffVar")
      tryCatch(
        p.vals.var <- diffVar(X,inds.g1,inds.g2,adjustment.table=adjustment.table,paired = paired),
        error = function(ee) {
          logger.warning(c("Could not compute p-values using diffVar:",ee$message))
          logger.completed()
        }
      )
    } else if (variability.method == "iEVORA"){
      if(paired){
        logger.warning("Cannot conduct paired variability analysis with iEVORA, changing to diffVar.")
        rnb.options("differential.variability.method"="diffVar")
        tryCatch(
          p.vals.var <- diffVar(X,inds.g1,inds.g2,adjustment.table=adjustment.table,paired=paired),
          error = function(ee) {
            logger.warning(c("Could not compute p-values using diffVar:",ee$message))
            logger.completed()
          }
        )
      }else{
        tryCatch(
          p.vals.var <- apply.iEVORA(X,inds.g1,inds.g2),
          error = function(ee) {
            logger.warning(c("Could not compute p-values using iEVORA:",ee$message))
            logger.completed()
          }
        )
      }
    }
  } else {
    logger.warning("Skipping p-value computation due to insufficient sample numbers")
  }
  p.vals.is.na <- is.na(p.vals.var)
  if (!all(p.vals.is.na)){
    if (any(p.vals.is.na)){
      logger.info(c(sum(p.vals.is.na),"p-values are NA. They are treated as 1 in FDR adjustment"))
      p.vals.var[is.na(p.vals.t.na.adj)] <- 1
    }
    p.vals.var.adj <- p.adjust(p.vals.var, method = "fdr")
  } else {
    p.vals.var.adj <- rep(NA,length(p.vals.var))
  }
  tab.g1 <- X[,inds.g1,drop=FALSE]
  tab.g2 <- X[,inds.g2,drop=FALSE]
  if(length(inds.g1)<2) {
    logger.info("Group 1 has less than 2 members")
    tab.g1 <- as.matrix(tab.g1)
  }
  if(length(inds.g2)<2) {
    logger.info("Group 2 has less than 2 members")
    tab.g2 <- as.matrix(tab.g2)
  }
  
  var.g1 <- apply(tab.g1,1,var)
  var.g2 <- apply(tab.g2,1,var)
  if(paired){
    var.diff <- apply(tab.g1 - tab.g2,1,var)
    var.log.ratio <- apply(X,1,function(x){
      log2(var(x[inds.g1]+eps)/var(x[inds.g2]+eps))
    })
  }else{
    var.diff <- var.g1-var.g2
    var.log.ratio <- ifelse(var.g1==0|var.g2==0,1,log2(var.g1/var.g2))
  }
  neg.log10.p <- -log10(p.vals.var)
  neg.log10.fdr <- -log10(p.vals.var.adj)
  tt <- data.frame(var.g1=var.g1,var.g2=var.g2,var.diff=var.diff,var.log.ratio=var.log.ratio,diffVar.p.val=p.vals.var,diffVar.p.adj.fdr=p.vals.var.adj,log10P=neg.log10.p,
                   log10FDR=neg.log10.fdr)
}

#' computeDiffVar.default.region
#' @noRd
computeDiffVar.default.region <- function(dmtp,regions2sites){
  n.regs.with.sites <- length(regions2sites)
  col.id.var.g1 <- "var.g1"
  col.id.var.g2 <- "var.g2"
  col.id.diff.var <- "var.diff"
  col.id.quot.var <- "var.log.ratio"
  col.id.p.var <- "diffVar.p.val"
  mean.var.g1 <- rep(NA,n.regs.with.sites)
  mean.var.g2 <- rep(NA,n.regs.with.sites)
  diff.var <- rep(NA,n.regs.with.sites)
  quot.var <- rep(NA,n.regs.with.sites)
  p.vals.var <- rep(NA,n.regs.with.sites)
  col.vec <- c(col.id.var.g1, col.id.var.g2, col.id.diff.var, col.id.quot.var, col.id.p.var)
  
  dmt4fastProc <- dmtp[,col.vec]
  if(parallel.isEnabled()){
    dm <- foreach(i=1:n.regs.with.sites, .combine='rbind',.multicombine=TRUE,.maxcombine=200) %dopar% {
      pids <- regions2sites[[i]]
      subtab <- dmt4fastProc[pids,]#these lookups take up most of the time
      
      mean.var.g1   <- mean(subtab[,1],na.rm=TRUE)
      mean.var.g2   <- mean(subtab[,2],na.rm=TRUE)
      diff.var      <- mean(subtab[,3],na.rm=TRUE)
      quot.var      <- mean(subtab[,4],na.rm=TRUE)
      res <- combineTestPvalsMeth(na.omit(subtab[,5]),correlated=TRUE)
      p.vals.var <- NA
      if (length(res)>0) p.vals.var <- res
      c(mean.var.g1,mean.var.g2,diff.var,quot.var,p.vals.var)
    }
    mean.var.g1                  <- dm[,1]
    mean.var.g2                  <- dm[,2]
    diff.var                     <- dm[,3]
    quot.var                     <- dm[,4]
    p.vals.var                  <- dm[,5]
    } else {
      dummy <- sapply(1:n.regs.with.sites,FUN=function(i){
        pids <- regions2sites[[i]]
        subtab <- dmt4fastProc[pids,]#these lookups take up most of the time
        
        mean.var.g1[i] <<- mean(subtab[,1],na.rm=TRUE)
        mean.var.g2[i] <<- mean(subtab[,2],na.rm=TRUE)
        diff.var[i] <<- mean(subtab[,3],na.rm=TRUE)
        quot.var[i] <<- mean(subtab[,4],na.rm=TRUE)
        res <- combineTestPvalsMeth(na.omit(subtab[,5]),correlated=TRUE)
        if (length(res)>0) p.vals.var[i]  <<- res
        return(TRUE)
      })
    }
  p.vals.var.na.adj <- p.vals.var
  p.vals.var.is.na <- is.na(p.vals.var)
  if (any(p.vals.var.is.na)){
    logger.info(c(sum(p.vals.var.is.na),"p-values are NA. They are treated as 1 in FDR adjustment"))
    p.vals.var.na.adj[is.na(p.vals.var.na.adj)] <- 1
  }
  p.vals.var.adj <- p.adjust(p.vals.var.na.adj, method = "fdr")
  tt <- cbind(data.frame(mean.var.g1=mean.var.g1,mean.var.g2=mean.var.g2,mean.var.diff=diff.var,
                            mean.var.log.ratio=quot.var,comb.p.val.var=p.vals.var,comb.p.adj.var.fdr=p.vals.var.adj))
  return(tt)
}

#' computeDiffVar.bin.region
#' @noRd
computeDiffVar.bin.region <- function(rnbSet,dmtp,inds.g1,inds.g2,region.types=rnb.region.types(assembly(rnbSet)), ...){
  if (length(union(inds.g1,inds.g2)) != (length(inds.g1)+length(inds.g2))){
    logger.error("Overlapping sample sets in differential methylation analysis")
  }
  logger.start('Computing Differential Variability Tables (Region Level)')

  diffmeth.tabs <- list()
  for (rt in region.types){
    regions2sites <- regionMapping(rnbSet,rt)
    regions2sites.is.all.na <- sapply(regions2sites,FUN=function(x){all(is.na(x))})
    if (any(regions2sites.is.all.na)) {
      stop(paste("Region mapping of RnBSet from sites to regions is inconsistent (",rt,")"))
    }
    dmtr <- computeDiffVar.default.region(dmtp,regions2sites)
    diff.var.ranks <- cols.to.rank.region(dmtr)
    comb.rank.var <- combinedRanking.tab(diff.var.ranks,rerank=FALSE)
    dmtr$combinedRank.var <- comb.rank.var
    
    diffmeth.tabs <- c(diffmeth.tabs,list(dmtr))
    logger.status(c("Computed table for", rt))
  }
  names(diffmeth.tabs) <- region.types
  logger.completed()
  return(diffmeth.tabs)
}

########################################################################################################################
## normalizationMethods.R
## created: 2021-02-04
## creator: Pavlo Lutsik
## ---------------------------------------------------------------------------------------------------------------------
## Implementation of the normalization methods.
########################################################################################################################

## F U N C T I O N S ###################################################################################################

# reimplementation of the basic scaling normalization method for dye bias correction
#
# method='reference', default Illumina scaling normalization to a reference sample, taken from methylumi
# method='internal', singe-sample correlction, taken from minfi
#

rnb.norm.scaling<-function(rnb.set, method=c("internal", "reference")[1L], ref.sample=NULL){
    
    if(!method %in% c("reference", "internal")){
        rnb.error("Unsupported value for method")
    }
    
    qc_int<-qc(rnb.set)
    ctrl_target<-gsub("probes", "controls", rnb.set@target)
    qc_annot<-rnb.get.annotation(ctrl_target, rnb.set@assembly)
    annot<-annotation(rnb.set)  
    
    channels<-c("Cy5", "Cy3")
    tokens<-get.platform.tokens(rnb.set@target)
    
    get_norm_int<-function(ch){
        ids<-qc_annot[[tokens$id_col]][grep(tokens$norm_token[ch], qc_annot[[tokens$trg_col]])]
        ids<-as.character(ids)
        qc_int[[ch]][ids,]
    }
    
    norm_int_list<-lapply(channels, get_norm_int)
    names(norm_int_list)<-channels
    
    #taken from methylumi
    Grn.avg <- colMeans(norm_int_list[["Cy3"]])
    Red.avg <- colMeans(norm_int_list[["Cy5"]])
    R.G.ratio = Red.avg/Grn.avg
    if(method=="reference"){
        if(is.null(ref.sample)) ref.sample <- which.min( abs(R.G.ratio-1) )
        rnb.info(paste('Using sample number', ref.sample, 'as reference level...'))
        
        ref <- (Grn.avg + Red.avg)[ref.sample]/2
        if(is.na(ref)) rnb.error("'reference' refers to an array that is not present")
        Grn.factor <- ref/Grn.avg
        Red.factor <- ref/Red.avg
    }else if(method=="internal"){
        Grn.factor <- rep(1, length(R.G.ratio))
        Red.factor <- 1/R.G.ratio
    }
    
    probe.categories<-INTENSITY.SUMMARIZATION.INFO[[rnb.set@target]]

    if(!is.null(rnb.set@status) && rnb.set@status$disk.dump){
        M_temp<-convert.to.ff.matrix.tmp(matrix(NA_real_, nrow=dim(rnb.set@M)[1], ncol=dim(rnb.set@M)[2]))
        U_temp<-convert.to.ff.matrix.tmp(matrix(NA_real_, nrow=dim(rnb.set@U)[1], ncol=dim(rnb.set@U)[2]))
        M0_temp<-convert.to.ff.matrix.tmp(matrix(NA_real_, nrow=dim(rnb.set@M0)[1], ncol=dim(rnb.set@M0)[2]))
        U0_temp<-convert.to.ff.matrix.tmp(matrix(NA_real_, nrow=dim(rnb.set@U0)[1], ncol=dim(rnb.set@U0)[2]))
    }else{
        M_temp<-matrix(NA_real_, nrow=nrow(rnb.set@M), ncol=ncol(rnb.set@M))
        U_temp<-matrix(NA_real_, nrow=nrow(rnb.set@U), ncol=ncol(rnb.set@U))
        M0_temp<-matrix(NA_real_, nrow=nrow(rnb.set@M0), ncol=ncol(rnb.set@M0))
        U0_temp<-matrix(NA_real_, nrow=nrow(rnb.set@U0), ncol=ncol(rnb.set@U0))
    }
    
    for(pc in probe.categories){
        
        if(pc$Color=="Both"){
            M.factor<-Grn.factor
            U.factor<-Red.factor
            M0.factor<-Red.factor
            U0.factor<-Grn.factor
        }else if(pc$Color=="Red"){
            M.factor<-Red.factor
            U.factor<-Red.factor
            M0.factor<-Grn.factor
            U0.factor<-Grn.factor
        }else if(pc$Color=="Grn"){
            M.factor<-Grn.factor
            U.factor<-Grn.factor
            M0.factor<-Red.factor
            U0.factor<-Red.factor
        }
        indices<-which(annot$Design==pc$Design & annot$Color==pc$Color)
        
        M_temp[indices,]<-sweep(rnb.set@M[indices,], 2, FUN="*", M.factor)
        U_temp[indices,]<-sweep(rnb.set@U[indices,], 2, FUN="*", U.factor)
        M0_temp[indices,]<-sweep(rnb.set@M0[indices,], 2, FUN="*", M0.factor)
        U0_temp[indices,]<-sweep(rnb.set@U0[indices,], 2, FUN="*", U0.factor)
        
    }
    
    if(!is.null(rnb.set@status) && rnb.set@status$disk.dump && isTRUE(rnb.set@status$discard.ff.matrices)){
        delete(rnb.set@M)
        delete(rnb.set@U)
        delete(rnb.set@M0)
        delete(rnb.set@U0)
    }
    
    rnb.set@M<-M_temp
    rnb.set@U<-U_temp
    rnb.set@M0<-M0_temp
    rnb.set@U0<-U0_temp
    
    if(!is.null(rnb.set@status) && rnb.set@status$disk.dump){
        rm(M_temp); rm(U_temp); rm(M0_temp); rm(U0_temp); rnb.cleanMem()
    }
    
    rnb.set<-update.meth(rnb.set)
    
    return(rnb.set)
}



# reimplementation of the basic subtraction  normalization method

rnb.bgcorr.subtr<-function(rnb.set, bg.quant=0.05, offset=0){
    
    qc_int<-qc(rnb.set)
    ctrl_target<-gsub("probes", "controls", rnb.set@target)
    qc_annot<-rnb.get.annotation(ctrl_target, rnb.set@assembly)
    annot<-annotation(rnb.set)
    
    channels<-c("Cy5", "Cy3")
    tokens<-get.platform.tokens(rnb.set@target)
    
    get_bg_int<-function(ch){
        ids<-qc_annot[[tokens$id_col]][grep(tokens$bg_token, qc_annot[[tokens$trg_col]])]
        ids<-as.character(ids)
        qc_int[[ch]][ids,]
    }
    
    bg_int_list<-lapply(channels, get_bg_int)
    names(bg_int_list)<-channels
    
    bg<-list()
    for(ch in channels) {
        bg[[ch]]<-colQuantiles(bg_int_list[[ch]], probs=bg.quant)
    }
    
    probe.categories<-INTENSITY.SUMMARIZATION.INFO[[rnb.set@target]]
    
    if(!is.null(rnb.set@status) && rnb.set@status$disk.dump){
        M_temp<-convert.to.ff.matrix.tmp(matrix(NA_real_, nrow=dim(rnb.set@M)[1], ncol=dim(rnb.set@M)[2]))
        U_temp<-convert.to.ff.matrix.tmp(matrix(NA_real_, nrow=dim(rnb.set@U)[1], ncol=dim(rnb.set@U)[2]))
    }else{
        M_temp<-matrix(NA_real_, nrow=nrow(rnb.set@M), ncol=ncol(rnb.set@M))
        U_temp<-matrix(NA_real_, nrow=nrow(rnb.set@U), ncol=ncol(rnb.set@U))
    }
    
    for(pc in probe.categories){
        
        if(pc$Color=="Both"){
            bg.M<-bg[["Cy5"]]
            bg.U<-bg[["Cy3"]]
        }else if(pc$Color=="Red"){
            bg.M<-bg[["Cy3"]]
            bg.U<-bg[["Cy3"]]
        }else if(pc$Color=="Grn"){
            bg.M<-bg[["Cy5"]]
            bg.U<-bg[["Cy5"]]
        }
        indices<-which(annot$Design==pc$Design & annot$Color==pc$Color)
        for(i in 1:ncol(rnb.set@M)) {
            M_temp[indices,i] <- pmax(rnb.set@M[indices,i] - bg.M[i], 1)
            U_temp[indices,i] <- pmax(rnb.set@U[indices,i] - bg.U[i], 1)
        }
    }
    
    if(!is.null(rnb.set@status) && rnb.set@status$disk.dump && isTRUE(rnb.set@status$discard.ff.matrices)){
        delete(rnb.set@M)
        delete(rnb.set@U)
    }
    
    rnb.set@M<-M_temp
    rnb.set@U<-U_temp
    
    if(!is.null(rnb.set@status) && rnb.set@status$disk.dump){
        rm(M_temp); rm(U_temp); rnb.cleanMem()
    }
    
    rnb.set<-update.meth(rnb.set)
    
    return(rnb.set)
    
}

#
# NOOB method as implemented in SeSAME
#
noob.sesame <- function(rnb.set, background=c("internal", "bleed-through")[2L], offset=15) {
    
    annot<-annotation(rnb.set)
    
    ## if no Infinium-I probes
    #if (nrow(InfIG(sdf)) == 0 && nrow(InfIR(sdf)) == 0) { return(sdf) }
    
    ##################### GRN CHANNEL ESTIMATION
    ## background
    oobG <- rbind(
            rnb.set@M0[annot$Design=="I" & annot$Color=="Red",], 
            rnb.set@U0[annot$Design=="I" & annot$Color=="Red",])
    oobG[is.na(oobG)] <-0
    #oobG[oobG == 0] <- 1
    
    ## if not enough out-of-band signal
    if (sum(oobG > 0, na.rm=TRUE) < 100) { return(rnb.set) }
    
    ## foreground
    #ibG = c(InfIG(sdf)$MG, InfIG(sdf)$UG, InfII(sdf)$UG)
    ibG <- rbind(
            rnb.set@M[annot$Design=="I" & annot$Color=="Grn",], 
            rnb.set@U[annot$Design=="I" & annot$Color=="Grn",]
            )

    if(background == "internal") ibG <- rbind(ibG, rnb.set@M[annot$Design=="II",])

    ibG[is.na(ibG)] <- 0
    #ibG[ibG == 0] <- 1 # set signal to 1 if 0
    
   
    ##################### RED CHANNEL ESTIMATION
    oobR <- rbind(
            rnb.set@M0[annot$Design=="I" & annot$Color=="Grn",], 
            rnb.set@U0[annot$Design=="I" & annot$Color=="Grn",])
    oobR[is.na(oobR)] <-0
    #oobR[oobR == 0] <- 1
    
    if (sum(oobR > 0, na.rm=TRUE) < 100) { return(rnb.set) }
    
    ibR <- rbind(
            rnb.set@M[annot$Design=="I" & annot$Color=="Red",], 
            rnb.set@U[annot$Design=="I" & annot$Color=="Red",]
            )
    
    if(background == "internal") ibR <- rbind(ibR, rnb.set@U[annot$Design=="II",])
    
    ibR[is.na(ibR)] <- 0
    #ibR[ibR == 0] <- 1 # set signal to 1 if 0
    
    if(background=="internal"){
        
        #fitG = backgroundCorrectionNoobFit(ibG, oobG)
        oobG[oobG == 0] <- 1
        fitG <- sapply(seq(ncol(ibG)), function(i) backgroundCorrectionNoobFit(ibG[,i],oobG[,i]))
        
        #fitR = backgroundCorrectionNoobFit(ibR, oobR)
        oobR[oobR == 0] <- 1
        fitR <- sapply(seq(ncol(ibR)), function(i) backgroundCorrectionNoobFit(ibR[,i],oobR[,i]))
        
    }else if(background=="bleed-through"){
        
        ibR_other<-rbind(oobG, rnb.set@M[annot$Design=="II",])
        ibR_other[is.na(ibR_other)] <- 0
        
        ibG_other<-rbind(oobR, rnb.set@U[annot$Design=="II",])
        ibG_other[is.na(ibG_other)] <- 0
        
        oobR[oobR == 0] <- 1
        bg.GpredictR <- lapply(seq(ncol(ibG)), function(i) {f<-train.model.lm(ibG[,i], oobR[,i]); return(f)})
        ##ibR.other.channel
        #ibR_other[ibR_other==0] <- 1
        pp.bg.ibR<-sapply(seq(ncol(ibR_other)), function(i) bg.GpredictR[[i]](ibR_other[,i]))
        ibR_other<-NULL; rm(ibR_other)
        ##sset$IG
        pp.bg.oobR<-sapply(seq(ncol(ibG)), function(i) bg.GpredictR[[i]](ibG[,i]))
        
        ibRR<-rbind(ibR, rnb.set@U[annot$Design=="II",])
        ibRR[ibRR==0] <- 1
        alphaR<-sapply(seq(ncol(ibRR)), function(i) pmax(MASS::huber(ibRR[,i] - pp.bg.ibR[,i]$mu)$mu, offset-5))
        ibRR<-NULL; rm(ibRR)
        
        oobG[oobG == 0] <- 1
        bg.RpredictG <- lapply(seq(ncol(ibR)), function(i) {f<-train.model.lm(ibR[,i], oobG[,i]); return(f)})
        ##ibG.other.channel

        #ibG_other[ibG_other==0] <- 1
        pp.bg.ibG<-sapply(seq(ncol(ibG_other)), function(i) bg.RpredictG[[i]](ibG_other[,i]))
        ibG_other<-NULL; rm(ibG_other)
        ##sset$IR
        pp.bg.oobG<-sapply(seq(ncol(ibR)), function(i) bg.RpredictG[[i]](ibR[,i]))
        
        ibGG<-rbind(ibG, rnb.set@M[annot$Design=="II",])
        ibGG[ibGG==0] <- 1
        alphaG<-sapply(seq(ncol(ibGG)), function(i) pmax(MASS::huber(ibGG[,i] - pp.bg.ibG[,i]$mu)$mu, offset-5))
        ibGG<-NULL; rm(ibGG)

    }
    ####### CORRECTION
    tII.len<-sum(annot$Design=="II")
    mg.ind<-c(which(annot$Color=="Grn"), which(annot$Design=="II"))
    ug.ind<-which(annot$Color=="Grn" )
    
    mr.ind<-which(annot$Color=="Red" )
    ur.ind<-c(which(annot$Color=="Red"), which(annot$Design=="II"))
    
    slot_names<-c("M", "U", "M0", "U0")
    ind.lists<-list("Grn"=list(), "Red"=list())
    
    ind.lists[["Grn"]][["M"]]<-mg.ind
    ind.lists[["Grn"]][["U"]]<-ug.ind
    ind.lists[["Grn"]][["M0"]]<-mr.ind
    ind.lists[["Grn"]][["U0"]]<-mr.ind
    
    ind.lists[["Red"]][["M"]]<-mr.ind
    ind.lists[["Red"]][["U"]]<-ur.ind
    ind.lists[["Red"]][["M0"]]<-ug.ind
    ind.lists[["Red"]][["U0"]]<-ug.ind
    
    if(background=="internal"){
        
        fit_param<-list("Grn"=fitG, "Red"=fitR)
        
    }else if(background=="bleed-through"){
        
        ir<-expand.grid(seq(nrow(pp.bg.ibG)), seq(ncol(pp.bg.ibG)))
        
        pp.bg.ibG_M<-pp.bg.ibG
        for(ind in 1:nrow(ir)) pp.bg.ibG_M[[ir[ind,1],ir[ind,2]]]<-pp.bg.ibG_M[[ir[ind,1],ir[ind,2]]][c(seq(ug.ind),(2*length(ug.ind)+1):(2*length(ug.ind)+tII.len))]
        pp.bg.ibG_U<-pp.bg.ibG
        for(ind in 1:nrow(ir)) pp.bg.ibG_U[[ir[ind,1],ir[ind,2]]]<-pp.bg.ibG_U[[ir[ind,1],ir[ind,2]]][c((length(ug.ind)+1):(2*length(ug.ind)))]
        pp.bg.ibR_M<-pp.bg.ibR
        for(ind in 1:nrow(ir)) pp.bg.ibR_M[[ir[ind,1],ir[ind,2]]]<-pp.bg.ibR_M[[ir[ind,1],ir[ind,2]]][c(seq(mr.ind))]
        pp.bg.ibR_U<-pp.bg.ibR
        for(ind in 1:nrow(ir)) pp.bg.ibR_U[[ir[ind,1],ir[ind,2]]]<-pp.bg.ibR_U[[ir[ind,1],ir[ind,2]]][c((length(mr.ind)+1):(2*length(mr.ind)+tII.len))]

        pp.bg.oobG_M<-pp.bg.oobG
        for(ind in 1:nrow(ir)) pp.bg.oobG_M[[ir[ind,1],ir[ind,2]]]<-pp.bg.oobG_M[[ir[ind,1],ir[ind,2]]][seq(mr.ind)]
        pp.bg.oobG_U<-pp.bg.oobG
        for(ind in 1:nrow(ir)) pp.bg.oobG_U[[ir[ind,1],ir[ind,2]]]<-pp.bg.oobG_U[[ir[ind,1],ir[ind,2]]][(length(mr.ind)+1):(2*length(mr.ind))]
        pp.bg.oobR_M<-pp.bg.oobR
        for(ind in 1:nrow(ir)) pp.bg.oobR_M[[ir[ind,1],ir[ind,2]]]<-pp.bg.oobR_M[[ir[ind,1],ir[ind,2]]][seq(ug.ind)]
        pp.bg.oobR_U<-pp.bg.oobR
        for(ind in 1:nrow(ir)) pp.bg.oobR_U[[ir[ind,1],ir[ind,2]]]<-pp.bg.oobR_U[[ir[ind,1],ir[ind,2]]][(length(ug.ind)+1):(2*length(ug.ind))]
        
        fit_param<-list(
                "Grn"=list(
                        "M"=list(
                            "IB"=pp.bg.ibG_M,
                            "OB"=pp.bg.oobG_M
                        ),
                        "U"=list(
                             "IB"=pp.bg.ibG_U,
                             "OB"=pp.bg.oobG_U
                        ),
                        "alpha"=alphaG
                        ), 
                "Red"=list(
                        "M"=list(
                            "IB"=pp.bg.ibR_M,
                            "OB"=pp.bg.oobR_M
                        ),
                        "U"=list(
                            "IB"=pp.bg.ibR_U,
                            "OB"=pp.bg.oobR_U
                        ),
                        "alpha"=alphaR
                        )
        )
    }
    
    for(sl in slot_names){
        if(!is.null(rnb.set@status) && rnb.set@status$disk.dump){
            mat_temp<-convert.to.ff.matrix.tmp(matrix(NA_real_, nrow=dim(rnb.set@M)[1], ncol=dim(rnb.set@M)[2]))
        }else{
            mat_temp<-matrix(NA_real_, nrow=nrow(rnb.set@M), ncol=ncol(rnb.set@M))
        }
        for(col in c("Grn", "Red")){
            for(i in seq(ncol(rnb.set@M))){
                if(background=="internal"){
                    mat_temp[ind.lists[[col]][[sl]], i]<-normExpSignal(
                            fit_param[[col]][,i]$mu, 
                            fit_param[[col]][,i]$sigma, 
                            fit_param[[col]][,i]$alpha, 
                            slot(rnb.set,sl)[ind.lists[[col]][[sl]],i]) + offset
                }else{
                    token_mu<-gsub("0","",sl)
                    token_band<-c("IB", "OB")[1L+grepl("0", sl)]
                    input<-slot(rnb.set,sl)[ind.lists[[col]][[sl]],i]
                    input[is.na(input) | input == 0]<-1L
                    mat_temp[ind.lists[[col]][[sl]], i]<-backgroundCorrCh1(
                            fit_param[[col]][[token_mu]][[token_band]][,i],
                            fit_param[[col]]$alpha[i], 
                            input) + offset
                }
            }
        }
        if(!is.null(rnb.set@status) && rnb.set@status$disk.dump && isTRUE(rnb.set@status$discard.ff.matrices)){
            delete(slot(rnb.set,sl))
        }
        
        slot(rnb.set,sl)<-mat_temp
        
        if(!is.null(rnb.set@status) && rnb.set@status$disk.dump){
            rm(mat_temp); rnb.cleanMem()
        }
    }
    
    rnb.set<-update.meth(rnb.set)
    
    return(rnb.set)
}

backgroundCorrectionNoobFit <- function(ib, bg) {
    e = MASS::huber(bg)
    mu = e$mu
    sigma = e$s
    alpha = pmax(MASS::huber(ib)$mu-mu, 10)
    list(mu = mu, sigma = sigma, alpha = alpha)
}

# the following is adapted from Limma
## normal-exponential deconvolution (conditional expectation of
## xs|xf; WEHI code)
normExpSignal <- function(mu, sigma, alpha, x)  {
    
    sigma2 <- sigma * sigma
    
    if (alpha <= 0)
        stop("alpha must be positive")
    if (sigma <= 0)
        stop("sigma must be positive")
    
    mu.sf <- x - mu - sigma2/alpha
    signal <- mu.sf + sigma2 * exp(
            dnorm(0, mean = mu.sf, sd = sigma, log = TRUE) -
                    pnorm(
                            0, mean = mu.sf, sd = sigma,
                            lower.tail = FALSE, log.p = TRUE))
    
    o <- !is.na(signal)
    if (any(signal[o] < 0)) {
        warning("Limit of numerical accuracy reached with very
                        low intensity or very high background:\nsetting adjusted intensities
                        to small value")
        signal[o] <- pmax(signal[o], 1e-06)
    }
    signal
}

#####

train.model.lm <- function(input, output) {
    fitdata <- data.frame(IB=as.vector(input)+1, LOB=log(as.vector(output)+1))
    m <- lm(LOB~IB, data=fitdata)
    #m <- MASS::rlm(LOB~IB, data=fitdata)
    function(d) {
        force(m)
        pp <- predict(m, newdata=data.frame(IB=as.vector(d)), interval='prediction', level=0.8)
        # use upper bound for mu since true signal is often much higher than noise
        list(mu=exp(pp[,'fit']), sigma=(exp(pp[,'upr'])-exp(pp[,'lwr']))/10.13)
        # list(mu=exp(pp[,'upr']), sigma=log(exp(pp[,'upr'])-exp(pp[,'lwr'])))
    }
}

backgroundCorrCh1 <- function(pp, alpha, x) {
    mu.bg <- pp$mu
    sigma <- pp$sigma
    sigma2 <- sigma * sigma
    if (alpha <= 0)
        stop("alpha must be positive")
    if (any(na.omit(sigma) <= 0))
        stop("sigma must be positive")
    mu.sf <- x - mu.bg - sigma2/alpha
    signal <- mu.sf + sigma2 * exp(
            dnorm(0, mean = mu.sf, sd = sigma, log = TRUE) -
                    pnorm(0, mean = mu.sf, sd = sigma, lower.tail = FALSE, log.p = TRUE))
    o <- !is.na(signal)
    if (any(signal[o] < 0)) {
        warning("Limit of numerical accuracy reached with very low intensity or very high background:\nsetting adjusted intensities to small value")
        signal[o] <- pmax(signal[o], 1e-06)
    }
    signal.min <- min(signal, na.rm = TRUE)
    signal <- signal - signal.min
    signal
}

get.platform.tokens<-function(platform){
    
    channels<-c("Cy5", "Cy3")
    token<-setNames(rep(NA_character_, 2), channels)
    
    dict<-list()
    
    if(platform=="probes27"){
        dict$norm_token["Cy3"] <- 'Norm.G'
        dict$norm_token["Cy5"] <- 'Norm.R'
        dict$bg_token<-"Negative"
        dict$id_col<-"ID"
        dict$trg_col<-"Target"
    } else if(platform=="probes450"){
        dict$norm_token["Cy3"] <- 'NORM_(C|G)'
        dict$norm_token["Cy5"] <- 'NORM_(A|T)'
        dict$bg_token<-"NEGATIVE"
        dict$id_col<-"ID"
        dict$trg_col<-"Target"
    } else if(platform=="probesEPIC"){
        dict$norm_token["Cy3"] <- 'NORM_(C|G)'
        dict$norm_token["Cy5"] <- 'NORM_(A|T)'
        dict$bg_token<-"NEGATIVE"
        dict$id_col<-"ID"
        dict$trg_col<-"Target"
    } else if(platform=="probesEPICv2"){
        dict$norm_token["Cy3"] <- 'NORM_(C|G)'
        dict$norm_token["Cy5"] <- 'NORM_(A|T)'
        dict$bg_token<-"NEGATIVE"
        dict$id_col<-"ID"
        dict$trg_col<-"Target"
    } else if(platform=="probesMMBC"){
        dict$norm_token["Cy3"] <- 'NORM_(C|G)'
        dict$norm_token["Cy5"] <- 'NORM_(A|T)'
        dict$bg_token<-"NEGATIVE"
        dict$id_col<-"ID"
        dict$trg_col<-"Target"
    }
    
    return(dict)
}

################################################################################
# Cluster Architecture Descriptions
################################################################################
################################################################################
# Concrete implementations for the SLURM environment
################################################################################

#' ClusterArchitectureSLURM Class
#'
#' A child class of \code{\linkS4class{ClusterArchitecture}} implementing specifications of Simple Linux Utility for Resource Management (SLURM) architectures.
#'
#' @details
#' Follow this template if you want to create your own ClusterArchitecture class.
#'
#' @section Slots:
#' see \code{\linkS4class{ClusterArchitecture}}
#'
#' @section Methods:
#' \describe{
#'   \item{\code{\link{getSubCmdTokens,ClusterArchitectureSGE-method}}}{Returns a vector of command line tokens corresponding to submitting
#'   a job with the given command to the cluster}
#' }
#'
#' @name ClusterArchitectureSLURM-class
#' @rdname ClusterArchitectureSLURM-class
#' @author Michael Scherer
#' @exportClass ClusterArchitecture
setClass("ClusterArchitectureSLURM",
	contains = "ClusterArchitecture"
)

#' initialize.ClusterArchitectureSLURM
#'
#' Initialize an ClusterArchitecture object for a SLURM
#' 
#' @param .Object New instance of \code{ClusterArchitectureSLURM}.
#' @param name A name or identifier
#' @param ... arguments passed on to the constructor of \code{\linkS4class{ClusterArchitecture}} (the parent class)
#'
#' @export
#' @author Michael Scherer
#' @docType methods
setMethod("initialize","ClusterArchitectureSLURM",
	function(
		.Object,
		name="ClusterArchitectureSLURM",
		...
	) {
		.Object <- callNextMethod(.Object=.Object, name=name, ...)
		.Object <- setExecutable(.Object,"R","R")
		.Object <- setExecutable(.Object,"Rscript","Rscript")
		.Object <- setExecutable(.Object,"python","python")
		.Object@getSubCmdTokens.optional.args <- c("sub.binary","quote.cmd")
		.Object
	}
)

#' getSubCmdTokens-methods
#'
#' Returns a string for the of command line corresponding to submitting
#' a job with the given command to the cluster.
#' @details
#' For a concrete child class implementation for a SLURM architecture specification see \code{\linkS4class{ClusterArchitectureSLURM}}
#'
#' @param object \code{\linkS4class{ClusterArchitectureSLURM}} object
#' @param cmd.tokens a character vector specifying the executable command that should be wrapped in the cluster submission command
#' @param log file name and path of the log file that the submitted job writes to
#' @param job.name name of the submitted job
#' @param res.req named vector of requested resources. Two options are available: \code{"clock.limit"} and \code{"memory.size"}
#' @param sub.binary flag indicating if the command is to be submitted using the \code{"wrap"} option of SLURM
#' @param depend.jobs character vector containg names or ids of jobs the submitted job will depend on.
#' @param quote.cmd Flag indicating whether the submitted cammed should also be wrapped in quotes
#' @return A character vector containing the submission command tokens
#'
#' @rdname getSubCmdTokens-ClusterArchitectureSLURM-methods
#' @docType methods
#' @aliases getSubCmdTokens,ClusterArchitectureSLURM-method
#' @author Michael Scherer
#' @export
#' @examples
#' \donttest{
#' arch <- new("ClusterArchitectureSLURM",
#' 	name="my_slurm_architecture"
#' )
#' getSubCmdTokens(arch,c("Rscript","my_great_script.R"),"my_logfile.log")
#' }
setMethod("getSubCmdTokens",
	signature(
		object="ClusterArchitectureSLURM"
	),
	function(
	  object,
	  cmd.tokens,
	  log,
	  job.name = "",
	  res.req = character(0),
	  depend.jobs = character(0),
	  sub.binary = TRUE,
	  quote.cmd = TRUE
	) {
	  res.req.token <- NULL
		if(length(res.req)>0){
		  if("clock.limit" %in% names(res.req)){
		    res.req.token <- paste(res.req.token,"-t",res.req["clock.limit"]," ",collapse = "")
		  }
		  if("mem.size" %in% names(res.req)){
		    res.req.token <- paste0(res.req.token,"--mem=",res.req["mem.size"],collapse="")
		    
		  }
		}
		log.token <- NULL
		if (nchar(log)>0) {
			log.token <- c("-o",log)
		}
		job.name.token <- NULL
		if (nchar(job.name)>0) {
			job.name.token <- paste0(job.name.token,"--job-name=",job.name,collapse = "")
		}
		dependency.token <- NULL
		if (length(depend.jobs)>0){
		  get.job.id <- function(x){
		    cmd <- paste0("echo $(squeue --noheader --format %i --name ",x,")")
		    system(cmd,intern = T)
		  }
		  depend.jobs <- sapply(depend.jobs,get.job.id)
			dependency.token <- paste0(dependency.token, "--depend=", paste0(paste(depend.jobs,collapse=",")),collapse = "")
		}
		wrap.token <- NULL
		if(sub.binary){
		  if (quote.cmd){
		    cmd.tokens <- paste0("--wrap=",paste0("'",paste(cmd.tokens,collapse=" "),"'"),collapse="")
		  }else{
		    cmd.tokens <- paste0("--wrap=",paste(cmd.tokens,collapse=" "),collapse="")
		  }
		}else{
		  if (quote.cmd){
		    cmd.tokens <- paste0("'",paste(cmd.tokens,collapse=" "),"'")
		  }else{
		    cmd.tokens <- paste(cmd.tokens,collapse=" ")
		  }
		}

		res <- c(
			"sbatch",
			"--export=ALL",
			res.req.token,
			log.token,
			job.name.token,
			dependency.token,
			wrap.token,
			cmd.tokens
		)
		return(res)
	}
)

########################################################################################################################
## RnBeadRawSet-class.R
## created: 2013-xx-xx
## creator: Pavlo Lutsik
## ---------------------------------------------------------------------------------------------------------------------
## RnBeadRawSet class definition.
########################################################################################################################

## ---------------------------------------------------------------------------------------------------------------------
## CLASS DEFINITIONS
## ---------------------------------------------------------------------------------------------------------------------

#' RnBeadRawSet-class 
#' 
#' Main class for storing HumanMethylation micorarray data which includes intensity information 
#' 
#' @section Slots:
#' \describe{
#'	\item{\code{pheno}}{Phenotypic data.}
#'	\item{\code{M}}{\code{matrix} of intensities for the probes measuring the abundance of methylated molecules.} 
#'	\item{\code{U}}{\code{matrix} of intensities for the probes measuring the abundance of unmethylated molecules.} 
#' 	\item{\code{M0}}{\code{matrix} of "out-of-band" intensities for the probes measuring the abundance of methylated molecules.} 
#' 	\item{\code{U0}}{\code{matrix} of "out-of-band" intensities for the probes measuring the abundance of unmethylated molecules.}
#' 	\item{\code{bead.counts.M}}{\code{matrix} of bead counts per probe.}
#' 	\item{\code{bead.counts.U}}{\code{matrix} of bead counts per probe.}
#' }
#' 
#' @section Methods and Functions:
#' \describe{
#'   \item{\code{samples}}{Gets the identifiers of all samples in the dataset.}
#'   \item{\code{\link[=M,RnBeadRawSet-method]{M}}}{Get the matrix of intensities for the probes measuring the abundance of methylated molecules.}
#'   \item{\code{\link[=U,RnBeadRawSet-method]{U}}}{Get the matrix of intensities for the probes measuring the abundance of unmethylated molecules.}
#'   \item{\code{\link{intensities.by.color}}}{Get probe intensities in each color channel.}
#' } 
#' 
#' @name RnBeadRawSet-class
#' @rdname RnBeadRawSet-class
#' @author Pavlo Lutsik
#' @exportClass RnBeadRawSet
#' @include RnBeadSet-class.R
setClass("RnBeadRawSet",
		representation(M="matrixOrffOrNULL",
				U="matrixOrffOrNULL",
				M0="matrixOrffOrNULL",
				U0="matrixOrffOrNULL",
				bead.counts.M="matrixOrffOrNULL",
				bead.counts.U="matrixOrffOrNULL"
		),
		contains="RnBeadSet",
		prototype(#pheno=data.frame(),
				#betas=matrix(),
				#meth.sites=matrix(),
				#pval.sites=NULL,
				#pval.regions=NULL,
				#qc=NULL, 
				#status=NULL,
				M=matrix(),
				U=matrix(),
				M0=NULL,
				U0=NULL,
				bead.counts.M=NULL,
				bead.counts.U=NULL
		),
		package = "RnBeads"
)

RNBRAWSET.SLOTNAMES<-c("M","U","M0","U0","bead.counts.M", "bead.counts.U")

########################################################################################################################

## initialize.RnBeadRawSet
##
## Direct slot filling.
## 
## #docType methods
## #rdname RnBeadRawSet-class
setMethod("initialize", "RnBeadRawSet",
		function(.Object,
				pheno = data.frame(),
				sites = matrix(ncol=0, nrow=0),
				meth.sites = matrix(ncol=0, nrow=0),
				M = matrix(ncol=0, nrow=0),
				U = matrix(ncol=0, nrow=0),
				M0 = NULL,
				U0 = NULL,
				bead.counts.M = NULL, 
				bead.counts.U = NULL,
				covg.sites = NULL,
				pval.sites = NULL,
				qc = NULL,
				target="probes450",
				status=list(normalized=FALSE, background=FALSE, disk.dump=FALSE)
				) {
			
			.Object@target<-target
					
			#.Object@pheno<-pheno
			#.Object@sites<-sites
			.Object@M<-M
			.Object@U<-U
			.Object@M0<-M0
			.Object@U0<-U0
			.Object@bead.counts.M<-bead.counts.M
			.Object@bead.counts.U<-bead.counts.U
			
			#.Object@meth.sites<-betas
			#.Object@pval.sites<-p.values
			#.Object@covg.sites<-bead.counts
			#.Object@regions<-list()
			
			#.Object@status<-list()
			
			#.Object@status[["normalized"]]<-"none"
			#.Object@status[["background"]]<-"none"
			#.Object@status[["disk.dump"]]<-useff
			
			#.Object@inferred.covariates <- list()
			
			#.Object@qc<-qc
			
			#.Object
			callNextMethod(.Object,
					pheno=pheno,
					sites=sites,
					meth.sites=meth.sites, 
					pval.sites=pval.sites,
					covg.sites=covg.sites,
					target=target,
					status=status,
					qc=qc)
		})

########################################################################################################################		

#' Wrapper function RnBeadRawSet
#'
#' @param pheno       		Phenotypic data.
#' @param probes			\code{character} vector of Infinium(R) probe identifiers
#' @param M       	  		Matrix of intensities for the probes measuring the abundance of methylated molecules 
#' @param U       	  		Matrix of intensities for the probes measuring the abundance of unmethylated molecules 
#' @param M0       	  		Matrix of "out-of-band" intensities for the probes measuring the abundance of methylated molecules 
#' @param U0       	  		Matrix of "out-of-band" intensities for the probes measuring the abundance of unmethylated molecules
#' @param bead.counts.M 	Matrix of bead counts per probe.
#' @param bead.counts.U 	Matrix of bead counts per probe.
#' @param p.values    		Matrix of detection p-values.
#' @param qc                ...
#' @param platform	   		\code{character} singleton specifying the microarray platform: \code{"450k"} corresponds to HumanMethylation450 microarray, and \code{"27k"} stands for HumanMethylation27.
#' @param region.types		A \code{character} vector specifying the region types, for which the methylation infromation will be summarized.
#' @param beta.offset		A regularization constant which is added to the denominator at beta-value calculation
#' @param summarize.bead.counts	If \code{TRUE} the coverage slot is filled by summarizing the \code{bead.counts.M} and \code{bead.counts.U} matrices. For type I probes the summarization is done using \code{min} operation, while for type II probes the bead counts should be identical in both supplied matrices
#' @param summarize.regions ...
#' @param useff		  		If \code{TRUE} the data matrices will be stored as \code{ff} objects
#' @param ffcleanup		  	If \code{TRUE} and disk dumping has been enabled the data of the input \code{ff} objects will be deleted 
#'
#' @return an object of class RnBeadRawSet
#' 
#' @name RnBeadRawSet
#' @rdname RnBeadRawSet-class
#' @aliases initialize,RnBeadRawSet-method
#' @export
RnBeadRawSet<-function(
		pheno,
		probes,
		M,
		U,
		M0=NULL,
		U0=NULL,
		bead.counts.M = NULL, 
		bead.counts.U = NULL,
		p.values=NULL,
		qc = NULL,
		platform = "450k",
		beta.offset=100,
		summarize.bead.counts=TRUE,
		summarize.regions=TRUE,
		region.types = rnb.region.types.for.analysis(ifelse(platform=="MMBC", "mm10", ifelse(target=="probesEPICv2", "hg38", rnb.getOption("assembly")))), ## TODO: Implement assembly selection
		useff=rnb.getOption("disk.dump.big.matrices"),
		ffcleanup=FALSE){
		
		if(missing(pheno)){
			stop("argument pheno should be supplied")
		}
		
		if(missing(probes)){
			if(!is.null(rownames(M))){
				probes<-rownames(M)
			}else if(!is.null(rownames(U))){
				probes<-rownames(U)
			}else{
				stop("If probes are not supplied, betas should have probe identifers as row names")
			}
		}
		
		if(!is.data.frame(pheno)){
			stop("invalid value for pheno: should be a data frame")
		}
		
		if(!any(c('matrix', 'ff_matrix') %in% class(M))){
			stop("invalid value for M: should be a matrix or an ff_matrix")
		}
		
		if(!any(c('matrix', 'ff_matrix') %in% class(U))){
			stop("invalid value for U: should be a matrix or an ff_matrix")
		}
		
		if(!is.null(p.values) && !any(c('matrix', 'ff_matrix') %in% class(p.values))){
			stop("invalid value for p.values: should be a matrix or an ff_matrix")
		}
		
		if(!is.null(bead.counts.M) && !any(c('matrix', 'ff_matrix') %in% class(bead.counts.M))){
			stop("invalid value for bead.counts.M: should be a matrix or an ff_matrix")
		}
		
		if(!is.null(bead.counts.U) && !any(c('matrix', 'ff_matrix') %in% class(bead.counts.U))){
			stop("invalid value for bead.counts.U: should be a matrix or an ff_matrix")
		}
		
		if(!is.null(qc)){
			if(!is.list(qc)){
				stop("invalid value for qc: should be a list")
			}
		}
		
		if(!is.character(platform) || length(platform)!=1L){
			stop("invalid value for platform: should be a character of length one")
		}
		
		if(!is.character(region.types)){
			stop("invalid value for region types: should be a character vector")
		}
		
		if(!is.numeric(beta.offset) || length(beta.offset)!=1L || beta.offset<0){
			stop("invalid value for beta.offset: should be a positive numeric of length one")
		}
		
		if(!is.logical(summarize.regions) || length(summarize.regions)!=1L){
			stop("invalid value for summarize.regions: should be a logical of length one")
		}
		
		if(!is.logical(summarize.bead.counts) || length(summarize.bead.counts)!=1L){
			stop("invalid value for summarize.bead.counts: should be a logical of length one")
		}
		
		if(!is.logical(useff) || length(useff)!=1L){
			stop("invalid value for useff: should be a logical of length one")
		}
		
		genome.assembly <- rnb.getOption("assembly")
		if (platform =="EPIC") {
			target <- "probesEPIC"
			assembly <- ifelse(genome.assembly == "hg19", "hg19", "hg38")
		}else if (platform =="EPICv2") {
			target <- "probesEPICv2"
			assembly <- "hg38"
		}else if (platform =="450k") {
			target <- "probes450"
			assembly <- ifelse(genome.assembly == "hg19", "hg19", "hg38")
		} else if(platform == "27k"){
			target <- "probes27"
			assembly <- "hg19"
        }else if(platform == "MMBC"){
            target <- "probesMMBC"
            assembly <- "mm10"
		}else{
			rnb.error("Invalid value for platform")
		}
		

		res<-match.probes2annotation(probes, target, assembly)
		sites<-res[[1]]
		site.ids<-res[[2]]
		fullmatch<-res[[3]]
		
		M<-prepare.slot.matrix(M, useff=useff, full.match=fullmatch,  subset=site.ids, cleanup=ffcleanup)
		
		U<-prepare.slot.matrix(U, useff=useff, full.match=fullmatch,  subset=site.ids, cleanup=ffcleanup)
		
		if(!is.null(M0)){
			M0<-prepare.slot.matrix(M0, useff=useff, full.match=fullmatch,  subset=site.ids, cleanup=ffcleanup)
		}
		if(!is.null(U0)){
			U0<-prepare.slot.matrix(U0, useff=useff, full.match=fullmatch,  subset=site.ids, cleanup=ffcleanup)
		}
		if(!is.null(bead.counts.M)){
			bead.counts.M<-prepare.slot.matrix(bead.counts.M, useff=useff, full.match=fullmatch,  subset=site.ids, cleanup=ffcleanup)
		}
		if(!is.null(bead.counts.U)){
			bead.counts.U<-prepare.slot.matrix(bead.counts.U, useff=useff, full.match=fullmatch,  subset=site.ids, cleanup=ffcleanup)
		}
		if(!is.null(p.values)){
			p.values<-prepare.slot.matrix(p.values, useff=useff, full.match=fullmatch,  subset=site.ids, cleanup=ffcleanup)
		}
		
		rownames(M)<-NULL			
		rownames(U)<-NULL
		
		if(!is.null(M0)){
			rownames(M0)<-NULL
		}
		if(!is.null(U0)){
			rownames(U0)<-NULL
		}
		if(!is.null(bead.counts.M)){
			rownames(bead.counts.M)<-NULL
			M[,][bead.counts.M[,]<1]<-NA
			if(!is.null(M0)){
				M0[,][bead.counts.M[,]<1]<-NA
			}
		}
		if(!is.null(bead.counts.U)){
			rownames(bead.counts.U)<-NULL
			U[,][bead.counts.U[,]<1]<-NA
			if(!is.null(U0)){
				U0[,][bead.counts.U[,]<1]<-NA
			}
		}
		
		if(!is.null(bead.counts.M) && !is.null(bead.counts.U) && summarize.bead.counts){
			bead.counts<-summarize.bead.counts(bead.counts.M[,,drop=FALSE],bead.counts.U[,,drop=FALSE])
		}else{
			bead.counts<-NULL
		}
		
		betas<-beta.value(M[,,drop=FALSE],U[,,drop=FALSE], beta.offset)
		
		if(useff){
			if(!is.null(bead.counts)){
				bead.counts<-convert.to.ff.matrix.tmp(bead.counts)
			}
		}
		
		if(useff){
			betas<-convert.to.ff.matrix.tmp(betas)
		}
		
		status<-list()
		status[["normalized"]]<-"none"
		status[["background"]]<-"none"
		status[["disk.dump"]]<-useff
		
		object<-new("RnBeadRawSet",
			pheno=pheno,
			sites=sites,
			meth.sites=betas,
			M=M,
			U=U,
			M0=M0,
			U0=U0,
			bead.counts.M=bead.counts.M,
			bead.counts.U=bead.counts.U,
			covg.sites=bead.counts,
			pval.sites=p.values,
			qc=qc,
			target=target,
			status=status
		)
		
		if(summarize.regions){
			for (region.type in region.types) {
				if (region.type %in% rnb.region.types(assembly)) {
					object <- summarize.regions(object, region.type)
				}
			}
		}
		return(object)
}
########################################################################################################################
		
## FIXME: dummy validity method
validRnBeadRawSetObject<-function(object){
	return(TRUE)
}

setValidity("RnBeadRawSet", method=validRnBeadRawSetObject)

########################################################################################################################

setMethod("show", "RnBeadRawSet", rnb.show.rnbeadset)

########################################################################################################################

#' Conversion to/from RnBeadRawSet
#' 
#' The \code{"as"} method can be used for the following conversions:
#' \itemize{
#'   \item{}{\code{MethyLumiSet} (in package \pkg{methylumi}) to \code{\linkS4class{RnBeadRawSet}}}
#'   \item{}{\code{RnBeadRawSet} to \code{MethyLumiSet}}
#'   \item{}{\code{RGChannelSet} (in package \pkg{minfi}) to \code{\linkS4class{RnBeadRawSet}}}
#' }
#' 
#' @name as.RnBeadRawSet
setAs("MethyLumiSet", "RnBeadRawSet",
		
		function(from, to){
			
			if(!inherits(from,"MethyLumiSet")){
				stop("not a MethyLumiSet object:", deparse(substitute(methylumi.set)))
			}
			
			m.data <- MethyLumiSet2RnBeadSet(from)
			
			if("methylated.N" %in% ls(from@assayData) && "unmethylated.N" %in% ls(from@assayData) ){
				meth.N.element<-"methylated.N"
				umeth.N.element<-"unmethylated.N"
			}else if("Avg_NBEADS_A" %in% ls(from@assayData) && "Avg_NBEADS_B" %in% ls(from@assayData)){
				meth.N.element<-"Avg_NBEADS_B"	
				umeth.N.element<-"Avg_NBEADS_A"
			}else{
				meth.N.element<-NULL
				umeth.N.element<-NULL
			}
			
			if("methylated.OOB" %in% ls(from@assayData) && "unmethylated.OOB" %in% ls(from@assayData)){
				meth.oob.element<-"methylated.OOB"
				umeth.oob.element<-"unmethylated.OOB"
			}else{
				meth.oob.element<-NULL
				umeth.oob.element<-NULL
			}
			
			if(annotation(from)=="IlluminaMethylationEPIC"){
				platform="EPIC"
			}else if(annotation(from)=="IlluminaMethylationEPICv2"){
				platform="EPICv2"
			}else if(annotation(from)=="IlluminaHumanMethylation450k"){
				platform="450k"
			}else if(annotation(from)=="IlluminaHumanMethylation27k"){
				platform="27k"
			}
			
			object<-RnBeadRawSet(
					pheno=m.data$pheno,
					probes=m.data$probes,
					M=methylated(from),
					U=unmethylated(from),
					p.values=m.data$p.values,
					M0=if(!is.null(meth.oob.element)) get(meth.oob.element,from@assayData) else NULL,
					U0=if(!is.null(umeth.oob.element)) get(umeth.oob.element,from@assayData) else NULL,
					bead.counts.M=if(!is.null(meth.N.element)) get(meth.N.element,from@assayData) else NULL,
					bead.counts.U=if(!is.null(umeth.N.element)) get(umeth.N.element,from@assayData) else NULL,
					platform=platform
					)
			
			if ("qc" %in% names(m.data)) {
				qc(object)<-m.data[["qc"]]
			}
						
			object
			
		})

########################################################################################################################
		
setAs("RnBeadRawSet","MethyLumiSet",
		
		function(from, to){
			
			if(!inherits(from,"RnBeadRawSet")){
				stop("not a RnBeadRawSet object:", deparse(substitute(methylumi.set)))
			}
				
			assd<-new.env()
			
			assign("betas", meth(from),  envir=assd)
			assign("pvals", dpval(from),  envir=assd)
			assign("methylated", M(from),  envir=assd)
			assign("unmethylated", U(from),  envir=assd)
			
			if(!is.null(M0(from))){
				assign("methylated.OOB", M0(from),  envir=assd)
			}
			if(!is.null(U0(from))){
				assign("unmethylated.OOB", U0(from),  envir=assd)
			}
			if(!is.null(bead.counts.M(from))){
				assign("methylated.N", bead.counts.M(from),  envir=assd)
			}
			if(!is.null(bead.counts.U(from))){
				assign("unmethylated.N", bead.counts.U(from),  envir=assd)
			}
			
			pd<-pheno(from)
			rownames(pd)<-colnames(meth(from))
			
			mset<-new(to, assd, as(pd, "AnnotatedDataFrame"))
			
			rm(assd)
			
			ann<-annotation(from, add.names=TRUE)
			featureData(mset)<-as(data.frame(COLOR_CHANNEL=ann$Color, rownames=rownames(ann)), "AnnotatedDataFrame")
			featureNames(mset)<-ann[["ID"]]
			
			if (!is.null(qc(from))){
				
				assd<-new.env()
				
				assign("methylated", qc(from)$Cy3,  envir=assd)
				assign("unmethylated", qc(from)$Cy5,  envir=assd)
				
				mset@QC<-new("MethyLumiQC", assd)
					
				if(from@target == "probesEPIC"){
					probeIDs<-rnb.get.annotation("controlsEPIC")[,"Target"]
					## TODO remove this after annotation has been fixed
					index<-rnb.update.controlsEPIC.enrich(rnb.get.annotation("controlsEPIC"))[,"Index"]
					probeIDs<-paste(probeIDs, index, sep=".")
				}else if(from@target == "probesEPICv2"){
					probeIDs<-rnb.get.annotation("controlsEPICv2", assembly = "hg38")[,"Target"]
					probeIDs<-paste(probeIDs, unlist(sapply(table(probeIDs)[unique(probeIDs)], seq, from=1 )), sep=".")
				}else if(from@target == "probes450"){
					probeIDs<-rnb.get.annotation("controls450")[,"Target"]
					probeIDs<-paste(probeIDs, unlist(sapply(table(probeIDs)[unique(probeIDs)], seq, from=1 )), sep=".")
				}else if(from@target == "probes27"){
					probeIDs<-rnb.get.annotation("controls27")[,"Name"]
				}
				
				featureData(mset@QC)<-as(data.frame(Address=rownames(qc(from)$Cy3), rownames=probeIDs), "AnnotatedDataFrame")
				featureNames(mset@QC)<-probeIDs
								
				if(from@target == "probesEPIC"){
					annotation(mset@QC) <- "IlluminaMethylationEPIC"
				}else if(from@target == "probesEPICv2"){
					annotation(mset@QC) <- "IlluminaMethylationEPICv2"
				}else if(from@target == "probes450"){
					annotation(mset@QC) <- "IlluminaHumanMethylation450k"
				}else if(from@target == "probes27"){
					annotation(mset) <- "IlluminaHumanMethylation27k"
				}
			}
			
			if(from@target == "probesEPIC"){
				annotation(mset) <- "IlluminaMethylationEPIC"
			}else if(from@target == "probesEPICv2"){
				annotation(mset) <- "IlluminaMethylationEPICv2"
			}else if(from@target == "probes450"){
				annotation(mset) <- "IlluminaHumanMethylation450k"
			}else if(from@target == "probes27"){
				annotation(mset) <- "IlluminaHumanMethylation27k"
			}
			
			mset
			
		})

########################################################################################################################

setAs("RGChannelSet", "RnBeadRawSet", function(from, to) {

		## Get Illumina assay (platform)
		target.info <- getManifest(from)
		assay.name <- target.info@annotation
		if (!(is.character(assay.name) && length(assay.name) == 1 && isTRUE(assay.name != ""))) {
			stop("Unsupported platform; expected one-element character")
		}
		if (assay.name == "IlluminaHumanMethylationEPIC") {
			assay.name <- "probesEPIC"
			platform.name <- "EPIC"
		} else if (assay.name == "IlluminaHumanMethylationEPICv2") {
			assay.name <- "probesEPICv2"
			platform.name <- "EPICv2"
		} else if (assay.name == "IlluminaHumanMethylation450k") {
			assay.name <- "probes450"
			platform.name <- "450k"
		} else if (assay.name == "IlluminaHumanMethylation27k") {
			assay.name <- "probes27"
			platform.name <- "27k"
		} else {
			stop(paste("Unsupported platform", assay.name))
		}
	
		## Use RnBeads' mapping from probe IDs to addresses
		probes.all <- rnb.get.annotation(assay.name, ifelse(assay.name=="probesEPICv2", "hg38", rnb.getOption("assembly"))) ## TODO: Impove genome build selection
		probes.all <- lapply(probes.all, function(x) {
				result <- as.data.frame(mcols(x)[, c("Design", "Color", "AddressA", "AddressB")])
				rownames(result) <- names(x)
				result
			}
		)
		probes.all <- do.call(rbind, unname(probes.all))
		controls.all <- rnb.get.annotation(sub("^probes", "controls", assay.name), ifelse(assay.name=="probesEPICv2", "hg38", rnb.getOption("assembly"))) ## TODO: Impove genome build selection
		controls.all <- controls.all[, "ID"]

		## Extract data on signals
		mm.green <- getGreen(from)
		mm.red <- getRed(from)
		addresses.raw <- as.integer(rownames(mm.green))
		probes.supported <- probes.all[, 3] %in% addresses.raw | probes.all[, 4] %in% addresses.raw
		probes.all <- probes.all[probes.supported, ]
		rm(target.info, assay.name, probes.supported)

		## Initialize matrices of signal intensities and bead counts
		M <- matrix(NA_real_, nrow = nrow(probes.all), ncol = ncol(mm.red))
		U <- matrix(NA_real_, nrow = nrow(probes.all), ncol = ncol(mm.red))
		M0 <- matrix(NA_real_, nrow = nrow(probes.all), ncol = ncol(mm.red))
		U0 <- matrix(NA_real_, nrow = nrow(probes.all), ncol = ncol(mm.red))
		if (inherits(from, "RGChannelSetExtended")) {
			mm.beads <- get("NBeads", pos = from@assayData)
			beads.M <- matrix(0L, nrow = nrow(probes.all), ncol = ncol(mm.red))
			beads.U <- matrix(0L, nrow = nrow(probes.all), ncol = ncol(mm.red))
		} else {
			beads.M <- NULL
			beads.U <- NULL
		}

		## Map signals on addresses to probe signal intensities
		ii <- which(probes.all[, "Design"] == "II")
		jj <- findInterval(probes.all[ii, "AddressA"], addresses.raw)
		M[ii, ] <- mm.green[jj, ]
		U[ii, ] <- mm.red[jj, ]
		if (!is.null(beads.M)) {
			beads.M[ii, ] <- mm.beads[jj, ]
			beads.U[ii, ] <- mm.beads[jj, ]
		}
		ii <- which(probes.all[, "Design"] == "I" & probes.all[, "Color"] == "Grn")
		jj <- findInterval(probes.all[ii, "AddressB"], addresses.raw)
		M[ii, ] <- mm.green[jj, ]
		M0[ii, ] <- mm.red[jj, ]
		if (!is.null(beads.M)) {
			beads.M[ii, ] <- mm.beads[jj, ]
		}
		jj <- findInterval(probes.all[ii, "AddressA"], addresses.raw)
		U[ii, ] <- mm.green[jj, ]
		U0[ii, ] <- mm.red[jj, ]
		if (!is.null(beads.M)) {
			beads.U[ii, ] <- mm.beads[jj, ]
		}
		ii <- which(probes.all[, "Design"] == "I" & probes.all[, "Color"] == "Red")
		jj <- findInterval(probes.all[ii, "AddressB"], addresses.raw)
		M[ii, ] <- mm.red[jj, ]
		M0[ii, ] <- mm.green[jj, ]
		if (!is.null(beads.M)) {
			beads.M[ii, ] <- mm.beads[jj, ]
		}
		jj <- findInterval(probes.all[ii, "AddressA"], addresses.raw)
		U[ii, ] <- mm.red[jj, ]
		U0[ii, ] <- mm.green[jj, ]
		if (!is.null(beads.M)) {
			beads.U[ii, ] <- mm.beads[jj, ]
		}

		## TODO: Extract detection p-values if present

		## Extract control probe signals if available
		jj <- findInterval(controls.all, addresses.raw, all.inside = TRUE)
		ii <- which(controls.all == addresses.raw[jj])
		if (length(ii) == 0) {
			qc <- NULL
		} else {
			jj <- jj[ii]
			qc <- matrix(NA_real_, nrow = length(controls.all), ncol = ncol(mm.red))
			rownames(qc) <- as.character(controls.all)
			qc <- list("Cy3" = qc, "Cy5" = qc)
			qc[[1]][ii, ] <- mm.green[jj, ]
			qc[[2]][ii, ] <- mm.red[jj, ]
			if (all(sapply(qc, function(x) { all(is.na(x)) }))) {
				qc <- NULL
			}
		}
    p.data <- as.data.frame(pData(from))
		## Construct the resulting object
		RnBeadRawSet(p.data, rownames(probes.all), M, U, M0, U0, beads.M, beads.U, NULL, qc, platform.name)
	}
)

setAs("RnBeadRawSet", "RGChannelSet", function(from, to){
	assay.name <- from@target
#	probes.all <- rnb.get.annotation(assay.name, "hg19")
#	probes.all <- lapply(probes.all, function(x) {
#			result <- as.data.frame(mcols(x)[, c("Design", "Color", "AddressA", "AddressB")])
#			rownames(result) <- names(x)
#			result
#		}
#	)
#	probes.all <- do.call(rbind, unname(probes.all))
	probes.all <- annotation(from)[,c("Design", "Color", "AddressA", "AddressB")]
	genome.assembly<-rnb.getOption("assembly")
	controls.all <- rnb.get.annotation(sub("^probes", "controls", assay.name), ifelse(assay.name=="probesEPICv2", "hg38", genome.assembly))
	controls.all <- controls.all[, "ID"]

	# Obtain methylated and unmethylated intensities
	M <- M(from)
	U <- U(from)
#	beads.M <- from@bead.counts.M
#	beads.U <- from@bead.counts.U
	M0 <- from@M0
	U0 <- from@U0

	#mm.green <- matrix(NA_real_, nrow=nrow(probes.all), ncol=ncol(M))
	#mm.red <- matrix(NA_real_, nrow=nrow(probes.all), ncol=ncol(M))

	is.type.II <- which(probes.all[,"Design"]%in%"II")
	addresses <- probes.all[is.type.II,"AddressA"]
	mm.green <- M[is.type.II,]
	mm.red <- U[is.type.II,]
#	mm.beads <- beads.M[is.type.II,,drop=F]
	row.names(mm.green) <- addresses
	row.names(mm.red) <- addresses

	is.type.I.green <- which(probes.all[, "Design"] == "I" & probes.all[, "Color"] == "Grn")
	addresses <- probes.all[is.type.I.green,"AddressB"]
	r.names <- c(row.names(mm.green),addresses)
	mm.green <- rbind(mm.green,M[is.type.I.green,])
	row.names(mm.green) <- r.names
	r.names <- c(row.names(mm.red),addresses)
	mm.red <- rbind(mm.red,M0[is.type.I.green,,drop=F])
	row.names(mm.red) <- r.names
#	mm.beads <- rbind(mm.beads,beads.M[is.type.I.green,,drop=F])
#	mm.beads <- rbind(mm.beads,beads.U[is.type.I.green,,drop=F])

	addresses <- probes.all[is.type.I.green,"AddressA"]
	r.names <- c(row.names(mm.green),addresses)
	mm.green <- rbind(mm.green,U[is.type.I.green,])
	row.names(mm.green) <- r.names
	r.names <- c(row.names(mm.red),addresses)
	mm.red <- rbind(mm.red,U0[is.type.I.green,,drop=F])
	row.names(mm.red) <- r.names

	is.type.I.red <- which(probes.all[, "Design"] == "I" & probes.all[, "Color"] == "Red")
	addresses <- probes.all[is.type.I.red,"AddressB"]
	r.names <- c(row.names(mm.red),addresses)
	mm.red <- rbind(mm.red,M[is.type.I.red,])
	row.names(mm.red) <- r.names
	r.names <- c(row.names(mm.green),addresses)
	mm.green <- rbind(mm.green,M0[is.type.I.red,,drop=F])
	row.names(mm.green) <- r.names
#	mm.beads <- rbind(mm.beads,beads.M[is.type.I.red,,drop=F])
#	mm.beads <- rbind(mm.beads,beads.U[is.type.I.red,,drop=F])

	addresses <- probes.all[is.type.I.red,"AddressA"]
	r.names <- c(row.names(mm.red),addresses)
	mm.red <- rbind(mm.red,U[is.type.I.red,])
	row.names(mm.red) <- r.names
	r.names <- c(row.names(mm.green),addresses)
	mm.green <- rbind(mm.green,U0[is.type.I.red,,drop=F])
	row.names(mm.green) <- r.names

	r.names <- c(row.names(mm.green),controls.all)
	mm.green <- rbind(mm.green,qc(from)[[1]])
	row.names(mm.green) <- r.names
	r.names <- c(row.names(mm.red),controls.all)
	mm.red <- rbind(mm.red,qc(from)[[2]])
	row.names(mm.red) <- r.names
#	mm.beads <- rbind(mm.beads,
#		matrix(NA,nrow=nrow(qc(from)[[1]]),ncol=ncol(mm.beads)))

	if(assay.name %in% "probesEPIC"){
		anno <- "IlluminaHumanMethylationEPIC"
	}else if(assay.name %in% "probesEPICv2"){
		anno <- "IlluminaHumanMethylationEPICv2"
	}else if(assay.name %in% "probes450"){
		anno <- "IlluminaHumanMethylation450k"
	}else if(assay.name %in% "probes27"){
		anno <- "IlluminaHumanMethylation27k"
	}else{
		stop("Unsupported platform")
	}
	p.data <- pheno(from)
	if(!is.null(rnb.getOption("identifiers.column"))){
		row.names(p.data) <- p.data[,rnb.getOption("identifiers.column")]
		colnames(mm.green) <- colnames(mm.red) <- p.data[,rnb.getOption("identifiers.column")]
	}
	to <- RGChannelSet(Green=mm.green,
			Red=mm.red,
	#		NBeads=mm.beads,
	#		GreenSD=matrix(NA,nrow=nrow(mm.green),ncol=ncol(mm.green)),
	#		RedSD=matrix(NA,nrow=nrow(mm.red),ncol=ncol(mm.red)),
			annotation=anno,
			colData=pheno(from))
	return(to)
})

########################################################################################################################
		
## ---------------------------------------------------------------------------------------------------------------------
## ACCESSORS
## ---------------------------------------------------------------------------------------------------------------------

if(!isGeneric("M")) setGeneric('M',
			function(object, ...) standardGeneric('M'))

#' M-methods
#'
#' Extract raw methylated probe intensity from an object of \code{RnBeadRawSet} class.
#'
#' @param object 		Dataset of interest.
#' @param row.names		Flag indicating whether the resulting matrix will be assigned row names
#'  
#' @return \code{matrix} of the methylated probe intensities
#'
#' @rdname M-methods
#' @docType methods
#' @export
#' @aliases M
#' @aliases M,RnBeadRawSet-method
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' M.intensity<-M(rnb.set.example)
#' head(M.intensity)
#' } 
#' 
setMethod("M", signature(object="RnBeadRawSet"),
		function(object, row.names=FALSE){
			get.dataset.matrix(object, "sites", row.names, object@M, object@meth.regions)
		})


if(!isGeneric("U")) setGeneric('U',
			function(object, ...) standardGeneric('U'))

########################################################################################################################

#' U-methods
#'
#' Extract raw unmethylated probe intensity from an object of \code{RnBeadRawSet} class.
#'
#' @param object 		Dataset of interest.
#' @param row.names		Flag indicating whether the resulting matrix will be assigned row names
#'  
#' @return \code{matrix} of the unmethylated probe intensities
#'
#' @rdname U-methods
#' @docType methods
#' @export
#' @aliases U
#' @aliases U,RnBeadRawSet-method
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' U.intensity<-U(rnb.set.example)
#' head(U.intensity)
#' } 
setMethod("U", signature(object="RnBeadRawSet"),
		function(object, row.names=FALSE){
			get.dataset.matrix(object, "sites", row.names, object@U, object@meth.regions)
		})

########################################################################################################################

setGeneric('M0',
			function(object, ...) standardGeneric('M0'))

setMethod("M0", signature(object="RnBeadRawSet"),
		function(object, row.names=FALSE){
			get.dataset.matrix(object, "sites", row.names, object@M0, object@meth.regions)
		})

########################################################################################################################

setGeneric('U0',
			function(object, ...) standardGeneric('U0'))

setMethod("U0", signature(object="RnBeadRawSet"),
		function(object, row.names=FALSE){
			get.dataset.matrix(object, "sites", row.names, object@U0, object@meth.regions)
		})
########################################################################################################################

setGeneric('bead.counts.M',
			function(object, ...) standardGeneric('bead.counts.M'))


setMethod("bead.counts.M", signature(object="RnBeadRawSet"),
		function(object, row.names=FALSE){
			get.dataset.matrix(object, "sites", row.names, object@bead.counts.M, object@meth.regions)
		})

########################################################################################################################

setGeneric('bead.counts.U',
			function(object, ...) standardGeneric('bead.counts.U'))


setMethod("bead.counts.U", signature(object="RnBeadRawSet"),
		function(object, row.names=FALSE){
			get.dataset.matrix(object, "sites", row.names, object@bead.counts.U, object@meth.regions)
		})

## ---------------------------------------------------------------------------------------------------------------------
## MODIFIERS
## ---------------------------------------------------------------------------------------------------------------------

setGeneric('M<-',
			function(object, value) standardGeneric('M<-'))

setMethod("M<-", signature(object="RnBeadRawSet", value="matrixOrffOrNULL"),
		function(object, value){
			if(object@status$disk.dump){
				# delete(object@M)
				object@M<-convert.to.ff.matrix.tmp(value)	
			}else{
				object@M<-value
			}
			
		})
########################################################################################################################

setGeneric('U<-',
			function(object, value) standardGeneric('U<-'))

setMethod("U<-", signature(object="RnBeadRawSet", value="matrixOrffOrNULL"),
		function(object, value){
			if(object@status$disk.dump){
				# delete(object@U)
				object@U<-convert.to.ff.matrix.tmp(value)	
			}else{
				object@U<-value
			}
		})

########################################################################################################################

setGeneric('M0<-',
			function(object, value) standardGeneric('M0<-'))

setMethod("M0<-", signature(object="RnBeadRawSet", value="matrixOrffOrNULL"),
		function(object, value){
			if(object@status$disk.dump){
				# delete(object@M0)
				object@M0<-convert.to.ff.matrix.tmp(value)	
			}else{
				object@M0<-value
			}
		})
########################################################################################################################

setGeneric('U0<-',
			function(object, value) standardGeneric('U0<-'))

setMethod("U0<-", signature(object="RnBeadRawSet", value="matrixOrffOrNULL"),
		function(object, value){
			if(object@status$disk.dump){
				# delete(object@U0)
				object@U0<-convert.to.ff.matrix.tmp(value)	
			}else{
				object@U0<-value
			}
		})
########################################################################################################################

setGeneric('bead.counts.M<-',
			function(object, value) standardGeneric('bead.counts.M<-'))

setMethod("bead.counts.M<-", signature(object="RnBeadRawSet", value="matrixOrffOrNULL"),
		function(object, value){
			if(object@status$disk.dump){
				# delete(object@bead.counts.M)
				object@bead.counts.M<-convert.to.ff.matrix.tmp(value)	
			}else{
				object@bead.counts.M<-value
			}
		})
########################################################################################################################

setGeneric('bead.counts.U<-',
			function(object, value) standardGeneric('bead.counts.U<-'))

setMethod("bead.counts.U<-", signature(object="RnBeadRawSet", value="matrixOrffOrNULL"),
		function(object, value){
			if(object@status$disk.dump){
				# delete(object@bead.counts.U)
				object@bead.counts.U<-convert.to.ff.matrix.tmp(value)	
			}else{
				object@bead.counts.U<-value
			}
		})
########################################################################################################################

if (!isGeneric("remove.sites")) {
	setGeneric("remove.sites", function(object, probelist, verbose = TRUE) standardGeneric("remove.sites"))
}

#' @rdname remove.sites-methods
#' @aliases remove.sites,RnBeadRawSet-method
#' @docType methods
#' @export
setMethod("remove.sites", signature(object = "RnBeadRawSet"),
		function(object, probelist, verbose = TRUE) {
			inds <- get.i.vector(probelist, rownames(object@sites))
			if (length(inds) != 0) {
				for(sl in RNBRAWSET.SLOTNAMES){
					if(!is.null(slot(object,sl))){
						if(!is.null(object@status) && object@status$disk.dump){
							new.matrix<-slot(object,sl)[-inds,,drop=FALSE]
							if(isTRUE(object@status$discard.ff.matrices)){
								delete(slot(object,sl))
							}
							slot(object,sl)<-convert.to.ff.matrix.tmp(new.matrix)
							rm(new.matrix); rnb.cleanMem()
						}else{
							slot(object,sl)<-slot(object,sl)[-inds,,drop=FALSE]
						}
					
					}
				}
			}
			
			callNextMethod()
		}
)

########################################################################################################################

if (!isGeneric("remove.samples")) {
	setGeneric("remove.samples", function(object, samplelist) standardGeneric("remove.samples"))
}

#' @rdname remove.samples-methods
#' @aliases remove.samples,RnBeadRawSet-method
#' @docType methods
#' @export
setMethod("remove.samples", signature(object = "RnBeadRawSet"),
		function(object, samplelist) {
			inds <- get.i.vector(samplelist, samples(object))
			if (length(inds) != 0) {
				for(sl in RNBRAWSET.SLOTNAMES){
					if(!is.null(slot(object,sl))){
						if(!is.null(object@status) && object@status$disk.dump){
							new.matrix<-slot(object,sl)[,-inds, drop=FALSE]
							if(isTRUE(object@status$discard.ff.matrices)){
								delete(slot(object,sl))
							}
							slot(object,sl)<-convert.to.ff.matrix.tmp(new.matrix)
							rm(new.matrix); rnb.cleanMem()
						}else{
							slot(object,sl)<-slot(object,sl)[,-inds, drop=FALSE]
						}
					}
				}
			}
			callNextMethod()
		}
)

#######################################################################################################################

#if (!isGeneric("update.meth")) {
	setGeneric("update.meth", function(object) standardGeneric("update.meth"))
#}

##  
## update.meth
## 
## Update the methylation calls, after the change of intensity values
##
## param object 		RnBeadRawSet object
##
## return Updated RnBeadRawSet object
##
setMethod("update.meth", signature(object="RnBeadRawSet"),
		function(object){
			
			if(object@status$disk.dump){
				object@meth.sites<-convert.to.ff.matrix.tmp(beta.value(object@M[,], object@U[,]))	
			}else{
				object@meth.sites<-beta.value(object@M, object@U)
			}
			return(object)
		})

#######################################################################################################################
## save, load and destroy 

setMethod("save.matrices", signature(object="RnBeadRawSet", path="character"),
		function(object, path){

			if(!is.null(object@status) && object@status$disk.dump){
				
				for(sl in RNBRAWSET.SLOTNAMES){
					if(!is.null(slot(object,sl))){
					
						if("ff" %in% class(slot(object,sl))){
							ffmatrix<-slot(object,sl)
							ffsave(ffmatrix, file=file.path(path, paste("rnb", sl, sep=".")),
									rootpath=getOption('fftempdir'))
							rm(ffmatrix)
							
						}
					}
				}
			}
			callNextMethod(object, path)
			
		})

#######################################################################################################################

setMethod("load.matrices", signature(object="RnBeadRawSet", path="character"),
		
		function(object, path, temp.dir=tempdir()){
			slot.names <- RNBRAWSET.SLOTNAMES
			for(sl in slot.names){
				if(!is.null(slot(object, sl))){
					
					if(paste("rnb",sl,"RData", sep=".") %in% list.files(path) &&
							paste("rnb",sl,"ffData", sep=".") %in% list.files(path)){
						load_env<-new.env()
						suppressMessages(ffload(file=file.path(path, paste("rnb", sl, sep=".")), 
										envir=load_env,rootpath=getOption("fftempdir")))
						slot(object, sl)<-get("ffmatrix", envir=load_env)
						rm(load_env)
					}
					
				}

			}
			
			callNextMethod(object=object, path=path, temp.dir=temp.dir)
			
		})

#######################################################################################################################

#' @rdname destroy-methods
#' @aliases destroy,RnBeadRawSet-method
#' @docType methods
#' @export
setMethod("destroy", signature(object="RnBeadRawSet"),
		function(object){
			
			if(object@status$disk.dump){
				for(sl in RNBRAWSET.SLOTNAMES){
					if(!is.null(slot(object,sl))){
						delete(slot(object, sl))	
					}
				}
			}
			callNextMethod()
			
		}
)

## ---------------------------------------------------------------------------------------------------------------------
## HELPER ROUTINES
## ---------------------------------------------------------------------------------------------------------------------

beta.value<-function(M,U,offset=100){
	M/(M+U+offset)
}

#######################################################################################################################

m.value<-function(M,U,offset=100){
	log2((M+offset)/(U+offset))
}

#######################################################################################################################

#' intensities.by.color
#' 
#' Rearranges information from "M" and "U" slots of a RnBeadsRawSet object by color channel.
#'
#' @param raw.set          Methylation dataset as an instance of \code{RnBeadRawSet} object.
#' @param address.rownames  if \code{TRUE} the rows of the returned matrices are named with the with the corresponding Illumina probe addresses 
#' @param add.oob			 if \code{TRUE} the "out-of-band" intensities are included
#' @param add.controls if \code{TRUE} the control probe intensities are included
#' @param add.missing  if \code{TRUE} the rows for the probes missing in \code{raw.set} is imputed with \code{NA} values
#' @param re.separate  if \code{TRUE} the type I and type II intensities, as well as the out-of-band and control probe intensities
#'                     (if set to \code{TRUE}), will be returned as separate elements per channel and not as concatenated rows.  
#' @return A \code{list} with elements \code{Cy3} and \code{Cy5} containing average bead intensities measured for each
#'         each probe in the green and red channels, respectively. Exception, if \code{re.separate} is \code{TRUE} a \code{list}
#'         with elements \code{Cy3.I}, \code{Cy5.I}, and \code{II} will be returned. The elements \code{Cy3.I.oob}, 
#'         \code{Cy5.I.oob} and also \code{Cy3.ctl}, \code{Cy5.ctl} will be returned 
#'         if the respective parameters (\code{add.oob} and \code{add.ctl}) are set to true. 
#' 
#' @author Pavlo Lutsik, Nathan Steenbuck

intensities.by.color<-function(raw.set,
                                   address.rownames = TRUE, 
                                   add.oob = all(!is.null(M0(raw.set)), !is.null(U0(raw.set))), 
                                   add.controls = !is.null(qc(raw.set)),
                                   add.missing = TRUE,
                                   re.separate = FALSE) {
  if (raw.set@target == "probesEPIC") { 
    rnb.require("IlluminaHumanMethylationEPICmanifest")
    manifest.object <- IlluminaHumanMethylationEPICmanifest

#   } else if (raw.set@target == "probesEPICv2") { ## FIXME: No minfi support for EPIC v2!
#     rnb.require("IlluminaHumanMethylationEPICv2manifest") ## TODO: Doesn't exist for EPIC v2!
#     manifest.object <- IlluminaHumanMethylationEPICv2manifest

  } else if (raw.set@target == "probes450") {
    rnb.require("IlluminaHumanMethylation450kmanifest")
    manifest.object <- IlluminaHumanMethylation450kmanifest
  } else {
    stop("Unsupported platform")
  }
  
  Mmatrix<-M(raw.set, row.names=TRUE)
  Umatrix<-U(raw.set, row.names=TRUE)
  if(add.oob){
      if(!is.null(M0(raw.set))){
        M0matrix<-M0(raw.set, row.names=TRUE)
        U0matrix<-U0(raw.set, row.names=TRUE)
    }else{
        M0matrix<-matrix(NA_integer_, ncol=ncol(Mmatrix), nrow=nrow(Umatrix))
        U0matrix<-matrix(NA_integer_, ncol=ncol(Umatrix), nrow=nrow(Umatrix))
        rownames(M0matrix)<-rownames(Mmatrix)
        rownames(U0matrix)<-rownames(Umatrix)
    }
  }
  pinfos <- annotation(raw.set, add.names=TRUE)
  
  if(add.missing){
    full.ann<-rnb.annotation2data.frame(rnb.get.annotation(raw.set@target))
    ann.missing<-full.ann[setdiff(rownames(full.ann), rownames(pinfos)),]
    pinfos<-rbind(pinfos, ann.missing[,colnames(full.ann)])
    filler<-matrix(NA_real_, nrow=nrow(ann.missing), ncol=length(samples(raw.set)))
    rownames(filler)<-rownames(ann.missing)
    
    Mmatrix<-rbind(Mmatrix, filler)
    Umatrix<-rbind(Umatrix, filler)
    
    if(add.oob){
      M0matrix<-rbind(M0matrix, filler)
      U0matrix<-rbind(U0matrix, filler)
    }
    
    rm(full.ann, ann.missing, filler)
  }
  
  rnb.set.probe.ids<-pinfos[["ID"]]
  dII.probes <- rnb.set.probe.ids[pinfos[, "Design"] == "II"]
  
  if (address.rownames) {
    tII<-rbind(as.data.frame(manifest.object@data$TypeII[,c("Name", "AddressA")]),
               as.data.frame(manifest.object@data$TypeSnpII[,c("Name", "AddressA")]))
    tII<-tII[match(dII.probes, tII$Name),]
  }
  dII.grn <- Mmatrix[pinfos[, "Design"] == "II", , drop = FALSE]
  dII.red <- Umatrix[pinfos[, "Design"] == "II", , drop = FALSE]
  if (address.rownames) {
    rownames(dII.grn) <- rownames(dII.red) <- tII$AddressA
  }
  
  dI.red.probes <- rnb.set.probe.ids[pinfos[, "Color"] == "Red"]
  dI.green.probes <- rnb.set.probe.ids[pinfos[, "Color"] == "Grn"]
  #dI.red.probes <- dI.red.probes[!grepl("rs", dI.red.probes)]
  #dI.green.probes <- dI.green.probes[!grepl("rs", dI.green.probes)]
  
  if (address.rownames) {
    tI <- c("Name","Color", "AddressA", "AddressB")
    tI <- lapply(c("TypeI", "TypeSnpI"), function(x) { as.data.frame(manifest.object@data[[x]][, tI]) })
    tI <- do.call(rbind, unname(tI))
    
    tI.red<-tI[tI$Color=="Red",]
    tI.red<-tI.red[match(dI.red.probes, tI.red$Name),]
    
    tI.grn<-tI[tI$Color=="Grn",]
    tI.grn<-tI.grn[match(dI.green.probes, tI.grn$Name),]
    rm(tI)
  }
  
  dI.red.meth <- Mmatrix[pinfos[, "Color"] == "Red", , drop = FALSE]
  dI.red.umeth <- Umatrix[pinfos[, "Color"] == "Red", , drop = FALSE]
  dI.grn.meth <- Mmatrix[pinfos[, "Color"] == "Grn", , drop = FALSE]
  dI.grn.umeth <- Umatrix[pinfos[, "Color"] == "Grn", , drop = FALSE]
  
  if (address.rownames) {
    rownames(dI.red.meth) <- tI.red[, "AddressB"]
    rownames(dI.red.umeth) <- tI.red[, "AddressA"]
    rownames(dI.grn.meth) <- tI.grn[, "AddressB"]
    rownames(dI.grn.umeth) <- tI.grn[, "AddressA"]
  }
  if (add.oob) {
    dI.red.meth.oob <- M0matrix[pinfos[, "Color"] == "Red", , drop = FALSE]
    dI.red.umeth.oob <- U0matrix[pinfos[, "Color"] == "Red", , drop = FALSE]
    dI.grn.meth.oob <- M0matrix[pinfos[, "Color"] == "Grn", , drop = FALSE]
    dI.grn.umeth.oob <- U0matrix[pinfos[, "Color"] == "Grn", , drop = FALSE]
    if (address.rownames) {
      rownames(dI.red.meth.oob) <- tI.red[, "AddressB"]
      rownames(dI.red.umeth.oob) <- tI.red[, "AddressA"]
      rownames(dI.grn.meth.oob) <- tI.grn[, "AddressB"]
      rownames(dI.grn.umeth.oob) <- tI.grn[, "AddressA"]
    }
  }
  
  if(re.separate){
    intensities.by.channel <- list("Cy3.I" = list("M" = dI.grn.meth, "U" = dI.grn.umeth),
                                   "Cy5.I" = list("M" = dI.red.meth, "U" = dI.red.umeth), 
                                   "II" = list("M" = dII.grn, "U" = dII.red))
    
    if(add.oob){
      intensities.by.channel[["Cy3.I.oob"]] = list("M" = dI.red.meth.oob, "U" = dI.red.umeth.oob)
      intensities.by.channel[["Cy5.I.oob"]] = list("M" = dI.grn.meth.oob, "U" = dI.grn.umeth.oob)
    } 
  }else{
    intensities.by.channel <- list(
      Cy3=rbind(dII.grn, dI.grn.meth, dI.grn.umeth, 
                if(add.oob) dI.red.meth.oob else NULL, if(add.oob) dI.red.umeth.oob else NULL),
      Cy5=rbind(dII.red, dI.red.meth, dI.red.umeth, 
                if(add.oob) dI.grn.meth.oob else NULL, if(add.oob) dI.grn.umeth.oob else NULL))
    
    rm(dII.grn, dI.grn.meth, dI.grn.umeth, dI.red.meth.oob, dI.red.umeth.oob, 
       dII.red, dI.red.meth, dI.red.umeth, dI.grn.meth.oob, dI.grn.umeth.oob)
    invisible(gc())
    
    if(add.oob & address.rownames) {
      intensities.by.channel$Cy5<-intensities.by.channel$Cy5[rownames(intensities.by.channel$Cy3),,drop=FALSE]
    }
  }
  
  if (add.controls) {
	if (raw.set@target == "probesEPICv2") {
		ncd <- rnb.get.annotation("controlsEPICv2", assembly = "hg38")
	} else {
		ncd <- rnb.get.annotation(ifelse(raw.set@target == "probesEPIC", "controlsEPIC", "controls450"))
	}
    #ncd <- ncd[ncd[["Target"]] == "NEGATIVE", ]
    ncd$Target <- tolower(ncd$Target)
    controls.by.channel <- qc(raw.set)
    controls.by.channel$Cy3 <- controls.by.channel$Cy3[as.character(ncd$ID), , drop=FALSE]
    controls.by.channel$Cy5 <- controls.by.channel$Cy5[as.character(ncd$ID), , drop=FALSE]
    
    if(re.separate){
      intensities.by.channel[["Cy3.ctl"]] <- controls.by.channel$Cy3
      intensities.by.channel[["Cy5.ctl"]] <- controls.by.channel$Cy5
      
    }
    else{
      intensities.by.channel$Cy3 <- rbind(intensities.by.channel$Cy3, controls.by.channel$Cy3)
      intensities.by.channel$Cy5 <- rbind(intensities.by.channel$Cy5, controls.by.channel$Cy5)
    }
  }
  return(intensities.by.channel)
}

########################################################################################################################

########################################################################################################################
## batch.quality.R
## created: 2012-06-28
## creator: Pavlo Lutsik
## ---------------------------------------------------------------------------------------------------------------------
## Batch effects associated with quality probe information.
########################################################################################################################

## F U N C T I O N S ###################################################################################################

#' rnb.execute.batch.qc
#'
#' Computation of correlations and permutation-based p-values for detecting quality-associated batch effects.
#'
#' @param rnb.set      HumanMethylation450K dataset as an object of type \code{\linkS4class{RnBeadSet}}.
#' @param pcoordinates Coordinates of the samples of \code{rnb.set} in the principal components space, as returned by
#'                     \code{\link{rnb.execute.dreduction}}.
#' @param permutations Matrix of sample index permutations, as returned by \code{\link{rnb.execute.batcheffects}}. If
#'                     this parameter is \code{NULL}, permutation-based p-values are not calculated.
#' @return \code{NULL} if no principal components for batch analysis are specified (
#'         \code{rnb.getOption("exploratory.principal.components") == 0}); otherwise, a hierarchical structure of
#'         matrices in the form of a nested list. The root branches are represented by the elements
#'         \code{"correlations"} and \code{"pvalues"}. Every element is a list of control probe types; each type is in
#'         turn a list of up to two matrices of correlations between probe values and principal components - one for the
#'         probes on the green channel and one for the red channel. Note that the \code{"pvalues"} branch is not
#'         returned when \code{permutations} is \code{NULL}.
#'
#' @author Pavlo Lutsik
#' @export
rnb.execute.batch.qc <- function(rnb.set, pcoordinates, permutations = NULL) {
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	if (is.null(qc(rnb.set))) {
		stop("invalid value for rnb.set; quality control data is missing")
	}
	nsamples <- validate.pcoordinates.all(pcoordinates)
	if (is.character(nsamples)) {
		stop(nsamples)
	}
	rm(nsamples)
	if (!(is.null(permutations) || is.valid.permutations(permutations))) {
		stop("invalid value for permutations")
	}

	pc.association.count <- rnb.getOption("exploratory.principal.components")
	if (pc.association.count == 0) {
		return(NULL)
	}

	## Initialize data points, control probe types and control probe information
	if(rnb.set@target=="probesEPIC"){
		CONTROL.TYPES <- rnb.infinium.control.targets("probesEPIC")[c(1:4, 6, 11:14)]
		id.col<-"ID"
		type.col<-"Target"
		ctrls<-"controlsEPIC"
	} else if(rnb.set@target=="probesEPICv2"){
		CONTROL.TYPES <- rnb.infinium.control.targets("probesEPICv2")[c(1:4, 6, 11:14)]
		id.col<-"ID"
		type.col<-"Target"
		ctrls<-"controlsEPICv2"
	} else if(rnb.set@target=="probes450"){
		CONTROL.TYPES <- rnb.infinium.control.targets("probes450")[c(1:4, 6, 11:14)]
		id.col<-"ID"
		type.col<-"Target"
		ctrls<-"controls450"
	}else if(rnb.set@target=="probes27"){
		CONTROL.TYPES <- rnb.infinium.control.targets("probes27")[c(1:4, 6, 9:11)]
		id.col<-"Address"
		type.col<-"Type"
		ctrls<-"controls27"
	}else if(rnb.set@target=="probesMMBC"){
        CONTROL.TYPES <- rnb.infinium.control.targets("probesMMBC")[c(1:4, 6, 11:14)]
        id.col<-"ID"
        type.col<-"Target"
        ctrls<-"controlsMMBC"
    }
	control.probe.infos <- rnb.get.annotation(ctrls, rnb.set@assembly)[, c(id.col, type.col)]
	channels <- list("green" = qc(rnb.set)$Cy3, "red" = qc(rnb.set)$Cy5)

	targets <- names(pcoordinates)
	names(targets) <- 1:length(targets)
	results<-lapply(1:length(targets), function(target.id){

		dpoints <- pcoordinates[[target.id]]$pca$x
		if (ncol(dpoints) > pc.association.count) {
			dpoints <- dpoints[, 1:pc.association.count]
		}

		result <- list("correlations" = list())
		if (!is.null(permutations)) {
			result[["pvalues"]] <- list()
		}

		get.probes.channel <- function(channel, ids) {
			i <- intersect(ids, rownames(channel))
			if (length(i) == 0) {
				return(NULL)
			}
			result <- channel[i, ]
			if (length(i) == 1) {
				result <- t(result)
				rownames(result) <- i
			}
			return(result)
		}
		init.matrix <- function(N) {
			matrix(as.double(NA), nrow = N, ncol = ncol(dpoints),
				dimnames = list("Probe" = 1:N, "Principal component" = 1:ncol(dpoints)))
		}

		## Compute correlations between QC probe values and principal components
		target2ids <- tapply(control.probe.infos[[id.col]], control.probe.infos[[type.col]], as.character)
		for (ci in 1:length(CONTROL.TYPES)) {
			ctype <- names(CONTROL.TYPES)[ci]
			ids <- list(target2ids[[CONTROL.TYPES[ci]]])
			p.channels <- mapply(get.probes.channel, channels, ids, SIMPLIFY = FALSE)
			if (all(sapply(p.channels, is.null))) {
				next
			}
			result[["correlations"]][[ctype]] <- list()
			if (!is.null(permutations)) {
				result[["pvalues"]][[ctype]] <- list()
			}
			for (c.channel in names(p.channels)) {
				matrix.channel <- p.channels[[c.channel]]
				if (is.null(matrix.channel)) {
					next
				}
				table.correlations <- init.matrix(nrow(matrix.channel))
				table.pvalues <- init.matrix(nrow(matrix.channel))

				for (i in 1:nrow(matrix.channel)) {
					for (j in 1:ncol(dpoints)) {
						t.result <- test.traits(matrix.channel[i, ], dpoints[, j], permutations)
						table.correlations[i, j] <- t.result[["correlation"]]
						table.pvalues[i, j] <- t.result[["pvalue"]]
					}
				}
				result[["correlations"]][[ctype]][[c.channel]] <- table.correlations
				if (!is.null(permutations)) {
					result[["pvalues"]][[ctype]][[c.channel]] <- table.pvalues
				}
			}
			rnb.status(c("Computed associations between", ncol(dpoints), "principal component(s) and", ctype, "for",
					targets[target.id]))
		}
		attr(result, "Target") <- targets[target.id]
		result
	})

	return(results)
}

########################################################################################################################

#' rnb.section.batch.qc
#'
#' Adds a section on quality-associated batch effects to the given report.
#'
#' @param report         Report to contain the quality-associated section. This must be an object of type
#'                       \code{\linkS4class{Report}}.
#' @param qccorrelations Nested lists of correlation and p-value matrices, as returned by
#'                       \code{\link{rnb.execute.batch.qc}}.
#' @return The modified report.
#'
#' @author Yassen Assenov
#' @noRd
rnb.section.batch.qc <- function(report, qccorrelations) {
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	## TODO: Validate qccorrelations

	targets<-sapply(qccorrelations, attr, "Target")
	report.descriptions <- c("correlations" = as.character(NA), "pvalues" = as.character(NA))

	channels<-NULL
	ctypes<-NULL
	pvals.present<-NULL
	table.pvalue.links<-list()

	all.plots<-lapply(1:length(targets), function(target.id){

		## Create heatmaps of correlations and p-values
		pvals.present <<- ("pvalues" %in% names(qccorrelations[[target.id]]))
		ctypes <<- names(qccorrelations[[target.id]][["correlations"]])
		channels <<- sort(unique(unlist(lapply(qccorrelations[[target.id]][["correlations"]], names))))
		probecount <- max(sapply(unlist(qccorrelations[[target.id]][["correlations"]], recursive = FALSE), nrow))
		imgheight <- list("correlations" = 1 + probecount * 0.4, "pvalues" = 1 + probecount * 0.25)
		report.plots <- list()

		if (pvals.present) {
			## Create a table of links to the p-value matrices
			table.pvalue.links[[target.id]] <<- matrix(as.character(NA), nrow = length(ctypes), ncol = length(channels),
				dimnames = list(ctypes, channels))
		}

		heatmap.functions <- list("correlations" = plot.heatmap.pc.correlations, "pvalues" = plot.heatmap.pc.pvalues)
		for (tblname in names(qccorrelations[[target.id]])) {
			heatmap.function <- heatmap.functions[[tblname]]
			report.plots[[tblname]] <- list()
			for (i in 1:length(ctypes)) {
	#i <- 1
				tables <- qccorrelations[[target.id]][[tblname]][[i]]
				for (cchannel in channels) {
	#cchannel <- "green"
					fname <- paste("heatmap", tblname, "pc", target.id, cchannel, i, sep = "_")
					if (cchannel %in% names(tables)) {
						## Create a heatmap of the table of correlations or pvalues
						rplot <- heatmap.function(report, tables[[cchannel]], fname, height = imgheight[[tblname]])
						if (is.na(report.descriptions[tblname])) {
							report.descriptions[tblname] <<- rplot$description
						}
						rplot <- rplot$plot

						## Save the table of p-values to a file
						if (tblname == "pvalues") {
							fname <- paste("pvalues_pc_", target.id, "_", cchannel, "_", i, ".csv", sep = "")
							fname.full <- file.path(rnb.get.directory(report, "data", absolute = TRUE), fname)
							tbl <- cbind("Probe \\ Principal component" = rownames(tables[[cchannel]]),
								as.data.frame(tables[[cchannel]], check.names = FALSE, stringsAsFactors = FALSE))
							utils::write.csv(tbl, file = fname.full, row.names = FALSE)
							table.pvalue.links[[target.id]][i,grep(cchannel, colnames(table.pvalue.links[[target.id]]))] <<- paste("<a href=\"", rnb.get.directory(report, "data"), "/",
								fname, "\">csv</a>", sep = "")

							rm(fname.full, tbl)
						}
					} else {
						## TODO: Create an image "n/a"
					}
					report.plots[[tblname]] <- c(report.plots[[tblname]], rplot)
				}
			}
		}
#		rm(imgheight, tblname, i, tables, cchannel, fname, rplot)
		report.plots
	})
	setting.names <- list("Location type" = targets, "Channel" = channels, "Probe group" = ctypes)
	names(setting.names[["Location type"]]) <- 1:length(targets)
	names(setting.names[["Channel"]]) <- channels
	names(setting.names[["Probe group"]]) <- 1:length(ctypes)

	txt <- c("The heatmaps below visualize the Pearson correlation coefficients between the principal ",
		"components and the signal levels of selected quality control probes.")
	rnb.add.paragraph(report, txt)

	report.plots.corrs<-unlist(lapply(all.plots, el, "correlations"), recursive = FALSE)
	report.plots.pvals<-unlist(lapply(all.plots, el, "pvalues"), recursive = FALSE)

	report <- rnb.add.figure(report, report.descriptions["correlations"], report.plots.corrs, setting.names)

	if (!is.null(report.plots.pvals) && all(!sapply(report.plots.pvals, is.null))) {
		txt <- c("In a complete analogy to the heatmaps above, the figure below visualizes the p-values calculated ",
			"using permutation tests.")
		rnb.add.paragraph(report, txt)
		report <- rnb.add.figure(report, report.descriptions["pvalues"], report.plots.pvals, setting.names)

		txt <- c("All computed p-values for associations are available as comma-separated files that accompany this ",
			"report. The links to the dedicated files are provided in the table below.")
		rnb.add.paragraph(report, txt)

		table.pvalue.links<-lapply(table.pvalue.links, function(tpl){
					rn<-rownames(table.pvalue.links[[1]])
					tpl<-cbind("Control probe type \\ Target - Channel" = rn, tpl)
					colnames(tpl) <- capitalize(colnames(tpl))
					tpl
				})
		names(table.pvalue.links) <- paste("batchQcPvalueTable", 1:length(targets), sep = "_")

		table.header <- c("<colgroup>", paste0("\t<col width=\"", c(72, 14, 14), "%\" />"), "</colgroup>")
		report <- rnb.add.tables(report, table.pvalue.links, setting.names[1], row.names = FALSE,
			first.col.header = TRUE, thead = table.header)
	}

	return(report)
}

########################################################################################################################

#' rnb.step.batch.qc
#'
#' Detection of quality-associated batch effects.
#'
#' @param rnb.set      HumanMethylation450K dataset as an object of type \code{\linkS4class{RnBeadSet}}.
#' @param report       Report to contain the quality-associated section. This must be an object of type
#'                     \code{\linkS4class{Report}}.
#' @param pcoordinates Coordinates of the samples of \code{rnb.set} in the principal components space, as returned by
#'                     \code{\link{rnb.execute.dreduction}}.
#' @param permutations Matrix of sample index permutations, as returned by \code{\link{rnb.execute.batcheffects}}.
#' @return The modified report.
#'
#' @author Pavlo Lutsik
#' @noRd
rnb.step.batch.qc <- function(rnb.set, report, pcoordinates, permutations = NULL) {
	if (!inherits(rnb.set, "RnBeadSet")){
		stop("invalid value for rnb.set")
	}
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	logger.start("Quality-associated Batch Effects")
	if (!is.null(pcoordinates)) {
		nsamples <- validate.pcoordinates.all(pcoordinates)
		if (is.character(nsamples)) {
			stop(nsamples)
		}
		rm(nsamples)
	}
	if (!(is.null(permutations) || is.valid.permutations(permutations))) {
		stop("invalid value for permutations")
	}

	if (is.null(qc(rnb.set))) {
		## No quality information is available; skip this step
		logger.warning("No quality information available")
		txt <- c("The studied dataset does not include quality probe signal values, therefore, it cannot be examined ",
			"for quality-associated batch effects.")
		report <- rnb.add.section(report, "Quality-associated Batch Effects", txt)
		logger.completed()
		return(report)
	}
	if (is.null(pcoordinates)) {
		txt <- c("No low-dimensional representation of the dataset is available, therefore, it cannot be examined for ",
			"quality-associated batch effects.")
		report <- rnb.add.section(report, "Quality-associated Batch Effects", txt)
		logger.completed()
		return(report)
	}
	txt <- "This section examines the methylation values of the dataset for quality-associated batch effects."
	report <- rnb.add.section(report, "Quality-associated Batch Effects", txt)

	## Compute correlations between QC probes and principal components
	qccorrelations <- rnb.execute.batch.qc(rnb.set, pcoordinates, permutations)

	## Add heatmaps of correlation between QC probes and principal components
	report <- rnb.section.batch.qc(report, qccorrelations)

	logger.status("Added heatmaps of correlation between principal components and signal levels of Q probes")

	logger.completed()
	return(report)
}

########################################################################################################################
## enmix.R
## created: 2016-10-02
## creator: Yassen Assenov
## ---------------------------------------------------------------------------------------------------------------------
## Functions for performing exponential-normal (EN) background subtraction on 450k, EPIC and EPICv2 datasets.
########################################################################################################################

## F U N C T I O N S ###################################################################################################

#' rnb.probe.types
#'
#' Gets the indices of the probes of specific type in the given dataset.
#'
#' @param dataset     Infinium 450k, EPIC or EPICv2 dataset as an instance of \code{\linkS4class{RnBeadRawSet}}.
#' @param include.snp Flag indicating if the SNP probes, if present in the dataset, are to be considered.
#' @return Probe indices, split by types as a \code{list} with three elements: \code{"IGrn"}, \code{"IRed"},
#'         \code{"II"}.
#'
#' @author Yassen Assenov
#' @noRd
rnb.probe.types <- function(dataset, include.snp = TRUE) {
	ptypes <- rnb.get.annotation(dataset@target, dataset@assembly)
	pindices <- tapply(dataset@sites[, 3], dataset@sites[, 2], unname, simplify = FALSE)
	i.chrom <- as.integer(names(pindices))
	probe.types <- list()
	for (i in 1:length(pindices)) {
		probe.types[[i]] <- as.data.frame(mcols(ptypes[[i.chrom[i]]])[pindices[[i]], c("Design", "Color")])
	}
	probe.types <- do.call(rbind, probe.types)
	rm(pindices, i.chrom)
	i.probes <- list()
	for (probe.design in levels(probe.types[, 1])) {
		i <- probe.types[, 1] == probe.design
		if (probe.design == "I") {
			for (probe.color in levels(probe.types[, 2])[-1]) {
				i.probes[[paste0(probe.design, probe.color)]] <-
					which(i & probe.types[, 2] == probe.color)
			}
		} else { # probe.design == "II"
			i.probes[[probe.design]] <- which(i)
		}
	}
	if (!include.snp) {
		is.snp <- which(unlist(lapply(ptypes, function(x) { grepl("^rs", names(x)) }), use.names = FALSE))
		i.probes <- lapply(i.probes, setdiff, y = is.snp)
	}
	i.probes
}

########################################################################################################################

#' rnb.enmix.oob
#' 
#' Runs the Exponential-truncated-normal (ENmix) background subtraction using out-of-band (OOB) intensities to estimate
#' the background (noise) distribution parameters.
#' 
#' @param dataset      Infinium 450k, EPIC or EPICv2 dataset as an instance of \code{\linkS4class{RnBeadRawSet}}.
#' @param update.betas Flag indicating if the beta value matrix is to be updated after correcting the M and U intensity
#'                     values.
#' @return The modified dataset.
#' 
#' @author Yassen Assenov
#' @noRd
rnb.enmix.oob <- function(dataset, update.betas = TRUE) {
	ptypes <- rnb.probe.types(dataset)
	if (parallel.isEnabled()) {
		## TODO: Update fexport after integrating this in RnBeads
		fexport <- c("rnb.enmix.em", "rnb.enmix.cm", "rnb.enmix.adjust", "rnb.enmix.oob.sample")
		results <- foreach(i = 1:nrow(dataset@pheno), .packages	= c("RnBeads", "MASS"), .export = fexport) %dopar%
			rnb.enmix.oob.sample(dataset, i, ptypes = ptypes)
		for (i in 1:length(results)) {
			dataset@M[, i] <- results[[i]][[1]]
			dataset@U[, i] <- results[[i]][[2]]
		}
	} else {
		for (i in 1:nrow(dataset@pheno)) {
			result <- rnb.enmix.oob.sample(dataset, i, ptypes)
			dataset@M[, i] <- result[[1]]
			dataset@U[, i] <- result[[2]]
		}
	}
	if (update.betas) {
		if (dataset@status$disk.dump) {
			dataset@meth.sites <- convert.to.ff.matrix.tmp(beta.value(dataset@M[,], dataset@U[,]))
		} else {
			dataset@meth.sites <- beta.value(dataset@M[,], dataset@U[,])
		}
	}
	dataset
}

########################################################################################################################

#' rnb.enmix.oob.sample
#'
#' Processes a single sample from the given dataset using ENmix.
#'
#' @param dataset  Infinium 450k, EPIC or EPICv2 dataset as an instance of \code{\linkS4class{RnBeadRawSet}}.
#' @param i.sample Index of the sample to be processed.
#' @param ptypes   Probe indices, split by types, as returned by \code{\link{rnb.probe.types}}.
#' @return Corrected intensity values for the i-th sample as a \code{list} of two \code{vectors}: one of the M, and
#'         one for the U values, respectively.
#'
#' @author Yassen Assenov
#' @noRd
rnb.enmix.oob.sample <- function(dataset, i.sample, ptypes = rnb.probe.types(dataset, FALSE)) {
	
	## Construct the resulting vectors
	Mhat <- dataset@M[, i.sample]
	Uhat <- dataset@U[, i.sample]
	
	## Compute estimates mu_B and s_B for the green channel
	oob <- c(dataset@M0[ptypes$IRed, i.sample], dataset@U0[ptypes$IRed, i.sample])
	oob[is.na(oob)] <- 10^(-6) # FIXME: these intensities should be set to zero upon loading
	oob[oob <= 0] <- 10^(-6)
	estimates <- unlist(MASS::huber(oob), use.names = FALSE)
	
	## Fit the green channel intensities and adjust to the fit
	Mhat[ptypes$IGrn] <- rnb.enmix.adjust(Mhat[ptypes$IGrn], estimates[1], estimates[2])
	Uhat[ptypes$IGrn] <- rnb.enmix.adjust(Uhat[ptypes$IGrn], estimates[1], estimates[2])
	Mhat[ptypes$II] <- rnb.enmix.adjust(Mhat[ptypes$II], estimates[1], estimates[2])
	
	## Compute estimates mu_B and s_B for the red channel
	oob <- c(dataset@M0[ptypes$IGrn, i.sample], dataset@U0[ptypes$IGrn, i.sample])
	oob[is.na(oob)] <- 10^(-6) # FIXME: these intensities should be set to zero upon loading
	oob[oob <= 0] <- 10^(-6)
	estimates <- unlist(huber(oob), use.names = FALSE)
	
	## Fit the red channel intensities and adjust to the fit
	Mhat[ptypes$IRed] <- rnb.enmix.adjust(Mhat[ptypes$IRed], estimates[1], estimates[2])
	Uhat[ptypes$IRed] <- rnb.enmix.adjust(Uhat[ptypes$IRed], estimates[1], estimates[2])
	Uhat[ptypes$II] <- rnb.enmix.adjust(Uhat[ptypes$II], estimates[1], estimates[2])
	
	return(list(Mhat, Uhat))
}

########################################################################################################################

#' rnb.enmix.adjust
#' 
#' Adjusts the intensity values of the given sample using Exponential-truncated-normal (EN) mixture.
#' 
#' @param x      Vector of intensity values to be adjusted.
#' @param muB    Estimated mu parameter for the background (noise).
#' @param sigmaB Estimated sigma parameter for the background (noise).
#' @return Corrected vector of intensity values.
#' 
#' @author Zongli Xu and Liang Niu, modified by Yassen Assenov
#' @noRd
rnb.enmix.adjust <- function(x, muB, sigmaB) {
	x[is.na(x)] <- 1
	x[x < 1] <- 1
	
	xx <- x[x >= muB] - muB
	estimates <- rnb.enmix.em(xx)
	lambda <- estimates[1]
	muS <- estimates[2]
	sigmaS <- ifelse(estimates[3] <= sigmaB, 0.1, sqrt(estimates[3]^2 - sigmaB^2))
	p <- (length(x) - length(xx) + estimates[4] * length(xx)) / length(x)
	
	result <- rnb.enmix.cm(lambda, muS, sigmaS, p, muB, sigmaB, x)
	result[result <= 0.01] <- 0.01
	return(result)
}

########################################################################################################################

#' rnb.enmix.em
#'
#' Runs the Expectation Maximization (EM) algorithm to fit a model: p * exp(lambda) + (1 - p) * N+(muS, sigmaS^2).
#'
#' @param x Set of read values, in the form of a \code{double} \code{vector}, against which the model is to be fit.
#' @return \code{vector} of the fitted parameters: lambda, mu, sigma, and p.
#'
#' @author Zongli Xu and Liang Niu, modified by Yassen Assenov
#' @noRd
rnb.enmix.em <- function(x) {
	epsilon = c(1e-04, 0.001, 0.001, 0.001)
	p <- 0.5
	lambda <- max(density(x)$y)
	mu <- mean(range(x))
	sigma2 <- (diff(range(x)) / 6)^2
	n <- as.double(length(x))
	repeat {
		z <- p / (p + (1 - p) * exp(dnorm(x, mu, sqrt(sigma2), log = TRUE) - dexp(x, lambda, log = TRUE)))
		sz <- sum(z)
		p.new <- sz / n
		lambda.new <- sz / sum(x * z)
		z <- 1 - z; sz <- n - sz
		mu.new <- sum(x * z) / sz
		sigma2.new <- sum(((x - mu.new)^2) * z) / sz
		finished <- abs(c(lambda.new - lambda, mu.new - mu, sigma2.new - sigma2, p.new - p))
		finished <- all(finished < epsilon)
		p <- p.new
		lambda <- lambda.new
		mu <- mu.new
		sigma2 <- sigma2.new
		if (finished) { break }
	}
	return(c(lambda, mu, sqrt(sigma2), p))
}

########################################################################################################################

#' rnb.enmix.cm
#' 
#' Computes corrected signal intensities given the observed intensities and EN model parameters.
#' 
#' @param lambda The lambda parameter.
#' @param muS    The mu parameter for the signal.
#' @param sigmaS The sigma paraemter for the signal.
#' @param p      The p parameter.
#' @param muB    The mu parameter for the background (noise).
#' @param sigmaB The sigma parameter for the background (noise).
#' @param x      Observed intensities given as a \code{vector} of type \code{double}.
#' 
#' @author Zongli Xu and Liang Niu, modified by Yassen Assenov
#' @noRd
rnb.enmix.cm <- function(lambda, muS, sigmaS, p, muB, sigmaB, x) {
	a <- muB + lambda * sigmaB^2
	C <- (sigmaS^2 * (x - muB) + sigmaB^2 * muS) / (sigmaS^2 + sigmaB^2)
	D <- sigmaS * sigmaB / sqrt(sigmaS^2 + sigmaB^2)
	E <- pnorm(x, C, D) - pnorm(0, C, D)
	G <- pnorm(x, a, sigmaB) - pnorm(0, a, sigmaB)
	T1 <- p * lambda * exp(lambda^2 * sigmaB^2/2 - lambda * (x - muB)) * G
	T2 <- (1 - p) * dnorm(x, muS + muB, sqrt(sigmaS^2 + sigmaB^2)) / (1 - pnorm(0, muS, sigmaS))
	D1 <- T1 * (x - a + sigmaB * (dnorm((x - a) / sigmaB) - dnorm(a / sigmaB)) / G)
	D2 <- T2 * (C * E + D * (exp(-C^2/(2 * D^2)) - exp(-(x - C)^2 / (2 * D^2))) / sqrt(2 * pi))
	(D1 + D2) / (T1 + T2 * E)
}

########################################################################################################################
## RnBDiffMeth-class.R
## created: 2012-08-30
## creator: Fabian Mueller
## ---------------------------------------------------------------------------------------------------------------------
## RnBDiffMeth class definition.
########################################################################################################################

setClassUnion("characterOrNULL", c("character", "NULL"))

#' RnBDiffMeth Class
#'
#' A class for storing differential methylation data.
#' 
#' @details
#' Contains differential methylation tables (DMT) for multiple comparisons and region types. DMTs can be stored in memory
#' as R objects or on disk
#'
#' @section Slots:
#' \describe{
#'   \item{\code{sites}}{List of differential methylation tables on site level (see \code{computeDiffMeth.bin.site} for details).
#' 					Indexed by comparison.}
#'   \item{\code{regions}}{List of lists of differential methylation tables on region levels (see \code{computeDiffMeth.bin.region} for details).
#' 					Indexed by region type on the top level and comparison on the lower level.}
#'   \item{\code{comparisons}}{character vector of all comparisons stored in the objects. Vector indices correspond to indices in the \code{sites} and
#' 					\code{regions} list slots.}
#'   \item{\code{region.types}}{character vector of all region types stored in the objects. Vector indices correspond to indices in
#' 					the \code{regions} list slot.}
#'   \item{\code{comparison.grouplabels}}{A character matrix with 2 columns containing group labels of all comparisons in the object}
#'   \item{\code{comparison.info}}{A list containing comparison information for each comparison. See \code{\link{get.comparison.info}} for details.}
#'   \item{\code{includesSites}}{Logical indicating whether the object contains site-level differential methylation information.}
#'   \item{\code{site.test.method}}{method which was applied to obtain the site-level p-values.}
#'   \item{\code{variability.method}}{method to be used to detect differentially variable sites.}
#'   \item{\code{covg.thres}}{coverage threshold. Important for certain columns of the differential methylation tables.}
#'   \item{\code{disk.dump}}{Flag indicating whether the tables should be stored on disk rather than in the main memory}
#'   \item{\code{disk.path}}{path on the disk for DMTs.Only meaningful if \code{disk.dump} is \code{TRUE}}
#'   \item{\code{residSites}}{List of residual matrices on site level}
#'   \item{\code{residRegions}}{List of lists of residual matrices on region levels}
#' }
#'
#' @section Methods:
#' \describe{
#'   \item{\code{\link{destroy,RnBDiffMeth-method}}}{remove tables stored to disk from the file system}
#'   \item{\code{\link{get.region.types,RnBDiffMeth-method}}}{Gets all region types represented in the object as character vector}
#' 	 \item{\code{\link{get.comparisons,RnBDiffMeth-method}}}{Gets all comparisons represented in the object as character vector}
#'   \item{\code{\link{get.comparison.grouplabels,RnBDiffMeth-method}}}{Gets all comparison group names as a matrix}
#'   \item{\code{\link{get.covg.thres,RnBDiffMeth-method}}}{Gets the coverage threshold employed for obtaining statistics in the differential methylation tables}
#'   \item{\code{\link{get.table,RnBDiffMeth-method}}}{Gets a differential methylation table}
#'   \item{\code{\link{addDiffMethTable,RnBDiffMeth-method}}}{Adds a differential methylation table}
#' 	 \item{\code{\link{reload,RnBDiffMeth-method}}}{relink disk dumped tables. Useful if the files are manually copied or if the object is loaded again}
#'   \item{\code{\link{save.tables,RnBDiffMeth-method}}}{save disk dumped tables as binaries and zip them.
#' 						Useful if the files are copied or shared.}
#'   \item{\code{\link{join.diffMeth}}}{Merges two disjoint RnBDiffMeth objects into one}
#' }
#'
#' @name RnBDiffMeth-class
#' @rdname RnBDiffMeth-class
#' @author Fabian Mueller
#' @exportClass RnBDiffMeth
setClass("RnBDiffMeth",
	slots = list(
		sites="list",
		regions="list",
		residSites="list",
		residRegions="list",
		comparisons="character",
		region.types="character",
		comparison.grouplabels="matrix",
		comparison.info="list",
		includesSites="logical",
		site.test.method="characterOrNULL",
		variability.method="characterOrNULL",
		covg.thres="integer",
		disk.dump="logical",
		disk.path="characterOrNULL"
	),
	prototype = prototype(
		sites=list(),
		regions=list(),
		comparisons=character(),
		region.types=character(),
		comparison.grouplabels=matrix(ncol=2,nrow=0),
		comparison.info=list(),
		includesSites=FALSE,
		residSites=list(),
		residRegions=list(),
		site.test.method=NULL,
		variability.method=NULL,
		covg.thres=-1L,
		disk.dump=FALSE,
		disk.path=NULL
	),
	package = "RnBeads")

#' initialize.RnBDiffMeth
#'
#' Initialize an RnBDiffMeth object
#' 
#' @param .Object New instance of \code{RnBDiffMeth}.
#' @param site.test.method method which was applied to obtain the site-level p-values.
#' @param covg.thres  coverage threshold. Important for certain columns of the differential methylation tables.
#' 					  See \code{computeDiffMeth.bin.site} and \code{computeDiffMeth.bin.region} for details.
#' @param variability.method method to be used to calculate differentially variable sites. Has to be one of: `diffVar' or 
#'            `iEVORA'.
#' @param disk.dump   Flag indicating whether the tables should be stored on disk rather than in the main memory
#' @param disk.path   Path on the disk for DMTs.Only meaningful if \code{disk.dump} is \code{TRUE}
#'
#' @export
#' @author Fabian Mueller
#' @docType methods
setMethod("initialize", "RnBDiffMeth",
	function(.Object,
		site.test.method=rnb.getOption("differential.site.test.method"),
		variability.method=rnb.getOption("differential.variability.method"),
		covg.thres=rnb.getOption("filtering.coverage.threshold"),
		disk.dump=FALSE,disk.path=NULL){
			#create directory in which to dump the matrices to file. must be a non-existing directory
			if (disk.dump){
				if (!is.null(disk.path)){
					if (!create.path(disk.path, FALSE, showWarnings = FALSE)) {
						stop(paste("Could not create directory (RnBDiffMeth):",disk.path))
					}
				}
			}
			
			.Object@sites <- list()
			.Object@regions <- list()
			.Object@residSites <- list()
			.Object@residRegions <- list()
			.Object@comparisons <- character()
			.Object@region.types <- character()
			.Object@comparison.grouplabels <- matrix(ncol=2,nrow=0)
			.Object@includesSites <- FALSE
			.Object@site.test.method <- site.test.method
			.Object@variability.method <- variability.method
			.Object@covg.thres <- covg.thres
			.Object@comparison.info <- list()
			.Object@disk.dump <- disk.dump
			.Object@disk.path <- disk.path
			.Object
	}
)

#remove the directory containing the dumped matrices
if (!isGeneric("destroy")) setGeneric("destroy", function(object) standardGeneric("destroy"))
#' destroy-methods
#'
#' remove tables stored to disk from the file system. Useful for cleaning up disk dumped objects.
#' CAUTION: currently only works with reloaded objects
#'
#' @param object \code{\linkS4class{RnBDiffMeth}} object
#' @return Nothing of particular interest
#'
#' @rdname destroy-RnBDiffMeth-methods
#' @docType methods
#' @aliases destroy,RnBDiffMeth-method
#' @author Fabian Mueller
#' @export
setMethod("destroy", signature(object="RnBDiffMeth"),
	function(object){
		n.comps <- length(object@comparisons)
		n.region.types <- length(object@region.types)
		if (object@disk.dump){
			logger.start("Deleting RnBDiffMeth disk dump files from disk")
			for (cci in 1:n.comps) {
				cc <- object@comparisons[cci]
				if (!is.null(object@sites[[cci]])){
					delete(object@sites[[cci]])
				}
				if (length(object@residSites) >= cci && !is.null(object@residSites[[cci]])) {
					delete(object@residSites[[cci]])
				}
				if (n.region.types > 0){
					for (rri in 1:n.region.types){
						rr <- object@region.types[rri]
						if (!is.null(object@regions[[rri]][[cci]])){
							delete(object@regions[[rri]][[cci]])
						}
						if (length(object@residRegions) >= rri) {
						  resid_target <- object@residRegions[[rri]]
						  if (length(resid_target) >= cci
							&& !is.null(resid_target[[cci]])
						  ) {
							 delete(resid_target[[cci]])
						  }
						}
					}
				}
			}
			unlink(object@disk.path,recursive=TRUE)
			logger.completed()
		}
	}
)

if (!isGeneric("get.region.types")) setGeneric("get.region.types", function(object) standardGeneric("get.region.types"))
#' get.region.types-methods
#'
#' Gets all region types represented in the object as character vector
#'
#' @param object \code{\linkS4class{RnBDiffMeth}} object
#' @return character vector containing region types
#'
#' @rdname get.region.types-RnBDiffMeth-methods
#' @docType methods
#' @author Fabian Mueller
#' @aliases get.region.types
#' @aliases get.region.types,RnBDiffMeth-method
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
#' get.region.types(dm)
#' }
setMethod("get.region.types", signature(object="RnBDiffMeth"),
	function(object){
		return(object@region.types)
	}
)

if (!isGeneric("get.comparisons")) setGeneric("get.comparisons", function(object) standardGeneric("get.comparisons"))
#' get.comparisons-methods
#'
#' Gets all comparisons represented in the object as character vector
#'
#' @param object \code{\linkS4class{RnBDiffMeth}} object
#' @return character vector containing comparisons
#'
#' @rdname get.comparisons-RnBDiffMeth-methods
#' @docType methods
#' @author Fabian Mueller
#' @aliases get.comparisons
#' @aliases get.comparisons,RnBDiffMeth-method
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
#' get.comparisons(dm)
#' }
setMethod("get.comparisons", signature(object="RnBDiffMeth"),
	function(object){
		return(object@comparisons)
	}
)

if (!isGeneric("get.comparison.grouplabels")) setGeneric("get.comparison.grouplabels", function(object) standardGeneric("get.comparison.grouplabels"))
#' get.comparison.grouplabels-methods
#'
#' Gets all comparison grouplabels represented in the object as character matrix of dimension n.comparisons x 2
#' where the columns specify group names 1 and 2 respectively
#'
#' @param object \code{\linkS4class{RnBDiffMeth}} object
#' @return character matrix containing comparison group names
#'
#' @rdname get.comparison.grouplabels-RnBDiffMeth-methods
#' @docType methods
#' @author Fabian Mueller
#' @aliases get.comparison.grouplabels
#' @aliases get.comparison.grouplabels,RnBDiffMeth-method
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
#' get.comparison.grouplabels(dm)
#' }
setMethod("get.comparison.grouplabels", signature(object="RnBDiffMeth"),
		function(object){
			return(object@comparison.grouplabels)
		}
)
if (!isGeneric("get.comparison.groupsizes")) setGeneric("get.comparison.groupsizes", function(object) standardGeneric("get.comparison.groupsizes"))
#' get.comparison.groupsizes-methods
#'
#' Gets all comparison group sizes represented in the object as character matrix of dimension n.comparisons x 2
#' where the columns specify sizes of groups 1 and 2 respectively
#'
#' @param object \code{\linkS4class{RnBDiffMeth}} object
#' @return character matrix containing comparison group sizes
#'
#' @rdname get.comparison.groupsizes-RnBDiffMeth-methods
#' @docType methods
#' @author Fabian Mueller
#' @aliases get.comparison.groupsizes
#' @aliases get.comparison.groupsizes,RnBDiffMeth-method
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
#' get.comparison.groupsizes(dm)
#' }
setMethod("get.comparison.groupsizes", signature(object="RnBDiffMeth"),
		function(object){
			res <- do.call("rbind",lapply(object@comparisons, FUN=function(ccn){
				cmp.info <- object@comparison.info[[ccn]]
				c(length(cmp.info$group.inds$group1),length(cmp.info$group.inds$group2))
			}))
			return(res)
		}
)

if (!isGeneric("get.site.test.method")) setGeneric("get.site.test.method", function(object) standardGeneric("get.site.test.method"))
#' get.site.test.method-methods
#'
#' Gets the site testing method used to obtain the p-values in the differential methylation tables
#'
#' @param object RnBDiffMeth object
#' @return character describing the site test method
#'
#' @rdname get.site.test.method-RnBDiffMeth-methods
#' @docType methods
#' @author Fabian Mueller
#' @aliases get.site.test.method
#' @aliases get.site.test.method,RnBDiffMeth-method
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
#' get.site.test.method(dm)
#' }
setMethod("get.site.test.method", signature(object="RnBDiffMeth"),
	function(object){
		if (.hasSlot(object,"site.test.method")) { #.hasSlot ensure backwards compatibility
			return(object@site.test.method)
		} else {
			return(rnb.getOption("differential.site.test.method"))
		}
	}
)

if (!isGeneric("get.variability.method")) setGeneric("get.variability.method", function(object) standardGeneric("get.variability.method"))
#' get.variability.method-methods
#'
#' Gets the variability testing method used to obtain the p-values in the differential varibiality tables
#'
#' @param object RnBDiffMeth object
#' @return character describing the variability method
#'
#' @rdname get.variability.method-RnBDiffMeth-methods
#' @docType methods
#' @author Michael Scherer
#' @aliases get.variability.method
#' @aliases get.variability.method,RnBDiffMeth-method
#' @export
setMethod("get.variability.method", signature(object="RnBDiffMeth"),
          function(object){
            if (.hasSlot(object,"variability.method")) { 
              return(object@variability.method)
            } else {
              return(rnb.getOption("differential.variability.method"))
            }
          }
)

if (!isGeneric("get.covg.thres")) setGeneric("get.covg.thres", function(object) standardGeneric("get.covg.thres"))
#' get.covg.thres-methods
#'
#' Gets the coverage threshold employed for obtaining statistics in the differential methylation tables
#'
#' @param object RnBDiffMeth object
#' @return integer coverage threshold
#'
#' @rdname get.covg.thres-RnBDiffMeth-methods
#' @docType methods
#' @author Fabian Mueller
#' @aliases get.covg.thres
#' @aliases get.covg.thres,RnBDiffMeth-method
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
#' get.covg.thres(dm)
#' }
setMethod("get.covg.thres", signature(object="RnBDiffMeth"),
	function(object){
		return(object@covg.thres)
	}
)

if (!isGeneric("includes.sites")) setGeneric("includes.sites", function(object) standardGeneric("includes.sites"))
#' includes.sites-methods
#'
#' Returns \code{TRUE} if the differential methylation object contains site-level information
#'
#' @param object RnBDiffMeth object
#' @return \code{TRUE} if the differential methylation object contains site-level information. \code{FALSE} otherwise
#'
#' @rdname includes.sites-RnBDiffMeth-methods
#' @docType methods
#' @author Fabian Mueller
#' @aliases includes.sites
#' @aliases includes.sites,RnBDiffMeth-method
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
#' includes.sites(dm)
#' }
setMethod("includes.sites", signature(object="RnBDiffMeth"),
	function(object){
		if (.hasSlot(object,"includesSites")) { #.hasSlot ensure backwards compatibility
			return(object@includesSites)
		} else {
			return(all(sapply(object@sites,is.null)))
		}
	}
)

if (!isGeneric("get.table")) setGeneric("get.table", function(object,...) standardGeneric("get.table"))
#' get.table-methods
#'
#' Gets a differential methylation table
#'
#' @param object \code{\linkS4class{RnBDiffMeth}} object
#' @param comparison character or index of the comparison of the table to retrieve
#' @param region.type character or index of the region type of the table to retrieve
#' @param undump Flag indicating whether to convert the table into a matrix instead of using the file descriptor.
#' 		         Only meaningful if the if the objects's \code{disk.dump} slot is true.
#' @param return.data.frame should a data.frame be returned instead of a matrix?
#' @return differential methylation table. See \code{computeDiffMeth.bin.site} and \code{computeDiffMeth.bin.region} for details.
#'
#' @rdname get.table-RnBDiffMeth-methods
#' @docType methods
#' @author Fabian Mueller
#' @aliases get.table
#' @aliases get.table,RnBDiffMeth-method
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
#' dm.promoters <- get.table(dm,get.comparisons(dm)[1],"promoters",return.data.frame=TRUE)
#' summary(dm.promoters)
#' }
setMethod("get.table", signature(object="RnBDiffMeth"),
	function(object,comparison,region.type,undump=TRUE,return.data.frame=FALSE){
		if (!is.element(comparison,object@comparisons)) {
			stop(paste("invalid comparison:",comparison))
		}
		if (!is.element(region.type,c("sites",object@region.types))) {
			stop(paste("invalid region.type:",region.type))
		}
		if (region.type == "sites"){
			res <- object@sites[[comparison]]
		} else {
			res <- object@regions[[region.type]][[comparison]]
		}
		if (object@disk.dump && undump && !is.null(res)){
			res <- res[,]
		}
		if (return.data.frame){
			res <- data.frame(res)
		}
		return(res)
	}
)

if (!isGeneric("addDiffMethTable")) setGeneric("addDiffMethTable", function(object,...) standardGeneric("addDiffMethTable"))
#' addDiffMethTable-methods
#'
#' Adds a differential methylation table
#'
#' @param object \code{\linkS4class{RnBDiffMeth}} object
#' @param dmt    Differential methylation table to add
#' @param comparison character or index of the comparison of the table to retrieve
#' @param grp.labs character vector of length 2 specifying the names of the groups being compared
#' @param region.type character or index of the region type of the table to retrieve
#' @return the updated RnBDiffMeth object
#'
#' @note Caveat: if disk dumping is enabled the resulting object tables will be stored in the initial location of the object.
#' @rdname addDiffMethTable-RnBDiffMeth-methods
#' @docType methods
#' @author Fabian Mueller
#' @aliases addDiffMethTable
#' @aliases addDiffMethTable,RnBDiffMeth-method
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' dm <- rnb.execute.computeDiffMeth(rnb.set.example,"Sample_Group",c("genes","tiling"))
#' s.groups <- rnb.sample.groups(rnb.set.example,"Sample_Group")[[1]]
#' dmt.sites <- computeDiffTab.extended.site(meth(rnb.set.example),s.groups[[1]],s.groups[[2]])
#' map.regions.to.sites <- regionMapping(rnb.set.example,"promoters")
#' dmt.promoters <- computeDiffTab.default.region(dmt.sites,map.regions.to.sites)
#' cmp.name <- get.comparisons(dm)[1]
#' grp.labs <- get.comparison.grouplabels(dm)[1,]
#' #add the promoter level differential methylation table
#' dm.add <- addDiffMethTable(dm,dmt.promoters,cmp.name,"promoters",grp.labs)
#' get.region.types(dm.add)
#' }
setMethod("addDiffMethTable", signature(object="RnBDiffMeth"),
	function(object,dmt,comparison,region.type,grp.labs=c("group1","group2")){
		if(!(is.character(comparison) && length(comparison)==1)) stop("Invalid argument: comparison")
		if(!(is.character(grp.labs) && length(grp.labs)==2)) stop("Invalid argument: grp.labs")
		if(!(is.character(region.type) && length(region.type)==1)) stop("Invalid argument: region.type")
		if(!(is.data.frame(dmt) || is.matrix(dmt))) stop("Invalid argument: dmt")
		
		disk.dump <- object@disk.dump
		disk.path <- object@disk.path
		res.opt <- rnb.getOption("differential.residuals.output")
		
		#check if there is already an entry for the diff meth table
		if (region.type == "sites"){
			if (!is.null(object@sites[[comparison]])) {
				warning(paste("DiffMethTable already exists for combination:",comparison," , sites . Returning unmodified object"))
				return(object)
			}
		} else {
			if (!is.null(object@regions[[region.type]][[comparison]])) {
				warning(paste("DiffMethTable already exists for combination:",comparison,",",reg.type,". Returning unmodified object"))
				return(object)
			}
		}
		
		#check if the comparison is already there
		if(!is.element(comparison,object@comparisons)){
			object@comparisons <- c(object@comparisons,comparison)
			n.comps <- length(object@comparisons)
			names(object@comparisons)[n.comps] <- paste0("cmp",n.comps)
			object@comparison.grouplabels <- rbind(object@comparison.grouplabels,grp.labs)
			rownames(object@comparison.grouplabels) <- object@comparisons
			cmp.i <- length(object@comparisons)
			#append comparison info
			object@comparison.info <- c(object@comparison.info,list(NULL))
			names(object@comparison.info)[cmp.i] <- comparison
			#append the comparison to all sites and regions objects
			object@sites <- c(object@sites,list(NULL))
			names(object@sites)[cmp.i] <- comparison
			# append placeholder for residuals
			object@residSites <- c(object@residSites,list(NULL))
			names(object@residSites)[cmp.i] <- comparison
			for (rr in object@region.types){
				object@regions[[rr]] <- c(object@regions[[rr]],list(NULL))
				names(object@regions[[rr]])[cmp.i] <- comparison
				object@residRegions[[rr]] <- c(object@residRegions[[rr]],list(NULL))
				names(object@residRegions[[rr]])[cmp.i] <- comparison
			}
		}
		cmp.i <- which(object@comparisons == comparison) #index of the comparison
		cmp.fname <- paste0("cmp",cmp.i)
		
		if (region.type == "sites"){
			reg.i <- 0
			table.obj <- as.matrix(dmt)
			rownames(table.obj) <- NULL #make certain that rownames do not exist. they are inefficient when using ff
			if (disk.dump){
				#create the ff matrix object
				fileN <- paste0(paste("sites",cmp.fname,sep="_"),".ff")
				table.obj <- ff(table.obj,dim=dim(table.obj),dimnames=dimnames(table.obj),filename=file.path(disk.path,fileN))
			}
			object@sites[[comparison]] <- table.obj

			# if a residual matrix attribute is present on the data.frame, store it (ff if dumped)
			resid_mat <- attr(dmt, "resid_mat")
			if (!is.null(res.opt) && !is.null(resid_mat)){
					if (disk.dump && !is.null(disk.path)){
						fileN <- paste0(paste("residuals_site",cmp.fname,sep="_"),".ff")
						res_ff <- ff(resid_mat, dim=dim(resid_mat), dimnames=dimnames(resid_mat), filename=file.path(disk.path,fileN))
						object@residSites[[comparison]] <- res_ff
					} else {
						# fallback to in-memory storage if ff requested but no disk dump
						object@residSites[[comparison]] <- resid_mat
					}
			}

			object@includesSites <- TRUE
		} else {
			#check if the region type is already there
			if(!is.element(region.type,object@region.types)){
				object@region.types <- c(object@region.types,region.type)
				n.reg.types <- length(object@region.types)
				names(object@region.types)[n.reg.types] <- paste0("reg",n.reg.types)
				empty.cmp.list.4.regs <- rep(list(NULL),length(object@comparisons))#empty list of comparisons
				names(empty.cmp.list.4.regs) <- object@comparisons
				object@regions <- c(object@regions,list(empty.cmp.list.4.regs))
				names(object@regions)[length(object@regions)] <- region.type
				object@residRegions <- c(object@residRegions,list(empty.cmp.list.4.regs))
				names(object@residRegions)[length(object@residRegions)] <- region.type
			}
			reg.i <- which(object@region.types == region.type)
			reg.dir.name <- paste0("reg",reg.i)
			table.obj <- as.matrix(dmt)
			rownames(table.obj) <- NULL #make certain that rownames do not exist. they are inefficient when using ff
			if (disk.dump){
				#create the ff matrix object
				fileN <- paste0(paste("regions",reg.dir.name,cmp.fname,sep="_"),".ff")
				table.obj <- ff(table.obj,dim=dim(table.obj),dimnames=dimnames(table.obj),filename=file.path(disk.path,fileN))
			}
			object@regions[[region.type]][[comparison]] <- table.obj

				# if a residual matrix attribute is present on the data.frame, store it (ff if dumped)
				resid_mat <- attr(dmt, "resid_mat")
				if (!is.null(res.opt) && !is.null(resid_mat)){
						# logger.info("addTable: residual matrix found in the input table. storing it in the RnBDiffMeth object.")
						if (disk.dump && !is.null(disk.path)){
							fileN <- paste0(paste("residuals_region",reg.dir.name,cmp.fname,sep="_"),".ff")
							res_ff <- ff(resid_mat, dim=dim(resid_mat), dimnames=dimnames(resid_mat), filename=file.path(disk.path,fileN))
							object@residRegions[[region.type]][[comparison]] <- res_ff
						} else {
							object@residRegions[[region.type]][[comparison]] <- resid_mat
						}
					
				}
		}
		
		return(object)
	}
)

if (!isGeneric("save.tables")) setGeneric("save.tables", function(object,...) standardGeneric("save.tables"))
#' save.tables-methods
#'
#' save the disk dumped tables to an ff archive for later reloading
#'
#' @param object \code{\linkS4class{RnBDiffMeth}} object
#' @param file path on the disk to save to.
#' @return success
#'
#' @rdname save.tables-RnBDiffMeth-methods
#' @docType methods
#' @author Fabian Mueller
#' @aliases save.tables
#' @aliases save.tables,RnBDiffMeth-method
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' pcols <- c("Sample_Group","Treatment")
#' tdir <- tempfile()
#' dm <- rnb.execute.computeDiffMeth(rnb.set.example,pcols,disk.dump=TRUE,disk.dump.dir=tdir)
#' save.tables(dm,tempfile())
#' }
setMethod("save.tables", signature(object="RnBDiffMeth"),
	function(object,file){
		n.comps <- length(object@comparisons)
		n.region.types <- length(object@region.types)
		if (object@disk.dump){
			ee <- new.env()
			for (cci in 1:n.comps) {
				cc <- object@comparisons[cci]
				ccn <- paste0("cmp",cci)
				if (includes.sites(object)){
					if (!is.null(object@sites[[cci]])){
						ee[[paste("sites",ccn,sep=".")]] <- object@sites[[cci]]
					}
					if (!is.null(object@residSites[[cci]])){
						ee[[paste("residuals.sites",ccn,sep=".")]] <- object@residSites[[cci]]
					}
				}
				if (n.region.types > 0){
					for (rri in 1:n.region.types){
						rr <- object@region.types[rri]
						rrn <- paste0("reg",rri)
						if (!is.null(object@regions[[rri]][[cci]])){
							ee[[paste("regions",rrn,ccn,sep=".")]] <- object@regions[[rri]][[cci]]
						}
						if (!is.null(object@residRegions[[rri]][[cci]])){
							ee[[paste("residuals.regions",rrn,ccn,sep=".")]] <- object@residRegions[[rri]][[cci]]
						}
					}
				}
			}
			ffsave(list=ls(ee),envir=ee,rootpath=object@disk.path,file=file)
		}
	}
)
			
if (!isGeneric("reload")) setGeneric("reload", function(object,...) standardGeneric("reload"))
#' reload-methods
#'
#' reload disk dumped tables. Useful if the table files are manually copied or if the object is loaded again.
#'
#' @param object \code{\linkS4class{RnBDiffMeth}} object
#' @param save.file location of the ff data saved to disk (i.e. save in save.RData and save.ffData)
#' @param disk.path path on the disk for DMTs. can be new or be the same as in the original object
#' @return the updated RnBDiffMeth object
#'
#' @rdname reload-RnBDiffMeth-methods
#' @docType methods
#' @author Fabian Mueller
#' @aliases reload
#' @aliases reload,RnBDiffMeth-method
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' #compute differential methylation
#' pcols <- c("Sample_Group","Treatment")
#' tdir <- tempfile(pattern="working")
#' dm <- rnb.execute.computeDiffMeth(rnb.set.example,pcols,disk.dump=TRUE,disk.dump.dir=tdir)
#' #get temporary file names
#' fn.save.tabs <- tempfile(pattern="saveTables")
#' fn.save.obj  <- tempfile(pattern="saveObject")
#' #save the object and the tables to disk
#' save(dm,file=fn.save.obj)
#' save.tables(dm,fn.save.tabs)
#' #delete the object from the workspace
#' destroy(dm)
#' rm(dm)
#' #reload the object and tables
#' load(fn.save.obj)
#' dm.new <- reload(dm,fn.save.tabs)
#' }
setMethod("reload", signature(object="RnBDiffMeth"),
	function(object,save.file,disk.path=tempfile(pattern="diffmeth_", tmpdir=getOption('fftempdir'))){
		if (!object@disk.dump){
			warning("RnBDiffMeth object is not dumped to disk. Returning unmodified object")
			return(object)
		}

		n.comps <- length(object@comparisons)
		n.region.types <- length(object@region.types)
		
		logger.start("Relinking RnBDiffMeth object")
		object@disk.path <- disk.path
		
		ee <- new.env(parent=emptyenv())
		ffload(save.file,rootpath=disk.path,envir=ee)
		
		for (cci in 1:n.comps) {
			cc <- object@comparisons[cci]
			ccn <- paste0("cmp",cci)
			site.obj.name <- paste("sites",ccn,sep=".")
			if (includes.sites(object)){
				if (exists(site.obj.name,ee)){
					object@sites[[cci]] <- get(site.obj.name,ee)
				} else {
					logger.warning(c("Could not relink:","sites","--",cc))
					object@sites[cci] <- list(NULL)
				}
			} else {
				object@sites[cci] <- list(NULL)
			}
			# relink residuals for sites if present
			resid.site.name <- paste("residuals.sites",ccn,sep=".")
			if (exists(resid.site.name,ee)){
				object@residSites[[cci]] <- get(resid.site.name,ee)
			} else {
				object@residSites[cci] <- list(NULL)
			}
			if (n.region.types > 0){
				for (rri in 1:n.region.types){
					rr <- object@region.types[rri]
					rrn <- paste0("reg",rri)
					reg.obj.name <- paste("regions",rrn,ccn,sep=".")
					if (exists(reg.obj.name,ee)){
						object@regions[[rri]][[cci]] <- get(reg.obj.name,ee)
					} else {
						logger.warning(c("Could not relink:",rr,"--",cc))
						object@regions[[rri]][cci] <- list(NULL)
					}
						resid.reg.name <- paste("residuals.regions",rrn,ccn,sep=".")
						if (exists(resid.reg.name,ee)){
							object@residRegions[[rri]][[cci]] <- get(resid.reg.name,ee)
						} else {
							object@residRegions[[rri]][cci] <- list(NULL)
						}
				}
			}
		}
		logger.completed()
		
		return(object)
	}
)

#' save.rnb.diffmeth
#'
#' save an \code{\linkS4class{RnBDiffMeth}} object to disk
#'
#' @param object \code{\linkS4class{RnBDiffMeth}} object
#' @param path path on the disk to save to.
#' @author Fabian Mueller
#' @aliases save.rnb.diffmeth
#' @export
save.rnb.diffmeth <- function(object,path){
	if (file.exists(path)){
		stop("Saving unsuccessfull. Target directory already exists")
	}

	dir.create(path, showWarnings=FALSE, recursive=FALSE)
	save(object,file=file.path(path,"rnbDiffMeth.RData"))
	if (object@disk.dump){
		if(.Platform$OS == "windows" && Sys.getenv("R_ZIPCMD")==""){
			rnb.warning(c("Zip not found on this Windows system, this RnBDiffMeth object will not be saved.",
					"See the instructions for installing ZIP on Windows in the FAQ section of the RnBeads website."))
			return(invisible(FALSE))
		}
		save.tables(object,file.path(path,"rnbDiffMeth_tables"))
	}
}
#' load.rnb.diffmeth
#'
#' load a saved \code{\linkS4class{RnBDiffMeth}} object from disk
#'
#' @param path path of the saved object (a directory containing a corresponding \code{rnbDiffMeth.RData} file and possibly \code{rnbDiffMeth_tables} files)
#' @return the loaded \code{\linkS4class{RnBDiffMeth}} object
#' @author Fabian Mueller
#' @aliases load.rnb.diffmeth
#' @export
load.rnb.diffmeth <- function(path){
	if (!file.exists(path)){
		stop("Loading unsuccessfull. Path does not exist")
	}
	load.env <- new.env(parent=emptyenv())
	load(file.path(path,"rnbDiffMeth.RData"),envir=load.env)
	object <- get("object", load.env)
	if (object@disk.dump){
		object <- reload(object,file.path(path,"rnbDiffMeth_tables"))
	}
	return(object)
}

setGeneric("addComparisonInfo", function(object,...) standardGeneric("addComparisonInfo"))

#' addComparisonInfo-methods
#'
#' Adds a differential methylation comparison information.
#'
#' @param object \code{\linkS4class{RnBDiffMeth}} object
#' @param cmp.info	Comparison information as returned by \code{\link{get.comparison.info}}
#' @return the updated RnBDiffMeth object
#'
#' @rdname addComparisonInfo-RnBDiffMeth-methods
#' @docType methods
#' @author Fabian Mueller
#' @aliases addComparisonInfo,RnBDiffMeth-method
#' @noRd
setMethod("addComparisonInfo", signature(object="RnBDiffMeth"),
	function(object,cmp.info){
		if (!is.list(cmp.info)) {
			stop("invalid value for cmp.info. expected list")
		}
		if (length(cmp.info) < 1 || length(names(cmp.info)) < 1) {
			stop("invalid value for cmp.info: list is too short or is unnamed")
		}
		if (!all(names(cmp.info) %in% names(object@comparison.info))) {
			stop(paste("cmp.info does not match comparisons. The following comparison names are not",
					   "present in the RnBDiffMeth object: ",
					   paste(setdiff(names(cmp.info),names(object@comparison.info)),collapse=","))
			)
		}
		for (cn in names(cmp.info)){
			if (!is.null(cmp.info[[cn]])) {
				if (is.null(object@comparison.info[[cn]])) {
					object@comparison.info[[cn]] <- cmp.info[[cn]]
				} else {
					warning(paste("Did not overwrite comparison information for comparison:",cn,""))
				}
			}
		}
		return(object)
	}
)


if (!isGeneric("join.diffMeth")) setGeneric("join.diffMeth", function(obj1,obj2,...) standardGeneric("join.diffMeth"))
#' join.diffMeth-methods
#'
#' Merges two disjoint RnBDiffMeth objects into one. Disjoint here means, that no differential methylation table is specified in both
#' objects.
#'
#' @param obj1 \code{\linkS4class{RnBDiffMeth}} object. Its base properties will be used to create the joint object
#' 			   this is particularly imported for disk dumped objects as its path will be used and tables from the second
#' 			   object will be copied there
#' @param obj2 \code{\linkS4class{RnBDiffMeth}} object
#' @return the merged \code{\linkS4class{RnBDiffMeth}} object
#'
#' @note Caveat: if disk dumping is enabled the resulting object tables will be stored in the initial location of the first object to be joined
#' I.e. deleting the first object will lead to a broken joined object and deleting the joined object will lead to an broken first object.
#' @rdname join.diffMeth-methods
#' @docType methods
#' @author Fabian Mueller
#' @aliases join.diffMeth
#' @aliases join.diffMeth,RnBDiffMeth,RnBDiffMeth-method
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' dm1 <- rnb.execute.computeDiffMeth(rnb.set.example,"Sample_Group",c("genes","tiling"))
#' dm2 <- rnb.execute.computeDiffMeth(rnb.set.example,c("Sample_Group","Treatment"),"promoters")
#' dm.join1 <- join.diffMeth(dm1,dm2)
#' #The following joint object is invalid due to missing region type - comparison combinations
#' is.valid(dm.join1)
#' dm3 <- rnb.execute.computeDiffMeth(rnb.set.example,"Treatment",c("genes","tiling"))
#' dm.join2 <- join.diffMeth(dm.join1,dm3)
#' #After joining the missing information, the new object is valid
#' is.valid(dm.join2)
#' }
setMethod("join.diffMeth", signature(obj1="RnBDiffMeth",obj2="RnBDiffMeth"),
	function(obj1,obj2){
		is.compatible <- (includes.sites(obj1) == includes.sites(obj2)) &&
						 (obj1@site.test.method == obj2@site.test.method) &&
						 (obj1@covg.thres == obj2@covg.thres) && 
		         (obj1@variability.method == obj2@variability.method) &&
						 (obj1@disk.dump == obj2@disk.dump)
		if (!is.compatible){
			stop("incompatible RnBDiffMeth objects")
		}
#		logger.start("Combining RnBDiffMeth objects")
		res <- obj1
		new.regs <- setdiff(obj2@region.types,obj1@region.types)
		n.new.regs <- length(new.regs)
		res@region.types <- c(res@region.types,new.regs)
		names(res@region.types) <- paste0("reg",1:length(res@region.types))
		
		new.comps <- setdiff(obj2@comparisons,obj1@comparisons)
		n.new.comps <- length(new.comps)
		res@comparisons <- c(res@comparisons,new.comps)
		names(res@comparisons) <- paste0("cmp",1:length(res@comparisons))
		new.comps.gns <- obj2@comparison.grouplabels[new.comps,]
		res@comparison.grouplabels <- rbind(res@comparison.grouplabels,new.comps.gns)
		rownames(res@comparison.grouplabels) <- res@comparisons
		
		#check for conflicting definitions, i.e. if anything is defined in both sets
		comps.intersect <- intersect(obj1@comparisons,obj2@comparisons)
		region.types.intersect <- intersect(obj1@region.types,obj2@region.types)
		#sites
		is.def.obj1 <- !vapply(obj1@sites[comps.intersect],is.null,FALSE)
		is.def.obj2 <- !vapply(obj2@sites[comps.intersect],is.null,FALSE)
		#comparison information: don't overwrite obj1
		cmp.info.names.obj1 <- names(obj1@comparison.info)
		cmp.info.names.obj2 <- names(obj2@comparison.info)
		cmp.info.names.intersect <- intersect(cmp.info.names.obj1,cmp.info.names.obj2)
		if (length(cmp.info.names.intersect)>0) {
			warning(paste("Join RnBDiffMeth: Comparison info defined in both objects:",cmp.info.names.intersect,
						  "--> keeping obj1"))
		}
		res@comparison.info <- rep(list(NULL),length(res@comparisons))
		names(res@comparison.info) <- res@comparisons
		res <- suppressWarnings(addComparisonInfo(res,obj1@comparison.info))
		res <- suppressWarnings(addComparisonInfo(res,obj2@comparison.info))

		#is any comparison defined on both objects: site level
		is.def.both <- (is.def.obj1+is.def.obj2)>1
		if (any(is.def.both)) {
			warning(paste("Join RnBDiffMeth: Comparison on site level defined in both objects:",comps.intersect[is.def.both],
						  "--> keeping obj1"))
		}
		#regions
		for (rr in region.types.intersect){
			is.def.obj1 <- !vapply(obj1@regions[[rr]][comps.intersect],is.null,FALSE)
			is.def.obj2 <- !vapply(obj2@regions[[rr]][comps.intersect],is.null,FALSE)
			#is any comparison defined on both objects
			is.def.both <- (is.def.obj1+is.def.obj2)>1
			if (any(is.def.both)) {
				warning(paste("Join RnBDiffMeth: Comparison on region (",rr,") level defined in both objects:",comps.intersect[is.def.both],
							  "--> keeping obj1"))
			}
		}
		#add empty lists for sites and regions
		new.comp.list <- rep(list(NULL),n.new.comps)
		names(new.comp.list) <- new.comps
		if (includes.sites(obj1)) res@sites <- c(res@sites,new.comp.list)
		for (rr in obj1@region.types){
			res@regions[[rr]] <- c(res@regions[[rr]],new.comp.list) 
		}
		n.all.comps <- length(res@comparisons)
		new.reg.list <- rep(list(NULL),n.all.comps)
		names(new.reg.list) <- res@comparisons
		new.reg.list.list <- rep(list(new.reg.list),n.new.regs)
		names(new.reg.list.list) <- new.regs
		res@regions <- c(res@regions,new.reg.list.list)
		
		iis <- match(obj2@comparisons,res@comparisons)
		jjs <- match(obj2@region.types,res@region.types)
		for (i.old in 1:length(obj2@comparisons)){
			i.new <- iis[i.old]
			cc <- obj2@comparisons[i.old]
			
			cmp.fname <- paste0("cmp",i.new)
			if (!is.null(obj2@sites[[cc]]) && is.null(res@sites[[cc]])) {
				#if dumped, copy the matrix of obj2 to the location of obj1
				if (obj2@disk.dump) {
					base.path <- res@disk.path
					fileN <- paste0(paste("sites",cmp.fname,sep="_"),".ff")
					res@sites[[cc]] <- clone(obj2@sites[[cc]],vmode=vmode(obj2@sites[[cc]]),filename=file.path(base.path,fileN))
				} else {
					res@sites[[cc]] <- obj2@sites[[cc]]
				}
			}
			for (j.old in 1:length(obj2@region.types)){
				j.new <- jjs[j.old]
				rr <- obj2@region.types[j.old]
				reg.dir.name <- paste0("reg",j.new)
				if (!is.null(obj2@regions[[rr]][[cc]]) && is.null(res@regions[[rr]][[cc]])){
					#if dumped, copy the matrix of obj2 to the location of obj1
					if (obj2@disk.dump) {
						base.path <- res@disk.path
						fileN <- paste0(paste("regions",reg.dir.name,cmp.fname,sep="_"),".ff")
						res@regions[[rr]][[cc]] <- clone(obj2@regions[[rr]][[cc]],vmode=vmode(obj2@regions[[rr]][[cc]]),filename=file.path(base.path,fileN))
					} else {
						res@regions[[rr]][[cc]] <- obj2@regions[[rr]][[cc]]
					}
				}
			}
		}
#		logger.completed()
		return(res)
	}
)

if (!isGeneric("is.valid")) setGeneric("is.valid", function(object,...) standardGeneric("is.valid"))
#' is.valid-methods
#'
#' Validate an RnBDiffMeth object, ie. verify that all differential methylation tables are specified
#' and accounted for
#'
#' @param object \code{\linkS4class{RnBDiffMeth}} object
#' @param verbose print more info to the logger
#' @return TRUE iff all differential methylation tables are present and accounted for
#'
#' @rdname is.valid-RnBDiffMeth-methods
#' @docType methods
#' @author Fabian Mueller
#' @aliases is.valid
#' @aliases is.valid,RnBDiffMeth-method
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' dm1 <- rnb.execute.computeDiffMeth(rnb.set.example,"Sample_Group",c("genes","tiling"))
#' dm2 <- rnb.execute.computeDiffMeth(rnb.set.example,c("Sample_Group","Treatment"),"promoters")
#' dm.join1 <- join.diffMeth(dm1,dm2)
#' #The following joint object is invalid due to missing region type - comparison combinations
#' is.valid(dm.join1)
#' dm3 <- rnb.execute.computeDiffMeth(rnb.set.example,c("Treatment"),c("genes","tiling"))
#' dm.join2 <- join.diffMeth(dm.join1,dm3)
#' #After joining the missing information, the new object is valid
#' is.valid(dm.join2)
#' }
setMethod("is.valid", signature(object="RnBDiffMeth"),
	function(object,verbose=FALSE){
		n.comps <- length(object@comparisons)
		n.region.types <- length(object@region.types)
		
		for (cci in 1:n.comps) {
			cc <- object@comparisons[cci]
			ccn <- paste0("cmp",cci)
			if (includes.sites(object)){ #.hasSlot for backwards compatibility
				if (is.null(object@sites[[cc]])){
					if (verbose) logger.info(paste0("No table found for comparison '",cc,"' (sites)"))
					return(FALSE)
				}
				if (object@disk.dump){
					fileN <- file.path(object@disk.path,paste0(paste("sites",ccn,sep="_"),".ff"))
					if (!file.exists(fileN)){
						if (verbose) logger.info(paste0("Disk dump file ['",fileN,"'] not found for comparison '",cc,"' (sites)"))
						return(FALSE)
					}
				}
			}
			if (n.region.types > 0){
				for (rri in 1:n.region.types){
					rr <- object@region.types[rri]
					rrn <- paste0("reg",rri)
					if (is.null(object@regions[[rr]][[cc]])){
						if (verbose) logger.info(paste0("No table found for comparison '",cc,"' (region: '",rr,"')"))
						return(FALSE)
					}
					if (object@disk.dump){
						fileN <- file.path(object@disk.path,paste0(paste("regions",rrn,ccn,sep="_"),".ff"))
						if (!file.exists(fileN)){
							if (verbose) logger.info(paste0("Disk dump file ['",fileN,"'] not found for comparison '",cc,"' (region: '",rr,"')"))
							return(FALSE)
						}
					}
				}
			}
		}
		return(TRUE)
	}
)

library(argparse)
suppressPackageStartupMessages(library(RnBeads))

ap <- ArgumentParser()
ap$add_argument("-x", "--xml", action="store", help="Configuration xml file")
ap$add_argument("-s", "--rnbSet", action="store", dest="rnbSetLoc", help="Location of stored RnBSet")
ap$add_argument("-d", "--diffMethChunkId", action="append", dest="diffMethChunkIds", help="Ids for chunks for which differential methylation has already been computed. Will be used to construct the input file locations.")
ap$add_argument("-o", "--output", action="store", help="Output directory")
ap$add_argument("-c", "--cores", action="store", type="integer", default=1, help="Number of cores used for the analysis")
cmdArgs <- ap$parse_args()
module.name <- "differential_wrapup"

logger.start(fname=NA)
logger.status(c("...Started module:",module.name))

logger.start("Configuring Analysis")
	rnb.settings <- rnb.xml2options(cmdArgs$xml,return.full.structure=TRUE)

	report.dir <- rnb.settings$analysis.params[["dir.reports"]]
	analysis.options <- rnb.settings$options
	dm.chunk.ids <- cmdArgs$diffMethChunkIds

	if ("preanalysis.script" %in% names(rnb.settings)){
		source(rnb.settings$preanalysis.script)
	} 
	## Set options
	if (length(analysis.options) != 0) {
		do.call(rnb.options, analysis.options)
	}

	logger.machine.name()

	if (cmdArgs$cores > 1) {
		parallel.setup(cmdArgs$cores)
	}

	aname <- rnb.getOption("analysis.name")
	if (!(is.null(aname) || is.na(aname) || nchar(aname) == 0)) {
		logger.info(c("Analysis Title:", aname))
	}
	ncores <- parallel.getNumWorkers()
	if (ncores == -1) {
		ncores <- 1L
	}
	logger.info(c("Number of cores:", ncores))
	rm(aname, ncores)
logger.completed()

logger.start("Loading RnBSet")
	rnb.set <- load.rnb.set(cmdArgs$rnbSetLoc)
logger.completed()

logger.start(fname=c(file.path(report.dir,paste0("analysis_",module.name,".log")),NA))

################################################################################
# main script
################################################################################
if (length(dm.chunk.ids)<1){
	stop("Specify at least one id for a differential methylation chunk")
}

disk.dump <- rnb.getOption("disk.dump.big.matrices")

logger.start("Loading and Combining DiffMeth Files")
	chunk.id <- dm.chunk.ids[1]
	diffmeth.path <- file.path(cmdArgs$output,paste0("differential_chunk_",chunk.id,"_rnbDiffMeth"))
	logger.info(c("Loading:",diffmeth.path))
	diffmeth <- load.rnb.diffmeth(diffmeth.path)
	diffmeth.go.enrichment <- NULL
	enrichment.go.file <- file.path(diffmeth.path, "enrichment_go.RData")
	if (file.exists(enrichment.go.file)){
		logger.info(c("Loading:",enrichment.go.file))
		load(enrichment.go.file) #loads 'diffmeth.go.enrichment' object
	}
	diffmeth.lola.enrichment <- NULL
	enrichment.lola.file <- file.path(diffmeth.path, "enrichment_lola.RData")
	if (file.exists(enrichment.lola.file)){
		logger.info(c("Loading:",enrichment.lola.file))
		load(enrichment.lola.file) #loads 'diffmeth.lola.enrichment' object
	}

	diffmeth.res <- list(report=NULL,diffmeth=diffmeth,dm.go.enrich=diffmeth.go.enrichment,dm.lola.enrich=diffmeth.lola.enrichment)

	rm(diffmeth,diffmeth.go.enrichment,diffmeth.lola.enrichment) # to make sure a result.diffmeth object is not stored twice if a following file does not contain one
	RnBeads:::rnb.cleanMem()

	if (length(dm.chunk.ids)>1){
		for (chunk.id in dm.chunk.ids[2:length(dm.chunk.ids)]){
			diffmeth.path <- file.path(cmdArgs$output,paste0("differential_chunk_",chunk.id,"_rnbDiffMeth"))
			logger.info(c("Loading:",diffmeth.path))
			diffmeth <- load.rnb.diffmeth(diffmeth.path)
			enrichment.go.file <- file.path(diffmeth.path, "enrichment_go.RData")
			diffmeth.go.enrichment <- NULL
			if (file.exists(enrichment.go.file)){
				logger.info(c("Loading:",enrichment.go.file))
				load(enrichment.go.file) #loads 'diffmeth.go.enrichment' object
			}
			enrichment.lola.file <- file.path(diffmeth.path, "enrichment_lola.RData")
			diffmeth.lola.enrichment <- NULL
			if (file.exists(enrichment.lola.file)){
				logger.info(c("Loading:",enrichment.lola.file))
				load(enrichment.lola.file) #loads 'diffmeth.lola.enrichment' object
			}
			diffmeth.res.cur <- list(report=NULL,diffmeth=diffmeth,dm.go.enrich=diffmeth.go.enrichment,dm.lola.enrich=diffmeth.lola.enrichment)

			ll <- list(diffmeth.res,diffmeth.res.cur)
			diffmeth.res <- RnBeads:::combine.diffMeth.objs(ll)

			destroy(diffmeth)
			rm(diffmeth.res.cur,diffmeth,diffmeth.go.enrichment,diffmeth.lola.enrichment) # to make sure a result.diffmeth object is not stored twice if a following file does not contain one
			RnBeads:::rnb.cleanMem()
		}
	}
	if (!is.valid(diffmeth.res$diffmeth,verbose=TRUE)){
		stop("RnBDiffMeth object is invalid")
	}
logger.completed()

logger.start("Saving combined data")
	diffmeth.path <- file.path(cmdArgs$output,paste0("differential_rnbDiffMeth"))
	save.rnb.diffmeth(diffmeth.res$diffmeth, diffmeth.path)
	diffmeth.go.enrichment <- diffmeth.res$dm.go.enrich
	if (!is.null(diffmeth.go.enrichment)){
		save(diffmeth.go.enrichment, file=file.path(diffmeth.path, "enrichment_go.RData"))
	}
	diffmeth.lola.enrichment <- diffmeth.res$dm.lola.enrich
	if (!is.null(diffmeth.lola.enrichment)){
		save(diffmeth.lola.enrichment, file=file.path(diffmeth.path, "enrichment_lola.RData"))
	}
logger.completed()

logger.start("Differential Methylation")
	logger.start("Report Generation")
		aname <- rnb.getOption("analysis.name")
		if (!(is.null(aname) || is.na(aname) || nchar(aname) == 0)) {
			logger.info(c("Analysis Title:", aname))
		}
		ncores <- parallel.getNumWorkers()
		if (ncores == -1) {
			ncores <- 1L
		}
		logger.info(c("Number of cores:", ncores))
		rm(aname, ncores)

		diffmeth <- diffmeth.res$diffmeth
		dm.go.enrich <- diffmeth.res$dm.go.enrich
		dm.lola.enrich <- diffmeth.res$dm.lola.enrich

		init.configuration <- !file.exists(file.path(report.dir, "configuration"))
		report <- RnBeads:::init.pipeline.report("differential_methylation", report.dir, init.configuration)
		optionlist <- rnb.options("analyze.sites","region.types", "differential.permutations", "differential.comparison.columns",
			"differential.comparison.columns.all.pairwise","columns.pairing","differential.site.test.method","covariate.adjustment.columns",
			"differential.adjustment.sva","differential.adjustment.celltype","differential.enrichment.go","differential.enrichment.lola", "differential.enrichment.lola.dbs")
		report <- RnBeads:::rnb.add.optionlist(report, optionlist)
		
		if (is.null(diffmeth)){
			txt <- "Differential methylation analyis was skipped because no valid grouping information could be found."
			report <- rnb.add.section(report, "Differential Methylation Analysis", txt)
		} else {
			gz <- rnb.getOption("gz.large.files")
			includeSites <- rnb.getOption("analyze.sites") && rnb.getOption("differential.report.sites")
			report <- RnBeads:::rnb.section.diffMeth.introduction(diffmeth,report)
			if (includeSites){
				report <- RnBeads:::rnb.section.diffMeth.site(rnb.set,diffmeth,report,gzTable=gz)
			}
			if (length(get.region.types(diffmeth))>0){
				report <- RnBeads:::rnb.section.diffMeth.region(rnb.set,diffmeth,report,dm.go.enrich=dm.go.enrich,dm.lola.enrich=dm.lola.enrich,gzTable=gz)
			}
		}

		off(report)

	logger.completed()
logger.completed()

logger.status(c("...Completed module:",module.name))
quit(save='no')

########################################################################################################################
## sex.R
## created: 2014-02-28
## creator: Yassen Assenov
## ---------------------------------------------------------------------------------------------------------------------
## Functions to infer sex from Infinium 450k, EPIC, EPICv2 and bisulfite sequencing data, and to visualize the results.
########################################################################################################################

## G L O B A L S #######################################################################################################

## Columns added to the sample annotation table when sex is predicted
RNB.COLUMNS.PREDICTED.SEX <- c("Predicted Male Probability", "Predicted Sex")

## Coefficients of a logistic regression model to predict sex based on signal increases of the sex chromosomes
RNB.SEX.COEFFICIENTS <- c(-3, 3, -1)

## Coefficients of a logistic regression model to predict sex based different coverages on the sex chromosomes
RNB.SEX.COEFFICIENTS.BISEQ <- c(2.06, 1.66, -0.23)

## F U N C T I O N S ###################################################################################################

#' rnb.contains.sex
#'
#' Checks if the given dataset contains sites on sex chromosomes.
#'
#' @param rnb.set Methylation dataset as an object of type inheriting \code{RnBSet}.
#' @return \code{TRUE} if the dataset contains sites on the X or Y chromosome; \code{FALSE} otherwise.
#' @author Yassen Assenov
#' @noRd
rnb.contains.sex <- function(rnb.set) {
	ci.sex <- which(rnb.get.chromosomes(rnb.set@assembly) %in% c("X", "Y"))
	any(rnb.set@sites[, 2] %in% ci.sex)
}

########################################################################################################################

#' rnb.get.XY.shifts
#'
#' Calculates the increase of signal on the sex chromosomes in the given dataset.
#'
#' @param rnb.set     Dataset of interest. Currently only \code{\linkS4class{RnBeadRawSet}} is supported.
#' @param signal.type Matrix or matrices to use in calculating the shifts. Currently ignored.
#' @return Calculated shifts in the form of a matrix in which every row corresponds to a sample in \code{rnb.set} and
#'         the columns denote the shifts in the X and Y chromosomes.
#'
#' @noRd
#' @author Yassen Assenov
rnb.get.XY.shifts <- function(rnb.set, signal.type = "raw") {

  target <- rnb.set@target
  genome.build <- rnb.set@assembly
	probes.bad <- rnb.get.annotation(target, rnb.set@assembly)
	probes.max <- sapply(probes.bad, length)
	if (genome.build == "hg38") {
		#' There is no 'cross-active' annotation available for probes in the hg38 annotation
		if(target == 'probesEPIC'){
	  		probes.bad <- lapply(probes.bad, function(x) { which((mcols(x)[, "SNPs 3 Alternative"] != 0)) })
		} else if(target == 'probesEPICv2'){
			probes.bad <- lapply(probes.bad, function(x) { which((mcols(x)[, "SNPs 3 Alternative"] != 0)) })
		} else if (target == 'probes450'){
			probes.bad <- lapply(probes.bad, function(x) { which((mcols(x)[, "SNPs 3"] != 0)) })
		}
	} else { ## hg19
		if(target == 'probesEPIC'){
	  		#' There is no 'cross-active' annotation available for probes in the EPIC annotation
			probes.bad <- lapply(probes.bad, function(x) { which((mcols(x)[, "SNPs 3"] != 0)) })
		} else {
	  		probes.bad <- lapply(probes.bad, function(x) { which((mcols(x)[, "SNPs 3"] != 0) | (mcols(x)[, "Cross-reactive"] != 0)) })
		}
	}
	probes.max <- probes.max - sapply(probes.bad, length)

	## Identify the indices of sites on the sex chromosomes and autosomes
	sex.chroms <- c("X" = "chrX", "Y" = "chrY")
	tokeep <- tapply(rnb.set@sites[, 3], rnb.set@sites[, 2], unname)
	names(tokeep) <- names(probes.bad)[as.integer(names(tokeep))]
	if (!all(c(sex.chroms) %in% names(tokeep))) {
		return(NULL)
	}
	isgood <- function(x, y) match(x, y, nomatch = 0L) == 0L
	tokeep <- mapply(isgood, x = tokeep, y = probes.bad[names(tokeep)], SIMPLIFY = FALSE)
	inds <- cumsum(sapply(tokeep, length))
	inds <- cbind("last" = inds, "first" = 1L + c(0L, unname(inds[-length(inds)])))
	ii <- lapply(sex.chroms, function(cn) { (inds[cn, "first"]:inds[cn, "last"])[tokeep[[cn]]] })
	ii$autosome <- setdiff((1:nrow(rnb.set@sites))[unlist(tokeep, use.names = FALSE)], unlist(ii, use.names = FALSE))
	rm(tokeep, isgood, inds)

	## Validate that not too many sites are missing
	probes.max <- c(probes.max[sex.chroms], sum(probes.max) - sum(probes.max[sex.chroms]))
	min.fraction <- 0.2
	if (!all(sapply(ii, length) / probes.max >= min.fraction)) {
		return(NULL)
	}

	## Calculate signal shifts
	shifts <- t(apply(rnb.set@M[, , drop = FALSE] + rnb.set@U[, , drop = FALSE], 2, function(x) {
			t.signals <- sapply(ii, function(i) { mean(x[i], na.rm = TRUE) })
			c(t.signals[1], t.signals[2]) - t.signals[3]
		})
	)
	return(shifts / 1000)
}

########################################################################################################################

#' rnb.get.XY.shifts.biseq
#'
#' Calculates the increase of coverage on the sex chromosomes in the given sequencing dataset.
#'
#' @param rnb.set     Dataset of interest. Only \code{\linkS4class{RnBiseqSet}} applicable here.
#' @return Calculated shifts in the form of a matrix in which every row corresponds to a sample in \code{rnb.set} and
#'         the columns denote the shifts in the X and Y chromosomes.
#'
#' @author Michael Scherer
#' @noRd
rnb.get.XY.shifts.biseq <- function(rnb.set) {
  
  #probes.bad <- rnb.get.annotation('CpG', rnb.set@assembly)
  probes.bad <- annotation(rnb.set)
  probes.bad <- GRanges(Rle(probes.bad$Chromosome),IRanges(start = probes.bad$Start,end = probes.bad$End),strand=Rle(probes.bad$Strand),probes.bad[,-(1:4)])
  probes.bad <- split(probes.bad,seqnames(probes.bad))
  probes.max <- sapply(probes.bad, length)
  if("SNPs"%in%colnames(mcols(probes.bad[[1]]))){
    probes.bad <- lapply(probes.bad, function(x) { which((mcols(x)[, "SNPs"] != 0)) })
    probes.max <- probes.max - sapply(probes.bad, length)
  }else{
    probes.bad <- lapply(probes.bad,function(x)which(!is.null(mcols(x))))
  }
  
  ## Identify the indices of sites on the sex chromosomes and autosomes
  sex.chroms <- c("X" = "chrX", "Y" = "chrY")
  tokeep <- tapply(rnb.set@sites[, 3], rnb.set@sites[, 2], unname)
  names(tokeep) <- names(probes.bad)[as.integer(names(tokeep))]
  if (!all(c(sex.chroms) %in% names(tokeep))) {
    return(NULL)
  }
  isgood <- function(x, y) match(x, y, nomatch = 0L) == 0L
  tokeep <- mapply(isgood, x = tokeep, y = probes.bad[names(tokeep)], SIMPLIFY = FALSE)
  inds <- cumsum(sapply(tokeep, length))
  inds <- cbind("last" = inds, "first" = 1L + c(0L, unname(inds[-length(inds)])))
  ii <- lapply(sex.chroms, function(cn) { (inds[cn, "first"]:inds[cn, "last"])[tokeep[[cn]]] })
  ii$autosome <- setdiff((1:nrow(rnb.set@sites))[unlist(tokeep, use.names = FALSE)], unlist(ii, use.names = FALSE))
  rm(tokeep, isgood, inds)
  
  ## Validate that not too many sites are missing
  probes.max <- c(probes.max[sex.chroms], sum(probes.max) - sum(probes.max[sex.chroms]))
  min.fraction <- 0.1
  if (!all(sapply(ii, length) / probes.max >= min.fraction)) {
    return(NULL)
}
  
  ## Calculate signal shifts
#  shifts <- t(apply(rnb.set@covg.sites[, , drop = FALSE], 2, function(x) {
#    t.signals <- sapply(ii, function(i) { mean(x[i], na.rm = TRUE) })
#    c(t.signals[1], t.signals[2]) - t.signals[3]
#  })
#  )
  # Use the resource saving version
  shifts <- t(sampleCovgApply(rnb.set,fn=function(x){
	t.signals <- sapply(ii, function(i) { mean(x[i], na.rm = TRUE) })
   c(t.signals[1], t.signals[2]) - t.signals[3]
 }))
  ## Scale the shifts for coverage differences, the number is the average coverage of the data
  ## set on which logistic regression coefficients were calculated
  scale <- mean(sampleCovgApply(rnb.set,function(x)mean(x,na.rm=TRUE)),na.rm=TRUE)
  return(shifts/(scale/10))
}

sampleCovgApply <- function(object, fn, type="sites", ...) {
		if (!(is.character(type) && length(type) == 1 && (!is.na(type)))) {
			stop("invalid value for type")
		}
		if (type %in% c("sites", object@target)) {
			result <- nrow(object@covg.sites)
		} else if (!(type %in% names(object@regions))) {
			stop("unsupported region type")
		}
		res <- sapply(1:length(samples(object)), FUN=function(j){
			fn(covg(object, type=type, j=j), ...)
		})
		return(res)
	}

########################################################################################################################

#' rnb.set.update.predicted.sex
#'
#' Adds two columns (RNB.COLUMNS.PREDICTED.SEX) to the sample annotation of the given methylation dataset, based
#' on the calculated methylation shifts. Also sets the sex inferred covariate to \code{TRUE}.
#'
#' @param rnb.set         Dataset of interest. Currently only \code{\linkS4class{RnBeadRawSet}} is supported.
#' @param shifts          Matrix of calculated mean signal increases, as returned by \code{\link{rnb.get.XY.shifts}}.
#' @param pr.coefficients 3-element vector storing the coefficients of the logistic regression model used for the
#'                        prediction of sex. The first element of this vector must denote the intercept.
#' @return The possibly modified dataset with two columns added to its sample annotation table. If \code{shifts} is
#'         \code{NULL}, the returned dataset is \code{rnb.set}.
#'
#' @author Yassen Assenov
#' @noRd
rnb.set.update.predicted.sex <- function(rnb.set, shifts, pr.coefficients = RNB.SEX.COEFFICIENTS) {
  if(inherits(rnb.set,'RnBiseqSet')){
    pr.coefficients <- RNB.SEX.COEFFICIENTS.BISEQ
  }
	if (!is.null(shifts)) {
		male.probabilities <- as.vector(1 / (1 + exp(shifts %*% pr.coefficients[-1] + pr.coefficients[1])))
		rnb.set@pheno[, RNB.COLUMNS.PREDICTED.SEX[1]] <- male.probabilities
		p.sex <- factor(ifelse(male.probabilities > 0.5, "male", "female"), levels = c("female", "male", "unknown"))
		p.sex[is.na(p.sex)] <- "unknown"
		rnb.set@pheno[, RNB.COLUMNS.PREDICTED.SEX[2]] <- p.sex
		rnb.set@inferred.covariates$sex <- TRUE
	}
	rnb.set
}

########################################################################################################################

#' rnb.execute.sex.prediction
#'
#' Infers the sex of every sample in the given dataset, based on average signal intensity values on
#' the autosomes and the sex chromosomes.
#'
#' @param rnb.set Methylation dataset as an object of type \code{\linkS4class{RnBeadRawSet}}.
#' @return The possibly modified dataset. If sex could be predicted, the sample annotation table is enriched with
#' two more columns - \code{"Predicted Male Probability"} and \code{"Predicted Sex"}.
#'
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' rnb.set.example <- rnb.execute.sex.prediction(rnb.set.example)
#' table(rnb.set.example[, "Predicted Sex"])
#' }
#' @author Yassen Assenov
#' @export
rnb.execute.sex.prediction <- function(rnb.set) {
  if(!inherits(rnb.set,"RnBiseqSet") && !inherits(rnb.set, "RnBeadRawSet")){
    stop("invalid value for rnb.set")
  }
	if (inherits(rnb.set, "RnBeadRawSet")) {
	  if (rnb.set@target != "probes450" && rnb.set@target != "probesEPIC" && rnb.set@target != "probesEPICv2") {
		  stop("unsupported platform")
	  }
	  shifts <- rnb.get.XY.shifts(rnb.set)
	}else{
	  shifts <- rnb.get.XY.shifts.biseq(rnb.set)
	}
  
	if (is.null(shifts)) {
		rnb.warning("Prediction skipped due to incomplete data")
	}
	rnb.set.update.predicted.sex(rnb.set, shifts)
}

########################################################################################################################

#' rnb.execute.gender.prediction
#' 
#' Deprecated function name, now called \code{\link{rnb.execute.sex.prediction}}.
#' @param rnb.set Methylation dataset after running the sex prediction step, as an object of type
#'                \code{\linkS4class{RnBSet}}.
#' @return The possibly modified dataset. If sex could be predicted, the sample annotation table is enriched with
#' @seealso rnb.execute.sex.prediction
rnb.execute.gender.prediction <- function(rnb.set){
  rnb.execute.sex.prediction(rnb.set)
}

########################################################################################################################

#' rnb.section.sex.prediction
#'
#' Adds a dedicated section on sex prediction results to the given report.
#'
#' @param rnb.set Methylation dataset after running the sex prediction step, as an object of type
#'                \code{\linkS4class{RnBSet}}.
#' @param shifts  Matrix of calculated mean signal increases, as returned by \code{\link{rnb.get.XY.shifts}}.
#' @param report  Report on annotation inferrence to contain the sex prediction section. This must be an object of
#'                type \code{\linkS4class{Report}}.
#' @return The modified report.
#'
#' @seealso \code{\link{rnb.execute.sex.prediction}} for performing sex prediction
#' @author Yassen Assenov
#' @noRd
rnb.section.sex.prediction <- function(rnb.set, shifts, report) {
	if (inherits(rnb.set, "RnBeadRawSet") || inherits(rnb.set, "RnBiseqSet")) {
		if (is.null(shifts)) {
			if (rnb.contains.sex(rnb.set)) {
			  if(inherits(rnb.set,'RnBeadRawSet')){
				  txt <- c("Sex prediction was not performed because the dataset contains too few sites. Please note ",
					          "that sex prediction is started when the dataset contains reliable measurements for at least ",
				          	"20% of the sites on autosomes and on each sex chromosome.")
			  } else {
			    txt <- c("Sex prediction was not performed because the dataset contains too few sites. Please note ",
			             "that sex prediction is started when the dataset contains reliable measurements for at least ",
			             "1% of the sites on autosomes and on each sex chromosome.")
			  }
			} else {
				txt <- c("Sex prediction was not performed because the dataset contains no sites on sex ",
					"chromosomes. The prediction is based on signal intensities on the sex chromosomes relative to ",
					"the ones on autosomes.")
				if (rnb.getOption("filtering.sex.chromosomes.removal")) {
					txt <- c(txt, " Please disable sex chromosomes removal (analysis option ",
						"<code>filtering.sex.chromosomes.removal</code>) in order to enable sex prediction.")
				}
			}
		} else {
			pred.sex <- rnb.set@pheno[, RNB.COLUMNS.PREDICTED.SEX[2]]
			colors.sex <- c(muted("pink"), muted("blue"), "#808080")
			names(colors.sex) <- levels(pred.sex)
			pred.sex <- table(pred.sex)
			txt <- c('RnBeads predicted the sex of the samples in the dataset using a logistic regression model. ',
				'The results are summarized in the table below.')
			pred.sex <- data.frame("Sex" = names(pred.sex), "Samples" = as.integer(pred.sex),
				check.names = FALSE, stringsAsFactors = FALSE)
		}
	} else {
		txt <- c("Sex prediction is skipped because this operation is not supported for the dataset. Currently, ",
			"sex prediction can be performed only on raw Infinium 450k, EPIC and EPICv2 datasets, as well as sequencing data sets.", 
      "These are, for example, datasets loaded from IDAT or BED-like files.")
		return(report)
	}
	report <- rnb.add.section(report, "Sex Prediction", txt)

	if (exists("colors.sex", inherits = FALSE)) {
		rnb.add.table(report, pred.sex)

		## Display the shifts and the separation line
	  if(inherits(rnb.set,'RnBeadRawSet')){
	  	txt <- c("Sex was predicted based on the increase (or decrease) of mean signal intensities in the sex ",
			"chromosomes w.r.t. the corresponding value in autosomes. The figure below displays these characteristics ",
			"of the samples.")
	  } else {
	    txt <- c("Sex was predicted based on the average coverage (number of reads) at the sex ",
	             "chromosomes w.r.t. the corresponding value in autosomes. The figure below displays these characteristics ",
	             "of the samples.")
	  }
		rnb.add.paragraph(report, txt)
		s.colorings <- c("prob" = "predicted male probability", "sex" = "predicted sex")
		dframe <- data.frame(
			prob = rnb.set@pheno[, RNB.COLUMNS.PREDICTED.SEX[1]],
			sex = rnb.set@pheno[, RNB.COLUMNS.PREDICTED.SEX[2]], X = shifts[, 1], Y = shifts[, 2])
		rplots <- list()
		for (s.coloring in names(s.colorings)) {
			a.labels <- paste("Mean signal increase in the", c("X", "Y"), "chromosome")
			c.label <- gsub("predicted ", "", s.colorings[s.coloring], fixed = TRUE)
			pp <- ggplot2::ggplot(dframe, aes_string(x = 'X', y = 'Y', color = s.coloring)) + ggplot2::geom_point() +
				ggplot2::coord_fixed() + ggplot2::labs(x = a.labels[1], y = a.labels[2], color = c.label)
			if (s.coloring == "prob") {
				pp <- pp + ggplot2::scale_color_gradient2(limits = c(0, 1), low = colors.sex[1], mid = "white",
						high = colors.sex[2], midpoint = 0.5, na.value = colors.sex[3])
			} else {
				pp <- pp + ggplot2::scale_color_manual(na.value = colors.sex[3], values = colors.sex)
			}
			if(inherits(rnb.set,'RnBeadRawSet')){
			  xslope <- -RNB.SEX.COEFFICIENTS[2] / RNB.SEX.COEFFICIENTS[3]
			  intc <- -RNB.SEX.COEFFICIENTS[1] / RNB.SEX.COEFFICIENTS[3]
			  pp <- pp + ggplot2::geom_abline(intercept = intc, slope = xslope) +
			    ggplot2::theme(plot.margin = unit(0.1 + c(0, 1, 0, 0), "in")) +
			    ggplot2::theme(legend.position = c(1, 0.5), legend.justification = c(0, 0.5))
			} else {
			  xslope <- -RNB.SEX.COEFFICIENTS.BISEQ[2] / RNB.SEX.COEFFICIENTS.BISEQ[3]
			  intc <- -RNB.SEX.COEFFICIENTS.BISEQ[1] / RNB.SEX.COEFFICIENTS.BISEQ[3]
			  pp <- pp + ggplot2::geom_abline(intercept = intc, slope = xslope) +
			    ggplot2::theme(plot.margin = unit(0.1 + c(0, 1, 0, 0), "in")) +
			    ggplot2::theme(legend.position = c(1, 0.5), legend.justification = c(0, 0.5))+
			    ggplot2::xlab('Mean coverage increase in the X chromosome')+
			    ggplot2::ylab('Mean coverage increase in the Y chromosome')
			}
			fname <- paste0("sex_prediction_signals_", s.coloring)
			rplot <- createReportPlot(fname, report, width = 8.2, height = 7.2)
			print(pp)
			rplots <- c(rplots, off(rplot))
		}
		if(inherits(rnb.set,'RnBeadRawSet')){
		  txt <- c("Sex prediction based on mean signal increase. The decision boundary between the two sexes is ",
			"visualized by a black line. Sample colors denote predicted male probability / sex.")
		} else {
		  txt <- c("Sex prediction based on coverage differences. The decision boundary between the two sexes is ",
		           "visualized by a black line. Sample colors denote predicted male probability / sex.")
		}
		report <- rnb.add.figure(report, txt, rplots, list("Colors denote" = s.colorings))
		rm(pred.sex, txt, s.colorings, dframe, rplots, s.coloring, pp, fname, rplot)
	}
	report
}

########################################################################################################################

#' rnb.step.sex.prediction
#'
#' Executes the sex prediction step and adds a dedicated section to the report.
#'
#' @param rnb.set Methylation dataset as an object of type \code{\linkS4class{RnBSet}}.
#' @param report  Report on annotation inferrence to contain the sex prediction section. This must be an object of
#'                type \code{\linkS4class{Report}}.
#' @return List of two elements:
#'         \describe{
#'           \item{\code{"dataset"}}{The dataset, possibly enriched with predicted sex annotation.}
#'           \item{\code{"report"}}{The modified report.}
#'         }
#'
#' @note This function is not used because sex prediction is performed in the data import module, whereas results are
#'       moved to a different module - quality control.
#' @author Yassen Assenov
#' @noRd
rnb.step.sex.prediction <- function(rnb.set, report) {
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}

	if (inherits(rnb.set, "RnBeadRawSet")) {
		shifts <- rnb.get.XY.shifts(rnb.set)
	} else if (inherits(rnb.set, "RnBiseqSet")){
		shifts <- rnb.get.XY.shifts.biseq(rnb.set)
	}
	rnb.set <- rnb.set.update.predicted.sex(rnb.set, shifts)
	report <- rnb.section.sex.prediction(rnb.set, shifts, report)
	return(list(dataset = rnb.set, report = report))
}

########################################################################################################################
## filteringSummary.R
## created: 2013-12-12
## creator: Yassen Assenov
## ---------------------------------------------------------------------------------------------------------------------
## Implementation of the summary section after all filtering steps have been performed.
########################################################################################################################

## F U N C T I O N S ###################################################################################################

## rnb.get.filtered.sites.samples
##
## Validates the given datasets before and after filtering and extracts the lists of filtered sites and samples.
##
## @param old.set Methylation dataset before filtering as an object of type inheriting \code{\linkS4class{RnBSet}}.
## @param new.set Methylation dataset after filtering as an object of type inheriting \code{\linkS4class{RnBSet}}.
## @return \code{list} of three elements: \code{"mm"}, \code{"samples"} and \code{"sites"}.
##
## @author Yassen Assenov
rnb.get.filtered.sites.samples <- function(old.set, new.set) {

	## Validate that new.set is a subset of old.set
	mm.old <- meth(old.set, row.names = TRUE)
	mm.new <- meth(new.set, row.names = TRUE)
	sites.old <- rownames(mm.old)
	sites.new <- rownames(mm.new)
	samples.old <- colnames(mm.old)
	samples.new <- colnames(mm.new)
	rm(mm.new)
	if (length(setdiff(sites.new, sites.old)) != 0) {
		stop("inconsistent sites in old.set and new.set")
	}
	if (length(setdiff(samples.new, samples.old)) != 0) {
		stop("inconsistent samples in old.set and new.set")
	}

	removed.samples <- which(!(samples.old %in% samples.new))
	removed.sites <- which(!(sites.old %in% sites.new))
	return(list(mm = mm.old, samples = removed.samples, sites = removed.sites))
}

########################################################################################################################

#' rnb.filter.dataset
#' 
#' Modifies the given methylation dataset after a sequence of filtering steps has been performed and summarized.
#' 
#' @param rnb.set   Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param r.samples Indices of all samples to be removed as a result of performing filtering steps.
#' @param r.sites   Indices of all sites/probes to be removed as a result of performing filtering steps.
#' @param mask      Optionally, masking of low quality measures to be applied. If specified, this parameter must be a
#'                  \code{matrix} of \code{logical} values with dimensions corresponding to the size of \code{rnb.set}.
#' @return The possibly modified dataset.
#' 
#' @author Yassen Assenov
#' @noRd
rnb.filter.dataset <- function(rnb.set, r.samples, r.sites, mask = NULL) {
	logger.start("Manipulating the object")
	needs.summary <- (!is.null(mask))
	if (needs.summary) {
		rnb.set <- mask.sites.meth(rnb.set, mask)
		logger.status("Updated NA masking")
	}
	if (length(r.samples) != 0) {
		if(rnb.getOption("enforce.destroy.disk.dumps")){
			rnb.set@status$discard.ff.matrices<-TRUE
		}
		rnb.set <- remove.samples(rnb.set, r.samples)
		if(isTRUE(rnb.set@status$discard.ff.matrices)){
			rnb.set@status$discard.ff.matrices<-NULL
		}
		# needs.summary <- FALSE
		logger.status(sprintf("Removed %d samples", length(r.samples)))
	}
	if (length(r.sites) != 0) {
		if(rnb.getOption("enforce.destroy.disk.dumps")){
			rnb.set@status$discard.ff.matrices<-TRUE
		}
		rnb.set <- remove.sites(rnb.set, r.sites)
		# rnb.set <- remove.sites(rnb.set, r.sites, verbose=TRUE) #DEBUG
		if(isTRUE(rnb.set@status$discard.ff.matrices)){
			rnb.set@status$discard.ff.matrices<-NULL
		}
		needs.summary <- FALSE # remove.sites already takes care of region summaries
		logger.status(sprintf("Removed %d sites (probes)", length(r.sites)))
	}
	if (needs.summary) {
		rnb.set <- updateRegionSummaries(rnb.set)
		logger.status("Updated region-level data")
	}
	logger.info(c("Retained", nsites(rnb.set), "sites and", length(samples(rnb.set)), "samples"))
	logger.completed()
	rnb.set
}

########################################################################################################################

#' rnb.execute.filter.summary
#'
#' Calculates a table summarizing the effect of the applied filtering procedures.
#'
#' @param old.set Methylation dataset before filtering as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param new.set Methylation dataset after filtering as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @return \code{matrix} summarizing the number of removed and retained sites, samples, and (optionally) reliable and
#'         unreliable measurements.
#'
#' @details
#' This function expects that the sites and samples in \code{new.set} are subsets of the sites and samples in
#' \code{old.set}, respectively. If this is not the case, it exists with an error.
#'
#' @seealso \code{\link{rnb.run.preprocessing}} for running the whole preprocessing module
#'
#' @author Yassen Assenov
#' @export
rnb.execute.filter.summary <- function(old.set, new.set) {
	if (!inherits(old.set, "RnBSet")) {
		stop("invalid value for old.set")
	}
	if (!inherits(new.set, "RnBSet")) {
		stop("invalid value for new.set")
	}

	removed <- rnb.get.filtered.sites.samples(old.set, new.set)
	rnb.execute.filter.summary.internal(old.set, removed$samples, removed$sites)
}

rnb.execute.filter.summary.internal <- function(rnb.set, removed.samples, removed.sites) {
	cont.matrix <- rbind(
		"Sites" = as.double(c(nsites(rnb.set) - length(removed.sites), length(removed.sites))),
		"Samples" = as.double(c(length(samples(rnb.set)) - length(removed.samples), length(removed.samples))))
	rownames(cont.matrix)[1] <- capitalize(rnb.get.row.token(class(rnb.set), plural = TRUE))
	colnames(cont.matrix) <- c("Retained", "Removed")
	if (rnb.has.reliability.info(rnb.set)) {
		cmatrix <- matrix(0, nrow = 2, ncol = 2)
		rownames(cmatrix) <- paste(c("Reliable", "Unreliable"), "measurements")

		if (length(removed.sites) != 0) {
			relCounts.remSites <- rnb.get.reliability.counts.per.sample(rnb.set, removed.sites)
			notRelCounts.remSites <- length(removed.sites) - relCounts.remSites
			relCounts.retSites <- rnb.get.reliability.counts.per.sample(rnb.set, -removed.sites)
			notRelCounts.retSites <- nsites(rnb.set) - length(removed.sites) - relCounts.retSites
			if (length(removed.samples) != 0) {
				cmatrix[1,1] <- sum(as.numeric(relCounts.retSites[-removed.samples]))
				cmatrix[1,2] <- sum(as.numeric(relCounts.remSites[-removed.samples])) + sum(as.numeric(relCounts.retSites[removed.samples]) + as.numeric(relCounts.remSites[removed.samples]))
				cmatrix[2,1] <- sum(as.numeric(notRelCounts.retSites[-removed.samples]))
				cmatrix[2,2] <- sum(as.numeric(notRelCounts.remSites[-removed.samples])) + sum(as.numeric(notRelCounts.retSites[removed.samples]) + as.numeric(notRelCounts.remSites[removed.samples]))
			} else {
				cmatrix[1,1] <- sum(as.numeric(relCounts.retSites))
				cmatrix[1,2] <- sum(as.numeric(relCounts.remSites))
				cmatrix[2,1] <- sum(as.numeric(notRelCounts.retSites))
				cmatrix[2,2] <- sum(as.numeric(notRelCounts.remSites))
			}
		} else { # length(removed.sites) == 0
			relCounts <- rnb.get.reliability.counts.per.sample(rnb.set)
			notRelCounts <- nsites(rnb.set) - relCounts
			if (length(removed.samples) != 0) {
				cmatrix[1,1] <- sum(as.numeric(relCounts[-removed.samples]))
				cmatrix[1,2] <- sum(as.numeric(relCounts[ removed.samples]))
				cmatrix[2,1] <- sum(as.numeric(notRelCounts[-removed.samples]))
				cmatrix[2,2] <- sum(as.numeric(notRelCounts[ removed.samples]))
			} else {
				cmatrix[1,1] <- sum(as.numeric(relCounts))
				cmatrix[1,2] <- as.numeric(0)
				cmatrix[2,1] <- sum(as.numeric(notRelCounts))
				cmatrix[2,2] <- as.numeric(0)
			}
		}
		cont.matrix <- rbind(cont.matrix, cmatrix)
	}
	cbind(cont.matrix, "Total" = rowSums(cont.matrix))
}

########################################################################################################################

#' rnb.step.filter.summary
#'
#' Calculates a table summarizing the effect of the applied filtering procedures and adds a corresponding section to the
#' given report.
#'
#' @param old.set Methylation dataset before filtering as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param new.set Methylation dataset after filtering as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param report  Report to summarize the outcome of this procedure. This must be an object of type
#'                \code{\linkS4class{Report}}.
#' @return The report, possibly modified.
#'
#' @details
#' This function expects that the sites and samples in \code{new.set} are subsets of the sites and samples in
#' \code{old.set}, respectively. If this is not the case, it exists with an error. If \code{old.set} and \code{new.set}
#' are identical, no information is added to the given report.
#'
#' @seealso \code{\link{rnb.execute.filter.summary}}, \code{\link{rnb.run.filtering}}
#'
#' @author Yassen Assenov
#' @noRd
rnb.step.filter.summary <- function(old.set, new.set, report) {
	if (!inherits(old.set, "RnBSet")) {
		stop("invalid value for old.set")
	}
	if (!inherits(new.set, "RnBSet")) {
		stop("invalid value for new.set")
	}
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	if (rnb.getOption("logging") && logger.isinitialized() == FALSE) {
		logger.start(fname = NA) # initialize console logger
	}

	removed <- rnb.get.filtered.sites.samples(old.set, new.set)
	rnb.step.filter.summary.internal(old.set, removed$samples, removed$sites, report,
		TRUE)
}

rnb.step.filter.summary.internal <- function(rnb.set, removed.samples, removed.sites, report,
	log.section = FALSE, section.name = "Filtering Summary") {

	if (log.section) {
		logger.start("Summary of Filtering Procedures")
	}
	section.suffix <- gsub("^Filtering Summary( (.+))?$", "\\2", section.name)
	if (section.suffix != "") {
		section.suffix <- paste0("_", which(c("I", "II", "III", "IV") == section.suffix))
	}

	## Create a summary table of removed sites, samples and unreliable measurements
	table.summary <- rnb.execute.filter.summary.internal(rnb.set, removed.samples, removed.sites)
	logger.status("Created summary table of removed sites, samples and unreliable measurements")
	if (all(table.summary[, "Removed"] == 0)) {
		if (log.section) logger.completed()
		return(report)
	}

	## Save table and create figure
	fname <- paste0("summary", section.suffix, ".csv")
	utils::write.csv(table.summary, file = file.path(rnb.get.directory(report, "data", TRUE), fname))
	dframe <- table.summary[, -3]
	colnames(dframe) <- tolower(colnames(dframe))
	dframe <- data.frame(
		x = as.vector(dframe),
		item = factor(rep(rownames(dframe), ncol(dframe)), levels = rev(rownames(dframe))),
		group = factor(rep(colnames(dframe), each = nrow(dframe)), levels = colnames(dframe)))
	pp <- ggplot(dframe, aes_string("item", weight = "x", fill = "factor(group, levels = rev(levels(group)))")) +
		scale_y_continuous(limits = c(0, 1), expand = c(0, 0), labels = percent_format()) +
		labs(x = NULL, y = "Fraction", fill = "Group") + theme(axis.ticks.y = element_blank()) +
		ggplot2::geom_bar(position = "fill", width = 0.7) + coord_flip() +
		theme(plot.margin = unit(0.1 + c(0, 0, 0, 0), "in"))
	pname <- paste0("summary", section.suffix, "_barchart")
	rplot <- createReportPlot(pname, report, width = 7, height = 0.55 + 0.4 * nrow(table.summary))
	suppressWarnings(print(pp))
	rplot <- off(rplot)
	fname <- paste(rnb.get.directory(report, "data"), fname, sep = "/")
	dataset.class <- class(rnb.set)
	txt.site <- rnb.get.row.token(dataset.class)
	txt.sites <- rnb.get.row.token(dataset.class, plural = TRUE)
	rems <- table.summary[c(capitalize(txt.sites), "Samples"), "Removed"]
	txt <- c("As a final outcome of the filtering procedures, ", rems[1], " ",
		ifelse(rems[1] == 1, txt.site, txt.sites), " and ", rems[2], " sample", ifelse(rems[2] != 1, "s", ""), " were ",
		"removed (", length(samples(rnb.set))-rems[2], " samples and ", nsites(rnb.set)-rems[1], " ", txt.sites, " were retained). ",
		"These statistics are presented in <a href=\"", fname, "\">a dedicated table</a> that accompanies ",
		"this report and visualized in the figure below.")
	report <- rnb.add.section(report, section.name, txt)
	txt <- "Fractions of removed values in the dataset after applying filtering procedures."
	report <- rnb.add.figure(report, txt, rplot)
	rm(table.summary, fname, dframe, pp, pname, rplot, rems, txt)
	logger.status("Added summary table of removed and retained items")

	## Construct vectors of removed and retained betas
	siteInds <- NULL
	# subsample sites from removed and retained sites in order to reduce memory footprint 
	if (rnb.getOption("distribution.subsample") > 1){
		numSites <- rnb.getOption("distribution.subsample")
		if (length(removed.sites) != 0) {
			if (length(removed.sites) > numSites){
				siteInds <- sort(sample(removed.sites, size=numSites))
			} else {
				siteInds <- removed.sites
			}
			retained.sites <- setdiff(1:nsites(rnb.set), removed.sites)
			if (length(retained.sites) > numSites){
				siteInds <- sort(c(siteInds, sample(retained.sites, size=numSites)))
			} else {
				siteInds <- sort(c(siteInds, retained.sites))
			}
		} else {
			siteInds <- sort(sample.int(nsites(rnb.set), size=numSites))
		}
	}
	if (!is.null(siteInds)){
		logger.info(c("Subsampling", length(siteInds), "sites for plotting density distributions"))
		# new site indices in subsampled set
		indexMap <- rep(NA, nsites(rnb.set))
		indexMap[siteInds] <- 1:length(siteInds)
		removed.sites <- na.omit(indexMap[removed.sites])
	}
	
	mm <- meth(rnb.set, i=siteInds)
	if (length(removed.samples) != 0) {
		betas.removed <- as.vector(mm[, removed.samples])
		mm <- mm[, -removed.samples]
	} else {
		betas.removed <- double()
	}
	if (length(removed.sites) != 0) {
		betas.removed <- c(betas.removed, mm[removed.sites, ])
		mm <- mm[-removed.sites, ]
	}
	beta.values <- list(Removed = betas.removed, Retained = as.vector(mm))
	for (i in 1:length(beta.values)) {
		beta.values[[i]] <- beta.values[[i]][!is.na(beta.values[[i]])]
	}
	logger.status("Constructed sequences of removed and retained methylation values")
	rm(betas.removed, mm, i)
	rnb.cleanMem()

	## Compare removed vs. retained methylation beta values
	if ((!is.null(beta.values)) && min(sapply(beta.values, length)) >= 501) {
		report.plots <- rnb.plot.beta.comparison(beta.values, paste0("summary", section.suffix, "_betas"), report)
		setting.names <- list("Plot type" =
			c("density" = "density estimation", "histogram" = "histograms", "qq" = "quantile-quantile plot"))
		txt <- c("The figure below compares the distributions of the removed methylation &beta; values and of the ",
			"retained ones.")
		rnb.add.paragraph(report, txt)
		txt <- "Comparison of removed and retained &beta; values."
		txt <- c(txt, add.text.subsampling(attr(report.plots, "subsampled"), paste(names(beta.values), "betas")))
		report <- rnb.add.figure(report, txt, report.plots, setting.names)
		logger.status("Added comparison between removed and retained beta values")
	}

	if (log.section) {
		logger.completed()
	}
	return(report)
}

## E N D ###############################################################################################################

########################################################################################################################
## ReportPlot-methods.R
## created: 2012-04-16
## creator: Yassen Assenov
## ---------------------------------------------------------------------------------------------------------------------
## ReportPlot class definition.
########################################################################################################################

## M E T H O D S #######################################################################################################

setValidity("ReportPlot",
	function(object) {
		if (object@fname == "") {
			return("file name must not be an empty character")
		}
		if (object@width <= 0) {
			return("width must be a positive value")
		}
		if (object@height <= 0) {
			return("height must be a positive value")
		}
#		if (!(object@create.pdf || object@low.png > 0 || object@high.png > 0)) {
#			return("no image type specified to create")
#		}
		TRUE
	}
)

########################################################################################################################

setMethod("initialize", "ReportPlot",
	function(.Object, fname="temp", report=NULL, width=7, height=7, create.pdf=FALSE, low.png=0L, high.png=0L, skip.dev.setup=FALSE) {
		.Object@fname <- fname
		.Object@width <- width
		.Object@height <- height
		.Object@create.pdf <- create.pdf
		.Object@low.png <- low.png
		.Object@high.png <- high.png
		## Use report to set directories
		if (!is.null(report)) {
			.Object@dir.pdf <- rnb.get.directory(report, "pdfs", absolute = TRUE)
			.Object@dir.png.low <- rnb.get.directory(report, "images", absolute = TRUE)
			.Object@dir.png.high <- rnb.get.directory(report, "images-high", absolute = TRUE)
		} else {
			.Object@dir.pdf <- getwd()
			.Object@dir.png.low <- getwd()
			.Object@dir.png.high <- getwd()
		}
		validObject(.Object)
		if (!skip.dev.setup){
			if (create.pdf) {
				validate.dir(.Object@dir.pdf)
				fn <- file.path(.Object@dir.pdf, paste0(fname, ".pdf"))
				pdf(fn, width = width, height = height)
			} else if (high.png > 0) {
				validate.dir(.Object@dir.png.high)
				if (low.png > 0 && .Object@dir.png.high == .Object@dir.png.low) {
					fname <- paste0(fname, "_high_resolution")
				}
				fn <- file.path(.Object@dir.png.high, paste0(fname, ".png"))
				png(fn, width = width * high.png, height = height * high.png)
			} else if (low.png > 0) {
				validate.dir(.Object@dir.png.low)
				fn <- file.path(.Object@dir.png.low, paste0(fname, ".png"))
				png(fn, width = width * low.png, height = height * low.png)
			} else {
				skip.dev.setup <- TRUE
			}
		}
		if (!skip.dev.setup) {
			dev.control(displaylist = "enable")
			par(mar = c(4, 4, 1, 1) + 0.1)
		}
		.Object
	}
)

########################################################################################################################

#' get.files
#'
#' Gets the list of all files that are planned to be generated, or were already generated by the given report plot.
#'
#' @param report.plot Report plot of interest. This must be an object of type \code{\linkS4class{ReportPlot}}.
#' @return Non-empty \code{character} vector of absolute file names.
#'
#' @examples
#' \donttest{
#' plot.image <- createReportPlot('scatterplot', high.png = 200)
#' get.files(plot.image)
#' }
#' @author Yassen Assenov
#' @export
get.files <- function(report.plot) {
	if (!inherits(report.plot, "ReportPlot")) {
		stop("invalid value for report.plot")
	}
	result <- character(0)
	fname <- report.plot@fname
	if (report.plot@create.pdf) {
		result <- file.path(report.plot@dir.pdf, paste(fname, "pdf", sep = "."))
	}
	if (report.plot@low.png > 0) {
		result <- c(result, file.path(report.plot@dir.png.low, paste(fname, "png", sep = ".")))
	}
	if (report.plot@high.png > 0) {
		if (report.plot@low.png && report.plot@dir.png.low == report.plot@dir.png.high) {
			fname <- paste(fname, "_high_resolution", sep = "")
		}
		result <- c(result, file.path(report.plot@dir.png.high, paste(fname, "png", sep = ".")))
	}
	return(result)
}

########################################################################################################################

#' @rdname off
#' @export
setMethod("off", signature(object="ReportPlot"),
	function(object) {
		convert.f <- function(fname, ...) {
			doerror <- function(e) {
				dev.off()
				em <- paste("Could not create file.", e$message)
				if (logger.isinitialized()) {
					logger.error(em)
				} else {
					stop(em)
				}
			}

			tryCatch(
				dev2bitmap(fname, type = "pngalpha", height = object@height, width = object@width,
					method = "pdf", ...),
				warning = function(e) {
					if (grepl(" had status 1$", e$message)) {
						doerror(e)
					} else if (logger.isinitialized()) {
						logger.warning(e$message)
					} else {
						invisible(e$message)
					}
				},
				error = doerror)
		}
		if (object@create.pdf) {
			if (object@high.png > 0) {
				validate.dir(object@dir.png.high)
				fname <- object@fname
				if (object@low.png > 0 && object@dir.png.high == object@dir.png.low) {
					fname <- paste0(fname, "_high_resolution")
				}
				fname <- file.path(object@dir.png.high, paste0(fname, ".png"))
				convert.f(fname, res = object@high.png, fonts = c("Helvetica", "sans"))
			}
			if (object@low.png > 0) {
				validate.dir(object@dir.png.low)
				fname <- file.path(object@dir.png.low, paste0(object@fname, ".png"))
				convert.f(fname, res = object@low.png, fonts = c("Helvetica", "sans"))
			}
		} else if (object@low.png > 0 && object@high.png > 0) {
			validate.dir(object@dir.png.low)
			fname <- file.path(object@dir.png.low, paste0(object@fname, ".png"))
			convert.f(fname, res = object@low.png)
		}
		dev.off()
		return(invisible(object))
	}
)

########################################################################################################################
########################################################################################################################

#' createReportPlot
#'
#' Initializes a report plot and opens a device to create it. The type of the device created depends on the
#' parameters \code{create.pdf}, \code{low.png} and \code{high.png}. If \code{create.pdf} is \code{TRUE}, a PDF device
#' is opened and its contents are later copied to PNG device(s) if needed. Otherwise, a PNG device is opened. Note that
#' at least one of the following conditions must be met:
#' \itemize{
#'   \item{}{\code{create.pdf == TRUE}}
#'   \item{}{\code{low.png > 0}}
#'   \item{}{\code{high.png > 0}}
#' }
#'
#' @param fname      \code{character} vector with one element storing the name of the output file, without the
#'                   extension. The initialized object appends \code{.pdf} and/or \code{.png} to this name.
#' @param report     Report (object of type \code{\linkS4class{Report}}) to which this plot is going to be added. This
#'                   is used to set the directories for PDF and/or PNG files generated for these plots. If this
#'                   parameter is \code{NULL}, the current working directory is used to host all generated images.
#' @param width      \code{numeric} storing the width of the device in inches. The length of this vector must be
#'                   \code{1}.
#' @param height     \code{numeric} storing the height of the device in inches. The length of this vector must be
#'                   \code{1}.
#' @param create.pdf Flag indicating if a PDF image is to be created. The length of this vector must be \code{1}.
#' @param low.png    Resolution, in dots per inch, used for the figure image. Set this to \code{0} or a negative value
#'                   to disable the creation of a low resolution image. The length of this vector must be \code{1}.
#' @param high.png   Resolution, in dots per inch, used for a dedicated image. Set this to \code{0} or a negative value
#'                   to disable the creation of a high resolution image. The length of this vector must be \code{1}.
#' @return Newly created \code{ReportPlot} object.
#'
#' @seealso \code{\link{pdf}} for manually initializing a graphics device; \code{\linkS4class{Report}} for other
#'   functions adding contents to an HTML report
#'
#' @examples
#' \donttest{
#' plot.image <- createReportPlot('scatterplot_tumors')
#' plot(x = c(0.4, 1), y = c(9, 3), type = 'p', main = NA, xlab = expression(beta), ylab = 'Measure')
#' off(plot.image)
#' }
#'
#' @details
#' In order to ensure independence of the operating system, there are strong restrictions on the name of the file. It
#' can consist of the following symbols only: Latin letters, digits, dot (\code{.}), dash (\code{-}) and underline
#' (\code{_}). The name must not include paths, that is, slash (\code{/}) or backslash (\code{\\}) cannot be used.
#'
#' @author Yassen Assenov
#' @export
createReportPlot <- function(fname, report = NULL, width = 7, height = 7, create.pdf = TRUE, low.png = 100L,
	high.png = 0L) {
	validate.vector <- function(x, vtype) {
		return(class(x) == vtype && length(x) == 1 && (!is.na(x)))
	}
	if (!validate.vector(fname, "character")) {
		stop("invalid value for fname")
	}
	if (!grepl("^[A-Za-z0-9._-]+$", fname)) {
		stop("invalid value for fname")
	}
	if (!(is.null(report) || class(report) == "Report")) {
		stop("invalid value for report")
	}
	if (is.integer(width)) {
		width <- as.numeric(width)
	}
	if (is.integer(height)) {
		height <- as.numeric(height)
	}
	if (!validate.vector(width, "numeric")) {
		stop("invalid value for width")
	}
	if (!validate.vector(height, "numeric")) {
		stop("invalid value for height")
	}
	if (!validate.vector(create.pdf, "logical")) {
		stop("invalid value for create.pdf")
	}
	if (is.numeric(low.png) && isTRUE(all(low.png == as.integer(low.png)))) {
		low.png <- as.integer(low.png)
	}
	if (!validate.vector(low.png, "integer")) {
		stop("invalid value for low.png")
	}
	if (is.numeric(high.png) && isTRUE(all(high.png == as.integer(high.png)))) {
		high.png <- as.integer(high.png)
	}
	if (!validate.vector(high.png, "integer")) {
		stop("invalid value for high.png")
	}
	## Initialize report plots
	return(new("ReportPlot", fname, report, width, height, create.pdf, low.png, high.png))
}



## ReportGgPlot M E T H O D S #######################################################################################################

setValidity("ReportGgPlot",
		function(object) {
			if (object@fname == "") {
				return("file name must not be an empty character")
			}
			if (object@width <= 0) {
				return("width must be a positive value")
			}
			if (object@height <= 0) {
				return("height must be a positive value")
			}
			if (!(is.ggplot(object@ggp) | is.null(object@ggp))){
				return("ggp must be a ggplot object")
			}
			TRUE
		}
)


setMethod("initialize", "ReportGgPlot",
		function(.Object, ggp=ggplot(), ...) {
			.Object <- callNextMethod(.Object=.Object,skip.dev.setup=TRUE,...)
			.Object@ggp <- ggp
			validObject(.Object)
			.Object
		}
)

#' @rdname off
#' @export
setMethod("off", signature(object="ReportGgPlot"),
	function(object,handle.errors=FALSE) {
		do.it <- function(obj){
			if (obj@create.pdf) {
				validate.dir(obj@dir.pdf)
				fn <- file.path(obj@dir.pdf, paste(obj@fname, "pdf", sep = "."))
				ggplot2::ggsave(fn,obj@ggp,width=obj@width,height=obj@height)
			}
			if (obj@high.png > 0) {
				validate.dir(obj@dir.png.high)
				fname <- obj@fname
				if (obj@low.png > 0 && obj@dir.png.high == obj@dir.png.low) {
					fname <- paste(fname, "_high_resolution", sep = "")
				}
				fn <- file.path(obj@dir.png.high, paste(fname, "png", sep = "."))
				ggplot2::ggsave(fn,obj@ggp,width=obj@width,height=obj@height,dpi=obj@high.png)
			}
			if (obj@low.png > 0) {
				validate.dir(obj@dir.png.low)
				fn <- file.path(obj@dir.png.low, paste(obj@fname, "png", sep = "."))
				ggplot2::ggsave(fn,obj@ggp,width=obj@width,height=obj@height,dpi=obj@low.png)
			}
		}
		if (handle.errors){
			tryCatch(
				do.it(object),
				error=function(ee){
					logger.warning(c("ReportGgPlot error ('off' method):",ee$message))
				  object@ggp <<- rnb.message.plot("plotting error")
					do.it(object)
				}
			)
		} else {
			do.it(object)
		}
		object@ggp <- NULL
		return(invisible(object))
	}
)

#' createReportGgPlot
#'
#' creates a report plot containing a \code{ggplot object}. Except for the \code{ggp} parameter, the signature and
#' behavior is identical to \code{\link{createReportPlot}}.
#' 
#' @param ggp        \code{ggplot} object to be plotted
#' @param fname      \code{character} vector with one element storing the name of the output file, without the
#'                   extension. The initialized object appends \code{.pdf} and/or \code{.png} to this name.
#' @param report     Report (object of type \code{\linkS4class{Report}}) to which this plot is going to be added. This
#'                   is used to set the directories for PDF and/or PNG files generated for these plots. If this
#'                   parameter is \code{NULL}, the current working directory is used to host all generated images.
#' @param width      \code{numeric} storing the width of the device in inches. The length of this vector must be
#'                   \code{1}.
#' @param height     \code{numeric} storing the height of the device in inches. The length of this vector must be
#'                   \code{1}.
#' @param create.pdf Flag indicating if a PDF image is to be created. The length of this vector must be \code{1}.
#' @param low.png    Resolution, in dots per inch, used for the figure image. Set this to \code{0} or a negative value
#'                   to disable the creation of a low resolution image. The length of this vector must be \code{1}.
#' @param high.png   Resolution, in dots per inch, used for a dedicated image. Set this to \code{0} or a negative value
#'                   to disable the creation of a high resolution image. The length of this vector must be \code{1}.
#' @return Newly created \code{ReportGgPlot} object.
#'
#' @author Fabian Mueller
#' @export
createReportGgPlot <- function(ggp, fname, report = NULL, width = 7, height = 7, create.pdf = TRUE, low.png = as.integer(100),
		high.png = as.integer(0)) {
	validate.vector <- function(x, vtype) {
		return(class(x) == vtype && length(x) == 1 && (!is.na(x)))
	}
	if (!is.ggplot(ggp)){
		stop("invalid value for ggp")
	}
	if (!validate.vector(fname, "character")) {
		stop("invalid value for fname")
	}
	if (!grepl("^[A-Za-z0-9._-]+$", fname)) {
		stop("invalid value for fname")
	}
	if (!(is.null(report) || class(report) == "Report")) {
		stop("invalid value for report")
	}
	if (is.integer(width)) {
		width <- as.numeric(width)
	}
	if (is.integer(height)) {
		width <- as.numeric(height)
	}
	if (!validate.vector(width, "numeric")) {
		stop("invalid value for width")
	}
	if (!validate.vector(height, "numeric")) {
		stop("invalid value for width")
	}
	if (!validate.vector(create.pdf, "logical")) {
		stop("invalid value for create.pdf")
	}
	if (is.numeric(low.png) && isTRUE(all(low.png == as.integer(low.png)))) {
		low.png <- as.integer(low.png)
	}
	if (!validate.vector(low.png, "integer")) {
		stop("invalid value for low.png")
	}
	if (is.numeric(high.png) && isTRUE(all(high.png == as.integer(high.png)))) {
		high.png <- as.integer(high.png)
	}
	if (!validate.vector(high.png, "integer")) {
		stop("invalid value for high.png")
	}
	## Initialize report plots
	return(new("ReportGgPlot", ggp, fname, report, width, height, create.pdf, low.png, high.png))
}

########################################################################################################################
## plottingUtils.R
## created: 2012-05-12
## creator: Pavlo Lutsik
## ---------------------------------------------------------------------------------------------------------------------
## Constants and functions related to plotting.
########################################################################################################################

## G L O B A L S #######################################################################################################

## Full legends for site and region annotations that are supported by RnBeads
LEGENDS.ALL <- list(
	"CGI Relation" = c("Open Sea" = "#0000FF", "Shelf" = "#00FFFF", "Shore" = "#B200FF", "Island" = "#FF0000"),
	"Design" = c("I" = "#FF8080", "II" = "#8080FF"),
	"CpG Density" = 2L)

## color palettes to be used when there are multiple density estimates in one plot
## (e.g. for scatterplots or differential methylation)
## colors based on ColorBrewer color palette "Paired", pairs 1 and 3
DENS.COLORS.LOW  <- c("#1F78B4","#E31A1C") 
DENS.COLORS.HIGH <- c("#A6CEE3","#FB9A99")

## F U N C T I O N S ###################################################################################################

## Applies the given coloring scheme to the vector of values.
##
## @param val    a vector of values
## @param rng	a 2-tuple descibing the possible range of values
## @param colscheme.col.val the colorscheme that is applied
## @return a vector of color values
## @author Fabian Mueller
colorize.value <- function(val, rng=c(min(val),max(val)),
	colscheme.col.val=c(gplots::colorpanel(100,"blue","white"),gplots::colorpanel(100,"white","red")))
{
	#require(gplots)
	return(colscheme.col.val[round((val - rng[1]) / (rng[2] -rng[1])  * (length(colscheme.col.val)-1),0)+1])
}

## Returns a color panel of color values for methylation levels from the "colors.meth option"
##
## @author Fabian Mueller
get.methylation.color.panel <- function()
{
	meth.color.base <- rnb.getOption("colors.meth")
	if (length(meth.color.base) == 3) {
		meth.color.panel <- gplots::colorpanel(100,meth.color.base[1],meth.color.base[2],meth.color.base[3])
	} else { # length(meth.color.base) == 2
		meth.color.panel <- gplots::colorpanel(100,meth.color.base[1],meth.color.base[2])
	}
	return(meth.color.panel)
}

########################################################################################################################

## Applies the given coloring scheme to the vector of values.
##
## @param x      Non-empty vector of values.
## @param cols   Coloring scheme to be applied. This must be either a non-empty \code{character} vector storing
##               different colors, or a function that takes one integer argument \code{n} and returns a vector of
##               \code{n} different colors.
## @param na.col Color to be used for \code{NA} elements of \code{x}. If the value vector contains \code{NA}s, this
##               color is appended to the mapping and used to indicate a missing value.
## @return List of two elements:
##         \itemize{
##           \item{"colors" }{Vector of length \code{length(x)} containing the assigned colors for the values in
##                \code{x}.}
##           \item{"mapping" }{\code{character} vector storing the applied mapping from value to color.}
##         }
## @author Yassen Assenov
rnb.get.cols <- function(x, cols, na.col = "#C0C0C0") {
	contains.na <- any(is.na(x))
	xvalues <- unique(sort(x))
	if (typeof(cols) == "closure") {
		color.map <- as.character(cols(length(xvalues)))
	} else if (is.character(cols) && length(cols) != 0) {
		if (length(cols) < length(xvalues)) {
			cols <- rep(cols, ceiling(length(xvalues) / length(cols)))
		}
		color.map <- cols[1:length(xvalues)]
	} else {
		stop("invalid value for cols")
	}
	names(color.map) <- as.character(xvalues)
	result <- color.map[as.character(x)]
	if (any(is.na(x))) {
		color.map <- c(color.map, na.col)
		names(color.map)[length(color.map)] <- NA
		result[is.na(result)] <- na.col
	}
	return(list("colors" = result, "mapping" = color.map))
}

########################################################################################################################

#' get.site.and.region.types
#'
#' Initializes color legends and extracts annotation values of site and/or region types covered in the given methylation
#' dataset.
#'
#' @param rnb.set Methylation dataset of interest.
#' @return List of mappable annotation values, one \code{data.frame} per site or region type. The columns of this
#'         \code{data.frame} are the supported annotations. Every table also contains an attribute named \code{"legend"}
#'         that contains a \code{list} of the legends for all columns in the respective annotation table. Every legend
#'         item is either a \code{character} vector storing the mapping from values to colors, or a single
#'         \code{integer} value, signifying the number of colors in a gradient.
#' @author Yassen Assenov, Fabian Mueller
#' @noRd
get.site.and.region.types <- function(rnb.set) {
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	result <- list()
	rtypes <- rnb.region.types.for.analysis(rnb.set)
	if (rnb.getOption("analyze.sites")) {
		rtypes <- c("sites", rtypes)
	}
	for (rtype in rtypes) {
		anno.table <- tryCatch(annotation(rnb.set, type = rtype), error = function(e) { NULL })
		if (is.null(anno.table)) {
			next
		}
		## Add a column for CpG Density
		if (!("CpG Density" %in% colnames(anno.table)) && ("CpG" %in% colnames(anno.table))) {
			if (rtype == "sites") {
				LENGTH.NEIGHBORHOOD <- 100L # TODO: Get this value from the data package
				anno.table[, "CpG Density"] <- anno.table[, "CpG"] / LENGTH.NEIGHBORHOOD
			} else {
				anno.table[, "CpG Density"] <- anno.table[, "CpG"] / (anno.table[, "End"] - anno.table[, "Start"] + 1)
			}
		}

		## Extract suitable columns from the annotation table
		c.names <- intersect(names(LEGENDS.ALL), colnames(anno.table))
		if (length(c.names) == 0) {
			next
		}
		if (length(c.names) != 1) {
			anno.table <- anno.table[, c.names]
		} else {
			anno.table <- data.frame("1" = anno.table[, c.names])
			colnames(anno.table) <- c.names
		}
		## Adjust the CGI Relation values if necessary
		if (("CGI Relation" %in% c.names) &&
			(!identical(levels(anno.table[, "CGI Relation"]), names(LEGENDS.ALL[["CGI Relation"]])))) {
			cgi.relations <- as.character(anno.table[, "CGI Relation"])
			cgi.relations[cgi.relations %in% c("North Shelf", "South Shelf")] <- "Shelf"
			cgi.relations[cgi.relations %in% c("North Shore", "South Shore")] <- "Shore"
			anno.table[, "CGI Relation"] <- factor(cgi.relations, levels = names(LEGENDS.ALL[["CGI Relation"]]))
		}

		## Construct color legends
		legend <- LEGENDS.ALL[c.names]
		for (c.name in c.names) {
			lg <- legend[[c.name]]
			if (is.character(lg)) {
				lg <- lg[intersect(names(lg), unique(anno.table[, c.name]))]
				legend[[c.name]] <- lg
			}
		}
		attr(anno.table, "legend") <- legend
		result[[rtype]] <- anno.table
	}
	return(result)
}

########################################################################################################################

## get.site.and.region.types.colors
##
## Takes the returned value from \code{\link{get.site.and.region.types}} and returns a vector containing the color
## values for the given locus type.
##
## @param snrt      List of site and region types, as returned by \code{get.site.and.region.types}.
## @param type      Site or region type.
## @param annot.ind Column index or name for \code{snrt[[type]]}, i.e. the index of the annotation in the object.
## @return a vector containing a color value for each locus
get.site.and.region.types.colors <- function(snrt,type,annot.ind){
	vals <- snrt[[type]][,annot.ind]
	leg <- attr(snrt[[type]],"legend")[[annot.ind]]
	if (is.character(leg)){
		return(leg[vals])
	}
	else if (leg == 2){
		colors.grad <- rnb.getOption("colors.gradient")
		return(colorize.value(vals,colscheme.col.val=c(gplots::colorpanel(100,colors.grad[1],colors.grad[2]))))	
	} else {
		stop("Error in converting values to colors: invalid legend")
	}
}

########################################################################################################################

## rnb.pheno2colors
##
## Maps the given sample phenotypic information to colors.
##
## @param sample.table Phenotypical information of a single trait, stored in a non-empty \code{vector} or \code{factor};
##                     or of multiple traits, stored in a non-empty table (\code{matrix} or \code{data.frame}). In case
##                     this is a table, rows are treated as samples, and columns - as traits. Note that missing values
##                     (\code{NA}), if present in a trait, are assigned a hard-coded color irrespective of the provided
##                     scheme.
## @param cols         Color scheme(s) to be mapped to the values in \code{sample.table}. This may be specified in one
##                     of three forms: \code{character} vector, function, or a list of schemes, each element in the list
##                     being a vector or a function. If the number of the given color schemes is smaller than the number
##                     of traits in \code{sample.table}, the schemes are recycled.
## @return     List of two elements:
##             \itemize{
##                \item{"colors" }{\code{character} vector or matrix of the same dimensions as \code{sample.table},
##                     containing the assigned colors based on the given values.}
##                \item{"mapping" }{\code{character} vector storing the applied mapping from value to color. or a list
##                     of such vector, one for each column of \code{colors}.}
##             }
## @author Yassen Assenov
rnb.pheno2colors <- function(sample.table, cols = rnb.getOption("colors.category")) {
	if ((is.vector(sample.table) || is.factor(sample.table)) && length(sample.table) != 0) {
		if (is.list(cols) && length(cols) != 0) {
			cs <- cols[[1]]
		} else if (is.character(cols) || typeof(cols) == "closure") {
			cs <- cols
		} else {
			stop("invalid value for cols")
		}
		return(rnb.get.cols(sample.table, cs))
	}
	if (!(is.matrix(sample.table) || is.data.frame(sample.table))) {
		stop("invalid value for sample.table")
	}
	if (nrow(sample.table) == 0 || ncol(sample.table) == 0) {
		stop("invalid value for sample.table")
	}
	if (is.list(cols) && length(cols) != 0) {
		ci <- 0
	} else if (is.character(cols) || typeof(cols) == "closure") {
		cs <- cols
	} else {
		stop("invalid value for cols")
	}
	result <- list(
		"colors" = matrix("", nrow = nrow(sample.table), ncol = ncol(sample.table), dimnames = dimnames(sample.table)),
		"mapping" = list())
	for (i in 1:ncol(sample.table)) {
		if (is.list(cols)) {
			ci <- ci %% length(cols) + 1
			cs <- cols[[ci]]
		}
		ri <- rnb.get.cols(sample.table[, i], cs)
		result$colors[, i] <- ri$colors
		result$mapping[[i]] <- ri$mapping
	}
	return(result)
}

########################################################################################################################

#' rnb.plot.beta.comparison
#'
#' Draws plots that compare two distributions of beta values.
#'
#' @param beta.values      Two beta value sequences in the form of a named \code{list} of two non-empty vectors of type
#'                         \code{double}. If any of the vectors contains \code{NA}s, this method may exit with an error.
#' @param fprefix          File name prefix for the plots. This function appends the suffixes \code{"_density"},
#'                         \code{"_histogram"} and \code{"_qq"} to this prefix.
#' @param report           Report to which the plots are to be added.
#' @param qq.length        Positive \code{integer} value showing the number of quantiles to be calculated and presented
#'                         in the generated Q-Q plot.
#' @param points.per.group Maximum number of values to use in plotting a group's distribution. Groups that contain more
#'                         observations than this threshold are subsampled. Setting this parameter to a value less than
#'                         \code{2} disables subsampling.
#' @return List of all generated plots, each being an object ot type \code{\linkS4class{ReportPlot}}.
#'
#' @author Yassen Assenov
#' @export
rnb.plot.beta.comparison <- function(beta.values, fprefix, report = NULL, qq.length = 501L,
	points.per.group = rnb.getOption("distribution.subsample")) {

	if (!(is.list(beta.values) && length(beta.values) == 2 && all(sapply(beta.values, is.double)))) {
		stop("invalid value for beta.values")
	}
	if (any(sapply(beta.values, length) == 0)) {
		stop("invalid value for beta.values; both sequences must be non-empty")
	}
	if (!(is.character(fprefix) && length(fprefix) == 1 && (!is.na(fprefix)))) {
		stop("invalid value for fprefix")
	}
	if (!(is.null(report) || inherits(report, "Report"))) {
		stop("invalid value for report")
	}
	if (is.double(qq.length) && isTRUE(all(as.integer(qq.length) == qq.length))) {
		qq.length <- as.integer(qq.length)
	}
	if (!(is.integer(qq.length) && length(qq.length) == 1 && isTRUE(qq.length > 0))) {
		stop("invalid value for qq.length; expected positive integer")
	}
	if (is.double(points.per.group) && isTRUE(all(as.integer(points.per.group) == points.per.group))) {
		points.per.group <- as.integer(points.per.group)
	}
	if (!(is.integer(points.per.group) && length(points.per.group) == 1 && (!is.na(points.per.group)))) {
		stop("invalid value for points.per.group")
	}

	## Perform subsampling, if required
	subsampled <- rep(FALSE, length(beta.values))
	if (points.per.group > 1) {
		for (i in 1:length(beta.values)) {
			if (length(beta.values[[i]]) > points.per.group) {
				beta.values[[i]] <- sample(beta.values[[i]], size = points.per.group)
				subsampled[i] <- TRUE
			}
		}
	}

	## Construct a data.frame to plot
	dframe <- data.frame(
	 	"value" = unlist(beta.values, use.names = FALSE),
	 	"vtype" = factor(rep(1:length(beta.values), sapply(beta.values, length))), check.names = FALSE)
	levels(dframe[[2]]) <- names(beta.values)

	report.plots <- list()
	fnames <- paste(fprefix, c("density", "histogram", "qq"), sep = "_")
	pmargins <- unit(c(0.1, 1.6, 0.1, 0.1), "in")

	## Create a density estimation plot
	rplot <- createReportPlot(fnames[1], report, width = 6.8, height = 5.2)
	pp <- ggplot(dframe, aes_string(x = "value")) +
		labs(x = expression(beta), y = "Density", color = "Values") +
		geom_density(aes_string(color = "vtype"), kernel = "gaussian") +
		theme(plot.margin = pmargins, legend.justification = c(0, 0.5), legend.position = c(1, 0.5))
	print(pp)
	off(rplot)
	report.plots <- c(report.plots, rplot)
	
	## Create histograms
	rplot <- createReportPlot(fnames[2], report, width = 6.8, height = 5.2)
	pp <- ggplot(dframe, aes_string(x = "value")) + coord_cartesian(xlim = c(0, 1)) +
		labs(x = expression(beta), y = "Density") +
		geom_histogram(aes(y = after_stat(density)), binwidth = 0.02) +
		facet_grid(vtype ~ .) + theme(plot.margin = pmargins)
	print(pp)
	off(rplot)
	report.plots <- c(report.plots, rplot)
	
	## Create a Q-Q plot
	v.quantiles <- as.data.frame(lapply(beta.values, quantile, probs = seq(0, 1, length.out = qq.length)))
	v.diagonal <- data.frame("x" = c(0, 1), "y" = c(0, 1))
	colnames(v.quantiles) <- colnames(v.diagonal) <- names(beta.values)
	cnames <- paste0("`", colnames(v.quantiles), "`")
	rplot <- createReportPlot(fnames[3], report, width = 6.8, height = 5.2)
	pp <- ggplot(v.quantiles, aes_string(x = cnames[1], y = cnames[2])) +
		coord_cartesian(xlim = c(0, 1), ylim = c(0, 1)) + coord_fixed() +
		geom_path(data = v.diagonal, color = "#A0A0A0") + geom_path(lineend = "round", size = 2) +
		theme(plot.margin = pmargins)
	print(pp)
	off(rplot)
	report.plots <- c(report.plots, rplot)

	attr(report.plots, "subsampled") <- subsampled
	return(report.plots)
}

########################################################################################################################

## add.text.subsampling
##
## Generates, if necessary, text about subsampling. This text should be added to a figure description.
##
## @param subsampled       \code{logical} vector of length \code{2} containing information if each of the targeted two
##                         value sequences was subsampled.
## @param group.names      Names of the two groups of values.
## @param points.per.group Size of sample, if sampling was applied.
## @return \code{character} vector containing descriptive text about subsampling; an empty vector if subsampling was not
##         performed.
## @author Yassen Assenov
add.text.subsampling <- function(subsampled, group.names, points.per.group = rnb.getOption("distribution.subsample")) {
	if (any(subsampled)) {
		if (all(subsampled)) {
			txt <- "Both distributions are"
		} else {
			txt <- paste(" The distribution of", tolower(group.names[which(subsampled)]), "is")
		}
		txt <- c(txt, " estimated by randomly sampling ", points.per.group, " values",
			ifelse(all(subsampled), " in each group.", "."))
	} else {
		txt <- character()
	}
	return(txt)
}

########################################################################################################################

#' rnb.message.plot
#'
#' Creates a plot, using \pkg{ggplot2}, with a single text message.
#'
#' @param txt Text to be plotted.
#' @return The newly initialized \code{ggplot} instance.
#'
#' @examples
#' \donttest{
#' x11(width = 5, height = 5)
#' rnb.message.plot("Missing data")
#' }
#' @author Yassen Assenov
#' @export
rnb.message.plot <- function(txt) {
	if (!(is.character(txt) && length(txt) == 1 && (!is.na(txt)))) {
		stop("invalid value for txt")
	}
	ggplot(data.frame(x = 1, y = 1, labeltext = txt), aes_string("x", "y", label = "labeltext")) +
		geom_text(color = "grey50") +
		theme(axis.line = element_blank(), axis.title = element_blank(), axis.text = element_blank(),
			axis.ticks = element_blank(), panel.border = element_blank(), panel.grid = element_blank(),
			panel.background = element_blank(), plot.background = element_blank())
}

########################################################################################################################

#' rnb.color.legends
#'
#' Creates a figure in the given report that contains one or more color legends.
#'
#' @param report        Report to contain the legend figure. This must be an object of type \code{\linkS4class{Report}}.
#' @param legends       Color legend in the form of a non-empty \code{character} vector. Element names denote legend
#'                      labels, and the elements themselves specify colors. This parameter can also be a \code{list} of
#'                      color legends. Special restrictions apply to the names of the list elements, see \emph{Details}.
#' @param fprefix       File name or prefix for the plot files.
#' @param description   Text of the figure description. See the correponding parameter in
#'                      \code{\link{rnb.add.figure}} for more details.
#' @param setting.names One-element list containing a plot file descriptor, when \code{legends} is a list. See the
#'                      corresponding parameter in \code{\link{rnb.add.figure}} for more details. If this is set to
#'                      \code{NULL} (default), the list is automatically created using \code{names(legends)} (when
#'                      \code{legends} is a list), or as an empty list (when \code{legends} is a vector).
#' @param size.factor   Relative size, in inches of the plots. Legends are displayed in columns of up to 10 items; each
#'                      column is effectively a square with the specified size.
#' @return The modified report.
#'
#' @details In case \code{legends} specifies multiple legends in the form of a list, \code{names(legends)} are appended
#'          to \code{fprefix} to generate file names. In order to ensure independence of the operating system, there are
#'          strong restrictions on these names. They can consist of the following symbols only: Latin letters, digits,
#'          dot (\code{.}), dash (\code{-}) and underline (\code{_}).
#'
#' @author Yassen Assenov
#' @export
rnb.color.legends <- function(report, legends, fprefix = ifelse(is.character(legends), "legend", "legend_"),
	description = "Color legend.", setting.names = NULL, size.factor = 3) {
	if (!(is.character(legends) && length(legends) != 0)) {
		if (!(is.list(legends) && all(sapply(legends, is.character)) && all(sapply(legends, length) != 0))) {
			stop("invalid value for legends; expected character or list")
		}
		lnames <- names(legends)
		if (!(is.character(lnames) && all(sapply(lnames, is.valid.relative)))) {
			stop("invalid value for legends; missing or invalid legend names")
		}
		if (anyDuplicated(lnames) != 0) {
			stop("invalid value for legends; duplicated legend names")
		}
		if (!all(grepl("^[A-Za-z0-9._-]+$", lnames))) {
			stop("invalid value for legends; unsupported legend names")
		}
		if (is.null(setting.names)) {
			setting.names <- list("Based on" = lnames)
			names(setting.names[[1]]) <- lnames
		}
	} else {
		setting.names <- list()
	}
	if (!inherits(report, "Report")) {
		stop("invalid value for report; expected Report")
	}
	if (!(is.vector(fprefix) && length(fprefix) == 1 && (!is.na(fprefix)))) {
		stop("invalid value for fprefix; expected single-element vector")
	}
	fprefix <- as.character(fprefix)
	if (!((is.double(size.factor) || is.integer(size.factor)) && length(size.factor) == 1 && (!is.na(size.factor)))) {
		stop("invalid value for size.factor; expected scalar of type integer or double")
	}

	## Create plots for the legends
	plotlegend <- function(fname, report, legendinfo) {
		legend.columns <- (length(legendinfo) - 1) %/% 10 + 1
		rplot <- createReportPlot(fname, report, width = size.factor * legend.columns, height = size.factor)
		par(mar = c(0, 0, 0, 0) + 0.1)
		plot.new()
		legend("center", legend = names(legendinfo), fill = legendinfo, bty = "n", ncol = legend.columns)
		off(rplot)
		return(rplot)
	}
	if (is.character(legends)) {
		report.plots <- plotlegend(fprefix, report, legends)
	} else { # is.list(legends) && all(sapply(legends, is.character))
		report.plots <- lapply(names(legends), function(lname) {
			plotlegend(paste(fprefix, lname, sep = ""), report, legends[[lname]])
		})
	}

	## Add a figure to the report
	return(rnb.add.figure(report, description, report.plots, setting.names))
}

########################################################################################################################

#' rnb.plot.pheno.categories
#'
#' Generates bar charts summarizing the categorical traits in a sample annotation table.
#'
#' @param annotations  Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}, or its sample
#'                     annotations in the form of a \code{data.frame}. If this parameter is a dataset, the annotation
#'                     information is extracted using the method \code{\link[=pheno,RnBSet-method]{pheno}}.
#' @param columns      Optional; predefined column names (in the form of a \code{character} vector) or indices (an
#'                     \code{integer} vector) to consider. All other columns in the annotation table will be ignored.
#' @param fileprefix   \code{character} vector with one element storing the file name prefix of the output files,
#'                     without the extension. Only a limited set of symbols is allowed to be used in this prefix.
#' @param report       Report to contain the generated plots. If specified, this must be an object of type
#'                     \code{\linkS4class{Report}}.
#' @param color.values Non-empty \code{character} vector containing the color scheme to be mapped to the categories
#'                     defined in the annotation table. Colors are recycled if necessary, that is, if the length of this
#'                     vector is smaller than the number of categories in a trait.
#' @return List of report plots. The names in this list are the column names in the annotation table that were selected
#'         for visualization. In case no suitable categorical traits are found among the provided annotations, this
#'         function returns an empty list.
#' 
#' @details This function identifies the traits that define sample subgroups and then generates one report plot per
#' trait. Every report plot consists of two files. File names are formed by appending an index and file extension to
#' \code{fileprefix}. Thus, the suffixes appended are \code{"_1.pdf"}, \code{"_1.png"}, \code{"_2.pdf"},
#' \code{"_2.png"}, ... Existing files with the generated filenames are overwritten.
#' 
#' @seealso \code{\link{rnb.sample.groups}} for identifying traits in the annotation table that define sample subgroups;
#'          \code{\link{createReportPlot}} for the allowed symbols to be used in \code{fileprefix}
#' 
#' @author Yassen Assenov
#' @export
rnb.plot.pheno.categories <- function(annotations, columns = NULL, fileprefix = "barchart_pheno", report = NULL,
	color.values = rnb.getOption("colors.category")) {

	## Validate the function's parameters
	pheno.groups <- lapply(rnb.sample.groups(annotations, columns), sapply, length)
	if (!(is.character(fileprefix) && length(fileprefix) == 1 && (!is.na(fileprefix)))) {
		stop("invalid value for fileprefix")
	}
	if (!grepl("^[A-Za-z0-9._-]+$", fileprefix)) {
		stop("invalid value for fileprefix")
	}
	if (!(is.null(report) || inherits(report, "Report"))) {
		stop("invalid value for report")
	}
	if (!(is.character(color.values) && length(color.values) != 0 && all(!is.na(color.values)))) {
		stop("invalid value for color.values")
	}

	result <- list()
	if (length(pheno.groups) == 0) {
		names(result) <- character()
		return(result)
	}

	## Recycle colors if necessary
	color.values <- rep_len(color.values, max(sapply(pheno.groups, length)))

	## Create bar charts
	for (i in 1:length(pheno.groups)) {
		x <- pheno.groups[[i]]
		x <- data.frame(x = 1L, t = factor(names(x), levels = names(x)), count = as.integer(x))
		pp <- ggplot(x, aes_string(x = "x", y = "count", fill = "t")) +
			ggplot2::geom_bar(stat = "identity", position = "stack") +
			labs(x = NULL, y = "Samples", fill = names(pheno.groups)[i]) +
			scale_fill_manual(na.value = "C0C0C0", values = color.values, guide = guide_legend(reverse = TRUE)) +
			theme(axis.ticks.x = element_blank(), axis.text.x = element_blank(),
				legend.justification = c(0, 0.5), legend.position = c(1, 0.5),
				plot.margin = unit(c(0.1, 2.1, 0.1, 0.1), "in"))
		fname <- paste(fileprefix, i, sep = "_")
		rplot <- createReportPlot(fname, report, width = 3.1, height = 5.2)
		print(pp)
		result[[names(pheno.groups)[i]]] <- off(rplot)
	}
	result 
}

########################################################################################################################

#' rnb.plot.dreduction
#' 
#' Creates a dimension reduction plot based on the methylation values of the given dataset.
#' 
#' @param rnb.set         Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}. This dataset
#'                        must contain at least four samples.
#' @param plot.type       Type of plot to be created. This must be one of \code{"pca"} (projection to two principal
#'                        components), \code{"mds"} (multidimensional scaling to two dimensions) or \code{"tsne"}
#'                        (t-distributed stochastic neighbor embedding to two dimensions). The section
#'                        \emph{Details} provides more details on how the dimension reduction techniques are applied.
#' @param dimensions      Vector of two positive \code{integer} values giving the principle components to be shown in
#'                        the horizontal and vertical axis of the plot. This parameter is considered only when
#'                        \code{plot.type} is \code{"pca"}.
#' @param distance.metric Distance metric to be applied when reducing the dimensionality of the methylation data. This
#'                        must be one of \code{"eucledian"} or \code{"manhattan"}. The second metric is not supported by
#'                        principal component analysis.
#' @param target          Site or region type to be used in the dimension reduction technique. This must be either
#'                        \code{"sites"} (individual CpGs) or one of the region types summarized in \code{rnb.set}.
#' @param point.types     Trait, specified as column name or index in the sample annnotation table of \code{rnb.set}, to
#'                        be used to define point types in the plot. Setting this parameter to zero (default) or to a
#'                        trait that does not define categories results in all samples being displayed as filled
#'                        circles. If this parameter specifies a column that can be used as sample identifiers, the plot
#'                        displays the samples as identifiers instead of points.
#' @param point.colors    Trait, specified as column name or index in the sample annnotation table of \code{rnb.set}, to
#'                        be used to define sample colors in the plot. Setting this parameter to zero (default) or to a
#'                        trait that does not define numerical values or categories results in all samples being
#'                        displayed in black.
#' @param legend.space    Width, in inches, of the space dedicated for legends that will be assigned on the right side
#'                        of the plot. This parameter is considered only if legends are actually included, that is, if
#'                        sample traits are mapped to point types and/or colors.
#' @return The generated plot as an object of type \code{\link[ggplot2:ggplot]{ggplot}}. The object also contains an
#'         attribute \code{"info"}, which is a list with the following elements:
#'         \describe{
#'           \item{\code{"Target" }}{Targeted sites or regions; the value of the parameter \code{target}.}
#'           \item{\code{"Technique" }}{Dimension reduction technique applied; one of \code{"PCA"} or \code{"MDS"}.}
#'           \item{\code{"All" }}{Total number of sites or regions defining the high dimensional methylation space.}
#'           \item{\code{"Missing" }}{Number of dimensions ignored because they contain (only) missing values.}
#'           \item{\code{"Selected" }}{Number of dimensions used when applying a dimension reduction technique.}
#'           \item{\code{"Explained" }}{Value between \code{0} and \code{1} showing the variance explained by the
#'             selected dimensions, as a fraction of the total variance of all dimensions.}
#'         }
#' 
#' @details
#' The analysis option \code{"exploratory.top.dimensions"} controls whether dimension reduction is applied on all
#' probes, sites or regions available in the given dataset, or only on the most variable ones. In case a trait is mapped
#' to point types, the shapes to use are taken from the option \code{"points.category"}. Similary, the option
#' \code{"colors.category"} determines which colors are used when mapping sample categories to color. In cases when
#' numerical values are mapped to color, the option \code{"colors.3.gradient"} is used. If the set of value contains
#' both positive and negative numbers, the middle point in the color legend is set to zero. See
#' \emph{\link[=rnb.options]{RnBeads Options}} for more information on the options mentioned above.
#' 
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' pdf("PCA.pdf", width = 7.2, height = 5.2)
#' print(rnb.plot.dreduction(rnb.set.example, point.colors="Sample_Group"))
#' dev.off()
#' }
#' @seealso \code{\link[=summarized.regions,RnBSet-method]{summarized.regions}} for listing all region types summarized
#'          in a dataset
#' 
#' @author Yassen Assenov
#' @export
rnb.plot.dreduction <- function(rnb.set, plot.type = "pca", dimensions = 1:2, distance.metric = "euclidean",
	target = "sites", point.types = 0L, point.colors = 0L, legend.space = 2) {

	## Validate parameters
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	if (!(is.character(plot.type) && length(plot.type) == 1 && tolower(plot.type) %in% c("mds", "pca", "tsne"))) {
		stop("invalid value for plot.type")
	}
	plot.type <- tolower(plot.type)
	if (plot.type == "pca") {
		if (is.double(dimensions) && isTRUE(all(dimensions == as.integer(dimensions)))) {
			dimensions <- as.integer(dimensions)
		}
		if (!(is.integer(dimensions) && length(dimensions) == 2 && isTRUE(all(dimensions > 0)))) {
			stop("invalid value for dimensions")
		}
	} else if (plot.type == "tsne") {
		rnb.require("tsne")
	}
	if (!(is.character(distance.metric) && length(distance.metric) == 1 &&
		  	distance.metric %in% c("manhattan", "euclidean"))) {
		stop("invalid value for distance.metric")
	}
	if (plot.type == "pca" && distance.metric != "euclidean") {
		stop(paste("invalid distance metric for", plot.type))
	}
	if (!(is.character(target) && length(target) == 1 && (!is.na(target)))) {
		stop("invalid value for target")
	}
	if (!(target %in% c("sites", summarized.regions(rnb.set)))) {
		stop("unsupported region type")
	}
	validate.column <- function(x, column.names) {
		if (!(length(x) == 1 && !is.na(x))) {
			return(-1L)
		}
		if (is.double(x) && isTRUE(x == as.integer(x))) {
			x <- as.integer(x)
		}
		if (is.integer(x)) {
			if (x < 0) {
				x <- -1L
			} else if (x > length(column.names)) {
				x <- integer()
			}
		} else if (is.character(x)) {
			x <- which(column.names == x)
		} else {
			x <- -1L
		}
		x
	}
	s.annotation <- pheno(rnb.set)
	point.types <- validate.column(point.types, colnames(s.annotation))
	if (length(point.types) == 0) {
		stop("unsupported column in point.types")
	}
	if (point.types == -1L) {
		stop("invalid value for point.types")
	}
	point.colors <- validate.column(point.colors, colnames(s.annotation))
	if (length(point.colors) == 0) {
		stop("unsupported column in point.colors")
	}
	if (point.colors == -1L) {
		stop("invalid value for point.colors")
	}
	if (!((is.double(legend.space) || is.integer(legend.space)) && length(legend.space) == 1 && (!is.na(legend.space)) &&
		  	0 <= legend.space)) {
		stop("invalid value for legend.space")
	}

	## Extract high-dimensional methylation values
	X <- t(meth(rnb.set, target))
	if (nrow(X) < 4) {
		stop("too few samples")
	}

	site.nas <- colMeans(is.na(X))
	if (plot.type == "mds") {
		i <- which(site.nas == 1)
	} else { # plot.type %in% c("pca", "tsne")
		i <- which(site.nas != 0)
	}
	info <- list("Target" = target, "Technique" = toupper(substr(plot.type, 1, 3)),
		"All" = ncol(X), "Missing" = length(i), "Selected" = ncol(X) - length(i), "Explained" = 1)
	if (length(i) != 0) {
		if (ncol(X) - length(i) < 4) {
			stop("too many missing values")
		}
		## Remove sites that contain (only) NAs
		X <- X[, -i]
		site.nas <- site.nas[-i]
	}
	top.sites <- rnb.getOption("exploratory.top.dimensions")
	site.nas <- which(site.nas != 0)
	if (0 < top.sites && top.sites < ncol(X)) {
		if (top.sites < 4) {
			stop("too few sites selected")
		}
		site.vars <- colVars(X, na.rm = TRUE)
		i <- order(site.vars, decreasing = TRUE)[1:top.sites]
		info$selected <- top.sites
		info$explained <- sum(site.vars[i]) / sum(site.vars)
		X <- X[, i]
	}

	## Perform dimension reduction
	if (plot.type == "mds") {
		X <- mds(X, distance.metric)
		colnames(X) <- paste("MDS dimension", 1:2)
	} else if (plot.type == "pca") {
		X <- prcomp(X, center = TRUE, scale. = FALSE)$x
		if (ncol(X) < max(dimensions)) {
			stop(paste("unsupported value for dimensions;", ncol(X), "principal components available"))
		}
		X <- X[, dimensions]
		colnames(X) <- paste("Principal component", dimensions)
	} else { # plot.type == "tsne"
		X <- tsne(dist(X, method = distance.metric), epoch = 2000)
		colnames(X) <- paste("Dimension", 1:2)
	}

	## Define the data and mappings to plot
	dframe <- data.frame(x = X[, 1], y = X[, 2])
	p.aes <- list("x" = "x", "y" = "y")
	plot.labs <- list("x" = colnames(X)[1], "y" = colnames(X)[2])
	if (point.colors != 0) {
		i <- s.annotation[, point.colors]
		if (all(is.na(i))) {
			rnb.warning("No data to visualize in point.colors")
			point.colors <- 0L
		} else if (is.logical(i) || is.character(i) || is.factor(i)) {
			dframe$pcolor <- as.factor(i)
		} else if (is.integer(i) || is.double(i)) {
			dframe$pcolor <- as.double(i)
		} else { # the trait cannot be mapped to point colors
			rnb.warning("Cannot map point.colors to colors")
			point.colors <- 0L
		}
		if (point.colors != 0) {
			p.aes[["color"]] <- "pcolor"
			plot.labs[["color"]] <- colnames(s.annotation)[point.colors]
		}
	}
	if (point.types != 0) {
		i <- s.annotation[, point.types]
		if ((is.character(i) || is.factor(i) || is.integer(i)) && (!any(is.na(i))) && anyDuplicated(i) == 0) {
			dframe$id <- as.character(i)
			p.aes[["label"]] <- "id"
		} else if (point.types == point.colors && (is.factor(dframe[, p.aes$color]))) {
			p.aes[["shape"]] <- p.aes[["color"]]
			plot.labs[["shape"]] <- plot.labs[["color"]]
		}  else if (is.logical(i) || is.character(i) || is.factor(i) || is.integer(i)) {
			dframe$ptype <- as.factor(i)
			p.aes[["shape"]] <- "ptype"
			plot.labs[["shape"]] <- colnames(s.annotation)[point.types]
		} else { # the trait cannot be mapped to point types
			rnb.warning("Cannot map point.types to point types")
		}
	}

	## Create the ggplot object
	pp <- ggplot2::ggplot(dframe) + do.call(ggplot2::aes_string, p.aes) + do.call(ggplot2::labs, plot.labs) +
		ggplot2::coord_fixed()
	if ("id" %in% colnames(dframe)) {
		pp <- pp + ggplot2::geom_text()
	} else {
		pp <- pp + ggplot2::geom_point()
	}
	if ("pcolor" %in% colnames(dframe)) {
		if (is.factor(dframe$pcolor)) {
			v2c <- rep_len(rnb.getOption("colors.category"), length.out = nlevels(dframe$pcolor))
			pp <- pp + ggplot2::scale_color_manual(na.value = "#C0C0C0", values = v2c)
		} else { # is.double(dframe$pcolor)
			v2c <- rnb.getOption("colors.3.gradient")
			mp <- range(dframe$pcolor, na.rm = TRUE)
			if (mp[1] < 0 && 0 < mp[2]) {
				mp <- 0
			} else {
				mp <- mean(mp)
			}
			pp <- pp + ggplot2::scale_color_gradient2(low = v2c[1], mid = v2c[2], high = v2c[3], midpoint = mp)
		}
	}
	if ("shape" %in% names(p.aes)) {
		v2shape <- rep(rnb.getOption("points.category"), length.out = nlevels(dframe[, p.aes[["shape"]]]))
		pp <- pp + ggplot2::scale_shape_manual(na.value = 1L, values = v2shape)
	}
	if (length(intersect(c("pcolor", "ptype"), colnames(dframe))) == 0) {
		pp <- pp + theme(plot.margin = unit(rep(0.1, 4), "in"))
	} else {
		pp <- pp + theme(plot.margin = unit(0.1 + c(0, legend.space, 0, 0), "in"),
			legend.justification = c(0, 0.5), legend.position = c(1.04, 0.5))
	}

	attr(pp, "info") <- info
	pp
}

########################################################################################################################

#' plotcdf
#'
#' Plots the cumulative distribution function estimated from the given sequence of values.
#'
#' @param fname  Base file name for the generated plot.
#' @param report Report that will contain the plot.
#' @param values Sequence of values sampled from a distribution.
#' @param width  Width, in inches, of the generated PDF image file.
#' @param height Height, in inches, of the generated PDF image file.
#' @param main   Title of the plot.
#' @param xlab   Label of the x axis.
#' @return Newly generated plots as an object of type \code{\linkS4class{ReportPlot}}.
#'
#' @author Yassen Assenov
#' @noRd
plotcdf <- function(fname, report, values, width = 6.2, height = 6.2, main = NA, xlab = "Count", ...) {
	rplot <- createReportPlot(fname, report, width = width, height = height)
	cd.function <- ecdf(values)
	xs <- knots(cd.function)
	dframe <- data.frame("x" = xs, "y" = cd.function(xs), "xend" = c(xs[-1], xs[length(xs)]))
	pp <- ggplot(dframe, aes_string(x = "x", y = "y")) + labs(x = xlab, y = "Fn(x)") +
		geom_segment(aes_string(xend = "xend", yend = "y"), color = "blue") +
		coord_cartesian(xlim = range(values), ylim = c(0, 1)) + theme(plot.margin = unit(0.1 + c(0, 0, 0, 0), "in"))
	print(pp)
	return(off(rplot))
}

########################################################################################################################

## Creates a deviation plot using the supplied statistics.
##
## @param stats           ...
## @param ylim            ...
## @param additional.info ...
## @param line.col        ...
## @param fill.col        ...
## @param border.col      ...
## @param subtitle        ...
## @param xlab            ...
## @param mar             ...
## @param ...             ...
## @return
## @author Yassen Assenov
deviation.plot <- function(stats, ylim = range(stats), additional.info = NULL,
	line.col = "blue", fill.col = "#FFFF00", border.col = "#BEBEBE",
	subtitle = NA, xlab = NA, mar = c(3, 4, 1, 1) + 0.1, ...) {
	s.indices <- NA
	N <- ncol(stats)
	if (is.matrix(additional.info)) {
		xlabcols <- (ncol(additional.info) == N)
	} else if (is.vector(additional.info)) {
		xlabcols <- (length(additional.info) == N)
	} else {
		xlabcols <- FALSE
	}
	if (xlabcols) {
		screens <- rbind(c(0, 1, 0.20, 1), c(0, 1, 0, 0.20))
		s.indices <- split.screen(screens)
		screen(s.indices[1])
		mar.bottom <- 0.5
	} else {
		mar.bottom <- mar[1]
	}
	par(mar = c(mar.bottom, mar[2:4]), las = 1)
	med.index <- ifelse(nrow(stats) == 1, 1, 2)
	plot(x = c(1, N), y = range(stats), type = "n",
		bty = "n", ylim = ylim, xaxs = "i", xaxt = "n", lwd = 2,
		col = line.col, xlab = NA, ...)
	if (!is.na(subtitle)) {
		mtext(subtitle)
	}
	if (med.index == 2) {
		xx <- c(1:N, N:1)
		yy <- c(stats[1, 1:N], stats[3, N:1])
		polygon(xx, yy, col = fill.col, border = border.col, lwd = 1)
	}
	lines(x = 1:N, stats[med.index, 1:N], lwd = 2, col = line.col)
	polygon(x = c(1, N, N, 1), y = rep(ylim, each = 2))

	if (xlabcols) {
		## Add color bar below the deviation plot
		screen(s.indices[2])
		par(mar = c(mar[1], mar[2], 0, mar[4]))
		if (is.matrix(additional.info)) {
			plot(c(1, N), c(0, 1), type = "n", main = NA, xlab = NA, ylab = NA, xaxs = "i", axes = FALSE)
			for (i in 1:nrow(additional.info)) {
				if (i == 1) {
					ys.start <- rep(0, N)
				} else {
					ys.start <- additional.info[i - 1, ]
				}
				ys.end <- additional.info[i, ]
				j <- which(ys.start != ys.end)
				if (length(j) != 0) {
					ys <- as.vector(rbind(ys.start[j], ys.end[j], rep(as.double(NA), length(j))))
					lines(rep(j, each = 3), ys, col = rownames(additional.info)[i], lwd = 1)
				}
			}
		} else { # is.vector(additional.info)
			plot(rep(1, N), type = "h", col = additional.info,
				ylim = c(0, 1), main = NA, sub = NA, xlab = NA, ylab = NA,
				xaxs = "i", axes = FALSE)
		}
		if (!is.na(xlab)) {
			mtext(xlab, side = 1, line = 1)
		}
		close.screen(all.screens = TRUE)
	} else if (!is.na(xlab)) {
		mtext(xlab, side = 1, line = 1)
	}

	if (med.index == 2) {
		return(mean(stats[3, ] - stats[1, ]))
	}
	return(0)
}

########################################################################################################################

## deviation.plot.beta.get.cuts
##
## Determines the grouping of sites or regions (if necessary) for a deviation plot.
## 
## @param bstats Total number of sites, probes or regions that will be summarized in a deviation plot.
## @return \code{integer} vector of length \code{N}, storing the group indices for every site or region; \code{NULL} if
##         no grouping is necessery.
## @author Yassen Assenov
deviation.plot.beta.get.cuts <- function(N) {
	if (N <= 4000L) {
		return(NULL)
	}
	if (N <= 40000L) {
		return(cut(1:N, breaks = 2000L, labels = FALSE))
	}
	# N > 40000
	return(cut(1:N, breaks = 4000L, labels = FALSE))
}

########################################################################################################################

#' deviation.plot.beta
#'
#' Creates a deviation plot based on the methylation beta values of a population.
#'
#' @param betas    Non-empty numeric \code{matrix} of methylation beta values. Rows in this matrix must denote sites or
#'                 regions, and columns - samples. If a locus (row in the matrix) contains missing values only, it is
#'                 not included in the plot.
#' @param c.values Vector (usually a \code{factor}) storing category or quantitative values for each site or region. The
#'                 length of this vector must be equal to \code{nrow(betas)}, the \emph{i}-th element storing the
#'                 property values for the \emph{i}-th locus in \code{betas}. Note that this vector's names, if present,
#'                 are ignored.
#' @param c.legend If \code{c.values} stores categories, this parameter specifies the mapping from property values to
#'                 colors. The mapping is in the form of a named \code{character} vector. All values that appear in
#'                 \code{c.values} must be present among the names of this vector. The order of the values in this
#'                 mapping determines in which order the colors are stacked (when the number of loci is large). If
#'                 \code{c.values} denotes a quantitative measure, this parameter is a singleton \code{integer},
#'                 specifying the color scheme for visualizing the values. Currently, the only supported values are
#'                 \code{2} and \code{3}. See \code{\link{rnb.options}} for more details.
#' @return Methylation variability as a number between \code{0} and \code{1}, invisibly. This number denotes the relative
#'         area of variation in the generated plot.
#'
#' @author Yassen Assenov
#' @export
deviation.plot.beta <- function(betas, c.values = NULL, c.legend = NULL) {
	if (!(is.numeric(betas) && is.matrix(betas))) {
		stop("invalid value for betas; numeric matrix expected")
	}
	if (nrow(betas) * ncol(betas) == 0) {
		stop("invalid value for betas; unexpected dimensions")
	}
	if (is.null(c.values)) {
		if (!is.null(c.legend)) {
			stop("invalid value for c.legend; expected NULL")
		}
	} else { # !is.null(c.values)
		if (!is.vector(c.values)) {
			stop("invalid value for c.values")
		}
		if (is.character(c.legend)) {
			val.occurring <- unique(c.values)
			val.legend <- names(c.legend)
			if (is.null(val.legend)) {
				stop("invalid value for c.legend; expected named character")
			}
			if (length(setdiff(val.legend, val.occurring)) != 0) {
				stop("invalid value for c.legend; mapping is incomplete")
			}
			c.legend <- c.legend[intersect(val.legend, val.occurring)]
		} else if (!(is.integer(c.legend) && length(c.legend) == 1 && c.legend %in% c(2L, 3L))) {
			stop("invalid value for c.legend")
		}
	}
	bstats <- apply(betas, 1, quantile, probs = c(0.05, 0.5, 0.95), na.rm = TRUE)
	site.order <- order(bstats[2, ], bstats[1, ], bstats[3, ], na.last = NA)
	bstats <- bstats[, site.order]
	c.values <- c.values[site.order]
	invisible(deviation.plot.beta.internal(bstats, c.values, c.legend))
}

deviation.plot.beta.internal <- function(bstats, c.values, c.legend, cuts = deviation.plot.beta.get.cuts(ncol(bstats))) {
	N <- ncol(bstats)
	additional.info <- NULL
	if (is.null(cuts)) {
		if (!is.null(c.values)) {
			additional.info <- c.legend[c.values]
		}
	} else {
		bstats <- sapply(tapply(1:N, cuts, function(x) {
					if (length(x) != 1) { return(rowMeans(bstats[, x])) }
					bstats[, x]
				}), identity)
		if (!is.null(c.values)) {
			if (is.character(c.legend)) {
				value.stats <- rep.int(0L, length(c.legend))
				names(value.stats) <- names(c.legend)
				additional.info <- sapply(tapply(c.values, cuts, function(x) {
							result <- value.stats
							values <- table(x)
							result[names(values)] <- values
							return(result / sum(result))
						}), cumsum)
				if (is.matrix(additional.info)) {
					rownames(additional.info) <- c.legend
				}
			} else { # is.integer(c.legend)
				if (c.legend == 2) {
					colors.grad <- rnb.getOption("colors.gradient")
					color.scheme <- gplots::colorpanel(20, low = colors.grad[1], high = colors.grad[2])
				} else { # c.legend == 3
					colors.grad <- rnb.getOption("colors.3.gradient")
					color.scheme <- gplots::colorpanel(20, low = colors.grad[1], mid = colors.grad[2], high = colors.grad[3])
				}
				v.range <- range(c.values, na.rm = TRUE)
				v.sections <- seq(v.range[1], v.range[2], length.out = 21)[-1]
				get.fractions <- function(x) { ecdf(x)(v.sections) }
				additional.info <- sapply(tapply(c.values, cuts, get.fractions), identity)
				if (is.matrix(additional.info)) {
					rownames(additional.info) <- color.scheme
				}
			}
			
		}
	}

	deviation.plot(bstats, ylim = c(0, 1), additional.info, mar = c(1, 4, 1, 1) + 0.1, ylab = expression(beta))
}


#### modified CGH profile plotting function from GLAD package


plotCGHProfile <- function(profileCGH, variable="LogRatio", Chromosome=NULL,
		Smoothing=NULL, GNL="ZoneGNL", Bkp=FALSE,
		labels=TRUE, plotband=TRUE, unit=0,
		colDAGLAD=c("black","blue","red","green","yellow"),
		pchSymbol=c(20,13),
		colCytoBand=c("white","darkblue"),
		colCentro="red", text=NULL, cytoband = NULL,
		main="", ylim=NULL, ...)
{
	
	if(is.null(cytoband))
	{
		stop("Error: cytoband must be provided")
	}
	
	if (length(intersect(names(profileCGH$profileValues),"PosBase"))<1)
	{
		stop("Error in plotProfile.profileCGH: PosBase is not available")
	}
	
	if (!is.null(Smoothing))
	{
		if (length(intersect(names(profileCGH$profileValues),Smoothing))<1)
		{
			print(paste("Warning in plotProfile.profileCGH:", Smoothing," is not available"))
		}
	}
	
	if (Bkp)
	{
		if (length(intersect(names(profileCGH$profileValues),"Breakpoints"))<1)
		{
			print("Warning in plotProfile.profileCGH: Breakpoints is not available")
		}
	}
	
	profileCGH$profileValues$VarToPlot <- profileCGH$profileValues[,variable]
	profileCGH$profileValues$Chromosome <- ChrNumeric(profileCGH$profileValues$Chromosome)
	
	ChrNum <- TRUE
	
	
	indexna <- attr(na.omit(profileCGH$profileValues[variable]),"na.action")
	if(!is.null(indexna))
	{
		profileCGH$profileValues <- profileCGH$profileValues[-indexna,]
	}
	
#    cytoband <- NULL
#    data("cytoband")
	
	if (!is.null(Chromosome))
	{
		ind <- NULL
		for (Chr in Chromosome)
		{
			indChr <- which(profileCGH$profileValues$Chromosome==Chr)
			ind <- c(ind,indChr)
		}
		profileCGH$profileValues <- profileCGH$profileValues[ind,]
		profileCGH$profileValues <- profileCGH$profileValues[order(profileCGH$profileValues$PosOrder),]
	}
	
	
	LabelChr <- unique(na.omit(profileCGH$profileValues$Chromosome))
	NbChr <- length(LabelChr)
	LabelChr <- data.frame(Chromosome=LabelChr)
	
	### Information dans les cytobandes
	genomeInfo <- aggregate(cytoband$End, list(Chromosome=cytoband$Chromosome, ChrNumeric=cytoband$ChrNumeric), max, na.rm=TRUE)
	names(genomeInfo) <- c("Chromosome", "ChrNumeric", "Length")
	genomeInfo$Chromosome <- as.character(genomeInfo$Chromosome)
	genomeInfo$ChrNumeric <- as.integer(as.character(genomeInfo$ChrNumeric))
	
	if (ChrNum)
	{        
		LabelChr <- merge(LabelChr, genomeInfo[,c("ChrNumeric","Length")], by.x="Chromosome", by.y="ChrNumeric", all.x=TRUE)
		LabelChr <- LabelChr[order(LabelChr$Chromosome),]
	}
	else
	{
		LabelChr <- merge(LabelChr, genomeInfo, by="Chromosome", all.x=TRUE)
		LabelChr <- LabelChr[order(LabelChr$ChrNumeric),]
	}
	
	
	
	if (NbChr > 1)
	{
		gap <- 100000000/3
		LabelChr$Length <- LabelChr$Length + gap
		
		cumulLength <- cumsum(LabelChr$Length)
		LabelChr$Length <- c(0, cumulLength[1:(NbChr-1)])                                                             
		LabelChr$Length <- LabelChr$Length/(10^unit)
	}
	
	else
	{
		LabelChr$Length <- 0
	}
	
	
	if (ChrNum)
	{
		cytobandNew <- subset(cytoband, select=setdiff(names(cytoband),"Chromosome"))
		cytobandNew <- merge(LabelChr, cytobandNew, by.x="Chromosome", by.y="ChrNumeric")
	}
	else
	{
		cytobandNew <- subset(cytoband, select=setdiff(names(cytoband),"ChrNumeric"))
		cytobandNew <- merge(LabelChr, cytobandNew, by="Chromosome") 
	}
	
	cytobandNew$Start <- cytobandNew$Start/(10^unit)
	cytobandNew$End <- cytobandNew$End/(10^unit)
	
	cytobandNew$Start <- cytobandNew$Start + cytobandNew$Length
	cytobandNew$End <- cytobandNew$End +  cytobandNew$Length
	
	profileCGH$profileValues <- merge(profileCGH$profileValues, LabelChr, by="Chromosome")
	
	profileCGH$profileValues$NewPosBase <- profileCGH$profileValues$PosBase + profileCGH$profileValues$Length
	
	def.par <- par(no.readonly = TRUE)
	
	##    if (plotband)
	##       {
	##         layout(c(1,2), heights=c(4,1))
	##         par(mar=c(0,4,5,2))
	##       }
	
	if (plotband)
	{
		### Cytobandes
		
		layout(c(1,2), heights=c(1,4))
		
		par(mar=c(0,4,4,2))
		
		plot(0, type="n", xlim=c(0, max(cytobandNew$End)),
				ylim=c(-1.5,1.5), xaxt="n", yaxt="n", ylab="", xlab="")
		
		LabelChrCyto <- unique(cytobandNew$Chromosome)
		
		
		for (i in 1:NbChr)
		{
            logger.warning('GLAD package currently not available')
			#GLAD::plotCytoBand(cytobandNew, Chromosome=LabelChrCyto[i], labels=labels, y=0, height=2, colCytoBand=colCytoBand, colCentro=colCentro)
		}
		
		par(mar=c(4,4,0,2))
	}
	
	if (!is.null(Smoothing))
	{
		profileCGH$profileValues <- profileCGH$profileValues[order(profileCGH$profileValues$Chromosome,profileCGH$profileValues$PosBase),]
		NbPos <- length(profileCGH$profileValues[,1])
		PosMax <- max(profileCGH$profileValues$NewPosBase) + 1
		Pos <- profileCGH$profileValues$NewPosBase[1:(NbPos-1)]
		PosNext <- profileCGH$profileValues$NewPosBase[2:NbPos]
		InterPos <- Pos + (PosNext-Pos)/2
		InterPos <- c(0, InterPos, PosMax)
		
		SmtStart <- profileCGH$profileValues[,Smoothing][1]
		SmtEnd <- profileCGH$profileValues[,Smoothing][NbPos]
		
		Smt1 <- profileCGH$profileValues[,Smoothing][1:(NbPos-1)]
		Smt1 <- c(SmtStart, Smt1, SmtEnd)
		
		Smt2 <- profileCGH$profileValues[,Smoothing][2:NbPos]
		Smt2 <- c(SmtStart, Smt2, SmtEnd)
		
		
		datasmt <- data.frame(PosBase=c(InterPos,InterPos),Smoothing=c(Smt1,Smt2))
		datasmt <- unique(datasmt)
		datasmt <- datasmt[order(datasmt$PosBase),]
	}
	
	if (length(intersect(names(profileCGH$profileValues),GNL))>=1)
	{
		
		col <- rep(colDAGLAD[5],length(profileCGH$profileValues$PosOrder))
		col[which(profileCGH$profileValues[GNL]==-1)] <- colDAGLAD[4]
		col[which(profileCGH$profileValues[GNL]==1)] <- colDAGLAD[3]
		col[which(profileCGH$profileValues[GNL]==2)] <- colDAGLAD[2]
		col[which(profileCGH$profileValues[GNL]==-10)] <- colDAGLAD[1]
		
		
		outliers <- rep(pchSymbol[1],length(profileCGH$profileValues$PosOrder))
		outliers[which(profileCGH$profileValues$OutliersTot!=0)] <- pchSymbol[2]
		
		if (plotband)
		{
			
			plot(VarToPlot ~ NewPosBase, data=profileCGH$profileValues,
					pch=outliers, col=col, xaxt="n", xlab=main,
					ylab=variable, ylim=ylim, xlim=c(0,max(cytobandNew$End)),...)
		}
		else
		{
			plot(VarToPlot ~ NewPosBase, data=profileCGH$profileValues,
					pch=outliers, col=col, xaxt="n", xlab="", ylab=variable, main=main, ylim=ylim, ...)
		}
		
		
		if (!is.null(Smoothing))
		{
			lines(datasmt$Smoothing ~ datasmt$PosBase, col="black")
		}
		
		if (Bkp)
		{
			if (is.data.frame(profileCGH$BkpInfo))
			{
				profileCGH$BkpInfo <- merge(profileCGH$BkpInfo, LabelChr, by="Chromosome")
				profileCGH$BkpInfo$NewPosBase <- profileCGH$BkpInfo$PosBase + profileCGH$BkpInfo$Length
				abline(v=profileCGH$BkpInfo$NewPosBase+0.5, col="red", lty=2)
			}
		}
	}
	else
	{
		if (plotband)
		{
			plot(VarToPlot ~ NewPosBase, data=profileCGH$profileValues,
					pch=20, xaxt="n", xlab=main, ylab=variable, ylim=ylim, xlim=c(0,max(cytobandNew$End)), ...)
		}
		else
		{
			plot(VarToPlot ~ NewPosBase, data=profileCGH$profileValues,
					pch=20, xaxt="n", xlab="", ylab=variable, main=main, ylim=ylim, ...)
		}
		
		if (Bkp)          
		{
			if (is.data.frame(profileCGH$BkpInfo))
			{
				profileCGH$BkpInfo <- merge(profileCGH$BkpInfo, LabelChr, by="Chromosome")
				profileCGH$BkpInfo$NewPosBase <- profileCGH$BkpInfo$PosBase + profileCGH$BkpInfo$Length
				abline(v=profileCGH$BkpInfo$NewPosBase+0.5, col="red", lty=2)
			}
		}
		
		if (!is.null(Smoothing))
		{
			lines(datasmt$Smoothing ~ datasmt$PosBase, col="red")
		}
	}
	
	if (!is.null(text))
	{
		text(text$x, text$y, labels=text$labels, cex=text$cex)
	}
	
#    par(def.par)
}

########################################################################################################################

#' densRanks
#'
#' Rank the points accordind to density of the region they fall in. Densities are computed
#' as Kernel Density estimates. The method and parameters are implemented in analogy to
#' \code{grDevices::densCols}
#' @param x x-coordinate
#' @param y y-coordinate
#' @param nbin number of bins
#' @param bandwidth bandwidth
#' @author Fabian Mueller
densRanks <- function (x, y = NULL, nbin = 128, bandwidth) 
{
    xy <- xy.coords(x, y)
    select <- is.finite(xy$x) & is.finite(xy$y)
    x <- cbind(xy$x, xy$y)[select, ]
    map <- grDevices:::.smoothScatterCalcDensity(x, nbin, bandwidth)
    mkBreaks <- function(u) u - diff(range(u))/(length(u) - 1)/2
    xbin <- cut(x[, 1], mkBreaks(map$x1), labels = FALSE)
    ybin <- cut(x[, 2], mkBreaks(map$x2), labels = FALSE)
    dens <- map$fhat[cbind(xbin, ybin)]
    dens[is.na(dens)] <- 0

    res <- rep(NA_integer_, length(select))
    rrs <- rank(dens,ties.method="max")
    res[select] <- rrs
    res
}

########################################################################################################################

#' create.densityScatter
#' 
#' Creates a density scatterplot highlighting points in sparsely populated plot regions
#' as well as points marked as special in a seperate color
#' @param df2p \code{data.frame} to be plotted. Only the fist two columns are taken into account as
#' 			x and y coordinates respectively
#' @param is.special boolean vector of length equal to the number of rows in \code{df2p}. Specifies
#' 			which points should be highlighed seperately in a different color
#' @param dens.subsample if the number of points exceeds this number, subsample the number of points for the
#'			density estimation to that number. Any non-numeric value disables subsampling.
#' @param dens.special Flag indicating whether the points of the special population should be colored
#' 			according to their density
#' @param sparse.points Either percentage (\code{<=1,>=0}) or the absolute number 
#'          of points in the sparsely populated area that should be drawn seperately. A value of 0 means that these points
#'			will not be drawn.
#' @param dens.n passed on to \code{ggplot2::stat_density2d}: argument: \code{n}
#' @param add.text.cor flag indicating whether a text token with the correlation coefficient should be included in the lower
#'          right corner of the plot
#' @return \code{ggplot} object
#' @author Fabian Mueller
#' @export
#' @examples
#' \donttest{
#' d <- data.frame(x=rnorm(1000),y=rnorm(1000))
#' s <- rep(FALSE,1000)
#' s[sample(1:length(s),100)] <- TRUE
#' create.densityScatter(d,s)
#' }
create.densityScatter <- function(df2p,is.special=NULL,dens.subsample=FALSE,dens.special=TRUE,
		sparse.points=0.01,dens.n=100,add.text.cor=FALSE){
	if (!(is.numeric(sparse.points) && sparse.points>=0)) {
		stop("Invalid parameter value: sparse.points")
	}
	if (!is.null(is.special)) is.special[is.na(is.special)] <- FALSE
	if (sum(is.special)<1){
		is.special <- NULL
	}
	if (!is.null(is.special)){
		df2p$is.special <- is.special
	}
	if (is.null(df2p) || nrow(df2p)<1){
		logger.warning(c("Could not create density scatterplot"))
		pp <- rnb.message.plot("Could not create plot")
		return(pp)
	}
	df2p <- na.omit(df2p)
	if (is.null(df2p) || nrow(df2p)<1){
		logger.warning(c("Could not create density scatterplot (NA omission removed all entries)"))
		pp <- rnb.message.plot("Could not create plot")
		return(pp)
	}
	df2p.sub <- df2p
	dens.ranks <- NULL
	tryCatch(
		dens.ranks <- densRanks(x=df2p[,1],y=df2p[,2]),
		error=function(ee){
			logger.warning(c("Could not assess density ranking:",ee$message))
		}
	)
	if (is.numeric(dens.subsample) && dens.subsample>0){
		ss <- as.integer(dens.subsample)
		if (nrow(df2p) > ss) {
			df2p.sub <- df2p[sample(nrow(df2p),ss),]
		}
	}

	#the standard bandwith function of MASS::kde2d is unstable when looking at
	#distributions with very low variance. Here's a more stable version
	stable.bandwidth.fun <- function(x,eps=1e-4){
	    r <- quantile(x, c(0.25, 0.75))
	    h <- (r[2] - r[1])/1.34
	    if (h==0) h <- eps
	    4 * 1.06 * min(sqrt(var(x)), h) * length(x)^(-1/5)
	}
	stable.h <- c(stable.bandwidth.fun(df2p.sub[,1]),stable.bandwidth.fun(df2p.sub[,2]))

	if (is.null(dens.ranks)){
  		pp <- rnb.message.plot("Could not assess density")
  	} else {		
  		pp <- ggplot(df2p.sub) + aes_string(x=colnames(df2p)[1],y=colnames(df2p)[2]) + 
		  stat_density2d(geom="tile", fill=DENS.COLORS.LOW[1], aes(alpha=after_stat(density)^0.25), contour=FALSE, n=dens.n, h=stable.h) +
		  scale_alpha(range = c(0.0, 1))
		if (sparse.points > 0){
			if (sparse.points <= 1){
				thres <- ceiling(nrow(df2p)*sparse.points)
			} else {
				thres <- sparse.points
			}
			df2p.loose <- df2p[dens.ranks<=thres,]#the sub data.frame in of the least dens points
			pp <- pp + geom_point(data=df2p.loose,aes_string(x=colnames(df2p)[1],y=colnames(df2p)[2]),colour=DENS.COLORS.LOW[1],size=0.4)
		}
		if (!is.null(is.special)){
			df2p.special <- df2p[df2p$is.special,]
			colors.dmp <- DENS.COLORS.LOW[2]
			if (dens.special && nrow(df2p.special) > 1){
				tryCatch(
					colors.dmp   <- densCols(x=df2p.special[,1],y=df2p.special[,2],colramp = colorRampPalette(c(DENS.COLORS.LOW[2],DENS.COLORS.HIGH[2]))),
					error=function(ee){
						logger.warning(c("Could not assess density colors using densCols:",ee$message))
					}
				)
			}
			df2p.special$color <- colors.dmp

			pp <- pp + geom_point(data=df2p.special,aes_string(x=colnames(df2p)[1],y=colnames(df2p)[2],colour="color"),size=1) + scale_color_identity()
		}
		if (add.text.cor) {
			cc <- cor(df2p[,1],df2p[,2],use="pairwise.complete.obs")
			txt.cor <- paste0('rho',paste0("==",round(cc,4)))
			pp <- pp + annotate("text", x=max(df2p[,1],na.rm=TRUE),y=min(df2p[,2],na.rm=TRUE),label=txt.cor,parse=TRUE,hjust=1,vjust=1,size=4)
		}
		pp <- pp + theme(legend.position="none")
  	}
	return(pp)
}

########################################################################################################################
#' create.scatter.dens.points
#' 
#' Creates a scatterplot containing all points in a given data.frame. Points are colored according to point
#' density. Optionally, a selection of points are shown in a different color
#' @param df2p \code{data.frame} to be plotted. Only the fist two columns are taken into account as
#' 			x and y coordinates respectively
#' @param is.special boolean vector of length equal to the number of rows in \code{df2p}. Specifies
#' 			which points should be highlighed seperately in a different color
#' @param dens.special Flag indicating whether the points of the special population should be colored
#' 			according to their density
#' @param mock Should only the axis be plotted? useful when exporting scatterplots with lots of points
#'        as immage and the corresponding axis as vector graphics.
#' @return \code{ggplot} object
#' @author Fabian Mueller
#' @export
#' @examples
#' \donttest{
#' d <- data.frame(x=rnorm(1000),y=rnorm(1000))
#' s <- rep(FALSE,1000)
#' s[sample(1:length(s),100)] <- TRUE
#' create.scatter.dens.points(d,s)
#' }
create.scatter.dens.points <- function(df2p,is.special=NULL,dens.special=TRUE,mock=FALSE){
	if (!is.null(is.special)) is.special[is.na(is.special)] <- FALSE
	if (sum(is.special)<1){
		is.special <- NULL
	}
	if (!is.null(is.special)){
		df2p$is.special <- is.special
	}
	df2p <- na.omit(df2p)

	#plot order: plot DMRs last
	n.points <- nrow(df2p)
	df2p$plotOrder <- NA
	num.not.special <- sum(!df2p$is.special)
	df2p$plotOrder[!df2p$is.special] <- seq_len(num.not.special)
	df2p$plotOrder[df2p$is.special] <- seq((num.not.special+1),n.points)
	df2p <- df2p[order(df2p$plotOrder, decreasing=FALSE, na.last=FALSE),]
	
	df2p$color <- NA
	if (sum(!df2p$is.special)>1){
		colors.nodmr <- DENS.COLORS.LOW[1]
		tryCatch(
			colors.nodmr <- densCols(x=df2p[!df2p$is.special,1],y=df2p[!df2p$is.special,2],colramp = colorRampPalette(c(DENS.COLORS.LOW[1],DENS.COLORS.HIGH[1]))),
			error=function(ee){
				logger.warning(c("Could not assess density colors using densCols:",ee$message))
			}
		)
		df2p[!df2p$is.special,"color"] <- colors.nodmr
	} else if (sum(!df2p$is.special)==1){
		df2p[!df2p$is.special,"color"] <- DENS.COLORS.LOW[1]
	}
	if (sum(df2p$is.special)>1){
		colors.dmr <- DENS.COLORS.LOW[2]
		if (dens.special) {
			tryCatch(
				colors.dmr   <- densCols(x=df2p[ df2p$is.special,1],y=df2p[ df2p$is.special,2],colramp = colorRampPalette(c(DENS.COLORS.LOW[2],DENS.COLORS.HIGH[2]))),
				error=function(ee){
					logger.warning(c("Could not assess density colors using densCols:",ee$message))
				}
			)
		}
		df2p[df2p$is.special,"color"] <- colors.dmr
		
	} else if (sum(df2p$is.special)==1){
		df2p[df2p$is.special,"color"] <- DENS.COLORS.LOW[2]
	}
	pp <- ggplot(df2p) + aes_string(x=colnames(df2p)[1],y=colnames(df2p)[2])
	if (mock){
		pp <- pp + geom_blank()
	} else {
		pp <- pp + geom_point(aes_string(color="color")) + scale_color_identity()
	}
	
	return(pp)
}

########################################################################################################################
#' create.hex.summary.plot
#' 
#' Creates a summary plot binning the data given by a certain quantity in heagonal bins
#' @param df2p \code{data.frame} to be plotted.
#' @param x name of the variable in \code{df2p} considered as x-axis
#' @param y name of the variable in \code{df2p} considered as y-axis
#' @param q name of the variable in \code{df2p} considered as quantity to be summarized over bins
#' @param bins,fun,... arguments to be passed on to \code{stat_summary_hex}
#' @return \code{ggplot} object
#' @author Fabian Mueller
create.hex.summary.plot <- function(df2p,x=colnames(df2p)[1],y=colnames(df2p)[2],q=colnames(df2p)[3],
	bins=128,fun=median,...){
	pp <- ggplot(df2p) + aes_string(x=x,y=y,z=q) +
			stat_summary_hex(bins=bins,fun=fun,...) + scale_fill_gradientn(colours=rev(rnb.getOption("colors.gradient")))
	return(pp)
}

########################################################################################################################
## batch.R
## created: 2012-06-21
## creator: Yassen Assenov
## ---------------------------------------------------------------------------------------------------------------------
## Implementation of batch effects analysis and correction.
########################################################################################################################

## G L O B A L S #######################################################################################################

## Minimum percentage of variance explained; the first n principal components that explain the variance are considered
VAR.EXPLAINED <- 95

## Color codes used to represent tests for association between traits
COLORS.TEST <- c("Correlation" = "#F8766D", "Fisher" = "#7CAE00", "Wilcoxon" = "#00BFC4", "Kruskal-Wallis" = "#C77CFF")

## Color codes used to represent non-significant and significant p-values
COLORS.SIGNIFICANT <- c("FALSE" = "#9C9CCC", "TRUE" = "#F1B1B1")

## Point shapes used to represent reasons for failure of an association test
SHAPES.FAILURES <- c("not enough common values" = 22, "not enough categories" = 24, "test failed" = 21)

## F U N C T I O N S ###################################################################################################

#' mds
#'
#' Performs Kruskal's non-metric multidimensional scaling using the supplied distance metric.
#'
#' @param m           Data matrix to be scaled. Rows should denote observations, and columns - dimensions.
#' @param dist.metric Distance metric. Must be one of \code{"euclidean"} or \code{"manhattan"}.
#' @return            Two-column matrix of transformed points.
#' @author Yassen Assenov
#' @noRd
mds <- function(m, dist.metric) {
	dist.matrix <- dist(m, method = dist.metric)
	if (any(dist.matrix == 0)) {
		## Get rid of zeros in the distance matrix by adding an epsilon
		dist.matrix <- dist.matrix + min(dist.matrix[dist.matrix != 0]) / 1000
	}
	isoMDS(dist.matrix, k = 2, maxit = 100, trace = FALSE)$points
}

########################################################################################################################

#' validate.pcoordinates
#'
#' Validates that the given object is returned by \code{\link{rnb.execute.dreduction}}.
#'
#' @param pcoordinates Object to validate.
#' @return If \code{pcoordinates} is valid - the number of samples in the dataset that were processed; otherwise - an
#'         error message in the form of a one-element \code{character}.
#'
#' @author Yassen Assenov
#' @noRd
validate.pcoordinates <- function(pcoordinates) {
	## Validate the data structure
	if (!is.list(pcoordinates)) {
		return("invalid value for pcoordinates")
	}
	if (!setequal(names(pcoordinates), c("pca", "mds"))) {
		return("invalid value for pcoordinates")
	}
	if (!(is.null(pcoordinates[["pca"]]) || class(pcoordinates[["pca"]]) == "prcomp")) {
		return("invalid value for pcoordinates$pca")
	}
	pc.mds <- pcoordinates[["mds"]]
	if (!is.list(pc.mds)) {
		return("invalid value for pcoordinates$mds")
	}
	if (length(pc.mds) != 0) {
		if (!(setequal(names(pc.mds), c("euclidean", "manhattan")) &&
			  	all(sapply(pc.mds, is.matrix)) && all(sapply(pc.mds, is.double)))) {
			return("invalid value for pcoordinates$mds")
		}
		nsamples <- sapply(pc.mds, dim)
		if (any(nsamples[2,] != 2)) {
			return("invalid value for pcoordinates$mds; expected 2 dimensions")
		}
		if (any(nsamples[1, ] != nsamples[1, 1])) {
			return("invalid value for pcoordinates$mds; inconsistent number of samples")
		}
		nsamples <- as.integer(nsamples[1, 1])
	} else {
		nsamples <- 0L
	}

	## Validate the number of samples is consistent
	if (!is.null(pcoordinates[["pca"]])) {
		if (nsamples == 0) {
			nsamples <- nrow(pcoordinates[["pca"]]$x)
		} else if (!isTRUE(nsamples == nrow(pcoordinates[["pca"]]$x))) {
			return("invalid value for pcoordinates; inconsistent number of samples")
		}
	}
	return(nsamples)
}

########################################################################################################################

## validate.pcoordinates.all
##
## Validates that the given object is either returned by \code{\link{rnb.execute.dreduction}}, or is list of objects
## returned by that functions, applied on the same samples.
##
## @param pcoordinates Object to validate.
## @return If \code{pcoordinates} is valid - the number of samples in the dataset that were processed; otherwise - an
##         error message in the form of a one-element \code{character}.
##
## @author Yassen Assenov
validate.pcoordinates.all <- function(pcoordinates) {
	if (!(is.list(pcoordinates) && length(pcoordinates) != 0)) {
		return("invalid value for pcoordinates")
	}
	if (setequal(names(pcoordinates), c("mds", "pca"))) {
		nsamples <- validate.pcoordinates(pcoordinates)
	} else {
		nsamples <- lapply(pcoordinates, validate.pcoordinates)
		i.error <- which(sapply(nsamples, is.character))
		if (length(i.error) != 0) {
			nsamples <- nsamples[[i.error[1]]]
		} else {
			nsamples <- sapply(nsamples, identity)
			nsamples.max <- max(nsamples)
			if (!all(nsamples[nsamples != 0] == nsamples.max)) {
				return("invalid value for pcoordinates; inconsistent number of samples")
			}
			nsamples <- nsamples.max
		}
	}
	return(nsamples)
}

########################################################################################################################

## Checks if the given argument is a valid sample index permutation matrix.
##
## @param perm.matrix Matrix to be validated.
## @return \code{TRUE} if \code{perm.matrix} is a valid index permutation matrix, that is, an integer matrix in which
##         every column denotes a permutation; \code{FALSE} otherwise.
## @author Yassen Assenov
is.valid.permutations <- function(perm.matrix) {
	if (!(is.integer(perm.matrix) && is.matrix(perm.matrix) && nrow(perm.matrix) > 0 && ncol(perm.matrix) > 0)) {
		return(FALSE)
	}
	all.indices <- 1:nrow(perm.matrix)
	if (!identical(perm.matrix[, 1], all.indices)) {
		return(FALSE)
	}
	if (ncol(perm.matrix) > 1) {
		if (!all(apply(perm.matrix[, -1], 2, function(x) { identical(sort(x), all.indices) }))) {
			return(FALSE)
		}
	}
	return(TRUE)
}

########################################################################################################################

#' Tests for association between two traits.
#'
#' @param x           Sample values for the first trait. This must be a vector of type \code{factor}, \code{integer} or
#'                    \code{numeric}.
#' @param y           Sample values for the second trait. This must be a vector of type \code{factor}, \code{integer} or
#'                    \code{numeric}.
#' @param perm.matrix Matrix of sample permutations to be used in case none of the traits is a \code{factor}, and thus
#'                    permutation-based p-value from correlations is computed. If this parameter is \code{NULL} and
#'                    both \code{x} and \code{y} are sequences of numbers, no p-value is calculated.
#' @return            List of four elements:
#'                    \describe{
#'                      \item{error}{Error, if any, that prevented this function from computing a p-value for trait
#'                           association.}
#'                      \item{test}{Type of test performed. This is one of \code{"Fisher"}, \code{"Wilcoxon"},
#'                           \code{"Kruskal-Wallis"}, \code{"Correlation"} or \code{NA}. The last value indicates that
#'                           the traits cannot be tested for association.}
#'                      \item{correlation}{Value of the pearson correlation coefficient between \code{x} and \code{y},
#'                           or \code{NA} if any of them is \code{factor}.}
#'                      \item{pvalue}{Calculated p-value, or \code{NA} if the traits cannot be tested for association.}
#'                    }
#' @author Yassen Assenov
#' @noRd
test.traits <- function(x, y, perm.matrix = NULL) {
	result <- list(
			"error" = as.character(NA),
			"test" = as.character(NA),
			"correlation" = as.double(NA),
			"pvalue" = as.double(NA))

	## Focus on common values
	if (class(x) == "Date") { x <- as.integer(x) }
	if (class(y) == "Date") { y <- as.integer(y) }
	inds <- which(!(is.na(x) | is.na(y)))
	if (length(inds) < 2) {
		result[["error"]] <- "not enough common values"
		return(result)
	}
	x <- x[inds]
	if (is.factor(x)) {
		x <- as.factor(as.character(x))
		if (nlevels(x) < 2) {
			## Not enough categories in y
			result[["error"]] <- "not enough categories"
			return(result)
		}
	}
	y <- y[inds]
	if (is.factor(y)) {
		y <- as.factor(as.character(y))
		if (nlevels(y) < 2) {
			## Not enough categories in y
			result[["error"]] <- "not enough categories"
			return(result)
		}
	}

	## Perform a test or compute correlation
	get.p <- function(expr) { tryCatch(suppressWarnings(expr$p.value), error = function(er) { as.double(NA) }) }
	if (is.factor(x)) {
		if (is.factor(y)) {
			simulate <- (nlevels(x) > 2 || nlevels(y) > 2)
			result[["test"]] <- "Fisher"
			result[["pvalue"]] <- get.p(fisher.test(x, y, conf.int = FALSE, simulate.p.value = simulate, B = 50000))
		} else if (nlevels(x) == 2) {
			result[["test"]] <- "Wilcoxon"
			values <- tapply(y, x, identity)
			result[["pvalue"]] <- get.p(wilcox.test(values[[1]], values[[2]], alternative = "two.sided"))
		} else {
			result[["test"]] <- "Kruskal-Wallis"
			result[["pvalue"]] <- get.p(kruskal.test(y, x))
		}
	} else if (is.factor(y)) {
		if (nlevels(y) == 2) {
			result[["test"]] <- "Wilcoxon"
			values <- tapply(x, y, identity)
			result[["pvalue"]] <- get.p(wilcox.test(values[[1]], values[[2]], alternative = "two.sided"))
		} else {
			result[["test"]] <- "Kruskal-Wallis"
			result[["pvalue"]] <- get.p(kruskal.test(x, y))
		}
	} else {
		result[["test"]] <- "Correlation"
		if (is.null(perm.matrix)) {
			result[["correlation"]] <- cor(x, y)
		} else {
			N <- length(inds)
			values <- apply(perm.matrix, 2, function(i) { cor(x[i[i <= N]], y) })
			result[["correlation"]] <- values[1]
			values <- abs(values)
			result[["pvalue"]] <- mean(values[1] <= values)
		}
	}

	if (is.na(result[["pvalue"]])) {
		result[["error"]] <- "test failed"
	}
	return(result)
}

########################################################################################################################

## plot.heatmap.pc.correlations
##
## Creates a heatmap displaying a table of correlation values.
##
## @param report Report to contain the generated heatmap.
## @param tbl    Table of correlation values to be plotted.
## @param fname  File name for the plot.
## @param width  Width of the plot in inches. If this is set to \code{NULL} (default), the width is calculated based on
##               the number of columns in \code{tbl}.
## @param height Height of the plot in inches. If this is set to \code{NULL} (default), the height is calculated based
##               on the number of rows in \code{tbl}.
## @return \code{list} with two elements: \code{"plot"} and \code{"description"} containing the \code{\link{ReportPlot}}
##         object and plot's description, respectively.
## @author Yassen Assenov
plot.heatmap.pc.correlations <- function(report, tbl, fname, width = NULL, height = NULL) {
	tbl.melt <- reshape2::melt(tbl, varnames = c("x", "y"))
	colnames(tbl.melt)[3] <- "correlation"
	tbl.melt[[1]] <- factor(as.character(tbl.melt[[1]]), levels = rev(rownames(tbl)))
	tbl.melt[[2]] <- factor(as.character(tbl.melt[[2]]), levels = colnames(tbl))
	xlab <- names(dimnames(tbl))[2]
	if ((!is.null(xlab)) && (is.na(xlab) || identical(xlab, ""))) { xlab <- NULL }
	ylab <- names(dimnames(tbl))[1]
	if ((!is.null(ylab)) && (is.na(ylab) || identical(ylab, ""))) { ylab <- NULL }
	xsize <- ifelse(is.null(xlab), 2, 0.25)
	ysize <- ifelse(is.null(ylab), 2, 0.25)
	if (is.null(width)) {
		width <- ysize + ncol(tbl) * 0.28 + 1.6
	}
	if (is.null(height)) {
		height <- xsize + nrow(tbl) * 0.28 + 0.2
	}
	colors.g <- rnb.getOption("colors.3.gradient")
	pp <- ggplot(tbl.melt) + aes_string("y", "x") + labs(x = xlab, y = ylab) + coord_fixed() +
		geom_tile(aes_string(fill = "correlation"), color = "white") +
		scale_fill_gradient2(limits = c(-1, 1), low = colors.g[1], mid = colors.g[2], high = colors.g[3]) +
		scale_x_discrete(expand = c(0, 0)) + scale_y_discrete(expand = c(0, 0)) +
		theme(axis.ticks = element_blank(), legend.justification = c(0, 0.5), legend.position = c(1, 0.5)) +
		theme(panel.border = element_blank(), plot.margin = unit(c(0.1, 1.5, 0.1, 0.1), "in"))
	pp <- suppressWarnings(ggplot_gtable(ggplot_build(pp)))
	pp$widths[[3]] <- unit(ysize, "in")
	pp$heights[[length(pp$heights) - 2L]] <- unit(xsize, "in")
	rplot <- createReportPlot(fname, report, width = width, height = height)
	grid.newpage()
	grid.draw(pp)
	rplot <- off(rplot)
	txt <- "Heatmap presenting a table of correlations. Grey cells, if present, denote missing values."
	return(list("plot" = rplot, "description" = txt))
}

########################################################################################################################

plot.heatmap.pc.pvalues <- function(report, tbl, fname, width = NULL, height = NULL) {
	tbl.melt <- reshape2::melt(tbl, varnames = c("x", "y"))
	colnames(tbl.melt)[3] <- "pvalue"
	tbl.melt[[1]] <- factor(as.character(tbl.melt[[1]]), levels = rev(rownames(tbl)))
	tbl.melt[[2]] <- factor(as.character(tbl.melt[[2]]), levels = colnames(tbl))
	tbl.melt[["significant"]] <-
		as.character(tbl.melt[, "pvalue"] < rnb.getOption("exploratory.correlation.pvalue.threshold"))
	tbl.melt[is.na(tbl.melt[["significant"]]), "significant"] <- "NA"
	tbl.melt[["pvaltext"]] <- toupper(format(tbl.melt[, "pvalue"], digits = 2, scientific = TRUE))
	tbl.melt[["pvaltext"]] <- ifelse(tbl.melt[["significant"]] == "TRUE", tbl.melt[["pvaltext"]], "")
	xlab <- names(dimnames(tbl))[2]
	if ((!is.null(xlab)) && (is.na(xlab) || identical(xlab, ""))) { xlab <- NULL }
	ylab <- names(dimnames(tbl))[1]
	if ((!is.null(ylab)) && (is.na(ylab) || identical(ylab, ""))) { ylab <- NULL }
	if (is.null(width)) {
		width <- ifelse(is.null(ylab), 2, 1) + ncol(tbl) * 0.62
	}
	if (is.null(height)) {
		height <- ifelse(is.null(xlab), 2, 1) + nrow(tbl) * 0.25
	}
	pp <- ggplot(tbl.melt, aes_string("y", "x", label = "pvaltext")) + labs(x = xlab, y = ylab) +
		coord_fixed(ratio = 0.25 / 0.62) + geom_tile(aes_string(fill = "significant"), color = "white") +
		scale_fill_manual(values = c("NA" = "#D0D0D0", "FALSE" = "#9C9CCC", "TRUE" = "#F1B1B1")) +
		geom_text(size = 3) + scale_x_discrete(expand = c(0, 0)) + scale_y_discrete(expand = c(0, 0)) +
		theme(panel.border = element_blank(), panel.grid = element_blank()) +
		theme(plot.margin = unit(0.1 + c(0, 0, 0, 0), "in")) +
		theme(axis.ticks = element_blank(), legend.position = "none")
	## Fix the areas for x and y axis labels
	pp <- suppressWarnings(ggplot_gtable(ggplot_build(pp)))
	if (is.null(ylab)) {
		pp$widths[[3]] <- unit(2, "in")
	}
	if (is.null(xlab)) {
		pp$heights[[length(pp$heights) - 2L]] <- unit(2, "in")
	}
	rplot <- createReportPlot(fname, report, width = width, height = height)
	grid.newpage()
	grid.draw(pp)
	txt <- paste0("Heatmap presenting a table of p-values. Significant p-values (less than ",
		rnb.getOption("exploratory.correlation.pvalue.threshold"), ") are printed in pink boxes. Non-significant ",
		"values are represented by blue boxes. Bright grey cells, if present, denote missing values.")
	return(list("plot" = off(rplot), "description" = txt))
}

########################################################################################################################

## plot.heatmap.symm
##
## Creates a lower-diagonal heatmap representing the selected tests applied between pairs of traits, or the resulting
## p-values.
##
## @param report       Report to contain the generated plot(s).
## @param tbl.symm     Symmetric \code{matrix} of type \code{character} storing test names, or of type \code{double}
##                     storing resulting p-values. This function expects the test names to be among the following:
##                     \code{"Correlation"}, \code{"Fisher"}, \code{"Wilcoxon"}, \code{"Kruskal-Wallis"}.
## @param tbl.failures Optional, a symmetric \code{matrix} of type \code{character} storing explanations for test
##                     failures. If provided, this matrix must have the same dimensions and dimension names as
##                     \code{tbl.symm} and its values must be among the following: \code{"not enough common values"},
##                     \code{"not enough categories"} and \code{"test failed"}.
## @param fname        File name for the plot.
## @return Generated plot(s) as an object of type \code{\linkS4class{ReportPlot}} or a \code{list} of such objects.
##
## @details
## The type of heatmap to be draw is determined based on the type of the provided symmetric matrix:
## \code{typeof(tbl.symm)}. Test names are assumed when this type is \code{character}, and p-values otherwise.
##
## @author Yassen Assenov
plot.heatmap.symm <- function(report, tbl.symm, tbl.failures = NULL, fname) {
	do.tests <- (typeof(tbl.symm) == "character")
	# Note that tbl.symm must be symmetric of size at least 2x2; this is not checked

	## Create a data frame with tests/p-values
	tbl <- tbl.symm
	tbl[upper.tri(tbl)] <- NA
	tbl <- tbl[-1, -ncol(tbl), drop = FALSE]
	tbl.melt <- reshape2::melt(tbl, varnames = c("x", "y"))
	colnames(tbl.melt)[3] <- "test"
	if (do.tests) {
		col.mapping <- COLORS.TEST
		tbl.melt[["test"]] <- factor(as.character(tbl.melt[["test"]]), levels = names(col.mapping))
		tbl.melt[["pvaltext"]] <- ""
	} else {
		col.mapping <- COLORS.SIGNIFICANT
		tbl.melt[["significant"]] <-
			as.character(tbl.melt[, "test"] < rnb.getOption("exploratory.correlation.pvalue.threshold"))
		tbl.melt[["pvaltext"]] <- toupper(format(tbl.melt[, "test"], digits = 2, scientific = TRUE))
		tbl.melt[["pvaltext"]] <- ifelse(tbl.melt[["significant"]] == "TRUE", tbl.melt[["pvaltext"]], "")
		tbl.melt[["test"]] <- tbl.melt[["significant"]]
	}
	tbl.melt[[1]] <- factor(as.character(tbl.melt[[1]]), levels = rev(rownames(tbl)))
	tbl.melt[[2]] <- factor(as.character(tbl.melt[[2]]), levels = colnames(tbl))

	if (is.matrix(tbl.failures) && (!all(is.na(tbl.failures)))) {
		## Create a data frame with test failures
		tbl <- tbl.failures
		tbl[upper.tri(tbl)] <- NA
		tbl <- tbl[-1, -ncol(tbl), drop = FALSE]
		tbl.f.melt <- reshape2::melt(tbl, varnames = c("x", "y"))
		colnames(tbl.f.melt)[3] <- "failure"
		tbl.f.melt[[1]] <- factor(as.character(tbl.f.melt[[1]]), levels = levels(tbl.melt[[1]]))
		tbl.f.melt[[2]] <- factor(as.character(tbl.f.melt[[2]]), levels = levels(tbl.melt[[2]]))
		tbl.f.melt[[3]] <- factor(as.character(tbl.f.melt[[3]]), levels = names(SHAPES.FAILURES))
		tbl.f.melt <- tbl.f.melt[!is.na(tbl.f.melt[[3]]), ]
	}

	## Create the heatmap
	height <- 2.2 + nrow(tbl) * 0.25
	width <- 4 + nrow(tbl) * 0.62
	pp <- ggplot(tbl.melt, aes_string("y", "x", label = "pvaltext")) + labs(x = NULL, y = NULL) +
		coord_fixed(ratio = 0.25 / 0.62) + geom_tile(aes_string(fill = "test"), color = "white") +
		scale_x_discrete(expand = c(0, 0)) + scale_y_discrete(expand = c(0, 0)) +
		scale_fill_manual(na.value = "white", values = col.mapping)
	if (exists("tbl.f.melt")) {
		pp <- pp + geom_point(aes_string(shape = "failure", label = NULL), data = tbl.f.melt, color = "#808080",
				fill = "#808080") + scale_shape_manual(values = SHAPES.FAILURES)
	}
	if (do.tests) {
		pp <- pp + theme(legend.justification = c(0, 1), legend.position = c(1, 1))
	} else {
		pp <- pp + geom_text(size = 3) + theme(legend.position = "none")
	}
	pp <- pp + theme(axis.ticks = element_blank(), axis.text.x = element_text(angle = 90, hjust = 1)) +
		theme(panel.grid.major = element_blank(), panel.background = element_blank()) +
		theme(panel.border = element_blank(), plot.margin = unit(c(0.1, 1.9, 0.1, 0.1), "in"))
	## Fix the areas for x and y axis labels
	pp <- suppressWarnings(ggplot_gtable(ggplot_build(pp)))
	i <- which(pp$layout$name == "axis-b")
	pp$layout[i, "clip"] <- "on"
	pp$heights[pp$layout[i, "t"]] <- unit(2, "in")
	i <- which(pp$layout$name == "axis-l")
	pp$layout[i, "clip"] <- "on"
	pp$width[pp$layout[i, "l"]] <- unit(2, "in")
	rplot <- createReportPlot(fname, report, width = width, height = height)
	grid.newpage()
	grid.draw(pp)
	return(off(rplot))
}

########################################################################################################################
########################################################################################################################

#' rnb.execute.dreduction
#'
#' Performs principal component analysis (PCA) and multi-dimensional scaling (MDS) of the samples in the given
#' methylation dataset.
#'
#' @param rnb.set Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}. This dataset
#'                must contain at least four samples.
#' @param target  \code{character} singleton specifying the level of DNA methylation infromation. If this is
#' 				  \code{"sites"}, the DNA methylation information for the individual sites or probes is analyzed.
#' 				  Otherwise, this should be one of the supported region types, as returned by
#'                \code{\link{rnb.region.types}}.
#' @return Results of the dimension reduction in the form of a list with the following elements:
#'         \describe{
#'           \item{\code{pca}}{Results of the PCA as returned by the function \code{\link{prcomp}}.}
#'           \item{\code{mds}}{List of two elements - \code{"manhattan"} and \code{"euclidean"}, each of which is a
#'                two-column \code{matrix} storing the coordinates of the samples in a two-dimensional space. The
#'                matrices are computed using the function \code{\link{isoMDS}}.}
#'         }
#'
#' @details
#' Row names in the returned matrices are sample identifiers, determined based on the package option
#' \code{"identifiers.column"}. See \emph{\link[=rnb.options]{RnBeads Options}} for more information on this option.
#'
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' regs <- c("sites", summarized.regions(rnb.set.example))
#' dreduction <- function(x) rnb.execute.dreduction(rnb.set.example, x)
#' pcoordinates <- lapply(regs, dreduction)
#' names(pcoordinates) <- regs
#' str(pcoordinates)
#' }
#' @seealso \code{\link{rnb.run.exploratory}} for running the whole exploratory analysis module
#' @author Yassen Assenov
#' @export
rnb.execute.dreduction <- function(rnb.set, target = "sites") {
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	if (!(is.character(target) && length(target) == 1 && (!is.na(target)))) {
		stop("invalid value for target")
	}
	X <- t(meth(rnb.set, target))
	if (nrow(X) < 4) {
		stop("too few samples")
	}
	rownames(X) <- samples(rnb.set)

	pcoordinates <- list(mds = list(), pca = NULL)
	top.sites <- rnb.getOption("exploratory.top.dimensions")
	site.nas <- colMeans(is.na(X))
	i <- which(site.nas == 1)
	if (length(i) != 0) {
		if (ncol(X) - length(i) < 4) {
			stop("too many missing values")
		}
		## Remove sites that contain NAs only
		X <- X[, -i]
		site.nas <- site.nas[-i]
	}
	dframe.selected <- data.frame( # table with statistics on used sites
		target = rep(target, 2),
		technique = c("MDS", "PCA"),
		all = rep(ncol(X), 2),
		na = rep(length(i), 2),
		selected = rep(ncol(X) - length(i), 2),
		explained = c(1, 1), row.names = c("mds", "pca"), stringsAsFactors = FALSE)
	site.nas <- which(site.nas != 0)
	if (top.sites != 0) {
		select.top <- function(X, dr.method) {
			site.vars <- colVars(X, na.rm = TRUE)
			i.sites <- order(site.vars, decreasing = TRUE)[1:top.sites]
			dframe.selected[dr.method, "selected"] <<- top.sites
			dframe.selected[dr.method, "explained"] <<- sum(site.vars[i.sites]) / sum(site.vars, na.rm=TRUE)
			rnb.status(c("Selected the", top.sites, "dimensions with highest variance"))
			X[, i.sites, drop = FALSE]
		}
	}

	## Run MDS
	rnb.logger.start("MDS")
	if (0 != top.sites && top.sites < ncol(X)) {
		X.mds <- select.top(X, "mds")
		if (length(site.nas) == 0) {
			X <- X.mds
			dframe.selected["pca", "selected"] <- dframe.selected["mds", "selected"]
			i <- gc()
		}
	} else {
		X.mds <- X
	}
	for (dist.metric in c("manhattan", "euclidean")) {
		pcoordinates$mds[[dist.metric]] <- tryCatch(mds(X.mds, dist.metric), error = function(er) { NULL })
		rnb.status(c("Calculated MDS coordinates using", dist.metric, "distance"))
	}
	rnb.logger.completed()
	rm(X.mds)

	## Run PCA
	rnb.logger.start("PCA")
	if (length(site.nas) != 0) {
		## Remove sites that contain NA in any sample
		dframe.selected["pca", "na"] <- dframe.selected["pca", "na"] + length(site.nas)
		dframe.selected["pca", "selected"] <- dframe.selected["pca", "selected"] - length(site.nas)
		if (length(site.nas) >= ncol(X) - nrow(X)) {
			attr(pcoordinates, "selected") <- dframe.selected
			rnb.info("Skipped due to too many missing values")
			rnb.logger.completed()
			return(pcoordinates)
		}
		X <- X[, -site.nas]
		rnb.info(c("Removed", length(site.nas), "loci (", target, ") because they contain missing values"))
		if (0 != top.sites && top.sites < ncol(X)) {
			X <- select.top(X, "pca")
		}
	}
	suppressWarnings(rm(site.nas, top.sites, i, select.top))
	pcoordinates$pca <- prcomp(X, center = TRUE, scale. = FALSE)
	pcoordinates$pca$rotation <- NULL # remove rotation matrix to reduce memory load
	attr(pcoordinates, "selected") <- dframe.selected
	rnb.logger.completed()
	return(pcoordinates)
}

########################################################################################################################

#' rnb.section.dreduction
#'
#' Creates a report section dedicated to dimension reduction.
#'
#' @param report            Report to contain the section. This must be an object of type \code{\linkS4class{Report}}.
#' @param pcoordinates      Coordinates of data points, as an object returned by \code{\link{rnb.execute.dreduction}},
#'                          or a list of such objects.
#' @param sample.phenotypes Table of sample phenotype information in the form of a \code{data.frame}. The number of rows
#'                          in this table must be consistent with the samples in \code{pcoordinates}. This parameter is
#'                          used in determining point colors and/or shapes when plotting the samples in the reduced
#'                          dimensional representations. Set this parameter to \code{NULL} if no phenotype data is to be
#'                          visualized.
#' @return The modified report.
#'
#' @seealso \code{\link{rnb.step.dreduction}}
#' @author Yassen Assenov
#' @noRd
rnb.section.dreduction <- function(report, pcoordinates, sample.phenotypes = NULL) {
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	nsamples <- validate.pcoordinates.all(pcoordinates)

	if (is.character(nsamples)) {
		stop(nsamples)
	}
	if (!is.null(sample.phenotypes)) {
		if (!is.data.frame(sample.phenotypes)) {
			stop("invalid value for sample.phenotypes")
		}
		if (nrow(sample.phenotypes) != nsamples) {
			stop("invalid value for sample.phenotypes; unexpected number of samples")
		}
	}
	if (rnb.getOption("logging") && logger.isinitialized() == FALSE) {
		logger.start(fname = NA) # initialize console logger
	}
	if (setequal(names(pcoordinates), c("mds", "pca"))) {
		pcoordinates <- list("unknown" = pcoordinates)
	}
	rnb.section.dreduction.internal(report, pcoordinates, sample.phenotypes, "Low-dimensional Representation")
}

rnb.section.dreduction.internal <- function(report, pcoordinates, sample.phenotypes, section.title) {

	targets <- names(pcoordinates)
	names(targets) <- 1:length(targets)

	## Define point colors and types based on the phenotypic information
	col.visual <- names(rnb.sample.groups(sample.phenotypes, rnb.getOption("exploratory.columns")))
	use.colors <- (length(col.visual) != 0)
	logger.info(c("Mapping", length(col.visual), "traits to point colors and types"))

	setting.names <- list("Location type" = targets,
		"Principal components" = c("1" = "first and second", "2" = "second and third"),
		"Distance" = c("manhattan" = "Manhattan", "euclidean" = "Euclidean"))
	snames <- c("identifiers", "circles")
	if (use.colors) {
		snames <- c(snames, paste("based on", col.visual))
	}
	names(snames) <- 1:length(snames)
	setting.names[["Sample representation"]] <- snames
	if (use.colors) {
		snames <- c("black", paste("based on", col.visual))
		names(snames) <- 0:length(col.visual)
		setting.names[["Sample color"]] <- snames
	}
	rm(snames)

	create.scatters <- function(dpoints, fprefix) {
		report.plots <- list()
		dframe <- data.frame(x = dpoints[, 1], y = dpoints[, 2], id = rownames(dpoints))
		for (ptype in 1:(length(col.visual) + 2)) {
			plot.labs <- list("x" = colnames(dpoints)[1], "y" = colnames(dpoints)[2])
			if (ptype > 2) {
				sample.categories <- length(unique(sample.phenotypes[, col.visual[ptype - 2]]))
				dframe[["pvalues"]] <- as.factor(sample.phenotypes[, col.visual[ptype - 2]])
				plot.labs[["shape"]] <- col.visual[ptype - 2]
			}
			for (i in 0:length(col.visual)) {
				fname <- paste(fprefix, ptype, sep = "_")
				if (use.colors) {
					fname <- paste(fname, i, sep = "_")
				}
				rplot <- createReportPlot(fname, report, width = 5.4 + 2.6 * use.colors, height = 5.4)
				if (i != 0) {
					sample.categories <- length(unique(sample.phenotypes[, col.visual[i]]))
					dframe[["color"]] <- as.factor(sample.phenotypes[, col.visual[i]])
					cvalues <- rep(rnb.getOption("colors.category"), length.out = sample.categories)
					plot.labs[["color"]] <- col.visual[i]
				}
				if (i != 0) {
					pp <- ggplot(dframe, aes_string(x = "x", y = "y", color = "color", label = "id")) +
							scale_colour_manual(na.value = "#C0C0C0", values = cvalues)
				} else {
					pp <- ggplot(dframe, aes_string(x = "x", y = "y", label = "id"))
				}
				pp <- pp + do.call(labs, plot.labs)
				if (ptype == 1) { # plot sample IDs
					pp <- pp + geom_text(size = 2)
				} else if (ptype == 2) { # all samples are circles
					pp <- pp + geom_point()
				} else {
					ptvalues <- rep(rnb.getOption("points.category"), length.out = nlevels(dframe[, "pvalues"]))
					if (ptype - 2 == i) {
						pp <- pp + geom_point(aes_string(shape = "color"))
					} else {
						pp <- pp + geom_point(aes_string(shape = "pvalues"))
					}
					pp <- pp + scale_shape_manual(na.value = 1L, values = ptvalues)
				}
				if (use.colors) {
					pp <- pp + theme(plot.margin = unit(0.1 + c(0, 2.6, 0, 0), "in"),
							legend.justification = c(0, 0.5), legend.position = c(1, 0.5))
				}
				suppressWarnings(print(pp)) # "Removed ... rows containing missing values (geom_point)"
				report.plots <- c(report.plots, off(rplot))
			}
		}
		return(report.plots)
	}

	## Start the section on dimension reduction
	stext <- c("Dimension reduction is used to visually inspect the dataset for a strong signal in the methylation ",
		"values that is related to samples\' clinical or batch processing annotation. RnBeads implements two methods ",
		"for dimension reduction - principal component analysis (PCA) and multidimensional scaling (MDS).")
	if (is.null(section.title)) {
		rnb.add.paragraph(report, stext)
	} else {
		report <- rnb.add.section(report, "Low-dimensional Representation", stext)
	}

	## Mention that only some sites and/or regions are selected
	if (!is.null(attr(pcoordinates, "selected"))) {
		selected <- attr(pcoordinates, "selected")
#		save(selected, file = "selected.RData", compression_level = 9L)
		top.selected <- isTRUE(any(selected$explained != 1))
		if (top.selected || isTRUE(any(selected$na != 0))) {
			if (top.selected) {
				stext <- c("The analyses in the following sections are based on selected sites and/or regions with ",
					"highest variability in methylation across all samples. The following table shows the maximum ",
					"dimensionality and the selected dimensions in each setting (column names <i>Dimensions</i> ",
					"and <i>Selected</i>, respectively).")
			} else {
				stext <- c(ifelse(length(pcoordinates) == 1, "The methylation matrix",
						"One or more of the methylation matrices"), " was augmented before applying the dimension ",
					"reduction techniques because it contains missing values.")
			}
			selected <- data.frame(
				"Sites/regions" = selected$target,
				"Technique" = selected$technique,
				"Dimensions" = selected$all,
				"Missing" = selected$na,
				"Selected" = selected$selected,
				"Variance explained" = sprintf("%.1f", selected$explained * 100),
				check.names = FALSE, stringsAsFactors = FALSE)
			if (!top.selected) {
				selected <- selected[, setdiff(colnames(selected), "Variance explained")]
			}
			if (isTRUE(any(selected[, "Missing"] != 0))) {
				stext <- c(stext, " The column <i>Missing</i> lists the number of dimensions ignored due to ",
					"missing values. In the case of MDS, dimensions are ignored only if they contain missing ",
					"values for all samples. In contrast, sites or regions with missing values in any sample ",
					"are ignored prior to PCA.")
			} else {
				selected <- selected[, setdiff(colnames(selected), "Missing")]
			}
			rnb.add.paragraph(report, stext)
			rnb.add.table(report, selected, row.names = FALSE)
		}
		rm(top.selected)
	}

	## Create scatter plots of all samples in the reduced representation by MDS
	stext <- c("The scatter plot below visualizes the samples transformed into a two-dimensional space using MDS.")
	report <- rnb.add.section(report, "Multidimensional Scaling", stext, level = 2)
	report.plots <- list()

	for (target.id in 1:length(targets)) {
		for (dist.metric in names(pcoordinates[[target.id]][["mds"]])) {
			dpoints <- pcoordinates[[target.id]][["mds"]][[dist.metric]]
			colnames(dpoints) <- paste("MDS dimension", 1:ncol(dpoints))
			fprefix <- paste("scatter", "mds", target.id, dist.metric, sep = "_")
			report.plots <- c(report.plots, create.scatters(dpoints, fprefix))
		}
	}
	stext <- "Scatter plot showing samples after performing Kruskal\'s non-metric mutidimensional scaling."
	sn.indices <- match("Principal components", names(setting.names))
	tryCatch(
		report <- rnb.add.figure(report, stext, report.plots, setting.names[-sn.indices],
			selected.image = 2 + length(col.visual)),
		error = function(err) {
			print(setting.names[-sn.indices])
			print(sapply(report.plots, slot, 'fname'))
		}
	)

	## Create scatter plots of all samples in the first few principle components
	max.component <- 3L
	report <- rnb.add.section(report, "Principal Component Analysis", NULL, level = 2)
	txt <- "Similarly, the figure below shows the values of selected principal components in a scatter plot."
	report.plots <- list()
	for (target.id in 1:length(targets)) {
		dpoints <- pcoordinates[[target.id]][["pca"]]
		if (length(dpoints) != 0) {
			dpoints <- dpoints$x[, 1:max.component]
			colnames(dpoints) <- paste("Principal component", 1:ncol(dpoints))
			for (k in 1:(max.component - 1)) {
				fprefix <- paste("scatter", "pca", target.id, k, sep = "_")
				rplot <- create.scatters(dpoints[, c(k, k + 1)], fprefix)
				report.plots <- c(report.plots, rplot)
			}
		} else if (length(txt) == 1) {
			txt <- c(txt, " Note that PCA plots are not available for all site and region types; in some cases PCA ",
					"was not performed due to too many missing values.")
		}
	}
	if (length(report.plots) == 0) {
		txt <- "PCA was skipped in all studied site and/or region types due to too many missing values."
		rnb.add.paragraph(report, txt)
		return(report)
	}
	rnb.add.paragraph(report, txt)

	txt <- "Scatter plot showing the samples\' coordinates on principal components."
	sn.indices <- match("Distance", names(setting.names))
	report <- rnb.add.figure(report, txt, report.plots, setting.names[-sn.indices])
	suppressWarnings(rm(max.component, txt, report.plots, target.id, dpoints, k, fprefix, rplot, sn.indices))

	## Calculate variance explained by the top principle components
	var.tbl <- NULL
	rplots <- list()
	for (target.id in 1:length(targets)) {
		var.fraction <- pcoordinates[[target.id]][["pca"]]$sdev^2
		if (length(var.fraction) == 0) {
			next
		}
		var.fraction <- 100 * var.fraction / sum(var.fraction)
		var.explained <- cumsum(var.fraction)
		max.comp <- min(which(var.explained >= VAR.EXPLAINED))
		txt <- c("Principal components that explain at least", VAR.EXPLAINED, "% of the total variance:", max.comp)
		logger.info(txt)
		var.percentage <- data.frame(
			"Principal Component" = 1:length(var.fraction),
			"Percentage of Total Variance" = var.fraction,
			"Cumulative Variance Explained" = var.explained,
			check.names = FALSE, stringsAsFactors = FALSE)

		fname <- sprintf("pca_variance_explained_%s.csv", target.id)
		fname.full <- file.path(rnb.get.directory(report, "data", absolute = TRUE), fname)
		utils::write.csv(var.percentage, file = fname.full, row.names = FALSE)
		logger.info(c("Saved percentage of total variance to", fname))

		fname.full <- paste0('<a href="', rnb.get.directory(report, "data"), '/', fname, '">csv</a>')
		var.tbl <- rbind(var.tbl, c(targets[target.id], as.character(max.comp), fname.full))

		if (max.comp == 1) {
			rplot <- list(NULL)
		} else { # max.comp > 1

			## Plot a CDF of variance explained
			dframe <- data.frame(x = 1:max.comp, y = var.explained[1:max.comp])
			ylab <- paste0(ifelse(is.null(selected), "Total v", "V"), "ariance explained (%)")
			fname <- paste0("cdf_pca_variability_regions_", target.id)
			rplot <- createReportPlot(fname, report, width = 6.2, height = 6.2)
			pp <- ggplot(dframe, aes_string(x = "x", y = "y")) + labs(x = "Number of components", y = ylab) +
				geom_line(color = "blue") + geom_point(color = "blue") +
				theme(plot.margin = unit(0.1 + c(0, 0, 0, 0), "in"))
			print(pp)
			rplot <- off(rplot)
			rm(dframe, ylab, pp)
		}
		rm(var.explained, max.comp, txt, var.percentage, fname, fname.full)
		rplots <- c(rplots, rplot)
		rm(rplot)
	}
	rm(target.id, var.fraction)

	## Create a figure of variance explained
	i.targets <- which(!sapply(rplots, is.null))
	if (length(i.targets) != 0) {
		stext <- c("The figure below shows the cumulative distribution functions of variance explained by the ",
			"principal components.")
		rnb.add.paragraph(report, stext)
		stext <- ifelse(is.null(selected), "total ", "")
		stext <- c("Cumulative distribution function of percentange of ", stext, "variance explained.")
		tryCatch(
			report <- rnb.add.figure(report, stext, rplots[i.targets], setting.names[1L]),
			error = function(err) {
				print(setting.names[1])
				print(sapply(rplots, slot, 'fname'))
			}
		)
	}

	## Add links to the tables of principal component variances
	if (!is.null(var.tbl)) {
		if (nrow(var.tbl) == 1) {
			txt <- c("table", "is", "a ", "file")
		} else { # nrow(var.tbl) > 1
			txt <- c("tables", "are", "", "files")
		}
		txt <- c("The table below gives for each location type a number of principal components that explain at least ",
			VAR.EXPLAINED, " percent of the total variance. The full ", txt[1], " of variances explained by all ",
			"components ", txt[2], " available in ", txt[3], "comma-separated values ", txt[4], " accompanying this ",
			"report.")
		rnb.add.paragraph(report, txt)

		colnames(var.tbl) <- c("Location Type", "Number of Components", "Full Table File")
		rnb.add.table(report, var.tbl, row.names = FALSE)
	}

	return(report)
}

########################################################################################################################

#' rnb.step.dreduction
#'
#' Executes the dimension reduction step and adds a dedicated section to the given report.
#'
#' @param rnb.set            Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param report             Report to contain the dimension reduction section. This must be an object of type
#'                           \code{\linkS4class{Report}}.
#' @param return.coordinates Flag indicating if the computed sample coordinates in the prinicipal component space should
#'                           also be returned.
#' @return If \code{return.coordinates} is \code{FALSE}: the modified report. Otherwise, a list of two elements:
#'         \describe{
#'           \item{report}{The modified report.}
#'           \item{pcoordinates}{Coordinates of the samples in the dataset, as returned by
#'                \code{\link{rnb.execute.dreduction}}.}
#'         }
#'
#' @seealso \code{\link{rnb.execute.dreduction}}, \code{\link{rnb.section.dreduction}}
#' @author Yassen Assenov
#' @noRd
rnb.step.dreduction <- function(rnb.set, report, return.coordinates = FALSE) {
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnbSet")
	}
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	if (!parameter.is.flag(return.coordinates)) {
		stop("invalid value for return.coordinates; expected TRUE or FALSE")
	}
	if (rnb.getOption("logging") && logger.isinitialized() == FALSE) {
		logger.start(fname = NA) # initialize console logger
	}

	## Extract list of site and region types to analyze
	section.title <- "Low-dimensional Representation"
	targets <- rnb.step.analyze.targets(rnb.set, report, section.title)
	if (length(targets) == 0) {
		if (return.coordinates) {
			return(list(report = report, pcoordinates = list()))
		}
		return(report)
	}
	if (!is.null(attr(targets, "warning"))) {
		txt <- attr(targets, "warning")
		report <- rnb.add.section(report, section.title, txt)
		section.title <- NULL
	}

	## Map the samples to a low-dimensional space
	logger.start("Dimension Reduction Techniques")
	names(targets) <- targets
	if (parallel.isEnabled()) {
		pcoordinates <- foreach(target = targets) %dopar%
			tryCatch(RnBeads::rnb.execute.dreduction(rnb.set, target = target), error = function(e) { e$message } )
		names(pcoordinates) <- names(targets)
	} else {
		pcoordinates <- lapply(targets, function(target) {
			tryCatch(rnb.execute.dreduction(rnb.set, target = target), error = function(e) { e$message } )
		})
	}

	selected <- lapply(pcoordinates, attr, which = "selected")
	selected <- selected[!sapply(selected, is.null)]
	if (length(selected) != 0) {
		attr(pcoordinates, "selected") <- do.call(rbind, selected)
		for (i in 1:length(pcoordinates)) {
			attr(pcoordinates[[i]], "selected") <- NULL
		}
		rm(i)
	}
	rm(selected)

	i.failed <- which(sapply(pcoordinates, is.character))
	if (length(i.failed) != 0) {
		if (length(i.failed) == length(pcoordinates)) {
			if (identical(pcoordinates[[1]], "too few samples")) {
				txt <- "Dimension reduction techniques were not applied because the dataset contains too few samples."
				logger.warning("Skipped due to too few samples")
			} else { # too many sites with missing values
				txt <- c("Dimension reduction techniques were not applied.", "")
				logger.warning("Skipped due to too many missing values")
			}
		} else {
			txt <- paste0("<b>", names(pcoordinates)[i.failed], "</b>", collapse = ", ")
			txt <- c("Dimension reduction techniques were not applied for the following types: ", txt, ".")
		}
		if (length(txt) != 1) {
			txt <- c(txt, " The most likely reason for this error is inability to calculate a distance between a pair ",
				"of samples due to (too many) missing values in the respective methylation ",
				ifelse(length(i.failed) == 1, "matrix", "matrices"), ".")
		}
		if (is.null(section.title)) {
			rnb.add.paragraph(report, txt)
		} else {
			report <- rnb.add.section(report, section.title, txt)
			section.title <- NULL
		}
		pcoordinates <- pcoordinates[setdiff(1:length(pcoordinates), i.failed)]
	}

	if (length(pcoordinates) != 0) {
		## Create plots of PCA and MDS
		report <- rnb.section.dreduction.internal(report, pcoordinates, pheno(rnb.set), section.title)
		logger.status("Created scatter plots and CDFs summarizing the reduced dimensional representations")
	}
	logger.completed()
	if (return.coordinates) {
		return(list(report = report, pcoordinates = pcoordinates))
	}
	return(report)

}

########################################################################################################################

#' rnb.execute.batcheffects
#'
#' Performs tests for association between traits and principal components.
#'
#' @param rnb.set      Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param pcoordinates Coordinates of the samples of \code{rnb.set} in the principal components space, as returned by
#'                     \code{\link{rnb.execute.dreduction}}.
#' @return Results of attempted tests for associations in the form of a list with up to three elements:
#'         \describe{
#'           \item{\code{"permutations"}}{\code{integer} matrix of index permutations. The number of rows in the matrix
#'                is \emph{N} - the number of samples in \code{rnb.set}. Every column in this matrix denotes a sample
#'                permutation; the first column is the sequence 1 to \emph{N}. This element is included only when
#'                \code{rnb.getOption("exploratory.correlation.permutations")} is non-zero and there are numeric traits
#'                to be tested.}
#'           \item{\code{"pc"}}{List of four matrices named \code{"failures"}, \code{"tests"}, \code{"correlations"}
#'                and \code{"pvalues"}. The rows in each of these matrices correspond to the first several principal
#'                components, and the columns - to selected traits. This element is not included in the returned list
#'                when \code{pcoordinates} is \code{NULL}.}
#'           \item{\code{"traits"}}{List of four square symmetric matrices named \code{"failures"}, \code{"tests"},
#'                \code{"correlations"} and \code{"pvalues"}, containing information about the performed tests for
#'                pairwise trait association. This element is included only if two or more traits were tested.}
#'         }
#'
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' regs <- c("sites", summarized.regions(rnb.set.example))
#' dreduction <- function(x) rnb.execute.dreduction(rnb.set.example, x)
#' pcoordinates <- lapply(regs, dreduction)
#' names(pcoordinates) <- regs
#' result <- rnb.execute.batcheffects(rnb.set.example, pcoordinates)
#' }
#' @seealso \code{\link{rnb.run.exploratory}} for running the whole exploratory analysis module
#' @author Yassen Assenov
#' @export
rnb.execute.batcheffects <- function(rnb.set, pcoordinates = NULL) {
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	pheno.table <- pheno(rnb.set)
	if (is.null(pheno.table)) {
		stop("invalid value for rnb.set; missing phenotype data")
	}
	if (!is.null(pcoordinates)) {
		nsamples <- validate.pcoordinates.all(pcoordinates)
		if (is.character(nsamples)) {
			stop(nsamples)
		}
	}
	if (rnb.getOption("logging") && logger.isinitialized() == FALSE) {
		logger.start(fname = NA) # initialize console logger
	}
	logger.start("Tests for Associations")

	## Get a list of comparable traits
	predefined.columns <- rnb.getOption("exploratory.columns")
	if (!is.null(predefined.columns)) {
		if (is.character(predefined.columns)) {
			predefined.columns <- intersect(predefined.columns, colnames(pheno.table))
		} else { # is.integer(columns)
			predefined.columns <- intersect(predefined.columns, 1:ncol(pheno.table))
		}
		pheno.table <- pheno.table[, predefined.columns]
	}
	traits <- lapply(pheno.table, function(x) {
				if (length(unique(na.omit(x))) < 2) {
					return(NULL)
				}
				if (is.character(x)) {
					x <- as.factor(x)
				}
				if (is.logical(x)) {
					x <- as.factor(x)
				}
				if (is.factor(x)) {
					y <- na.omit(x)
					if (anyDuplicated(y) == 0) {
						return(NULL)
					}
				}
				return(x)
			}
	)
	traits <- traits[sapply(traits, is.null) == FALSE]
	NT <- length(traits)
	if (NT == 0) {
		logger.info("No suitable traits found")
		logger.completed()
		return(NULL)
	}
	logger.info(c("Testing the following traits for associations:", paste(names(traits), collapse = "; ")))

	result <- list()

	## Create sample permutations if necessary
	perm.matrix <- NULL
	perm.count <- rnb.getOption("exploratory.correlation.permutations")
	if (perm.count != 0 && ((!is.null(pcoordinates)) || sum(!sapply(traits, is.factor)) >= 2)) {
		perm.matrix <- mapply(sample, rep(nrow(pheno.table), times = perm.count))
		perm.matrix[, 1] <- 1:nrow(perm.matrix)
		result[["permutations"]] <- perm.matrix
		logger.status(c("Created", ncol(perm.matrix), "sample permutations"))
	}

	pc.association.count <- rnb.getOption("exploratory.principal.components")
	if ((!is.null(pcoordinates)) && pc.association.count != 0) {
		## Compute correlations between principle components and traits
		result[["pc"]] <- list()
		for (target in names(pcoordinates)) {
			dpoints <- pcoordinates[[target]]$pca$x
			if(!is.null(dpoints)){
				if (ncol(dpoints) > pc.association.count) {
					dpoints <- dpoints[, 1:pc.association.count]
				}

				init.matrix <- function(x) {
					matrix(x, nrow = NT, ncol = ncol(dpoints),
							dimnames = list(names(traits), "Principal component" = 1:ncol(dpoints)))
				}
				table.test.failures <- init.matrix(as.character(NA))
				table.test.names <- init.matrix(as.character(NA))
				table.correlations <- init.matrix(as.double(NA))
				table.pvalues <- init.matrix(as.double(NA))

				for (i in 1:NT) {
					for (j in 1:ncol(dpoints)) {
						t.result <- test.traits(traits[[i]], dpoints[, j], perm.matrix)
						table.test.failures[i, j] <- t.result[["error"]]
						table.test.names[i, j] <- t.result[["test"]]
						table.correlations[i, j] <- t.result[["correlation"]]
						table.pvalues[i, j] <- t.result[["pvalue"]]
					}
				}
				result[["pc"]][[target]] <- list(
						"failures" = table.test.failures,
						"tests" = table.test.names,
						"correlations" = table.correlations,
						"pvalues" = table.pvalues)
			}
		}
		rm(dpoints, init.matrix, table.test.failures, table.test.names, table.correlations, table.pvalues)
		rm(i, j, t.result)
		logger.status("Computed correlations between principal components and traits.")
	}

	## Compute correlations and perform tests
	if (NT > 1) {
		init.matrix <- function(x) {
			matrix(x, nrow = NT, ncol = NT, dimnames = list(names(traits), names(traits)))
		}
		table.test.failures <- init.matrix(as.character(NA))
		table.test.names <- init.matrix(as.character(NA))
		table.correlations <- init.matrix(as.double(NA))
		table.pvalues <- init.matrix(as.double(NA))
		for (i in 1:(NT - 1)) {
			for (j in (i + 1):NT) {
				t.result <- test.traits(traits[[i]], traits[[j]], perm.matrix)
				table.test.failures[i, j] <- table.test.failures[j, i] <- t.result[["error"]]
				table.test.names[i, j] <- table.test.names[j, i] <- t.result[["test"]]
				table.correlations[i, j] <- table.correlations[j, i] <- t.result[["correlation"]]
				table.pvalues[i, j] <- table.pvalues[j, i] <- t.result[["pvalue"]]
			}
		}
		rm(perm.matrix, NT, i, j, t.result)
		result[["traits"]] <- list(
				"failures" = table.test.failures,
				"tests" = table.test.names,
				"correlations" = table.correlations,
				"pvalues" = table.pvalues)
		logger.status("Computed pairwise correlations between traits.")
	}
	logger.completed()
	return(result)
}

########################################################################################################################

#' rnb.section.batcheffects
#'
#' Creates a report section summarizing associations between traits and principal components, as well as trait pairwise
#' associations.
#'
#' @param report       Report to contain the section. This must be an object of type \code{\linkS4class{Report}}.
#' @param batcheffects Results of attempted tests for associations, as returned by
#'                     \code{\link{rnb.execute.batcheffects}}.
#' @return The modified report.
#'
#' @seealso \code{\link{rnb.step.batcheffects}}, \code{\link{rnb.run.exploratory}}
#' @author Yassen Assenov
#' @noRd
rnb.section.batcheffects <- function(report, batcheffects) {
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	if (!is.null(batcheffects)) {
		if (!(is.list(batcheffects) && all(names(batcheffects) %in% c("permutations", "pc", "traits")))) {
			stop("invalid value for batcheffects")
		}
		if ("permutations" %in% names(batcheffects)) {
			be <- batcheffects[names(batcheffects) != "permutations"]
			if (!is.valid.permutations(batcheffects$permutations)) {
				stop("invalid value for batcheffects$permutations")
			}
		} else {
			be <- batcheffects
		}
		check.batcheffect.matrices <- function(x) {
			is.list(x) && identical(names(x), c("failures", "tests", "correlations", "pvalues")) &&
				all(sapply(x, is.matrix)) && all(sapply(x[1:2], is.character)) && all(sapply(x[3:4], is.numeric))
		}
		if (!all(sapply(be[names(be) != "pc"], check.batcheffect.matrices))) {
			stop("invalid value for batcheffects")
		}
		if (!all(sapply(be[["pc"]], check.batcheffect.matrices))) {
			stop("invalid value for batcheffects")
		}
		rm(be, check.batcheffect.matrices)
	} else {
		txt <- c("Batch effects were not studied because none of the traits can be tested for association with sample ",
			"coordinates in the principal components space.")
		report <- rnb.add.section(report, "Batch Effects", txt)
		return(report)
	}
	if (rnb.getOption("logging") && logger.isinitialized() == FALSE) {
		logger.start(fname = NA) # initialize console logger
	}

	txt <- c("In this section, different properties of the dataset are tested for significant associations. The ",
			"properties can include sample coordinates in the principal component space, phenotype traits and ",
			"intensities of control probes. The tests used to calculate a p-value given two properties depend on the ",
			"essence of the data:")
	report <- rnb.add.section(report, "Batch Effects", txt)
	nperm <- rnb.getOption("exploratory.correlation.permutations")
	txt <- list(
		c("If both properties contain categorical data (e.g. tissue type and sample processing date), the test of ",
				"choice is a two-sided Fisher's exact test."),
		c("If both properties contain numerical data (e.g. coordinates in the first principal component and age of ",
				"individual), the correlation coefficient between the traits is computed. ",
				ifelse(nperm > 0, paste("A p-value is estimated using permutation tests with", nperm, "permutations."),
						"No p-value is estimated in this case because permutation tests are disabled.")),
		c("If property <i>A</i> is categorical and property <i>B</i> contains numeric data, p-value for association ",
				"is calculated by comparing the values of <i>B</i> for the different categories in <i>A</i>. The test ",
				"of choice is a two-sided Wilcoxon rank sum test (when <i>A</i> defines two categories) or ",
				"a Kruskal-Wallis one-way analysis of variance (when <i>A</i> separates the samples into three or more ",
				"categories)."))
	rnb.add.list(report, txt)
	txt <- "Note that the p-values presented in this report are <em>not corrected</em> for multiple testing."
	rnb.add.paragraph(report, txt)
	data.dir.abs <- rnb.get.directory(report, "data", TRUE)
	data.dir.rel <- rnb.get.directory(report, "data")

	## -----------------------------------------------------------------------------------------------------------------
	## Summarize the results of association between principal components and traits

	if ("pc" %in% names(batcheffects) && length(batcheffects[["pc"]])>0) {

		targets <- names(batcheffects[["pc"]])
		names(targets) <- 1:length(targets)
		setting.names <- list("Region type" = targets)

		txt <- ifelse(is.null(rnb.getOption("exploratory.columns")), "available", "specified")
		txt <- c("The computed sample coordinates in the principal component space were tested for association with ",
			"the ", txt, " traits. Below is a list of the traits and the tests performed.")
		report <- rnb.add.section(report, "Associations between Principal Components and Traits", txt, level = 2)

		## Display summary table of tests performed
		pc.tables <- batcheffects$pc[[1]]
		tbl <- matrix(c(rownames(pc.tables$tests), pc.tables$tests[, 1]), ncol = 2)
		colnames(tbl) <- c("Trait", "Test")
		rnb.add.table(report, tbl)

		append.table <- function(tbl, fnames, heatmap.fun) {
			mytbl <- cbind("Trait \\ Principal Component" = rownames(tbl), as.data.frame(tbl, check.names = FALSE))
			fnames.full <- file.path(data.dir.abs, fnames)
			fnames[1] <- paste(data.dir.rel, fnames[1], sep = "/")
			utils::write.csv(mytbl, file = fnames.full[1], na = "", row.names = FALSE)
			sprintf("<a href=\"%s\">csv</a>", fnames[1])
			hmap <- heatmap.fun(report, tbl, fnames[2])
			list(link = sprintf("<a href=\"%s\">csv</a>", fnames[1]), plot = hmap$plot, description = hmap$description)
		}

		## Create heatmaps of correlations and save the values to CSV files
		file.tbl <- matrix(character(), nrow = 0, ncol = 2, dimnames = list(NULL, c("Location Type", "File Name")))
		rplots <- list()
		for (target.id in 1:length(targets)) {
			tbl <- batcheffects$pc[[target.id]]$correlations
			if (!all(is.na(tbl))) {
				fnames <- paste0(c("correlations_pc_trait_", "heatmap_pc_correlations_"), target.id, c(".csv", ""))
				result <- append.table(tbl, fnames, plot.heatmap.pc.correlations)
				file.tbl <- rbind(file.tbl, c(targets[target.id], result$link))
				rplots[[targets[target.id]]] <- result$plot
				description <- result$description
			}
		}
		tbl.correlations.present <- (length(rplots) != 0)
		if (tbl.correlations.present) {
			txt <- c("The next figure shows the computed correlations between the first ",
				rnb.getOption("exploratory.principal.components"), " principal components and the sample traits.")
			rnb.add.paragraph(report, txt)
			report <- rnb.add.figure(report, description, rplots, setting.names)
			colnames(file.tbl) <- c("Location type", "Table file")
			rnb.add.exported.tables(report, file.tbl)
		}

		## Create heatmaps of p-values and save the values to CSV files
		file.tbl <- matrix(character(), nrow = 0, ncol = 2, dimnames = list(NULL, c("Location Type", "File Name")))
		rplots <- list()
		for (target.id in 1:length(targets)) {
			pc.tables <- batcheffects$pc[[target.id]]
			tbl <- pc.tables$pvalues
			if (!all(is.na(tbl))) {
				fnames <- paste0(c("pvalues_pc_trait_", "heatmap_pc_pvalues_"), target.id, c(".csv", ""))
				result <- append.table(tbl, fnames, plot.heatmap.pc.pvalues)
				file.tbl <- rbind(file.tbl, c(targets[target.id], result$link))
				rplots[[targets[target.id]]] <- result$plot
				description <- result$description
			}
		}
		tbl.pvalues.present <- (length(rplots) != 0)
		if (nperm == 0) {
			if (tbl.correlations.present) {
				txt <- c("Since permutation tests were disabled, no p-values were estimated from the correlations ",
					"shown above.")
				rnb.add.paragraph(report, txt)
			}
		} else if (tbl.pvalues.present) {
			txt <- c("The heatmap below summarizes the results of permutation tests performed for associations. ",
				"Significant p-values (values less than ", rnb.getOption("exploratory.correlation.pvalue.threshold"),
				") are displayed in pink background.")
			rnb.add.paragraph(report, txt)
			report <- rnb.add.figure(report, description, rplots, setting.names)
			txt <- c("The full tables of p-values for each location type are available in CSV (comma-separated ",
				"value) files below.")
			rnb.add.paragraph(report, txt)
			rnb.add.table(report, file.tbl)
		} else {
			## TODO: All p-value tables are full of NAs
		}

		rm(pc.tables)
	}

	## -----------------------------------------------------------------------------------------------------------------
	## Summarize the results of trait association

	if ("traits" %in% names(batcheffects)) {
		trait.tables <- batcheffects$traits
		txt <- c("This section summarizes the associations between pairs of traits.")
		report <- rnb.add.section(report, "Associations between Traits", txt, level = 2)

		## Create a triangular heatmap of the performed tests for associations
		plots.associations <- list(NULL, NULL)
		txt <- c("The figure below visualizes the tests that were performed on trait pairs based on the description ",
			"provided above.")
		tbl <- trait.tables$tests
		tbl.failures <- trait.tables$failures
		if (!all(is.na(tbl.failures))) {
			txt <- c(txt, " In some cases, pairs of traits could not be tested for associations. These scenarios are ",
				"marked by grey shapes, and the underlying reason is given in the figure legend.")
		} else {
			tbl.failures <- NULL
		}
		plots.associations[[1]] <- plot.heatmap.symm(report, tbl, tbl.failures, "heatmap_traits_tests")

		## Create a triangular heatmap of the calculated p-values for associations
		tbl <- trait.tables$pvalues
		plots.associations[[2]] <- plot.heatmap.symm(report, tbl, NULL, "heatmap_traits_pvalues")

		## Attach the table of the calculated p-values for associations
		tbl <- cbind("Traits" = rownames(tbl), as.data.frame(tbl, check.names = FALSE))
		fname <- "pvalues_traits.csv"
		utils::write.csv(tbl, file = file.path(data.dir.abs, fname), na = "", row.names = FALSE)

		## Add a figure of the triangular heatmaps
		txt <- c(txt, " In addition, the calculated p-values for associations between traits are shown. ",
			"Significant p-values (values less than ", rnb.getOption("exploratory.correlation.pvalue.threshold"),
			") are displayed in pink background. The full table of p-values is available in a <a href=\"",
			data.dir.rel, "/", fname, "\">dedicated file</a> that accompanies this report.")
		rnb.add.paragraph(report, txt)
		txt <- c("(1) Table of performed tests on pairs of traits. Test names (Correlation + permutation test, ",
			"Fisher's exact test, Wilcoxon rank sum test and/or Kruskal-Wallis one-way analysis of variance) are ",
			"color-coded according to the legend given above.<br />")
		txt <- c(txt, "(2) Table of resulting p-values from the performed tests on pairs of traits. Significant ",
			"p-values (less than ", rnb.getOption("exploratory.correlation.pvalue.threshold"), ") are printed in pink boxes ",
			"Non-significant values are represented by blue boxes. White cells, if present, denote missing values.")
		setting.names <- list("Heatmap of" = c("tests" = "tests performed", "pvalues" = "p-values"))
		report <- rnb.add.figure(report, txt, plots.associations, setting.names,  selected.image = 2)
		rm(plots.associations, txt, tbl, tbl.failures, fname, setting.names)

		if (!all(is.na(trait.tables$correlations))) {
			## Attach the table of correlations to the report
			txt <- "In some cases, a correlation was computed between a pair of traits."
			if (nperm > 0) {
				txt <- c(txt, " As described earlier in this report, these correlation values are used as the basis ",
					"for a permutation-based test.")
			}
			tbl <- trait.tables$correlations
			tbl <- cbind("Traits" = rownames(tbl), as.data.frame(tbl, check.names = FALSE))
			fname <- "correlations_traits.csv"
			utils::write.csv(tbl, file = file.path(data.dir.abs, fname), na = "", row.names = FALSE)
			txt <- c(txt, " The table of computed correlations is available as a <a href=\"",
				data.dir.rel, "/", fname, "\">comma-separated file</a> accompanying this report.")
			rnb.add.paragraph(report, txt)
		}
	}

	return(report)
}

########################################################################################################################

#' rnb.step.batcheffects
#'
#' Executes the batch effects step and adds a dedicated section to the given report.
#'
#' @param rnb.set             Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param report              Report to contain the dimension reduction section. This must be an object of type
#'                            \code{\linkS4class{Report}}.
#' @param pcoordinates        Coordinates of the samples of \code{rnb.set} in the principal components space, as returned
#'                            by \code{\link{rnb.execute.dreduction}}.
#' @param return.permutations Flag indicating if the generated sample permutations should also be returned.
#' @return If \code{return.permutations} is \code{FALSE}: the modified report. Otherwise, a list of two elements:
#'         \describe{
#'           \item{report}{The modified report.}
#'           \item{permutations}{\code{integer} matrix of sample index permutations. See the return value of
#'                \code{\link{rnb.execute.batcheffects}} for more information. This element is \code{NULL} if no
#'                permutations were generated.}
#'         }
#'
#' @seealso \code{\link{rnb.execute.batcheffects}}, \code{\link{rnb.section.batcheffects}},
#'   \code{\link{rnb.run.exploratory}}
#' @author Yassen Assenov
#' @noRd
rnb.step.batcheffects <- function(rnb.set, report, pcoordinates, return.permutations = FALSE) {
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	nsamples <- validate.pcoordinates.all(pcoordinates)
	if (is.character(nsamples)) {
		stop(nsamples)
	}
	rm(nsamples)
	if (!parameter.is.flag(return.permutations)) {
		stop("invalid value for return.permutations; expected TRUE or FALSE")
	}
	if (rnb.getOption("logging") && logger.isinitialized() == FALSE) {
		logger.start(fname = NA) # initialize console logger
	}

	if (is.null(pheno(rnb.set))) {
		txt <- "Batch effects were not studied because the dataset contains no phenotype data."
		report <- rnb.add.section(report, "Batch Effects", txt)
		return(report)
	}

	batcheffects <- rnb.execute.batcheffects(rnb.set, pcoordinates)
	report <- rnb.section.batcheffects(report, batcheffects)

	if (return.permutations) {
		return(list(report = report, permutations = batcheffects$permutations))
	}
	return(report)
}

########################################################################################################################
## dataImport.R
## created: 2012-04-04
## creator: Pavlo Lutsik
## ---------------------------------------------------------------------------------------------------------------------
## Data reading routines.
########################################################################################################################

## G L O B A L S #######################################################################################################

## Strings in a sample annotation table that are read as missing values (NAs)
NA.STRINGS <- c("NA","N/A","N.A.","n/a","n.a.","")

## Regular expression for a section header in a CSV file exported from Genome Studio
REGEX.CSV.HEADER <- "^\\[(.+)\\],*$"

## probe category definitions for Infinium arrays
INTENSITY.SUMMARIZATION.INFO<-list(
        probesEPIC=list(
                "typeIred"=list(Design="I", Color="Red", Msource="Red", Usource="Red", Maddress="AddressB", Uaddress="AddressA"),
                "typeIgrn"=list(Design="I", Color="Grn", Msource="Grn", Usource="Grn", Maddress="AddressB", Uaddress="AddressA"),
                "typeII"=list(Design="II", Color="Both", Msource="Grn", Usource="Red", Maddress="AddressA", Uaddress="AddressA")),
		probesEPICv2=list(
                "typeIred"=list(Design="I", Color="Red", Msource="Red", Usource="Red", Maddress="AddressB", Uaddress="AddressA"),
                "typeIgrn"=list(Design="I", Color="Grn", Msource="Grn", Usource="Grn", Maddress="AddressB", Uaddress="AddressA"),
                "typeII"=list(Design="II", Color="Both", Msource="Grn", Usource="Red", Maddress="AddressA", Uaddress="AddressA")),
        probes450=list(
                "typeIred"=list(Design="I", Color="Red", Msource="Red", Usource="Red", Maddress="AddressB", Uaddress="AddressA"),
                "typeIgrn"=list(Design="I", Color="Grn", Msource="Grn", Usource="Grn", Maddress="AddressB", Uaddress="AddressA"),
                "typeII"=list(Design="II", Color="Both", Msource="Grn", Usource="Red", Maddress="AddressB", Uaddress="AddressA")),
        probes27=list(
                "typeIred"=list(Design="I", Color="red", Msource="Red", Usource="Red", Maddress="AddressB", Uaddress="AddressA"),
                "typeIgrn"=list(Design="I", Color="green", Msource="Grn", Usource="Grn", Maddress="AddressB", Uaddress="AddressA")),
        probesMMBC=list(
                "typeIred"=list(Design="I", Color="Red", Msource="Red", Usource="Red", Maddress="AddressB", Uaddress="AddressA"),
                "typeIgrn"=list(Design="I", Color="Grn", Msource="Grn", Usource="Grn", Maddress="AddressB", Uaddress="AddressA"),
                "typeII"=list(Design="II", Color="Both", Msource="Grn", Usource="Red", Maddress="AddressA", Uaddress="AddressA"))
)



## F U N C T I O N S ###################################################################################################

#' check.barcode
#'
#' Validates that the table of sample phenotype information contains a column named \code{"barcode"}, and tries to
#' construct such a column ifit doesn't exist.
#'
#' @param dframe Table of sample phenotype data.
#' @return The possibly modified data frame; \code{NULL} ifthe provided data frame does not a column \code{"barcode"}
#'         and such a column cannot be inferred.
#' @author Yassen Assenov
#' @noRd
check.barcode <- function(dframe){
	if("barcode" %in% colnames(dframe)){
		return(dframe)
	}
	if(all(c("Sentrix_ID", "Sentrix_Position") %in% colnames(dframe))){
		if(any(is.na(dframe[, "Sentrix_ID"])) || any(is.na(dframe[, "Sentrix_Position"]))){
			stop("Invalid Sentrix_ID and/or Sentrix_Position column(s): missing values")
		}
		dframe[, "barcode"] <- paste(dframe[, "Sentrix_ID"], dframe[, "Sentrix_Position"], sep = "_")
		return(dframe)
	}
	if(all(c("Sentrix ID", "Sentrix Position") %in% colnames(dframe))){
		if(any(is.na(dframe[, "Sentrix ID"])) || any(is.na(dframe[, "Sentrix Position"]))){
			stop("Invalid Sentrix_ID and/or Sentrix_Position column(s): missing values")
		}
		dframe[, "barcode"] <- paste(dframe[, "Sentrix ID"], dframe[, "Sentrix Position"], sep = "_")
		return(dframe)
	}
	id.column <- grep("Sentrix_ID", colnames(dframe), fixed = TRUE)[1]
	position.column <- grep("Sentrix_Position", colnames(dframe), fixed = TRUE)[1]
	if(!(is.na(id.column) || is.na(position.column))){
		if(any(is.na(dframe[, position.column])) || any(is.na(dframe[, position.column]))){
			stop("Invalid Sentrix_ID and/or Sentrix_Position column(s): missing values")
		}
		dframe[, "barcode"] <- paste(dframe[, id.column], dframe[, position.column], sep = "_")
		return(dframe)
	}
	return(NULL)
}

########################################################################################################################

#' read.sample.annotation
#'
#' Reads Illumina Infinium sample annotation.
#'
#' @param fname Name of text file that contains a sample annotation table with a header. This method handles a variety
#'              of file formats, including comma-separated values file exported from Genome Studio.
#' @param sep   One-element \code{character} used as field separator in the tables file.
#' @return Sample annotation table in the form of a \code{data.frame}, in which every row corresponds to a sample, and
#'         every column - to a trait.
#'
#' @author Pavlo Lutsik
#' @examples
#' \donttest{
#'
#'   annotation.file<-system.file("")
#'   sa<-read.sample.annotation(annotation.file)
#'   sa
#' }
#' @export
read.sample.annotation <- function(fname, sep = rnb.getOption("import.table.separator")) {
	if(!(is.character(fname) && length(fname) == 1 && (!is.na(fname)))) {
		stop("invalid value for fname")
	}
	if(!(is.character(sep) && length(sep) == 1 && (!is.na(sep)) && (nchar(sep) != 0))) {
		stop("invalid value for sep")
	}

	file.contents <- scan(fname, what = "", sep = "\n", quiet = TRUE)
	i.headers <- grep(REGEX.CSV.HEADER, file.contents)
	i.first = 1L
	i.last = length(file.contents)
	if(length(i.headers) != 0) {
		headers <- toupper(sub(REGEX.CSV.HEADER, "\\1", file.contents[i.headers]))
		i.data <- which(headers == "DATA")
		if(length(i.data) == 0) {
			rnb.warning(c("File", fname, "may contain sections but [Data] was not found"))
		} else {
			if(length(i.data) > 1) {
				rnb.warning(c("File", fname, "contains multiple sections [Data]; only the first one is read"))
				i.data <- i.data[1]
			}
			i.first <- i.headers[i.data] + 1L
			if(i.data < length(i.headers)) {
				i.last <- i.headers[i.data + 1] - 1L
			}
			if(i.first > i.last) {
				logger.error(c("File", fname, "contains empty section [Data]"))
			}
		}
		rm(i.data)
	}
	rm(i.headers)

	comment.char <- ifelse(sep == "\t", "#", "")
	result <- read.table(header = TRUE, sep = sep, quote = "\"", comment.char = comment.char,
		na.strings = NA.STRINGS, check.names = FALSE, text = file.contents[i.first:i.last])
	if(ncol(result) < 2) {
		rnb.warning("The sample sheet table has only one column. Please check import.table.separator option")
	}

	i <- which(colnames(result) == "")
	if (length(i) != 0) {
		colnames(result)[i] <- paste("Unknown Column", 1:length(i))
		rnb.warning(paste0("The sample sheet contained ", ifelse(length(i) == 1, "one column", paste(i, "columns")),
			" with empty names; renamed to Unknown Column No."))
	}
	result
}

#######################################################################################################################

#' read.data.dir
#'
#' Reads in a directory with Illumina Infinium HumanMethylation450 data. The files shoudl be stored as data
#'
#' @param dir directory containing the table files
#' @param pheno a file containing data sample annotations and phenotypic information
#' @param betas a file containing the beta values. If not supplied, the routine will look in dir for a file containing "beta" token in the filename
#' @param p.values a file containing the detection p values. If not supplied, the routine will look in dir for a file containing "pval" token in the filename
#' @param bead.counts a file containing the bead counts (optional). If not supplied, the routine will look in dir for a file containing "bead" token in the filename
#' @param sep character used as field separator in the tables files. Default value is taken by the call to \code{rnb.getOption("import.table.separator")}
#' @param verbose Flag indicating ifthe messages to the logger should be sent. Note that the logger must be initialized prior to calling this function.
#' 				  Logging is useful for keeping a record of the downloaded and processed samples. Also, informative messages are stored in case of an error.
#' @details Colnames in all files should match. They will be returned as the samples element of the list.
#' @return Object of type \code{\linkS4class{RnBeadSet}}.
#'
#' @author Pavlo Lutsik
#' @export
read.data.dir<-function(dir,
				pheno,
				betas,
				p.values,
				bead.counts,
				sep=rnb.getOption("import.table.separator"),
				verbose=TRUE){

			if(verbose){
				rnb.logger.start("Loading Data from Table Files")
			}

			if(!missingArg(dir)){
				files<-list.files(dir, full.names=TRUE)
			}else{
				files<-character()
			}

			if(length(files)>10){
				stop(paste("Too many files in directory:", dir), length(files))
			}

			if(missingArg(pheno)){
				pheno<-grep("Sample|sample", files, value=TRUE)
				if(length(pheno)==0){
					stop("Missing file with sample information:", files)
				}else if(length(pheno)!=1){
					stop("More than one file with sample information:", files)
				}
			}

			if(missingArg(betas)){
				if(length(grep("beta", files))==1){
					betas<-files[grep("beta", files)]
				}else{
					stop("Missing or more than one file with betas:", files)
				}
			}

			if(missingArg(p.values)){
				if(length(grep("pval", files))==1){
					p.values<-files[grep("pval", files)]
				}else{
					p.values<-NULL
				}
				#else stop("Missing or more than one file with p values:", files)
			}

			if(missingArg(bead.counts)){
				if(length(grep("bead", files)==1)){
					bead.counts<-files[grep("bead", files)]
				}else{
					bead.counts<-NULL
				}
				#else stop("Missing or more than one file with bead counts:", files)
			}

			pheno.table<-read.sample.annotation(pheno, sep=sep)
			if(verbose){
				rnb.info("Read table with sample information")
			}

			beta.table<-read.table(betas, sep=sep, header=TRUE, check.names=FALSE)
			if(verbose){
				rnb.info("Read table with betas")
			}
			if(ncol(beta.table)<2){
				rnb.warning("Beta-value table has less than two columns: check the value of sep")
			}
			if(is.null(rownames(beta.table))){
				rnb.error("Beta-value table has no row names: cannot match probes to the annotation")
			}

			if(!is.null(p.values)){
				p.value.table<-read.table(p.values, sep=sep, header=TRUE, check.names=FALSE)
				if(verbose){
					rnb.info("Read table with p-values")
				}
				if(ncol(p.value.table)<2){
					rnb.warning("P-value table has less than two columns: check the value of sep")
				}
			}else p.value.table<-NULL

			if(!is.null(bead.counts)){
				bead.count.table<-read.table(bead.counts, sep=sep, header=TRUE, check.names=FALSE)
				if(verbose){
					rnb.info("Read table with bead counts")
				}
				if(ncol(bead.count.table)<2){
					rnb.warning("Bead count table has less than two columns: check the value of sep")
				}
			}else {
				bead.count.table<-NULL
			}

			if(!is.null(p.value.table)){
				if(sum(colnames(beta.table) != colnames(p.value.table))>0){
					stop("The columns of p-value table do not match the columns of the beta value table ", "")
				}
			}
			if(!is.null(bead.count.table)){
				if(sum(colnames(beta.table) != colnames(bead.count.table))>0){
					stop("The columns of bead counts table do not match the columns of the beta value table ", "")
				}
			}
			
			platform <- rnb.getOption("import.idat.platform")
			if(platform=="auto"){
				platform <- ifelse(nrow(beta.table)>500000L,ifelse(nrow(beta.table)>900000L,"EPICv2","EPIC"),
							ifelse(nrow(beta.table)<30000L,"27k","450k"))
			}else{
				platform.dict <- list("probesEPIC" = "EPIC", "probesEPICv2" = "EPICv2",
									  "probes27" = "27k", "probes450" = "450k")
				platform <- platform.dict$platform
				rm(platform.dict)
			}				
			
			data.set<-RnBeadSet(
					pheno=pheno.table,
					probes=rownames(beta.table),
					betas=as.matrix(beta.table),
					p.values=if(is.null(p.value.table)) p.value.table else as.matrix(p.value.table),
					bead.counts=if(is.null(bead.count.table)) bead.count.table else as.matrix(bead.count.table),
					platform=platform)

			if(verbose) {
				rnb.logger.completed()
			}
			return(data.set)

}
########################################################################################################################

#' read.idat.files2
#'
#' Reads a directory of \code{.idat} files and initializes an object of type \code{\linkS4class{MethyLumiSet}}.
#'
#' @param base.dir       Directory that contains the \code{.idat} files to be read; or a character vector of such
#'                       directories.
#' @param barcodes       Optional non-empty \code{character} vector listing the barcodes of the samples that should be
#'                       loaded. If supplied, this vector must not contain \code{NA} among its elements.
#' @param sample.sheet   Optional file name containing a table of sample annotation data, or the table itself in the
#'                       form of a \code{\link{data.frame}} or \code{matrix}. Only (and all) samples defined in this
#'                       table will be loaded. The table is expected to contain a column named \code{"barcode"} that
#'                       lists the samples' Sentrix  barcodes. If such a column is not present, this function searches
#'                       for columns \code{"Sentrix_ID"} and \code{"Sentrix_Position"} (or similar) that build a
#'                       barcode.
#' @param sep.samples	 character used as field separator in the sample sheet file.
#' 						 Default value is taken by the call to \code{rnb.getOption("import.table.separator")}
#' @param load.chunk	 \code{integer} of size one, giving the number of IDAT files which should be loaded in
#' 						 one loading cycle or \code{NULL}, in which case an attempt will be made to load all files
#' 						 in one go. Should be assigned in case the number of IDATs is more than one thousand.
#' @param keep.methylumi a flag indicating whether the a \code{MethyLumiSet}
#'						 object should be returned instead of a \code{RnBeadRawSet}.
#' @param verbose        Flag indicating ifthe messages to the logger should be sent. Note that the logger
#'                       must be initialized prior to calling this function. Logging is useful for keeping a
#'                       record of the downloaded and processed samples. Also, informative messages are
#'                       stored in case of an error.
#' @return Loaded dataset of HumanMethylation450K samples, encapsulated in an object of type \code{MethyLumiSet}.
#'
#' @details
#' If neither \code{barcodes}, nor \code{sample.sheet} are specified, the function attempts to locate a file in
#' \code{base.dir} containing sample annotation information. It fails ifsuch a file cannot be (unambiguously)
#' identified. If both \code{barcodes} and \code{sample.sheet} are supplied, only \code{sample.sheet} is used in loading
#' methylation data. The value of \code{barcodes} is tested for validity but it is not used as a filter.
#'
#' @seealso \code{\link{methylumIDAT}} in package \pkg{methylumi}
#'
#' @author Pavlo Lutsik
read.idat.files2 <- function(base.dir,
		barcodes = NULL,
		sample.sheet = NULL,
		sep.samples=rnb.getOption("import.table.separator"),
		load.chunk=NULL,
		keep.methylumi=FALSE,
		verbose = TRUE){

	if(!(is.character(base.dir) && length(base.dir) == 1 && (!is.na(base.dir)))) {
		stop("invalid value for base.dir")
	}
	if(!is.null(barcodes)) {
		if(!is.character(barcodes) && length(barcodes) != 0 && (!any(is.na(barcodes)))) {
			stop("invalid value for barcodes; expected non-empty character vector or NULL")
		}
	}
	if(!parameter.is.flag(verbose)){
		stop("invalid value for verbose; expected TRUE or FALSE")
	}

	if(verbose){
		rnb.logger.start("Loading Data from IDAT Files")
	}

	if(is.null(barcodes) && is.null(sample.sheet)){
		## Attempt to locate sample annotation file
		sample.sheet <- list.files(base.dir, pattern = "[Ss]ample", full.names = TRUE)
		sample.sheet <- sample.sheet[file.info(sample.sheet)[, "isdir"] == FALSE]
		if(length(sample.sheet) > 1) {
			rnb.error(paste("No unique candidate for sample annotation found in", base.dir))
		}
		if(length(sample.sheet) == 0 || is.na(sample.sheet)){
			rnb.error(paste("No candidate for sample information found in", base.dir))
		}
	}
	if(!is.null(sample.sheet)) {
		if(is.character(sample.sheet) && (!is.matrix(sample.sheet))){
			if(length(sample.sheet) != 1) {
				stop("invalid value for sample.sheet")
			}
			sample.sheet <- read.sample.annotation(sample.sheet, sep=sep.samples)
		} else if(is.matrix(sample.sheet)){
			sample.sheet <- as.data.frame(sample.sheet, check.names = FALSE, stringsAsFactors = FALSE)
		}
		if(!is.data.frame(sample.sheet)){
			stop("invalid value for sample.sheet")
		}
		if(!("barcode" %in% colnames(sample.sheet))){
			sample.sheet <- check.barcode(sample.sheet)
			if(is.null(sample.sheet)){
				rnb.error("sample.sheet does not contain column barcode")
			}
			if(verbose) {
				rnb.info("Added column barcode to the provided sample annotation table")
			}
		}
	}
	if(is.null(barcodes) && is.null(sample.sheet)){
		stop("invalid value for barcodes and/or sample.sheet")
	}
	if(ncol(sample.sheet) < 2){
		rnb.error("The sample annotation table has less than two columns. Check the \"default.table.separator\" option")
	}

	## Check subdirectories
	if(check.idat.subdirs(base.dir)){
		## Found subdirectories that contain .idat files
		if(verbose) {
			rnb.info("Found subdirectores with idat files; creating symbolic links")
		}
		base.dir <- prepare.idat.dir(base.dir)
		if(is.null(base.dir)){
			rnb.error("Could not create temporary directory or create links to idat files")
		}
		fn <- attr(base.dir, "failed")
		if(fn != 0) {
			rnb.warning(paste("Could not create links to", fn, "idat files; these files will be ignored"))
		}
		rm(fn)
	}

	## Check whether all files a present

	if(!is.null(sample.sheet)){
		fn.base<-sample.sheet[,"barcode"]
	}else{
		fn.base<-barcodes
	}

	fn.expected<-paste(fn.base, c("Red.idat", "Grn.idat"), sep="_")
	fn.available<-list.files(base.dir, pattern="idat")
	fn.missing<-setdiff(fn.expected, fn.available)
	if(length(fn.missing)>0){
		rnb.error(sprintf("Some IDAT files are not present in the supplied base directory, for instance %s",
				paste(fn.missing[1:min(6,length(fn.missing))], collapse=", ")))
	}

	## Read the data using the methylumi package
	methylumi.params <- list(idatPath = base.dir, n = TRUE)
	if(is.null(sample.sheet)){
		if(!is.null(load.chunk) && load.chunk>length(barcodes)){
			rnb.warning("The value of the load.chunk parameter is incorrect.")
			load.chunk=NULL
			nsamp<-length(barcodes)
		}
	}else{
		if(!is.null(load.chunk) && load.chunk>nrow(sample.sheet)){
			rnb.warning("The value of the load.chunk parameter is incorrect.")
			load.chunk=NULL
			nsamp<-nrow(sample.sheet)
		}
	}

	if(is.null(load.chunk)){
		if(is.null(sample.sheet)){
			methylumi.params$barcodes <- barcodes
		}else{ # is.data.frame(sample.sheet)
			methylumi.params$pdat <- sample.sheet
		}
		mls <- suppressWarnings(suppressMessages(do.call(methylumIDAT, methylumi.params)))
		if(!keep.methylumi){
			mls<-as(mls, "RnBeadRawSet")
		}
	}else{
		# load the data chunk-by-chunk, converting to RnBeadRawSet
        # and combining them after every step

		mls <- NULL
		if(is.null(sample.sheet)) {
			nsamp<-length(barcodes)
		}else{
			nsamp<-nrow(sample.sheet)
		}
		nch <- nsamp %/% load.chunk

		## TODO: remove this strange thing
		# temporarily disable region summarization
		rt<-rnb.getOption("region.types")
		rnb.options(region.types=character())
		####

		for(ch in 1:(nch+(nsamp %% load.chunk > 0))){
			if(verbose){
				rnb.info(sprintf("Loading sample chunk %d", ch))
			}

			ixxs <- (load.chunk*(ch-1L)+1L):min((load.chunk*ch), nsamp)

			if(is.null(sample.sheet)){
				methylumi.params$barcodes <- barcodes[ixxs]
			} else { # is.data.frame(sample.sheet)
				methylumi.params$pdat <- sample.sheet[ixxs,]
			}

			mlsch <- suppressWarnings(suppressMessages(do.call(methylumIDAT, methylumi.params)))

			rnbch <- as(mlsch, "RnBeadRawSet")

			if(is.null(mls)){
				mls<-rnbch
			}else{
				mls.old <- mls
				# a couple of disk space saving hacks
				if(rnb.getOption("enforce.destroy.disk.dumps")){
					if(mls.old@status$disk.dump){
						mls.old@status$discard.ff.matrices<-TRUE
					}
					if(rnbch@status$disk.dump){
						rnbch@status$discard.ff.matrices<-TRUE
					}
				}
				mls<-combine(mls.old, rnbch)
				if(rnb.getOption("enforce.destroy.disk.dumps")){
					if(mls@status$disk.dump){
						mls@status$discard.ff.matrices<-NULL
					}
				}
				rnb.call.destructor(mls.old)
				rnb.call.destructor(rnbch)
			}

			rm(rnbch)
			rnb.cleanMem()
		}
		#gc()

		rnb.options(region.types=rt)
		genome.assembly<-rnb.getOption("assembly") ## TODO: Improve genome build selection 
		for (region.type in rnb.region.types.for.analysis(genome.assembly)) {
			mls <- summarize.regions(mls, region.type)
		}
		rnb.cleanMem()

	}

	if(verbose){
		rnb.logger.completed()
	}

	return(mls)

}

########################################################################################################################

#' read.idat.files
#'
#' Reads a directory of \code{.idat} files and initializes an object of type \code{\linkS4class{MethyLumiSet}}.
#'
#' @param base.dir       Directory that contains the \code{.idat} files to be read; or a character vector of such
#'                       directories.
#' @param barcodes       Optional non-empty \code{character} vector listing the barcodes of the samples that should be
#'                       loaded. If supplied, this vector must not contain \code{NA} among its elements.
#' @param sample.sheet   Optional file name containing a table of sample annotation data, or the table itself in the
#'                       form of a \code{\link{data.frame}} or \code{matrix}. Only (and all) samples defined in this
#'                       table will be loaded. The table is expected to contain a column named \code{"barcode"} that
#'                       lists the samples' Sentrix  barcodes. If such a column is not present, this function searches
#'                       for columns \code{"Sentrix_ID"} and \code{"Sentrix_Position"} (or similar) that build a
#'                       barcode.
#' @param sep.samples	 \code{character} string used as field separator in the sample sheet file.
#' 						 Default value is taken by the call to \code{rnb.getOption("import.table.separator")}
#' @param dpval.method   \code{character} indicating which method is to be used to compute detection p-values
#' 				By default, \code{'controls'} uses the built-in control probes, while the
#' 				\code{'pOOBAH'} method from the sesame package is also available.
#' @param useff			 If \code{TRUE} \code{ff} package is used to store large matrices on the hard disk
#' @param verbose        Flag specifying whether the messages to the logger should be sent. Note that the logger
#'                       must be initialized prior to calling this function. Logging is useful for keeping a
#'                       record of the downloaded and processed samples. Also, informative messages are
#'                       stored in case of an error.
#' @return Loaded dataset of HumanMethylation450K samples, encapsulated in an object of type \code{MethyLumiSet}.
#'
#' @details
#' If neither \code{barcodes}, nor \code{sample.sheet} are specified, the function attempts to locate a file in
#' \code{base.dir} containing sample annotation information. It fails ifsuch a file cannot be (unambiguously)
#' identified. If both \code{barcodes} and \code{sample.sheet} are supplied, only \code{sample.sheet} is used in loading
#' methylation data. The value of \code{barcodes} is tested for validity but it is not used as a filter.
#'
#' @seealso \code{\link{methylumIDAT}} in package \pkg{methylumi}
#'
#' @author Pavlo Lutsik
#' @export
read.idat.files <- function(base.dir,
		barcodes = NULL,
		sample.sheet = NULL,
		sep.samples=rnb.getOption("import.table.separator"),
		dpval.method="controls",
		useff = FALSE,
		verbose = TRUE){

	if(!(is.character(base.dir) && length(base.dir) == 1 && (!is.na(base.dir)))) {
		stop("invalid value for base.dir")
	}
	if(!is.null(barcodes)) {
		if(!is.character(barcodes) && length(barcodes) != 0 && (!any(is.na(barcodes)))) {
			stop("invalid value for barcodes; expected non-empty character vector or NULL")
		}
	}
	if(!parameter.is.flag(verbose)){
		stop("invalid value for verbose; expected TRUE or FALSE")
	}

	if(verbose){
		rnb.logger.start("Loading Data from IDAT Files")
	}

	if(is.null(barcodes) && is.null(sample.sheet)){
		## Attempt to locate sample annotation file
		sample.sheet <- list.files(base.dir, pattern = "[Ss]ample", full.names = TRUE)
		sample.sheet <- sample.sheet[file.info(sample.sheet)[, "isdir"] == FALSE]
		if(length(sample.sheet) > 1) {
			rnb.error(paste("No unique candidate for sample annotation found in", base.dir))
		}
		if(length(sample.sheet) == 0 || is.na(sample.sheet)){
			rnb.error(paste("No candidate for sample information found in", base.dir))
		}
	}
	if(!is.null(sample.sheet)) {
		if(is.character(sample.sheet) && (!is.matrix(sample.sheet))){
			if(length(sample.sheet) != 1) {
				stop("invalid value for sample.sheet")
			}
			sample.sheet <- read.sample.annotation(sample.sheet, sep=sep.samples)
		} else if(is.matrix(sample.sheet)){
			sample.sheet <- as.data.frame(sample.sheet, check.names = FALSE, stringsAsFactors = FALSE)
		}
		if(!is.data.frame(sample.sheet)){
			stop("invalid value for sample.sheet")
		}
		if(!("barcode" %in% colnames(sample.sheet))){
			sample.sheet <- check.barcode(sample.sheet)
			if(is.null(sample.sheet)){
				rnb.error("sample.sheet does not contain column barcode")
			}
			if(verbose) {
				rnb.info("Added column barcode to the provided sample annotation table")
			}
		}
	}
	if(is.null(barcodes) && is.null(sample.sheet)){
		stop("invalid value for barcodes and/or sample.sheet")
	}
	if(!is.null(barcodes) && any(is.na(barcodes))){
		stop("invalid value for barcodes: missing values")
	}
	if(ncol(sample.sheet) < 2){
		rnb.error("The sample annotation table has less than two columns. Check the \"default.table.separator\" option")
	}

	## Check subdirectories
	if(check.idat.subdirs(base.dir)){
		## Found subdirectories that contain .idat files
		if(verbose) {
			rnb.info("Found subdirectores with idat files; creating symbolic links")
		}
		base.dir <- prepare.idat.dir(base.dir)
		if(is.null(base.dir)){
			rnb.error("Could not create temporary directory or create links to idat files")
		}
		fn <- attr(base.dir, "failed")
		if(fn != 0) {
			rnb.warning(paste("Could not create links to", fn, "idat files; these files will be ignored"))
		}
		rm(fn)
	}

	## Check whether all files a present

	if(!is.null(sample.sheet)){
		fn.base<-sample.sheet[,"barcode"]
		nsamp<-nrow(sample.sheet)
	}else{
		fn.base<-barcodes
		nsamp<-length(barcodes)
	}

	fn.expected<-unlist(lapply(fn.base, paste, c("Red.idat", "Grn.idat"), sep="_"))
	fn.available<-list.files(base.dir, pattern="idat", recursive=TRUE, full.names=TRUE)
	fn.available.short<-sapply(strsplit(fn.available, split=.Platform$file.sep), function(x) x[length(x)])
	fn.map<-match(fn.expected, fn.available.short)
	fn.missing<-which(is.na(fn.map))
	if(length(fn.missing)>0){
		rnb.error(sprintf("Some IDAT files are not present in the supplied base directory, for instance %s",
						paste(fn.expected[fn.missing[1:min(6,length(fn.missing))]], collapse=", ")))
	}
	idat.fnames<-fn.available[fn.map]
	idat.red.fnames<-idat.fnames[2*(1:nsamp)-1]
	idat.grn.fnames<-idat.fnames[2*(1:nsamp)]

	## Detect Infinium platform
	platform<-rnb.detect.infinium.platform(idat.fnames)
	genome.assembly<-rnb.getOption("assembly")
	if(verbose) {
		txt <- c("probes27"="HumanMethylation27",
			"probes450"="HumanMethylation450",
			"probesEPIC"="MethylationEPIC",
			"probesEPICv2"="MethylationEPICv2",
			"probesMMBC"="MouseMethylationBeadChip")
		rnb.info(paste("Detected platform:", txt[platform]))
		if (platform == "probesEPICv2" && genome.assembly != "hg38") {
			rnb.info(paste0("MethylationEPICv2 is not supported for this session's genome assembly: ", genome.assembly, ". Changing genome assembly to: hg38"))
			rnb.options(assembly = "hg38") ## EPICv2 is only annotated in the RnBeads.hg38 package
			genome.assembly<-rnb.getOption("assembly")
		}
		if (platform == "probes450" && genome.assembly == "hg38") {
		  rnb.info(paste0("Methylation450k is not supported for this session's genome assembly: ", genome.assembly, ". Changing genome assembly to: hg19"))
		  rnb.options(assembly = "hg19") ## 450k only for hg19
		  genome.assembly<-rnb.getOption("assembly")
		}
		if (platform == "probes27" && genome.assembly == "hg38") {
		  rnb.info(paste0("Methylation27k is not supported for this session's genome assembly: ", genome.assembly, ". Changing genome assembly to: hg19"))
		  rnb.options(assembly = "hg19") ## 450k only for hg19
		  genome.assembly<-rnb.getOption("assembly")
		}
		rnb.info(paste("Annotation package genome assembly version:", genome.assembly))
		rm(txt)
	}
	
    genome<-c(
            "probes27"="hg19",
            "probes450"=ifelse(genome.assembly == "hg19", "hg19", "hg38"),
            "probesEPIC"=ifelse(genome.assembly == "hg19", "hg19", "hg38"),
			"probesEPICv2"="hg38",
            "probesMMBC"="mm10")
    
    annot_gr<-rnb.get.annotation(platform, genome[platform])
    ### TODO: remove after fixing the annotation
#    if(platform=="probesMMBC"){
#        annot_gr<-endoapply(annot_gr, function(ag) {names(ag)<-mcols(ag)[["ID"]]; return(ag)})
#    }
	annot<-rnb.annotation2data.frame(annot_gr)
    annot.ctrls<-rnb.get.annotation(gsub("probes","controls",platform), genome[platform])
	nprobes<-sum(rnb.annotation.size(platform, genome[platform]))
	ncprobes<-nrow(annot.ctrls)
	
	if(platform=="probesEPIC"){
        id.col<-"ID"
		ctrls.address.col<-"ID"
		ctrls.target.col<-"Target"
		neg.ctrl.indexes<-which(annot.ctrls[["Target"]]=="NEGATIVE")
	}else if(platform=="probesEPICv2"){
        id.col<-"ID"
		ctrls.address.col<-"ID"
		ctrls.target.col<-"Target"
		neg.ctrl.indexes<-which(annot.ctrls[["Target"]]=="NEGATIVE")
	}else if(platform=="probes450"){
        id.col<-"ID"
		ctrls.address.col<-"ID"
		ctrls.target.col<-"Target"
		neg.ctrl.indexes<-which(annot.ctrls[["Target"]]=="NEGATIVE")
	}else if(platform=="probes27"){
        id.col<-"ID"
		ctrls.address.col<-"Address"
		ctrls.target.col<-"Type"
		neg.ctrl.indexes<-which(annot.ctrls[["Type"]]=="Negative")
	}else if(platform=="probesMMBC"){
        id.col<-"ID"
        ctrls.address.col<-"ID"
        ctrls.target.col<-"Target"
        neg.ctrl.indexes<-which(annot.ctrls[["Target"]]=="NEGATIVE")
    }

	tIred<-which(annot$Color=="Red")
	tIgrn<-which(annot$Color=="Grn")
	tII<-which(annot$Design=="II")

	probes<-character(length=nprobes)

	if(useff){
		dpvals<-ff(NA_real_, dim=c(nprobes, nsamp), dimnames=list(NULL, NULL), vmode="double")
		M<-ff(NA_real_, dim=c(nprobes, nsamp), dimnames=list(NULL, NULL), vmode="double")
		U<-ff(NA_real_, dim=c(nprobes, nsamp), dimnames=list(NULL, NULL), vmode="double")
		M0<-ff(NA_real_, dim=c(nprobes, nsamp), dimnames=list(NULL, NULL), vmode="double")
		U0<-ff(NA_real_, dim=c(nprobes, nsamp), dimnames=list(NULL, NULL), vmode="double")
		beadsM<-ff(NA_integer_, dim=c(nprobes, nsamp), dimnames=list(NULL, NULL))
		beadsU<-ff(NA_integer_, dim=c(nprobes, nsamp), dimnames=list(NULL, NULL))
	}else{
		dpvals<-matrix(NA_real_, nrow=nprobes, ncol=nsamp)
		M<-matrix(NA_real_, nrow=nprobes, ncol=nsamp)
		U<-matrix(NA_real_, nrow=nprobes, ncol=nsamp)
		M0<-matrix(NA_real_, nrow=nprobes, ncol=nsamp)
		U0<-matrix(NA_real_, nrow=nprobes, ncol=nsamp)
		beadsM<-matrix(NA_integer_, nrow=nprobes, ncol=nsamp)
		beadsU<-matrix(NA_integer_, nrow=nprobes, ncol=nsamp)
	}

	qc.int<-list()
	qc.int$Cy3<-matrix(NA_real_, nrow=ncprobes, ncol=nsamp, dimnames = list(annot.ctrls[[ctrls.address.col]], NULL))
	qc.int$Cy5<-matrix(NA_real_, nrow=ncprobes, ncol=nsamp, dimnames = list(annot.ctrls[[ctrls.address.col]], NULL))

	probe.categories<-INTENSITY.SUMMARIZATION.INFO[[platform]]

	get.OOB.channel<-function(channel){
		if(channel=="Red"){
			return("Grn")
		}else if(channel=="Grn"){
			return("Red")
		}else{
			stop("Wrong input for channel")
		}
	}

	translate.channel.name<-function(channel){

		if(channel=="Red"){
			return("Cy5")
		}else if(channel=="Cy5"){
			return("Red")
		}else if(channel=="Grn"){
			return("Cy3")
		}else if(channel=="Cy3"){
			return("Grn")
		}else{
			stop("Unrecognized channel")
		}
	}

	probes<-annot[[id.col]]
	for(pcind in 1:length(probe.categories)){
		probe.categories[[pcind]]$Indices<-which(annot$Design==probe.categories[[pcind]]$Design &
						annot$Color==probe.categories[[pcind]]$Color)
		probe.categories[[pcind]]$Maddress<-annot[probe.categories[[pcind]]$Indices,probe.categories[[pcind]]$Maddress]
		probe.categories[[pcind]]$Uaddress<-annot[probe.categories[[pcind]]$Indices,probe.categories[[pcind]]$Uaddress]
		#probes[probe.categories[[pcind]]$Indices]<-annot[probe.categories[[pcind]]$Indices,"ID"]
	}

  mouse_fractions <- c()
	for(sid in 1:nsamp){

		#barcode<-sample.sheet$barcode[sid]
		idatfile.red<-idat.red.fnames[sid]#file.path(base.dir, sprintf("%s_Red.idat",barcode))
		idatfile.grn<-idat.grn.fnames[sid]#file.path(base.dir, sprintf("%s_Grn.idat",barcode))

		if(!file.exists(idatfile.red)){
			rnb.error(sprintf("IDAT file not found: %s", idatfile.red))
		}
		if(!file.exists(idatfile.grn)){
			rnb.error(sprintf("IDAT file not found: %s", idatfile.grn))
		}

		int.files<-list()
		int.files[["Red"]]<-readIDAT(idatfile.red)
		int.files[["Grn"]]<-readIDAT(idatfile.grn)
      
    if(rnb.getOption('inference.mouse.fraction')){
			res_fraction <- rnb.run_xeno(gsub("_Red.idat", "", idatfile.red))
      mouse_fractions <- c(mouse_fractions, res_fraction$mouse_fraction)
    }

		ctrls.indexes<-match(annot.ctrls[[ctrls.address.col]], int.files[["Red"]]$MidBlock)

		qc.int$Cy5[!is.na(ctrls.indexes),sid]<-int.files[["Red"]]$Quant[na.omit(ctrls.indexes),1L]
		qc.int$Cy3[!is.na(ctrls.indexes),sid]<-int.files[["Grn"]]$Quant[na.omit(ctrls.indexes),1L]

		if(dpval.method=="pOOBAH"){
 			cr<-unlist(annot[annot$"Cross-reactive"!=0,c("AddressA", "AddressB")])
			grn_probes<-setdiff(probe.categories[["typeIgrn"]]$Maddress, cr)
			red_probes<-setdiff(probe.categories[["typeIred"]]$Maddress, cr)

			Gmap<-match(grn_probes, int.files[["Red"]]$MidBlock)
			Rmap<-match(red_probes, int.files[["Grn"]]$MidBlock)
			bgG <- int.files[["Red"]]$Quant[Gmap,1L]
			bgR <- int.files[["Grn"]]$Quant[Rmap,1L]
			oob.bg<-list("Red"=ecdf(bgG), "Grn"=ecdf(bgR))
			rm(bgG); rm(bgR)
		}
		for(pc in probe.categories){

			Mmap<-match(pc$Maddress, int.files[[pc$Msource]]$MidBlock)
			Umap<-match(pc$Uaddress, int.files[[pc$Usource]]$MidBlock)

			M[pc$Indices,sid]<-int.files[[pc$Msource]]$Quant[Mmap,1L]
			U[pc$Indices,sid]<-int.files[[pc$Usource]]$Quant[Umap,1L]

			if(pc$Design=="I"){
				M0[pc$Indices,sid]<-int.files[[get.OOB.channel(pc$Msource)]]$Quant[Mmap,1L]
				U0[pc$Indices,sid]<-int.files[[get.OOB.channel(pc$Usource)]]$Quant[Umap,1L]
			}

			beadsM[pc$Indices,sid]<-int.files[[pc$Msource]]$Quant[Mmap,3L]
			beadsU[pc$Indices,sid]<-int.files[[pc$Usource]]$Quant[Umap,3L]
			if(dpval.method=="pOOBAH"){
				Mdist.fun<-oob.bg[[pc$Msource]]
				Udist.fun<-oob.bg[[pc$Usource]]
			}else{
				Mdist.fun<-ecdf(qc.int[[translate.channel.name(pc$Msource)]][neg.ctrl.indexes,sid])
				Udist.fun<-ecdf(qc.int[[translate.channel.name(pc$Usource)]][neg.ctrl.indexes,sid])
			}

			mpval<-1-Mdist.fun(M[pc$Indices,sid])
			upval<-1-Udist.fun(U[pc$Indices,sid])

			dpvals[pc$Indices,sid]<-rowMins(cbind(
							mpval,
							upval
			))
		}
	}

	if(verbose){
		rnb.logger.completed()
	}

	genome.assembly<-rnb.getOption("assembly")
	if(platform %in% "probes27"){
		rnb.platform<-"27k"
        assembly<-"hg19"
	}else if(platform %in% "probes450"){
		rnb.platform<-"450k"
		assembly<-ifelse(genome.assembly == "hg19", "hg19", "hg38")
	}
	else if(platform %in% "probesEPIC"){
		rnb.platform<-"EPIC"
        assembly<-ifelse(genome.assembly == "hg19", "hg19", "hg38")
	}else if(platform %in% "probesEPICv2"){
		rnb.platform<-"EPICv2"
        assembly<-"hg38"
	}else{
        rnb.platform<-"MMBC"
        assembly<-"mm10"
    }

	if(is.null(sample.sheet)){
		sample.sheet<-data.frame(barcodes=barcode)
	}

	if(rnb.getOption('inference.mouse.fraction')){
      sample.sheet$MouseFraction <- mouse_fractions
	}
	#  saveRDS(M, "/Users/baris.kalem/Code/RnBeads_Project/Kaur_CompareEPICv1_EPICv2/duplicated_probes_bug/M_good.RDS")
    
    ### solve the problem of duplicated probes
    ### in each pair select those that have a lower detection p-value
    if(rnb.platform=="MMBC" || rnb.platform=="EPICv2"){
       
       probe_names<-annot[["Name"]]
       dup_probe_names<-unique(probe_names[duplicated(probe_names)])
       dup_ind<-which(probe_names %in% dup_probe_names)
       dup_ind<-dup_ind[order(probe_names[dup_ind])]
       dup_probe_names<-probe_names[dup_ind]
   
	   if (dim(dpvals)[2] > 1){
		keep<-unlist(tapply(dup_ind, dup_probe_names, function(ind) ind[which.max(rowSums(dpvals[ind,]==colMins(dpvals[ind,])))]))
		is.one.dimensional <- FALSE
	   } else {
		keep<-unlist(tapply(dup_ind, dup_probe_names, function(ind) ind[which.max(dpvals[ind,]==dpvals[ind,])]))
		is.one.dimensional <- TRUE
	   }
       remove<-setdiff(dup_ind,keep)

	   if (is.one.dimensional) {
		probes<-data.matrix(probes[-remove])
        M<-data.matrix(M[-remove,])
        U<-data.matrix(U[-remove,])
        M0<-data.matrix(M0[-remove,])
        U0<-data.matrix(U0[-remove,])
        beadsM<-data.matrix(beadsM[-remove,])
        beadsU<-data.matrix(beadsU[-remove,])
        dpvals<-data.matrix(dpvals[-remove,])
	   } else {
		probes<-probes[-remove]
		M<-M[-remove,]
		U<-U[-remove,]
		M0<-M0[-remove,]
		U0<-U0[-remove,]
		beadsM<-beadsM[-remove,]
		beadsU<-beadsU[-remove,]
		dpvals<-dpvals[-remove,]
	   }
    }

	object<-RnBeadRawSet(
			pheno = sample.sheet,
			probes = probes,
			platform = rnb.platform,
			M = M,
			U = U,
			M0 = M0,
			U0 = U0,
			bead.counts.M = beadsM,
			bead.counts.U = beadsU,
			p.values=dpvals,
			qc=qc.int,
			region.types = rnb.region.types.for.analysis(assembly),
			useff=useff,
			summarize.bead.counts=TRUE,
			ffcleanup = rnb.getOption("enforce.destroy.disk.dumps")
			)

	return(object)
}

########################################################################################################################

#' read.GS.report
#'
#' Reads in a Genome Studio report, exported as a single file.
#'
#' @param gsReportFile  location of the GS report file
#' @param pd			alternative sample annotation, if the \code{gsReporFile} is missing the sample section as
#' 						\code{data.frame} of \code{character} singleton with the file name
#' @param sep       	character used as field separator in the sample sheet file and in the GS report file
#' 						 (should be identical).
#' 						Default value is taken by the call to \code{rnb.getOption("import.table.separator")}
#' @param keep.methylumi a flag indicating whether the a \code{MethyLumiSet}
#'						 object should be returned instead of a \code{RnBeadRawSet}.
#' @param verbose Flag indicating ifthe messages to the logger should be sent. Note that the logger must be initialized prior to calling this function.
#' 			     Logging is useful for keeping a record of the downloaded and processed samples. Also, informative messages are stored in case of an error.
#'
#' @return  MethylumiSet object with the data from the report
#'
#' @export
read.GS.report<-function(
		gsReportFile,
		pd=NULL,
		sep=rnb.getOption("import.table.separator"),
		keep.methylumi=FALSE,
		verbose=TRUE){

	if(verbose && logger.isinitialized()){
		rnb.error <- logger.error
	} else {
		rnb.error <- function(msg) { stop(paste(msg)) }
	}

	if(verbose){
		rnb.logger.start("Loading Data from a GenomeStudio Report File")
	}

	if(!(is.null(pd) || is.character(pd) || is.data.frame(pd))){
		rnb.error("Invalid value for sample descriptions")
	}

	if(is.character(pd)){
		pd<-read.sample.annotation(pd, sep=sep)
	}

	methylumiSet<-methylumiR(gsReportFile, sampleDescriptions=pd, sep=sep)

	if(is.null(annotation(methylumiSet)) || length(annotation(methylumiSet))==0){
		if(length(featureNames(methylumiSet))<30000L){
			annotation(methylumiSet)<-"IlluminaHumanMethylation27k"
		}else{
			annotation(methylumiSet)<-"IlluminaHumanMethylation450k"
		}
	}

	if(verbose){
		rnb.logger.completed()
	}

	if(keep.methylumi){
		return(methylumiSet)
	}else{
		return(as(methylumiSet, "RnBeadRawSet"))
	}

}

###########################################################################################################################

#' read.bed.files
#'
#' Reads a reduced-representation/whole-genome bisulfite sequencing data set from a set of BED files
#'
#' @param base.dir 		Directory with BED files contatining processed methylation data
#' @param file.names    Optional non-empty \code{character} vector listing the names of the files that should be
#'                      loaded relative to \code{base.dir}. If supplied, this vector must not contain \code{NA}
#' 						among its elements.
#' @param sample.sheet  Optional file name containing a table of sample annotation data, or the table itself in the form
#'                      of a \code{\link{data.frame}} or \code{matrix}. Only (and all) samples defined in this table
#'                      will be loaded. The table is expected to contain a column named \code{"barcode"} that lists the
#'                      samples' Sentrix  barcodes. If such a column is not present, this function searches for columns
#'                      \code{"Sentrix_ID"} and \code{"Sentrix_Position"} (or similar) that build a barcode.
#' @param file.names.col Column of the sample sheet which contains the file names (integer singleton). If \code{NA}
#' 						an attempt will be made to find a suiting column automatically.
#' @param assembly		Genome assembly. Defaults to human (\code{"hg19"})
#' @param region.types	\code{character} vector storing the types of regions for which the methylation information is to
#'                      be summarized. The function \code{\link{rnb.region.types}} provides the list of all supported
#'                      regions. Setting this to \code{NULL} or an empty vector restricts the dataset to site
#'                      methylation only.
#' @param pos.coord.shift	The frame shift between the the CpG annotation (1-based) and the coordinates in the loaded BEDs.
#' 						If BEDs have 0-based coordinates, \code{pos.coord.shift=1} (default).
#' @param skip.lines	The number of top lines to skip while reading the BED files
#' @param sep.samples	\code{character} singleton used as field separator in the sample sheet file.
#' 						Default value is taken by the call to \code{rnb.getOption("import.table.separator")}
#' @param merge.bed.files In case multiple BED files are specified for each sample, the flag indicates whether the
#' 						methylation calls should be merged after reading
#' @param useff			If \code{TRUE}, functionality provided by the \code{ff} package will be used to read the data efficiently.
#' @param usebigff		flag specifying whether the extended ff functionality should be used (large matrix support for ff)
#' @param verbose       Flag indicating ifthe messages to the logger should be sent. Note that the logger
#'                      must be initialized prior to calling this function. Logging is useful for keeping a
#'                      record of the downloaded and processed samples. Also, informative messages are
#'                      stored in case of an error.
#' @param ...			Further arguments which are passed to the internal function \code{read.single.bed} and to \code{read.table}
#'
#' @return an object of class \code{\linkS4class{RnBiseqSet}}
#'
#' @details To control the BED column assignment, one should also supply arguments to \code{read.single.bed}.
#'
#' @author Pavlo Lutsik
#' @export
read.bed.files<-function(base.dir=NULL,
		file.names = NULL,
		sample.sheet = NULL,
		file.names.col=0,
		assembly=rnb.getOption("assembly"),
		region.types=rnb.region.types.for.analysis(assembly),
		pos.coord.shift=1L,
		skip.lines=1,
		sep.samples=rnb.getOption("import.table.separator"),
		merge.bed.files=TRUE,
		useff=rnb.getOption("disk.dump.big.matrices"),
		usebigff=rnb.getOption("disk.dump.bigff"),
		verbose = TRUE,
		...){

	if(!parameter.is.flag(verbose)) {
		stop("Invalid value for verbose; expected TRUE or FALSE")
	}
	if(!(is.character(assembly) && length(assembly) == 1 && (!is.na(assembly)))) {
		stop("invalid value for assembly")
	}

	if(verbose){
		rnb.logger.start("Loading Data From BED Files")
	}

	if(all(is.null(base.dir), is.null(file.names), is.null(sample.sheet))){
		rnb.error("one of base.dir, file.names, sample.sheet should be supplied")
	}

	if(!is.null(base.dir) && (!is.character(base.dir) || length(base.dir)!=1)){
		rnb.error("Invalide value for parameter base.dir")
	}

	if(!is.null(file.names)) {
		if(!is.character(file.names) && length(file.names) != 0 && (!any(is.na(file.names)))) {
			rnb.error("Invalid value for file.names; expected non-empty character vector or NULL")
		}
	}

	if(!(assembly %in% rnb.get.assemblies())) {
		rnb.error("unsupported assembly")
	}
	if(!(is.character(region.types) && (!any(is.na(region.types))))) {
		rnb.error("Invalid value for region.types")
	}
	if(usebigff){
		bff.finalizer <- rnb.getOption("disk.dump.bigff.finalizer")
	}

	if(is.null(file.names) && is.null(sample.sheet)) {
		## Attempt to locate sample annotation file
		sample.sheet <- list.files(base.dir, pattern = "[Ss]ample", full.names = TRUE)
		sample.sheet <- sample.sheet[file.info(sample.sheet)[, "isdir"] == FALSE]
		if(length(sample.sheet) > 1) {
			rnb.error(c("No unique candidate for sample annotation found in", base.dir))
		}
		if(length(sample.sheet) == 0 || is.na(sample.sheet)) {
			rnb.error(c("No candidate for sample information found in", base.dir))
		}
	}

	if(!is.null(sample.sheet)){
#		if(verbose){
#			rnb.status(c("Supplied a sample annotation, parsing ..."))
#		}
#
		if(is.character(sample.sheet) && (!is.matrix(sample.sheet))) {
			if(length(sample.sheet) != 1) {
				rnb.error("invalid value for sample.sheet")
			}
			sample.sheet <- read.sample.annotation(sample.sheet, sep=sep.samples)
		} else if(is.matrix(sample.sheet)) {
			sample.sheet <- as.data.frame(sample.sheet, check.names = FALSE, stringsAsFactors = FALSE)
		}
		if(!is.data.frame(sample.sheet)) {
			rnb.error("invalid value for sample.sheet")
		}

		if(!is.null(rnb.getOption("identifiers.column")))
			sample.names<-as.character(sample.sheet[,rnb.getOption("identifiers.column")]) else
			sample.names<-as.character(sample.sheet[,1])
	}else{
		sample.names<-sub("\\.bed", "", file.names)
	}

	if(is.null(file.names)){

		if(is.na(file.names.col)){
			bed.col<-find.bed.column(sample.sheet)
			file.names.col <- bed.col[[1]]
			file.names<-sample.sheet[,bed.col[[1]]]
		}else if(is.integer(file.names.col) && file.names.col>0 && file.names.col<=ncol(sample.sheet)){
			file.names<-sample.sheet[,file.names.col]
		}else if(is.character(file.names.col) && length(file.names.col)==1 && file.names.col %in% colnames(sample.sheet)){
			file.names<-sample.sheet[,file.names.col]
		}else{
			file.names.col<-grep("BED|bed", colnames(sample.sheet))
			file.names<-sample.sheet[,file.names.col[1]]
		}
		file.names<-as.character(file.names)
		if(length(file.names)!=nrow(sample.sheet)){
			rnb.error("Could not find a sample annotation column with valid BED file names. Exiting...")
		}
		### manage the multiple file names

		fn.lists<-strsplit(file.names, split=";")
		if(any(sapply(fn.lists, length)>1)){
			multiple.files<-TRUE
			n.files<-sapply(fn.lists, length)
			new.sample.sheet<-do.call("rbind",
					lapply(1:nrow(sample.sheet), function(index)
								sample.sheet[rep(index, n.files[index]),]))

			if(merge.bed.files){
				new.sample.sheet$AutomaticUniqueSampleID<-do.call("c", lapply(1:length(n.files),function(index) rep(index, n.files[index])))
			}
			file.names<-unlist(fn.lists)
			sample.names<-sub("\\.bed", "", file.names)
			new.sample.sheet[,file.names.col]<-file.names
			sample.sheet<-new.sample.sheet
		}else{
			multiple.files<-FALSE
		}

		if(!is.null(base.dir)){
			present.files <- list.files(base.dir, pattern="*", full.names=FALSE)
			if(length(setdiff(file.names, present.files))){
				rnb.error(c("These BED files", setdiff(file.names, present.files),"are missing in", base.dir))
			}
			file.names<-file.path(base.dir, file.names)
		}else{
			existing.files<-sapply(file.names, file.exists )
			if(!all(existing.files)){
				rnb.error(c("Theses BED files do not exist:", file.names[!existing.files]))
			}
		}
	}

	#skip.lines <- 1
	data.matrices <- lapply(file.names, read.single.bed, context="cg",
			...,
#			chr.col=1L,
#			start.col=2L,
#			end.col=NA,
#			c.col=4L,
#			t.col=5L,
#			strand.col=3L,
#			mean.meth.col=NA,
#			coverage.col=NA,
#			coord.shift=0L,
#			nrows=10000L,
			skip=skip.lines, ffread=useff)

	if(verbose){
		rnb.status(sprintf("Read %d BED files", length(data.matrices)))
	}

	found.chroms<-character()
	found.strands<-character()
	site.ints<-NULL
	site.indices<-list()

	chr.offset<-100e9
	strand.offset<-1e9

	for(dmi in 1:length(data.matrices)){
		if(useff){
			open(data.matrices[[dmi]])
		}
		chrs.dm <- as.character(unique(data.matrices[[dmi]][,1L]))
		found.chroms <- c(found.chroms, unique(chrs.dm[!(chrs.dm %in% found.chroms)]))
		chr.match <- match(data.matrices[[dmi]][,1L], found.chroms)

		strands.dm <- as.character(unique(data.matrices[[dmi]][,3L]))
		found.strands <- c(found.strands, strands.dm[!(strands.dm %in% found.strands)])
		strand.match <- match(data.matrices[[dmi]][,3L], found.strands)
		site.ints.dm <- chr.match*chr.offset + strand.match*strand.offset + data.matrices[[dmi]][,2L]
		if(is.null(site.ints)){
			site.ints<-site.ints.dm
			site.indices[[dmi]] <- 1:nrow(data.matrices[[dmi]])
			next
		}else{
			site.ints<-c(site.ints, unique(site.ints.dm[!(site.ints.dm %in% site.ints)]))
			site.indices[[dmi]] <- match(site.ints.dm, site.ints)
		}
		if(useff){
			close(data.matrices[[dmi]])
		}
	}
	if (verbose) rnb.status(c("Matched chromosomes and strands to annotation"))

	dm.subsample<-list()
	if(useff && (prod(length(site.ints), length(data.matrices))>.Machine$integer.max) && !usebigff){
		sites.allowed <- as.integer(.Machine$integer.max/length(data.matrices))
		sample.site.inds <- sort(sample.int(length(site.ints),sites.allowed))
		msg <- c("Full dataset is too large to be supported by ff. --> downsampling to",sites.allowed,"( of",length(site.ints),") sites")
		rnb.warning(msg)
		all.sites <- site.ints[sample.site.inds]
		for(dmi in 1:length(data.matrices)){
			if(useff){
				open(data.matrices[[dmi]])
			}
			site.ints.dm <- site.ints[site.indices[[dmi]]]
			new.sites <- match(site.ints.dm, all.sites)
			dm.subsample[[dmi]] <- which(!is.na(new.sites))
			site.indices[[dmi]] <- new.sites[!is.na(new.sites)]
			if(useff){
				close(data.matrices[[dmi]])
			}
		}
	}else{
		all.sites<-site.ints
		for(dmi in 1:length(data.matrices)){
			dm.subsample[[dmi]]<-1:dim(data.matrices[[dmi]])[1]
		}
	}

	covg.present <- sapply(1:length(data.matrices), function(indx){
		if(useff){
			open(data.matrices[[indx]])
		}
		present<-dim(data.matrices[[indx]])[2]==5L
		if(useff){
			close(data.matrices[[indx]])
		}
		return(present)
	})

	if(!all(covg.present)){
		rnb.warning(c("Coverage information is not present for the following BED files: ", paste(file.names[which(!covg.present)], sep=", ")))
		rnb.warning(c("Discarded coverage information"))
	}
	if (verbose) rnb.status(c("Checked for the presence of sites and coverage"))

	sites <- matrix(nrow=length(all.sites), ncol=3, dimnames=list(NULL, c("chr", "coord", "strand")))
	if(!useff){
		meth <- matrix(nrow=length(all.sites), ncol=length(data.matrices), dimnames=list(NULL, sample.names))
		if(all(covg.present)){
			covg <- matrix(nrow=length(all.sites), ncol=length(data.matrices), dimnames=list(NULL, sample.names))
		}else{
			covg <- NULL
		}
	}else{
		#sites<-ff(NA_integer_, dim=c(length(all.sites), 3), dimnames=list(NULL, c("chr", "coord", "strand")))
		if (usebigff){
			meth <- BigFfMat(row.n=length(all.sites), col.n=length(data.matrices), row.names=NULL, col.names=sample.names, finalizer=bff.finalizer)
		} else {
			meth <- ff(NA, dim=c(length(all.sites), length(data.matrices)), dimnames=list(NULL, sample.names), vmode="double")
		}
		if(all(covg.present)){
			if (usebigff){
				covg <- BigFfMat(row.n=length(all.sites), col.n=length(data.matrices), row.names=NULL, col.names=sample.names, na.prototype=as.integer(NA), finalizer=bff.finalizer)
			} else {
				covg <- ff(NA_integer_, dim=c(length(all.sites), length(data.matrices)), dimnames=list(NULL, sample.names))
			}
		}else{
			covg <- NULL
		}
	}
	if (verbose) rnb.status(c("Initialized meth/covg matrices"))

	## Detect the scale of the mean methylation data
	## A very naive method to guess whether the methylation is  
	## given as a fraction of 1, percentage or on 0 to 1000 scale
	nSites <- nrow(data.matrices[[1]])
	test.set  <- data.matrices[[1]][,4L]
	if (nSites > 10000){
		sampleInds <- sample.int(nSites, 10000)
		test.set <- test.set[sampleInds]
	}
	perc.meth <- sum(test.set[!is.na(test.set)]>10.0)>5
	prom.meth <- sum(test.set[!is.na(test.set)]>110.0)>5
	
	## A workaround for chromosome names, not starting with "chr"
	chroms<-names(rnb.get.chromosomes(assembly=assembly))
	if(!any(grepl("^chr", found.chroms))){
		chroms<-gsub("chr", "", chroms)
	}
	dump<-sapply(1:length(data.matrices), function(indx){
				if(useff){
					open(data.matrices[[indx]])
				}
				# if (verbose) rnb.status(c("Reading data matrix", indx))
				dm<-data.matrices[[indx]]
				#dm.rows2<-intersect(all.sites, rownames(dm))
				#dm.rows<-match(all.sites, rownames(dm))
				#dm<-dm[dm.rows[!is.na(dm.rows)],]
				#rows<-which(all.sites %in% rownames(dm))
				rows<-site.indices[[indx]]
				rows.dm<-dm.subsample[[indx]]
				sites[rows,1] <<- as.integer(match(dm[rows.dm,1L], chroms))
				sites[rows,3] <<- as.integer(factor(dm[rows.dm,3L], levels=c("+","-","*")))
				sites[rows,2] <<- as.integer(dm[rows.dm,2L]) + c(pos.coord.shift, 0L, 0L)[sites[rows,3]]
				if(prom.meth){
					meth[rows,indx] <<- as.numeric(dm[rows.dm,4L])/1000.0
				}else if(perc.meth){
					meth[rows,indx] <<- as.numeric(dm[rows.dm,4L])/100.0
				}else{
					meth[rows,indx] <<- as.numeric(dm[rows.dm,4L])
				}
				if(!is.null(covg)){
					covg[rows,indx] <<- as.integer(dm[rows.dm,5L])
				}
				if(useff){
					delete(dm)
				}
				return(NULL)		
			})
	rm(dump)
	#clean the memory of big objects
	rm(data.matrices)
	rm(site.indices)
	rm(site.ints)
	rm(dm.subsample)
	rnb.cleanMem()

	if(verbose){
		rnb.status(sprintf("Combined a data matrix with %d sites and %d samples", dim(sites)[1], dim(meth)[2]))
	}

	if(verbose){
		rnb.status("Processed all BED files")
	}

	if(is.null(sample.sheet)){
		sample.sheet<-data.frame(Sample_Label=sample.names, rownames=sample.names)
	}

	rnb.cleanMem()

	if(verbose){
		rnb.logger.start("Creating RnBiseqSet object")
	}

	object<-RnBiseqSet(
			pheno=sample.sheet,
			sites=sites,
			meth=meth,
			covg=covg,
			assembly=assembly,
			target="CpG",
			useff=useff,
			usebigff=usebigff,
			verbose=verbose
			)

	if(verbose){
		rnb.logger.completed()
	}
	if(verbose){
		rnb.logger.completed()
	}

	return(object)
}

########################################################################################################################
#' read.single.bed
#'
#' reads a BED file with methylation information
#'
#' @param file				the input BED file
#' @param chr.col			chromosome column index
#' @param start.col		start column index
#' @param end.col			end column index
#' @param strand.col		strand column index
#' @param mean.meth.col	mean methylation column index
#' @param c.col			converted C counts column index
#' @param t.col			unconverted C counts column index
#' @param coverage.col		column with coverage information
#' @param is.epp.style		Flag for custom Broad Epigenome Pipeline (EPP) bed style (columns \code{"chrom"}, \code{"start"},
#'                          \code{"end"}, \code{"methylated_count/total_count"}, \code{"meth_score_scaled_0_1000"} and
#'                          \code{"strand"} in this order). Setting this to \code{TRUE} overwrites all other parameters except
#'                          \code{file}, and also neglects \code{...}.
#' @param coord.shift 		An integer specifying the coordinate adjustment applied to the start and end coordinates.
#' @param ffread			Use \code{ff} package functionality
#' @param context			prefix for the output rownames
#' @param ...				further arguments to \code{read.table} or \code{read.table.ffdf}
#'
#' @details Missing columns should be assigned with \code{NA}. In case \code{mean.meth.col} is absent at least \code{coverage.col}
#' 			and one of \code{c.col} or \code{t.col} should be specified.
#'
#' @return a \code{data.frame} or \code{ff.data.frame} object with DNA methylation and coverage information. The row names are formed by the following convension:
#'  		\code{context\\.read.delim(file,...)[,chr.col]\\.read.delim(file,...)[,start.col]\\.read.delim(file,...)[,strand.col]}.
#'
#' @author Pavlo Lutsik
#' @export
read.single.bed<-function(file,
		chr.col=1L,
		start.col=2L,
		end.col=3L,
		strand.col=6L,
		mean.meth.col=7L,
		coverage.col=8L,
		c.col=NA,
		t.col=NA,
		is.epp.style=FALSE,
		coord.shift=0L,
		ffread=FALSE,
		context="cg",
		...){

	if(all(is.na(mean.meth.col), any(is.na(c(c.col, t.col)))) && !is.epp.style){
		stop("No methylation information columns supplied")
	}

	if((is.null(mean.meth.col) || is.na(mean.meth.col)) && all(is.na(coverage.col), any(is.na(c(c.col, t.col)))) && !is.epp.style){
		stop("If the mean methylation column is absent,
			either coverage column and one of C- or T-count column, or both C- and T-count columns should be specified.")
	}

	if(any(!is.integer(c(chr.col, start.col, coverage.col, coord.shift, c(end.col, mean.meth.col, c.col,t.col)[!is.na(c(end.col, mean.meth.col, c.col,t.col))])))){
		stop("Invalid parameter values supplied, column indices should be integer")
	}

	if(min(na.omit(c(chr.col, start.col, end.col, strand.col, mean.meth.col, coverage.col, c.col, t.col)))<1){
		stop("Invalid parameter values supplied, column indices should not be negative or zero")
	}

	if(!is.character(file) || length(file)!=1)
		stop("Invalid file name")

	rnb.info(c("Reading BED file:", file))

	## read top of the file to determine the column classes
	if(ffread){
		mc<-list(...)
		meth.tab.init <- read.table(file, skip=mc$skip, nrows=1000, header=FALSE, sep="\t", stringsAsFactors=FALSE)
		columnClasses <- get.bed.column.classes(meth.tab.init,
				chr.col=chr.col,
				start.col=start.col,
				end.col=end.col,
				strand.col=strand.col,
				mean.meth.col=mean.meth.col,
				coverage.col=coverage.col,
				c.col=c.col,
				t.col=t.col,
				useff=TRUE)
	}

	if(is.epp.style){

		if(!ffread){
			meth.tab <- read.table(file,...,header=FALSE, sep="\t",stringsAsFactors=FALSE)
			mm <- strsplit(meth.tab[,4],split="/")
			mm <- t(sapply(mm,as.numeric))
			data.set <- data.frame(chrom=meth.tab[,1],start=meth.tab[,2]+1,strand=meth.tab[,6],meth=mm[,1]/mm[,2]*100,cov=mm[,2]) #+1 to account for 0-based coordinates
		}else{
			csf<-getOption("stringsAsFactors")
			options(stringsAsFactors=TRUE)
			data.set<-read.delim.ffdf(x=NULL, file=file, next.rows=10000L, ..., colClasses=columnClasses, header=FALSE, sep="\t")
			options(stringsAsFactors=csf)
			mm <- strsplit(gsub("\\'", "", data.set[,4]),split="/")
			mm <- t(sapply(mm,as.numeric))
			data.set<-data.set[c(1,2,6)]
			colnames(data.set)<-c("chrom", "start", "strand")
			data.set$start<-ff(data.set[,2]+1)
			data.set$meth<-ff(mm[,1]/mm[,2]*100)
			data.set$cov<-ff(mm[,2])
			rm(mm);rnb.cleanMem()
		}
		#rownames(data.set)<-paste(rep(context, dim(data.set)[1]), data.set[,1], data.set[,2], data.set[,3], sep=".")
		if(ffread){
			close(data.set)
		}
		return(data.set)
	} else {

		if(!ffread){
			data.set<-read.delim(file, ..., stringsAsFactors=FALSE, header=FALSE)
		}else{
			csf<-getOption("stringsAsFactors")
			options(stringsAsFactors=TRUE)
			data.set<-read.delim.ffdf(x=NULL,
					file=file, next.rows=10000L, ..., colClasses=columnClasses, header=FALSE)
			options(stringsAsFactors=csf)
		}

		#checking the validity

		if(max(chr.col, start.col, end.col, strand.col, mean.meth.col, coverage.col, c.col, t.col, na.rm=TRUE)>ncol(data.set)){
			stop(sprintf("The loaded bed file %s has less columns than expected, check the correctness of column assignment (or the BED style option)", file))
		}

		#TODO: strand information
		if(is.na(strand.col)){
			if(!ffread){
				data.set$strand <- factor(rep("*",nrow(data.set)))
			}else{
				data.set$strand <- as.ff(factor(rep("*",nrow(data.set))))
			}
			strand.col <- which(colnames(data.set)=="strand")
		} # else {
		# 	possible.strands <- unique(data.set[,strand.col])
		# 	valid.strand.info <- all(possible.strands %in% c("+","-"))
		# 	if(!valid.strand.info){
		# 		stop("Only + and - are allowed in the strand column")
		# 	}
		# }
		data.set[,strand.col]<-sub("\\.", "*", data.set[,strand.col])

		if(coord.shift!=0L){
			data.set[,start.col] <- data.set[,start.col] + coord.shift
			if(!is.na(end.col)){
				data.set[,end.col] <- data.set[,end.col] + coord.shift
			}
		}
#		rn<-paste(rep(context, dim(data.set)[1]), data.set[,chr.col], data.set[,start.col], data.set[,strand.col], sep=".")
#		if(!ffread){
#			rownames(data.set)<-rn
#		}else{
#			row.names(data.set)<-rn
#		}
#		rm(rn)
		rnb.cleanMem()

		if(!is.null(mean.meth.col) && !is.na(mean.meth.col)){

			columns<-c(chr.col, start.col, strand.col, mean.meth.col)
			if(!is.na(coverage.col)){
				columns<-c(columns, coverage.col)
			}
			if(!ffread){
				return(data.set[,columns])
			}else{
				res<-data.set[columns]
				close(res); close(data.set)
				return(res)
			}

		}else if((is.na(coverage.col) && all(!is.na(c(c.col,t.col)))) || (!is.na(coverage.col) && any(!is.na(c(c.col, t.col))))){

			if(is.na(coverage.col)){
				cvg <- data.set[,t.col]+data.set[,c.col]
			}else{
				cvg <- data.set[,coverage.col]
			}
			if(!is.na(c.col)){
				if(!ffread){
					return(cbind(data.set[,c(chr.col,start.col,strand.col)],data.set[,c.col]/cvg*100, cvg))
				}else{
					res<-data.set[c(chr.col,start.col,strand.col)]
					res$mean.meth<-ff(data.set[,c.col]/cvg*100)
					res$cvg<-ff(cvg)
					close(res); close(data.set)
					return(res)
				}
			}else{
				if(!ffread){
					return(cbind(data.set[,c(chr.col,start.col,strand.col)],(1-data.set[,t.col]/cvg)*100, cvg))
				}else{
					res<-data.set[c(chr.col,start.col,strand.col)]
					res$mean.meth<-ff((1-data.set[,t.col]/cvg)*100)
					res$cvg<-ff(cvg)
					close(res); close(data.set)
					return(res)
				}
			}
		}else{
			stop("Could not process the BED file: no column with mean methylation specified, and no way to calculate it")
		}
	}
}

########################################################################################################################
##
##	find.bed.column
##
##	Automatically parses the provided bisulfite sequencing annotation file
##	and tries to guess the column with file names
##
##	@author Pavlo Lutsik
##
find.bed.column<-function(annotation,
		verbose=TRUE){


	if(verbose){
		rnb.logger.start("Automatically parsing the provided sample annotation file")
	}

	# find a column with file names

	classes<-sapply(annotation, class)

	potential.fnames<-which(classes%in%c("character","factor"))

	candidate.fname<-sapply(potential.fnames, function(cix){
				extensions<-grepl("\\.bed|\\.BED|\\.cov|\\.COV",annotation[,cix])
				all(extensions)
			})

	if(!any(candidate.fname)){
		msg<-"Could not identify a column with file names in the supplied sample annotations file"
		if(verbose){
			rnb.error(msg)
		}
	}else if(length(which(candidate.fname))==1){
		msg<-sprintf("Potential file names found in column %s of the supplied annotation table",
				potential.fnames[which(candidate.fname)])

		if(verbose){
			rnb.status(msg)
		}
	}else{
		msg<-paste("Multiple columns containing file names detected (", potential.fnames[which(candidate.fname)],
				"). Please disable automatic parsing and specify the column containing the file names explicitly")
		if(verbose){
			rnb.error(msg)
		}
	}

	absolute<-sapply(potential.fnames[candidate.fname], function(cix){
				paths<-sapply(annotation[,cix], grepl, pattern="/|\\\\" )
				all(paths)
			})

	if(verbose){
		rnb.logger.completed()
	}

	return(list(potential.fnames[candidate.fname], absolute))
}
########################################################################################################################
##
##	get.bed.column.classes
##
##	Tries to infer classes of the BED columns based on several top lines of the file
##
##	@author Pavlo Lutsik
##
get.bed.column.classes<-function(bed.top,
		chr.col=1L,
		start.col=2L,
		end.col=3L,
		strand.col=6L,
		mean.meth.col=7L,
		coverage.col=8L,
		c.col=NA,
		t.col=NA,
		useff=FALSE){

	classes<-sapply(bed.top, class)
	if(useff){
		classes[grep("logical", classes)]<-"numeric"
		classes[grep("character", classes)]<-"factor"
		classes[grep("integer", classes)]<-"numeric"
	}
	if(useff){
		classes[chr.col]<-"factor"
		classes[strand.col]<-"factor"
	}else{
		classes[chr.col]<-"character"
		classes[strand.col]<-"character"
	}
	classes[start.col]<-"integer"
	if(!is.na(end.col)){
		classes[end.col]<-"integer"
	}
	classes[coverage.col]<-"integer"
	if(!is.na(mean.meth.col)){
		classes[mean.meth.col]<-"numeric"
	}
	if(useff){
		classes[chr.col]<-"factor"
	}else{
		classes[chr.col]<-"character"
	}
	if(!is.na(c.col)){
		classes[c.col]<-"integer"
	}
	if(!is.na(t.col)){
		classes[t.col]<-"integer"
	}
	names(classes)<-NULL
	return(classes)
}

########################################################################################################################

#' rnb.detect.infinium.platform
#'
#' Attempts to infer the version of the Infinium platform based on the names and/or sizes of the IDAT files.
#'
#' @param idat.fnames Full names of the IDAT files comprising the dataset of interest.
#' @return One of \code{"probes27"}, \code{"probes450"} or \code{"probes450"}.
#
#' @author Pavlo Lutsik
#' @noRd
rnb.detect.infinium.platform <- function(idat.fnames){
  
  platform <- rnb.getOption("import.idat.platform")
  if(platform=="auto"){
    
  	inf27k.idats.present <- any(grepl("_[ABCDEFGHIJKL]_", idat.fnames))
  	inf450kEPIC.idats.present <- any(grepl("_R0[1-6]C0[1-2]_", idat.fnames))
  
  	if (inf27k.idats.present) {
  		if (inf450kEPIC.idats.present) {
  			rnb.error("Undefined platform; detected HumanMethylation27 and HumanMethylation450")
  		}
  		return("probes27")
  	}
  	if (inf450kEPIC.idats.present) {
  		
  		file.sizes <- as.numeric(na.omit(file.info(idat.fnames)[, "size"]))
  		if (length(file.sizes) == 0) {
  			rnb.error("Undefined platform; cannot read the specified IDAT files")
  		}
        if (all(file.sizes>14000000)) {
  			return("probesEPICv2")
  		}
  		if (all(file.sizes>10000000)) {
  			return("probesEPIC")
  		}
        if (all(file.sizes<5000000)) {
            return("probesMMBC")
        }
  		if (all(file.sizes<10000000)) {
  			return("probes450")
  		}
        
  		rnb.error("Undefined platform; detected several different platforms")
  	}
  	rnb.error("Undefined platform; unexpected or missing IDAT files")
  }else{
    return(platform)
  }
}

########################################################################################################################

########################################################################################################################
## RnBSet-class.R
## created: 2012-04-06
## creator: Pavlo Lutsik
## ---------------------------------------------------------------------------------------------------------------------
## RnBSet class definition.
########################################################################################################################
## GLOBALS

RNBSET.SLOTNAMES<-c("meth.sites", "covg.sites")

##
## ---------------------------------------------------------------------------------------------------------------------
## CLASS DEFINITIONS
## ---------------------------------------------------------------------------------------------------------------------
#' @include bigFf.R
setOldClass(c("ff_matrix"))
setClassUnion("matrixOrff", c("matrix", "ff_matrix"))
setClassUnion("matrixOrffOrBigFfMat", c("matrix", "ff_matrix", "BigFfMat"))
setClassUnion("matrixOrffOrNULL", c("matrix", "ff_matrix", "NULL"))
setClassUnion("matrixOrffOrBigFfMatOrNULL", c("matrix", "ff_matrix", "BigFfMat", "NULL"))
setClassUnion("listOrNULL", c("list", "NULL"))
setClassUnion("characterOrNULL", c("character", "NULL"))

#' RnBSet Class
#'
#' Basic class for storing DNA methylation and experimental quality information
#'
#' @details
#' It is a virtual class and objects of type \code{RnBSet} should not be instantiated. Instead, the child classes are
#' used: \code{\linkS4class{RnBeadRawSet}} and \code{\linkS4class{RnBeadSet}} for Infinium HumanMethylation and
#' \code{\linkS4class{RnBiseqSet}} for bisulfite sequencing data
#'
#' @section Slots:
#' \describe{
#'   \item{\code{pheno}}{Sample annotations (phenotypic and processing data) in the form of a \code{data.frame}.}
#'   \item{\code{sites}}{A \code{matrix} object storing the identifiers of the methylation sites for which the
#' 		methylation information is present}
#'   \item{\code{meth.sites}}{\code{matrix} of methylation values. Every row corresponds to a methylation site,
#' 		and every column - to a sample.}
#'   \item{\code{covg.sites}}{\code{matrix} of coverage values. Every row corresponds to a methylation site,
#' 		and every column - to a sample.}
#'   \item{\code{regions}}{\code{list} of all identifiers of methylation sites for which methylation information
#' 		is available.}
#'   \item{\code{meth.regions}}{\code{list} of methylation \code{matrix} objects, one per available region type. Every row in a
#' 		matrix corresponds to a methylation site, and every column - to a sample.}
#'   \item{\code{covg.regions}}{\code{list} of coverage \code{matrix} objects, one per available region type.
#' 		Every row corresponds to a region, and every column - to a sample.}
#' 	 \item{\code{status}}{\code{list} with meta-information about the object.}
#' 	 \item{\code{assembly}}{\code{character} vector of length one, specifying the genome assembly which the object is linked to, e.g. "hg19".}
#'   \item{\code{target}}{\code{character} vector of length one, specifying the feature class:
#' 		\code{"CpG"} for sequencing data, \code{"probes450"} and \code{"probes27"} for
#' 		HumanMethylation450 and HumanMethylation27 microarrays respectively.}
#'   \item{\code{inferred.covariates}}{\code{list} with covariate information.
#' 		Can contain elements \code{"sva"} and \code{"cell.types"}.}
#' 	 \item{\code{version}}{Package version in which the dataset was created.}
#' 	 \item{\code{imputed}}{Flag indicating if methylation matrix has been imputed.}
#' }
#'
#' @section Methods and Functions:
#' \describe{
#'   \item{\code{\link[=pheno,RnBSet-method]{pheno}}}{Gets the phenotypic and processing data of the dataset.}
#'   \item{\code{\link[=samples,RnBSet-method]{samples}}}{Gets the identifiers of all samples in the dataset.}
#'   \item{\code{\link[=summarized.regions,RnBSet-method]{summarized.regions}}}{Gets the genomic annotations for
#'   which methylation data is present.}
#' 	 \item{\code{\link[=meth,RnBSet-method]{meth}}}{Gets a \code{matrix} of methylation values in the dataset.}
#' 	 \item{\code{\link[=mval,RnBSet-method]{mval}}}{Gets a \code{matrix} of M values in the dataset.}
#'   \item{\code{\link[=covg,RnBSet-method]{covg}}}{Gets the \code{matrix} of coverage values of the dataset.}
#'   \item{\code{\link[=remove.sites,RnBSet-method]{remove.sites}}}{Removes sites from the dataset.}
#'   \item{\code{\link[=remove.samples,RnBSet-method]{remove.samples}}}{Removes samples from the dataset.}
#'   \item{\code{\link[=addPheno,RnBSet-method]{addPheno,RnBSet-method}}}{Add sample annotation to the dataset.}
#'   \item{\code{\link[BiocGenerics]{combine}}}{Combines two datasets.}
#'   \item{\code{\link{regionMapping,RnBSet-method}}}{Retrieve the sites mapping to a given region type}
#'   \item{\code{\link[=rnb.sample.summary.table,RnBSet-method]{rnb.sample.summary.table}}}{Creates a sample summary table from an RnBSet object.}
#'   \item{\code{\link{isImputed,RnBSet-method}}}{Getter for the imputation slot.}
#' }
#'
#' @name RnBSet-class
#' @rdname RnBSet-class
#' @author Pavlo Lutsik
#' @exportClass RnBSet
setClass("RnBSet",
		representation(pheno="data.frame",
				sites="matrix",
				meth.sites="matrixOrffOrBigFfMat",
				covg.sites="matrixOrffOrBigFfMatOrNULL",
				regions="list",
				meth.regions="list",
				covg.regions="listOrNULL",
				status="listOrNULL",
				assembly="character",
				target="characterOrNULL",
				inferred.covariates="list",
				version="characterOrNULL",
		    imputed="logical"),
		prototype(pheno=data.frame(),
				sites=matrix(nrow=0, ncol=0),
				meth.sites=matrix(nrow=0, ncol=0),
				covg.sites=NULL,
				regions=list(),
				meth.regions=list(),
				covg.regions=NULL,
				status=NULL,
				assembly="hg19",
				target=NULL,
				inferred.covariates=list(),
				version=as.character(packageVersion("RnBeads")),
		    imputed=FALSE),
		contains = "VIRTUAL",
		package = "RnBeads")

## ---------------------------------------------------------------------------------------------------------------------
## DUMMY CONSTRUCTOR
## ---------------------------------------------------------------------------------------------------------------------
#
#setMethod("initialize", "RnBSet",
#		function(pheno=data.frame(),
#				sites=matrix(),
#				meth.sites=matrix(),
#				covg.sites=NULL,
#				regions=list(),
#				meth.regions=list(),
#				covg.regions=NULL,
#				status=NULL,
#				assembly="hg19",
#				target=NULL,
#				inferred.covariates=list()
#				){
#					.Object@pheno<-pheno
#					.Object@sites<-sites
#					.Object@meth.sites<-betas
#					.Object@covg.sites<-covg.sites
#
#					.Object@status<-status
#
#					.Object@target<-target
#
#
#				})

## ---------------------------------------------------------------------------------------------------------------------
## ACCESSORS
## ---------------------------------------------------------------------------------------------------------------------

if (!isGeneric("pheno")) setGeneric("pheno", function(object) standardGeneric("pheno"))

#' pheno-methods
#'
#' Extracts sample phenotype and/or processing information.
#'
#' @param object Dataset of interest.
#' @return Sample annotation information available for the dataset in the form of a \code{data.frame}.
#'
#' @rdname pheno-methods
#' @docType methods
#' @aliases pheno
#' @aliases pheno,RnBSet-method
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' pheno(rnb.set.example)
#' }
setMethod("pheno", signature(object="RnBSet"), function(object) object@pheno)

########################################################################################################################

if (!isGeneric("samples")) {
	setGeneric("samples", function(object) standardGeneric("samples"))
}

#' samples-methods
#'
#' Extracts sample identifiers
#'
#' @param object Dataset of interest.
#'
#' @details The column of the sample annotation table which contains identifiers is globally controlled via the
#'  \code{"identifiers.column"} option. In case the latter is \code{NULL} column names of the matrix returned
#' by the \code{meth} method are treated as sample identifiers. In case the latter are also missing, a \code{character}
#' vector with sample numbers is returned.
#'
#' @return \code{character} vector of sample identifiers.
#'
#' @rdname samples-methods
#' @docType methods
#' @aliases samples
#' @aliases samples,RnBSet-method
#' @aliases samples,RnBeadClustering-method
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' samples(rnb.set.example)
#' }
setMethod("samples", signature(object="RnBSet"),
	function(object) {
		pheno.table <- pheno(object)
		id.column <- rnb.getOption("identifiers.column")
		ids <- NULL
		if (!(is.null(pheno.table) || is.null(id.column))) {
			if (is.character(id.column)) {
				if (id.column %in% colnames(pheno.table)) {
					ids <- pheno.table[, id.column]
				}
			} else if (1L <= id.column && id.column <= ncol(pheno.table)) {
				ids <- pheno.table[, id.column]
			}

			if (is.null(ids) || any(is.na(ids)) || anyDuplicated(ids) != 0) {
				rnb.warning("The supplied identifiers column is not found or is not suitable")
                ids <- as.character(1:nrow(object@pheno))
			}
			ids <- as.character(ids)
		} else if (!is.null(colnames(object@meth.sites))) {
			ids <- colnames(object@meth.sites)
		} else {
			ids <- as.character(1:nrow(object@pheno))
		}
		ids
	}
)

########################################################################################################################

if(!isGeneric("sites")) setGeneric("sites",
			function(object) standardGeneric("sites"))

#' sites-methods
#'
#' Methylation sites object information for which is present in the \code{RnBSet} object.
#'
#' @param object Dataset of interest.
#'
#' @return A matrix of type \code{integer} describing the sites, information for which is
#' present in the \code{object}
#'
#' @rdname sites-methods
#' @docType methods
#' @aliases sites
#' @aliases sites,RnBSet-method
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' sites(rnb.set.example)
#' }
setMethod("sites", signature(object="RnBSet"),
		function(object){
			return(object@sites)
		})

if(!isGeneric("regions")) setGeneric("regions",
			function(object, ...) standardGeneric("regions"))
########################################################################################################################

#' regions-methods
#'
#' Methylation regions, information for which is present in the \code{RnBSet} object.
#'
#' @param object Dataset of interest.
#' @param type   Region type(s) of interest as a \code{character} vector. If this is set to \code{NULL}, all region
#'               types summarized in the object are returned.
#' @return Methylation site and region assignment. If \code{type} is singleton, a \code{matrix} is returned. The first
#' 		   column corresponds to the methylation context index. The second column is the index of the chromosome in
#'         the genome, and the third is the index of the region in the \code{GRanges} object of the region type
#'         annotation. When \code{length(type)>1}, a list of such matrices is returned for each element of \code{type}.
#' 		   If \code{type} is \code{NULL}, matrices for all summarized region types are returned.
#'
#' @note
#' Methylation context index is an integer number denoting the sequence context of the cytosine of interest. Index
#' \code{1} corresponds to \code{CpG}, the only supported index in bisulfite sequencing datasets.
#'
#' @rdname regions-methods
#' @docType methods
#' @aliases regions
#' @aliases regions,RnBSet-method
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' head(regions(rnb.set.example))
#' }
#' @seealso \code{\link[=summarized.regions,RnBSet-method]{summarized.regions}} for all summarized region types in a dataset;
#'   \code{\link{rnb.get.chromosomes}} listing all supported chromosomes for a given genome assembly
#' @author Pavlo Lutsik
#' @export
setMethod("regions", signature(object="RnBSet"),
		function(object, type=NULL){
			if(!(is.character(type)))
				stop("Invalid argument type")

			if(is.null(object@regions)){
				warning("No region information present, returning NULL")
				return(NULL)
			}
			if(!is.null(type)){
				if(!all(type %in% names(object@regions)))
					stop(sprintf("No information for type %s",type))
				if(length(type==1))
				return(object@regions[[type]]) else
				return(object@regions[type])
			}else{
				return((object@regions))
			}
		})

########################################################################################################################

if (!isGeneric("summarized.regions")) {
	setGeneric("summarized.regions", function(object) standardGeneric("summarized.regions"))
}

#' summarized.regions-methods
#'
#' Gets the genomic annotations for which methylation data is present in the \code{RnBSet} object.
#'
#' @param object Methylation dataset of interest.
#'
#' @return \code{character} vector listing all genomic annotations summarized in the given dataset. If the dataset
#'         contains methylation in sites only, an empty vector is returned.
#'
#' @seealso \code{\link[=summarize.regions,RnBSet-method]{summarize.regions}} for calculating region-wise methylation in a dataset;
#'          \code{\link{rnb.set.annotation}} for adding or replacing a region annotation table
#'
#' @rdname summarized.regions-methods
#' @docType methods
#' @aliases summarized.regions
#' @aliases summarized.regions,RnBSet-method
#' @author Yassen Assenov
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' summarized.regions(rnb.set.example)
#' }
setMethod("summarized.regions", signature(object = "RnBSet"),
	function(object) {
		result <- names(object@regions)
		if (is.null(result)) {
			result <- character()
		}
		result
	}
)

########################################################################################################################

## get.dataset.matrix
##
## Extracts a specific data matrix from the given methylation dataset and sets row names if necessary.
##
## @param object     Methylation dataset as an object of class inheriting \code{RnBSet}.
## @param type       Site type (e.g. \code{"sites"} or \code{"probes450"}) for site/probe matrix, or region name for
##                   the corresponding region-based matrix.
## @param row.names  Flag indicating if row names must be generated.
## @param mm.sites   Data matrix for the site level.
## @param mm.regions List of data matrices, one per supported region type.
## @param i          indices of sites/regions to be retrieved (index or logical). retrieves all if \code{NULL} (default).
## @param j          indices of samples to be retrieved (index or logical). retrieves all if \code{NULL} (default).
## @return Requested data matrix. Note that this might be \code{NULL}.
## @author Pavlo Lutsik
get.dataset.matrix <- function(object, type, row.names, mm.sites, mm.regions, i=NULL, j=NULL) {
	if (!(is.character(type) && length(type) == 1 && (!is.na(type)))) {
		stop("invalid value for type")
	}
	if (!parameter.is.flag(row.names)) {
		stop("invalid value for row.names; expected TRUE or FALSE")
	}
	if (!is.element(class(i), c("NULL", "integer", "numeric", "logical"))) {
		stop("invalid value for i; expected NULL, index or logical")
	}
	if (!is.element(class(j), c("NULL", "integer", "numeric", "logical", "character"))) {
		stop("invalid value for j; expected NULL, index, character or logical")
	}
	if (is.character(j)){
		j <- match(j, samples(object))
		if (any(is.na(j))){
			stop("invalid sample names")
		}
	}
	if (type %in% c("sites", object@target)) {
		if (is.null(mm.sites)) {
			return(NULL)
		}
		if("ff" %in% class(mm.sites)){
			open(mm.sites)
		}
		if (is.null(i) && is.null(j)){
			result <- mm.sites[, , drop = FALSE]
		} else if(is.null(i)){
			result <- mm.sites[, j, drop = FALSE]
		} else if(is.null(j)){
			result <- mm.sites[i, , drop = FALSE]
		} else {
			result <- mm.sites[i, j, drop = FALSE]
		}
	} else if (!(type %in% names(object@regions))) {
		stop("unsupported region type")
	} else if (is.null(mm.regions[[type]])) {
		return(NULL)
	} else {
		if (is.null(i) && is.null(j)){
			result <- mm.regions[[type]][, , drop = FALSE]
		} else if(is.null(i)){
			result <- mm.regions[[type]][, j, drop = FALSE]
		} else if(is.null(j)){
			result <- mm.regions[[type]][i, , drop = FALSE]
		} else {
			result <- mm.regions[[type]][i, j, drop = FALSE]
		}
	}
	if (is.null(j)){
		colnames(result) <- samples(object)
	} else {
		colnames(result) <- samples(object)[j]
	}
	if (row.names) {
		if (is.null(i)){
			rownames(result) <- get.row.names(object, type)
		} else {
			rownames(result) <- get.row.names(object, type)[i]
		}
	} else {
		rownames(result) <- NULL
	}
	return(result)
}

########################################################################################################################

if(!isGeneric("mval")) setGeneric("mval", function(object, ...) standardGeneric("mval"))
#' mval-methods
#'
#' Extracts DNA methylation information (M values) for a specified set of genomic features.
#'
#' @param object 	dataset of interest.
#' @param type 		\code{character} singleton. If this is set to \code{"sites"} (default), DNA methylation information
#'                  for each available site is returned. Otherwise, this should be one of region types for for which
#'                  summarized DNA methylation information is computed in the given dataset.
#' @param row.names	Flag indicating of row names are to be generated in the result.
#' @param epsilon   Threshold of beta values to use when adjusting for potential M values close to +infinity or
#'                  -infinity. See \code{\link{rnb.beta2mval}} for more details.
#'
#' @return \code{matrix} with methylation M values.
#'
#' @seealso \code{\link[=meth,RnBSet-method]{meth}} for extracting methylation beta values
#' @rdname mval-methods
#' @docType methods
#' @aliases mval
#' @aliases mval,RnBSet-method
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' ## per-site M-value matrix
#' mm<-mval(rnb.set.example, row.names=TRUE)
#' head(mm)
#' ## M-values for each covered gene
#' gmm<-mval(rnb.set.example, type="gene", row.names=TRUE)
#' head(gmm)
#' }
#' @export
setMethod("mval", signature(object = "RnBSet"),
		  function(object, type = "sites", row.names = FALSE, epsilon = 0) {
		  	beta.values <- get.dataset.matrix(object, type, row.names, object@meth.sites, object@meth.regions)
		  	rnb.beta2mval(beta.values, epsilon)
		  }
)

if(!isGeneric("meth")) setGeneric("meth", function(object, ...) standardGeneric("meth"))
#' meth-methods
#'
#' Extracts DNA methylation information (beta values) for a specified set of genomic features.
#'
#' @param object 	dataset of interest.
#' @param type 		\code{character} singleton. If this is set to \code{"sites"} (default), DNA methylation information
#'                  for each available site is returned. Otherwise, this should be one of region types for for which
#'                  summarized DNA methylation information is computed in the given dataset.
#' @param row.names	flag indicating if row names are to be generated in the result.
#' @param i     	indices of sites/regions to be retrieved. By default (\code{NULL}), all will be retrieved.
#' @param j     	indices of samples to be retrieved. By default (\code{NULL}), all will be retrieved.
#'
#' @return \code{matrix} with methylation beta values.
#'
#' @seealso \code{\link[=mval,RnBSet-method]{mval}} for calculating M values
#' @rdname meth-methods
#' @docType methods
#' @aliases meth
#' @aliases meth,RnBSet-method
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' ## per-site beta-value matrix
#' mm<-meth(rnb.set.example, row.names=TRUE)
#' head(mm)
#' ## beta-values for each covered gene
#' gmm<-meth(rnb.set.example, type="gene", row.names=TRUE)
#' head(gmm)
#' }
#' @export
setMethod("meth", signature(object = "RnBSet"),
	function(object, type="sites", row.names=FALSE, i=NULL, j=NULL) {
		get.dataset.matrix(object, type, row.names, object@meth.sites, object@meth.regions, i=i, j=j)
	}
)

if(!isGeneric("hasCovg")) setGeneric("hasCovg", function(object,...) standardGeneric("hasCovg"))
#' hasCovg-methods
#'
#' Returns \code{TRUE} if the \code{RnBSet} object contains coverage information for sites or the specified region type.
#'
#' @param object 		\code{RnBSet} of interest.
#' @param type 			\code{character} singleton. If \code{sites} or a region type summarized in the object
#'
#' @return \code{TRUE} if the \code{RnBSet} object contains coverage information for sites or the specified region type. \code{FALSE} otherwise
#'
#' @rdname hasCovg-methods
#' @docType methods
#' @export
#' @aliases hasCovg
#' @aliases hasCovg,RnBSet-method
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' ## per-site beta-value matrix
#' hasCovg(rnb.set.example)
#' }
setMethod("hasCovg", signature(object="RnBSet"),
	function (object, type="sites") {
		if (!(is.character(type) && length(type) == 1 && (!is.na(type)))) {
			stop("invalid value for type")
		}
		if (type %in% c("sites", object@target)) {
			result <- !is.null(object@covg.sites)
		} else if (!(type %in% names(object@regions))) {
			stop("unsupported region type")
		} else {
			result <- !is.null(object@covg.regions[[type]])
		}
		return(result)
	}
)


if(!isGeneric("covg")) setGeneric("covg", function(object,...) standardGeneric("covg"))
#' covg-methods
#'
#' Extract coverage information from an object of \code{RnBSet} class.
#'
#' @param object 		Dataset of interest.
#' @param type 			\code{character} singleton. If \code{sites} DNA methylation information per each available
#' 						site is returned. Otherwise should be one of region types for for which the summarized
#' 						coverage information is available
#' @param row.names	    Flag indicating of row names are to be generated in the result.
#' @param i     	indices of sites/regions to be retrieved. By default (\code{NULL}), all will be retrieved.
#' @param j     	indices of samples to be retrieved. By default (\code{NULL}), all will be retrieved.
#'
#' @return coverage information available for the dataset in the form of a \code{matrix}.
#'
#' @rdname covg-methods
#' @docType methods
#' @export
#' @aliases covg
#' @aliases covg,RnBSet-method
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' ## per-site beta-value matrix
#' cvg<-covg(rnb.set.example, row.names=TRUE)
#' head(cvg)
#' }
setMethod("covg", signature(object="RnBSet"),
	function (object, type="sites", row.names=FALSE, i=NULL, j=NULL) {
		m<-get.dataset.matrix(object, type, row.names, object@covg.sites, object@covg.regions, i=i, j=j)
		m
	}
)

if(!isGeneric("geno")) setGeneric("geno", function(object, ...) standardGeneric("geno"))
#' geno-methods
#'
#' Extracts genotyping information from either the SNP (rs) or mutation (nv) probes
#'
#' @param object 	dataset of interest.
#' @param type 		\code{character} singleton. Available options are \code{NULL} (all genotyping probes), \code{'nv'} for the
#'      genotyping probes of the arrays, or \code{'rs'} for the SNP probes.
#'
#' @return \code{matrix} with genotyping information in the form of beta values. \code{0} stands for homozygous allele 1, \code{0.5}
#'      for heterozygous and \code{1} for homozygous allele 2.
#'
#' @rdname geno
#' @docType methods
#' @aliases geno
#' @aliases geno,RnBSet-method
#' @export
setMethod("geno", signature(object = "RnBSet"),
  function(object, type=NULL) {
    dat <- get.dataset.matrix(object, type='sites', row.names=TRUE, object@meth.sites, object@meth.regions, remove.nvs=FALSE)
    if(!any(row.names(dat)%in%c('nv', 'rs'))){
      rnb.error("No genotyping information for this dataset. During preprocessing, RnBeads removes these sites.")
    }
    if(!is.null(type)){
      if(!type%in%c('nv', 'rs')){
        rnb.error("Invalid value for type, has to be 'nv' or 'rs'")
      }
      dat <- dat[grepl(type, row.names(dat)),]
    }else{
      dat <- dat[grepl('nv', row.names(dat))|grepl('rs', row.names(dat)),]
    }
    return(dat)
  }
)



if(!isGeneric("nsites")) setGeneric("nsites", function(object, ...) standardGeneric("nsites"))
#' nsites-methods
#'
#' Returns the number of sites/regions for a given \code{RnBSet} object
#'
#' @param object 	\code{RnBSet} of interest.
#' @param type 		\code{character} singleton. If this is set to \code{"sites"} (default), the number of sites is returned.
#'                  Otherwise, this should be one of region types for for which the number of regions is returned.
#'
#' @return \code{integer} stating the number of sites/regions. \code{NA} if the regions have not been summarized yet.
#'
#' @seealso \code{\link[=meth,RnBSet-method]{meth}} Retrieving the matrix of methylation values
#' @rdname nsites-methods
#' @docType methods
#' @aliases nsites
#' @aliases nsites,RnBSet-method
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' nsites(rnb.set.example)
#' }
#' @export
setMethod("nsites", signature(object = "RnBSet"),
	function(object, type="sites") {
		if (!(is.character(type) && length(type) == 1 && (!is.na(type)))) {
			stop("invalid value for type")
		}
		if (type %in% c("sites", object@target)) {
			result <- nrow(object@meth.sites)
		} else if (!(type %in% names(object@regions))) {
			stop("unsupported region type")
		} else if (is.null(object@meth.regions[[type]])) {
			result <- NA
		} else {
			result <- nrow(object@meth.regions[[type]])
		}
		return(result)
	}
)

########################################################################################################################

if (!isGeneric("assembly")) {
	setGeneric("assembly", function(object) standardGeneric("assembly"))
}

#' assembly-methods
#'
#' Extracts information about assembly
#'
#' @param object Dataset of interest.
#' @return Sample annotation information available for the dataset in the form of a \code{data.frame}.
#'
#' @rdname assembly-methods
#' @docType methods
#' @aliases assembly
#' @aliases assembly,RnBSet-method
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' assembly(rnb.set.example) # "hg19"
#' }
setMethod("assembly", signature(object="RnBSet"),
		  function(object){
		  	return(object@assembly)
		  })


## ---------------------------------------------------------------------------------------------------------------------
## MODIFIERS
## ---------------------------------------------------------------------------------------------------------------------

if (!isGeneric("updateRegionSummaries")) {
	setGeneric("updateRegionSummaries", function(object) standardGeneric("updateRegionSummaries"))
}

#' updateRegionSummaries
#'
#' Updates the region information present in an RnBSet by invoking summarize.regions on all region types
#' present in the object
#'
#' @param object Dataset of interest.
#' @return Sample annotation information available for the dataset in the form of a \code{data.frame}.
#'
#' @rdname updateRegionSummaries
#' @docType methods
#' @aliases updateRegionSummaries
#' @aliases updateRegionSummaries,RnBSet-method
#' @export
setMethod("updateRegionSummaries", signature(object="RnBSet"),
		function(object){
			if (length(object@meth.regions) != 0) {
				region.types <- names(object@meth.regions)
				aggregations <- sapply(object@meth.regions, attr, "aggregation")
				for (i in 1:length(region.types)) {
					object <- summarize.regions(object, region.types[i], aggregations[i])
				}
			}
			object
		}
)

########################################################################################################################

if (!isGeneric("remove.sites")) {
	setGeneric("remove.sites", function(object, probelist, verbose = TRUE) standardGeneric("remove.sites"))
}

#' remove.sites-methods
#'
#' Removes the specified probes from the dataset.
#'
#' @param object    Dataset of interest.
#' @param probelist List of probes to be removed in the form of a \code{logical}, \code{integer} or \code{character}
#'                  vector. If this parameter is \code{logical}, it is not recycled; its length must be equal to the
#'                  number of probes in \code{object}. If it is \code{integer} or \code{character}, it must list only
#'                  probes that exist in the dataset. Specifying probe indices larger than the number of probes, or
#'                  non-existent probe identifiers results in an error.
#' @param verbose	if \code{TRUE} additional diagnostic output is generated
#'
#' @return The modified dataset.
#'
#' @seealso \code{\link[=remove.samples,RnBSet-method]{remove.samples}} for removing samples from a methylation dataset
#'
#' @rdname remove.sites-methods
#' @aliases remove.sites
#' @aliases remove.sites,RnBSet-method
#' @docType methods
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' print(rnb.set.example)
#' ## remove 100 random sites
#' s2r<-sample.int(nrow(sites(rnb.set.example)), 100)
#' rnb.set.f<-remove.sites(rnb.set.example, s2r)
#' print(rnb.set.f)
#' }
setMethod("remove.sites", signature(object = "RnBSet"),
		function(object, probelist, verbose=FALSE) {
			inds <- get.i.vector(probelist, rownames(object@sites))
			if(verbose) {
				rnb.logger.start("Removing sites")
			}
			## Delete methylation sites
			if(length(inds) != 0) {
					object@sites <- object@sites[-inds, ]
					if(!is.null(object@status) && object@status$disk.dump){
						doBigFf <- !is.null(object@status$disk.dump.bigff)
						bff.finalizer <- NULL
						if (doBigFf) doBigFf <- object@status$disk.dump.bigff
						if (doBigFf) bff.finalizer <- rnb.getOption("disk.dump.bigff.finalizer")
						nSites.new <- nrow(object@meth.sites) - length(inds)
						nSamples <- length(samples(object))
						# methylation
						newMat <- NULL
						if (doBigFf){
							newMat <- BigFfMat(row.n=nSites.new, col.n=nSamples, row.names=NULL, col.names=samples(object), finalizer=bff.finalizer)
						} else {
							newMat <- ff(NA, dim=c(nSites.new, nSamples), dimnames=list(NULL, samples(object)), vmode="double")
						}
						for (j in 1:nSamples){
							newMat[,j] <- object@meth.sites[-inds,j]
						}
						if(isTRUE(object@status$discard.ff.matrices)){
							delete(object@meth.sites)
						}
						object@meth.sites <- newMat

						# coverage
						if(!is.null(object@covg.sites)) {
							newMat <- NULL
							if (doBigFf){
								newMat <- BigFfMat(row.n=nSites.new, col.n=nSamples, row.names=NULL, col.names=samples(object), na.prototype=as.integer(NA), finalizer=bff.finalizer)
							} else {
								newMat <- ff(NA_integer_, dim=c(nSites.new, nSamples), dimnames=list(NULL, samples(object)))
							}
							for (j in 1:nSamples){
								newMat[,j] <- object@covg.sites[-inds,j]
							}
							if(isTRUE(object@status$discard.ff.matrices)){
								delete(object@covg.sites)
							}
							object@covg.sites <- newMat
						}
					} else {
						object@meth.sites <- object@meth.sites[-inds, ,drop=FALSE]
						if(!is.null(object@covg.sites)) {
							object@covg.sites <- object@covg.sites[-inds, ,drop=FALSE]
						}
					}
			}

			## Update region methylation
			if(length(object@meth.regions) != 0){
				region.types <- names(object@meth.regions)
				aggregations <- sapply(object@meth.regions, attr, "aggregation")
				for(i in 1:length(region.types)){
					if(verbose){
						rnb.status(c("summarizing regions:",region.types[i]))
					}
					object <- summarize.regions(object, region.types[i], aggregations[i])
				}
			}

			## Remove information on inferred covariates (they are likely to change when sites are removed)
			if (.hasSlot(object, "inferred.covariates")) {
				i.covariates <- setdiff(names(object@inferred.covariates), "sex")
				if (length(i.covariates) != 0) {
					object@inferred.covariates[i.covariates] <- NULL
					if(verbose){
						rnb.info("removed information on inferred covariates")
					}
				}
			}
			if(verbose){
				rnb.logger.completed()
			}
			object
		}
)

########################################################################################################################

if (!isGeneric("updateMethylationSites")) {
  setGeneric("updateMethylationSites", function(object, meth.data, verbose = TRUE) standardGeneric("updateMethylationSites"))
}

#' updateMethylationSites-methods
#'
#' Replaces the methylation info with the specified data frame.
#'
#' @param object    Dataset of interest.
#' @param meth.data This object has to be a \code{data.frame} of equal dimension than the one already contained in 
#'                  \code{object}, containing the methylation info that should be associated with the object.
#' @param verbose	if \code{TRUE} additional diagnostic output is generated
#'
#' @return The modified dataset.
#'#'
#' @rdname updateMethylationSites-methods
#' @aliases updateMethylationSites
#' @aliases updateMethylationSites,RnBSet-method
#' @docType methods
#' @export
setMethod("updateMethylationSites", signature(object = "RnBSet"),
          function(object, meth.data, verbose=FALSE) {
            if(verbose) {
              rnb.logger.start("Updating sites")
            }
            if(!is.null(object@status) && object@status$disk.dump){
              doBigFf <- !is.null(object@status$disk.dump.bigff)
              bff.finalizer <- NULL
              if (doBigFf) doBigFf <- object@status$disk.dump.bigff
              if (doBigFf) bff.finalizer <- rnb.getOption("disk.dump.bigff.finalizer")
              nSites <- nrow(object@meth.sites)
              if(nSites!=nrow(meth.data)){
                stop("Dimensions of provided and existing methylation info do not match.")
              }
              nSamples <- length(samples(object))
              if(nSites!=nrow(meth.data)||nSamples!=ncol(meth.data)){
                stop("Dimensions of provided and existing methylation info do not match.")
              }
              # methylation
              newMat <- NULL
              if (doBigFf){
                newMat <- BigFfMat(row.n=nSites, col.n=nSamples, row.names=NULL, col.names=samples(object), finalizer=bff.finalizer)
              } else {
                newMat <- ff(NA, dim=c(nSites, nSamples), dimnames=list(NULL, samples(object)), vmode="double")
              }
              for (j in 1:nSamples){
                newMat[,j] <- meth.data[,j]
              }
              if(isTRUE(object@status$discard.ff.matrices)){
                delete(object@meth.sites)
              }
              object@meth.sites <- newMat
                
            } else {
              nSites <- nrow(object@meth.sites)
              if(nSites!=nrow(meth.data)){
                stop("Dimensions of provided and existing methylation info do not match.")
              }
              nSamples <- length(samples(object))
              if(nSites!=nrow(meth.data)||nSamples!=ncol(meth.data)){
                stop("Dimensions of provided and existing methylation info do not match.")
              }
              object@meth.sites <- meth.data
            }
            if(verbose){
              logger.completed()
            }
            if(verbose){
              logger.start("Update regional methylation")
            }
            object <- updateRegionSummaries(object)
            if(verbose){
              logger.completed()
            }
            object
          }
)

########################################################################################################################

if (!isGeneric("mask.sites.meth")) {
	setGeneric("mask.sites.meth", function(object, mask, verbose=FALSE) standardGeneric("mask.sites.meth"))
}

#' mask.sites.meth-methods
#'
#' Given a logical matrix, sets corresponding entries in the methylation table to NA (masking).
#' Low memory footprint
#'
#' @param object    Dataset of interest.
#' @param mask      logical matrix indicating which sites should be masked
#' @param verbose	if \code{TRUE} additional diagnostic output is generated
#'
#' @return The modified dataset.
#'
#' @rdname mask.sites.meth-methods
#' @aliases mask.sites.meth
#' @aliases mask.sites.meth,RnBSet-method
#' @docType methods
setMethod("mask.sites.meth", signature(object = "RnBSet"),
	function(object, mask, verbose=FALSE) {
		if(!is.null(object@status) && object@status$disk.dump){
			nSamples <- length(samples(object))
			for (j in 1:nSamples){
				object@meth.sites[mask[,j],j] <- NA
			}
		} else {
			object@meth.sites[,][mask] <- NA
			if(inherits(object, "RnBeadRawSet")){
				object@M[,][mask] <- NA
				object@U[,][mask] <- NA
				if(!is.null(object@M0)){
					object@M0[,][mask] <- NA
				}
				if(!is.null(object@U0)){
					object@U0[,][mask] <- NA
				}
			}
		}
		object
	}
)

########################################################################################################################

if (!isGeneric("remove.samples")) {
	setGeneric("remove.samples", function(object, samplelist) standardGeneric("remove.samples"))
}

#' remove.samples-methods
#'
#' Removes the specified samples from the dataset.
#'
#' @param object     Dataset of interest.
#' @param samplelist List of samples to be removed in the form of a \code{logical}, \code{integer} or \code{character}
#'                   vector. If this parameter is \code{logical}, it is not recycled; its length must be equal to the
#'                   number of samples in \code{object}. If it is \code{integer} or \code{character}, it must list only
#'                   samples that exist in the dataset. Specifying sample indices larger than the number of samples, or
#'                   non-existent sample identifiers results in an error.
#' @return The modified dataset.
#'
#' @seealso \code{\link[=remove.sites,RnBSet-method]{remove.sites}} for removing sites or probes from a methylation dataset
#'
#' @rdname remove.samples-methods
#' @aliases remove.samples
#' @aliases remove.samples,RnBSet-method
#' @docType methods
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' samples(rnb.set.example)
#' ## remove 3 random samples
#' s2r<-sample.int(length(samples(rnb.set.example)), 3)
#' rnb.set.f<-remove.samples(rnb.set.example, s2r)
#' samples(rnb.set.f)
#' }
setMethod("remove.samples", signature(object = "RnBSet"),
		function(object, samplelist) {
			object.old <- object
			inds <- get.i.vector(samplelist, samples(object))
			bff.finalizer <- rnb.getOption("disk.dump.bigff.finalizer")
			if (length(inds) != 0) {
				if(object@status$disk.dump){
					doBigFf <- !is.null(object@status$disk.dump.bigff)
					if (doBigFf) doBigFf <- object@status$disk.dump.bigff

					mat <- object@meth.sites[,]
					new.matrix <- mat[,-inds, drop=FALSE]
					# delete(object@meth.sites)
					if(isTRUE(object@status$discard.ff.matrices)){
						delete(object@meth.sites)
					}
					if (doBigFf){
						object@meth.sites <- BigFfMat(new.matrix, finalizer=bff.finalizer)
					} else {
						object@meth.sites <- convert.to.ff.matrix.tmp(new.matrix)
					}
				}else{
					object@meth.sites <- object@meth.sites[,-inds, drop=FALSE]
				}

				if (!is.null(object@pheno)) {
					object@pheno <- object@pheno[-inds, ,drop=FALSE]
				}
				if (!is.null(object@covg.sites)) {
					if(object@status$disk.dump){
						doBigFf <- !is.null(object@status$disk.dump.bigff)
						if (doBigFf) doBigFf <- object@status$disk.dump.bigff

						mat <- object@covg.sites[,]
						new.matrix <- mat[,-inds, drop=FALSE]
						# delete(object@covg.sites)
						if(isTRUE(object@status$discard.ff.matrices)){
							delete(object@covg.sites)
						}
						if (doBigFf){
							object@covg.sites <- BigFfMat(new.matrix, finalizer=bff.finalizer)
						} else {
							object@covg.sites <- convert.to.ff.matrix.tmp(new.matrix)
						}
					}else{
						object@covg.sites <- object@covg.sites[,-inds, drop=FALSE]
					}
				}
				for (region in names(object@regions)) {
					if(object@status$disk.dump){
						doBigFf <- !is.null(object@status$disk.dump.bigff)
						if (doBigFf) doBigFf <- object@status$disk.dump.bigff

						mat <- object@meth.regions[[region]][,]
						meth.matrix <- mat[, -inds, drop=FALSE]
						if(isTRUE(object@status$discard.ff.matrices)){
							delete(object@meth.regions[[region]])
						}
						if (doBigFf){
							object@meth.regions[[region]] <- BigFfMat(meth.matrix, finalizer=bff.finalizer)
						} else {
							object@meth.regions[[region]] <- convert.to.ff.matrix.tmp(meth.matrix)
						}
						if(!is.null(object@covg.regions)){
							mat <- object@covg.regions[[region]][,]
							covg.matrix <- mat[, -inds, drop=FALSE]
							if(isTRUE(object@status$discard.ff.matrices)){
								delete(object@covg.regions[[region]])
							}
							if (doBigFf){
								object@covg.regions[[region]] <- BigFfMat(covg.matrix, finalizer=bff.finalizer)
							} else {
								object@covg.regions[[region]] <- convert.to.ff.matrix.tmp(covg.matrix)
							}
						}
						# delete(object@meth.regions[[region]])
						# delete(object@covg.regions[[region]])
					}else{
						object@meth.regions[[region]] <- object@meth.regions[[region]][, -inds, drop=FALSE]
						if(!is.null(object@covg.regions)){
							object@covg.regions[[region]] <- object@covg.regions[[region]][, -inds, drop=FALSE]
						}
					}

					attr(object@meth.regions[[region]], "aggregation")<-attr(object.old@meth.regions[[region]], "aggregation")
				}

				## Remove information on inferred covariates (they are likely to change when samples are removed)
				if (.hasSlot(object, "inferred.covariates")) {
					i.covariates <- setdiff(names(object@inferred.covariates), "sex")
					if (length(i.covariates) != 0) {
						## FIXME: Wouldn't it make more sense to simply take the samples out?
						object@inferred.covariates[i.covariates] <- NULL
					}
				}
			}
			object
		}
)

########################################################################################################################

if (!isGeneric("mergeSamples")) {
	setGeneric("mergeSamples", function(object, ...) standardGeneric("mergeSamples"))
}

#' mergeSamples
#'
#' Take an RnBSet object and merge methylation and phenotype information given a grouping column in the pheno table
#' coverage is combined by taking the sum of coverages
#' pheno is combined by concatenating entries from all samples
#' @param object input RnBSet object
#' @param grp.col a column name (string) of \code{pheno(rnb.set)} that contains unique identifiers for sample groups/replicates
#' 		  to be combined
#' @return the modified RnBSet object
#' @details combines phenotype information, coverage information and methylation information
#' methylation is combined by taking the average. Detection p-values are combined using Fisher's method.
#' For methylation arrays, bead counts are currently not taken into account.
#' objects of class \code{RnBeadRawSet} are automatically converted to \code{RnBeadSet}.
#' @note Requires the packages \pkg{foreach} and \pkg{doParallel}.
#'
#' @rdname mergeSamples-methods
#' @aliases mergeSamples
#' @aliases mergeSamples,RnBSet-method
#' @docType methods
#'
#' @author Fabian Mueller
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' rnb.set.example
#' rnb.set.merged <- mergeSamples(rnb.set.example,"Cell_Line")
#' rnb.set.merged
#' pheno(rnb.set.merged)
#' }
# TODOs:
# - incorporate weighted methylation average (coverage)
setMethod("mergeSamples", signature(object = "RnBSet"),
	function(object, grp.col){
		ph <- pheno(object)
		if (!is.element(grp.col,colnames(ph))){
			stop("Could not merge samples: phenotype column does not exist")
		}
		res <- object
		replicate.list <- getMergeList(object, grp.col)
		num.replicates <- sapply(replicate.list,length)
		phm <- sapply(ph, format, trim=TRUE, justify="none") #fomat to matrix, avoiding padded whitespaces
		ph.t <- t(phm)
		mf.pheno <- function(X.sub){
			sapply(1:nrow(X.sub),FUN=function(i){
				if (length(unique(X.sub[i,]))==1 && sum(is.na(X.sub[i,]))==0) {
					return(X.sub[i,1])
				} else if (all(is.na(X.sub[i,]))) {
					return(NA)
				} else {
					return(paste(X.sub[i,],collapse=";"))
				}
			})
		}
		pheno.new <- t(mergeColumns(ph.t,replicate.list,mergeFun=mf.pheno))
		pheno.new <- cbind(pheno.new,num.replicates)
		colnames(pheno.new) <- c(colnames(ph),"rnb_number_merged_samples")

		if (class(object) == "RnBiseqSet"){
			meth.site.new <- mergeColumns(meth(object,type="sites",row.names=FALSE),replicate.list)
			covg.site.new <- NULL
			if (!is.null(object@covg.sites)){
				covg.site.new <- mergeColumns(covg(object,type="sites"),replicate.list,mergeFun=function(X.sub){rowSums(X.sub,na.rm=TRUE)})
			}
			# res <- new("RnBiseqSet",
			# 		pheno=data.frame(pheno.new),
			# 		sites=object@sites,
			# 		meth.sites=meth.site.new,
			# 		covg.sites=covg.site.new,
			# 		region.types=summarized.regions(object),
			# 		assembly=object@assembly)
			aa <- annotation(object,"sites")
			sites.obj <- data.frame(chrom=as.character(aa$Chromosome),start=aa$Start,strand=as.character(aa$Strand),stringsAsFactors=FALSE)
			doBigFf <- !is.null(object@status$disk.dump.bigff)
			if (doBigFf) doBigFf <- object@status$disk.dump.bigff
			res <- RnBiseqSet(
				pheno=data.frame(pheno.new),
				sites=sites.obj,
				meth=meth.site.new,
				covg=covg.site.new,
				region.types=summarized.regions(object),
				assembly=object@assembly,
				useff=object@status$disk.dump,
				usebigff=doBigFf
			)
		} else if (is.element(class(object),c("RnBeadSet","RnBeadRawSet"))) {
			meth.site.new <- mergeColumns(meth(object,type="sites",row.names=TRUE),replicate.list)
			p.vals <- NULL
			if (!is.null(object@pval.sites)){
				p.vals <- mergeColumns(dpval(object,row.names=TRUE),replicate.list,
					mergeFun=function(X.sub){
						apply(X.sub,1,function(x){combineTestPvalsMeth(na.omit(x),correlated=FALSE)})
					}
				)
			}
			b.counts <- NULL
			if(object@target=="probesEPIC"){
				platform<-"EPIC"
			}else if (object@target=="probesEPICv2"){
				platform<-"EPICv2"
			}else if (object@target=="probes450"){
				platform<-"450k"
			}else if(object@target=="probes27"){
				platform<-"27k"
			}
			# res <- new("RnBeadSet",
			# 	data.frame(pheno.new),
			# 	meth.site.new,
			# 	p.values=p.vals,
			# 	bead.counts=b.counts,
			# 	platform=platform,
			# 	region.types=summarized.regions(object)
			# )
			res <- RnBeadSet(
				pheno=data.frame(pheno.new),
				betas=meth.site.new,
				p.values=p.vals,
				bead.counts=b.counts,
				platform=platform,
				region.types=summarized.regions(object),
				useff=object@status$disk.dump
			)
		} else {
			stop("Could not merge samples: Invalid class of object")
		}
		return(res)
	}
)
########################################################################################################################

setGeneric("combine.rnb.sets", function(x,y, ...) standardGeneric("combine.rnb.sets"))

#' combine.rnb.sets-methods
#'
#' Combine two objects inheriting from \code{\linkS4class{RnBSet}} class
#'
#' @param x,y 		\code{\linkS4class{RnBeadSet}}, \code{\linkS4class{RnBeadRawSet}}
#' 					or \code{\linkS4class{RnBiseqSet}} object
#' @param type		\code{character} singleton defining the set operation applied to the two site sets, 
#' 					one of "all", "all.x", "all.y" or "common"
#' 
#' @details Combine method supports a merge of any two RnBSet objects that contain data of the same specie.
#' In case a non-synonymous merge is performed, the class conversion will follow the following hierarchy: 
#' \code{\linkS4class{RnBeadSet}} < \code{\linkS4class{RnBeadRawSet}} < \code{\linkS4class{RnBiseqSet}}.
#' In case \code{x} and \code{y} are both array data containers (\code{RnBeadSet} or \code{RnBeadRawSet}), 
#' the resulting object will have an annotation that corresponds to the newer array version 
#' (\code{27k} < \code{450k} < \code{EPIC} < \code{EPICv2}).
#' The sample sets of \code{x} and \code{y} should be unique. Sample annotation information is merged only for columns 
#' which have identical names in both objects. CpG sites of the new object are a union of those present in both objects.
#'
#' @return combined \code{\linkS4class{RnBeadSet}}, \code{\linkS4class{RnBeadRawSet}} or
#' \code{\linkS4class{RnBiseqSet}} object
#'
#' @rdname combine.rnb.sets-methods
#' @docType methods
#' @export
#' @aliases combine.rnb.sets
#' @aliases combine.rnb.sets,RnBSet-method
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' r1 <- rnb.set.example
#' r1 <- remove.samples(r1,samples(rnb.set.example)[1:5])
#' i <- which(r1@@sites[,2] == 15 | r1@@sites[,2] == 21)
#' sites.rem.r1 <- union(sample(1:nrow(meth(rnb.set.example)),500),i)
#' r1 <- remove.sites(r1,sites.rem.r1)
#' r2 <- rnb.set.example
#' r2 <- remove.samples(r2,samples(rnb.set.example)[6:12])
#' sites.rem.r2 <- sample(1:nrow(meth(rnb.set.example)),800)
#' r2 <- remove.sites(r2,sites.rem.r2)
#' rc <- combine.rnb.sets(r1,r2)
#' #assertion: check the number of sites
#' sites.rem.c <- intersect(sites.rem.r1,sites.rem.r2)
#' (nrow(meth(rnb.set.example))-length(sites.rem.c)) == nrow(meth(rc))
#' }
setMethod("combine.rnb.sets", signature(x="RnBSet", y="RnBSet"),
        function(x, y, type="all"){
            if(class(x)==class(y)){
                if(inherits(x, "RnBeadSet")){
                    rnb.combine.arrays(x, y, type=type)
                }else if(inherits(x, "RnBiseqSet")){
                    rnb.combine.seq(x, y, type=type)
                }else{
                    rnb.error("This combine operation is currently not supported")
                }
            }else{
                if(inherits(x, "RnBiseqSet")){
                    y.seq<-as(y, "RnBiseqSet")
                    rnb.combine.seq(x, y.seq, type=type)
                }else if(inherits(y, "RnBiseqSet")){
                    x.seq<-as(x, "RnBiseqSet")
                    rnb.combine.seq(x.seq, y, type=type)
                }
            }
        }
)

########################################################################################################################
if (!isGeneric("addPheno")) {
	setGeneric("addPheno", function(object, ...) standardGeneric("addPheno"))
}
#' addPheno
#'
#' Adds phenotypic or processing information to the sample annotation table of the given \code{RnBSet} object.
#'
#' @param object \code{\linkS4class{RnBSet}} of interest.
#' @param trait   Trait as a non-empty \code{vector} or \code{factor}. The length of this vector must be equal to the
#'                number of samples in \code{object}, the i-th element storing the value for the i-th sample. Note that
#'                names, if present, are ignored.
#' @param header  Trait name given as a one-element \code{character}. This is the heading to be used for the sample
#'                annotation table. This method fails if such a trait already exists; in other words, if
#'                \code{header \%in\% names(pheno(object))}.
#' @return The modified dataset as an object of type \code{\linkS4class{RnBSet}}.
#'
#' @author Fabian Mueller
#' @export
#' @docType methods
#' @rdname addPheno-RnBSet-methods
#' @aliases addPheno
#' @aliases addPheno,RnBSet-method
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' is.hiPSC <- pheno(rnb.set.example)[, "Sample_Group"]=="hiPSC"
#' rnb.set.mod <- addPheno(rnb.set.example, is.hiPSC, "is_hiPSC")
#' pheno(rnb.set.mod)
#' }
setMethod("addPheno", signature(object="RnBSet"),
	function(object, trait, header) {
		if (!((is.vector(trait) || is.factor(trait)) && length(trait) == nrow(pheno(object)))) {
			stop(paste("invalid value for trait; expected vector of length", nrow(pheno(object))))
		}
		if (!(is.character(header) && length(header) == 1 && (!is.na(header)))) {
			stop("invalid value for header; expected one-element character")
		}
		if (is.element(header, names(pheno(object)))) {
			stop(paste("trait", header, "already exists in the sample annotation table"))
		}

		object@pheno[[header]] <- trait
		return(object)
	}
)
########################################################################################################################


if (!isGeneric("summarize.regions")) {
	setGeneric("summarize.regions", function(object, ...) standardGeneric("summarize.regions"))
}
#' summarize.regions-methods
#'
#' Summarize DNA methylation information for which is present in the \code{RnBSet} object.
#'
#' @param object Dataset of interest.
#' @param region.type Type of the region annotation for which the summarization will be performed or \code{"strands"} for summarizing the methylation values from both strands
#' @param aggregation Operation to summarize the methylation values. Currently supported values are \code{"mean"}, \code{"median"}, \code{"min"}, \code{"max"} and \code{"coverage.weighted"}
#' @param overwrite If \code{TRUE} the existing region-level information for \code{region.type} is discarded
#'
#' @return object of the same class as the supplied one containing the summarized methylation information for the specified region types
#'
#' @rdname summarize.regions-methods
#' @docType methods
#' @aliases summarize.regions
#' @aliases summarize.regions,RnBSet-method
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' rnb.set.summarized<-summarize.regions(rnb.set.example, "genes", overwrite=TRUE)
#' head(meth(rnb.set.summarized, type="genes", row.names=TRUE))
#' }
setMethod("summarize.regions", signature(object="RnBSet"),
		function(object, region.type, aggregation = rnb.getOption("region.aggregation"), overwrite = TRUE) {
			if (!(is.character(region.type) && length(region.type) == 1 && (!is.na(region.type)))) {
				stop("invalid value for region.type")
			}
			if (!(is.character(aggregation) && length(aggregation) == 1 && (!is.na(aggregation)))) {
				stop("invalid value for aggregation; expected single character")
			}
			## FIXME: Some of these aren't implemented; and I need them (min and max in particular)
			##        Is there a measurable improvement over the simple get(...) implementation that was dropped?
			aggregation <- aggregation[1]
			if (!(aggregation %in% c("min", "max", "mean", "median", "sum", "coverage.weighted"))) {
				stop("invalid value for aggregation; expected one of \"min\", \"max\", \"mean\", \"median\", \"sum\" or \"coverage.weighted\"")
			}
			if (overwrite == FALSE && region.type %in% names(object@meth.regions)) {
				stop("invalid region type; methylation data already present")
			}
#			aggregate.f <- get(aggregation)
#			aggregate.function <- function(x) {
#				tryCatch(aggregate.f(x, na.rm = TRUE), warning = function(w) { as.double(NA) })
#			}
			## Extract the full annotation tables for the regions and the sites
			if (!(region.type %in% c(rnb.region.types(object@assembly),"strands"))){
				stop("unsupported region type")
			}

			if (region.type =="strands" && !inherits(object, "RnBiseqSet")){
				stop("cannot summarize the strand-specific information for objects other than RnBiseqSet")
			}

			if (aggregation == "coverage.weighted" && !inherits(object, "RnBiseqSet")){
				stop("coverage.weighted aggregation is allowed only for objects of type RnBiseqSet")
			}

			if (aggregation == "coverage.weighted" && is.null(object@covg.sites)){
				stop("cannot apply coverage.weighted aggregation method to an RnBiseqSet object with
						missing coverage information")
			}
			bff.finalizer <- rnb.getOption("disk.dump.bigff.finalizer")

			if(region.type=="strands"){
				annot.sizes <- rnb.annotation.size(assembly=object@assembly)
				mapping <- sapply(names(rnb.get.chromosomes(assembly=object@assembly)), function(chr){
					num.sites <- annot.sizes[[chr]]
					#TODO:this is not really robust
					IRanges(start=(1:(num.sites/2))*2-1, width=2, names=(1:(num.sites/2))*2-1)
				})
			}else{
				mapping <- rnb.get.mapping(region.type, object@target, object@assembly)
			}

			chromInds <- unique(object@sites[,2])
			#construct the overlap data structure for retrieving other information
			regMap.ov.str <- lapply(chromInds, function(chr.id){
				chr.map <- object@sites[,2]==chr.id
				names(chr.map) <- NULL
				site.ranges <- IRanges(start=object@sites[chr.map,3], width=1)
				chr.name <- names(rnb.get.chromosomes(assembly=object@assembly))[chr.id]
				mapping.contains.chrom <- chr.name %in% names(mapping)
				if(!mapping.contains.chrom){
					return(NULL)
				}
				chr.mapping.ind <- match(chr.name,names(mapping))
				olap <- IRanges::as.matrix(findOverlaps(mapping[[chr.mapping.ind]], site.ranges))

				if(nrow(olap)<1) return(NULL)
				return(list(
					chr.id=chr.id,
					chr.name=chr.name,
					chr.mapping.ind=chr.mapping.ind,
					chr.match.inds=which(chr.map),
					olap=olap
				))
			})
			# logger.info(c("DEBUG:","Generated mapping structure for all chromosomes"))
			region.indices <- do.call("rbind", lapply(regMap.ov.str, function(x){
				if (is.null(x)) return(NULL)
				indOnChrom <- unique(x$olap[,1])
				regInd <- as.integer(names(mapping[[x$chr.mapping.ind]][indOnChrom]))
				cbind(rep(1, length(regInd)), rep(x$chr.id, length(regInd)), regInd)
			}))
			# logger.info(c("DEBUG:","Generated region index data frame"))
			regions2sites <- unlist(lapply(regMap.ov.str, function(x){
				if (is.null(x)) return(list())
				tapply(x$chr.match.inds[x$olap[,2]], factor(x$olap[,1], levels=unique(x$olap[,1])), list)
			}), recursive=FALSE)
			names(regions2sites) <- NULL
			# regions2sites.tab <- do.call("rbind",lapply(1:length(regions2sites), FUN=function(i){
			# 	cbind(rep(i, length(regions2sites[[i]])), regions2sites[[i]])
			# }))
			# regions2sites.tab.fac <- factor(regions2sites.tab[,1], levels=unique(regions2sites.tab[,1]))
			# logger.info(c("DEBUG:","Generated mapping of regions to sites"))

			nSamples <- length(samples(object))

			aggr.f <- NULL
			if (aggregation=="mean"){
				aggr.f <- function(siteInds, siteVec, covgVec=NULL){
					mean(siteVec[siteInds], na.rm=TRUE)
					# 0.666
				}
			} else if (is.element(aggregation, c("min", "max", "mean", "median", "sum"))){
				aggr.f <- function(siteInds, siteVec, covgVec=NULL){
					do.call(aggregation, list(siteVec[siteInds], na.rm=TRUE))
				}
			} else if (aggregation=="coverage.weighted"){
				aggr.f <- function(siteInds, siteVec, covgVec){
					cTotal <- sum(covgVec[siteInds], na.rm=TRUE)
					sum(siteVec[siteInds]*covgVec[siteInds], na.rm=TRUE)/cTotal
				}
			}
			site.meth <- object@meth.sites
			site.covg <- object@covg.sites
			aggr.meth.sample <- function(j){
				siteVec <- site.meth[,j]
				covgVec <- NULL
				if (aggregation=="coverage.weighted") covgVec <- site.covg[,j]
				vapply(regions2sites, aggr.f, numeric(1), siteVec=siteVec, covgVec=covgVec)
			}
			aggr.covg.sample <- function(j){
				siteVec <- site.covg[,j]
				vapply(regions2sites, function(siteInds){
					sum(siteVec[siteInds], na.rm=TRUE)
				}, numeric(1))
			}

			## Assign the resulting matrices to the object
			if (region.type=="strands"){
				if(!is.null(object@status) && object@status$disk.dump){
					doBigFf <- !is.null(object@status$disk.dump.bigff)
					if (doBigFf) doBigFf <- object@status$disk.dump.bigff

					# delete(object@meth.sites)
					if (doBigFf) {
						object@meth.sites <- BigFfMat(row.n=nrow(region.indices), col.n=nSamples, col.names=samples(object), finalizer=bff.finalizer)
						# logger.info(c("DEBUG:","Created BigFfMat for meth"))
					} else {
						object@meth.sites <- convert.to.ff.matrix.tmp(matrix(numeric(0), nrow=nrow(region.indices), ncol=nSamples, dimnames=list(NULL,samples(object))))
					}
				} else{
					object@meth.sites <- matrix(numeric(0), nrow=nrow(region.indices), ncol=nSamples, dimnames=list(NULL,samples(object)))
				}
				for (j in 1:nSamples){
					# logger.info(c("DEBUG:","Aggregating methylation for sample",j))
					object@meth.sites[,j] <- aggr.meth.sample(j)
				}
				if (!is.null(object@covg.sites)) {
					if(!is.null(object@status) && object@status$disk.dump){
						doBigFf <- !is.null(object@status$disk.dump.bigff)
						if (doBigFf) doBigFf <- object@status$disk.dump.bigff

						# delete(object@covg.sites)
						if (doBigFf) {
							object@covg.sites <- BigFfMat(row.n=nrow(region.indices), col.n=nSamples, col.names=samples(object), finalizer=bff.finalizer)
						} else {
							object@covg.sites <- convert.to.ff.matrix.tmp(matrix(integer(0), nrow=nrow(region.indices), ncol=nSamples, dimnames=list(NULL,samples(object))))
						}
					} else {
						object@covg.sites <- matrix(integer(0), nrow=nrow(region.indices), ncol=nSamples, dimnames=list(NULL,samples(object)))
					}
					for (j in 1:nSamples){
						# logger.info(c("DEBUG:","Aggregating coverage for sample",j))
						object@covg.sites[,j] <- aggr.covg.sample(j)
					}
				} else {
					object@covg.sites <- NULL
				}
				object@sites <- region.indices
			} else if(!is.null(region.indices)){
				if(!is.null(object@status) && object@status$disk.dump){
					doBigFf <- !is.null(object@status$disk.dump.bigff)
					if (doBigFf) doBigFf <- object@status$disk.dump.bigff
					# if(!is.null(object@meth.regions[[region.type]])){
					# 	delete(object@meth.regions[[region.type]])
					# }
					if(rnb.getOption("enforce.destroy.disk.dumps")){
						delete(object@meth.regions[[region.type]])
					}
					if (doBigFf){
						object@meth.regions[[region.type]] <- BigFfMat(row.n=nrow(region.indices), col.n=nSamples, col.names=samples(object), finalizer=bff.finalizer)
						# logger.info(c("DEBUG:","Created BigFfMat for meth"))
					} else {
						object@meth.regions[[region.type]] <- convert.to.ff.matrix.tmp(matrix(numeric(0), nrow=nrow(region.indices), ncol=nSamples, dimnames=list(NULL,samples(object))))
					}
				} else {
					object@meth.regions[[region.type]] <- matrix(numeric(0), nrow=nrow(region.indices), ncol=nSamples, dimnames=list(NULL,samples(object)))
				}
				for (j in 1:nSamples){
					# logger.info(c("DEBUG:","Aggregating methylation for sample",j))
					object@meth.regions[[region.type]][,j] <- aggr.meth.sample(j)
				}
				if(!is.null(object@covg.sites)) {
					if(!is.null(object@status) && object@status$disk.dump){
						doBigFf <- !is.null(object@status$disk.dump.bigff)
						if (doBigFf) doBigFf <- object@status$disk.dump.bigff
						# if(!is.null(object@covg.regions[[region.type]])) {
						# 	delete(object@covg.regions[[region.type]])
						# }
						if(rnb.getOption("enforce.destroy.disk.dumps")){
							delete(object@covg.regions[[region.type]])
						}
						if (doBigFf){
							if (is.null(object@covg.regions)) object@covg.regions <- list()
							object@covg.regions[[region.type]] <- BigFfMat(row.n=nrow(region.indices), col.n=nSamples, col.names=samples(object), finalizer=bff.finalizer)
						} else {
							object@covg.regions[[region.type]] <- convert.to.ff.matrix.tmp(matrix(integer(0), nrow=nrow(region.indices), ncol=nSamples, dimnames=list(NULL,samples(object))))
						}
					}else{
						object@covg.regions[[region.type]] <- matrix(integer(0), nrow=nrow(region.indices), ncol=nSamples, dimnames=list(NULL,samples(object)))
					}
					for (j in 1:nSamples){
						# logger.info(c("DEBUG:","Aggregating coverage for sample",j))
						object@covg.regions[[region.type]][,j] <- aggr.covg.sample(j)
					}
				}else{
					object@covg.regions <- NULL
				}

				attr(object@meth.regions[[region.type]], "aggregation") <- aggregation
				object@regions[[region.type]] <- region.indices
			}else{ #no valid regions found
				object@meth.regions[[region.type]] <- matrix(0L, nrow=0, ncol=ncol(object@meth.sites))
				if(!is.null(object@covg.sites)) object@covg.regions[[region.type]] <- matrix(0L, nrow=0, ncol=ncol(object@meth.sites))
				attr(object@meth.regions[[region.type]], "aggregation") <- aggregation
				object@regions[[region.type]] <- matrix(0L, nrow=0, ncol=3)
			}
			rm(site.meth) #for ff and BigFfMat, the finalizer should be "delete" and thus the objects should be deleted from disk when this function terminates
			rm(site.covg)
			object
		}
)
########################################################################################################################

if (!isGeneric("remove.regions")) {
	setGeneric("remove.regions", function(object, ...) standardGeneric("remove.regions"))
}

#' remove.regions-methods
#'
#' Remove the summarized methylation information for a given region type from an \code{RnBSet} object.
#'
#' @param object Dataset of interest.
#' @param region.type Type of the region annotation for which the summarization should be removed
#'
#' @return object of the same class as the supplied one without the summarized methylation information for the specified region type
#'
#' @rdname remove.regions-methods
#' @docType methods
#' @aliases remove.regions
#' @aliases remove.regions,RnBSet-method
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' summarized.regions(rnb.set.example)
#' rnb.set.reduced<-remove.regions(rnb.set.example, "genes")
#' summarized.regions(rnb.set.reduced)
#' }
setMethod("remove.regions", signature(object="RnBSet"),
	function(object, region.type) {
		object@regions[[region.type]] <- NULL
		object@meth.regions[[region.type]] <- NULL
		if(!is.null(object@covg.sites)) object@covg.regions[[region.type]] <- NULL
		return(object)
	}
)

########################################################################################################################

if (!isGeneric("regionMapping")) {
	setGeneric("regionMapping", function(object, ...) standardGeneric("regionMapping"))
}

#' regionMapping-methods
#'
#' get the mapping of regions in the RnBSet object to methylation site indices in the RnBSet object
#'
#' @param object Dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param region.type  region type. see \code{\link{rnb.region.types}} for possible values
#' @return A list containing for each region the indices (as integers) of sites that belong to that region
#'
#' @rdname regionMapping-methods
#' @docType methods
#' @aliases regionMapping
#' @aliases regionMapping,RnBSet-method
#' @author Fabian Mueller
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' promoter.probe.list <- regionMapping(rnb.set.example,"promoters")
#' #get the number of CpGs per promoter in the dataset:
#' sapply(promoter.probe.list,length)
#' }
setMethod("regionMapping", signature(object = "RnBSet"),
function(object, region.type) {

			if (!inherits(object, "RnBSet")) {
				stop("invalid value for object; expected RnBSet")
			}
			if (!(is.character(region.type) && length(region.type) == 1 && (!is.na(region.type)))) {
				stop("invalid value for type")
			}

			if (!(region.type %in% rnb.region.types(object@assembly))) {
				stop(paste0("unsupported annotation type (annotation): ",region.type))
			}
			if (!(region.type %in% names(object@regions))) {
				stop(paste0("unsupported annotation type (RnBSet): ",region.type))
			}
			chrom.maps <- rnb.get.mapping(region.type, object@target, object@assembly)

			chrom.integer2name <- names(rnb.get.chromosomes(assembly=object@assembly))
			obj.sites <- data.frame(object@sites)
			region.map <- object@regions[[region.type]]
			chr.inds.reg <- unique(region.map[,2])

			obj.sites[,2] <- factor(chrom.integer2name[obj.sites[,2]],levels=chrom.integer2name[unique(obj.sites[,2])])
#			obj.sites[,2] <- factor(chrom.integer2name[obj.sites[,2]],levels=chrom.integer2name)
#			obj.sites[,2] <- as.factor(chrom.integer2name[obj.sites[,2]])
			chrom.site.inds <- tapply(obj.sites[,3],obj.sites[,2],FUN=function(x){
				IRanges(start=x,width=1)
			})

			chrom.offsets <- sapply(chrom.site.inds,length)
			chrom.offsets <-cumsum(c(0,chrom.offsets[-length(chrom.offsets)]))
			names(chrom.offsets) <- names(chrom.site.inds)

			result <- lapply(chr.inds.reg,FUN=function(chr){

				curChromName <- chrom.integer2name[chr]
				rnbs.regs <- region.map[region.map[,2]==chr,3]
				rnbs.regs.char <- format(rnbs.regs,trim=TRUE,scientific=FALSE)
				rrRanges <- chrom.maps[[curChromName]]
				#only take the regions that are also in the RnBSet object
				if (!all(rnbs.regs.char %in% names(rrRanges))) {stop(paste("Not all regions in RnBSet are present in the annotation (",curChromName,")"))}
				rrRanges <- rrRanges[rnbs.regs.char,]
				olap<-as.matrix(findOverlaps(chrom.site.inds[[curChromName]], rrRanges))

				olap[,1]<-olap[,1]+chrom.offsets[curChromName]
				res<-tapply(olap[,1], olap[,2], list)

				return(res)
			})

			result<-unlist(result, recursive=FALSE)
			names(result)<-NULL
			if (dim(region.map)[1] != length(result)){
				stop("regionMapping failed")
			}
			return(result)

		}
)

########################################################################################################################
#' annotation-methods
#'
#' Genomic annotation of the methylation sites or regions covered in the supplied dataset.
#'
#' @param object dataset as an object of type inheriting \code{RnBSet}.
#' @param type   loci or regions for which the annotation should be obtained. If the value of this parameter is
#'               \code{"sites"} (default), individual methylation sites are annotated. Otherwise, this must be one of
#'               the available region types, as returned by \code{\link{rnb.region.types}}.
#' @param add.names flag specifying whether the unique site identifiers should be used as row names of the
#' 					resulting data frame
#' @param include.regions if \code{TRUE} one additional column is added to the returned annotation dat frame
#' 						  for each of the available region types, giving the indices of the
#'
#' @return Annotation table in the form of a \code{data.frame}.
#'
#' @rdname annotation-methods
#' @docType methods
#' @aliases annotation
#' @aliases annotation,RnBSet-method
#' @author Pavlo Lutsik
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' ## show present sites
#' head(annotation(rnb.set.example, add.names=TRUE))
#' ## show promoters
#' ann.prom<-annotation(rnb.set.example, type="promoters", add.names=TRUE)
#' head(ann.prom)
#' }
setMethod("annotation", signature(object = "RnBSet"),
		function(object, type="sites", add.names=FALSE, include.regions=FALSE) {

			if (!inherits(object, "RnBSet")) {
				stop("invalid value for object; expected RnBSet")
			}
			if (!(is.character(type) && length(type) == 1 && (!is.na(type)))) {
				stop("invalid value for type")
			}

			if (type == "sites") {
				type <- object@target
				subsets <- object@sites
			} else {
				if (!(type %in% rnb.region.types(object@assembly))) {
					stop(paste0("unsupported annotation type (annotation): ",type))
				}
				if (!(type %in% names(object@regions))) {
					## This region type is not initialized with summarize.regions
					## FIXME: Report an error or initialize.
					stop(paste0("unsupported annotation type (RnBSet): ",type))
				}
				subsets <- object@regions[[type]]
			}

			annot <- rnb.get.annotation(type, object@assembly)
			ind.shift<-rnb.annotation.size(type, object@assembly)
			ind.shift<-cumsum(c(0,ind.shift[-length(ind.shift)]))
			subsets.full<-subsets[,3]+ind.shift[subsets[,2]]

			result<-rnb.annotation2data.frame(annot, add.names=add.names)[subsets.full,]

			if(include.regions){
				dump<-sapply(names(object@regions), function(rt){
					result[,rt]<<-rep(0L,nrow(result))
					map<-regionMapping(object, rt)
					index_map<-lapply(1:length(map), function(ix) rep(ix, length(map[[ix]])))
					result[unlist(map),rt]<<-unlist(index_map)
				})
			}

			return(result)
		}
)
########################################################################################################################

#if (!isGeneric("save.matrices")) {
	setGeneric("save.matrices", function(object, path, ...) standardGeneric("save.matrices"))
#}

setMethod("save.matrices", signature(object="RnBSet", path="character"),
		function(object, path){

			if(!is.null(object@status) && object@status$disk.dump){
				if("ff" %in% class(object@meth.sites)){
					ffmatrix <- object@meth.sites
					ffsave(ffmatrix,file=file.path(path, "rnb.meth"),rootpath=getOption('fftempdir'))
					rm(ffmatrix)
				} else if("BigFfMat" %in% class(object@meth.sites)){
					save.bigFfMat(object@meth.sites, file=file.path(path, "rnb.meth"), rootpath=getOption('fftempdir'))
				}

				if("ff" %in% class(object@covg.sites)){
					ffmatrix <- object@covg.sites
					ffsave(ffmatrix, file=file.path(path, "rnb.covg"),rootpath=getOption('fftempdir'))
					rm(ffmatrix)
				} else if("BigFfMat" %in% class(object@covg.sites)){
					save.bigFfMat(object@covg.sites, file=file.path(path, "rnb.covg"), rootpath=getOption('fftempdir'))
				}

				if(length(object@regions) != 0){

					for(rgn in 1:length(object@regions)){

						rgnpath<-file.path(path,rgn)
						if(!file.exists(rgnpath)){
							dir.create(rgnpath)
						}

						if("ff" %in% class(object@meth.regions[[rgn]])){
							ffmatrix<-object@meth.regions[[rgn]]
							ffsave(ffmatrix, file=file.path(path, rgn, "rnb.meth"),rootpath=getOption('fftempdir'))
							rm(ffmatrix)
						} else if("BigFfMat" %in% class(object@meth.regions[[rgn]])){
							save.bigFfMat(object@meth.regions[[rgn]], file=file.path(path, rgn, "rnb.meth"), rootpath=getOption('fftempdir'))
						}

						if("ff" %in% class(object@covg.regions[[rgn]])){
							ffmatrix<-object@covg.regions[[rgn]]
							ffsave(ffmatrix, file=file.path(path, rgn, "rnb.covg"),rootpath=getOption('fftempdir'))
							rm(ffmatrix)
						} else if("BigFfMat" %in% class(object@covg.regions[[rgn]])){
							save.bigFfMat(object@covg.regions[[rgn]], file=file.path(path, rgn, "rnb.covg"), rootpath=getOption('fftempdir'))
						}

					}

				}
			}


		})

########################################################################################################################

setGeneric("load.matrices",
		function(object, path, ...) standardGeneric("load.matrices"))

setMethod("load.matrices", signature(object="RnBSet", path="character"),
		function(object, path, temp.dir=tempdir()){
			doBigFf <- !is.null(object@status)
			if (doBigFf) doBigFf <- !is.null(object@status$disk.dump.bigff)
			if (doBigFf) doBigFf <- object@status$disk.dump.bigff

			if (doBigFf){
				object@meth.sites <- load.bigFfMat(file.path(path, "rnb.meth"), rootpath=getOption("fftempdir"))
				if(!is.null(object@covg.sites)){
					object@covg.sites <- load.bigFfMat(file.path(path, "rnb.covg"), rootpath=getOption("fftempdir"))
				}
			} else {
				if(sum(grepl("rnb.meth", list.files(path)))==2){
					load_env<-new.env()
					suppressMessages(ffload(file=file.path(path, "rnb.meth"), envir=load_env,rootpath=getOption("fftempdir")))
					object@meth.sites<-get("ffmatrix", envir=load_env)
					rm(load_env)
				}

				if(sum(grepl("rnb.covg", list.files(path)))==2){
					load_env<-new.env()
					suppressMessages(ffload(file=file.path(path, "rnb.covg"), envir=load_env,rootpath=getOption("fftempdir")))
					object@covg.sites<-get("ffmatrix", envir=load_env)
					rm(load_env)
				}
			}

			rgns <- names(object@regions)
			if(!is.null(rgns)){
				if (.hasSlot(object, 'version')) {
					rgns <- 1:length(rgns)
				}

				for(rgn in rgns){
					if (doBigFf){
						object@meth.regions[[rgn]] <- load.bigFfMat(file.path(path, rgn, "rnb.meth"), rootpath=getOption("fftempdir"))
						if(!is.null(object@covg.regions[[rgn]])){
							object@covg.regions[[rgn]] <- load.bigFfMat(file.path(path, rgn, "rnb.covg"), rootpath=getOption("fftempdir"))
						}
					} else {
						if(sum(grepl("rnb.meth",list.files(file.path(path, rgn))))==2){
							load_env<-new.env()
							suppressMessages(ffload(file=file.path(path, rgn, "rnb.meth"), envir=load_env, rootpath=getOption("fftempdir")))
							object@meth.regions[[rgn]]<-get("ffmatrix", envir=load_env)
							rm(load_env)
						}
						if(sum(grepl("rnb.covg",list.files(file.path(path, rgn))))==2){
							load_env<-new.env()
							suppressMessages(ffload(file=file.path(path, rgn, "rnb.covg"), envir=load_env, rootpath=getOption("fftempdir")))
							object@covg.regions[[rgn]]<-get("ffmatrix", envir=load_env)
							rm(load_env)
						}
					}
				}
			}

			return(object)

		})

########################################################################################################################

#' save.rnb.set
#'
#' Consistent saving of an \code{RnBSet} objects with large matrices of type \link{ff}.
#'
#' @param object     \code{RnBSet}-inheriting object.
#' @param path	      the name of the output file (or directory if \code{archive} is \code{FALSE})
#' 					  without an extension. If only the file name is given the object will be saved
#' 					  in the current working directory.
#' @param archive     if \code{TRUE} (default value) the output is a ZIP-file.
#'
#' @details 		  The saved object can be reloaded with the \link{load.rnb.set} function.
#'
#' @return 			  invisibly, the full path to the ZIP file (if \code{archive} is \code{TRUE}),
#' 					  or to the output directory (otherwise)
#'
#' @author Pavlo Lutsik
#' @export
save.rnb.set<-function(object, path, archive=TRUE){

	## Validate parameters
	if (!inherits(object, "RnBSet")) {
		stop("invalid value for object")
	}
	if (!(is.character(path) && length(path) == 1 && isTRUE(!grepl("^[/\\.]*$", path)))) {
		stop("invalid value for path")
	}
	if (!parameter.is.flag(archive)) {
		stop("invalid value for archive")
	}

	if(object@status$disk.dump && .Platform$OS == "windows" && Sys.getenv("R_ZIPCMD")==""){
		rnb.warning(c("Zip not found on this Windows system, this RnBSet object will not be saved.",
				"See the instructions for installing ZIP on Windows in the FAQ section of the RnBeads website."))
		return(invisible(path))
	}

	## Get the full path of the file or directory to be created
	fullpath <- normalizePath(gsub("/$", "", gsub("\\", "/", path, fixed = TRUE)), winslash = "/", mustWork = FALSE)

	## Create or overwrite a directory to store the files
	if (unlink(fullpath, recursive = TRUE) == 1) {
		stop("Specified path already exists and cannot be overwritten")
	}
	if (archive) {
		if (unlink(paste0(fullpath, ".zip"), recursive = TRUE) == 1) {
			stop("Specified path already exists and cannot be overwritten")
		}
	}
	if (!dir.create(fullpath, showWarnings = FALSE, recursive = TRUE)) {
		stop("Could not create output directory")
	}

	## Save all data structures
	save.matrices(object, fullpath)
	save(object, file=file.path(fullpath, "rnb.set.RData"))

	## Create a ZIP archive of the whole directory
	if(archive){
		currdir <- setwd(fullpath)
		zip(paste0(fullpath, ".zip"), dir(), flags = "-rm9X")
		while(length(list.files(path))>0){
			TRUE;
		}
		setwd(currdir)
		if (unlink(fullpath, recursive = TRUE) == 1) {
			rnb.warning("Could not clean output directory after zipping")
		}
		fullpath <- paste0(fullpath, ".zip")
	}

	return(invisible(fullpath))
}

########################################################################################################################

#' load.rnb.set
#'
#' Loading of the \code{RnBSet} objects with large matrices of type \pkg{ff}.
#'
#' @param path			full path of the file or directory. If \code{archive} is \code{FALSE})
#' 					  	without an extension.
#' @param temp.dir		\code{character} singleton which specifies temporary directory, used while loading
#'
#' @return				Loaded object
#'
#' @author Pavlo Lutsik
#' @export
load.rnb.set<-function(path, temp.dir=tempdir()){

	## Validate parameters
	if (!(is.character(path) && length(path) == 1 && isTRUE(!grepl("^[/\\.]*$", path)))) {
		stop("invalid value for path")
	}
	if (!(is.character(temp.dir) && length(temp.dir) == 1 && isTRUE(!grepl("^[/\\.]*$", temp.dir)))) {
		stop("invalid value for temp.dir")
	}
	if (!file.exists(path)) {
		stop("invalid value for path; the path does not exist")
	}
	if (!isTRUE(file.info(temp.dir)[1, "isdir"])) {
		stop("invalid value for temp.dir; the path does not exist or is not a directory")
	}

	if(.Platform$OS == "windows" && Sys.getenv("R_ZIPCMD")==""){
	 	method="internal"
	}else{
		method="unzip"
	}
  
  if(grepl("rnb.set.RData",path)){
    logger.info("The path to the data set directory should be provided, not to the invidual file. Changing to parent directory.")
    path <- dirname(path)
  }

	if(!file.info(path)[["isdir"]]){
		td<-tempfile("extraction", temp.dir)
		unzip(path, exdir=td, unzip=method)
	}else{
		td<-path
	}

	load_env<-new.env(parent=emptyenv())
	load(file.path(td, "rnb.set.RData"),envir=load_env)
	load.matrices(get("object", load_env), td, temp.dir=temp.dir)
}

########################################################################################################################

if (!isGeneric("destroy")) setGeneric("destroy", function(object) standardGeneric("destroy"))
#' destroy-methods
#'
#' Remove tables stored to disk from the file system. Useful for cleaning up disk dumped objects.
#'
#' @param object object inheriting from \code{\linkS4class{RnBSet}}
#' @return Nothing of particular interest
#'
#' @rdname destroy-methods
#' @docType methods
#' @aliases destroy
#' @aliases destroy,RnBSet-method
#' @export
setMethod("destroy", signature(object="RnBSet"),
		function(object){

			if(object@status$disk.dump){
				delete(object@meth.sites)

				if(!is.null(object@covg.sites)){
					delete(object@covg.sites)
				}

				if(!is.null(object@regions)){
					for(rgn in names(object@regions)){
						delete(object@meth.regions[[rgn]])
						if(!is.null(object@covg.regions))
						{
							delete(object@covg.regions[[rgn]])
						}
					}
				}
			}
			return(invisible(TRUE))
		}
)

########################################################################################################################

## meth.matrices
##
## Creates a list of methylation value (beta) matrices for the given dataset.
##
## @param object        Methylation dataset object of type that inherits \code{RnBSet}.
## @param include.sites Flag indicating if the methylation matrix of sites or probes is to be included in the result.
## @return Non-empty \code{list} of matrices of beta values. If \code{include.sites} is \code{TRUE}, the first matrix in
##         the list is the one based on sites or probes. Other matrices store region-based methylation for (some of) the
##         regions addressed in the option \code{"region.types"}.
## @author Yassen Assenov
meth.matrices <- function(object, include.sites = rnb.getOption("analyze.sites")) {
	result <- list()
	if (include.sites) result[["sites"]] <- meth(object)
	for (rtype in rnb.region.types.for.analysis(object)) {
		X <- tryCatch(meth(object, rtype), error = function(e) { NULL })
		if (!is.null(X)) {
			result[[rtype]] <- X
		}
	}
	return(result)
}

########################################################################################################################

## get.row.names
##
## Generates row names based on the genomic location.
##
## @param object \code{RnBSet} object.
## @return \code{character} vector of row names.
## @author Pavlo Lutsik
get.row.names<-function(object, type="sites"){
	if(type=="sites"){
		target<-object@target
		subsets<-object@sites
	}else if(type %in% names(object@regions)){
		target<-type
		subsets<-object@regions[[type]]
	}else stop("unsupported region type")

	loc.info<-annotation(object, type=type, add.names=TRUE)
	if ("ID" %in% colnames(loc.info) && anyDuplicated(loc.info[, "ID"]) == 0) {
		result <- loc.info[,"ID"]
	} else if (!is.null(rownames(loc.info))) {
		result <- rownames(loc.info)
	} else {
		result <- paste(loc.info[,"Chromosome"], loc.info[,"Start"], as.character(loc.info[,"Strand"]), sep=".")
	}
	result
}

########################################################################################################################

## rnb.get.row.token
##
## Gets the methylation target, that is, the basic methylation feature of a dataset based on its platform.
##
## @param object Methylation dataset of interest, an object of type inheriting \code{MethyLumiSet} or \code{RnBSet}.
## @param plural Flag, indicating if the plural form of the word.
## @return Word or phrase denoting the term for a single target of the platform.
## @author Pavlo Lutsik
rnb.get.row.token<-function(object, plural = FALSE){

	if (is.character(object)) {
		result <- ifelse(object %in% c("RnBiseqSet", "RnBSet"), "site", "probe")
	} else if (inherits(object, "MethyLumiSet")){
		result <- "probe"
	} else if (object@target == "CpG") {
		result <- "site"
	} else { # object@target == "probes450"
		result <- "probe"
	}

	ifelse(plural, paste0(result, "s"), result)
}

########################################################################################################################

## rnb.get.covg.token
##
## Gets the measure of coverage of a dataset based on its platform.
##
## @param object  Methylation dataset of interest, an object of type inheriting \code{MethyLumiSet} or \code{RnBSet}.
## @param capital Flag, indicating if the first letter of the returned phrase should be capitalized.
## @return Word or phrase denoting the term for depth of coverage.
## @author Pavlo Lutsik
rnb.get.covg.token<-function(object, capital=FALSE){

	if (is.character(object)) {
		result <- ifelse(object %in% c("RnBiseqSet", "RnBSet"), "coverage", "bead counts")
	} else if (inherits(object, "MethyLumiSet")) {
		result <- "bead counts"
	} else if (object@target == "CpG") {
		result <- "coverage"
	} else { # object@target == "probes450"
		result <- "bead counts"
	}

	ifelse(capital, capitalize(result), result)
}

########################################################################################################################
if(!isGeneric("sampleMethApply")) setGeneric("sampleMethApply", function(object, ...) standardGeneric("sampleMethApply"))
#' sampleMethApply-methods
#'
#' Applies a function over the methylation values for all samples in an \code{RnBSet} using a low memory footprint.
#' 
#' @param object object inheriting from \code{\linkS4class{RnBSet}}
#' @param fn function to be applied
#' @param type \code{character} singleton. Specify "sites" (default) or a region type over which the function is applied
#' @param ... arguments passed on to the function
#' @return Result analogous to \code{apply(meth(rnbSet, type), 2, FUN=FUN)}
#'
#' @seealso \code{\link[=meth,RnBSet-method]{meth}} Retrieving the matrix of methylation values
#' @rdname sampleMethApply-methods
#' @docType methods
#' @aliases sampleMethApply
#' @aliases sampleMethApply,RnBSet-method
setMethod("sampleMethApply", signature(object = "RnBSet"),
	function(object, fn, type="sites", ...) {
		if (!(is.character(type) && length(type) == 1 && (!is.na(type)))) {
			stop("invalid value for type")
		}
		if (type %in% c("sites", object@target)) {
			result <- nrow(object@meth.sites)
		} else if (!(type %in% names(object@regions))) {
			stop("unsupported region type")
		}
		res <- sapply(1:length(samples(object)), FUN=function(j){
			fn(meth(object, type=type, j=j), ...)
		})
		return(res)
	}
)
if(!isGeneric("sampleCovgApply")) setGeneric("sampleCovgApply", function(object, ...) standardGeneric("sampleCovgApply"))
#' sampleCovgApply-methods
#'
#' Applies a function over the coverage values for all samples in an \code{RnBSet} using a low memory footprint.
#' @param object object inheriting from \code{\linkS4class{RnBSet}}
#' @param fn function to be applied
#' @param type \code{character} singleton. Specify "sites" (default) or a region type over which the function is applied
#' @param ... arguments passed on to the function
#' @return Result analogous to \code{apply(covg(rnbSet, type), 2, FUN=FUN)}
#'
#' @seealso \code{\link[=meth,RnBSet-method]{covg}} Retrieving the matrix of coverage values
#' @rdname sampleCovgApply-methods
#' @docType methods
#' @aliases sampleCovgApply
#' @aliases sampleCovgApply,RnBSet-method
setMethod("sampleCovgApply", signature(object = "RnBSet"),
	function(object, fn, type="sites", ...) {
		if (!(is.character(type) && length(type) == 1 && (!is.na(type)))) {
			stop("invalid value for type")
		}
		if (type %in% c("sites", object@target)) {
			result <- nrow(object@covg.sites)
		} else if (!(type %in% names(object@regions))) {
			stop("unsupported region type")
		}
		res <- sapply(1:length(samples(object)), FUN=function(j){
			fn(covg(object, type=type, j=j), ...)
		})
		return(res)
	}
)
########################################################################################################################
if(!isGeneric("getNumNaMeth")) setGeneric("getNumNaMeth", function(object, ...) standardGeneric("getNumNaMeth"))
#' getNumNaMeth-methods
#'
#' for each site/region, the getNumNaMeth retrieves the number of NA values accross all samples.
#' Does this efficiently by breaking down the methylation matrix into submatrices
#' @param object object inheriting from \code{\linkS4class{RnBSet}}
#' @param type "sites" or region type
#' @param chunkSize size of each submatrix (performance tuning parameter)
#' @param mask logical matrix. its entries will also be considered NAs in counting
#' @return vector containing the number of NAs per site/region
#'
#' @rdname getNumNaMeth-methods
#' @docType methods
#' @aliases getNumNaMeth
#' @aliases getNumNaMeth,RnBSet-method
setMethod("getNumNaMeth", signature(object = "RnBSet"),
	function(object, type="sites", chunkSize=1e5, mask=NULL) {
		if (!(is.character(type) && length(type) == 1 && (!is.na(type)))) {
			stop("invalid value for type")
		}
		if (!(type %in% c("sites", object@target, names(object@regions)))) {
			stop("unsupported region type")
		}
		#get start and end indices for the chunks
		n <- nsites(object, type)
		indStarts <- seq(1,n,by=chunkSize)
		indEnds <- c(indStarts[-1]-1, n)
		#apply to each chunk
		res <- unlist(lapply(1:length(indStarts), FUN=function(i){
			indsCur <- indStarts[i]:indEnds[i]
			mm <- meth(object, type=type, i=indsCur)
			isNaMat <- is.na(mm)
			if (!is.null(mask)) isNaMat <- isNaMat | mask[indsCur,]
			return(as.integer(rowSums(isNaMat)))
		}))
		
		return(res)
	}
)
if(!isGeneric("isImputed")) setGeneric("isImputed", function(object, ...) standardGeneric("isImputed"))
#' isImputed
#' 
#' Getter for the imputation field. Return TRUE, if the object has been imputed and FALSE otherwise.
#' @param object Object for which the information should be returned
#' @return TRUE, if the object has been imputed and FALSE otherwise.
#' @author Michael Scherer
#' @aliases isImputed
#' @aliases isImputed,RnBSet-method
#' @export
setMethod("isImputed",signature(object="RnBSet"),
  function(object){
    if(.hasSlot(object,"imputed")){
      return(object@imputed)
    }
    return(FALSE)
  }          
)

########################################################################################################################
#' rnb.sample.summary.table
#'
#' Creates a sample summary table from an RnBSet object
#'
#' @param rnbSet \code{\linkS4class{RnBSet}} of interest.
#' @return a summary table (as data.frame) with the following variables for each sample (rows):
#' \item{sampleName}{Name of the sample}
#' \item{*_num (* can be 'sites' or a region type)}{Number of sites or regions with coverage in the sample}
#' \item{*_covgMean (\code{RnBiseqSet} only)}{Mean coverage of sites or regions in the sample}
#' \item{*_covgMedian (\code{RnBiseqSet} only)}{Median coverage of sites or regions in the sample}
#' \item{*_covgPerc25 (\code{RnBiseqSet} only)}{25 percentile of coverage of sites or regions in the sample}
#' \item{*_covgPerc75 (\code{RnBiseqSet} only)}{75 percentile of coverage of sites or regions in the sample}
#' \item{*_numCovg5,10,30,60 (\code{RnBiseqSet} only)}{Number of sites or regions with coverage greater or equal to 5,10,30,60}
#' \item{sites_numDPval5em2,1em2,1em3 (\code{RnBeadSet} only)}{Number of sites with a detection p-value smaller than 0.05,0.01,0.001}
#' \item{**_numSitesMean (** is any region type)}{Mean number of sites in a region}
#' \item{**_numSitesMedian}{Median number of sites in a region}
#' \item{**_numSites2,5,10,20}{Number of regions with at least 2,5,10,20 sites with valid methylation measurements}
#' @author Fabian Mueller
#' @aliases rnb.sample.summary.table,RnBSet-method
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' rnb.sample.summary.table(rnb.set.example)
#' }
rnb.sample.summary.table <- function(rnbSet) {
	is.biseq <- "RnBiseqSet" %in% class(rnbSet)
	is.beads <- "RnBeadSet" %in% class(rnbSet)
	df.empty <- data.frame(matrix(nrow=length(samples(rnbSet)),ncol=0))
	rownames(df.empty) <- samples(rnbSet)
	tt <- data.frame(df.empty,sampleName=samples(rnbSet),stringsAsFactors=FALSE)
	reg.types.regions <- summarized.regions(rnbSet)
	reg.types <- c("sites",reg.types.regions)
	for (rr in reg.types){
		# logger.status(c("Region type:",rr))
		# rnb.cleanMem()
		tt.cur <- df.empty
		tt.cur$num <- sampleMethApply(rnbSet, function(x){sum(!is.na(x))}, type=rr)
		if (is.biseq){
			covgStats <- do.call("rbind", lapply(1:length(samples(rnbSet)), FUN=function(j){
				# logger.status(c("  Sample:",j))
				mm <- as.vector(meth(rnbSet, rr, j=j))
				cc <- as.vector(covg(rnbSet, rr, j=j))
				cc[cc==0] <- NA
				cc[is.na(mm)] <- NA
				qq <- quantile(cc, probs = c(0.25,0.75), na.rm=TRUE)
				res <- c(
					mean(cc, na.rm=TRUE),
					median(cc, na.rm=TRUE),
					qq[1],
					qq[2],
					sum(cc>=5, na.rm=TRUE),
					sum(cc>=10, na.rm=TRUE),
					sum(cc>=30, na.rm=TRUE),
					sum(cc>=60, na.rm=TRUE)
				)
				return(res)
			}))
			colnames(covgStats) <- c("covgMean", "covgMedian", "covgPerc25", "covgPerc75", "numCovg5", "numCovg10", "numCovg30", "numCovg60")
			tt.cur <- cbind(tt.cur, covgStats)

		}
		if (is.beads){
			if (rr == "sites"){
				pp <- dpval(rnbSet,type=rr)
				if (!is.null(pp)) {
					tt.cur$numDPval5em2 <- colSums(pp < 5e-2, na.rm=TRUE)
					tt.cur$numDPval1em2 <- colSums(pp < 1e-2, na.rm=TRUE)
					tt.cur$numDPval1em3 <- colSums(pp < 1e-3, na.rm=TRUE)
				}
			}
		}

		if (rr %in% reg.types.regions){
			regions2sites <- regionMapping(rnbSet,region.type=rr)
			#compute the number of sites per region and sample
			nsamples <- length(samples(rnbSet))
			num.sites <- sapply(1:nsamples,function(i){
				# logger.status(c("  Sample:",i))
				mm.s.nna <- !is.na(as.vector(meth(rnbSet, j=i)))
				sapply(1:nsites(rnbSet, rr),function(j){
					sum(mm.s.nna[regions2sites[[j]]])
				})
			})
			# num.sites2 <- t(sapply(1:nsites(rnbSet, rr),function(i){
			# 	# logger.status(c("  Site/Region:",i))
			# 	colSums(!is.na(meth(rnbSet, i=regions2sites[[i]])))
			# })) # a bit slower, but more memory effective, if to include later, check the code again
			tt.cur$numSitesMean   <- colMeans(num.sites, na.rm=TRUE)
			tt.cur$numSitesMedian <- colMedians(num.sites, na.rm=TRUE)
			tt.cur$numSites2  <- colSums(num.sites>=2, na.rm=TRUE)
			tt.cur$numSites5  <- colSums(num.sites>=5, na.rm=TRUE)
			tt.cur$numSites10 <- colSums(num.sites>=10,na.rm=TRUE)
			tt.cur$numSites20 <- colSums(num.sites>=20,na.rm=TRUE)
		}
		colnames(tt.cur) <- paste(rr,colnames(tt.cur),sep="_")

		tt <- data.frame(tt,tt.cur)
	}
	return(tt)
}
########################################################################################################################

test_report_2 <- function() {

	report.dir <- "RnBeads_report_testing"

	rnb.initialize.reports(report.dir)
	report <- createReport(file.path(report.dir, "myreport.html"), "An Eye Opener", page.title = "Fascinating Report", authors = c("Me","You"))

	stext <- "Here is some text for our awesome report"
	report <- rnb.add.section(report, "Adding a text section", stext)
	lorem1 <- c("Lorem ipsum dolor sit amet, consectetur adipiscing elit. Maecenas vestibulum placerat lobortis.
				Ut viverra fringilla urna at rutrum. In hac habitasse platea dictumst.
				Vestibulum adipiscing rutrum libero et interdum. Etiam sed odio ac nibh ullamcorper congue.
				Proin ac ipsum elit. Ut porta lorem sed lorem molestie pharetra.",
				"Vestibulum ante ipsum primis in faucibus orci luctus et ultrices posuere cubilia Curae;
				Cras ac augue eu turpis dignissim aliquam. Vivamus at arcu ligula, vel scelerisque nisi.
				Vivamus ac lorem libero, quis venenatis metus. Fusce et lectus at lectus vestibulum
				faucibus ac id sem.")
	report <- rnb.add.section(report,"A subsection",lorem1,level=2)
	lorem2 <- "Nunc congue molestie fringilla. Aliquam erat volutpat.
	Integer consequat turpis nec dolor pulvinar et vulputate magna adipiscing.
	Curabitur purus dolor, porttitor vel dapibus quis, eleifend at lacus.
	Cras at mauris est, quis aliquam libero. Nulla facilisi.
	Nam facilisis placerat aliquam. Morbi in odio non ligula mollis rhoncus
	et et erat. Maecenas ut dui nisl. Mauris consequat cursus augue quis euismod."
	rnb.add.paragraph(report,lorem2)

	rnb.add.paragraph(report,"TODO: Add more content here",paragraph.class="task")
	rnb.add.paragraph(report,"To be, or not to be, that is the question",paragraph.class="note")
	ll <- lapply(LETTERS[1:10],FUN=function(x){paste(rep(x,3),collapse="")})
	rnb.add.list(report,ll,type="u")
	rnb.add.list(report,ll,type="o")
	tt <- matrix(sapply(LETTERS[1:24],FUN=function(x){paste(rep(x,3),collapse="")}),ncol=4)
	colnames(tt) <- paste("Col",1:4)
	rownames(tt) <- paste("Row",1:6)
	rnb.add.table(report,tt,row.names = TRUE, first.col.header = TRUE,
			  tcaption = "A table")

	stext <- c("<p>Some German umlauts:
			   <ul>
			   <li>&Auml;</li>
			   <li>&Ouml;</li>
			   <li>&Uuml;</li>
			   </ul>
			   </p>",
			   '<p> A link: <a href="http://rnbeads.org/">RnBeads website</a></p>')
	report <- rnb.add.section(report, "HTML code", stext)

	report <- rnb.add.section(report, 'Plots', "")
	report.plot <- createReportPlot("hist_normal",report,create.pdf=TRUE,high.png=200)
	hist(rnorm(1000),breaks=50,col="blue")
	off(report.plot)
	desc <- 'A histogramm of samples drawn from the normal distribution
	Click <a href="myreport_pdfs/hist_normal.pdf">
	here</a> for the pdf version.'
	report <- rnb.add.figure(report, desc, report.plot)

	plotSine <- function(a,b,fname){
		report.plot <- createReportPlot(fname,report,create.pdf=FALSE,high.png=200)
		curve(sin(a*x)+b, -2*pi, 2*pi,ylim=c(-2,2),col="blue",lwd=2)
		off(report.plot)
		return(report.plot)
	}

	period.params <- c(a05=0.5,a1=1,a2=2)
	intercept.params <- c(im1=-1,i0=0,i1=1)
	plot.list <- list()
	for (aa in names(period.params)){
		for (bb in names(intercept.params)){
			fname <- paste("sinePlot",aa,bb,sep="_")
			current.plot <- plotSine(period.params[aa],intercept.params[bb],fname)
			plot.list <- c(plot.list,list(current.plot))
		}
		
	}
	setting.names <- list('period parameter' = period.params,
						  'intercept parameter' = intercept.params)
	description <- 'Sine for various parameters'
	report <- rnb.add.figure(report, description, plot.list, setting.names, selected.image=5)

	off(report)

	unlink(report.dir,recursive=TRUE)
}

########################################################################################################################
## ReportPlot-class.R
## created: 2012-04-16
## creator: Yassen Assenov
## ---------------------------------------------------------------------------------------------------------------------
## ReportPlot class definition.
########################################################################################################################

## C L A S S ###########################################################################################################

#' ReportPlot Class
#'
#' Information about the files created to store one generated plot in a report. Report plots are initialized using the
#' function \code{\link{createReportPlot}}.
#'
#' @section Slots:
#' \describe{
#'   \item{\code{fname}}{Relative file name. It does not include path or extension.}
#'   \item{\code{width}}{Width of the image in inches.}
#'   \item{\code{height}}{Height of the image in inches.}
#'   \item{\code{create.pdf}}{Flag indicating if a PDF image is created.}
#'   \item{\code{low.png}}{Resolution, in dots per inch, used for the figure image.}
#'   \item{\code{high.png}}{Resolution, in dots per inch, used for the high-resolution image.}
#'   \item{\code{dir.pdf}}{Directory that contains the generated PDF file.}
#'   \item{\code{dir.png.low}}{Directory that contains the generated figure image file.}
#'   \item{\code{dir.png.high}}{Directory that contains the generated high-resolution image file.}
#' }
#'
#' @section Methods and Functions:
#' \describe{
#'   \item{\code{\link{get.files}}}{Gets the list of all files that are planned to be generated,
#'        or were already generated by the report plot.}
#'   \item{\code{\link[=off,ReportPlot-method]{off}}}{Copies the figure to a PNG file (if needed) and closes the device
#'        associated with the report plot.}
#' }
#'
#' @name ReportPlot-class
#' @rdname ReportPlot-class
#' @aliases initialize,ReportPlot-method
#' @author Yassen Assenov
#' @exportClass ReportPlot
setClass("ReportPlot",
	representation(fname = "character", width = "numeric", height = "numeric",
		create.pdf = "logical", low.png = "integer", high.png = "integer",
		dir.pdf = "character", dir.png.low = "character", dir.png.high = "character"),
	prototype = prototype(fname = "temp", width = 7, height = 7,
		create.pdf = TRUE, low.png = as.integer(100), high.png = as.integer(0),
		dir.pdf = ".", dir.png.low = ".", dir.png.high = "."),
	package = "RnBeads")


#' ReportGgPlot Class
#'
#' Information about the files created to store one generated plot in a report. Report plots are initialized using the
#' function \code{\link{createReportGgPlot}}. It inherits from the \code{\linkS4class{ReportPlot}} class and handling is
#' analogous, except that it contains an additional slot to store a \code{ggplot} object.
#'
#' @section Slots:
#' \describe{
#'   \item{\code{ggp}}{\code{ggplot} object to be printed}
#' }
#' @section Notes:
#' No device is being opened until \code{off(reportGgPlot)} is called.
#'
#' @name ReportGgPlot-class
#' @rdname ReportGgPlot-class
#' @aliases initialize,ReportGgPlot-method
#' @author Fabian Mueller
#' @exportClass ReportGgPlot
setClass("ReportGgPlot",
		slots= c(ggp="ANY"),
		contains="ReportPlot",
		prototype = prototype(ggp=ggplot()),
		package = "RnBeads")

########################################################################################################################
## normalization.R
## created: 2012-05-31
## creator: Pavlo Lutsik
## ---------------------------------------------------------------------------------------------------------------------
## Implementation of the normalization step.
########################################################################################################################

## F U N C T I O N S ###################################################################################################

#' rnb.execute.normalization
#'
#' Performs normalization of the provided HumanMethylation450 data set.
#'
#' @param object Methylation dataset as an object of type \code{\linkS4class{MethyLumiSet}} or
#'               \code{\linkS4class{RnBSet}}.
#' @param method Normalization method, must be one of \code{"none"}, \code{"illumina"}, \code{"swan"},
#'               \code{"minfi.funnorm"}, \code{"bmiq"}, or \code{wm.*} where \code{*} stands for one of the methods
#'               implemented in \pkg{wateRmelon} package.
#'               Note that the execution of methods SWAN and minfi.funnorm requires packages \pkg{minfi} and
#'               \pkg{IlluminaHumanMethylation450kmanifest}. The BMIQ method requires the package \pkg{RPMM}. The
#'               \code{wm.*} methods naturally require \pkg{wateRmelon}.
#' @param bgcorr.method Character singleton specifying which background subtraction should be used. Only methods impemented
#'               in the \pkg{methylumi} package are supported at the moment, namely \code{methylumi.noob}, \code{methylumi.goob}
#'               and \code{methylumi.doob}. See Triche et al. for detailed description of the methods.
#' @param verbose flag specifying whether diagnostic output should be written to the console or to the RnBeads logger
#' 				 in case the latter is initialized
#'
#' @return Normalized dataset as an object of type \code{\linkS4class{RnBeadSet}}.
#'
#' @references 1. Triche, Timothy J., Jr., Weisenberger, Daniel J., Van Den Berg, David, Laird, Peter W. and Siegmund, Kimberly D. (2013)
#'             Low-level processing of Illumina Infinium DNA Methylation BeadArrays.
#'             Nucleic Acids Research 41(7):e90-e90.
#'
#' @author Pavlo Lutsik
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' rnb.set.norm<-rnb.execute.normalization(rnb.set.example, method="illumina", bgcorr.method="none")
#' }
#' @export
rnb.execute.normalization<-function(
		object,
		method=rnb.getOption("normalization.method"),
		bgcorr.method=rnb.getOption("normalization.background.method"),
		verbose=TRUE){

	if(!(inherits(object,"MethyLumiSet") || inherits(object,"RnBSet"))) {
		stop("invalid value for object; expected MethyLumiSet, RnBeadSet or RnBiseqSet")
	}
	accepted <- .rnb.options[["accepted"]][["normalization.method"]]
	accepted <- c(accepted, paste0(setdiff(accepted, c("none", "bmiq")), "+bmiq"))
	if (!(is.character(method) && length(method) == 1 && isTRUE(method %in% accepted))) {
		msg <- paste0('"', accepted, '"', collapse = ", ")
		stop(paste("invalid value for method; expected one of", msg))
	}
	accepted <- .rnb.options[["accepted"]][["normalization.background.method"]]
	if (!(is.character(bgcorr.method) && length(bgcorr.method) == 1 && isTRUE(bgcorr.method %in% accepted))) {
		stop("invalid value for bgcorr.method")
	}

	## Split the double normalization
	if (grepl("\\+bmiq$", method)) {
		method <- substr(method, 1, nchar(method) - 5)
		secondary.bmiq <- TRUE
	} else {
		secondary.bmiq <- FALSE
	}
	method.to.set <- method
	bgcorr.method.to.set <- bgcorr.method
	disable.method <- function(is.method, txt = '') {
		txt <- paste0('Incompatible dataset and ', ifelse(is.method, 'normalization', 'background correction'),
					  ' method', ifelse(txt == '', '', ': '), txt, '. Changed the method to "none"')
		rnb.warning(txt)
		if (is.method) {
		  rnb.options(normalization.method = "none")
			method <<- "none"
			method.to.set <<- object@status$normalized
			secondary.bmiq <<- FALSE
		} else {
			rnb.options(normalization.background.method = "none")
			bgcorr.method <<- "none"
			bgcorr.method.to.set <<- object@status$background
		}
	}

	## Ignore normalization for bisulfite sequencing datasets
	if (inherits(object, "RnBiseqSet")) {
		if (method != "none") {
			disable.method(TRUE)
		}
		if (bgcorr.method != "none") {
			disable.method(FALSE)
		}
		return(object)
	}

	## Validate there are enough samples present
	if (inherits(object, "MethyLumiSet")) {
		nsamples <- ncol(exprs(object))
	} else {
		nsamples <- ncol(object@meth.sites)
	}
	accepted <- c("none", "bmiq")
	if (nsamples < 2 && !(method %in% accepted)) {
		disable.method(TRUE, 'too few samples')
	}
	rm(nsamples)

	if (inherits(object, "RnBeadSet")) {
		if (object@status$normalized != "none") {
			## Allow only BMIQ as a secondary normalization
			if (method == "bmiq" && (!grepl("bmiq$", object@status$normalized))) {
				method.to.set <- paste0(object@status$normalized, "+bmiq")
			} else if (method != "none") {
				disable.method(TRUE, 'dataset already normalized')
			}
		} else if (!(method %in% c("none", "bmiq") || inherits(object, "RnBeadRawSet"))) {
			disable.method(TRUE, 'missing intensity data')
		} else if (method == "illumina" && is.null(qc(object))) {
			disable.method(TRUE, 'missing data on quality control probes')
		}
	}

	## Perform background subtraction
	if (bgcorr.method != "none") {
		if (inherits(object, "MethyLumiSet")) {

			if (annotation(object) == "IlluminaHumanMethylation27") {
				disable.method(FALSE, 'not supported for Infinium 27k')
			} else if (grepl("enmix", bgcorr.method)) {
				disable.method(FALSE, 'not supported for MethyLumiSet')
			} else if (bgcorr.method == "methylumi.noob") {
				if (!all(c("methylated.OOB", "unmethylated.OOB") %in% ls(object@assayData))) {
					disable.method(FALSE, 'missing out-of-band intensities')
				}
			}

			if (grepl("methylumi", bgcorr.method)) {
				bgcorr.methylumi<-gsub("methylumi\\.", "", bgcorr.method)
				pheno.columns<-colnames(phenoData(object)@data)
				suppressMessages({
					sinkfile<-ifelse("Windows" %in% Sys.info(),"NUL", "/dev/null")
					sink(sinkfile)
					object<-methylumi.bgcorr(object, method=bgcorr.methylumi)
					sink()
				})
				#removing the introduced columns
				phenoData(object)<-phenoData(object)[,pheno.columns]
			}

		} else if (inherits(object, "RnBeadRawSet")) {

			if (object@target == "probes27") {
				disable.method(FALSE, 'not supported for Infinium 27k')
			} else if (bgcorr.method == "methylumi.noob" && object@target == "probesEPICv2") {
				disable.method(FALSE, 'methylumi.noob is not supported for MethylationEPICv2')
            } else if (bgcorr.method == "methylumi.noob" && object@target == "probesEPIC") {
				disable.method(FALSE, 'methylumi.noob is not supported for MethylationEPIC')
            } else if (bgcorr.method == "methylumi.noob" && object@target == "probesMMBC") {
                disable.method(FALSE, 'methylumi.noob is not supported for MethylationMMBC')
			} else if (grepl("oob$", bgcorr.method) && (is.null(M0(object)) || is.null(U0(object)))) {
				disable.method(FALSE, 'missing out-of-band intensities')
			} else if (object@status$normalized == "swan") {
				disable.method(FALSE, 'dataset already normalized using SWAN')
			}

			if (bgcorr.method=="subtraction"){
                object<-rnb.bgcorr.subtr(object)
            }else if (any(grepl("sesame.noob", bgcorr.method))){
                if(bgcorr.method=="sesame.noobsb"){
                    object<-noob.sesame(object, background="bleed-through")
                }else{
                    object<-noob.sesame(object, background="internal")
                }
            }else if (any(grepl("methylumi", bgcorr.method))) {
				bgcorr.methylumi<-gsub("methylumi\\.", "", bgcorr.method)
				pheno.columns<-colnames(pheno(object))
				inferred.covariates <- object@inferred.covariates
				old.obj<-object
				object<-as(object, "MethyLumiSet")
				if(isTRUE(old.obj@status$discard.ff.matrices)){
					rnb.call.destructor(old.obj)
					rm(old.obj)
				}
				rnb.cleanMem()
				suppressMessages({
					sinkfile <- ifelse("Windows" %in% Sys.info(), "NUL", "/dev/null")
					sink(sinkfile)
					object<-methylumi.bgcorr(object, method=bgcorr.methylumi)
					sink()
				})
				#removing the introduced columns
				phenoData(object)<-phenoData(object)[,pheno.columns]
				object<-as(object, "RnBeadRawSet")
				if(object@status$disk.dump && rnb.getOption("enforce.destroy.disk.dumps")){
					object@status$discard.ff.matrices<-TRUE
				}
			} else if (any(grepl("enmix", bgcorr.method))) {
				bgcorr.enmix<-gsub("enmix\\.", "", bgcorr.method)
				object<-rnb.enmix.oob(object)
			}

			object@status$background <- bgcorr.method
		}
		if (verbose) {
			rnb.status(c("Performed background subtraction with method", bgcorr.method))
		}
		rnb.cleanMem()
	}

	## Validate the normalization method is supported for the given data type
	accepted <- c("none", "illumina", "methylumi.illumina")
	if (inherits(object, "MethyLumiSet") && annotation(object) == "IlluminaHumanMethylation27" &&
		!(method %in% accepted)) {
		disable.method(TRUE, 'not supported for Infinium 27k MethyLumiSet')
	}
	if (inherits(object, "RnBeadSet") && object@target == "probes27" && !(method %in% accepted)) {
		disable.method(TRUE, 'not supported for Infinium 27k')
	}
	accepted <- c("none", "scaling", "scaling.internal", "scaling.reference", "bmiq", "swan", "minfi.funnorm", "wm.dasen")

	if (inherits(object, "RnBeadSet") && object@target == "probesEPIC" && !(method %in% accepted)) {
		disable.method(TRUE, 'not supported for HumanMethylationEPIC')
	}
	if (inherits(object, "RnBeadSet") && object@target == "probesEPICv2" && !(method %in% accepted)) {
		disable.method(TRUE, 'not supported for HumanMethylationEPICv2')
	}
    accepted<-setdiff(.rnb.options[["accepted"]][["normalization.method"]], c("illumina", "minfi.funnorm", "swan", "wm.tost","wm.fuks", "wm.swan"))
    if (inherits(object, "RnBeadSet") && object@target == "probesMMBC" && !(method %in% accepted)) {
        disable.method(TRUE, 'not supported for Mouse Methylation Bead Chip')
    }
    
	## Perform normalization
	if (any(grepl("scaling", method))) {
        
        if(method=="scaling") {
            sc.method="internal"
        }else{
            sc.method<-gsub("scaling.", "", method)
        }
        
        object<-rnb.norm.scaling(object, method=sc.method)
        
        object@status$normalized<-"scaling"
        object@status$background<-bgcorr.method
        rnb.cleanMem()
        
    } else if (method=="illumina" || method == "methylumi.illumina") {
		if (inherits(object, "RnBeadRawSet")) {
			inferred.covariates <- object@inferred.covariates
			object <- as(object, "MethyLumiSet")
		}
		object <- suppressMessages(as(normalizeMethyLumiSet(object), "RnBeadRawSet"))
		object@status$normalized<-"illumina"
		object@status$background<-bgcorr.method
		rnb.cleanMem()

	}else if (method=="swan"){

		rnb.require("minfi")
		rnb.require("IlluminaHumanMethylation450kmanifest") 
		if(inherits(object,"MethyLumiSet") && (is.null(methylated(object))||is.null(unmethylated(object)))) {
			rnb.error("Invalid value for object; missing intensity information")
		}

		rga <- c("IlluminaHumanMethylationEPIC", "ilm10b2.hg19", "IlluminaHumanMethylation450k", "ilmn12.hg19")
		rga <- matrix(rga, 2, 2, TRUE, list(c("EPIC", "450"), c("array", "annotation")))
		if(inherits(object,"MethyLumiSet")){
			intensities.by.channel<-methylumi.intensities.by.color(object)
			## FIXME: Update this MethyLumiSet can contain EPIC data as well
			rga <- rga["450", ]
		}else if(inherits(object,"RnBeadRawSet")){
			intensities.by.channel<-intensities.by.color(object)
			rga <- rga[gsub("^probes", "", object@target), ]
		}
		if (grepl("EPIC", rga[1])) {
			rnb.require("IlluminaHumanMethylationEPICmanifest")
		} else {
			rnb.require("IlluminaHumanMethylation450kmanifest")
		}

		rg.set<-RGChannelSet(intensities.by.channel$Cy3, intensities.by.channel$Cy5)
		annotation(rg.set) <- rga
		suppressMessages({
				sinkfile<-ifelse("Windows" %in% Sys.info(), "NUL", "/dev/null")
				sink(sinkfile); methyl.set<-preprocessSWAN(rg.set); sink()
		})

		meth.minfi<-getMeth(methyl.set)
		umeth.minfi<-getUnmeth(methyl.set)

		if(inherits(object, "MethyLumiSet")){
			methylated(object)<-meth.minfi[match(rownames(meth.minfi), featureNames(object)),]#+methylated(object)[setdiff(featureNames(object), rownames(meth.minfi)),]
			unmethylated(object)<-umeth.minfi[match(rownames(umeth.minfi), featureNames(object)),]#+unmethylated(object)[setdiff(featureNames(object), rownames(umeth.minfi)),]
			rm(rg.set,methyl.set, meth.minfi, umeth.minfi)
			betas(object)<-rbind(methylated(object)/(methylated(object)+unmethylated(object)),
				betas(object)[setdiff(featureNames(object), rownames(methylated(object))),])
			object<-as(object, "RnBeadSet")
		}else if(inherits(object, "RnBeadRawSet")){
			probe.ids<-rownames(annotation(object))
			## rs probes are "lost" during SWAN

			meth.minfi<-meth.minfi[rownames(meth.minfi) %in% probe.ids,]
			umeth.minfi<-umeth.minfi[rownames(umeth.minfi) %in% probe.ids,]

			object@M[,][match(rownames(meth.minfi),probe.ids),]<-meth.minfi
			object@U[,][match(rownames(umeth.minfi),probe.ids),]<-umeth.minfi

			#update.meth(object)
			if(object@status$disk.dump){
				object@meth.sites<-convert.to.ff.matrix.tmp(beta.value(object@M[,], object@U[,]))
			}else{
				object@meth.sites<-beta.value(object@M[,,drop=FALSE], object@U[,,drop=FALSE])
			}
			rm(rg.set,methyl.set, meth.minfi, umeth.minfi)
		}

	} else if (method == "bmiq") {

		object <- rnb.execute.normalization.bmiq(object)

	}else if(grepl("wm\\.",method)[1]){

		rnb.require("wateRmelon")
		wm.method<-gsub("wm\\.","", method)

		if(inherits(object, "MethyLumiSet")){

			object<-do.call(wm.method, list(object))

			object<-as(object, "RnBeadSet")

		}else if(inherits(object, "RnBeadRawSet")){

			if(wm.method %in% c("betaqn", "fuks")){

				ann<-annotation(object, add.names=TRUE)

				if(wm.method=="betaqn"){
					betas.norm<-do.call(wm.method, list(meth(object)))
				}else{
					colnames(ann)[5]<-"DESIGN"
					betas.norm<-do.call(wm.method, list(meth(object), anno=ann))
				}
				object<-as(object, "RnBeadSet")
				if(object@status$disk.dump){
					object@meth.sites<-convert.to.ff.matrix.tmp(betas.norm)
				}else{
					object@meth.sites<-betas.norm
				}
				rm(betas.norm)

			}else{

				Mmatrix<-M(object, row.names=TRUE)
				Umatrix<-U(object, row.names=TRUE)

				ann.full<-rnb.annotation2data.frame(rnb.get.annotation(object@target, object@assembly))
				ann<-annotation(object, add.names=TRUE)
				probe.names<-ann[["ID"]]

				if(nrow(Mmatrix)<nrow(ann.full)){
					filler<-matrix(NA_real_, nrow=nrow(ann.full)-nrow(ann), ncol=length(samples(object)))
					rownames(filler)<-rownames(ann.full[!rownames(ann.full) %in% rownames(ann),])
					ann<-rbind(ann, ann.full[!rownames(ann.full) %in% rownames(ann),colnames(ann.full)])
					Mmatrix<-rbind(Mmatrix, filler)
					Umatrix<-rbind(Umatrix, filler)
					rm(filler)
				}
				rm(ann.full)
				rownames(Mmatrix)<-ann[["ID"]]
				rownames(Umatrix)<-ann[["ID"]]

				if(wm.method %in% c("swan")){

					if(wm.method %in% c("swan")){
						betas.norm<-suppressMessages({do.call(wm.method, list(Mmatrix, Umatrix,qc=qc(object)))})
						rnb.warning("Methylation calls at SNP-tagging probes were not normalized")
					}

#					rownames(betas.norm)<-probe.names
#					qcl<-qc(object)
#					object<-new("RnBeadSet",
#							pheno(object),
#							betas.norm[1:nrow(meth(object)),],
#							dpval(object, row.names=TRUE),
#							covg(object, row.names=TRUE))
#					qc(object)<-qcl
					object<-as(object, "RnBeadSet")
					betas.new<-meth(object)
					betas.new[match(rownames(betas.norm), probe.names),]<-betas.norm
					if(object@status$disk.dump){
						object@meth.sites<-convert.to.ff.matrix.tmp(betas.new)
					}else{
						object@meth.sites<-betas.new
					}
					rm(betas.norm, betas.new)

				}else if(wm.method %in% c("tost")){
					cgi.relation<-rep("Sea",nrow(ann))
					cgi.relation[unlist(regionMapping(object, "cpgislands"))]<-"Island"
					ann.tost<-data.frame(
							TargetID=ann[["ID"]],
							#TargetID=as.character(1:nrow(ann)),
							COLOR_CHANNEL=ann[["Color"]],
							INFINIUM_DESIGN_TYPE=ann[["Design"]],
							RELATION_TO_UCSC_CPG_ISLAND=cgi.relation,
							CHROMOSOME=ann[["Chromosome"]],
							POSITION=ann[["Start"]],
							row.names=rownames(ann))
					betas.norm<-do.call(wm.method, list(Mmatrix, Umatrix,
									da=ann.tost, pn=dpval(object)))
					object<-as(object, "RnBeadSet")
					betas.new<-meth(object)
					betas.new[match(rownames(betas.norm), probe.names),]<-betas.norm
					if(object@status$disk.dump){
						object@meth.sites<-convert.to.ff.matrix.tmp(betas.new)
					}else{
						object@meth.sites<-betas.new
					}
					rm(betas.norm, betas.new)

				}else{

					if(wm.method %in% c("nasen", "naten", "nanet")){
						list.norm<-do.call(wm.method, list(Mmatrix, Umatrix,
										ann[,"Design"], ret2=TRUE))
					}else{
						list.norm<-do.call(wm.method, list(Mmatrix, Umatrix,
										ann[,"Design"], ret2=TRUE, roco=pheno(object)[,grep("Sentrix[ |_]Position", colnames(pheno(object)))]))
					}
					object@M[,]<-list.norm[[1]][1:nrow(meth(object)),]
					object@U[,]<-list.norm[[2]][1:nrow(meth(object)),]
					rm(list.norm)
					#update.meth(object)
					if(object@status$disk.dump){
						object@meth.sites<-convert.to.ff.matrix.tmp(beta.value(object@M[,], object@U[,]))
					}else{
						object@meth.sites<-beta.value(object@M[,], object@U[,])
					}
				}
			}
		}

	}else if(method == "minfi.funnorm"){

		rnb.require("minfi")
		rnb.require("IlluminaHumanMethylation450kmanifest")
		rnb.require("IlluminaHumanMethylation450kanno.ilmn12.hg19")
		if(inherits(object,"MethyLumiSet") && (is.null(methylated(object))||is.null(unmethylated(object)))) {
			rnb.error("Invalid value for object; missing intensity information")
		}

		if(inherits(object,"MethyLumiSet")){

			if(nrow(methylated(object))<sum(elementNROWS(rnb.get.annotation("probes450"))) ||
					sum(is.na(methylated(object)))>0 || sum(is.na(unmethylated(object)))>0){
				rnb.warning("Funtional normalization is only supported for unfiltered data sets where intensity values are present for all probes. Skipping normalization")
				object<-as(object, "RnBeadRawSet")
				object@status$normalized<-"none"
				object@status$background<-bgcorr.method
				rnb.cleanMem()
				return(object)
			}
			intensities.by.channel<-methylumi.intensities.by.color(object)

		}else if(inherits(object,"RnBeadRawSet")){

			if(nrow(sites(object))<sum(elementNROWS(rnb.get.annotation("probes450"))) ||
					sum(is.na(M(object)))>0 || sum(is.na(U(object)))>0){
				rnb.warning("Funtional normalization is only supported for unfiltered data sets where intensity values are present for all probes. Skipping normalization")
				object@status$normalized<-"none"
				object@status$background<-bgcorr.method
				rnb.cleanMem()
				return(object)
			}
			intensities.by.channel<-intensities.by.color(object)
		}

		rg.set<-RGChannelSet(intensities.by.channel$Cy3, intensities.by.channel$Cy5)
		annotation(rg.set)<-c(array="IlluminaHumanMethylation450k", annotation=minfi:::.default.450k.annotation)
		suppressMessages({
					#sinkfile<-ifelse("Windows" %in% Sys.info(),"NUL", "/dev/null")
					#sink(sinkfile);
					#methyl.set<-preprocessFunnorm(rg.set);
					rg.set <- updateObject(rg.set)
					gmSet <- mapToGenome(rg.set)
					extractedData <- minfi:::.extractFromRGSet450k(rg.set)

					gmSet <- addSex(gmSet, getSex(gmSet, cutoff = -3))
					sex <- rep(1L, length(gmSet$predictedSex))
					sex[gmSet$predictedSex == "F"] <- 2L

					rm(rg.set)
					CN <- getCN(gmSet)
					methyl.set <- minfi:::.normalizeFunnorm450k(object = gmSet,
							extractedData = extractedData,
							sex = NULL, nPCs = 2, verbose = 0)

					#sink()
				})

		meth.minfi<-getMeth(methyl.set)
		umeth.minfi<-getUnmeth(methyl.set)

		meth.minfi<-pmax(meth.minfi, 0)
		umeth.minfi<-pmax(umeth.minfi, 0)

		if(inherits(object, "MethyLumiSet")){

			methylated(object)<-meth.minfi[match(rownames(meth.minfi), featureNames(object)),]#+methylated(object)[setdiff(featureNames(object), rownames(meth.minfi)),]
			umeth.minfi<-getUnmeth(methyl.set)
			unmethylated(object)<-umeth.minfi[match(rownames(umeth.minfi), featureNames(object)),]#+unmethylated(object)[setdiff(featureNames(object), rownames(umeth.minfi)),]
			rm(methyl.set, meth.minfi, umeth.minfi)
			betas(object)<-rbind(methylated(object)/(methylated(object)+unmethylated(object)),
					betas(object)[setdiff(featureNames(object), rownames(methylated(object))),])
			object<-as(object, "RnBeadSet")

		}else if(inherits(object, "RnBeadRawSet")){

			probe.ids<-rownames(annotation(object, add.names=TRUE))
			## rs probes are "lost" during SWAN

			meth.minfi<-meth.minfi[rownames(meth.minfi) %in% probe.ids,]
			umeth.minfi<-umeth.minfi[rownames(umeth.minfi) %in% probe.ids,]

			object@M[,][match(rownames(meth.minfi),probe.ids),]<-meth.minfi
			object@U[,][match(rownames(umeth.minfi),probe.ids),]<-umeth.minfi

			#update.meth(object)
			if(object@status$disk.dump){
				object@meth.sites<-convert.to.ff.matrix.tmp(beta.value(object@M[,], object@U[,]))
			}else{
				object@meth.sites<-beta.value(object@M[,], object@U[,])
			}
			rm(methyl.set, meth.minfi, umeth.minfi)
		}

	} else { # method == "none"
		if (inherits(object, "MethyLumiSet")) {
			object <- as(object, "RnBeadSet")
		}
	}

	## Display
	if (method != "none") {
		if (verbose) {
			rnb.status(c("Performed normalization with method", method))
		}
		rnb.cleanMem()
	}

	## Apply BMIQ as a secondary normalization method
	if (secondary.bmiq) {
		object <- rnb.execute.normalization.bmiq(object)
		if (verbose) {
			rnb.status("Performed normalization with method bmiq")
		}
		rnb.cleanMem()
	}

	## Update the dataset instance
	object@status$normalized <- method.to.set
	object@status$background <- bgcorr.method.to.set
	if (method != "none" || bgcorr.method !="none") {
		object <- updateRegionSummaries(object)
	}
	if (base::exists("inferred.covariates", inherits = FALSE)) {
		object@inferred.covariates <- inferred.covariates
	}

	object
}

########################################################################################################################

#' rnb.execute.normalization.bmiq
#'
#' Performs BMIQ normalization on the given dataset.
#' @param object Methylation dataset as an object of type \code{\linkS4class{MethyLumiSet}} or
#'               \code{\linkS4class{RnBSet}}.
#' @return The normalized dataset.
#'
#' @author Yassen Assenov
#' @noRd
rnb.execute.normalization.bmiq <- function(object) {

	## Extract methylation value matrix and probe design information
	if (inherits(object, "MethyLumiSet")) {
		m.data <- MethyLumiSet2RnBeadSet(object)
		beta.vals <- m.data$betas
		probe.design <- rnb.annotation2data.frame(rnb.get.annotation(object@target), add.names = TRUE)
		probe.design <- as.integer(probe.design[rownames(beta.vals), "Design"])
	} else {
		beta.vals <- object@meth.sites[, , drop = FALSE]
		probe.design <- as.integer(annotation(object)[, "Design"])
	}

	## Perform BMIQ
	samples.skipped <- integer()
	if (parallel.isEnabled() && ncol(beta.vals) > 1) {
		beta.names <- dimnames(beta.vals)
		beta.vals <- foreach(beta.v = as.data.frame(beta.vals), .combine = cbind, .packages = "RPMM",
							 .export = c("BMIQ", "betaEst2", "blc2"),
							 .noexport = c("bgcorr.method", "beta.names", "method", "object")) %dopar% {
			i <- which(!is.na(beta.v))
			p.design <- probe.design[i]
			if (all(tabulate(p.design, 2L) >= 50000L)) {
				x <- BMIQ(beta.v[i], p.design)
				if (!is.null(x)) {
					beta.v[i] <- x$all
				}
			}
			beta.v
		}
		dimnames(beta.vals) <- beta.names
		rm(beta.names)
	} else {
		for (j in 1:ncol(beta.vals)) {
			i <- which(!is.na(beta.vals[, j]))
			p.design <- probe.design[i]
			if (any(tabulate(p.design, 2L) < 50000L)) {
				## There are not enough probes of types I and/or II
				samples.skipped <- c(samples.skipped, j)
				rnb.status(c("Skipped sample", j))
			} else {
				x <- BMIQ(beta.vals[i, j], p.design)
				if (is.null(x)) {
					samples.skipped <- c(samples.skipped, j)
					rnb.status(c("Could not normalize sample", j))
				} else {
					beta.vals[i, j] <- x$all
					rnb.status(c("Normalized sample", j))
				}
			}
		}
		suppressWarnings(rm(j, i, p.design, x))
	}
	if (length(samples.skipped) != 0) {
		## Some samples were skipped due to not enough observations or unsuccessful fits
		rnb.warning(c("The following samples were not normalized:", paste(samples.skipped, collapse = ", ")))
	}
	rm(probe.design, samples.skipped)

	## Construct the resulting dataset
	if (inherits(object, "MethyLumiSet")) {
		object <- new("RnBeadSet", pheno=m.data$pheno, betas=beta.vals, p.values=m.data$p.values,
					  bead.counts=m.data$bead.counts)
		if ("qc" %in% names(m.data)) {
			qc(object) <- m.data[["qc"]]
		}
	} else {
		if (rnb.getOption("disk.dump.big.matrices")) {
			object@meth.sites <- convert.to.ff.matrix.tmp(beta.vals)
		} else {
			object@meth.sites <- beta.vals
		}
		for (region.type in rnb.region.types.for.analysis(object@assembly)) {
			object <- summarize.regions(object, region.type)
		}
	}
	object
}

########################################################################################################################
########################################################################################################################

#' rnb.section.normalization.shifts
#'
#' Generates a report sub-section on methylation beta values before and after correction.
#'
#' @param report       The report to contain the generated section.
#' @param betas.before Vector of raw beta values (before correction).
#' @param shifts       Vector of methylation value shifts (corrections). The \code{i}-th element in this vector must
#'                     correspond to the shift of the \code{i}-th element in \code{betas.before}.
#' @param bgcorr       Flag indicating if background subtraction was performed.
#' @return The modified report.
#'
#' @author Yassen Assenov
#' @noRd
rnb.section.normalization.shifts <- function(report, betas.before, shifts, bgcorr) {

	## Calculate frequencies of observed shifts and beta values
	shift.break.width <- 0.01
	shift.max <- ceiling(max(abs(shifts)) * 10) / 10
	shift.break.max <- shift.max + shift.break.width / 2
	shift.breaks <- seq(-shift.break.max, shift.break.max, by = shift.break.width)
	shift.mids <- seq(-shift.max, shift.max, length.out = length(shift.breaks) - 1)
	beta.break.width <- 0.025
	beta.breaks <- seq(0, 1, by = beta.break.width)
	beta.mids <- (beta.breaks[-length(beta.breaks)] + beta.breaks[-1]) / 2
	i.sb <- ceiling(betas.before / beta.break.width)
	i.sb[i.sb == 0] <- 1L
	i.sb <- as.integer(ceiling((shifts - shift.breaks[1]) / shift.break.width) - 1) * length(beta.mids) + i.sb
	i.sb <- tabulate(i.sb, length(shift.mids) * length(beta.mids))
	i.sb <- data.frame(
		xmin = rep(beta.breaks[-length(beta.breaks)], length(shift.mids)),
		xmax = rep(beta.breaks[-1], length(shift.mids)),
		ymin = rep(shift.breaks[-length(shift.breaks)], each = length(beta.mids)),
		ymax = rep(shift.breaks[-1], each = length(beta.mids)),
		freq = i.sb)

	## Plot histogram of shifts (differences)
	dframe <- data.frame(x = shift.mids, y = tapply(i.sb$freq, i.sb$ymin, sum))
	pp <- ggplot(dframe, aes_string(x = "x", y = "y")) + ggplot2::geom_bar(stat = "identity", width = 0.01) +
		scale_x_continuous(limits = c(-shift.break.max, shift.break.max), expand = c(0, 0)) +
		scale_y_continuous(expand = c(0, 0)) + labs(x = 'Shift', y = 'Frequency')
	rplot <- createReportPlot("correction_shifts", report, width = 7, height = 5.2)
	print(pp)
	off(rplot)
	txt <- c("The next figure gives an idea of the magnitude of the correction by showing the distribution of shifts, ",
		"i.e. degrees of modification of the raw methylation values.")
	rnb.add.paragraph(report, txt)
	txt <- "Histogram of observed magnitude of &beta; value correction."
	report <- rnb.add.figure(report, txt, rplot)
	logger.status("Added histogram of observed beta shifts (magnitude of correction)")
	rm(shift.break.width, shift.max, shift.breaks, shift.mids, beta.break.width, beta.breaks, beta.mids)
	rm(dframe, pp)

	## Plot 2D histogram of beta value (before normalization) and shift
	pp <- ggplot(i.sb, aes_string(xmin = "xmin", xmax = "xmax", ymin = "ymin", ymax = "ymax", fill = "freq")) +
		labs(x = expression(beta), y = "Shift", fill = "frequency") + geom_rect() +
		scale_x_continuous(limits = c(0, 1), expand = c(0, 0)) +
		scale_fill_gradient(low = "#FFFFFF", high = "#000000") +
		scale_y_continuous(limits = c(-shift.break.max, shift.break.max), expand = c(0, 0)) +
		theme(legend.justification = c(0, 0.5), legend.position = c(1, 0.5)) +
		theme(panel.border = element_blank(), panel.background = element_blank(), panel.grid = element_blank()) +
		theme(plot.margin = unit(c(0.1, 1.1, 0.1, 0.1), "in"))
	rplot <- createReportPlot("betas_shifts", report, width = 7.2, height = 6.2)
	print(pp)
	off(rplot)
	txt <- c("The figure below gives a more detailed view. This color-coded 2D histogram shows the uncorrected &beta; ",
		"values and their respective shifts after performing the normalization procedure.")
	rnb.add.paragraph(report, txt)
	txt <- "2D histogram showing the raw &beta; values and the magnitude of the corrections."
	report <- rnb.add.figure(report, txt, rplot)
	logger.status("Added 2D histogram of observed beta values and shifts")

	return(report)
}

########################################################################################################################

#' rnb.section.normalize.regions
#'
#' Adds a section summarizing the requested regions.
#'
#' @param report  Report on loading to contain the newly constructed section.
#' @param rnb.set Methylation dataset to be analyzed.
#' @param regions Non-empty \code{character} vector of region names.
#' @return The modified report.
#'
#' @author Yassen Assenov
#' @noRd
rnb.section.normalize.regions <- function(report, rnb.set, regions) {
	msg <- function(txt, e.class = "disabled") {
		paste0('<span class="', e.class, '">', txt, '</span>')
	}
	reg.counts <- sapply(regions, function(region) {
			tryCatch(as.character(nrow(meth(rnb.set, region))), error = function(e) { msg("not supported") })
		}
	)
	get.r.description <- function(reg) {
		result <- attr(rnb.get.annotation(reg, assembly = assembly(rnb.set)), "description")
		if (is.null(result) || identical("", result)) {
			result <- msg("n.a.")
		} else {
			result <- paste0('<p style="font-weight:normal;text-align:left">', result, '</p>')
		}
		result
	}
	reg.descriptions <- sapply(regions, function(region) {
			tryCatch(get.r.description(region), error = function(e) { msg("not imported", "outdated") })
		}
	)
	table.statistics <- data.frame(
		"Annotation" = regions, "Description" = reg.descriptions, "Regions in the Dataset" = reg.counts,
		check.names = FALSE, stringsAsFactors = FALSE)
	txt <- c(ifelse(rnb.getOption("analyze.sites"), "In addition to CpG sites, there", "There"),
		ifelse(length(regions) == 1, " is one set", paste(" are", length(regions), "sets")),
		" of genomic regions to be covered in the analysis. The table below gives a summary of these annotations.")
	report <- rnb.add.section(report, "Region Annotations", txt, level = 2)
	table.header <- c("<colgroup>", paste0('\t<col width="', c(210, 420, 150), 'px" />'), "</colgroup>")
	rnb.add.table(report, table.statistics, row.names = FALSE, thead = table.header)

	return(report)
}

#######################################################################################################################

#' rnb.section.normalization
#'
#' Performs normalization of the loaded methylation dataset.
#'
#' @param report    Report to contain the normalization section. This must be an object of type
#'                  \code{\linkS4class{Report}}.
#' @param rnb.set   Normalized methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param betas.raw Matrix of methylation beta values for CpG sites, as they would appear without (prior to)
#'                  normalization. If this parameter is \code{NULL}, the effect of normalization on beta values is
#'                  not examined and visualized.
#' @return The modified report.
#'
#' @details
#' If specified \code{betas.raw}, must be a matrix of methylation values with dimensions identical to the one
#' encapsulated in \code{rnb.set}. The former matrix is expected to store beta values before normalization,
#' whereas the latter one - after normalization. If the provided matrices contain sufficient amounts of non-missing
#' values, this function creates figures that compare the two distributions of values and examines the magnitute of
#' modifications.
#'
#' @author Pavlo Lutsik
#' @noRd
rnb.section.normalization <- function(report, rnb.set, betas.raw = NULL) {
	if (!inherits(report, "Report")) {
		stop("Invalid value for report")
	}
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	if (!is.null(betas.raw)) {
		if (!(is.matrix(betas.raw) && (is.double(betas.raw) || is.integer(betas.raw)))) {
			stop("invalid value for betas.raw")
		}
		betas.after <- meth(rnb.set, row.names = TRUE)
		if (!(nrow(betas.raw) == nrow(betas.after) && ncol(betas.raw) == ncol(betas.after) &&
			setequal(rownames(betas.raw), rownames(betas.after)) &&
			setequal(colnames(betas.raw), colnames(betas.after)))) {
			stop("incompatible values for rnb.set and betas.raw")
		}
		betas.after <- betas.after[rownames(betas.raw), colnames(betas.raw)]
	}

	txt.methylumi <- "<a href=\"http://www.bioconductor.org/packages/release/bioc/html/methylumi.html\">methylumi</a>"

	bgcorr.method <- rnb.set@status$background
	bgcorr <- (bgcorr.method != "none")
	bgcorr.method <- gsub("methylumi\\.","", bgcorr.method)
	if (bgcorr) {
		refText <- c("Triche, T.J. Jr, Weisenberger, D.J., Van Den Berg, D., Laird, P.W., and Siegmund, K.D. (2013)",
			"Low-level processing of Illumina Infinium DNA Methylation BeadArrays. ",
			"<i>Nucleic Acids Research</i> <b>41</b>(7), e90")
		report <- rnb.add.reference(report, refText)
		txt <- c("The background was subtracted using the ", txt.methylumi, " package (method \"",
			bgcorr.method, "\") ", rnb.get.reference(report, refText), ".")
		txt.methylumi <- "methylumi"
	} else {
		txt <- ""
	}
	method <- rnb.set@status$normalized
	if (method == "illumina") {
		txt <- c(txt, "The signal intensity values were normalized using Illumina's recommended normalization method, ",
			"as implemented in the ", txt.methylumi, " package.")
	} else if (method == "swan") {
		txt <- c(txt, "The signal intensity values were normalized using the SWAN normalization method, as ",
			"implemented in the ",
			"<a href=\"http://www.bioconductor.org/packages/release/bioc/html/minfi.html\">minfi</a> package.")
	} else if (method == "bmiq") {
		refText <- c("Teschendorff, A. E., Marabita, F., Lechner, M., Bartlett, T., Tegner, J., Gomez-Cabrero, D. and ",
			"Beck, S. (2013) A beta-mixture quantile normalization method for correcting probe design bias in ",
			"Illumina Infinium 450 k DNA methylation data. <i>Bioinformatics</i> <b>29</b>(2), 189-196")
		report <- rnb.add.reference(report, refText)
		txt <- c(txt, "The methylation &beta; values were normalized using the BMIQ normalization method ",
			rnb.get.reference(report, refText), ".")
	} else if (grepl("wm\\.", method)){
		refText <- c("Pidsley, R., Wong, C.,  Volta, M., Lunnon, K., Mill, J., and Schalkwyk, L. (2013)",
				"A data-driven approach to preprocessing Illumina 450K methylation array data."," <i>BMC Genomics</i>, <b>14<b>(1), 293")
		report <- rnb.add.reference(report, refText)
		wm.method<-gsub("wm\\.", "", method)
		txt<-c(txt, sprintf("The data was normalized using method %s from %s.", wm.method, rnb.get.reference(report,refText)))
	}else if (method == "minfi.funnorm"){
		refText<-c("Fortin, J., Labbe, A., Lemire, M., Zanke, B. W., Hudson, T. J., Fertig, E. J., Greenwood, C.M.T., ",
			"Hansen, K. D. (2014) Functional normalization of 450k methylation array data improves replication in ",
			"large cancer studies. <i>BioRxiv</i>.")
		report <- rnb.add.reference(report, refText)
		txt<-c(txt, sprintf("The data was normalized using the functional normalization method from %s.", rnb.get.reference(report,refText)))

	}else{ # method == "none"
		txt <- c(txt, "The measurements in this dataset were not normalized after ",
			ifelse(txt == "", "loading", "background subtraction"), ".")
		if(rnb.getOption("normalization.method")=="minfi.funnorm"){
			if(nrow(sites(rnb.set))<sum(elementNROWS(rnb.get.annotation("probes450")))){
				txt<-c(txt, "The desired normalization method \"minfi.funnorm\" was not applied since the data set did not
					contain information for all probes. This is most likely because of the preceding quality filtering steps.",
					"In order to apply minfi.funnorm in the full RnBeads analysis you should disable SNP filtering and Greedycut.",
					"Otherwise, you apply rnb.execute.normalization() function to an unfiltered RnBead(Raw)Set after loading and start
					the full pipeline with the returned object as input.")
			}else if(sum(is.na(M(rnb.set)))>0 || sum(is.na(U(rnb.set)))>0){
				txt<-c(txt, "The desired normalization method \"minfi.funnorm\" was not applied because intensity information contains
						missing values.", "Try to apply rnb.execute.normalization() function to an unfiltered RnBead(Raw)Set after loading and start
								the full pipeline with the returned object as input.")
			}
		}
	}
	report <- rnb.add.section(report, "Normalization", txt)

	## Add comparison of betas before and after correction
	if ((!is.null(betas.raw)) && (bgcorr || method != "none")) {
		betas.raw <- as.vector(betas.raw)
		betas.after <- as.vector(betas.after)
		min.observations <- 501L

		i.before <- !is.na(betas.raw); i.after <- !is.na(betas.after)
		i.both <- which(i.before & i.after)
		i.before <- which(i.before); i.after <- which(i.after)

		section.title <- "Effect of Correction"
		if (length(i.before) >= min.observations && length(i.after) >= min.observations) {
			txt <- c("This section shows the influence of the applied normalization procedure on CpG methylation ",
				"values. The following figure compares the distributions of the &beta; values before and after ",
				"performing normalization.")
			report <- rnb.add.section(report, section.title, txt, level = 2)

			## Distributions before and after normalization
			beta.values <- list("Before correction" = betas.raw[i.before], "After correction" = betas.after[i.after])
			report.plots <- rnb.plot.beta.comparison(beta.values, "correction_comparison", report, min.observations)
			txt <- "Comparison of &beta; values before and after correction."
			txt <- c(txt, add.text.subsampling(attr(report.plots, "subsampled"), paste("betas", names(beta.values))))
			setting.names <- list("Plot type" =
				c("density" = "density estimation", "histogram" = "histograms", "qq" = "quantile-quantile plot"))
			report <- rnb.add.figure(report, txt, report.plots, setting.names)
			rm(i.before, i.after, beta.values, report.plots, setting.names)
			rnb.cleanMem()
			logger.status("Added comparison between non-normalized and normalized beta values")

			if (rnb.getOption("normalization.plot.shifts")) {
				if (length(i.both) != 0) {
					betas.raw <- betas.raw[i.both]
					betas.after <- betas.after[i.both] - betas.raw
					report <- rnb.section.normalization.shifts(report, betas.raw, betas.after, bgcorr)
				} else {
					txt <- c("No plots on &beta; value shifts were created because there are not enough data to ",
						"visualize.")
					rnb.add.paragraph(report, txt)
				}
			}
		} else {
			txt <- c("The effects of the applied procedure on CpG methylation values is not summarized because there ",
				"are not enough &beta; values to accurately present the observed distributions before and after ",
				"normalization.")
			report <- rnb.add.section(report, section.title, txt)
		}
		rm(betas.raw, betas.after)
	}

	if (!inherits(rnb.set, "RnBeadSet")) {
		return(report)
	}

	## Add point-and-whisker plots of mean methylation for every slide
	report.plots <- rnb.plot.sentrix.distributions(rnb.set, report = report, width = 6, height = 6)
	if (is.null(report.plots)) {
		txt <- "Sample average methylation cannot be visualized because "
		if (length(samples(rnb.set)) == 0) {
			txt <- c(txt, "the normalized dataset is empty.")
		} else if (all(is.na(meth(rnb.set)))) {
			txt <- c(txt, "the dataset contains no valid methylation beta values.")
		} else {
			txt <- c(txt, "no valid Sentrix ID and Sentrix Position information could be extracted from the ",
				"sample annotation table.")
		}
	} else {
		txt <- "The following figure visualizes the average methylation per sample. Samples are grouped by slide."
	}
	report <- rnb.add.section(report, "Sample Mean Methylations", txt, level = 2)
	if (!is.null(report.plots)) {
		description <- "Point-and-whisker plot showing mean and standard deviation among all beta values in a sample."
		if (inherits(report.plots, "ReportPlot")) {
			setting.names <- list()
		} else { # is.list(report.plots)
			setting.names <- list("Slide number" = names(report.plots))
			names(setting.names[[1]]) <- 1:length(report.plots)
		}
		report <- rnb.add.figure(report, description, report.plots, setting.names)
	}

	r.types <- rnb.getOption("region.types")
	if (is.null(r.types)) {
		r.types <- summarized.regions(rnb.set)
	}
	if (length(r.types) != 0) {
		report <- rnb.section.normalize.regions(report, rnb.set, r.types)
	}

	return(report)
}

#######################################################################################################################

#' rnb.step.normalization
#'
#' Performs normalization of the loaded HumanMethylation450 dataset and adds a corresponding section to the given
#' report.
#'
#' @param object Methylation dataset as an object of type inheriting \code{\linkS4class{MethyLumiSet}} or
#'               \code{\linkS4class{RnBSet}}.
#' @param report Report to contain the normalization section. This must be an object of type
#'               \code{\linkS4class{Report}}.
#' @param method Normalization method, must be one of \code{"illumina"}, \code{"swan"} or \code{"none"}. Note that
#'               the execution of method SWAN requires packages \pkg{minfi} and
#'               \pkg{IlluminaHumanMethylation450kmanifest}.
#'
#' @return List with two elements:
#'         \describe{
#'           \item{\code{dataset}}{Normalized dataset as an object of type \code{\linkS4class{RnBeadSet}}.}
#'           \item{\code{report}}{the modified report.}
#'         }
#'
#' @author Pavlo Lutsik
#' @noRd
rnb.step.normalization<-function(object, report, method = rnb.getOption("normalization.method")){

	if(!(inherits(object,"MethyLumiSet") || inherits(object,"RnBSet"))) {
		stop("Invalid value for object; expected MethyLumiSet, RnBeadSet or RnBiseqSet")
	}
	if (!inherits(report, "Report")) {
		stop("Invalid value for report")
	}
	accepted <- .rnb.options[["accepted"]][["normalization.method"]]
	if (!(is.character(method) && length(method) == 1 && isTRUE(method %in% accepted))) {
		msg <- paste0('"', accepted, '"', collapse = ", ")
		stop(paste("invalid value for method; expected one of", msg))
	}

	logger.start("Normalization Procedure")


	if (method == "none") {
		betas.raw <- NULL
	} else if (inherits(object, "MethyLumiSet")) {
		betas.raw <- betas(object)
		pheno.table <- phenoData(object)@data
		id.column <- rnb.getOption("identifiers.column")
		if (!(is.null(pheno.table) || is.null(id.column))) {
			ids <- NA
			if (is.character(id.column) && (id.column %in% colnames(pheno.table))) {
				ids <- pheno.table[, id.column]
			} else if (1 <= id.column && id.column <= ncol(pheno.table)) {
				ids <- pheno.table[, id.column]
			}
			if (any(is.na(ids)) == FALSE && anyDuplicated(ids) == 0) {
				colnames(betas.raw) <- as.character(ids)
			}
			rm(ids)
		}
		rm(pheno.table, id.column)
		rnb.cleanMem()
	} else { # inherits(object, "RnBSet")
		betas.raw <- meth(object, row.names = TRUE)
	}
	#only destroy an object if the new object is not the same as the old object,
	#i.e. if normalization or background subtraction took place
	#if ((rnb.set@status$normalized != "none") || (rnb.set@status$background != "none")){
	#	rnb.call.destructor(object)
	#}

	if(object@status$disk.dump && rnb.getOption("enforce.destroy.disk.dumps")){
		object@status$discard.ff.matrices<-TRUE
	}
	object <- rnb.execute.normalization(object, method)
	rnb.cleanMem()
	if(isTRUE(object@status$discard.ff.matrices)){
		object@status$discard.ff.matrices<-NULL
	}
#	if(object@status$background!="none"){
#		logger.status(sprintf("Performed background correction with method \"%s\"", object@status$background))
#	}
	if(object@status$normalized!="none"){
		logger.status(sprintf("Performed normalization with method \"%s\"", object@status$normalized))
	}

	report<-rnb.section.normalization(report, object, betas.raw)
	logger.status("Added normalization section")

	logger.completed()
	return(list(dataset=object, report=report))
}

#######################################################################################################################

#' mean.imputation
#'
#' Performs mean imputation either for all samples (way=1) or for all CpGs (way=2).
#'
#'@param rnb.set Object containing the methylation information to be changed. Has to be of
#'               type \code{\linkS4class{RnBeadSet}} or \code{\linkS4class{RnBiseqSet}} or a data matrix containing
#'               the methylation values.
#'@param way Should the sample-wise mean (1) or CpG-wise mean (2) be used to replace the missing value.
#'@return Modified rnb.set object.
#'
#'@author Michael Scherer
#'@noRd
mean.imputation <- function(rnb.set,way=1){
  if(inherits(rnb.set,"RnBSet")){
    methData <- meth(rnb.set)
  }else{
    methData <- as.matrix(rnb.set)
  }
  nas <- is.na(methData)
  if(any(apply(nas,way,all))){
    logger.warning(paste("There are", ifelse(way==1,"CpG sites","sampels"),"that have missing values in all",
                         ifelse(way==1,"samples","CpG sites"),", imputation not performed."))
    return(NULL)
  }
  means <- apply(methData,way,mean,na.rm=TRUE)
  has.nas <- which(apply(nas,way,any))
  if(way==1){
    for(i in has.nas){
      methData[i,nas[i,]] <- means[i]
    }
  }else if(way==2){
    for(i in has.nas){
        methData[nas[,i],i] <- means[i]
    }
  }
  return(methData)
}

#######################################################################################################################

#' imputation.low.memory.samples
#'
#' Performs (sample-wise) imputation in a low memory-footprint way with the specified method
#'
#'@param rnb.set Object containing the methylation information to be changed. Has to be of
#'               type \code{\linkS4class{RnBeadSet}} or \code{\linkS4class{RnBiseqSet}} or a data matrix containing
#'               the methylation values.
#'@param method Method to be used, should be either 'mean' or 'median'
#'@return Modified rnb.set object.
#'
#'@author Michael Scherer
#'@noRd
imputation.low.memory.samples <- function(rnb.set,method=mean){
 for(i in 1:nsites(rnb.set)){
    cpg <- meth(rnb.set,i=i)
    if(any(is.na(cpg))){
      rnb.set@meth.sites[i,which(is.na(cpg))] <- method(cpg,na.rm=T)
    }
  }
  return(rnb.set)
}

#######################################################################################################################

#' imputation.low.memory.cpgs
#'
#' Performs (cpg-wise) imputation in a low memory-footprint way with the specified method
#'
#'@param rnb.set Object containing the methylation information to be changed. Has to be of
#'               type \code{\linkS4class{RnBeadSet}} or \code{\linkS4class{RnBiseqSet}} or a data matrix containing
#'               the methylation values.
#'@param method Method to be used, should be either 'mean' or 'median'
#'@return Modified rnb.set object.
#'
#'@author Michael Scherer
#'@noRd
imputation.low.memory.cpgs <- function(rnb.set,method=mean){
  if(any(sampleMethApply(rnb.set,function(x){all(is.na(x))}))){
    logger.warning("There are samples that have missing values at all CpG sites, imputation not performed.")
    return(rnb.set)
  }
  for(i in 1:length(samples(rnb.set))){
    ss <- meth(rnb.set,j=i)
    if(any(is.na(ss))){
      rnb.set@meth.sites[which(is.na(ss)),i] <- method(ss,na.rm=T)
    }
  }
  return(rnb.set)
}


#######################################################################################################################

#' random.imputation
#'
#' Performs random imputation by replacing missing values with randomly selecting (with replacement) from the same CpG site in the samples
#' that do not contain missing values.
#'
#'@param rnb.set Object containing the methylation information to be changed. Has to be of
#'               type \code{\linkS4class{RnBeadSet}} or \code{\linkS4class{RnBiseqSet}} or a data matrix containing
#'               the methylation values.
#'@return Modified rnb.set object.
#'
#'@author Michael Scherer
#'@noRd
random.imputation <- function(rnb.set){
  if(inherits(rnb.set,"RnBSet")){
    methData <- meth(rnb.set)
  }else{
    methData <- as.matrix(rnb.set)
  }
  nas <- is.na(methData)
  if(any(apply(nas,1,all))){
    logger.warning("There are CpG sites that have missing values in all samples, imputation not performed.")
    return(NULL)
  }
  has.nas <- which(apply(nas,1,any))
  for(i in has.nas){
    row <- methData[i,]
    without_nas <- row[!nas[i,]]
    replacement <- sample(without_nas,sum(nas[i,]),replace=TRUE)
    methData[i,nas[i,]] <- replacement
  }
  return(methData)
}
#######################################################################################################################

#' knn.imputation
#'
#' This function performns k nearest neighbors imputation by calling the \code{impute.knn} function from the
#' \pkg{impute} package.
#'
#'@param rnb.set Object containing the methylation information to be changed. Has to be of
#'               type \code{\linkS4class{RnBeadSet}} or \code{\linkS4class{RnBiseqSet}} or a data matrix containing
#'               the methylation values.
#'@param k parameter defining the number of nearest neighbors from which the missing value should be inferred
#'@return Modified rnb.set object.
#'
#'@author Michael Scherer
#'@noRd
knn.imputation <- function(rnb.set,k=10){
  rnb.require('impute')
  if(inherits(rnb.set,"RnBSet")){
    methData <- meth(rnb.set)
  }else{
    methData <- as.matrix(rnb.set)
  }
  dummy <- capture.output(methData <- (impute.knn(methData,colmax=1,k=k))$data)
  rm(dummy)
  return(methData)
}

#######################################################################################################################

#' median.imputation
#'
#' Performs median imputation either for all samples (way=1) or for all CpGs (way=2).
#'
#'@param rnb.set Object containing the methylation information to be changed. Has to be of
#'               type \code{\linkS4class{RnBeadSet}} or \code{\linkS4class{RnBiseqSet}} or a data matrix containing
#'               the methylation values.
#'@param way Should the sample-wise median (1) or CpG-wise median (2) be used to replace the missing value.
#'@return Modified rnb.set object.
#'
#'@author Michael Scherer
#'@noRd
median.imputation <- function(rnb.set,way=1){
  if(inherits(rnb.set,"RnBSet")){
    methData <- meth(rnb.set)
  }else{
    methData <- as.matrix(rnb.set)
  }
  nas <- is.na(methData)
  if(any(apply(nas,way,all))){
    logger.warning(paste("There are", ifelse(way==1,"CpG sites","sampels"),"that have missing values in all",
                ifelse(way==1,"samples","CpG sites"),", imputation not performed."))
    return(NULL)
  }
  medians <- apply(methData,way,median,na.rm=TRUE)
  has.nas <- which(apply(nas,way,any))
  if(way==1){
    for(i in has.nas){
      methData[i,nas[i,]] <- medians[i]
    }
  }else if(way==2){
    for(i in has.nas){
      methData[nas[,i],i] <- medians[i]
    }
  }
  return(methData)
}

#######################################################################################################################

#' rnb.execute.imputation
#'
#' Removes missing methylation values in the methylation matrix of the given object
#'
#' @param rnb.set Dataset object inheriting from \code{\linkS4class{RnBSet}}.
#' @param method Imputation method to be used, must be one of \code{"mean.cpgs"}, \code{"mean.samples"},
#'                \code{"random"}, \code{"knn"}, \code{"median.cpgs"}, \code{"median.samples"}, or \code{"none"}.
#' @param update.ff flag indicating if the disk based matrices should be updated. Should be set to FALSE, if methylation
#'                  matrix should only temporarly be changed. If this value is FALSE, the region level methylation values
#'                  are not updated and only the site-wise matrix is changed temporarly.
#' @param ... Optional arguments passed to knn.imputation
#' @return The modified rnb.set object without missing methylation values.
#' 
#' @details Imputes missing values by applying on the following methods:
#'          \describe{
#'            \item{mean.cpgs:}{missing values are inferred as the average methylation value from all other 
#'            (non-mising) CpGs in this sample}
#'            \item{mean.samples:}{missing values are inferred as the average methylation value from all other 
#'            (non-mising) values at this CpG sites in all other samples}
#'            \item{random:}{missing values are inferred by randomly selecting a (non-missing) methylation value
#'            from any other sample at this CpG site}
#'            \item{knn:}{missing values are inferred by k-nearest neighbors imputation (see \pkg{impute})}
#'            \item{median.cpgs:}{missing values are inferred as the median methylation value from all other 
#'            (non-mising) CpGs in this sample}
#'            \item{median.samples:}{missing values are inferred as the median methylation value from all other 
#'            (non-mising) values at this CpG sites in all other samples}
#'            \item{none:}{imputation should not be performed}
#'          }
#'
#' @author Michael Scherer
#'
#' @export
rnb.execute.imputation <- function(rnb.set,method=rnb.getOption("imputation.method"),update.ff=TRUE,...){
  if(!(inherits(rnb.set,"RnBSet")||is.matrix(rnb.set)||is.data.frame(rnb.set))){
    stop("Invalid value for input object, has to be of type RnBeadSet or RnBiseqSet")
  }
  if(inherits(rnb.set,"RnBSet")&&isImputed(rnb.set)){
    logger.info("RnBSet already imputed, imputation skipped.")
    return(rnb.set)
  }
  if(inherits(rnb.set,"RnBSet") && (length(samples(rnb.set))==1) && !(method%in%c("median.cpgs","mean.cpgs"))){
    logger.info(paste("Imputation cannot be employed with method",method,"median.cpgs used"))
    method <- "median.cpgs"
  }
  if(!(method%in%c('mean.cpgs','mean.samples','random','knn','median.cpgs','median.samples'))){
    if(method=='none'){
      if(inherits(rnb.set,"RnBeadSet")){
        logger.info("No imputation method selected, 'knn' method used.")
        rnb.options(imputation.method='knn')
        method = 'knn'
      }else if (inherits(rnb.set,"RnBiseqSet")||is.matrix(rnb.set)||is.data.frame(rnb.set)){
        logger.info("No imputation method selected, 'mean.samples' method used.")
        rnb.options(imputation.method='mean.samples')
        method = 'mean.samples'
      }
    }else{
      stop("Invalid option for imputation method, has to be one of 'mean.cpgs','mean.samples','random','knn'")
    }
  }
  if(inherits(rnb.set,"RnBiseqSet") && rnb.getOption("imputation.method")=="knn"){
    rnb.options("imputation.method"="mean.samples")
    method = "mean.samples"
    logger.info("Knn imputation not applicable to sequencing data sets, switched to 'mean.samples' method")
    rnb.options("imputation.method"=method)
  }
  if(rnb.getOption("enforce.memory.management")){
    if(!method%in%c("mean.cpgs","median.cpgs")){
      logger.info(sprintf("Low memory imputation not compatible with method %s, switched to mean.cpgs",method))
      method <- "mean.cpgs"
      rnb.options("imputation.method"=method)
    }
    if(method=="mean.cpgs"){
      if(is.data.frame(rnb.set)||is.matrix(rnb.set)){
        logger.info("No low memory footprint imputation available for matrix")
        meth.data <- mean.imputation(rnb.set,way=2)
        return(meth.data)
      }
      logger.start("Low memory footprint version of imputation")
      rnb.set <- imputation.low.memory.cpgs(rnb.set,mean)
      logger.completed()
    }
    if(method=="median.cpgs"){
      if(is.data.frame(rnb.set)||is.matrix(rnb.set)){
        logger.info("No low memory footprint imputation available for matrix")
        meth.data <- median.imputation(rnb.set,way=2)
        return(meth.data)
      }
      logger.start("Low memory footprint version of imputation")
      rnb.set <- imputation.low.memory.cpgs(rnb.set,median)
      logger.completed()
    }
    if(inherits(rnb.set,"RnBSet")){
      rnb.set <- updateRegionSummaries(rnb.set)
      rnb.set@imputed <- TRUE
    }
    return(rnb.set)
  }
  logger.start(sprintf("Imputation procedure %s ",method))
  if(method=='mean.cpgs'){
    meth.data <- mean.imputation(rnb.set,2)
  }
  if(method=='mean.samples'){
    meth.data <- mean.imputation(rnb.set,1)
  }
  if(method=='random'){
    meth.data <- random.imputation(rnb.set)
  }
  if(method=='knn'){
    meth.data <- knn.imputation(rnb.set,...)
  }
  if(method=='median.cpgs'){
    meth.data <- median.imputation(rnb.set,2)
  }
  if(method=='median.samples'){
    meth.data <- median.imputation(rnb.set,1)
  }
  if(!is.null(meth.data)){
    if(inherits(rnb.set,"RnBSet")){
      if(update.ff){
        rnb.set <- updateMethylationSites(rnb.set,meth.data)
        rnb.set@imputed <- TRUE
      }else{
        rnb.set@meth.sites <- meth.data
      }
    }else{
      rnb.set <- meth.data
    }
  }
  logger.completed()
  return(rnb.set)
}

#######################################################################################################################

#' rnb.section.imputation
#'
#' Adds information and plots describing the effect of imputation on the dataset
#'
#' @param report Report to which the information should be added (\code{\linkS4class{Report}}).
#' @param rnb.set Dataset on which imputation was performed (\code{\linkS4class{RnBSet}}).
#' @param old.values Methylation matrix before imputation procedure.
#'
#' @return The modified report object
#' @author Michael Scherer
#' @noRd
rnb.section.imputation <- function(report,rnb.set,old.values){
  if(rnb.getOption('imputation.method')=='none'){
    return(report)
  }
  txt <- c("Imputation was performed ")
  if(rnb.getOption('imputation.method')=='mean.samples'){
    txt <- c(txt,"by calculating the mean methylation level for each CpG site across all samples",
             " and replacing all missing values for this CpG site in individual samples with the mean across all samples.\n")
  }
  if(rnb.getOption('imputation.method')=='mean.cpgs'){
    txt <- c(txt,"by calculating the mean methylation level for each sample across all CpG sites",
             " and replacing all missing values for this sample at an individual CpG site with the mean across all CpGs in the sample.\n")
  }
  if(rnb.getOption('imputation.method')=='random'){
    txt <- c(txt,"by randomly selecting values from non-missing CpGs in other samples.",
             " Sampling was done with replacement such that the number of missing values can be larger as the number of existing.\n")
  }
  if(rnb.getOption('imputation.method')=='knn'){
    txt <- c(txt,"by k nearest neighbors imputation from the <a href=https://bioconductor.org/packages/release/bioc/html/impute.html>impute</a> package. Briefly, missing values were replaced",
             " by the average of the methylation values at that CpG in the closest samples. Closeness was defined with",
             " the euclidean distance.\n")
  }
  if(rnb.getOption('imputation.method')=='median.samples'){
    txt <- c(txt,"by calculating the median methylation level for each CpG site across all samples",
             " and replacing all missing values for this CpG site in individual samples with the median across all samples.\n")
  }
  if(rnb.getOption('imputation.method')=='median.cpgs'){
    txt <- c(txt,"by calculating the median methylation level for each sample across all CpG sites",
             " and replacing all missing values for this sample at an individual CpG site with the median across all CpGs in the sample.\n")
  }
  new.values <- meth(rnb.set)
  missing.values <- apply(old.values,2,function(x)sum(is.na(x)))
  median <- median(missing.values)
  txt <- c(txt," Imputation replaced a median of ",median," missing values per sample by estimations.")
  report <- rnb.add.section(report,'Imputation',txt)
  if(sum(is.na(new.values))==sum(is.na(old.values))){
    txt <- "Imputation failed, see logger output for more information."
    report <- rnb.add.paragraph(report,txt)
    return(report)
  }
  old.values <- reshape2::melt(old.values)
  old.values <- old.values$value
  old.values <- na.omit(old.values)
  new.values <- reshape2::melt(new.values)
  new.values <- new.values$value
  min.observations <- 501L
  beta.values <- list("NAs removed" = old.values, "After imputation" = new.values)
  report.plot <- rnb.plot.beta.comparison(beta.values, "imputation_comparison", report, min.observations)
  setting.names <- list("Plot type" =
                          c("density" = "density estimation", "histogram" = "histograms", "qq" = "quantile-quantile plot"))
  descr <- 'Visulisation of the effect of Imputation on the methylation distribution.'
  report <- rnb.add.figure(report,descr,report.plot,setting.names)
  return(report)
}
#######################################################################################################################

#' rnb.step.imputation
#'
#' Imputes missing values from the given dataset with the specified method
#'
#' @param rnb.set Dataset object inheriting from \code{\linkS4class{RnBSet}}.
#' @param report Report object to contain imputation information. Has to be of class
#'                \code{\linkS4class{Report}}.
#' @param method Imputation method to be used, must be one of \code{"mean.cpgs"}, \code{"mean.samples"},
#'                \code{"random"}, \code{"knn"} or \code{"none"}.
#'
#' @return List with two elements:
#'         \describe{
#'           \item{\code{dataset}}{Dataset as an object of type \code{\linkS4class{RnBSet}}
#'                 containing the imputed methylation matrix.}
#'           \item{\code{report}}{the modified report.}
#'         }
#'
#' @author Michael Scherer
#' @noRd
rnb.step.imputation <- function(rnb.set, report, method=rnb.getOption("imputation.method")){
  if(!(inherits(rnb.set,"RnBSet"))) {
    stop("Invalid value for object; RnBeadSet or RnBiseqSet required")
  }
  if (!inherits(report, "Report")) {
    stop("Invalid value for report")
  }
  old.data <- meth(rnb.set)
  if(sum(is.na(old.data))==0){
    logger.info("No missing values present, imputation skipped")
    return(list(dataset=rnb.set,report=report))
  }
  if(!(method%in%c('none'))){
    rnb.set <- rnb.execute.imputation(rnb.set)
  }
  report <- rnb.section.imputation(report,rnb.set,old.data)
  return(list(dataset=rnb.set,report=report))
}

#######################################################################################################################

################################################################################
# Cluster Architecture Descriptions
################################################################################
################################################################################
# Concrete implementations for the LSF environment
################################################################################

#' ClusterArchitectureLSF Class
#'
#' A child class of \code{\linkS4class{ClusterArchitecture}} implementing specifications of IBM LSF architectures.
#'
#' @details
#' Follow this template if you want to create your own ClusterArchitecture class.
#'
#' @section Slots:
#' see \code{\linkS4class{ClusterArchitecture}}
#'
#' @section Methods:
#' \describe{
#'   \item{\code{\link{getSubCmdTokens,ClusterArchitectureLSF-method}}}{Returns a vector of command line tokens corresponding to submitting
#'   a job with the given command to the cluster}
#' }
#'
#' @name ClusterArchitectureLSF-class
#' @rdname ClusterArchitectureLSF-class
#' @author Michael Scherer
#' @exportClass ClusterArchitecture
setClass("ClusterArchitectureLSF",
	contains = "ClusterArchitecture"
)

#' initialize.ClusterArchitectureLSF
#'
#' Initialize an ClusterArchitecture object for a LSF
#' 
#' @param .Object New instance of \code{ClusterArchitectureLSF}.
#' @param name A name or identifier
#' @param ... arguments passed on to the constructor of \code{\linkS4class{ClusterArchitecture}} (the parent class)
#'
#' @export
#' @author Michael Scherer
#' @docType methods
setMethod("initialize","ClusterArchitectureLSF",
	function(
		.Object,
		name="ClusterArchitectureLSF",
		...
	) {
		.Object <- callNextMethod(.Object=.Object, name=name, ...)
		.Object <- setExecutable(.Object,"R","R")
		.Object <- setExecutable(.Object,"Rscript","Rscript")
		.Object <- setExecutable(.Object,"python","python")
		.Object
	}
)

#' getSubCmdTokens-methods
#'
#' Returns a string for the of command line corresponding to submitting
#' a job with the given command to the cluster.
#' @details
#' For a concrete child class implementation for a LSF architecture specification see \code{\linkS4class{ClusterArchitectureLSF}}
#'
#' @param object \code{\linkS4class{ClusterArchitectureLSF}} object
#' @param cmd.tokens a character vector specifying the executable command that should be wrapped in the cluster submission command
#' @param log file name and path of the log file that the submitted job writes to
#' @param job.name name of the submitted job
#' @param res.req named vector of requested resources. Two options are available: \code{"clock.limit"} and \code{"memory.size"}
#' @param depend.jobs character vector containg names or ids of jobs the submitted job will depend on.
#' @return A character vector containing the submission command tokens
#'
#' @rdname getSubCmdTokens-ClusterArchitectureLSF-methods
#' @docType methods
#' @aliases getSubCmdTokens,ClusterArchitectureLSF-method
#' @author Michael Scherer
#' @export
#' @examples
#' \donttest{
#' arch <- new("ClusterArchitectureLSF",
#' 	name="my_lsf_architecture"
#' )
#' getSubCmdTokens(arch,c("Rscript","my_great_script.R"),"my_logfile.log")
#' }
setMethod("getSubCmdTokens",
	signature(
		object="ClusterArchitectureLSF"
	),
	function(
	  object,
	  cmd.tokens,
	  log,
	  job.name = "",
	  res.req = character(0),
	  depend.jobs = character(0)
	) {
	  res.req.token <- NULL
	  queue <- "short"
		if(length(res.req)>0){
		  if("clock.limit" %in% names(res.req)){
		    res.req.token <- paste(res.req.token,"-W ",res.req["clock.limit"]," ",collapse = "")
		  }
		  if("mem.size" %in% names(res.req)){
		    res.req.token <- paste0(res.req.token,'-R "rusage[mem=', res.req["mem.size"], ']"', collapse="")
		  }
		  if("queue" %in% names(res.req)){
		    queue <-  res.req["queue"]
		  }
		}
		log.token <- NULL
		if (nchar(log)>0) {
			log.token <- c("-e", log, "-o", log)
		}
		job.name.token <- NULL
		if (nchar(job.name)>0) {
			job.name.token <- paste0(job.name.token,"-J ",job.name,collapse = "")
		}
		dependency.token <- NULL
		if (length(depend.jobs)>0){
			dependency.token <- paste0(dependency.token, "-w ", paste0('"', paste(paste0('done(', depend.jobs, ')'),collapse="&&"), '"'),collapse = "")
		}

		res <- c(
			"bsub",
			paste("-q", queue),
			res.req.token,
			log.token,
			job.name.token,
			dependency.token,
			cmd.tokens
		)
		print(paste(res, collapse=" "))
		return(res)
	}
)

########################################################################################################################
## RnBiseqSet-class.R
## created: 2012-10-29
## creator: Pavlo Lutsik
## ---------------------------------------------------------------------------------------------------------------------
## RnBiseqSet class definition.
########################################################################################################################

#' RnBiseqSet Class
#'
#' A class for storing the DNA methylation and quality information from bisulfite sequencing experiments 
#'
#' @details TBA
#'
#' @section Slots:
#' \describe{
#'   \item{\code{status}}{Normalization status.}
#' }
#'
#' @section Methods and Functions:
#' \describe{
#'   \item{\code{\link[BiocGenerics]{combine}}}{Combines two datasets.}
#' }
#'
#' @name RnBiseqSet-class
#' @rdname RnBiseqSet-class
#' @author Pavlo Lutsik
#' @exportClass RnBiseqSet
setClass("RnBiseqSet",
		representation(status="listOrNULL"),
		contains="RnBSet",
		prototype(pheno=data.frame(), meth.sites=matrix(nrow=0,ncol=0), covg.sites=NULL, target="CpG", status=NULL),
		package = "RnBeads")

## ---------------------------------------------------------------------------------------------------------------------
## CONSTRUCTORS
## ---------------------------------------------------------------------------------------------------------------------

setMethod("initialize", "RnBiseqSet",
	function(.Object,
			pheno=data.frame(),
			sites=matrix(ncol=0,nrow=0),
			meth.sites=matrix(ncol=0,nrow=0),
			covg.sites=matrix(ncol=0,nrow=0),
			regions=list(),
			meth.regions=list(),
			assembly="hg19",
			target="CpG",
			status=list(),
			inferred.covariates=list()
			){
		
		.Object@pheno<-pheno	
		.Object@sites<-sites
		.Object@meth.sites<-meth.sites
		.Object@covg.sites<-covg.sites
		.Object@regions<-regions
		.Object@meth.regions<-meth.regions
		.Object@assembly<-assembly
		.Object@target<-target
		.Object@status<-status
		.Object@inferred.covariates <- inferred.covariates
		
		.Object
		
	}
)
		

#' Wrapper function RnBiseqSet
#' 
#' @param pheno       	phenotypic data.
#' @param sites       	CpG site definition, as a \code{data.frame}
#' 						with 3 variables: chromosome (of type
#' 						\code{character}), position (\code{integer}) and
#' 						strand (\code{character}, one of \code{"+"}, \code{"-"} or \code{"*"}
#' @param meth  		summarized methylation calls as a \code{matrix} or \code{ff_matrix}
#' @param covg 			read coverage information as a \code{matrix} or \code{ff_matrix}
#' @param assembly		the genome assembly
#' @param target		target DNA methylation features (CpG sites)
#' @param summarize.regions ...
#' @param region.types	region annotations for which the methylation data should be summarized 
#' @param useff			flag specifying whether the ff functionality should be used
#' @param usebigff		flag specifying whether the extended ff functionality should be used (large matrix support for ff)
#' @param verbose		flag specifying whether the diagnostic messages should be written to the 
#' 						console or to the RnBeads logger, if the latter is initialized
#' 
#' @return an object of class RnBiseqSet
#' 
#' @name RnBiseqSet
#' @rdname RnBiseqSet-class
#' @aliases initialize,RnBiseqSet-method
#' @export
RnBiseqSet<-function(
		pheno,
		sites,
		meth,
		covg = NULL,
		assembly="hg19",
		target="CpG",
		summarize.regions=TRUE,
		region.types=rnb.region.types.for.analysis(assembly),
		useff=rnb.getOption("disk.dump.big.matrices"),
		usebigff=rnb.getOption("disk.dump.bigff"),
		verbose=FALSE){
	
	if(missing(pheno)){
		stop("argument pheno should be supplied")
	}
	
	if(missing(sites)){
		#warning("Valid RnBeadSet object was not created: pheno and/or betas missing")
		stop("argument meth should be supplied")
	}
	
	if(missing(meth)){
		#warning("Valid RnBeadSet object was not created: pheno and/or betas missing")
		stop("argument meth should be supplied")
	}
	
	if(!is.data.frame(pheno)){
		stop("invalid value for pheno: should be a data frame")
	}
	
	if(!(any(c('data.frame', 'matrix') %in% class(sites)))){
		stop("invalid value for sites: should be a data frame or a matrix of type")
	}
	
	if(!any(c('matrix', 'ff_matrix', 'BigFfMat') %in% class(meth))){
		stop("invalid value for meth: should be a matrix or an ff_matrix")
	}
	
	if(!is.null(covg) && !any(c('matrix', 'ff_matrix', 'BigFfMat') %in% class(covg))){
		stop("invalid value for covg: should be a matrix or an ff_matrix")
	}
	
	if(!is.character(assembly) || length(assembly)!=1L){
		stop("invalid value for assembly: should be a character of length one")
	}
	
	if(!is.character(target) || length(target)!=1L){
		stop("invalid value for target: should be a character of length one")
	}
	
	if(!is.logical(summarize.regions) || length(summarize.regions)!=1L){
		stop("invalid value for summarize.regions: should be a logical of length one")
	}
	
	if(!is.logical(useff) || length(useff)!=1L){
		stop("invalid value for useff: should be a logical of length one")
	}
	
	if (nrow(sites)!= nrow(meth)){
		msg<-c("Inconsistent values for sites and meth")
		rnb.error(msg)
	}
	if(usebigff){
		bff.finalizer <- rnb.getOption("disk.dump.bigff.finalizer")
	}
	
	if(!is.null(rnb.getOption("identifiers.column"))){
		sample.names<-pheno[[rnb.getOption("identifiers.column")]]
	}else if(!is.null(colnames(meth))){
		sample.names<-colnames(meth)
	}else{
		sample.names<-as.character(1:ncol(meth))
	}
	#
	# prepare sites definition
	#
	chroms<-names(rnb.get.chromosomes(assembly=assembly))
	
	if(is.data.frame(sites)){
		sites.df<-sites
		sites<-matrix(nrow=nrow(sites.df), ncol=3, dimnames=list(NULL, c("chr", "coord", "strand")))
		
		sites[,1L]<-as.integer(match(sites.df[,1L], chroms))
		sites[,2L]<-as.integer(sites.df[,2L])
		sites[,3L]<-as.integer(factor(sites.df[,3L], levels=c("+","-","*")))
		
		rm(sites.df)
	}
	
	num.all.sites<-nrow(sites)
	
	#remove sites with illegal chromosomes
	legal.sites <- rowSums(is.na(sites))==0
	num.illegal <- sum(!legal.sites)
	
	sites <- sites[legal.sites,,drop=FALSE]
	#meth <- meth[legal.sites,,drop=FALSE]
	#covg <- covg[legal.sites,,drop=FALSE]
	
	if(verbose && num.illegal > 0) {
		rnb.info(c("Removed", num.illegal, "sites with unknown chromosomes"))
	}
	
	if(nrow(sites)<1L){
		rnb.warning("All sites have been removed, returning NULL")
		return(invisible(NULL))
	}
		
	#
	# match to annotation
	#
	guess.strand <- all(sites[,3L]==3)
	if(verbose && guess.strand) {
		rnb.info("Inferring strand information from annotation enabled")
	}
	# Chromosome ordering fix. 
	#
	#chrdata<-sapply(sort(unique(sites[,1L])), function(chr){
	chrdata<-lapply(sort(unique(sites[,1L])), function(chr){
				#dump<-sapply(unique(sites[,1L]), function(chr){
				chr.subset<-which(sites[,1L]==chr)
				# version with proper ordering
				#
				# chr.subset<-sites[,1L]==chr
				if(guess.strand){
					site.ranges <- GRanges(rep(chroms[chr], length(chr.subset)), 
							IRanges(start=sites[chr.subset,2], 
									width=rep(1,length(chr.subset))), 
							rep("*",length(chr.subset)))
					c.coords.ref <- GenomicRanges::flank(rnb.get.annotation(assembly=assembly)[[chr]],-1)
					
					match <- GenomicRanges::findOverlaps(c.coords.ref, site.ranges, type="start")
				} else {
					site.ranges<-GRanges(rep(chroms[chr], length(chr.subset)), 
							IRanges(start=sites[chr.subset,2L], 
									width=rep(1,length(chr.subset))), 
							c("+","-","*")[sites[chr.subset,3L]])
					
					#				match<-GenomicRanges::findOverlaps(site.ranges, rnb.get.annotation(assembly=assembly)[[chr]], type="start")
					#				sites[chr.subset[queryHits(match)],2]<<-subjectHits(match)
					#				sites[chr.subset[-queryHits(match)],2]<<-NA
					#				return(NULL)
					
					# version with proper ordering
					#			
					match<-GenomicRanges::findOverlaps(rnb.get.annotation(assembly=assembly)[[chr]], site.ranges, type="start")
				}
				bed.h <- subjectHits(match)
				ref.h <- queryHits(match)
				multi.hit.ref <- duplicated(ref.h)
				if(any(multi.hit.ref)){
					rnb.warning(paste("Multiple entries detected in the bed file that match to the same reference C. --> Picking the first one"))
					bed.h <- bed.h[!multi.hit.ref]
					ref.h <- ref.h[!multi.hit.ref]
				}
				multi.hit.bed <- duplicated(bed.h)
				if(any(multi.hit.bed)){
					rnb.warning(paste("Some entries in the bed file match multiple reference Cs. --> Picking the first one"))
					bed.h <- bed.h[!multi.hit.bed]
					ref.h <- ref.h[!multi.hit.bed]
				}
				
                if(length(bed.h)!=length(ref.h)){
                    stop("Something went wrong while matching the sites, exiting")
                } 
                
				n.match <- length(bed.h)
				
				#chrmeths<-meth[chr.subset,,drop=FALSE][bed.h,,drop=FALSE]
				#chrcovgs<-covg[chr.subset,,drop=FALSE][bed.h,,drop=FALSE]
				#indices<-chr.subset[bed.h]
				chrsites<-cbind(rep(1L,n.match),rep(chr, n.match), ref.h, chr.subset[bed.h])
				#return(list(chrsites,chrmeths,chrcovgs))
				chrsites
				
			})
	
	rnb.cleanMem()
	#version with proper ordering
	#sites<-do.call("rbind", chrdata[1,])
	sites<-do.call("rbind", chrdata)
	#meth<-do.call("rbind", chrdata[2,])
	#covg<-do.call("rbind", chrdata[3,])
	valid<-rowSums(is.na(sites))==0
	
	
	if(nrow(sites)<1L){
		rnb.warning("All sites have been removed, returning NULL")
		return(invisible(NULL))
	}else{
	
		if(verbose){
			rnb.status(c("Matched", sum(valid),"of", num.all.sites,"methylation sites to the annotation"))
		}
	}
	
	if(!is.null(covg)){
		if(verbose) rnb.status(c("Checking site coverage")) #TODO: debug info: remove me
		zero.covg.sites <- rowSums(covg[sites[,4L],,drop=FALSE], na.rm=TRUE)==0 #TODO: could this be too memory wasteful?
		if(verbose && any(zero.covg.sites)) {
			rnb.status(c("Removed",length(intersect(which(zero.covg.sites), which(valid))),"of",
							sum(valid),"methylation sites because they were not covered in any sample"))
		}
		valid<-which(valid | zero.covg.sites)
		rnb.cleanMem()
		if(!useff){
			covg[is.na(covg)]<-0L
		}else{
			# if(verbose) rnb.status(c("Setting zero coverages")) #TODO: debug info: remove me
			for(ci in 1:ncol(covg)){
				covg[is.na(covg[,ci]),ci]<-0L
			}
		}
	}
	rnb.cleanMem()
	
	sites <- sites[valid,]
	if(!useff){
		meth <- meth[which(legal.sites)[sites[,4L]],,drop=FALSE]
		if(!is.null(covg)){
			covg <- covg[which(legal.sites)[sites[,4L]],,drop=FALSE]
		}
	}else{
		meth.old <- meth
		if(verbose) rnb.status(c("Creating methylation matrix")) #TODO: debug info: remove me
		if (usebigff) {
			meth <- BigFfMat(row.n=nrow(sites), col.n=ncol(meth.old), row.names=NULL, col.names=sample.names, finalizer=bff.finalizer)
		} else {
			meth <- ff(NA, dim=c(nrow(sites), ncol(meth.old)), dimnames=list(NULL, sample.names), vmode="double")
		}
		# if(verbose) rnb.status(c("Filling methylation matrix")) #TODO: debug info: remove me
		# meth[,] <- meth.old[which(legal.sites)[sites[,4L]],]
		site.inds <- which(legal.sites)[sites[,4L]]
		for (j in 1:ncol(meth.old)) meth[,j] <- meth.old[site.inds,j]
		rm(meth.old)
		rnb.cleanMem()
		if(!is.null(covg)){
			covg.old <- covg
			if(verbose) rnb.status(c("Creating coverage matrix")) #TODO: debug info: remove me
			if (usebigff) {
				covg <- BigFfMat(row.n=nrow(sites), col.n=ncol(covg.old), row.names=NULL, col.names=sample.names, na.prototype=as.integer(NA), finalizer=bff.finalizer)
			} else {
				covg <- ff(NA_integer_, dim=c(nrow(sites), ncol(covg.old)), dimnames=list(NULL, sample.names))
			}
			# if(verbose) rnb.status(c("Filling coverage matrix")) #TODO: debug info: remove me
			# covg[,] <- covg.old[which(legal.sites)[sites[,4L]],]
			for (j in 1:ncol(covg.old)) covg[,j] <- covg.old[site.inds,j]
			rm(covg.old)
			rnb.cleanMem()
		}
	}
	sites <- sites[,-4L]
	colnames(sites)<-c("context", "chr", "index")
	
	meth.sample.n.valid<-integer()
	for(ci in 1:ncol(meth)){
		meth.sample.n.valid[ci] <- sum(!is.na(meth[,ci]))
	}
	
	if(any(meth.sample.n.valid<1)){
		rnb.error(c("The following samples have no valid methylation values:",colnames(meth)[meth.sample.n.valid<1]))
		#stop("invalid sample methylation values")
	}
	
#	sites<-cbind(rep(1, nrow(sites)), sites)
#	sites<-sites[,2:4]
#	colnames(sites)<-c("context", "chr", "index")
	
	#be memory efficient by cleaning up
	# rownames take a lot of space
	
#	sites <- sites[valid,,drop=FALSE]
#	meth  <- meth[valid,,drop=FALSE]
#	covg  <- covg[valid,,drop=FALSE]
	rnb.cleanMem()
#	if(!is.null(covg)){
#		zero.covg.sites <- rowSums(covg, na.rm=TRUE)==0
#		if(any(zero.covg.sites)){
#			valid <- !zero.covg.sites
#			sites <- sites[valid,,drop=FALSE]
#			meth  <- meth[valid,,drop=FALSE]
#			covg  <- covg[valid,,drop=FALSE]
#			rnb.cleanMem()
#
#			if(verbose) {
#				rnb.status(c("Removed",sum(zero.covg.sites),"of",
#								length(valid),"methylation sites because they were not covered in any sample"))
#			}
#		}
#	}
	
	status <- list(disk.dump=FALSE)
	if(useff){
		#subsampling for large datasets (ff currently supports only objects of size .Machine$integer.max)
		if (prod(dim(meth))>.Machine$integer.max & !usebigff){
			sites.allowed <- as.integer(.Machine$integer.max/ncol(meth))
			sample.site.inds <- sort(sample.int(nrow(meth),sites.allowed))
			msg<-c("Full dataset is too large to be supported by ff. --> downsampling to",sites.allowed,"( of",nrow(meth),") sites")
			rnb.warning(msg)
			meth <- meth[sample.site.inds,]
			if(!is.null(covg)){
				covg <- covg[sample.site.inds,]
			}
			sites <- sites[sample.site.inds,]
		}
		
		if("matrix" %in% class(meth)){
			if (usebigff){
				meth <- BigFfMat(meth, finalizer=bff.finalizer)
			} else {
				meth <- convert.to.ff.matrix.tmp(meth)
			}
		}
		
		if(!is.null(covg)){
			if("matrix" %in% class(covg)){
				if (usebigff){
					covg <- BigFfMat(covg, finalizer=bff.finalizer)
				} else {
					covg <- convert.to.ff.matrix.tmp(covg)
				}
			}
		}
		status <- list(disk.dump=TRUE, disk.dump.bigff=usebigff)
	}

	if(verbose) rnb.status(c("Creating object")) #TODO: debug info: remove me
	object<-new("RnBiseqSet",
			pheno=pheno,
			sites=sites,
			meth.sites=meth,
			covg.sites=covg,
			status=status,
			assembly=assembly,
			target=target)
	
	if(!rnb.getOption("strand.specific")){
		msg<-c("Summarizing","strand","methylation")
		if(verbose){
			rnb.status(msg)
		}
		if(is.null(covg)){
			aggr<-"mean"
		}else{
			aggr<-"coverage.weighted"
		}
		object <- summarize.regions(object, "strands", aggregation=aggr)
		rnb.cleanMem()
	}
	
	for (region.type in region.types) {
		if (region.type %in% rnb.region.types(assembly)) {
			msg<-c("Summarizing",region.type,"methylation")
			if(verbose){
				rnb.status(msg)
			}
			object <- summarize.regions(object, region.type)
			rnb.cleanMem()
		}
	}
	object
}


## ---------------------------------------------------------------------------------------------------------------------
## VALIDITY
## ---------------------------------------------------------------------------------------------------------------------
#
# check.rnb.biseq.set
#
# A routine that checks the validity of a freshly loaded RnBiseqSet object
# 
# #param object RnBiseqSet object to test
# #param verbose A flag specifying whether the logger is initialized
#
# #return TRUE if all checks are successful
# 
# Pavlo Lutsik
#
check.rnb.biseq.set<-function(object, verbose=TRUE){
	
	
	if(!inherits(object, "RnBiseqSet")){
		if(verbose){ 
			rnb.info("The supplied object is not of class RnBiseqSet. Breaking the check...")
		}
		return(FALSE)
	}else{
		if(verbose){
			rnb.info("Checking the supplied RnBiseqSet object")
		}
		valid<-TRUE
	}
	
	#check the methylation data
	
	mm <- meth(object)
	
	if(nrow(mm)<1){
		if(verbose){
			rnb.info("The object contains information for 0 methylation sites")
		}
		valid<-FALSE
	}else{
		if(verbose){
			rnb.info(sprintf("The object contains information for %d methylation sites",nrow(mm)))
		}
	}
	
	if(ncol(mm)<1){
		if(verbose){
			rnb.info("The object contains information for 0 samples")
		}
		valid<-FALSE
	}else{
		if(verbose){
			rnb.info(sprintf("The object contains information for %d samples",ncol(mm)))
		}
	}
	
	if(sum(!is.na(mm))<1){
		if(verbose){
			rnb.info("All methylation values are missing")
		}
		valid<-FALSE
	}else{
		if(verbose){
			rnb.info(sprintf("The object contains %d missing methylation values",sum(is.na(mm))))
		}
	}
	
	if(sum(mm[!is.na(mm)]<0 & mm[!is.na(mm)]>1)>0){
		if(verbose){
			rnb.info("The object contains incorrect methylation values (less than 0 or greater than 1)")
		}
		valid<-FALSE
	}else{
		if(verbose){
			rnb.info("Methylation values are within the expected range")
		}
	}
	
	#check the coverage data
	
	if(is.null(object@covg.sites)){
		if(verbose){
			rnb.info("No coverage information found")
		}
		valid<-FALSE
		return(valid)
	}else{
		if(verbose){
			rnb.info("The object contains coverage information")
		}
		cvg<-covg(object)
	}
	
	if(sum(cvg[!is.na(cvg)]<0)>0){
		if(verbose){
			rnb.info("The object contains incorrect coverage values (negative values)")
		}
		valid<-FALSE
	}else if(sum(cvg[!is.na(cvg)]>1000)>0.5*length(as.numeric(cvg))){
		if(verbose){
			rnb.info("The object contains incorrect coverage values (half of the sites have coverage > 1000 )")
		}
	}else{
		if(verbose){
			rnb.info("Coverage values are within the expected range")
		}
	}
	
	return(valid)
	
}

########################################################################################################################

setMethod("show", "RnBiseqSet",
	function(object) {
		cat("Object of class RnBiseqSet\n")
		cat(sprintf("%8d samples\n", nrow(object@pheno)))
		cat(sprintf("%8d methylation sites\n", nrow(object@sites)))
		if (!is.null(object@regions)){
			cat("Region types:\n")
			for (rn in names(object@regions)) {
				cat(sprintf("\t%8d regions of type %s\n", nrow(object@regions[[rn]]), rn))
			}
		}
		cat(sprintf("Coverage information is %s\n", ifelse(is.null(object@covg.sites), "absent", "present")))
		if(isImputed(object)){
		  cat("Data set was imputed.\n")
		}
	}
)

########################################################################################################################
#' as("RnBeadSet", "RnBiseqSet")
#'
#' Convert a \code{\linkS4class{RnBeadSet}} object to a "mock" \code{\linkS4class{RnBiseqSet}} object
#' (used in the combine method)
#' 
#' @name coercion-methods
#' 
setAs("RnBeadSet", "RnBiseqSet",
        
        function(from, to){
            
            ann<-annotation(from)
            site.info<-ann[,c("Chromosome", "Start", "Strand")]
            
            object<-RnBiseqSet(
                    pheno=pheno(from),
                    sites=site.info,
                    meth=meth(from),
                    covg=NULL,
                    assembly=from@assembly,
                    target="CpG",
                    useff=isTRUE(from@status$disk.dump),
                    usebigff=isTRUE(from@status$disk.dump.bigff),
                    verbose=FALSE
            )
            
            object
        })
########################################################################################################################

########################################################################################################################
## segmentation.R
## created: 2019-01-04
## creator: Michael Scherer
## ---------------------------------------------------------------------------------------------------------------------
## Functions for segmentation of the methylome through MethylSeekR
########################################################################################################################

#' rnb.execute.segmentation
#' 
#' This function computes methylation segmentation by MethylSeekR into PMDs, UMRs/LMRs, and HMDs. It is recommened to only
#' execute this function on WGBS data  (with coverage >=10 according to the developer's recommendation), but could also be
#' used with RRBS_HaeIII without guarantee and the results should be interpreted carefully.
#' 
#' @param rnb.set An object of type \code{\link{RnBiseqSet-class}} containing methylation and coverage information.
#' @param sample.name The sample for which segmentation is to be executed. Segemntation can only be exectued for each sample
#'                     individually.
#' @param meth.level Methylation cutoff to be used in UMR/LMR computation
#' @param fdr False discovery rate cutoff to be used in percent
#' @param min.cover The coverage threshold
#' @param n.cores The number of cores available for analysis
#' @param chr.sel Selected chromosome for model training in "chrAB" format. Defaults to "chr2".
#' @param plot.path Location on disk on which diagnostic plots are to be stored. Defaults to the working directory.
#' @param temp.dir The temporary directory. Defaults to the R temporary directory.
#' @return The input RnBSet object with segementation added as an additional region type. Furthermore, three new annotations
#'          are set globally containing segmentation into PMDs, UMRs/LMRs, and HMDs for the sample that was specified.
#' @details For further descriptions on the methods, see \code{MethylSeekR}-documentation. The new annotations can be accessed
#'          via \code{rnb.get.annotation("[PMDs,UMRs,LMRs,HMDs]_[sample.name]")}.
#' @references 1. Burger, Lukas, Gaidatzis, Dimos, Schuebeler, Dirk, and Stadler, Michael B. (2013)
#'             Identification of active regulatory regions from DNA methylation data.
#'             Nucleic Acids Research 41(16):e155.
#' @author Michael Scherer, based on a script by Abdulrahman Salhab
#' @export
rnb.execute.segmentation <- function(rnb.set,
                                     sample.name,
                                     meth.level=0.5,
                                     fdr=5,
                                     min.cover=5,
                                     n.cores=1,
                                     chr.sel="chr2",
                                     plot.path=getwd(),
                                     temp.dir=tempdir()){
  if(!inherits(rnb.set,"RnBiseqSet")){
    logger.error("Invalid value for rnb.set, needs to be RnBiseqSet")
  }
  if(length(sample.name)>1){
    logger.error("Only single sample can be analysed")
  }
  rnb.require("MethylSeekR")
  asb <- assembly(rnb.set)
  if(asb %in% c("hg19","hg38")){
    rnb.require(paste0("BSgenome.Hsapiens.UCSC.",asb))
    myGenomeSeq <- Hsapiens
  }else if(asb %in% "mm10"){
    rnb.require(paste0("BSgenome.Mmusculus.UCSC.",asb))
    myGenomeSeq <- Mmusculus
  }else{
    logger.info("Invalid assembly; Segmentation only supported for hg19, hg38 and mm10")
  }

  sLengths <- seqlengths(rnb.get.annotation("CpG",assembly = asb))
  sLengths.gr <- GRanges(Rle(names(sLengths)),IRanges(start = 1,end=sLengths))
  CpGislands.gr <- rnb.get.annotation("cpgislands",assembly = asb)
  CpGislands.gr <-suppressWarnings(resize(CpGislands.gr, 5000, fix="center"))
  session <- browserSession(url = 'http://genome-euro.ucsc.edu/cgi-bin/')
  genome(session) <- asb
  query <- ucscTableQuery(session, table="gap")
  gaps <- getTable(query)
  gaps.gr <- makeGRangesFromDataFrame(gaps, starts.in.df.are.0based=TRUE) # the second argument to be discussed 0/1 based
  
  if(any(!(rnb.region.types.for.analysis(rnb.set) %in% rnb.region.types(assembly(rnb.set))))){
    rnb.load.annotation.from.db(rnb.region.types.for.analysis(rnb.set)[!(rnb.region.types.for.analysis(rnb.set) %in% rnb.region.types(assembly(rnb.set)))],assembly(rnb.set))
  }
  sel.sample <- samples(rnb.set) %in% sample.name
  if(!any(sel.sample)){
    logger.error(paste("Specified sample",sample.name,"not in RnBSet"))
  }
  sel.sample <- which(sel.sample)
  meth.rnb <- rnb.set@meth.sites[,sel.sample]
  is.na.meth <- is.na(meth.rnb)
  covg.rnb <- rnb.set@covg.sites[,sel.sample]
  meth.matrix <- round(meth.rnb*covg.rnb)
  nas <- is.na(covg.rnb) | is.na.meth
  anno.rnb <- data.frame2GRanges(annotation(rnb.set),assembly=assembly(rnb.set))
  anno.rnb <- sort(anno.rnb)
  anno.rnb <- anno.rnb[!nas]
  tmp.anno <- anno.rnb
  values(tmp.anno) <- data.frame(T=covg.rnb[!nas],M=meth.matrix[!nas])
  tmp.file.meth <- file.path(temp.dir,"GRanges_for_MethylSeekR.RDS")
  saveRDS(tmp.anno,tmp.file.meth)
  data.gr <- readMethylome(tmp.file.meth, seqLengths=sLengths, format="GRanges")
  
  ### read SNPs
  anno.rnb <- anno.rnb[!is.na(values(anno.rnb)$SNPs)]
  values(anno.rnb) <- values(anno.rnb)$SNPs
  tmp.file.snps <- file.path(temp.dir,"GRangesSNP_for_MethylSeekR.RDS")
  saveRDS(anno.rnb,tmp.file.snps)
  snps.gr <- readSNPTable(tmp.file.snps, seqLengths=sLengths, format="GRanges")
  
  ### removing SNPs
  if(length(snps.gr)>0){
    cat("Filtering the SNPs ......\n")
    data.gr <- removeSNPs(data.gr, snps.gr)
  }
  
  ### find PMD and plot
  logger.start("Detecting PMDs")
  PMDsegments.gr <- segmentPMDs(m=data.gr,pdfFilename=file.path(plot.path,paste(sample.name,"alpha.model.fit.pdf",sep=".")), chr.sel=chr.sel,seqLengths=sLengths, num.cores=n.cores)
  logger.completed()
  
  logger.start("Plotting alpha distribution")
  plotAlphaDistributionOneChr(m=data.gr, chr.sel=chr.sel,pdfFilename=file.path(plot.path,paste(sample.name,"alpha_distribution.pdf",sep=".")),num.cores=n.cores)
  logger.completed()
  
  ### FDR calculation
  logger.start("FDR calculation")
  stats <- calculateFDRs(m=data.gr, CGIs=CpGislands.gr,PMDs=PMDsegments.gr, nCpG.cutoffs =seq(1, 17, by=3),pdfFilename=file.path(plot.path,paste(sample.name,"FDR.pdf",sep=".")),num.cores=n.cores)
  FDR.cutoff <- as.numeric(fdr)
  m.sel <- as.numeric(meth.level)
  n.sel <- as.integer(names(stats$FDRs[as.character(m.sel), ][stats$FDRs[as.character(m.sel), ]<FDR.cutoff])[1])
  logger.info(paste("Minimum number of CpGs in LMRs:",n.sel,"CpGs"))
  logger.completed()
  
  ### find UMR and LMR
  logger.start("Detecting UMRs and LMRs")
  UMRLMRsegments.gr <- segmentUMRsLMRs(m=data.gr, meth.cutoff=m.sel,nCpG.cutoff=n.sel, PMDs=PMDsegments.gr,pdfFilename=file.path(plot.path,paste(sample.name,"UMR.LMR.scatter.pdf",sep=".")),num.cores=n.cores, myGenomeSeq=myGenomeSeq,seqLengths=sLengths, minCover = min.cover)
  logger.completed()
  
  # create final segmentation
  logger.start("Create final segmentation")
  PMDsegments.gr <- PMDsegments.gr[PMDsegments.gr$type=="PMD"]
  PMDsegments.gr <- setdiff(PMDsegments.gr,gaps.gr)
  PMDsegments.gr$type <- "PMD"
  hmr.segments<- setdiff(sLengths.gr,c(PMDsegments.gr,UMRLMRsegments.gr,gaps.gr))
  values(hmr.segments)$HMR <- rep("HMR",length(hmr.segments))
  logger.completed()
  
  # set new annotations PMD, UMR/LMR, HMR
  logger.start("Set new annotations and summarize methylation")
  pmd.frame <- data.frame(Chromosome=seqnames(PMDsegments.gr),Start=start(PMDsegments.gr),End=end(PMDsegments.gr),
                           PMDs=values(PMDsegments.gr)$type)
  rnb.set.annotation(paste0("PMDs_",sample.name),regions=pmd.frame,description = "Partially Methylated Domains by MethylSeekR",assembly = asb)
  umr.lmr.frame <- data.frame(Chromosome=seqnames(UMRLMRsegments.gr),Start=start(UMRLMRsegments.gr),End=end(UMRLMRsegments.gr),
                              UMRsLMRs=values(UMRLMRsegments.gr)$type)
  umr.frame <- umr.lmr.frame[umr.lmr.frame$UMRsLMRs=="UMR",]
  rnb.set.annotation(paste0("UMRs_",sample.name),regions=umr.frame,description = "Unmethylated Regions by MethylSeekR",assembly = asb)  
  lmr.frame <- umr.lmr.frame[umr.lmr.frame$UMRsLMRs=="LMR",]
  rnb.set.annotation(paste0("LMRs_",sample.name),regions=lmr.frame,description = "Lowly Methylated Regions by MethylSeekR",assembly = asb)
  hmr.frame <- data.frame(Chromosome=seqnames(hmr.segments),Start=start(hmr.segments),End=end(hmr.segments),
                          HMDs=values(hmr.segments)$HMR)
  rnb.set.annotation(paste0("HMDs_",sample.name),regions=hmr.frame,description = "Highly Methylated Regions by MethylSeekR",assembly = asb)
  
  rnb.set <- summarize.regions(rnb.set,paste("PMDs",sample.name,sep="_"))
  rnb.set <- summarize.regions(rnb.set,paste("UMRs",sample.name,sep="_"))
  rnb.set <- summarize.regions(rnb.set,paste("LMRs",sample.name,sep="_"))
  rnb.set <- summarize.regions(rnb.set,paste("HMDs",sample.name,sep="_"))
  logger.completed()
  
  unlink(tmp.file.meth)
  unlink(tmp.file.snps)
  return(rnb.set)
}

#' rnb.bed.from.segmentation
#' 
#' This function creates a BED file from the segmentation result of \code{rnb.execute.segmentation} and stores it on disk.
#' 
#' @param rnb.set An \code{\link{RnBSet-class}} object obtained by executing \code{rnb.execute.segmentation}.
#' @param sample.name The sample name for which segmentation was computed.
#' @param type The type of segmentation (\code{PMDs}, \code{UMRs}, \code{LMRs}, \code{HMDs} or \code{final}).
#' @param store.path Path to which the BED file is to be stored.
#' @author Michael Scherer
#' @export
rnb.bed.from.segmentation <- function(rnb.set,
                                      sample.name,
                                      type="final",
                                      store.path=getwd()){
  if(!type %in% c("PMDs","UMRs","LMRs","HMDs","final")){
    logger.error("Invalid value for type, needs to be PMDs, UMRs, LMRs or HMDs.")
  }
  if(!(sample.name %in% samples(rnb.set))){
    logger.error("Specify a sample that is available in the rnb.set")
  }
  if(type != "final"){
    region.name <- paste(type,sample.name,sep = "_")
    if(!(region.name %in% summarized.regions(rnb.set))){
      logger.error("Segmentation not yet available, execute rnb.execute.segementation first")
    }
    bed.frame <- annotation(rnb.set,region.name)
    meth.seg <- meth(rnb.set,region.name)[,sample.name]
    bed.frame <- data.frame(bed.frame[,c("Chromosome","Start","End",type)],AvgMeth=meth.seg)
  }else{
    bed.frame <- rnb.final.segmentation(rnb.set,sample.name)
  }
  write.table(bed.frame,file.path(store.path,paste0(sample.name,"_",type,".bed")),sep="\t",row.names=F,col.names=F,quote = F)
}

#' rnb.boxplot.from.segmentation
#' 
#' This function creates a boxplot from the segmentation result of \code{rnb.execute.segmentation}.
#' 
#' @param rnb.set An \code{\link{RnBSet-class}} object obtained by executing \code{rnb.execute.segmentation}.
#' @param sample.name The sample name for which segmentation was computed.
#' @param type The type of segmentation (\code{PMDs}, \code{UMRs}, \code{LMRs}, \code{HMDs} or \code{final}).
#' @return An object of type \code{ggplot} visualizing the methylation values in the segments.
#' @author Michael Scherer
#' @export
rnb.boxplot.from.segmentation <- function(rnb.set,
                                          sample.name,
                                          type="final"){
  if(!type %in% c("PMDs","UMRs","LMRs","HMDs","final")){
    logger.error("Invalid value for type, needs to be PMDs, UMRs, LMRs or HMDs.")
  }
  if(!(sample.name %in% samples(rnb.set))){
    logger.error("Specify a sample that is available in the rnb.set")
  }
  if(type != "final"){
    region.name <- paste(type,sample.name,sep = "_")
    if(!(region.name %in% summarized.regions(rnb.set))){
      logger.error("Segmentation not yet available, execute rnb.execute.segementation first")
    }
    bed.frame <- annotation(rnb.set,region.name)
    meth.seg <- meth(rnb.set,region.name)[,sample.name]
    to.plot <- data.frame(bed.frame[,type],AvgMeth=meth.seg)
    colnames(to.plot)[1] <- "Segment"
  }else{
    to.plot <- rnb.final.segmentation(rnb.set,sample.name)
    colnames(to.plot)[4] <- "Segment"
    to.plot[,"Segment"] <- factor(to.plot[,"Segment"], levels=c("HMR","PMD","LMR","UMR"))
  }
  plot <- ggplot(to.plot,aes(x=Segment,y=AvgMeth,fill=Segment))+geom_boxplot()+scale_fill_manual(values=rnb.getOption("colors.category"))
  return(plot)
}  

#' rnb.final.segmentation
#' 
#' This function creates a single segmentation, assigning each region to PMD, HMR, UMR or LMR.
#' 
#' @param rnb.set The \code{\link{RnBSet-class}}-object, for which segmentation was computed using \code{\link{rnb.execute.segmentation}}
#' @param sample.name The sample name in \code{rnb.set} for which segmentation was conducted.
#' @author Michael Scherer
#' @noRd
rnb.final.segmentation <- function(rnb.set,
                                   sample.name){
  if(!(sample.name %in% samples(rnb.set))){
    logger.error("Specify a sample that is available in the rnb.set")
  }
  region.names <- c(paste("PMDs",sample.name,sep = "_"),paste("HMDs",sample.name,sep = "_"),paste("UMRs",sample.name,sep = "_"),paste("LMRs",sample.name,sep = "_"))
  if(any(!(region.names %in% summarized.regions(rnb.set)))){
    logger.error("Segmentation not yet available, execute rnb.execute.segementation first")
  }
  pmd.frame <- annotation(rnb.set,region.names[1])
  pmd.meth <- meth(rnb.set,region.names[1])[,sample.name]
  pmd.frame <- data.frame(pmd.frame,AvgMeth=pmd.meth)
  colnames(pmd.frame)[5] <- "Segment"
  hmr.frame <- annotation(rnb.set,region.names[2])
  hmr.meth <- meth(rnb.set,region.names[2])[,sample.name]
  hmr.frame <- data.frame(hmr.frame,AvgMeth=hmr.meth)
  colnames(hmr.frame)[5] <- "Segment"
  umr.frame <- annotation(rnb.set,region.names[3])
  umr.meth <- meth(rnb.set,region.names[3])[,sample.name]
  umr.frame <- data.frame(umr.frame,AvgMeth=umr.meth)
  colnames(umr.frame)[5] <- "Segment"
  lmr.frame <- annotation(rnb.set,region.names[4])
  lmr.meth <- meth(rnb.set,region.names[4])[,sample.name]
  lmr.frame <- data.frame(lmr.frame,AvgMeth=lmr.meth)
  colnames(lmr.frame)[5] <- "Segment"
  final.frame <- data.frame(rbind(pmd.frame,rbind(hmr.frame,umr.frame,lmr.frame)))
  final.frame <- final.frame[order(final.frame$Chromosome,final.frame$Start,final.frame$End),]
  color.code <- rep("202,0,32",nrow(final.frame))
  color.code[final.frame$Segment %in% "PMD"] <- "244,165,130"
  color.code[final.frame$Segment %in% "UMR"] <- "5,113,176"
  color.code[final.frame$Segment %in% "LMR"] <- "146,197,222"
  final.frame <- data.frame(final.frame[,c(1,2,3,5,6)],rep(".",nrow(final.frame)),final.frame[,c(2,3)],color.code)
  return(final.frame)
}

#' rnb.plot.segmentation.final
#' 
#' This functions plots the final segmentation result.
#' 
#' @param data.gr The data object as \code{\link{GRanges}} object
#' @param UMR.LMR.segments.gr Segmentation into UMR/LMR as GRanges
#' @param PMD.segments Segmentation into PMDs/notPMDs as GRanges
#' @param n.regions Number of regions
#' @param meth.cutoff The methylation cutoff
#' @author Michael Scherer
#' @noRd
rnb.plot.segmentation.final <- function(data.gr,
                                        UMR.LMR.segments.gr,
                                        PMD.segments,
                                        n.regions=4,
                                        meth.cutoff){
  logger.start("Plotting final segmentation")
  plotFinalSegmentation(m=data.gr, segs=UMR.LMR.segments.gr,PMDs=PMD.segments,numRegions = n.regions,meth.cutoff=meth.cutoff)
  logger.completed()
}

#'rnb.plot.segmentation.distributions
#'
#'Plots the distributions of methylation and coverage.
#'
#'@param data.gr The data object as \code{\link{GRanges}} object
#'@author Michael Scherer
#'@noRd
rnb.plot.segmentation.distributions <- function(data.gr){
  df <- as.data.frame(data.gr)
  df$meth <- df$M/df$T
  meth.plot <- ggplot(df[df$T>=5,], aes(x=df[df$T>=5,"meth"])) + geom_density(colour="dodgerblue1",size=1) +  ylab("density") + xlab("beta value") + ggtitle("Methylation level density")
  covg.plot <- ggplot(df, aes(x=df$T)) + geom_histogram(binwidth=1,alpha=.5,position="identity",colour = "dodgerblue1", fill = "dodgerblue1") + geom_vline(xintercept=mean(df$T),colour="black", linetype = "longdash")  + ylab("Frequency") + xlab("read coverage per CpG") + geom_text(aes(x2,y2,label = texthere),data.frame(x2=mean(df$T), y2=max(table(df$T)), texthere=round(mean(df$T),2)))
  return((list(Methylation=meth.plot,Coverage=covg.plot)))
}

################################################################################
# RnBeadsDJ - RnBeads Data Juggler
# A Shiny app for configuring and running RnBeads analyses
#-------------------------------------------------------------------------------
# Main app file
#-------------------------------------------------------------------------------
# created: 2017-10-04
# creator: Fabian Mueller
#-------------------------------------------------------------------------------
# Run using:
# shiny::runApp(file.path('RnBeadsDJ'))
################################################################################
library(shiny)
library(shinyjs)
# library(shinyFiles)
library(RnBeads)

################################################################################
# Options
################################################################################
options(shiny.maxRequestSize=50*1024^2)

################################################################################
# globals
################################################################################
REFRESH.TIME <- 5000
RNB.MODULES <- c(
	"Data Import"="data_import",
	"Quality Control"="quality_control",
	"Preprocessing"="preprocessing",
	"Tracks and Tables"="tracks_and_tables",
	"Covariate Inference"="covariate_inference",
	"Exploratory Analysis"="exploratory_analysis",
	"Differential Methylation"="differential_methylation"
)
RNB.MODULES.LOG.MSG <- c(
	"data_import"="Loading Data",
	"quality_control"="Quality Control",
	"preprocessing"="Preprocessing",
	"tracks_and_tables"="Tracks and Tables",
	"covariate_inference"="Covariate Inference",
	"exploratory_analysis"="Exploratory Analysis",
	"differential_methylation"="Differential Methylation"
)
RNB.PLATFORMS <- c("Bisulfite Sequencing"="biseq", "Illumina EPIC"="illEpic", "Illumina EPIC v2"="illEpicv2", "Illumina 450k"="ill450k", "Illumina27k"="ill27k", 'Illumina MMBC'='illMMBC')
RNB.ASSEMBLIES <- rnb.get.assemblies()
RNB.TABLE.SEPS <- c("comma" = ",", "tab"="\t")
RNB.BED.STYLES <- c("BisSNP"="BisSNP", "ENCODE"="Encode", "EPP"="EPP", "Bismark cytosine"="bismarkCytosine", "Bismark coverage"="bismarkCov")
RNB.FILTERING.SNP <- c("No filtering"="no", "3 SNPs"="3", "5 SNPs"="5", "Any SNPs"="any") ## TODO: Include SNPs MAF from the new hg38 annotation
RNB.NORMALIZATION.METHODS=c("none", "bmiq", "illumina", "swan", "minfi.funnorm", "wm.dasen", "wm.nasen", "wm.betaqn", "wm.naten", "wm.nanet", "wm.nanes", "wm.danes", "wm.danet", "wm.danen", "wm.daten1", "wm.daten2", "wm.tost", "wm.fuks", "wm.swan", "scaling.internal")
RNB.NORMALIZATION.BG.METHODS <- c("none", "methylumi.noob", "methylumi.goob", "enmix.oob", "sesame.noobsb")
RNB.IMPUTATION.METHODS <- c("none", "mean.cpgs", "mean.samples", "random", "knn")
RNB.TRACKHUB.FORMATS <- c("bigBed", "bigWig")
RNB.SVA.NUM.METHODS <- c("leek", "be")
RNB.DIFFMETH.TEST.METHODS <- c("limma", "refFreeEWAS")
RNB.DIFFVAR.METHODS <- c("diffVar", "iEVORA")
RNB.COLSCHEMES.CATEGORY <- list(
	default=c("#1B9E77","#D95F02","#7570B3","#E7298A","#66A61E","#E6AB02","#A6761D","#666666"),
	extended=c("#1B9E77","#D95F02","#7570B3","#E7298A","#66A61E","#E6AB02","#A6761D","#666666","#2166AC","#B2182B","#00441B","#40004B","#053061","#003D7C","#D50911")
)
RNB.COLSCHEMES.METH <- list(
	default=c("#AD0021","#909090","#39278C"),
	YlBl=c("#EDF8B1","#41B6C4","#081D58")
)

RNB.OPTION.DESC.TAB <- RnBeads:::rnb.options.description.table()
RNB.OPTION.DESC <- sapply(names(rnb.options()), FUN=function(x){
	if (is.element(x, rownames(RNB.OPTION.DESC.TAB))){
		return(RNB.OPTION.DESC.TAB[x, "desc"])
	} else {
		return("See the help pages: '?rnb.options'")
	}
})

RNB.GROUP.SIZE.RANGE  <- c(1, 20)
RNB.GROUP.COUNT.RANGE <- c(2, 20)

RNB.OPTION.PROFILES.PATH <- system.file(file.path("extdata", "optionProfiles"), package="RnBeads")
RNB.OPTION.PROFILES <- gsub("\\.xml$", "", list.files(path=RNB.OPTION.PROFILES.PATH, pattern="\\.xml$"))
################################################################################
# Choose local file or directory
# adapted from https://github.com/wleepang/shiny-directory-input
################################################################################
# Interactive Choosers for MacOS and Linux (For Windos this is already implemented in the utils package)
#' Choose a Folder Interactively (Mac OS X)
#'
#' Display a folder selection dialog under Mac OS X
#'
#' @param default which folder to show initially
#' @param caption the caption on the selection dialog
#'
#' @details
#' Uses an Apple Script to display a folder selection dialog.  With \code{default = NA},
#' the initial folder selection is determined by default behavior of the
#' "choose folder" Apple Script command.  Otherwise, paths are expanded with
#' \link{path.expand}.
#'
#' @return
#' A length one character vector, character NA if 'Cancel' was selected.
#'
if (Sys.info()['sysname'] == 'Darwin') {
	choose.dir <- function(default = NA, caption = NA, ...) {
		command = 'osascript'
		args = '-e "POSIX path of (choose folder{{prompt}}{{default}})"'

		if (!is.null(caption) && !is.na(caption) && nzchar(caption)) {
			prompt = sprintf(' with prompt \\"%s\\"', caption)
		} else {
			prompt = ''
		}
		args = sub('{{prompt}}', prompt, args, fixed = T)

		if (!is.null(default) && !is.na(default) && nzchar(default)) {
			default = sprintf(' default location \\"%s\\"', path.expand(default))
		} else {
			default = ''
		}
		args = sub('{{default}}', default, args, fixed = T)

		suppressWarnings({
			path = system2(command, args = args, stderr = TRUE)
			path <- path[length(path)]
		})
		if (!file.exists(path)) {
			# user canceled
		 	path = NA
		}

		return(path)
	}

	choose.files <- function(default = NA, caption = NA, ...) {
		command = 'osascript'
		args = '-e "POSIX path of (choose file{{prompt}}{{default}})"'

		if (!is.null(caption) && !is.na(caption) && nzchar(caption)) {
			prompt = sprintf(' with prompt \\"%s\\"', caption)
		} else {
			prompt = ''
		}
		args = sub('{{prompt}}', prompt, args, fixed = T)

		if (!is.null(default) && !is.na(default) && nzchar(default)) {
			default = sprintf(' default location \\"%s\\"', path.expand(default))
		} else {
			default = ''
		}
		args = sub('{{default}}', default, args, fixed = T)

		suppressWarnings({
			path = system2(command, args = args, stderr = TRUE)
			path <- path[length(path)]
		})
		if (!file.exists(path)) {
			# user canceled
			path = NA
		}

		return(path)
	}
} else if (Sys.info()['sysname'] == 'Linux') {
	library(tcltk)
	choose.dir <- tk_choose.dir
	choose.files <- tk_choose.files
}

#' Directory Selection Control
#'
#' Create a directory selection control to select a directory on the server
#'
#' @param inputId The \code{input} slot that will be used to access the value
#' @param label Display label for the control, or NULL for no label
#' @param value Initial value.  Paths are exapnded via \code{\link{path.expand}}.
#'
#' @details
#' This widget relies on \link{\code{choose.dir}} to present an interactive
#' dialog to users for selecting a directory on the local filesystem.  Therefore,
#' this widget is intended for shiny apps that are run locally - i.e. on the
#' same system that files/directories are to be accessed - and not from hosted
#' applications (e.g. from shinyapps.io).
#'
#' @return
#' A directory input control that can be added to a UI definition.
#'
#' @seealso
#' \link{updateDirectoryInput}, \link{readDirectoryInput}, \link[utils]{choose.dir}
directoryInput = function(inputId, label, value = NULL) {
	if (!is.null(value) && !is.na(value)) {
		value = path.expand(value)
	}

	tagList(
		singleton(
			tags$head(
				tags$script(src = 'js/directory_input_binding.js')
			)
		),

		div(
			class = 'form-group directory-input-container',
			tags$label(label),
			div(
				span(
					class = 'col-xs-9 col-md-11',
					style = 'padding-left: 0; padding-right: 5px;',
					div(
						class = 'input-group shiny-input-container',
						style = 'width:100%;',
						div(class = 'input-group-addon', icon('folder-o')),
						tags$input(
							id = sprintf('%s__chosen_dir', inputId),
							value = value,
							type = 'text',
							class = 'form-control directory-input-chosen-dir',
							readonly = 'readonly'
						)
					)
				),
				span(
					class = 'shiny-input-container',
					tags$button(
						id = inputId,
						class = 'btn btn-default directory-input',
						'...'
					)
				)
			)
		)
	)
}

#' Local File Selection Control
#'
#' Create a local file selection control to select a local file on the server
#'
#' @param inputId The \code{input} slot that will be used to access the value
#' @param label Display label for the control, or NULL for no label
#' @param value Initial value.  Paths are exapnded via \code{\link{path.expand}}.
#'
#' @details
#' This widget relies on \link{\code{choose.files}} to present an interactive
#' dialog to users for selecting files on the local filesystem.  Therefore,
#' this widget is intended for shiny apps that are run locally - i.e. on the
#' same system that files/directories are to be accessed - and not from hosted
#' applications (e.g. from shinyapps.io).
#'
#' @return
#' A local file input control that can be added to a UI definition.
#'
#' @seealso
#' \link{updateLocalFileInput}, \link{readLocalFileInput}, \link[utils]{choose.files}
localFileInput = function(inputId, label, value = NULL) {
	if (!is.null(value) && !is.na(value)) {
		value = path.expand(value)
	}

	tagList(
		singleton(
			tags$head(
				tags$script(src = 'js/localfile_input_binding.js')
			)
		),

		div(
			class = 'form-group localfile-input-container',
			tags$label(label),
			div(
				span(
					class = 'col-xs-9 col-md-11',
					style = 'padding-left: 0; padding-right: 5px;',
					div(
						class = 'input-group shiny-input-container',
						style = 'width:100%;',
						div(class = 'input-group-addon', icon('file-o')),
						tags$input(
							id = sprintf('%s__chosen_file', inputId),
							value = value,
							type = 'text',
							class = 'form-control localfile-input-chosen-file',
							readonly = 'readonly'
						)
					)
				),
				span(
					class = 'shiny-input-container',
					tags$button(
						id = inputId,
						class = 'btn btn-default localfile-input',
						'...'
					)
				)
			)
		)
	)
}

#' Change the value of a directoryInput on the client
#'
#' @param session The \code{session} object passed to function given to \code{shinyServer}.
#' @param inputId The id of the input object.
#' @param value A directory path to set
#' @param ... Additional arguments passed to \link{\code{choose.dir}}.  Only used
#'    if \code{value} is \code{NULL}.
#'
#' @details
#' Sends a message to the client, telling it to change the value of the input
#' object.  For \code{directoryInput} objects, this changes the value displayed
#' in the text-field and triggers a client-side change event.  A directory
#' selection dialog is not displayed.
#'
updateDirectoryInput = function(session, inputId, value = NULL, ...) {
  if (is.null(value)) {
    value = choose.dir(...)
  }
  session$sendInputMessage(inputId, list(chosen_dir = value))
}

#' Change the value of a localFileInput on the client
#'
#' @param session The \code{session} object passed to function given to \code{shinyServer}.
#' @param inputId The id of the input object.
#' @param value A file path to set
#' @param ... Additional arguments passed to \link{\code{choose.files}}.  Only used
#'    if \code{value} is \code{NULL}.
#'
#' @details
#' Sends a message to the client, telling it to change the value of the input
#' object.  For \code{localFileInput} objects, this changes the value displayed
#' in the text-field and triggers a client-side change event.  A file
#' selection dialog is not displayed.
#'
updateLocalFileInput = function(session, inputId, value = NULL, ...) {
  if (is.null(value)) {
    value = choose.files(...)
  }
  session$sendInputMessage(inputId, list(chosen_file = value))
}

#' Read the value of a directoryInput
#'
#' @param session The \code{session} object passed to function given to \code{shinyServer}.
#' @param inputId The id of the input object
#'
#' @details
#' Reads the value of the text field associated with a \code{directoryInput}
#' object that stores the user selected directory path.
#'
readDirectoryInput = function(session, inputId) {
  session$input[[sprintf('%s__chosen_dir', inputId)]]
}

#' Read the value of a localFileInput
#'
#' @param session The \code{session} object passed to function given to \code{shinyServer}.
#' @param inputId The id of the input object
#'
#' @details
#' Reads the value of the text field associated with a \code{localFileInput}
#' object that stores the user selected file path.
#'
readLocalFileInput = function(session, inputId) {
  session$input[[sprintf('%s__chosen_file', inputId)]]
}

observeDirectoryInput <- function(input, session, inputId){
	observeEvent(ignoreNULL=TRUE, eventExpr={input[[inputId]]},
		handlerExpr={
			if (input[[inputId]] > 0) {	      
				# launch the directory selection dialog with initial path read from the widget
				defaultPath <- readDirectoryInput(session, inputId)
				if (defaultPath == "[NULL]") defaultPath <- NA
				path = choose.dir(default = defaultPath)
				# update the widget value
				updateDirectoryInput(session, inputId, value=path)
			}
		}
	)
}

observeLocalFileInput <- function(input, session, inputId){
	observeEvent(ignoreNULL=TRUE, eventExpr={input[[inputId]]},
		handlerExpr={
			if (input[[inputId]] > 0) {
				# launch the file selection dialog with initial path read from the widget 
				filt <- matrix(c("comma-separated", ".csv", "tab-separated", ".tsv", "Text", ".txt", "All files", "*"), 4, 2, byrow = TRUE)
				defaultPath <- readLocalFileInput(session, inputId)
				if (defaultPath == "[NULL]") defaultPath <- NA
				path = choose.files(default = defaultPath, filters=filt)
				# update the widget value
				updateLocalFileInput(session, inputId, value=path)
			}
		}
	)
}

################################################################################
# Little Helpers
################################################################################
plotColPal <- function(colpal){
	require(plotrix)
	par(mar=c(0,0,0,0))
	plot.new()
	gradient.rect(0,0,1,1,col=colpal,nslices=length(colpal),gradient="x",border=NA)
}

getRnbStatusFromLog <- function(logFile){
	if (!file.exists(logFile)) return(NULL)
	ll <- readLines(logFile)
	res <- data.frame(
		module=RNB.MODULES,
		status=NA,
		scheduled=NA
	)
	rownames(res) <- RNB.MODULES
	for (mm in RNB.MODULES){
		if(sum(grepl(paste0("COMPLETED ", RNB.MODULES.LOG.MSG[mm]), ll))>0){
			res[mm, "status"] <- "completed"
		} else if(sum(grepl(paste0("STARTED ", RNB.MODULES.LOG.MSG[mm]), ll))>0){
			res[mm, "status"] <- "started"
		}
	}
	#check the options if the anlysis is scheduled
	res["data_import", "scheduled"]              <- rnb.getOption("import")
	res["quality_control", "scheduled"]          <- rnb.getOption("qc")
	res["preprocessing", "scheduled"]            <- rnb.getOption("preprocessing")
	res["tracks_and_tables", "scheduled"]        <- rnb.getOption("export.to.csv") || rnb.getOption("export.to.bed") || length(rnb.getOption("export.to.trackhub")) > 0
	res["covariate_inference", "scheduled"]      <- rnb.getOption("inference")
	res["exploratory_analysis", "scheduled"]     <- rnb.getOption("exploratory")
	res["differential_methylation", "scheduled"] <- rnb.getOption("import")
	return(res)
}

checkReportDir <- function(repDir){
	res <- list(valid=FALSE)
	res$reportHtml <- rep(NA, length(RNB.MODULES))
	names(res$reportHtml) <- RNB.MODULES
	res$moduleStatus <- list(NULL)
	res$createdByDj <- FALSE # check if this report directory was created by RnBeadsDJ

	if (!dir.exists(repDir)) return(res)
	contents <- list.files(repDir, include.dirs=TRUE, all.files=TRUE)
	res$valid <- all(c("configuration", "analysis.log") %in% contents)
	if (!res$valid) return(res)
	res$createdByDj <- is.element(".RnBeadsDJ", contents) 
	res$valid <- res$createdByDj || all(c("index.html", "analysis_options.xml") %in% contents)
	if (!res$valid) return(res)
	htmlModules <- paste0(RNB.MODULES, ".html")
	htmlModules.exist <- htmlModules %in% contents
	res$reportHtml[htmlModules.exist] <- htmlModules[htmlModules.exist]
	res$logFile <- file.path(repDir, "analysis.log")
	res$moduleStatus <- getRnbStatusFromLog(res$logFile)
	return(res)
}
# mark a report directory as created by RnBeadsDJ
markDirDJ <- function(repDir){
	repStatus <- checkReportDir(repDir)
	if (!repStatus$createdByDj){
		file.create(file.path(repDir, ".RnBeadsDJ"))
	}
	invisible(NULL)
}
################################################################################
# UI configuration
################################################################################
ui <- tagList(useShinyjs(), navbarPage(
	windowTitle="RnBeadsDJ",
	tags$p(tags$a(href="https://rnbeads.org", tags$img(width=145, height=50, src="img/rnbeads_logo.png")), "DJ"),
	# tabPanel("Sandbox", icon=icon("dropbox"),
	# 	verbatimTextOutput("sandboxOut"),
	# 	localFileInput('sandboxIn2', label='select a local file'),
	# 	verbatimTextOutput("sandboxOut2")
	# ),
	tabPanel("About", icon=icon("book"),
		tags$h1("Welcome to the RnBeads Data Juggler"),
		tags$p("Here, you can configure and run your RnBeads analyses. You can ..."),
		tags$ul(
			tags$li("... run new analyses by specifying a non-existing data directory in the", "'Analysis'", "tab."),
			tags$li("... view the status of an RnBeads analysis by specifying an existing report directory in the", "'Analysis'", "tab."),
			tags$li("... configure, load and save option settings for your analyses in the", "'Analysis Options'", "tab."),
			tags$li("... run the complete RnBeads pipeline using the", "'Input'",  " and 'Run'", "tabs."),
			tags$li("... run individual RnBeads modules for new or existing RnBeads analysis runs via the", "'Modules'", "tab.")
		)
	),
	tabPanel(tags$div(title="Specify general parameters, such as report (output) directories, number of cores to use, etc. You can configure new analyses or inspect the status of previous RnBeads runs.", icon("bar-chart"), "Analysis"),
		sidebarPanel(
			tags$div(title="Specify parent directory where the analysis reports will be stored.", directoryInput('outDir', label = 'Select analysis directory', value = '~')),
			tags$div(title="Specify the report directory (subdirectory of the analysis directory). Specify a non-existing directory for a new RnBeads analysis or an existing one to inspect the status and/or continue a previously started analysis.", textInput("reportSubDir", "Choose the name of the report directory", "rnbeads_report")),
			tags$div(title="Select the number of cores to use for the analysis. Note that selecting more cores will speed up analysis, but might lead to significantly increased memory usage.", sliderInput('numCores', "Select the number of cores to use", min=1, max=detectCores(), value=1, step=1)),
			tags$div(title="Select a color scheme for the plots generated by RnBeads.", selectInput('ggplotTheme', "Select a theme for the plots", c("Black & White"="bw", "Grey"="grey")))
		),
		mainPanel(
			uiOutput("anaStatus")
		)
	),
	tabPanel(tags$div(title="Specify input parameters, such as where to find data and which format and assay platform to expect.", icon("sign-in"), "Input"),
		sidebarPanel(
			tags$div(title="Select a file which contains a valid annotation table with rows corresponding to each sample in the analysis.", localFileInput("sampleAnnotFile", "Select sample annotation file")),
			tags$div(title="Select the character that is used to separate the columns in the sample annotation table.", selectInput("rnbOptsI.import.table.separator", "Separator:", RNB.TABLE.SEPS)),
			tags$div(title="Select the experimental platform that is used for DNA methylation quantification.", selectInput("platform", "Platform", RNB.PLATFORMS)),
			tags$div(title="Specify the directory where the input methylation data (IDAT or methylation call files) can be found.", directoryInput('dataDir', label='Select input data directory'))
		),
		mainPanel(
			uiOutput("inputStatus")
		)
	),
	tabPanel(tags$div(title="Specify analysis parameters in order to finetune your methylation analysis. Parameter settings can be configured, saved and loaded.", icon("sliders"), "Analysis Options"),
		fluidRow(
			column(9,
				wellPanel(
					tags$h3("Load Option Profile"),
					fluidRow(
						column(3, tags$div(title="Load option profile store from a preexisting analysis. Existing analysis directory must be specified.", wellPanel(
							actionButton("loadOptsAnaDirDo", "Load from Analysis Directory")
						))),
						column(5, tags$div(title="Load option profile from XML file.", wellPanel(
							localFileInput("loadOptsXmlFile", "XML file"),
							actionButton("loadOptsXmlDo", "Load from XML")
						))),
						column(4, tags$div(title="Load option profile from preset. Several options for more or less comprehensive and resource-demanding analyses are available.", wellPanel(
							selectInput("loadOptsProfileSel", "Predefined Option Profile", RNB.OPTION.PROFILES),
							actionButton("loadOptsProfileDo", "Load Option Profile")
						)))
					)
				)
			),
			column(3,
				tags$div(title="Save the current option profile to an XML file. This file can be loaded later for running a different analysis with the same parameters.", wellPanel(
					tags$h3("Saving Option Profile"),
					downloadButton("saveOptsXml", "Save to XML")
				))
			)
		),
		tabsetPanel(
			tabPanel("General",
				tags$table(class="table table-hover",
					tags$thead(tags$tr(
						tags$th("Name"),
						tags$th("Setting"),
						tags$th("Value")
					)),
					tags$tbody(
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["analysis.name"], tags$code("analysis.name"))
							),
							tags$td(
								textInput("rnbOptsI.analysis.name", NULL, "RnBeads Analysis")
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.analysis.name")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["assembly"], tags$code("assembly"))
							),
							tags$td(
								selectInput("rnbOptsI.assembly", NULL, RNB.ASSEMBLIES, selected="hg38")
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.assembly", placeholder=TRUE)
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["region.types"], tags$code("region.types"))
							),
							tags$td(
								uiOutput("selRegionTypes")
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.region.types", placeholder=TRUE)
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["identifiers.column"], tags$code("identifiers.column"))
							),
							tags$td(
								uiOutput('selColumn.id')
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.identifiers.column", placeholder=TRUE)
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["min.group.size"], tags$code("min.group.size"))
							),
							tags$td(
								sliderInput("rnbOptsI.min.group.size", NULL, min=RNB.GROUP.SIZE.RANGE[1], max=RNB.GROUP.SIZE.RANGE[2], value=2)
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.min.group.size")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["max.group.count"], tags$code("max.group.count"))
							),
							tags$td(
								sliderInput("rnbOptsI.max.group.count", NULL, min=RNB.GROUP.COUNT.RANGE[1], max=RNB.GROUP.COUNT.RANGE[2], value=RNB.GROUP.COUNT.RANGE[2])
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.max.group.count")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["colors.category"], tags$code("colors.category"))
							),
							tags$td(
								selectInput("rnbOptsI.colors.category", NULL, names(RNB.COLSCHEMES.CATEGORY))
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.colors.category"),
								plotOutput("rnbOptsOP.colors.category", height="30px")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["colors.meth"], tags$code("colors.meth"))
							),
							tags$td(
								selectInput("rnbOptsI.colors.meth", NULL, names(RNB.COLSCHEMES.METH))
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.colors.meth"),
								plotOutput("rnbOptsOP.colors.meth", height="30px")
							)
						)
					)
				)
			),
			tabPanel("Import",
				tags$table(class="table table-hover",
					tags$thead(tags$tr(
						tags$th("Name"),
						tags$th("Setting"),
						tags$th("Value")
					)),
					tags$tbody(
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["import.default.data.type"], tags$code("import.default.data.type"))
							),
							tags$td(
								"Defined per 'Platform' in the 'Input' section"
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.import.default.data.type")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["import.table.separator"], tags$code("import.table.separator"))
							),
							tags$td(
								"Defined per 'Separator' in the 'Input' section"
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.import.table.separator")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["import.bed.style"], tags$code("import.bed.style"))
							),
							tags$td(
								selectInput("rnbOptsI.import.bed.style", NULL, RNB.BED.STYLES)
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.import.bed.style")
							)
						)
					)
				)
			),
			tabPanel("Quality Control",
				tags$table(class="table table-hover",
					tags$thead(tags$tr(
						tags$th("Name"),
						tags$th("Setting"),
						tags$th("Value")
					)),
					tags$tbody(
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["qc"], tags$code("qc"))
							),
							tags$td(
								checkboxInput("rnbOptsI.qc", "Enable", value=TRUE)
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.qc")
							)
						)
					)
				)
			),
			tabPanel("Preprocessing",
				tags$table(class="table table-hover",
					tags$thead(tags$tr(
						tags$th("Name"),
						tags$th("Setting"),
						tags$th("Value")
					)),
					tags$tbody(
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["preprocessing"], tags$code("preprocessing"))
							),
							tags$td(
								checkboxInput("rnbOptsI.preprocessing", "Enable", value=TRUE)
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.preprocessing")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["filtering.coverage.threshold"], tags$code("filtering.coverage.threshold"))
							),
							tags$td(
								sliderInput("rnbOptsI.filtering.coverage.threshold", NULL, min=1, max=100, value=5)
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.filtering.coverage.threshold")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["filtering.low.coverage.masking"], tags$code("filtering.low.coverage.masking"))
							),
							tags$td(
								checkboxInput("rnbOptsI.filtering.low.coverage.masking", "Enable", value=FALSE)
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.filtering.low.coverage.masking")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["filtering.high.coverage.outliers"], tags$code("filtering.high.coverage.outliers"))
							),
							tags$td(
								checkboxInput("rnbOptsI.filtering.high.coverage.outliers", "Enable", value=FALSE)
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.filtering.high.coverage.outliers")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["filtering.missing.value.quantile"], tags$code("filtering.missing.value.quantile"))
							),
							tags$td(
								sliderInput("rnbOptsI.filtering.missing.value.quantile", NULL, min=0, max=1, value=1)
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.filtering.missing.value.quantile")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["filtering.greedycut"], tags$code("filtering.greedycut"))
							),
							tags$td(
								checkboxInput("rnbOptsI.filtering.greedycut", "Enable", value=FALSE)
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.filtering.greedycut")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["filtering.sex.chromosomes.removal"], tags$code("filtering.sex.chromosomes.removal"))
							),
							tags$td(
								checkboxInput("rnbOptsI.filtering.sex.chromosomes.removal", "Enable", value=FALSE)
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.filtering.sex.chromosomes.removal")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["filtering.snp"], tags$code("filtering.snp"))
							),
							tags$td(
								selectInput("rnbOptsI.filtering.snp", NULL, RNB.FILTERING.SNP, selected="3")
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.filtering.snp")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["filtering.cross.reactive"], tags$code("filtering.cross.reactive"))
							),
							tags$td(
								checkboxInput("rnbOptsI.filtering.cross.reactive", "Enable", value=FALSE)
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.filtering.cross.reactive")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["filtering.whitelist"], tags$code("filtering.whitelist"))
							),
							tags$td(
									localFileInput("rnbOptsI.filtering.whitelist", NULL),
									actionButton("rnbOptsResetWhitelist", "Reset whitelist")
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.filtering.whitelist")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["filtering.blacklist"], tags$code("filtering.blacklist"))
							),
							tags$td(
									localFileInput("rnbOptsI.filtering.blacklist", NULL),
									actionButton("rnbOptsResetblacklist", "Reset blacklist")
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.filtering.blacklist")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["normalization.method"], tags$code("normalization.method"))
							),
							tags$td(
								selectInput("rnbOptsI.normalization.method", NULL, RNB.NORMALIZATION.METHODS, selected="scaling.internal")
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.normalization.method")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["normalization.background.method"], tags$code("normalization.background.method"))
							),
							tags$td(
								selectInput("rnbOptsI.normalization.background.method", NULL, RNB.NORMALIZATION.BG.METHODS, selected="sesame.noobsb")
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.normalization.background.method")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["imputation.method"], tags$code("imputation.method"))
							),
							tags$td(
								selectInput("rnbOptsI.imputation.method", NULL, RNB.IMPUTATION.METHODS)
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.imputation.method")
							)
						)
					)
				)
			),
			tabPanel("Export",
				tags$table(class="table table-hover",
					tags$thead(tags$tr(
						tags$th("Name"),
						tags$th("Setting"),
						tags$th("Value")
					)),
					tags$tbody(
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["export.to.bed"], tags$code("export.to.bed"))
							),
							tags$td(
								checkboxInput("rnbOptsI.export.to.bed", "Enable", value=TRUE)
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.export.to.bed")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["export.to.csv"], tags$code("export.to.csv"))
							),
							tags$td(
								checkboxInput("rnbOptsI.export.to.csv", "Enable", value=FALSE)
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.export.to.csv")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["export.to.trackhub"], tags$code("export.to.trackhub"))
							),
							tags$td(
								selectInput("rnbOptsI.export.to.trackhub", NULL, RNB.TRACKHUB.FORMATS, multiple=TRUE, selected=RNB.TRACKHUB.FORMATS)
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.export.to.trackhub", placeholder=TRUE)
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["export.types"], tags$code("export.types"))
							),
							tags$td(
								uiOutput('selRegions.export')
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.export.types", placeholder=TRUE)
							)
						)
					)
				)
			),
			tabPanel("Inference",
				tags$table(class="table table-hover",
					tags$thead(tags$tr(
						tags$th("Name"),
						tags$th("Setting"),
						tags$th("Value")
					)),
					tags$tbody(
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["inference"], tags$code("inference"))
							),
							tags$td(
								checkboxInput("rnbOptsI.inference", "Enable", value=FALSE)
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.inference")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["inference.age.prediction"], tags$code("inference.age.prediction"))
							),
							tags$td(
								checkboxInput("rnbOptsI.inference.age.prediction", "Enable", value=TRUE)
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.inference.age.prediction")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["inference.age.column"], tags$code("inference.age.column"))
							),
							tags$td(
								uiOutput('selColumn.agepred')
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.inference.age.column", placeholder=TRUE)
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["inference.targets.sva"], tags$code("inference.targets.sva"))
							),
							tags$td(
								uiOutput('selColumn.sva')
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.inference.targets.sva", placeholder=TRUE)
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["inference.sva.num.method"], tags$code("inference.sva.num.method"))
							),
							tags$td(
								selectInput("rnbOptsI.inference.sva.num.method", NULL, RNB.SVA.NUM.METHODS)
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.inference.sva.num.method", placeholder=TRUE)
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["inference.reference.methylome.column"], tags$code("inference.reference.methylome.column"))
							),
							tags$td(
								uiOutput('selColumn.cellTypeRef')
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.inference.reference.methylome.column", placeholder=TRUE)
							)
						)
					)
				)
			),
			tabPanel("Exploratory",
				tags$table(class="table table-hover",
					tags$thead(tags$tr(
						tags$th("Name"),
						tags$th("Setting"),
						tags$th("Value")
					)),
					tags$tbody(
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["exploratory"], tags$code("exploratory"))
							),
							tags$td(
								checkboxInput("rnbOptsI.exploratory", "Enable", value=TRUE)
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.exploratory")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["exploratory.columns"], tags$code("exploratory.columns"))
							),
							tags$td(
								uiOutput('selColumn.ex')
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.exploratory.columns", placeholder=TRUE)
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["exploratory.intersample"], tags$code("exploratory.intersample"))
							),
							tags$td(
								checkboxInput("rnbOptsI.exploratory.intersample", "Enable", value=TRUE)
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.exploratory.intersample")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["exploratory.beta.distribution"], tags$code("exploratory.beta.distribution"))
							),
							tags$td(
								checkboxInput("rnbOptsI.exploratory.beta.distribution", "Enable", value=TRUE)
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.exploratory.beta.distribution")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["exploratory.correlation.qc"], tags$code("exploratory.correlation.qc"))
							),
							tags$td(
								checkboxInput("rnbOptsI.exploratory.correlation.qc", "Enable", value=TRUE)
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.exploratory.correlation.qc")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["exploratory.region.profiles"], tags$code("exploratory.region.profiles"))
							),
							tags$td(
								uiOutput('selRegionProfiles.ex')
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.exploratory.region.profiles", placeholder=TRUE)
							)
						)
					)
				)
			),
			tabPanel("Differential",
				tags$table(class="table table-hover",
					tags$thead(tags$tr(
						tags$th("Name"),
						tags$th("Setting"),
						tags$th("Value")
					)),
					tags$tbody(
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["differential"], tags$code("differential"))
							),
							tags$td(
								checkboxInput("rnbOptsI.differential", "Enable", value=TRUE)
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.differential")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["differential.comparison.columns"], tags$code("differential.comparison.columns"))
							),
							tags$td(
								uiOutput('selColumn.diff')
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.differential.comparison.columns", placeholder=TRUE)
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["covariate.adjustment.columns"], tags$code("covariate.adjustment.columns"))
							),
							tags$td(
								uiOutput('selAdjColumns')
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.covariate.adjustment.columns", placeholder=TRUE)
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["differential.site.test.method"], tags$code("differential.site.test.method"))
							),
							tags$td(
								selectInput("rnbOptsI.differential.site.test.method", NULL, RNB.DIFFMETH.TEST.METHODS)
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.differential.site.test.method", placeholder=TRUE)
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["differential.report.sites"], tags$code("differential.report.sites"))
							),
							tags$td(
								checkboxInput("rnbOptsI.differential.report.sites", "Enable", value=TRUE)
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.differential.report.sites")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["differential.variability"], tags$code("differential.variability"))
							),
							tags$td(
								checkboxInput("rnbOptsI.differential.variability", "Enable", value=FALSE)
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.differential.variability")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["differential.variability.method"], tags$code("differential.variability.method"))
							),
							tags$td(
								selectInput("rnbOptsI.differential.variability.method", NULL, RNB.DIFFVAR.METHODS)
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.differential.variability.method")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["differential.enrichment.go"], tags$code("differential.enrichment.go"))
							),
							tags$td(
								checkboxInput("rnbOptsI.differential.enrichment.go", "Enable", value=FALSE)
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.differential.enrichment.go")
							)
						),
						tags$tr(
							tags$td(
								tags$div(title=RNB.OPTION.DESC["differential.enrichment.lola"], tags$code("differential.enrichment.lola"))
							),
							tags$td(
								checkboxInput("rnbOptsI.differential.enrichment.lola", "Enable", value=FALSE)
							),
							tags$td(
								verbatimTextOutput("rnbOptsO.differential.enrichment.lola")
							)
						)
					)
				)
			),
			tabPanel("R Object",
				verbatimTextOutput("rnbOpts")
			)
		)
	),
	tabPanel(tags$div(title="Run a full RnBeads analysis from scratch once you specified your analysis parameters.", icon("play"), "Run"),
		uiOutput("runHint"),
		actionButton("runRnb", "Run Analysis", class="btn-primary"),
		textOutput("runStatusMsg")
	),
	tabPanel(tags$div(title="Run individual steps of an RnBeads analysis using preexisting datasets as input.", icon("cubes"), "Modules"),
		tabsetPanel(
			tabPanel("Data Import",
				sidebarPanel(
					tags$h1("Import from ..."),
					wellPanel(
						tags$h4("... new Dataset"),
						tags$p("Import using the inputs and options specified in other tabs."),
						uiOutput("modImportNew.about"),
						actionButton("modImportNew", "Import New Data", class="btn-primary")
					),
					wellPanel(
						tags$h4("... existing Analysis Directory"),
						uiOutput("modImportAnaDir.about"),
						actionButton("modImportAnaDir", "Import from Analysis Directory", class="btn-primary")
					),
					wellPanel(
						tags$h4("... RnBeads Objects"),
						tags$div(title="Specify the directory of an existing RnBSet object. Be sure to select a directory (not an RData file).", directoryInput('modImportRnBSetDir', label = 'Select directory containing an RnBSet object', value = '~')),
						# localFileInput('modImportOptionsFile', label = 'Select Options XML file (optional)'),
						actionButton("modImportRObjects", "Import", class="btn-primary")
					),
					wellPanel(
						tags$h4("Reset"),
						actionButton("modImportReset", "Unload dataset")
					)
				),
				mainPanel(
					tags$h1("Loading Status"),
					uiOutput("modImport.status"),
					uiOutput("rnbSetInfo")
				)
			),
			tabPanel("Quality Control",
				sidebarPanel(
					checkboxInput("modQC.overwrite", "Overwrite existing report", value=FALSE),
					actionButton("modQC.run", "Run Quality Control", class="btn-primary")
				),
				mainPanel(
					uiOutput("modQC.status")
				)
			),
			tabPanel("Preprocessing",
				sidebarPanel(
					checkboxInput("modPreprocessing.overwrite", "Overwrite existing report", value=FALSE),
					checkboxInput("modPreprocessing.save", "Save RnBSet object", value=TRUE),
					actionButton("modPreprocessing.run", "Run Preprocessing", class="btn-primary")
				),
				mainPanel(
					uiOutput("modPreprocessing.status")
				)
			),
			tabPanel("Tracks and Tables",
				sidebarPanel(
					checkboxInput("modTNT.overwrite", "Overwrite existing report", value=FALSE),
					actionButton("modTNT.run", "Run Tracks and Tables", class="btn-primary")
				),
				mainPanel(
					uiOutput("modTNT.status")
				)
			),
			tabPanel("Covariate Inference",
				sidebarPanel(
					checkboxInput("modInference.overwrite", "Overwrite existing report", value=FALSE),
					checkboxInput("modInference.save", "Save RnBSet object", value=TRUE),
					actionButton("modInference.run", "Run Covariate Inference", class="btn-primary")
				),
				mainPanel(
					uiOutput("modInference.status")
				)
			),
			tabPanel("Exploratory Analysis",
				sidebarPanel(
					checkboxInput("modExploratory.overwrite", "Overwrite existing report", value=FALSE),
					actionButton("modExploratory.run", "Run Exploratory Analysis", class="btn-primary")
				),
				mainPanel(
					uiOutput("modExploratory.status")
				)
			),
			tabPanel("Differential Methylation",
				sidebarPanel(
					checkboxInput("modDifferential.overwrite", "Overwrite existing report", value=FALSE),
					actionButton("modDifferential.run", "Run Differential Methylation", class="btn-primary")
				),
				mainPanel(
					uiOutput("modDifferential.status")
				)
			)
		)
	)
))
################################################################################
# Server configuration
################################################################################
server <- function(input, output, session) {
	############################################################################
	# SANDBOX
	# file and directory handling
	# shinyDirChoose(input, "outDirSel", roots=c(disk="/", home="~", wd="."), session=session)

	# dynamic outputs
	observeLocalFileInput(input, session, 'sandboxIn2')
	output$sandboxOut2 <- renderPrint({readLocalFileInput(session, 'sandboxIn2')})
	############################################################################

	# output$outDirSel <- renderPrint({input$outDirSel})
	# output$outDirSel <- renderPrint({readDirectoryInput(session, 'outDirSel')})


	############################################################################
	# Analysis status
	############################################################################
	refreshTimer <- reactiveTimer(REFRESH.TIME)
	observeDirectoryInput(input, session, 'outDir')
	reportDir <- reactive({
		file.path(readDirectoryInput(session, 'outDir'), input$reportSubDir)
	})
	setNumCoresFromSlider <- reactive({
		res <- input$numCores
		if (res > 1){
			parallel.setup(res)
		} else {
			parallel.teardown()
		}
		logger.close()
		res
	})
	anaStatus <- reactive({
		refreshTimer()
		res <- list(status="invalid", statusTab=NULL, rnbSet.paths=c(), logFile=NA)
		if (dir.exists(reportDir())){
			reportStatus <- checkReportDir(reportDir())
			if (reportStatus$valid){
				#Valid existing report directory
				statusTab <- data.frame(
					Module=names(RNB.MODULES),
					Report=NA,
					Status=NA
				)
				rownames(statusTab) <- RNB.MODULES
				for (mm in RNB.MODULES){
					if (!is.na(reportStatus$reportHtml[mm])){
						statusTab[mm, "Report"] <- as.character(tags$a(href=paste0("file://", file.path(reportDir(), reportStatus$reportHtml[mm])), tags$code("html")))
						if (reportStatus$moduleStatus[mm, "status"]=="completed"){
							statusTab[mm, "Status"] <- as.character(tags$span(style="color:green", icon("check")))
						} else if (reportStatus$moduleStatus[mm, "status"]=="started"){
							statusTab[mm, "Status"] <- as.character(tags$span(style="color:orange", icon("play")))
						}
						
					} else {
						statusTab[mm, "Status"] <- as.character(tags$span(style="color:red", icon("times")))
					}
				}
				res$rnbSet.paths <- file.path(reportDir(), grep("rnbSet_", list.dirs(reportDir(), full.names=FALSE, recursive=FALSE), value=TRUE))
				if (!modImportStatus$dataset.loaded) shinyjs::enable("modImportAnaDir")
				res$logFile <- reportStatus$logFile
				res$status <- "reportDir"
				res$statusTab <- statusTab
				shinyjs::enable("loadOptsAnaDirDo")
			} else {
				# Existing directory, but no valid RnBeads report
				shinyjs::disable("modImportAnaDir")
				shinyjs::disable("loadOptsAnaDirDo")
			}	
		} else {
			# New RnBeads analysis
			shinyjs::disable("modImportAnaDir")
			shinyjs::disable("loadOptsAnaDirDo")
			res$status <- "new"
		}
		res
	})
	output$anaStatus <- renderUI({
		curStatus <- anaStatus()
		if (curStatus$status=="new"){
			tagList(
				tags$h1("Configure new analysis"),
				tags$p(
					"Configure the input and analysis options by clicking on the tabs above",
					"and selecting corresponding values. Finally go to 'Run' to start your analysis."
				),
				tags$p(
					"A new report will be created in", tags$code(reportDir())
				)
			)
		} else if (curStatus$status=="reportDir"){
			tagList(
				tags$h1("Existing RnBeads analysis"),
				renderTable(curStatus$statusTab, sanitize.text.function=function(x){x}, striped=TRUE, hover=TRUE, bordered=TRUE),
				tags$div(title="If the link does not open use right-click, copy the link and paste the address in a new browser window", tags$p(tags$span(icon("file-text-o"), tags$a(href=paste0("file://", curStatus$logFile), "Log File"))))
			)
		} else if (curStatus$status=="invalid"){
			tagList(
				tags$h1("Invalid analysis directory"),
				tags$p("Please select a non-existing analysis directory for new analyses or an existing RnBeads report directory.")
			)
		}
	})

	############################################################################
	# Input
	############################################################################
	observeLocalFileInput(input, session, 'sampleAnnotFile')
	observeDirectoryInput(input, session, 'dataDir')

	tabSep <- reactive({input$rnbOptsI.import.table.separator})
	sampleAnnotFile <- reactive({
		readLocalFileInput(session, 'sampleAnnotFile')
	})
	inputDataDir <- reactive({
		readDirectoryInput(session, 'dataDir')
	})

	sannot.fromFile <- reactive({
		sampleAnnotFn <- sampleAnnotFile()
		if (is.null(sampleAnnotFn) || sampleAnnotFn=="") return(NULL)
		tryCatch(
			{
				RnBeads::read.sample.annotation(sampleAnnotFn, sep=tabSep())
			},
			error = function(err) {
				data.frame(Error="Unable to load table", Why=err$message)
				NULL
			}
		)
	})
	sannot <- reactive({
		# first look if an RnBSet object has been loaded. If so, take the sample annotation from there
		# otherwise require the user to upload an annotation table via file input
		if (!is.null(rnbData$rnbSet)){
			pheno(rnbData$rnbSet)
		} else {
			sannot.fromFile()
		}
	})
	sannot.nsamples <- reactive({
		if (is.null(sannot())){
			0
		} else {
			nrow(sannot())
		}
	})
	sannot.cols <- reactive({
		cnames <- colnames(sannot())
		if (length(cnames)==2 && cnames==c("Error", "Why")) cnames <- NULL #error --> no column names
		cnames
	})
	sannot.cols.plusNone <- reactive(c("[None]", sannot.cols()))
	sannot.cols.plusDefault <- reactive(c("[default]", sannot.cols()))
	sannot.cols.grps <- reactive({
		# print("DEBUG: updated sannot.cols.grps")
		depDummy <- input$rnbOptsI.min.group.size
		depDummy <- input$rnbOptsI.max.group.count
		if(length(sannot.cols())>0) {
			names(rnb.sample.groups(sannot()))
		} else {
			NULL
		}
	})
	sannot.cols.plusAutomatic <- reactive(c("[automatic]", sannot.cols()))
	output$sampleAnnotContent <- renderTable({sannot.fromFile()}, striped=TRUE, hover=TRUE, bordered=TRUE)

	isBiseq <- reactive({
		selPlatform <- input$platform
		if (!is.null(rnbData$rnbSet) && (inherits(rnbData$rnbSet, "RnBeadSet") || inherits(rnbData$rnbSet, "RnBiseqSet"))){
			res <- inherits(rnbData$rnbSet, "RnBiseqSet")
		} else {
			res <- selPlatform == "biseq"
		}
		if (res) {
			shinyjs::enable("rnbOptsI.assembly")
			shinyjs::enable("rnbOptsI.import.bed.style")
			shinyjs::enable("rnbOptsI.filtering.coverage.threshold")
			shinyjs::enable("rnbOptsI.filtering.low.coverage.masking")
			shinyjs::enable("rnbOptsI.filtering.high.coverage.outliers")
			updateCheckboxInput(session, "rnbOptsI.filtering.greedycut", value=FALSE)
			shinyjs::disable("rnbOptsI.filtering.greedycut")
			# shinyjs::disable("rnbOptsI.filtering.whitelist")
			# shinyjs::disable("rnbOptsI.filtering.blacklist")
			shinyjs::disable("rnbOptsI.filtering.cross.reactive")
			shinyjs::disable("rnbOptsI.normalization.method")
			shinyjs::disable("rnbOptsI.normalization.background.method")
			shinyjs::disable("rnbOptsI.exploratory.correlation.qc")
		} else {
			#shinyjs::disable("rnbOptsI.assembly")
			shinyjs::disable("rnbOptsI.import.bed.style")
			shinyjs::disable("rnbOptsI.filtering.coverage.threshold")
			shinyjs::disable("rnbOptsI.filtering.low.coverage.masking")
			shinyjs::disable("rnbOptsI.filtering.high.coverage.outliers")
			updateCheckboxInput(session, "rnbOptsI.filtering.greedycut", value=TRUE)
			shinyjs::enable("rnbOptsI.filtering.greedycut")
			shinyjs::enable("rnbOptsI.filtering.cross.reactive")
			# shinyjs::enable("rnbOptsI.filtering.whitelist")
			# shinyjs::enable("rnbOptsI.filtering.blacklist")
			shinyjs::enable("rnbOptsI.normalization.method")
			shinyjs::enable("rnbOptsI.normalization.background.method")
			shinyjs::enable("rnbOptsI.exploratory.correlation.qc")
		}
		res
	})

	output$selColumn.id <- renderUI({
		selectInput('rnbOptsI.identifiers.column', NULL, sannot.cols.plusNone(), selected=optionSettingObserver$selColumn.id)
	})
	output$selColumn.agepred <- renderUI({
		selectInput('rnbOptsI.inference.age.column', NULL, sannot.cols.plusDefault(), selected=optionSettingObserver$selColumn.agepred)
	})
	output$selColumn.sva<- renderUI({
		selectInput('rnbOptsI.inference.targets.sva', NULL, sannot.cols.grps(), multiple=TRUE, selected=optionSettingObserver$selColumn.sva)
	})
	output$selColumn.cellTypeRef <- renderUI({
		selectInput('rnbOptsI.inference.reference.methylome.column', NULL, sannot.cols.plusNone(), selected=optionSettingObserver$selColumn.cellTypeRef)
	})
	output$selColumn.ex <- renderUI({
		selectInput('rnbOptsI.exploratory.columns', NULL, sannot.cols.plusAutomatic(), multiple=TRUE, selected=optionSettingObserver$selColumn.ex)
	})
	output$selColumn.diff <- renderUI({
		selectInput('rnbOptsI.differential.comparison.columns', NULL, sannot.cols.plusAutomatic(), multiple=TRUE, selected=optionSettingObserver$selColumn.diff)
	})
	output$selAdjColumns <- renderUI({
		selectInput('rnbOptsI.covariate.adjustment.columns', NULL, sannot.cols(), multiple=TRUE, selected=optionSettingObserver$selColumn.adj)
	})

	output$inputStatus <- renderUI({
		res <- list()
		showAnnotTab <- TRUE
		sampleAnnotFn <- sampleAnnotFile()
		if (is.null(sampleAnnotFn) || sampleAnnotFn=="") showAnnotTab <- FALSE
		if (showAnnotTab){
			res <- c(res,
				list(tags$h1("Preview of the sample annotation table")),
				list(tableOutput("sampleAnnotContent"))
			)
		} else {
			res <- c(res,
				list(tags$p("No sample annotation loaded"))
			)
		}
		tagList(res)
	})

	############################################################################
	# Analysis Options
	############################################################################
	# autoOpts <- c("analysis.name", "import.table.separator")
	# for (oo in autoOpts){
	# 	output[[paste0("rnbOptsO.", oo)]] <- renderText({
	# 		args <- list()
	# 		args[[oo]] <- input[[paste0("rnbOptsI.", oo)]]
	# 		do.call("rnb.options", args)
	# 		rnb.getOption(oo)
	# 	})
	# }

	# a helper container for storing variable, option-related information
	optionSettingObserver <- reactiveValues(
		group.count.range.max=RNB.GROUP.COUNT.RANGE[2],
		colors.category.list=RNB.COLSCHEMES.CATEGORY,
		selRegs=NULL,
		selRegs.export=NULL,
		selRegs.exploratory.profiles=NULL,
		selRegs.differential=NULL,
		selColumn.id="[None]",
		selColumn.agepred="[default]",
		selColumn.sva=character(0),
		selColumn.cellTypeRef="[None]",
		selColumn.ex="[automatic]",
		selColumn.diff="[automatic]",
		selColumn.adj=character(0)
	)

	optList <- reactive({
		depDummy <- assemblySel()
		depDummy <- isBiseq() #dummy to create dependency upon update of platform selection
		depDummy <- optsImportDataType()
		# auto update on any option change to rnbOptsI
		for (oo in grep("^rnbOptsI.", names(input), value=TRUE)){
			input[[oo]]
		}
		rnb.options()
	})
	output$rnbOpts <- renderPrint({
		optList()
	})
	output$rnbOptsO.analysis.name <- renderText({
		rnb.options(analysis.name=input$rnbOptsI.analysis.name)
		rnb.getOption("analysis.name")
	})
	#default setting for assembly if platform is not bisulfite and thus no assembly input has been given
	assemblySel <- reactive({
		interfaceSetting <- input$rnbOptsI.assembly
		res <- "hg19"
		if (!is.null(interfaceSetting)){
			res <- interfaceSetting
		}
		rnb.options(assembly=res)
		res
	})
	output$rnbOptsO.assembly <- renderText({
		depDummy <- assemblySel()
		rnb.getOption("assembly")
	})
	regTypes.all <- reactive({
		depDummy <- assemblySel() # dummy to update on dependency: assembly
		rnb.region.types(rnb.getOption("assembly"))
	})
	output$selRegionTypes <- renderUI({
		rrs <- optionSettingObserver$selRegs
		if (is.null(rrs)){
			rrs <- regTypes.all()
			optionSettingObserver$selRegs <- rrs
		}
		selectInput('rnbOptsI.region.types', NULL, regTypes.all(), multiple=TRUE, selected=rrs)
	})
	regTypes <- reactive({
		input$rnbOptsI.region.types
	})
	regTypes.plus.sites <- reactive({c("sites", regTypes())})
	output$rnbOptsO.region.types <- renderText({
		rnb.options(region.types=regTypes())
		rnb.getOption("region.types")
	})
	output$selRegionProfiles.ex <- renderUI({
		selRegs <- optionSettingObserver$selRegs
		rrs <- optionSettingObserver$selRegs.exploratory.profiles
		if (is.null(rrs)) {
			rrs <- intersect(c("genes", "promoters", "cpgislands"), selRegs)
			optionSettingObserver$selRegs.exploratory.profiles <- rrs
		}
		selectInput('rnbOptsI.exploratory.region.profiles', NULL, selRegs, selected=rrs, multiple=TRUE)
	})
	output$selRegions.export <- renderUI({
		selRegs <- regTypes.plus.sites()
		rrs <- optionSettingObserver$selRegs.export
		if (is.null(rrs)) {
			rrs <- "sites"
			optionSettingObserver$selRegs.export <- rrs
		}
		selectInput('rnbOptsI.export.types', NULL, selRegs, selected=optionSettingObserver$selRegs.export, multiple=TRUE)
	})
	output$rnbOptsO.identifiers.column <- renderText({
		cname <- input$rnbOptsI.identifiers.column
		if (is.null(cname) || cname=="[None]") cname <- NULL
		rnb.options(identifiers.column=cname)
		rnb.getOption("identifiers.column")
	})
	output$rnbOptsO.min.group.size <- renderText({
		rnb.options(min.group.size=input$rnbOptsI.min.group.size)
		rnb.getOption("min.group.size")
	})
	observe({
		val.min.group.size <- input$rnbOptsI.min.group.size
		val.max.group.count <- input$rnbOptsI.max.group.count
		if (sannot.nsamples() > 0) optionSettingObserver$group.count.range.max <- sannot.nsamples()
		updateSliderInput(session, "rnbOptsI.min.group.size", value=val.min.group.size, min=RNB.GROUP.SIZE.RANGE[1], max=RNB.GROUP.SIZE.RANGE[2], step=1)
		updateSliderInput(session, "rnbOptsI.max.group.count", value=val.max.group.count, min=RNB.GROUP.COUNT.RANGE[1], max=optionSettingObserver$group.count.range.max, step=1)
	})
	output$rnbOptsO.max.group.count <- renderText({
		rnb.options(max.group.count=input$rnbOptsI.max.group.count)
		rnb.getOption("max.group.count")
	})
	output$rnbOptsO.colors.category <- renderText({
		cols <- optionSettingObserver$colors.category.list[[input$rnbOptsI.colors.category]]
		rnb.options(colors.category=cols)
		rnb.getOption("colors.category")
	})
	output$rnbOptsOP.colors.category <- renderPlot({
		cols <- optionSettingObserver$colors.category.list[[input$rnbOptsI.colors.category]]
		plotColPal(cols)
	})
	output$rnbOptsO.colors.meth <- renderText({
		cols <- RNB.COLSCHEMES.METH[[input$rnbOptsI.colors.meth]]
		rnb.options(colors.meth=cols)
		rnb.getOption("colors.meth")
	})
	output$rnbOptsOP.colors.meth <- renderPlot({
		cols <- RNB.COLSCHEMES.METH[[input$rnbOptsI.colors.meth]]
		# rnb.options(colors.meth=cols)
		# rnb.getOption("colors.meth")
		plotColPal(cols)
	})
	optsImportDataType <- reactive({
		res <- ""
		if (isBiseq()){
			res <- "bed.dir"
		} else {
			res <- "idat.dir"
		}
		rnb.options(import.default.data.type=res)
		res
	})
	output$rnbOptsO.import.default.data.type <- renderText({
		rnb.getOption("import.default.data.type")
	})
	output$rnbOptsO.import.table.separator <- renderText({
		rnb.options(import.table.separator=tabSep())
		rnb.getOption("import.table.separator")
	})
	output$rnbOptsO.import.bed.style <- renderText({
		interfaceSetting <- input$rnbOptsI.import.bed.style
		res <- rnb.getOption("import.bed.style")
		if (isBiseq()){
			res <- interfaceSetting
			rnb.options(import.bed.style=res)
		}
		res
	})
	doQc <- reactive({
		res <- input$rnbOptsI.qc
		oNames <- grep("^rnbOptsI.qc.", names(input), value=TRUE)
		if (res){
			for (oo in oNames){
				shinyjs::enable(oo)
			}
		} else {
			for (oo in oNames){
				shinyjs::disable(oo)
			}
		}
		res
	})
	output$rnbOptsO.qc <- renderText({
		rnb.options(qc=doQc())
		rnb.getOption("qc")
	})
	doPreprocessing <- reactive({
		res <- input$rnbOptsI.preprocessing
		oNames <- grep("^rnbOptsI.preprocessing.", names(input), value=TRUE)
		oNames <- union(oNames, grep("^rnbOptsI.normalization.", names(input), value=TRUE))
		oNames <- union(oNames, grep("^rnbOptsI.filtering.", names(input), value=TRUE))
		if (res){
			for (oo in oNames){
				shinyjs::enable(oo)
			}
		} else {
			for (oo in oNames){
				shinyjs::disable(oo)
			}
		}
		res
	})
	output$rnbOptsO.preprocessing <- renderText({
		rnb.options(preprocessing=doPreprocessing())
		rnb.getOption("preprocessing")
	})
	output$rnbOptsO.filtering.coverage.threshold <- renderText({
		interfaceSetting <- input$rnbOptsI.filtering.coverage.threshold
		res <- rnb.getOption("filtering.coverage.threshold")
		if (isBiseq()){
			res <- interfaceSetting
			rnb.options(filtering.coverage.threshold=res)
		}
		res
	})
	output$rnbOptsO.filtering.low.coverage.masking <- renderText({
		interfaceSetting <- input$rnbOptsI.filtering.low.coverage.masking
		res <- rnb.getOption("filtering.low.coverage.masking")
		if (isBiseq()){
			res <- interfaceSetting
			rnb.options(filtering.low.coverage.masking=res)
		}
		res
	})
	output$rnbOptsO.filtering.high.coverage.outliers <- renderText({
		interfaceSetting <- input$rnbOptsI.filtering.high.coverage.outliers
		res <- rnb.getOption("filtering.high.coverage.outliers")
		if (isBiseq()){
			res <- interfaceSetting
			rnb.options(filtering.high.coverage.outliers=res)
		}
		res
	})
	output$rnbOptsO.filtering.missing.value.quantile <- renderText({
		rnb.options(filtering.missing.value.quantile=input$rnbOptsI.filtering.missing.value.quantile)
		rnb.getOption("filtering.missing.value.quantile")
	})
	output$rnbOptsO.filtering.greedycut <- renderText({
		rnb.options(filtering.greedycut=input$rnbOptsI.filtering.greedycut)
		rnb.getOption("filtering.greedycut")
	})
	output$rnbOptsO.filtering.sex.chromosomes.removal <- renderText({
		rnb.options(filtering.sex.chromosomes.removal=input$rnbOptsI.filtering.sex.chromosomes.removal)
		rnb.getOption("filtering.sex.chromosomes.removal")
	})
	output$rnbOptsO.filtering.snp <- renderText({
		rnb.options(filtering.snp=input$rnbOptsI.filtering.snp)
		rnb.getOption("filtering.snp")
	})
	output$rnbOptsO.filtering.cross.reactive <- renderText({
		rnb.options(filtering.cross.reactive=input$rnbOptsI.filtering.cross.reactive)
		rnb.getOption("filtering.cross.reactive")
	})
	output$rnbOptsO.filtering.whitelist <- renderText({
		wl <- rnbOpts.filtering.whitelist.fn()
		if (is.null(wl) || nchar(wl) < 1 || wl=="[NULL]") wl <- NULL
		rnb.options(filtering.whitelist=wl)
		res <- rnb.getOption("filtering.whitelist")
		if (is.null(res)) res <- "NULL"
		res
	})
	output$rnbOptsO.filtering.blacklist <- renderText({
		bl <- rnbOpts.filtering.blacklist.fn()
		if (is.null(bl) || nchar(bl) < 1 || bl=="[NULL]") bl <- NULL
		rnb.options(filtering.blacklist=bl)
		res <- rnb.getOption("filtering.blacklist")
		if (is.null(res)) res <- "NULL"
		res
	})
	output$rnbOptsO.normalization.method <- renderText({
		interfaceSetting <- input$rnbOptsI.normalization.method
		res <- rnb.getOption("normalization.method")
		if (!isBiseq()){
			res <- interfaceSetting
			rnb.options(normalization.method=res)
		}
		res
	})
	output$rnbOptsO.normalization.background.method <- renderText({
		interfaceSetting <- input$rnbOptsI.normalization.background.method
		res <- rnb.getOption("normalization.background.method")
		if (!isBiseq()){
			res <- interfaceSetting
			rnb.options(normalization.background.method=res)
		}
		res
	})
	output$rnbOptsO.imputation.method <- renderText({
		rnb.options(imputation.method=input$rnbOptsI.imputation.method)
		rnb.getOption("imputation.method")
	})
	output$rnbOptsO.export.to.bed<- renderText({
		rnb.options(export.to.bed=input$rnbOptsI.export.to.bed)
		rnb.getOption("export.to.bed")
	})
	output$rnbOptsO.export.to.csv <- renderText({
		rnb.options(export.to.csv=input$rnbOptsI.export.to.csv)
		rnb.getOption("export.to.csv")
	})
	output$rnbOptsO.export.to.trackhub <- renderText({
		rnb.options(export.to.trackhub=input$rnbOptsI.export.to.trackhub)
		rnb.getOption("export.to.trackhub")
	})
	output$rnbOptsO.export.types <- renderText({
		rts <- input$rnbOptsI.export.types
		if (length(rts)<1) rts <- NULL
		rnb.options(export.types=rts)
		rnb.getOption("export.types")
	})
	doInference <- reactive({
		res <- input$rnbOptsI.inference
		res.agepred <- input$rnbOptsI.inference.age.prediction
		inferenceOptNames <- grep("^rnbOptsI\\.inference\\.", names(input), value=TRUE)
		inferenceOptNames.agepred <- setdiff(grep("^rnbOptsI\\.inference\\.age\\.", names(input), value=TRUE), "rnbOptsI.inference.age.prediction")
		if (res){
			for (oo in inferenceOptNames){
				if (is.element(oo, inferenceOptNames.agepred)){
					if (res.agepred){
						shinyjs::enable(oo)
					} else {
						shinyjs::disable(oo)
					}
				} else {
					shinyjs::enable(oo)
				}
			}
		} else {
			for (oo in inferenceOptNames){
				shinyjs::disable(oo)
			}
		}
		res
	})
	output$rnbOptsO.inference <- renderText({
		rnb.options(inference=doInference())
		rnb.getOption("inference")
	})
	output$rnbOptsO.inference.age.prediction <- renderText({
		rnb.options(inference.age.prediction=input$rnbOptsI.inference.age.prediction)
		rnb.getOption("inference.age.prediction")
	})
	output$rnbOptsO.inference.age.column <- renderText({
		cname <- input$rnbOptsI.inference.age.column
		if (is.null(cname) || cname=="[default]") cname <- "age"
		rnb.options(inference.age.column=cname)
		rnb.getOption("inference.age.column")
	})
	output$rnbOptsO.inference.targets.sva <- renderText({
		cnames <- input$rnbOptsI.inference.targets.sva
		if (length(cnames)<1) cnames <- character(0)
		rnb.options(inference.targets.sva=cnames)
		rnb.getOption("inference.targets.sva")
	})
	output$rnbOptsO.inference.sva.num.method <- renderText({
		rnb.options(inference.sva.num.method=input$rnbOptsI.inference.sva.num.method)
		rnb.getOption("inference.sva.num.method")
	})
	output$rnbOptsO.inference.reference.methylome.column <- renderText({
		cname <- input$rnbOptsI.inference.reference.methylome.column
		if (is.null(cname) || cname=="[None]") cname <- NULL
		rnb.options(inference.reference.methylome.column=cname)
		rnb.getOption("inference.reference.methylome.column")
	})
	doExploratory <- reactive({
		res <- input$rnbOptsI.exploratory
		oNames <- grep("^rnbOptsI.exploratory.", names(input), value=TRUE)
		if (res){
			for (oo in oNames){
				shinyjs::enable(oo)
			}
		} else {
			for (oo in oNames){
				shinyjs::disable(oo)
			}
		}
		res
	})
	output$rnbOptsO.exploratory <- renderText({
		rnb.options(exploratory=doExploratory())
		rnb.getOption("exploratory")
	})
	output$rnbOptsO.exploratory.columns <- renderText({
		cnames <- input$rnbOptsI.exploratory.columns
		if (length(cnames)<1) cnames <- NULL
		if (length(cnames)==1 && cnames=="[automatic]") cnames <- NULL
		cnames <- setdiff(cnames, "[automatic]")
		rnb.options(exploratory.columns=cnames)
		rnb.getOption("exploratory.columns")
	})
	output$rnbOptsO.exploratory.intersample <- renderText({
		rnb.options(exploratory.intersample=input$rnbOptsI.exploratory.intersample)
		rnb.getOption("exploratory.intersample")
	})
	output$rnbOptsO.exploratory.beta.distribution <- renderText({
		rnb.options(exploratory.beta.distribution=input$rnbOptsI.exploratory.beta.distribution)
		rnb.getOption("exploratory.beta.distribution")
	})
	output$rnbOptsO.exploratory.correlation.qc <- renderText({
		interfaceSetting <- input$rnbOptsI.exploratory.correlation.qc
		res <- rnb.getOption("exploratory.correlation.qc")
		if (!isBiseq()){
			res <- interfaceSetting
			rnb.options(exploratory.correlation.qc=res)
		}
		res
	})
	output$rnbOptsO.exploratory.region.profiles <- renderText({
		rts <- input$rnbOptsI.exploratory.region.profiles
		if (length(rts)<1) rts <- character(0)
		rnb.options(exploratory.region.profiles=rts)
		rnb.getOption("exploratory.region.profiles")
	})
	doDifferential <- reactive({
		res <- input$rnbOptsI.differential
		oNames <- grep("^rnbOptsI.differential.", names(input), value=TRUE)
		if (res){
			for (oo in oNames){
				shinyjs::enable(oo)
			}
		} else {
			for (oo in oNames){
				shinyjs::disable(oo)
			}
		}
		res
	})
	output$rnbOptsO.differential <- renderText({
		rnb.options(differential=doDifferential())
		rnb.getOption("differential")
	})
	output$rnbOptsO.differential.comparison.columns <- renderText({
		cnames <- input$rnbOptsI.differential.comparison.columns
		if (length(cnames)<1) cnames <- NULL
		if (length(cnames)==1 && cnames=="[automatic]") cnames <- NULL
		cnames <- setdiff(cnames, "[automatic]")
		rnb.options(differential.comparison.columns=cnames)
		rnb.getOption("differential.comparison.columns")
	})
	output$rnbOptsO.covariate.adjustment.columns <- renderText({
		cnames <- input$rnbOptsI.covariate.adjustment.columns
		if (length(cnames)<1) cnames <- NULL
		rnb.options(covariate.adjustment.columns=cnames)
		rnb.getOption("covariate.adjustment.columns")
	})
	output$rnbOptsO.differential.site.test.method <- renderText({
		rnb.options(differential.site.test.method=input$rnbOptsI.differential.site.test.method)
		rnb.getOption("differential.site.test.method")
	})
	output$rnbOptsO.differential.report.sites <- renderText({
		rnb.options(differential.report.sites=input$rnbOptsI.differential.report.sites)
		rnb.getOption("differential.report.sites")
	})
	doDiffVar <- reactive({
		res <- input$rnbOptsI.differential.variability
		if (res){
			shinyjs::enable("rnbOptsI.differential.variability.method")
		} else {
			shinyjs::disable("rnbOptsI.differential.variability.method")
		}
		res
	})
	output$rnbOptsO.differential.variability <- renderText({
		rnb.options(differential.variability=doDiffVar())
		rnb.getOption("differential.variability")
	})
	output$rnbOptsO.differential.variability.method <- renderText({
		rnb.options(differential.variability.method=input$rnbOptsI.differential.variability.method)
		rnb.getOption("differential.variability.method")
	})
	output$rnbOptsO.differential.enrichment.go <- renderText({
		rnb.options(differential.enrichment.go=input$rnbOptsI.differential.enrichment.go)
		rnb.getOption("differential.enrichment.go")
	})
	output$rnbOptsO.differential.enrichment.lola <- renderText({
		rnb.options(differential.enrichment.lola=input$rnbOptsI.differential.enrichment.lola)
		rnb.getOption("differential.enrichment.lola")
	})

	#apply the option setting 'ovalue' for option with name 'oname'
	applyOptValue <- function(oname, ovalue, fallback=FALSE){
		if (oname=="analysis.name") {
			updateTextInput(session, "rnbOptsI.analysis.name", value=ovalue)
		} else if (oname=="assembly") {
			if (is.element(ovalue, RNB.ASSEMBLIES)){
				updateSelectInput(session, "rnbOptsI.assembly", selected=ovalue)
			} else {
				stop(paste0("Invalid assembly: ", ovalue))
			}
		} else if (oname=="region.types") {
			if (is.null(ovalue) || all(ovalue %in% regTypes.all())){
				if (is.null(ovalue)) ovalue <- regTypes.all()
				optionSettingObserver$selRegs <- ovalue
				updateSelectInput(session, "rnbOptsI.region.types", selected=ovalue)
			} else {
				stop(paste0("Region type(s) not supported by current assembly (", rnb.getOption("assembly"), "): ", paste(setdiff(ovalue, regTypes.all()), collapse=", ")))
			}
		} else if (oname=="identifiers.column") {
			if (is.null(ovalue) || is.element(ovalue, sannot.cols.plusNone())){
				if (is.null(ovalue)) ovalue <- "[None]"
				optionSettingObserver$selColumn.id <- ovalue
				updateSelectInput(session, "rnbOptsI.identifiers.column", selected=ovalue)
			} else {
				stop(paste0("Sample annotation column not supported"))
			}
		} else if (oname=="min.group.size") {
			if (ovalue >= RNB.GROUP.SIZE.RANGE[1] && ovalue <= RNB.GROUP.SIZE.RANGE[2]){
				# print("DEBUG: updating min.group.size option")
				updateSliderInput(session, "rnbOptsI.min.group.size", value=ovalue)
			} else {
				stop(paste0("Not within expected range [", RNB.GROUP.SIZE.RANGE[1], "-", RNB.GROUP.SIZE.RANGE[2],"]: ", ovalue))
			}
		} else if (oname=="max.group.count") {
			if (is.null(ovalue) || (ovalue >= RNB.GROUP.COUNT.RANGE[1] && ovalue <= optionSettingObserver$group.count.range.max)){
				updateSliderInput(session, "rnbOptsI.max.group.count", value=ovalue)
			} else {
				stop(paste0("Not within expected range [", RNB.GROUP.COUNT.RANGE[1], "-", optionSettingObserver$group.count.range.max,"]: ", ovalue))
			}
		} else if (oname=="colors.category") {
			selVal <- ovalue
			if (length(ovalue)>1){
				selVal <- "[custom]"
				optionSettingObserver$colors.category.list[[selVal]] <- ovalue
			}
			updateSelectInput(session, "rnbOptsI.colors.category", choices=names(optionSettingObserver$colors.category.list), selected=selVal)
		} else if (oname=="import.default.data.type") {
			if (ovalue != optsImportDataType()){
				stop(paste0("Incompatible option with currently selected platform"))
			}
		} else if (oname=="import.table.separator") {
			if (is.element(ovalue, RNB.TABLE.SEPS)){
				updateSelectInput(session, "rnbOptsI.import.table.separator", selected=ovalue)
			} else {
				stop(paste0("Invalid table separator: ", ovalue))
			}
		} else if (oname=="import.bed.style") {
			if (is.element(ovalue, RNB.BED.STYLES)){
				updateSelectInput(session, "rnbOptsI.import.bed.style", selected=ovalue)
			} else {
				stop(paste0("Invalid import.bed.style: ", ovalue))
			}
		} else if (oname=="filtering.coverage.threshold") {
			if (ovalue >= 1 && ovalue <= 100){
				updateSliderInput(session, "rnbOptsI.filtering.coverage.threshold", value=ovalue)
			} else {
				stop(paste0("Not within expected range [", 1, "-", 100, "]: ", ovalue))
			}
		} else if (oname=="filtering.low.coverage.masking") {
			updateCheckboxInput(session, "rnbOptsI.filtering.low.coverage.masking", value=ovalue)
		} else if (oname=="filtering.high.coverage.outliers") {
			updateCheckboxInput(session, "rnbOptsI.filtering.high.coverage.outliers", value=ovalue)
		} else if (oname=="filtering.missing.value.quantile") {
			if (ovalue >= 0 && ovalue <= 1){
				updateSliderInput(session, "rnbOptsI.filtering.missing.value.quantile", value=ovalue)
			} else {
				stop(paste0("Not within expected range [", 0, "-", 1, "]: ", ovalue))
			}
		} else if (oname=="filtering.greedycut") {
			updateCheckboxInput(session, "rnbOptsI.filtering.greedycut", value=ovalue)
		} else if (oname=="filtering.sex.chromosomes.removal") {
			updateCheckboxInput(session, "rnbOptsI.filtering.sex.chromosomes.removal", value=ovalue)
		} else if (oname=="filtering.snp") {
			if (is.element(ovalue, RNB.FILTERING.SNP)){
				updateSelectInput(session, "rnbOptsI.filtering.snp", selected=ovalue)
			} else {
				stop(paste0("Invalid selection: ", ovalue))
			}
		} else if (oname=="filtering.cross.reactive") {
			updateCheckboxInput(session, "rnbOptsI.filtering.cross.reactive", value=ovalue)
		} else if (oname=="normalization.method") {
			if (is.element(ovalue, RNB.NORMALIZATION.METHODS)){
				updateSelectInput(session, "rnbOptsI.normalization.method", selected=ovalue)
			} else {
				stop(paste0("Invalid selection: ", ovalue))
			}
		} else if (oname=="normalization.background.method") {
			if (is.element(ovalue, RNB.NORMALIZATION.BG.METHODS)){
				updateSelectInput(session, "rnbOptsI.normalization.background.method", selected=ovalue)
			} else {
				stop(paste0("Invalid selection: ", ovalue))
			}
		} else if (oname=="imputation.method") {
			if (is.element(ovalue, RNB.IMPUTATION.METHODS)){
				updateSelectInput(session, "rnbOptsI.imputation.method", selected=ovalue)
			} else {
				stop(paste0("Invalid selection: ", ovalue))
			}
		} else if (oname=="export.to.trackhub") {
			if (is.null(ovalue) || all(ovalue %in% RNB.TRACKHUB.FORMATS)){
				if (is.null(ovalue)) ovalue <- character(0)
				updateSelectInput(session, "rnbOptsI.export.to.trackhub", selected=ovalue)
			} else {
				stop(paste0("Trackhub formats not supported: ", paste(setdiff(ovalue, RNB.TRACKHUB.FORMATS), collapse=", ")))
			}
		} else if (oname=="export.types") {
			# allowedVals <- regTypes.plus.sites()
			allowedVals <- c("sites", optionSettingObserver$selRegs)
			if (fallback) ovalue <- intersect(ovalue, allowedVals) #if resetting to old options, make sure that the old regions are contained in the allowed regions
			if (is.null(ovalue) || all(ovalue %in% allowedVals)){
				if (is.null(ovalue)) ovalue <- character(0)
				optionSettingObserver$selRegs.export <- ovalue
				updateSelectInput(session, "rnbOptsI.export.types", selected=ovalue)
			} else {
				stop(paste0("Export types not supported: ", paste(setdiff(ovalue, allowedVals), collapse=", ")))
			}
		} else if (oname=="inference") {
			updateCheckboxInput(session, "rnbOptsI.inference", value=ovalue)
		} else if (oname=="inference.age.prediction") {
			updateCheckboxInput(session, "rnbOptsI.inference.age.prediction", value=ovalue)
		} else if (oname=="inference.targets.sva") {
			print(ovalue)
			print(sannot.cols.grps())
			if (all(ovalue %in% sannot.cols.grps())){
				# print("DEBUG: updating inference.targets.sva option")
				optionSettingObserver$selColumn.sva <- ovalue
				updateSelectInput(session, "rnbOptsI.inference.targets.sva", selected=ovalue)
			} else {
				stop(paste0("Sample annotation column(s) not supported"))
			}
		} else if (oname=="inference.sva.num.method") {
			if (is.element(ovalue, RNB.SVA.NUM.METHODS)){
				updateSelectInput(session, "rnbOptsI.inference.sva.num.method", selected=ovalue)
			} else {
				stop(paste0("Invalid selection: ", ovalue))
			}
		} else if (oname=="inference.reference.methylome.column") {
			if (is.null(ovalue) || is.element(ovalue, sannot.cols.plusNone())){
				if (is.null(ovalue)) ovalue <- "[None]"
				optionSettingObserver$selColumn.cellTypeRef <- ovalue
				updateSelectInput(session, "rnbOptsI.inference.reference.methylome.column", selected=ovalue)
			} else {
				stop(paste0("Sample annotation column not supported"))
			}
		} else if (oname=="exploratory.columns") {
			if (is.null(ovalue) || all(ovalue %in% sannot.cols.plusAutomatic())){
				if (is.null(ovalue)) ovalue <- "[automatic]"
				optionSettingObserver$selColumn.ex <- ovalue
				updateSelectInput(session, "rnbOptsI.exploratory.columns", selected=ovalue)
			} else {
				stop(paste0("Sample annotation column(s) not supported"))
			}
		} else if (oname=="exploratory.intersample") {
			updateCheckboxInput(session, "rnbOptsI.exploratory.intersample", value=ovalue)
		} else if (oname=="exploratory.beta.distribution") {
			updateCheckboxInput(session, "rnbOptsI.exploratory.beta.distribution", value=ovalue)
		} else if (oname=="exploratory.correlation.qc") {
			updateCheckboxInput(session, "rnbOptsI.exploratory.correlation.qc", value=ovalue)
		} else if (oname=="exploratory.region.profiles") {
			allowedVals <- optionSettingObserver$selRegs
			if (fallback) ovalue <- intersect(ovalue, allowedVals) #if resetting to old options, make sure that the old regions are contained in the allowed regions
			if (is.null(ovalue) || all(ovalue %in% allowedVals)){
				if (is.null(ovalue)) ovalue <- intersect(c("genes", "promoters", "cpgislands"), allowedVals)
				optionSettingObserver$selRegs.exploratory.profiles <- ovalue
				updateSelectInput(session, "rnbOptsI.exploratory.region.profiles", selected=ovalue)
			} else {
				stop(paste0("Region profiles not supported for region types: ", paste(setdiff(ovalue, allowedVals), collapse=", ")))
			}
		} else if (oname=="differential.comparison.columns") {
			if (is.null(ovalue) || all(ovalue %in% sannot.cols.plusAutomatic())){
				if (is.null(ovalue)) ovalue <- "[automatic]"
				optionSettingObserver$selColumn.diff <- ovalue
				updateSelectInput(session, "rnbOptsI.differential.comparison.columns", selected=ovalue)
			} else {
				stop(paste0("Sample annotation column(s) not supported"))
			}
		} else if (oname=="covariate.adjustment.columns") {
			if (is.null(ovalue) || all(ovalue %in% sannot.cols())){
				if (is.null(ovalue)) ovalue <- character(0)
				optionSettingObserver$selColumn.adj <- ovalue
				updateSelectInput(session, "rnbOptsI.covariate.adjustment.columns", selected=ovalue)
			} else {
				stop(paste0("Sample annotation column(s) not supported"))
			}
		} else if (oname=="differential.site.test.method") {
			if (is.element(ovalue, RNB.DIFFMETH.TEST.METHODS)){
				updateSelectInput(session, "rnbOptsI.differential.site.test.method", selected=ovalue)
			} else {
				stop(paste0("Invalid selection: ", ovalue))
			}
		} else if (oname=="differential.report.sites") {
			updateCheckboxInput(session, "rnbOptsI.differential.report.sites", value=ovalue)
		} else if (oname=="differential.variability") {
			updateCheckboxInput(session, "rnbOptsI.differential.variability", value=ovalue)
		} else if (oname=="differential.variability.method") {
			if (is.element(ovalue, RNB.DIFFVAR.METHODS)){
				updateSelectInput(session, "rnbOptsI.differential.variability.method", selected=ovalue)
			} else {
				stop(paste0("Invalid selection: ", ovalue))
			}
		} else if (oname=="differential.enrichment.go") {
			updateCheckboxInput(session, "rnbOptsI.differential.enrichment.go", value=ovalue)
		} else if (oname=="differential.enrichment.lola") {
			updateCheckboxInput(session, "rnbOptsI.differential.enrichment.lola", value=ovalue)
		} 
	}

	#apply the options stored in list 'ol'
	applyOptList <- function(ol, ol.old=list()){
		for (oname in names(ol)){
			# print(paste("DEBUG: Reading XML option:", oname))
			rr <- tryCatch(
				applyOptValue(oname, ol[[oname]]),
				error = function(err) {
					showNotification(tags$span(style="color:red", icon("warning"), paste0("Could not update option '", oname, " (", err$message, ")")))
					if (is.element(oname, names(ol.old))){
						# print(paste("DEBUG: FAILED: Resetting to old option"))
						applyOptValue(oname, ol.old[[oname]], fallback=TRUE)
						optSettingList <- list(ol.old[[oname]])
						names(optSettingList) <- oname
						do.call("rnb.options", optSettingList)
					}
				}
			)
		}
		showNotification(tags$span(style="color:green", icon("check"), paste0("Option settings applied")))
	}

	observeLocalFileInput(input, session, 'loadOptsXmlFile')
	loadOptsXml.fName <- reactive({
		readLocalFileInput(session, 'loadOptsXmlFile')
	})
	observeEvent(input$loadOptsAnaDirDo, {
		curStatus <- anaStatus()
		isValid <- curStatus$status=="reportDir"
		xmlFile <- file.path(reportDir(), "analysis_options.xml")
		if (isValid && file.exists(xmlFile)){
			rnbOpts.old <- rnb.options()
			optList <- tryCatch({
					dummy <- rnb.xml2options(xmlFile)
					rnb.options()
				},
				error = function(err) {
					showNotification(tags$span(style="color:red", icon("warning"), paste0("Could not load option file from analysis directory:", err$message)))
					NULL
				}
			)
			if (length(optList) > 0){
				applyOptList(optList, rnbOpts.old)
			}
		} else {
			showNotification(tags$span(style="color:red", icon("warning"), paste0("Option file does not exist in analysis directory")))
		}
	})
	observeEvent(input$loadOptsXmlDo, {
		# print("DEBUG: Reading XML file")
		xmlFile <- loadOptsXml.fName()
		if (file.exists(xmlFile)){
			rnbOpts.old <- rnb.options()
			# print("DEBUG: Old option settings")
			# print(rnbOpts.old)
			optList <- tryCatch({
					dummy <- rnb.xml2options(xmlFile)
					rnb.options()
				},
				error = function(err) {
					showNotification(tags$span(style="color:red", icon("warning"), paste0("Could not load option file: ", err$message)))
					NULL
				}
			)
			if (length(optList) > 0){
				applyOptList(optList, rnbOpts.old)
			}
		} else {
			showNotification(tags$span(style="color:red", icon("warning"), paste0("Option file does not exist")))
		}
	})
	observeEvent(input$loadOptsProfileDo, {
		xmlFile <- file.path(RNB.OPTION.PROFILES.PATH, paste0(input$loadOptsProfileSel, ".xml"))
		if (file.exists(xmlFile)){
			rnbOpts.old <- rnb.options()
			optList <- tryCatch({
					dummy <- rnb.xml2options(xmlFile)
					rnb.options()
				},
				error = function(err) {
					showNotification(tags$span(style="color:red", icon("warning"), paste0("Could not load option profile:", err$message)))
					NULL
				}
			)
			if (length(optList) > 0){
				applyOptList(optList, rnbOpts.old)
			}
		} else {
			showNotification(tags$span(style="color:red", icon("warning"), paste0("Option profile does not exist")))
		}
	})

	observeLocalFileInput(input, session, 'rnbOptsI.filtering.whitelist')
	rnbOpts.filtering.whitelist.fn <- reactive({
		readLocalFileInput(session, 'rnbOptsI.filtering.whitelist')
	})
	observeEvent(input$rnbOptsResetWhitelist, {
		updateLocalFileInput(session, 'rnbOptsI.filtering.whitelist', value='[NULL]')
	})
	observeLocalFileInput(input, session, 'rnbOptsI.filtering.blacklist')
	rnbOpts.filtering.blacklist.fn <- reactive({
		readLocalFileInput(session, 'rnbOptsI.filtering.blacklist')
	})
	observeEvent(input$rnbOptsResetblacklist, {
		updateLocalFileInput(session, 'rnbOptsI.filtering.blacklist', value='[NULL]')
	})

	output$saveOptsXml <- downloadHandler(
		filename="rnb_options.xml",
		content=function(fname){
			cat(rnb.options2xml(), file=fname)
		}
	)

	############################################################################
	# Run
	############################################################################
	enableRun <- reactive({
		res <- TRUE
		if (dir.exists(reportDir())) res <- FALSE #directory exists
		if (!file.exists(sampleAnnotFile())) res <- FALSE #sample annotation file does not exist
		if (!dir.exists(inputDataDir())) res <- FALSE # data directory does not exist
		return(res)
	})
	output$runHint <- renderUI({
		if (enableRun()){
			shinyjs::enable("runRnb")
			tags$p(tags$span(style="color:green", icon("play"), "Ready to run"))
		} else {
			shinyjs::disable("runRnb")
			tags$p(tags$span(style="color:red", icon("warning"), "Unable to run the analysis. Please make sure that you selected a non-existing report directory ('Analysis' tab) and that you specified the sample annotation file and data directory correctly ('Input' tab)"))
		}
	})
	observeEvent(input$ggplotTheme, {
		eval(parse(text=paste0("theme_set(theme_", input$ggplotTheme, "())")))
	})
	observeEvent(input$runRnb, {
		if(input$runRnb == 0) return()
        shinyjs::disable("runRnb")
    	setNumCoresFromSlider()

    	logger.close() # close the server's current logger s.t. a new log file is created in the report directory
		withProgress({
			rnb.run.analysis(
				dir.reports=reportDir(),
				sample.sheet=sampleAnnotFile(),
				data.dir=inputDataDir(),
				data.type=rnb.getOption("import.default.data.type"),
				save.rdata=TRUE
			)
		}, message="Runnning RnBeads analysis")
	})
	# output$runStatusMsg <- renderText({
	# 	refreshTimer()
	# 	print(input$runRnb)
	# 	if(input$runRnb == 0) return("Analysis not started")
	# 	res <- "Analysis started"
	# 	# logFn <- file.path(reportDir(), "analysis.log")
	# 	# status <- getRnbStatusFromLog(logFn)
	# 	# status <- status[status$scheduled,]
	# 	# if (all(status$status=="completed")) return("Analysis completed")
	# 	# startedModules <- which(status$status=="started")
	# 	# if (length(startedModules) > 0){
	# 	# 	mm <- status$module[startedModules[length(startedModules)]] # pick last started module
	# 	# 	res <- paste0("Running ", mm)
	# 	# }
	# 	return(res)
	# })

	############################################################################
	# Modules
	############################################################################
	# IMPORT
	output$modImportNew.about <- renderUI({
		if (enableRun()){
			if (!modImportStatus$dataset.loaded){
				shinyjs::enable("modImportNew")
				shinyjs::enable("modImportNew.save")
			}
			tagList(
				tags$p(tags$span(icon("play"), "Ready to run")),
				checkboxInput("modImportNew.save", "Save RnBSet object", value=TRUE)
			)
		} else {
			shinyjs::disable("modImportNew")
			shinyjs::disable("modImportNew.save")
			tags$p(tags$span(icon("warning"), "Unable to run. Please make sure that you selected a non-existing report directory ('Analysis' tab) and that you specified the sample annotation file and data directory correctly ('Input' tab)"))
		}
	})
	output$modImportAnaDir.about <- renderUI({
		curStatus <- anaStatus()
		if (curStatus$status=="reportDir"){
			if (length(curStatus$rnbSet.paths) > 0){
				pp <- curStatus$rnbSet.paths
				names(pp) <- basename(pp)
				tagList(
					tags$p("An existing RnBeads report has been specified. Ready to import."),
					selectInput("modImportAnaDir.rnbSet", "Select RnBSet to load", pp)
				)
			} else {
				tagList(tags$p(icon("warning"), "The analysis directory does not contain RnBSet objects."))
			}
		} else {
			tagList(tags$p(icon("warning"), "Please specify an existing RnBeads report in the 'Analysis' tab to continue."))
		}
	})
	output$rnbSetInfo <- renderUI({
		if (is.null(rnbData$rnbSet)) return(NULL)
		tagList(
			tags$h2("RnBSet Object:"),
			renderPrint({methods::show(rnbData$rnbSet)}),
			tags$h2("Sample Annotation:"),
			renderTable({sannot()}, striped=TRUE, hover=TRUE, bordered=TRUE)
		)
	})
	modImportStatus <- reactiveValues(
		dataset.loaded=FALSE,
		dataset.loaded.nsamples=-1
	)
	rnbData <- reactiveValues(
		rnbSet=NULL
	)
	# watch if dataset has been loaded
	reactDataset <- observeEvent(modImportStatus$dataset.loaded, {
		if (modImportStatus$dataset.loaded){
			shinyjs::disable("modImportNew")
			shinyjs::disable("modImportNew.save")
			shinyjs::disable("modImportAnaDir")
			shinyjs::disable("modImportRObjects")
			shinyjs::enable("modImportReset")
		} else {
			if (enableRun()) {
				shinyjs::enable("modImportNew")
				shinyjs::enable("modImportNew.save")
			}
			if (anaStatus()$status=="reportDir" && length(anaStatus()$rnbSet.paths) > 0) shinyjs::enable("modImportAnaDir")
			shinyjs::enable("modImportRObjects")
			shinyjs::disable("modImportReset")
		}
	})
	output$modImport.status <- renderUI({
		# curStatus <- modImportStatus()
		res <- list()
		if (modImportStatus$dataset.loaded){
			res <- c(res, list(tags$p(tags$span(style="color:green", icon("check"), "Loaded dataset containing", modImportStatus$dataset.loaded.nsamples, "samples"))))
		} else {
			res <- c(res, list(tags$p(tags$span(style="color:red", icon("times"), "No dataset loaded"))))
		}
		res <- c(res, list(tags$p("The current report directory is ", tags$code(reportDir()), ". This can be configured in the 'Analysis' tab.")))
		tagList(res)
	})
	observeEvent(input$modImportNew, {
		modImportStatus$dataset.loaded <- TRUE
		withProgress({
			tryCatch({
					setNumCoresFromSlider()
					if(!dir.exists(reportDir())) rnb.initialize.reports(reportDir())
					logger.start(fname=file.path(reportDir(), "analysis.log"))
					res <- rnb.run.import(c(inputDataDir(), sampleAnnotFile()), data.type=rnb.getOption("import.default.data.type"), dir.reports=reportDir())
					logger.close()
					rnbData$rnbSet <- res$rnb.set
					markDirDJ(reportDir())
					if (input$modImportNew.save) save.rnb.set(rnbData$rnbSet, file.path(reportDir(), "rnbSet_import"), archive=FALSE)
					modImportStatus$dataset.loaded.nsamples <- length(samples(rnbData$rnbSet))
				},
				error = function(err) {
					rnbData$rnbSet <- NULL
					modImportStatus$dataset.loaded <- FALSE
					showNotification(tags$span(style="color:red", icon("warning"), paste0("Failed to import data: ", err$message)))
				}
			)
		}, message="Importing dataset")
	})
	observeEvent(input$modImportAnaDir, {
		modImportStatus$dataset.loaded <- TRUE
		withProgress({
			tryCatch({
					rnbData$rnbSet <- load.rnb.set(input$modImportAnaDir.rnbSet)
					modImportStatus$dataset.loaded.nsamples <- length(samples(rnbData$rnbSet))
				},
				error = function(err) {
					rnbData$rnbSet <- NULL
					modImportStatus$dataset.loaded <- FALSE
					showNotification(tags$span(style="color:red", icon("warning"), paste0("Failed to load RnBSet object: ", err$message)))
				}
			)
		}, message="Loading dataset")
		#TODO: import options
	})
	observeDirectoryInput(input, session, 'modImportRnBSetDir')
	# observeLocalFileInput(input, session, 'modImportOptionsFile')
	observeEvent(input$modImportRObjects, {
		modImportStatus$dataset.loaded <- TRUE
		withProgress({
			tryCatch({
					rnbData$rnbSet <- load.rnb.set(readDirectoryInput(session, 'modImportRnBSetDir'))
					modImportStatus$dataset.loaded.nsamples <- length(samples(rnbData$rnbSet))
				},
				error = function(err) {
					rnbData$rnbSet <- NULL
					modImportStatus$dataset.loaded <- FALSE
					showNotification(tags$span(style="color:red", icon("warning"), paste0("Failed to load RnBSet object: ", err$message)))
				}
			)
		}, message="Loading dataset")
		#TODO: import options
	})
	observeEvent(input$modImportReset, {
		modImportStatus$dataset.loaded <- FALSE
		modImportStatus$dataset.loaded.nsamples <- -1
		rnbData$rnbSet <- NULL
	})

	# QUALITY CONTROL
	reportExists.quality_control <- reactive({
		curStatus <- anaStatus()
		if (is.null(curStatus$statusTab)) return(FALSE)
		!is.na(curStatus$statusTab["quality_control", "Report"])
	})
	output$modQC.status <- renderUI({
		reportStatus <- checkReportDir(reportDir())
		rdy <- FALSE
		res <- list()
		if (modImportStatus$dataset.loaded){
			rdy <- TRUE
			res <- c(res, list(tags$p(tags$span(style="color:green", icon("check"), "Dataset loaded."))))
		} else {
			res <- c(res, list(tags$p(tags$span(style="color:red", icon("times"), "No dataset loaded. Please load a dataset using the 'Data Import' tab."))))
		}
		res <- c(res, list(tags$p("The current report directory is ", tags$code(reportDir()), ". This can be configured in the 'Analysis' tab.")))
		if (reportExists.quality_control()){
			if (rdy){
				shinyjs::enable("modQC.overwrite")
			}
			if (!input$modQC.overwrite){
				rdy <- FALSE
			}
			res <- c(res, list(tags$div(title="If the link does not open use right-click, copy the link and paste the address in a new browser window", tags$p(tags$span(icon("search"), tags$a(href=paste0("file://", file.path(reportDir(), reportStatus$reportHtml["quality_control"])), "View report"))))))
		} else {
			shinyjs::disable("modQC.overwrite")
		}
		if (rdy){
			shinyjs::enable("modQC.run")
		} else {
			shinyjs::disable("modQC.run")
		}
		tagList(res)
	})
	observeEvent(input$modQC.run, {
		withProgress({
			tryCatch({
					setNumCoresFromSlider()
					if(!dir.exists(reportDir())) rnb.initialize.reports(reportDir())
					if (reportExists.quality_control() && input$modQC.overwrite){
						unlink(file.path(reportDir(), paste0("quality_control","*")), recursive=TRUE)
						showNotification(tags$span(icon("trash"), paste0("Deleted previous report")))
					}
					updateCheckboxInput(session, "modQC.overwrite", value=FALSE)
					logger.start(fname=file.path(reportDir(), "analysis.log"))
					res <- rnb.run.qc(rnbData$rnbSet, dir.reports=reportDir())
					logger.close()
					markDirDJ(reportDir())
					showNotification(tags$span(style="color:green", icon("check"), paste0("Analysis (Quality Control) completed")))
				},
				error = function(err) {
					showNotification(tags$span(style="color:red", icon("warning"), paste0("Analysis (Quality Control) failed:", err$message)))
				}
			)
		}, message="Performing analysis: Quality Control")
	})

	# PREPROCESSING
	reportExists.preprocessing <- reactive({
		curStatus <- anaStatus()
		if (is.null(curStatus$statusTab)) return(FALSE)
		!is.na(curStatus$statusTab["preprocessing", "Report"])
	})
	output$modPreprocessing.status <- renderUI({
		reportStatus <- checkReportDir(reportDir())
		rdy <- FALSE
		res <- list()
		if (modImportStatus$dataset.loaded){
			rdy <- TRUE
			res <- c(res, list(tags$p(tags$span(style="color:green", icon("check"), "Dataset loaded."))))
		} else {
			res <- c(res, list(tags$p(tags$span(style="color:red", icon("times"), "No dataset loaded. Please load a dataset using the 'Data Import' tab."))))
		}
		res <- c(res, list(tags$p("The current report directory is ", tags$code(reportDir()), ". This can be configured in the 'Analysis' tab.")))
		if (reportExists.preprocessing()){
			if (rdy){
				shinyjs::enable("modPreprocessing.overwrite")
			}
			if (!input$modPreprocessing.overwrite){
				rdy <- FALSE
			}
			res <- c(res, list(tags$div(title="If the link does not open use right-click, copy the link and paste the address in a new browser window", tags$p(tags$span(icon("search"), tags$a(href=paste0("file://", file.path(reportDir(), reportStatus$reportHtml["preprocessing"])), "View report"))))))
		} else {
			shinyjs::disable("modPreprocessing.overwrite")
		}
		if (rdy){
			shinyjs::enable("modPreprocessing.run")
		} else {
			shinyjs::disable("modPreprocessing.run")
		}
		tagList(res)
	})
	observeEvent(input$modPreprocessing.run, {
		withProgress({
			tryCatch({
					setNumCoresFromSlider()
					if(!dir.exists(reportDir())) rnb.initialize.reports(reportDir())
					if (reportExists.preprocessing() && input$modPreprocessing.overwrite){
						unlink(file.path(reportDir(), paste0("preprocessing","*")), recursive=TRUE)
						showNotification(tags$span(icon("trash"), paste0("Deleted previous report")))
					}
					updateCheckboxInput(session, "modPreprocessing.overwrite", value=FALSE)
					logger.start(fname=file.path(reportDir(), "analysis.log"))
					res <- rnb.run.preprocessing(rnbData$rnbSet, dir.reports=reportDir())
					logger.close()
					markDirDJ(reportDir())
					rnbData$rnbSet <- res$rnb.set
					if (input$modPreprocessing.save) {
						rnbsPath <- file.path(reportDir(), "rnbSet_preprocessed")
						if (dir.exists(rnbsPath)){
							unlink(rnbsPath, recursive=TRUE)
							showNotification(tags$span(icon("trash"), paste0("Deleted previous RnBSet object from report directory")))
						}
						save.rnb.set(rnbData$rnbSet, rnbsPath, archive=FALSE)
					}
					showNotification(tags$span(style="color:green", icon("check"), paste0("Analysis (Preprocessing) completed")))
					showNotification(tags$span(style="color:green", icon("warning"), paste0("Replaced loaded RnBSet object with preprocessed object")))
				},
				error = function(err) {
					showNotification(tags$span(style="color:red", icon("warning"), paste0("Analysis (Preprocessing) failed:", err$message)))
				}
			)
		}, message="Performing analysis: Preprocessing")
	})

	# TRACKS AND TABLES
	reportExists.tracks_and_tables <- reactive({
		curStatus <- anaStatus()
		if (is.null(curStatus$statusTab)) return(FALSE)
		!is.na(curStatus$statusTab["tracks_and_tables", "Report"])
	})
	output$modTNT.status <- renderUI({
		reportStatus <- checkReportDir(reportDir())
		rdy <- FALSE
		res <- list()
		if (modImportStatus$dataset.loaded){
			rdy <- TRUE
			res <- c(res, list(tags$p(tags$span(style="color:green", icon("check"), "Dataset loaded."))))
		} else {
			res <- c(res, list(tags$p(tags$span(style="color:red", icon("times"), "No dataset loaded. Please load a dataset using the 'Data Import' tab."))))
		}
		res <- c(res, list(tags$p("The current report directory is ", tags$code(reportDir()), ". This can be configured in the 'Analysis' tab.")))
		if (reportExists.tracks_and_tables()){
			if (rdy){
				shinyjs::enable("modTNT.overwrite")
			}
			if (!input$modTNT.overwrite){
				rdy <- FALSE
			}
			res <- c(res, list(tags$div(title="If the link does not open use right-click, copy the link and paste the address in a new browser window", tags$p(tags$span(icon("search"), tags$a(href=paste0("file://", file.path(reportDir(), reportStatus$reportHtml["tracks_and_tables"])), "View report"))))))
		} else {
			shinyjs::disable("modTNT.overwrite")
		}
		if (rdy){
			shinyjs::enable("modTNT.run")
		} else {
			shinyjs::disable("modTNT.run")
		}
		tagList(res)
	})
	observeEvent(input$modTNT.run, {
		withProgress({
			tryCatch({
					setNumCoresFromSlider()
					if(!dir.exists(reportDir())) rnb.initialize.reports(reportDir())
					if (reportExists.tracks_and_tables() && input$modTNT.overwrite){
						unlink(file.path(reportDir(), paste0("tracks_and_tables","*")), recursive=TRUE)
						showNotification(tags$span(icon("trash"), paste0("Deleted previous report")))
					}
					updateCheckboxInput(session, "modTNT.overwrite", value=FALSE)
					logger.start(fname=file.path(reportDir(), "analysis.log"))
					res <- rnb.run.tnt(rnbData$rnbSet, dir.reports=reportDir())
					logger.close()
					markDirDJ(reportDir())
					showNotification(tags$span(style="color:green", icon("check"), paste0("Analysis (Tracks and Tables) completed")))
				},
				error = function(err) {
					showNotification(tags$span(style="color:red", icon("warning"), paste0("Analysis (Tracks and Tables) failed:", err$message)))
				}
			)
		}, message="Performing analysis: Tracks and Tables")
	})

	# COVARIATE INFERENCE
	reportExists.covariate_inference <- reactive({
		curStatus <- anaStatus()
		if (is.null(curStatus$statusTab)) return(FALSE)
		!is.na(curStatus$statusTab["covariate_inference", "Report"])
	})
	output$modInference.status <- renderUI({
		reportStatus <- checkReportDir(reportDir())
		rdy <- FALSE
		res <- list()
		if (modImportStatus$dataset.loaded){
			rdy <- TRUE
			res <- c(res, list(tags$p(tags$span(style="color:green", icon("check"), "Dataset loaded."))))
		} else {
			res <- c(res, list(tags$p(tags$span(style="color:red", icon("times"), "No dataset loaded. Please load a dataset using the 'Data Import' tab."))))
		}
		res <- c(res, list(tags$p("The current report directory is ", tags$code(reportDir()), ". This can be configured in the 'Analysis' tab.")))
		if (reportExists.covariate_inference()){
			if (rdy){
				shinyjs::enable("modInference.overwrite")
			}
			if (!input$modInference.overwrite){
				rdy <- FALSE
			}
			res <- c(res, list(tags$div(title="If the link does not open use right-click, copy the link and paste the address in a new browser window", tags$p(tags$span(icon("search"), tags$a(href=paste0("file://", file.path(reportDir(), reportStatus$reportHtml["covariate_inference"])), "View report"))))))
		} else {
			shinyjs::disable("modInference.overwrite")
		}
		if (rdy){
			shinyjs::enable("modInference.run")
		} else {
			shinyjs::disable("modInference.run")
		}
		tagList(res)
	})
	observeEvent(input$modInference.run, {
		withProgress({
			tryCatch({
					setNumCoresFromSlider()
					if(!dir.exists(reportDir())) rnb.initialize.reports(reportDir())
					if (reportExists.covariate_inference() && input$modInference.overwrite){
						unlink(file.path(reportDir(), paste0("covariate_inference","*")), recursive=TRUE)
						showNotification(tags$span(icon("trash"), paste0("Deleted previous report")))
					}
					updateCheckboxInput(session, "modInference.overwrite", value=FALSE)
					logger.start(fname=file.path(reportDir(), "analysis.log"))
					res <- rnb.run.inference(rnbData$rnbSet, dir.reports=reportDir())
					logger.close()
					markDirDJ(reportDir())
					rnbData$rnbSet <- res$rnb.set
					if (input$modInference.save) {
						rnbsPath <- file.path(reportDir(), "rnbSet_inference")
						if (dir.exists(rnbsPath)){
							unlink(rnbsPath, recursive=TRUE)
							showNotification(tags$span(icon("trash"), paste0("Deleted previous RnBSet object from report directory")))
						}
						save.rnb.set(rnbData$rnbSet, rnbsPath, archive=FALSE)
					}
					showNotification(tags$span(style="color:green", icon("check"), paste0("Analysis (Covariate Inference) completed")))
					showNotification(tags$span(style="color:green", icon("warning"), paste0("Replaced loaded RnBSet object with inference object")))
				},
				error = function(err) {
					showNotification(tags$span(style="color:red", icon("warning"), paste0("Analysis (Covariate Inference) failed:", err$message)))
				}
			)
		}, message="Performing analysis: Covariate Inference")
	})

	# EXPLORATORY ANALYSIS
	reportExists.exploratory_analysis <- reactive({
		curStatus <- anaStatus()
		if (is.null(curStatus$statusTab)) return(FALSE)
		!is.na(curStatus$statusTab["exploratory_analysis", "Report"])
	})
	output$modExploratory.status <- renderUI({
		reportStatus <- checkReportDir(reportDir())
		rdy <- FALSE
		res <- list()
		if (modImportStatus$dataset.loaded){
			rdy <- TRUE
			res <- c(res, list(tags$p(tags$span(style="color:green", icon("check"), "Dataset loaded."))))
		} else {
			res <- c(res, list(tags$p(tags$span(style="color:red", icon("times"), "No dataset loaded. Please load a dataset using the 'Data Import' tab."))))
		}
		res <- c(res, list(tags$p("The current report directory is ", tags$code(reportDir()), ". This can be configured in the 'Analysis' tab.")))
		if (reportExists.exploratory_analysis()){
			if (rdy){
				shinyjs::enable("modExploratory.overwrite")
			}
			if (!input$modExploratory.overwrite){
				rdy <- FALSE
			}
			res <- c(res, list(tags$div(title="If the link does not open use right-click, copy the link and paste the address in a new browser window", tags$p(tags$span(icon("search"), tags$a(href=paste0("file://", file.path(reportDir(), reportStatus$reportHtml["exploratory_analysis"])), "View report"))))))
		} else {
			shinyjs::disable("modExploratory.overwrite")
		}
		if (rdy){
			shinyjs::enable("modExploratory.run")
		} else {
			shinyjs::disable("modExploratory.run")
		}
		tagList(res)
	})
	observeEvent(input$modExploratory.run, {
		withProgress({
			tryCatch({
					setNumCoresFromSlider()
					if(!dir.exists(reportDir())) rnb.initialize.reports(reportDir())
					if (reportExists.exploratory_analysis() && input$modExploratory.overwrite){
						unlink(file.path(reportDir(), paste0("exploratory_analysis","*")), recursive=TRUE)
						showNotification(tags$span(icon("trash"), paste0("Deleted previous report")))
					}
					updateCheckboxInput(session, "modExploratory.overwrite", value=FALSE)
					logger.start(fname=file.path(reportDir(), "analysis.log"))
					res <- rnb.run.exploratory(rnbData$rnbSet, dir.reports=reportDir())
					logger.close()
					markDirDJ(reportDir())
					showNotification(tags$span(style="color:green", icon("check"), paste0("Analysis (Explorarory Analysis) completed")))
				},
				error = function(err) {
					showNotification(tags$span(style="color:red", icon("warning"), paste0("Analysis (Explorarory Analysis) failed:", err$message)))
				}
			)
		}, message="Performing analysis: Explorarory Analysis")
	})

	# DIFFERENTIAL METHYLATION
	reportExists.differential_methylation <- reactive({
		curStatus <- anaStatus()
		if (is.null(curStatus$statusTab)) return(FALSE)
		!is.na(curStatus$statusTab["differential_methylation", "Report"])
	})
	output$modDifferential.status <- renderUI({
		reportStatus <- checkReportDir(reportDir())
		rdy <- FALSE
		res <- list()
		if (modImportStatus$dataset.loaded){
			rdy <- TRUE
			res <- c(res, list(tags$p(tags$span(style="color:green", icon("check"), "Dataset loaded."))))
		} else {
			res <- c(res, list(tags$p(tags$span(style="color:red", icon("times"), "No dataset loaded. Please load a dataset using the 'Data Import' tab."))))
		}
		res <- c(res, list(tags$p("The current report directory is ", tags$code(reportDir()), ". This can be configured in the 'Analysis' tab.")))
		if (reportExists.differential_methylation()){
			if (rdy){
				shinyjs::enable("modDifferential.overwrite")
			}
			if (!input$modDifferential.overwrite){
				rdy <- FALSE
			}
			res <- c(res, list(tags$div(title="If the link does not open use right-click, copy the link and paste the address in a new browser window", tags$p(tags$span(icon("search"), tags$a(href=paste0("file://", file.path(reportDir(), reportStatus$reportHtml["differential_methylation"])), "View report"))))))
		} else {
			shinyjs::disable("modDifferential.overwrite")
		}
		if (rdy){
			shinyjs::enable("modDifferential.run")
		} else {
			shinyjs::disable("modDifferential.run")
		}
		tagList(res)
	})
	observeEvent(input$modDifferential.run, {
		withProgress({
			tryCatch({
					setNumCoresFromSlider()
					if(!dir.exists(reportDir())) rnb.initialize.reports(reportDir())
					if (reportExists.differential_methylation() && input$modDifferential.overwrite){
						unlink(file.path(reportDir(), paste0("differential_methylation","*")), recursive=TRUE)
						showNotification(tags$span(icon("trash"), paste0("Deleted previous report")))
					}
					updateCheckboxInput(session, "modDifferential.overwrite", value=FALSE)
					logger.start(fname=file.path(reportDir(), "analysis.log"))
					res <- rnb.run.differential(rnbData$rnbSet, dir.reports=reportDir())
					logger.close()
					markDirDJ(reportDir())
					showNotification(tags$span(style="color:green", icon("check"), paste0("Analysis (Differential Methylation) completed")))
				},
				error = function(err) {
					showNotification(tags$span(style="color:red", icon("warning"), paste0("Analysis (Differential Methylation) failed:", err$message)))
				}
			)
		}, message="Performing analysis: Differential Methylation")
	})
}

################################################################################
# Main
################################################################################
shinyApp(ui = ui, server = server)

################################################################################
# Sandbox
################################################################################
# useful link for themes/layout
# https://shiny.rstudio.com/gallery/shiny-theme-selector.html

################################################################################
# TODOs:
# - sannot.cols.grps updates last during XML file loading --> SVA column option has to be loaded twice
################################################################################

########################################################################################################################
## test_logger.R
## creator: Yassen Assenov
## ---------------------------------------------------------------------------------------------------------------------
## Simple tests on the logging functionality.
########################################################################################################################

test_logger <- function() {
	rnb.options(logging.memory = TRUE)
	logger.close()
	checkTrue(logger.isinitialized() == FALSE)

	## Print a dummy log in a file
	logfiles <- c("testme.log", NA)
	logger.start("Analysis", fname = logfiles)
	checkTrue(logger.isinitialized())
	logger.info(c("Loaded information from", "data.RData"))
	logger.start("Processing Detection P-values")
	logger.info(c("Removed", 3979, "probes that overlap with SNPs"))
	logger.info(c("Completed Greedycut on", 510, "samples"))
	logger.completed()
	logger.warning(c("File not found:", "data2.RData"))
	logger.completed()
	logger.close()
	checkTrue(logger.isinitialized() == FALSE)

	## Delete the generated file
	logfiles <- logfiles[!is.na(logfiles)]
	if (length(logfiles) != 0) {
		file.remove(logfiles)
	}
}

################################################################################
# Wrapper for ff objects with more than INT_MAX slots
################################################################################
setClassUnion("characterOrNULL", c("character", "NULL"))

#' BigFfMat Class
#'
#' A class implementing matrices as lists of ff objects by column
#' 
#' @details
#' useful to get rid of the INT_MAX limitatation of ff which is easily reached when specifying large matrices
#'
#' @section Slots:
#' \describe{
#'   \item{\code{cols}}{List of ff objects to store the columns of the matrix}
#'   \item{\code{colNames}}{vector of column names or NULL}
#'   \item{\code{rowNames}}{vector of row names or NULL} 
#'   \item{\code{colN}}{number of matrix columns} 
#'   \item{\code{rowN}}{number of matrix rows} 
#' }
#'
#' @section Methods:
#' \describe{
#'   \item{\code{show}}{Display the object}
#'   \item{\code{nrow}}{get the number of rows in the matrix}
#'   \item{\code{ncol}}{get the number of columns in the matrix}
#'   \item{\code{dim}}{vector of the number of rows and columns in the matrix}
#'   \item{\code{rownames}}{get row names}
#'   \item{\code{rownames<-}}{assign row names}
#'   \item{\code{colnames}}{get column names}
#'   \item{\code{colnames<-}}{assign column names}
#'   \item{\code{[}}{array indexing of the stored matrix}
#'   \item{\code{[<-}}{assign values by array indexing in the stored matrix}
#' }
#'
#' @name BigFfMat-class
#' @noRd
#' @author Fabian Mueller
#' @examples
#' myDf <- data.frame(normal=rnorm(10), integers=sample(1:100,10), index=1:10)
#' tt <- BigFfMat(row.n=4, col.n=3, col.names=LETTERS[1:3])
#' ttr <- BigFfMat(myDf)
#' nrow(tt)
#' ncol(tt)
#' dim(tt)
#' colnames(tt)
#' colnames(tt) <- c("blabb","bli","blubb")
#' colnames(tt)[1:2] <- c("bla","padoink")
#' rownames(tt) <- paste0("r",1:nrow(tt))
#' ttr[6,2]
#' ttr[5:7,c(TRUE,TRUE,FALSE)]
#' ttr[,2]
#' ttr[1:2,]
#' rownames(ttr) <- paste0("r",1:nrow(ttr))
#' ttr[c("r5","r7"),c("integers","normal")]
#' tt[4,2] <- 555
#' tt[1,] <- 1:3
#' tt[2,] <- 6
#' tt[3:4,2:3] <- matrix(10,nrow=2,ncol=2)
#' tt[c("r2","r3"),c("padoink","blubb")] <- -8
#' tt[3:4] <- -666 # currently assigns the value to the entire rows instead of using matrix indices
#' # tt[3:5] <- 666 #does not work
#' tt[,1][3:4] <- -1
#' tt[2:3,][,2] <- c(77,777)
#' tt[2:3,2:3][1:4] <- 1:4
#' mask <- matrix(c(TRUE,FALSE,FALSE,TRUE),ncol=2)
#' tt[2:3,2:3][mask] <- c(1000,9999)
#' tt
#' ttr
setClass("BigFfMat",
	slots=list(cols="list", colNames="characterOrNULL", rowNames="characterOrNULL", colN="integer", rowN="integer", activeClosing="logical")
)

#' Construct BigFfMat objects
#' 
#' @param df       	a matrix or data.frame to be stored
#' @param row.n     specify the number of rows. only plays a role if \code{df} is missing.
#' @param col.n     specify the number of columns. only plays a role if \code{df} is missing.
#' @param row.names specify row names. only plays a role if \code{df} is missing.
#' @param col.names specify column names. only plays a role if \code{df} is missing.
#' @param na.prototype type of NA object that is used for filling empty matrices. only plays a role if \code{df} is missing.
#' @param ...		paramters for \code{ff}
#' 
#' @return an object of class BigFfMat
#' 
#' @name BigFfMat
#' @noRd
BigFfMat <- function(df, row.n, col.n, row.names=NULL, col.names=NULL, na.prototype=as.numeric(NA), active.closing=TRUE, ...){
	#initialize empty matrix if the df object is missing
	if (missing(df)){
		if (missing(row.n) | missing(col.n)){
			stop("Must specify row.n and col.n")
		}
		# df <- matrix(na.prototype, nrow=row.n, ncol=col.n)
		ffList <- lapply(1:col.n, FUN=function(j){
			res <- ff(rep(na.prototype, row.n), ...)
			if (active.closing) close(res)
			res
		})
		row.n <- as.integer(row.n)
		col.n <- as.integer(col.n)
		if (!is.null(row.names)){
			if (is.factor(row.names)){
				row.names <- as.character(row.names)
			}
		}
		if (!is.null(col.names)){
			if (is.factor(col.names)){
				col.names <- as.character(col.names)
			}
		}
	} else {	
		if (is.data.frame(df)){
			df <- as.matrix(df)
		}
		if (is.character(df)){
			stop("Character matrices currently not supported")
		}
		row.n <- nrow(df)
		col.n <- ncol(df)
		ffList <- lapply(1:col.n, FUN=function(j){
			res <- ff(df[,j], ...)
			if (active.closing) close(res)
			res
		})
		row.names <- rownames(df)
		col.names <- colnames(df)
	}
	res <- new("BigFfMat", cols=ffList, colNames=col.names, rowNames=row.names, colN=col.n, rowN=row.n, activeClosing=active.closing)
}

setMethod("show", "BigFfMat",
	function(object){
		cat("BigFfDf object of dimension ", object@rowN, " x ", object@colN, "\n", sep="")
		if (!is.null(object@rowNames)){
			if (object@rowN > 10) {
				rowNstr <- paste(paste(rep(object@rowNames, length.out=10), collapse=", "),", ...")
			} else {
				rowNstr <- paste(object@rowNames, collapse=", ")
			}
			cat("  row names: ", rowNstr, "\n", sep="")
		}
		if (!is.null(object@colNames)){
			if (object@colN > 10) {
				colNstr <- paste(paste(rep(object@colNames, length.out=10), collapse=", "),"...")
			} else {
				colNstr <- paste(object@colNames, collapse=", ")
			}
			cat("  col names: ", colNstr, "\n", sep="")
		}
		# show(object[,])
		invisible(NULL)	
	}
)
setMethod("nrow", signature(x="BigFfMat"),
	function(x){
		x@rowN
	}
)
setMethod("ncol", signature(x="BigFfMat"),
	function(x){
		x@colN
	}
)
#' Dimensions of BigFfMat
#'
#' @param x BigFfMat object
setMethod("dim", signature(x="BigFfMat"),
	function(x){
		c(x@rowN, x@colN)
	}
)
setMethod("rownames", signature(x="BigFfMat"),
	function(x){
		x@rowNames
	}
)
setReplaceMethod("rownames", signature(x="BigFfMat"),
	function(x, value){
		if (!is.null(value) && length(value)!=x@rowN){
				stop("Number of replacement items does not fit")
		}
		x@rowNames <- value
		invisible(x)
	}
)
setMethod("colnames", signature(x="BigFfMat"),
	function(x){
		x@colNames
	}
)
setReplaceMethod("colnames", signature(x="BigFfMat"),
	function(x, value){
		if (!is.null(value) && length(value)!=x@colN){
			stop("Number of replacement items does not fit")
		}
		x@colNames <- value
		invisible(x)
	}
)

#' Extract parts of BigFfMat
#'
#' @param x BigFfMat object
#' @param i row indices (integer, logical, character are allowed)
#' @param j column indices (integer, logical, character are allowed)
#' @param drop analogous to generic drop
setMethod("[", "BigFfMat",
	function(x, i, j, drop=TRUE){
		if (missing(i)) i <- 1:x@rowN
		if (missing(j)) j <- 1:x@colN
		if (is.logical(i)) i <- which(i)
		if (is.logical(j)) j <- which(j)
		if (is.character(i)){
			if (all(i %in% x@rowNames)){
				i <- match(i, x@rowNames)
			} else {
				stop("Invalid row names specified")
			}
		}
		if (is.character(j)){
			if (all(j %in% x@colNames)){
				j <- match(j, x@colNames)
			} else {
				stop("Invalid col names specified")
			}
		}
		if (!is.vector(i)){
			stop("Invalid row selection (non-vector)")
		}
		if (!is.vector(j)){
			stop("Invalid column selection (non-vector)")
		}
		# print(paste("get: i:",paste(i,collapse=","),"j:",paste(j,collapse=",")))
		res <- do.call("cbind", lapply(x@cols[j],FUN=function(cc){
			if (x@activeClosing) open(cc)
			res <- suppressMessages(cc[i])
			if (x@activeClosing) close(cc)
			res
		}))
		colnames(res) <- x@colNames[j]
		rownames(res) <- x@rowNames[i]
		if (drop) res <- drop(res)
		return(res)
	}
)

#' Replace parts of BigFfMat
#'
#' @param x BigFfMat object
#' @param i row indices (integer, logical, character are allowed)
#' @param j column indices (integer, logical, character are allowed)
#' @param value replacement values
setReplaceMethod("[", "BigFfMat",
	function(x, i, j, value){
		if (missing(i)) i <- 1:x@rowN
		if (missing(j)) j <- 1:x@colN
		if (is.logical(i)) i <- which(i)
		if (is.logical(j)) j <- which(j)
		if (is.character(i)){
			if (all(i %in% x@rowNames)){
				i <- match(i, x@rowNames)
			} else {
				stop("Invalid row names specified")
			}
		}
		if (is.character(j)){
			if (all(j %in% x@colNames)){
				j <- match(j, x@colNames)
			} else {
				stop("Invalid col names specified")
			}
		}
		if (!is.vector(i)){
			stop("Invalid row selection (non-vector)")
		}
		if (!is.vector(j)){
			stop("Invalid column selection (non-vector)")
		}
		# print(paste("set: i:",paste(i,collapse=","),"j:",paste(j,collapse=",")))

		if (is.vector(value)){
			if (length(value)==1){
				if (x@activeClosing) {
					for (jjj in j) open(x@cols[[jjj]])
				}
				x@cols[j] <- lapply(1:length(j), FUN=function(jj){
					ccMod <- x@cols[j][[jj]]
					ccMod[i] <- value
					return(ccMod)
				})
				if (x@activeClosing) {
					for (jjj in j) close(x@cols[[jjj]])
				}
			} else if (length(i)==length(value) && length(j)==1){
				if (x@activeClosing) open(x@cols[[j]])
				x@cols[[j]][i] <- value 
				if (x@activeClosing) close(x@cols[[j]])
			} else if (length(j)==length(value) && length(i)==1){
				if (x@activeClosing) {
					for (jjj in j) open(x@cols[[jjj]])
				}
				x@cols[j] <- lapply(1:length(value), FUN=function(jj){
					ccMod <- x@cols[j][[jj]]
					ccMod[i] <- value[jj]
					return(ccMod)
				})
				if (x@activeClosing) {
					for (jjj in j) close(x@cols[[jjj]])
				}
			} else {
				stop("Invalid specification for replacement object of type vector.")
			}
		} else {
			if (nrow(value)!=length(i) || ncol(value)!=length(j)){
				stop("Incompatible dimensions of replacement object and indices")
			}
			# x@cols[j] <- lapply(1:ncol(value), FUN=function(jj){
			# 	ccMod <- x@cols[j][[jj]]
			# 	ccMod[i] <- value[,jj]
			# 	return(ccMod)
			# })
			dummy <- lapply(1:ncol(value), FUN=function(jj){
				if (x@activeClosing) open(x@cols[j][[jj]])
				x@cols[j][[jj]][i] <<- value[,jj]
				if (x@activeClosing) close(x@cols[j][[jj]])
				return(NULL)
			})
		}
		invisible(x)
	}
)

if (!isGeneric("delete")) setGeneric("delete", function(x) standardGeneric("delete"))
setMethod("delete", signature(x="BigFfMat"),
	function(x){
		# message("removing BigFfMat object") #TODO: remove me
		for (j in 1:x@colN){
			ff::delete(x@cols[[j]])
		}
		invisible(NULL)
	}
)
# adapt vmode from ff package
setMethod("vmode", signature(x="BigFfMat"),
	function(x){
		if (ncol(x)<1) return(NA)
		return(vmode(x@cols[[1]]))
	}
)

#' save.bigFfMat
#'
#' save an \code{\linkS4class{BigFfMat}} object to disk
#'
#' @param bff \code{\linkS4class{BigFfMat}} object
#' @param file path on the disk to save to.
#' @param ...  arguments passed on to \code{ffsave}
#' @return Nothing of particular interest
#' @author Fabian Mueller
#' @aliases save.bigFfMat
#' @noRd
save.bigFfMat <- function(bff, file, ...){
	if (dir.exists(file) || file.exists(file)){
		stop("Destination directory or file already exists")
	}
	dir.create(file)
	objFn <- file.path(file,"bff.RData")
	save(bff, file=objFn)
	ee <- new.env()
	for (j in 1:bff@colN){
		cnCur <- paste0("col",j)
		ee[[cnCur]] <- bff@cols[[j]]
	}
	ffFn <- file.path(file,"bff_data")
	ffsave(list=ls(ee), envir=ee, file=ffFn, ...)
	invisible(NULL)
}

#' load.bigFfMat
#'
#' loads a saved \code{\linkS4class{BigFfMat}} object from disk
#'
#' @param path path of the saved object (a directory containing a corresponding \code{bff.RData} file and possibly \code{bff_data} files)
#' @param ...  arguments passed on to \code{ffload}
#' @return the loaded \code{\linkS4class{BigFfMat}} object
#' @author Fabian Mueller
#' @aliases load.bigFfMat
#' @noRd
load.bigFfMat <- function(path, ...){
	if (!file.exists(path)){
		stop("Loading unsuccessfull. Path does not exist")
	}
	load.env <- new.env(parent=emptyenv())
	load(file.path(path,"bff.RData"),envir=load.env)
	bff <- get("bff", load.env)
	eec <- new.env(parent=emptyenv())
	save.file <- file.path(path,"bff_data")
	ffload(save.file, envir=eec, ...)
	bff@cols <- lapply(1:bff@colN, FUN=function(j){
		cnCur <- paste0("col",j)
		get(cnCur,eec)
	})
	return(bff)
}

########################################################################################################################
## CNV.R
## created: 2013-06-11
## creator: Pavlo Lutsik, edited by Michael Scherer
## ---------------------------------------------------------------------------------------------------------------------
## Copy number variation based on HumanMethylation450 data.
########################################################################################################################

## utility functions

#' rnb.get.cv.annotations
#' 
#' This function returns reference profiles for CNV computation. Only 450k data is
#' supported in the moment.
#' 
#' @param platform The platform of the refernce data set to be returned. Acceptible
#'                  values are \code{"probes450}.
#' @return A list with three elements stored in the \code{\link{RnBeads.hg19}} package.
#'          \itemize{
#'             \item{\code{seq.info.hg19}} A Seqinfo object containing general sequence information.
#'             \item{\code{ref.intensity}} Reference intensity values used to compute CNVs
#'             \item{\code{bac.array.cpg}} CpG identifieres of the entries in \code{ref.intensity}
#'          }
#' @details The reference profiles have been obtained as the median signal intensities of twins from two twin studies:
#'    GSE85647 and GSE100940. 
#' @noRd
#' @author Pavlo Lutsik, with modifications by Michael Scherer         
rnb.get.cnv.annotations<-function(platform="probes450"){
  if(!platform %in% c("probes450")){
    logger.warning("Reference dataset is only available for HumanMethylation450 chips.")
    return(NULL)
  }
  rnb.require("RnBeads.hg19")
  ref.loc <- system.file("extdata/cnv.reference.RDS", package="RnBeads.hg19")
  if(file.exists(ref.loc)){
    cnv.reference.data <- readRDS(ref.loc)
  }else{
    logger.warning("Reference dataset for CNV estimation not available. Please update RnBeads.hg19.")
    cnv.reference.data <- NULL
  }
	return(cnv.reference.data)
}

#######################################################################################################################

#' getGLADProfiles
#' 
#' This function computes and returns GLAD profiles.
#' 
#' @param rnb.set An object of type \code{\link{RnBeadRawSet}} containing intensity information for
#'                 computing the GLAD profiles.
#' @return GLAD profiles as produced by \code{\link{daglad}}, with names corresponding
#'          to the samples of \code{rnb.set}
#' @noRd
#' @author Pavlo Lutsik, with modifications by Michael Scherer and Baris Kalem         
getGLADProfiles<-function(rnb.set,refbased=TRUE){
	
	if(!inherits(rnb.set,"RnBeadRawSet")){
		stop("RnBeadRawSet object expected")
	}
	
  annot <- annotation(rnb.set)
  target <- rnb.set@target
  if(target %in% c("probes27","probes450","probesEPIC", "probesEPICv2")){
    target <- "probes450"
  }
  cnv.reference.data<-rnb.get.cnv.annotations(target)
  if(rnb.getOption("qc.cnv.refbased") && !is.null(cnv.reference.data)){
	  matchi <- rownames(annot) %in% cnv.reference.data$bac.array.cps
	  annot<-annot[matchi,]
	  ref.int <- cnv.reference.data$ref.intensity[cnv.reference.data$bac.array.cps %in% row.names(annot)]
  
  	I<-M(rnb.set)+U(rnb.set)
  	I<-I[matchi,]
  }else{
    I<-M(rnb.set)+U(rnb.set)
    ref.int <- rowMeans(I)
  }
	
	sample.names<-samples(rnb.set)
	
	primary.data<-lapply(sample.names, function(cl){
	  log2(I[,cl]/ref.int)
	})
	names(primary.data)<-sample.names
	
	if(parallel.isEnabled()){	
  	ncores <- parallel.getNumWorkers()
  	cgh.profiles <- mclapply(primary.data, function(df){
  	  df.complete <- data.frame(
  	    Clone=row.names(annot),
  	    PosOrder=1:nrow(annot), 
  	    LogRatio=df,
  	    PosBase=annot[,"Start"],
  	    Chromosome=annot[,"Chromosome"],
  	    BAC=row.names(annot)
  	  )
  	  dummy <- capture.output(res <- as.profileCGH(df.complete, value=3))
  	  return(res)
  	}
  	                         ,mc.cores = ncores)
  	names(cgh.profiles)<-sample.names
  	glad.profiles<-mclapply(cgh.profiles, function(cl){
  	  dummy <- capture.output(res <- daglad(cl,mediancenter=TRUE,param=c(d=9)))
  	  return(res)
  	},mc.cores = ncores)
  	names(glad.profiles)<-sample.names
	}else{
	  cgh.profiles <- lapply(primary.data, function(df){
	    df.complete <- data.frame(
	      Clone=row.names(annot),
	      PosOrder=1:nrow(annot), 
	      LogRatio=df,
	      PosBase=annot[,"Start"],
	      Chromosome=annot[,"Chromosome"],
	      BAC=row.names(annot)
	    )
	    dummy <- capture.output(res <- as.profileCGH(df.complete, value=3))
	    return(res)
	  })
	  names(cgh.profiles)<-sample.names
	  glad.profiles<-lapply(cgh.profiles, function(cl){
	    dummy <- capture.output(res <- daglad(cl,mediancenter=TRUE,param=c(d=9)))
	    return(res)
	  })
	  names(glad.profiles)<-sample.names
	}
	
	glad.profiles
	
}

#######################################################################################################################

#' rnb.plot.GLAD.profile
#' 
#' This functions returns a ReportPlot object with the GLAD profile figures added.
#' 
#' @param glad.profiles GLAD profiles as produced by \code{\link{getGLADprofiles}}.
#' @param label Name of the sample to be plotted.
#' @param sample.names Character vector containing all sample names.
#' @param numeric.names Something is replaced.
#' @param ... Further arguments passed to \code{createReportPlot}
#' @return A ReportPlot object with the GLAD profile plots added.
#' @author Pavlo Lutsik, with modifications by Michael Scherer
#' @noRd
rnb.plot.GLAD.profile<-function(glad.profile, label, sample.names = NA, numeric.names = FALSE, ...){
	
	cytoband_env<-new.env()
	data(cytoband, package="GLAD", envir = cytoband_env)
	
	plot.file<-createReportPlot(paste('GLADProfilePlot',  
					ifelse(!is.na(sample.names) && numeric.names, 
							match(label, sample.names), gsub("[ |_]", ".", label)) , sep="_"), ...)
		
	#GLAD::plotProfile(glad.profile,cytoband=get("cytoband", envir = cytoband_env), Bkp=TRUE, Smoothing="Smoothing", main=paste(label),
	#                  labels=F)
		
	off(plot.file)
	return(plot.file)
}

#######################################################################################################################

#' getCGCounts
#' 
#' This function returns the CpG-wise copy number alterations for each sample.
#' 
#' @param cnv.profiles CNV (GLAD) profiles as produced by \link{getGLADProfiles}.
#' @param rnb.set An object of type \code{\link{RnBeadRawSet}}.
#' @return CNV counts for each CpG in all samples
#' @author Pavlo Lutsik, with modifications by Michael Scherer
#' @noRd
getCGCounts<-function(cnv.profiles, rnb.set){
	target <- rnb.set@target
    assembly<-rnb.set@assembly
	cnv.reference.data<-rnb.get.cnv.annotations(target)
	chrom.data <- seqlengths(rnb.get.annotation(target, assembly))
	
	si<-cnv.reference.data$seq.info.hg19
	annot<-annotation(rnb.set)
	
	results<-sapply(cnv.profiles, function(profile){
		
		chr.basic<-GRanges(seqnames=ChrNumeric(names(chrom.data)), 
				IRanges(start=rep(1,length(chrom.data)), end=chrom.data), 
				strand=(rep("*", length(chrom.data))), regid=rep("native", length(chrom.data)))#,
				#seqinfo=si)		
				
		sample.cgh<-as.data.frame(profile)
		sample.bkps<-profile$BkpInfo
		
		if(!any(is.na(sample.bkps))){			
	  	sample.ranges<-GRanges(seqnames=sample.bkps$Chromosome, 
		  		IRanges(start=sample.bkps$PosBase,width=1), strand=rep("*", nrow(sample.bkps)),
			  	#seqinfo=si,
				  regid=sample.bkps[,1])
		}else{
		  sample.ranges <- GRanges()
		}
		sample.ranges<-c(chr.basic, sample.ranges)
		sample.ranges<-disjoin(sample.ranges)
		start(sample.ranges[which(width(sample.ranges)==1)+1L])<-start(sample.ranges[which(width(sample.ranges)==1)+1L])-1
		sample.ranges<-sample.ranges[width(sample.ranges)>1]
					
		sample.prof.ranges<-GRanges(seqnames=ChrNumeric(sample.cgh$Chromosome), 
				IRanges(start=sample.cgh$PosBase, width=1), strand="*")#, seqinfo=si)
		sample.olaps<-findOverlaps(sample.prof.ranges, sample.ranges)
		
		sample.ranges <- sample.ranges[unique(subjectHits(sample.olaps))]
		mean.zonegnl<-tapply(sample.cgh[queryHits(sample.olaps),"ZoneGNL"], subjectHits(sample.olaps), mean)
		Level<-round(mean.zonegnl) # expecting 2n, can be very wrong if the sample is on average kn, k>2
		sample.counts<-cbind(as.data.frame(sample.ranges),Level)	
		
		sample.counts$chromosome <- seqnames(sample.ranges)
		sample.counts$seqnames<-NULL
		sample.counts<-sample.counts[,c(6,1,2,4,5)]			
		
		cg.ranges<-GRanges(seqnames=ChrNumeric(annot$Chromosome), 
				IRanges(start=annot$Start, width=1), strand="*")#, seqinfo=si)
		cg.olaps<-findOverlaps(cg.ranges, sample.ranges)

		cnv.vector<-sample.counts[subjectHits(cg.olaps),"Level"]
	})
	
	colnames(results)<-names(cnv.profiles)
	row.names(results)<-row.names(annot)
	results
	
}

#######################################################################################################################
#' rnb.execute.cnv
#'
#' Copy number variation calling using GLAD
#'
#' @param object \code{\linkS4class{RnBeadRawSet}} object
#' @return a \code{list} with elements \code{glad.profiles} and \code{cg.counts}
#'
#' @author Pavlo Lutsik
#' @noRd

rnb.execute.cnv<-function(object){
	
    logger.warning("CNV calling currently not supported due to dependency issues")
	#glad.profiles <- getGLADProfiles(object)
	#cg.counts <- getCGCounts(glad.profiles, object)
    glad.profiles <- NULL
    cg.counts <- NULL
	
	list(cnv.profiles=glad.profiles, cg.counts=cg.counts)
	
}

#######################################################################################################################

#' rnb.section.cnv
#'
#' Adds CNV section to quality control report
#'
#' @param report analysis report
#' @param cnv.data a \code{list} output by \code{rnb.execute.cnv}
#'
#' @return The modified report object
#'
#' @author Pavlo Lutsik
#' @noRd
rnb.section.cnv<-function(report, cnv.data){
	
	logger.start("Copy Number Variation Section")
	
	report <- rnb.add.section(report, "Copy number variation analysis", 
			"Visualization and analysis of copy number variations (CNVs) based on Infinium data.")
	
	txt <- 'CNV profile plots visualizes the results of CNV analysis using the <a href="http://bioconductor.org/packages/release/bioc/html/GLAD.html">GLAD</a> package.'
	if(rnb.getOption("qc.cnv.refbased")){
	  txt <- c(txt," \n A reference dataset was used to compute copy number gains and losses. The reference intensity values were obtained ",
	           "from a twin dataset (<a href='https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE100940'>GSE100940</a>), as the median intensity value for each position in the twins.")
	}else{
	  txt <- c(txt," \n The mean intensity value for each CpG in all samples was used as a reference to compute copy number alterations.")
	}
	rnb.add.paragraph(report, c("<b>CNV profiles:</b> ", txt))
	report<-add.profile.plots(report, cnv.data$cnv.profiles)
	logger.status("Added CNV profile plots")
	
	fname <- "cnv_per_cpg.csv"
	fname.full <- file.path(rnb.get.directory(report, "data", absolute = TRUE), fname)
	utils::write.csv(cnv.data$cg.counts, file = fname.full, na = "")
	
	txt<-c("Based on the segments with the identical copy number, one can calculate copy number for each CpG. ",
			"The table of computed copy numbers is available as a <a href=\"",
			rnb.get.directory(report, "data"), "/", fname, "\">comma-separated file</a> accompanying this report.",
			"The meanings of each entry are listed below.")
	rnb.add.paragraph(report, c("<b>Copy gains per CpG:</b> ", txt))
	exp.list <- list("<b>-1:</b> represents a copy number loss",
	                 "<b>1:</b> represents a copy number gain",
	                 "<b>2:</b> represents an amplicon",
	                 "<b>-10:</b> represents a deletion",
	                 "<b>0:</b> represents the normal state")
	rnb.add.list(report,exp.list)
	
	logger.status("Added CpG counts")
	
	logger.completed()
	return(report)
	
}
#######################################################################################################################

#' rnb.step.cnv
#'
#' Performs copy number calling from the Infinium intenstity data and adds the results to the report
#'
#' @param rnb.set	An object of type \code{\linkS4class{RnBeadRawSet}}
#' @param report  Report on quality control to contain the generated sections. This must be an object of type
#'                		\code{\linkS4class{Report}}.
#' @return The modified report.
#'
#' @author Pavlo Lutsik
#' @export
rnb.step.cnv<-function(rnb.set, report){

    logger.warning("CNV calling currently not supported due to dependency issues")
  
	#rnb.require("GLAD")

	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	
	logger.start("Copy Number Variation")
	
	cnv.data<-rnb.execute.cnv(rnb.set)
	
	#report<-rnb.section.cnv(report, cnv.data)
	
	logger.completed()
	
	report
	
}

#######################################################################################################################
#######################################################################################################################

#' add.profile.plots
#' 
#' Produces GLAD profile plots for each sample and adds them to the report.
#' 
#' @param report The \code{\link{Report}} object, to which the plots should be added.
#' @param cnv.profiles CNV (GLAD) profiles as produced by \link{getGLADProfiles}.
#' @return The modified report object.
#' @author Pavlo Lutsik, with modifications by Pavlo Lutsik
#' @noRd
add.profile.plots<-function(report, cnv.profiles){
	
	descr <- c("Profiles visualize copy number variation across the genome of Infinium profiled samples. Each point represents a CpG on ",
"the chip. Yellow is the normal state, black is a deletion, blue an amplification, red a gain and green a loss. The CpGs are annotated ",
"to their position in the genome, with the centromers of the chromosome depicted in red. Red dashed lines represent breakpoints of regions ",
"with the identical amount of DNA.")
	
	ids<-names(cnv.profiles)
	
	if(parallel.isEnabled()){
	  cplots <- mclapply(ids, function(id) {
	    rnb.plot.GLAD.profile(glad.profile=cnv.profiles[[id]], label=id, sample.names=ids, 
	                          report=report, numeric.names=TRUE, width=8, height=7, low.png=100, high.png=300, create.pdf=F)
	  },mc.cores=parallel.getNumWorkers())
	}else{
	  cplots<-lapply(ids, function(id) {
				rnb.plot.GLAD.profile(glad.profile=cnv.profiles[[id]], label=id, sample.names=ids, 
						report=report, numeric.names=TRUE, width=8, height=7, low.png=100, high.png=300, create.pdf=F)
			})
	}
	
	names(cplots)<-1:length(ids)
	
	sn<-list("Sample labels" = ids)
	names(sn[[1]])<-1:length(ids)
	
	report<-rnb.add.figure(report, description=descr, report.plots=cplots, setting.names=sn)
	report
}
#######################################################################################################################

library(argparse)
suppressPackageStartupMessages(library(RnBeads))

ap <- ArgumentParser()
ap$add_argument("-x", "--xml", action="store", help="Configuration xml file")
ap$add_argument("-s", "--rnbSet", action="store", dest="rnbSetLoc", help="Location of stored RnBSet")
ap$add_argument("-c", "--cores", action="store", type="integer", default=1, help="Number of cores used for the analysis")
cmdArgs <- ap$parse_args()
module.name <- "exploratory"

logger.start(fname=NA)
logger.status(c("...Started module:",module.name))

logger.start("Configuring Analysis")
	rnb.settings <- rnb.xml2options(cmdArgs$xml,return.full.structure=TRUE)

	report.dir <- rnb.settings$analysis.params[["dir.reports"]]
	analysis.options <- rnb.settings$options

	if ("preanalysis.script" %in% names(rnb.settings)){
		source(rnb.settings$preanalysis.script)
	} 
	## Set options
	if (length(analysis.options) != 0) {
		do.call(rnb.options, analysis.options)
	}

	if(rnb.getOption("region.subsegments")>1L){
		logger.start("Loading region subsegmentation data")
			subseg.annot.dir <- file.path(report.dir,"preprocessing_data","subsegments")
			load.region.subsegment.annotation(rnb.set,subseg.annot.dir)
		logger.completed()
	}

	logger.machine.name()

	if (cmdArgs$cores > 1) {
		parallel.setup(cmdArgs$cores)
	}

	aname <- rnb.getOption("analysis.name")
	if (!(is.null(aname) || is.na(aname) || nchar(aname) == 0)) {
		logger.info(c("Analysis Title:", aname))
	}
	ncores <- parallel.getNumWorkers()
	if (ncores == -1) {
		ncores <- 1L
	}
	logger.info(c("Number of cores:", ncores))
	rm(aname, ncores)
logger.completed()

logger.start("Loading RnBSet")
	rnb.set <- load.rnb.set(cmdArgs$rnbSetLoc)
logger.completed()

logger.start(fname=c(file.path(report.dir,paste0("analysis_",module.name,".log")),NA))

################################################################################
# main script
################################################################################

rnb.run.exploratory(rnb.set, report.dir, close.report = TRUE)

logger.status(c("...Completed module:",module.name))
quit(save='no')

########################################################################################################################
## profiles.R
## created: 2012-05-28
## creator: Yassen Assenov
## ---------------------------------------------------------------------------------------------------------------------
## Implementation of the methylation profiles step of the analysis pipeline.
########################################################################################################################

## F U N C T I O N S ###################################################################################################

#' rnb.section.sample.groups
#'
#' Creates leading section ("Sample Groups") in the report on methylation profiles.
#'
#' @param rnb.set Methylation dataset of interest.
#' @param report  Report to contain the Sample Groups section. 
#' @return List of two elements:
#'         \describe{
#'           \item{\code{"sample.inds"}}{List of traits defining subgroups in the dataset, as returned by
#'                \code{\link{rnb.sample.groups}}.}
#'           \item{\code{"report"}}{The modified report.}
#'         }
#'
#' @author Yassen Assenov
#' @noRd
rnb.section.sample.groups <- function(rnb.set, report) {
	predefined.columns <- rnb.getOption("exploratory.columns")
	sample.inds <- rnb.sample.groups(rnb.set, columns = predefined.columns)
	pheno.columns <- data.frame(
		"Trait" = names(sample.inds),
		"Number of groups" = sapply(sample.inds, length), check.names = FALSE, stringsAsFactors = FALSE)
	if (is.null(predefined.columns)) {
		if (is.null(pheno(rnb.set))) {
			txt <- "Since no phenotypic data are available, no predefined sample groups were examined."
		} else if (nrow(pheno.columns) == 0) {
			txt <- c("The table of phenotypic information contains no traits that meet the criteria for defining ",
				"sample groups.")
		} else {
			txt <- c("Traits in the table of phenotypic information were automatically selected based on criteria for ",
				"defining sample groups.")
		}
	} else {
		if (is.null(pheno(rnb.set))) {
			txt <- "Since no phenotypic data are available, no predefined sample groups were examined."
		} else if (nrow(pheno.columns) == 0) {
			txt <- c("None of the predefined traits meets the criteria for defining sample groups.")
		} else {
			txt <- c("The specified traits were tested based on criteria for defining sample groups.")
		}
	}
	
	if (nrow(pheno.columns) != 0) {
		## Display a summary table of traits
		txt <- c(txt, " The table below summarizes these traits.")
		report <- rnb.add.section(report, "Sample Groups", txt)
		rnb.add.table(report, pheno.columns, row.names = FALSE)
	} else {
		report <- rnb.add.section(report, "Sample Groups", txt)
	}
	rnb.status("Designed color mappings for probe type and CGI status")
	return(list(sample.inds = sample.inds, report = report))
}

########################################################################################################################

## Creates a density estimation plot of beta values for the specified sample and/or probe group.
##
## @param dframe        \code{data.frame} containing at least 2 variables:
##						\code{beta:} Vector of beta values of the distributions to be displayed.
##                      \code{group:} Vector of group labels for the values.
## @param annotation    Name of the annotation being visualized, in the form of a \code{character} vector of length 1.
## @param cvalues       Vector storing the color mapping to be applied based on the groups. This is passed to the
##                      function \code{\link{scale_color_manual}}.
## @param legend.margin Length, in inches, of the space reserved for the plot legend. Legend is placed to the right of
##                      the plot.
## @return The generated ggplot object.
## @author Fabian Mueller
rnb.plot.beta.density.group <- function(dframe, annotation, cvalues, legend.margin = 2.2) {
	
	if (length(cvalues) > 1) {
		pp <- ggplot(dframe, aes(x = beta, color = group)) +
			labs(x = expression(beta), y = "Density", color = annotation) +
			geom_density() + scale_color_manual(values = cvalues) +
			theme(plot.margin = unit(c(1, legend.margin, 0.5, 0.5), c("lines", "in", "lines", "lines")),
				legend.justification = c(0, 0.5), legend.position = c(1, 0.5))
	} else {
		pp <- ggplot(dframe, aes(x = beta)) + labs(x = expression(beta), y = "Density") + geom_density() +
			theme(plot.margin = unit(c(1, legend.margin, 0.5, 0.5), c("lines", "in", "lines", "lines")))
	}
	return(pp)
}

########################################################################################################################

## transform a list of disjoint integer vectors to one factor vector. The result is a vector
## of length n where the indices specified in the input list
## correspond to the level specified by the list element name
## Useful for transforming result from \code{rnb.sample.groups} to factor vectors
##
## @param ll   index list
## @param n    length of the output vector
## @return factor vector with levels corresponding to names(ll)
## @author Fabian Mueller
index.list.2.factor <- function(ll,n){
	res <- rep(NA,n)
	ggs <- names(ll)
	if(is.null(ggs)) ggs <- 1:length(ll)
	inds.names.na <- which(is.na(ggs)|(ggs==""))
	ggs[inds.names.na] <- inds.names.na
	for(i in 1:length(ll)){
		if (any(!is.na(res[ll[[i]]]))) stop("Non-disjoint index sets")
		res[ll[[i]]] <- ggs[i]
	}
	return(factor(res,levels=ggs))
}

########################################################################################################################

## Given a metrix of beta values and a vector of sample or site group memberships,
## get a data.frame that can be passed on to \code{rnb.plot.beta.density.group}. 
## NA values are ommitted.
## If subsampling, is enabled (i.e. \code{points.per.group}>0), 
## observations per group are subsampled according to the following procedure:
## Given a data.frame with K groups and number of observations
## N_1,...,N_K, target number of points per group T, the total number of points N = sum(N_1,...,N_K) is computed
## Afterwards the proportions p_k = N_k/N is computed and from each group, S_k = p_k*(K*T) observations
## are randomly selected from all observations belonging to group k.
## @param beta.matrix matrix containing the values to be plotted. in this package, mostly 
##                    beta methylation values
## @param groups      group memberships as factor vector for either the columns or rows of \code{beta.matrix}
## @param byrow		  dows the grouping corrospond to the rows of beta.matrix. If so, beta.matrix will be transposed
##					  and the rest of function is handled in complete analogy to the column case
## @param log.name    character to print as name for the logger. If NULL, logging is ommited
## @param points.per.group the targeted number of points (T) per group. Set this to a value < 1 to disable subsampling
## @return \code{data.frame} containing at least 2 variables:
##				\code{beta:} Vector of beta values of the distributions to be displayed.
##              \code{group:} Vector of group labels for the values.
## @author Fabian Mueller
get.density.dframe <- function(beta.matrix,groups,byrow=FALSE,log.name=NULL,points.per.group=0){
	if (byrow){
		beta.matrix <- t(beta.matrix)
	}
	if (length(groups)!=ncol(beta.matrix)){
		stop("Grouping does not match beta.matrix")
	}
	ggs <- levels(groups)
	n.grps <- length(ggs)
	grp.ncol <- table(groups)
	group.mats <- lapply(ggs,FUN=function(gg){
		beta.matrix[,groups==gg]
	})
	names(group.mats) <- ggs
	grp.n.obs <- sapply(group.mats,FUN=function(x){
		sum(!is.na(x))
	})
	N <- sum(grp.n.obs)
	
	if (!is.null(log.name)){
		grp.na.str <- paste(paste0(ggs,":",grp.n.obs,"/",grp.ncol*nrow(beta.matrix)),collapse="; ")
		logger.info(c("Density estimation (",log.name,"): Groupwise retained observations after missing value removal:",grp.na.str))
	}
	
	N.sub <- points.per.group*n.grps
	do.subsample <- points.per.group>0 && N.sub < N
	
	grp.beta.list <- lapply(ggs,FUN=function(gg){
		#select observations based on not missing and subsampling
		obs.sel <- which(!is.na(group.mats[[gg]]))
		if (do.subsample){
			p.sub <- N.sub/N
			target.num <- round(grp.n.obs[gg]*p.sub)
			if (target.num < grp.n.obs[gg]) {
				obs.sel <- sample(obs.sel,target.num)
			}
		}
		as.vector(group.mats[[gg]][obs.sel])
	})
	names(grp.beta.list) <- ggs
	grp.n.obs.subsample <- sapply(grp.beta.list,length)
	if (!is.null(log.name) && do.subsample){
		grp.ss.str <- paste(paste0(ggs,":",grp.n.obs.subsample,"/",grp.n.obs),collapse="; ")
		logger.info(c("Density estimation (",log.name,"): Groupwise retained observations after subsampling:",grp.ss.str))
	}
	return(data.frame(beta=unlist(grp.beta.list,use.names=FALSE),
				group=factor(rep(ggs,grp.n.obs.subsample),levels=ggs)))
}

########################################################################################################################

#' rnb.plot.betadistribution.sampleGroups
#'
#' Plots beta value distrubions given a sample grouping.
#'
#' @param beta.matrix       Beta values in the form of a non-empty \code{matrix} of type \code{double}. Rows in this
#'                          matrix must correspond to Infinium probes, and columns - to samples.
#' @param sample.group.inds Named \code{list} that contains indices for the samples contained in the groups in
#' 						    \code{beta.matrix}. The number of groups is determined by the length of the list, and its
#'                          names are used as group names.
#' @param annotation        Name of the annotation being visualized, in the form of a \code{character} vector of length 1.
#' @param log.str		    string specifying more details for the log file
#' @param points.per.group the targeted number of points per group. Set this to a value < 1 to disable subsampling. More
#' 				  information in the Details section of \code{\link{rnb.step.betadistribution}}
#' @return the plot as a \code{ggplot2} object
#'
#' @seealso rnb.plot.betadistribution.probeCategories
#' @author Fabian Mueller
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' meth.mat <- meth(rnb.set.example)
#' sample.groups <- rnb.sample.groups(rnb.set.example)[[1]]
#' rnb.plot.betadistribution.sampleGroups(meth.mat,sample.groups)
#' }
rnb.plot.betadistribution.sampleGroups <- function(beta.matrix, sample.group.inds, annotation = "Group",
		log.str=NULL,points.per.group=rnb.getOption("distribution.subsample")) {
	if (!(is.matrix(beta.matrix) && is.double(beta.matrix) && nrow(beta.matrix) * ncol(beta.matrix) != 0)) {
		stop("invalid value for beta.matrix")
	}
	## TODO: Validate sample.group.inds
	if (!(is.character(annotation) && length(annotation) == 1 && (!is.na(annotation)))) {
		stop("invalid value for annotation")
	}
	grps <- index.list.2.factor(sample.group.inds,ncol(beta.matrix))
	dframe <- get.density.dframe(beta.matrix,grps,byrow=FALSE,log.name=paste(c(annotation,log.str),collapse="--"),
				points.per.group=points.per.group)
	cvalues <- rep(rnb.getOption("colors.category"), length.out = length(sample.group.inds)) #adjust length of the color vector to the appropriate size
	rnb.plot.beta.density.group(dframe, annotation, cvalues)
}

########################################################################################################################

#' rnb.plot.betadistribution.probeCategories
#'
#' plot beta value distrubions given probe categories
#'
#' @param beta.matrix  Beta values in the form of a non-empty \code{matrix} of type \code{double}. Rows in this matrix
#'                     must correspond to Infinium probes, and columns - to samples.
#' @param probe.cat    \code{factor} vector of length \code{nrow(beta.matrix)} corresponding to the
#'                     probe categories.
#' @param annotation   Name of the annotation being visualized, in the form of a \code{character} vector of length 1.
#' @param color.legend Color legend to use in the form of a \code{character} vector with element names. The values in
#'                     this vector should encode colors. All values in \code{probe.cat} must be present in the names of
#'                     this color legend. If this parameter is \code{NULL}, a default color legend is be constructed.
#' @param log.str	   string specifying more details for the log file
#' @param points.per.group the targeted number of points per group. Set this to a value < 1 to disable subsampling. More
#' 				  information in the Details section of \code{\link{rnb.step.betadistribution}}
#' @return The plot as a \code{ggplot2} object.
#'
#' @seealso rnb.plot.betadistribution.sampleGroups
#' @author Fabian Mueller
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' meth.mat <- meth(rnb.set.example)
#' probe.types <- annotation(rnb.set.example)[, "Design"]
#' rnb.plot.betadistribution.probeCategories(meth.mat,probe.types,annotation="Infinium probe type")
#' }
rnb.plot.betadistribution.probeCategories <- function(beta.matrix, probe.cat, annotation = "Group", color.legend = NULL,
		log.str=NULL,points.per.group=rnb.getOption("distribution.subsample")) {
	if (!(is.matrix(beta.matrix) && is.double(beta.matrix) && nrow(beta.matrix) * ncol(beta.matrix) != 0)) {
		stop("invalid value for beta.matrix")
	}
	if (!(is.factor(probe.cat) && length(probe.cat)==nrow(beta.matrix))) {
		stop("invalid value for probe.cat")
	}
	if (!(is.character(annotation) && length(annotation) == 1 && (!is.na(annotation)))) {
		stop("invalid value for annotation")
	}
	if (is.null(color.legend)) {
		## Use default palette if none is provided
		color.legend <- rep(rnb.getOption("colors.category"), length.out = length(unique(probe.cat)))
	} else if (!(is.character(color.legend) && length(color.legend) != 0 && (!any(is.na(color.legend))))) {
		stop("invalid value for color.legend")
	}
	dframe <- get.density.dframe(beta.matrix,probe.cat,byrow=TRUE,log.name=paste(c(annotation,log.str),collapse="--"),
					points.per.group=points.per.group)
	rnb.plot.beta.density.group(dframe,annotation,color.legend)
}
	
########################################################################################################################

#' rnb.step.betadistribution
#'
#' Computes the distributions of beta values across various sample groups and adds a corresponding section to the
#' report.
#'
#' @param rnb.set HumanMethylation450K dataset as an object of type \code{\linkS4class{RnBSet}}.
#' @param report  Report to contain the methylation deviation section. This must be an object of type
#'                \code{\linkS4class{Report}}.
#' @param columns Optional; predefined column names (in the form of a \code{character} vector) or indices (an
#'                \code{integer} vector) in the sample annotation table. Only these columns are considered for grouping
#'                samples and defining profiles. All other columns in the phenotype table are ignored.
#' @param points.per.group the targeted number of points (T) per group. Set this to a value < 1 to disable subsampling. More
#' 				  information in the Details section
#' @return The modified report.
#' @section Details:
#' If subsampling is enabled (i.e. \code{points.per.group}>0), 
#' observations per group are subsampled according to the following procedure:
#' Given K groups and numbers of observed beta values per group
#' N_1,...,N_K, and the target number of points per group T: the total number of points N = sum(N_1,...,N_K) is computed
#' Afterwards the proportions p_k = N_k/N is computed and from each group, S_k = p_k*(K*T) observations
#' are randomly selected from all observations belonging to group k.
#'
#' @author Fabian Mueller
#' @export
rnb.step.betadistribution <- function(rnb.set, report, columns = rnb.getOption("exploratory.columns"),
		points.per.group=rnb.getOption("distribution.subsample")) {
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	## rnb.sample.groups validates the value of columns
	sample.inds <- rnb.sample.groups(rnb.set, columns = columns)
	if (rnb.getOption("analyze.sites") == FALSE) {
		return(report)
	}
	logger.start("Methylation Value Distribution Plots")
	pinfos <- get.site.and.region.types(rnb.set)[[1]]
	return(rnb.step.betadistribution.internal(rnb.set, report, sample.inds, pinfos, points.per.group))
}

rnb.step.betadistribution.internal <- function(rnb.set, report, sample.inds, pinfos,
		points.per.group=rnb.getOption("distribution.subsample")) {
#	init.plot <- function(fname) {
#		createReportPlot(fname, report, width = 7.8, height = 5.6, high.png = 200L)
#	}
	do.ggplot <- function(pp,fname) {
		res <- createReportGgPlot(pp, fname, report, width = 7.8, height = 5.6, high.png = 200L)
		res <- off(res,handle.errors=TRUE)
		return(res)
	}

	## Add subsection for beta value densities of different sample groupings
	beta.value.output <- rnb.section.beta.value.density.plot(report, rnb.set, sample.inds, pinfos, points.per.group)
	
	report <- beta.value.output$report
	pinfos <- beta.value.output$pinfos
	X <- beta.value.output$meth.matrices
	txt.site <- beta.value.output$txt.site

	## Beta value distributions for different site types
	logger.start(c("Methylation Value Distributions -", capitalize(txt.site), "Categories"))
	site.categories <- colnames(pinfos)
	if(ncol(pinfos)>0){
		names(site.categories) <- 1:length(site.categories)
	}

	#skip this section if all the site annotations are either not categorical or only have 1 category
	skip <- all(sapply(pinfos,FUN=function(x){
		!is.factor(x) || length(unique(x)) < 2
	}))
	if (skip){
		logger.info(c("Site categories are non-categorical. --> skipped"))
	} else {
		X <- meth.matrices(rnb.set)
		if (length(sample.inds) != 0) {
			sample.inds.ext <- unlist(sample.inds, recursive = FALSE, use.names = FALSE)
			names(sample.inds.ext) <- paste(unlist(lapply(sample.inds, names), use.names = FALSE),
					rep(paste("(based on ", names(sample.inds), ")", sep = ""), sapply(sample.inds, length)))
		} else {
			sample.inds.ext <- list()
		}
		sample.inds.ext <- c(list("all samples" = 1:ncol(X[[1]])), sample.inds.ext)

		figure.plots <- list()
		for (i in 1:length(sample.inds.ext)){
			bbs <- as.matrix(X[[1]][, sample.inds.ext[[i]]])
			if (parallel.isEnabled()) {
				figure.plots <- c(figure.plots, foreach(j = 1:length(site.categories)) %dopar% {
					fname <- paste("beta_density_sites", i, j, sep = "_")
#					rplot <- init.plot(fname)
					pp <- rnb.plot.betadistribution.probeCategories(bbs, pinfos[,j],
						annotation = colnames(pinfos)[j], attr(pinfos, "legend")[[j]],
						log.str=names(sample.inds.ext)[i], points.per.group=points.per.group)
#					print(pp)
#					off(rplot)
					rplot <- do.ggplot(pp,fname)
					rplot
				})
			} else {
				for (j in 1:length(site.categories)){
					fname <- paste("beta_density_sites", i, j, sep = "_")
#					rplot <- init.plot(fname)
					pp <- rnb.plot.betadistribution.probeCategories(bbs, pinfos[,j],
						annotation = colnames(pinfos)[j], attr(pinfos, "legend")[[j]],
						log.str=names(sample.inds.ext)[i], points.per.group=points.per.group)
#					print(pp)
#					off(rplot)
					rplot <- do.ggplot(pp,fname)
					figure.plots <- c(figure.plots,list(rplot))
				}
			}
		}

		txt <- c("In a similar fashion, the plot below compares the distributions of beta values in different ",
			txt.site, " types.")
		report <- rnb.add.section(report, paste("Methylation Value Densities of", capitalize(txt.site), "Categories"),
			txt, level = 2)

		groups <- names(sample.inds.ext)
		names(groups) <- 1:length(groups)
		setting.names <- list("Sample group" = groups, "category" = site.categories)
		names(setting.names)[2] <- paste(capitalize(txt.site), names(setting.names)[2])
		description <- c("Methylation value density estimation according to sample grouping and ", txt.site, " category.")
		report <- rnb.add.figure(report, description, figure.plots, setting.names)
	}

	logger.completed()
	return(report)
}

########################################################################################################################

rnb.section.beta.value.density.plot <- function(report, object, sample.inds, pinfos, points.per.group, for.nv.probes = FALSE) {
	## Beta value density plot ##
	do.ggplot <- function(pp,fname) {
		res <- createReportGgPlot(pp, fname, report, width = 7.8, height = 5.6, high.png = 200L)
		res <- off(res,handle.errors=TRUE)
		return(res)
	}
	## Keep only site annotations that define categories
	is.category <- sapply(attr(pinfos, "legend"), is.character)
	pinfos.legend <- attr(pinfos, "legend")[is.category]
	pinfos <- as.data.frame(pinfos[, is.category])
	colnames(pinfos) <- names(pinfos.legend)
	attr(pinfos, "legend") <- pinfos.legend
	rm(is.category, pinfos.legend)
	
	txt.site <- rnb.get.row.token(object)

	if (for.nv.probes) {
		X <- list("nvProbes" = rnb.get.nv.probes.matrix(object))
	} else {
		X <- meth.matrices(object)
		txt.covered <- paste(txt.site, ifelse(length(X) != 1, " and region", ""), sep = "")
		txt <- c("Methylation value distributions were assessed based on selected sample groups. This was done on ",
				txt.covered, " levels. This section contains the generated density plots.")
		report <- rnb.add.section(report, paste("Methylation Value Distributions"), txt, level = 2)
	}

	sample.inds.ext <- c(list("all samples" = list("all" = 1:ncol(X[[1]]))), sample.inds)

	## Beta value distributions for different sample groups
	logger.start(paste(ifelse(for.nv.probes, "Beta", "Methylation"), "Value Distributions - Sample Groups"))
	param.combinations <- mapply(c, rep(1:length(sample.inds.ext), each = length(X)),
		rep(1:length(X), length(sample.inds.ext)), SIMPLIFY = FALSE)
	create.plot <- function(params) {
		i <- params[1]
		j <- params[2]
		if (for.nv.probes) {
			pname <- paste("beta_density_samples_nv_probes", i, sep = "_")
		} else {
			pname <- paste("beta_density_samples", i, sep = "_")
		}
		
		if (length(X) != 1) {
			pname <- paste(pname, j, sep = "_")
		}
		logger.info(c("processing",pname))
#		rplot <- init.plot(pname)
#		print(rnb.plot.betadistribution.sampleGroups(X[[j]], sample.inds.ext[[i]], names(sample.inds.ext)[i]))
#		off(rplot)
		pp <- rnb.plot.betadistribution.sampleGroups(X[[j]], sample.inds.ext[[i]], names(sample.inds.ext)[i],
				log.str=names(X)[j],points.per.group=points.per.group)
		rplot <- do.ggplot(pp,pname)
		rnb.cleanMem()
		return(rplot)
	}
	if (parallel.isEnabled()) {
		figure.plots <- foreach(params = param.combinations,
			.export = c("logger.info", "rnb.plot.betadistribution.sampleGroups", "rnb.cleanMem")) %dopar% create.plot(params)
	} else {
		figure.plots <- lapply(param.combinations, create.plot)
	}
	if (for.nv.probes) {
		txt <- "The plots below compare the distributions of Beta values of nv probes in different sample groups."
	} else {
		txt <- "The plots below compare the distributions of Methylation values in different sample groups, as defined by the traits listed above."
	}
	report <- rnb.add.section(report, paste(ifelse(for.nv.probes, "Beta", "Methylation"), "Value Densities of Sample Groups"), txt, 
							  level = 2)
	groupings <- names(sample.inds.ext)
	names(groupings) <- 1:length(groupings)
	setting.names <- list("Sample trait" = groupings)
	if (length(X) != 1) {
		i <- length(setting.names) + 1
		setting.names[[paste(ifelse(for.nv.probes, "Beta values", "Methylation"),"of")]] <- c(rnb.get.row.token(object, plural = TRUE), names(X)[-1])
		names(setting.names[[i]]) <- 1:length(setting.names[[i]])
	}
	description <- paste(ifelse(for.nv.probes, "Beta", "Methylation"), "value density estimation according to sample grouping.")
	report <- rnb.add.figure(report, description, figure.plots, setting.names)
	logger.completed()
	return(list("report" = report, "pinfos" = pinfos, "meth.matrices" = X, "txt.site" = txt.site))
}

#######################################################################################################################

#' rnb.section.nv.probes.heatmap
#'
#' Adds a section on the nv probes to the given report.
#'
#' @param report Report to contain the new section. This must be an object of type \code{\linkS4class{Report}}.
#' @param object Methylation dataset as an object of type \code{\linkS4class{RnBeadSet}}.
#' @return The (possibly modified) report.
#'
#' @author Pavlo Lutsik, Baris Kalem
#' @noRd
rnb.section.nv.probes.heatmap <- function(report, object, sample.inds, pinfos) {
	nv.probes <- rnb.get.nv.probes.matrix(object)
	add.info <- function(stitle, ffunction, txt) {
		result <- rnb.add.section(report, stitle, txt, level = 2)
		result <- ffunction(result, nv.probes)
		rnb.status(c("Added", stitle))
		result
	}

	if (length(samples(object)) < 3) {
		txt <- "Note that the nv-probe heatmap is generated only when the dataset contains at least 3 samples."
		report <- rnb.add.section(report, "nv probes Heatmap", txt, level = 2)
	} else {
		txt <- "Heatmap of the beta values from the nv probes."
		report <- add.info("nv probes Heatmap", rnb.add.nv.heatmap, txt)
	}
	return(report)
}

rnb.section.nv.probes.beta.distribution <- function(report, object, sample.inds, pinfos) {
	points.per.group=rnb.getOption("distribution.subsample")
	report <- rnb.section.beta.value.density.plot(report, object, sample.inds,
												  pinfos, points.per.group, for.nv.probes = TRUE)$report
}

#######################################################################################################################

#' rnb.step.nv.probes.heatmap
#'
#' Computes statistics on the nv probes in the given dataset and creates a corresponding section in the report.
#'
#' @param object a \code{\linkS4class{RnBeadSet}} object
#' @param report Report to contain the new section. This must be an object of
#'               type \code{\linkS4class{Report}}.
#' @return the modified report object
#'
#' @author Pavlo Lutsik, Baris Kalem
#' @noRd
rnb.step.nv.probes.heatmap<-function(object, report, sample.inds, pinfos){
	if(!inherits(object,"RnBSet")){
		stop("Supplied object is not of the class inheriting from RnBSet")
	}
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}

	logger.start("nv Probe Heatmap Section")
	report <- rnb.section.nv.probes.heatmap(report,object, sample.inds, pinfos)
	logger.completed()

	return(report)
}

#######################################################################################################################

#' rnb.step.nv.probes.beta.distribution
#'
#' Plots beta distribution graph of the nv probes in the given dataset and creates a corresponding section in the report.
#'
#' @param object a \code{\linkS4class{RnBeadSet}} object
#' @param report Report to contain the new section. This must be an object of
#'               type \code{\linkS4class{Report}}.
#' @return the modified report object
#'
#' @author Pavlo Lutsik, Baris Kalem
#' @noRd

rnb.step.nv.probes.beta.distribution <- function(object, report, sample.inds, pinfos) {
	if(!inherits(object,"RnBSet")){
		stop("Supplied object is not of the class inheriting from RnBSet")
	}
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}

	logger.start("Plotting beta distribution of nv probes")
	report <- rnb.section.nv.probes.beta.distribution(report,object, sample.inds, pinfos)
	logger.completed()

	return(report)
}

#######################################################################################################################

rnb.add.nv.heatmap <- function(report, object){
	if (inherits(object, "RnBeadSet")) {
		object <- rnb.get.nv.probes.matrix(dataset)
	}
	if (!(is.matrix(object) && is.numeric(object) && length(object) != 0)) {
		stop("invalid value for object")
	}
	txt <- paste0("Heatmap of the nv probes. Euclidean distance and complete linkage are used for constructing the ",
		"dendrograms.")
	rplot <- rnb.plot.nv.heatmap(object, writeToFile=TRUE, report=report, width=8, height=9, low.png=100, high.png=300)
	rnb.add.figure(report, txt, rplot)
}

#######################################################################################################################

#' rnb.plot.nv.heatmap
#'
#' Heatmap of beta values from nv probes.
#'
#' @param dataset     Dataset as an object of type inheriting \code{\linkS4class{RnBeadSet}}, or a matrix of
#'                    methylation beta values.
#' @param writeToFile Flag specifying whether the output should be saved as \code{\linkS4class{ReportPlot}}.
#' @param ...         Additional named arguments passed to \code{\link{createReportPlot}}. These are used only if
#'                    \code{writeToFile} is \code{TRUE}.
#' @return If \code{writeToFile} is \code{TRUE}, plot as an object of type \code{\linkS4class{ReportPlot}}. Otherwise,
#'         there is no value returned (invisible \code{NULL}).
#'
#' @export
#' @author Pavlo Lutsik, Baris Kalem

rnb.plot.nv.heatmap <- function(dataset, writeToFile = FALSE, ...) {
	if (inherits(dataset, "RnBeadSet")) {
		dataset <- rnb.get.nv.probes.matrix(dataset)
	}
	if (!parameter.is.flag(writeToFile)) {
		stop("invalid value for writeToFile; expected TRUE or FALSE")
	}

	if (writeToFile) {
		plot.file <- createReportPlot('nvHeatmap', ...)
	}
	if (nrow(dataset) < 2 || ncol(dataset) < 2) {
		rnb.message.plot("Heatmap is not available due to insufficient data.")
	} else {
		sample.ids <- abbreviate.names(colnames(dataset))
		meth.colors <- get.methylation.color.panel()
		meth.breaks <- seq(0, 1, length.out = length(meth.colors) + 1L)

		## Trim rownames of nv probes -> assumes this format: nv-GRCh38-chr19-52213081-52213081-C-G_BC11
		nv.probe.ids <- strsplit(rownames(dataset), split = "-")
		rnb.require("biomaRt")
		mart <- useMart("ENSEMBL_MART_ENSEMBL", dataset = "hsapiens_gene_ensembl",host="https://sep2025.archive.ensembl.org")
		for (i in 1:length(nv.probe.ids)) {
			chr <- nv.probe.ids[[i]][3]
			start <- nv.probe.ids[[i]][4]
			end <- nv.probe.ids[[i]][5]
			gene.symbol <- get.genesymbol.of.coordinate(start, end, chr, mart) ## Get gene symbol of this nv probe
			rownames(dataset)[i] <- paste(c(chr, start, end, gene.symbol), collapse = "-")
		}

		suppressWarnings(heatmap.2(dataset, scale = "none", na.rm = FALSE,
				breaks = meth.breaks, col = meth.colors, trace = "none",
				margins = c(13, 7), labCol = sample.ids, labRow = FALSE,
				cexRow = 0.45, ## nv probes have long ids. Reduce font size
				density.info = "density", key.title = NA, key.xlab = expression(beta), key.ylab = "Density"))
	}
	if (writeToFile) {
		off(plot.file)
		return(plot.file)
	}
	return(invisible(NULL))
}

#######################################################################################################################

#' rnb.get.nv.probes.matrix
#' 
#' Gets the matrix with "beta" values of nv probes in the given dataset.
#' 
#' @param dataset       Microarray-based methylation dataset as an object of type inheriting \code{RnBeadSet}.
#' @param threshold.nas Threshold for removing probes (rows) and samples (columns). If more than this fraction of
#'                      \code{NA}s are present in the nv probe matrix, the corresponding row or column is removed.
#' @return Two-dimensional \code{matrix} with the values at the nv probes.
#' 
#' @author Yassen Asenov, Baris Kalem
#' @noRd
rnb.get.nv.probes.matrix <- function(dataset, threshold.nas = 1) {
	if (dataset@target == "probesEPICv2") {
		result <- meth(dataset, row.names=TRUE)
		result <- result[grep("^nv", rownames(result)), , drop = FALSE]
	} else {
		stop("invalid value for dataset")
	}
	if (length(result) == 0) {
		stop("invalid value for dataset; no SNP probes found")
	}
	
	i <- which(apply(is.na(result), 1, mean) > threshold.nas)
	if (length(i) != 0) {
		if (length(i) == nrow(result)) {
			rnb.error("Not enough nv probe data available (too many missing values per probe)")
		}
		result <- result[-i, , drop = FALSE]
		rnb.warning(paste(length(i), "probes ignored because they contain too many NAs"))
	}
	i <- which(apply(is.na(result), 2, mean) > threshold.nas)
	if (length(i) != 0) {
		if (length(i) == ncol(result)) {
			rnb.error("Not enough nv probe data available (too many missing values per sample)")
		}
		result <- result[, -i, drop = FALSE]
		rnb.warning(paste(length(i), "samples ignored because their nv probes contain too many NAs"))
	}
	return(result)
}

########################################################################################################################

#' rnb.step.intersample
#'
#' Generates one or more deviation plots for the variation step of the methylation profiles module and adds a dedicated
#' section to the given report.
#'
#' @param rnb.set Methylation dataset as an object of type \code{\linkS4class{RnBSet}}.
#' @param report  Report on methylation profiles to contain the methylation deviation section. This must be an object of
#'                type \code{\linkS4class{Report}}.
#' @param columns Optional; predefined column names (in the form of a \code{character} vector) or indices (an
#'                \code{integer} vector) in the sample annotation table. Only these columns are considered for grouping
#'                samples and defining profiles. All other columns in the phenotype table are ignored.
#' @return The modified report.
#'
#' @author Yassen Assenov
#' @noRd
rnb.step.intersample <- function(rnb.set, report, columns = rnb.getOption("exploratory.columns")) {
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	## rnb.sample.groups validates the value of columns
	sample.inds <- rnb.sample.groups(rnb.set, columns = columns)
	rinfos <- get.site.and.region.types(rnb.set)
	rnb.step.intersample.internal(rnb.set, report, sample.inds, rinfos)
}

rnb.step.intersample.internal <- function(rnb.set, report, sample.inds, rinfos) {
	X <- meth.matrices(rnb.set)
	if (length(X) == 0) {
		return(report)
	}

	## Extract sample group definitions
	logger.start("Scatter Plots of Mean Beta vs Variance")
	if (length(sample.inds) != 0) {
		logger.info(c("Sample subgroups are defined by:", paste(names(sample.inds), collapse = "; ")))
		sample.group.traits <- rep(names(sample.inds), sapply(sample.inds, length))
		sample.group.names <- unlist(lapply(sample.inds, names), use.names = FALSE)
		sample.inds <- unlist(sample.inds, recursive = FALSE, use.names = FALSE)
		names(sample.inds) <- paste(sample.group.names, " (based on ", sample.group.traits, ")", sep = "")
	} else {
		logger.info("None of the traits passes the criteria for sample subgroup definition")
		sample.group.traits <- character()
		sample.group.names <- character()
	}
	sample.inds <- c(list("all samples" = 1:ncol(X[[1]])), sample.inds)
	sample.group.traits <- c("", sample.group.traits)
	sample.group.names <- c(names(sample.inds)[1], sample.group.names)

	## Measure site and/or region variance
	sample.inds.2 <- sample.inds[sapply(sample.inds, length) >= 2]
	sites.supported <- rnb.getOption("analyze.sites")
	regions.supported <- character(0)
	if (sites.supported) {
		if (length(X) == 1) {
			txt.position <- "locus"
			txt.positions <- "locations"
		} else { # length(X) > 1
			txt.position <- "locus/region"
			txt.positions <- "locations/regions"
			regions.supported <- names(X)[-1]
		}
	} else {
		txt.position <- "region"
		txt.positions <- "regions"
		regions.supported <- names(X)
	}
	txt <- c("The variability of the methylation values is measured in two aspects: (1) intra-sample variance, ",
		"that is, differences of methylation between genomic ", txt.positions, " within the same sample, and (2) ",
		"inter-sample variance, i.e. variability in the methylation degree at a specific ", txt.position,
		" across a group of samples.")
	report <- rnb.add.section(report, "Inter-sample Variability", txt, level = 2)

	txt.site <- rnb.get.row.token(rnb.set)
	txt.sites <- rnb.get.row.token(rnb.set, plural = TRUE)
	txt <- c("The following figure shows the relationship between average methylation and methylation variability of ",
		"a ", ifelse(sites.supported, txt.site, "region"), ".")
	if (!identical(sample.inds.2, sample.inds)) {
		txt <- c(txt, " Only groups of 2 or more samples are considered here.")
	}
	rnb.add.paragraph(report, txt)

	scatter.var <- function(fprefix, beta.values, c.values, c.legends) {
		report.plots <- list()
		for (i in 1:length(sample.inds.2)) {
			x.vars <- data.frame(
				"mean" = rowMeans(beta.values[, sample.inds.2[[i]]], na.rm = TRUE),
				"var" = rowVars(beta.values[, sample.inds.2[[i]]], na.rm = TRUE))
			if (!is.null(c.values)) {
				x.vars <- cbind(x.vars, c.values)
			}
			na.rows <- is.na(x.vars[, 1]) | is.na(x.vars[, 2])
			if (any(na.rows)) {
				x.vars <- x.vars[!na.rows, ]
				logger.warning(c("Removed", sum(na.rows), "loci with missing values in group", names(sample.inds.2)[i]))
			}
			for (k in 0:length(c.legends)) {
				fname <- paste(fprefix, i, sep = "_")
				if (length(c.legends) != 0) {
					fname <- paste(fname, k, sep = "_")
				}
				rplot <- createReportPlot(fname, report, create.pdf = FALSE, high.png = 200)
				if (nrow(x.vars) == 0) {
					print(rnb.message.plot("Data not available"))
					pp <- NULL
				} else if (k == 0) {
					pp <- ggplot(x.vars, aes_string(x = "mean", y = "var"))
				} else if (c.legends[k] %in% colnames(c.values)) {
					cl <- paste("`", c.legends[k], "`", sep = "")
					pp <- ggplot(x.vars, aes_string(x = "mean", y = "var", colour = cl))
					cl <- attr(c.values, "legend")[[c.legends[k]]]
					if (is.character(cl)) {
						pp <- pp + scale_colour_manual(values = cl)
					} else { # is.integer(cl) && length(cl) == 1
						if (cl == 2L) {
							g.colors <- rnb.getOption("colors.gradient")
						} else { # cl == 3L
							g.colors <- rnb.getOption("colors.3.gradient")
						}
						pp <- pp + scale_color_gradientn(colours = g.colors, na.value = "grey50")
					}
				} else {
					print(rnb.message.plot("Annotation not available"))
					pp <- NULL
				}
				if (!is.null(pp)) {
					pp <- pp + geom_point() + labs(x = "Mean methylation", y = "Variance")
					pp <- pp + theme(legend.justification = c(0, 0.5), legend.position = c(1, 0.5),
						plot.margin = unit(c(0.5, 1.5, 0.5, 0.5), c("lines", "in", "lines", "lines")))
					print(pp)
				}
				report.plots <- c(report.plots, off(rplot))
			}
		}
		return(report.plots)
	}

	setting.names <- list()
	setting.names.common <- list("Sample group" = names(sample.inds.2))
	names(setting.names.common[[1]]) <- 1:length(sample.inds.2)
	description <- paste("Scatter plot showing the correlation betweeen %s mean methylation and the variance across",
		"a group of samples. Every point corresponds to one %s.")
	if (sites.supported) {
		lg <- names(attr(rinfos[[1]], "legend"))
		fplots <- scatter.var("scatter_meanvariance_sites", X[[1]], rinfos[[1]], lg)
		names(lg) <- 1:length(lg)
		lg <- c("0" = "nothing (all black)", lg)
		setting.names$sites <- setting.names.common
		setting.names$sites[["Point color based on"]] <- lg
		f.description <- sprintf(description, txt.site, txt.site)
		report <- rnb.add.figure(report, f.description, fplots, setting.names$sites)
		logger.status(c("Created", length(fplots), "scatter plots of methylation variance on the site level"))
		rm(lg, fplots, f.description)
	}
	if (length(regions.supported) != 0) {
		ri <- rinfos[regions.supported]
		lg <- unique(unlist(lapply(ri, function(x) { names(attr(x, "legend")) }), use.names = FALSE))
		if (is.null(lg)) { lg <- character(0) }
		fplots <- list()
		for (i in 1:length(regions.supported)) {
			fprefix <- paste("scatter_meanvariance_regions", i, sep = "_")
			fplots <- c(fplots, scatter.var(fprefix, X[[regions.supported[i]]], ri[[i]], lg))
		}
		setting.names$regions <- c(list("Regions" = regions.supported), setting.names.common)
		names(setting.names$regions[[1]]) <- 1:length(setting.names$regions[[1]])
		if (length(lg) != 0) {
			names(lg) <- 1:length(lg)
			lg <- c("0" = "nothing (all black)", lg)
			setting.names$regions[["Point color based on"]] <- lg
		}
		f.description <- sprintf(description, "region", "region")
		if (sites.supported) {
			txt <- c("In a complete analogy to the plots above, the figure below shows the relationship between ",
				"average methylation and methylation variability of a genomic region.")
			rnb.add.paragraph(report, txt)
		}
		report <- rnb.add.figure(report, f.description, fplots, setting.names$regions)
		logger.status(c("Created", length(fplots), "scatter plots of methylation variance on the region level"))
		rm(ri, lg, fplots, i, f.description)
	}
	rm(setting.names.common, description)
	logger.completed()

	## -----------------------------------------------------------------------------------------------------------------
	## Create deviation plots

	o.deviation <- rnb.getOption("exploratory.deviation.plots")
	if (!(isTRUE(o.deviation) || (is.null(o.deviation) && inherits(rnb.set, "RnBeadSet")))) {
		return(report)
	}

	logger.start("Deviation Plots")
	description <- paste0("Deviation plot of a sample group. %s are sorted in increasing order of their median ",
		"methylation%s. The horizontal axis in the plot iterates over %s, and the vertical axis measures methylation ",
		"degree. Median &beta; values are depicted by a blue curve. Grey borders mark the 5th and 95th percentiles of ",
		"&beta; values in a %s%s, ensuring that 90 percent of the observed values lie in the yellow area.<br />",
		"Relative frequencies of %s categories%s are color-coded and plotted below the horizontal ",
		"axis. Every segment in the color legend shows the distribution of %s categories ",
		"that underlie the corresponding segment in the deviation plot above it.")
	txt <- c("The figure below shows a methylation deviation plot for all samples in the dataset",
		ifelse(length(sample.inds) == 1, ".",
			", as well as other sample groups inferred from the table of phenotypic information."))
	rnb.add.paragraph(report, txt)

	setting.names <- lapply(setting.names, function(sn) {
			sn[["Sample group"]] <- names(sample.inds)
			names(sn[["Sample group"]]) <- 1:length(sample.inds)
			i <- which(names(sn) == "Point color based on")
			if (length(i) != 0) {
				names(sn)[i] <- "Color legend based on"
				sn[[i]] <- sn[[i]][-1]
			}
			return(sn)
		})
	sample.group.stats <- data.frame(
		"Loci/regions" = "",
		"Sample Group" = sample.group.names,
		"Based on Trait" = sample.group.traits,
		"Size" = sapply(sample.inds, length),
		"Variability" = as.double(NA), check.names = FALSE, stringsAsFactors = FALSE)
	sample.group.stats <- rep(list(sample.group.stats), length(X))
	report.plots <- list()
	bin.sizes <- rep.int(1L, length(X))
	for (j in 1:length(X)) {
		sample.group.stats[[j]][, "Loci/regions"] <- names(X)[j]
		for (i in 1:length(sample.inds)) {
			## Reorder sites or regions according to their median methylation
			if (length(sample.inds[[i]]) > 1) {
				bstats <- t(rowQuantiles(X[[j]][, sample.inds[[i]]], probs = c(0.05, 0.5, 0.95), na.rm = TRUE))
			} else {
				bstats <- matrix(rep(X[[j]][, sample.inds[[i]]], each = 3), nrow = 3)
			}
			site.order <- order(bstats[2, ], bstats[1, ], bstats[3, ], na.last = NA)
			bstats <- bstats[, site.order]
			sample.group.stats[[j]][i, "Variability"] <- mean(bstats[3, ] - bstats[1, ])
			cuts <- deviation.plot.beta.get.cuts(ncol(bstats))
			if (!is.null(cuts)) {
				bin.sizes[j] <- max(bin.sizes[j], max(table(cuts)))
			}
			if (j == 1 && sites.supported) {
				for (k in 1:ncol(rinfos[[1]])) {
					c.values <- rinfos[[1]][site.order, k]
					c.legend <- attr(rinfos[[1]], "legend")[[k]]
					fname <- paste("deviation_sites", i, k, sep = "_")
					rplot <- createReportPlot(fname, report, width = 8, height = 5)
					deviation.plot.beta.internal(bstats, c.values, c.legend, cuts)
					report.plots <- c(report.plots, off(rplot))
				}
				next
			}
			# length(regions.supported) != 0
			if ("Color legend based on" %in% names(setting.names$regions)) {
				knames <- setting.names$regions[["Color legend based on"]]
				kvalues <- 1:length(knames)
				ri <- rinfos[[names(X)[j]]]
			} else {
				knames <- character()
				kvalues <- 0L
				ri <- NULL
			}
			for (k in kvalues) {
				if ((!is.null(ri)) && (knames[k] %in% colnames(ri))) {
					c.values <- ri[site.order, knames[k]]
					c.legend <- attr(ri, "legend")[[knames[k]]]
				} else {
					c.values <- rep.int("1", ncol(bstats))
					c.legend <- c("1" = "white")
				}
				fname <- paste("deviation_regions", j - sites.supported, i, sep = "_")
				if (!identical(kvalues, 0L)) {
					fname <- paste(fname, k, sep = "_")
				}
				rplot <- createReportPlot(fname, report, width = 8, height = 5)
				deviation.plot.beta.internal(bstats, c.values, c.legend, cuts)
				report.plots <- c(report.plots, off(rplot))
			}
			rm(knames, kvalues, ri)
		}
		if (j == 1 && sites.supported) {
			is.binned <- (bin.sizes[1] != 1)
			f.description <- sprintf(description, capitalize(txt.sites),
				ifelse(is.binned, paste(" and are binned in groups of up to", bin.sizes[1]), ""),
				ifelse(is.binned, paste(txt.site, "groups"), txt.sites), txt.site,
				ifelse(is.binned, " (averaged over the group)", ""), txt.site,
				ifelse(is.binned, " in every group", ""), txt.site)
			report <- rnb.add.figure(report, f.description, report.plots, setting.names$sites)
			logger.status(c("Created", length(report.plots), "deviation plots on the site level"))
			report.plots <- list()
		}
	}
	if (length(report.plots) != 0) {
		if (sites.supported) {
			txt <- "In a similar fashion, the figure below shows deviation plots on the region level."
			rnb.add.paragraph(report, txt)
			bin.sizes <- bin.sizes[-1]
		}
		is.binned <- any(bin.sizes != 1)
		txt <- paste(ifelse(all(bin.sizes != 1), " and", ", and in some cases"), " are binned in groups of up to ",
			max(bin.sizes), sep = "")
		f.description <- sprintf(description, "Regions",
			ifelse(is.binned, txt, ""),
			ifelse(is.binned, "region groups", "regions"), "region",
			ifelse(is.binned, " (averaged over the group)", ""), "region",
			ifelse(is.binned, " in every group", ""), "region")
		report <- rnb.add.figure(report, f.description, report.plots, setting.names$regions)
		logger.status(c("Created", length(report.plots), "deviation plots on the region level"))
	}
	rm(report.plots, j, k, c.values, c.legend, fname, rplot, is.binned, f.description)

	sample.group.stats <- do.call(rbind, sample.group.stats)
	variabilities <- sprintf("%1.4f", sample.group.stats[["Variability"]])
	variabilities[is.na(sample.group.stats[["Variability"]])] <- as.character(NA)
	sample.group.stats[["Variability"]] <- variabilities

	## Add group statistics to the report
	txt <- c("The <i>variability</i> of a sample group is the span between 5th and 95th percentile of &beta; values , ",
		"averaged over all valid ", txt.positions, ". This amounts to a number between 0 and 1 and corresponds to the ",
		"relative area of deviation in the plots presented above. The table below lists the variabilities of the ",
		"studied sample groups.")
	rnb.add.paragraph(report, txt)
	rnb.add.table(report, sample.group.stats, row.names = FALSE)
	logger.status("Added summary table of variabilities")

	logger.completed()
	return(report)
}

########################################################################################################################
## annotations.R
## created: 2012-08-16
## creator: Yassen Assenov
## ---------------------------------------------------------------------------------------------------------------------
## Collection of helper constants and functions related to the management of probe and region annotations.
########################################################################################################################

#' RnBeads Annotation Tables
#'
#' RnBeads uses sets of annotation tables and mappings (from regions to sites) for each of the supported genomes. The
#' structures for one assembly are stored in a separate dedicated annotation package. The following annotation packages
#' are available in Bioconductor:
#' \describe{
#' 	 \item{\pkg{RnBeads.hg38}}{for \code{"hg38"}}
#'   \item{\pkg{RnBeads.hg19}}{for \code{"hg19"}}
#'   \item{\pkg{RnBeads.mm10}}{for \code{"mm10"}}
#'   \item{\pkg{RnBeads.mm9}}{for \code{"mm9"}}
#'   \item{\pkg{RnBeads.rn5}}{for \code{"rn5"}}
#' }
#'
#' @details
#' An assembly-specific scaffold is automatically loaded upon initialization of its annotation, that is, by the first
#' valid call to any of the following functions: \code{\link{rnb.get.chromosomes}},
#' \code{\link{rnb.get.annotation}}, \code{\link{rnb.set.annotation}}, \code{\link{rnb.get.mapping}},
#' \code{\link{rnb.annotation.size}}. Adding an annotation amounts to attaching its table(s) and mapping structures to
#' the scaffold.
#'
#' @docType data
#' @keywords datasets
#' @name RnBeads.data
#' @format \code{list} of four elements - \code{"regions"}, \code{"sites"}, \code{"controls"} and \code{"mappings"}.
#'         These elements are described below.
#'         \describe{
#'           \item{\code{"regions"}}{\code{list} of \code{NULL}s; the names of the elements correspond to the built-in
#'                region annotation tables. Once the default annotations are loaded, the attribute \code{"builtin"} is
#'                a \code{logical} vector storing, for each region annotation, whether it is the default (built-in) or
#'                custom.}
#'           \item{\code{"sites"}}{\code{list} of \code{NULL}s; the names of the elements correspond to the site and
#'                probe annotation tables.}
#'           \item{\code{"controls"}}{\code{list} of \code{NULL}s; the names of the elements correspond to the control
#'                probe annotation tables. The attribute \code{"sites"} is a \code{character} vector pointing to the
#'                site annotation that encompasses the respective control probes.}
#'           \item{\code{"mappings"}}{\code{list} of \code{NULL}s; the names of the elements correspond to the built-in
#'                region annotation tables.}
#'         }
#' @author Yassen Assenov
NULL

## G L O B A L S #######################################################################################################

## Environment to contain all probe, site and region annotation tables.
##
##  hg19
##      $GENOME                 character
##      $CHROMOSOMES            character
##      $regions
##          $tiling             GRangesList
##          $genes              GRangesList
##          $promoters          GRangesList
##          $cpgislands         GRangesList
##      $sites
##          $CpG                GRangesList
##          $probes450          GRangesList
##      $controls
##          $controls450        data.frame
##      $mappings
##          $tilinig
##              $CpG            list of IRanges
##              $probes450      list of IRanges
##          $genes
##              $CpG            list of IRanges
##              $probes450      list of IRanges
##          $promoters
##              $CpG            list of IRanges
##              $probes450      list of IRanges
##          $cpgislands
##              $CpG            list of IRanges
##              $probes450      list of IRanges
##      $lengths                int[ <chromosomes> , <annotations> ]
.rnb.annotations <- new.env()

## Control probe types
EPIC.CONTROL.TARGETS <- c(
	"bisulfite conversion I" = "BISULFITE CONVERSION I",
	"bisulfite conversion II" = "BISULFITE CONVERSION II",
	"extension" = "EXTENSION",
	"hybridization" = "HYBRIDIZATION",
	"negative control" = "NEGATIVE",
	"non-polymorphic" = "NON-POLYMORPHIC",
	"norm A" = "NORM_A",
	"norm C" = "NORM_C",
	"norm G" = "NORM_G",
	"norm T" = "NORM_T",
	"restoration" = "RESTORATION",
	"specificity I" = "SPECIFICITY I",
	"specificity II" = "SPECIFICITY II",
	"staining" = "STAINING",
	"target removal" = "TARGET REMOVAL")

EPICv2.CONTROL.TARGETS <- c( ## TODO: Not validated for EPIC v2 (Most likely it's compatible)
	"bisulfite conversion I" = "BISULFITE CONVERSION I",
	"bisulfite conversion II" = "BISULFITE CONVERSION II",
	"extension" = "EXTENSION",
	"hybridization" = "HYBRIDIZATION",
	"negative control" = "NEGATIVE",
	"non-polymorphic" = "NON-POLYMORPHIC",
	"norm A" = "NORM_A",
	"norm C" = "NORM_C",
	"norm G" = "NORM_G",
	"norm T" = "NORM_T",
	"restoration" = "RESTORATION",
	"specificity I" = "SPECIFICITY I",
	"specificity II" = "SPECIFICITY II",
	"staining" = "STAINING",
	"target removal" = "TARGET REMOVAL")

HM450.CONTROL.TARGETS <- c(
	"bisulfite conversion I" = "BISULFITE CONVERSION I",
	"bisulfite conversion II" = "BISULFITE CONVERSION II",
	"extension" = "EXTENSION",
	"hybridization" = "HYBRIDIZATION",
	"negative control" = "NEGATIVE",
	"non-polymorphic" = "NON-POLYMORPHIC",
	"norm A" = "NORM_A",
	"norm C" = "NORM_C",
	"norm G" = "NORM_G",
	"norm T" = "NORM_T",
	"specificity I" = "SPECIFICITY I",
	"specificity II" = "SPECIFICITY II",
	"staining" = "STAINING",
	"target removal" = "TARGET REMOVAL")

HM27.CONTROL.TARGETS<-c(
	"bisulfite conversion" = "Bisulfite conversion",
	"extension" = "Extension",
	"hybridization" = "Hybridization",
	"negative control" = "Negative",
	"SNP" = "Genotyping",
	"non-polymorphic" = "Non-Polymorphic",
	"norm Grn" = "Normalization-Green",
	"norm Red" = "Normalization-Red",
	"specificity" = "Specificity",
	"staining" = "Staining",
	"target removal" = "Target Removal",
	"pACYC174" = "pACYC174",
	"pUC19" = "pUC19",
	"phiX174" = "phiX174")

MMBC.CONTROL.TARGETS <- c(
        "bisulfite conversion I" = "BISULFITE CONVERSION I",
        "bisulfite conversion II" = "BISULFITE CONVERSION II",
        "extension" = "EXTENSION",
        "hybridization" = "HYBRIDIZATION",
        "negative control" = "NEGATIVE",
        "non-polymorphic" = "NON-POLYMORPHIC",
        "norm A" = "NORM_A",
        "norm C" = "NORM_C",
        "norm G" = "NORM_G",
        "norm T" = "NORM_T",
        "restoration" = "RESTORATION",
        "specificity I" = "SPECIFICITY I",
        "specificity II" = "SPECIFICITY II",
        "staining" = "STAINING",
        "target removal" = "TARGET REMOVAL")

## Sample-independent control probe types (subset of CONTROL.TARGETS)
CONTROL.TARGETS.SAMPLE.INDEPENDENT <- c("STAINING", "HYBRIDIZATION", "TARGET REMOVAL", "EXTENSION")

## Genotyping probes on the 27k microarray
HM27.CY3.SNP.PROBES<-c(
		"rs798149",
		"rs2959823",
		"rs2235751",
		"rs2125573",
		"rs2804694")

HM27.CY5.SNP.PROBES<-c(
		"rs1941955",
		"rs845016",
		"rs866884",
		"rs739259",
		"rs1416770",
		"rs1019916",
		"rs2521373",
		"rs10457834",
		"rs6546473",
		"rs5931272",
		"rs264581")

## F U N C T I O N S ###################################################################################################

#' rnb.fix.strand
#'
#' Converts, if necessary, the provided strand information into a factor vector with levels \code{"+"}, \code{"-"} and
#' \code{"*"}.
#'
#' @param values Vector of strand information to be converted.
#' @return The (possibly modified) strand information as a factor vector.
#'
#' @author Yassen Assenov
#' @noRd
rnb.fix.strand <- function(values) {
	if (is.factor(values) && setequal(levels(values), c("+", "-", "*"))) {
		values[is.na(values)] <- "*"
	} else {
		values <- as.character(values)
		values[is.na(values)] <- "*"
		i.positive <- values %in% c("+", "1", "+1", "F", "f", "TOP")
		i.negative <- values %in% c("-", "-1", "R", "r", "BOT")
		values[i.positive] <- "+"
		values[i.negative] <- "-"
		values[!(i.positive | i.negative)] <- "*"
		values <- factor(values, levels = c("+", "-", "*"))
	}
	return(values)
}

########################################################################################################################

#' rnb.sort.regions
#'
#' Sorts the given regions based on start and end position.
#'
#' @param x Genomic regions as an object of type \code{\link{GRanges}} or \code{\link{GRangesList}}.
#' @return Set of the same regions as x, sorted based on start and end positions.
#'
#' @author Yassen Assenov
#' @noRd
rnb.sort.regions <- function(x) {
	if (inherits(x, "GRanges")) {
		return(x[order(start(x), end(x), as.integer(strand(x))), ])
	}
	if (inherits(x, "GRangesList")) {
		return(endoapply(x, function(y) { y[order(start(y), end(y), as.integer(strand(y))), ] }))
	}
	stop("invalid value for x")
}

########################################################################################################################

#' Get chromosome sizes
#' 
#' Gets the lengths of the supported chromosomes for the given genome assembly.
#' 
#' @param assembly   Genome assembly of interest. Must be one of \code{\link{rnb.get.assemblies}}.
#' @param file.chrom Optionally, the name of the text file to save the chromosome lengths to. If this file exists, it
#'                   will be overwritten.
#' @return Invisibly, a named \code{integer} vector listing the lengths, in base pairs, of all supported chromosomes.
#' @author Yassen Assenov
#' @noRd
rnb.chromosome.lengths <- function(assembly, file.chrom = NULL) {
	a.table <- NULL
	if (exists(assembly, .rnb.annotations, inherits = FALSE)) {
		a.table <- get(assembly, .rnb.annotations, inherits = FALSE)[["sites"]]
		if (length(a.table) == 0) {
			a.table <- NULL
		} else {
			a.table <- a.table[[1]]
		}
	}
	if (is.null(a.table)) {
		a.table <- rnb.get.annotation(assembly = assembly)
	}
	c.lengths <- seqlengths(a.table)
	if (!is.null(file.chrom)) {
		write(paste0(names(c.lengths), "\t", c.lengths), file.chrom, sep = "\t")
	}
	invisible(c.lengths)
}

########################################################################################################################

#' rnb.load.bed
#'
#' Loads a BED file into a \code{data.frame} with fixed column names. The file contents is validated for structure
#' (at least 3 columns), as well as for integer values in columns 2 and 3.
#'
#' @param fname BED file to load.
#' @return \code{data.frame} with at least 3 and at most 6 columns. The column names are: \code{"Chromosome"},
#'         \code{"Start"}, \code{"End"}, \code{"id"}, \code{"score"} and \code{"strand"}. Columns after the sixth one,
#'         if present, are dropped.
#'
#' @author Yassen Assenov
#' @noRd
rnb.load.bed <- function(fname) {
	BED.COLUMNS <- c("Chromosome", "Start", "End", "id", "score", "strand")
	tbl <- tryCatch(suppressWarnings(read.delim(fname, header = FALSE, quote = "", comment.char = "#",
				stringsAsFactors = FALSE, na.strings = "")), error = function(e) { e })
	if (inherits(tbl, "error")) {
		if (grepl("cannot open", tbl, fixed = TRUE)) {
			stop("cannot open file")
		}
		stop("invalid file format")
	}
	if (ncol(tbl) < 3) {
		stop("invalid file format; expected at least 3 columns")
	}
	if (!(is.integer(tbl[[2]]) && is.integer(tbl[[3]]))) {
		stop("invalid file format; expected start and end positions in columns 2 and 3, respectively")
	}
	if (length(BED.COLUMNS) < ncol(tbl)) {
		tbl <- tbl[, 1:length(BED.COLUMNS)]
	}
	colnames(tbl) <- BED.COLUMNS[1:ncol(tbl)]
	tbl[[1]] <- as.character(tbl[[1]])
	if (4 <= ncol(tbl)) {
		tbl[[4]] <- as.character(tbl[[4]])
		if (anyDuplicated(tbl[[4]]) == 0) {
			rownames(tbl) <- tbl[[4]]
			tbl <- tbl[, -4]
		}
	}
	return(tbl)
}

########################################################################################################################

#' get.cpg.stats
#'
#' Computes CpG-related statistics for the specified regions.
#'
#' @param chrom.sequence Chromosome sequence, usually obtained from the assembly's genome definition. This must be an
#'                       object of type \code{MaskedDNAString}.
#' @param starts         \code{integer} vector of start positions for the regions of interest.
#' @param ends           \code{integer} vector of end positions for the regions of interest.
#' @return Table of statistics for the regions in the form of a \code{matrix} with the following columns:
#'         \code{"CpG"} and \code{"GC"}. The columns contain the number of CpG dinucleoties and the number of C and G
#'         bases in each region.
#'
#' @author Yassen Assenov
#' @export
get.cpg.stats <- function(chrom.sequence, starts, ends) {
	if (!(inherits(chrom.sequence, "MaskedDNAString") | inherits(chrom.sequence, "DNAString"))) {
		stop("invalid value for chrom.sequence")
	}
	if (!(is.integer(starts) && !any(is.na(starts)))) {
		stop("invalid value for starts")
	}
	if (!(is.integer(ends) && !any(is.na(ends)))) {
		stop("invalid value for ends")
	}
	chrom.regions <- suppressWarnings(Views(chrom.sequence, start = starts, end = ends))
	cbind(
		"CpG" = dinucleotideFrequency(chrom.regions)[, "CG"],
		"GC" = as.integer(rowSums(letterFrequency(chrom.regions, c("C", "G")))))
}

########################################################################################################################
#' match.chrom.names
#'
#' invariant chromosome name matching. I.e. find the occurrences of the names in the first
#' vector in the second vector, neglecting the "chr" prefix
#'
#' @param q chromosome name query
#' @param q chromosome name reference
#' @return the names from r corresponding to the entries in q
#'
#' @author Fabian Mueller
#' @noRd
match.chrom.names <- function(q, r) {
	rr <- gsub("^chr","",r)
	qq <- gsub("^chr","",q)
	res <- r[match(qq,rr)]
	names(res) <- q
	return(res)
}

#' append.cpg.stats
#'
#' Appends additional metadata columns for CpG count and GC density to the specified regions.
#'
#' @param genome.data Genome of interest.
#' @param regionlist  Genomic regions as a list of \code{GRanges} objects (or an obect of type \code{GRangesList}),
#'                    containing one set of regions per chromosome.
#' @return The modified \code{regionlist}. Two columns are appedned to the metadata of each element in this list -
#'         \code{"CpG"} and \code{"GC"}. If the metadata already contains these columns, this function appends columns
#'         with similar names.
#'
#' @author Yassen Assenov
#' @noRd
append.cpg.stats <- function(genome.data, regionlist) {
	cpg.stats <- function(chrom) {
		chromName.gd <- match.chrom.names(chrom,seqnames(genome.data))
		stats <- get.cpg.stats(genome.data[[chromName.gd]], start(regionlist[[chrom]]), end(regionlist[[chrom]]))
		result <- regionlist[[chrom]]
		mcols(result) <- IRanges::cbind(mcols(result), DataFrame(stats))
		result
	}
	regions.enriched <- suppressWarnings(
		foreach(chrom = names(regionlist), .packages = "GenomicRanges",
			.export = c("get.cpg.stats", "genome.data", "regionlist")) %dopar% cpg.stats(chrom))
	names(regions.enriched) <- names(regionlist)
	return(GRangesList(regions.enriched))
}

########################################################################################################################

#' data.frame2GRanges
#'
#' Converts a \code{data.frame} that defines genomic regions to object of type \code{GRanges}.
#'
#' @param dframe        Table defining genomic regions.
#' @param ids           Region names (identifiers) as a \code{character} vector, or \code{NULL} if no names are present.
#' @param chrom.column  Column name or index that lists the chromosome names.
#' @param start.column  Column name or index that lists the start positions of the regions.
#' @param end.column    Column name or index that lists the end positions of the regions.
#' @param strand.column Column name or index that lists the strands on which the regions are located. Set this to
#'                      \code{NULL} if this region set is not strand-specific.
#' @param assembly      Genome assembly of interest. See \code{\link{rnb.get.assemblies}} for the list of supported
#'                      genomes.
#' @param sort.result   Should the resulting table be sorted
#' @return \code{GRanges} object encapsulating all well defined regions on supported chromosomes, contained in
#'         \code{dframe}. Columns other that the ones listed as parameters in this function are included as metadata.
#'
#' @author Yassen Assenov
#' @export
data.frame2GRanges <- function(dframe, ids = rownames(dframe), chrom.column = "Chromosome", start.column = "Start",
	end.column = "End", strand.column = NULL, assembly = "hg19", sort.result = TRUE) {
	if (is.character(chrom.column)) { chrom.column <- which(colnames(dframe) == chrom.column) }
	if (is.character(start.column)) { start.column <- which(colnames(dframe) == start.column) }
	if (is.character(end.column)) { end.column <- which(colnames(dframe) == end.column) }
	if (is.character(strand.column)) { strand.column <- which(colnames(dframe) == strand.column) }

	## Process the chromosome names
	chroms <- as.character(dframe[, chrom.column])
	chroms <- paste0("chr", sub("^chr", "", chroms))
	param.list <- list()
	param.list[["seqnames"]] <- chroms

	## Process the regions
	dframe <- dframe[!(is.na(dframe[, start.column]) | is.na(dframe[, end.column])), ]
	param.list[["ranges"]] <- IRanges(start = dframe[, start.column], end = dframe[, end.column], names = ids)

	## Process the strands
	if (!is.null(strand.column)) {
		param.list[["strand"]] <- rnb.fix.strand(dframe[, strand.column])
	}

	for (cname in colnames(dframe)[-c(chrom.column, start.column, end.column, strand.column)]) {
		param.list[[cname]] <- dframe[[cname]]
	}
	if (!is.null(assembly)) {
		CHROMOSOMES <- names(.rnb.annotations[[assembly]][['CHROMOSOMES']])
		i.valid <- (chroms %in% CHROMOSOMES)
		param.list <- lapply(param.list, function(x) { x[i.valid] })
	}
	result <- do.call(GRanges, param.list)
	if (!is.null(assembly)) {
		seqlevels(result) <- CHROMOSOMES
		genome(result) <- assembly
	}
	if (sort.result){
		result <- rnb.sort.regions(result)
	}
	return(result)
}

########################################################################################################################

#' load.annotations
#'
#' Loads the specifieid annotation tables and mappings, if they are not already loaded.
#'
#' @param assembly Genome assembly regions to load.
#' @param sites    Sites and mappings to load. This parameter is considered only if \code{assembly} is not \code{NULL}.
#' @return \code{TRUE} if the required assembly (and sites) were successfully loaded; \code{FALSE} if such annotations
#'         do not exist.
#'
#' @author Yassen Assenov
#' @noRd
load.annotations <- function(assembly = NULL, sites = NULL) {
	load.f <- function(fname, pname = paste0("RnBeads.", assembly)) {
		if (pname != "RnBeads") {
			if (!suppressPackageStartupMessages(require(pname, character.only = TRUE))) {
				return(invisible(FALSE))
			}
		}
		fp <- system.file(file.path("data", fname), package=pname)
		tryCatch(load(fp, envir = .rnb.annotations),
			error = function(e) {
				stop(paste0("Internal error in ", pname, ": loading required file '", fp, "' failed"))
			}
		)
		invisible(TRUE)
	}

	if (!is.null(assembly)) {
		updated <- character()
		if (!exists(assembly, where = .rnb.annotations, inherits = F)) {
			## Add scaffold for sites, probes and control probes
			if (!load.f(paste0(assembly, ".RData"))) {
				return(FALSE)
			}
			## Load built-in regions
			if (!load.f("regions.RData")) {
				return(FALSE)
			}
#			assign(assembly, list("sites" = list(), "controls" = list(), "mappings" = list()), .rnb.annotations)
#			vnames <- c("CHROMOSOMES", "GENOME", "regions")
#			for (vname in vnames) {
#				.rnb.annotations[[assembly]][[vname]] <- get(vname, .rnb.annotations, inherits = FALSE)
#			}
#			rm(list = vnames, pos = .rnb.annotations)
			.rnb.annotations[[assembly]][["regions"]] <- .rnb.annotations[["regions"]]
			updated <- names(.rnb.annotations[[assembly]][["regions"]])
			rm(list = "regions", pos = .rnb.annotations)

			## Create matrix of elements per chromosome
			CHROMOSOMES <- names(.rnb.annotations[[assembly]][["CHROMOSOMES"]])
			s.names <- c(names(.rnb.annotations[[assembly]][["sites"]]), updated)
			.rnb.annotations[[assembly]][["lengths"]] <-
				matrix(0L, nrow = length(CHROMOSOMES), ncol = length(s.names), dimnames = list(CHROMOSOMES, s.names))
			rm(CHROMOSOMES, s.names)
		}
		if (!is.null(sites)) {
			if (sites %in% names(.rnb.annotations[[assembly]][["controls"]])) {
				c.sites <- attr(.rnb.annotations[[assembly]][["controls"]], "sites")
				sites <- c.sites[sites]
				rm(c.sites)
			}
			if (!(sites %in% names(.rnb.annotations[[assembly]][["sites"]]))) {
				return(FALSE)
			}
			if (is.null(.rnb.annotations[[assembly]][["sites"]][[sites]])) {
				## Load the required sites
				load.f(paste0(sites, ".RData"))
				sinfo <- get("sites", .rnb.annotations, inherits = FALSE)
				rm(list = "sites", pos = .rnb.annotations)
				.rnb.annotations[[assembly]][["sites"]][[sites]] <- sinfo$sites
				for (cnames in setdiff(names(sinfo), c("sites", "mappings"))) {
					.rnb.annotations[[assembly]][["controls"]][[cnames]] <- sinfo[[cnames]]
				}
				## Assign and/or create mappings
				region.bi <- attr(.rnb.annotations[[assembly]][["regions"]], "builtin")
				for (rname in names(.rnb.annotations[[assembly]][["regions"]])) {
					if (region.bi[rname]) {
						rmapping <- sinfo$mappings[[rname]]
					} else {
						rmapping <- rnb.regions2sites(.rnb.annotations[[assembly]][["regions"]][[rname]], sinfo$sites)
					}
					.rnb.annotations[[assembly]][["mappings"]][[rname]][[sites]] <- rmapping
					rm(rmapping)
				}
				suppressWarnings(rm(sinfo, region.bi, rname))
				updated <- sites
			}
		}
		if (length(updated) != 0) {
			## Update annotations lengths for the newly loaded regions and/or sites
			genome.info <- get(assembly, .rnb.annotations, inherits = FALSE)
			for (a.name in updated) {
				.rnb.annotations[[assembly]][["lengths"]][, a.name] <- 0L
				if (a.name %in% names(genome.info$regions)) {
					a.lengths <- elementNROWS(genome.info$regions[[a.name]])
				} else { # a.name %in% names(genome.info$sites)
					a.lengths <- elementNROWS(genome.info$sites[[a.name]])
				}
				.rnb.annotations[[assembly]][["lengths"]][names(a.lengths), a.name] <- a.lengths
			}
		}
	}

	return(TRUE)
}

########################################################################################################################

#' unload.annotations
#'
#' Removes all loaded annotation tables and mappings (if such exist).
#'
#' @author Yassen Assenov
#' @noRd
unload.annotations <- function() {
	rm(list = ls(envir = .rnb.annotations, all.names = TRUE), envir = .rnb.annotations)
}

########################################################################################################################

#' rnb.update.annotation.infos
#'
#' Updates or adds supporting attributes for a new region annotation.
#'
#' @param type     One-element \code{character} vector giving the name of the freshly added/updated region annotation.
#' @param assembly Genome assembly of interest. See \code{\link{rnb.get.assemblies}} for the list of supported genomes.
#'
#' @author Yassen Assenov
#' @noRd
rnb.update.annotation.infos <- function(type, assembly) {

	## Set the annotation for built-in regions
	region.builtin <- attr(.rnb.annotations[[assembly]][["regions"]], "builtin")
	if (type %in% names(region.builtin)) {
		region.builtin[type] <- FALSE
	} else {
		region.builtin <- c(region.builtin, FALSE)
		names(region.builtin)[length(region.builtin)] <- type
	}
	attr(.rnb.annotations[[assembly]][["regions"]], "builtin") <- region.builtin

	## Set the information for number of regions per chromosome
	CHROMOSOMES <- names(.rnb.annotations[[assembly]][['CHROMOSOMES']])
	a.lengths.full <- rep.int(0L, length(CHROMOSOMES))
	names(a.lengths.full) <- CHROMOSOMES
	a.lengths <- elementNROWS(.rnb.annotations[[assembly]][["regions"]][[type]])
	a.lengths.full[names(a.lengths)] <- a.lengths
	i.type <- which(colnames(.rnb.annotations[[assembly]][["lengths"]]) == type)
	if (length(i.type) == 0) {
		lengths <- cbind(.rnb.annotations[[assembly]][["lengths"]], a.lengths.full)
		colnames(lengths)[ncol(lengths)] <- type
	} else {
		lengths <- .rnb.annotations[[assembly]][["lengths"]]
		lengths[, i.type] <- a.lengths.full
	}
	.rnb.annotations[[assembly]][["lengths"]] <- lengths
}

########################################################################################################################

#' rnb.annotation.size
#'
#' Gets the size, in number of genomic elements, of the specified annotation.
#'
#' @param type     Name of annotation. Control probe annotations are not accepted.
#' @param assembly Genome assembly of interest. See \code{\link{rnb.get.assemblies}} for the list of supported genomes.
#' @return \code{integer} vector showing the number of elements the specified annotation contains per chromosome. The
#'         names of the vector are the names of \code{\link{rnb.get.chromosomes}} for the given genome assembly.
#'         Chromosomes that are not covered by the annotation have their respective value set to \code{0} (zero).
#'
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' rnb.annotation.size("probes450")
#' }
#' @seealso \code{\link{rnb.region.types}} for a list of supported region annotations
#'
#' @author Yassen Assenov
#' @export
rnb.annotation.size <- function(type = "CpG", assembly = "hg19") {
	if (!(is.character(type) && length(type) == 1 && (!is.na(type)))) {
		stop("invalid value for type")
	}
	if (!(is.character(assembly) && length(assembly) == 1 && (!is.na(assembly)))) {
		stop("invalid value for assembly")
	}
	assembly <- tolower(assembly)
	if (!load.annotations(assembly)) {
		stop("unsupported assembly")
	}
	load.annotations(assembly, type)
	if (!(type %in% colnames(.rnb.annotations[[assembly]][["lengths"]]))) {
		stop("unsupported annotation type")
	}
	.rnb.annotations[[assembly]][["lengths"]][, type]
}

########################################################################################################################

#' rnb.annotation2data.frame
#'
#' Transform the specified site, probe or region annotation to \code{data.frame}.
#'
#' @param annotation.table Annotation in the form of non-empty \code{GRangesList} object, as returned by
#'                         \code{\link{rnb.get.annotation}}.
#' @param add.names        Flag indicating if element names should be extracted and returned also as a column named
#'                         \code{"ID"} in the resulting \code{data.frame}. Note that element names, if present, are
#'                         set to be the row names of the table.
#' @return Annotation in the form of a single \code{data.frame}. The columns in this table include, among other,
#'         \code{"Chromosome"}, \code{"Start"} and \code{"End"}.
#'
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' head(rnb.annotation2data.frame(rnb.get.annotation("probes450")))
#' }
#' @author Yassen Assenov
#' @export
rnb.annotation2data.frame <- function(annotation.table, add.names = TRUE) {
	if (!(inherits(annotation.table, "GRangesList") && length(annotation.table) != 0)) {
		stop("invalid value for annotation.table")
	}
	if (!parameter.is.flag(add.names)) {
		stop("invalid value for add.names")
	}

	## Convert the GRangesList object to a data.frame
	el.names <- do.call("c", lapply(annotation.table, names))
	if (is.null(el.names) || anyDuplicated(el.names) != 0) {
		result <- GenomicRanges::as.data.frame(annotation.table, row.names = NULL)
		el.names <- NULL
	} else {
		result <- GenomicRanges::as.data.frame(annotation.table)
		rownames(result) <- el.names
	}
	if (ncol(result) > 2 && identical(colnames(result)[1:2], c("group", "group_name"))) {
		result <- result[, c(-1, -2), drop = FALSE]
	}

	## Adjust column and row names
	c.names <- colnames(mcols(annotation.table[[1]]))
	c.names <- c(colnames(result)[1:(ncol(result) - length(c.names))], c.names)
	c.names[c.names == "CGI.Relation"] <- "CGI Relation"
	c.names[c.names == "strand"] <- "Strand"
	c.names[c.names == "start"] <- "Start"
	c.names[c.names == "end"] <- "End"
	c.names[c.names == "seqnames"] <- "Chromosome"
	c.names[c.names == "element"] <- "ID"
	colnames(result) <- c.names
	if (add.names && (!is.null(el.names))) {
		result[["ID"]] <- el.names
		i <- which(c.names == "width")
	} else {
		i <- which(c.names %in% c("ID", "width"))
	}
	if (length(i) != 0) {
		result <- result[, -i]
	}
	return(result)
}

########################################################################################################################

#' rnb.get.assemblies
#'
#' Gets the supported genome assemblies.
#'
#' @return All supported genome assemblies in the form of a \code{character} vector. These are \code{"hg19"},
#'         \code{"mm10"}, \code{"mm9"} and \code{"rn5"}.
#'
#' @examples
#' \donttest{
#' "hg19" %in% rnb.get.assemblies()
#' }
#' @author Yassen Assenov
#' @export
rnb.get.assemblies <- function() {
	package.regex <- "^RnBeads\\.(.+)$"
	pnames <- grep(package.regex, rownames(installed.packages(noCache = TRUE, fields = character())), value = TRUE)
	sort(sub(package.regex, "\\1", pnames))
}

########################################################################################################################

#' rnb.get.annotation
#'
#' Extracts the requested annotation for the given genome.
#'
#' @param type     Name of annotation.
#' @param assembly Genome assembly of interest. See \code{\link{rnb.get.assemblies}} for the list of supported genomes.
#' @return Probe, site or region annotation table. If the specified type refers to control probes, the returned value is
#'         a \code{data.frame} listing all respective control probes. Otherwise, this function returns an object of type
#'         \code{\link{GRangesList}} - a list of consistent \code{\link{GRanges}} objects, one per chromosome.
#'
#' @details
#' When the returned value is of type \code{GRangesList}, it defines the genomic positions of the requested sites,
#' probes or regions. Identifiers, if present, can be obtained using the \code{names} method. Strand information is also
#' included when applicable. Any additional annotation is stored as metadata in the respective \code{GRanges} objects.
#'
#' @seealso \code{\link{rnb.set.annotation}} for adding annotation;
#'   \code{\link{rnb.region.types}} for all loaded region types in a genome assembly
#' @author Fabian Mueller
#' @export
#' @examples
#' \donttest{
#' rnb.get.annotation("promoters")
#' }
rnb.get.annotation <- function(type = "CpG", assembly = "hg19") {
	if (type == "probesEPICv2" || type == "controlsEPICv2") {
		assembly = "hg38"
	}

	if (!(is.character(type) && length(type) == 1 && (!is.na(type)))) {
		stop("invalid value for type")
	}
	if (!(is.character(assembly) && length(assembly) == 1 && (!is.na(assembly)))) {
		stop("invalid value for assembly")
	}
	assembly <- tolower(assembly)
	if (!load.annotations(assembly)) {
		stop("unsupported assembly")
	}
	load.annotations(assembly, type)
	if (type %in% names(.rnb.annotations[[assembly]][["controls"]])) {
		result <- .rnb.annotations[[assembly]][["controls"]][[type]]
	} else if (type %in% names(.rnb.annotations[[assembly]][["sites"]])) {
		result <- .rnb.annotations[[assembly]][["sites"]][[type]]
	} else if (type %in% names(.rnb.annotations[[assembly]][["regions"]])) {
		result <- .rnb.annotations[[assembly]][["regions"]][[type]]
	} else {
		stop("unsupported annotation type")
	}
	return(result)
}

########################################################################################################################

#' rnb.set.annotation
#'
#' Adds or replaces a region annotation table.
#'
#' @param type        One-element \code{character} vector giving the name of the annotation. If this region type is
#'                    already available, it will be overwritten for the current session. The type cannot be one of
#'                    \code{"CpG"}, \code{"probes450"} or \code{"controls450"}, because these names are reserved for the
#'                    annotation tables of CpG dinucleotides, and Infinium methylation and control probes, respectively.
#' @param regions     BED file defining regions (see \emph{Details}). Alternatively, the value of this parameter can be
#'                    a table of genomic regions in the form of a \code{\link{data.frame}}, containing at least the
#'                    following three columns - \code{"Chromosome"}, \code{"Start"} and \code{"End"} (notice the upper
#'                    case). The \code{"chromosome"} column must be a \code{character} or \code{factor} vector that
#'                    lists chromosome names. The \code{"start"} and \code{"end"} columns are expected to contain
#'                    genomic positions as \code{integer}s. The row names of this \code{data.frame} are used as region
#'                    identifiers.
#' @param description Optional; short description in the form of a non-empty \code{character} vector. The elements in
#'                    this vector are concatenated without a separator to form the description of the annotation.
#' @param assembly    Genome assembly of interest. See \code{\link{rnb.get.assemblies}} for the list of supported
#'                    genomes.
#' @return Invisibly, \code{TRUE} if an existing annotation was replaced and \code{FALSE} otherwise.
#'
#' @details
#' In case the parameter \code{regions} specifies an existing BED file, regions are loaded from this file. The number of
#' columns defined must be at least 3. Columns after the sixth one, if present, are dropped. The columns are given the
#' following names: \code{"chromosome"}, \code{"start"}, \code{"end"}, \code{"id"}, \code{"score"} and \code{"strand"}.
#'
#' The annotation tables in \pkg{RnBeads} focus on chromosomes \code{"chr1"}, \code{"chr2"}, ..., \code{"chr22"},
#' \code{"chrX"} and \code{"chrY"}. Regions on other chromosomes are ignored. This function also recognizes the
#' convention of chromosome names such as \code{"1"}, adopted, for example, by \href{http://www.ensembl.org/}{Ensembl}.
#' Apart from this, the region definition table is not examined in details by this function; therefore, regions located
#' on unsupported chromosomes or having invalid (e.g. negative) genomic coordinates are simply not mapped to any sites
#' or probes.
#'
#' @examples
#' \donttest{
#' my.regions <- data.frame(
#'     chromosome = c("chr1", "chr1"),
#'     start = c(49242278L, 49242372L),
#'     end = c(49242590L, 49242810L),
#'     rownames = c("BEND5E1", "CpG:38"))
#' txt <- "First exon of the BEND5 gene and an overlapping CpG island."
#' rnb.set.annotation("my regions", my.regions, txt)
#' }
#' @seealso \code{\link{rnb.get.annotation}} for extracting annotation;
#'   \code{\link{rnb.region.types}} for all loaded region types in a genome assembly
#' @author Yassen Assenov
#' @export
rnb.set.annotation <- function(type, regions, description = NULL, assembly = "hg19") {
	if (!(is.character(type) && length(type) == 1 && (!is.na(type)))) {
		stop("invalid value for type")
	}
	if (!(is.character(assembly) && length(assembly) == 1 && (!is.na(assembly)))) {
		stop("invalid value for assembly")
	}
	if (!is.data.frame(regions)) {
		if (!(is.character(regions) && length(regions) == 1 && (!is.na(regions)))) {
			stop("invalid value for regions")
		}
		if (!file.exists(regions)) {
			stop("invalid value for regions; file does not exist")
		}
		regions <- rnb.load.bed(regions)
	}
	if (!(ncol(regions) >= 3 && all(c("Chromosome", "Start", "End") %in% colnames(regions)) &&
			(is.character(regions[["Chromosome"]]) || is.factor(regions[["Chromosome"]])) &&
			is.numeric(regions[["Start"]]) && is.numeric(regions[["End"]]))) {
		stop("invalid format for regions")
	}
	if (!is.null(description)) {
		if (!(is.character(description) && length(description) != 0)) {
			stop("invalid value for description")
		}
		description <- paste(description, collapse = "")
	}
	assembly <- tolower(assembly)
	if (!load.annotations(assembly)) {
		stop("unsupported assembly")
	}
	s.names <- c(names(.rnb.annotations[[assembly]][["controls"]]), names(.rnb.annotations[[assembly]][["sites"]]))
	if (type %in% s.names) {
		stop("invalid value for type; specified name is not allowed")
	}

	## Construct list of region annotation tables with sorted coordinates, one per chromosome
	strand.column <- which(colnames(regions) == "Strand")
	if (length(strand.column) == 0) {
		strand.column <- NULL
	}
	regions <- data.frame2GRanges(regions, strand.column = strand.column, assembly = assembly)
	regions <- GenomicRanges::split(regions, seqnames(regions))
	attr(regions, "description") <- description

	## Create mappings from the regions to all defined sites
	sites <- .rnb.annotations[[assembly]][["sites"]]
	sites <- sites[!sapply(sites, is.null)]
	mappings <- lapply(sites, function(x) { rnb.regions2sites(regions, sites = x) })

	## Add the region annotation table to the environment
	annotation.exists <- isTRUE(type %in% names(.rnb.annotations[[assembly]][["regions"]]))
	.rnb.annotations[[assembly]][["regions"]][[type]] <- regions
	## Add mappings to the environment
	.rnb.annotations[[assembly]][["mappings"]][[type]] <- mappings
	## Set annotation attributes and sizes (lengths)
	rnb.update.annotation.infos(type, assembly)
	return(invisible(annotation.exists))
}

########################################################################################################################

#' rnb.set.annotation.and.cpg.stats
#'
#' wrapper for \code{\link{rnb.set.annotation}} to accept the region format as output by \code{annotation(rnb.set)}.
#' Additionally, CpG statistics are added to the annotation.
#' @param regions a data.frame handled similarly as by \code{\link{rnb.set.annotation}} with the exception that
#'                 the genomic location columns should be specified using upper case first letters
#' @param type,description,assembly Parameters handled exactly as in \code{\link{rnb.set.annotation}}
#' @return Invisibly, \code{TRUE} if an existing annotation was replaced and \code{FALSE} otherwise.
#'
#' @seealso \code{\link{rnb.set.annotation}}
#' @author Fabian Mueller
#' @export
rnb.set.annotation.and.cpg.stats <- function(type, regions, description = NULL, assembly = "hg19"){
	## FIXME: Incorporate this function as a parameter in rnb.set.annotation
	GENOME <- .rnb.annotations[[assembly]][['GENOME']]
	rnb.require(GENOME)
	genome.data <- get(GENOME)
	regs.gr <- data.frame2GRanges(regions, chrom.column = "Chromosome", start.column = "Start",
		end.column = "End", strand.column = "Strand", assembly = assembly)
	regs.grl <- GenomicRanges::split(regs.gr, seqnames(regs.gr))

	regs.grl <- append.cpg.stats(genome.data, regs.grl)
	regs.gr <- BiocGenerics::unlist(regs.grl,use.names=FALSE)

	regs.df <- GenomicRanges::as.data.frame(regs.gr)
	colnames(regs.df)[colnames(regs.df)=="seqnames"] <- "Chromosome"
	colnames(regs.df)[colnames(regs.df)=="start"] <- "Start"
	colnames(regs.df)[colnames(regs.df)=="end"] <- "End"
	regs.df$width <- NULL

	rnb.set.annotation(type=type, regions=regs.df, description=description, assembly = assembly)
}

########################################################################################################################

#' rnb.save.annotation
#'
#' Saves the specified region annotation table and its accompanying data structures to a binary file.
#'
#' @param fname    One-element \code{character} vector giving the name of the file to contain the annotation data. If
#'                 this file already exists, it will be overwritten.
#' @param type     One-element \code{character} vector giving the name of the region annotation.
#' @param assembly Genome assembly of interest. See \code{\link{rnb.get.assemblies}} for the list of supported genomes.
#' @return \code{TRUE}, invisibly.
#'
#' @details
#' This function is used in combination with \code{\link{rnb.load.annotation}} to enable fast reloading of custom region
#' annotations. If can also be used to save a build-in region annotation (e.g. before overwriting it) but not site or
#' control probe annotations.
#'
#' @seealso \code{\link{rnb.load.annotation}} for loading a saved annotation
#' @author Yassen Assenov
#' @export
rnb.save.annotation <- function(fname, type, assembly = "hg19") {
	if (!(is.character(fname) && length(fname) == 1 && (!is.na(fname)))) {
		stop("invalid value for fname")
	}
	if (!(is.character(type) && length(type) == 1 && (!is.na(type)))) {
		stop("invalid value for type")
	}
	if (!(is.character(assembly) && length(assembly) == 1 && (!is.na(assembly)))) {
		stop("invalid value for assembly")
	}
	assembly <- tolower(assembly)
	if (!load.annotations(assembly)) {
		stop("unsupported assembly")
	}
	if (!(type %in% names(.rnb.annotations[[assembly]][["regions"]]))) {
		stop("unsupported annotation type")
	}

	regions <- .rnb.annotations[[assembly]][["regions"]][[type]]
	mappings <- .rnb.annotations[[assembly]][["mappings"]][[type]]
	tryCatch(save(assembly, regions, mappings, file = fname, compression_level = 9L),
		error = function(er) { stop(paste("unable to save objects to", fname)) })
	return(invisible(TRUE))
}

########################################################################################################################

#' rnb.export.annotation
#'
#' Export the annotation to a defined format (currently only bed is supported
#'
#' @param fname    One-element \code{character} vector giving the name of the file to contain the annotation data. If
#'                 this file already exists, it will be overwritten.
#' @param type     One-element \code{character} vector giving the name of the region annotation.
#' @param assembly Genome assembly of interest. See \code{\link{rnb.get.assemblies}} for the list of supported genomes.
#' @param format   Output format. currently only \code{"bed"} is supported.
#' @return \code{TRUE}, invisibly.
#'
#' @author Fabian Mueller
#' @export
#' @examples
#' \donttest{
#' rnb.export.annotation(tempfile(pattern="promoters",fileext=".bed"),"promoters")
#' }
rnb.export.annotation <- function(fname, type, assembly = "hg19", format = "bed") {
	if (!(is.character(fname) && length(fname) == 1 && (!is.na(fname)))) {
		stop("invalid value for fname")
	}
	if (!(is.character(type) && length(type) == 1 && (!is.na(type)))) {
		stop("invalid value for type")
	}
	if (!(is.character(format) && length(format) == 1 && (!is.na(format)) && format=="bed")) {
		stop("invalid value for format")
	}
	assembly <- tolower(assembly)
	if (!load.annotations(assembly)) {
		stop("unsupported assembly")
	}
	if (!(type %in% c("CpG",names(.rnb.annotations[[assembly]][["regions"]])))) {
		stop("unsupported annotation type")
	}
	aa <- rnb.get.annotation(type,assembly)
	names(aa) <- NULL
	aa <- BiocGenerics::unlist(aa)
	names(aa) <- NULL
	aa.df <- GenomicRanges::as.data.frame(aa)
	colnames(aa.df)[colnames(aa.df)=="seqnames"] <- "chrom"
	aa.df$width <- NULL
	ids <- rownames(aa.df)
	if (length(ids)!=nrow(aa.df)){
		ids <- 1:nrow(aa.df)
	}
	if (format=="bed"){
		tab.2.export <- data.frame(chrom=aa.df$chrom,start=aa.df$start,end=aa.df$end,strand=aa.df$strand,id=ids)
		write.table(format(tab.2.export,scientific=FALSE),file=fname,
				quote=FALSE,sep="\t",row.names=FALSE,col.names=TRUE,na=".")
	}
	return(invisible(TRUE))
}

########################################################################################################################

#' rnb.export.all.annotation
#'
#' Wrapper for exporting all annotation sets
#'
#' @param out.dir  The directory to write the files to
#' @param types    One-element \code{character} vector giving the name of the region annotation.
#' @param assembly Genome assembly of interest. See \code{\link{rnb.get.assemblies}} for the list of supported genomes.
#' @param format   output format. currently only \code{"bed"} is supported.
#' @return \code{TRUE}, invisibly.
#'
#' @author Fabian Mueller
#' @export
#' @examples
#' \donttest{
#' logger.start(fname=NA)
#' rnb.export.all.annotation(tempdir(),c("genes","promoters"))
#' }
rnb.export.all.annotation <- function(out.dir, types=c("CpG",rnb.region.types(assembly)), assembly = "hg19", format = "bed") {
	if (!(is.character(out.dir) && length(out.dir) == 1 && (!is.na(out.dir)))) {
		stop("invalid value for fname")
	}
	if (!(is.character(types) && (!any(is.na(types))))) {
		stop("invalid value for type")
	}
	if (!(is.character(format) && length(format) == 1 && (!is.na(format)) && format=="bed")) {
		stop("invalid value for format")
	}
	for (tt in types){
		if (logger.isinitialized()) {
			logger.start(c("Exporting Region Type",tt))
		}
		rnb.export.annotation(file.path(out.dir,paste(assembly,tt,format,sep=".")), tt, assembly = assembly, format = "bed")
		if (logger.isinitialized()) {
			logger.completed()
		}
	}
	return(invisible(TRUE))
}

########################################################################################################################

#' rnb.load.annotation
#'
#' Loads a previously saved custom region annotation from a binary (RData) file.
#'
#' @param fname One-element \code{character} vector giving the name of the file that contains the annotation data.
#' @param type  One-element \code{character} vector giving the name of the region annotation. If this annotation
#'              is already available, it will be overwritten for the current session.
#' @return Invisibly, \code{TRUE} if the annotation was loaded successfully; an error message if the objects in the
#'         given file do not encode an annotation.
#'
#' @details
#' If the region annotation cannot be loaded from the specified location, this function exits with an error message in
#' the form \code{"unable to load object from ..."}. This could happen, for example, when \code{fname} does not refer to
#' a valid RData file, or the file cannot be accessed due to security restrictions.
#'
#' If the file is loaded in the current session, but no annotation was added, the function returns invisibly one of the
#' following short failure messages:
#' \describe{
#'   \item{\code{"invalid format"}}{The RData file does not store exactly the following three objects - \code{assembly},
#'        \code{regions}, and \code{mapping}, or they are not of the expected type.}
#'   \item{\code{"unsupported assembly"}}{The specified assembly is unknown.}
#'   \item{\code{"invalid format of regions"}}{The specified region annotation table is invalid.}
#'   \item{\code{"invalid format of mappings"}}{The specified region mapping tables are invalid.}
#' }
#'
#' @seealso \code{\link{rnb.save.annotation}} for saving annotation to a binary file; \code{\link{rnb.set.annotation}}
#'          for loading an annotation from a BED file.
#' @author Yassen Assenov
#' @export
rnb.load.annotation <- function(fname, type) {
	if (!(is.character(fname) && length(fname) == 1 && (!is.na(fname)))) {
		stop("invalid value for fname")
	}
	if (!(is.character(type) && length(type) == 1 && (!is.na(type)))) {
		stop("invalid value for type")
	}
	result <- tryCatch(load(fname), error = function(err) { return(NULL) })
	if (is.null(result)) {
		stop(paste("unable to load object from", fname))
	}
	if (!setequal(c("assembly", "regions", "mappings"), result)) {
		return(invisible("invalid format"))
	}
	if (!(is.character(assembly) && length(assembly) == 1 && (!is.na(assembly)))) {
		return(invisible("invalid format"))
	}
	if (!load.annotations(assembly)) {
		return(invisible("unsupported assembly"))
	}
	## TODO: validate regions and validate mappings

	## Compare currently loaded sites with the ones avaialable in the mappings
	site.names.all <- names(.rnb.annotations[[assembly]][["sites"]])
	site.names.mapped <- names(mappings)
	if (!all(site.names.mapped %in% site.names.all)) {
		return(invisible("invalid format of mappings"))
	}
	## Load all unloaded sites
	## TODO: Would it be better to simply ignore the unloaded sites?
	sites.loaded <- .rnb.annotations[[assembly]][["sites"]]
	sites.loaded <- sites.loaded[!sapply(sites.loaded, is.null)]
	site.names.loaded <- names(sites.loaded)
	for (sname in setdiff(site.names.mapped, site.names.loaded)) {
		load.annotations(assembly, sname)
	}
	## Construct additional mappings if necessary
	for (sname in setdiff(site.names.loaded, site.names.mapped)) {
		mappings[[sname]] <- rnb.regions2sites(regions, sites = sites.loaded[[sname]])
	}
	## Put the objects in the .rnb.annotation environment
	sites.all <- .rnb.annotations[[assembly]][["sites"]]
	site.names.loaded <- names(sites.all[!sapply(sites.all, is.null)])
	.rnb.annotations[[assembly]][["regions"]][[type]] <- regions
	.rnb.annotations[[assembly]][["mappings"]][[type]] <- mappings[site.names.loaded]
	rnb.update.annotation.infos(type, assembly)
	invisible(TRUE)
}

########################################################################################################################

#' rnb.load.annotation.from.db
#'
#' Loads a previously region annotation from the RnBeads resource database
#'
#' @param types  One-element \code{character} vector giving the name of the region annotation. If this annotation
#'              is already available, it will be overwritten for the current session.
#' @param assembly Genome assembly of interest. See \code{\link{rnb.get.assemblies}} for the list of supported genomes.
#' @return Invisibly, \code{TRUE} if the annotation was loaded successfully; an error message if the objects in the
#'         given file do not encode an annotation.
#'
#' @details
#' This function checks whether a region annotation is present in the RnBeads resources,
#' downloads the corresponding annotation file(s) from the and then runs \code{\link{rnb.load.annotation}}
#' to import the annotation.
#' 
#' @examples
#' \donttest{
#' rnb.region.types() 
#' rnb.load.annotation.from.db(c("tiling1kb", "dynamicMethZiller2013"))
#' rnb.region.types()
#' }
#' 
#' @seealso \code{\link{rnb.load.annotation}} for loading annotation from a binary file
#' @author Fabian Mueller
#' @export
rnb.load.annotation.from.db <- function(types, assembly="hg19") {
	# db.url <- "http://rnbeads.mpi-inf.mpg.de/publication/data/regiondb"
	db.url <- "https://rnbeads.org/materials/data/regiondb"
	if (!(is.character(types) && length(types) > 0 && (!any(is.na(types))))) {
		stop("invalid value for types")
	}
	if (!(is.character(assembly) && length(assembly) == 1 && (!is.na(assembly)))) {
		stop("invalid value for assembly")
	}
	for (tt in types){
		fname <- paste0("annotation_", assembly, "_", tt, ".RData")
		tmpFn <- tempfile(fileext=".RData")
		success <- tryCatch(
			download.file(paste(db.url, fname, sep="/"), destfile=tmpFn, mode = "wb"),
			error = function(err) { return(-1) }
		)
		if (success != 0){
			logger.error(c("Failed to download region set annotation:", tt))
		}
		rnb.load.annotation(tmpFn, tt)
		unlink(tmpFn, recursive=FALSE) # remove temp files
	}
	invisible(TRUE)
}

########################################################################################################################

#' rnb.remove.annotation
#'
#' Deletes a region annotation table. Use this function with caution; its operation cannot be undone.
#'
#' @param type     One-element \code{character} vector giving the name of the region annotation.
#' @param assembly Genome assembly of interest. See \code{\link{rnb.get.assemblies}} for the list of supported genomes.
#' @return Invisibly, \code{TRUE} if the annotation has been successfully deleted, or \code{FALSE} if the specified
#'         region type is not supported.
#'
#' @examples
#' \donttest{
#' t.regions <- rnb.get.annotation("tiling")
#' rnb.remove.annotation("tiling")
#' }
#'
#' @seealso \code{\link{rnb.get.annotation}}, \code{\link{rnb.region.types}}
#' @author Fabian Mueller
#' @export
rnb.remove.annotation <- function(type, assembly = "hg19") {
	if (!(is.character(type) && length(type) == 1 && (!is.na(type)))) {
		stop("invalid value for type")
	}
	if (!(is.character(assembly) && length(assembly) == 1 && (!is.na(assembly)))) {
		stop("invalid value for assembly")
	}
	assembly <- tolower(assembly)
	if (!load.annotations(assembly)) {
		stop("unsupported assembly")
	}
	if (!(type %in% names(.rnb.annotations[[assembly]][["regions"]]))) {
		warning("unsupported annotation type")
		return(invisible(FALSE))
	}
	.rnb.annotations[[assembly]][["regions"]][[type]]  <- NULL
	.rnb.annotations[[assembly]][["mappings"]][[type]] <- NULL
	i.type <- which(colnames(.rnb.annotations[[assembly]][["lengths"]]) == type)
	.rnb.annotations[[assembly]][["lengths"]] <- .rnb.annotations[[assembly]][["lengths"]][, -i.type]
	invisible(TRUE)
}

########################################################################################################################

#' rnb.region.types
#'
#' Gets the supported region annotations for a given genome assembly.
#'
#' @param assembly Genome assembly of interest. See \code{\link{rnb.get.assemblies}} for the list of supported genomes.
#' @return Region types supported by \pkg{RnBeads} in the form of a \code{character} vector. The built-in ones are
#'         \code{"cpgislands"}, \code{"genes"}, \code{"promoters"} and \code{"tiling"}. The names of all custom region
#'         definitions are also included in the returned vector.
#'
#' @examples
#' \donttest{
#' "promoters" %in% rnb.region.types() # TRUE
#' }
#'
#' @seealso \code{\link{rnb.get.annotation}}, \code{\link{rnb.set.annotation}}
#' @author Yassen Assenov
#' @export
rnb.region.types <- function(assembly = "hg19") {
	if (!(is.character(assembly) && length(assembly) == 1 && (!is.na(assembly)))) {
		stop("invalid value for assembly")
	}
	assembly <- tolower(assembly)
	if (!load.annotations(assembly)) {
		stop("unsupported assembly")
	}
	names(.rnb.annotations[[assembly]][["regions"]])
}

########################################################################################################################

#' rnb.get.mapping
#'
#' Gets the mapping information used for a region type. These are structures used to map regions to the genomic loci (or
#' Infinium probes) that target them.
#'
#' @param region.type Region type. The built-in types are \code{"cpgislands"}, \code{"genes"}, \code{"promoters"} and
#'                    \code{"tiling"}.
#' @param target.type Target type for sites.
#' @param assembly    Genome assembly of interest. See \code{\link{rnb.get.assemblies}} for the list of supported
#'                    genomes.
#' @return \code{list} of mapping structures, one per chromosome. Every mapping structure is an object of type
#'         \code{\link{IRanges}} and stores the range of indices of all sites contained in the respective region.
#'         Regions that do not contain sites are left out of the mapping.
#' @examples
#' \donttest{
#' promoters2probes <- rnb.get.mapping("promoters", "probes450")
#' promoters2probes[["chr21"]]
#' }
#' @author Yassen Assenov
#' @export
rnb.get.mapping <- function(region.type, target.type, assembly = "hg19") {
	if (!(is.character(region.type) && length(region.type) == 1 && (!is.na(region.type)))) {
		stop("invalid value for region.type")
	}
	if (!(is.character(target.type) && length(target.type) == 1 && (!is.na(target.type)))) {
		stop("invalid value for target.type")
	}
	if (!(is.character(assembly) && length(assembly) == 1 && (!is.na(assembly)))) {
		stop("invalid value for assembly")
	}
	assembly <- tolower(assembly)
	if (!load.annotations(assembly)) {
		stop("unsupported assembly")
	}
	if (!load.annotations(assembly, target.type)) {
		stop("unsupported target.type")
	}
	if (!(target.type %in% names(.rnb.annotations[[assembly]][["sites"]]))) {
		stop("unsupported target.type")
	}
	if (is.null(.rnb.annotations[[assembly]][["regions"]][[region.type]])) {
		stop("unsupported region.type")
	}
	.rnb.annotations[[assembly]][["mappings"]][[region.type]][[target.type]]
}

########################################################################################################################

#' rnb.get.chromosomes
#'
#' Gets the chromosome names supported for the specified assembly.
#'
#' @param assembly Genome assembly of interest. See \code{\link{rnb.get.assemblies}} for the list of supported genomes.
#' @return \code{character} vector of supported chromosomes for the specified genome assembly. The elements of the
#'         vector follow the \href{http://www.ensembl.org/}{Ensembl} convention (\code{"1"}, \code{"2"}, ...), and the
#'         names of this vector - the convention of the \href{http://genome.ucsc.edu/}{UCSC Genome Browser}
#'         (\code{"chr1"}, \code{"chr2"}, ...).
#'
#' @examples
#' \donttest{
#' "chrX" %in% names(rnb.get.chromosomes())
#' }
#' @author Pavlo Lutsik
#' @export
rnb.get.chromosomes <- function(assembly = "hg19") {
	if (!(is.character(assembly) && length(assembly) == 1 && (!is.na(assembly)))) {
		stop("invalid value for assembly")
	}
	assembly <- tolower(assembly)
	if (!load.annotations(assembly)) {
		stop("unsupported assembly")
	}
	return(.rnb.annotations[[assembly]][['CHROMOSOMES']])
}

########################################################################################################################

#' rnb.infinium.control.targets
#'
#' Extracts all control probe types in the HumanMethylation450 assay.
#'
#' @param target  A singleton of type \code{character}, specifying the microarray platform.
#' 				 \code{"probesEPICv2"},\code{"probesEPIC"},\code{"probes450"} and \code{"probes27"} correspond to MethylationEPICv2, 
#' 				 Methylation EPIC, HumanMethylation450, and HumanMethylation27 microarrays respectively.
#'
#' @return \code{character} vector of control targets.
#'
#' @examples
#' \donttest{
#' "NEGATIVE" %in% rnb.infinium.control.targets()
#' }
#' @author Pavlo Lutsik
#' @export
rnb.infinium.control.targets <- function(target="probes450") {
	if(target=="probesEPIC"){
		return(EPIC.CONTROL.TARGETS)
	}else if(target=="probesEPICv2"){
		return(EPICv2.CONTROL.TARGETS)
	}else if(target=="probes450"){
		return(HM450.CONTROL.TARGETS)
	}else if(target=="probes27"){
		return(HM27.CONTROL.TARGETS)
    }else if(target=="probesMMBC"){
        return(MMBC.CONTROL.TARGETS)
	}else{
		stop("Unsupported platform")
	}
}

########################################################################################################################
## differentialMethylation.R
## created: 2017-07-04
## creator: Fabian Mueller
## ---------------------------------------------------------------------------------------------------------------------
## Methods for enrichment analysis.
########################################################################################################################

################################################################################
# GO enrichment analyses
################################################################################
#' performGOenrichment.diffMeth.entrez
#'
#' performs Gene Ontology (GO) enrichment analysis for a list of Entrez identifiers
#' @param gids gene ids to test (entrez IDs)
#' @param uids ids to test against (universe)
#' @param ontology which ontology should be used (see \code{GOHyperGParams} from the \code{GOstats} package for details)
#' @param assembly Genome to be used. One of the following: hg19, mm9, mm10 or rn5
#' @param ... arguments passed on to the parameters of \code{GOHyperGParams} from the \code{GOstats} package
#' @return a \code{GOHyperGresult} object (see the \code{GOstats} package for further details)
#' @author Fabian Mueller
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
#' dmt <- get.table(dm,get.comparisons(dm)[1],"promoters")
#' annot <- annotation(rnb.set.example,"promoters")
#' all.promoters <- annot$entrezID
#' #get the hypermethylated promoters
#' hyper.promoters <- annot$entrezID[dmt[,"mean.mean.diff"]>0]
#' result <- performGOenrichment.diffMeth.entrez(hyper.promoters,all.promoters,"BP",assembly="hg19")
#' }
performGOenrichment.diffMeth.entrez <- function(gids,uids,ontology,assembly="hg19",...){
	rnb.require("GOstats")
#	#testing
#	dmt <- diffmeth$region[[1]][["promoters"]]
#	annot <- rnb.get.annotations(type = "promoters")
#	uids <- unique(unlist(sapply(annot[rownames(dmt),]$entrezid,FUN=function(idstr){
#		unlist(strsplit(idstr, ";", fixed = TRUE))
#	})))
#	dmrs <- dmt[dmt[,"combinedRank"] < 1000,]
#	gids <- unique(unlist(sapply(annot[rownames(dmrs),]$entrezid,FUN=function(idstr){
#		unlist(strsplit(idstr, ";", fixed = TRUE))
#	})))
#	#/testing
	if (is.element(assembly,c("hg19","hg38"))){
		ass <- "org.Hs.eg.db"
	} else if (is.element(assembly,c("mm9","mm10"))){
		ass <- "org.Mm.eg.db"
	} else if (is.element(assembly,c("rn5"))){
		ass <- "org.Rn.eg.db"
	} else if (is.element(assembly,c("zv9"))){
		ass <- "org.Dr.eg.db"
	} else {
		stop("Unsupported assembly")
	}
	params <- new("GOHyperGParams",annotation=ass,geneIds = gids, universeGeneIds = uids, 
			ontology = ontology,conditional = TRUE, testDirection = "over",...)
	nonMatchIdErrorMsg <- c(
		"The genes you are testing do not have any corresponding GO terms for the ontology you are searching.",
		"genes being tested do not have corresponding GO terms"
	)
	hgOver <- tryCatch({
			GOstats::hyperGTest(params)
		}, error = function(ee) {
			if (is.element(ee$message, nonMatchIdErrorMsg)){
				logger.info("Could not conduct enrichment analysis as associated genes are not in GO database.")
				NULL
			} else {
				logger.warning(c("Could not conduct enrichment analysis:", ee$message))
				NULL
			}
		}
	)
	return(hgOver)
}

#' performGoEnrichment.diffMeth
#'
#' performs Geno Ontology (GO) enrichment analysis for a given differential methylation table.
#' @author Fabian Mueller
#' @param rnbSet RnBSet object for which dirrential methylation was computed
#' @param diffmeth RnBDiffMeth object. See \code{\link{RnBDiffMeth-class}} for details.
#' @param ontologies GO ontologies to use for enrichment analysis
#' @param rank.cuts.region Cutoffs for combined ranking that are used to determine differentially methylated regions
#' @param add.auto.rank.cut flag indicating whether an automatically computed cut-off should also be considered.
#' @param rerank For deterimining differential methylation: should the ranks be ranked again or should the absolute ranks be used.
#' @param verbose Enable for detailed status report
#' @param ... arguments passed on to the parameters of \code{GOHyperGParams} from the \code{GOstats} package
#' @return a DiffMeth.go.enrich object (S3) containing the following attributes
#' \item{region}{Enrichment information for differential methylation on the region level. See \code{GOHyperGresult} from the \code{GOstats} package for furthert details}
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
#' res <- performGoEnrichment.diffMeth(rnb.set.example,dm)
#' }
performGoEnrichment.diffMeth <- function(rnbSet,diffmeth,ontologies=c("BP","MF"),rank.cuts.region=c(100,500,1000),add.auto.rank.cut=TRUE,rerank=TRUE,verbose=TRUE,...){
	gene.id.col <- "entrezID"
	logger.start("Differential Methylation GO Enrichment Analysis")
	if (!is.element(assembly(rnbSet),c("hg19","hg38","mm9","mm10","rn5","zv9"))){
		logger.warning(c("Enrichment analysis currently not supported for genome assembly:",assembly(rnbSet),"--> skipping enrichment analysis"))
		logger.completed()
		return(NULL)
	}
	comps <- get.comparisons(diffmeth)
	region.types <- get.region.types(diffmeth)
	skipSites <- !includes.sites(diffmeth)
	diff.col.reg <- "mean.mean.diff"
	if (skipSites) diff.col.reg <- "mean.diff"
	dm.go.enrich <- list(probe=list(),region=list())
	for (cc in comps){
		logger.start(c("Comparison: ",cc))
		dm.go.enrich$probe[[cc]] <- list()
		dm.go.enrich$region[[cc]] <- list()
		for (oo in ontologies){
			logger.start(c("Ontology: ",oo))
			logger.start("Region Level")
			dm.en.region.list.list <- list()
			for (rt in region.types){
				logger.start(c("Region Type:",rt))
				#rt <- "promoters"
				annot <- annotation(rnbSet,type = rt)
				if (!(gene.id.col %in% colnames(annot))){
					logger.info(c("Not annotated with",gene.id.col,"--> Skipped"))
					logger.completed()
					next
				}
#				dmt <- diffmeth$region[[cc]][[rt]]
				dmt <- get.table(diffmeth,cc,rt,undump=TRUE,return.data.frame=TRUE)
				rrs <- dmt[,"combinedRank"]
				rrs.hyper <- rrs
				rrs.hypo <- rrs
				rrs.hyper[dmt[,diff.col.reg] <= 0] <- NA
				rrs.hypo[dmt[,diff.col.reg] >= 0] <- NA
				#automatically select rank cutoff
				rc.auto <- 0L
				if (add.auto.rank.cut){
					rc.auto <- as.integer(auto.select.rank.cut(dmt$comb.p.adj.fdr,dmt$combinedRank))
				}

				if (rerank){
					rrs <- rank(rrs,na.last="keep",ties.method="min")
					rrs.hyper <- rank(rrs.hyper,na.last="keep",ties.method="min")
					rrs.hypo <- rank(rrs.hypo,na.last="keep",ties.method="min")
					if (add.auto.rank.cut && rc.auto > 0L){
						rc.auto <- na.omit(rrs[dmt[,"combinedRank"]==rc.auto])[1] #arbitrarily select the first rerank if the rank cut threshold is occupied by multiple ranks
					}
				}
				
				gene.ids.chr <- as.character(annot[,gene.id.col])
				uids <- unique(unlist(sapply(gene.ids.chr,FUN=function(idstr){
					unlist(strsplit(idstr, ";", fixed = TRUE))
				})))
				uids <- na.omit(uids)
				dm.en.region.list <- list()
				rank.cuts <- rank.cuts.region
				if (add.auto.rank.cut){
					rank.cuts <- c(rank.cuts,"autoSelect")
				}
				for (i in 1:length(rank.cuts)){
					rc <- rank.cuts[i]
					if (rc == "autoSelect") {
						rc <- rc.auto
						if (verbose) logger.info(c("Rank cutoff:",rc,"(auto-select)"))
					} else {
						rc <- as.integer(rc)
						if (verbose) logger.info(c("Rank cutoff:",rc))
					}
					is.dmr.hyper <- rrs.hyper <= rc
					is.dmr.hypo <- rrs.hypo <= rc
					gids.hyper <- unique(unlist(sapply(gene.ids.chr[is.dmr.hyper],FUN=function(idstr){
						unlist(strsplit(idstr, ";", fixed = TRUE))
					})))
					gids.hyper <- na.omit(gids.hyper)
					if (length(gids.hyper)>0 && length(uids)>0){
						dm.en.region.hyper <- performGOenrichment.diffMeth.entrez(gids.hyper,uids,ontology=oo,assembly=assembly(rnbSet),...)
					} else {
						dm.en.region.hyper <- NULL
					}
					
					gids.hypo <- unique(unlist(sapply(gene.ids.chr[is.dmr.hypo],FUN=function(idstr){
						unlist(strsplit(idstr, ";", fixed = TRUE))
					})))
					gids.hypo <- na.omit(gids.hypo)
					if (length(gids.hypo)>0 && length(uids)>0){
						dm.en.region.hypo <- performGOenrichment.diffMeth.entrez(gids.hypo,uids,ontology=oo,assembly=assembly(rnbSet),...)
					} else {
						dm.en.region.hypo <- NULL
					}
					dm.en.region.list <- c(dm.en.region.list,list(list(hyper=dm.en.region.hyper,hypo=dm.en.region.hypo)))
				}
				names(dm.en.region.list) <- paste("rankCut_",rank.cuts,sep="")
				dm.en.region.list.list <- c(dm.en.region.list.list,list(dm.en.region.list))
				names(dm.en.region.list.list)[length(dm.en.region.list.list)] <- rt
				logger.completed()
			}
			dm.go.enrich$region[[cc]][[oo]] <- dm.en.region.list.list
			logger.completed()
			logger.completed()
		}
		logger.completed()
	}
	class(dm.go.enrich) <- "DiffMeth.go.enrich"
	logger.completed()
	return(dm.go.enrich)
}

################################################################################
# LOLA enrichment analyses
################################################################################
#' downloadLolaDbs
#' 
#' Downloading prepared LOLA DBs from server
#' @param dest    destination directory
#' @param dbs     vector of names of LOLA DBs to be downloaded. Currently 'LOLACore' and 'LOLAExt'
#'                are supported
#' @details
#' Requires a stable internet connection. Could take a while depending on the size of the database and the internet connection
#' @return a list containing vectors of directory names for each available genome assembly
#' @author Fabian Mueller
#' @export
#' @examples
#' \donttest{
#' lolaDest <- tempfile()
#' dir.create(lolaDest)
#' lolaDirs <- downloadLolaDbs(lolaDest, dbs="LOLACore")
#' }
downloadLolaDbs <- function(dest, dbs=c("LOLACore")){
	res <- list()
	for (dbName in dbs){
		dbDownloadLink <- NULL
		if (dbName == "LOLACore"){
			# dbDownloadLink <- "http://regiondb.s3.amazonaws.com/LOLACoreCaches_170206.tgz"
			dbDownloadLink <- "http://cloud.databio.org.s3.amazonaws.com/regiondb/LOLACoreCaches_170206.tgz"
		} else if (dbName == "LOLAExt"){
			# dbDownloadLink <- "http://regiondb.s3.amazonaws.com/LOLAExtCaches_170206.tgz"
			dbDownloadLink <- "http://cloud.databio.org.s3.amazonaws.com/regiondb/LOLAExtCaches_170206.tgz"
		}
		if (is.null(dbDownloadLink)){
			logger.error(c("Invalid DB name:", dbName))
		}

		dbDir <- file.path(dest, dbName)
		if (dir.exists(dbDir)){
			logger.warning(c("Directory", dbDir, "already existing --> skipping download"))
		} else {
			tmpFn <- tempfile(fileext=".tgz")
			logger.start(c("Downloading from", dbDownloadLink))
			download.file(dbDownloadLink, destfile=tmpFn, mode = "wb")
			exDir <- tempfile()
			untar(tmpFn, exdir=exDir)
			baseDir <- grep(paste0(dbName, "$"), list.dirs(exDir, recursive=TRUE), value=TRUE)
			if (length(baseDir) < 1) logger.error(c("Downloaded archive does not contain a subdirectory", dbName))
			dir.create(dbDir)
			file.copy(baseDir, dest, recursive=TRUE)
			unlink(c(tmpFn, exDir), recursive=TRUE) # remove temp files
			logger.completed()
		}
		assemblies <- list.dirs(dbDir, full.names=FALSE, recursive=FALSE)
		for (aa in assemblies){
			res[[aa]] <- c(res[[aa]], file.path(dbDir, aa))
		}
	}
	return(res)
}

#' prepLolaDbPaths
#' 
#' prepare LOLA Database paths from options
#' @param assembly name of the assembly/subdirectory of the DB path
#' @param dbs  LOLA database paths, potentially containing placeholders
#' @param downloadDir directory where default LOLA DBs corresponding to placeholders will be downloaded to
#'             (provided that placeholders are contained in \code{dbs})
#' 
#' @return vector of LOLA DB paths
#' @author Fabian Mueller
#' @noRd
prepLolaDbPaths <- function(assembly, dbs=rnb.getOption("differential.enrichment.lola.dbs"), downloadDir=tempdir()){
	#download DBs for placeholders
	phPattern <- "^\\$\\{(.+)\\}$"
	isPh <- grepl(phPattern, dbs)
	res <- file.path(dbs[!isPh], assembly)
	if (any(isPh)){
		logger.start("Downloading LOLA databases")
			if (!dir.exists(downloadDir)) dir.create(downloadDir)
			downloadDbs <- gsub(phPattern, "\\1", dbs[isPh])
			downloadedPaths <- downloadLolaDbs(downloadDir, dbs=downloadDbs)
		logger.completed()
		res <- c(res, downloadedPaths[[assembly]])
	}
	nonExisting <- !dir.exists(res)
	if (any(nonExisting)){
		logger.error(c("The following LOLA DB directories are missing:", paste(res[nonExisting], collapse="; ")))
	}
	return(res)
}

#' loadLolaDbs
#' 
#' Load LOLA databases from disk and merge them
#' @param lolaDbPaths  vector of names of LOLA DB paths to be loaded
#' 
#' @return LOLA DB list as returned by \code{LOLA::loadRegionDB}
#' @author Fabian Mueller
#' @export
#' @examples
#' \donttest{
#' # download LOLA DB
#' lolaDest <- tempfile()
#' dir.create(lolaDest)
#' lolaDirs <- downloadLolaDbs(lolaDest, dbs="LOLACore")
#' lolaDb <- loadLolaDbs(lolaDirs[["hg19"]])
#' }
loadLolaDbs <- function(lolaDbPaths){
	rnb.require("data.table") #explicitely load data.table to adress LOLA namespace issues
	rnb.require("LOLA")
	rnb.require("simpleCache") # TODO: include requirement in dependencies
	logger.start("Loading LOLA DBs")
		lolaDb <- loadRegionDB(lolaDbPaths[1])
		if (length(lolaDbPaths)>1){
			for (i in 2:length(lolaDbPaths)){
				lolaDb <- mergeRegionDBs(lolaDb, loadRegionDB(lolaDbPaths[i]))
			}
		}
	logger.completed()
	return(lolaDb)
}

#' performLolaEnrichment.diffMeth
#'
#' performs LOLA enrichment analysis for a given differential methylation table.
#' @author Fabian Mueller
#' @param rnbSet RnBSet object for which dirrential methylation was computed
#' @param diffmeth RnBDiffMeth object. See \code{\link{RnBDiffMeth-class}} for details.
#' @param lolaDbPaths LOLA database paths
#' @param rank.cuts.region Cutoffs for combined ranking that are used to determine differentially methylated regions
#' @param add.auto.rank.cut flag indicating whether an automatically computed cut-off should also be considered.
#' @param rerank For deterimining differential methylation: should the ranks be ranked again or should the absolute ranks be used.
#' @param verbose Enable for detailed status report
#' @return a DiffMeth.lola.enrich object (S3) containing the following attributes
#' \item{region}{Enrichment information for differential methylation on the region level. A \code{data.table} object
#' as returned by the \code{runLOLA} function from the \code{LOLA} package for furthert details. Each element will contain different
#' user sets for different rank cutoffs and hyper/hypomethylation events(\code{userSet} column)}
#' \item{lolaDb}{The loaded \code{lolaDb} object containing the merged databases as returned by \code{\link{loadLolaDbs}}}
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' # compute differential methylation
#' dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
#' # download LOLA DB
#' lolaDest <- tempfile()
#' dir.create(lolaDest)
#' lolaDirs <- downloadLolaDbs(lolaDest, dbs="LOLACore")
#' # perform enrichment analysis
#' res <- performLolaEnrichment.diffMeth(rnb.set.example,dm,lolaDirs[["hg19"]])
#' }
performLolaEnrichment.diffMeth <- function(rnbSet, diffmeth, lolaDbPaths, rank.cuts.region=c(100,500,1000), add.auto.rank.cut=TRUE, rerank=TRUE, verbose=TRUE){
	nCores <- ifelse(parallel.isEnabled(), parallel.getNumWorkers(), 1)
	lolaDb <- loadLolaDbs(lolaDbPaths)
	comps <- get.comparisons(diffmeth)
	region.types <- get.region.types(diffmeth)
	skipSites <- !includes.sites(diffmeth)
	diff.col.reg <- "mean.mean.diff"
	if (skipSites) diff.col.reg <- "mean.diff"
	dm.lola.enrich <- list(probe=list(),region=list())
	for (cc in comps){
		dm.lola.enrich$probe[[cc]] <- list()
		dm.lola.enrich$region[[cc]] <- list()
	}
	logger.start("Differential Methylation LOLA Enrichment Analysis")
	for (rt in region.types){
		logger.start(c("Region Type:",rt))
			annot <- annotation(rnbSet, type=rt)
			# convert annotaiton to GRanges object
			univGr <- data.frame2GRanges(annot, chrom.column="Chromosome", start.column="Start", end.column="End", assembly=assembly(rnbSet), sort.result=FALSE)

			lolaUserSets <- list()

			for (i in 1:length(comps)){
				cc <- comps[i]
				compKey <- paste0("comp", i)
				logger.start(c("Comparison: ",cc))
					dmt <- get.table(diffmeth,cc,rt,undump=TRUE,return.data.frame=TRUE)

					rrs <- dmt[,"combinedRank"]
					rrs.hyper <- rrs
					rrs.hypo <- rrs
					rrs.hyper[dmt[,diff.col.reg] <= 0] <- NA
					rrs.hypo[dmt[,diff.col.reg] >= 0] <- NA
					#automatically select rank cutoff
					rc.auto <- 0L
					if (add.auto.rank.cut){
						rc.auto <- as.integer(auto.select.rank.cut(dmt$comb.p.adj.fdr,dmt$combinedRank))
					}

					if (rerank){
						rrs <- rank(rrs,na.last="keep",ties.method="min")
						rrs.hyper <- rank(rrs.hyper,na.last="keep",ties.method="min")
						rrs.hypo <- rank(rrs.hypo,na.last="keep",ties.method="min")
						if (add.auto.rank.cut && rc.auto > 0L){
							rc.auto <- na.omit(rrs[dmt[,"combinedRank"]==rc.auto])[1] #arbitrarily select the first rerank if the rank cut threshold is occupied by multiple ranks
						}
					}

					rank.cuts <- rank.cuts.region
					if (add.auto.rank.cut){
						rank.cuts <- c(rank.cuts,"autoSelect")
					}
					for (j in 1:length(rank.cuts)){
						rcn <- paste("rankCut_", rank.cuts[j] ,sep="")

						rc <- rank.cuts[j]
						if (rc == "autoSelect") {
							rc <- rc.auto
							if (verbose) logger.info(c("Rank cutoff:",rc,"(auto-select)"))
						} else {
							rc <- as.integer(rc)
							if (verbose) logger.info(c("Rank cutoff:",rc))
						}
						is.dmr.hyper <- rrs.hyper <= rc
						is.dmr.hyper[is.na(is.dmr.hyper)] <- FALSE
						is.dmr.hypo <- rrs.hypo <= rc
						is.dmr.hypo[is.na(is.dmr.hypo)] <- FALSE

						lolaUserSets <- c(lolaUserSets, list(univGr[is.dmr.hyper]), list(univGr[is.dmr.hypo]))
						N <- length(lolaUserSets)
						names(lolaUserSets)[c(N-1,N)] <- paste(compKey, rcn, c("hyper", "hypo"), sep="_")
					}
				logger.completed()
			}

			lolaUserSets <- GRangesList(lolaUserSets)
			logger.start("Running LOLA")
				lolaRes <- runLOLA(lolaUserSets, univGr, lolaDb, cores=nCores)
			logger.completed()

			for (i in 1:length(comps)){
				cc <- comps[i]
				curPat <- paste0("^comp",i,"_")
				curSubset <- grepl(curPat, lolaRes[["userSet"]])
				lolaRes.cur <- lolaRes[curSubset,]
				lolaRes.cur[["userSet"]] <- gsub(curPat, "", lolaRes.cur[["userSet"]])
				dm.lola.enrich$region[[cc]][[rt]] <- lolaRes.cur
			}

		logger.completed()
	}
	dm.lola.enrich[["lolaDb"]] <- lolaDb
	class(dm.lola.enrich) <- "DiffMeth.lola.enrich"
	logger.completed()
	return(dm.lola.enrich)
}

#' performGOEnrichment.diffVar
#'
#' performs Geno Ontology (GO) enrichment analysis for a given differential variability table.
#' @author Fabian Mueller and Michael Scherer
#' @param rnbSet RnBSet object for which dirrential variability was computed
#' @param diffmeth RnBDiffMeth object. See \code{\link{RnBDiffMeth-class}} for details.
#' @param enrich.diffMeth Result of \code{performGOEnrichment.diffMeth}. NULL, if enrichment should only be performed for differential variability.
#' @param ontologies GO ontologies to use for enrichment analysis
#' @param rank.cuts.region Cutoffs for combined ranking that are used to determine differentially variable regions
#' @param add.auto.rank.cut flag indicating whether an automatically computed cut-off should also be considered.
#' @param rerank For deterimining differential variability: should the ranks be ranked again or should the absolute ranks be used.
#' @param verbose Enable for detailed status report
#' @param ... arguments passed on to the parameters of \code{GOHyperGParams} from the \code{GOstats} package
#' @return a DiffMeth.enrich object (S3) containing the following attributes
#' \item{region}{Enrichment information for differential variability on the region level. See \code{GOHyperGresult} from the \code{GOstats} package for furthert details}
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' dm <- rnb.execute.diffVar(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
#' res <- performEnrichment.diffVar(rnb.set.example,dm)
#' }
performGOEnrichment.diffVar <- function(rnbSet,diffmeth,enrich.diffMeth=NULL,ontologies=c("BP","MF"),rank.cuts.region=c(100,500,1000),add.auto.rank.cut=TRUE,rerank=TRUE,verbose=TRUE,...){
  gene.id.col <- "entrezID"
  logger.start("Differential Variability Enrichment Analysis")
  if (!is.element(assembly(rnbSet),c("hg19","hg38","mm9","mm10","rn5","zv9"))){
    logger.warning(c("Enrichment analysis currently not supported for genome assembly:",assembly(rnbSet),"--> skipping enrichment analysis"))
    logger.completed()
    return(NULL)
  }
  comps <- get.comparisons(diffmeth)
  region.types <- get.region.types(diffmeth)
  skipSites <- !includes.sites(diffmeth)
  diff.col.reg <- "mean.var.diff"
  if (skipSites) diff.col.reg <- "var.diff"
  if(!is.null(enrich.diffMeth)){
    enrich.diffMeth$probe_var <- list()
    enrich.diffMeth$region_var <- list()
    dv.enrich <- enrich.diffMeth
    rm(enrich.diffMeth)
  }else{
    dv.enrich <- list(probe_var=list(),region_var=list())
  }
  for (cc in comps){
    logger.start(c("Comparison: ",cc))
    dv.enrich$probe_var[[cc]] <- list()
    dv.enrich$region_var[[cc]] <- list()
    for (oo in ontologies){
      logger.start(c("Ontology: ",oo))
      logger.start("Region Level")
      dm.en.region.list.list <- list()
      for (rt in region.types){
        logger.start(c("Region Type:",rt))
        #rt <- "promoters"
        annot <- annotation(rnbSet,type = rt)
        if (!(gene.id.col %in% colnames(annot))){
          logger.info(c("Not annotated with",gene.id.col,"--> Skipped"))
          logger.completed()
          next
        }
        dmt <- get.table(diffmeth,cc,rt,undump=TRUE,return.data.frame=TRUE)
        rrs <- dmt[,"combinedRank.var"]
        rrs.hyper <- rrs
        rrs.hypo <- rrs
        rrs.hyper[dmt[,diff.col.reg] <= 0] <- NA
        rrs.hypo[dmt[,diff.col.reg] >= 0] <- NA
        #automatically select rank cutoff
        rc.auto <- 0L
        if (add.auto.rank.cut){
          rc.auto <- as.integer(auto.select.rank.cut(dmt$comb.p.adj.var.fdr,dmt$combinedRank.var))
        }
        
        if (rerank){
          rrs <- rank(rrs,na.last="keep",ties.method="min")
          rrs.hyper <- rank(rrs.hyper,na.last="keep",ties.method="min")
          rrs.hypo <- rank(rrs.hypo,na.last="keep",ties.method="min")
          if (add.auto.rank.cut && rc.auto > 0L){
            rc.auto <- na.omit(rrs[dmt[,"combinedRank.var"]==rc.auto])[1] #arbitrarily select the first rerank if the rank cut threshold is occupied by multiple ranks
          }
        }
        
        gene.ids.chr <- as.character(annot[,gene.id.col])
        uids <- unique(unlist(sapply(gene.ids.chr,FUN=function(idstr){
          unlist(strsplit(idstr, ";", fixed = TRUE))
        })))
        uids <- na.omit(uids)
        dm.en.region.list <- list()
        rank.cuts <- rank.cuts.region
        if (add.auto.rank.cut){
          rank.cuts <- c(rank.cuts,"autoSelect")
        }
        for (i in 1:length(rank.cuts)){
          rc <- rank.cuts[i]
          if (rc == "autoSelect") {
            rc <- rc.auto
            if (verbose) logger.info(c("Rank cutoff:",rc,"(auto-select)"))
          } else {
            rc <- as.integer(rc)
            if (verbose) logger.info(c("Rank cutoff:",rc))
          }
          is.dmr.hyper <- rrs.hyper <= rc
          is.dmr.hypo <- rrs.hypo <= rc
          gids.hyper <- unique(unlist(sapply(gene.ids.chr[is.dmr.hyper],FUN=function(idstr){
            unlist(strsplit(idstr, ";", fixed = TRUE))
          })))
          gids.hyper <- na.omit(gids.hyper)
          if (length(gids.hyper)>0 && length(uids)>0){
            dm.en.region.hyper <- performGOenrichment.diffMeth.entrez(gids.hyper,uids,ontology=oo,assembly=assembly(rnbSet),...)
          } else {
            dm.en.region.hyper <- NULL
          }
          
          gids.hypo <- unique(unlist(sapply(gene.ids.chr[is.dmr.hypo],FUN=function(idstr){
            unlist(strsplit(idstr, ";", fixed = TRUE))
          })))
          gids.hypo <- na.omit(gids.hypo)
          if (length(gids.hypo)>0 && length(uids)>0){
            dm.en.region.hypo <- performGOenrichment.diffMeth.entrez(gids.hypo,uids,ontology=oo,assembly=assembly(rnbSet),...)
          } else {
            dm.en.region.hypo <- NULL
          }
          dm.en.region.list <- c(dm.en.region.list,list(list(hyper=dm.en.region.hyper,hypo=dm.en.region.hypo)))
        }
        names(dm.en.region.list) <- paste("rankCut_",rank.cuts,sep="")
        dm.en.region.list.list <- c(dm.en.region.list.list,list(dm.en.region.list))
        names(dm.en.region.list.list)[length(dm.en.region.list.list)] <- rt
        logger.completed()
      }
      dv.enrich$region_var[[cc]][[oo]] <- dm.en.region.list.list
      logger.completed()
      logger.completed()
    }
    logger.completed()
  }
  class(dv.enrich) <- "DiffMeth.go.enrich"
  logger.completed()
  return(dv.enrich)
}

#' performLolaEnrichment.diffVar
#'
#' performs LOLA enrichment analysis for a given differential variability table.
#' @author Michael Scherer and Fabian Mueller
#' @param rnbSet RnBSet object for which differential variability was computed
#' @param diffmeth RnBDiffMeth object. See \code{\link{RnBDiffMeth-class}} for details.
#' @param enrich.diffMeth Enrichment object as obtained from \code{\link{performLolaEnrichment.diffMeth}}. If it is not provided
#'          a new object is created.
#' @param lolaDbPaths LOLA database paths
#' @param rank.cuts.region Cutoffs for combined ranking that are used to determine differentially variable regions
#' @param add.auto.rank.cut flag indicating whether an automatically computed cut-off should also be considered.
#' @param rerank For deterimining differential variability: should the ranks be ranked again or should the absolute ranks be used.
#' @param verbose Enable for detailed status report
#' @return a DiffMeth.lola.enrich object (S3) containing the following attributes
#' \item{region}{Enrichment information for differential variability on the region level. A \code{data.table} object
#' as returned by the \code{runLOLA} function from the \code{LOLA} package for further details. Each element will contain different
#' user sets for different rank cutoffs and hyper/hypomethylation events(\code{userSet} column)}
#' \item{lolaDb}{The loaded \code{lolaDb} object containing the merged databases as returned by \code{\link{loadLolaDbs}}}
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' # compute differential methylation
#' dm <- rnb.execute.diffVar(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
#' # download LOLA DB
#' lolaDest <- tempfile()
#' dir.create(lolaDest)
#' lolaDirs <- downloadLolaDbs(lolaDest, dbs="LOLACore")
#' # perform enrichment analysis
#' res <- performLolaEnrichment.diffVar(rnb.set.example,dm,lolaDirs[["hg19"]])
#' }
performLolaEnrichment.diffVar <- function(rnbSet, diffmeth, enrich.diffMeth=NULL, lolaDbPaths, rank.cuts.region=c(100,500,1000), add.auto.rank.cut=TRUE, rerank=TRUE, verbose=TRUE){
  nCores <- ifelse(parallel.isEnabled(), parallel.getNumWorkers(), 1)
  lolaDb <- loadLolaDbs(lolaDbPaths)
  comps <- get.comparisons(diffmeth)
  region.types <- get.region.types(diffmeth)
  skipSites <- !includes.sites(diffmeth)
  diff.col.reg <- "mean.var.diff"
  if (skipSites) diff.col.reg <- "var.diff"
  if(!is.null(enrich.diffMeth)){
    enrich.diffMeth$region_var <- list()
    dv.lola.enrich <- enrich.diffMeth
    rm(enrich.diffMeth)
  }else{
    dv.lola.enrich <- list(region_var=list())
  }
  for (cc in comps){
    dv.lola.enrich$probe_var[[cc]] <- list()
    dv.lola.enrich$region_var[[cc]] <- list()
  }
  logger.start("Differential Variability LOLA Enrichment Analysis")
  for (rt in region.types){
    logger.start(c("Region Type:",rt))
    annot <- annotation(rnbSet, type=rt)
    # convert annotaiton to GRanges object
    univGr <- data.frame2GRanges(annot, chrom.column="Chromosome", start.column="Start", end.column="End", assembly=assembly(rnbSet), sort.result=FALSE)
    
    lolaUserSets <- list()
    
    for (i in 1:length(comps)){
      cc <- comps[i]
      compKey <- paste0("comp", i)
      logger.start(c("Comparison: ",cc))
      dmt <- get.table(diffmeth,cc,rt,undump=TRUE,return.data.frame=TRUE)
      
      if(!is.element("combinedRank.var",colnames(dmt))){
        stop("No differential variability information available. Be sure to execute differential variability module beforehand.")
      }
      rrs <- dmt[,"combinedRank.var"]
      rrs.hyper <- rrs
      rrs.hypo <- rrs
      rrs.hyper[dmt[,diff.col.reg] <= 0] <- NA
      rrs.hypo[dmt[,diff.col.reg] >= 0] <- NA
      #automatically select rank cutoff
      rc.auto <- 0L
      if (add.auto.rank.cut){
        rc.auto <- as.integer(auto.select.rank.cut(dmt$comb.p.adj.var.fdr,dmt$combinedRank.var))
      }
      
      if (rerank){
        rrs <- rank(rrs,na.last="keep",ties.method="min")
        rrs.hyper <- rank(rrs.hyper,na.last="keep",ties.method="min")
        rrs.hypo <- rank(rrs.hypo,na.last="keep",ties.method="min")
        if (add.auto.rank.cut && rc.auto > 0L){
          rc.auto <- na.omit(rrs[dmt[,"combinedRank.var"]==rc.auto])[1] #arbitrarily select the first rerank if the rank cut threshold is occupied by multiple ranks
        }
      }
      
      rank.cuts <- rank.cuts.region
      if (add.auto.rank.cut){
        rank.cuts <- c(rank.cuts,"autoSelect")
      }
      for (j in 1:length(rank.cuts)){
        rcn <- paste("rankCut_", rank.cuts[j] ,sep="")
        
        rc <- rank.cuts[j]
        if (rc == "autoSelect") {
          rc <- rc.auto
          if (verbose) logger.info(c("Rank cutoff:",rc,"(auto-select)"))
        } else {
          rc <- as.integer(rc)
          if (verbose) logger.info(c("Rank cutoff:",rc))
        }
        is.dmr.hyper <- rrs.hyper <= rc
        is.dmr.hyper[is.na(is.dmr.hyper)] <- FALSE
        is.dmr.hypo <- rrs.hypo <= rc
        is.dmr.hypo[is.na(is.dmr.hypo)] <- FALSE
        
        lolaUserSets <- c(lolaUserSets, list(univGr[is.dmr.hyper]), list(univGr[is.dmr.hypo]))
        N <- length(lolaUserSets)
        names(lolaUserSets)[c(N-1,N)] <- paste(compKey, rcn, c("hyper", "hypo"), sep="_")
      }
      logger.completed()
    }
    
    lolaUserSets <- GRangesList(lolaUserSets)
    logger.start("Running LOLA")
    lolaRes <- runLOLA(lolaUserSets, univGr, lolaDb, cores=nCores)
    logger.completed()
    
    for (i in 1:length(comps)){
      cc <- comps[i]
      curPat <- paste0("^comp",i,"_")
      curSubset <- grepl(curPat, lolaRes[["userSet"]])
      lolaRes.cur <- lolaRes[curSubset,]
      lolaRes.cur[["userSet"]] <- gsub(curPat, "", lolaRes.cur[["userSet"]])
      dv.lola.enrich$region_var[[cc]][[rt]] <- lolaRes.cur
    }
    
    logger.completed()
  }
  if(!is.element("lolaDb",names(dv.lola.enrich))){
    dv.lola.enrich[["lolaDb"]] <- lolaDb
  }
  class(dv.lola.enrich) <- "DiffMeth.lola.enrich"
  logger.completed()
  return(dv.lola.enrich)
}

########################################################################################################################
## dataExport.R
## created: 2013-01-15
## creator: Fabian Mueller
## ---------------------------------------------------------------------------------------------------------------------
## Data exporting routines
########################################################################################################################

#' rnb.export.fail
#'
#' Adds a paragraph describing why exporting data for a specific region type might fail.
#'
#' @param report Report to contain the description.
#' @param txt Text to be prepended to the paragraph.
#' @author Yassen Assenov
#' @noRd
rnb.export.fail <- function(report, txt = character()) {
	txt <- c(txt, "There are several reasons why a certain output file cannot be (fully) generated. Examples include:")
	rnb.add.paragraph(report, txt)
	txt <- list(
		"The corresponding region type is invalid.",
		"The corresponding region type is not supported by the dataset.",
		"Due to security restrictions, the creation of files in the output directory is not allowed.",
		"A file or directory with the same name exists and cannot be overwritten.",
		"The disk is full or the user quota is exceeded.")
	rnb.add.list(report, txt)
}

########################################################################################################################

#' rnb.RnBSet.to.GRangesList
#'
#' convert an \code{\linkS4class{RnBSet}} object to a \code{GRangesList} object
#' @param rnb.set Object of class \code{\linkS4class{RnBSet}}
#' @param reg.type region type to be converted
#' @param return.regular.list flag indicating whether a regular \code{list} object should be returned instead
#'             of a \code{GRangesList}. Might improve performance in some cases
#' @return a \code{GRangesList} or \code{list} object with one list element (\code{GRanges}) for each sample in \code{rnb.set}
#' @author Fabian Mueller
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' result <- rnb.RnBSet.to.GRangesList(rnb.set.example)
#' }
rnb.RnBSet.to.GRangesList <- function(rnb.set,reg.type="sites",return.regular.list=FALSE){
	if (!inherits(rnb.set, "RnBSet")) {
		stop("Invalid value for rnb.set: Expected RnBSet")
	}
	if (!(reg.type %in% c(rnb.region.types(assembly(rnb.set)),"sites"))){
		stop("Unsupported region type")
	}
	# logger.status("Gathering data") #DEBUG MESSAGE
	anno.sites <- annotation(rnb.set,type=reg.type)
	anno.sites <- anno.sites[,c("Chromosome","Start","End","Strand")]
	granges.coords <- data.frame2GRanges(anno.sites, ids = NULL, chrom.column = "Chromosome", start.column = "Start",
						end.column = "End", strand.column = "Strand", assembly = assembly(rnb.set), sort.result=FALSE)
	# logger.status("...getting data into the correct order") #DEBUG MESSAGE
	oo <- order(as.integer(seqnames(granges.coords)),start(granges.coords), end(granges.coords), as.integer(strand(granges.coords)))
	granges.coords <- granges.coords[oo]
	mm <- meth(rnb.set,type=reg.type)[oo,,drop=FALSE]
	ccov <- covg(rnb.set,type=reg.type)[oo,,drop=FALSE]

	# logger.status("Creating list for GRanges objects") #DEBUG MESSAGE
	rnbs.grl <- lapply(samples(rnb.set),FUN=function(ss){
		if (!is.null(ccov)){
			dd <- data.frame(score=mm[,ss],coverage=ccov[,ss])
		} else {
			dd <- data.frame(score=mm[,ss])
		}
		rows.no.nas <- !is.na(dd$score)
		granges.cur <- granges.coords[rows.no.nas]
		elementMetadata(granges.cur) <- dd[rows.no.nas,,drop=FALSE]
		# logger.status(c("...done processing sample",ss)) #DEBUG MESSAGE
		return(granges.cur)
	})
	names(rnbs.grl) <- samples(rnb.set)
	# logger.status("sorting entries")
	# rnbs.grl <- lapply(rnbs.grl , function(y) { y[order(as.integer(seqnames(y)),start(y), end(y), as.integer(strand(y))), ] })
	if (!return.regular.list){
		logger.status("converting to GRangesList") #DEBUG MESSAGE
		rnbs.grl <- GRangesList(rnbs.grl) #this is very memory intense for larger WGBS datasets
	}
	# #alternative: endoapply
	# rnbs.grl <- rep(GRangesList(granges.coords),length(samples(rnb.set)))
	# names(rnbs.grl) <- samples(rnb.set)
	# logger.status("Adding sample information") #DEBUG MESSAGE
	# i <- 1
	# rnbs.grl <- endoapply(rnbs.grl , function(gr){
	# 	#a bit of a hack getting the sample name using i in endoapply.
	# 	#If you can think of a better way, let me know.
	# 	#endoapply was necessary for performance reasons in large WGBS datasets
	# 	ss <- names(rnbs.grl)[i]
	# 	i <<- i + 1
	# 	logger.status(c("...processing sample",ss)) #DEBUG MESSAGE
	# 	if (!is.null(ccov)){
	# 		dd <- data.frame(score=mm[,ss],coverage=ccov[,ss])
	# 	} else {
	# 		dd <- data.frame(score=mm[,ss])
	# 	}
	# 	logger.status(c("......setting elementMetadata")) #DEBUG MESSAGE
	# 	elementMetadata(gr) <- dd
	# 	logger.status(c("......retrieving non-NAs")) #DEBUG MESSAGE
	# 	subset(gr,!is.na(elementMetadata(gr)[,"score"]))
	# })

	# rnbs.grl <- endoapply(rnbs.grl , function(y) { y[order(as.integer(seqnames(y)),start(y), end(y), as.integer(strand(y))), ] })
	return(rnbs.grl)
}

########################################################################################################################

#' Export to BED files
#'
#' Exports the beta values from a methylation dataset to BED files.
#' 
#' @param rnb.set Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param out.dir Output directory. If not existing, it will be created. otherwise files in that directory are overwritten.
#' @param reg.type Region type to be extracted.
#' @param names.quant.meth should the names of the bed regions contain information on the methylation level.
#' 		   If TRUE the following format is applied: meth_percent%[coverage]. Coverage is available only when
#' 			\code{covg(rnb.set)} is not NULL
#' @param add.track.line Add a track line to the bed file to enable browsers like IGV to display the data better
#' @param lexicographic Should lexicographic ordering be used for chromosome names
#' @param verbose More detailed logger output
#' @return (invisibly) a summary list containing information on the conversion step.
#'         elements are \code{filenames} (a table containing information on which sample has been written to what filename)
#'         and \code{assembly} (a string indicating the assembly used by \code{rnb.set}).
#' @details Details on the BED file format can be found in the \href{http://genome.ucsc.edu/FAQ/FAQformat.html}{UCSC Genome Browser
#'          documentation}.  Each methylation site is an entry in the resulting bed file. The Score column corresponds
#'          to a site's methylation value in the interval \code{[0,1]}.
#' @author Fabian Mueller
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' rnb.RnBSet.to.bed(rnb.set.example,tempdir())
#' }
rnb.RnBSet.to.bed <- function(rnb.set,out.dir,reg.type="sites",names.quant.meth=TRUE,add.track.line=TRUE,lexicographic=FALSE,verbose=TRUE){
	if (!inherits(rnb.set, "RnBSet")) {
		stop("inavlid value for rnb.set")
	}
	if (!file.exists(out.dir)){
		dir.create(out.dir)
	}
	if (!(reg.type %in% c(rnb.region.types(assembly(rnb.set)),"sites"))){
		stop("Unsupported region type")
	}
	#convert RnBSet to GRangesList
	if (verbose) logger.status("Converting to GRangesList")
	rnb.set.grl <- rnb.RnBSet.to.GRangesList(rnb.set,reg.type=reg.type,return.regular.list=TRUE) #might take a while

	## Extract annotation table and methylation values
	tbl <- annotation(rnb.set, type = reg.type)[, c("Chromosome", "Start", "End", "Strand")]
	is.disjoint <- isDisjoint(GRanges(tbl$Chromosome, IRanges(tbl$Start, tbl$End), tbl$Strand))
	levels(tbl$Strand) <- c("+", "-", ".")
	tbl[, 2] <- tbl[, 2] - 1L # switch from 1-based closed intervals to 0-based half-open
	tbl[["ID"]] <- FALSE
	tbl[["Value"]] <- 0
	tbl <- tbl[, c(1:3, 5:6, 4)]
	if (lexicographic) {
		i.order <- order(as.integer(factor(as.character(tbl$Chromosome))))
	}

	## Prepare the resulting structure
	sample.ids <- samples(rnb.set)
	res <- list()
	res[["filenames"]] <- data.frame(i=1:length(sample.ids),sample=sample.ids,filename=paste0("rnbeads_sample",1:length(sample.ids),".bed"))
	res[["assembly"]] <- assembly(rnb.set)
	res[["contains.overlapping.regions"]] <- is.disjoint

	## output the text files
	for (i in 1:length(sample.ids)) {
		if (verbose) logger.status(c("Exporting sample", sample.ids[i]))
		mm <- as.vector(meth(rnb.set, type = reg.type, j = i))
		ii <- which(!is.na(mm))
		if (names.quant.meth) {
			cvg <- covg(rnb.set, reg.type, j = i)
			if (is.null(cvg)) {
				cvg <- "%"
			} else {
				cvg <- paste0("%[", cvg, "]")
			}
			bed.names <- paste0("'", round(mm * 100), cvg, "'")
		} else {
			bed.names <- rownames(tbl)
		}
		tbl$Value <- as.integer(round(mm * 1000))
		fn <- file.path(out.dir, res[["filenames"]][i, "filename"])
		if (add.track.line) {
			trackLine <- paste0("track name=\"",sample.ids[i],"\" description=\"",sample.ids[i]," methylation (",reg.type,")\" color=0,60,120 useScore=1")
			write(trackLine,file=fn)
		}
		if (lexicographic) {
			x <- tbl[intersect(i.order, ii), , drop = FALSE]
		} else {
			x <- tbl[ii, , drop = FALSE]
		}
		write.table(x,file=fn,quote=FALSE,sep="\t",row.names=FALSE,col.names=FALSE,na=".",append=add.track.line)
	}
	invisible(res)
}

########################################################################################################################

#' rnb.RnBSet.to.bedGraph
#'
#' Exports the methylation data of an \code{\linkS4class{RnBSet}} object to \code{*.bedGraph} files.
#'
#' @param rnb.set    Dataset as an instance of class \code{\linkS4class{RnBSet}}.
#' @param out.dir    One-element \code{character} vector signifying the output directory in which to create
#'                   \code{bedGraph} files. Setting this to \code{"."} (default) uses the current working directory. If
#'                   the output directory does not exist, this function attempts to create it. Any existing files in
#'                   this directory could be overwritten.
#' @param reg.type   Site or region type to be exported.
#' @param parameters Named \code{character} vector storing parameters (other than \code{"type"} and \code{"name"}) to
#'                   include in the track definition line. The names of this vector must be the parameter names, and its
#'                   elements - the corresponding values; missing values (\code{NA}s) are allowed neither for names, nor
#'                   for values. This function does not test if all provided parameter names and values conform to the
#'                   BedGraph track specification.
#' @param digits     Optionally, number of significant digits after the decimal point to round methylation values to. If
#'                   specified, this parameter must be an \code{integer} between \code{0} and \code{10}.
#' @return (invisibly) a summary list containing information on the conversion step.
#'         elements are \code{filenames} (a table containing information on which sample has been written to what filename)
#'         and \code{assembly} (a string indicating the assembly used by \code{rnb.set}).
#' @details The description of the BedGraph track format can be found \href{http://genome.ucsc.edu/goldenPath/help/bedgraph.html}{here}.
#'          Each methylation site is an entry in the resulting bedGraph file. The Score column corresponds to a site's
#'          methylation value in the interval \code{[0,1]}.
#' @author Fabian Mueller
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' rnb.RnBSet.to.bedGraph(rnb.set.example,tempdir())
#' }
rnb.RnBSet.to.bedGraph <- function(rnb.set,out.dir=".",reg.type="sites",parameters=character(),digits=NULL){
	## Validate parameters
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	if (!(is.character(out.dir) && length(out.dir) == 1 && isTRUE(out.dir != ""))) {
		stop("invalid value for out.dir")
	}
	if (!(is.character(reg.type) && length(reg.type) == 1 && isTRUE(reg.type != ""))) {
		stop("invalid value for reg.type")
	}
	if (!is.character(parameters)) {
		stop("invalid value for parameters")
	}
	if (length(parameters) != 0) {
		if (any(is.na(parameters)) || is.null(names(parameters)) || any(is.na(names(parameters)))) {
			stop("invalid value for parameters")
		}
		parameters <- paste0(" ", paste(names(parameters), parameters, sep = "=", collapse = " "))
	}
	if (!is.null(digits)) {
		if (isTRUE(is.double(digits) && all(digits == as.integer(digits)))) {
			digits <- as.integer(digits)
		}
		if (!(is.integer(digits) && length(digits) == 1 && isTRUE(0L <= digits && digits <= 10L))) {
			stop("invalid value for digits")
		}
	}

	## Create output directory if needed
	if (file.exists(out.dir)) {
		if (!file.info(out.dir)[, "isdir"]) {
			stop("out.dir is an existing file")
		}
	} else if (!dir.create(out.dir, showWarnings = FALSE, recursive = TRUE)) {
		stop(paste("Could not create directory", out.dir))
	}

	## Extract annotation table and methylation values
	sample.ids <- samples(rnb.set)
	tbl <- annotation(rnb.set, type = reg.type)[, c("Chromosome", "Start", "End")]
	tbl[, 2] <- tbl[, 2] - 1L # switch from 1-based closed intervals to 0-based half-open
	tbl[["Value"]] <- 0
	mm <- meth(rnb.set, type = reg.type)

	## Export to bedGraph files
	nd <- as.integer(ceiling(log10(length(sample.ids)))) + 1L
	filenames <- data.frame(
		"Sample" = sample.ids,
		"File" = sprintf(paste0("rnbeads_sample_%0", nd, "d.bedGraph"), 1:length(sample.ids)),
		stringsAsFactors = FALSE)
	for (i in 1:length(sample.ids)) {
		fname <- file.path(out.dir, filenames[i, 2L])
		if (is.null(digits)) {
			tbl[, "Value"] <- mm[, i]
		} else {
			tbl[, "Value"] <- round(mm[, i], digits = digits)
		}
		j <- which(!is.na(mm[, i]))
		cat(paste0('track type=bedGraph name="', sample.ids[i], '"', parameters, "\n"), file = fname)
		write.table(tbl[j, ], fname, TRUE, FALSE, "\t", row.names = FALSE, col.names = FALSE)
	}
	are.disjoint <- tapply(1:nrow(tbl), as.character(tbl[, 1]), function(ii) {
		(length(ii) < 2) || all(tbl[ii[-length(ii)], 3] <= tbl[ii[-1], 2])
	})
	invisible(
		list("filenames" = filenames, "assembly" = assembly(rnb.set), "contains.overlapping.regions" = !all(are.disjoint)))
}

########################################################################################################################
########################################################################################################################

## rnb.diffmeth.to.EpiExplorer.file
##
## Export the differential methylation information to a file that can be read by EpiExplorer
## @author Fabian Mueller
rnb.diffmeth.to.EpiExplorer.file <- function(rnb.set, diffmeth, fname, comp.name, reg.type, referenceSet) {
	if (!(is.character(fname) && length(fname) == 1 && (!is.na(fname)))) {
		stop("invalid value for fname")
	}
	if (!(reg.type %in% c(rnb.region.types(assembly(rnb.set)),"sites")) || !(reg.type %in% c(names(rnb.set@regions),"sites"))){
		stop("Unsupported region type")
	}
	get.type <- reg.type
	if (reg.type=="sites") {
		get.type <- "CpG"
		dmt <- get.table(diffmeth,comp.name,"sites",return.data.frame=TRUE)
		col.rank <- "combinedRank"
		col.diff <- "mean.diff"
		col.quot <- "mean.quot.log2"
		col.p.val <- "diffmeth.p.adj.fdr"
		index.map <- rnb.set@sites
	} else {
		dmt <- get.table(diffmeth,comp.name,reg.type,return.data.frame=TRUE)
		col.rank <- "combinedRank"
		col.diff <- "mean.mean.diff"
		col.quot <- "mean.mean.quot.log2"
		col.p.val <- "comb.p.adj.fdr"
		index.map <- rnb.set@regions[[reg.type]]
	}
	aa.rnbd <- rnb.get.annotation(get.type,assembly(rnb.set))

	#go over each chromosome and get the corresponding diffmeth data as GRanges object
	dm.grl <- lapply(1:length(aa.rnbd),FUN=function(cci){
		res <- aa.rnbd[[cci]]
		cur.chrom.members <- index.map[,2]==cci #get all the indexes belonging to the current chromosome
		inds <- index.map[cur.chrom.members,3]
		dm.df <- dmt[cur.chrom.members,c(col.rank,col.diff,col.quot,col.p.val)]
		dm.df <- data.frame(inRnBSet=rep(TRUE,nrow(dm.df)),dm.df)
		na.vec <- rep(NA,length(res))
		empty.df <- data.frame(inRnBSet=rep(FALSE,length(res)),col.rank=na.vec,col.diff=na.vec,col.quot=na.vec,col.p.val=na.vec)
		names(empty.df) <- c("inRnBSet",col.rank,col.diff,col.quot,col.p.val)
		if (length(inds > 0)){
			empty.df[inds,] <- dm.df
		}
		elementMetadata(res) <- empty.df
		return(res)
	})
	dm.gr <- do.call(c,dm.grl)
	df2export <- as.data.frame(elementMetadata(dm.gr))

	write.table(format(df2export,scientific=FALSE),file=fname,
			quote=FALSE,sep="\t",row.names=FALSE,col.names=TRUE,na="")
	return(fname)
}

########################################################################################################################

## create.ucsc.track.hub
##
## creates the directory structure and essential files for a UCSC track hub.
## @param hub.dir directory where the track hub should be created. Must be nonexisting.
## @param rnb.set Object of class \code{RnBSet} for which the hub is to be created.
## @param data.type either "bigBed" or "bigWig"
## @param ana.name a name for the analysis
## @param ana.desc a decription of the analysis
## @param email email address of the creator
## @return nothing of particular interest
## @details For details on UCSC track hubs see \href{http://genome.ucsc.edu/goldenPath/help/hgTrackHubHelp.html}{here}.
## @author Fabian Mueller
create.ucsc.track.hub <- function(hub.dir,rnb.set,data.type="bigBed",ana.name="RnBeads Analysis",ana.desc="Methylation data processed by RnBeads",email="-@-.com"){
	if (file.exists(hub.dir)){
		stop("Could not create UCSC Hub directory as the specified directory already exists")
	}
	if (!(data.type %in% c("bigBed","bigWig"))){
		stop("Unsupported data type")
	}
	hub.name <- "RnBeads Analysis Hub"
	#follow the steps from http://genome.ucsc.edu/goldenPath/help/hgTrackHubHelp.html
	#step1 is handled by rnb.export.to.ucsc and manual conversion to binary format
	#Step2
	dir.create(hub.dir)
	#Step3 has to be done manually
	#Step4
	fileConn<-file(file.path(hub.dir,"hub.txt"))
	writeLines(c(paste("hub",hub.name),
				 paste("shortLabel",ana.name),
				 paste("longLabel",ana.desc),
				 "genomesFile genomes.txt",
				 paste("email",email)
	),fileConn)
	close(fileConn)
	#Step5
	fileConn<-file(file.path(hub.dir,"genomes.txt"))
	writeLines(c(paste("genome",assembly(rnb.set)),
				 paste("trackDb ",assembly(rnb.set),"/trackDb.txt",sep="")
	),fileConn)
	close(fileConn)
	#Step6
	assembly.subdir <- file.path(hub.dir,assembly(rnb.set))
	if (!file.exists(assembly.subdir)){
		dir.create(assembly.subdir)
	}
	#Step7
	type.str <- data.type
	if (data.type == "bigBed"){
		type.str <- "bigBed 5 ."
	}
	lines2write <- c()
	for (i in 1:length(samples(rnb.set))){
		ss <- samples(rnb.set)[i]
		if (data.type=="bigBed"){
			lines2write <- c(lines2write,
				c(paste("track",paste("RnBeadsMethylationTrack",i,sep="")),
						paste("bigDataUrl",paste("rnbeads_sample",i,".",data.type,sep="")),
						paste("shortLabel",paste(ss)),
						paste("longLabel",paste("RnBeads methylation track for sample",ss)),
						paste("visibility","dense"),
						paste("color","0,60,120"),
						paste("spectrum","on"),
						paste("type ",type.str,"\n",sep=""))
			)
		} else if (data.type=="bigWig"){
			lines2write <- c(lines2write,
				     c(paste("track",paste("RnBeadsMethylationTrack",i,sep="")),
					 paste("bigDataUrl",paste("rnbeads_sample",i,".",data.type,sep="")),
					 paste("shortLabel",paste(ss)),
					 paste("longLabel",paste("RnBeads methylation track for sample",ss)),
					 paste("visibility","full"),
					 paste("viewLimits","0:1"), #bigWig
					 paste("viewLimitsMax","0:1"), #bigWig
					 paste("maxHeightPixels","100:30:10"), #bigWig
					 paste("color","0,60,120"),
					 paste("spectrum","on"),
	 				 paste("type ",type.str,"\n",sep=""))
			)
		}
	}
	fileConn<-file(file.path(hub.dir,assembly(rnb.set),"trackDb.txt"))
	writeLines(lines2write,fileConn)
	close(fileConn)
	#Step8
	#Maybe later
}

########################################################################################################################

#' rnb.convert.bedGraph.to.bigWig
#'
#' Converts \code{*.bedGraph} files to \code{*.bigWig} files.
#' 
#' @param files.inp  File names of the \code{*.bedGraph} files (input files).
#' @param files.out  File names of the \code{*.bigWig} files (output files).
#' @param file.chrom Existing text file storing chromosome lengths in base pairs.
#' @param file.exec  Full path to the tool bedGraphToBigWig.
#' @return Invisibly, number of successfully converted bedGraph files.
#'
#' @details Supported operating systems are currently Unix and MacOS only. The corresponding
#' 			binaries of \code{bedGraphToBigWig} are installed along with \pkg{RnBeads}. So are
#'          the chromosome sizes files.
#' @author Fabian Mueller
#' @noRd
rnb.convert.bedGraph.to.bigWig <- function(files.inp, files.out, file.chrom, file.exec) {
	result <- 0L
	for (i in 1:length(files.inp)) {
		cmd <- paste0('"', file.exec, '" "', files.inp[i], '" "', file.chrom, '" "', files.out[i], '"')
		cmd <- suppressWarnings(system(cmd, intern = TRUE, ignore.stderr = TRUE))
		result <- result + is.null(attr(cmd, "status"))
	}
	invisible(result)
}

########################################################################################################################

#' rnb.convert.bed.to.bigBed
#'
#' converts \code{*.bed} files to \code{*.bigBed} files.
#'
#' @param files.inp  File names of the \code{*.bed} files (input files).
#' @param files.out  File names of the \code{*.bigBed} files (output files).
#' @param file.chrom Existing text file storing chromosome lengths in base pairs.
#' @param file.exec  Full path to the tool bedToBigBed.
#' @return Invisibly, number of successfully converted bedGraph files.
#'
#' @author Fabian Mueller
#' @noRd
rnb.convert.bed.to.bigBed <- function(files.inp, files.out, file.chrom, file.exec){

	result <- 0L
	for (i in 1:length(files.inp)) {
		bedFn <- files.inp[i]

		## Remove the header (track line) if such exists
#		header.found <- isTRUE(grepl("^track", readLines(bedFn, n = 1)))
#		if (header.found) {
#			fileContent <- readLines(bedFn)[-1]
#			bedFn <- tempfile(pattern = "bedFile", fileext = ".bed")
#			writeLines(fileContent, bedFn)
#		}

		## Run bed to bigBed conversion
		cmd <- paste0('"', file.exec, '" "', bedFn, '" "', file.chrom, '" "', files.out[i], '"')
		cmd <- suppressWarnings(system(cmd, intern = TRUE, ignore.stderr = TRUE))
		result <- result + is.null(attr(cmd, "status"))

		## Clean up the temporary file if created
#		if (header.found) {
#			file.remove(bedFn)
#		}
	}
	invisible(result)
}

########################################################################################################################

#' rnb.export.to.trackhub
#'
#' convert an \code{\linkS4class{RnBSet}}  object to a UCSC-style track hub.
#' @param rnb.set Object of class \code{\linkS4class{RnBSet}}
#' @param out.dir output directory. If not existing, it will be created. otherwise files in that directory are overwritten.
#' @param reg.type region type to be converted
#' @param data.type either "bigBed" or "bigWig"
#' @param ... parameters passed on to the track hub generating procedure
#' @return a list containing information on the export
#' @details During execution the RnBSet is converted to bed files. If the operating system is supported (currently Unix and MacOS only)
#'          these are automatically converted to bigBed files. If your operating system is not supported, you need to create them manually
#'          (see the \href{http://genome.ucsc.edu/FAQ/FAQformat.html}{UCSC Genome Browser documentation} for details).
#'          For details on UCSC track hubs see the
#'          \href{http://genome.ucsc.edu/goldenPath/help/hgTrackHubHelp.html}{UCSC tracks help page}.
#' @author Fabian Mueller
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' rnb.export.to.trackhub(rnb.set.example,tempdir())
#' }
rnb.export.to.trackhub <- function(rnb.set,out.dir,reg.type="sites",data.type="bigBed",...){
	## Validate paramter values
	if (!inherits(rnb.set, "RnBSet")) {
		stop("Invalid value for rnb.set: Expected RnBSet")
	}
	if (!(is.character(out.dir) && length(out.dir) == 1 && isTRUE(out.dir != ""))) {
		stop("Invalid value for out.dir")
	}
	if (!file.exists(out.dir)) {
		if (!dir.create(out.dir, FALSE, TRUE)) {
			rnb.error(c("Could not create directory", out.dir))
		}
	}
	if (!(is.character(reg.type) && length(reg.type) == 1 && isTRUE(reg.type != ""))) {
		stop("Invalid value for reg.type")
	}
	if (!(reg.type %in% c("sites", rnb.region.types(assembly(rnb.set))))) {
		stop(paste("Unsupported region type:", reg.type))
	}
	if (!(is.character(data.type) && length(data.type) == 1 && isTRUE(data.type != ""))) {
		stop("Invalid value for data.type")
	}
	if (!(data.type %in% c("bigBed","bigWig"))) {
		stop("Unsupported data type")
	}

	res <- list(assembly = assembly(rnb.set))
	if (data.type=="bigBed") {

		## Skip if executable is not found
		file.exec <- rnb.get.executable("bedToBigBed")
		if (length(file.exec) == 0) {
			txt <- paste("Skipped conversion bed -> bigBed for region type", reg.type)
			txt <- paste0(txt, "; could not find the tool bedToBigBed")
			rnb.warning(txt)
			res[["converted.bigBed"]] <- FALSE
			return(res)
		}
		logger.info(paste("Using", file.exec))

		## Expport all samples to bed files
		track.hub.dir <- file.path(out.dir, "trackHub_bigBed")
		bed.dir <- tempdir()
		rnb.logger.start("Conversion to BED")
		bed.conv <- rnb.RnBSet.to.bed(rnb.set, bed.dir, reg.type, inherits(rnb.set, "RnBiseqSet"), FALSE, TRUE, FALSE)
		rnb.logger.completed()

		## Prepare for conversion to bigWig
		file.chrom <- file.path(out.dir, "chromosomes.txt")
		in.file.list <- file.path(bed.dir, bed.conv$filenames$filename)
		out.file.list <- paste0("rnbeads_sample", bed.conv$filenames$i, ".bigBed")
		out.file.list <- file.path(track.hub.dir, assembly(rnb.set), out.file.list)
		res[["filenames.bed"]] <- bed.conv$filenames

		#@ Write the UCSC track hub structure
		rnb.logger.start("Creating Track Hub")
		create.ucsc.track.hub(track.hub.dir, rnb.set, data.type="bigBed", ...)

		## Convert bed to bigBed
		if (!file.exists(file.chrom)) {
			rnb.chromosome.lengths(rnb.set@assembly, file.chrom)
		}
		rnb.convert.bed.to.bigBed(in.file.list, out.file.list, file.chrom, file.exec)
		res[["converted.bigBed"]] <- TRUE
		rnb.logger.completed()

	} else if (data.type=="bigWig") {

		## Skip if executable is not found
		file.exec <- rnb.get.executable("bedGraphToBigWig")
		if (length(file.exec) == 0) {
			txt <- paste("Skipped conversion bedGraph -> bigWig for region type", reg.type)
			txt <- paste0(txt, "; could not find the tool bedGraphToBigWig")
			rnb.warning(txt)
			res[["converted.bigWig"]] <- FALSE
			return(res)
		}
		logger.info(paste("Using", file.exec))

		## Expport all samples to bedGraph files
		file.chrom <- file.path(out.dir, "chromosomes.txt")
		track.hub.dir <- file.path(out.dir, "trackHub_bigWig")
		bedgraph.dir <- file.path(out.dir, "bedgraph")
		rnb.logger.start("Conversion to bedGraph")
		bedGraph.conv <- rnb.RnBSet.to.bedGraph(rnb.set, bedgraph.dir, reg.type=reg.type)
		rnb.logger.completed()

		## Prepare for conversion to bigWig
		in.file.list <- file.path(bedgraph.dir, bedGraph.conv$filenames$File)
		out.file.list <- gsub("bedGraph$", "bigWig", bedGraph.conv$filenames$File)
		out.file.list <- file.path(track.hub.dir, assembly(rnb.set), out.file.list)

		## Skip if overlapping regions are found
		res[["contains.overlapping.regions"]] <- bedGraph.conv[["contains.overlapping.regions"]]
		res[["filenames.bedGraph"]] <- bedGraph.conv$filenames
		if (res[["contains.overlapping.regions"]]) {
			txt <- paste("Skipped conversion bedGraph -> bigWig for region type", reg.type)
			txt <- paste0(txt, "; overlapping fragments found")
			rnb.warning(txt)
			res[["converted.bigWig"]] <- FALSE
			unlink(bedgraph.dir, recursive = TRUE, force = TRUE)
			return(res)
		}

		## Write the UCSC track hub structure
		rnb.logger.start("Creating Track Hub")
		create.ucsc.track.hub(track.hub.dir, rnb.set, data.type="bigWig", ...)

		## Convert bedGraph to bigWig
		if (!file.exists(file.chrom)) {
			rnb.chromosome.lengths(rnb.set@assembly, file.chrom)
		}
		rnb.convert.bedGraph.to.bigWig(in.file.list,out.file.list,file.chrom,file.exec)
		res[["converted.bigWig"]] <- TRUE

		## Delete bedGraph files after successful conversion
		unlink(bedgraph.dir, recursive = TRUE, force = TRUE)
		rnb.logger.completed()
	}

	invisible(res)
}

########################################################################################################################
########################################################################################################################

#' rnb.export.to.ewasher
#'
#' Data exported to a format compatible with the FaST-LMM-EWASher tool for cell-mixture adjustment.
#' see \href{http://www.nature.com/nmeth/journal/v11/n3/full/nmeth.2815.html}{Zou, J., et al., Nature Methods, 2014}
#' for further details on the tool.
#' @param rnb.set Object of class \code{\linkS4class{RnBSet}}
#' @param out.dir output directory. If not existing, it will be created and all exported files will be placed here.
#' 				  If existing, this functions results in an error.
#' @param reg.type region type to be exported
#' @param ... passed on to \code{get.comparison.info}
#' @return a list containing information on the export
#' @author Fabian Mueller
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' rnb.export.to.ewasher(rnb.set.example,tempfile(pattern="forEwasher"))
#' }
rnb.export.to.ewasher <- function(rnb.set, out.dir, reg.type="sites", ...){
	#example: rnb.export.to.ewasher(rnb.set,out.dir="~/tmp/rnb4ewasher",pheno.cols=rnb.getOption("differential.comparison.columns"))
	res <- list()
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set; expected RnBSet")
	}
	if (file.exists(out.dir)){
		stop("Could not create output directory as the specified directory already exists")
	}
	dir.create(out.dir)

	data.dir <- file.path(out.dir,"data_file")
	dir.create(data.dir)
	pheno.dir <- file.path(out.dir,"phenotype_file")
	dir.create(pheno.dir)
	covar.dir <- file.path(out.dir,"covar_file")
	dir.create(covar.dir)
	map.dir <- file.path(out.dir,"map_file")
	dir.create(map.dir)
	res[["data.subdir"]] <- "data_file"
	res[["pheno.subdir"]] <- "phenotype_file"
	res[["covar.subdir"]] <- "covar_file"
	res[["map.subdir"]] <- "map_file"


	cmp.info <- get.comparison.info(rnb.set,adjust.celltype=FALSE,...)

	mm <- meth(rnb.set,type=reg.type)
	aa <- annotation(rnb.set,type=reg.type)
	site.ids <- rownames(aa)
	sample.ids <- samples(rnb.set)
	# check if the sample ids start with a letter. If not prepend "S_"
	# because EWASher has problems with sample names starting with non-letter characters
	illegal.sample.id <- !grepl("^[[:alpha:]]", sample.ids)
	sample.ids[illegal.sample.id] <- paste0("S_",sample.ids[illegal.sample.id])

	cmp.ids   <- paste0("cmp",1:length(cmp.info))
	cmp.desc <- sapply(cmp.info,FUN=function(x){x$comparison})
	tab.summary <- data.frame(
		comparison_id=cmp.ids,
		comparison_desc=cmp.desc,
		stringsAsFactors=FALSE)
	res[["comparison.table"]] <- tab.summary

	#write the methylation data
	logger.start("Exporting methylation data")
	tab.data <- data.frame(
		ID=site.ids,
		mm
	)
	colnames(tab.data) <- c("ID",sample.ids)
	fn <- file.path(data.dir,"methylation.tsv")
	write.table(tab.data, file=fn,quote=FALSE,sep="\t",row.names=FALSE,col.names=TRUE)
	res[["data.files"]] <- rep("methylation.tsv",length(cmp.info))
	logger.completed()

	#write the location mapping data
	logger.start("Exporting genome mapping data")
	map.data <- data.frame(
		chrom=gsub("chr", "", aa$Chromosome),
		ID=site.ids,
		chromStart=aa$Start,
		chromEnd=aa$End-1
	)
	fn <- file.path(map.dir,"mapping.map")
	write.table(map.data, file=fn,quote=FALSE,sep="\t",row.names=FALSE,col.names=FALSE)
	res[["map.files"]] <- rep("mapping.map",length(cmp.info))
	logger.completed()

	res[["pheno.files"]] <- rep(NA,length(cmp.info))
	res[["covar.files"]] <- rep(NA,length(cmp.info))
	for (i in 1:length(cmp.info)){
		logger.start(c("Exporting data for comparison:",cmp.info[[i]]$comparison))
		#write the phenotype data
		logger.status("Exporting phenotype table...")
		grps <- rep(NA,length(sample.ids))
		grps[cmp.info[[i]]$group.inds$group1] <- 1
		grps[cmp.info[[i]]$group.inds$group2] <- 0
		tab.pheno <- data.frame(
			sample=sample.ids,
			pheno=grps
		)
		fnn <- paste0("pheno_",cmp.ids[i],".tsv")
		fn <- file.path(pheno.dir,fnn)
		write.table(tab.pheno, file=fn,quote=FALSE,sep="\t",row.names=FALSE,col.names=FALSE)
		res[["pheno.files"]][i] <- fnn

		#covariates
		pairing.info <- NULL
		covariates <- cmp.info[[i]]$adjustment.table
		if (cmp.info[[i]]$paired){
			pairing.info <- rep(NA,length(sample.ids))
			#the sample indices are assumed to be in pairing order and the number of samples in each group are equal
			num.pairs <- length(cmp.info[[i]]$group.inds$group1)
			pairing.info[cmp.info[[i]]$group.inds$group1] <- paste0("pair",1:num.pairs)
			pairing.info[cmp.info[[i]]$group.inds$group2] <- paste0("pair",1:num.pairs)
			pairing.info <- as.factor(pairing.info)
			if (is.null(covariates)){
				covariates <- data.frame(matrix(NA,nrow=length(sample.ids),ncol=0))
			}
			covariates <- data.frame(covariates,pairs=pairing.info)
		}
		if (!is.null(covariates)){
			logger.status("Exporting covariate table...")
			opts.na.orig <- options("na.action")[[1]] #for converting the dataframe and keeping NA's temporarily change the NA action
			options(na.action='na.pass')
			covariates <- model.matrix(~0+.,data=covariates)
			options(na.action=opts.na.orig)
			tab.covar <- data.frame(
				intercept=1,
				sample=sample.ids,
				covariates
			)
			fnn <- paste0("covar_",cmp.ids[i],".tsv")
			fn <- file.path(covar.dir,fnn)
			write.table(tab.covar, file=fn,quote=FALSE,sep="\t",row.names=FALSE,col.names=FALSE)
			res[["covar.files"]][i] <- fnn
		}
		logger.completed()
	}

	fn <- file.path(out.dir,"summary.tsv")
	write.table(tab.summary, file=fn,quote=FALSE,sep="\t",row.names=FALSE,col.names=TRUE)
	return(res)
}

########################################################################################################################

### rnb.section.export.ct.adj
###
### add information to the report for export
### @author Fabian Mueller
### @aliases rnb.section.export
### @param ewasher.obj Ewasher export results object. See \code{\link{rnb.export.to.ewasher}} for details.
### @param rnbSet RnBSet object
### @param report report object to which the content is added
### @return the updated report object
rnb.section.export.ct.adj <- function(ewasher.obj,rnbSet,report){
	res.ewasher <- ewasher.obj
	ewasher.dir <- file.path(rnb.get.directory(report, "data"),"sites","ewasher")
	sectionText <- c("Site-level methylation and differential methylation data has been exported to formats compatible with tools for ",
		"cell-mixture adjustment.")
	report <- rnb.add.section(report, 'Export to Other Cell Type Adjustment Tools', sectionText)
	sectionText <- c("In order to analyze the exported data with FaST-LMM-EWASher, first obtain either the ",
		"<a href=\"http://research.microsoft.com/en-us/downloads/472fe637-7cb9-47d4-a0df-37118760ccd1/\">R-version</a> ",
		"or the <a href=http://research.microsoft.com/en-us/downloads/8af2ab12-a205-4bbb-809c-a333ecaffa40/>Python-version</a> ",
		"of the tool and install it on your machine. ",
		"Note that the FaST-LMM-EWASher software is currently in development. Hence, installation (particularly on non-Windows-computers) ",
		"is a non-trivial task. We recommend to contact the FaST-LMM-EWASher developers in case of technical difficulties. ",
		"RnBeads exported files that can serve as an input to FaST-LMM-EWASher. ",
		"These input files can be found in the ",paste("<a href=\"",ewasher.dir,"\">data directory</a> ",sep=""),
		"accompanying this report. The following table contains the filenames for the input data for each differential methylation comparison:")
	report <- rnb.add.section(report, 'FaST-LMM-EWASher', sectionText, level=2)
	data.file.dir <- file.path(rnb.get.directory(report, "data"),"sites","ewasher",res.ewasher$data.subdir)
	data.file.links <- paste("<a href=\"",file.path(data.file.dir,res.ewasher$data.files),"\">",res.ewasher$data.files,"</a>",sep="")
	map.file.dir <- file.path(rnb.get.directory(report, "data"),"sites","ewasher",res.ewasher$map.subdir)
	map.file.links <- paste("<a href=\"",file.path(map.file.dir,res.ewasher$map.files),"\">",res.ewasher$map.files,"</a>",sep="")
	pheno.file.dir <- file.path(rnb.get.directory(report, "data"),"sites","ewasher",res.ewasher$pheno.subdir)
	pheno.file.links <- paste("<a href=\"",file.path(pheno.file.dir,res.ewasher$pheno.files),"\">",res.ewasher$pheno.files,"</a>",sep="")
	covar.file.dir <- file.path(rnb.get.directory(report, "data"),"sites","ewasher",res.ewasher$covar.subdir)
	covar.file.links <- ifelse(!is.na(res.ewasher$covar.files),
		paste("<a href=\"",file.path(covar.file.dir,res.ewasher$covar.files),"\">",res.ewasher$covar.files,"</a>",sep=""),
		"N/A")
	tt.ewasher <- data.frame(
		comparison=res.ewasher$comparison.table$comparison_desc,
		comparison_id=res.ewasher$comparison.table$comparison_id,
		data_file=data.file.links,
		pheno_file=pheno.file.links,
		map_file=map.file.links,
		covar_file=covar.file.links,
		stringsAsFactors=FALSE
	)
	rnb.add.table(report,tt.ewasher)

	sectionText <- c("To run the Python-version of FaST-LMM-EWASher with the output file use the following syntax for your shell or console ",
		"commands:")
	rnb.add.paragraph(report, sectionText)
	cmdSyntax <- paste0("<code>","python fastlmm-ewasher.py data_file pheno_file [-covar covar_file] [-map map_file]","</code>")
	rnb.add.paragraph(report,cmdSyntax)
	sectionText <- c("To run the R-version of FaST-LMM-EWASher with the output file use the following syntax for your shell or console ",
		"commands:")
	rnb.add.paragraph(report, sectionText)
	cmdSyntax <- paste0("<code>","Rscript FLE-script.r data_file pheno_file [-covar covar_file] [-map map_file]","</code>")
	rnb.add.paragraph(report,cmdSyntax)
	return(report)
}

### rnb.section.tnt
###
### add information to the report for export
### @author Fabian Mueller
### @aliases rnb.section.export
### @param res.exp Exporting results object. See \code{\link{rnb.execute.tnt}} for details.
### @param rnbSet RnBSet object
### @param report report object to which the content is added
### @return the updated report object
rnb.section.tnt <- function(res.exp,rnbSet,report){
	if (is.null(res.exp)){
		sectionText <- "Nothing was exported"
		report <- rnb.add.section(report, 'Nothing Exported', sectionText)
	} else {
		is.biseq <- "RnBiseqSet" %in% class(rnbSet)
		is.beads <- "RnBeadSet" %in% class(rnbSet)
		export.ucsc.types <- unique(unlist(lapply(res.exp,FUN=function(x){x$export.trackhub})))
		any.export.bed <- any(unlist(lapply(res.exp,FUN=function(x){x$export.bed})))
		export.ewasher <- is.element("sites",names(res.exp))
		if (export.ewasher) export.ewasher <- is.element("export.ewasher",names(res.exp$sites))
		sectionText <- "Methylation Data was exported to the following formats:<ul>\n"
		if (any.export.bed){
			sectionText <- paste(sectionText,"<li><a href=\"http://genome.ucsc.edu/FAQ/FAQformat.html#format1\">bed</a></li>")
		}
		if ("bigBed" %in% export.ucsc.types){
			sectionText <- paste(sectionText,"<li><a href=\"http://genome.ucsc.edu/goldenPath/help/hgTrackHubHelp.html\">Track hub</a>: bigBed</li>")
		}
		if ("bigWig" %in% export.ucsc.types){
			sectionText <- paste(sectionText,"<li><a href=\"http://genome.ucsc.edu/goldenPath/help/hgTrackHubHelp.html\">Track hub</a>: bigWig</li>")
		}
		if (export.ewasher){
			refText <- c("Zou, J., Lippert, C., Heckerman, D., Aryee, M., Listgarten, J. (2014). ",
				"Epigenome-wide association studies without the need for cell-type composition. ",
				"<i>Nature Methods</i>, 11(3), 309-311")
			report <- rnb.add.reference(report, refText)
			sectionText <- paste(sectionText, "<li>Input for the FaST-LMM-EWASher tool for cell-mixture adjustment ",
				rnb.get.reference(report, refText), "</li>")
		}
		sectionText <- paste(sectionText,"</ul>")
		report <- rnb.add.section(report, 'Methylation Data', sectionText)
		summary.tab <- rnb.sample.summary.table(rnbSet)
		fname <- paste("sample_methylation_summary.csv",sep="")
		fname <- rnb.write.table(summary.tab,fname,fpath=rnb.get.directory(report, "data", absolute = TRUE),format="csv",gz=FALSE,row.names = FALSE,quote=FALSE)

		sectionText <- paste("This section contains summary metrics for each sample as well as links to exported files. The summary table below contains the following metrics. It is also available as ",
			"<a href=\"", rnb.get.directory(report, "data"), "/", fname,"\">csv file</a>.",sep="")
		column.description <- c("<ul>",
									"<li>sampleName: Name of the sample</li>",
									"<li>*_num (* can be 'sites' or a region type): Number of sites or regions with coverage in the sample</li>")
		if (is.biseq){
			column.description <- c(column.description,
									"<li>*_covgMean: Mean coverage of sites or regions in the sample</li>",
									"<li>*_covgMedian: Median coverage of sites or regions in the sample</li>",
									"<li>*_covgPerc25,75: 25 and 75 percentile of coverage of sites or regions in the sample</li>",
									"<li>*_numCovg5,10,30,60: Number of sites or regions with coverage greater or equal to 5,10,30,60</li>")
		}
		if (is.beads){
			if (!is.null(dpval(rnbSet))){
				column.description <- c(column.description,
										"<li>sites_numDPval5em2,1em2,1em3: Number of sites with a detection p-value smaller than 0.05,0.01,0.001</li>")
			}
		}
		column.description <- c(column.description,
									"<li>**_numSitesMean (** is any region type): Mean number of sites in a region</li>",
									"<li>**_numSitesMedian (** is any region type): Median number of sites in a region</li>",
									"<li>**_numSites2,5,10,20: Number of regions with at least 2,5,10,20 sites with valid methylation measurements</li>",
							   "</ul>")

		sectionText <- paste(sectionText,paste(column.description,collapse="\n"))
		if (any.export.bed){
			sectionText <- c(sectionText," Bed files for each sample contain the locations of methylation sites and regions the methylation level
						   (in the score column). The following table links to the files. The files can be quite large in size, so you might want to
							consider using the \"save as\" option instead of opening the links directly.")
			tt <- data.frame(sample=as.character(res.exp[[1]]$filenames.bed[,"sample"]),stringsAsFactors=FALSE)
			for (i in 1:length(res.exp)){
				rr <- names(res.exp)[i]
				ttt <- res.exp[[i]]$filenames.bed[,c("sample","filename")]
				ttt$filename <- paste("<a href=\"",file.path(rnb.get.directory(report, "data"),rr,"bed",ttt$filename),"\">","bed","</a>",sep="")
				tt <- data.frame(tt,ttt$filename,stringsAsFactors=FALSE)
			}
			colnames(tt)[2:ncol(tt)] <- paste0(names(res.exp),"_bedFile")
			rownames(tt) <- tt$sample
			bed.df <- data.frame(tt[summary.tab$sampleName,2:ncol(tt)],stringsAsFactors=FALSE)
			colnames(bed.df) <- colnames(tt)[2:ncol(tt)]
			summary.tab <- data.frame(sampleName=summary.tab$sampleName,bed.df,summary.tab[,2:ncol(summary.tab)],stringsAsFactors=FALSE)
		}
		if ("bigBed" %in% export.ucsc.types){
			tt <- data.frame(sample=as.character(res.exp[[1]]$filenames.bed[,"sample"]),stringsAsFactors=FALSE)
			for (i in 1:length(res.exp)){
				rr <- names(res.exp)[i]
				ttt <- res.exp[[i]]$filenames.bed[,c("sample","filename")]
				ttt$filename <- gsub(".bed$",".bigBed",ttt$filename) #replace bed with bigBed
				ttt$filename <- paste("<a href=\"",file.path(rnb.get.directory(report, "data"),rr,"trackHub_bigBed",assembly(rnbSet),ttt$filename),"\">","bigBed","</a>",sep="")
				tt <- data.frame(tt,ttt$filename,stringsAsFactors=FALSE)
			}
			colnames(tt)[2:ncol(tt)] <- paste0(names(res.exp),"_bigBedFile")
			rownames(tt) <- tt$sample
			bigbed.df <- data.frame(tt[summary.tab$sampleName,2:ncol(tt)],stringsAsFactors=FALSE)
			colnames(bigbed.df) <- colnames(tt)[2:ncol(tt)]
			summary.tab <- data.frame(sampleName=summary.tab$sampleName,bigbed.df,summary.tab[,2:ncol(summary.tab)],stringsAsFactors=FALSE)
		}

		report <- rnb.add.section(report, "Sample summary", sectionText, level=2)
		rnb.add.table(report,summary.tab,row.names=FALSE)

		if (length(export.ucsc.types)>0){

			sectionText <- paste("Track Hub data was generated for export to various genome browsers. Note that you need a server that is capable of
							serving the tracks to the genome browser via URL. Below, instructions are provided to view the tracks in the UCSC genome browser. Of course the files can also be viewed in other browsers such as the
							<a href=\"http://www.ensembl.org/\">Ensembl genome browser</a>.
							<ol>\n")
			anything.unconverted <- FALSE
			if ("bigBed" %in% export.ucsc.types) {
				if (!res.exp[[1]]$converted.bigBed) {
					anything.unconverted <- TRUE
				}
			}
			if ("bigWig" %in% export.ucsc.types) {
				if (!res.exp[[1]]$converted.bigWig) {
					anything.unconverted <- TRUE
				}
			}
			if (anything.unconverted) {
				sectionText <- paste(sectionText,"<li>Convert the bed/bedGraph files contained in the bed/bedGraph directories (see table below) attached to this report.",
									 "You can use the UCSC tool <code>bedTobigBed</code>/<code>bedGraphTobigWig</code> for this task. More information (e.g. where to obtain the tool)
									 on how to convert can be found <a href=\"http://genome.ucsc.edu/goldenPath/help/bigBed.html\">here</a>.",
									 "Make sure the resulting bigBed/bigWig files are moved to the corresponding UCSC track hub directory (see table(s) below).</li>")
			}
			sectionText <- paste(sectionText,"<li>Make sure you are viewing this report via the internet and not locally (simply look at the URL in the address line
							of your browser). If this is not the case, copy the files belonging to this report
							to a directory which is accessible via the internet and enter the corresponding URL in your browser.
							If you don't want to copy the entire report, it suffices to copy the track hub directories (see table(s) below).</li>")
			sectionText <- paste(sectionText,"<li>Make sure the",res.exp[[1]][["assembly"]],"subdirectories",
								"in the track hub directory (for its location be referred to the table(s) below) contain all the required bigBed/bigWig files</li>")
			sectionText <- paste(sectionText,"<li>Go to the <a href=\"http://genome.ucsc.edu/cgi-bin/hgHubConnect\">UCSC hub connection website</a>
							to the \"My Hubs\" tab and add the web-URL of the <code>hub.txt</code> file in the directory you just copied.
							Hint: You can use the copy link functionality of your browser (right mouse click) on the track hub txt column in the table(s) below.
							Don't forget to click on \"Load Selected Hubs\".</li>")
			sectionText <- paste(sectionText,paste("<li>Make sure you are looking at the correct genome (",res.exp[[1]][["assembly"]],").
							The loaded hubs should now appear below the browsing window where you can modify their display settings.</li>",sep=""))
			sectionText <- paste(sectionText,"</ol>\nMore information on UCSC track hubs can be found
						   <a href=\"http://genome.ucsc.edu/goldenPath/help/hgTrackHubHelp.html\">here</a>. Below, you find table(s) containing the directory names
						   for the bed/bedGraph and track hub directories for all exported data, site and region types.\n")
			report <- rnb.add.section(report, 'Tracks Hubs', sectionText, level=2)

			rrs <- names(res.exp)
			if ("bigBed" %in% export.ucsc.types){
				track.hub.dirs <- file.path(rnb.get.directory(report, "data"),rrs,"trackHub_bigBed")
				track.hub.urls <- file.path(rnb.get.directory(report, "data"),rrs,"trackHub_bigBed","hub.txt")
				bed.dirs <- file.path(rnb.get.directory(report, "data"),rrs,"bed")
				track.hub.dirs.links <- paste("<a href=\"",track.hub.dirs,"\">",track.hub.dirs,"</a>",sep="")
				track.hub.urls.links <- paste("<a href=\"",track.hub.urls,"\">",track.hub.urls,"</a>",sep="")
				bed.dirs.links <- paste("<a href=\"",bed.dirs,"\">",bed.dirs,"</a>",sep="")
				tt.bb <- data.frame("bed directory"=bed.dirs.links,"track hub directory"=track.hub.dirs.links,"track hub txt"=track.hub.urls.links)
				rownames(tt.bb) <- rrs
				report <- rnb.add.section(report, "bigBed", NULL, level=3)
				rnb.add.table(report,tt.bb)
			}
			if ("bigWig" %in% export.ucsc.types){
				regions.contain.overlap <- unlist(lapply(res.exp,FUN=function(x){
					base::ifelse(is.null(x[["contains.overlapping.regions"]]),FALSE,x[["contains.overlapping.regions"]])
				}))

				if (sum(!regions.contain.overlap)>0){
					track.hub.dirs <- file.path(rnb.get.directory(report, "data"),rrs[!regions.contain.overlap],"trackHub_bigWig")
					track.hub.urls <- file.path(rnb.get.directory(report, "data"),rrs[!regions.contain.overlap],"trackHub_bigWig","hub.txt")
					track.hub.dirs.links <- paste("<a href=\"",track.hub.dirs,"\">",track.hub.dirs,"</a>",sep="")
					track.hub.urls.links <- paste("<a href=\"",track.hub.urls,"\">",track.hub.urls,"</a>",sep="")
					tt.bw <- data.frame("track hub directory"=track.hub.dirs.links,"track hub txt"=track.hub.urls.links)
					if (!res.exp[[1]]$converted.bigWig){
						bed.dirs <- file.path(rnb.get.directory(report, "data"),rrs,"bedgraph")
						bed.dirs.links <- paste("<a href=\"",bed.dirs,"\">",bed.dirs,"</a>",sep="")
						tt.bw <- data.frame("bedgraph directory"=bed.dirs.links,tt.bw)
					}
					rownames(tt.bw) <- rrs[!regions.contain.overlap]
					report <- rnb.add.section(report, "bigWig", NULL, level=3)
					rnb.add.table(report,tt.bw)
				}
			}
		}
	}
	return(report)
}

########################################################################################################################

#' rnb.execute.tnt
#'
#' export RnBSet to various output data formats
#' @author Fabian Mueller
#' @aliases rnb.execute.tnt
#' @param rnb.set \code{\linkS4class{RnBSet}} object
#' @param out.dir output directory.
#' @param exp.bed A character vector indicating which data types should be exported to UCSC. Possible values in the vector are \code{bigBed} and \code{bigWig}.
#' 				  If \code{NULL}, UCSC export is disabled
#' @param exp.trackhub file types which should be exported to a trackhub structure.
#' @param region.types a character vector indicating region types to be exported
#' @param ... Arguments passed to \code{\link{rnb.export.to.trackhub}}
#' @return a list containing information on the export
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' rnb.execute.tnt(rnb.set.example,tempdir())
#' }
rnb.execute.tnt <- function(rnb.set,out.dir,exp.bed=rnb.getOption("export.to.bed"),
		exp.trackhub=rnb.getOption("export.to.trackhub"),region.types=rnb.getOption("export.types"),...){
	logger.start("Generating Tracks and Tables")
	reg.types <- intersect(region.types,c(rnb.region.types(assembly(rnb.set)),"sites"))
	if (length(reg.types)<1){
		stop("Region types not found")
	}
	if (length(reg.types) < length(region.types)){
		logger.warning("Not all region types were found in the annotation")
	}
	ress <- list()
	for (rr in reg.types){
		logger.start(c("Exporting",rr))
		out.dir.cur <- file.path(out.dir,rr)
		dir.create(out.dir.cur)
		res <- list(export.bed = exp.bed, export.trackhub = character())
		if (exp.bed) {
			logger.start("Creating BED Files")
			bed.dir <- file.path(out.dir.cur,"bed")
			res.exp <- rnb.RnBSet.to.bed(rnb.set,bed.dir,reg.type=rr,names.quant.meth=TRUE)
			logger.completed()
			res[["filenames.bed"]] <- res.exp[["filenames"]]
		}
		if ("bigBed" %in% exp.trackhub) {
			logger.start("Creating Track Hub -- bigBed")
			res.exp <- rnb.export.to.trackhub(rnb.set,out.dir.cur,reg.type=rr,
							ana.name=paste("RnBeads Analysis -- ",rr,",bigBed",sep=""),
							ana.desc=paste(rr, "methylation (bigBed)"),data.type="bigBed",...)
			logger.completed()
			res <- c(res,res.exp)
			res[["export.trackhub"]] <- c(res[["export.trackhub"]],"bigBed")
		}
		if ("bigWig" %in% exp.trackhub){
			logger.start("Creating UCSC Track Hub -- bigWig")
			res.exp <- rnb.export.to.trackhub(rnb.set,out.dir.cur,reg.type=rr,
					ana.name=paste0("RnBeads Analysis -- ",rr,",bigWig"),
					ana.desc=paste(rr, "methylation (bigWig)"),data.type="bigWig",...)
			logger.completed()
			res <- c(res,res.exp)
			if (res[["converted.bigWig"]]) {
				res[["export.trackhub"]] <- c(res[["export.trackhub"]],"bigWig")
			}
		}
		ress <- c(ress,list(res))
		logger.completed()
	}
	names(ress) <- reg.types
	logger.completed()
	invisible(ress)
}

########################################################################################################################
########################################################################################################################

## rnb.execute.export.csv.save
##
## Attempts to save the methylation values matrix of the given region type to the specified file.
##
## @param rnb.set         Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
## @param region.type     Site or region type to be exported.
## @param output.location Target file path.
## @param fname           Target file name. If this file already exists, it will be overwritten.
## @param gz              Flag indicating whether the file should be zipped in \code{gz} format.
## @return The (possibly updated) target file name, invisibly. If \code{gz} is \code{TRUE}, the string \code{".gz"} will
##         be appended to \code{fname}.
##
## @seealso \code{\link{rnb.write.table}}
## @author Yassen Assenov
rnb.execute.export.csv.save <- function(rnb.set, region.type, output.location, fname,
	gz = rnb.getOption("gz.large.files")) {
	mm <- meth(rnb.set, region.type, row.names = TRUE)
	reg.annotation <- annotation(rnb.set, region.type)
	accepted.columns <- tolower(colnames(reg.annotation)) %in% c("chrom", "chromosome", "start", "end", "strand")
	mm <- cbind(data.frame(ID = rownames(mm), check.names = FALSE, stringsAsFactors = FALSE),
		reg.annotation[, accepted.columns], mm)
	rnb.write.table(mm, fname, output.location, gz = gz, row.names = FALSE, quote = FALSE)
}

########################################################################################################################

#' rnb.execute.export.csv
#'
#' Exports (selected) methylation tables of the given dataset to comma-separated value files.
#'
#' @param rnb.set         Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param output.location \code{character} or \code{\linkS4class{Report}} specifying the output directory. If this is a
#'                        report, the output directory is set to be a subdirectory named \code{csv} of the report's data
#'                        directory. Set this parameter to the empty string (\code{""}) or \code{NA} to use the current
#'                        working directory. If the given path does not exist, this function attempts to create it.
#' @param region.types    \code{character} vector indicating region types to be exported.
#' @return \code{character} vector containing the names of the files to which data were exported; prepended by
#'         \code{output.location}. In case a certain region type could not be exported (see the \emph{Details} section),
#'         the corresponding element of this vector is \code{NA}.
#'
#' @details
#' The names of the generated output files are formed by the prefix \code{"betas_"}, followed by a number between
#' 1 and \code{length(region.types)}. The extension is \code{.csv} or \code{.csv.gz}, depending on the value of the
#' \pkg{RnBeads} option \code{"gz.large.files"}. Any such files that already exist in the output directory, are
#' overwritten.
#'
#' There are several reasons why a certain output file cannot be (fully) generated. Examples for failures are listed
#' below:
#' \itemize{
#'   \item{}{The corresponding region type is invalid.}
#'   \item{}{The corresponding region type is not supported by the dataset. If the type is loaded in \pkg{RnBeads},
#'           use the \code{\link[=summarize.regions,RnBSet-method]{summarize.regions}} method prior to calling this function,
#'           in order to include the support of this region type in the dataset.}
#'   \item{}{Due to security restrictions, the creation of files in the output directory is not allowed.}
#'   \item{}{A file or directory with the same name exists and cannot be overwritten.}
#'   \item{}{The disk is full or the user quota is exceeded.}
#' }
#'
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' rnb.execute.export.csv(rnb.set.example, "", summarized.regions(rnb.set.example))
#' }
#' @author Yassen Assenov
#' @export
rnb.execute.export.csv <- function(rnb.set, output.location, region.types = rnb.getOption("export.types")) {
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	if (inherits(output.location, "Report")) {
		fprefix <- paste0(rnb.get.directory(output.location, "data", absolute = FALSE), "/csv/")
		output.location <- file.path(rnb.get.directory(output.location, "data", absolute = TRUE), "csv")
	} else {
		if (!(length(output.location) == 1 && (is.character(output.location) || is.na(output.location)))) {
			stop("invalid value for output.location; expected Report or character of length 1")
		}
		if (is.na(output.location) || output.location == ".") {
			output.location <- ""
		} else if (output.location != "") {
			output.location <- sub("^(.+)[/\\\\]$", "\\1", output.location) # remove trailing slash or backslash
		}
		fprefix <- ifelse(output.location == "", "", paste0(gsub("\\", "/", output.location, fixed = TRUE), "/"))
	}
	if (is.null(region.types)) {
		region.types <- character()
	} else if (!(is.character(region.types))) {
		stop("invalid value for region.types")
	}
	if (output.location != "") {
		if (!file.exists(output.location)) {
			## Attempt to create output.location
			if (!dir.create(output.location, showWarnings = FALSE, recursive = TRUE)) {
				rnb.error(c("Could not create directory", output.location))
			}
		} else if (!file.info(output.location)[1, "isdir"]) {
			rnb.error(c("Specified path is not a directory:", output.location))
		}
	}

	result <- region.types
	names(result) <- region.types
	if (length(region.types) != 0) {
		for (i in 1:length(region.types)) {
			fname <- paste0("betas_", i, ".csv")
			fname <- tryCatch(
				rnb.execute.export.csv.save(rnb.set, region.types[i], output.location, fname),
				error = function(e) { return(as.character(NA)) })
			if (!is.na(fname)) {
				fname <- paste0(fprefix, fname)
			}
			result[i] <- fname
		}
	}
	return(result)
}

########################################################################################################################

#' rnb.section.export.csv
#'
#' Creates a report section dedicated to exporting (selected) methylation tables to comma-separated value files.
#'
#' @param report         Report on data export to contain the CSV section. This must be an object of type
#'                       \code{\linkS4class{Report}}.
#' @param export.summary Result of the exporting procedure, as returned by \code{\link{rnb.execute.export.csv}}.
#' @return The (possibly) modified report.
#'
#' @author Yassen Assenov
#' @noRd
rnb.section.export.csv <- function(report, export.summary) {
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	if (!(is.character(export.summary) && (!is.null(names(export.summary))))) {
		stop("invalid value for export.summary")
	}
	if (length(export.summary) == 0) {
		return(report)
	}

	txt <- "This section summarizes the result of exporting methylation data to comma-separated value (CSV) files."
	report <- rnb.add.section(report, "Export to Comma-separated Value Files", txt)

	if (all(is.na(export.summary))) {
		txt <- "Methylation matrices for none of the requested region types could be exported. "
		rnb.export.fail(report, txt)
		return(report)
	}
	txt <- c("The table below contains links to all generated CSV files that store exported methylation &beta; values.")
	if (any(is.na(export.summary))) {
		txt <- c(txt, " Note that the data for ", sum(is.na(export.summary)), " region types could not be exported.")
	}
	rnb.add.paragraph(report, txt)
	ftypes <- rep(".csv", length(export.summary))
	ftypes[grep("\\.csv\\.gz$", export.summary)] <- ".csv.gz"
	tsummary <- data.frame(
		"Region type" = names(export.summary),
		"File" = paste("<a href=\"", export.summary, "\">", ftypes, "</a>", sep = ""),
		check.names = FALSE, stringsAsFactors = FALSE)
	tsummary[is.na(export.summary), "File"] <- as.character(NA)
	rnb.add.table(report, tsummary, row.names = FALSE)
	if (any(is.na(export.summary))) {
		rnb.export.fail(report)
	}

	return(report)
}

########################################################################################################################
## bmiq.R
## created: 2013-04-01
## creator: Yassen Assenov
## ---------------------------------------------------------------------------------------------------------------------
## Beta-mixture quantile normalization method, as implemented by Andrew Teschendorff and improved by Steve Horvath.
########################################################################################################################

## F U N C T I O N S ###################################################################################################

### Author: Andrew Teschendorff
### Date: 4th Oct 2012

#' BMIQ
#'
#' Performs Beta-mixture quantile normalization, adjusting for type II bias in Infinium 450K data.
#'
#' @param beta.v   \code{double} vector consisting of beta values. Missing values (\code{NA}s) cannot be handled, so
#'                 these must be removed or imputed prior to running BMIQ. Before normalization, beta values that are
#'                 exactly 0 and exactly 1 are replaced by the minimum positive and maximum value below 1, respectively.
#' @param design.v \code{integer} vector of length \code{length(beta.v)}, containing the values \code{1} and \code{2}
#'                 only. These values specify probe design type.
#' @param doH      Flag indicating if normalization for hemimethylated type II probes is to be performed.
#' @param nfit     Number of probes of a given design to use for the fitting. Smaller values will make BMIQ faster at
#'                 the expense of accuracy. Values between 10000 and 50000 seem to work well.
#' @param th1.v    Thresholds "type 1" to use for the initialization of the EM algorithm. These values should represent
#'                 best guesses for calling type I probes hemi-methylated and methylated, and are refined in further
#'                 steps by the algorithm.
#' @param th2.v    Thresholds "type 2" to used for the initialization of the EM algorithm. These values should represent
#'                 best guesses for calling type II probes hemi-methylated and methylated, and are refined in further
#'                 steps by the EM algorithm. If this is \code{NULL} (default), the thresholds are estimated based on
#'                 \code{th1.v} and a modified PBC correction method.
#' @param niter    Maximum number of EM iterations to be performed.
#' @param tol      Tolerance threshold for EM algorithm.
#' @return List with the following elements:
#'         \describe{
#'           \item{\code{"all"}}{The normalised beta-profile for the sample.}
#'           \item{\code{"class1"}}{Methylation state assigned to the type I probes.}
#'           \item{\code{"class2"}}{Methylation state assigned to the type II probes.}
#'           \item{\code{"av1"}}{Mean beta values for the \code{nL} classes for type I probes.}
#'           \item{\code{"av2"}}{Mean beta values for the \code{nL} classes for type II probes.}
#'           \item{\code{"hf"}}{Hubble dilation factor.}
#'           \item{\code{"th1"}}{Estimated thresholds used for type I probes.}
#'           \item{\code{"th2"}}{Estimated thresholds used for type II probes.}
#'         }
#' 
#' @description This function makes 3 indipendent attempts to fit a 3-state beta mixture model on the provided type I
#'              probes. An attempt is successful if at least 4 probes are assigned to each level. In case all attempts
#'              fail, the return value is \code{NULL}.
#' 
#' @author Andrew Teschendorff and Steve Horvath; with minor modifications by Yassen Assenov
#' @export
BMIQ <- function(beta.v,design.v,doH=TRUE,nfit=50000,th1.v=c(0.2,0.75),th2.v=NULL,niter=5,tol=0.001){

	rnb.require("RPMM")

	type1.idx <- which(design.v==1L)
	type2.idx <- which(design.v==2L)
	beta1.v <- beta.v[type1.idx]
	beta2.v <- beta.v[type2.idx]

	## Replace zeros by minimum non-zero
	zero.idx <- which(beta1.v==0)
	suppressWarnings(beta1.v[zero.idx] <- min(beta1.v[-zero.idx]))
	zero.idx <- which(beta2.v==0)
	suppressWarnings(beta2.v[zero.idx] <- min(beta2.v[-zero.idx]))
	rm(zero.idx)

	## Estimates initial weight matrix and trains the model
	fit.blc <- function(betam.v, th.v, maxiter = niter, toler = tol) {
		w.m <- matrix(0,nrow=length(betam.v),ncol=3L)
		w.m[which(betam.v <= th.v[1]),1] <- 1
		w.m[which(th.v[1] < betam.v & betam.v <= th.v[2]),2] <- 1
		w.m[which(betam.v > th.v[2]),3] <- 1
		blc2(matrix(betam.v,ncol=1),w=w.m,maxiter=maxiter,tol=toler,verbose=FALSE)
	}

	### Fit type I
	lt <- 1L # number of attempts made to fit type I
	repeat {
		betam.v <- sample(beta1.v,nfit)
		em1.o <- fit.blc(betam.v, th1.v)
		subsetclass.v <- apply(em1.o$w,1,which.max)
		i2 <- which(subsetclass.v==2L)
		subsetth1.v <- c(
			mean(c(max(betam.v[subsetclass.v==1L]),min(betam.v[i2]))),
			mean(c(max(betam.v[i2]),min(betam.v[subsetclass.v==3L]))))
		class1.v <- rep(2L,length(beta1.v))
		class1.v[which(beta1.v < subsetth1.v[1])] <- 1L
		class1.v[which(beta1.v > subsetth1.v[2])] <- 3L
		if (all(tabulate(class1.v, 3L) > 3L)) {
			break
		}
		if (lt == 3L) {
			return(NULL)
		}
		lt <- lt + 1L
	}

	## Estimate modes
	d1U.o <- density(beta1.v[class1.v==1L])
	d1M.o <- density(beta1.v[class1.v==3L])
	mod1U <- d1U.o$x[which.max(d1U.o$y)]
	mod1M <- d1M.o$x[which.max(d1M.o$y)]
	
	d2U.o <- density(beta2.v[which(beta2.v<0.4)])
	d2M.o <- density(beta2.v[which(beta2.v>0.6)])
	mod2U <- d2U.o$x[which.max(d2U.o$y)]
	mod2M <- d2M.o$x[which.max(d2M.o$y)]

	### Deal with type II fit
	if (is.null(th2.v)) {
		th2.v <- c(subsetth1.v[1] + (mod2U-mod1U), subsetth1.v[2] + (mod2M-mod1M))
	}
	betam.v <- sample(beta2.v, nfit)
	em2.o <- fit.blc(betam.v,th2.v)
	
	### For type II probes: assign to state (unmethylated, hemi- or fully methylated)
	subsetclass.v <- apply(em2.o$w,1,which.max)
	i2 <- which(subsetclass.v==2L)
	subsetth2.v <- c(
		mean(c(max(betam.v[subsetclass.v==1L]),min(betam.v[i2]))),
		mean(c(max(betam.v[i2]),min(betam.v[subsetclass.v==3L]))))
	class2.v <- rep(2L,length(beta2.v))
	class2.v[which(beta2.v < subsetth2.v[1])] <- 1L
	class2.v[which(beta2.v > subsetth2.v[2])] <- 3L

	classAV1.v <- as.vector(em1.o$mu)
	classAV2.v <- as.vector(em2.o$mu)

	### Start normalising type II probes
	nbeta2.v <- beta2.v
	## Select unmethylated probes
	lt <- 1L
	selU.idx <- which(class2.v == lt)
	selUR.idx <- selU.idx[which(beta2.v[selU.idx] > classAV2.v[lt])]
	selUL.idx <- selU.idx[which(beta2.v[selU.idx] < classAV2.v[lt])]
	## Probability according to type II distribution -> corresponding quantile in type I distribution
	p.v <- pbeta(beta2.v[selUR.idx], em2.o$a[lt,1], em2.o$b[lt,1], lower.tail = FALSE)
	nbeta2.v[selUR.idx] <- qbeta(p.v, em1.o$a[lt,1], em1.o$b[lt,1], lower.tail = FALSE)
	p.v <- pbeta(beta2.v[selUL.idx], em2.o$a[lt,1], em2.o$b[lt,1], lower.tail = TRUE)
	nbeta2.v[selUL.idx] <- qbeta(p.v, em1.o$a[lt,1], em1.o$b[lt,1], lower.tail = TRUE)

	## Select methylated probes
	lt <- 3L
	selM.idx <- which(class2.v == lt)
	selMR.idx <- selM.idx[which(beta2.v[selM.idx] > classAV2.v[lt])]
	selML.idx <- selM.idx[which(beta2.v[selM.idx] < classAV2.v[lt])]
	## Probability according to type II distribution -> corresponding quantile in type I distribution
	p.v <- pbeta(beta2.v[selMR.idx], em2.o$a[lt,1], em2.o$b[lt,1], lower.tail = FALSE)
	nbeta2.v[selMR.idx] <- qbeta(p.v, em1.o$a[lt,1], em1.o$b[lt,1], lower.tail = FALSE)

	if (doH) {
		## Select hemimethylated probes and include ML probes
		lt <- 2L
		selH.idx <- c(which(class2.v == lt), selML.idx)
		minH <- min(beta2.v[selH.idx])
		maxH <- max(beta2.v[selH.idx])

		## Set new maximum and minimum of hemimethylated probes
		nmaxH <- min(nbeta2.v[selMR.idx]) - min(beta2.v[selMR.idx]) + maxH
		nminH <- max(nbeta2.v[selU.idx]) - max(beta2.v[selU.idx]) + minH
		
		## Postulate a conformal transformation (shift + dilation)
		## new_beta_H(i) = a + hf * (beta_H(i) - minH)
		hf <- (nmaxH - nminH) / (maxH - minH)
		nbeta2.v[selH.idx] <- nminH + hf * (beta2.v[selH.idx] - minH)
	}

	pnbeta.v <- beta.v
	pnbeta.v[type1.idx] <- beta1.v
	pnbeta.v[type2.idx] <- nbeta2.v

	return(list(all = pnbeta.v, class1 = class1.v, class2 = class2.v, av1 = classAV1.v, av2 = classAV2.v,
			hf = hf, th1 = subsetth1.v, th2 = th2.v))
}

########################################################################################################################

## @author Steve Horvath
betaEst2 <- function(y, w, weights) {
	yobs = which(!is.na(y))
	if (length(yobs) <= 1) {
		return(c(1, 1))
	}
	y <- y[yobs]
	w <- w[yobs]
	weights <- weights[yobs]
	N <- sum(weights * w)
	p <- sum(weights * w * y ) / N
	v <- sum(weights * w * y * y) / N - p * p
	logab = log(c(p, 1 - p)) + log(pmax(1e-06, p * (1 - p) / v - 1))
	if (sum(yobs) == 2) {
		return(exp(logab))
	}
	opt <- try(optim(logab, RPMM::betaObjf, ydata = y, wdata = w, weights = weights, method = "Nelder-Mead",
			control = list(maxit = 50)), silent = TRUE)
	if (inherits(opt, "try-error")) {
		return(c(1, 1))
	}
	exp(opt$par)
}

########################################################################################################################

## @author Steve Horvath
blc2 <- function(Y, w, maxiter = 25, tol = 1e-06, weights = NULL, verbose = TRUE) {
	Ymn = min(Y[Y > 0], na.rm = TRUE)
	Ymx = max(Y[Y < 1], na.rm = TRUE)
	Y = pmax(Y, Ymn/2)
	Y = pmin(Y, 1 - (1 - Ymx)/2)
	Yobs = !is.na(Y)
	J = dim(Y)[2]
	K = dim(w)[2]
	n = dim(w)[1]
	if (n != dim(Y)[1]) 
		stop("Dimensions of w and Y do not agree")
	if (is.null(weights)) 
		weights = rep(1, n)
	mu = a = b = matrix(Inf, K, J)
	crit = Inf
	for (i in 1:maxiter) {
		warn0 = options()$warn
		options(warn = -1)
		eta = apply(weights * w, 2, sum)/sum(weights)
		mu0 = mu
		for (k in 1:K) {
			for (j in 1:J) {
				ab = betaEst2(Y[, j], w[, k], weights)
				a[k, j] = ab[1]
				b[k, j] = ab[2]
				mu[k, j] = ab[1]/sum(ab)
			}
		}
		ww = array(0, dim = c(n, J, K))
		for (k in 1:K) {
			for (j in 1:J) {
				ww[Yobs[, j], j, k] = dbeta(Y[Yobs[, j], j], 
					a[k, j], b[k, j], log = TRUE)
			}
		}
		options(warn = warn0)
		w = apply(ww, c(1, 3), sum, na.rm = TRUE)
		wmax = apply(w, 1, max)
		for (k in 1:K) w[, k] = w[, k] - wmax
		w = t(eta * t(exp(w)))
		like = apply(w, 1, sum)
		w = (1/like) * w
		llike = weights * (log(like) + wmax)
		crit = max(abs(mu - mu0))
		if (verbose) 
			print(crit)
		if (crit < tol) 
			break
	}
	return(list(a = a, b = b, eta = eta, mu = mu, w = w, llike = sum(llike)))
}

########################################################################################################################
## greedycut.R
## created: 2012-03-27
## creator: Yassen Assenov
## ---------------------------------------------------------------------------------------------------------------------
## Implementation of the iterative procedure for filtering probe and samples based on the given scores.
########################################################################################################################

## F U N C T I O N S ###################################################################################################

#' rnb.greedycut.table
#'
#' Extracts the indication matrix to be used by Greedycut. This function also prints the appropriate messages to the
#' log.
#'
#' @param rnb.set Methylation dataset of interest.
#' @param report  Report to contain the Greedycut section. This parameter is only used in case rnb.set does not contain
#'                the required data for estimating beta value quality. If this is set to \code{NULL} (default), adding a
#'                section is disabled.
#' @return \code{list} of two items, named \code{"matrix"} and \code{"report"}. The first item is a \code{logical}
#'         matrix of the same dimensions as the beta value matrix of \code{rnb.set}. Every row signifies a site, and
#'         every column - a sample in the dataset. Unreliable measurements are encoded by \code{TRUE}. The second item
#'         in the returned list is the possibly modified report.
#' @author Yassen Assenov
#' @noRd
rnb.greedycut.table <- function(rnb.set, pval.threshold=rnb.getOption("filtering.greedycut.pvalue.threshold"), min.coverage=rnb.getOption("filtering.coverage.threshold"), report = NULL) {
	if (pval.threshold<1 && inherits(rnb.set, "RnBeadSet")) {
		result.pval <- dpval(rnb.set)
		if (is.null(result.pval)) {
			rnb.info("Omitting Greedycut because detection p-values are missing")
			if (!is.null(report)) {
				txt <- "Greedycut was not executed because detection p-values are missing."
				report <- rnb.add.section(report, "Greedycut", txt)
			}
		} else {
			rnb.info(c("Working with a p-value threshold of", pval.threshold))
			result.pval[is.na(result.pval)] <- 1
			result.pval <- (result.pval >= pval.threshold)
		}
	}else{
        result.pval<-NULL
    }
    if(min.coverage>0 && inherits(rnb.set, "RnBiSeq")){
		result.covg <- covg(rnb.set)
		if (is.null(result.covg))	{
			rnb.info("Omitting Greedycut because read coverage data are missing")
			if (!is.null(report)) {
				txt <- "Greedycut was not executed because read coverage data are missing."
				report <- rnb.add.section(report, "Greedycut", txt)
			}
		} else {
			rnb.info(c("Working with a minimal acceptable coverage of", min.coverage))
			result.covg[is.na(result.covg)] <- 0L
			result.covg <- (result.covg < min.coverage)
		}
	}else{
        result.covg<-NULL
    }
    
    if(is.null(result.pval) && is.null(result.covg)){
        result<-NULL
    }else if(is.null(result.pval)){
        result<-result.covg
    }else if(is.null(result.covg)){
        result<-result.pval
    }else{
        result<-result.pval | result.covg
    }
    
	return(list(matrix = result, report = report))
}

########################################################################################################################

#' rnb.execute.greedycut
#'
#' Executes the Greedycut procedure for probe and sample filtering based on the detection p-values, and calculates
#' statistics on its iterations.
#'
#' @param rnb.set HumanMethylation450K dataset as an object of type \code{\linkS4class{RnBeadSet}}.
#' @param pval.threshold The P-value threshold. For further information, see the option \code{"filtering.greedycut.pvalue.threshold"}
#'                    in \code{\link{rnb.options}}.
#' @param min.coverage The coverage threshold. For further information, see the option \code{"filtering.coverage.threshold"}
#'                    in \code{\link{rnb.options}}.
#' @param rc.ties Flag indicating what the behaviour of the algorithm should be in case of ties between values of rows
#'                (probes) and columns (samples). See the corresponding parameter in
#'                \code{\link{greedycut.filter.matrix}} for more details.
#' @return \code{NULL} if \code{rnb.set} does not contain a matrix of detection p-values, or if all p-values denote
#'         reliable measurements. Otherwise, a list of the following elements:
#'         \describe{
#'           \item{"infos"}{Table summarizing the iterations of the algorithm, as returned by
#'                \code{\link{greedycut.filter.matrix}}.}
#'           \item{"statistics"}{Additional statistics on all iterations, as returned by
#'                \code{\link{greedycut.get.statistics}}.}
#'           \item{"iteration"}{Number of Greedycut iterations + \code{1} applied to the dataset, that is,
#'                a value of 1 indicates that the dataset was not modified.}
#'           \item{"sites"}{Indices of all sites to be removed.}
#'           \item{"samples"}{Indices of all samples to be removed.}
#'         }
#'
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' greedy.result <- rnb.execute.greedycut(rnb.set.example)
#' # Number of applied iterations
#' greedy.result$iteration
#' }
#' @author Yassen Assenov
#' @export
rnb.execute.greedycut <- function(rnb.set, pval.threshold=rnb.getOption("filtering.greedycut.pvalue.threshold"), min.coverage=rnb.getOption("filtering.coverage.threshold"), rc.ties = rnb.getOption("filtering.greedycut.rc.ties")) {
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	if (!(is.character(rc.ties) && length(rc.ties) == 1 && (rc.ties %in% c("row", "column", "any")))) {
		stop("invalid value for rc.ties; expected one of: row, column, any")
	}
	rc.ties <- rc.ties[1]
	if (pval.threshold < 1.0 && is.null(dpval(rnb.set))) {
		return(NULL)
	}
    
    if (min.coverage>0 && is.null(covg(rnb.set))) {
        return(NULL)
    }
    
    ### temporary: disable bead count-based filtering for array data
    if(inherits(rnb.set, "RnBeadSet")){
        min.coverage <- 0
    }
    
	beta.state <- rnb.greedycut.table(rnb.set, pval.threshold, min.coverage)$matrix
	if (is.null(beta.state)) {
		return(NULL)
	}

	rnb.execute.greedycut.internal(beta.state, integer(), rc.ties)
}

rnb.execute.greedycut.internal <- function(beta.state, sites2ignore, rc.ties) {
	
	## Run the iterative algorithm for filtering out probes and samples
	iter.infos <- greedycut.filter.matrix(beta.state, sites2ignore, rc.ties)
	if (nrow(iter.infos) == 1) {
		rnb.info("No unreliable measurements found")
		return(NULL) # all(beta.state == 0)
	}
	rnb.status(c("Calculated a total of", nrow(iter.infos), "iterations"))

	## Calculate FPR and sensitivity
	iter.statistics <- greedycut.get.statistics(iter.infos)

	## Determine number of iterations to execute
	ind.iteration <- which.max(iter.statistics[["D"]])
	rnb.info(c("Optimal number of iterations is", ind.iteration))

	if (ind.iteration != 1) {
		## Update the matrices of beta and detection p-values
		rcindices <- iter.infos[1:ind.iteration, "Index"]
		rctypes <- iter.infos[1:ind.iteration, "Type"]
		filtered.sites <- rcindices[rctypes == "r"]
		filtered.samples <- rcindices[rctypes == "c"]
	} else {
		filtered.sites <- integer()
		filtered.samples <- integer()
	}

	return(list(infos = iter.infos, statistics = iter.statistics, iteration = ind.iteration,
		sites = filtered.sites, samples = filtered.samples))
}

########################################################################################################################

#' rnb.step.greedycut
#'
#' Performs the Greedycut procedure (if applicable) to filter the given methylation dataset and adds a
#' corresponding section to the specified report.
#'
#' @param rnb.set Dataset to be filtered. This must be an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param report  Report to summarize the outcome of this procedure. This must be an object of type
#'                \code{\linkS4class{Report}}.
#' @param rc.ties Flag indicating what the behaviour of the algorithm should be in case of ties between values of rows
#'                (sites or probes) and columns (samples). See the corresponding parameter in
#'                \code{\link{greedycut.filter.matrix}} for more details.
#' @return        List of three elements:
#'                \describe{
#'                  \item{\code{"dataset"}}{The (possibly modified) dataset after performing the Greeducyt procedure.}
#'                  \item{\code{"report"}}{The modified report.}
#'                  \item{\code{"filtered"}}{Indices of the sites in \code{rnb.set} that were filtered out.}
#'                }
#'
#' @seealso \code{\link{greedycut.filter.matrix}} for running all Greedycut iterations
#'
#' @author Yassen Assenov
#' @noRd
rnb.step.greedycut <- function(rnb.set, report, rc.ties = rnb.getOption("filtering.greedycut.rc.ties")) {
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	if (!(is.character(rc.ties) && length(rc.ties) == 1 && (rc.ties %in% c("row", "column", "any")))) {
		stop("invalid value for rc.ties; expected one of: row, column, any")
	}
	rc.ties <- rc.ties[1]
	rnb.step.greedycut.internal(rnb.set, integer(), annotation(rnb.set, add.names = inherits(rnb.set, "RnBeadSet")),
		rc.ties)
}

rnb.step.greedycut.internal <- function(rnb.set, sites2ignore, report, anno.table,
	rc.ties = rnb.getOption("filtering.greedycut.rc.ties")) {

	result <- rnb.greedycut.table(rnb.set, report=report)
	beta.state <- result$matrix
	report <- result$report
	rm(result)
	if (is.null(beta.state)) {
		return(list(sites = integer(), samples = integer(), report = report))
	}
	logger.start("Greedycut")

	result <- rnb.execute.greedycut.internal(beta.state, sites2ignore, rc.ties)
	if (is.null(result)) {
		logger.info("All measurements are reliable based on the specified threshold")
		txt <- "Greedycut was not executed because all measurements are reliable based on the specified threshold"
		report <- rnb.add.section(report, "Greedycut", txt)
		logger.completed()
		return(list(sites = integer(), samples = integer(), report = report))
	}
	unreliable.counts <- list("persite" = rowSums(beta.state), "persample" = colSums(beta.state))
	iter.infos <- result$infos
	iter.statistics <- result$statistics
	ind.iteration <- result$iteration
	rm(beta.state)

	## ----------------------------------------
	## Generate text and figures for the report

	txt.site <- rnb.get.row.token(rnb.set)
	txt.sites <- rnb.get.row.token(rnb.set, plural = TRUE)
	txt <- ifelse(grepl("Bead", class(rnb.set)), "detection p-value", "read coverage")
	txt <- c("The Greedycut algorithm iteratively removes from the dataset ", txt.sites, " and samples of highest ",
		"impurity. These correspond to the rows and columns in the ", txt, " table that contain the largest fraction ",
		"of unreliable measurements. This section summarizes the results of applying Greedycut on the analyzed ",
		"dataset.")
	report <- rnb.add.section(report, "Greedycut", txt)
	if (inherits(rnb.set, "RnBeadSet")) {
		txt <- c("We considered every &beta; value to be unreliable when its corresponding detection p-value is ",
			"not below the threshold <i>T</i>:")
		txt.formula <- c("<i>p</i> &ge; <i>T</i> = ", rnb.getOption("filtering.greedycut.pvalue.threshold"))
	} else {
		txt <- c("We considered every methylation value to be unreliable when its corresponding read coverage is ",
			"below the threshold <i>T</i>:")
		txt.formula <- c("<i>c</i> &lt; <i>T</i> = ", rnb.getOption("filtering.coverage.threshold"))
	}
	report <- rnb.add.section(report, "Unreliable Measurements", txt, level = 2)
	rnb.add.paragraph(report, txt.formula, "centered")
	rm(txt.formula)

	## Generate text and figures for the report using unreliable.counts, iter.infos, iter.statistics, ind.iteration
	report.plots <- list()
	for (iname in names(unreliable.counts)) {
		fname <- paste0("greedycut_cdf_", iname)
		report.plots <- c(report.plots, plotcdf(fname, report, unreliable.counts[[iname]]))
	}
	txt <- "The figure below summarizes the observed number of unreliable measurements per probe and per sample."
	rnb.add.paragraph(report, txt)
	txt <- c("Cumulative distribution function of number of unreliable values per ", txt.site, "/sample.")
	setting.names <- list("Number of values per" = c(txt.site, "sample"))
	names(setting.names[[1]]) <- names(unreliable.counts)
	report <- rnb.add.figure(report, txt, report.plots, setting.names)

	report.plots <- list()
	mdims <- iter.infos[, c("Columns", "Rows")]
	mdims[, 2] <- mdims[, 2] / 1000
	colnames(mdims) <- c("Samples", paste(capitalize(txt.sites), "(in thousand)"))
	iter.statistics[["Iteration"]] <- 1:nrow(iter.statistics)

	## Plot ROC curve
	rpoints <- unique(iter.statistics[, c("False positive rate", "Sensitivity")])
	colnames(rpoints) <- c("x", "y")
	plotroc <- function(fname, report, rocpoints, ind.iteration = NA) {
		rplot <- createReportPlot(fname, report, width = 6.2, height = 6.2)
		pp <- ggplot(data.frame(x = c(0, 1), y = c(0, 1)), aes_string(x = "x", y = "y")) +
			labs(x = "False positive rate", y = "Sensitivity") + coord_fixed(xlim = c(0, 1), ylim = c(0, 1)) +
			geom_line(col = "#A0A0A0") + geom_line(data = rocpoints, col = "#000080", lwd = 2)
		if (!is.na(ind.iteration)) {
			pp <- pp + geom_point(data = rocpoints[ind.iteration, ], col = "#FF0000", pch = 20)
		}
		pp <- pp + theme(plot.margin = unit(0.1 + c(0, 0, 0, 0), "in"))
		print(pp)
		return(off(rplot))
	}
	report.plots <- c(report.plots, plotroc("greedycut_roc_all", report, rpoints, ind.iteration))
	i2plot <- list("all" = 1:nrow(iter.infos))
	if (ind.iteration != 1 && ind.iteration != nrow(iter.infos)) {
		i2plot[["stop"]] <- 1:ind.iteration
		report.plots <- c(report.plots, plotroc("greedycut_roc_stop", report, rpoints[1:ind.iteration, ]))
	}
	logger.status("Created ROC plot")

	## Create line plots of matrix dimensions, FPR, sensitivity and more
	plotline <- function(fname, dframe, ind.stop = NA, col = "#000080", col.stop = "#800000", ...) {
		rplot <- createReportPlot(fname, report, width = 6.2, height = 6.2)
		vnames <- paste0("`", colnames(dframe), "`")
		pp <- ggplot(dframe, aes_string(x = vnames[1], y = vnames[2])) + geom_path(col = col)
		if (!is.na(ind.stop)) {
			pp <- pp + geom_point(data = dframe[ind.stop, ], col = col.stop, pch = 20)
		}
		pp <- pp + theme(plot.margin = unit(0.1 + c(0, 0, 0, 0), "in"))
		print(pp)
		return(off(rplot))
	}
	for (i in 1:length(i2plot)) {
		ind.stop <- ifelse(length(i2plot) == 2 && i == 1, ind.iteration, NA)
		dframe <- iter.statistics[i2plot[[i]], ]
		icolumn <- which(colnames(dframe) == "Iteration")
		if (nrow(dframe) > 10000) {
			dframe[, icolumn] <- dframe[, icolumn] / 1000
			colnames(dframe)[icolumn] <- "Iteration (in thousand)"
		}
		icolumn <- colnames(dframe)[icolumn]
		## Plot matrix dimensions
		fname <- paste0("greedycut_stairs_", names(i2plot)[i])
		rplot <- plotline(fname, mdims[i2plot[[i]], ], ind.stop)
		report.plots <- c(report.plots, rplot)
		## Plot False positive rate
		fname <- paste0("greedycut_fpr_", names(i2plot)[i])
		rplot <- plotline(fname, dframe[, c(icolumn, "False positive rate")], ind.stop)
		report.plots <- c(report.plots, rplot)
		## Plot sensitivity
		fname <- paste0("greedycut_sensitivity_", names(i2plot)[i])
		rplot <- plotline(fname, dframe[, c(icolumn, "Sensitivity")], ind.stop)
		report.plots <- c(report.plots, rplot)
		## Plot distance from the diagonal
		fname <- paste0("greedycut_ddiagonal_", names(i2plot)[i])
		rplot <- plotline(fname, dframe[, c(icolumn, "D")], ind.stop)
		report.plots <- c(report.plots, rplot)
	}
	rm(mdims, rpoints, plotroc, plotline, i2plot, i, ind.stop, dframe, icolumn, fname, rplot)
	txt <- c("RnBeads executed Greedycut using the threshold given above and applied all its steps. ",
		"Briefly, Greedycut is an iterative algorithm that filters out the ", txt.site, " or sample with the highest ",
		"fraction of unreliable measurements one at a time. Note that every iteration of the algorithm produces a ",
		"matrix of retained measurements and a set of removed ones.")
	report <- rnb.add.section(report, paste("Filtered", capitalize(txt.sites), "and Samples"), txt, level = 2)
	txt <- c("We calculated false positive rate (<i>&alpha;</i>) and sensitivity (<i>s</i>) when the retained ",
		"measurements are considered as prediction for the reliable ones. Among all matrices produced by Greedycut, ",
		"we selected the one that maximizes the value of the expression <i>s</i> + 1 - <i>&alpha;</i>, thereby giving ",
		"equal weights to the sensitivity and specificity. Presented geometrically on a ROC curve, this is the point ",
		"that is furthest from the diagonal. The results of the Greedycut procedure and the selected iteration are ",
		"presented in the figure below.")
	rnb.add.paragraph(report, txt)
	txt <- c("Change of table dimensions / metric related to accuracy as Greedycut progressively removes ", txt.sites,
		" and samples. Accuracy is calculated by treating the retained entries as predictive of reliable ",
		"measurements. The red circle, if present, marks the last iteration that was executed.")
	setting.names <- list(
		"Metric" = c(
			"stairs" = "table dimensions",
			"fpr" = "false positive rate",
			"sensitivity" = "sensitivity",
			"roc" = "ROC curve",
			"ddiagonal" = "distance to the diagonal in a ROC plot"),
		"Iterations to show" = c("all" = "all", "stop" = "executed only"))
	report <- rnb.add.figure(report, txt, report.plots, setting.names)
	logger.status("Created line plots for matrix dimensions and other statistics")

	## Generate tables with the filtered sites and samples
	r.removed <- length(result$sites)
	c.removed <- length(result$samples)
	if (r.removed != 0 || c.removed != 0) {
		tbl.removed <- data.frame(
			Type = c(capitalize(txt.sites), "Samples"),
			Removed = c(r.removed, c.removed),
			Table = as.character(c(NA, NA)), stringsAsFactors = FALSE)

		if (r.removed != 0) {
			fname <- "removed_sites_greedycut.csv"
			fname.full <- rnb.save.removed.sites(anno.table[result$sites, , drop = FALSE], report, fname)
			tbl.removed[1, "Table"] <- paste0('<a href="', fname.full, '">', fname, '</a>')
		}
		if (c.removed != 0) {
			fname <- "removed_samples_greedycut.csv"
			fname.full <- file.path(rnb.get.directory(report, "data", TRUE), fname)
			write.csv(pheno(rnb.set)[result$samples, , drop = FALSE], fname.full, row.names = FALSE)
			fname <- paste0('<a href="', rnb.get.directory(report, "data"), '/', fname, '">', fname, '</a>')
			tbl.removed[2, "Table"] <- fname
		}

		txt <- c("Based on the criteria described above, ", r.removed, " ", ifelse(r.removed == 1, txt.site, txt.sites),
			" and ", c.removed, " sample", ifelse(c.removed == 1, "", "s"), " were filtered out. Links to the lists of ",
			"removed items are given below.")
		rnb.add.paragraph(report, txt)
		rnb.add.table(report, tbl.removed, row.names = FALSE, na = "")
	}

	logger.completed()
	return(list(sites = result$sites, samples = result$samples, report = report))
}

########################################################################################################################

#' greedycut.filter.matrix
#'
#' Performs all iterations of the Greedycut algorithm for removing rows and columns from the given matrix.
#'
#' @param mm          Numeric matrix to filter.
#' @param rows2ignore \code{integer} vector containing indices of rows in \code{mm} to be ignored by this  function.
#' @param rc.ties     Flag indicating what the behaviour of the algorithm should be in case of ties between values of
#'                    rows and columns. The value of this parameter must be one of \code{"row"}, \code{"column"} or
#'                    \code{"any"} (the last one indicating random choice).
#' @return Table summarizing the iterations of the algorithm in the form of a \code{data.frame} with the following
#'         columns : Index, Type, Score, Normalized score, Rows, Columns.
#'
#' @seealso \code{\link{greedycut.get.submatrix}} for extracting the resulting matrix after filtering
#' @author Yassen Assenov
#' @export
greedycut.filter.matrix <- function(mm, rows2ignore = integer(), rc.ties = "row") {
	if (any(mm < 0)) {
		stop("Negative values found")
	}
	if (!(rc.ties %in% c("row", "column", "any"))) {
		stop("Invalid value for rc.ties")
	}

	mm[rows2ignore, ] <- 0
	N <- nrow(mm); M <- ncol(mm)
	indices <- rep.int(as.integer(0), times = N + M)
	names(indices) <- rep.int("x", times = N + M)
	i <- 1
	fullscores <- c(rowSums(mm), colSums(mm))
	normscores <- c(rowMeans(mm), colMeans(mm))
	dimensions <- matrix(0, nrow = N + M, ncol = 2,
		dimnames = list(NULL, c("Rows", "Columns")))
	dimensions[1, ] <- c(N, M)

	maxfullscores <- rep(0, times = N + M)
	maxnormscores <- rep(0, times = N + M)

	repeat {
		## Find the maximum row or column score
		maxscore <- max(normscores)
		if (maxscore <= 0) {
			break
		}
		i <- i + 1
		max.ind <- which(normscores == maxscore)
		if (length(max.ind) != 1) {
			## Make a (random) selection in case of ties
			if (rc.ties == "row") {
				i1 <- max.ind[max.ind <= N]
				i2 <- max.ind[max.ind > N]
			} else if (rc.ties == "column") {
				i1 <- max.ind[max.ind > N]
				i2 <- max.ind[max.ind <= N]
			} else { # rc.ties = "any"
				i1 <- max.ind
				i2 <- c()
			}
			l <- length(i1)
			max.ind <- base::ifelse(l > 0, base::ifelse(l > 1, sample(i1, size = 1), i1), sample(i2, size = 1))
			rm(i1, i2, l)
		}

		## Save the maximum score and row or column number
		maxnormscores[i] <- maxscore
		maxfullscores[i] <- fullscores[max.ind]
		fullscores[max.ind] <- 0
		normscores[max.ind] <- 0
		if (max.ind <= N) {
			indices[i] <- max.ind
			names(indices)[i] <- "r"
			dimensions[i, ] <- dimensions[i - 1, ] - c(1, 0)
			fscores <- abs(fullscores[(N + 1):length(fullscores)] - mm[max.ind, ])
			fullscores[(N + 1):length(fullscores)] <- fscores
			normscores[(N + 1):length(normscores)] <- fscores / dimensions[i, 1]
			mm[max.ind, ] <- 0
		} else { # nrow(mm) < max.ind <= nrow(mm) + ncol(mm)
			max.ind <- max.ind - N
			indices[i] <- max.ind
			dimensions[i, ] <- dimensions[i - 1, ] - c(0, 1)
			names(indices)[i] <- "c"
			fscores <- abs(fullscores[1:N] - mm[, max.ind])
			fullscores[1:N] <- fscores
			normscores[1:N] <- fscores / dimensions[i, 2]
			mm[, max.ind] <- 0
		}
	}

	return(data.frame(
			"Index" = indices[1:i],
			"Type" = names(indices)[1:i],
			"Score" = maxfullscores[1:i],
			"Normalized score" = maxnormscores[1:i],
			"Rows" = dimensions[1:i, "Rows"] - length(rows2ignore),
			"Columns" = dimensions[1:i, "Columns"], check.names = FALSE, stringsAsFactors = TRUE))
}

########################################################################################################################

#' greedycut.get.statistics
#'
#' Calculates various statistics on the iterations of Greedycut.
#'
#' @param filterinfo Information on the filtering iterations as a \code{data.frame} returned by
#'                   \code{\link{greedycut.filter.matrix}}.
#' @return Additional statistics on the iterations in the form of a \code{data.frame} with the following columns:
#'         \code{"Elements retained"}, \code{"Elements removed"}, \code{"Mismatches retained"},
#'         \code{"Mismatches removed"}, \code{"False Positive Rate"}, \code{"Sensitivity"}, \code{"D"}. The last column
#'         signifies distance from the diagonal in a ROC curve. 
#'
#' @author Yassen Assenov
#' @export
greedycut.get.statistics <- function(filterinfo) {
	elements.retained <- filterinfo[, "Rows"] * filterinfo[, "Columns"]
	elements <- elements.retained[1]
	elements.removed <- elements - elements.retained
	mismatches.removed <- cumsum(filterinfo[["Score"]])
	mismatches <- mismatches.removed[length(mismatches.removed)]
	mismatches.retained <- abs(mismatches - mismatches.removed)
	fprs <- mismatches.retained / mismatches
	sensitivities <- (elements.retained - mismatches.retained) / (elements - mismatches)
	return(data.frame(
			"Elements retained" = elements.retained,
			"Elements removed" = elements.removed,
			"Mismatches retained" = mismatches.retained,
			"Mismatches removed" = mismatches.removed,
			"False positive rate" = fprs,
			"Sensitivity" = sensitivities,
			"D" = sqrt((fprs - sensitivities)^2 / 2),
			check.names = FALSE))
}

########################################################################################################################

#' greedycut.get.submatrix
#'
#' Filters a data matrix executing the given number of iterations of Greedycut.
#'
#' @param mm          Data \code{matrix} to be filtered.
#' @param filter.info Information on the filtering iterations as a \code{data.frame} returned by
#'                    \code{\link{greedycut.filter.matrix}}.
#' @param it.num      Number of iterations to execute. Defaults to all iterations.
#' @return Data matrix containing subsets of the rows and columns of \code{mm}.
#'
#' @author Yassen Assenov
#' @export
greedycut.get.submatrix <- function(mm, filter.info, it.num = nrow(filter.info) - as.integer(1)) {
	if (!is.matrix(mm)) {
		stop("invalid value for mm; expected matrix")
	}
	if (!(is.data.frame(filter.info) &&
			identical(colnames(filter.info), c("Index", "Type", "Score", "Normalized score", "Rows", "Columns")))) {
		stop("invalid value for filterinfo")
	}
	if (is.double(it.num) && all(it.num == as.integer(it.num))) {
		it.num <- as.integer(it.num)
	}
	if (!(is.integer(it.num) && length(it.num) == 1 && (!is.na(it.num)))) {
		stop("invalid value for it.num; expected single integer")
	}
	it.num <- it.num[1]
	if (!(1 <= it.num && it.num < nrow(filter.info))) {
		stop("invalid value for it.num")
	}
	i2remove <- filter.info[2:(it.num + 1), "Index"]
	what2remove <- filter.info[2:(it.num + 1), "Type"]
	ri2remove <- i2remove[what2remove == "r"]
	ci2remove <- i2remove[what2remove == "c"]
	if (any(i2remove < 1) || any(ri2remove > nrow(mm)) || any(ci2remove > ncol(mm))) {
		stop("unexpected dimensions of the data matrix")
	}
	ris <- setdiff(1:nrow(mm), ri2remove)
	cis <- setdiff(1:ncol(mm), ci2remove)
	return(mm[ris, cis])
}

## E N D ###############################################################################################################

########################################################################################################################
## clustering.R
## created: 2012-06-18
## creator: Yassen Assenov
## ---------------------------------------------------------------------------------------------------------------------
## Definition and implementation of clustering algorithms and visualization.
########################################################################################################################

## G L O B A L S #######################################################################################################

## Dissimilarity metrics used in clustering
DIST.METRICS <- c(
	"correlation" = "correlation-based",
	"manhattan" = "Manhattan distance",
	"euclidean" = "Euclidean distance")

## Clustering algorithms applied
ALGORITHMS <- c("hierarchical" = "hierarchical")

## Linkage methods used in hierarchical clustering
AGGLOMERATIONS <- c("average" = "average", "complete" = "complete", "median" = "median-based")

## C L A S S ###########################################################################################################

#' RnBeadClustering Class
#'
#' Storage class for the results of a clustering algorithm applied on an \code{\linkS4class{RnBSet}} dataset.
#'
#' @section Slots:
#' \describe{
#'   \item{dissimilarity}{Dissimilarity metric used in the form of a one-element \code{character} vector.}
#'   \item{dimensionality}{Dimensionality of the clustered points in the form of a one-element \code{integer} vector.}
#'   \item{algorithm}{Clustering algorithm (and optionally, type) as a \code{character} vector of length 1 or 2.}
#'   \item{result}{Resulting object after applying the clustering algorithm on a dataset.}
#'   \item{assignments}{Cluster assignments for the samples in the dataset as a matrix. Row names in this matrix are
#'        sample identifiers, and each column is dedicated to partitioning into \emph{k} clusters for a fixed \emph{k}.}
#'   \item{silhouettes}{\code{numeric} vector of mean silhouette values for each tested value of \emph{k}.}
#' }
#'
#' @section Methods and Functions:
#' \describe{
#'   \item{\code{samples}}{Gets the identifiers of all samples used in the clustering.}
#' }
#'
#' @name RnBeadClustering-class
#' @rdname RnBeadClustering-class
#' @aliases initialize,RnBeadClustering-method
#' @author Yassen Assenov
#' @exportClass RnBeadClustering
setClass("RnBeadClustering",
	representation(
		dissimilarity = "character",
		dimensionality = "integer",
		algorithm = "character",
		result = "list",
		assignments = "matrix",
		silhouettes = "numeric"),
	package = "RnBeads")

## M E T H O D S #######################################################################################################

setValidity("RnBeadClustering",
	function(object) {
		x <- object@dissimilarity
		if (!(is.character(x) && length(x) == 1 && (!is.na(x)))) {
			return("invalid value for dissimilarity; expected single character")
		}
		if (!(x %in% names(DIST.METRICS))) {
			return("unsupported value for dissimilarity")
		}
		x <- object@dimensionality
		if (!(is.integer(x) && length(x) == 1 && isTRUE(x > 0))) {
			return("invalid value for dimensionality; expected single positive integer")
		}
		x <- object@algorithm
		if (!(is.character(x) && (length(x) %in% c(1, 2)) && (!any(is.na(x))))) {
			return("invalid value for algorithm; expected character")
		}
		if (!(x[1] %in% names(ALGORITHMS))) {
			return("unsupported value for algorithm")
		}
		if (length(x) == 2) {
			if (!(x[2] %in% names(AGGLOMERATIONS))) {
				return("unsupported value for algorithm; unsupported linkage method")
			}
		}
		if (!(length(object@result) != 0)) {
			return("invalid value for result")
		}
		x <- object@assignments
		if (!(is.integer(x) && is.matrix(x) && (!any(is.na(x))))) {
			return("invalid value for assignments")
		}
		## TODO: Validate that colnames(x) are "integer" values in the range 2:nrow(x)
		y <- object@silhouettes
		if (!(is.numeric(y) && length(y) == ncol(x))) {
			return("invalid value for silhouettes")
		}
		if (!identical(colnames(x), names(y))) {
			return("mismatch between assignments and silhouettes")
		}
		TRUE
	}
)

########################################################################################################################

setMethod("initialize", "RnBeadClustering",
	function(.Object, dissimilarity, dimensionality, algorithm, result, assignments, silhouettes) {
		.Object@dissimilarity <- dissimilarity
		.Object@dimensionality <- dimensionality
		.Object@algorithm <- algorithm
		.Object@result <- result
		.Object@assignments <- assignments
		.Object@silhouettes <- silhouettes
		validObject(.Object)
		.Object
	}
)

########################################################################################################################

if (!isGeneric("samples")) {
	setGeneric("samples", function(object) standardGeneric("samples"))
}

#' @rdname samples-methods
#' @export
setMethod("samples", signature(object = "RnBeadClustering"),
	function(object) {
		rownames(object@assignments)
	}
)

## F U N C T I O N S ###################################################################################################

## dist.correlation
##
## Computes correlation-based dissimilarities between the row vectors of the given matrix.
##
## @param x Matrix to use in calculating dissimilarities.
## @return Object of class \code{"dist"}.
## @author Yassen Assenov
dist.correlation <- function(x) {
	res <- as.dist(1 - cor(t(x), use = "pairwise.complete.obs"))
	#take care of the following type of warnings that occur for invariable rows:
	#	Warning message:
	#	In cor(t(x), use = "pairwise.complete.obs") :
	#	the standard deviation is zero
	res[is.na(res)] <- 1
	return(res)
}

########################################################################################################################

## get.silhouette
##
## Computes the average silhouette value of the specified clustering using the given distance metric.
##
## @param x    \code{integer} vector with k different integer cluster codes.
## @param dist Distance metric to use.
## @return Average among the silhouette values of all points in \code{x}.
## @author Yassen Assenov
get.silhouette <- function(x, dist) {
	mean(silhouette(x, dist = dist)[, "sil_width"])
}

########################################################################################################################

## plot.heatmap.rand
##
## Creates a heatmap visualizing the given Rand indices.
##
## @param tbl    Matrix of computed adjusted Rand indices. All values are expected to be between -1 and 1.
## @param fname  File name to contain the generated plot. If this is \code{NA} (default), the heatmap is plotted to the
##               currently active graphics device.
## @param report Report to host the generated image. This parameter is used only when \code{fname} is not \code{NA}. If
##               provided, this parameter must be an instance of class \code{Report}.
##
## @author Yassen Assenov
plot.heatmap.rand <- function(tbl, fname = NA, report = NULL) {
	tbl.melt <- reshape2::melt(tbl, varnames = c("x", "y"))
	tbl.melt[[1]] <- factor(as.character(tbl.melt[[1]]), levels = rev(rownames(tbl)))
	tbl.melt[[2]] <- factor(as.character(tbl.melt[[2]]), levels = colnames(tbl))
	colnames(tbl.melt)[3] <- "rindex"
	if (!is.na(fname)) {
		width <- 2 + ncol(tbl) * 0.4 + 1.5
		height <- 3 + nrow(tbl) * 0.4
		rplot <- createReportPlot(fname, report, width = width, height = height)
	}
	colors.g <- rnb.getOption("colors.3.gradient")
	pp <- ggplot(tbl.melt) + aes_string("y", "x") + coord_fixed() + labs(x = NULL, y = NULL, fill = "Rand index") +
		geom_tile(aes_string(fill = "rindex"), color = "white") +
		scale_fill_gradient2(limits = c(-1, 1), low = colors.g[1], mid = colors.g[2], high = colors.g[3]) +
		scale_x_discrete(expand = c(0, 0)) + scale_y_discrete(expand = c(0, 0)) +
		theme(axis.ticks = element_blank(), axis.text.x = element_text(angle = 90, hjust = 1),
			plot.margin = unit(c(0.5, 1.5, 0.5, 0.5), c("lines", "in", "lines", "lines")),
			legend.justification = c(0, 0.5), legend.position = c(1, 0.5))
	print(pp)
	if (!is.na(fname)) {
		return(invisible(off(rplot)))
	}
}

########################################################################################################################

#' rnb.execute.clustering
#'
#' Performs hierarchical clustering on the samples of the given dataset using multiple distance metrics and
#' agglomeration methods for a single given region type.
#'
#' @param rnb.set     Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param region.type the clustering is performed on methylation levels from regions of that type. 
#' 					  see \code{\link{rnb.region.types}} for possible values.
#' @return List of clustering results, whereby each element is an object of type \code{\linkS4class{RnBeadClustering}}.
#'         In case clustering cannot be performed, the return value is \code{NULL}. Reasons for a failure include, among
#'         others, the case when \code{rnb.set} contains less than 3 samples, or undefined distances between a pair of
#'         samples due to (too many) missing values in the respective methylation matrix.
#'         
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' results <- rnb.execute.clustering(rnb.set.example, "promoters")
#' # List applied dissimilarity metrics
#' sapply(results, slot, "dissimilarity")
#' # List applied clustering algorithms
#' str(lapply(results, slot, "algorithm"))
#' }
#' @author Yassen Assenov
#' @export
rnb.execute.clustering <- function(rnb.set, region.type = "sites") {
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}

	X <- t(meth(rnb.set, type = region.type))
	if (nrow(X) < 3) {
		## Too few samples; cannot compute silhouette values
		rnb.status(c("Skipped clustering on", region.type, ": too few samples"))
		return(NULL)
	}
	if (rnb.getOption("exploratory.clustering") == "top") {
		N <- max(rnb.getOption("exploratory.clustering.top.sites"))
		if (N < ncol(X)) {
			X <- X[, order(colVars(X, na.rm = TRUE), decreasing = TRUE)[1:N], drop = FALSE]
		}
		rm(N)
	}
	results <- list()
	for (dist.metric in names(DIST.METRICS)) {
		if (dist.metric == "correlation") {
			dmatrix <- dist.correlation(X)
		} else {
			dmatrix <- dist(X, method = dist.metric)
		}
		for (agglomeration in names(AGGLOMERATIONS)) {
			result <- tryCatch(hclust(dmatrix, method = agglomeration), error = function(e) { NULL })
			if (is.null(result)) {
				logger.warning(c("Failed clustering on", region.type, "( using", dist.metric, ")"))
				return(NULL)
			}
			cluster.assignments <- as.matrix(cutree(result, k = c(2:(nrow(X) - 1))))
			silhouette.values <- apply(cluster.assignments, 2, get.silhouette, dist = dmatrix)
			attr(result, "class") <- NULL # remove S3 class information in order to store it in RnBeadClustering
			result <- new("RnBeadClustering", dist.metric, ncol(X), c("hierarchical", agglomeration), result,
				cluster.assignments, silhouette.values)
			results <- c(results, result)
		}
		rnb.status(c("Performed clustering on", region.type, "using", dist.metric, "as a distance metric"))
	}
	return(results)
}

########################################################################################################################

#' rnb.execute.clustering.all
#'
#' Performs hierarchical clustering on the samples of the given dataset using multiple distance metrics and
#' agglomeration methods for all suggested site and region types.
#'
#' @param rnb.set Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @return List of list of clustering results; each element corresponds to one region type and is a list of objects
#'         of type \code{\linkS4class{RnBeadClustering}}.
#'
#' @seealso \code{\link{rnb.execute.clustering}} for performing clustering using a single site or region type.
#'
#' @author Fabian Mueller
#' @export
rnb.execute.clustering.all <- function(rnb.set) {
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}

	result <- list()
	if (rnb.getOption("analyze.sites")) {
		result[["sites"]] <- rnb.execute.clustering(rnb.set, "sites")
	}
	for (rtype in rnb.region.types.for.analysis(rnb.set)) {
		clust.res <- rnb.execute.clustering(rnb.set, rtype)
		if (is.null(clust.res)){
			result[rtype] <- list(NULL)
		} else {
			result[[rtype]] <- clust.res
		}
	}

	return(result)
}

########################################################################################################################

### Just a helper function to not copy and paste code
rnb.section.clustering.add.heatmap <- function(report, X, fname, cluster.rows, clust.result, sample.ids, locus.colors,
	sample.colors = NULL) {

	dist.metric <- clust.result@dissimilarity
	agglomeration <- clust.result@algorithm[2]
	rplot <- createReportPlot(fname, report, width = 7.2, height = 7.2,
		create.pdf = rnb.getOption("exploratory.clustering.heatmaps.pdf"), high.png = 200)
	cresult <- clust.result@result
	attr(cresult, "class") <- "hclust"
	cresult <- as.dendrogram(cresult)

	meth.colors <- get.methylation.color.panel()
	meth.breaks <- seq(0, 1, length.out = length(meth.colors) + 1L)
	heatmap.parameters <- list(x = X, Rowv = FALSE, Colv = cresult, dendrogram = "column", scale = "none",
		breaks = meth.breaks, col = meth.colors, trace = "none",
		density.info = "density", key.title = NA, key.xlab = expression(beta), key.ylab = "Density")
	if (cluster.rows) {
		## Plotting sites/regions, attempt to cluster rows
		heatmap.parameters$labRow <- NA
		heatmap.parameters$margins <- c(4, 1)
		if (dist.metric == "correlation") {
			distfun <- dist.correlation
		} else {
			distfun <- function(x) { dist(x, method = dist.metric) }
		}
		cc <- tryCatch(hclust(distfun(X), method = agglomeration), error = function(err) {
				## Row dendrogram cannot be drawn
				logger.warning(c("Could not cluster rows of the heatmap with method", dist.metric, "and linkage",
						agglomeration, ". Discarding row dendrogram"))
				return(NULL)
			}
		)
		if (!is.null(cc)) {
			heatmap.parameters$Rowv <- as.dendrogram(cc)
			heatmap.parameters$dendrogram <- "both"
		}
	} else {
		## Plotting quantiles
		heatmap.parameters$labRow <- rownames(X)
		heatmap.parameters$margins <- c(4, 3)
	}
	if(any(nchar(sample.ids)>10)){
		heatmap.parameters$margins[1] <- 10 
	}
	heatmap.parameters$labCol <- sample.ids
	heatmap.parameters$RowSideColors <- locus.colors
	heatmap.parameters$ColSideColors <- sample.colors
	tryCatch(suppressWarnings(do.call(heatmap.2, heatmap.parameters)), error = function(err) {
			print(rnb.message.plot("Heatmap could not be created"))
			list()
		}
	)
	return(off(rplot))
}

########################################################################################################################

#' rnb.section.clustering
#'
#' Creates a report section dedicated to sample clustering.
#'
#' @param report          Report on methylation profiles to contain the clustering section. This must be an object of
#'                        type \code{\linkS4class{Report}}.
#' @param rnb.set         Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param clust.results   Results of applying hierarchical clustering as an object of type
#'                        \code{\linkS4class{RnBeadClustering}}, or a non-empty list of such objects.
#' @param site.reg.info   Information about traits to be visualized and tested for associations with clusters, as
#'                        returned by \code{\link{get.site.and.region.types}}.
#' @param section.started Flag indicating if the clustering section in the given report is already created.
#' @return The modified report.
#'
#' @author Yassen Assenov, Fabian Mueller
#' @noRd
rnb.section.clustering <- function(report, rnb.set, clust.results, site.reg.info, section.started = FALSE) {

	rnb.require("mclust")
	beta.values <- meth.matrices(rnb.set)

	if (inherits(clust.results, "RnBeadClustering")) {
		clust.results <- list(clust.results)
	}
	if (!(is.list(clust.results) && length(clust.results) != 0 &&
			all(sapply(unlist(clust.results,recursive=FALSE), inherits, "RnBeadClustering")))) {
		stop("invalid value for clust.results; expected list of list of RnBeadClustering objects")
	}
	if (!all(sapply(names(clust.results), function(rr) { all(sapply(clust.results[[rr]],
						FUN=function(x){identical(samples(x), colnames(beta.values[[rr]]))})) }))) {
		stop("invalid value for clust.results; does not match rnb.set")
	}
	dist.metrics <- unique(sapply(unlist(clust.results,recursive=FALSE), slot, "dissimilarity"))
	algorithms <- lapply(clust.results, FUN=function(x){
		lapply(x, slot, "algorithm")
	})
	if (!all(sapply(unlist(algorithms,recursive=FALSE), length) == 2)) {
		stop("unsupported clustering algorithms")
	}
	agglomerations <- unique(sapply(unlist(algorithms,recursive=FALSE), "[", 2))
	if (any(sapply(clust.results,FUN=function(x){length(dist.metrics) * length(agglomerations) != length(x)}))) {
		stop("invalid value for clust.results; list is incomplete")
	}
	if (rnb.getOption("logging") && logger.isinitialized() == FALSE) {
		logger.start(fname = NA) # initialize console logger
	}
	logger.start("Clustering Section")

	## Define sample IDs and coloring
	sample.ids <- samples(rnb.set)
	pheno.columns <- names(rnb.sample.groups(rnb.set,rnb.getOption("exploratory.columns")))
	if (length(pheno.columns) != 0) {
		pheno.colors <- rnb.pheno2colors(pheno(rnb.set)[, pheno.columns])
		if (length(pheno.columns) == 1) {
			pheno.colors$colors <- as.matrix(pheno.colors$colors)
			colnames(pheno.colors$colors) <- pheno.columns
			pheno.colors$mapping <- list(pheno.colors$mapping)
		}
		scolors <- 1:length(pheno.columns)
	} else {
		pheno.colors <- NULL
		scolors <- 0L
	}

	locus.columns <- unique(unlist(lapply(site.reg.info,FUN=function(x){colnames(x)})))
	lcolors <- 0L
	if (length(locus.columns) != 0) {
		lcolors <- 1:length(locus.columns)
	}
	
	reg.types <- intersect(names(clust.results),names(beta.values))

	## Create heatmaps
	description <- "The figure below shows clustering of samples using several algorithms and distance metrics."
	if (section.started) {
		rnb.add.paragraph(report, description)
	} else {
		report <- rnb.add.section(report, "Clustering", description)
	}
	probe.order <- lapply(beta.values,FUN=function(x){ 
		order(rowVars(x,na.rm = TRUE), decreasing = TRUE)
	})
	logger.start("Generating Heatmaps")
	report.plots <- list(quant=list(),loc=list())
	topprobes <- c(0L, rnb.getOption("exploratory.clustering.top.sites"))
	locus.colors <- list()
	for (ri in 1:length(reg.types)) {
		rt <- reg.types[ri]
		if (rt %in% names(site.reg.info)) {
			cur.annot <- colnames(site.reg.info[[rt]])
			locus.colors[[rt]] <- lapply(locus.columns,FUN=function(aa){
				if (aa %in% cur.annot){
					get.site.and.region.types.colors(site.reg.info,rt,aa)
				} else {
					NULL
				}
			})
			names(locus.colors[[rt]]) <- locus.columns
		} else {
			locus.colors[[rt]] <- NULL
		}
		
		logger.start(c("Region type:",rt))
		clust.res <- clust.results[[rt]]
		for (tprobes in topprobes){
			tprobes.trunc <- tprobes #truncate if tprobes exceeds the number of sites/regions
			if (tprobes == 0) {
				X <- apply(beta.values[[rt]], 2, quantile, probs = seq(0, 1, length = 101), na.rm = TRUE)
				rownames(X)[1:nrow(X) %% 10 != 1] <- ""
			} else {
				tprobes.trunc <- min(c(tprobes,length(probe.order[[rt]])))
				X <- beta.values[[rt]][probe.order[[rt]][1:tprobes.trunc], ]
			}
			for (i in 1:length(clust.res)){
				clust.result <- clust.res[[i]]
				dist.metric <- clust.result@dissimilarity
				agglomeration <- clust.result@algorithm[2]
				fname.base <- paste(ri, dist.metric, agglomeration, sep = "_")
				for (si in scolors) {
					
					sample.colors <- NULL
					locus.colors.cur <- NULL
					fname <- fname.base
					if (si != 0) {
						fname <- paste(fname.base, si, sep = "_")
						sample.colors <- pheno.colors$colors[, si]
					}
					if (tprobes != 0){
						fname.base.2 <- paste("heatmapLoc",fname,sep="_")
						fname <- fname.base.2
						for (li in lcolors) {
							if (li != 0) {
								fname <- paste(fname.base.2, li, sep = "_")
								locus.colors.cur <- locus.colors[[rt]][[locus.columns[li]]][probe.order[[rt]][1:tprobes.trunc]]
							}
							fname <- paste(fname, tprobes, sep = "_")
							rplot <- rnb.section.clustering.add.heatmap(report,X,fname,TRUE,clust.result,sample.ids,locus.colors.cur,sample.colors)

							report.plots[["loc"]] <- c(report.plots[["loc"]],rplot)
						}
					} else {
						fname <- paste("heatmapQuant",fname,sep="_")
						rplot <- rnb.section.clustering.add.heatmap(report,X,fname,FALSE,clust.result,sample.ids,NULL,sample.colors)
						report.plots[["quant"]] <- c(report.plots[["quant"]],rplot)
					}
				}
			}
		}
		logger.completed()
	}
	setting.names.common <- list(
		"Site/region level" = reg.types,
		"Dissimilarity metric" = DIST.METRICS[dist.metrics],
		"Agglomeration strategy (linkage)" = AGGLOMERATIONS[agglomerations])
	names(setting.names.common[["Site/region level"]]) <- 1:length(reg.types)

	setting.names <- setting.names.common
	if (max(scolors) != 0) {
		sam.colors <- colnames(pheno.colors$colors)
		names(sam.colors) <- 1:ncol(pheno.colors$colors)
		setting.names[["Sample color based on"]] <- sam.colors
		rm(sam.colors)
	}
	txt.clustering <- paste0("Hierarchical clustering of samples based on ",
		ifelse(rnb.getOption("exploratory.clustering") == "all", "all methylation values",
		paste(max(rnb.getOption("exploratory.clustering.top.sites")), "most variable loci")), ". The heatmap displays ")
	description <- c(txt.clustering, "methylation percentiles per sample. ",
		"The legend for sample coloring can be found in the figure below.")
	report <- rnb.add.figure(report, description, report.plots[["quant"]], setting.names)

	if (max(lcolors) != 0) {
		lcolors <- locus.columns
		names(lcolors) <- 1:length(locus.columns)
		setting.names[["Site/region color based on"]] <- lcolors
	}
	setting.names[["Visualize"]] <- paste(topprobes[-1], "most variable loci")
	names(setting.names[["Visualize"]]) <- topprobes[-1]
	description <- c(txt.clustering, "only selected sites/regions with the highest variance across all samples. ",
		"The legend for locus and sample coloring can be found in the figure below.")
	report <- rnb.add.figure(report, description, report.plots[["loc"]], setting.names)

	logger.status(c("Created", length(report.plots[["quant"]]) + length(report.plots[["loc"]]),
		"heatmaps based on the clustering results"))
	logger.completed()

	## Add color legends
	logger.start("Adding Color Legends")
	if (!is.null(pheno.colors) || length(locus.columns) != 0) {
		add.legend <- function(s.index, legend.colors) {
			screen(s.index)
			par(mar = c(0, 0, 0, 0) + 0.1)
			plot.new()
			legend("center", legend = names(legend.colors), fill = legend.colors, bty = "n", ncol = 1)
		}
		ns <- length(scolors)
		nl <- length(lcolors)
		## TODO: Simplify this weird parameter computations
		param.mat <- c()
		if (max(scolors) == 0) {
			param.mat <- cbind(0,1:nl)
		} else {
			for (i in 1:ns){
				if (nl == 0) {
					param.mat <- rbind(param.mat,cbind(i,0))
				} else {
					param.mat <- rbind(param.mat,cbind(i,1:nl))
				}
			}
		}

		report.plots <- list()
		for (ri in 1:length(reg.types)) {
			rt <- reg.types[ri]
			for (i in 1:(dim(param.mat)[1])) {
				fname <- paste("heatmapLegend",ri, param.mat[i,1],param.mat[i,2], sep = "_")
				rplot <- createReportPlot(fname, report, width = 7.2, height = 1.8, high.png = 200)
				screens <- rbind(c(0, 0.5, 0, 1), c(0.5, 1, 0, 1))
				s.indices <- split.screen(screens)
				if (param.mat[i,2] != 0){
					leg.loc <-  attr(site.reg.info[[rt]],"legend")[[lcolors[param.mat[i,2]]]]
					if (!is.null(leg.loc)){
						#convert color gradient
						if (!is.character(leg.loc)){
							if (leg.loc == 2){
								colors.grad <- rnb.getOption("colors.gradient")
								vals <- round(site.reg.info[[rt]][,lcolors[param.mat[i,2]]],2)
								leg.loc <- colors.grad[1:2]
								names(leg.loc) <- c(min(vals),max(vals))
							} else {
								stop("Invalid legend")
							}
						}
						add.legend(s.indices[1],leg.loc)
					}

				}
				if (param.mat[i,1] != 0) {
					add.legend(s.indices[2], pheno.colors$mapping[[param.mat[i,1]]])
				}
				close.screen(all.screens = TRUE)
				off(rplot)
				report.plots <- c(report.plots,rplot)
			}
		}
		if (max(lcolors) == 0) {
			lcolors <- "n.a."
			names(lcolors) <- 0
		}
		if (max(scolors) == 0) {
			scolors <- "n.a."
			names(scolors) <- 0
		} else {
			scolors <- colnames(pheno.colors$colors)
			names(scolors) <- 1:ncol(pheno.colors$colors)
		}
		regs <- reg.types
		names(regs) <- 1:length(reg.types)
		setting.names.leg <- list(
				"Site/region level" = regs,
				"Sample color based on" = scolors,
				"Site/region color based on" = lcolors)
		
		description <- c("Probe and sample colors used in the heatmaps in the previous figures.")
		report <- rnb.add.figure(report, description, report.plots, setting.names.leg)
	}
	rm(scolors, description, probe.order, topprobes, tprobes, X)
	logger.completed()

	## Create plots of silhouette values
	logger.start("Estimating Optimal Numbers of Clusters")
	refText <- c("Rousseeuw, P. J. (1987) Silhouettes: A graphical aid to the interpretation and validation of ",
		"cluster analysis. <i>Journal of Computational and Applied Mathematics</i>, <b>20</b>, 53-65")
	report <- rnb.add.reference(report, refText)
	description <- c("Using the average silhouette value as a measure of cluster assignment ",
		rnb.get.reference(report, refText), ", it is possible to infer the number of clusters produced by each of the ",
		"studied methods. The figure below shows the corresponding mean silhouette value for every observed ",
		"separation into clusters.")
	report <- rnb.add.section(report, "Identified Clusters", description, level = 3)
	## TODO: Support also different lengths of silhouettes
	colnames.sil <- lapply(reg.types,FUN=function(rt){
		as.character(AGGLOMERATIONS[sapply(algorithms[[rt]], "[", 2)])
	})
	names(colnames.sil) <- reg.types
	silhouettes <- lapply(reg.types,FUN=function(rt){
			sils <- as.matrix(sapply(clust.results[[rt]], slot, "silhouettes"))
			colnames(sils) <- colnames.sil[[rt]]
			return(sils)
	})
	names(silhouettes) <- reg.types
	algorithm.names <- lapply(algorithms,FUN=function(algs){
		sapply(algs, function(a.name) {
			base::ifelse(length(a.name) == 2, paste0(a.name[1], " (", a.name[2], " linkage)"), a.name)
		})
	})
	clusters.identified.inner <- data.frame(
		"Metric" = character(0),
		"Algorithm" = character(0),
		"Clusters" = integer(0),
		stringsAsFactors = FALSE)
	clusters.identified <- list()
	report.plots.all <- list()
	for (rt in 1:length(reg.types)){
		rt <- reg.types[ri]
		clusters.identified.cur <- clusters.identified.inner
		report.plots <- lapply(dist.metrics, function(dist.metric) {
			cinds <- (sapply(clust.results[[rt]], slot, "dissimilarity") == dist.metric)
			for (i in which(cinds)) {
				cn <- as.integer(rownames(silhouettes[[rt]])[which.max(silhouettes[[rt]][, i])])
				clusters.identified.cur[nrow(clusters.identified.cur) + 1, "Metric"] <<- DIST.METRICS[dist.metric]
				clusters.identified.cur[nrow(clusters.identified.cur), "Algorithm"] <<- algorithm.names[[rt]][i]
				clusters.identified.cur[nrow(clusters.identified.cur), "Clusters"] <<- cn
			}
			dframe <- data.frame(
				"clusters" = rep(as.integer(rownames(silhouettes[[rt]])), sum(cinds)),
				"silhouette" = as.vector(silhouettes[[rt]][, cinds]),
				"linkage" = rep(colnames(silhouettes[[rt]])[cinds], each = nrow(silhouettes[[rt]])), check.names = FALSE)
			fname <- paste("silhouette", ri, dist.metric, sep = "_")
			rplot <- createReportPlot(fname, report, width = 7, height = 5)
			pp <- ggplot(dframe, aes_string(x = "clusters", y = "silhouette", colour = "linkage", group = "linkage")) +
				scale_colour_manual(values = rnb.getOption("colors.category")) +
				labs(x = paste("Number of clusters"), y = paste("Silhouette value")) + geom_line()
			print(pp)
			off(rplot)
			rplot
		})
		report.plots.all <- c(report.plots.all,report.plots)
		clusters.identified <- c(clusters.identified,list(clusters.identified.cur))
	}
	names(clusters.identified) <- 1:length(reg.types)
	description <- c("Line plot visualizing mean silhouette values of the clustering algorithm outcomes for each ",
		"applicable value of <i>K</i> (number of clusters).")
	report <- rnb.add.figure(report, description, report.plots.all, setting.names.common[1:2])
	description <- "The table below summarizes the number of clusters identified by the algorithms."
	rnb.add.paragraph(report, description)
	report <- rnb.add.tables(report, clusters.identified, setting.names.common[1], row.names = FALSE)
	logger.status("Estimated number of clusters based on mean silhouette value")
	logger.completed()

	logger.start("Overlapping Clusters with Sample Traits")
	if (!is.null(pheno.colors)) {
		report <- rnb.add.section(report, "Clusters and Traits", NULL, level = 3)

		## Compute Rand indices to test for associations between clusterings and traits	
		randIndices.all <- list()
		csv.files <- sapply(reg.types, function(x) { as.character(NA) })
		for (ri in 1:length(reg.types)){
			rt <- reg.types[ri]
			randIndices <- matrix(as.double(NA), nrow = nrow(clusters.identified[[ri]]), ncol = ncol(pheno.colors$colors))
			dimnames(randIndices) <- list(clusters.identified[[ri]][, "Algorithm"], colnames(pheno.colors$colors))
			dist.metrics <- DIST.METRICS[sapply(clust.results[[rt]], slot, "dissimilarity")]
			for (i in 1:nrow(clusters.identified[[ri]])) {
				j <- which(dist.metrics == clusters.identified[[ri]][i, "Metric"] &
						algorithm.names[[rt]] == clusters.identified[[ri]][i, "Algorithm"])
				cl.assignments <- clust.results[[rt]][[j]]@assignments[, as.character(clusters.identified[[ri]][i, "Clusters"])]
				for (j in colnames(randIndices)) {
					tr.assignments <- as.integer(as.factor(pheno(rnb.set)[, j]))
					valids <- (!is.na(tr.assignments))
					randIndices[i, j] <- mclust::adjustedRandIndex(cl.assignments[valids], tr.assignments[valids])
				}
			}
			randIndices.all <- c(randIndices.all,list(randIndices))
			rm(dist.metrics, i, j, cl.assignments, tr.assignments, valids)
	
			## Save the table to an external file
			rIndices <- cbind(clusters.identified[[ri]][, c("Metric", "Algorithm")], randIndices)
			csv.files[rt] <- paste("adjusted_rand_indices_",ri,".csv",sep="")
			fname.full <- file.path(rnb.get.directory(report, "data", absolute = TRUE), csv.files[rt])
			utils::write.csv(rIndices, file = fname.full, row.names = FALSE)
			csv.files[rt] <- paste(rnb.get.directory(report, "data"), csv.files[rt], sep = "/")
			logger.status(c("Computed adjusted rand indices and saved to", csv.files[rt]))
		}
		names(randIndices.all) <- reg.types
		refText <- c("Hubert, L. and Arabie, P. (1985) Comparing partitions. ",
			"<i>Journal of Classification</i>, <b>2</b>(1), 193-218")
		report <- rnb.add.reference(report, refText)
		txt <- c("The figure below shows associations between clusterings and the examined traits. Associations are ",
			"quantified using the adjusted Rand index ", rnb.get.reference(report, refText), ". Rand indices near 1 ",
			"indicate high agreement while values close to -1 indicate seperation. The full table of all computed ",
			"indices is stored in the following comma separated files:")
		rnb.add.paragraph(report, txt)
		## Add a list of links to the created CSV files
		txt <- as.list(paste("<a href=\"", csv.files, "\">", names(csv.files), " (csv)</a>", sep = ""))
		rnb.add.list(report, txt)
		rm(csv.files, ri, rt, randIndices, rIndices, fname.full, refText, txt)

		## Create heatmaps of adjusted Rand indices
		report.plots.all <- list()
		for (ri in 1:length(reg.types)){
			rt <- reg.types[ri]
			report.plots <- lapply(names(DIST.METRICS), function(dist.metric) {
				fname <- paste("heatmap", "rindex", ri, dist.metric, sep = "_")
				cinds <- which(DIST.METRICS[dist.metric] == clusters.identified[[ri]][, "Metric"])
				return(plot.heatmap.rand(t(randIndices.all[[rt]][cinds, ]), fname, report))
			})
			report.plots.all <- c(report.plots.all,report.plots)
		}
		description <- c("Heatmap visualizing Rand indices computed between sample traits (rows) and clustering ",
			"algorithm outcomes (columns).")
		report <- rnb.add.figure(report, description, report.plots.all, setting.names.common[1:2])
	}
	logger.completed()
	logger.completed()
	return(report)
}

########################################################################################################################

#' rnb.step.clustering
#'
#' Performs hierarchical clustering on the samples of the given dataset and generates a corresponding section in the
#' report.
#'
#' @param rnb.set Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param report  Report on methylation profiles to contain the clustering section. This must be an object of type
#'                \code{\linkS4class{Report}}.
#' @return The modified report.
#'
#' @seealso \code{\link{rnb.execute.clustering}}, \code{\link{rnb.section.clustering}}
#' @author Yassen Assenov
#' @noRd
rnb.step.clustering <- function(rnb.set, report) {
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	rnb.step.clustering.internal(rnb.set, report, get.site.and.region.types(rnb.set))
}

rnb.step.clustering.internal <- function(rnb.set, report, rinfo) {
	logger.start("Sample Clustering")

	## Perform clustering algorithms
	logger.start("Agglomerative Hierarchical Clustering")
	clust.results <- rnb.execute.clustering.all(rnb.set)
	logger.completed()

	## Handle the cases when clusterings on some or all of the methylation matrices failed
	clust.failed <- sapply(clust.results, is.null)
	clust.edited <- sum(clust.failed) != 0
	if (clust.edited) {
		failed.single <- sum(clust.failed) == 1
		failed.all <- sum(clust.failed) == length(clust.results)
		txt <- "The clustering algorithm was not completed"
		if (failed.all) {
			txt <- c(txt, ifelse(failed.single, ".", " for all requested levels of methylation."))
		} else {
			txt <- c(txt, " for the following requested level", ifelse(failed.single, "", "s"), " of methylation - ")
			txt <- c(txt, paste("<b>", names(clust.results)[clust.failed], "</b>", sep = "", collapse = ", "), ".")
		}
		if (length(samples(rnb.set)) < 3) {
			txt <- c(txt, " Note that clustering is performed only when the dataset contains at least 3 samples.")
		} else {
			txt <- c(txt, " The most likely reason for this error is inability to calculate a distance between a pair ",
				"of samples due to (too many) missing values in the respective methylation ",
				ifelse(failed.single, "matrix", "matrices"), ".")
		}
		report <- rnb.add.section(report, "Clustering", txt)
		if (failed.all) {
			logger.completed()
			return(report)
		}
		clust.results <- clust.results[!clust.failed]
		rm(failed.single, failed.all, txt)
	}
	rm(clust.failed)

	## Generate clustering section in the report
	report <- rnb.section.clustering(report, rnb.set, clust.results, rinfo, clust.edited)
	logger.completed()
	return(report)
}

########################################################################################################################
## dataset_generator.R
## created: 2012-11-27
## creator: Yassen Assenov
## ---------------------------------------------------------------------------------------------------------------------
## Routines for generating random data to be used in unit tests.
########################################################################################################################

## F U N C T I O N S ###################################################################################################

## rnb.generate.betas
##
## Generates a matrix of random values in the range [0, 1].
##
## @param site.names   \code{character} vector to use for row names of the matrix.
## @param sample.names \code{character} vector to use for column names of the matrix.
## @return Matrix of beta values with dimension names \code{site.names} and \code{sample.names}. There are no missing
##         values (\code{NA}s) in the matrix.
##
## @author Yassen Assenov
rnb.generate.betas <- function(site.names, sample.names = LETTERS[1:12]) {
	N <- length(site.names) * length(sample.names)
	N.bump <- as.integer(0.03 * N) # maximum number of values to direct towards the "bump" at 0.5
	values.bump <- rnorm(N.bump, mean = 0.5, sd = 0.15)
	values.bump <- values.bump[0 <= values.bump & values.bump <= 1]
	values.peaks <- rbeta(N - length(values.bump), 0.5, 0.5, ncp = 0)
	values <- sample(c(values.bump, values.peaks))
	matrix(values, nrow = length(site.names), ncol = length(sample.names), dimnames = list(site.names, sample.names))
}

########################################################################################################################

## rnb.generate.intensities
##
## Generates a list of two matrices with intensity values corresponding to a given beta-value matrix. The matrix should
## have row and column names
##
## @param beta.values   \code{numeric} matrix of beta-values
## @param average.intensity \code{numeric} vector of length 1 giving mean total intensity
## @param sd.intensity	\code{numeric} vector of length 1 giving standard deviation of the total intensity
##
## @return List of two matrices with dimension equal to \code{beta.values}. There are no missing
##         values (\code{NA}s) in the matrix.
##
## @author Yassen Assenov
rnb.generate.intensities <- function(beta.values, average.intensity=20000, sd.intensity=2000) {
	
	nsites<-nrow(beta.values)
	nsamples<-ncol(beta.values)
	
	site.total<-rnorm(nsites, average.intensity, sd.intensity)
	
	total<-t(sapply(site.total, rnorm, n=nsamples, sd=sd.intensity/100))
	colnames(total)<-colnames(beta.values)
	rownames(total)<-rownames(beta.values)
	
	methylated<-beta.values * total
	unmethylated<-(1-beta.values) * total
			
	list(M=methylated, U=unmethylated)
}

########################################################################################################################

## rnb.generate.pvalues
##
## Generates a matrix of random p-values, the vast majority of which are very small.
##
## @param site.names   \code{character} vector to use for row names of the matrix.
## @param sample.names \code{character} vector to use for column names of the matrix.
## @return Matrix of p-values values with dimension names \code{site.names} and \code{sample.names}. There are no
##         missing values (\code{NA}s) in the matrix.
##
## @author Yassen Assenov
rnb.generate.pvalues <- function(site.names, sample.names = LETTERS[1:12]) {
	N <- length(site.names) * length(sample.names)
	values <- rexp(as.integer(N * 0.1), rate = 12) / 12
	values <- sample(c(rep(min(values) / 2, N - length(values)), values))
	matrix(values, nrow = length(site.names), ncol = length(sample.names), dimnames = list(site.names, sample.names))
}

########################################################################################################################

## rnb.generate.annotation
##
## Generates an annotation table for 12 samples coming from the same array.
##
## @return Sample annotation table in the form of a \code{data.frame}.
##
## @author Yassen Assenov
rnb.generate.annotation <- function() {
	sentrix.id <- paste("1", paste(sample.int(10, size = 9, replace = TRUE) - 1L, collapse = ""), sep = "")
	sentrix.id <- as.integer(sentrix.id)
	sentrix.pos <- paste("R0", rep(1:6, each = 2), "C0", rep(1:2, 6), sep = "")
	tissue <- "lung"
	disease.states <- c("healthy", "primary tumor")
	disease <- factor(disease.states[c(1, 1, 2, 2, 2, 1, 2, NA, NA, 2, 2, 2)])
	diagnoses <- c("2012-11-26", "2012-11-26", "2012-11-26", "2012-11-26", "2012-11-27", "2012-11-27", NA, NA,
		"2012-12-01", "2012-12-01", "2012-12-01", "2012-12-02")
	result <- data.frame(
		"Sentrix_ID" = sentrix.id,
		"Sentrix_Position" = sentrix.pos,
		"Tissue" = tissue,
		"Disease" = disease,
		"Diagnose Date" = diagnoses,
		"barcode" = paste(sentrix.id, sentrix.pos, sep = "_"), check.names = FALSE, stringsAsFactors = FALSE)
	rownames(result) <- LETTERS[1:nrow(result)]
	return(result)
}

########################################################################################################################

## rnb.generate.infinium450.dataset
##
## Generates an object of type \code{RnBeadSet} containing approximately 4800 probes and 12 samples.
##
## @return The newly initialized dataset.
##
## @author Yassen Assenov
rnb.generate.infinium450.dataset <- function() {
	## Select randomly 1% of the probe names
	annot <- rnb.annotation2data.frame(rnb.get.annotation("probes450"))
	site.names <- rownames(annot)[sort(sample(1:nrow(annot), size = round(nrow(annot) * 0.01)))]
	## Generate data matrices and sample annotation table
	beta.matrix <- rnb.generate.betas(site.names)
	pval.matrix <- rnb.generate.pvalues(site.names)
	phenotypes <- rnb.generate.annotation()
	## Initialize an object
	new("RnBeadSet", pheno = phenotypes, betas = beta.matrix, p.values = pval.matrix, bead.counts = NULL)
}

## rnb.generate.infinium450.raw.dataset
##
## Generates an object of type \code{RnBeadRawSet} containing approximately 4800 probes and 12 samples.
##
## @return The newly initialized dataset.
##
## @author Yassen Assenov
rnb.generate.infinium450.raw.dataset <- function() {
	## Select randomly 1% of the probe names
	annot <- rnb.annotation2data.frame(rnb.get.annotation("probes450"))
	site.names <- rownames(annot)[sort(sample(1:nrow(annot), size = round(nrow(annot) * 0.01)))]
	## Generate data matrices and sample annotation table
	beta.matrix <- rnb.generate.betas(site.names)
	intensities <- rnb.generate.intensities(beta.matrix)
	pval.matrix <- rnb.generate.pvalues(site.names)
	phenotypes <- rnb.generate.annotation()
	## Initialize an object
	new("RnBeadRawSet", pheno = phenotypes, M=intensities$M, U=intensities$U, p.values = pval.matrix, bead.counts.M = NULL, bead.counts.U = NULL)
}

########################################################################################################################
## filtering.R
## created: 2012-06-04
## creator: Yassen Assenov
## ---------------------------------------------------------------------------------------------------------------------
## Implementation of the SNP removal and NA removal steps of the filtering steps of the preprocessing module.
########################################################################################################################

## G L O B A L S #######################################################################################################

HIGH.COVER.OUTLIER.QUANTILE <- 0.95 #used for high coverage outlier filtering
HIGH.COVER.OUTLIER.FACTOR <- 50 #used for high coverage outlier filtering

## F U N C T I O N S ###################################################################################################

## rnb.filtering.results
##
## Constructs the list with common elements returned by execute functions in the filtering steps.
##
## @param rnb.set  Methylation dataset before filtering.
## @param filtered \code{logical} vector signifying which of the sites in \code{rnb.set} will be removed.
## @return List of three or four elements:
##         \describe{
##           \item{\code{"dataset.before"}}{Copy of \code{rnb.set}.}
##           \item{\code{"dataset"}}{The (possibly modified) dataset after performing site removal.}
##           \item{\code{"filtered"}}{Indices of sites in \code{rnb.set} that were removed.}
##           \item{\code{"betas"}}{\code{matrix} of the beta values for the sites that were removed from the dataset.
##                This element is added to the list only when \code{filtered} is non-empty.}
##         }
## @author Yassen Assenov
rnb.filtering.results <- function(rnb.set, filtered = NULL) {
	if (is.null(filtered)) {
		filtered <- integer(0)
	} else {
		filtered <- which(filtered)
	}
	if (length(filtered) != 0) {
		return(list(dataset.before = rnb.set, dataset = remove.sites(rnb.set, filtered, verbose = TRUE),
				filtered = filtered, betas = meth(rnb.set)[filtered, ]))
	}
	rnb.cleanMem()
	return(list(dataset.before = rnb.set, dataset = rnb.set, filtered = filtered))
}

########################################################################################################################

## validate.stats
##
## Validates that the given statistics are (partially) produced by \code{\link{rnb.filtering.results}}.
##
## @param stats Filtering result statistics.
## @ author Yassen Assenov
validate.stats <- function(stats) {
	if (!is.list(stats) && all(c("dataset.before", "dataset", "filtered") %in% names(stats))) {
		stop("invalid value for stats")
	}
	if (!inherits(stats$dataset.before, "RnBSet")) {
		stop("invalid value for stats$dataset.before; expected methylation dataset")
	}
	if (!inherits(stats$dataset, "RnBSet")) {
		stop("invalid value for stats$dataset; expected methylation dataset")
	}
	samples.before <- colnames(meth(stats$dataset.before))
	samples.after <- colnames(meth(stats$dataset))
	if (!identical(samples.after, samples.before)) {
		stop("invalid value for stats; incompatible dataset.before and dataset")
	}
	site.count.before <- nsites(stats$dataset.before)
	site.count.after <- nsites(stats$dataset)
	if (!(site.count.after <= site.count.before)) {
		stop("invalid value for stats; incompatible dataset.before and dataset")
	}
	filtered <- stats$filtered
	if (!(is.integer(filtered) && all(!is.na(filtered)) && all(1L <= filtered) && anyDuplicated(filtered) == 0)) {
		stop("invalid value for stats$filtered")
	}
	if (!(all(filtered <= site.count.before) && site.count.before == length(filtered) + site.count.after)) {
		stop("invalid value for stats; incompatible dataset.before, dataset and filtered")
	}
}

########################################################################################################################

## rnb.save.removed.sites
##
## Creates a table with information on the specified removed sites or probes and saves it to a file.
##
## @param anno.table     Table of site or probe annotation.
## @param report         Report to link to the generated file with an annotation table.
## @param fname          Name of the file to store the created annotation table.
## @param p.columns      Column names from the complete annotation table to include in the created one.
## @param p.columns.file Column names to use for the created table.
## @param filtered       Indices to include in the created annotation table.
## @param ...            Any additional annotation columns, specified as named parameters.
## @return Name of the generated file, as a local path with respect to the given report.
##
## @author Yassen Assenov
rnb.save.removed.sites <- function(anno.table, report, fname, p.columns = c("ID", "Chromosome", "Start", "End"),
	p.columns.file = p.columns, ...) {

	p.columns <- intersect(p.columns, colnames(anno.table))
	p.infos <- anno.table[, p.columns, drop = FALSE]

	additional.values <- list(...)
	for (i in names(additional.values)) {
		p.infos[, i] <- additional.values[[i]]
	}
	fname.full <- file.path(rnb.get.directory(report, "data", absolute = TRUE), fname)
	utils::write.csv(p.infos, file = fname.full, row.names = FALSE)
	rnb.status(c("Saved removed sites to", fname.full))
	return(paste(rnb.get.directory(report, "data"), fname, sep = "/"))
}

########################################################################################################################

#' rnb.execute.context.removal
#'
#' Removes all probes that belong to specific context from the given dataset.
#'
#' @param rnb.set  Methylation dataset as an object of type \code{\linkS4class{RnBeadSet}}.
#' @param contexts Probe contexts to be filtered out.
#' @return List of three or four elements:
#'         \describe{
#'           \item{\code{"dataset.before"}}{Copy of \code{rnb.set}.}
#'           \item{\code{"dataset"}}{The (possibly modified) \code{RnBeadSet} object after performing the missing
#'                value removal.}
#'           \item{\code{"filtered"}}{\code{integer} vector storing the indices of all removed probes in
#'                \code{dataset.before}.}
#'           \item{\code{"contexts"}}{The value of the parameter \code{contexts}.}
#'         }
#'
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' contexts.to.ignore <- c("CC", "CAG", "CAH")
#' rnb.set.filtered <- rnb.execute.context.removal(rnb.set.example, contexts.to.ignore)$dataset
#' identical(rnb.set.example, rnb.set.filtered) # FALSE
#' }
#' @author Yassen Assenov
#' @export
rnb.execute.context.removal <- function(rnb.set, contexts = rnb.getOption("filtering.context.removal")) {
	if (!inherits(rnb.set, "RnBeadSet")) {
		stop("invalid value for rnb.set")
	}
	## TODO: Validate contexts

	filtered <- rnb.execute.context.removal.internal(integer(), contexts, annotation(rnb.set, add.names = TRUE))
	return(list(dataset.before = rnb.set, dataset = remove.sites(rnb.set, filtered), filtered = filtered,
			contexts = contexts))
}

rnb.execute.context.removal.internal <- function(sites2ignore, contexts, anno.table) {
	# if (any(grepl("^nv", anno.table[, "ID"]))) {
	# 	logger.info("Detected nv probes in the dataset. Will not remove any nv probes based on context")
	# 	return(setdiff(which(anno.table[, "Context"] %in% contexts & !grepl("^nv",anno.table[, "ID"])), sites2ignore))
	# }
	setdiff(which(anno.table[, "Context"] %in% contexts), sites2ignore)
}

########################################################################################################################

#' rnb.section.context.removal
#'
#' Adds a section on context-specific probe removal to the specified report.
#'
#' @param report Report to summarize the outcome of the procedure. This must be an object of type
#'               \code{\linkS4class{Report}}.
#' @param stats  Statistics on context-specific probe filtering, as returned by
#'               \code{\link{rnb.execute.context.removal}}. See the documentation of the function for more details.
#' @return The modified report.
#'
#' @author Yassen Assenov
#' @noRd
rnb.section.context.removal <- function(report, stats) {
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	validate.stats(stats)
	## TODO: Validate stats$contexts
	rnb.section.context.removal.internal(report, stats, annotation(stats$dataset.before, add.names = TRUE))
}

rnb.section.context.removal.internal <- function(report, contexts, filtered, anno.table) {

	if (length(contexts) == 0) {
		return(report)
	}
	f.count <- length(filtered)
	txt.title <- "Context-specific Probe Removal"
	txt <- paste("The studied dataset contains",
		ifelse(f.count == 0, "no probes", ifelse(f.count == 1, "1 (one) probe", paste("in total", f.count, "probes"))),
		"of the specified", ifelse(length(contexts) == 1, "context.", "contexts."))
	if (f.count == 0) {
		report <- rnb.add.section(report, txt.title, txt)
		return(report)
	}

	## Construct a table of selected information for all removed probes
	p.columns <- c("ID", "Chromosome", "Start", "End", "Context")
	fname <- "removed_sites_context.csv"
	fname <- rnb.save.removed.sites(anno.table[filtered, ], report, fname, p.columns)
	txt <- c(txt, ifelse(f.count == 1, " This (removed) probe is", " All these (removed) probes are "),
		"available in a <a href=\"", fname, "\">dedicated table</a> accompanying this report.")
	rm(p.columns, fname)

	## Construct a summary table of number of removed probes per context
	probe.infos <- as.character(anno.table[filtered, "Context"])
	removed.summary <- data.frame(
		"Context" = contexts,
		"Probes" = sapply(contexts, function(x) { sum(probe.infos == x) }))

	if (nrow(removed.summary) == 1) {
		report <- rnb.add.section(report, txt.title, txt)
		return(report)
	}
	txt <- c(txt, " The table below summarizes the number of removed probes per context.")
	report <- rnb.add.section(report, txt.title, txt)
	rnb.add.table(report, removed.summary, row.names = FALSE)
	return(report)
}

########################################################################################################################

#' rnb.step.context.removal
#'
#' Performs the procedure for context-specific probe removal (if applicable) to filter the given methylation dataset
#' and adds a corresponding section to the specified report.
#'
#' @param rnb.set Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param report  Report to summarize the outcome of this procedure. This must be an object of type
#'                \code{\linkS4class{Report}}.
#' @return List of up to three elements:
#'         \describe{
#'           \item{\code{"dataset"}}{The (possibly modified) \code{RnBeadSet} object after performing context-specific
#'                probe removal.}
#'           \item{\code{"report"}}{The modified report.}
#'           \item{\code{"filtered"}}{}
#'           \item{\code{"betas"}}{\code{matrix} of the beta values for probes that have undesired contexts. These
#'                values were essentially removed from the dataset. This element is added to the list only when
#'                \code{rnb.set} contains such probes.}
#'         }
#'
#' @author Yassen Assenov
#' @noRd
rnb.step.context.removal <- function(rnb.set, report) {
	if (!inherits(rnb.set, "RnBeadSet")) {
		stop("invalid value for rnb.set")
	}
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	result <- rnb.step.context.removal.internal(integer(), report, annotation(rnb.set, add.names = TRUE))
	return(list(dataset = remove.sites(rnb.set, result$filtered), report = result$report))
}

rnb.step.context.removal.internal <- function(sites2ignore, report, anno.table) {
	logger.start("Probe Context Removal")
	contexts <- rnb.getOption("filtering.context.removal")
	filtered <- rnb.execute.context.removal.internal(sites2ignore, contexts, anno.table)
	logger.status(c("Removed", length(filtered), "probe(s) having not acceptable context"))
	report <- rnb.section.context.removal.internal(report, contexts, filtered, anno.table)
	logger.status("Added a corresponding section to the report")
	logger.completed()
	list(report = report, filtered = filtered)
}

########################################################################################################################

#' rnb.execute.cross.reactive.removal
#'
#' Removes all probes defined as cross-reactive from the given dataset.
#'
#' @param rnb.set Methylation dataset as an object of type inheriting \code{\linkS4class{RnBeadSet}}.
#' @return \code{list} of four elements:
#'         \describe{
#'           \item{\code{"dataset.before"}}{Copy of \code{rnb.set}.}
#'           \item{\code{"dataset"}}{The (possibly) modified dataset object after removing probes that have a high
#'                likelihood of cross-hybridization.}
#'           \item{\code{"filtered"}}{\code{integer} vector storing the indices (in beta matrix of the unfiltered
#'                dataset) of all removed probes.}
#'         }
#'
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' rnb.set.filtered <- rnb.execute.cross.reactive.removal(rnb.set.example)$dataset
#' identical(meth(rnb.set.example), meth(rnb.set.filtered)) # FALSE
#' }
#' @author Yassen Assenov
#' @export
rnb.execute.cross.reactive.removal <- function(rnb.set) {
	if (!inherits(rnb.set, "RnBeadSet")) {
		stop("invalid value for rnb.set")
	}
	filtered <- rnb.execute.cross.reactive.removal.internal(integer(), annotation(rnb.set))
	list(dataset.before = rnb.set, dataset = remove.sites(rnb.set, filtered), filtered = filtered)
}

rnb.execute.cross.reactive.removal.internal <- function(sites2ignore, anno.table) {
	if (!("Cross-reactive" %in% colnames(anno.table))) {
		stop("no Cross-reactive information in the annotation table")
	}
	setdiff(which(anno.table[, "Cross-reactive"] != 0), sites2ignore)
}

########################################################################################################################

#' rnb.section.cross.reactive.removal
#'
#' Adds a section on removing cross-reactive probes to the specified report.
#'
#' @param report  Report to contain the new section. This must be an object of type \code{\linkS4class{Report}}.
#' @param rnb.set Methylation dataset before filtering as an object of type inheriting \code{\linkS4class{RnBeadSet}}.
#' @param stats   Statistics on cross-reactive probe filtering, as returned by
#'                \code{\link{rnb.execute.cross.reactive.removal}}. See the documentation of the function for more
#'                details.
#' @return The possibly modified report.
#'
#' @seealso \code{\link{rnb.execute.cross.reactive.removal}}, \code{\link{rnb.step.cross.reactive.removal}},
#'          \code{\link{rnb.run.filtering}}
#'
#' @author Yassen Assenov
#' @noRd
rnb.section.cross.reactive.removal <- function(report, rnb.set, stats) {
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	validate.stats(stats)
	if (!inherits(stats$dataset, "RnBeadSet")) {
		stop("invalid value for stats$dataset; expected RnBeadSet")
	}
	anno.table <- annotation(stats$dataset.before, add.names = TRUE)
	filtered <- stats$filtered
	if (length(filtered) != 0 && max(filtered) > nrow(anno.table)) {
		stop("invalid value for stats$filtered")
	}
	rnb.section.cross.reactive.removal.internal(report, filtered, anno.table)
}

rnb.section.cross.reactive.removal.internal <- function(report, filtered, anno.table) {
	txt.site <- rnb.get.row.token("RnBeadSet")
	txt.sites <- rnb.get.row.token("RnBeadSet", plural = TRUE)
	txt.title <- paste("Removal of Cross-reactive", capitalize(txt.sites))
	if (is.null(filtered)) {
		txt <- paste0("Filtering of cross-reactive probes was not performed because the probe annotation table does ",
			"not include information on cross-hybridization.")
		report <- rnb.add.section(report, txt.title, txt)
		return(report)
	}

	if (nrow(anno.table) < 500000L) {
		refText <- paste0("Chen, Y., Lemire, M., Choufani, S., Butcher, D.T.,  Grafodatskaya, D., Zanke, B.W., ",
			"Gallinger, S., Hudson, T.J., Weksberg, R. (2013) Discovery of cross-reactive probes and polymorphic ",
			"CpGs in the Illumina Infinium HumanMethylation450 microarray. <i>Epigenetics</i>, <b>8</b>(2), 203-209")
	} else {
		refText <- paste0("Pidsley, R., Zotenko, E., Peters, T.J., Lawrence, M.G., Risbridger, G.P., Molloy, P., ",
			"Djik, S.V., Muhlhausler, B., Strizaker, C., Clark, S.J. (2016) Critical evaluation of the Illumina ",
			"MethylationEPIC BeadChip microarray for whole-genome DNA methylation profiling. <i>Genome Biology</i>, ",
			"<b>17</b>:208")
	}
	report <- rnb.add.reference(report, refText)
	txt <- " non-specific and "
	N <- length(filtered)
	if (N == 0) {
		txt <- paste0("No probes were found that have sequences which are", txt, "have")
	} else if (N == 1) {
		txt <- paste0("<b>One</b> probe was removed because its sequence is", txt, "has")
	} else {
		txt <- paste0("<b>", N, "</b> probes were removed because their sequences are", txt, "have")
	}
	txt <- paste0(txt, " a high likelihood of cross-hybridization ", rnb.get.reference(report, refText), ".")

	if (N != 0) {
		p.columns <- c("ID", "Chromosome", "Start", "End", "Cross-reactive")
		fname <- "removed_sites_cross_reactive.csv"
		fname <- rnb.save.removed.sites(anno.table[filtered, ], report, fname, p.columns)
		txt <- paste(txt, "The", ifelse(N == 1, paste("removed", txt.site), paste("list of removed", txt.sites)),
			' is available in a <a href="', fname, '">dedicated table</a> accompanying this report.')
	}
	return(rnb.add.section(report, txt.title, txt))
}

########################################################################################################################

#' rnb.step.cross.reactive.removal
#'
#' Performs the procedure for removal of cross-reactive probes (if any) given an Infinium methylation dataset and adds a
#' corresponding section to the specified report.
#'
#' @param rnb.set Methylation dataset as an object of type inheriting \code{\linkS4class{RnBeadSet}}.
#' @param report  Report to summarize the outcome of this procedure. This must be an object of type
#'                \code{\linkS4class{Report}}.
#' @return List of two elements:
#'         \describe{
#'           \item{\code{"dataset"}}{The (possibly modified) dataset after removing cross-reactive probes.}
#'           \item{\code{"report"}}{The modified report.}
#'         }
#'
#' @seealso \code{\link{rnb.execute.cross.reactive.removal}}, \code{\link{rnb.section.cross.reactive.removal}},
#'          \code{\link{rnb.run.filtering}}
#'
#' @author Yassen Assenov
#' @noRd
rnb.step.cross.reactive.removal <- function(rnb.set, report) {
	if (!inherits(rnb.set, "RnBeadSet")) {
		stop("invalid value for rnb.set")
	}
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	if (rnb.getOption("logging") && logger.isinitialized() == FALSE) {
		logger.start(fname = NA) # initialize console logger
	}
	result <- rnb.step.cross.reactive.removal.internal(integer(), report, annotation(rnb.set, add.names = TRUE))
	return(list(dataset = remove.sites(rnb.set, result$filtered), report = result$report))
}

rnb.step.cross.reactive.removal.internal <- function(sites2ignore, report, anno.table) {
	logger.start("Removal of Cross-reactive Probes")
	filtered <- tryCatch(
		rnb.execute.cross.reactive.removal.internal(sites2ignore, anno.table), error = function(x) { NULL })
	if (is.null(filtered)) {
		filtered <- integer()
		report <- rnb.section.cross.reactive.removal.internal(report, NULL, anno.table)
	} else {
		msg <- paste("Removed", length(filtered), ifelse(length(filtered) == 1, "site", "sites"))
		logger.status(msg)
		report <- rnb.section.cross.reactive.removal.internal(report, filtered, anno.table)
	}
	logger.status("Added a corresponding section to the report")
	logger.completed()
	list(report = report, filtered = filtered)
}

########################################################################################################################

#' rnb.execute.snp.removal
#'
#' Removes all probes overlapping with single nucleotide polymorphisms (SNPs) from the given dataset.
#'
#' @param rnb.set Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param snp     Criterion for the removal of sites or probes based on overlap with SNPs. Possible values are
#'                \code{"no"}, \code{"3"}, \code{"5"}, \code{"any"} or \code{"yes"}. See the documentation of
#'                \code{\link{rnb.options}} for a detailed explanation of the procedures these values encode.
#' @return \code{list} of four elements:
#'         \describe{
#'           \item{\code{"dataset.before"}}{Copy of \code{rnb.set}.}
#'           \item{\code{"dataset"}}{The (possibly) modified dataset object after removing probes that overlap
#'                with SNPs.}
#'           \item{\code{"filtered"}}{\code{integer} vector storing the indices (in beta matrix of the unfiltered
#'                dataset) of all removed sites or probes.}
#'           \item{\code{"snp"}}{The value of the \code{snp} parameter.}
#'         }
#'
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' rnb.set.filtered <- rnb.execute.snp.removal(rnb.set.example, "any")$dataset
#' identical(meth(rnb.set.example), meth(rnb.set.filtered)) # FALSE
#' }
#' @author Yassen Assenov
#' @export
rnb.execute.snp.removal <- function(rnb.set, snp = rnb.getOption("filtering.snp")) {
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	if ((is.double(snp) || is.integer(snp)) && length(snp) == 1) {
		snp <- as.character(snp)
	}
	accepted <- .rnb.options[["accepted"]][["filtering.snp"]]
	if (!(is.character(snp) && length(snp) == 1 && isTRUE(snp %in% accepted))) {
		msg <- paste0('"', accepted, '"', collapse = ", ")
		stop(paste("invalid value for snp; expected one of", msg))
	}
	filtered <- rnb.execute.snp.removal.internal(integer(), snp[1], inherits(rnb.set, "RnBeadSet"),
		annotation(rnb.set))

	list(dataset.before = rnb.set, dataset = remove.sites(rnb.set, filtered), filtered = filtered, snp = snp)
}

rnb.execute.snp.removal.internal <- function(sites2ignore, snp, is.infinium, anno.table) {
	if (snp == "no") {
		filtered <- NULL
	} else {
		if (is.infinium) {
			## Infinium datasets
			snp.overlap.column <- paste("SNPs", ifelse(snp %in% c("3", "5"), snp, "Full"))

			if ("SNPs 3 Alternative" %in% colnames(anno.table)) { ## To detect EPICv2: only EPICv2 and EPIC anno has this column
				snp.overlap.column <- paste(snp.overlap.column, "Alternative")
			}

			if (snp.overlap.column %in% colnames(anno.table)) {
				filtered <- setdiff(which(anno.table[, snp.overlap.column] > 0), sites2ignore)
			} else {
				stop("no SNP annotation")
			}
		} else {
			## Bisulfite sequencing datasets
			snp.overlap.column <- "SNPs"
			if (snp.overlap.column %in% colnames(anno.table)) {
				filtered <- setdiff(which(!is.na(anno.table[, snp.overlap.column])), sites2ignore)
			} else {
				stop("no SNP annotation")
			}
		}
	}
	filtered
}

########################################################################################################################

#' rnb.section.snp.removal
#'
#' Adds a section on removing SNP-overlapping sites or probes to the specified report.
#'
#' @param report  Report to contain the new section. This must be an object of type \code{\linkS4class{Report}}.
#' @param rnb.set Methylation dataset before filtering as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param stats   Statistics on SNP-overlapping probe filtering, as returned by
#'                \code{\link{rnb.execute.snp.removal}}. See the documentation of the function for more details.
#' @return The possibly modified report.
#'
#' @seealso \code{\link{rnb.execute.snp.removal}}, \code{\link{rnb.step.snp.removal}}, \code{\link{rnb.run.filtering}}
#'
#' @author Yassen Assenov
#' @noRd
rnb.section.snp.removal <- function(report, rnb.set, stats) {
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	validate.stats(stats)
	anno.table <- annotation(stats$dataset.before, add.names = inherits(stats$dataset, "RnBeadSet"))
	filtered <- stats$filtered
	if (!(is.integer(filtered) && anyDuplicated(filtered) == 0 && 1 <= min(filtered) &&
		  	max(filtered) <= nrow(anno.table))) {
		stop("invalid value for stats$filtered")
	}
	snp <- stats$snp
	if ((is.double(snp) || is.integer(snp)) && length(snp) == 1) {
		snp <- as.character(snp)
	}
	accepted <- .rnb.options[["accepted"]][["filtering.snp"]]
	if (!(is.character(snp) && length(snp) == 1 && isTRUE(snp %in% accepted))) {
		msg <- paste0('"', accepted, '"', collapse = ", ")
		stop(paste("invalid value for stats$snp; expected one of", msg))
	}
	rnb.section.snp.removal.internal(report, class(stats$dataset), filtered, anno.table, snp)
}

rnb.section.snp.removal.internal <- function(report, dataset.class, filtered, anno.table, snp) {
	txt.site <- rnb.get.row.token(dataset.class)
	txt.sites <- rnb.get.row.token(dataset.class, plural = TRUE)
	txt.title <- paste("Removal of SNP-enriched", capitalize(txt.sites))
	if (snp == "no") {
		return(report)
	}
	if (is.null(filtered)) {
		txt <- "SNP-based filtering was not performed because the site annotation does not include SNP information."
		report <- rnb.add.section(report, txt.title, txt)
		return(report)
	}
	N <- length(filtered)

	if (dataset.class == "RnBeadSet") {
		if (N == 0) {
			txt <- "No probes were found for which"
		} else if (N == 1) {
			txt <- "<b>One</b> probe was removed because"
		} else {
			txt <- paste0("<b>", N, "</b> probes were removed because")
		}
		if (snp %in% c("3", "5")) {
			txt <- paste(txt, "the last", snp, "bases of")
		}
		txt <- paste(txt, "their sequences overlap with SNPs.")
	} else { # dataset.class == "RnBiseqSet" ## FIXME: This passes for illumina platforms
		if (N == 0) {
			txt <- "No sites were found that overlap"
		} else if (N == 1) {
			txt <- "<b>One</b> site was removed because it overlaps"
		} else {
			txt <- paste0("<b>", N, "</b> sites were removed because they overlap")
		}
		txt <- paste(txt, "with SNPs.")
	}

	if (N != 0) {
		snp.overlap.column <- paste("SNPs", ifelse(snp %in% c("3", "5"), snp, "Full"))
		is.epicv2 = FALSE
		if ("SNPs 3 Alternative" %in% colnames(anno.table)) { ## To detect EPICv2: only EPICv2 anno has this column
			snp.overlap.column <- paste(snp.overlap.column, "Alternative")
			is.epicv2 = TRUE
		}
		p.columns <- c("ID", "Chromosome", "Start", "End", snp.overlap.column)
		names(p.columns) <- c(p.columns[1:(length(p.columns) - 1)], "SNPs")
		fname <- "removed_sites_snp.csv"
		fname <- rnb.save.removed.sites(anno.table[filtered, ], report, fname, p.columns, names(p.columns))
		txt <- paste(txt, "The", ifelse(N == 1, paste("removed", txt.site), paste("list of removed", txt.sites)),
					 ' is available in a <a href="', fname, '">dedicated table</a> accompanying this report.')
		if (is.epicv2) {
			txt <- paste(txt, "For this step, dbSNP data from the Illumina manifest file is utilized.")
		}
	}
	report <- rnb.add.section(report, txt.title, txt)
	return(report)
}

########################################################################################################################

#' rnb.step.snp.removal
#'
#' Performs the procedure for removal of sites or probes (if any) that overlap with many SNPs to filter the given
#' methylation dataset and adds a corresponding section to the specified report.
#'
#' @param rnb.set Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param report  Report to summarize the outcome of this procedure. This must be an object of type
#'                \code{\linkS4class{Report}}.
#' @return List of two elements:
#'         \describe{
#'           \item{\code{"dataset"}}{The (possibly modified) dataset after removing sites that overlap with SNPs.}
#'           \item{\code{"report"}}{The modified report.}
#'         }
#'
#' @seealso \code{\link{rnb.execute.snp.removal}}, \code{\link{rnb.section.snp.removal}}, \code{\link{rnb.run.filtering}}
#'
#' @author Yassen Assenov
#' @noRd
rnb.step.snp.removal <- function(rnb.set, report) {
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	if (rnb.getOption("logging") && logger.isinitialized() == FALSE) {
		logger.start(fname = NA) # initialize console logger
	}
	result <- rnb.step.snp.removal.internal(class(rnb.set), integer(), report,
		annotation(rnb.set, add.names = TRUE))
	return(list(dataset = remove.sites(rnb.set, result$filtered), report = result$report))
}

rnb.step.snp.removal.internal <- function(dataset.class, sites2ignore, report, anno.table) {
	logger.start("Removal of SNP-enriched Sites")
	snp <- rnb.getOption("filtering.snp")
	is.infinium <- !(dataset.class %in% c("RnBiseqSet", "RnBSet"))
	filtered <- tryCatch(rnb.execute.snp.removal.internal(sites2ignore, snp, is.infinium, anno.table),
			error = function(x) { NULL })
	if (is.null(filtered)) {
		filtered <- integer()
		report <- rnb.section.snp.removal.internal(report, dataset.class, NULL, anno.table, snp)
	} else {
		msg <- paste("Removed", length(filtered), ifelse(length(filtered) == 1, "site", "sites"))
		logger.status(paste0(msg, ' using SNP criterion "', snp, '"'))
		report <- rnb.section.snp.removal.internal(report, dataset.class, filtered, anno.table, snp)
	}
	logger.status("Added a corresponding section to the report")
	logger.completed()
	list(report = report, filtered = filtered)
}

########################################################################################################################

#' rnb.execute.sex.removal
#'
#' Removes all sites in sex chromosomes from the given dataset.
#'
#' @param rnb.set Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @return List of three elements:
#'         \describe{
#'           \item{\code{"dataset.before"}}{Copy of \code{rnb.set}.}
#'           \item{\code{"dataset"}}{The (possibly) modified dataset after retaining sites on autosomes only.}
#'           \item{\code{"filtered"}}{\code{integer} vector storing the indices (in beta matrix of the unfiltered
#'                dataset) of all removed probes.}
#'         }
#'
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' rnb.set.filtered <- rnb.execute.sex.removal(rnb.set.example)$dataset
#' identical(meth(rnb.set.example), meth(rnb.set.filtered)) # FALSE
#' }
#' @author Yassen Assenov
#' @export
rnb.execute.sex.removal <- function(rnb.set) {
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	filtered <- rnb.execute.sex.removal.internal(integer(),
		annotation(rnb.set, add.names = inherits(rnb.set, "RnBeadSet")))

	if (length(filtered) != 0) {
		dataset <- remove.sites(rnb.set, filtered)
	} else {
		dataset <- rnb.set
	}
	return(list(dataset.before = rnb.set, dataset = dataset, filtered = filtered))
}

rnb.execute.sex.removal.internal <- function(sites2ignore, anno.table) {
	setdiff(which(anno.table[, "Chromosome"] %in% c("chrX", "chrY")), sites2ignore)
}

########################################################################################################################

#' rnb.section.sex.removal
#'
#' Adds a section on removing sex chromosome sites to the specified report.
#'
#' @param report Report to contain the new section. This must be an object of type \code{\linkS4class{Report}}.
#' @param stats  Statistics on sex chromosome site filtering, as returned by \code{\link{rnb.execute.sex.removal}}.
#'               See the documentation of the function for more details.
#' @return The modified report.
#'
#' @seealso \code{\link{rnb.execute.sex.removal}}, \code{\link{rnb.step.sex.removal}}, \code{\link{rnb.run.filtering}}
#'
#' @author Yassen Assenov
#' @noRd
rnb.section.sex.removal <- function(report, stats) {
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	validate.stats(stats)
	rnb.section.sex.removal.internal(report, class(stats$dataset), stats$filtered,
		annotation(stats$dataset.before, add.names = inherits(stats$dataset, "RnBeadSet")))
}

rnb.section.sex.removal.internal <- function(report, dataset.class, filtered, anno.table) {
	txt.site <- rnb.get.row.token(dataset.class)
	txt.sites <- rnb.get.row.token(dataset.class, plural = TRUE)
	pcount <- length(filtered)
	if (pcount == 0) {
		txt <- c("No ", txt.sites, " located on sex chromosomes were found.")
	} else {
		fname <- "removed_sites_sex.csv"
		fname <- rnb.save.removed.sites(anno.table[filtered, ], report, fname)
		txt <- c(pcount, " ", ifelse(pcount == 1, txt.site, txt.sites), " on sex chromosomes ",
			ifelse(pcount == 1, "was", "were"), " removed at this step. The ",
			ifelse(pcount == 1, paste("removed", txt.site), paste("list of removed", txt.sites)),
			" is available in a <a href=\"", fname, "\">dedicated table</a> accompanying this report.")
	}
	report <- rnb.add.section(report, paste("Removal of", capitalize(txt.sites), "on Sex Chromosomes"), txt)
	return(report)
}

########################################################################################################################

#' rnb.step.sex.removal
#'
#' Performs the procedure for removal of sites or probes on sex chromosomes (if any) to filter the given methylation
#' dataset and adds a corresponding section to the specified report.
#'
#' @param rnb.set Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param report  Report to summarize the outcome of this procedure. This must be an object of type
#'                \code{\linkS4class{Report}}.
#' @return List of two elements:
#'         \describe{
#'           \item{\code{"dataset"}}{The (possibly modified) dataset after removing sites on sex chromosomes.}
#'           \item{\code{"report"}}{The modified report.}
#'         }
#'
#' @seealso \code{\link{rnb.execute.sex.removal}}, \code{\link{rnb.section.sex.removal}}, \code{\link{rnb.run.filtering}}
#'
#' @author Yassen Assenov
#' @noRd
rnb.step.sex.removal <- function(rnb.set, report) {
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	if (rnb.getOption("logging") && logger.isinitialized() == FALSE) {
		logger.start(fname = NA) # initialize console logger
	}
	result <- rnb.step.sex.removal.internal(class(rnb.set), integer(), report,
		annotation(rnb.set, add.names = inherits(rnb.set, "RnBeadSet")))
	return(list(dataset = remove.sites(rnb.set, result$filtered), report = result$report))
}

rnb.step.sex.removal.internal <- function(dataset.class, sites2ignore, report, anno.table) {
	logger.start("Removal of Sites on Sex Chromosomes")
	filtered <- rnb.execute.sex.removal.internal(sites2ignore, anno.table)
	logger.status(c("Removed", length(filtered), "site(s) on sex chromosomes"))
	report <- rnb.section.sex.removal.internal(report, dataset.class, filtered, anno.table)
	logger.status("Added a corresponding section to the report")
	logger.completed()
	list(report = report, filtered = filtered)
}

########################################################################################################################
########################################################################################################################

#' rnb.execute.low.coverage.masking
#'
#' Replaces all low coverage sites by \code{NA}.
#'
#' @param rnb.set        Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param covg.threshold Threshold for minimal acceptable coverage, given as a non-negative \code{integer} value. All
#'                       methylation measurements with lower coverage than this threshold are set to \code{NA}. If this
#'                       parameter is \code{0}, calling this method has no effect.
#' @return List of three elements:
#'         \describe{
#'           \item{\code{"dataset.before"}}{Copy of \code{rnb.set}.}
#'           \item{\code{"dataset"}}{The (possibly) modified dataset after retaining sites on autosomes only.}
#'           \item{\code{"mask"}}{A logical matrix of dimension \code{meth(rnb.set,type="sites")} indicating which
#' 				   methylation values have been masked}
#'         }
#'
#' @author Fabian Mueller
#' @export
rnb.execute.low.coverage.masking <- function(rnb.set, covg.threshold = rnb.getOption("filtering.coverage.threshold")) {
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	if (is.double(covg.threshold) && all(covg.threshold == as.integer(covg.threshold))) {
		covg.threshold <- as.integer(covg.threshold)
	}
	if (!(is.integer(covg.threshold) && length(covg.threshold) == 1 && isTRUE(0 <= covg.threshold))) {
		stop("invalid value for covg.threshold")
	}
	mask <- rnb.execute.low.coverage.masking.internal(rnb.set, integer(), covg.threshold)
	dataset <- rnb.set
	if (any(mask)) {
		dataset@meth.sites[,][mask] <- NA
		if (inherits(dataset, "RnBeadRawSet")) {
			dataset@M[,][mask] <- NA
			dataset@U[,][mask] <- NA
			dataset@M0[,][mask] <- NA
			dataset@U0[,][mask] <- NA
		}
		dataset <- updateRegionSummaries(dataset)
	}
	list(dataset.before = rnb.set, dataset = dataset, mask = mask)
}

rnb.execute.low.coverage.masking.internal <- function(rnb.set, sites2ignore, covg.threshold) {
	# coverage.matrix <- covg(rnb.set)
	if (!hasCovg(rnb.set)) {
		return(NULL)
	}
	# mask <- (coverage.matrix < covg.threshold) & (!is.na(coverage.matrix)) & (!is.na(meth(rnb.set)))
	N <- nsites(rnb.set)
	M <- length(samples(rnb.set))
	mask <- matrix(NA, nrow=N, ncol=M)
	colnames(mask) <- samples(rnb.set)
	for (j in 1:M){
		cm <- covg(rnb.set, j=j)[,1]
		mask[,j] <- (cm < covg.threshold) & !is.na(cm) & !is.na(meth(rnb.set, j=j)[,1])
		if (length(sites2ignore) != 0) {
			mask[sites2ignore, j] <- FALSE
		}
	}
	return(mask)
}


########################################################################################################################

#' rnb.section.low.coverage.masking
#'
#' Adds a section on masking low coverage sites with NAs to the specified report.
#'
#' @param report Report to contain the new section. This must be an object of type \code{\linkS4class{Report}}.
#' @param stats a list as outputted from \code{rnb.execute.low.coverage.masking}
#' @return The modified report.
#'
#' @seealso \code{\link{rnb.execute.low.coverage.masking}}, \code{\link{rnb.run.filtering}}
#'
#' @author Fabian Mueller
#' @noRd
rnb.section.low.coverage.masking <- function(report, stats,
	covg.threshold = rnb.getOption("filtering.coverage.threshold")) {
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	## TODO: Validate stats
	if (is.double(covg.threshold) && all(covg.threshold == as.integer(covg.threshold))) {
		covg.threshold <- as.integer(covg.threshold)
	}
	if (!(is.integer(covg.threshold) && length(covg.threshold) == 1 && (!is.na(covg.threshold)) &&
		  0 <= covg.threshold)) {
		stop("invalid value for covg.threshold")
	}
	rnb.section.low.coverage.masking.internal(report, class(stats$dataset), stats$mask,
		annotation(stats$dataset.before, add.names = inherits(stats$dataset, "RnBeadSet")), covg.threshold)
}

rnb.section.low.coverage.masking.internal <- function(report, dataset.class, mask, anno.table, covg.threshold) {
	txt.site <- rnb.get.row.token(dataset.class, )
	txt.sites <- rnb.get.row.token(dataset.class, plural = TRUE)
	num.masked <- colSums(mask)
	num.masked.total <- sum(num.masked)
	if (num.masked.total < 1) {
		txt <- c("No ", txt.sites, " were masked")
	} else {
		masked.table <- data.frame(Sample = names(num.masked), masked = num.masked, check.names = FALSE)
		colnames(masked.table)[2] <- paste("Masked", txt.sites)
		fname <- "masked_sites_coverage.csv"
		fname.full <- file.path(rnb.get.directory(report, "data", absolute = TRUE), fname)
		utils::write.csv(masked.table, file = fname.full, row.names = FALSE)
		rnb.status(c("Saved numbers of masked sites per sample to", fname.full))
		fname.relative <- paste(rnb.get.directory(report, "data"), fname, sep = "/")
		txt <- c("A total of ",num.masked.total," ",txt.sites, " with coverage less than ",covg.threshold," were masked by NA in the methylation table")
		txt <- c(txt, paste(" The numbers of masked",txt.sites,"per sample"))
		txt <- c(txt, " are available in a <a href=\"", fname.relative, "\">dedicated table</a> accompanying this report.")
	}
	report <- rnb.add.section(report, paste("Masking of", capitalize(txt.sites), "with Low Coverage"), txt)
	return(report)
}

########################################################################################################################

#' rnb.step.low.coverage.masking
#'
#' Sets methylation values of sites with low coverage to NA
#'
#' @param rnb.set Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param report  Report to summarize the outcome of this procedure. This must be an object of type
#'                \code{\linkS4class{Report}}.
#' @return List of up to three elements:
#'         \describe{
#'           \item{\code{"dataset"}}{The (possibly modified) dataset after replacing low coverage sites}
#'           \item{\code{"report"}}{The modified report.}
#'         }
#'
#' @seealso \code{\link{rnb.run.filtering}},  \code{\link{rnb.execute.low.coverage.masking}},  \code{\link{rnb.section.low.coverage.masking}}
#'
#' @author Fabian Mueller
#' @noRd
rnb.step.low.coverage.masking <- function(rnb.set, report, covg.threshold = rnb.getOption("filtering.coverage.threshold")) {
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	if (is.double(covg.threshold) && all(covg.threshold == as.integer(covg.threshold))) {
		covg.threshold <- as.integer(covg.threshold)
	}
	if (!(is.integer(covg.threshold) && length(covg.threshold) == 1 && (!is.na(covg.threshold)) &&
		  	0 <= covg.threshold)) {
		stop("invalid value for covg.threshold")
	}
	if (rnb.getOption("logging") && logger.isinitialized() == FALSE) {
		logger.start(fname = NA) # initialize console logger
	}
	rnb.step.low.coverage.masking.internal(rnb.set, integer(), report,
		annotation(rnb.set, add.names = inherits(rnb.set, "RnBeadSet")), covg.threshold)
}

rnb.step.low.coverage.masking.internal <- function(rnb.set, sites2ignore, report, anno.table, covg.threshold) {
	logger.start("Replacing Low Coverage Sites by NA")
	mask <- rnb.execute.low.coverage.masking.internal(rnb.set, sites2ignore, covg.threshold)
	logger.status(c("Masked ", sum(colSums(mask)), " site(s) based on coverage threshold ", covg.threshold))
	report <- rnb.section.low.coverage.masking.internal(report, class(rnb.set), mask, anno.table, covg.threshold)
	logger.status("Added a corresponding section to the report")
	logger.completed()
	list(report = report, mask = mask)
}

########################################################################################################################

#' rnb.execute.high.coverage.removal
#'
#' Removes methylation sites with a coverage larger than 100 times the 95-percentile of coverage in each sample.
#'
#' @param rnb.set Methylation dataset as an object of type inheriting \code{\linkS4class{RnBiseqSet}}.
#' @return \code{list} of two elements:
#'         \describe{
#'           \item{\code{"dataset"}}{The (possibly) modified dataset after retaining sites on autosomes only.}
#'           \item{\code{"filtered"}}{\code{integer} vector storing the indices of all removed sites.}
#'         }
#'
#' @author Fabian Mueller
#' @export
rnb.execute.high.coverage.removal <- function(rnb.set) {
	if (!inherits(rnb.set, "RnBiseqSet")) {
		stop("invalid value for rnb.set")
	}
	filtered <- rnb.execute.high.coverage.removal.internal(rnb.set, integer())
	if (is.null(filtered)) {
		stop("invalid value for rnb.set; no coverage information present")
	}
	if (length(filtered) != 0) {
		dataset <- remove.sites(rnb.set, filtered)
	} else {
		dataset <- rnb.set
	}
	return(list(dataset.before = rnb.set, dataset = dataset, filtered = filtered))
}

rnb.execute.high.coverage.removal.internal <- function(rnb.set, sites2ignore) {
	if (!hasCovg(rnb.set)) {
		return(NULL)
	}
	filtered <- rep(FALSE, nsites(rnb.set))
	for (j in 1:length(samples(rnb.set))) {
		cv <- covg(rnb.set, j=j)[,1] # get coverage VECTOR for current sample
		cv[cv<1] <- NA
		cvq <- cv
		if (length(sites2ignore) != 0) {
			cvq <- cv[-sites2ignore]
		}
		if (sum(!is.na(cv)) != 0) {
			qqs <- ceiling(quantile(cvq, probs = HIGH.COVER.OUTLIER.QUANTILE, na.rm = TRUE)) * HIGH.COVER.OUTLIER.FACTOR
			filtered <- filtered | cv > qqs
		}
	}
	filtered <- which(filtered)
	setdiff(filtered, sites2ignore)
}

########################################################################################################################

#' rnb.section.high.coverage.removal
#'
#' Adds a section on masking low coverage sites with NAs to the specified report.
#'
#' @param report Report to contain the new section. This must be an object of type \code{\linkS4class{Report}}.
#' @param masking.result a list as outputted from \code{rnb.execute.high.coverage.removal}
#' @return The modified report.
#'
#' @seealso \code{\link{rnb.execute.high.coverage.removal}}, \code{\link{rnb.run.filtering}}
#'
#' @author Fabian Mueller
#' @noRd
rnb.section.high.coverage.removal <- function(report, stats) {
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	validate.stats(stats)
	rnb.section.high.coverage.removal.internal(report, class(stats$dataset), stats$filtered, annotation(stats$dataset.before))
}

rnb.section.high.coverage.removal.internal <- function(report, dataset.class, filtered, anno.table) {
	txt.site <- rnb.get.row.token(dataset.class)
	txt.sites <- rnb.get.row.token(dataset.class, plural = TRUE)
	if (is.null(filtered)) {
		txt <- c("No high coverage outlier ", txt.sites, " were found.")
	} else {
		fname <- "removed_sites_high_coverage.csv"
		fname <- rnb.save.removed.sites(anno.table[filtered, ], report, fname)

		txt <- length(filtered)
		txt <- c(ifelse(txt == 1, paste("One", txt.site, "was"), paste(txt, txt.sites, "were")), " detected as high coverage ",
			"outlier in at least one sample and removed at this step. An outlier site is defined as one whose ",
			"coverage exceeds ", HIGH.COVER.OUTLIER.FACTOR, " times the ", HIGH.COVER.OUTLIER.QUANTILE, "-quantile ",
			"of coverage values in its sample. The list of removed ", txt.sites, " is available in a <a href=\"", fname,
			"\">dedicated table</a> accompanying this report.")
	}

	report <- rnb.add.section(report, "Removal of High Coverage Outlier Sites", txt)
	return(report)
}

########################################################################################################################

#' rnb.step.high.coverage.removal
#'
#' Removes methylation sites with a coverage larger than 100 times the 95-percentile of coverage in each sample.
#'
#' @param rnb.set Methylation dataset as an object of type inheriting \code{\linkS4class{RnBiseqSet}}.
#' @param report  Report to summarize the outcome of this procedure. This must be an object of type
#'                \code{\linkS4class{Report}}.
#' @return List of up to three elements:
#'         \describe{
#'           \item{\code{"dataset"}}{The (possibly modified) dataset after removing high coverage outlier sites}
#'           \item{\code{"report"}}{The modified report.}
#'         }
#'
#' @seealso \code{\link{rnb.execute.high.coverage.removal}}, \code{\link{rnb.section.high.coverage.removal}}
#'
#' @author Fabian Mueller
#' @noRd
rnb.step.high.coverage.removal <- function(rnb.set, report) {
	if (!inherits(rnb.set, "RnBiseqSet")) {
		stop("invalid value for rnb.set")
	}
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	if (rnb.getOption("logging") && logger.isinitialized() == FALSE) {
		logger.start(fname = NA) # initialize console logger
	}
	result <- rnb.step.high.coverage.removal.internal(rnb.set, integer(), report, annotation(rnb.set))
	return(list(dataset = remove.sites(rnb.set, result$filtered), report = result$report))
}

rnb.step.high.coverage.removal.internal <- function(rnb.set, sites2ignore, report, anno.table) {
	logger.start("Removal of High Coverage (Outlier) Sites")
	filtered <- rnb.execute.high.coverage.removal.internal(rnb.set, sites2ignore)
	if (is.null(filtered)) {
		filtered <- integer()
		report <- rnb.section.high.coverage.removal.internal(report, class(rnb.set), NULL, anno.table)
		logger.warning("No coverage information present")
	} else {
		logger.status(c("Removed", length(filtered), "high coverage outlier sites"))
		report <- rnb.section.high.coverage.removal.internal(report, class(rnb.set), filtered, anno.table)
		logger.status("Added a corresponding section to the report")
	}
	logger.completed()
	list(report = report, filtered = filtered)
}

########################################################################################################################

#' rnb.execute.na.removal
#'
#' Removes all probes with missing value (if such exists) from the given dataset.
#'
#' @param rnb.set   Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param threshold Maximum quantile of \code{NA}s allowed per site. This must be a value between 0 and 1.
#' @return List of four or five elements:
#'         \describe{
#'           \item{\code{"dataset.before"}}{Copy of \code{rnb.set}.}
#'           \item{\code{"dataset"}}{The (possibly modified) dataset after performing the missing value removal.}
#'           \item{\code{"filtered"}}{\code{integer} vector storing the indices (in beta matrix of the unfiltered
#'                dataset) of all removed sites.}
#' 			 \item{\code{"threshold"}}{Copy of \code{threshold}.}
#' 			 \item{\code{"naCounts"}}{Vector storing the number of NAs per site}
#'         }
#'
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' rnb.set.filtered <- rnb.execute.na.removal(rnb.set.example, 0)$dataset
#' identical(meth(rnb.set.example), meth(rnb.set.filtered)) # TRUE
#' }
#' @author Yassen Assenov
#' @export
rnb.execute.na.removal <- function(rnb.set, threshold = rnb.getOption("filtering.missing.value.quantile")) {
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	if (!(is.double(threshold) && length(threshold) == 1 && (!is.na(threshold)))) {
		stop("invalid value for threshold")
	}
	if (!(0 <= threshold && threshold <= 1)) {
		stop("invalid value for threshold; expected a value between 0 and 1")
	}
	filterRes <- rnb.execute.na.removal.internal(rnb.set, NULL, threshold)
	list(dataset.before = rnb.set, dataset = remove.sites(rnb.set, filterRes$filtered), filtered = filterRes$filtered,
		threshold = threshold, naCounts=filterRes$naCounts)
}

rnb.execute.na.removal.internal <- function(rnb.set, sites2ignore, threshold, mask=NULL) {
	naCounts <- getNumNaMeth(rnb.set, mask=mask)
	filtered <- which((naCounts/length(samples(rnb.set))) > threshold)
	if (length(sites2ignore) > 0){
		filtered <- setdiff(filtered, sites2ignore)
	}
	res <- list(naCounts=naCounts, filtered=filtered)
	return(res)
}

########################################################################################################################

#' rnb.section.na.removal
#'
#' Adds a section on removing sites or probes with missing values to the specified report.
#'
#' @param report Report to summarize the outcome of the procedure. This must be an object of type
#'               \code{\linkS4class{Report}}.
#' @param stats  Statistics on filtering based on missing values, as returned by \code{\link{rnb.execute.na.removal}}.
#'               See the documentation of the function for more details.
#' @return The modified report.
#'
#' @author Yassen Assenov
#' @noRd
rnb.section.na.removal <- function(report, stats) {
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	validate.stats(stats)
	if (!(is.double(stats$threshold) && length(stats$threshold) == 1 && (!is.na(stats$threshold)))) {
		stop("invalid value for stats$threshold")
	}
	if (!(0 <= stats$threshold && stats$threshold <= 1)) {
		stop("invalid value for stats$threshold; expected a value between 0 and 1")
	}
	rnb.section.na.removal.internal(report, class(stats$dataset), length(samples(stats$dataset)), stats$naCounts,
		stats$filtered, stats$threshold, annotation(stats$dataset.before, add.names = TRUE))
}

rnb.section.na.removal.internal <- function(report, dataset.class, numSamples, naCounts, filtered, threshold, anno.table) {
	txt.site <- rnb.get.row.token(dataset.class)
	txt.sites <- rnb.get.row.token(dataset.class, plural = TRUE)
	threshold.abs <- as.integer(floor(threshold * numSamples))
	txt.title <- paste("Removal of", capitalize(txt.sites), "with (Many) Missing Values")
	na.counts <- as.integer(naCounts)
	removed.count <- length(filtered)
	if (removed.count == 0) {
		txt <- "No sites with too many missing values were found in the methylation table."
		report <- rnb.add.section(report, txt.title, txt)
		return(report)
	}

	## Create a table of sites that contain missing values
	na.c <- na.counts[filtered]
	fname <- "removed_sites_na.csv"
	fname <- rnb.save.removed.sites(anno.table[filtered, ], report, fname, `Number of Missing Values` = na.c)
	txt <- c(removed.count, " ", ifelse(removed.count == 1, paste(txt.site, "was"), paste(txt.sites, "were")),
		" removed because ", ifelse(removed.count == 1, "it contains", "they contain"), " more than ", threshold.abs,
		" missing values in the methylation table. This threshold corresponds to ", round(threshold * 100, 1),
		"% of all samples. The total number of missing values in the methylation table before this filtering step was ",
		sum(na.counts), ". A <a href=\"", fname, "\">dedicated table of all removed ", txt.sites,
		"</a> is attached to this report.")
	report <- rnb.add.section(report, txt.title, txt)
	rm(na.c, fname, txt)

	if (removed.count != 1) {
		## Create a histogram of number of NAs per site
		values <- list("all" = list(values = na.counts))
		if (sum(na.counts == 0) > 1/3 * length(na.counts)) {
			values[["positive"]] <- list(values = na.counts[filtered])
		}
		for (i in names(values)) {
			values[[i]][["fname"]] <- paste("histogram_na_counts", i, sep = "_")
		}
		report.plots <- lapply(values, function(x) {
				dframe <- data.frame(x = x$values)
				binwidth <- range(x$values)
				binwidth <- max((binwidth[2] - binwidth[1]) / 40, 1)
				rplot <- createReportPlot(x$fname, report, width = 5, height = 5)
				pp <- ggplot(dframe, aes_string(x = "x")) + labs(x = "Number of missing values", y = "Frequency") +
					geom_histogram(aes(y = after_stat(count)), binwidth = binwidth)
				if (0 < threshold && threshold < 1) {
					pp <- pp + geom_vline(xintercept = threshold * numSamples, linetype = "dotted")
				}
				print(pp)
				return(off(rplot))
			})
		setting.names <- list()
		if (length(report.plots) > 1) {
			ename <- paste(capitalize(txt.sites), "to include")
			setting.names[[ename]] <- c("all" = "all", "positive" = paste(txt.sites, "with missing values"))
		}
		txt <- c("The figure below shows the distribution of missing values per ", txt.site, ".")
		rnb.add.paragraph(report, txt)
		txt <- c("Histogram of number of ", txt.sites, " that contain missing values.")
		if (0 < threshold && threshold < 1) {
			txt <- c(txt, " The vertical line, if visible, denotes the applied threshold.")
		}
		report <- rnb.add.figure(report, txt, report.plots, setting.names)
	}

	return(report)
}

########################################################################################################################

#' rnb.step.na.removal
#'
#' Performs the procedure for missing value removal (if applicable) to filter the given methylation dataset and adds a
#' corresponding section to the provided report.
#'
#' @param rnb.set   Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param report    Report to summarize the outcome of this procedure. This must be an object of type
#'                  \code{\linkS4class{Report}}.
#' @param threshold Maximum quantile of \code{NA}s allowed per site. This must be a value between 0 and 1.
#' @return List up to three elements:
#'         \describe{
#'           \item{\code{"dataset"}}{The (possibly modified) dataset after performing the missing value removal.}
#'           \item{\code{"report"}}{The modified report.}
#'           \item{\code{"betas"}}{\code{matrix} of the beta values for sites that contain missing values. These values
#'                were essentially removed from the dataset. This element is added to the list only when \code{rnb.set}
#'                contains such sites.}
#'         }
#'
#' @author Yassen Assenov
#' @noRd
rnb.step.na.removal <- function(rnb.set, report, threshold = 0) {
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	if (rnb.getOption("logging") && logger.isinitialized() == FALSE) {
		logger.start(fname = NA) # initialize console logger
	}
	if (!(is.double(threshold) && length(threshold) == 1 && (!is.na(threshold)))) {
		stop("invalid value for threshold")
	}
	if (!(0 <= threshold && threshold <= 1)) {
		stop("invalid value for threshold; expected a value between 0 and 1")
	}
	result <- rnb.step.na.removal.internal(rnb.set, sites2ignore, report, annotation(rnb.set, add.names = TRUE),
		threshold)
	return(list(dataset = remove.sites(rnb.set, result$filtered), report = result$report))
}

rnb.step.na.removal.internal <- function(rnb.set, sites2ignore, report, anno.table, threshold, mask=NULL) {
	dataset.class <- class(rnb.set)
	logger.start("Missing Value Removal")
	logger.status(c("Using a sample quantile threshold of", threshold))
	filterRes <- rnb.execute.na.removal.internal(rnb.set, sites2ignore, threshold, mask)
	logger.status(c("Removed", length(filterRes$filtered), "site(s) with too many missing values"))
	report <- rnb.section.na.removal.internal(report, dataset.class, length(samples(rnb.set)), filterRes$naCounts, filterRes$filtered, threshold, anno.table)
	logger.status("Added a corresponding section to the report")
	logger.completed()
	list(report = report, filtered = filterRes$filtered)
}

########################################################################################################################

#' rnb.execute.variability.removal
#'
#' Removes all sites or probes with low variability from the given dataset.
#'
#' @param rnb.set       Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param min.deviation Threshold for standard deviation per site. This must be a scalar between 0 and 1. All sites, for
#'                      which the standard deviation of methylation values (for all samples in \code{rnb.set}) is lower
#'                      than this threshold, will be filtered out.
#' @return List of four elements:
#'         \describe{
#'           \item{\code{"dataset.before"}}{Copy of \code{rnb.set}.}
#'           \item{\code{"dataset"}}{The (possibly modified) dataset after removing sites with low variability.}
#'           \item{\code{"filtered"}}{\code{integer} vector storing the indices (in beta matrix of the unfiltered
#'                dataset) of all removed sites.}
#' 			 \item{\code{"threshold"}}{The value of the given parameter \code{min.deviation}.}
#'         }
#'
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' rnb.set.filtered <- rnb.execute.variability.removal(rnb.set.example, 0.01)
#' }
#' @author Yassen Assenov
#' @export
rnb.execute.variability.removal <- function(rnb.set, min.deviation = rnb.getOption("filtering.deviation.threshold")) {
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	if (is.integer(min.deviation)) {
		ovalue <- as.double(min.deviation)
	}
	if (!(is.double(min.deviation) && length(min.deviation) == 1 && (!is.na(min.deviation)))) {
		stop("invalid value for min.deviation")
	}
	min.deviation <- min.deviation[1]
	if (min.deviation < 0 || min.deviation > 1) {
		stop("invalid value for min.deviation; expected a number between 0 and 1")
	}
	filtered <- which(apply(meth(rnb.set), 1, sd, na.rm = TRUE) < min.deviation)
	list(dataset.before = rnb.set, dataset = remove.sites(rnb.set, filtered), filtered = filtered,
		threshold = min.deviation)
}

rnb.execute.variability.removal.internal <- function(mm, sites2ignore, threshold) {
	site.deviations <- apply(mm, 1, sd, na.rm = TRUE)
	filtered <- setdiff(which(site.deviations < threshold), sites2ignore)
	list(deviations = site.deviations, filtered = filtered)
}

########################################################################################################################

#' rnb.section.variability.removal
#'
#' Adds a section on removing low-variable probes to the specified report.
#'
#' @param report        Report to summarize the outcome of the procedure. This must be an object of type
#'                      \code{\linkS4class{Report}}.
#' @param dataset.class Type of the dataset object, e.g. \code{"RnBiseqSet"}.
#' @param deviations    Vector of type \code{double} storing the standard deviations of all sites.
#' @param filtered      Vector of type \code{integer} storing the indices of all removed sites.
#' @param threshold     Threshold for minimal deviation; sites with lower deviations should have been filtered out.
#' @param anno.table    Annotation information for all sites (before filtering) in the form of a \code{data.frame}.
#' @return The modified report.
#'
#' @author Yassen Assenov
#' @noRd
rnb.section.variability.removal <- function(report, dataset.class, deviations, filtered, threshold, anno.table) {

	txt.site <- rnb.get.row.token(dataset.class)
	txt.sites <- rnb.get.row.token(dataset.class, plural = TRUE)
	txt.title <- paste("Removal of Consistent", capitalize(txt.sites))
	removed.count <- length(filtered)
	if (removed.count == 0) {
		txt <- c("No ", txt.sites, " in the methylation table were found that exhibit standard deviation lower than ",
			threshold, ".")
		report <- rnb.add.section(report, txt.title, txt)
		return(report)
	}

	## Create a table of probes that have low methylation variability
	fname <- "removed_sites_variability.csv"
	fname <- rnb.save.removed.sites(anno.table[filtered, ], report, fname,
		`Standard Deviation` = deviations[filtered])
	plural <- (removed.count != 1)
	txt <- c(removed.count, ifelse(plural, paste(txt.sites, " were"), paste(txt.site, " was")), " removed because ",
		ifelse(plural, "their", "its"), " beta values exhibit standard deviation lower than ", threshold,
		". A <a href=\"", fname, "\">dedicated table of all removed ", txt.sites, "</a> is attached to this report.")
	report <- rnb.add.section(report, txt.title, txt)
	rm(fname, plural, txt)

	## Create a density estimation plot
	dframe <- data.frame(x = deviations)
	xlim <- range(c(range(deviations, na.rm = TRUE), threshold))
	rplot <- createReportPlot("site_deviations", report, width = 6, height = 5)
	pp <- ggplot(dframe, aes_string(x = "x")) + coord_cartesian(xlim = xlim) +
		labs(x = "Standard deviation", y = "Density") + geom_vline(xintercept = threshold, color = "#FF0000") +
		geom_density(kernel = "gaussian", color = "#000080") + theme(plot.margin = unit(0.1 + c(0, 0, 0, 0), "in"))
	print(pp)
	rplot <- off(rplot)
	txt <- c("Density plot of observed standard deviations of beta values per ", txt.site, ". The red vertical line ",
		"shows the applied threshold for ", txt.site, " variability.")
	report <- rnb.add.figure(report, txt, rplot)

	return(report)
}

########################################################################################################################

#' rnb.step.variability.removal
#'
#' Performs the procedure for removal of probes with low variability (if applicable) to filter the given methylation
#' dataset and adds a corresponding section to the specified report.
#'
#' @param rnb.set Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param report  Report to summarize the outcome of this procedure. This must be an object of type
#'                \code{\linkS4class{Report}}.
#' @return List of up to three elements:
#'         \describe{
#'           \item{\code{"dataset"}}{The (possibly modified) dataset after removing probes with low variability in
#'                methylation.}
#'           \item{\code{"report"}}{The modified report.}
#'           \item{\code{"betas"}}{\code{matrix} of the beta values for sites that exhibit low variability. These values
#'                were essentially removed from the dataset. This element is added to the list only when \code{rnb.set}
#'                contains such sites.}
#'         }
#'
#' @author Yassen Assenov
#' @noRd
rnb.step.variability.removal <- function(rnb.set, report) {
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	if (rnb.getOption("logging") && logger.isinitialized() == FALSE) {
		logger.start(fname = NA) # initialize console logger
	}
	threshold <- rnb.getOption("filtering.deviation.threshold")
	rnb.step.variability.removal.internal(rnb.set, report,
		annotation(rnb.set, add.names = inherits(rnb.set, "RnBeadSet")))
}

rnb.step.variability.removal.internal <- function(dataset.class, mm, sites2ignore, report, anno.table, threshold) {
	logger.start("Site Removal Based on Standard Deviation")
	logger.info(c("Using standard deviation threshold of", threshold))
	result <- rnb.execute.variability.removal.internal(mm, sites2ignore, threshold)
	logger.status(c("Removed", length(result$filtered), "site(s) with variance lower than the threshold"))
	report <- rnb.section.variability.removal(report, dataset.class, result$deviations, result$filtered,
		threshold, anno.table)
	logger.status("Added a corresponding section to the report")
	list(report = report, filtered = result$filtered)
}

########################################################################################################################

#' rnb.execute.high.dpval.masking
#'
#' Replaces all low coverage sites by \code{NA}.
#'
#' @param rnb.set        Methylation dataset as an object of type inheriting \code{\linkS4class{RnBeadSet}}.
#' @param dpval.threshold Threshold for maximal acceptable detection p-value, given as a non-negative \code{numeric} value between 0 and 1. All
#'                       methylation measurements with detection p-value than this threshold are set to \code{NA}. If this
#'                       parameter is \code{0}, calling this method has no effect.
#' @return List of three elements:
#'         \describe{
#'           \item{\code{"dataset.before"}}{Copy of \code{rnb.set}.}
#'           \item{\code{"dataset"}}{The (possibly) modified dataset after retaining sites on autosomes only.}
#'           \item{\code{"mask"}}{A logical matrix of dimension \code{meth(rnb.set,type="sites")} indicating which
#' 				   methylation values have been masked}
#'         }
#'
#' @author Fabian Mueller
#' @export
rnb.execute.high.dpval.masking <- function(rnb.set, dpval.threshold = 0.05) {
    if (!inherits(rnb.set, "RnBSet")) {
        stop("invalid value for rnb.set")
    }
    if (!(is.numeric(dpval.threshold) && length(dpval.threshold) == 1 && isTRUE(0 <= dpval.threshold | dpval.threshold > 1.0))) {
        stop("invalid value for dpval.threshold")
    }
    mask <- rnb.execute.high.dpval.masking.internal(rnb.set, integer(), dpval.threshold)
    dataset <- rnb.set
    if (any(mask)) {
        dataset@meth.sites[,][mask] <- NA
        if (inherits(dataset, "RnBeadRawSet")) {
            dataset@M[,][mask] <- NA
            dataset@U[,][mask] <- NA
            dataset@M0[,][mask] <- NA
            dataset@U0[,][mask] <- NA
        }
        dataset <- updateRegionSummaries(dataset)
    }
    list(dataset.before = rnb.set, dataset = dataset, mask = mask)
}

rnb.execute.high.dpval.masking.internal <- function(rnb.set, sites2ignore, dpval.threshold) {
    # coverage.matrix <- covg(rnb.set)
    if (is.null(rnb.set@pval.sites)) {
        return(NULL)
    }
    # mask <- (coverage.matrix < covg.threshold) & (!is.na(coverage.matrix)) & (!is.na(meth(rnb.set)))
    N <- nsites(rnb.set)
    M <- length(samples(rnb.set))
    mask <- matrix(NA, nrow=N, ncol=M)
    colnames(mask) <- samples(rnb.set)
    for (j in 1:M){
        dpm <- dpval(rnb.set, j=j)[,1]
        mask[,j] <- (dpm > dpval.threshold) & !is.na(dpm) & !is.na(meth(rnb.set, j=j)[,1])
        if (length(sites2ignore) != 0) {
            mask[sites2ignore, j] <- FALSE
        }
    }
    return(mask)
}

## E N D ###############################################################################################################

########################################################################################################################
## immune.R
## created: 2017-11-20
## creator: Yassen Assenov
## ---------------------------------------------------------------------------------------------------------------------
## Functions related to the estimation of immune cell infiltration.
########################################################################################################################

## F U N C T I O N S ###################################################################################################

#' Leukocytes unmethylation for purity
#'
#' Implementation of the LUMP (Leukocytes UnMethylation for Purity) algorithm for purity estimation on methylation
#' datasets.
#'
#' @param dataset Methylation dataset to study, provided as an object of type inheriting \code{RnBSet}.
#' @return Purity esimates provided as a \code{vector} of values in the range \code{[0, 1]}. The attribute
#'         \code{"sites"} contains the number of sites used in estimating the immune cell proportions. In case the
#'         dataset does not contain measurements for any of the sites on which LUMP focuses, the return values is
#'         \code{NULL}.
#'
#' @details The LUMP algorithm is developed by Dvir Aran, Marina Sirota and Atul J. Buttea.
#  It is described in their publication "Systematic pan-cancer analysis of tumour purity" in 2015.
#'
#' @author Yassen Assenov
#' @export
rnb.execute.lump <- function(dataset) {
	if (!inherits(dataset, "RnBSet")) {
		stop("Invalid value for dataset")
	}

	## Load the sites used by the LUMP algorithm
	load(system.file("data/lump.RData", package = "RnBeads"))
	x <- get0(paste0("lump.", dataset@assembly), inherits = FALSE)
	if (is.null(x)) {
		stop("Unsupported assembly")
	}
	x <- x[[dataset@target]]
	if (is.null(x)) {
		stop("Unsupported platform")
	}

	## Identify the indices of the sites in the dataset
	i.sites <- rep(0L, nrow(x)) # indices in the methylation matrix of the selected sites
	for (i.chrom in unique(x[, 1])) {
		i.x <- which(x[, 1] == i.chrom)
		i.d <- which(dataset@sites[, 2] == i.chrom)
		for (i in i.x) {
			j <- which(dataset@sites[i.d, 3] == x[i, 2])
			if (length(j) == 1) { i.sites[i] <- i.d[j] }
		}
	}
	rm(i.chrom, i.x, i.d, i, j)
	i.sites <- i.sites[i.sites != 0]
	if (length(i.sites) == 0) {
		stop(NULL)
	}

	result <- apply(dataset@meth.sites[i.sites, , drop = FALSE], 2, mean, na.rm = TRUE)
	result <- 1 - pmin(result / 0.85, 1)
	attr(result, "sites") <- length(i.sites)
	result
}

########################################################################################################################

#' rnb.section.lump
#'
#' Creates a report section dedicated to immune cell content estimation.
#'
#' @param report         Report on covariate inference to contain the immune cell content section. This must be an
#'                       object of type \code{\linkS4class{Report}}.
#' @param immune.content Estimation of the immune cell content, as returned by \code{\link{rnb.execute.lump}}.
#' @param s.groups       Sample groups as defined by \code{\link{rnb.sample.groups}}
#' @return The modified report.
#'
#' @author Yassen Assenov
#' @noRd
rnb.section.lump <- function(report, immune.content,s.groups) {
	s.title <- "Immune Cell Content Estimation"
	txt <- NULL
	refText <- paste("Aran, D. et al. (2015) Systematic pan-cancer analysis of tumour purity.",
		"<i>Nature Communications</i> <b>6</b>:8971.")

	if (is.null(immune.content)) {
		report <- rnb.add.reference(report, refText)
		txt <- c("Immune cell content estimation was not performed because the dataset does not contain measurements ",
			"for any of the sites used in the LUMP algorithm ", rnb.get.reference(report, refText), ".")
		return(rnb.add.section(report, s.title, txt))
	} else if (is.character(immune.content)) {
		txt <- gsub("^Unsupported ", "", immune.content)
		txt <- c("Immune cell content estimation was not performed because the ", txt, " is not supported.")
		return(rnb.add.section(report, s.title, txt))
	}

	report <- rnb.add.reference(report, refText)
	iss <- length(immune.content) == 1
	txt <- c("Immune cell content estimation was performed using the LUMP algorithm ",
		rnb.get.reference(report, refText), ". ", ifelse(iss, "The estimate is a value", "Estimates are values"),
		" between 0 and 1 and ", ifelse(iss, "is", "are") , " based on ",
		ifelse(iss, "", "up to "), attr(immune.content, "sites"), " sites", ifelse(iss, "", " per sample"), ".")
	if (!iss) {
		txt <- c(txt, " Blood samples, for example, should have a LUMP value close to 1. The figure below shows ",
			"the distribution of immune cell content values.")
	}
	report <- rnb.add.section(report, s.title, txt)
	if (!iss) {
		pp <- ggplot(data.frame(x = immune.content), aes_string("x")) +
			geom_histogram(breaks = seq(0, 1, length.out = 51)) + labs(x = "LUMP Estimate", y = "Frequency") +
			scale_x_continuous(breaks = seq(0, 1, length.out = 11), limits = c(0, 1), expand = c(0, 0)) +
			scale_y_continuous(expand = c(0, 0)) + theme(plot.margin = grid::unit(0.1 + c(0, 0.1, 0, 0), "in"))
		rplot <- createReportPlot("histogram_lump", report, width = 6.2, height = 5.2)
		print(pp)
		rplot <- off(rplot)
		txt <- paste("Histogram of all", length(immune.content), "estimates obtained after runnin LUMP.")
		report <- rnb.add.figure(report, txt, rplot)
		txt <- "Plotting immune cell content in different sample groups"
		report <- rnb.add.section(report, "Immune Stratification Plot", txt, level = 2)
		report <- add.stratification.plot.immune(report,immune.content,s.groups)
		rnb.status(c("Added", "Immune Stratification Plot"))
	}

	return(report)
}

#######################################################################################
#' add.stratification.plot.immune
#'
#' This function creates a plot comparing estimated immune cell content 
#' in the different sample groups defined by \code{\link{rnb.sample.groups}}
#'
#' @param report	report object to be modified.
#' @param immune.contents		estimated immune cell contents for the samples.
#' @param sample.groups sample groups as defined by \code{\link{rnb.sample.groups}}
#'
#' @return		modified report object with the immune cell stratification stratification plot
#'
#' @author	Michael Scherer
#' @noRd

add.stratification.plot.immune <- function(report,immune.contents,sample.groups){
  report.plots <- list()
  nsamples <- length(immune.contents)
  report.plots <- foreach(c=1:length(sample.groups),.combine = c) %dopar%{
    sample.group <- sample.groups[[c]]
    cvalues <- rep(rnb.getOption("colors.category"),length.out=length(sample.group))
    cat <- rep(NA,nsamples)
    for(i in 1:length(sample.group)){
      cat[sample.group[[i]]] <- names(sample.group)[i]
    }
    trait <- gsub("[[:punct:]]","",names(sample.groups)[c])
    trait <- gsub("[^ -~]","",trait)
    trait <- gsub(" ","",trait)
    to.plot <- data.frame(Immune=immune.contents,Group=cat)
    plot <- ggplot(to.plot,aes(x=Group,y=Immune,fill=Group))+geom_boxplot()+scale_fill_manual(values=cvalues)+ylab("LUMP estimate")+
      theme(axis.text.x=element_text(angle=90,hjust=1))
    report.plot <- createReportPlot(paste("lump_stratification",trait,sep="_"),report)
    print(plot)
    report.plot <- off(report.plot)
    return(c(report.plots,report.plot))
  }
  s.groups <- gsub("[[:punct:]]","",names(sample.groups))
  s.groups <- gsub("[^ -~]","",s.groups)
  s.groups <- gsub(" ","",s.groups)
  names(s.groups) <- s.groups
  s.names <- list(Group=s.groups)
  descr <- "Immune Cell Content estimated by LUMP is stratified over different sample groups."
  report <- rnb.add.figure(report,descr,report.plots,s.names)
  return(report)
}

################################################################################
# RnBClusterRun
################################################################################

#' RnBClusterRun Class
#'
#' A class for configuring and running RnBeads on a scientific compute cluster.
#' 
#'
#' @section Slots:
#' \describe{
#'   \item{\code{architecture}}{A \code{\linkS4class{ClusterArchitecture}} object managing the settings for a scientific compute cluster}
#'   \item{\code{modules}}{A vector of pipeline modules} 
#'   \item{\code{module.res.req}}{Stores the resource requirements for each module. A list containing named vectors for the resources} 
#'   \item{\code{module.num.cores}}{Stores the number of cores for each module}
#' }
#'
#' @section Methods:
#' \describe{
#'   \item{\code{\link{setModuleResourceRequirements,RnBClusterRun,character,character-method}}}{Sets the resource requirements for the different pipeline modules}
#'   \item{\code{\link{setModuleNumCores,RnBClusterRun,integer,character-method}}}{Sets the number of cores used by the different pipeline modules}
#'   \item{\code{\link{getModuleNumCores,RnBClusterRun-method}}}{Gets the number of cores used by the different pipeline modules}
#'   \item{\code{\link{run,RnBClusterRun-method}}}{Submit the pipeline modules to the cluster}
#' }
#'
#' @name RnBClusterRun-class
#' @rdname RnBClusterRun-class
#' @author Fabian Mueller
#' @exportClass RnBClusterRun
setClass("RnBClusterRun",
	slots = list(
		architecture = "ClusterArchitecture",
		modules = "character",
		module.res.req = "list",
		module.num.cores = "integer"
	)
)

#' initialize.RnBClusterRun
#'
#' Initialize an RnBClusterRun object
#' 
#' @param .Object New instance of \code{RnBClusterRun}.
#' @param architecture A \code{\linkS4class{ClusterArchitecture}} object managing the settings for a scientific compute cluster.
#'
#' @author Fabian Mueller
#' @docType methods
setMethod("initialize","RnBClusterRun",
	function(
		.Object,
		architecture
	) {
		.Object@architecture <- architecture
		.Object@modules <- c (
			import        = "data import",
			qc            = "quality control",
			preprocessing = "preprocessing",
			tnt           = "tracks and tables",
			inference     = "covariate inference",
			exploratory   = "exploratory analysis",
			differential  = "differential methylation",
			differential_chunk   = "differential methylation (chunk)",
			differential_wrapup  = "differential methylation (wrapup)",
			wrapup        = "wrapup"
		)
		module.res.req <- rep(list(character(0)),length(.Object@modules))
		names(module.res.req) <- names(.Object@modules)

		module.num.cores <- rep(1L,length(.Object@modules))
		names(module.num.cores) <- names(.Object@modules)

		.Object
	}
)


if (!isGeneric("setModuleResourceRequirements")) setGeneric("setModuleResourceRequirements", function(object,resources,modules) standardGeneric("setModuleResourceRequirements"))
#' setModuleResourceRequirements-methods
#'
#' Specifies resource requirements for the different pipeline modules
#'
#' @param object \code{\linkS4class{RnBClusterRun}} object
#' @param resources A NAMED character vector containing the resource reuirements as value and the resource name as name
#' @param modules vector of applicable pipeline modules. Can be \code{"all"} to specify all modules
#' @return The modified object
#'
#' @rdname setModuleResourceRequirements-RnBClusterRun-methods
#' @docType methods
#' @aliases setModuleResourceRequirements
#' @aliases setModuleResourceRequirements,RnBClusterRun-method
#' @author Fabian Mueller
#' @export
setMethod("setModuleResourceRequirements",
	signature(
		object="RnBClusterRun",
		resources="character",
		modules="character"
	),
	function(
		object,
		resources,
		modules="all"
	) {
		if (length(modules)<1 || !all(modules %in% c(names(object@modules),"all"))){
			stop("Invalid modules specified")
		}
		if (length(resources) < 1){
			stop("Invalid resources parameter. Has to have at least length 1.")
		}
		if (any(is.na(names(resources))) || is.null(names(resources))){
			stop("Invalid resource requirement specification. Need names for all requirements")
		}
		if (any(modules=="all")){
			modules = names(object@modules)
		}
		for (mm in modules){
			object@module.res.req[[mm]][names(resources)] <- resources
		}
		return(object)
	}
)

if (!isGeneric("setModuleNumCores")) setGeneric("setModuleNumCores", function(object,num.cores,modules) standardGeneric("setModuleNumCores"))
#' setModuleNumCores-methods
#'
#' Specifies the number of cores used by the different pipeline modules
#'
#' @param object \code{\linkS4class{RnBClusterRun}} object
#' @param num.cores an integer specifying the number of cores to be used
#' @param modules vector of applicable pipeline modules. Can be \code{"all"} to specify all modules
#' @return The modified object
#'
#' @rdname setModuleNumCores-RnBClusterRun-methods
#' @docType methods
#' @aliases setModuleNumCores
#' @aliases setModuleNumCores,RnBClusterRun-method
#' @author Fabian Mueller
#' @export
setMethod("setModuleNumCores",
	signature(
		object="RnBClusterRun",
		num.cores="integer",
		modules="character"
	),
	function(
		object,
		num.cores,
		modules="all"
	) {
		if (length(modules)<1 || !all(modules %in% c(names(object@modules),"all"))){
			stop("Invalid modules specified")
		}
		if (any(num.cores<1)){
			stop("invalid number of cores specified. Must be 1 or more.")
		}
		num.cores <- num.cores[1]
		if (any(modules=="all")){
			modules = names(object@modules)
		}
		for (mm in modules){
			object@module.num.cores[mm] <- num.cores
		}
		return(object)
	}
)

if (!isGeneric("getModuleNumCores")) setGeneric("getModuleNumCores", function(object) standardGeneric("getModuleNumCores"))
#' getModuleNumCores-methods
#'
#' Retrieves the number of cores used by each module
#'
#' @param object \code{\linkS4class{RnBClusterRun}} object
#' @return A named vector containing the number of cores for each module
#'
#' @rdname getModuleNumCores-RnBClusterRun-methods
#' @docType methods
#' @aliases getModuleNumCores
#' @aliases getModuleNumCores,RnBClusterRun-method
#' @author Fabian Mueller
#' @export
setMethod("getModuleNumCores",
	signature(
		object="RnBClusterRun"
	),
	function(
		object
	) {
		return(object@module.num.cores)
	}
)

if (!isGeneric("run")) setGeneric("run", function(object,...) standardGeneric("run"))
#' run-methods
#'
#' Runs the analysis by submitting jobs for each module to the compute cluster
#'
#' @param object \code{\linkS4class{RnBClusterRun}} object
#' @param analysis.id analysis id. used for naming submitted jobs and log files
#' @param config.xml XML file specifying the analysis options and parameter settings
#' @param split.differential flag indicating whether to split the differnetial methylation module
#'        into seperate jobs according to sample annotation column and region type.
#' @param dry.run Prevent the actual job submission. Rather only write to a shell script file
#' @param long.cmd.thres commands that are longer than this number will be encapsulated in shell scripts
#' 		  rather than being submitted as direct command
#' @param queue The name of the queue the jobs are going to be submitted to
#' @return Nothing of importance
#'
#' @rdname run-RnBClusterRun-methods
#' @docType methods
#' @aliases run
#' @aliases run,RnBClusterRun-method
#' @author Fabian Mueller
#' @export
#' @examples
#' \donttest{
#' #specify the xml file for your analysis
#' xml.file <- "MY_ANALYSIS_SETTINGS.XML"
#' #set the cluster architecture specific to your environment
#' arch <- new("ClusterArchitectureSGE")
#' rnb.cr <- new("RnBClusterRun",arch)
#' #set up the cluster so that 32GB of memory are required (SGE resource is called "mem_free")
#' rnb.cr <- setModuleResourceRequirements(rnb.cr,c(mem_free="32G"),"all")
#' #set up the cluster to use 4 cores on each node for all modules
#' rnb.cr <- setModuleNumCores(rnb.cr,4L,"all")
#' #set up the cluster to use 2 cores for the exploratory analysis module
#' rnb.cr <- setModuleNumCores(rnb.cr,2L,"exploratory")
#' #run the actual analysis (remove dry.run=TRUE, to really submit the jobs)
#' run(rnb.cr, "rnbeads_analysis", xml.file, dry.run=TRUE)
#' }
setMethod("run",
	signature(
		object="RnBClusterRun"
	),
	function(
		object,
		analysis.id,
		config.xml,
		split.differential=TRUE,
		dry.run=FALSE,
		long.cmd.thres=1024L,
        queue=NULL
	) {
		rnb.cr <- object
		arch = rnb.cr@architecture
		r.exec <- getExecutable(arch,"Rscript")

		rnb.settings <- rnb.xml2options(config.xml,return.full.structure=TRUE)
		if (!("dir.reports" %in% names(rnb.settings$analysis.params))) {
			stop("Invalid analysis parameters: 'dir.reports' must be specified")
		}
		dir.reports <- rnb.settings$analysis.params[["dir.reports"]]
		data.type   <- rnb.settings$analysis.params[["data.type"]]
		if (!rnb.initialize.reports(dir.reports)) {
			stop(paste("Could not initialize reports in", dir.reports, "; make sure this path does not exist."))
		}
		cluster.dir <- file.path(dir.reports,"cluster_run")
		create.path(cluster.dir)
		log.dir <- cluster.dir #file.path(cluster.dir,"logs")
		#create.path(log.dir)
		logger.start("RnBeads Cluster Run",fname=c(file.path(log.dir,"cluster_run.log"),NA))

		if ("preanalysis.script" %in% names(rnb.settings)){
			source(rnb.settings$preanalysis.script)
		}

		do.call(rnb.options, rnb.settings$options)
		analysis.options <- rnb.options()

		## Set options
		if (length(analysis.options) != 0) {
			do.call(rnb.options, analysis.options)
		}

		logger.start("Saving settings")
		settings.file <- file.path(cluster.dir,"clusterRunSettings.RData")
		save(rnb.cr,rnb.settings,analysis.options,file=settings.file)
		file.copy(config.xml, file.path(cluster.dir,"options.xml"))
		logger.completed()

		#Can we submit shell scripts instead of binary commands for long commands?
		shell.script.for.long.commands <- is.element("sub.binary",arch@getSubCmdTokens.optional.args) && long.cmd.thres > 0L
		shell.script.dir <- cluster.dir
		submit.job <- function(name,cmd.tokens,...){
			r.cmd <- paste(cmd.tokens,collapse=" ")
            if(is.null(queue)){
    			cmd <- getSubCmdStr(arch, cmd.tokens, ...)
            }else{
    			cmd <- getSubCmdStr(arch, cmd.tokens, queue=queue, ...)
            }
			#make sure the command is not too long. else, wrap it in a shell script
			if (shell.script.for.long.commands){
				if (nchar(r.cmd)>long.cmd.thres){
					shell.script.file <- file.path(shell.script.dir,paste0(name,".sh"))
					fileConn<-file(shell.script.file)
					writeLines(c("#!/bin/sh",r.cmd), fileConn)
					close(fileConn)
					Sys.chmod(shell.script.file, mode = "0755")
					cmd.tokens.shell <- c(shell.script.file)
                    if(is.null(queue)){
					    cmd <- getSubCmdStr(arch, cmd.tokens.shell, sub.binary=FALSE, ...)
                    }else{
					    cmd <- getSubCmdStr(arch, cmd.tokens.shell, sub.binary=FALSE, queue=queue, ...)
                    }
				}
			}

			#actually submit
			if (!dry.run){
				system(cmd)
			}
			logger.info(c("Command used:",cmd))
			return(cmd)
		}

		cmds.submit <- c()
		deps.wrapup <- c()

		mm <- "import"
		logger.start(c("Running:",mm))
			log.file <- file.path(log.dir,paste0(mm,".log"))
			jid <- paste(analysis.id,mm,sep="_")
			res.req <- rnb.cr@module.res.req[[mm]]
			script.file <- system.file(file.path("extdata","Rscript",paste0("rscript_",mm,".R")), package = "RnBeads")

			cmd.tokens <- c(r.exec,script.file, "-x",config.xml, "-o",cluster.dir, "-c",getModuleNumCores(rnb.cr)[mm])
			cmds.submit[mm] <- submit.job(mm, cmd.tokens, log=log.file, job.name=jid, res.req=res.req)
			deps.wrapup <- c(deps.wrapup,jid)
		logger.completed()
		jid.import <- jid

		rnb.set.file <- file.path(cluster.dir,paste0("import","_RnBSet"))
		depend.jobs <- jid.import

		if (rnb.getOption("qc")){
			mm <- "qc"
			logger.start(c("Running:",mm))
				log.file <- file.path(log.dir,paste0(mm,".log"))
				jid <- paste(analysis.id,mm,sep="_")
				res.req <- rnb.cr@module.res.req[[mm]]
				script.file <- system.file(file.path("extdata","Rscript",paste0("rscript_",mm,".R")), package = "RnBeads")
				cmd.tokens <- c(
					r.exec,script.file,
					"-x",config.xml,
					"-s",rnb.set.file,
					"-c",getModuleNumCores(rnb.cr)[mm]
				)
				cmds.submit[mm] <- submit.job(mm, cmd.tokens, log=log.file, job.name=jid, res.req=res.req, depend.jobs=depend.jobs)
				deps.wrapup <- c(deps.wrapup,jid)
				depend.jobs <- jid
			logger.completed()
		}

		mm <- "preprocessing"
		logger.start(c("Running:",mm))
			log.file <- file.path(log.dir,paste0(mm,".log"))
			jid <- paste(analysis.id,mm,sep="_")
			res.req <- rnb.cr@module.res.req[[mm]]
			script.file <- system.file(file.path("extdata","Rscript",paste0("rscript_",mm,".R")), package = "RnBeads")
			
			cmd.tokens <- c(
				r.exec,script.file,
				"-x",config.xml,
				"-s",rnb.set.file,
				"-o",cluster.dir,
				"-c",getModuleNumCores(rnb.cr)[mm]
			)
			cmds.submit[mm] <- submit.job(mm, cmd.tokens, log=log.file, job.name=jid, res.req=res.req, depend.jobs=depend.jobs)
			deps.wrapup <- c(deps.wrapup,jid)
		logger.completed()
		jid.preprocessing <- jid

		rnb.set.file <- file.path(cluster.dir,paste0("preprocessing","_RnBSet"))
		depend.jobs <- jid.preprocessing

		if (rnb.getOption("export.to.bed") || (length(rnb.getOption("export.to.trackhub")) > 0)){
			mm <- "tnt"
			logger.start(c("Running:","Tracks and Tables (tnt)"))
				log.file <- file.path(log.dir,paste0(mm,".log"))
				jid <- paste(analysis.id,mm,sep="_")
				res.req <- rnb.cr@module.res.req[[mm]]
				script.file <- system.file(file.path("extdata","Rscript",paste0("rscript_",mm,".R")), package = "RnBeads")
				
				cmd.tokens <- c(
					r.exec,script.file,
					"-x",config.xml,
					"-s",rnb.set.file,
					"-c",getModuleNumCores(rnb.cr)[mm]
				)
				cmds.submit[mm] <- submit.job(mm, cmd.tokens, log=log.file, job.name=jid, res.req=res.req, depend.jobs=depend.jobs)
				deps.wrapup <- c(deps.wrapup,jid)
			logger.completed()
		}

		if (rnb.getOption("inference")){
			mm <- "inference"
			logger.start(c("Running:",mm))
				log.file <- file.path(log.dir,paste0(mm,".log"))
				jid <- paste(analysis.id,mm,sep="_")
				res.req <- rnb.cr@module.res.req[[mm]]
				script.file <- system.file(file.path("extdata","Rscript",paste0("rscript_",mm,".R")), package = "RnBeads")
				cmd.tokens <- c(
					r.exec,script.file,
					"-x",config.xml,
					"-s",rnb.set.file,
					"-o",cluster.dir,
					"-c",getModuleNumCores(rnb.cr)[mm]
				)
				cmds.submit[mm] <- submit.job(mm, cmd.tokens, log=log.file, job.name=jid, res.req=res.req, depend.jobs=depend.jobs)
				deps.wrapup <- c(deps.wrapup,jid)
			logger.completed()

			jid.inference <- jid
			rnb.set.file <- file.path(cluster.dir,paste0("inference","_RnBSet"))
			depend.jobs <- jid.inference
		}

		if (rnb.getOption("exploratory")){
			mm <- "exploratory"
			logger.start(c("Running:",mm))
				log.file <- file.path(log.dir,paste0(mm,".log"))
				jid <- paste(analysis.id,mm,sep="_")
				res.req <- rnb.cr@module.res.req[[mm]]
				script.file <- system.file(file.path("extdata","Rscript",paste0("rscript_",mm,".R")), package = "RnBeads")
				cmd.tokens <- c(
					r.exec,script.file,
					"-x",config.xml,
					"-s",rnb.set.file,
					"-c",getModuleNumCores(rnb.cr)[mm]
				)
				cmds.submit[mm] <- submit.job(mm, cmd.tokens, log=log.file, job.name=jid, res.req=res.req, depend.jobs=depend.jobs)
				deps.wrapup <- c(deps.wrapup,jid)
			logger.completed()
		}

		if (rnb.getOption("differential")){
			logger.start(c("Running:","differential"))
				cmp.cols <- rnb.getOption("differential.comparison.columns")
				if ((rnb.getOption("differential.permutations") != 0L) && split.differential){
					logger.warning(c("Option 'differential.permutations' is currently not compatible with splitting jobs for differential methylation on a cluster",
								"--> proceed without splitting the differential methylation job"))
					split.differential <- FALSE
				}
				if (is.null(cmp.cols) && split.differential){
					logger.warning(c("differential.comparison.columns=NULL is currently not compatible with splitting differential methylation task into chunks","--> proceed without splitting the differential methylation job"))
					split.differential <- FALSE
				}
				if (split.differential){
					logger.info(c("Splitting up into chunks"))
					if (rnb.getOption("differential.enrichment.lola")){
						logger.start(c("Checking/Downloading LOLA databases"))
							dbDownloadDir <- file.path(cluster.dir,paste0("lolaDbs"))
							lolaDbPaths <- prepLolaDbPaths(rnb.getOption("assembly"), downloadDir=dbDownloadDir)
							if (length(lolaDbPaths) > 0){
								saveRDS(lolaDbPaths, file.path(cluster.dir, "lolaDbPaths.rds"))
							} else {
								logger.warning(c("No LOLA DB found for assembly", rnb.getOption("assembly"), "--> continuing without LOLA enrichment"))
							}
						logger.completed()
					}
					logger.start(c("Running chunks"))
						mm <- "differential_chunk"
						res.req <- rnb.cr@module.res.req[[mm]]
						script.file <- system.file(file.path("extdata","Rscript",paste0("rscript_",mm,".R")), package = "RnBeads")

						reg.types <- rnb.region.types.for.analysis(rnb.getOption("assembly"))
						if(rnb.getOption("region.subsegments")>1L){
							reg.types <- c(reg.types,paste(reg.types,"subsegments",sep="."))
						}
						if (length(reg.types)>0){
							logger.info(c("Using region types:",paste(reg.types,collapse=",")))
						} else {
							logger.info(c("Skipping region level analysis"))
						}
						#split by pheno column and region type
						do.regions <- length(reg.types)>0
						combinations <- data.frame(Var1=cmp.cols,Var2=rep(NA,length(cmp.cols)))
						if (do.regions){
							combinations <- expand.grid(cmp.cols,reg.types)
						}

						#submit the jobs for each chunk
						chunk.ids <- c()
						depend.jobs.wrapup <- c()
						for (i in 1:nrow(combinations)){
							pp <- cmp.cols[combinations[i,1]]
							rr <- reg.types[combinations[i,2]]
							chunk.id.cur <-  paste0(pp,"_")
							if (do.regions){
								chunk.id.cur <- paste(pp,rr,sep="_")
							}

							mmm <- paste0(mm,"_",chunk.id.cur)
							log.file <- file.path(log.dir,paste0(mmm,".log"))
							jid <- paste(analysis.id,mmm,sep="_")
							chunk.ids <- c(chunk.ids,chunk.id.cur)
							depend.jobs.wrapup <- c(depend.jobs.wrapup,jid)

							r.tokens <- NULL
							if (do.regions){
								r.tokens <- c("-r",rr)
							}
							cmd.tokens <- c(
								r.exec,script.file,
								"-x",config.xml,
								"-s",rnb.set.file,
								"-i",chunk.id.cur,
								"-o",cluster.dir,
								"-p",pp,
								r.tokens,
								"-c",getModuleNumCores(rnb.cr)[mm]
							)
							cmds.submit[mmm] <- submit.job(mmm, cmd.tokens, log=log.file, job.name=jid, res.req=res.req, depend.jobs=depend.jobs)
						}
					logger.completed()
					logger.start(c("Running wrapup"))
						mm <- "differential_wrapup"
						log.file <- file.path(log.dir,paste0(mm,".log"))
						jid <- paste(analysis.id,mm,sep="_")
						res.req <- rnb.cr@module.res.req[[mm]]
						script.file <- system.file(file.path("extdata","Rscript",paste0("rscript_",mm,".R")), package = "RnBeads")

						chunk.tokens <- as.vector(rbind(rep("-d",length(chunk.ids)),chunk.ids))

						cmd.tokens <- c(
							r.exec,script.file,
							"-x",config.xml,
							"-s",rnb.set.file,
							"-o",cluster.dir,
							chunk.tokens,
							"-c",getModuleNumCores(rnb.cr)[mm]
						)
						cmds.submit[mm] <- submit.job(mm, cmd.tokens, log=log.file, job.name=jid, res.req=res.req, depend.jobs=depend.jobs.wrapup)
						deps.wrapup <- c(deps.wrapup,jid)
					logger.completed()

				} else {
					mm <- "differential"
					log.file <- file.path(log.dir,paste0(mm,".log"))
					jid <- paste(analysis.id,mm,sep="_")
					res.req <- rnb.cr@module.res.req[[mm]]
					script.file <- system.file(file.path("extdata","Rscript",paste0("rscript_",mm,".R")), package = "RnBeads")
					cmd.tokens <- c(
						r.exec,script.file,
						"-x",config.xml,
						"-s",rnb.set.file,
						"-o",cluster.dir,
						"-c",getModuleNumCores(rnb.cr)[mm]
					)
					cmds.submit[mm] <- submit.job(mm, cmd.tokens, log=log.file, job.name=jid, res.req=res.req, depend.jobs=depend.jobs)
					deps.wrapup <- c(deps.wrapup,jid)
				}
			logger.completed()
		}

		mm <- "wrapup"
		logger.start(c("Running:",mm))
			log.file <- file.path(log.dir,paste0(mm,".log"))
			jid <- paste(analysis.id,mm,sep="_")
			res.req <- rnb.cr@module.res.req[[mm]]
			script.file <- system.file(file.path("extdata","Rscript",paste0("rscript_",mm,".R")), package = "RnBeads")
			cmd.tokens <- c(
				r.exec,script.file,
				"-x",config.xml,
				"-c",getModuleNumCores(rnb.cr)[mm]
			)
			cmds.submit[mm] <- submit.job(mm, cmd.tokens, log=log.file, job.name=jid, res.req=res.req, depend.jobs=deps.wrapup)
		logger.completed()

		#save the submission commands to a shell script
		cmd.shellscript.file <- file(file.path(cluster.dir,"submitted_jobs.sh"))
		writeLines(c("#!/bin/sh",cmds.submit), cmd.shellscript.file)
		close(cmd.shellscript.file)

		logger.completed()
		invisible(TRUE)
	}
)

################################################################################
# Helper and utility functions
################################################################################

#' logger.machine.name
#'
#' Log the machine name the analysis is run on
#' 
#' @return None (invisible \code{NULL}).
#'
#' @author Fabian Mueller
#' @export
logger.machine.name <- function(){
	uname <- Sys.info()[["nodename"]]
	if (!is.null(uname)) logger.info(c("Machine name:",uname))
}

#' combine.diffMeth.objs
#'
#' combine differential methylation objects (output from \code{rnb.run.differential}).
#' To be more precise, the \code{diffmeth} and \code{dm.go.enrich} are merged.
#' individual objects that are merged are assumed to belong to the same analysis
#' and vary only in their indexing of region types and comparisons
#' @param obj.list a list containing outputs from \code{rnb.run.differential}
#' @author Fabian Mueller
combine.diffMeth.objs <- function(obj.list){
	if (length(obj.list) < 1){
		return(NULL)
	}
	if (length(obj.list) == 1){
		return(obj.list[[1]])
	}
	ontologies <- c("BP","MF")
	ontol.list.empty <- rep(list(list()),length(ontologies))
	names(ontol.list.empty) <- ontologies
	dm.go.enrich.comb <- list(probe=list(),region=list())
	dm.lola.enrich.comb <- list(probe=list(),region=list(),lolaDb=NULL)
	if(is.element("region_var",names(obj.list[[1]]$dm.go.enrich))){
	  dm.go.enrich.comb$region_var <- list()
	}
  if(is.element("region_var",names(obj.list[[1]]$dm.lola.enrich))){
    dm.lola.enrich.comb$region_var <- list()
  }

	# helper to check whether two lola DB objects are the same
	# necessary to check for compatibility
	isIdenticalLolaDb <- function(db1, db2){
		res <- length(db1$dbLocation) > 0 && length(db2$dbLocation) > 0 && length(db1$regionAnno$filename) > 0 && length(db2$regionAnno$filename) > 0
		res <- res && all(db1$dbLocation==db2$dbLocation) && all(db1$regionAnno$filename==db2$regionAnno$filename)
		return(res)
	}
	
	diffmeth <- obj.list[[1]]$diffmeth
	for (i in 1:length(obj.list)){
		dm <- obj.list[[i]]$diffmeth
		if (i > 1){
			diffmeth <- join.diffMeth(diffmeth,dm)
		}
		
		new.comps <- get.comparisons(dm)
		if (i > 1){
			new.comps <- new.comps[!(new.comps %in% get.comparisons(diffmeth))]
		}
		n.new.comps <- length(new.comps)
		#merge dm.go.enrich parts
		if (!is.null(obj.list[[i]]$dm.go.enrich)) {
			dmer <- obj.list[[i]]$dm.go.enrich$region
			dmer.var <- NULL
			#add empty lists for new comparisons
			new.comp.list.empty <- rep(list(ontol.list.empty),n.new.comps)
			names(new.comp.list.empty) <- new.comps
			dm.go.enrich.comb$region <- c(dm.go.enrich.comb$region,new.comp.list.empty)
			if(is.element("region_var",names(obj.list[[i]]$dm.go.enrich))){
			  dmer.var <- obj.list[[i]]$dm.go.enrich$region_var
			  dm.go.enrich.comb$region_var <- c(dm.go.enrich.comb$region_var,new.comp.list.empty)
			}
			#fill in the empty entries with the new entries
			for (cc in names(dmer)){
				for (oo in names(dmer[[cc]])){
					for (rr in names(dmer[[cc]][[oo]])){
						if (!is.element(rr,names(dm.go.enrich.comb$region[[cc]][[oo]]))){
							dm.go.enrich.comb$region[[cc]][[oo]][[rr]] <- dmer[[cc]][[oo]][[rr]]
							if(!is.null(dmer.var)){
							  dm.go.enrich.comb$region_var[[cc]][[oo]][[rr]] <- dmer.var[[cc]][[oo]][[rr]]
							}
						}
					}
				}
			}
		}
		#merge dm.lola.enrich parts
		if (!is.null(obj.list[[i]]$dm.lola.enrich)) {
			if (is.null(dm.lola.enrich.comb$lolaDb)){
				dm.lola.enrich.comb$lolaDb <- obj.list[[i]]$dm.lola.enrich$lolaDb
			} else {
				if (!isIdenticalLolaDb(dm.lola.enrich.comb$lolaDb, obj.list[[i]]$dm.lola.enrich$lolaDb)){
					logger.error("Encountered incompatible LOLA DB objects")
				}
			}
			dmer <- obj.list[[i]]$dm.lola.enrich$region
			dmer.var <- NULL
			#add empty lists for new comparisons
			new.comp.list.empty <- rep(list(list()),n.new.comps)
			names(new.comp.list.empty) <- new.comps
			dm.lola.enrich.comb$region <- c(dm.lola.enrich.comb$region,new.comp.list.empty)
			if(is.element("region_var",names(obj.list[[i]]$dm.lola.enrich))){
			  dmer.var <- obj.list[[i]]$dm.lola.enrich$region_var
			  dm.lola.enrich.comb$region_var <- c(dm.lola.enrich.comb$region_var,new.comp.list.empty)
			}
			#fill in the empty entries with the new entries
			for (cc in names(dmer)){
				for (rr in names(dmer[[cc]])){
					if (!is.element(rr,names(dm.lola.enrich.comb$region[[cc]]))){
						dm.lola.enrich.comb$region[[cc]][[rr]] <- dmer[[cc]][[rr]]
						if(!is.null(dmer.var)){
						  dm.lola.enrich.comb$region_var[[cc]][[rr]] <- dmer.var[[cc]][[rr]]
						}
					}
				}
			}
		}
	}
	
	#set emtpy GO enrichment analysis to NULL object
	if (length(dm.go.enrich.comb$region)==0){
		dm.go.enrich.comb <- NULL
	} else {
		class(dm.go.enrich.comb) <- "DiffMeth.go.enrich"
	}
	#set emtpy LOLA enrichment analysis to NULL object
	if (length(dm.lola.enrich.comb$region)==0){
		dm.lola.enrich.comb <- NULL
	} else {
		class(dm.lola.enrich.comb) <- "DiffMeth.lola.enrich"
	}

	res <- list(report=NULL,diffmeth=diffmeth,dm.go.enrich=dm.go.enrich.comb,dm.lola.enrich=dm.lola.enrich.comb)
	return(res)
}

########################################################################################################################
## logger.R
## created: 2012-03-28
## creator: Yassen Assenov
## ---------------------------------------------------------------------------------------------------------------------
## Logging functionality of the RnBeads modules.
########################################################################################################################

## G L O B A L S #######################################################################################################

.LOG.INFO <- new.env()

.LOG.INDENT <- "    "
.LOGGER <- "RnBeads"
.LOG.STAT <- c("INFO" = "INFO", "STATUS" = "STATUS", "WARNING" = "WARNING", "ERROR" = "ERROR")

## F U N C T I O N S ###################################################################################################

## get.memory.usage
##
## Gets the memory used by this process.
##
## @return Memory, in Gigabytes, used in this R session.
## @details
## In Windows, the returned value measures only the memory allocated by this session. It does not include the memory
## used by the R system itself (and many of the loaded libraries). In Linux and Mac OS, the returned value is the total
## memory used by the R process that runs this script.
## @author Yassen Assenov
get.memory.usage <- function() {
	if (.Platform$OS == "windows") {
		memused <- memory.size() / 1024
	} else if (Sys.info()["sysname"] == "Darwin") {
		processinfo <- system(paste('top -pid', Sys.getpid(), '-l 1 -stats MEM'), intern = TRUE)
		processinfo <- gsub("\\+$", "", processinfo[length(processinfo)])
		regex.process <- "(\\d+)([A-Z]?)$"
		if (grepl(regex.process, processinfo)) {
			memused <- as.double(gsub(regex.process, "\\1", processinfo))
			memscale <- gsub(regex.process, "\\2", processinfo)
			if (memscale == "") {
				memused <- (memused / 1048576) / 1024
			} else if (memscale == "K") {
				memused <- memused / 1048576
			} else if (memscale == "M") {
				memused <- memused / 1024
			} else if (memscale == "T") {
				memused <- memused * 1024
			}
		} else {
			memused <- 0
		}
	} else {
		processinfo <- paste("/proc", Sys.getpid(), "status", sep = "/")
		processinfo <- scan(processinfo, what = character(), sep = "\n", quiet = TRUE)
		memused <- grep("^VmSize\\:(.+)kB", processinfo, value = TRUE)
		memused <- as.double(gsub("\\s+", "", substr(memused, 8, nchar(memused) - 2))) / 1048576
	}
	return(memused)
}

########################################################################################################################

## get.disk.usage
##
## Gets the space used by all files and subdirectories of the given path.
##
## @param path Base directory to scan.
## @return Combined size, in Gigabytes, used by the files in the given path and in its subdirectories. The disk
##         space used by the directory entries themselves is not included.
## @author Yassen Assenov
get.disk.usage <- function(path = getOption("fftempdir")) {
	if (!isTRUE(file.info(path)[, "isdir"])) {
		stop("invalid value for path; expected existing directory")
	}
	sum(file.info(dir(path, full.names = TRUE, recursive = TRUE))[, "size"] / 1048576) / 1024
}

########################################################################################################################

format.usage <- function(usage.extractor) {
	tryCatch(format(usage.extractor(), digits = 1L, nsmall = 1L, justify = "right", width = 6),
		error = function(e) { "      " })
}

########################################################################################################################

## logger.format
##
## Memory usage formatter for the RnBeads text logger.
##
## @param record Log message in the form of a record. This message is expected to start with a status word, e.g. with
##               \code{WARNING}.
## @return       Character containing the formatted message.
logger.format <- function(record) {

	memory.used <- ifelse(.LOG.INFO[["memory"]], paste0(format.usage(get.memory.usage), " "), "")
	disk.used <- ifelse(.LOG.INFO[["disk"]], paste0(format.usage(get.disk.usage), " "), "")

	## Right-align the status word
	status.word <- sub("^([A-Z]+) .+", "\\1", record$msg)
	if (status.word %in% .LOG.STAT) {
		prefix <- paste(rep(" ", max(nchar(.LOG.STAT)) - nchar(status.word)), collapse = "")
	} else {
		prefix <- ""
	}
	## Prepend date, time, memory used and temporary disk space used
	paste0(record$timestamp, " ", memory.used, disk.used, prefix, record$msg)
}

########################################################################################################################

## Transforms a vector of message elements to a character.
##
## @param txt    Character vector with message elements.
## @param indent Flag indicating if the message must be prepended by \code{.LOG.INDENT} to indicate it belongs to
##               a specific section.
## @return       Single-element character vector concatenating all elements in \code{txt}, possibly with indentation.
logger.transform <- function(txt, indent = TRUE) {
	if (!(is.character(txt) && is.vector(txt))) {
		stop("invalid value for parameter txt")
	}
	if (length(txt) > 1) {
		txt <- paste(txt, collapse = " ")
	}
	if (indent && length(.LOG.INFO[["titles"]]) != 0) {
		txt <- paste(paste(rep(.LOG.INDENT, length(.LOG.INFO[["titles"]])), collapse = ""), txt, sep = "")
	}
	return(txt)
}

########################################################################################################################

logger.paste <- function(status.word, txt, logger = .LOGGER) {
	record <- list(
		timestamp = format(Sys.time(), "%Y-%m-%d %H:%M:%S "),
		msg = paste(.LOG.STAT[status.word], logger.transform(txt)))
	txt <- paste(.LOG.INFO[["formatter"]](record), "\n", sep = "")
	for (fname in .LOG.INFO[["logfiles"]]) {
		cat(txt, file = ifelse(is.na(fname), "", fname), append = TRUE)
	}
}

########################################################################################################################

#' logger.getfiles
#'
#' Gets the files currently used by the logger.
#'
#' @return Vector storing the full names of the files that are being used by the logger. This vector contains \code{NA}
#'         as an element if the logger is (also) using the console for its output. If logging functionality is disabled
#'         (see \code{\link{rnb.options}}) or the logger is not initialized, this function returns \code{NULL}.
#'
#' @examples
#' \donttest{
#' if (NA %in% logger.getfiles())
#'   cat("Console logger is enabled\n")
#' }
#' @seealso \code{\link{logger.isinitialized}} to check if logging is activated;
#'   \code{\link{logger.start}} for initializing a logger or starting a section
#' @author Yassen Assenov
#' @export
logger.getfiles <- function() {
	if (rnb.getOption("logging") && exists("logfiles", envir = .LOG.INFO, inherits = FALSE)) {
		return(get("logfiles", envir = .LOG.INFO, inherits = FALSE))
	}
	return(NULL)
}

########################################################################################################################

#' logger.isinitialized
#'
#' Checks if the logger is initialized.
#'
#' @return \code{TRUE} if the logger was initialized and is in use; \code{FALSE} otherwise.
#'
#' @examples
#' \donttest{
#' if (!logger.isinitialized())
#'   logger.start(fname = NA)
#' }
#' @seealso \code{\link{logger.start}} for initializing a logger or starting a section
#' @author Yassen Assenov
#' @export
logger.isinitialized <- function() {
	if (rnb.getOption("logging")) {
		return(exists("logfiles", envir = .LOG.INFO, inherits = FALSE))
	}
	return(FALSE)
}

########################################################################################################################

#' Writing text messages to the log file.
#'
#' Appends a single-line status message to the log text file. The message is prepended by its type, which is one of
#' \code{STATUS}, \code{INFO}, \code{WARNING} or \code{ERROR}.
#'
#' @rdname loggerMessages
#' @aliases logger.status
#' @aliases logger.info
#' @aliases logger.warning
#' @aliases logger.error
#'
#' @param txt       Text to add to the log file. This must be a \code{character} vector; its elements are concatenated
#'                  using a single space (\code{" "}) as a separator.
#' @param terminate Flag indicating if the execution is to be terminated after this error message is added to the log.
#' @return None (invisible \code{NULL}).
#'
#' @examples
#' \donttest{
#' if (!logger.isinitialized())
#'   logger.start(fname = NA)
#' logger.status(c("Reached step", 2))
#' logger.info(c("Provided email:", rnb.getOption("email")))
#' }
#' @seealso \code{\link{logger.isinitialized}}to check if logging is activated;
#'   \code{\link{logger.start}} for initializing a logger or starting a section
#' @author Yassen Assenov
#' @export
logger.status <- function(txt) {
	if (rnb.getOption("logging")) {
		if (!logger.isinitialized()) {
			# stop("logger is not initialized")
			logger.start(fname=NA) #initialize the logger to the console if not started yet
		}
		logger.paste("STATUS", txt)
	}
}

########################################################################################################################

#' @rdname loggerMessages
#' @export
logger.info <- function(txt) {
	if (rnb.getOption("logging")) {
		if (!logger.isinitialized()) {
			# stop("logger is not initialized")
			logger.start(fname=NA) #initialize the logger to the console if not started yet
		}
		logger.paste("INFO", txt)
	}
}

########################################################################################################################

#' @rdname loggerMessages
#' @export
logger.warning <- function(txt) {
	if (rnb.getOption("logging")) {
		if (!logger.isinitialized()) {
			# stop("logger is not initialized")
			logger.start(fname=NA) #initialize the logger to the console if not started yet
		}
		logger.paste("WARNING", txt)
	}
}

########################################################################################################################

#' @rdname loggerMessages
#' @export
logger.error <- function(txt, terminate = rnb.getOption("logging.exit.on.error")) {
	if (rnb.getOption("logging")) {
		if (!logger.isinitialized()) {
			stop("logger is not initialized")
			logger.start(fname=NA) #initialize the logger to the console if not started yet
		}
		logger.paste("ERROR", txt)
		if (terminate) {
			for (logfile in get("logfiles", envir = .LOG.INFO, inherits = FALSE)) {
				if (is.na(logfile)) {
					cat("\n")
				} else {
					cat("\n", file = logfile, append = TRUE)
				}
			}
			quit(save = "no", status = 1L)
		}
		stop(logger.transform(txt, FALSE))
	}
}

########################################################################################################################

#' Log File Management
#'
#' Functions for logger management.
#'
#' @rdname loggerManagement
#' @aliases logger.start
#' @aliases logger.completed
#' @aliases logger.close
#'
#' @param txt   Description to add to the log file. The words \code{STARTED} and \code{COMPLETED} are prepended to the
#'              message upon initialization and completion of the section, respectively.
#' @param fname Name of the log file and/or console. Note that at most one file name can be specified. The function
#'              \code{logger.start} normalizes the given name, that is, it converts it to an absolute name. If this
#'              parameter is \code{NA}, logger messages are printed to the console. If it is a two-element vector
#'              containing one file name and \code{NA}, the logger is (re)initialized to print messages both to the
#'              given file name and the console. A value of \code{NULL} (default) indicates the logger should continue
#'              using the previously specified file.
#' @return None (invisible \code{NULL}).
#'
#' @examples
#' \donttest{
#' if (!logger.isinitialized())
#'   logger.start(fname = NA)
#' logger.start("Tests for Significance")
#' logger.completed()
#' logger.close()
#' }
#' @section Details:
#' \code{logger.start} initializes the logger and/or starts a new section. \code{logger.completed} completes the last
#' (innermost) open section in the log. \code{logger.close} deinitializes the logger. Note that after reinitialization
#' or deinitialization, the information about the current output file, as well as any open sections, is deleted.
#'
#' @seealso logger.isinitialized
#' @author Yassen Assenov
#' @export
logger.start <- function(txt = character(0), fname = NULL) {
	if (rnb.getOption("logging")) {
		if (!logger.isinitialized()) {
			if (is.null(fname)) {
				logger.start(fname=NA) #initialize the logger to the console if not started yet
				# stop("logger is not initialized")
			}
		}
		if (!is.null(fname)) {
			if (!((length(fname) == 1 && (is.character(fname) || is.na(fname))) ||
				(length(fname) == 2 && (is.character(fname) && sum(is.na(fname)) == 1 &&
						sum(fname == "", na.rm = TRUE) == 0)))) {
				stop("invalid value for fname")
			}
			if (exists("logfiles", envir = .LOG.INFO, inherits = FALSE)) {
				logger.close()
			}
			.LOG.INFO[["memory"]] <- rnb.getOption("logging.memory")
			.LOG.INFO[["disk"]] <- (rnb.getOption("logging.disk") && rnb.getOption("logging.disk"))
			.LOG.INFO[["formatter"]] <- logger.format
			for (i in 1:length(fname)) {
				if (!is.na(fname[i])) {
					fname[i] <- normalizePath(fname[i], mustWork = FALSE)
				}
			}
			.LOG.INFO[["logfiles"]] <- as.character(fname)
			.LOG.INFO[["titles"]] <- character(0)
		}
		txt <- logger.transform(txt, indent = FALSE)
		if (length(txt) != 0) {
			logger.status(paste("STARTED", txt))
			.LOG.INFO[["titles"]] <- c(.LOG.INFO[["titles"]], txt)
		}
	}
}

########################################################################################################################

#' @rdname loggerManagement
#' @export
logger.completed <- function() {
	if (rnb.getOption("logging")) {
		if (!logger.isinitialized()) {
			stop("logger is not initialized")
		}
		N <- length(.LOG.INFO[["titles"]])
		if (length(.LOG.INFO[["titles"]]) == 0) {
			logger.error("No section to complete")
		}
		txt <- paste("COMPLETED ", .LOG.INFO[["titles"]][N], ifelse(N == 1, "\n", ""), sep = "")
		.LOG.INFO[["titles"]] <- .LOG.INFO[["titles"]][-N]
		logger.status(txt)
	}
}

########################################################################################################################

## logger.addfile
##
## Adds a new file (or console) to contain the messages of the logger.
##
## @param fname Name of the log file. This function normalizes the given file name, that is, it converts it to an
##              absolute name. Set this to \code{NA} in order to print log messages to the console.
##
## @seealso \code{\link{logger.start}} for re-initializing the logger
##
## @author Yassen Assenov
logger.addfile <- function(fname) {
	if (rnb.getOption("logging")) {
		if (!(length(fname) == 1 && (is.na(fname) || is.character(fname)))) {
			stop("invalid value for fname")
		}
		if (!logger.isinitialized()) {
			# stop("logger is not initialized")
			logger.start(fname=NA) #initialize the logger to the console if not started yet
		}
		logfiles <- get("logfiles", envir = .LOG.INFO, inherits = FALSE)
		if (!(fname %in% logfiles)) {
			if (any(sapply(logfiles, function(x) { is.character(x) && (!is.na(x)) }))) {
				## Adding a file when there is already a file
				stop("logger is already initialized to file")
			}
			.LOG.INFO[["logfiles"]] <- c(logfiles, as.character(fname))
		}
	}
}

########################################################################################################################

#' @rdname loggerManagement
#' @export
logger.close <- function() {
	if (rnb.getOption("logging")) {
		rm(list = ls(envir = .LOG.INFO), envir = .LOG.INFO)
	}
}

########################################################################################################################

#' logger.argument
#'
#' Reads a command-line argument supplied to a script.
#'
#' @param arg.names       \code{character} vector of acceptable argument names. This function scans the provided
#'                        arguments and performs a case insensitive match.
#' @param full.name       One-element \code{character} vector giving the argument's full name or description. This is
#'                        used in a log message in case of an error.
#' @param arg.type        Variable type of the argument. Must be one of \code{"character"}, \code{"logical"},
#'                        \code{"integer"}, \code{"double"}, \code{"numeric"} or \code{"real"}. The last three types are
#'                        all synonyms.
#' @param accepted.values Vector of accepted values for the argument. This must be of the type given in \code{arg.type}.
#'                        Set this to \code{NULL} if there are no restrictions on the argument values.
#' @param default         Default value for the argument in case it is not specified. Setting this to \code{NULL} makes
#'                        the argument required, that is, an error is generated if the argument is not specified. Set
#'                        this to \code{NA} if is not a required argument and it shouldn't default to a specific value.
#'                        Otherwise, if \code{accepted.values} is provided, this must be one of its elements.
#' @param arg.list        Vector of arguments provided at the execution of the script. The arguments should be provided
#'                        as \emph{name=value} pairs.
#' @return                Argument's value, or \code{NULL} if such is not provided.
#'
#' @details This is convenience function for reading parameters supplied to the script in the form \emph{name = value}.
#'          It expects that logging is enabled (see \code{\link{rnb.options}}). The function fails if this condition is
#'          not met.
#' @examples
#' \donttest{
#' n.iterations <- logger.argument("iterations", "number of iterations", "integer",
#'   accepted.values = 1:100, default = 1L)
#' logger.close()
#' }
#' @author Yassen Assenov
#' @export
logger.argument <- function(arg.names, full.name, arg.type = "character", accepted.values = NULL, default = NULL,
	arg.list = commandArgs()) {
	if (rnb.getOption("logging") == FALSE) {
		stop("logging is disabled")
	}
	if (!(is.character(arg.names) && length(arg.names) != 0 && (!any(is.na(arg.names))))) {
		stop("invalid value for arg.names")
	}
	if (!(is.character(full.name) && length(full.name) == 1 && (!is.na(full.name)))) {
		stop("invalid value for full.name")
	}
	if (!(is.character(arg.type) && length(arg.type) == 1 && (!is.na(arg.type)))) {
		stop("invalid value for arg.type")
	}
	if (!(arg.type %in% c("character", "logical", "integer", "double", "numeric", "real"))) {
		stop("invalid value for arg.type")
	}
	if (!(is.null(default) || length(default) == 1)) {
		stop("invalid value for default; expected single value or NULL")
	}
	if (!(is.null(accepted.values) || is.null(default))) {
		accepted.values <- c(NA, accepted.values)
		if (!(default %in% accepted.values)) {
			stop("invalid value for default; expected one of the accepted values")
		}
	}
	if (!(is.character(arg.list) && length(arg.list) != 0 && (!any(is.na(arg.list))))) {
		stop("invalid value for arg.list")
	}

	if (!logger.isinitialized()) {
		logger.start(fname = NA)
	}
	value <- NULL
	wrongType <- function(e) {
		logger.error(c("Invalid value for argument", full.name, ":", value))
	}
	arg.names <- tolower(arg.names)
	for (arg in strsplit(arg.list, split = "=", fixed = TRUE)) {
		if (length(arg) != 2) {
			next
		}
		arg[1] <- tolower(arg[1])
		if (arg[1] %in% arg.names) {
			if (!is.null(value)) {
				logger.error(c("Redefinition of", full.name))
			}
			## Perform type check
			value <- arg[2]
			if (arg.type == "character") {
				## No type conversion necessary
			} else if (arg.type == "logical") {
				value <- as.logical(toupper(arg[2]))
				if (is.na(value)) {
					value <- arg[2]
					wrongType("")
				}
			} else if (arg.type == "integer") {
				value <- tryCatch(as.integer(arg[2]), warning = wrongType, error = wrongType)
			} else { # arg.type %in% c("double", "numeric", "real")
				value <- tryCatch(as.numeric(arg[2]), warning = wrongType, error = wrongType)
			}
		}
	}
	if (is.null(value)) {
		if (is.null(default)) {
			logger.error(c("Missing required argument", full.name))
		}
		if (is.na(default)) {
			if (arg.type == "character") {
				value <- as.character(NA)
			} else if (arg.type == "logical") {
				value <- as.logical(NA)
			} else if (arg.type == "integer") {
				value <- as.integer(NA)
			} else {
				value <- as.numeric(NA)
			}
		} else {
			value <- default
		}
	}
	if (!(is.null(accepted.values) || (value %in% accepted.values))) {
		wrongType("")
	}
	return(value)
}

########################################################################################################################

#' logger.validate.file
#'
#' Validates the specified file or directory exists. Prints an error or a warning message to the log if it does not
#' exist, it is not of the accepted type or is not accessible.
#'
#' @param file      Name of file or directory to validate.
#' @param is.file   Flag indicating if the given name must denote an existing file. If this is \code{FALSE}, the given
#'                  name must denote a directory. Set this to \code{NA} if both types are an acceptable scenario.
#' @param terminate Flag indicating if the execution is to be terminated in case the validation fails. This parameter
#'                  determines if an error message (\code{terminate} is \code{TRUE}) or a warning message
#'                  (\code{terminate} is \code{FALSE}) is to be sent to the log when the specified file or directory
#'                  does not exist, is not of the accepted type or is not accessible.
#' @return Whether the validation succeeded or not, invisibly. Note that when \code{terminate} is \code{TRUE} and the
#'         validation fails, the R session is closed and thus no value is returned.
#'
#' @examples
#' \donttest{
#' if (!logger.isinitialized())
#'   logger.start(fname = NA)
#' # Validate the current working directory exists
#' logger.validate.file(getwd(), FALSE)
#' }
#' @author Yassen Assenov
#' @export
logger.validate.file <- function(file, is.file = TRUE, terminate = TRUE) {
	if (!(is.character(file) && length(file) == 1 && (!is.na(file)))) {
		stop("invalid value for file; expected single character")
	}
	if (!parameter.is.flag(is.file)) {
		stop("invalid value for is.file; expected TRUE or FALSE")
	}
	if (!parameter.is.flag(terminate)) {
		stop("invalid value for terminate; expected TRUE or FALSE")
	}
	file <- file[1]
	is.file <- is.file[1]
	terminate <- terminate[1]
	if (rnb.getOption("logging")) {
		if (!file.exists(file)) {
			msg <- ifelse(is.na(is.file), "File / directory", ifelse(is.file, "File", "Directory"))
			msg <- c(msg, "not found:", file)
			if (terminate) {
				logger.error(msg)
			}
			logger.warning(msg)
			return(invisible(FALSE))
		}
		if (!is.na(is.file)) {
			is.dir <- file.info(file)[1, "isdir"]
			if (is.file == is.dir) {
				msg <- c(file, "is a", ifelse(is.dir, "directory", "file"))
				if (terminate) {
					logger.error(msg)
				}
				logger.warning(msg)
				return(invisible(FALSE))
			}
		}
		return(invisible(TRUE))
	}
}

################################################################################
# Cluster Architecture Descriptions
################################################################################
################################################################################
# Concrete implementations for the SGE environment
# have to implement:
# Method: getSubCmdTokens
################################################################################

#' ClusterArchitectureSGE Class
#'
#' A child class of \code{\linkS4class{ClusterArchitecture}} implementing specifications of Sun Grid Engine (SGE) architectures.
#'
#' @details
#' Follow this template if you want to create your own ClusterArchitecture class.
#'
#' @section Slots:
#' see \code{\linkS4class{ClusterArchitecture}}
#'
#' @section Methods:
#' \describe{
#'   \item{\code{\link{getSubCmdTokens,ClusterArchitectureSGE-method}}}{Returns a vector of command line tokens corresponding to submitting
#'   a job with the given command to the cluster}
#' }
#'
#' @name ClusterArchitectureSGE-class
#' @rdname ClusterArchitectureSGE-class
#' @author Fabian Mueller
#' @exportClass ClusterArchitecture
setClass("ClusterArchitectureSGE",
	contains = "ClusterArchitecture"
)

#' initialize.ClusterArchitectureSGE
#'
#' Initialize an ClusterArchitecture object for a Sun Grid Engine (SGE)
#' 
#' @param .Object New instance of \code{ClusterArchitectureSGE}.
#' @param name A name or identifier
#' @param ... arguments passed on to the constructor of \code{\linkS4class{ClusterArchitecture}} (the parent class)
#'
#' @export
#' @author Fabian Mueller
#' @docType methods
setMethod("initialize","ClusterArchitectureSGE",
	function(
		.Object,
		name="ClusterArchitectureSGE",
		...
	) {
		.Object <- callNextMethod(.Object=.Object, name=name, ...)
		.Object <- setExecutable(.Object,"R","R")
		.Object <- setExecutable(.Object,"Rscript","Rscript")
		.Object <- setExecutable(.Object,"python","python")
		.Object@getSubCmdTokens.optional.args <- c("sub.binary","quote.cmd")
		.Object
	}
)

#' getSubCmdTokens-methods
#'
#' Returns a string for the of command line corresponding to submitting
#' a job with the given command to the cluster.
#' @details
#' For a concrete child class implementation for a sun grid architecture specification see \code{\linkS4class{ClusterArchitectureSGE}}
#'
#' @param object \code{\linkS4class{ClusterArchitectureSGE}} object
#' @param cmd.tokens a character vector specifying the executable command that should be wrapped in the cluster submission command
#' @param log file name and path of the log file that the submitted job writes to
#' @param job.name name of the submitted job
#' @param res.req character vector specifying required resources. The resource requirements should be the values of the vector,
#'                the names should specify the resource name
#' @param depend.jobs character vector containg names or ids of jobs the submitted job will depend on.
#' @param sub.binary treat the command as binary (see \code{-b} flag of \code{qsub} of the SGE documentation)
#' @param quote.cmd Flag indicating whether the submitted cammed should also be wrapped in quotes
#' @param queue The name of the queue to submit jobs to
#' @return A character vector containing the submission command tokens
#'
#' @rdname getSubCmdTokens-ClusterArchitectureSGE-methods
#' @docType methods
#' @aliases getSubCmdTokens,ClusterArchitectureSGE-method
#' @author Fabian Mueller
#' @export
#' @examples
#' \donttest{
#' arch <- new("ClusterArchitectureSGE",
#' 	name="my_sge_architecture"
#' )
#' getSubCmdTokens(arch,c("Rscript","my_great_script.R"),"my_logfile.log")
#' }
setMethod("getSubCmdTokens",
	signature(
		object="ClusterArchitectureSGE"
	),
	function(
		object,
		cmd.tokens,
		log,
		job.name = "",
		res.req = character(0),
		depend.jobs = character(0),
		sub.binary = TRUE,
		quote.cmd = TRUE,
        queue = NULL
	) {
		res.req.tokens <- NULL
		if (length(res.req) > 0){
			if (any(is.na(names(res.req))) || is.null(names(res.req))){
				stop("Invalid resource requirement specification. Need names for all requirements")
			}
			rr <- as.vector(rbind(rep("-l",length(res.req)),paste0(names(res.req),"=",res.req)))
			res.req.tokens <- c(res.req.tokens,rr)
		}
		log.token <- NULL
		if (nchar(log)>0) {
			log.token <- c("-o",log)
		}
		job.name.token <- NULL
		if (nchar(job.name)>0) {
			job.name.token <- c(job.name.token,"-N",job.name)
		}
		dependency.token <- NULL
		if (length(depend.jobs)>0){
			dependency.token <- c(dependency.token, "-hold_jid", paste0(paste(depend.jobs,collapse=",")))
		}
		if (quote.cmd){
			cmd.tokens <- paste0("'",paste(cmd.tokens,collapse=" "),"'")
		}
		bin.token <- ifelse(sub.binary,"y","n")
		res <- c(
			"qsub",
			"-V",
			res.req.tokens,
            ifelse(is.null(queue),"",paste0("-q ",queue)),
			log.token,
			"-j","y",
			job.name.token,
			dependency.token,
			"-b",bin.token,
			cmd.tokens
		)
		return(res)
	}
)

########################################################################################################################
## test_report_3.R
## creator: Yassen Assenov
## ---------------------------------------------------------------------------------------------------------------------
## Simple tests on creating and adding listing of tables to a report.
########################################################################################################################

test_report_3 <- function() {

	NAME.POOL <- c("Anna Prentice", "Annie Versary", "Ben Zeen", "Bill Ding", "Cole Desice", "Dee Zaster", "Eddie Bull",
		"Emma Fraid", "Flo Tinaway", "Hugh Manatty", "Isabel Ringing", "Joe King", "Justin Case", "Kerry Dowt",
		"Lois Steem", "Markus Absent", "Matt Schtick", "Maya Magination", "Miss Teri Novel", "Nick Rofillia",
		"Otto Matic", "Perry Scope", "Ray Neday", "Robbin Banks", "Shirley Knott", "Terry Torrie", "Trudy Lite",
		"Vic Tree", "Walter Mellon", "Warren Peece")

	## Generate a few random tables

	setting.names <- list(
		"Eye color" = c("blue" = "blue", "brown" = "brown", "black" = "black", "green" = "green"),
		"Hair color" = c("b" = "blond", "o" = "brown", "r" = "red", "k" = "black"))

	celebrities <- list()
	for (n1 in names(setting.names[[1]])) {
		for (n2 in names(setting.names[[2]])) {
			celebrities[[paste(n1, n2, sep = "_")]] <- matrix(sample(NAME.POOL, 12), nrow = 4, ncol = 3,
				dimnames = list(c("Europe", "Asia", "Africa", "Australia"), c("A", "B", "C")))
		}
	}
	rm(n1, n2)

	## Initialize a report
	report.directory <- "RnBeads_report_testing"
	report.file <- file.path(report.directory, "test_tables.html")
	dirs <- c("configuration" = "data", "data" = "data", "pdfs" = "data", "pngs" = "data", "high" = "data")
	report <- createReport(report.file, "Testing tables", dirs = dirs, init.configuration = TRUE)
	report <- rnb.add.section(report, "Celebrity Names", "There they are:")
	report <- rnb.add.tables(report, celebrities, setting.names, row.names = FALSE)
	off(report)

	unlink(report.directory, recursive = TRUE)
}

########################################################################################################################
## main.R
## created: 2012-05-15
## creator: Yassen Assenov
## ---------------------------------------------------------------------------------------------------------------------
## Main workflow of the RnBeads analysis pipeline.
########################################################################################################################

## F U N C T I O N S ###################################################################################################

#' rnb.tracks.to.export
#'
#' Checks if BED files or other tracks should be exported from the given dataset.
#'
#' @param rnb.set Methylation dataset currently being analyzed.
#' @return \code{TRUE} if the data export module can and should be executed on the dataset and should include track
#'         files; \code{FALSE} otherwise.
#'
#' @author Yassen Assenov
#' @noRd
rnb.tracks.to.export <- function(rnb.set) {
	any(rnb.getOption("export.to.bed"), length(rnb.getOption("export.to.trackhub")) > 0) &&
		(length(intersect(rnb.getOption("export.types"), c(rnb.region.types(assembly(rnb.set)), "sites"))) > 0)
}

########################################################################################################################

#' rnb.add.optionlist
#'
#' Adds a section to the given report displaying the specified module options and their values.
#'
#' @param report     Report to which the section of option values is to be added.
#' @param optionlist List of RnBeads module options. The attribute \code{"enabled"}, if present, is used to mark which
#'                   of these options were actually used.
#' @return The modified report.
#'
#' @author Yassen Assenov
#' @noRd
rnb.add.optionlist <- function(report, optionlist) {
	txt <- "The table below lists the options of the executed module."
	report <- rnb.add.section(report, "Parameter Overview", txt, collapsed = TRUE)
	write.table.options(optionlist, report)
	return(report)
}

########################################################################################################################

## rnb.build.index.internal
##
## Creates an HTML index file that contains listing of all generated and scheduled \pkg{RnBeads} reports.
##
## @param dir.reports        Directory that contains HTML reports generated by \pkg{RnBeads} modules. If this directory
##                           does not exist, is a regular file, is inaccessible, or does not contain any recognizable
##                           HTML report files, this function does not generate an HTML index file and produces an error
##                           or a warning message.
## @param fname              One-element \code{character} vector specifying the name of the index file to be generated.
##                           See the \emph{Details} section for restrictions on the name. The file will be created in
##                           \code{dir.reports}. If such a file already exists, it will be overwritten.
## @param dir.configuration  Subdirectory that hosts configuration files shared by the reports. This must be a
##                           \code{character} vector of length one that gives location as a path relative to
##                           \code{dir.reports}. Strong restrictions apply to the path name. See the description of the
##                           \code{\link{createReport}} function for more details.
## @param log.file           Name of log file to link to. This must be a \code{character} vector of lenght one that gives
##                           file name relative to \code{dir.reports}. If this file does not exist, no link to a log file
##                           is created and this function gives a warning message. Set this to \code{NULL} or an empty
##                           string if the index file should not include a link to a log file.
## @param export.enabled     Flag indicating if the data export module is scheduled to run; this parameter will disappear
##                           after the issues in rnb.run.tnt are fixed.
## @param current.report     File name of the report produced by the currently running module.
## @param open.index         Flag indicating if the index should be displayed after it is created. If this is
##                           \code{TRUE}, \code{\link{rnb.show.report}} is called to open the generated HTML file.
## @return Names of all HTML report files that were referenced in the newly generated index, invisibly. The order of the
##         file names is the same as the one they are listed in the index.
##
## @author Yassen Assenov
rnb.build.index.internal <- function(dir.reports, fname = "index.html", dir.configuration = "configuration",
	log.file = NULL, export.enabled = TRUE, current.report = NULL, open.index = FALSE) {

	## Load table of report descriptions
	tbl <- system.file("extdata/reportDescriptions.txt", package = "RnBeads")
	tbl <- read.delim(tbl, quote = "", row.names = 1, stringsAsFactors = FALSE)
	fname.base <- tolower(sub("\\.[^.]+$", "", fname))
	if (fname.base %in% rownames(tbl)) {
		stop("invalid value for fname; it matches a potential report file name")
	}

	## Extract files in dir.reports
	all.files <- dir(dir.reports)
	report.files <- all.files[!file.info(file.path(dir.reports, all.files))$isdir]
	report.files <- report.files[grep("\\.(htm|html|xhtml|xml)$", tolower(report.files))]
	if (fname %in% report.files) {
		report.files <- setdiff(report.files, fname)
	}

	## Identify RnBeads reports
	report.names <- sub("^(.+)\\.(htm|html|xhtml|xml)$", "\\1", tolower(report.files))
	if (is.null(current.report)) {
		tbl <- tbl[intersect(rownames(tbl), report.names), ]
	} else {
		## Focus only on reports that are scheduled for the current pipeline
		reports.skipped <- c("import", "qc", "preprocessing", "inference", "exploratory", "differential")
		names(reports.skipped) <- rownames(tbl)[c(1:3, 5:7)]
		reports.skipped <- sapply(reports.skipped, function(x) { rnb.getOption(x) == FALSE })
		reports.skipped <- names(reports.skipped)[reports.skipped]
		if (!export.enabled) {
			reports.skipped <- c(reports.skipped, "tracks_and_tables")
		}
		tbl <- tbl[setdiff(rownames(tbl), reports.skipped), ]
		rm(reports.skipped)
	}
	if (nrow(tbl) == 0) {
		logger.info(c("No RnBeads reports found in", dir.reports))
		return(invisible(character()))
	}

	## Initialize index HTML file as RnBeads report
	report.dirs <- all.files[file.info(file.path(dir.reports, all.files))$isdir]
	dir.images <- paste(fname.base, "images", sep = "_")
	if (dir.images %in% report.dirs) {
		unlink(file.path(dir.reports, dir.images), recursive = TRUE)
	}
	temp.dir <- 0L
	while (is.integer(temp.dir)) {
		if (as.character(temp.dir) %in% all.files) {
			temp.dir <- temp.dir + 1L
		} else {
			temp.dir <- as.character(temp.dir)
		}
	}
	analysis.name <- rnb.getOption("analysis.name")
	ptitle <- base::ifelse(is.null(analysis.name), "RnBeads report", paste("RnBeads:", analysis.name))
	ititle <- base::ifelse(is.null(analysis.name) || is.na(analysis.name), "RnBeads Analysis", analysis.name)
	index.dirs <- c(configuration = dir.configuration, data = temp.dir, pdfs = temp.dir, high = temp.dir)
	report <- createReport(file.path(dir.reports, fname), ititle, ptitle, dirs = index.dirs)
	unlink(file.path(dir.reports, temp.dir), recursive = TRUE)
	dir.images <- normalizePath(file.path(dir.reports, dir.images))
	file.copy(system.file("extdata/dna.png", package = "RnBeads", mustWork = TRUE), dir.images)
	logger.info(c("Initialized report index and saved to", fname))
	rm(fname.base, all.files, report.dirs, temp.dir, index.dirs)

	stext <- ifelse(is.null(current.report), "", "or scheduled ")
	stext <- c("The following listing contains links to all reports generated ", stext, "by RnBeads. ",
		"A short description of each report is also provided.")
	report <- rnb.add.section(report, "Table of Contents", stext, collapsed = "never")
	if (!is.null(log.file)) {
		stext <- c("The log file <a href=\"", log.file, "\">", log.file, "</a> presents a detailed account of all ",
			"performed activities.")
		rnb.add.paragraph(report, stext)
	}

	## Build table of contents
	rfiles.included <- character()
	stext <- c("<table class=\"pipeline\" style=\"background-image:url(index_images/dna.png);\">", "<colgroup>",
		"\t<col width=\"650\" />", "</colgroup>")
	for (rname in rownames(tbl)) {
		if (identical(rname, current.report)) {
			mod.open <- "<div class=\"running\" title=\"Analysis is currently in progress\">"
			mod.close <- "</div>"
		} else if (rname %in% report.names) {
			rfile <- report.files[which(report.names == rname)]
			if (length(rfile) != 1) {
				msg <- paste(rfile, collapse = ", ")
				logger.warning(paste0("Multiple files found that depict one RnBeads module report: ", msg,
						". Only the first file is linked in the index"))
				rfile <- rfile[1]
			}
			rfiles.included <- c(rfiles.included, rfile)
			mod.open <- paste("<a href=", rfile, "><div class=", "completed", ">", sep = "\"")
			mod.close <- "</div></a>"
		} else {
			mod.open <- "<div class=\"scheduled\" title=\"Module is scheduled to run\">"
			mod.close <- "</div>"
		}
		stext <- c(stext, "<tr>")
		stext <- c(stext, paste0("\t<td>", mod.open))
		stext <- c(stext, paste0("\t\t<h3>", tbl[rname, "Title"], "</h3>"))
		stext <- c(stext, paste0("\t\t<p>", tbl[rname, "Description"], "</p>"))
		stext <- c(stext, paste0("\t", mod.close, "</td>"))
		stext <- c(stext, "</tr>")
	}
	stext <- c(stext, "</table>")

	write.line(paste(stext, collapse = "\n"), report@fname)
	report <- complete.report(report, "index")
	if (open.index) {
		rnb.show.report(report)
	}
	invisible(rfiles.included)
}

########################################################################################################################

#' rnb.build.index
#'
#' Creates an HTML index file that contains listing of all available \pkg{RnBeads} reports. If no known reports are
#' found in the specified directory, no index is created.
#'
#' @param dir.reports       Directory that contains HTML reports generated by \pkg{RnBeads} modules. If this directory
#'                          does not exist, is a regular file, is inaccessible, or does not contain any recognizable
#'                          HTML report files, this function does not generate an HTML index file and produces an error
#'                          or a warning message.
#' @param fname             One-element \code{character} vector specifying the name of the index file to be generated.
#'                          See the \emph{Details} section for restrictions on the name. The file will be created in
#'                          \code{dir.reports}. If such a file already exists, it will be overwritten.
#' @param dir.configuration Subdirectory that hosts configuration files shared by the reports. This must be a
#'                          \code{character} vector of length one that gives location as a path relative to
#'                          \code{dir.reports}. Strong restrictions apply to the path name. See the description of the
#'                          \code{\link{createReport}} function for more details.
#' @param open.index        Flag indicating if the index should be displayed after it is created. If this is
#'                          \code{TRUE}, \code{\link{rnb.show.report}} is called to open the generated HTML file.
#' @return Names of all HTML report files that were referenced in the newly generated index, invisibly. The order of the
#'         file names is the same as the one they are listed in the index. If no known reports are found in the given
#'         directory, the returned value is an empty \code{character} vector.
#'
#' @details
#' In order to ensure independence of the operating system, there are strong restrictions on the name of the index file.
#' It can consist of the following symbols only: Latin letters, digits, dot (\code{.}), dash (\code{-}) and underline
#' (\code{_}). The extension of the file must be one of \code{htm}, \code{html}, \code{xhtml} or \code{xml}. The name
#' must not include paths, that is, slash (\code{/}) or backslash (\code{\\}) cannot be used. In addition, it cannot be
#' any of the recognized \pkg{RnBeads} report file names.
#'
#' @seealso \code{\link{rnb.run.analysis}}, \code{\link{rnb.initialize.reports}}
#' @author Yassen Assenov
#' @export
rnb.build.index <- function(dir.reports, fname = "index.html", dir.configuration = "configuration", open.index = TRUE) {
	if (!(is.character(dir.reports) && length(dir.reports) == 1 && (!is.na(dir.reports[1])))) {
		stop("invalid value for dir.reports")
	}
	if (!(is.character(fname) && length(fname) == 1 && (!is.na(fname[1])))) {
		stop("invalid value for fname")
	}
	if (!grepl("^[a-z0-9._-]+\\.(htm|html|xhtml|xml)$", tolower(fname))) {
		return("invalid value for fname")
	}
	if (!(is.character(dir.configuration) && length(dir.configuration) == 1 && (!is.na(dir.configuration)))) {
		stop("invalid value for dir.configuration")
	}
	if (!is.valid.relative(dir.configuration)) {
		stop("invalid value for dir.configuration")
	}
	if (rnb.getOption("logging") && logger.isinitialized() == FALSE) {
		logger.start(fname = NA) # initialize console logger
	}

	invisible(rnb.build.index.internal(dir.reports, fname, dir.configuration, open.index = open.index))
}

########################################################################################################################

#' rnb.validate.xml.option
#'
#' Checks if the specified XML node is a valid definition for an option.
#'
#' @param x XML node to test. This must be an object or type \code{XMLNodeList}.
#' @return \code{TRUE} if this node contains a single text node as a child; \code{FALSE} otherwise.
#'
#' @author Yassen Assenov
#' @noRd
rnb.validate.xml.option <- function(x) {
	if (length(XML::xmlChildren(x)) == 0) {
		return(TRUE)
	}
	set.null <- XML::xmlAttrs(x)["null"]
	if (!(is.null(set.null) || is.na(set.null))) {
		return(length(XML::xmlChildren(x)) == 0)
	}
	return(length(XML::xmlChildren(x)) == 1 && inherits(XML::xmlChildren(x)[[1]], "XMLTextNode"))
}

########################################################################################################################

#' rnb.xml2options
#'
#' Parses and partially validates parameters and RnBeads options from an XML tree.
#'
#' @param fname File name containing the XML analysis option values. The name of the root node in this document must be
#'              \code{"rnb.xml"}.
#' @param return.full.structure if enabled, return the full structure instead of just the option list
#' @return List of two sublists - \code{"analysis.params"} and \code{"options"}, storing the specified analysis
#'         parameters and previous values of the RnBeads options, respectively.
#'
#' @author Yassen Assenov
#' @examples
#' \donttest{
#' fname <- paste0("extdata/optionProfiles/",profile,".xml")
#' rnb.xml2options(system.file(fname,package="RnBeads"))
#' }
#' @export
rnb.xml2options <- function(fname,return.full.structure=FALSE) {

	rnb.require("XML")

	if (inherits(fname, "XMLNode")) {
		xml.tree <- fname
		analysis.params.required <- TRUE
	} else {
		if (!(is.character(fname) && length(fname) == 1 && isTRUE(fname != ""))) {
			stop("invalid value for fname")
		}
		xml.tree <- tryCatch(XML::xmlRoot(XML::xmlTreeParse(fname)), error = function(e) { return(e) })
		if (inherits(xml.tree, "error")) {
			stop(xml.tree$message)
		}
		analysis.params.required <- return.full.structure
	}

	if (!identical(XML::xmlName(xml.tree), "rnb.xml")) {
		stop("invalid XML format; expected root node rnb.xml")
	}
	xml.options <- XML::xmlChildren(xml.tree)
	if (!all(sapply(xml.options, rnb.validate.xml.option))) {
		stop("invalid XML format; expected text definitions for options")
	}
	result <- list(analysis.params = list(), options = list())
	i.data.source <- which(names(xml.options) == "data.source")
	if (analysis.params.required && length(i.data.source) == 0) {
		stop("invalid XML format; missing data.source")
	}
	i.data.type <- which(names(xml.options) == "data.type")
	i.dir.reports <- which(names(xml.options) == "dir.reports")
	if (analysis.params.required && length(i.dir.reports) == 0) {
		stop("invalid XML format; missing dir.reports")
	}
	i.save.rdata <- which(names(xml.options) == "save.rdata")
	i <- anyDuplicated(names(xml.options))
	if (i != 0) {
		stop(paste("invalid XML format; duplicated", names(xml.options)[i]))
	}
	option.values <- lapply(xml.options, XML::xmlValue)
	option.attrs <- lapply(xml.options, XML::xmlAttrs)
	i.null <- sapply(xml.options, function(x) {
			set.null <- XML::xmlAttrs(x)["null"]; !(is.null(set.null) || is.na(set.null))
		}
	)
	for (i in which(i.null)) {
		option.values[i] <- list(NULL)
	}

	## Extract parameters for the analysis function
	i <- integer() # indices of elements in option.values to be removed
	if (length(i.data.source) != 0) {
		val <- option.values[[i.data.source]]
		val <- strsplit(val, ",", fixed = TRUE)[[1]]
		result[["analysis.params"]][["data.source"]] <- val
		i <- c(i, i.data.source)
	}
	if (length(i.dir.reports) != 0) {
		result[["analysis.params"]][["dir.reports"]] <- option.values[[i.dir.reports]]
		i <- c(i, i.dir.reports)
	}
	if (length(i.data.type) != 0) {
		result[["analysis.params"]][["data.type"]] <- option.values[[i.data.type]]
		i <- c(i, i.data.type)
	}
	if (length(i.save.rdata) != 0) {
		result[["analysis.params"]][["data.type"]] <- tolower(option.values[[i.save.rdata]]) == "true"
		i <- c(i, i.save.rdata)
	}

	## Extract specified file with R preanalysis script, e.g. setting plotting themes, user defined regions, etc.
	i.preanalysis.script <- which(names(xml.options) == "preanalysis.script")
	if (length(i.preanalysis.script) != 0) {
		result[["preanalysis.script"]] <- option.values[[i.preanalysis.script]]
		i <- c(i, i.preanalysis.script)
	}
	if (length(i) != 0){
		option.values <- option.values[-i]
	}

	## Set RnBeads options
	convert.option.value <- function(otype, ovalue) {
		if (grepl("\\.vector$", otype)) {
			if (length(ovalue) != 0) {
				ovalue <- strsplit(ovalue, ",", fixed = TRUE)[[1]]
			}
			otype <- sub("\\.vector$", "", otype)
		}
		if (!grepl("^character", otype)) {
			do.convert <- get(paste0("as.", otype))
			result <- tryCatch(do.convert(ovalue), warning = function(w) { NULL }, error = function(e) { NULL })
			if (!is.null(result)) {
				return(result)
			}
		}
		return(ovalue)
	}
	get.names.from.attrs <- function(oattr){
		res <- NULL
		nns <- oattr["names"]
		if (length(nns) > 0){
			res <- unname(strsplit(nns, ",", fixed = TRUE)[[1]])
		}
		return(res)
	}
	for (oname in names(option.values)) {
		ovalue <- option.values[[oname]]
		if (!is.null(ovalue)) {
			#TODO: the following is a bit of a hack, but I could not do it otherwise
			if (oname=="import.table.separator"){
				ovalue <- gsub("\\\\t","\\\t",ovalue)
			}
			if (rnb.is.option(oname)) {
				option.values[[oname]] <- convert.option.value(.rnb.options[["infos"]][oname, "Type"], ovalue)
				if (!is.null(option.values[[oname]])){
					opt.names <- get.names.from.attrs(option.attrs[[oname]])
					if (length(opt.names)==length(option.values[[oname]])){
						names(option.values[[oname]]) <- opt.names
					}
				}
			}
		}
	}
	## Set the new option values
	r.options <- list()
	if (length(option.values) != 0) {
		r.options <- do.call(rnb.options, option.values)
	}
	if (!analysis.params.required) {
		return(r.options)
	}
	result[["options"]] <- option.values
	result
}

########################################################################################################################

#' rnb.run.xml
#'
#' Starts the analysis pipeline from an XML configuration file. This function uses the \pkg{XML} package to parse the
#' configuration file.
#'
#' @param fname            XML configuration file to read.
#' @param create.r.command Flag indicating if the R command(s) that correspond to the given XML configuration should be
#'                         generated. If this is set to \code{TRUE}, a file named \code{"analysis.R"} is created in the
#'                         reports directory.
#' @return Invisibly, the loaded, normalized and/or possibly filtered dataset as an object of type inheriting
#'         \code{\linkS4class{RnBSet}}.
#'
#' @details
#' Two values are required to be specified (as tags) in the configuration file - \code{data.source} and
#' \code{dir.reports}. They define the input and output directory, respectively. In addition, the file may define
#' analysis option values. The vignette \emph{Comprehensive DNA Methylation Analysis with RnBeads} describes in details
#' the syntax of the XML configuration file.
#'
#' The sample annotation table must be stored as a file in \code{data.source}. For more information about the required
#' parameters, see the documentation of \code{\link{rnb.run.analysis}}, which is called by this function.
#'
#' @seealso \code{\link{rnb.run.analysis}} for starting an analysis pipeline
#' @author Yassen Assenov
#' @export
rnb.run.xml <- function(fname, create.r.command = FALSE) {

	if (!(is.character(fname) && length(fname) == 1 && (!is.na(fname)))) {
		stop("invalid value for fname")
	}
	if (!parameter.is.flag(create.r.command)) {
		stop("invalid value for create.r.command; expected TRUE or FALSE")
	}
	rnb.require("XML")

	## Open and validate the structure of the XML file
	xml.tree <- tryCatch(XML::xmlRoot(XML::xmlTreeParse(fname)), error = function(e) { return(e) })
	if (inherits(xml.tree, "error")) {
		stop(xml.tree$message)
	}
	parsed <- rnb.xml2options(xml.tree)

	## Run preanalysis script
	if ("preanalysis.script" %in% names(parsed)) {
		if (file.exists(parsed$preanalysis.script)) {
			source(parsed$preanalysis.script, echo = TRUE, chdir = TRUE)
		}
	}

	## Start analysis
	result <- do.call(rnb.run.analysis, parsed$analysis.params)

	## Create analysis.R file
	if (create.r.command) {
		fname.analysis <- file.path(parsed$analysis.paramsdir.reports, "analysis.R")
		option.value2text <- function(ovalue) {
			if (is.null(ovalue)) {
				return("null")
			}
			if (length(ovalue) == 0) {
				return(paste0(typeof(ovalue), "()"))
			}
			quote.char <- ifelse(is.character(ovalue), '"', "")
			result <- paste0(quote.char, ovalue, quote.char, collapse = ", ")
			if (length(ovalue) != 1) {
				result <- paste0("c(", result, ")")
			}
			result
		}
		toappend <- FALSE
		if ("preanalysis.script" %in% names(parsed)) {
			txt <- parsed$preanalysis.script
			cat(ifelse(file.exists(txt), '', '# '), 'source("', txt, '")\n\n', file = fname.analysis, sep = "")
			toappend <- TRUE
		}
		if (length(parsed$options) != 0) {
			txt <- paste0("\t", names(parsed$options), " = ", sapply(parsed$options, option.value2text))
			cat("rnb.options(", paste(txt, collapse = "\n"), ")\n\n", file = fname.analysis, sep = "\n",
				append = toappend)
			toappend <- TRUE
		}
		txt <- paste0(names(parsed$analysis.params), ' = "', unlist(parsed$analysis.params), '"', collapse = ", ")
		cat("rnb.run.analysis(", txt, ")\n", file = fname.analysis, sep = "", append = toappend)
		rm(fname.analysis, option.value2text, txt)
	}

	invisible(result)
}

########################################################################################################################

#' rnb.run.dj
#'
#' Starts the RnBeads Data Juggler (RnBeadsDJ) for configuring and running RnBeads analyses from the web browser
#'
#' @return Nothing of particular interest
#'
#' @details
#' A Shiny app is launched in the web browser
#'
#' @seealso \code{\link{rnb.run.analysis}} for starting an analysis pipeline
#' @author Fabian Mueller
#' @export
rnb.run.dj <- function(){
	shiny::runApp(system.file("extdata/RnBeadsDJ", package = "RnBeads"))
	invisible(NULL)
}


########################################################################################################################

#' RnBeads Analysis Pipeline
#'
#' Starts the \pkg{RnBeads} analysis pipeline on the given dataset. It loads the dataset if it is specified as a
#' location.
#' @param dir.reports        Directory to host the generated report files. This must be a \code{character} of length one
#'                           that specifies either a non-existent path (when \code{initialize.reports} is \code{TRUE}),
#'                           or an existing directory (when \code{initialize.reports} is \code{FALSE}). In the latter
#'                           case, a call to \code{\link{rnb.initialize.reports}} might be required before viewing the
#'                           reports.
#' @param data.source        Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}, or a
#'                           \code{character} vector specifying the location of the data items on disk. The expected
#'                           length of the vector differs for different values of \code{data.type};
#'                           see \code{\link{rnb.execute.import}} for a more detailed description. If set, the parameters
#' 							 \code{sample.sheet}, \code{data.dir}, \code{GS.report}, \code{GEO.acc} will be ignored.
#' @param sample.sheet		 A spreadsheet-like text file with sample annotations. The required columns are different
#' 							 for different values of \code{data.type}.
#' @param data.dir			 For \code{data.type \%in\% c("data.dir", "idat.dir", "bed.dir")} a character singleton
#' 							 specifying the location of the directory with data files. The directory should have zero
#' 							 depth, i.e. should contain no subdirectories.
#' @param GS.report			 GenomeStudio report file. \code{data.type} will be automatically set to \code{"GS.report"}.
#' @param GEO.acc			 Gene Expression Omnibus accession of the data series with HumanMethylation450 data.
#' 							 \code{data.type} will be automatically set to \code{"GEO"}.
#' @param data.type          \code{character} vector of length one specifying the type of the input data.
#'                           The value must be one of \code{"data.dir"}, \code{"idat.dir"}, \code{"GS.report"},
#'                           \code{"GEO"} or \code{"rnb.set"}. See \code{\link{rnb.execute.import}} for a more detailed
#'                           description.
#' @param initialize.reports Flag indicating if the report's directory must be initialized. If this parameter is set to
#'                           \code{TRUE}, this function attempts to create the path specified by \code{dir.reports}.
#'                           Otherwise, \code{dir.reports} is expected to signify an existing directory.
#' @param build.index        Flag indicating if a report index file (named \code{"index.html"}) should be created after
#'                           all modules in the pipeline complete their analyses. If this is \code{TRUE}, the index file
#'                           is also displayed using the function \code{\link{rnb.show.report}}.
#' @param save.rdata		 Flag indicating whether important data objects (the filtered and unfiltered RnBSets,
#' 							 differential methylation) should be saved to an RData file in the reports folder.
#' @return Invisibly, the loaded, normalized and/or possibly filtered dataset as an object of type inheriting
#'         \code{\linkS4class{RnBSet}}.
#'
#' @seealso \link[=rnb.run.import]{RnBeads modules}
#' @author Yassen Assenov
#' @export
rnb.run.analysis <- function(dir.reports, data.source=NULL, sample.sheet=NULL, data.dir=NULL, GS.report=NULL, GEO.acc=NULL,
		data.type = rnb.getOption("import.default.data.type"),
	initialize.reports = TRUE, build.index = TRUE, save.rdata = TRUE) {

	if(all(is.null(c(sample.sheet, data.dir, GS.report, GEO.acc, data.source)))){
		stop("one of the sample.sheet, data.dir, GS.report, GEO.acc, data.source should be specified")
	}

	if(is.null(data.source)){

		if(!is.null(GS.report)){

			data.type<-"GS.report"
			data.source<-GS.report

		}else if(!is.null(GEO.acc)){

			data.type<-"GEO"
			data.source<-GEO.acc

		}else if(!is.null(sample.sheet) & !is.null(data.dir)){

			data.source<-list(data.dir=data.dir, sample.sheet=sample.sheet)

		}else if(!is.null(sample.sheet) & is.null(data.dir)){

			stop("data directory is missing")

		}else if(is.null(sample.sheet) & !is.null(data.dir)){

			stop("sample sheet is missing")

		}
	}

	if (!(is.character(dir.reports) && length(dir.reports) == 1 && (!is.na(dir.reports)))) {
		stop("invalid value for dir.reports; expected one-element character")
	}
	if (!parameter.is.flag(initialize.reports)) {
		stop("invalid value for initialize.reports; expected TRUE or FALSE")
	}
	if (!parameter.is.flag(build.index)) {
		stop("invalid value for build.index; expected TRUE or FALSE")
	}

	## Initialize the reports location and the log
	if (initialize.reports) {
		if (!rnb.initialize.reports(dir.reports)) {
			stop(paste("Could not initialize reports in", dir.reports, "; make sure this path does not exist."))
		}
	} else if (!isTRUE(file.info(dir.reports)[1, "isdir"])) {
		stop("invalid value for dir.reports; expected existing directory")
	}
	if (logger.isinitialized()) {
		logfile <- NULL
		log.file <- NULL
	} else {
		log.file <- "analysis.log"
		logfile <- c(file.path(dir.reports, log.file), NA)
	}
	logger.start("RnBeads Pipeline", fname = logfile)
	aname <- rnb.getOption("analysis.name")
	if (!(is.null(aname) || is.na(aname) || nchar(aname) == 0)) {
		logger.info(c("Analysis Title:", aname))
	}
	rm(aname)

	update.index <- function(dset, rname = "", skip.normalization = FALSE) {
		if (build.index) {
			if (is.null(dset)) {
				export.enabled <- TRUE
			} else {
				export.enabled <- rnb.getOption("export.to.csv") || rnb.tracks.to.export(dset)
			}
			rnb.build.index.internal(dir.reports, log.file = log.file, export.enabled = export.enabled,
				current.report = rname, open.index = (rname == ""))
		}
	}

	## Save the analysis options to an XML file
	cat(rnb.options2xml(), file = file.path(dir.reports, "analysis_options.xml"))

	## Loading
	if (rnb.getOption("import")) {
		if (is.character(data.source) || is.list(data.source) || inherits(data.source, "RnBSet")) {
			update.index(NULL, "data_import", data.type == "bed.dir")
			result <- rnb.run.import(data.source, data.type, dir.reports)
			rnb.set <- result$rnb.set
			rm(result); rnb.cleanMem()
		} else {
			stop("invalid value for data.source")
		}
	} else if (inherits(data.source, "RnBSet")) {
		rnb.set <- data.source
	} else if (inherits(data.source, "MethyLumiSet")) {
		rnb.set <- as(data.source, "RnBeadSet")
	} else {
		logger.warning("Cannot proceed with the supplied data.source. Check the option import")
		logger.completed()
		if (!is.null(logfile)) {
			logger.close()
		}
		return(invisible(NULL))
	}

	if (save.rdata){
		analysis.options <- rnb.options()
		# if(!is.null(rnb.set@status) && rnb.set@status$disk.dump){
		# 	save.matrices(rnb.set,
		# 			path=file.path(dir.reports, "rnbSet_unfiltered_ffmatrices"))
		# }
		save.rnb.set(rnb.set,file.path(dir.reports, "rnbSet_unnormalized"),
				archive=rnb.getOption("gz.large.files"))
		save(analysis.options,
				file = file.path(dir.reports, "analysis_options.RData"))
	}

	## Quality control
	if (rnb.getOption("qc")) {
		update.index(rnb.set, "quality_control")
		rnb.cleanMem()
		rnb.run.qc(rnb.set, dir.reports)
	}

	## Preprocessing
	if (rnb.getOption("preprocessing")) {
		update.index(rnb.set, "preprocessing")
		rnb.cleanMem()
		result <- rnb.run.preprocessing(rnb.set, dir.reports)
		rnb.set <- result$rnb.set
		rm(result); rnb.cleanMem()

		if (save.rdata){
			# if(!is.null(rnb.set@status) && rnb.set@status$disk.dump){
			# 	save.matrices(rnb.set,
			# 			path=file.path(dir.reports, "rnbSet_filtered_ffmatrices"))
			# }
			save.rnb.set(rnb.set,file.path(dir.reports, "rnbSet_preprocessed"),
					archive=rnb.getOption("gz.large.files"))
			# save(analysis.options,
			# 		file = file.path(dir.reports, "analysis_options.RData"))
		}
	}

	sample.count <- nrow(pheno(rnb.set))

	if (nsites(rnb.set) * sample.count != 0) {
		## Data export
		if (rnb.getOption("export.to.csv") || rnb.tracks.to.export(rnb.set)) {
			update.index(rnb.set, "tracks_and_tables")
			rnb.cleanMem()
			rnb.run.tnt(rnb.set, dir.reports)
		}

		## Annotation inference
		if (rnb.getOption("inference")) {
			update.index(rnb.set, "covariate_inference")
			rnb.set <- rnb.run.inference(rnb.set, dir.reports)$rnb.set
			if (save.rdata){
				save.rnb.set(rnb.set,file.path(dir.reports, "rnbSet_inference"),
					 archive=rnb.getOption("gz.large.files"))
			}
		}

		## Exploratory analysis
		if (rnb.getOption("exploratory")) {
			update.index(rnb.set, "exploratory_analysis")
			rnb.cleanMem()
			rnb.run.exploratory(rnb.set, dir.reports)
		}

		## Differential methylation
		if (rnb.getOption("differential")) {
			update.index(rnb.set, "differential_methylation")
			rnb.cleanMem()
			result.diffmeth <- rnb.run.differential(rnb.set, dir.reports)
		}
	}

	update.index(rnb.set)

	if (save.rdata){
		rnb.cleanMem()
		logger.start("Saving RData")
		# analysis.options <- rnb.options()
		if (exists("result.diffmeth")) {
			if (!is.null(result.diffmeth) && !is.null(result.diffmeth$diffmeth)){
				diffmeth.path <- file.path(dir.reports, "differential_rnbDiffMeth")
				save.rnb.diffmeth(result.diffmeth$diffmeth, diffmeth.path)
				diffmeth.go.enrichment <- result.diffmeth$dm.go.enrich
				if (!is.null(diffmeth.go.enrichment)){
					save(diffmeth.go.enrichment, file=file.path(diffmeth.path, "enrichment_go.RData"))
				}
				diffmeth.lola.enrichment <- result.diffmeth$dm.lola.enrich
				if (!is.null(diffmeth.lola.enrichment)){
					save(diffmeth.lola.enrichment, file=file.path(diffmeth.path, "enrichment_lola.RData"))
				}
			} else {
				logger.warning("Differential methylation object not saved")
			}
		}
		logger.completed()
	}
	rnb.cleanMem()
	logger.completed()
	if (!is.null(logfile)) {
		logger.close()
	}
	invisible(rnb.set)
}

########################################################################################################################

## validate.module.parameters
##
## Validates the values of parameters given to a module function.
validate.module.parameters <- function(rnb.set, dir.reports, close.report, show.report, methylumi.accepted = FALSE) {
	if (!inherits(rnb.set, "RnBSet")) {
		if (!(methylumi.accepted && inherits(rnb.set, "MethyLumiSet"))) {
			stop("invalid value for rnb.set")
		}
	}
	if (!(is.character(dir.reports) && length(dir.reports) == 1 && (!is.na(dir.reports)))) {
		stop("invalid value for dir.reports")
	}
	if (!parameter.is.flag(close.report)) {
		stop("invalid value for close.report; expected TRUE or FALSE")
	}
	if (!parameter.is.flag(show.report)) {
		stop("invalid value for show.report; expected TRUE or FALSE")
	}
}

########################################################################################################################

## Starts a section in the logger dedicated to a module. Also adds a log message on the number of cores.
##
## @param mtitle Module name to be set as a section title.
## @author Yassen Assenov
module.start.log <- function(mtitle) {
	if (rnb.getOption("logging") && logger.isinitialized() == FALSE) {
		logger.start(fname = NA) # initialize console logger
	}
	logger.start(mtitle)
	ncores <- parallel.getNumWorkers()
	if (ncores == -1) {
		ncores <- 1L
	}
	logger.info(c("Number of cores:", ncores))
}

########################################################################################################################

## init.pipeline.report
##
## Initializes a report that is part of the default analysis pipeline.
##
## @param fname              Desired file name (without extension) of the HTML report.
## @param dir.reports        Directory that will contain the generated report.
## @param init.configuration Flag indicating if the configuration directory should be created.
## @return Newly initialized report.
## @author Yassen Assenov
init.pipeline.report <- function(fname, dir.reports, init.configuration) {
	tbl <- system.file("extdata/reportDescriptions.txt", package = "RnBeads")
	tbl <- read.delim(tbl, quote = "", row.names = 1, stringsAsFactors = FALSE)

	ptitle <- rnb.getOption("analysis.name")
	ptitle <- ifelse(is.null(ptitle), "RnBeads report", paste("RnBeads:", ptitle))
	createReport(file.path(dir.reports, paste0(fname, ".html")), tbl[fname, "Title"], ptitle,
		init.configuration = init.configuration)
}

########################################################################################################################

## Completes a module by closing its section in the log and (optionally) its report.
##
## @param report       Report generated by the module.
## @param close.report Flag indicating if the report is to be closed.
## @param show.report  Flag indicating if the report is to be open in a browser.
## @author Yassen Assenov
module.complete <- function(report, close.report, show.report) {
	if (close.report) {
		off(report)
	}
	logger.completed()
	if (show.report) {
		rnb.show.report(report)
	}
}

########################################################################################################################

#' RnBeads Modules in the Analysis Pipeline
#'
#' Functions that start the predefined modules in the \pkg{RnBeads} analysis pipeline.
#'
#' @rdname rnb.runs
#' @aliases rnb.run.import
#' @aliases rnb.run.qc
#' @aliases rnb.run.preprocessing
#' @aliases rnb.run.inference
#' @aliases rnb.run.tnt
#' @aliases rnb.run.exploratory
#' @aliases rnb.run.differential
#'
#' @param data.source        \code{character} vector specifying the location of the data items on disk. The expected
#'                           length of the vector differs for different values of \code{data.type}; see
#'                           \code{\link{rnb.execute.import}} for a more detailed description.
#' @param data.type          \code{character} vector of length one specifying the type of the input data. The value of
#'                           this parameter must be one of \code{"idat.dir"}, \code{"data.dir"}, \code{"data.files"},
#'                           \code{"GS.report"}, \code{"GEO"} or \code{"rnb.set"}. See \code{\link{rnb.execute.import}}
#'                           for a more detailed description.
#' @param rnb.set            Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param dir.reports        Directory to host the generated report file. Note that if this directory contains files,
#'                           they may be overwritten.
#' @param init.configuration Flag indicating if the configuration directory (usually shared among reports) should also
#'                           be created.
#' @param close.report       Flag indicating if the created report is to be closed using the
#'                           \code{\link[=off,Report-method]{off}} method.
#' @param show.report        Flag indicating if the report is to be displayed after it is created. If this is,
#'                           \code{TRUE} \code{\link{rnb.show.report}} is called to open the generated HTML file.
#' @return For \code{rnb.run.import}, \code{rnb.run.preprocessing} and \code{rnb.run.inference}, the returned value is
#'         a list of two elements - the initialized or modified dataset and the created report. All other functions
#'         return the created report, invisibly.
#'
#' @details The functions start the import, quality control, preprocessing, covariate inference, tracks and tables,
#'          exploratory analysis and differential methylation modules, respectively.
#'
#' @examples
#' \donttest{
#' ### Running the modules step by step
#'
#' # Directory where your data is located
#' data.dir <- "~/RnBeads/data/Ziller2011_PLoSGen_450K"
#' idat.dir <- file.path(data.dir, "idat")
#' sample.annotation <- file.path(data.dir, "sample_annotation.csv")
#'
#' # Directory where the output should be written to
#' analysis.dir <- "~/RnBeads/analysis"
#' # Directory where the report files should be written to
#' report.dir <- file.path(analysis.dir, "reports_details")
#' rnb.initialize.reports(report.dir)
#' # Set some analysis options
#' rnb.options(filtering.sex.chromosomes.removal = TRUE, identifiers.column = "Sample_ID")
#' ## Restrict logging to the console only
#' logger.start(fname = NA)
#'
#' ## Data import
#' data.source <- c(idat.dir, sample.annotation)
#' result <- rnb.run.import(data.source=data.source, data.type="idat.dir", dir.reports=report.dir)
#' rnb.set <- result$rnb.set
#'
#' ## Quality Control
#' rnb.run.qc(rnb.set, report.dir)
#'
#' ## Preprocessing
#' rnb.set <- rnb.run.preprocessing(rnb.set, dir.reports=report.dir)$rnb.set
#'
#' ## Data export
#' rnb.options(export.to.csv = TRUE)
#' rnb.run.tnt(rnb.set, report.dir)
#'
#' ## Exploratory analysis
#' rnb.run.exploratory(rnb.set, report.dir)
#'
#' ## Differential methylation
#' rnb.run.differential(rnb.set, report.dir)
#' }
#'
#' @seealso \code{\link{rnb.run.analysis}} which executes these modules in the order given above
#' @author Yassen Assenov
#' @export
rnb.run.import <- function(data.source, data.type = rnb.getOption("import.default.data.type"), dir.reports,
		init.configuration = !file.exists(file.path(dir.reports, "configuration")), close.report = TRUE,
		show.report = FALSE) {

	## TODO: Validate parameters data.source and data.type
	if (!parameter.is.flag(close.report)) {
		stop("invalid value for close.report; expected TRUE or FALSE")
	}
	if (!parameter.is.flag(show.report)) {
		stop("invalid value for show.report; expected TRUE or FALSE")
	}
	module.start.log("Loading Data")

	report <- init.pipeline.report("data_import", dir.reports, init.configuration)
	optionlist <- rnb.options("import.default.data.type", "import.table.separator", "import.bed.style",
		"import.bed.columns", "import.bed.frame.shift")
	report <- rnb.add.optionlist(report, optionlist)

	result <- rnb.step.import(data.source, data.type, report)

	module.complete(result$report, close.report, show.report)
	return(result)
}

########################################################################################################################

#' @rdname rnb.runs
#' @export
rnb.run.qc <- function(rnb.set, dir.reports, init.configuration = !file.exists(file.path(dir.reports, "configuration")),
	close.report = TRUE, show.report = FALSE) {
	validate.module.parameters(rnb.set, dir.reports, close.report, show.report)
	module.start.log("Quality Control")

	report <- init.pipeline.report("quality_control", dir.reports, init.configuration)
	optionlist <- rnb.options("qc.boxplots", "qc.barplots", "qc.negative.boxplot")
	if (inherits(rnb.set, "RnBeadSet")) {
		snp.options <- list("qc.snp.heatmap", "qc.snp.barplot", "qc.snp.boxplot", "qc.snp.distances", "qc.snp.purity","qc.cnv","qc.cnv.refbased")
		snp.options <- do.call(rnb.options, snp.options)
		optionlist <- c(optionlist, snp.options)
		snp.options <- any(unlist(snp.options, use.names = FALSE))
	} else {
		snp.options <- FALSE
	}
	report <- rnb.add.optionlist(report, optionlist)

	report <- rnb.step.quality(rnb.set, report)
	if (snp.options) {
		report <- rnb.step.snp.probes(rnb.set, report)
	}
	if (.hasSlot(rnb.set, "inferred.covariates") && isTRUE(rnb.set@inferred.covariates$sex)) {
		if (inherits(rnb.set, "RnBeadRawSet")) {
			signal.increases <- rnb.get.XY.shifts(rnb.set)
		} else if (inherits(rnb.set, "RnBiseqSet")) {
			signal.increases <- rnb.get.XY.shifts.biseq(rnb.set)
		}
		report <- rnb.section.sex.prediction(rnb.set, signal.increases, report)
	}
	if(rnb.getOption("qc.cnv")){
	  if(inherits(rnb.set,"RnBeadRawSet")){
	   report <- rnb.step.cnv(rnb.set,report)
	  }else{
	    logger.info("CNV estimation only applicable for RnBeadRawSet objects")
	    txt <- "CNV estimation can only be performed for Illumina BeadChip data sets with signal intensity values available (RnBeadRawSet)"
	    report <- rnb.add.section(report,"Copy number variation analysis",description = txt)
	  }
	}
	module.complete(report, close.report, show.report)
	invisible(report)
}

########################################################################################################################

#' @rdname rnb.runs
#' @export
rnb.run.preprocessing <- function(rnb.set, dir.reports,
	init.configuration = !file.exists(file.path(dir.reports, "configuration")), close.report = TRUE,
	show.report = FALSE) {

	validate.module.parameters(rnb.set, dir.reports, close.report, show.report)
	module.start.log("Preprocessing")

	## Decide if a normalization procedure is to be executed
	do.normalization <- rnb.getOption("normalization")
	if (is.null(do.normalization)) {
		do.normalization <- inherits(rnb.set, "RnBeadSet")
	} else if (do.normalization && inherits(rnb.set, "RnBiseqSet")) {
		logger.warning("Skipped normalization module for sequencing data.")
		do.normalization <- FALSE
	}

	## Decide if Greedycut is to be executed
	do.greedycut <- rnb.getOption("filtering.greedycut")
	if (is.null(do.greedycut)) {
		do.greedycut <- inherits(rnb.set, "RnBeadSet")
	} else {
		if (!inherits(rnb.set, "RnBeadSet")){
			logger.warning("filtering.greedycut disabled for non-array datasets.")
			do.greedycut <- FALSE
		}
	}

	## Option list
	report <- init.pipeline.report("preprocessing", dir.reports, init.configuration)
	x.greedycut <- ifelse(inherits(rnb.set, "RnBeadSet"), "filtering.greedycut.pvalue.threshold",
		"filtering.coverage.threshold")
	optionlist <- rnb.options("filtering.whitelist", "filtering.blacklist", "filtering.snp", "filtering.cross.reactive",
		"filtering.greedycut", x.greedycut, "filtering.greedycut.rc.ties","imputation.method")
	attr.vec <- c(TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, do.greedycut, TRUE)
	if (!inherits(rnb.set, "RnBeadSet")) {
		optionlist <- optionlist[-4]
		attr.vec <- attr.vec[-4]
	}
	if (is.null(optionlist[["filtering.whitelist"]])) {
		optionlist[["filtering.whitelist"]] <- ""
	}
	if (is.null(optionlist[["filtering.blacklist"]])) {
		optionlist[["filtering.blacklist"]] <- ""
	}
	if (inherits(rnb.set, "RnBiseqSet")) {
		optionlist <- c(optionlist, rnb.options("filtering.high.coverage.outliers", "filtering.low.coverage.masking"))
		attr.vec <- c(attr.vec, TRUE, TRUE)
	}
	optionlist <- c(optionlist, rnb.options("normalization.method", "normalization.background.method",
			"normalization.plot.shifts", "filtering.context.removal", "filtering.missing.value.quantile",
			"filtering.sex.chromosomes.removal", "filtering.deviation.threshold", "distribution.subsample"))
	attr.vec <- c(attr.vec, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE)
	attr(optionlist, "enabled") <- attr.vec
	report <- rnb.add.optionlist(report, optionlist)
	rm(optionlist, x.greedycut, attr.vec)

	## Prefiltering
	logger.start(paste0("Filtering Procedures", ifelse(do.normalization, " I", "")))

	## Load whitelist and blacklist
	anno.table <- annotation(rnb.set, add.names = inherits(rnb.set, "RnBeadSet"))
	whitelist <- rnb.process.sitelist(rnb.getOption("filtering.whitelist"), anno.table)
	blacklist <- rnb.process.sitelist(rnb.getOption("filtering.blacklist"), anno.table)
	if (is.null(whitelist) && is.null(blacklist)) {
		whitelist <- integer()
		blacklist <- integer()
	} else {
		## Add a section on whitelisted and/or blacklisted sites
		txt <- character()
		if (is.null(whitelist)) {
			txt <- "A blacklist was"
			tbl <- rnb.sitelist.info(blacklist, "black")
			whitelist <- integer()
		} else if (is.null(blacklist)) {
			txt <- "A whitelist was"
			tbl <- rnb.sitelist.info(whitelist, "white")
			blacklist <- integer()
		} else {
			txt <- blacklist
			blacklist <- setdiff(blacklist, whitelist)
			attr(blacklist, "ignored") <- attr(txt, "ignored") + length(txt) - length(blacklist)
			attr(blacklist, "note") <- attr(txt, "note")
			tbl <- rbind(rnb.sitelist.info(whitelist, "white"), rnb.sitelist.info(blacklist, "black"))
			txt <- "A whitelist and a blacklist were"
		}
		txt <- c(txt, " constructed based on the specified file", ifelse(grepl("were$", txt), "s", ""), ". The table ",
			"below summarizes the number of identified records.")
		report <- rnb.add.section(report, "Whitelist and Blacklist", txt)
		rnb.add.table(report, tbl, FALSE)
		txt <- "A record in a site list file is ignored when one of the following conditions are met:"
		rnb.add.paragraph(report, txt)
		tbl <- inherits(rnb.set, "RnBeadSet")
		txt <- list(
			paste0("it does not define a valid genomic position", ifelse(tbl, " or probe identifier", ""), ";"),
			paste0("the position ", ifelse(tbl, "or probe ", ""), "it defines is not covered by the analyzed dataset;"),
			"it is a duplicate of another record within the same list;",
			"it is a record in the blacklist which is also present in the whitelist.")
		rnb.add.list(report, txt)
		txt <- "As described in the last condition above, the whitelist has a precedence over the blacklist."
		rnb.add.paragraph(report, txt)
		rm(txt, tbl)
	}
	removed.sites <- sort(union(blacklist, whitelist))
	removed.samples <- integer()

	if (rnb.getOption("filtering.snp") != "no") {
		result <- rnb.step.snp.removal.internal(class(rnb.set), removed.sites, report, anno.table)
		report <- result$report
		removed.sites <- sort(c(removed.sites, result$filtered))
	}
	if (rnb.getOption("filtering.cross.reactive")) {
		result <- rnb.step.cross.reactive.removal.internal(removed.sites, report, anno.table)
		report <- result$report
		removed.sites <- sort(c(removed.sites, result$filtered))
	}
	if (do.greedycut) {
		result <- rnb.step.greedycut.internal(rnb.set, removed.sites, report, anno.table)
		report <- result$report
		removed.sites <- sort(c(removed.sites, result$sites))
		removed.samples <- result$samples
	}
	if (rnb.getOption("filtering.high.coverage.outliers") && inherits(rnb.set, "RnBiseqSet")) {
		result <- rnb.step.high.coverage.removal.internal(rnb.set, removed.sites, report, anno.table)
		report <- result$report
		removed.sites <- sort(c(removed.sites, result$filtered))
	}
	mask <- NULL
	if (rnb.getOption("filtering.low.coverage.masking")) {
		result <- rnb.step.low.coverage.masking.internal(rnb.set, removed.sites, report, anno.table,
				covg.threshold = rnb.getOption("filtering.coverage.threshold"))
		if (any(result$mask)) {
			mask <- result$mask
		}
		report <- result$report
	}
	suppressWarnings(rm(result))
	rnb.cleanMem()

	logger.completed.filtering <- function(rnb.set, r.samples, r.sites) {
		retained.p <- nsites(rnb.set) - length(r.sites)
		retained.s <- length(samples(rnb.set)) - length(r.samples)
		logger.status(c("Retained", retained.s, "samples and", retained.p, "sites"))
		logger.completed()
	}

	if (do.normalization) {
		## Summary I
		removed.sites <- setdiff(removed.sites, whitelist)
		logger.completed.filtering(rnb.set, removed.samples, removed.sites)

		logger.start("Summary of Filtering Procedures I")
		report <- rnb.step.filter.summary.internal(rnb.set, removed.samples, removed.sites,
				report, section.name="Filtering Summary I")
		logger.completed()

		rnb.set <- rnb.filter.dataset(rnb.set, removed.samples, removed.sites, mask)
		mask <- NULL

		## Normalization
		normalization.result <- rnb.step.normalization(rnb.set, report)
		rnb.set <- normalization.result$dataset
		report <- normalization.result$report
		suppressWarnings(rm(normalization.result))
		rnb.cleanMem()

		logger.start("Filtering Procedures II")
		sn <- " II"
		anno.table <- annotation(rnb.set, add.names = inherits(rnb.set, "RnBeadSet"))
		whitelist <- rnb.process.sitelist(rnb.getOption("filtering.whitelist"), anno.table)
		removed.samples <- integer()
		removed.sites <- whitelist
	} else {
		sn <- ""
	}

	## Postfiltering
	mm <- NULL
	if (length(rnb.getOption("filtering.context.removal")) != 0 && inherits(rnb.set, "RnBeadSet")) {
		result <- rnb.step.context.removal.internal(removed.sites, report, anno.table)
		report <- result$report
		removed.sites <- sort(c(removed.sites, result$filtered))
	}
	if (rnb.getOption("filtering.sex.chromosomes.removal")) {
		result <- rnb.step.sex.removal.internal(class(rnb.set), removed.sites, report, anno.table)
		report <- result$report
		removed.sites <- sort(c(removed.sites, result$filtered))
	}
	ttt <- rnb.getOption("filtering.missing.value.quantile")
	if (ttt != 1) {
		# if (is.null(mm)) mm <- meth(rnb.set)
		result <- rnb.step.na.removal.internal(rnb.set, removed.sites, report, anno.table, ttt, mask)
		report <- result$report
		removed.sites <- sort(c(removed.sites, result$filtered))
	}
	ttt <- rnb.getOption("filtering.deviation.threshold")
	if (ttt > 0) {
		if (is.null(mm)) mm <- meth(rnb.set)
		result <- rnb.step.variability.removal.internal(class(rnb.set), mm, removed.sites, report, anno.table, ttt)
		report <- result$report
		removed.sites <- sort(c(removed.sites, result$filtered))
	}
	suppressWarnings(rm(result, ttt))

	## Summary (II)
	removed.sites <- setdiff(removed.sites, whitelist)
	logger.completed.filtering(rnb.set, removed.samples, removed.sites)
	logger.start(paste0("Summary of Filtering Procedures", sn))
	rnb.cleanMem()
	sn <- paste0("Filtering Summary", sn)
	report <- rnb.step.filter.summary.internal(rnb.set, removed.samples, removed.sites, report, section.name = sn)
	logger.completed()

	rnb.set <- rnb.filter.dataset(rnb.set, removed.samples, removed.sites, mask)

	## Imputation
	if((rnb.getOption("imputation.method")%in%c("none"))){
	  logger.info("Imputation was skipped, data set may still contain missing methylation values")
	}else{
	  imputation.result <- rnb.step.imputation(rnb.set,report)
	  rnb.set <- imputation.result$dataset
	  report <- imputation.result$report
	}
	
	if (rnb.getOption("region.subsegments") > 1L) {
		res <- rnb.step.region.subsegmentation(rnb.set, report, region.types=rnb.getOption("region.subsegments.types"))
		rnb.set <- res$rnb.set
		report <- res$report
	}
	rnb.cleanMem()

	module.complete(report, close.report, show.report)
	return(list(rnb.set = rnb.set, report = report))
}

########################################################################################################################

#' @rdname rnb.runs
#' @export
rnb.run.inference <- function(rnb.set, dir.reports,
	init.configuration = !file.exists(file.path(dir.reports, "configuration")), close.report = TRUE,
	show.report = FALSE) {

	validate.module.parameters(rnb.set, dir.reports, close.report, show.report)
	module.start.log("Covariate Inference")

	report <- init.pipeline.report("covariate_inference", dir.reports, init.configuration)
	optionlist <- rnb.options("inference.genome.methylation", "inference.targets.sva", "inference.sva.num.method",
		"covariate.adjustment.columns", "export.to.ewasher", "inference.age.prediction",
		"inference.age.prediction.training", "inference.age.prediction.predictor", "inference.age.column",
		"inference.age.prediction.cv", "inference.immune.cells")
	if (is.null(optionlist[[1]])) {
		optionlist[[1]] <- ""
	}
	report <- rnb.add.optionlist(report, optionlist)

	## Genome-wide methylation levels
	cname <- rnb.getOption("inference.genome.methylation")
	if (nchar(cname) != 0) {
		meth.levels <- rnb.execute.genomewide(rnb.set)
		report <- rnb.section.genomewide(report, meth.levels)
		if (!all(is.na(meth.levels))) {
			rnb.set@pheno[[cname]] <- meth.levels
		}
		rm(meth.levels)
	}

	if (inherits(rnb.set,"RnBSet") && rnb.getOption("inference.age.prediction") 
		&& !(rnb.set@target == "probesEPIC" | rnb.set@target == "probesEPICv2")){ ## There is no age prediction for EPICv1 & EPICv2
		ph <- pheno(rnb.set)
		ages <- ph$predicted_ages
		if(is.null(ages)){
			if(rnb.getOption("inference.age.prediction.training")){
				logger.start("Training new age predictor")
				report.data.dir <- file.path(dir.reports,rnb.get.directory(report,"data"))
				prediction_path <- trainPredictor(rnb.set,report.data.dir)
				logger.completed()
			}
			prediction_path <- rnb.getOption("inference.age.prediction.predictor")
			if(inherits(rnb.set,"RnBSet")){
				if(!is.null(prediction_path)&&prediction_path!=""){
					logger.start("Age Prediction using specified predictor")
					rnb.set <- agePredictor(rnb.set,prediction_path)
					logger.completed()
				}else{
				  logger.start("Age Prediction using predefined predictor")
				  rnb.set <- agePredictor(rnb.set)
				  logger.completed()
				}
			}
		}else{
			logger.info("We already have a predicted age in the phenotypic table of the dataset.")
		}
		report <- rnb.step.ageprediction(rnb.set,report)
	}

	## LUMP estimates
	if (rnb.getOption("inference.immune.cells")) {
		immune.content <- tryCatch(rnb.execute.lump(rnb.set), error = function(err) { err$message })
		report <- rnb.section.lump(report, immune.content,s.groups=rnb.sample.groups(rnb.set))
		if (is.double(immune.content)) {
			rnb.set@pheno$`Immune Cell Content (LUMP)` <- as.double(immune.content)
			rnb.set@inferred.covariates$`LUMP` <- TRUE
			rnb.status("Calculated LUMP estimates")
		} else if (is.null(immune.content)) {
			rnb.set@inferred.covariates$`LUMP` <- FALSE
			rnb.status("Could not calculate LUMP estimates")
		}
		rm(immune.content)
	}

	if (length(rnb.getOption("inference.targets.sva"))>0) {
		logger.start("Surrogate Variable Analysis (SVA)")
		result <- rnb.step.sva(rnb.set, report, cmp.cols=rnb.getOption("inference.targets.sva"),
							   columns.adj=rnb.getOption("covariate.adjustment.columns"),
							   numSVmethod=rnb.getOption("inference.sva.num.method"))
		report <- result$report
		rnb.set <- result$rnb.set
		logger.completed()
	}

	if(!is.null(rnb.getOption("inference.reference.methylome.column"))){
		result <- rnb.step.cell.types(rnb.set, report)
		report <- result$report
		rnb.set <- result$rnb.set
	}

	#EWASHER export
	if (rnb.getOption("export.to.ewasher")){
		logger.start("Exporting to FaST-LMM-EWASher")
		out.dir.cur <- file.path(rnb.get.directory(report, "data", absolute = TRUE),"ewasher")
		res.exp <- rnb.export.to.ewasher(rnb.set, out.dir.cur, reg.type="sites")
		logger.completed()
		report <- rnb.section.export.ct.adj(res.exp,rnb.set,report)
	}

	# Write new phenotypic table and add link in the report
	pheno.table <- pheno(rnb.set)
	fname.rel <- rnb.write.table(
	  pheno.table, fname="extended_pheno_table.csv", fpath=rnb.get.directory(report, "data", absolute = TRUE),
	  format="csv", gz=FALSE, row.names = FALSE, quote=FALSE
	)
	txt <- paste("The updated sample annotation sheet, with the inferred covariates added as additional columns, is available as a ",
	         "<a href=",paste0(rnb.get.directory(report, "data"), "/",fname.rel),">comma-separated file</a>.")
	report <- rnb.add.section(report,title="Updated Sample Sheet",description = txt)
	
	module.complete(report, close.report, show.report)
	return(list(rnb.set = rnb.set, report = report))
}

########################################################################################################################

#' @rdname rnb.runs
#' @export
rnb.run.tnt <- function(rnb.set, dir.reports,
	init.configuration = !file.exists(file.path(dir.reports, "configuration")), close.report = TRUE,
	show.report = FALSE) {
	validate.module.parameters(rnb.set, dir.reports, close.report, show.report)
	module.start.log("Tracks and Tables")

	report <- init.pipeline.report("tracks_and_tables", dir.reports, init.configuration)
	optionlist <- rnb.options("export.to.csv", "export.to.bed", "export.to.trackhub", "export.types")
	attr(optionlist, "enabled") <- c(TRUE, TRUE, TRUE,
		(optionlist[["export.to.bed"]] | length(optionlist[["export.to.trackhub"]]) > 0 | optionlist[["export.to.csv"]]))
	report <- rnb.add.optionlist(report, optionlist)

	if (rnb.getOption("export.to.csv")) {
		result <- rnb.execute.export.csv(rnb.set, report)
		logger.status("Exported data to CSV format")
		report <- rnb.section.export.csv(report, result)
		logger.status("Added \"CSV Export\" section to the report")
	}
	if (rnb.tracks.to.export(rnb.set)) {
		provided.email <- rnb.getOption("email")
		if (is.null(provided.email)) provided.email <- "-@-.com"
		res <- rnb.execute.tnt(rnb.set, rnb.get.directory(report, "data", absolute = TRUE), email = provided.email)
		logger.start("Writing export report")
		report <- rnb.section.tnt(res,rnb.set,report)
		logger.completed()
	}

	module.complete(report, close.report, show.report)
	invisible(report)
}

########################################################################################################################

#' @rdname rnb.runs
#' @export
rnb.run.exploratory <- function(rnb.set, dir.reports,
	init.configuration = !file.exists(file.path(dir.reports, "configuration")), close.report = TRUE,
	show.report = FALSE) {
	validate.module.parameters(rnb.set, dir.reports, close.report, show.report)
	module.start.log("Exploratory Analysis")
	report <- init.pipeline.report("exploratory_analysis", dir.reports, init.configuration)

	## Analysis options overview
	optionlist <- rnb.options("replicate.id.column", "exploratory.columns", "exploratory.top.dimensions",
		"exploratory.principal.components", "exploratory.correlation.pvalue.threshold",
		"exploratory.correlation.permutations", "exploratory.correlation.qc", "exploratory.beta.distribution",
		"exploratory.intersample", "exploratory.deviation.plots", "exploratory.clustering",
		"exploratory.clustering.top.sites", "exploratory.clustering.heatmaps.pdf", "distribution.subsample",
		"exploratory.gene.symbols","exploratory.custom.loci.bed")
	if (is.null(optionlist[["exploratory.deviation.plots"]])) {
		optionlist[["exploratory.deviation.plots"]] <- inherits(rnb.set, "RnBeadSet")
	}
	if (optionlist[["exploratory.top.dimensions"]] == 0L) {
		optionlist[["exploratory.top.dimensions"]] <- "all"
	}
	traits.to.test <- is.null(optionlist[["exploratory.columns"]]) || length(optionlist[["exploratory.columns"]] != 0)
	associ.tests <- traits.to.test || optionlist[["exploratory.correlation.qc"]]
	attr(optionlist, "enabled") <- c(TRUE, TRUE, TRUE, associ.tests, associ.tests, associ.tests, TRUE, TRUE, TRUE,
		optionlist[["exploratory.deviation.plots"]], TRUE, optionlist[["exploratory.clustering"]] != "none", TRUE,
		TRUE, TRUE, TRUE)
	report <- rnb.add.optionlist(report, optionlist)
	rm(optionlist, traits.to.test, associ.tests)

	## Sample groups
	result <- rnb.section.sample.groups(rnb.set, report)
	sample.inds <- result$sample.inds
	report <- result$report

	## Region descriptions
	r.types <- rnb.region.types.for.analysis(rnb.set)
	if (length(r.types) != 0) {
		report <- rnb.section.region.description(report, rnb.set, r.types)
	}
	rm(r.types)

	## Sample replicates
	if (!is.null(rnb.getOption("replicate.id.column"))) {
		replicateList <- rnb.sample.replicates(rnb.set, replicate.id.col = rnb.getOption("replicate.id.column"))
		if (length(replicateList) > 0) {
			region.types <- rnb.region.types.for.analysis(rnb.set)
			if (rnb.getOption("analyze.sites")){
				region.types <- c("sites", region.types)
			}
			report <- rnb.section.replicate.concordance(rnb.set, replicateList, types = region.types, report)
		}
	}

	## Low-dimensional representation
	result <- rnb.step.dreduction(rnb.set, report, return.coordinates = TRUE)

	## Batch effects detection
	pcoordinates <- result$pcoordinates
	if (length(pcoordinates) == 0) {
		report <- result$report
		stext <- "This procedure was skipped because no dimension reduction techniques were applied."
		report <- rnb.add.section(report, "Batch Effects", stext)
	} else {
		result <- rnb.step.batcheffects(rnb.set, result$report, pcoordinates, return.permutations = TRUE)
		report <- result$report

		## Quality-associated batch effects
		if (rnb.getOption("exploratory.principal.components") != 0 && rnb.getOption("exploratory.correlation.qc") &&
				inherits(rnb.set, "RnBeadSet")) {
			report <- rnb.step.batch.qc(rnb.set, report, pcoordinates, permutations = result$permutations)
		}
	}

	## Methylation value distributions
	rinfos <- get.site.and.region.types(rnb.set)
	if (rnb.getOption("exploratory.beta.distribution") && rnb.getOption("analyze.sites")) {
		report <- rnb.step.betadistribution.internal(rnb.set, report, sample.inds, rinfos[[1]])
	}

	## Inter-sample variability
	do.intersample <- rnb.getOption("exploratory.intersample")
	if(is.null(do.intersample)){
	  do.intersample <- inherits(rnb.set,"RnBeadSet")
	}
	if (do.intersample) {
		report <- rnb.step.intersample.internal(rnb.set, report, sample.inds, rinfos)
	}

	## Clustering
	if (rnb.getOption("exploratory.clustering") != "none") {
		report <- rnb.step.clustering.internal(rnb.set, report, rinfos)
	}

	## Regional methylation profiles
	profile.types <- rnb.getOption("exploratory.region.profiles")
	if (is.null(profile.types)) {
		profile.types <- intersect(summarized.regions(rnb.set), rnb.region.types(rnb.set@assembly))
	}
	profile.types.in.set <- intersect(profile.types, summarized.regions(rnb.set))
	profile.types.not.in.set <- setdiff(profile.types, summarized.regions(rnb.set))
	if (length(profile.types.not.in.set) > 0){
		logger.warning(c("The following region types are not contained in the RnBSet. They will be discarded for regional methylation profiling:", paste(profile.types.not.in.set, collapse=", ")))
	}
	if (length(profile.types.in.set) != 0) {
		report <- rnb.step.region.profiles(rnb.set, report, profile.types.in.set, subsample=rnb.getOption("distribution.subsample"))
	}
	custom.genes <- rnb.getOption("exploratory.gene.symbols")
	custom.loci.bed <- rnb.getOption("exploratory.custom.loci.bed")
	if (length(custom.genes) > 0 || length(custom.loci.bed) > 0) {
		if (is.null(custom.loci.bed)) custom.loci.bed <- ""
		report <- rnb.step.locus.profiles(rnb.set, report, locus.bed=custom.loci.bed, gene.list=custom.genes)
	}

	## EPICv2 nv probes visualization section
	section.title <- "Visualization of nv Probe Data"
	nv.probes <- tryCatch(rnb.get.nv.probes.matrix(rnb.set), error = function(err) { NULL })
	if (is.null(nv.probes) | rnb.getOption("nv.probe.tables") == FALSE) {
		txt <- c("Overview of nv probes and sample comparison based on them cannot be performed ",
			"because the dataset does not contain nv probes or nv.probe.tables is disabled.")
		report <- rnb.add.section(report, section.title, txt)
	} else if (rnb.set@target == "probesEPICv2" & (rnb.getOption("nv.heatmap") | rnb.getOption("nv.beta.distribution"))) {
		refText <- c("Kaur, D., Lee, S. M., Goldberg, D., Spix, N. J., Hinoue, T., Li, H.-T., Dwaraka, V. B., Smith, R., ",
					 "Shen, H., Liang, G., Renke, N., Laird, P. W., & Zhou, W. (2023). Comprehensive evaluation of the Infinium human ",
					 "MethylationEPIC v2 BeadChip. <i>Epigenetics Communications</i>, <b>3</b>(1), 6.")
		report <- rnb.add.reference(report, refText)
		txt <- paste0("Analysis of the beta values of the nv probes.\n", "EPICv2 introduced nv probes which target common somatic mutations that occur in human cancers. ",
				 	  "These probes measure DNA sequence variations rather than DNA cytosine methylation ", rnb.get.reference(report, refText), ".")

		## Attach a table of beta values and gene symbol of nv probes
		data.dir.abs <- rnb.get.directory(report, "data", TRUE)
		data.dir.rel <- rnb.get.directory(report, "data")
		nv.probe.ids <- strsplit(rownames(nv.probes), split = "-")
		rnb.require("biomaRt")
		ensembl <- useMart("ENSEMBL_MART_ENSEMBL", dataset = "hsapiens_gene_ensembl",host="https://sep2025.archive.ensembl.org")
		chromosomes <- sapply(nv.probe.ids, function(x) x[[3]])
		start <- sapply(nv.probe.ids, function(x) x[[4]])
		end <- sapply(nv.probe.ids, function(x) x[[5]])
		gene.symbols <- sapply(nv.probe.ids, function(x) get.genesymbol.of.coordinate(x[[4]], x[[5]], x[[3]], ensembl))
		tbl <- cbind("ProbeID" = rownames(nv.probes), "Chromosome" = chromosomes, "Start" = start, "End" = end, "GeneSymbol" = gene.symbols, nv.probes)
		fname <- "nv_probes_beta_values_and_genes.csv"
		utils::write.csv(tbl, file = file.path(data.dir.abs, fname), na = "", row.names = FALSE)

		txt <- c(txt, " The table of beta values of each sample and gene symbols of each nv probe is available as a <a href=\"",
				data.dir.rel, "/", fname, "\">comma-separated file</a> accompanying this report.")

		report <- rnb.add.section(report, section.title, txt)

		## EPICv2 nv probes Heatmap
		if (rnb.getOption("nv.heatmap")) {
			report <- rnb.step.nv.probes.heatmap(rnb.set, report, sample.inds, rinfos[[1]])
		}

		## EPICv2 nv probes beta distribution
		if (rnb.getOption("nv.beta.distribution")) {
			report <- rnb.step.nv.probes.beta.distribution(rnb.set, report, sample.inds, rinfos[[1]])
		}
	}

	module.complete(report, close.report, show.report)
	invisible(report)
}

########################################################################################################################

#' @rdname rnb.runs
#' @export
rnb.run.differential <- function(rnb.set, dir.reports,
	init.configuration = !file.exists(file.path(dir.reports, "configuration")), close.report = TRUE,
	show.report = FALSE) {
	validate.module.parameters(rnb.set, dir.reports, close.report, show.report)
	module.start.log("Differential Methylation")

	report <- init.pipeline.report("differential_methylation", dir.reports, init.configuration)
	optionlist <- rnb.options("analyze.sites", "differential.report.sites", "region.types", "differential.permutations", "differential.comparison.columns",
		"differential.comparison.columns.all.pairwise","columns.pairing","differential.site.test.method",
	  "differential.variability","differential.variability.method","covariate.adjustment.columns",
		"differential.adjustment.sva","differential.adjustment.celltype","differential.enrichment.go","differential.enrichment.lola","differential.enrichment.lola.dbs")
	report <- rnb.add.optionlist(report, optionlist)
	permutations <- rnb.getOption("differential.permutations")

	logger.start("Analysis")
		logger.info(c("Using",permutations,"permutation tests"))
		cmp.cols <- rnb.getOption("differential.comparison.columns")
		if (is.null(cmp.cols)){
			cmp.cols <- colnames(pheno(rnb.set))
		}
		logger.info(c("Using columns:",paste(cmp.cols,collapse=",")))
		reg.types <- rnb.region.types.for.analysis(rnb.set)
		if (length(reg.types)>0){
			logger.info(c("Using region types:",paste(reg.types,collapse=",")))
		} else {
			logger.info(c("Skipping region level analysis"))
		}
		disk.dump<-FALSE
		disk.dump.dir<-""
		if (rnb.getOption("disk.dump.big.matrices")){
			disk.dump<-TRUE
			#TODO[issue:0000133]: optionally set disk.dump.dir to local directory/temp directory
			# disk.dump.dir <- file.path(rnb.get.directory(report,dir="data",absolute=TRUE),"tableDump")
			disk.dump.dir <- tempfile(pattern="diffMethTables_")
		}
		rnb.cleanMem()
		diffmeth <- rnb.execute.computeDiffMeth(
			rnb.set,cmp.cols,region.types=reg.types,n.perm=permutations,
			covg.thres=rnb.getOption("filtering.coverage.threshold"),
			pheno.cols.all.pairwise=rnb.getOption("differential.comparison.columns.all.pairwise"),
			columns.pairs=rnb.getOption("columns.pairing"),
			columns.adj=rnb.getOption("covariate.adjustment.columns"),
			adjust.sva=rnb.getOption("differential.adjustment.sva"),
			pheno.cols.adjust.sva=rnb.getOption("inference.targets.sva"),
			adjust.celltype=rnb.getOption("differential.adjustment.celltype"),
			skip.sites=!rnb.getOption("analyze.sites"),
			disk.dump=disk.dump,disk.dump.dir=disk.dump.dir
		)
		rnb.cleanMem()

		dm.go.enrich <- NULL
		dm.lola.enrich <- NULL
		if (!is.null(diffmeth) && (length(reg.types)>0) && (rnb.getOption("differential.enrichment.go") || rnb.getOption("differential.enrichment.lola"))){
			if (rnb.getOption("differential.enrichment.go")){
				dm.go.enrich <- performGoEnrichment.diffMeth(rnb.set,diffmeth,verbose=TRUE)
				if(rnb.getOption("differential.variability")){
				  dm.go.enrich <- performGOEnrichment.diffVar(rnb.set,diffmeth,enrich.diffMeth = dm.go.enrich)
				}
				rnb.cleanMem()
			}
			if (rnb.getOption("differential.enrichment.lola")){
				lolaDbPaths <- prepLolaDbPaths(assembly(rnb.set), downloadDir=rnb.get.directory(report, "data", absolute=TRUE))
				if (length(lolaDbPaths) > 0){
					dm.lola.enrich <- performLolaEnrichment.diffMeth(rnb.set, diffmeth, lolaDbPaths, verbose=TRUE)
					if(rnb.getOption("differential.variability")){
					  dm.lola.enrich <- performLolaEnrichment.diffVar(rnb.set,diffmeth,enrich.diffMeth=dm.lola.enrich,lolaDbPaths,verbose=TRUE)
					}
					rnb.cleanMem()
				} else {
					logger.warning(c("No LOLA DB found for assembly", assembly(rnb.set), "--> continuing without LOLA enrichment"))
				}
			}
		} else {
			logger.info(c("Skipping enrichment analysis of differentially methylated regions"))
		}
	logger.completed()

	if (!is.null(diffmeth)){
		logger.start("Saving temp objects for debugging")
			dataDir <- rnb.get.directory(report, "data", absolute=TRUE)
			diffmeth.path <- file.path(dataDir, "differential_rnbDiffMeth")
			save.rnb.diffmeth(diffmeth, diffmeth.path)
			diffmeth.go.enrichment <- dm.go.enrich
			if (!is.null(diffmeth.go.enrichment)){
				save(diffmeth.go.enrichment, file=file.path(diffmeth.path, "enrichment_go.RData"))
			}
			diffmeth.lola.enrichment <- dm.lola.enrich
			if (!is.null(diffmeth.lola.enrichment)){
				save(diffmeth.lola.enrichment, file=file.path(diffmeth.path, "enrichment_lola.RData"))
			}
		logger.completed()
	}

	logger.start("Report Generation")
	if (is.null(diffmeth)){
		txt <- "Differential methylation analyis was skipped because no valid grouping information could be found."
		report <- rnb.add.section(report, "Differential Methylation Analysis", txt)
	} else {
		gz <- rnb.getOption("gz.large.files")
		includeSites <- rnb.getOption("analyze.sites") && rnb.getOption("differential.report.sites")
		report <- rnb.section.diffMeth.introduction(diffmeth,report)
		if (includeSites){
			report <- rnb.section.diffMeth.site(rnb.set,diffmeth,report,gzTable=gz)
		}
		if (length(get.region.types(diffmeth))>0){
			report <- rnb.section.diffMeth.region(rnb.set,diffmeth,report,dm.go.enrich=dm.go.enrich,dm.lola.enrich=dm.lola.enrich,gzTable=gz)
		}
	}
	logger.completed()

	module.complete(report, close.report, show.report)
	invisible(list(report=report,diffmeth=diffmeth,dm.go.enrich=dm.go.enrich,dm.lola.enrich=dm.lola.enrich))
}

########################################################################################################################

#' rnb.run.example
#'
#' Executes the analysis pipeline for an example from the RnBeads web site.
#'
#' @param index      Example to start. This must be one of \code{1}, \code{2}, \code{3} or \code{4}.
#' @param dir.output One-element \code{character} vector specifying the directory to contain the downloaded data files
#'                   and generated reports. This must be a non-existent path, as this function attempts to create it.
#' @return Invisibly, the loaded, normalized and/or possibly filtered dataset as an object of type inheriting
#'         \code{\linkS4class{RnBSet}}.
#'
#' @details
#' For more information about the examples, please visit the dedicated
#' \href{https://rnbeads.org/examples.html}{page on the RnBeads web site}.
#'
#' @examples
#' \donttest{
#' rnb.run.example()
#' }
#'
#' @seealso \code{\link{rnb.run.analysis}} for starting the analysis pipeline from a local data source
#' @author Yassen Assenov
#' @export
rnb.run.example <- function(index = 4L, dir.output = "example") {

	## Validate arguments
	if (is.double(index) && all(as.integer(index) == index)) {
		index = as.integer(index)
	}
	if (!(is.integer(index) && length(index) == 1 && index %in% 1:4)) {
		stop("invalid value for index; expected 1, 2, 3 or 4")
	}
	if (!(is.character(dir.output) && length(dir.output) == 1 && (!is.na(dir.output)))) {
		stop("invalid value for dir.output; expected one-element character")
	}
	if (file.exists(dir.output)) {
		stop(paste(dir.output, "already exists"))
	}
	if (!create.path(dir.output, accept.existing = FALSE, showWarnings = FALSE)) {
		stop(paste("could not create", dir.output))
	}
	if(index==2){
		if(!requireNamespace("RnBeads.mm9")){
			logger.warning("Missing required package RnBeads.mm9, downloading...")
			install("RnBeads.mm9")
		}
	}

	## Initialize the preprocessing log
	if (logger.isinitialized()) {
		logfile <- NULL
	} else {
		logfile <- c(file.path(dir.output, "preprocessing.log"), NA)
	}

	## Download and extract the example's files
	logger.start("Downloading and Unpacking Data Files", fname = logfile)
	logger.info(c("Processing example", index))
	url.file <- paste0("https://rnbeads.org/materials/data/example_", index, ".tar.gz")
	local.file <- file.path(dir.output, paste0("example_", index, ".tar.gz"))
	result <- download.file(url.file, destfile = local.file, mode = "wb")
	if (result != 0) {
		unlink(dir.output, recursive = TRUE)
		if (!is.null(logfile)) {
			logger.close()
		}
		stop(paste("Could not download", url.file))
	}
	logger.status(c("Downloaded", url.file))
	msg <- suppressWarnings(try(untar(local.file, exdir = dir.output, tar = "internal"), silent = TRUE))
	if (!is.null(attr(msg, "class")) && attr(msg, "class") == "try-error") {
		if (!is.null(logfile)) {
			logger.close()
		}
		stop("Could not unpack downloaded file")
	}
	unlink(local.file)
	logger.status(c("Unpacked downloaded file"))
	logger.completed()
	if (!is.null(logfile)) {
		logger.close()
	}

	## Start the pipeline
	wd <- setwd(dir.output)
	rnb.set <- rnb.run.xml("analysis.xml")
	setwd(wd)
	invisible(rnb.set)
}

########################################################################################################################
## utilities.R
## created: 2012-05-10
## creator: Yassen Assenov
## ---------------------------------------------------------------------------------------------------------------------
## Collection of (mostly internal) helper constants and functions.
########################################################################################################################

## F U N C T I O N S ###################################################################################################

#' rnb.beta2mval
#'
#' Transforms beta values to M values, adjusting for +infinity and -infinity.
#' 
#' @param betas   \code{numeric} vector or matrix of beta values to be transformed.
#' @param epsilon Single \code{numeric} in the range [0, 0.5], giving the threshold of beta values to use when
#'                adjusting for potential M values close to +infinity or -infinity. Setting this parameter to 0 (zero)
#'                disables stabilization; in which case M values of -infinity or +infinity could be returned.
#' @return The calculated and adjusted M values.
#'
#' @author Fabian Mueller
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' mvals <- rnb.beta2mval(meth(rnb.set.example))
#' summary(mvals)
#' }
rnb.beta2mval <- function(betas, epsilon = 0.00001) {
	if (!is.numeric(betas)) {
		stop("invalid value for betas")
	}
	if (!(is.numeric(epsilon) && length(epsilon) == 1 && (!is.na(epsilon)))) {
		stop("invalid value for epsilon")
	}
	if (epsilon < 0 || epsilon > 0.5) {
		stop("invalid value for epsilon; expected 0 <= epsilon <= 0.5")
	}
	betas[betas < epsilon] <- epsilon
	betas[betas > (1 - epsilon)] <- 1 - epsilon
	return(log2(betas / (1 - betas)))
}

########################################################################################################################

#' rnb.mval2beta
#'
#' Transforms M values to beta values.
#' 
#' @param mvals \code{numeric} vector or matrix of M values to be transformed.
#' 
#' @return The calculated beta values.
#'
#' @author Pavlo Lutsik
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' mvals <- rnb.beta2mval(meth(rnb.set.example))
#' bvals <- rnb.mval2beta(mvals)
#' all((bvals-meth(rnb.set.example))<1e-10)
#' }
rnb.mval2beta <- function(mvals){
	if (!is.numeric(mvals)) {
		stop("invalid value for mvals")
	}
	intmf<-2^(mvals)
	return(intmf/(intmf+1))
}

########################################################################################################################

#' rnb.require
#'
#' Loads the specified required package.
#'
#' @param pkg Package name to be loaded.
#'
#' @author Yassen Assenov
#' @noRd
rnb.require <- function(pkg) {
	if (!suppressPackageStartupMessages(do.call(require, list(package = pkg)))) {
		rnb.error(paste("Missing required package", pkg, "or its dependency"))
	}
}

########################################################################################################################

## rnb.cleanMem
##
## perform memory cleanup, i.e. invoke R's garbage collector
## does nothing, if \code{rnb.getOption("enforce.memory.management")==FALSE}
## 
## @param iter.gc number of times to invoke the garbage
## @return invisible \code{TRUE}
##
## @author Fabian Mueller
rnb.cleanMem <- function(iter.gc=10) {
	if (rnb.getOption("enforce.memory.management")){
		#it might be helpful to invoke gc() multiple times
		#see http://stackoverflow.com/questions/1467201/forcing-garbage-collection-to-run-in-r-with-the-gc-command
	#	logger.start("cleaning memory")
		for (i in 1:iter.gc){gc()}
	#	logger.completed()
	}
	invisible(TRUE)
}

########################################################################################################################

#' rnb.call.destructor
#'
#' calls the destructor of an RnBSet, RnBeadSet or RnBeadRawSet object
#' conditionally on whether the \code{enforce.destroy.disk.dumps} option is enabled.
#' 
#' @param object object to be destroyed
#' @param ...	 further arguments to the method \code{\link[=destroy,RnBSet-method]{destroy}}
#' @return invisible \code{TRUE}
#' 
#' @author Fabian Mueller
rnb.call.destructor <- function(object,...) {
	if (rnb.getOption("enforce.destroy.disk.dumps")){
		destroy(object,...)
	}
	invisible(TRUE)
}

########################################################################################################################

#' Get the path to an executable
#' 
#' Gets the full path to the requested executable file.
#' 
#' @param fname Name of the requested executable file. On Windows, this must not include the extension \code{.exe}.
#' @return Full path the specified executable file; an empty \code{character} vector if it could not be found.
#' 
#' @author Yassen Assenov
#' @noRd
rnb.get.executable <- function(fname) {
	dname <- Sys.info()["sysname"]
	if (dname == "Darwin") {
		dname <- "macOSX.i386"
		cmd.search <- "which"
	} else if (dname == "Linux") {
		dname <- "linux_x86.64"
		cmd.search <- "which"
	} else { # dname == "Windows"
		fname <- paste0(fname, ".exe")
		dname <- "windows"
		cmd.search <- "where"
	}
	dname <- system.file(file.path("bin", dname, fname), package = "RnBeads")
	if (dname != "") {
		return(dname)
	}
	searchResult <- suppressWarnings(system(paste(cmd.search, fname), intern = TRUE))
	validResult <- length(searchResult) == 1 && file.exists(searchResult)
	if (validResult){
		return(searchResult)
	} else {
		return(character(0))
	}
}

########################################################################################################################

#' rnb.sample.groups
#'
#' Identifies sample subgroups defined in the given annotation information.
#'
#' @param annotations     Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}},
#'                        or its sample annotations in the form of a \code{data.frame}. If this parameter is
#'                        a dataset, the annotation information is extracted using the method
#'                        \code{\link[=pheno,RnBSet-method]{pheno}}.
#' @param columns         Optional; predefined column names (in the form of a \code{character} vector) or indices (an
#'                        \code{integer} vector) to consider. All other columns in the annotation table will be ignored.
#' @param columns.pairs	  Optional; a NAMED vector containing for each column name for which paired comparisons should
#'                        be performed (say columnA) the name or index of another column (say columnB) in which same
#'                        values indicate the same pairing. columnA should be the name of the value columnB in this
#'                        vector.
#' @param min.group.size  Minimum number of samples in each subgroup. This must be a positive \code{integer}.
#' @param max.group.count Maxumum number of subgroups defined by a trait. This must be an \code{integer} greater than 1.
#' @return List of traits that define subgroups in the dataset. For each trait, the defined subgroups are represented by
#'         a list of \code{integer} vectors storing the corresponding sample indices.
#'
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' str(rnb.sample.groups(rnb.set.example))
#' }
#'
#' @author Yassen Assenov
#' @export
rnb.sample.groups <- function(annotations, columns = NULL, columns.pairs = NULL, min.group.size = rnb.getOption("min.group.size"),
		max.group.count = rnb.getOption("max.group.count")) {
	if (!is.null(annotations)) {
		if (inherits(annotations, "RnBSet")) {
			annotations <- pheno(annotations)
		} else if (!is.data.frame(annotations)) {
			stop("invalid value for annotations; expected object of type RnBSet or a data.frame")
		}
	}
	if (!(is.null(columns) || is.character(columns) || is.integer(columns))) {
		stop("invalid value for columns; expected character or integer")
	}
	if (!(is.null(columns.pairs) || is.character(columns.pairs) || is.integer(columns.pairs))) {
		stop("invalid value for columns.pairs; expected character or integer")
	}
	if (!is.null(columns.pairs)) {
		if (is.null(names(columns.pairs))) {
			stop("invalid value for columns.pairs; expected NAMED vector")
		}
		if (any(!(names(columns.pairs) %in% colnames(annotations)))) {
			stop("invalid value for columns.pairs; not all names are present in the annotation")
		}
	}
	if (is.numeric(min.group.size) && all(min.group.size == as.integer(min.group.size), na.rm = TRUE)) {
		min.group.size <- as.integer(min.group.size)
	}
	if (!(is.integer(min.group.size) && length(min.group.size) == 1 && (!is.na(min.group.size)))) {
		stop("invalid value for min.group.size; expected single integer")
	}
	min.group.size <- min.group.size[1]
	if (min.group.size < 1) {
		stop("invalid value for min.group.size; expected positive integer")
	}
	if (is.numeric(max.group.count) && all(max.group.count == as.integer(max.group.count), na.rm = TRUE)) {
		max.group.count <- as.integer(max.group.count)
	}
	#set the maximum number of groups to be N-1 for NULL values
	if (is.null(max.group.count)) {
		max.group.count <- nrow(annotations) - 1
		max.group.count <- as.integer(max(c(2,max.group.count)))
	}
	if (!(is.integer(max.group.count) && length(max.group.count) == 1 && (!is.na(max.group.count)))) {
		stop("invalid value for max.group.count; expected single integer")
	}
	max.group.count <- max.group.count[1]
	if (max.group.count < 2) {
		stop("invalid value for max.group.count; expected at least 2")
	}
	
	if (is.null(annotations)) {
		result <- list()
		names(result) <- character(0)
	} else {
		annotations.all <- annotations
		if (!is.null(columns)) {
			if (is.character(columns)) {
				columns <- intersect(columns, colnames(annotations))
			} else { # is.integer(columns)
				columns <- intersect(columns, 1:ncol(annotations))
			}
			pp <- annotations
			annotations <- annotations[, columns]
			if (length(columns) == 1){
				annotations <- data.frame(annotations)
				rownames(annotations) <- rownames(pp)
				colnames(annotations) <- columns
			}
		}
		result <- lapply(annotations, function(trait) {
					res <- tapply(1:nrow(annotations), trait, identity)
					if (length(res) < 2) return(NULL)
					res <- res[sapply(res, length) != 0] # ignore levels that are missing in the dataset
					res <- res[sapply(res, function(x){!any(is.na(x))})] # ignore levels that are missing in the dataset
					if (length(res) < 2 || max.group.count < length(res) || any(sapply(res, length) < min.group.size)) {
						return(NULL)
					}
					res
				}
		)
		result <- result[!sapply(result, is.null)]
		has.pairing <- rep(FALSE,length(result))
		#find pairing information
		if (!is.null(columns.pairs)){
			has.pairing <- names(result) %in% names(columns.pairs)
			pair.list <- lapply(names(columns.pairs),FUN=function(cc){
				annotations.all[,columns.pairs[cc]]
			})
			names(pair.list) <- names(columns.pairs)
			for (i in 1:length(result)){
				cc <- names(result)[i]
				if (has.pairing[i]){
					ggs <- result[[i]]
					ggs.pair.ids <- lapply(ggs,FUN=function(iis){
						pair.list[[cc]][iis]
					}) #pairing identifiers for the individual groups
					all.pair.ids <- factor(unlist(ggs.pair.ids))
					pair.id.order <- levels(all.pair.ids)
					is.valid.pairing <- TRUE
					ggs.sorted <- list()
					for (j in 1:length(ggs)) {
						ggn <- names(ggs)[j]
						if (any(duplicated(ggs.pair.ids[[ggn]])) || !all(pair.id.order %in% ggs.pair.ids[[ggn]])){
							is.valid.pairing <- FALSE
							break
						}
						ggs.cur <- ggs[[ggn]]
						names(ggs.cur) <- ggs.pair.ids[[ggn]]
						ggs.sorted[[j]] <- ggs.cur[pair.id.order]
					}
					if (is.valid.pairing){
						names(ggs.sorted) <- names(ggs)
						dd <- data.frame(ggs.sorted)
						rownames(dd) <- pair.id.order
						result[[i]] <- dd
					} else {
						logger.warning(paste("Invalid pairing information for column",cc,". --> Treating as unpaired."))
						has.pairing[i] <- FALSE
					}
				}
			}
		}
		attr(result,"paired") <- has.pairing
	}
	return(result)
}

########################################################################################################################

#' rnb.sample.replicates
#'
#' Identifies sample replicates defined in the given sample annotation table.
#'
#' @param rnb.set          Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param replicate.id.col Trait (column name in the sample annotation table) that indicates sample replicates.
#' 						   Replicates should have the same value for this trait, while samples without replicates are
#'                         expected to have unique values or missing values.
#' @return List of length of the number of replicates in the dataset. Each element is an \code{integer} vector storing
#'         the corresponding sample indices.
#'
#' @author Fabian Mueller
#' @export
rnb.sample.replicates <- function(rnb.set, replicate.id.col) {
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	result <- list()
	names(result) <- character(0)
	if (is.null(replicate.id.col)) {
		return(result)
	}
	if (!(is.character(replicate.id.col) && length(replicate.id.col) == 1 && (!is.na(replicate.id.col)))) {
		stop("invalid value for replicate.id.col")
	}
	if (!(replicate.id.col %in% colnames(pheno(rnb.set)))) {
		warning("specified replicate.id.column not found in rnb.set")
		return(result)
	}

	ids <- pheno(rnb.set)[, replicate.id.col]
	return(tapply(1:length(ids), ids, identity))
}

########################################################################################################################

#' rnb.show.report
#' 
#' Opens the given HMTL report file in the browser.
#' 
#' @param report \code{\linkS4class{Report}} object to open.
#' @return None (invisible \code{NULL}).
#' @author Pavlo Lutsik
#' @export
rnb.show.report <- function(report) {
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	
	tryCatch(browseURL(sprintf("file://%s", report@fname)),
		error = function(err) {
			rnb.warning(c("Could not display the report due to the following error message:", err$message))
			invisible(NULL)
		}
	)
}

########################################################################################################################

#' rnb.load.sitelist
#'
#' Loads a list of probe or site identifiers. This function is used in the preprocessing module for loading a whitelist
#' and/or a blacklist of identifiers.
#'
#' @param fname   File listing the identifiers, one per line.
#' @param verbose Flag indicating if messages are to be printed. If the values is \code{TRUE} and a logger is
#'                initialized, this function adds a message to the log.
#' @return The loaded list of identifiers, or \code{NULL} if \code{fname} could not be open.
#'
#' @seealso \code{\link{logger.start}} for initializing a logger
#' @author Yassen Assenov
#' @export
rnb.load.sitelist <- function(fname, verbose = FALSE) {
	if (length(fname) == 0) {
		return(NULL)
	}
	if (!(is.character(fname) && length(fname) == 1 && nchar(fname) != 0)) {
		stop("invalid value for fname; expected file name")
	}
	if (!parameter.is.flag(verbose)) {
		stop("invalid value for verbose; expected TRUE or FALSE")
	}

	result <- tryCatch(scan(fname, "", sep = "\n", quiet = TRUE), error = function(er) { NULL })
	if (verbose) {
		if (is.null(result)) {
			msg <- paste("Could not load site list from", fname)
			if (logger.isinitialized()) {
				logger.error(msg, terminate = FALSE)
			} else {
				message(msg)
			}
		} else {
			msg <- paste("Loaded", length(result), "site(s) from", fname)
			if (logger.isinitialized()) {
				rnb.status(msg)
			} else {
				message(msg)
			}
		}
	}
	result
}

########################################################################################################################

#' rnb.process.sitelist
#'
#' Loads and processes a list of probe or site identifiers.
#'
#' @param fname      File containing the list of identifiers to be processed.
#' @param anno.table Probe or site annotation table.
#' @return \code{integer} vector containing the indices in the annotation table that are targeted by the loaded list.
#'
#' @author Yassen Assenov
#' @noRd
rnb.process.sitelist <- function(fname, anno.table) {
	if (length(fname) == 0 || nchar(fname) == 0) {
		return(NULL)
	}
	id.list <- rnb.load.sitelist(fname, TRUE)
	if (is.null(id.list)) {
		result <- integer()
		attr(result, "ignored") <- 0L
		attr(result, "note") <- "Could not open the specified file."
	} else {
		id.list <- unique(id.list)
		result <- integer()
		## Parse and match IDs given as [chromosome]:[location]
		for (chrom in levels(anno.table$Chromosome)) {
			regex <- paste0("^", chrom, ":(\\d+)$")
			i <- grep(regex, id.list)
			if (length(i) != 0) {
				j <- which(anno.table$Chromosome == chrom)
				if (length(j) != 0) {
					i <- which(anno.table[j, "Start"] %in% as.integer(gsub(regex, "\\1", id.list[i])))
					result <- c(result, j[i])
				}
			}
		}
		## Parse and match IDs given as Illumina probe identifiers
		regex <- "^(c.\\d+)|(ch\\.(\\d+|X)\\.\\d+([FR]?))|(rs\\d+)$"
		if (any(grepl(regex, rownames(anno.table)))) {
			i <- grep(regex, id.list, value = TRUE)
			if (length(i) != 0) {
				result <- c(result, which(rownames(anno.table) %in% i))
			}
		}
		result <- unique(sort(result))
		attr(result, "ignored") <- length(id.list) - length(result)
		attr(result, "note") <- ""
	}
	result
}

########################################################################################################################

#' trimChar
#'
#' trim a character vector to have a desired length by taking the beginning of the string
#' and the end of the string
#'
#' @param ss       character vector
#' @param len.out  target output length to trim to
#' @param len.pref length of the prefix to be used from the original string
#' @param len.suf  length of the suffix to be used from the original string
#' @param trim.str string to place in between prefix and suffix
#' @return character vector in which each element has length\code{<=len.out}
#'
#' @author Fabian Mueller
#' @noRd
trimChar <- function(ss, len.out=50, trim.str="...", len.pref=ceiling((len.out-nchar(trim.str))/2), len.suf=len.out-len.pref-nchar(trim.str)){
	if (len.pref + len.suf + nchar(trim.str) > len.out){
		stop("Invalid string lengths specified")
	}
	lens <- nchar(ss)
	res <- ifelse(
		lens > len.out,
		paste0(substr(ss, 1, len.pref), trim.str, substr(ss, lens-len.suf+1, lens)),
		ss
	)
	return(res)
}

########################################################################################################################

#' rnb.sitelist.info
#' 
#' Constructs a table summarizing the loaded and identified sites to be included in a white- or blacklist.
#' 
#' @param sitelist List of loaded site indices in the form of an \code{integer} vector, as returned by
#'                 \code{\link{rnb.process.sitelist}}.
#' @param listtype One of \code{"white"} or \code{"black"}.
#' @return Table (\code{data.frame}) with four columns and a single row, summarizing the loaded and identified sites to
#'         be included in a white- or blacklist.
#' @author Yassen Assenov
#' @noRd
rnb.sitelist.info <- function(sitelist, listtype) {
	data.frame(
		"List" = ifelse(listtype == "white", "whitelist", "blacklist"),
		"Records used" = length(sitelist),
		"Records ignored" = attr(sitelist, "ignored"),
		"Note" = attr(sitelist, "note"),
		check.names = FALSE, stringsAsFactors = FALSE)
}

########################################################################################################################

#' rnb.write.table
#'
#' Writes a table to a file. Different formats and compression options are available.
#' 
#' @param tt     Table to be written to file, usually in the form of a \code{matrix} or \code{data.frame}.
#' @param fname  Target file name. If this file already exists, it will be overwritten.
#' @param fpath  Target file path. If "" (default value), \code{fname} is assumed to contain the absolute path.
#' @param format Target format; one of \code{"csv"}, \code{"tab"} or \code{"txt"}, denoting comma-separated,
#'               tab-separated and default text format, respectively. The last format allows for a user-specified
#'               delimiter through an additional parameter \code{sep}. See the documentation of
#'               \code{\link{write.table}} for more details.
#' @param gz     Flag indicating whether the file should be zipped in \code{gz} format.
#' @param ...    Any additional arguments to be passed on to \code{write.table} or \code{utils::write.csv}.
#' @return The (possibly updated) target file name, invisibly. If \code{gz} is \code{TRUE}, the string \code{".gz"} will
#'         be appended to \code{fname}.
#'
#' @seealso \code{\link{write.table}}
#' @author Fabian Mueller
#' @export
#' @examples
#' \donttest{
#' data(mtcars)
#' rnb.write.table(mtcars,tempfile(pattern="cars",fileext=".csv"))
#' }
rnb.write.table <- function(tt, fname, fpath = "", format = "csv", gz = FALSE, ...) {
	if (!(is.character(fname) && length(fname) == 1 && (!is.na(fname)))) {
		stop("invalid value for fname; expected single character")
	}
	if (!(is.character(fname) && length(fname) == 1)) {
		stop("invalid value for fpath; expected single character")
	} else if (is.na(fpath)) {
		fpath <- ""
	}
	if (!(is.character(format) && length(format) == 1)) {
		stop("invalid value for format")
	}
	if (!parameter.is.flag(gz)) {
		stop("invalid value for gz; expected TRUE or FALSE")
	}

	if (gz && grepl("\\.gz$", fname) == FALSE) {
		fname <- paste(fname, "gz", sep = ".")
	}
	fname.full <- ifelse(fpath != "", file.path(fpath, fname), fname)
	if (gz) {
		fname.full <- gzfile(fname.full, open = "w")
	}
	if (format == "csv") {
		utils::write.csv(tt, file = fname.full, ...)
	} else if (format == "tab") {
		write.table(tt, file = fname.full, sep = "\t", ...)
	} else {
		write.table(tt, file = fname.full, ...)
	}
	if (gz) {
		close(fname.full)
	}
	invisible(fname)
}

########################################################################################################################

#' rnb.region.types.for.analysis
#'
#' Identifies the region types that are summarized by the given dataset and pointed to for analysis.
#' 
#' @param rnb.set Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @return List of all region types to be analyzed in the current dataset in the form of a \code{character} vector.
#'
#' @details
#' This function intersects the value of the analysis option \code{"region.types"} with the region types that are
#' summarized in the provided dataset. In case the option's value is \code{NULL}, this function returns all summarized
#' region types in \code{rnb.set}.
#'
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' "promoters" %in% rnb.region.types.for.analysis(rnb.set.example)
#' }
#' @seealso \code{\link{rnb.getOption}} for checking the value of the \code{"region.types"} option;
#'   \code{\link[=summarized.regions,RnBSet-method]{summarized.regions}} for obtaining the region types summarized in a
#'   dataset
#' @author Yassen Assenov
#' @export
rnb.region.types.for.analysis <- function(rnb.set) {
	region.types <- rnb.getOption("region.types")
	if (is.character(rnb.set) && length(rnb.set) == 1 && (rnb.set %in% rnb.get.assemblies())) {
		## Handle the special (undocumented) case when an assembly is specified instead of a dataset
		if (is.null(region.types)) {
			region.types <- rnb.region.types(rnb.set)
		} else {
			region.types <- intersect(region.types, rnb.region.types(rnb.set))
		}
		return(region.types)
	}
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	region.types.dataset <- summarized.regions(rnb.set)
	if (is.null(region.types)) {
		region.types <- region.types.dataset
	} else {
		r.lost <- setdiff(region.types, region.types.dataset)
		if (logger.isinitialized() && length(r.lost) != 0) {
			txt <- c("The following", length(r.lost), "region types will not be included in the analysis:")
			logger.warning(c(txt, paste(r.lost, collapse = ", ")))
		}
		region.types <- intersect(region.types, region.types.dataset)
	}
	region.types
}

########################################################################################################################

## rnb.regions2sites
##
## Creates a mapping from regions to sites.
##
## @param regions GRangesList object storing genomic regions per chromosome. The number of elements and their names must
##                match the names of \code{sites}.
## @param sites   GRangesList object storing genomic loci (sites) per chromosome.
## @return Mapping from the specified regions to the sites contained in them, in the form of a list of mappings per
##         chromosome. Each entry in this list is an object of type \code{IRanges}, storing indices of \code{sites} on
##         the respective chromosome that are contained in the corresponding region. Regions that do not contain sites
##         are left out of the result.
##
## @author Fabian Mueller
rnb.regions2sites <- function(regions, sites) {
	if (length(setdiff(names(regions), names(sites))) != 0) {
		stop("sites is not defined for all chromosomes in regions")
	}
	ovl <- list()
	for (chrom in names(regions)) {
		ov <- GenomicRanges::findOverlaps(regions[[chrom]], sites[[chrom]], ignore.strand = TRUE)
		if (length(ov) != 0) {
			indexMap <- tapply(subjectHits(ov), queryHits(ov), range)
			reg.indices <- names(indexMap)
			indexMap <- matrix(unlist(indexMap, use.names = FALSE), nrow = 2)
			indexMap <- IRanges(start = indexMap[1, ], end = indexMap[2, ], names = reg.indices)
			ovl[[chrom]] <- indexMap
		}
	}
	return(ovl)
}

########################################################################################################################

## rnb.get.sentrix.data
##
## Extracts information on Sentrix ID and Sentrix position from the given sample information table.
##
## @param dframe Sample annotation in the form of a \code{data.frame}; or a dataset as an object of type inheriting
##               \code{\linkS4class{RnBeadSet}}.
## @return Table (\code{data.frame}) with two columns - \code{"Slide"} and \code{"Position"}, listing the slide number
##         location on the slide for each sample. If the provided annotation contains no valid sentrix ID and position
##         information, the returned value is \code{NULL}.
##
## @author Yassen Assenov
rnb.get.sentrix.data <- function(dframe) {
	if (inherits(dframe, "RnBeadSet")) {
		dframe <- pheno(dframe)
	}
	if (all(c("Sentrix_ID", "Sentrix_Position") %in% colnames(dframe))) {
		sentrix.ids <- dframe[, "Sentrix_ID"]
		sentrix.pos <- as.character(dframe[, "Sentrix_Position"])
	} else if (all(c("Sentrix ID", "Sentrix Position") %in% colnames(dframe))) {
		sentrix.ids <- dframe[, "Sentrix ID"]
		sentrix.pos <- as.character(dframe[, "Sentrix Position"])
	} else if ("barcode" %in% colnames(dframe)) {
		barcodes <- strsplit(as.character(dframe[, "barcode"]), "_", fixed = TRUE)
		if (all(sapply(barcodes, length) == 2L)) {
			sentrix.ids <- sapply(barcodes, '[', 1L)
			sentrix.pos <- sapply(barcodes, '[', 2L)
		}
	}
	if (exists("sentrix.ids", inherits = FALSE)) {
		positions <- paste0("R0", 1:6, "C0", rep(1:2, each = 6))
		if ((!(any(c(is.na(sentrix.ids), is.na(sentrix.pos)))) || any(sentrix.ids == "")) &&
			all(sentrix.pos %in% positions)) {
			return(data.frame("Slide" = sentrix.ids, "Position" = factor(sentrix.pos, levels = positions),
					stringsAsFactors = TRUE, check.names = FALSE))
		}
	}
	return(NULL)
}

########################################################################################################################

#' rnb.step.analyze.targets
#'
#' Gets the list of all site and region types that are both requested and supported.
#'
#' @param rnb.set       Dataset of interest.
#' @param report        Report to contain the section once the analysis is complete.
#' @param section.title Title of the report section in human-readable form.
#' @param analyze.sites Flag indicating if the sites are requested to be analyzed.
#' @return List of supported site and region types in the form of a \code{character} vector. If there are region types
#'         that are requested but not summarized in the dataset, this list contains a sentence as an attribute named
#'         \code{"warning"}.
#'
#' @author Yassen Assenov
#' @noRd
rnb.step.analyze.targets <- function(rnb.set, report, section.title, analyze.sites = rnb.getOption("analyze.sites")) {
	if (analyze.sites) {
		targets <- "sites"
	} else {
		targets <- character()
	}
	regions.requested <- rnb.region.types.for.analysis(rnb.set)
	regions.supported <- intersect(regions.requested, names(rnb.set@regions))
	regions.unsupported <- setdiff(regions.requested, regions.supported)
	targets <- c(targets, regions.supported)

	if (length(targets) == 0) {
		if (length(regions.unsupported) != 0) {
			txt <- c("This procedure was skipped because none of the requested region types are summarized in the ",
				"dataset.")
		} else {
			txt <- "This procedure was skipped because neither site nor region types are specified for analysis."
		}
		rnb.add.section(report, section.title, txt)
	} else if (length(regions.unsupported) != 0) {
		txt <- paste("<b>", regions.unsupported, "</b>", sep = "", collapse = ", ")
		txt <- paste("The following region types are not analyzed because they are not summarized in the dataset:", txt)
		attr(targets, "warning") <- txt
	}
	return(targets)
}

########################################################################################################################

## capitalize
##
## Makes the first letter of each of the given strings capital.
##
## @param x          \code{character} vector containing strings to be capitalized.
## @param every.word Flag indicating if the first letter of every word must be capitalized. Note that this method
##                   splits to words using the space character only, therefore, other whitespaces (tabulation or newline
##                   characters) are not recognized as word separators. If this parameter is \code{FALSE} (default),
##                   only the first letter of every string is capitalized. 
## @return \code{character} vector of the same length as \code{x}, in which every element is capitalized.
##
## @details
## If \code{every.word} is \code{TRUE}, this method removes a trailing space (if such exists) from every element of
## \code{x}. This is a side effect of splitting into words.
##
## @author Yassen Assenov
capitalize <- function(x, every.word = FALSE) {
	if (every.word) {
		result <- sapply(strsplit(x, split = " "), capitalize, USE.NAMES = !is.null(names(x)))
		result <- sapply(result, paste, collapse = " ")
	} else {
		result <- paste0(toupper(substr(x, 1, 1)), substring(x, 2))
		result[is.na(x)] <- NA
	}
	return(result)
}

########################################################################################################################

## symmetric.melt
##
## Constructs a \code{data.frame} for plotting the lower triangular values in the given symmetric matrix.
## 
## @param tbl.symmetric    Symmetric \code{matrix} of values with at least 2 rows.
## @param include.diagonal Flag indicating if the diagonal of \code{tbl.symmetric} should also be displayed.
## @return Table, in the form of a \code{data.frame} with three columns, to be passed to \pkg{ggplot}. Missing values
##         (\code{NA}) in the original matrix are omitted from the resulting table.
## @author Yassen Assenov
symmetric.melt <- function(tbl.symmetric, include.diagonal = TRUE) {
	tbl <- tbl.symmetric
	tbl[upper.tri(tbl)] <- NA
	if (!include.diagonal) {
		tbl <- tbl[-1, -ncol(tbl)]
		if (nrow(tbl.symmetric) == 2) {
			tbl <- matrix(tbl, nrow = 1, ncol = 1,
				dimnames = list(rownames(tbl.symmetric)[2], colnames(tbl.symmetric)[1]))
		}
	}
	tbl.melt <- reshape2::melt(tbl, varnames = c("x", "y"))
	tbl.melt[[1]] <- factor(as.character(tbl.melt[[1]]), levels = rev(rownames(tbl)))
	tbl.melt[[2]] <- factor(as.character(tbl.melt[[2]]), levels = colnames(tbl))
	tbl.melt[!is.na(tbl.melt[[3]]), ]
}

########################################################################################################################

## get.i.vector
##
## Transforms the given sublisting of items to indices (if necessary) and validates its scope.
##
## @param i.list Sublisting of items in the form of a \code{logical}, \code{integer} or \code{character} vector.
## @param items  Full list of items to be indexed.
## @return       Sorted \code{integer} vector representing indices in \code{items}.
## @author Yassen Assenov
get.i.vector <- function(i.list, items) {
	if (is.double(i.list) && all(i.list == as.integer(i.list), na.rm = TRUE)) {
		i.list <- as.integer(i.list)
	}
	if (is.logical(i.list)) {
		if (!is.null(items) && length(i.list) != length(items)) {
			stop("specified logical vector is of unexpected length")
		}
		inds <- which(i.list)
	} else if (is.character(i.list)) {
		inds <- which(items %in% i.list)
		if (length(i.list) != length(inds)) {
			stop("unknown names specified")
		}
	} else if (is.integer(i.list)) {
		inds <- sort(i.list)
		if (length(inds) != 0 && (min(inds) < 1 || (!is.null(items) && length(items) < max(inds)))) {
			stop("invalid indices specified")
		}
	} else {
		stop("invalid item list specified")
	}
	inds
}

########################################################################################################################

## parameter.is.flag
##
## Checks if the provided parameter value is a flag.
## 
## @param value Value to be tested.
## @return \code{TRUE} if \code{values} is \code{TRUE} or \code{FALSE}, \code{FALSE} otherwise.
## @author Yassen Assenov
parameter.is.flag <- function(value) {
	is.logical(value) && length(value) == 1 && (!is.na(value))
}

########################################################################################################################

## Validates the given vector or list contains a single element. This function is used in validating function or method
## arguments.
##
## @param x          Value vector or list to validate.
## @param param.name Name of parameter or slot that is validated. This is used in the generation of failing message.
## @return Short message that encodes the result of the validation in the form of a \code{character}. It is either the
##         string \code{ok}, or a short phrase describing the divergence from the "single value assumption".
## @author Yassen Assenov
validate.single <- function(x, param.name = "x") {
	if (is.null(x) || length(x) == 0) {
		result <- paste("missing value for", param.name)
	} else if (length(x) > 1) {
		result <- paste("multiple values for", param.name)
	} else if (is.na(x)) {
		result <- paste("missing value for", param.name)
	} else {
		result <- "ok"
	}
	return(result)
}

########################################################################################################################

## If there is a logger initialized, validates that the given directory exists.
##
## @param dname Name of directory to be validated.
## @author Yassen Assenov
validate.dir <- function(dname) {
	if (logger.isinitialized()) {
		logger.validate.file(dname, is.file = FALSE)
	}
}

########################################################################################################################

## write.line
##
## Writes a line of text to the specified text file. This function is used in the generation of HTML reports.
##
## @param txt    Character vector storing the text to be written. The elements of this vector are concatenated without
##               a separator.
## @param fname  Name of the file to write the text to.
## @param indent Indentation of the text, given as number of \code{TAB} characters.
## @param append Flag indicating if the line is to be appended to the text file. If this is \code{FALSE}, the file's
##               contents are overwritten.
## @author Yassen Assenov
write.line <- function(txt, fname, indent = 0, append = TRUE) {
	strprefix <- paste(rep("\t", times = indent), collapse = "")
	cat(strprefix, paste0(txt, collapse = ""), "\n", file = fname, sep = "", append = append)
}

########################################################################################################################

## write.table.options
##
## Generates HTML code for a table of module option values in the specified report.
##
## @param optionlist List of \pkg{RnBeads} module options. The attribute \code{"enabled"}, if present, is used to mark 
##                   which of these options were actually used.
## @param report     Report to which the table is to be added.
## @param indent     Default indentation, in number of tabulation characters, to apply to the \code{<table>} tag.
## @author Yassen Assenov
write.table.options <- function(optionlist, report, indent = 0L) {
	if (is.null(attr(optionlist, "enabled"))) {
		attr(optionlist, "enabled") <- rep.int(TRUE, length(optionlist))
	}

	wline <- function(txt, indentation = indent) {
		write.line(txt, report@fname, indent = indentation)
	}
	wline("<table class=\"tindex\">")
	wline("<thead>")
	wline("<tr>")
	wline("<th>Option</th>", indent + 1L)
	wline("<th>Value</th>", indent + 1L)
	wline("</tr>")
	wline("</thead>")
	wline("<tbody>")
	for (i in 1:length(optionlist)) {
		wline(c("<tr", ifelse(i %% 2 == 0, " class=\"darker\"", ""), ">"))
		tdopen <- c("<td", ifelse(attr(optionlist, "enabled")[i], "", " class=\"disabled\""), ">")
		wline(c(tdopen, names(optionlist)[i], "</td>"), indent + 1L)
		optionvalue <- optionlist[[i]]
		if (is.null(optionvalue)) {
			optionvalue <- "<span class=\"missing\">default</span>"
		} else if (length(optionvalue) != 1) {
			optionvalue <- paste(optionvalue, collapse = ", ")
		} else if (is.na(optionvalue)) {
			optionvalue <- "<span class=\"disabled\">n/a</span>"
		} else if (is.logical(optionvalue)) {
			optionvalue <- ifelse(optionvalue, "yes", "no")
		}
		wline(c(tdopen, optionvalue, "</td>"), indent + 1L)
		wline("</tr>")
	}
	wline("</tbody>")
	wline("</table>")
}

########################################################################################################################

#' rnb.add.exported.tables
#'
#' Adds a paragraph informing the readers that data are exported as CSV files, following by a table of links.
#'
#' @param report Report to be written to.
#' @param tbl    Table containing links to the exported CSV files.
#'
#' @author Yassen Assenov
#' @noRd
rnb.add.exported.tables <- function(report, tbl) {
	txt <- c("The values presented in the figure above are avaialable in CSV (comma-separated value) files ",
		"accompanying this report.")
	rnb.add.paragraph(report, txt)
	rnb.add.table(report, tbl, row.names = FALSE)
}

########################################################################################################################

#' rnb.get.reliability.matrix
#' 
#' Gets a matrix of reliability indications for every measurement in the given dataset.
#' 
#' @param rnb.set   Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param row.names	Flag indicating of row names are to be generated in the result.
#' @return \code{logical} matrix in which every row corresponds to a CpG site or probe and every column - to a patient.
#'         If the dataset does not contain coverage or detection p-value information, the returned value is \code{NULL}.
#' 
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' rnb.options(identifiers.column = "Sample_ID")
#' str(rnb.get.reliability.matrix(rnb.set.example))
#' }
#' @author Yassen Assenov
#' @export
rnb.get.reliability.matrix <- function(rnb.set, row.names = FALSE) {
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	if (inherits(rnb.set, "RnBeadSet")) {
		result <- dpval(rnb.set, row.names = row.names)
		if (!is.null(result)) {
			result <- (result < rnb.getOption("filtering.greedycut.pvalue.threshold"))
		}
	} else { # inherits(rnb.set, "RnBiseqSet")
		result <- covg(rnb.set, row.names = row.names)
		if (!is.null(result)) {
			result <- (result >= rnb.getOption("filtering.coverage.threshold"))
		}
	}
	result
}

#' rnb.has.reliability.info
#' 
#' Does the object contain info on reliability
#' 
#' @param rnb.set   Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @return \code{logical} indicating whether the object contains reliability information
#' 
#' @author Fabian Mueller
#' @noRd
rnb.has.reliability.info <- function(rnb.set) {
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	if (inherits(rnb.set, "RnBeadSet")) {
		return(!is.null(dpval(rnb.set)))
	} else { # inherits(rnb.set, "RnBiseqSet")
		return(hasCovg(rnb.set))
	}
}

#' rnb.get.reliability.counts.per.sample
#' 
#' Gets a vector with the counts of reliable measurements per sample
#' 
#' @param rnb.set   Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param siteIndices	vector of indices of sites that should be considered. indices can be integer or logical.
#' @return \code{numric} vector in which each entry corresponds to the number of reliable measurements in the corresponding sample
#'         If the dataset does not contain coverage or detection p-value information, the returned value is \code{NULL}.
#' 
#' @author Fabian Mueller
#' @noRd
rnb.get.reliability.counts.per.sample <- function(rnb.set, siteIndices=NULL) {

	## Extract reliability matrix and threshold values
	if (inherits(rnb.set, "RnBeadSet")) {
		mm <- rnb.set@pval.sites
		x.cutoff <- rnb.getOption("filtering.greedycut.pvalue.threshold")
	} else { # inherits(rnb.set, "RnBiseqSet")
		mm <- rnb.set@covg.sites
		x.cutoff <- rnb.getOption("filtering.coverage.threshold")
	}
	if (is.null(mm)) {
		return(NULL)
	}

	## Count number of reliable values per sample
	result <- rep(0L, ncol(mm))
	for (j in 1:ncol(mm)) {
		x <- as.vector(get.dataset.matrix(rnb.set, "sites", FALSE, mm, list(), i = siteIndices, j = j))
		if (inherits(rnb.set, "RnBeadSet")) {
			result[j] <- sum(x < x.cutoff, na.rm = TRUE)
		} else {
			result[j] <- sum(x >= x.cutoff, na.rm = TRUE)
		}
	}
	names(result) <- samples(rnb.set)
	result
}

########################################################################################################################

## methylumi.intensities.by.color
## 
## Rearranges information from "methylated" and "unmethylated" slots of a MethyLumiSet object by color channer.
##
## @param mset MethyLumiSet object
## @param address.rownames If \code{TRUE} the returned intensity matrices have Infinium probe address as row names   
## 
## @author Pavlo Lutsik
methylumi.intensities.by.color<-function(mset,address.rownames=TRUE){

	rnb.require("IlluminaHumanMethylation450kmanifest")

	pinfos <- rnb.annotation2data.frame(rnb.get.annotation("probes450"), add.names=TRUE)[featureNames(mset), ]
	dII.probes <- featureNames(mset)[pinfos[,"Design"] == "II"]
		
	dII.probes <- dII.probes[!grepl("rs", dII.probes)]
			
	if(address.rownames) tII<-IlluminaHumanMethylation450kmanifest@data$TypeII 
	
	if(address.rownames) tII<-tII[match(dII.probes, tII$Name),]
	
	dII.grn<-methylated(mset)[dII.probes,]
	if(address.rownames) rownames(dII.grn)<-tII$Address
	
	dII.red<-unmethylated(mset)[dII.probes,]
	if(address.rownames) rownames(dII.red)<-tII$Address

	dI.red.probes <- featureNames(mset)[pinfos[, "Color"] == "Red"]
	dI.red.probes <- dI.red.probes[!grepl("rs", dI.red.probes)]
	dI.green.probes <- featureNames(mset)[pinfos[, "Color"] == "Grn"]
	dI.green.probes <- dI.green.probes[!grepl("rs", dI.green.probes)]
	if(address.rownames) tI<-IlluminaHumanMethylation450kmanifest@data$TypeI
		
	if(address.rownames) tI.red<-tI[tI$Color=="Red",]
	if(address.rownames) tI.red<-tI.red[match(dI.red.probes, tI.red$Name),]
	
	if(address.rownames) tI.grn<-tI[tI$Color=="Grn",]
	if(address.rownames) tI.grn<-tI.grn[match(dI.green.probes, tI.grn$Name),]
	
	dI.red.meth<-methylated(mset)[dI.red.probes,]
	
	if(address.rownames) rownames(dI.red.meth)<-tI.red[,"AddressB"]
	
	dI.red.umeth<-unmethylated(mset)[dI.red.probes,]
	if(address.rownames) rownames(dI.red.umeth)<-tI.red[,"AddressA"]
	
	
	dI.red.meth.oob<-assayDataElement(mset, "methylated.OOB")[dI.red.probes,]
	if(address.rownames) rownames(dI.red.meth.oob)<-tI.red[,"AddressB"]
	
	dI.red.umeth.oob<-assayDataElement(mset, "unmethylated.OOB")(mset)[dI.red.probes,]
	if(address.rownames) rownames(dI.red.umeth.oob)<-tI.red[,"AddressA"]
	
	
	dI.grn.meth<-methylated(mset)[dI.green.probes,]
	if(address.rownames) rownames(dI.grn.meth)<-tI.grn[,"AddressB"]
	
	dI.grn.umeth<-unmethylated(mset)[dI.green.probes,]
	if(address.rownames) rownames(dI.grn.umeth)<-tI.grn[,"AddressA"]
	
	dI.grn.meth.oob<-assayDataElement(mset, "methylated.OOB")(mset)[dI.green.probes,]
	if(address.rownames) rownames(dI.grn.meth.oob)<-tI.grn[,"AddressB"]
	
	dI.grn.umeth.oob<-assayDataElement(mset, "unmethylated.OOB")[dI.green.probes,]
	if(address.rownames) rownames(dI.grn.umeth.oob)<-tI.grn[,"AddressA"]

	
	intensities.by.channel <- list(
			Cy3=rbind(dII.grn, dI.grn.meth, dI.grn.umeth, dI.red.meth.oob, dI.red.umeth.oob),
			Cy5=rbind(dII.red, dI.red.meth, dI.red.umeth, dI.grn.meth.oob, dI.grn.umeth.oob))

#	intensities.by.channel$Cy3<-rbind(intensities.by.channel$Cy3, 
#			matrix(0, nrow=length(setdiff(rownames(intensities.by.channel$Cy5),rownames(intensities.by.channel$Cy3))),
#					ncol=ncol(intensities.by.channel$Cy3),
#					dimnames=list(rownames=setdiff(rownames(intensities.by.channel$Cy5),rownames(intensities.by.channel$Cy3)))))
#	intensities.by.channel$Cy5<-rbind(intensities.by.channel$Cy5, 
#			matrix(0, nrow=length(setdiff(rownames(intensities.by.channel$Cy3),rownames(intensities.by.channel$Cy5))), 
#					ncol=ncol(intensities.by.channel$Cy5), 
#					dimnames=list(rownames=setdiff(rownames(intensities.by.channel$Cy3),rownames(intensities.by.channel$Cy5)))))

	intensities.by.channel$Cy5<-intensities.by.channel$Cy5[rownames(intensities.by.channel$Cy3),]

	ncd<-rnb.get.annotation("controls450")
	ncd<-ncd[ncd[["Target"]] == "NEGATIVE", ]
	ncd$Target<-tolower(ncd$Target)
	controls.by.channel<-intensitiesByChannel(controlData(mset))
	rownames(controls.by.channel$Cy3)<-tolower(rownames(controls.by.channel$Cy3))
	rownames(controls.by.channel$Cy5)<-tolower(rownames(controls.by.channel$Cy5))

	ncd<-ncd[1:length(grep("negative", rownames(controls.by.channel$Cy3))),]
	controls.by.channel$Cy3<-controls.by.channel$Cy3[sub("\\.0", "" ,paste(ncd$Target, 
							ncd$Index-ifelse("negative" %in% rownames(controls.by.channel$Cy3),1,0), sep=".")),]
	controls.by.channel$Cy5<-controls.by.channel$Cy5[sub("\\.0", "" ,paste(ncd$Target, 
							ncd$Index-ifelse("negative" %in% rownames(controls.by.channel$Cy3),1,0), sep=".")),]

	rownames(controls.by.channel$Cy3)<-ncd$ID
	rownames(controls.by.channel$Cy5)<-ncd$ID

	intensities.by.channel$Cy3<-rbind(intensities.by.channel$Cy3, controls.by.channel$Cy3)
	intensities.by.channel$Cy5<-rbind(intensities.by.channel$Cy5, controls.by.channel$Cy5)

	return(intensities.by.channel)
}

########################################################################################################################

## abbreviate.names
## 
## Creates short versions, if necessary, of the supplied vector labels.
##
## @param labels ...
## @param max.length ...
## @return ...
## @author Pavlo Lutsik
abbreviate.names <- function(labels, max.length = 15L) {
	N <- (max.length - 3) %/% 2
	ifelse(nchar(labels) <= max.length, labels,
		paste0(substring(labels, 1, N), "...", substring(labels, nchar(labels) - N + 1)))
}

########################################################################################################################

#' rnb.logger.start
#'
#' Starts a new section in the log (if it is initialized).
#' @param txt \code{character} vector storing the section title.
#' @details If the logger is not initialized, calling this method has no effect.
#'
#' @author Yassen Assenov
#' @noRd
rnb.logger.start <- function(txt) {
	if (logger.isinitialized()) {
		logger.start(txt)
	}
}

########################################################################################################################

#' rnb.logger.completed
#'
#' Completes the last open section in the log (if it is initialized).
#' @details If the logger is not initialized, calling this method has no effect.
#'
#' @author Yassen Assenov
#' @noRd
rnb.logger.completed <- function() {
	if (logger.isinitialized()) {
		logger.completed()
	}
}

########################################################################################################################

#' rnb.info
#'
#' Prints an information message to the log (if it is initialized).
#' @param txt \code{character} vector storing the status message.
#' @details If the logger is not initialized, calling this function has no effect.
#'
#' @author Yassen Assenov
#' @noRd
rnb.info <- function(txt) {
	if (logger.isinitialized()) {
		logger.info(txt)
	} else {
		message(paste(txt, collapse = " "))
	}
}

########################################################################################################################

#' rnb.status
#'
#' Prints a status message to the log (if it is initialized).
#' @param txt \code{character} vector storing the status message.
#' @details If the logger is not initialized, calling this function has no effect.
#'
#' @author Yassen Assenov
#' @noRd
rnb.status <- function(txt) {
	if (logger.isinitialized()) {
		logger.status(txt)
	} else {
		message(paste(txt, collapse = " "))
	}
}

########################################################################################################################

#' rnb.warning
#'
#' Prints a warning message to the log (if it is initialized) or the console (otherwise).
#' @param txt \code{character} vector storing the warning message.
#'
#' @author Yassen Assenov
#' @noRd
rnb.warning <- function(txt) {
	if (logger.isinitialized()) {
		logger.warning(txt)
	} else {
		warning(paste(txt, collapse = " "), call.=FALSE, immediate.=TRUE)
	}
}

########################################################################################################################

#' rnb.error
#'
#' Prints an error message to the log (if it is initialized) or executes an error action (otherwise).
#' @param txt \code{character} vector storing the error message.
#'
#' @author Yassen Assenov
#' @noRd
rnb.error <- function(txt) {
	if (logger.isinitialized()) {
		logger.error(txt)
	} else {
		stop(paste(txt, collapse = " "), call.=FALSE)
	}
}

########################################################################################################################

## convert.to.ff.tmp.file
##
## converts a table to an ff object
##
## @author Fabian Mueller
convert.to.ff.matrix.tmp <- function(x,...) {
	if(!"ff" %in% class(x)){
		ff(x,dim=dim(x),dimnames=dimnames(x),finalizer="delete",...)
	}else{
		x
	}
}

## additions with respect to a pached ff
convert.to.ff.matrix.tmp2 <- function(x,...) {
	if(is.integer(x)){
		vm<-'integer'
	}else{
		vm<-'double'
	}
	
	ffobj<-ff(vmode=vm, dim=dim(x),dimnames=dimnames(x),finalizer="delete",...)
	for(i in 1:ncol(x)){
		ff[,i]<-x[,i]
	}
	ffobj
}

## creating an empty ff matrix
create.empty.ff.matrix.tmp <- function(vm,dim, dimnames=NULL,...) {
	ff(vmode=vm,dim=dim,dimnames=dimnames,finalizer="delete",...)
}


get.matrix.from.ff<-function(x){

	nm<-matrix(NA,ncol=ncol(x), nrow=nrow(x))
	for(i in 1:ncol(x)){
		nm[,i]<-x[,i]
	}
	nm
}

get.genesymbol.of.coordinate <- function(start, end, chromosome, ensembl) {
	paste(getBM(attributes = "hgnc_symbol",
				filter = c("chromosome_name", "start", "end"),
				values = list(gsub("chr","",chromosome), start, end), 
				mart = ensembl)$hgnc_symbol, collapse = ";")
}

########################################################################################################################

## prepare.idat.dir
##
## Creates a temporary directory with symbolic links to all \code{idat} files located in the given directory or its
## subdirectories.
##
## @param base.dir Base directory to scan for \code{idat} files.
## @param temp.dir Temporary directory to be created for storing the symbolic links.
## @return \code{NULL} if the temporary directory could not be created or if no symbolic links could be created at all;
##         otherwise, the value of \code{temp.dir}. Additionally, an attribute named \code{"failed"} contains the number
##         of idat files in \code{rootdir} that could not be linked to.
## @author Pavlo Lutsik
prepare.idat.dir <- function(base.dir, temp.dir = tempfile(pattern = "idat")) {
	if (!dir.create(temp.dir)) {
		return(NULL)
	}
	idat.files <- list.files(base.dir, pattern = "^.+\\.idat$", recursive = TRUE)
	failed <- !sapply(idat.files, function(fname) {
		suppressWarnings(file.symlink(file.path(base.dir, fname), file.path(temp.dir, basename(fname))))
	})
	if (all(failed)) {
		return(NULL)
	}
	result <- temp.dir
	attr(result, "failed") <- as.integer(sum(failed))
	return(result)
}

########################################################################################################################

## check.idat.subdirs
##
## Checks whether the given directory contains subdirectories with IDAT files.
##
## @param rootdir Base directory to scan.
## @return \code{TRUE} if \code{base.dir} contains a subdirectory that contains one or more \code{idat} files;
##         \code{FALSE} otherwise.
## @author Pavlo Lutsik
check.idat.subdirs <- function(base.dir) {
	subdirectories <- dir(base.dir, full.names = TRUE)
	subdirectories <- subdirectories[file.info(subdirectories)[, "isdir"] == FALSE]
	for (subdir in subdirectories) {
		if (length(dir(subdir, pattern = "^.+\\.idat$", recursive = TRUE, include.dirs = FALSE)) != 0) {
			return(TRUE)
		}
	}
	return(FALSE)
}

########################################################################################################################

## getMergeList
##
## Given an RnBSet object and a column name of the sample annotation table, return a list with sample indices for each level
## level that column the RnBSet
## @param rnbs RnBSet object
## @param pheno.col column name specifying the grouping
## @return a list containing indices for each group
##
## @author Fabian Mueller
getMergeList <- function(rnbs, pheno.col){
	ph <- pheno(rnbs)
	replicate.ids <- ph[,pheno.col]
	replicate.ids.unique <- unique(na.omit(replicate.ids))
	#replace NAs by the sample names
	if (any(is.na(replicate.ids))){
		snames.na <- samples(rnbs)[is.na(replicate.ids)]
		#make sure the sample names do not also appear as group name
		snames.na <- ifelse(snames.na %in% replicate.ids.unique,paste("X",snames.na),snames.na)
		replicate.ids[is.na(replicate.ids)] <- snames.na
		replicate.ids.unique <- unique(replicate.ids)
	}
	if (length(replicate.ids.unique)==nrow(ph)){
		rnb.warning("Could not get merge list: phenotype column with unique entries selected")
		return(res)
	}
	res <- lapply(replicate.ids.unique,FUN=function(x){which(replicate.ids==x)})
	names(res) <- replicate.ids.unique
	return(res)
}

########################################################################################################################

## mergeColumns
##
## Given a matrix X and a list containing column indices for each group in the matrix apply mergeFun to each submatrix
## defined by the column indices in the list
## @param X the input matrix
## @param mergeList a list containing column indices defining subgroups in matrix X
## @param mergeFun the function to be applied to each submatrix
## @return the combined matrix
## @note Requires the packages \pkg{foreach} and \pkg{doParallel}.
##
## @author Fabian Mueller
mergeColumns <- function(X, mergeList, mergeFun=function(X.sub){rowMeans(X.sub, na.rm=TRUE)}){
	res <- do.call("cbind", lapply(mergeList,FUN=function(iis){
		do.call(mergeFun, list(X.sub=X[,iis, drop=FALSE]))
	}))
	return(res)
}

########################################################################################################################

prepare.slot.matrix<-function(slotmatrix, useff=FALSE, full.match=FALSE, subset=1:nrow(slotmatrix), cleanup=FALSE){
	if(useff){
		if(full.match){
			preparedmatrix<-convert.to.ff.matrix.tmp(slotmatrix)
		}else{
			preparedmatrix<-convert.to.ff.matrix.tmp(slotmatrix[subset,,drop=FALSE])
		}
		
	}else{
		if(full.match){
			preparedmatrix<-slotmatrix
		}else{
			preparedmatrix<-slotmatrix[subset,,drop=FALSE]
		}
	}
	if("ff" %in% class(slotmatrix) && cleanup && !full.match){
		delete(slotmatrix)
	}
	preparedmatrix
}


 ".stopImplicitCluster" <- function()
 {
    if(exists(".revoDoParCluster", where=doParallel:::.options) && 
      !is.null(doParallel:::.options[['.revoDoParCluster']]))
    {
      stopCluster(doParallel:::.options[['.revoDoParCluster']])
      remove('.revoDoParCluster', envir=doParallel:::.options)
     }
 }

########################################################################################################################
## genomewide.R
## created: 2018-01-11
## creator: Yassen Assenov
## ---------------------------------------------------------------------------------------------------------------------
## Functions related to the calculation of genome-wide methylation.
########################################################################################################################

## F U N C T I O N S ###################################################################################################

#' Genome-wide methylation level
#'
#' Computes genome-wide methylation levels per sample.
#'
#' @param dataset Methylation dataset to study, provided as an object of type inheriting \code{RnBSet}.
#' @return \code{vector} of values in the range \code{[0, 1]}, storing the average beta values per sample.
#'
#' @author Yassen Assenov
#' @export
rnb.execute.genomewide <- function(dataset) {
	if (!inherits(dataset, "RnBSet")) {
		stop("Invalid value for dataset")
	}

	N <- ncol(dataset@meth.sites)
	result <- rep(NA_real_, N)
	for (i in 1:N) {
		result[i] <- mean(dataset@meth.sites[, i], na.rm = TRUE)
	}
	result
}

########################################################################################################################

#' rnb.section.genomewide
#'
#' Creates a report section dedicated to the computed genome-wide methylation levels.
#'
#' @param report      Report on covariate inference to contain the genome-wide methylation section. This must be an
#'                    object of type \code{\linkS4class{Report}}.
#' @param meth.levels Calculated genome-wide methylation levels, as returned by \code{\link{rnb.execute.genomewide}}.
#' @return The modified report.
#'
#' @author Yassen Assenov
#' @noRd
rnb.section.genomewide <- function(report, meth.levels) {
	s.title <- "Genome-wide Methylation Levels"

	txt <- "because no beta value measurements are available"
	if (length(meth.levels) == 1) {
		if (is.na(meth.levels)) {
			txt <- paste0("Genome-wide methylation level could not be computed for the sample ", txt, ".")
		} else {
			txt <- c("Genome-wide methylation level was computed for the sample. The resulting value is ", meth.levels)
		}
		return(rnb.add.section(report, s.title, txt))
	}

	i.valid <- which(!is.na(meth.levels))
	M <- length(i.valid)
	if (M == 0) {
		txt <- paste0("Genome-wide methylation level could not be computed for any sample ", txt, ".")
		return(rnb.add.section(report, s.title, txt))
	}
	if (M == length(meth.levels)) {
		txt <- paste("Genome-wide methylation levels were computed for all", M, "samples.")
	} else {
		if (length(meth.levels) - M == 1L) {
			txt <- paste("It was not computed for the remaining sample", txt, "for it.")
		} else {
			txt <- paste("They were not computed for the remaining", length(meth.levels) - M, "samples", txt,
				"for them.")
		}
		if (M == 1) {
			txt <- paste("Genome-wide methylation level was computed for one sample only.", txt)
		} else {
			txt <- paste("Genome-wide methylation levels were computed for", M, "samples.", txt)
		}
	}
	txt <- c(txt, " The figure below shows ", ifelse(M == 1, "this value", "the distribution of these values"), ".")
	report <- rnb.add.section(report, s.title, txt)

	pp <- ggplot(data.frame(x = meth.levels[i.valid]), aes_string("x")) +
		geom_histogram(breaks = seq(0, 1, length.out = 51)) + labs(x = "Genome-wide methylation", y = "Frequency") +
		scale_x_continuous(breaks = seq(0, 1, length.out = 11), limits = c(0, 1), expand = c(0, 0)) +
		scale_y_continuous(expand = c(0, 0)) + theme(plot.margin = grid::unit(0.1 + c(0, 0.1, 0, 0), "in"))
	rplot <- createReportPlot("histogram_genomewide", report, width = 6.2, height = 5.2)
	print(pp)
	rplot <- off(rplot)
	txt <- c("Histogram of ", M, " genome-wide methylation level", ifelse(M == 1, "", "s"), ".")
	return(rnb.add.figure(report, txt, rplot))
}

########################################################################################################################
## RnBeadSet-class.R
## created: 2012-04-06
## creator: Pavlo Lutsik
## ---------------------------------------------------------------------------------------------------------------------
## RnBeadSet class definition.
########################################################################################################################
## GLOBALS

RNBEADSET.SLOTNAMES<-c("pval.sites")

## ---------------------------------------------------------------------------------------------------------------------
## CLASS DEFINITIONS
## ---------------------------------------------------------------------------------------------------------------------

#' RnBeadSet Class
#'
#' Stores the preprocessed information from HumanMethylation experiments
#'
#' @details
#' There are multiple ways to create an object of type \code{RnBeadSet}:
#' \describe{
#'  \item{Loading from files}{Dataset can be loaded from text or binary files. See the function
#'       \code{\link{rnb.execute.import}} for more details.}
#'  \item{Downloading from GEO}{See the function \code{\link{rnb.read.geo}} for details.}
#'  \item{Converting from \code{MethyLumiSet}}{...}
#' } 
#'
#' @section Slots:
#' \describe{
#'   \item{\code{pval.sites}}{\code{matrix} of detection p-values with the same dimensions as \code{betas}, or
#'        \code{NULL} if the detection p-values are not available.}
#'   \item{\code{pval.regions}}{\code{list} of methylation \code{matrix} objects, one per available region type. Every row in a 
#' 		matrix corresponds to a methylation site, and every column - to a sample.}
#'   \item{\code{covg.sites}}{\code{matrix} of bead counts per probe with the same dimensions as \code{betas}, or
#'        \code{NULL} if this data are not available.}
#'   \item{\code{qc}}{Quality control probe information in the form of a \code{list} of two elements - \code{"Cy3"} and
#'        \code{"Cy5"}, storing intensities of probes on the green and red channels, respectively. This slot's value is
#'        \code{NULL} if no control probe information is available.}
#' }
#'
#' @section Methods and Functions:
#' \describe{
#'   \item{\code{\link[=samples,RnBSet-method]{samples}}}{Gets the identifiers of all samples in the dataset.}
#'   \item{\code{\link[=pheno,RnBSet-method]{pheno}}}{Gets the phenotypic and processing data of the dataset.}
#'   \item{\code{\link[=meth,RnBSet-method]{meth}}}{Gets the \code{matrix} of methylation beta-values of the dataset.}
#'   \item{\code{\link[=dpval,RnBeadSet-method]{dpval}}}{Gets the \code{matrix} of detection p-values of the dataset.}
#'   \item{\code{\link[=covg,RnBSet-method]{covg}}}{Gets the \code{matrix} of bead counts of the dataset.}
#'   \item{\code{\link[=qc,RnBeadSet-method]{qc}}}{Gets the intensities of the quality control probes.}
#'   \item{\code{\link[=remove.sites,RnBSet-method]{remove.sites}}}{Removes probes from the dataset.}
#'   \item{\code{\link[=remove.samples,RnBeadSet-method]{remove.samples}}}{Removes samples from the dataset.}
#'   \item{\code{\link[BiocGenerics]{combine}}}{Combines two datasets.}
#' }
#'
#' @name RnBeadSet-class
#' @rdname RnBeadSet-class
#' @author Pavlo Lutsik
#' @exportClass RnBeadSet
setClass("RnBeadSet",
         representation(pval.sites="matrixOrffOrNULL",
				 		pval.regions="listOrNULL",
				 		qc="listOrNULL"#,
						#status="listOrNULL"
		#				bead.counts="listOrNULL"
		),
		 contains="RnBSet",
         prototype(#pheno=data.frame(),
				 #meth.sites=matrix(),
				 pval.sites=NULL,
				 pval.regions=NULL,
				 qc=NULL#, 
				 #status=NULL
#				 bead.counts=NULL
 		 ), package = "RnBeads"
)

## ---------------------------------------------------------------------------------------------------------------------
## VALIDITY
## ---------------------------------------------------------------------------------------------------------------------

##
## validRnBeadSetObject
##
## Valididty check for RnBeadSet-class
##
validRnBeadSetObject<-function(object){

  if(is.null(object@pheno) || is.null(object@meth.sites)) return("Pheno and betas slots can not be NULL")
  if(dim(object@pheno)[1L]!=dim(object@meth.sites)[2L]) return("The number of samples is ambiguous")

#  if(!is.null(object@pval.sites)){
#    if(!setequal(rownames(object@pval.sites), rownames(object@meth.sites))) return("Rows of the beta and p-value tables do not match")
#    if(!setequal(colnames(object@pval.sites), colnames(object@meth.sites))) return("Columns of the beta and p-value tables do not match")
#
#  }
#
#  if(!is.null(object@covg.sites)){
#    if(!setequal(rownames(object@covg.sites), rownames(object@meth.sites))) return("Rows of the beta and bead count tables do not match")
#    if(!setequal(colnames(object@covg.sites), colnames(object@meth.sites))) return("Columns of the beta and bead count tables do not match")
#  }

  return(TRUE)
}

setValidity("RnBeadSet", method=validRnBeadSetObject)

## ---------------------------------------------------------------------------------------------------------------------
## CONSTRUCTORS
## ---------------------------------------------------------------------------------------------------------------------

setMethod("initialize", "RnBeadSet",
		function(.Object,
				pheno=data.frame(),
				sites=matrix(nrow=0, ncol=0),
				meth.sites=matrix(nrow=0, ncol=0),
				pval.sites=NULL,
				covg.sites=NULL,
				qc=NULL,
				target="probes450",
				status=list(normalized=FALSE, background=FALSE, disk.dump=FALSE)
		) {
			
#			.Object@target<-target
#			
#			.Object@pheno<-pheno
#			.Object@sites<-sites
#			.Object@meth.sites<-betas
			.Object@pval.sites<-pval.sites
			#.Object@covg.sites<-bead.counts
			#.Object@regions<-list()
			
			status<-status
			
			#.Object@inferred.covariates <- list()
			
			.Object@qc<-qc
			
			genome.assembly<-rnb.getOption("assembly")

			callNextMethod(.Object,
					pheno=pheno,
					sites=sites,
					meth.sites=meth.sites,
					covg.sites=covg.sites,
					status=status,
					assembly=ifelse(target=="probesMMBC", "mm10", ifelse(target=="probesEPICv2", "hg38", genome.assembly)),
					target=target
			)
			
		})

########################################################################################################################

#' Wrapper function RnBeadSet
#'
#'
#' @param pheno       		Phenotypic data.
#' @param probes			\code{character} vector of Infinium(R) probe identifiers
#' @param betas      		\code{matrix} or \code{ff_matrix} of beta values. If \code{probes} are missing should contain Infinium probe identifiers as row names.
#' @param p.values    		\code{matrix} or \code{ff_matrix} of detection p-values.
#' @param bead.counts       ...
#' @param qc                ...
#' @param platform	   		\code{character} singleton specifying the microarray platform: \code{"450k"} corresponds to HumanMethylation450 microarray, and \code{"27k"} stands for HumanMethylation27.
#' @param summarize.regions ...
#' @param region.types		A \code{character} vector specifying the region types, for which the methylation infromation will be summarized.
#' @param useff		  		If \code{TRUE} the data matrices will be stored as \code{ff} objects
#' 
#' @return an object of class RnBeadSet
#' 
#' @name RnBeadSet
#' @rdname RnBeadSet-class
#' @aliases initialize,RnBeadSet-method
#' @export
RnBeadSet<-function(
		pheno,
		probes,
		betas,
		p.values = NULL, 
		bead.counts = NULL,
		qc = NULL,
		platform = "450k",
		summarize.regions = TRUE,
		region.types = rnb.region.types.for.analysis(ifelse(platform=="MMBC", "mm10", rnb.getOption("assembly"))),
		useff=rnb.getOption("disk.dump.big.matrices")
		){
		
		if(missing(pheno)){
			stop("argument pheno should be supplied")
		}
		
		if(missing(betas)){
			#warning("Valid RnBeadSet object was not created: pheno and/or betas missing")
			stop("argument betas should be supplied")
		}
		
		if(missing(probes)){
			if(!is.null(rownames(betas))){
				probes<-rownames(betas)
			}else{
				stop("If probes are not supplied, betas should have probe identifers as row names")
			}
		}
		
		if(!is.data.frame(pheno)){
			stop("invalid value for pheno: should be a data frame")
		}
		
		if(!any(c('matrix', 'ff_matrix') %in% class(betas))){
			stop("invalid value for betas: should be a matrix or an ff_matrix")
		}
	
		if(!is.null(p.values) && !any(c('matrix', 'ff_matrix') %in% class(p.values))){
			stop("invalid value for p.values: should be a matrix or an ff_matrix")
		}
		
		if(!is.null(bead.counts) && !any(c('matrix', 'ff_matrix') %in% class(bead.counts))){
			stop("invalid value for bead.counts: should be a matrix or an ff_matrix")
		}
		
		if(!is.null(qc)){
			if(!is.list(qc)){
				stop("invalid value for qc: should be a list")
			}
		}
		
		if(!is.character(platform) || length(platform)!=1L){
			stop("invalid value for platform: should be a character of length one")
		}
		
		if(!is.character(region.types)){
			stop("invalid value for region types: should be a character vector")
		}
		
		if(!is.logical(summarize.regions) || length(summarize.regions)!=1L){
			stop("invalid value for summarize.regions: should be a logical of length one")
		}
		
		if(!is.logical(useff) || length(useff)!=1L){
			stop("invalid value for useff: should be a logical of length one")
		}
		
		genome.assembly <- rnb.getOption("assembly")
		if (platform == "EPIC") {
			target <- "probesEPIC"
			assembly <- ifelse(genome.assembly == "hg19", "hg19", "hg38")
		} else if (platform == "EPICv2") {
			target <- "probesEPICv2"
			assembly <- "hg38"
		} else if (platform == "450k") {
			target <- "probes450"
			assembly <- ifelse(genome.assembly == "hg19", "hg19", "hg38")
		} else if(platform == "27k"){ 
			target <- "probes27"
			assembly <- "hg19"
		} else if(platform == "MMBC"){
            target <- "probesMMBC"
            assembly <- "mm10"
		} else{
			stop("Invalid value for platform")
		}
		mr<-match.probes2annotation(probes, target, assembly)
		sites<-mr[[1]]
		site.ids<-mr[[2]]
		
		if(useff){
			if(!"ff" %in% class(betas)){
				betas<-convert.to.ff.matrix.tmp(betas[site.ids,,drop=FALSE])
			}
			if(!is.null(p.values) && !"ff" %in% class(p.values)){
				p.values<-convert.to.ff.matrix.tmp(p.values)
			}
			if(!is.null(bead.counts) && !"ff" %in% class(bead.counts)){
				bead.counts<-convert.to.ff.matrix.tmp(bead.counts)
			}
		}else{
			betas<-betas[site.ids,,drop=FALSE]
			if(!is.null(p.values)){
				p.values<-p.values[site.ids,,drop=FALSE]
			}
			if(!is.null(bead.counts)){
				bead.counts<-bead.counts[site.ids,,drop=FALSE]
			}
		}
		status<-list()
		status[["normalized"]]<-"none"
		status[["background"]]<-"none"
		status[["disk.dump"]]<-useff
		
		object<-new("RnBeadSet",
				target = target,
				pheno,
				sites=sites,
				meth.sites=betas,
				pval.sites=p.values,
				covg.sites=bead.counts,
				qc=qc,
				status=status
				)
				
		if(summarize.regions){
			for (region.type in region.types) {
				if (region.type %in% rnb.region.types("hg19")) {
						object <- summarize.regions(object, region.type)
					}
				}
			}
			
		return(object)
}

########################################################################################################################

## Prints a summary of a RnBeadSet dataset. This is used in the \code{show} methods.
## 
## @param object The RnBeadSet object to be presented.
##
rnb.show.rnbeadset <- function(object) {
	cat("Object of class ", class(object), "\n", sep = "")
	cat(sprintf("%8d samples\n", nrow(object@pheno)))
	cat(sprintf("%8d probes\n", nrow(object@sites)))
	probe.types <- rownames(object@sites)
	if (!is.null(probe.types)) {
		probe.types.identifiers <- c("^cg", "^ch", "^rs")
		if (object@target == "probesEPICv2") {
			probe.types.identifiers <- c(probe.types.identifiers, "^nv")
		}
		probe.types <- sapply(probe.types.identifiers, function(type) { sum(grepl(type, probe.types)) })
		cat(sprintf("\tof which: %g CpG, %g CpH, %g rs and %g nv\n", 
					probe.types[1], probe.types[2], probe.types[3], ifelse(object@target == "probesEPICv2", probe.types[4], 0)
					))
	}
	if (!is.null(object@regions)) {
		cat("Region types:\n")
		for (rn in names(object@regions)) {
			cat(sprintf("\t%8d regions of type %s\n", nrow(object@regions[[rn]]), rn))
		}
	}
	cat(sprintf("Intensity information is %s\n", ifelse(inherits(object, "RnBeadRawSet"), "present", "absent")))
	cat(sprintf("Detection p-values are %s\n", ifelse(is.null(object@pval.sites), "absent", "present")))
	cat(sprintf("Bead counts are %s\n", ifelse(is.null(object@covg.sites), "absent", "present")))
	cat(sprintf("Quality control information is %s\n", ifelse(is.null(qc(object)), "absent", "present")))
	cat(sprintf("Summary of normalization procedures:\n"))
	
	if(!is.null(object@status) && !is.null(object@status$normalized)){
		if(object@status$normalized=="none"){
			cat(sprintf("\tThe methylation data was not normalized.\n"))
		}else{
			cat(sprintf("\tThe methylation data was normalized with method %s.\n", object@status$normalized))			
		}
	}
	
	if(!is.null(object@status) && !is.null(object@status$background)){
		if(object@status$background=="none"){
			cat(sprintf("\tNo background correction was performed.\n"))
		}else{
			cat(sprintf("\tBackground correction was performed with method %s.\n", object@status$background))			
		}
	}
	if(isImputed(object)){
	  cat("\tData set was imputed.\n")
	}
}

setMethod("show", "RnBeadSet", rnb.show.rnbeadset)

########################################################################################################################
## MethyLumi2RnBeadSet
##
## Extracts all methylation data from a MethyLumiSet object, necessary to instantiate RnBeadSet
##
## @param methySet \code{MethylumiSet} instance to be converted.
## @return \code{list} with up to 5 elements: \code{"pheno"}, \code{"betas"}, \code{"bead.counts"}, \code{"p.values"},
##         \code{"qc"}
##
## @author Pavlo Lutsik
MethyLumiSet2RnBeadSet <- function(methySet) {
	result <- list()
	result[["pheno"]] <- as(phenoData(methySet), "data.frame")
	beta.vals <- betas(methySet)

	if(all(c("methylated.N", "unmethylated.N") %in% assayDataElementNames(methySet))){

		## Set beta values with low coverage (bead.counts) to NA
		result[["bead.counts"]]<-methylated.N(methySet)+unmethylated.N(methySet)
		beta.vals[methylated.N(methySet)<2]<-NA
		beta.vals[unmethylated.N(methySet)<2]<-NA
		
	}
	result[["probes"]] <- featureNames(methySet)
	result[["betas"]] <- beta.vals
	result[["p.values"]] <- pvals(methySet)

	if(!is.null(controlData(methySet))){

		## Extract quality control data
		cd<-controlData(methySet)
		green<-intensitiesByChannel(cd,"Cy3")
		red<-intensitiesByChannel(cd,"Cy5")

		if(length(grep('[A-z]', rownames(green)))==dim(green)[1L]){

			if("ProbeID" %in% varLabels(featureData(cd)))
				probe.id.col<-"ProbeID" else if ("Address" %in% varLabels(featureData(cd))) probe.id.col<-"Address"

			probe.mapping<-as(featureData(cd)[,probe.id.col], "data.frame")
			rownames(green)<-probe.mapping[rownames(green), probe.id.col]
			rownames(red)<-probe.mapping[rownames(red), probe.id.col]
			
		}

		result[["qc"]]<-list(Cy3=green, Cy5=red)
	}
	result
}

#' as("RnBeadSet", "MethyLumiSet")
#'
#' Convert a \code{\linkS4class{RnBeadSet}} object to \code{\linkS4class{MethyLumiSet}}
#' 
#' @name coercion-methods
#' 
setAs("MethyLumiSet", "RnBeadSet",

	function(from, to){

		if(!inherits(from,"MethyLumiSet")){
			stop("not a MethyLumiSet object:", deparse(substitute(methylumi.set)))
		}

		m.data <- MethyLumiSet2RnBeadSet(from)
		object<-RnBeadSet(
				pheno=m.data$pheno,
				probes=m.data$probes,
				betas=m.data$betas,
				p.values=m.data$p.values,
				bead.counts=m.data$bead.counts)
		if ("qc" %in% names(m.data)) {
			qc(object)<-m.data[["qc"]]
		}

		object
})

## ---------------------------------------------------------------------------------------------------------------------
## GETTERS
## ---------------------------------------------------------------------------------------------------------------------

if(!isGeneric("dpval")) setGeneric('dpval',
           function(object, ...) standardGeneric('dpval'))

#' dpval-methods
#'
#' Extract detection p-values from an object of \code{\linkS4class{RnBeadSet}} class.
#'
#' @param object 		\code{\linkS4class{RnBeadSet}} or \code{\linkS4class{RnBeadRawSet}} object
#' @param type 			\code{character} singleton. If \code{sites} detection p-values per each available 
#' 						site is returned. Otherwise should be one of region types for for which the summarized 
#' 						p-values are available
#' @param row.names	    Flag indicating of row names are to be generated in the result.
#' @param i     	    Indices of sites/regions to be retrieved. By default (\code{NULL}), all will be retrieved.
#' @param j     	    Indices of samples to be retrieved. By default (\code{NULL}), all will be retrieved.
#' 
#' @return detection p-values available for the dataset in the form of a \code{matrix}.
#'
#' @rdname dpval-methods
#' @docType methods
#' @export
#' @aliases dpval
#' @aliases dpval,RnBeadSet-method
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' dp<-dpval(rnb.set.example, row.names=TRUE)
#' head(dp)
#' }
setMethod("dpval", signature(object = "RnBeadSet"),
	function(object, type = "sites", row.names = FALSE, i = NULL, j = NULL) {
		get.dataset.matrix(object, type, row.names, object@pval.sites, object@meth.regions, i, j)
	}
)

########################################################################################################################
  
setGeneric("qc", function(object) standardGeneric("qc"))

#' qc-methods
#'
#' Extracts HumanMethylation quality control information
#'
#' @param object Dataset of interest.
#' @return Quality control information available for the dataset in the form of a \code{list} with two elements:
#' \code{Cy3} and \code{Cy5}.
#'
#' @rdname qc-methods
#' @aliases qc
#' @aliases qc,RnBeadSet-method
#' @docType methods
#' @export 
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' qcinf<-dpval(rnb.set.example, row.names=TRUE)
#' head(qcinf$Cy3)
#' head(qcinf$Cy5)
#' }
setMethod("qc", signature(object="RnBeadSet"), function(object){ object@qc })

setGeneric("qc<-", function(object, value) standardGeneric("qc<-"))
setMethod("qc<-", signature(object = "RnBeadSet", value = "listOrNULL"),
	function(object, value) {
		## TODO: Add validation and export, or simply use @qc in our code
		object@qc <- value
		object
	}
)

## ---------------------------------------------------------------------------------------------------------------------
## MODIFIERS
## ---------------------------------------------------------------------------------------------------------------------

if (!isGeneric("remove.sites")) {
	setGeneric("remove.sites", function(object, probelist, verbose = TRUE) standardGeneric("remove.sites"))
}

#' @rdname remove.sites-methods
#' @aliases remove.sites,RnBeadSet-method
#' @docType methods
#' @export
setMethod("remove.sites", signature(object = "RnBeadSet"),
	function(object, probelist, verbose = TRUE) {
		inds <- get.i.vector(probelist, rownames(object@sites))
		if (length(inds) != 0) {
			if (!is.null(object@pval.sites)) {
				if(object@status$disk.dump){
					new.matrix<-object@pval.sites[-inds,,drop=FALSE]
					if(isTRUE(object@status$discard.ff.matrices)){
						delete(object@pval.sites)
					}
					object@pval.sites <- convert.to.ff.matrix.tmp(new.matrix)
					rm(new.matrix); rnb.cleanMem()
				}else{
					object@pval.sites <- object@pval.sites[-inds, ,drop=FALSE]
				}
			}
		}

		callNextMethod()
	}
)

########################################################################################################################

if (!isGeneric("remove.samples")) {
	setGeneric("remove.samples", function(object, samplelist) standardGeneric("remove.samples"))
}

#' @rdname remove.samples-methods
#' @aliases remove.samples,RnBeadSet-method
#' @docType methods
#' @export
setMethod("remove.samples", signature(object = "RnBeadSet"),
	function(object, samplelist) {
		inds <- get.i.vector(samplelist, samples(object))
		if (length(inds) != 0) {
			if (!is.null(object@pval.sites)) {
				if(object@status$disk.dump){
					new.matrix<-object@pval.sites[,-inds, drop=FALSE]
					if(isTRUE(object@status$discard.ff.matrices)){
						delete(object@pval.sites)
					}
					object@pval.sites <- convert.to.ff.matrix.tmp(new.matrix)
					rm(new.matrix); rnb.cleanMem()
				}else{
					object@pval.sites <- object@pval.sites[,-inds, drop=FALSE]
				}
			}
			if (!is.null(object@qc)) {
				object@qc$Cy3 <- object@qc$Cy3[,-inds, drop=FALSE]
				object@qc$Cy5 <- object@qc$Cy5[,-inds, drop=FALSE]
			}
		}
		callNextMethod()
	}
)

########################################################################################################################

setMethod("save.matrices", signature(object="RnBeadSet", path="character"),
		function(object, path){
			
			if(!is.null(object@pval.sites)){
				
				if(!is.null(object@status) && object@status$disk.dump){
					
					if("ff" %in% class(object@pval.sites)){
						ffmatrix<-object@pval.sites
						ffsave(ffmatrix, file=file.path(path, "rnb.pvals"),rootpath=getOption('fftempdir'))
						rm(ffmatrix)
						
					}
				}

				if(!is.null(object@regions)){
					
					for(rgn in 1:length(object@regions)){
						
						rgnpath<-file.path(path,rgn)
						if(!file.exists(rgnpath)){
							dir.create(rgnpath)	
						}

						if("ff" %in% class(object@pval.regions[[rgn]])){
							
							ffmatrix<-object@meth.regions[[rgn]]
							ffsave(ffmatrix, file=file.path(path, rgn, "rnb.pval"),rootpath=getOption('fftempdir'))
							rm(ffmatrix)
							
						}			
					}
				}
			}
			callNextMethod(object, path)
			
		})

########################################################################################################################
		
setMethod("load.matrices", signature(object="RnBeadSet", path="character"),
		
		function(object, path, temp.dir=tempdir()){
			
		if(!is.null(object@pval.sites)){
			
			if(sum(grepl("rnb.pvals",list.files(path)))==2){
				load_env<-new.env()
				suppressMessages(ffload(file=file.path(path, "rnb.pvals"), envir=load_env,rootpath=getOption("fftempdir")))
				object@pval.sites<-get("ffmatrix", envir=load_env)
				rm(load_env)
			}
			
			rgns <- object@regions
			if(!is.null(rgns)){
				if (.hasSlot(object, 'version')) {
					rngs <- 1:length(rgns)
				}
				for(rgn in rgns){
					if(sum(grepl("rnb.pvals",list.files(file.path(path, rgn))))==2){
						load_env<-new.env()
						suppressMessages(ffload(file=file.path(path, rgn, "rnb.pvals"), envir=load_env,rootpath=getOption("fftempdir")))
						object@pval.regions[[rgn]]<-get("ffmatrix", envir=load_env)
						rm(load_env)
					}		
				}
			}
		}
			
			callNextMethod(object=object, path=path, temp.dir=temp.dir)
			
		})

########################################################################################################################

#' @rdname destroy-methods
#' @aliases destroy,RnBeadSet-method
#' @docType methods
#' @export
setMethod("destroy", signature(object="RnBeadSet"),
		function(object){
			
			if(object@status$disk.dump){
				
				if(!is.null(object@pval.sites)){
					delete(object@pval.sites)
				}
			}
			callNextMethod()
			
		}
)

## ---------------------------------------------------------------------------------------------------------------------
## HELPER ROUTINES
## ---------------------------------------------------------------------------------------------------------------------
get.relative.covg<-function(m, design.vector){
	
	des1.quant<-sort(m[design.vector=="I",])[round(length(which(design.vector=="I"))*ncol(m)*0.001)]
	des2.quant<-sort(m[design.vector=="II",])[round(length(which(design.vector=="II"))*ncol(m)*0.001)]
	m[design.vector=="I",]<-m[design.vector=="I",]/as.double(des1.quant)
	m[design.vector=="II",]<-m[design.vector=="II",]/as.double(des2.quant)
	m
	
}
########################################################################################################################
summarize.bead.counts<-function(bead.counts.M, bead.counts.U, method="min"){
	
	nrow.M<-nrow(bead.counts.M); ncol.M<-ncol(bead.counts.M)
	nrow.U<-nrow(bead.counts.U); ncol.U<-ncol(bead.counts.U)
	
	if(nrow.M!=nrow.U || ncol.M!=ncol.U){
		stop("Dimensions of bead count matrices differ")
	}else{
		bead.counts<-matrix(NA,nrow.M,ncol.M)
		rownames(bead.counts)<-rownames(bead.counts.M)
	}
	
	if(method=="min"){
		index1<-bead.counts.M<=bead.counts.U
		index1[is.na(index1)]<-FALSE
		bead.counts[index1]<-bead.counts.M[index1]
		index2<-bead.counts.U<bead.counts.M
		index2[is.na(index2)]<-FALSE
		bead.counts[index2]<-bead.counts.U[index2]
		
	}
	
	bead.counts
	
}
########################################################################################################################
combine.sites<-function(sites1, sites2){
	
	common.chr<-intersect(unique(sites1[,2]), unique(sites2[,2]))
	
	common.sites<-lapply(common.chr, function(chr){
				
				sts<-intersect(sites1[sites1[,2]==chr,3],sites2[sites2[,2]==chr,3])				
				
				rbind(rep(1,length(sts)), rep(chr,length(sts)), sts)
				
			})
	
		
	if("ff_matrix" %in% c(class(sites1), class(sites2))){
	
		new.sites<-ff(vmode="integer", dim=c(sum(sapply(common.sites, nrow)),3))

		ixx<-1
		for(sts in common.sites){
			new.sites[,][ixx:(ixx+nrow(sts)),]<-sts
			ixx<-ixx+nrow(sts)+1
		}
		
	}else{
		new.sites<-do.call("rbind", common.sites)
	}
	
	
	new.sites
		
}
########################################################################################################################
match.probes2annotation<-function(probes, target="probes450", assembly="hg19"){
	
	if(!is.character(probes)){
		stop("wrong value for probes")
	}

	probe.identifiers.pattern <- "^cg|^ch|^rs"
	## EPIC v2 has a new probe type "nv"
	if (target == "probesEPICv2") {
		probe.identifiers.pattern <- paste(probe.identifiers.pattern, "|^nv", sep="")
	}
	
	if(!all(grepl(probe.identifiers.pattern, probes))){
		stop("probes contains invalid IDs")
	}
	
	dupl<-duplicated(probes)
	if(any(dupl)){
		stop(sprintf("some supplied probe IDs are duplicated: e.g. %s", paste(probes[which(dupl)[1:3]],collapse=", ")))
	}
		
	## Load probe annotation table
	probe.annotation <- rnb.get.annotation(target, assembly)
    ### TODO: remove after fixing the annotation
    if(target=="probesMMBC"){
        probe.annotation<-endoapply(probe.annotation, function(ag) {
                    names(ag)<-mcols(ag)[["ID"]]
                    mcols(ag)[["Context"]]<-factor(c("cg"="CG", "ch"="CH", "rs"="Other")[substr(names(ag), 1,2)], levels=c("CG", "CH", "Other"))
                    return(ag)})
    }
	## Construct the matrix of site indices
	#x.data<-GenomicRanges::as.data.frame(probe.annotation)
	
	chrs<-rnb.get.chromosomes(assembly)
	
	probe.id.column.name = "ID"
	is.combining.epicv2.arrays = FALSE
	annotated.probes<-do.call("c", lapply(probe.annotation, names))
	if (!grepl("_", probes[1]) & grepl("_", annotated.probes[1])){
		## FIXME: Bit of a hack - This part resolves the issue when combining
		## EPICv2 with previous arrays; since the IlmnID's do not match. i.e. cg123_BT12 vs cg123
		annotated.probes<-do.call("c", lapply(probe.annotation, function(x) mcols(x)[["Name"]]))
		probe.id.column.name = "Name" ## For EPICv2 annotation
		is.combining.epicv2.arrays = TRUE
	}
	if(length(which(probes %in% annotated.probes))<2){
		err<-"Annotations could be found for less than two rows from the supplied beta value table"
		rnb.error(err)		
	}
	
	if(!all(probes %in% annotated.probes)){
		warn<-"Some of the supplied probes are missing annotation and will be discarded"
		rnb.warning(warn)			
	}
	
	x.data<-rnb.annotation2data.frame(probe.annotation)
	if (!is.combining.epicv2.arrays) {
		rownames(x.data)<-annotated.probes
	}
	rm(annotated.probes)
	
	#site.ids<-character()
	site.ids<-list()
	p.infos <- lapply(unique(x.data[["Chromosome"]]), 
			function(chr) {
				chr.map<-which(x.data[["Chromosome"]]==chr)
				chr.ids<-x.data[chr.map, probe.id.column.name]
				table<-match(probes, chr.ids)
				present<-!is.na(table)
				po<-order(table[present])
				site.ids[[chr]]<<-which(present)[po]
				table<-table[present][po]
				matrix(c(
						as.integer(x.data[chr.map[table], "Context"]), 
						match(x.data[chr.map[table],"Chromosome"], names(chrs)), 
						table),
						ncol = 3, 
						dimnames = list(x.data[chr.map[table],probe.id.column.name], c("context", "chr", "index")))
			})
	
#	p.infos<-matrix(c(as.integer(x.data[site.ids, "Context"]), match(x.data[site.ids,"Chromosome"], names(chrs)), unlist(p.indices)),
#			ncol = 3, dimnames = list(site.ids, c("context", "chr", "index")))
	
	p.infos<-do.call("rbind", p.infos)
	site.ids<-do.call("c", site.ids)
			
	rownames(p.infos)<-probes[site.ids]

	fm <- FALSE
	if (length(site.ids)==nrow(x.data)){
		fm <- all(site.ids==1:nrow(x.data))
	}
	
	return(list(probe.infos=p.infos, ids=site.ids, full.match=fm))
	
}

########################################################################################################################
## cellTypeAdjustment.R
## created: 2014-04-01
## creator: Pavlo Lutsik
## ---------------------------------------------------------------------------------------------------------------------
## A set of routines for cell type heterogeneity adjustment.
########################################################################################################################

#' refFreeEWASP
#'
#' NOTE: This function is deprecated, since the RefFreeEWAS package is not supported and available anymore
#' Applies the reference-free cell-type heterogeneity adjustment model from [1] and 
#' returns corrected p-values
#' 
#' @param X Matrix on which the test is performed for every row
#' @param inds.g1 column indices of group 1 members
#' @param inds.g2 column indices of group 2 members
#' @param adjustment.table a \code{data.frame} containing variables to adjust for in the testing
#' @param paired should a paired analysis model be used. If so, the first index in \code{inds.g1} must correspond to the first
#'               index in  \code{inds.g2} and so on.
#' @param nboot The number of bootstrapping resamples 
#' @param ignore.na in this case all \code{NA} containing rows are removed 
#' @param rescale.residual rescale the residual matrix as z-scores
#' 
#' @return vector of p-values for the "adjusted" regression coefficients from the Reference-free EWAS model
#' 
#' @note Requires the package \pkg{RefFreeEWAS}.
#' 
#' @references 1. Houseman, E. Andres, John Molitor, and Carmen J. Marsit. "Reference-Free Cell Mixture Adjustments in Analysis of DNA Methylation Data." 
#' Bioinformatics (2014): btu029.
#' 
#' @author Pavlo Lutsik
#' @export
refFreeEWASP <- function(
		X,
		inds.g1,
		inds.g2=-inds.g1, 
		adjustment.table=NULL, 
		paired=FALSE, 
		nboot=100,
		ignore.na=TRUE,
		rescale.residual=TRUE) {

    rnb.warning(paste("This function is deprecated, since the RefFreeEWAS package is not supported and available anymore.",
        "If you have a local version of the package, check out the branch RefFreeEWAS in the RnBeads github https://github.com/epigen/RnBeads"))
#	rnb.require("RefFreeEWAS")
#	if (paired && utils::packageVersion("RefFreeEWAS")<'1.3'){
#		rnb.warning("RefFreeEWAS version >=1.3 is required for paired analysis, pairing will be disregarded")
#		paired<-FALSE
#	}
#	## TODO: Validate parameter values
#	if(rescale.residual){
#		ranges<-apply(X,1,range,na.rm=TRUE)
#		range.diff<-apply(ranges, 2, function(rng) rng[2]-rng[1])
#		if(any(range.diff==0)){
#			rescale.residual<-FALSE
#			rnb.warning("Found non-variable rows in the methylation data, residual scaling was disabled")
#		}
#	}
#	
#	ind.vec <- c(inds.g1,inds.g2)
#	if (length(ind.vec) < 2) rnb.error("need at least two samples indices to compare")
#	X <- X[,ind.vec,drop=FALSE]

#	rnb.logger.start("Fitting the reference-free EWAS model")

#	if (is.logical(inds.g1)) inds.g1 <- which(inds.g1)
#	if (is.logical(inds.g2)) inds.g2 <- which(inds.g2)
#	n.g1 <- length(inds.g1)
#	n.g2 <- length(inds.g2)

#	ncgs<-nrow(X)
#	if(ignore.na){
#		nnas<-apply(is.na(X), 1, sum)
#		notna.rows<-which(nnas==0)
#		X<-X[notna.rows,]
#	}else{
#		nnas<-rep(0,ncgs)
#	}
#	
#	design<-matrix(1L, ncol(X), 2)
#	design[inds.g2,2]<-0L
#	colnames(design)<-c("(Icept)", "group.f")

#	if(!is.null(adjustment.table)){
#		formula.text <- paste0(c("~0",colnames(adjustment.table)),collapse="+")
#		design.adj <- model.matrix(as.formula(formula.text),data=adjustment.table)
#		design<-cbind(design[,-1,drop=FALSE], design.adj)
#		rnb.warning("Adjusting for covariates in the RefFreeEWAS model is an experimental feature, use with caution.")		
#	}

#	tmpBstar <- (X %*% design %*% solve(t(design)%*%design))
#	
#	## rescaling the residuals, and addition by Andres
#	R <- X-tmpBstar %*% t(design)
#	if(rescale.residual){
#		R <- t(scale(t(R)))
#	}
#	d<-EstDimRMT(R)$dim
#	
#	rnb.info(c("Estimated number of latent components is", d))
#	
#	test <- RefFreeEwasModel(X, design, d)

#	rnb.status("Fitted the RefFreeEWAS model")
#	
#	if(paired){
#		pair.id <- rep(1:n.g1, 2)[order(c(inds.g1,inds.g2))]
#		testBoot <- PairsBootRefFreeEwasModel(test, nboot, pair.id)
#	}else{
#		testBoot <- BootRefFreeEwasModel(test,nboot)
#	}
#	rnb.status("Pefrormed the bootstrap")
#	
#	smry<-summary(testBoot)
#	rnb.status("Summarized the results")

#	if(!is.null(adjustment.table)){
#		tstatBeta<-smry[,1,1,1]/smry[,1,1,2]
#	}else{
#		tstatBeta<-smry[,2,1,1]/smry[,2,1,2]
#	}

#	# proper df calculation, added by Andres
#	pvals <- pt(-abs(tstatBeta), df=nrow(design)-ncol(design)-nnas)
#		
#	rnb.logger.completed()
#	
#	if(ignore.na){
#		notna.pvals<-pvals
#		pvals<-rep(NA,ncgs)
#		pvals[notna.rows]<-notna.pvals
#	}
#	
#	return(pvals)
    return(NA)
}

#######################################################################################################################

#' estimateProportionsCP
#' 
#' Estimates cell type proportions using the constrained projection method from Houseman et al. [1]
#' 
#' @param rnb.set			RnBSet object
#' @param cell.type.column	integer index or character identifier of a column
#' 							in the RnBSet object sample annotation table 
#' 							which gives the mapping to reference cell type
#' 							samples
#' @param n.most.variable	Singleton integer specifying how many top variable CpGs should be used for marker selection.
#' 							If this option is set to \code{NA} or \code{NULL}, all sites are considered. Please take
#'                          into account the extended computation time in such a case.
#' @param n.markers			singleton integer specifying how many CpGs should
#' 							be used as markers for fitting the projection model
#' @param constrained		if \code{TRUE} the returned cell type proportion estimates
#' 							are non-negative
#' @param full.output 		if \code{TRUE} not only the estimated proportions 
#' 							but also the intermediate analysis results are returned
#'	
#' @return a matrix of estimated cell type contributions (samples times cell types) or a list with results of the intermetidate steps (see details).
#' 					
#' @details This is a minimally customized implementation of the method by Houseman et al. [1] based on the orginial code kindly provided by 
#' Andres Houseman. Note that RnBeads does not provide any reference data sets, and the methylomes of purified cell types should be provided 
#' by the user as a part of the object supplied via \code{rnb.set}. The column specified by \code{cell.type.column} should 
#' give assignment of each reference methylome replicate to a cell type and missing values for all the target samples. 
#' First the marker selection model is fit to estimate association of each CpG with the given reference cell types (first expression in eq. (1) of [1]). 
#' The strength of association is expressed as an F-statistic.
#' Since fitting the marker selection model to all CpGs can take a lot of time, one can limit the marker search only to variable CpG positions
#' by setting \code{n.most.variable} to non-\code{NA} positive integer. The CpGs will be ranked using across-sample variance in the 
#' reference data set and \code{n.most.variable} will be taken to fit the marker selection model.
#' Coefficients of the fit, together with the F-statistic value for each CpG, are returned in case \code{full.output} is \code{TRUE}. 
#' Thereafter, \code{n.markers} are selected as true quantitative markers and the projection model (eq. [2]) is fit to estimate contributions of each cell type.
#' Depending on the value of \code{constrained} the returned coefficients can be either raw or enforced to attain values between 0 and 1 with within-sample sum 
#' less or equal to 1.
#' 
#' @note Requires the package \pkg{nlme}.
#' 
#' @references 1. Houseman, Eugene and Accomando, William and Koestler, Devin and Christensen, Brock and Marsit, Carmen and Nelson, 
#' 	Heather and Wiencke, John and Kelsey, Karl. DNA methylation arrays as surrogate measures of cell mixture distribution. 
#'  BMC Bioinformatics 2012, 13:86
#' 		
#' @author Pavlo Lutsik 
#' @export
estimateProportionsCP<-function(
		rnb.set, 
		cell.type.column, 
		n.most.variable=NA, 
		n.markers=500L, 
		constrained=TRUE, 
		full.output=FALSE){

	rnb.require("nlme")

	if(!inherits(rnb.set, "RnBSet")){
		rnb.error("invalid value for rnb.set: object of class RnBSet is expected")
	}
	
	if(!is.integer(cell.type.column) && !is.character(cell.type.column)){
		rnb.error("invalid value for cell.type.column: integer or character singleton is expected")		
	}
	
	if(is.integer(cell.type.column) && 
			(length(cell.type.column)!=1 || cell.type.column<0L || cell.type.column>ncol(pheno(rnb.set)))){
		rnb.error("invalid value for cell.type.column: integer index is out of bounds")		
	}	
	
	if(is.character(cell.type.column) && 
			(length(cell.type.column)!=1 || !cell.type.column %in% colnames(pheno(rnb.set)))){
		rnb.error("invalid value for cell.type.column: integer index is out of bounds")		
	}
	
	if(length(which(is.na(pheno(rnb.set)[[cell.type.column]])))==0){
		stop("invalid value for cell.type.column: should contain some missing values to define the target samples")
	}
	if (is.null(n.most.variable)) {
		n.most.variable <- NA_integer_
	}
	
	if(full.output){
		result<-list()
	}
	
	if(full.output){
		result$cell.type.column.name=cell.type.column
		result$cell.type.column=pheno(rnb.set)[,cell.type.column]
	}
	if (is.factor(pheno(rnb.set)[,cell.type.column])) {
		cell.types <- table(pheno(rnb.set)[,cell.type.column])
		cell.types <- names(cell.types)[cell.types != 0]
	} else {
		cell.types <- unique(na.omit(as.character(pheno(rnb.set)[,cell.type.column])))
	}

	if(length(cell.types)<2){
		stop("Found less than two reference cell types")
		if(full.output){
			return(result)
		}else{
			return(NULL)
		}
	}
	
	if(all(table(na.omit(pheno(rnb.set)[,cell.type.column]))==1)){
		stop("Found only one sample per each cell type")
		if(full.output){
			return(result)	
		}else{
			return(NULL)
		}
	}
	
	if(full.output){
		result$all.cell.types<-cell.types	
	}
	
	# Omit the last cell type as a referent, to avoid singularities during the model fitting 
	#referent.ct<-cell.types[length(cell.types)]
	#cell.types<-cell.types[-length(cell.types)]
	
#	if(full.output){
#		result$used.cell.types<-cell.types
#		#result$referent<-referent.ct
#	}
	
	# Load test data
	#load("Example-WBC-Data.RData")
	
	# Define validation model
	cell.types.fla<-paste("ct", 1:length(cell.types), sep="_")
	theModel = paste("y", paste(cell.types.fla, collapse="+"), sep="~")
	# remove the intercept term
	theModel = as.formula(paste(theModel, "1", sep="-"))
	theZeroModel = as.formula("y ~ 1")
	
	
	#sizeModel = length(cell.types)+1
	sizeModel = length(cell.types)
	
	mm<-meth(rnb.set, row.names=TRUE)
	nsites<-nrow(mm)
	if(is.na(n.most.variable)){
		M <- nrow(mm) # Number of CpGs on array (test data is a subset of 27K)
		validationData<-mm[,!is.na(pheno(rnb.set)[,cell.type.column])]
		targetData<-mm[,is.na(pheno(rnb.set)[,cell.type.column])]
	}else{
		if(n.most.variable<2 || n.most.variable>nrow(mm)){
			stop("invalid value for n.most.variable: either less than 2 or 
							more than CpGs in the RnBSet object")
		}
		M<-n.most.variable
		# get ranking by SD only based on the reference-methylome data 
		#sds<-apply(mm, 1, function(mmrow) sd(mmrow[!is.na(pheno(rnb.set)[,cell.type.column])]))
		sds<-apply(mm[,!is.na(pheno(rnb.set)[,cell.type.column])], 1, sd)
		
		cands<-order(sds, decreasing=TRUE)[1:n.most.variable]
		validationData<-mm[cands,!is.na(pheno(rnb.set)[,cell.type.column])]
		targetData<-mm[cands,is.na(pheno(rnb.set)[,cell.type.column])]
		if(full.output){
			result$most.variable<-cands
		}
	}
	rm(mm)
	# select most variable CpGs 
	
	cell.type.f<-as.character(pheno(rnb.set)[!is.na(pheno(rnb.set)[,cell.type.column]),cell.type.column])
	cell.type.contrasts<-sapply(cell.types, function(ct) as.integer(cell.type.f==ct))
	colnames(cell.type.contrasts)<-cell.types.fla
	plate.col<-grep("Sentrix[| |_]*ID", colnames(pheno(rnb.set)))
	
	if(length(plate.col)==1){
		validationData_Pheno<-data.frame(cell.type.contrasts)
		validationData_Pheno[["PLATE"]]<-pheno(rnb.set)[!is.na(pheno(rnb.set)[,cell.type.column]),plate.col]
	}else{
		validationData_Pheno<-data.frame(cell.type.contrasts)
	}
	
	#################### Detection of the marker set and estimation of the coefficients
	
	# Linear transformation of coefficient vector
	#  representing contrast to test F statistic
	#L.forFstat <- diag(sizeModel)[-1,,drop=FALSE]  #All non-intercept coefficients
	L.forFstat <- diag(sizeModel)[,drop=FALSE]
	
	# Initialize various containers
	sigmaResid <- sigmaIcept <- nObserved <- nClusters <- Fstat <- rep(NA, M)
	coefEsts <- matrix(NA, M, sizeModel)
	coefVcovs <- list()
	
	for(j in 1:M){ # For each CpG
		
		#Remove missing methylation values
		ii <- !is.na(validationData[j,])
		nObserved[j] <- sum(ii)
		validationData_Pheno[["y"]] <- validationData[j,]
		
		#if(j%%10==0) cat(j,"\n") # Report progress
		
		try({ # Try to fit a mixed model to adjust for plate
					if(length(plate.col)==1){
						fit <- try(lme(theModel, random=~1|PLATE, data=validationData_Pheno[ii,]), silent=TRUE)
						fit0 <- try(lme(theZeroModel, random=~1|PLATE, data=validationData_Pheno[ii,]), silent=TRUE)
					}
					
					if(length(plate.col)!=1 || inherits(fit,"try-error")){ # If LME can't be fit, just use OLS
						fit <- lm(theModel, data=validationData_Pheno[ii,])
						fit0 <- lm(theZeroModel, data=validationData_Pheno[ii,])
						
						fitCoef <- fit$coef
						sigmaResid[j] <- summary(fit)$sigma
						sigmaIcept[j] <- 0L
						nClusters[j] <- 0L
					}else{ 
						fitCoef <- fit$coef$fixed
						sigmaResid[j] <- fit$sigma
						sigmaIcept[j] <- sqrt(getVarCov(fit)[1])
						if(length(plate.col)==1){
							nClusters[j] <- length(fit$coef$random[[1]])
						}else{
							nClusters<-0L
						}
					}
					coefEsts[j,] <- fitCoef
					coefVcovs[[j]] <- vcov(fit)
					
#					useCoef <- L.forFstat[!is.na(fitCoef)[-1],!is.na(fitCoef),drop=FALSE] %*% fitCoef[!is.na(fitCoef)]
					#useCoef <- L.forFstat[!is.na(fitCoef),!is.na(fitCoef),drop=FALSE] %*% fitCoef[!is.na(fitCoef)]
#					useV <- L.forFstat[!is.na(fitCoef)[-1],!is.na(fitCoef),drop=FALSE] %*% coefVcovs[[j]] %*% t(L.forFstat[!is.na(fitCoef)[-1],!is.na(fitCoef),drop=FALSE])
					#useV <- L.forFstat[!is.na(fitCoef),!is.na(fitCoef),drop=FALSE] %*% coefVcovs[[j]] %*% t(L.forFstat[!is.na(fitCoef),!is.na(fitCoef),drop=FALSE])
					#Fstat[j] <- (t(useCoef) %*% solve(useV, useCoef))/sizeModel
					rss<-sum(residuals(fit)^2)
					rss0<-sum(residuals(fit0)^2)
					Fstat[j]<-((rss0-rss)/(sizeModel-1))/(rss/(nObserved[j]-sizeModel))
				})
	}
	
	# Name the rows so that they can be easily matched to the target data set
	rownames(coefEsts) <- rownames(validationData)
	#colnames(coefEsts) <- c(names(fitCoef)[1], cell.types)
	colnames(coefEsts) <- c(cell.types)
	
	# Get P values corresponding to F statistics
	Pval <- 1-pf(Fstat, sizeModel, nObserved - nClusters - sizeModel + 1)
	
	if(full.output){
		result$f.stat<-Fstat
		result$f.pval<-Pval
		result$coef.ests<-coefEsts
	}
	
	## order by F statistic since p values become indiscriminatory
	#
	inds<-order(Fstat, decreasing=TRUE)[1:n.markers]
	
	if(full.output){
		if(is.na(n.most.variable)){
			result$markers<-(1:nsites)[inds]
		}else{
			result$markers<-result$most.variable[inds]
		}
		result$n.markers<-n.markers		
	}
	
	# NOTE:  test data consists of CpGs having 500 largest F statistics
	#  in descending order of magnitude
	# For the full 27K array, it is necessary to sort by Pvalue and/or Fstatistic
	#  and choose the top CpGs
	#table(sign(diff(Fstat))) # Fstats decrease as j increases from 1 to M
	
	
	##################### Estimation of the cell type contributions 
	
#	valid.cts<-(colSums(is.na(coefEsts))==0)[-1L]
#	
#	if(full.output){
#		result$valid.cell.types<-valid.cts
#	}
	
	Lwbc <- diag(sizeModel) 
	#Lwbc <- diag(sizeModel)[-1L,,drop=FALSE]
	#Lwbc[,1] <- 1
	rownames(Lwbc) <- colnames(coefEsts)
	colnames(Lwbc) <- colnames(coefEsts)
	
	#Lwbc # View contrast matrix
	
	# CpGSelection = rownames(coefEsts)[inds] 
	# Use the top 100
	# Note:  if the CpGs were scattered throughout the array,
	#    you would want to select them by name as is indicated here.
	#    For this sample version, it would be easier just to use
	#    "[1:100]"
	
	
	####### Projections 
	
	if(full.output){
		
		coefsList<-list()
		
		for(constri in 1:2){
			coefsList[[constri]]<- projectWBC(
					targetData[inds,],
					coefEsts[inds,,drop=FALSE],    
					Lwbc, 
					nonnegative=c(FALSE, TRUE)[constri])
		}
		
		result$contributions<-coefsList[[1]]
		result$contributions.nonneg<-coefsList[[2]]
		
	}else{
		
		coefs <-  projectWBC(
				targetData[inds,],
				coefEsts[inds,,drop=FALSE],    
				Lwbc, nonnegative = constrained)
		
#		if(ncol(coefs)<length(cell.types) && constrained && normalized){
#			
#			pre.coefs<-coefs
#			coefs<-matrix(NA,nrow(pre.coefs),length(cell.types))
#			coefs[,valid.cts]<-pre.coefs/max(rowSums(pre.coefs))
#			coefs[,!valid.cts]<-(1-rowSums(pre.coefs)/(max(rowSums(pre.coefs))))/length(which(!valid.cts))
#			colnames(coefs)<-cell.types
#			rownames(coefs)<-rownames(pre.coefs)
#		}
		
	}
	
	if(full.output){
		return(result)
	}
	
	return(coefs)
}

#######################################################################################################################

#' rnb.plot.marker.fstat
#'
#' Plot the the cell type marker selection based on the reference methylome data  
#' 
#' @param ct.object   Object of class \code{CellTypeInferenceResult} as returned by \link{rnb.execute.ct.estimation}.
#' @param writeToFile If \code{TRUE}, the plot will be written to a file.
#' @param ...         Other arguments to \code{\link{createReportPlot}}.
#' 
#' @details	The F-statistic values from the cell type association model (first part of eqn. (1) in [1]) are plotted in decreasing order 
#' 			for all tested CpG positions. A vertical line gives a cut-off for the number of selected cell type markers.
#' 
#' @return				if \code{writeToFile=TRUE} an object of class \code{\linkS4class{ReportPlot}}, 
#' 						and the plotted reordered F-statistics vector otherwise  
#' 
#' @references  1. Houseman, Eugene and Accomando, William and Koestler, Devin and Christensen, Brock and Marsit, Carmen and Nelson, 
#' 	Heather and Wiencke, John and Kelsey, Karl. DNA methylation arrays as surrogate measures of cell mixture distribution. BMC Bioinformatics 2012, 13:86
#' 
#' @author Pavlo Lutsik
#' @export
rnb.plot.marker.fstat <- function(ct.object, writeToFile=FALSE, ...) {
	
	if(class(ct.object)!="CellTypeInferenceResult"){
		stop("invalid value for ct.object; expected CellTypeInferenceResult")
	}
	if(!isTRUE(ct.object$method == "houseman1")){
		stop("unsupported method in ct.object")
	}
	if (!parameter.is.flag(writeToFile)) {
		stop("invalid value for writeToFile; expected TRUE or FALSE")
	}

	if(writeToFile){
		plot.file<-createReportPlot("CellTypeMarkerFstatPlot", ...)
	}
	nna<-!is.na(ct.object$f.stat)
	dframe <- data.frame(x = 1:sum(nna), y = sort(ct.object$f.stat[nna], decreasing = TRUE))
	print(ggplot2::ggplot(dframe, aes(x = x, y = y)) + ggplot2::geom_point() +
		ggplot2::geom_vline(xintercept = ct.object$n.markers, col = muted("blue")) +
		ggplot2::labs(x = "CpGs", y = "F statistic"))

	if(writeToFile) {
		plot.file <- off(plot.file)
		return(plot.file)
	}
	return(invisible(dframe$y))
}

#######################################################################################################################

#' rnb.plot.ct.heatmap
#'
#' Plot contributions of the cell types  
#' 
#' @param ct.obj		Object of class \code{CellTypeInferenceResult} as returned by \link{rnb.execute.ct.estimation}.
#' @param type			Type of cell type contributions to plot.
#' @param writeToFile	If \code{TRUE}, the plot will be written to a file.
#' @param ...			Other arguments passed to \code{\link{createReportPlot}}.
#' 
#' @details				The cell type contributions are visualized as a heatmap
#' 
#' @return				if \code{writeToFile=TRUE} an object of class \code{\linkS4class{ReportPlot}}, 
#' 						or the protted matrix otherwise   
#' 
#' @author Pavlo Lutsik
#' @export
rnb.plot.ct.heatmap<-function(ct.obj, type="nonnegative", writeToFile=FALSE, ...) {
	
	if(class(ct.obj)!="CellTypeInferenceResult"){
		stop("Invalid value for ct.obj")
	}
	
	if(ct.obj$method!="houseman1"){
		stop("This plot requires the ct.obj obtained usign the houseman1 method")
	}
	
	
	if(type=="raw"){
		tbl<-ct.obj$contributions
	}else if(type=="nonnegative"){
		tbl<-ct.obj$contributions.nonneg
	}
	
	dframe <- data.frame(
		x = factor(rep(colnames(tbl), each = nrow(tbl)), levels = colnames(tbl)),
		y = factor(rep(rownames(tbl), ncol(tbl)), levels = rev(rownames(tbl))),
		v = as.vector(tbl))
	pp <- ggplot(dframe, aes(x, y, fill = v)) +
		geom_tile(colour = "white") +
		scale_fill_gradient(low ="white", high = "steelblue") +
		labs(x = NULL, y = NULL, fill = "Contribution") +
		scale_x_discrete(expand = c(0, 0)) +
		scale_y_discrete(expand = c(0, 0)) +
		theme(panel.background = element_blank(), panel.grid = element_blank()) +
		theme(panel.border = element_blank(), axis.line = element_blank(), axis.ticks = element_blank()) +
		theme(axis.text.x = element_text(size = 8, angle = 330, hjust = 0)) +
		theme(legend.position = c(1, 0.5), legend.justification = c(0, 0.5)) +
		theme(plot.margin = unit(0.1 + c(0, 1, 0, 0), "in"))

	if(writeToFile) {
		pp <- suppressWarnings(ggplot_gtable(ggplot_build(pp)))
		pp$widths[[3]] <- unit(2, "in")
		pp$heights[[length(pp$heights) - 2L]] <- unit(1, "in")

		fun.args <- list(...)
		if (!("fname" %in% names(fun.args))) {
			fun.args$fname <- "CellTypeContributionsPlot"
		}
		if (!("height" %in% names(fun.args))) {
			fun.args$height <- 1.2 + nrow(tbl)* 0.15
		}
		if (!("width" %in% names(fun.args))) {
			fun.args$width <- 3.2 + ncol(tbl) * 0.6
		}
		plot.file <- base::do.call(createReportPlot, fun.args)
		grid.newpage()
		grid.draw(pp)
		return(off(plot.file))
	}
	return(pp)
}

#######################################################################################################################
 
#' rnb.execute.ct.estimation
#'
#' Perform the estimation of the cell type contributions in each analyzed sample.
#' 
#' @param rnb.set			object of class \code{\linkS4class{RnBSet}}
#' @param cell.type.column	integer index or character identifier of a column in sample annotation table of \code{rnb.set}
#' 							which gives the mapping of samples to reference cell types 
#' @param test.max.markers	Maximal amount of CpG positions to use for marker selection. If this option is set to
#' 							\code{NA} or \code{NULL}, all sites are considered. Please take into account the extended
#'                          computation time in such a case.
#' @param top.markers		the number of markers to select 
#' @param method 			algorithm used for estmation of the cell type contributions
#' @param verbose			flag specifying whether diagnostic output should be written to the console 
#' 							or to the RnBeads logger in case the latter is initialized
#' 
#' @details The only supported method is the one from Houseman et al BMC Bioinformatics 2012 
#' 
#' @return object of class \code{CellTypeInferenceResult} 
#' 
#' @author Pavlo Lutsik
#' @export
rnb.execute.ct.estimation<-function(rnb.set, 
		cell.type.column=NA,
		test.max.markers=NA,
		top.markers=500,
		method="houseman1",
		verbose=TRUE){
		
	if(method=="houseman1"){
		result<-estimateProportionsCP(rnb.set, 
				cell.type.column, 
				n.most.variable=test.max.markers,
				n.markers=top.markers, full.output=TRUE)
	}else{
		rnb.warning("the supplied inference method is not supported yet")
		return(NULL)
	}
	
	result$method <- method
	class(result)<-"CellTypeInferenceResult"
	return(result)
}

#######################################################################################################################
#
# rnb.section.ct.estimation
#
#
rnb.section.ct.estimation<-function(report, ct.object){
		
	report <- rnb.add.section(report, "Estimation of Cell Type Heterogeneity Effects", NULL)
	
	if(!is.null(ct.object$cell.type.column.name)){
		
		report <- rnb.add.section(report, "Reference methylomes", NULL, level=2)
		
		intro <- c("The dataset contained reference methylomes defined by the sample annotation column <i>",
			ct.object$cell.type.column.name, "</i>.")

		if(length(which(!is.na(ct.object$cell.type.column)))>2){
			intro <- c(intro, " Detected reference methylomes were are summarized in the table below.")
		}
		ref.meth.table <- data.frame(table(na.omit(ct.object$cell.type.column)))
		colnames(ref.meth.table) <- c("Cell type", "Number of samples")

		rnb.add.paragraph(report, intro)

		rnb.add.table(report, ref.meth.table, row.names = FALSE, first.col.header = TRUE)
		
	}
			
	# plot the marker distribution
	if(ct.object$method=="houseman1"){
		
		report <- rnb.add.section(report, "Cell type contributions", NULL, level=2)

		refText <- c("Houseman, E. (2012) DNA methylation arrays as surrogate measures of cell mixture distribution. ",
			"<i>BMC Bioinformatics</i>, <b>13</b>(86)")
		report <- rnb.add.reference(report, refText)

		rnb.add.paragraph(report, c("The contributions of cell types were estimated using the method by Houseman <i>",
			"et al</i> ", rnb.get.reference(report, refText), "."))

		markers.text <- c("In the first step the reference methylomes were used to estimate the association of each ",
			"CpG position to each of the cell types. The stength of association was measured using F-test. ")
		ctMarkers <- rnb.getOption("inference.max.cell.type.markers")
		if(!(is.null(ctMarkers) || is.na(ctMarkers))){
			markers.text <- c(markers.text, "To decrease the computation load, only ", ctMarkers, " most variable ",
				"CpGs were considered. ")
		}
		n.markers <- ct.object$n.markers # rnb.getOption("inference.top.cell.type.markers")
		if (n.markers < length(ct.object$f.stat)) {
			borders <- sort(ct.object$f.stat, decreasing = TRUE)[0:1 + n.markers]
			i <- 10^(-1:4)
			cutoff <- floor(borders[1] * i) / i
			i <- which(cutoff > borders[2])
			cutoff <- base::ifelse(length(i) == 0, paste("approximately", cutoff[length(cutoff)]), cutoff[i[1]])
			rm(borders, i)
		} else { # n.markers == length(ct.object$f.stat)
			cutoff <- floor(min(ct.object$f.stat))
		}

		markers.text <- c(markers.text, "Finally, only ", n.markers, " CpGs ",
			"with the lowest F-test p-value were used in the contribution estimation. The plot below visualizes the ",
			"distribution of F statistic values for all tested CpGs. Note that selecting the most informative CpGs is ",
			"equivalent to applying an F statistic cut-off of ", cutoff, ".")
		report <- rnb.add.section(report, "Selection of the cell type markers", markers.text, level = 3)

		## Add a plot of F statistic values
		fstat.plot <- rnb.plot.marker.fstat(ct.object, writeToFile = TRUE, report = report)
		txt <- c("Scatter plot visualizing the F statistic of the cell type association model for each CpG position ",
			"from the tested subset. The vertical blue line, if present, reflects the selection of ", n.markers,
			" best markers for the projection.")
		report <- rnb.add.figure(report, txt, fstat.plot)
		rm(n.markers, cutoff)

		## Export the normalized (non-negative) contributions
		fname <- 'contributions_houseman1.csv'
		write.csv(ct.object$contributions.nonneg, file = file.path(rnb.get.directory(report, 'data', TRUE), fname))
		txt <- c("After the marker selection, a projection of the target data onto the space of the marker selection ",
			"model coefficients yields contributions of each reference cell type to each measured DNA methylation ",
			'profile. The resulting cell type contributions are available in a dedicated <a href="',
			rnb.get.directory(report, 'data'), '/', fname, '">comma-separated file</a> accompanying this report. These ',
			'values are also displayed in the heatmap below.', " The contributions are constrained to be greater or equal ",
			"to zero, and the per-sample sums are expected to be close to one, i.e. they are estimates of the cell type ",
			"proportions. Per-sample totals much larger than one may indicate the problems with the procedure, ",
			"e.g. bad correspondence of the target data to the reference methylomes, significant batch effects etc.")
	
		report <- rnb.add.section(report, "Cell type contributions via the coefficient projection", txt, level = 3)

		## Add a heatmap of normalized (non-negative) contributions
		contrib.plot <- rnb.plot.ct.heatmap(ct.object, writeToFile = TRUE, report = report)		
		txt <- "Heatmap visualizing estimated cell type contributions, scaled to the range [0, 1]."
		report <- rnb.add.figure(report, txt, contrib.plot)
	}
	
	return(report)
}

#######################################################################################################################
#
# rnb.step.cell.types
#
#
rnb.step.cell.types<-function(rnb.set, report){
	
	logger.start("Estimation of the cell type heterogeneity effects")
	
	cell.type.column<-rnb.getOption("inference.reference.methylome.column")
	
	if(is.na(cell.type.column)){
		rnb.warning("Invalid value for cell.type.column, non-NA value required. Skipped the cell type heterogeneity analysis")
		result<-NULL
		new.report<-report
	}else if((is.integer(cell.type.column) && (cell.type.column<0 || cell.type.column>length(samples(rnb.set))))){
		rnb.warning("Invalid value for cell.type.column, integer index not in permissible range. Skipped the cell type heterogeneity analysis")
		result<-NULL
		new.report<-report
	}else if(is.character(cell.type.column) && !cell.type.column %in% colnames(pheno(rnb.set))){
		rnb.warning("Invalid value for cell.type.column, specified column not found. Skipped the cell type heterogeneity analysis")
		result<-NULL
		new.report<-report
	}else{

		logger.start("Performing computations")
			result<-rnb.execute.ct.estimation(rnb.set, 
					cell.type.column=rnb.getOption("inference.reference.methylome.column"),
					test.max.markers=rnb.getOption("inference.max.cell.type.markers"), 
					top.markers=rnb.getOption("inference.top.cell.type.markers"),
					method="houseman1")
		logger.completed()
		
		logger.start("Adding a section to the report")
			new.report<-rnb.section.ct.estimation(report, result)
		logger.completed()
		
	}
	logger.completed()
	
	if(!is.null(result)){
		rnb.set<-set.covariates.ct(rnb.set, result)
		logger.info("Added cell type covariates to the RnBSet object")
	}
	return(list(rnb.set=rnb.set, report=new.report))
}

#######################################################################################################################

#' set.covariates.ct
#'
#' Adds the results of cell type estimation to an RnBSet
#' 
#' @param rnb.set The \code{RnBSet} object to which the results should be added
#' @param ct.obj An object of class \code{CellTypeInferenceResult} returned by \code{rnb.execute.ct.estimation}.
#' 
#' @return The modified \code{RnBSet}.
#' 
#' @export set.covariates.ct
set.covariates.ct <- function(rnb.set, ct.obj) {
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	if (!.hasSlot(rnb.set,"inferred.covariates")) { #.hasSlot ensure backwards compatibility
		rnb.set@inferred.covariates<-list()
	}
	if(is.null(rnb.set@inferred.covariates)){
		rnb.set@inferred.covariates<-list()
	}

	if (!is.null(ct.obj)) {
		## FIXME: Validate ct.obj and its elements
		contributions <- matrix(as.double(NA), nrow = length(samples(rnb.set)), ncol = ncol(ct.obj$contributions),
			dimnames = list(colnames(rnb.set@meth.sites), paste0("ct_", 1:ncol(ct.obj$contributions))))
		i.samples <- which(is.na(pheno(rnb.set)[, ct.obj$cell.type.column.name]))
		contributions[i.samples, ] <- ct.obj$contributions
		attr(contributions, "column") <- ct.obj$cell.type.column.name
		attr(contributions, "cell types") <- colnames(ct.obj$contributions)
		rnb.set@inferred.covariates[["cell.types"]] <- contributions
	}

	return(rnb.set)
}

#######################################################################################################################

#' get.covariates.ct
#'
#' Retrieves an NxK matrix of cell type contributions stored in an RnBSet for a given target variable
#' 
#' @param rnb.set \code{RnBSet} object
#' 
#' @return an NxK matrix of K cell types contributions for N samples of the \code{rnb.set}. \code{NULL}
#'		   if the components have not been computed or added to \code{rnb.set}.
#' 
#' @export
get.covariates.ct <- function(rnb.set) {
	if (!.hasSlot(rnb.set,"inferred.covariates")) { #.hasSlot ensure backwards compatibility
		return(NULL)
	}
	rnb.set@inferred.covariates[["cell.types"]]
}

#######################################################################################################################
#' has.covariates.ct
#'
#' Checks whether the given \code{RnBSet} object contains cell type contribution estimates
#' 
#' @param rnb.set \code{RnBSet} object
#' 
#' @return \code{TRUE} if the supplied object contains the cell type covariates information and \code{FALSE} otherwise
#' 
#' @export
has.covariates.ct <- function(rnb.set) {
	if (!.hasSlot(rnb.set,"inferred.covariates")) { #.hasSlot ensure backwards compatibility
		return(FALSE)
	}
	return(!is.null(rnb.set@inferred.covariates[["cell.types"]]))
}
#######################################################################################################################

########################################################################################################################
## qualityControl.R
## created: 2012-05-31
## creator: Pavlo Lutsik
## ---------------------------------------------------------------------------------------------------------------------
##
########################################################################################################################

#' rnb.step.quality
#'
#' Cteates quality control plots section of the quality control report.
#'
#' @param rnb.set Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param report  Report on quality control to contain the generated sections. This must be an object of type
#'                \code{\linkS4class{Report}}.
#' @return The modified report.
#'
#' @author Pavlo Lutsik
#' @noRd
rnb.step.quality<-function(rnb.set, report){

	if (!inherits(rnb.set, "RnBSet")){
		stop("invalid value for rnb.set")
	}
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}

	if(inherits(rnb.set,"RnBeadSet")){
		covg.lists <- NULL
	}else{ # inherits(rnb.set,"RnBiseqSet")
		covg.lists<-rnb.execute.quality(rnb.set)
	}
	report<-rnb.section.quality(report, rnb.set, covg.lists=covg.lists)

	return(report)
}

########################################################################################################################

#' rnb.execute.quality
#'
#' Performs quality control calculations on the loaded DNA methylation data set.
#'
#' @param object            Methylation dataset as an object of class \code{\linkS4class{RnBeadSet}},
#'                          \code{\linkS4class{RnBeadRawSet}} or \code{\linkS4class{RnBiseqSet}}.
#' @param type              \code{character} vector of length \code{1} giving the type of genomic regions for which the
#' 				            quality control information is summarized.
#' @param qc.coverage.plots Flag indicating if sequencing coverage information is summarized and returned. This
#'                          parameter is considered only when \code{object} is of type \code{\linkS4class{RnBiseqSet}}.
#' @param verbose	        Flag specifying whether diagnostic output should be written to the console or to the
#'                          RnBeads logger in case the latter is initialized.
#'
#' @details Currently, summarizing coverage for \code{\linkS4class{RnBiseqSet}} object is the only available function.
#'
#' @return \code{\linkS4class{RnBeadSet}} object with imputed quality control information
#'
#' @author Pavlo Lutsik
#' @export
rnb.execute.quality<-function(
		object,
		type="sites",
		qc.coverage.plots=rnb.getOption("qc.coverage.plots"),
		verbose=TRUE){

  if(verbose){
  	rnb.logger.start("Preparing Quality Control Information")
  }

  if(inherits(object, "RnBiseqSet")){
	  if(qc.coverage.plots){
		  covg.rnbs<-covg(object, type)
		  covg.rnbs[is.na(covg.rnbs)]<-0

		  if(type=="sites"){
			  type<-"CpG"
			  map<-sites(object)
		  }else{
			  map<-regions(object, type)
		  }

		  covg.lists<-lapply(samples(object), function(sample){

		  sample.id<-match(sample, samples(object))
		  assembly <- assembly(object)
		  covg.list<-lapply(names(rnb.get.chromosomes(assembly=assembly)), function(chr) {
					  chr.id<-match(chr, names(rnb.get.chromosomes(assembly=assembly)))
					  covg.weight<-rep(0, rnb.annotation.size(assembly=assembly)[chr.id])
					  covg.weight[map[map[,2]==chr.id,3]]<-covg.rnbs[map[,2]==chr.id,sample.id]
					  covg.weight
				  })
	  	  })
		  names(covg.lists)<-samples(object)
		  result<-covg.lists
  	  }else{
		  result<-NULL
	  }
  }else{
	  result<-object
  }
  if(verbose){
  	logger.completed()
  }
  return(result)
}

#######################################################################################################################

#' rnb.section.quality
#'
#' Adds quality control probe or coverage section to the quality control report.
#'
#' @param report             Analysis report to contain the newly generated section. This must be an object of type
#'                           \code{\linkS4class{Report}}.
#' @param rnb.set            Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param qc.boxplots        Flag indicating if quality control box plots are to be generated. This option has effect
#'                           only when \code{rnb.set} is a HumanMethylation450 dataset.
#' @param qc.barplots        Flag indicating if quality control bar plots are to be generated. This option has effect
#'                           only when \code{rnb.set} is a HumanMethylation450 dataset.
#' @param qc.negativeboxplot Flag indicating if plots showing the negative control probes for each sample are to be
#'                           generated. This option has effect only when \code{rnb.set} is a
#'                           HumanMethylation450 dataset.
#' @param covg.lists         ...
#' @return The modified report.
#'
#' @author Pavlo Lutsik
#' @noRd
rnb.section.quality<-function(report, rnb.set, qc.boxplots=rnb.getOption("qc.boxplots"),
	qc.barplots=rnb.getOption("qc.barplots"), qc.negative.boxplot=rnb.getOption("qc.negative.boxplot"),
	qc.coverage.plots=rnb.getOption("qc.coverage.plots"), qc.coverage.histograms=rnb.getOption("qc.coverage.histograms"),
	qc.coverage.violins=rnb.getOption("qc.coverage.violins"),
	qc.coverage.threshold.plot=rnb.getOption("qc.coverage.threshold.plot"),covg.lists=NULL){

	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	if (!parameter.is.flag(qc.boxplots)) {
		stop("invalid value for qc.boxplots; expected TRUE or FALSE")
	}
	if (!parameter.is.flag(qc.barplots)) {
		stop("invalid value for qc.barplots; expected TRUE or FALSE")
	}
	if (!parameter.is.flag(qc.negative.boxplot)) {
		stop("invalid value for qc.negative.boxplots; expected TRUE or FALSE")
	}
	if (!parameter.is.flag(qc.coverage.plots)) {
		stop("invalid value for qc.coverage.plots; expected TRUE or FALSE")
	}
	if (!parameter.is.flag(qc.coverage.histograms)) {
		stop("invalid value for qc.coverage.histograms; expected TRUE or FALSE")
	}
	if (!parameter.is.flag(qc.coverage.violins)) {
		stop("invalid value for qc.coverage.violins; expected TRUE or FALSE")
	}
	## TODO: Add error handling for qc.coverage.threshold.plot
	## TODO: Add error handling for covg.lists

	logger.start("Quality Control Section")
	report <- rnb.add.section(report, "Quality Control", NULL)
	txt <- NULL

	if(inherits(rnb.set, "RnBeadSet")){
		if(is.null(qc(rnb.set))){

			txt <- c("The supplied dataset contains no quality information, therefore, quality control graphics could ",
				"not be generated.")
			rnb.add.paragraph(report, txt)
			logger.info("No quality information present in the dataset")

		}else{

			txt <- "This section contains quality control plots and statistics for the methylation data."
			rnb.add.paragraph(report, txt)

			if(qc.boxplots) {
				txt <- c("Each box plot below shows the signal distribution of quality control probes across all ",
					"samples. The control box plots are separated by control types. Detailed description of the ",
					"control probes is given in the RnBeads vignette.")
				report <- rnb.add.section(report, "Quality Control Box Plots", txt, level = 2)
				report<-add.qc.boxplots(report, rnb.set)
				logger.status("Added quality control box plots")
			}

			if(qc.barplots) {
				txt <- c("The plots below visualize the exact signal levels at each quality control probe. Note that ",
					"the scale is not standardized. Background signal is usualy at the level of 1000 to 2000.")
				report <- rnb.add.section(report, "Quality Control Bar Plots", txt, level = 2)
				report<-add.qc.barplots(report, rnb.set, sample.batch.size=rnb.getOption("qc.sample.batch.size"))
				logger.status("Added quality control bar plots")
			}

			if(qc.negative.boxplot){
				txt <- c("Negative control box plots visualize background intensity distributions of all analyzed ",
					"samples. Samples with skewed distributions and high medians are likely to be of low quality and ",
					"should be discarded from the analysis.")
				report <- rnb.add.section(report, "Negative Control Box Plots", txt, level = 2)
				report<-add.negative.control.boxplot(report, rnb.set, sample.batch.size=rnb.getOption("qc.sample.batch.size"))
				logger.status("Added negative control boxplots")
			}
		}

	}else{ #inherits(rnb.set, "RnBiseqSet")


		if(qc.coverage.plots){
			txt <- c("The sequencing coverage plots visualized effective read coverage at all CpGs in the sequenced ",
					"genome. In case certain samples seem to have significantly decreased coverage, they should be excluded ",
					"from the analysis.")
			report <- rnb.add.section(report, "Sequencing Coverage Plots", txt, level = 2)
			report<-add.seq.coverage.plot(report, rnb.set, covg.lists)
			logger.status("Added sequencing coverage boxplots")
		}

		if(qc.coverage.histograms){
			txt <- c("The sequencing coverage histograms show distribution of coverage across all chromosomes. In case ",
			"certain samples seem to have significantly decreased coverage, they should be excluded from the analysis.")
			report <- rnb.add.section(report, "Sequencing Coverage Histograms", txt, level = 2)
			report <- add.seq.coverage.histograms(report, rnb.set)
			logger.status("Added sequencing coverage histograms")
		}
		report <- add.seq.coverage.num.sites.covg.tabs(report, rnb.set)
		logger.status("Added sample coverage section")

		if(qc.coverage.violins){
			txt <- c("The plots below show an alternative approach to visualizing the coverage distribution.")
			report <- rnb.add.section(report, "Sequencing Coverage Violin Plots", txt, level = 2)
			report<-add.seq.coverage.violins(report, rnb.set)
			logger.status("Added sequencing coverage violin plots")
		}

		rnb.cleanMem()

		if (length(qc.coverage.threshold.plot) != 0) {
			rplot <- rnb.plot.coverage.thresholds(rnb.set, qc.coverage.threshold.plot, fname = "coverage_interrogated",
				report = report)
			dframe.coverages <- attr(rplot, "data")
			fname <- "coverage_interrogated.csv"
			write.csv(dframe.coverages, file = file.path(rnb.get.directory(report, "data", TRUE), fname), row.names = FALSE)
			txt <- sprintf("%1.1f", range(dframe.coverages[dframe.coverages[, 2] == max(dframe.coverages[, 2]), 3]))
			txt <- c('In total, between ', txt[1], ' and ', txt[2], ' million sites are covered in all samples of the ',
				'dataset. The figure below shows the change in supports for different coverage thresholds. The exact ',
				'values are available in a dedicated <a href="', rnb.get.directory(report, 'data'), '/', fname,
				'">comma-separated file</a> accompanying this report.')
			report <- rnb.add.section(report, "Sequencing Coverage Thresholds", txt, level = 2)
			txt <- c("Line plot showing the number of CpG sites with a given support for different thresholds of ",
				"minimal coverage. The support of a CpG site is the minimal number of samples that interrogate it.")
			report <- rnb.add.figure(report, txt, rplot)
			rm(rplot, dframe.coverages, fname)
		}
	}

	if (is.null(txt)) {
		txt <- "No quality control plots are generated because all respective options are disabled."
		rnb.add.paragraph(report, txt)
	}

	logger.completed()
	return(report)
}

#######################################################################################################################

#' rnb.step.snp.probes
#'
#' Computes statistics on the SNP-based probes in the given dataset and creates a corresponding section in the report.
#'
#' @param object a \code{\linkS4class{RnBeadSet}} object
#' @param report Report on methylation profiles to contain the dimension reduction section. This must be an object of
#'               type \code{\linkS4class{Report}}.
#' @return the modified report object
#'
#' @author Pavlo Lutsik
#' @noRd
rnb.step.snp.probes<-function(object, report){

	if(!inherits(object,"RnBSet")){
		stop("Supplied object is not of the class inheriting from RnBSet")
	}
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}

	logger.start("Visualizing SNP Probe Data")
	if (any(unlist(rnb.options("qc.snp.heatmap", "qc.snp.boxplot", "qc.snp.barplot", "qc.snp.purity")))) {
		logger.start("Mixups Visualization Section")
		report <- rnb.section.snp.probes(report,object)
		logger.completed()
	}
	logger.completed()

	return(report)
}

#######################################################################################################################

#' rnb.section.snp.probes
#'
#' Adds a section on the SNP-based probes to the given report.
#'
#' @param report Report to contain the new section. This must be an object of type \code{\linkS4class{Report}}.
#' @param object Methylation dataset as an object of type \code{\linkS4class{RnBeadSet}}.
#' @return The (possibly modified) report.
#'
#' @author Pavlo Lutsik
#' @noRd
rnb.section.snp.probes <- function(report, object) {

	mm.snps <- tryCatch(rnb.get.snp.matrix(object), error = function(err) { NULL })
	section.title <- "Visualization of SNP Probe Data"
	if (is.null(mm.snps)) {
		txt <- c("Overview of SNP-based probes and sample comparison based on them cannot be performed ",
			"because the dataset does not contain such probes.")
		report <- rnb.add.section(report, section.title, txt)
		return(report)
	}
	if (ncol(mm.snps) == 1) {
		txt <- c("Overview of SNP-based probes and sample comparison based on them cannot be performed ",
			"because the dataset contains only one sample.")
		report <- rnb.add.section(report, section.title, txt)
		return(report)
	}
	
	txt <- c("Analysis of the values of the SNP-based probes can help identify sample mixups.")
	report <- rnb.add.section(report, section.title, txt)

	add.info <- function(stitle, ffunction, txt) {
		result <- rnb.add.section(report, stitle, txt, level = 2)
		result <- ffunction(result, mm.snps)
		rnb.status(c("Added", stitle))
		result
	}

	if (rnb.getOption("qc.snp.heatmap")) {
		txt <- "SNP heatmap enables the identification of sample mixups in particular for genetically matched designs."
		report <- add.info("SNP Heatmap", rnb.add.snp.heatmap, txt)
	}
	if (rnb.getOption("qc.snp.barplot")) {
		txt <- "SNP bar plots enable the identification of sample mixups in particular for genetically matched designs."
		report <- add.info("SNP Bar Plots", rnb.add.snp.barplot, txt)
	}
	if (rnb.getOption("qc.snp.boxplot")) {
		txt <- c("The SNP box plot is a visualization tool that can show sample mixups, especially in the case the ",
			"set of samples is genetically homogeneous.")
		report <- add.info("SNP Box Plot", rnb.add.snp.boxplot, txt)
	}
	if (rnb.getOption("qc.snp.distances")) {
		txt <- c("If we inspect the dataset in the space defined by the SNP probes only, samples appearing close to ",
			"each other are genetically similar.")
		report <- add.info("SNP-based Distances", rnb.add.snp.distances, txt)
	}
	if (rnb.getOption("qc.snp.purity")) {
		report <- rnb.add.snp.purity(report, mm.snps, object@pheno)
	}

	return(report)
}

########################################################################################################################
## UTILS
########################################################################################################################

add.qc.boxplots<-function(report, object){
  descr<-"Quality control box plots."

  if(object@target=="probesEPIC"){
	  ctypes<-rnb.infinium.control.targets(object@target)[c(14,4,3,15,1:2,12:13,6,11)]
  }else if(object@target=="probesEPICv2"){
	  ctypes<-rnb.infinium.control.targets(object@target)[c(14,4,3,15,1:2,12:13,6,11)]
  }else if(object@target=="probes450"){
  	ctypes<-rnb.infinium.control.targets(object@target)[c(13,4,3,14,1:2,11:12,6)]
  }else if(object@target=="probes27"){
	ctypes<-rnb.infinium.control.targets(object@target)[c(10,3,2,11,1,9,6)]
  }else if(object@target=="probesMMBC"){
      ctypes<-rnb.infinium.control.targets(object@target)[c(14,4,3,15,1:2,12:13,6,11)]
  }

  cplots<-lapply(ctypes, rnb.plot.control.boxplot, rnb.set=object, report=report, writeToFile=TRUE, numeric.names=TRUE, width=8, height=6, low.png=100, high.png=300)
  names(cplots)<-1:length(ctypes)

  sn<-list("Control probe type"=ctypes)
  names(sn[[1]])<-1:length(ctypes)

  report<-rnb.add.figure(report, description=descr, report.plots=cplots, setting.names=sn)
  report

}

#######################################################################################################################

add.qc.barplots<-function(report, object, sample.batch.size=50){
  descr="Quality control bar plots."

  if(object@target=="probesEPIC"){
	  cmd <- rnb.get.annotation("controlsEPIC", assembly = rnb.getOption("assembly"))
	  ctypes<-unique(cmd$Target)[unique(cmd$Target) %in% rnb.infinium.control.targets("probesEPIC")[c(14,4,3,15,1:2,12:13,6,11)]]
  }else if(object@target=="probesEPICv2"){
	  cmd <- rnb.get.annotation("controlsEPICv2", assembly = "hg38")
	  ctypes<-unique(cmd$Target)[unique(cmd$Target) %in% rnb.infinium.control.targets("probesEPICv2")[c(14,4,3,15,1:2,12:13,6,11)]]
  }else if(object@target=="probes450"){
  	cmd <- rnb.get.annotation("controls450", assembly = rnb.getOption("assembly"))
  	ctypes<-unique(cmd$Target)[unique(cmd$Target) %in% rnb.infinium.control.targets("probes450")[c(13,4,14,3,1:2,11:12,6)]]
  }else if(object@target=="probes27"){
	cmd <- rnb.get.annotation("controls27")
	ctypes<-unique(cmd$Type)[unique(cmd$Type) %in% rnb.infinium.control.targets("probes27")[c(10,3,2,11,1,9,6)]]
  }else if(object@target=="probesMMBC"){
    cmd <- rnb.get.annotation("controlsMMBC", assembly="mm10")
    ctypes<-unique(cmd$Target)[unique(cmd$Target) %in% rnb.infinium.control.targets("probesMMBC")[c(14,4,3,15,1:2,12:13,6,11)]]
  }
  nsamp<-length(samples(object))

  plot.names<-NULL

  if(nsamp %% sample.batch.size == 0){
	  portion.starts<-0:(nsamp %/% sample.batch.size - 1)*sample.batch.size+1
  }else if (nsamp %% sample.batch.size != 0){
	  portion.starts<-0:(nsamp %/% sample.batch.size)*sample.batch.size+1
  }
  portion.ends<-portion.starts+sample.batch.size-1
  portion.ends[length(portion.ends)]<-nsamp
  portions<-paste(portion.starts, portion.ends, sep="-")

  plots<-lapply(1:length(portions),function(portion.id){

	  cplots<-lapply(ctypes, function(type){

		if(object@target=="probes450" || object@target=="probesEPIC" || object@target=="probesMMBC" || object@target=="probesEPICv2"){
			cmdt <- cmd[cmd[["Target"]] == type, ]
			pn<-paste(type, 1:(dim(cmdt)[1]),  sep=".")
		}else if(object@target=="probes27"){
			cmdt <- cmd[cmd[["Type"]] == type, ]
			pn<-as.character(cmdt$Name)
		}

		if(portion.id==1) plot.names<<-c(plot.names, pn)
	    plots<-lapply(pn, rnb.plot.control.barplot, rnb.set=object,
				sample.subset=portion.starts[portion.id]:portion.ends[portion.id],
				report=report, writeToFile=TRUE, numeric.names=TRUE, width=8, height=6, low.png=100, high.png=300, verbose=TRUE,
				name.prefix=portions[portion.id])

		if(object@target=="probes450" || object@target=="probesEPIC" || object@target=="probesMMBC" || object@target=="probesEPICv2"){
			names(plots)<-paste(type, 1:(dim(cmdt)[1]))
		}else if(object@target=="probes27"){
			names(plots)<-as.character(cmdt$Name)
		}
	    plots

	  })

	  names(cplots)<-NULL

	  cplots<-unlist(cplots)
	  names(cplots)<-1:length(plot.names)
	  cplots
  })

  plots<-unlist(plots)

  sn<-list("Samples #: " = portions, "Control probe ID" = plot.names)


  names(sn[[1]])<-portions
  if(object@target=="probes450" || object@target=="probesEPIC" || object@target == "probesMMBC" || object@target == "probesEPICv2"){
  	names(sn[[2]])<-1:length(plot.names)
  }else if(object@target=="probes27"){
	names(sn[[2]])<-match(plot.names,cmd$Name[cmd$Type %in% rnb.infinium.control.targets("probes27")[c(10,3,2,11,1,9,6)]])
  }

  report<-rnb.add.figure(report, description=descr, report.plots=plots, setting.names=sn)

}

#######################################################################################################################

add.negative.control.boxplot<-function(report, object, sample.batch.size=50){

	descr<-"Box plots of the negative control probes."

#   cplots<-lapply(ctypes, control.boxplot, rnb.set=object, report=report, writeToFile=TRUE)
#   names(cplots)<-gsub(" ", ".", ctypes)
#
#   sn<-list(tolower(ctypes))
#   names(sn[[1]])<-gsub(" ", ".", ctypes)
	nsamp<-length(samples(object))
	if(nsamp %% sample.batch.size == 0){
 	 portion.starts<-0:(nsamp %/% sample.batch.size - 1)*sample.batch.size+1
	}else if (nsamp %% sample.batch.size != 0){
  	portion.starts<-0:(nsamp %/% sample.batch.size)*sample.batch.size+1
	}
	portion.ends<-portion.starts+sample.batch.size-1
	portion.ends[length(portion.ends)]<-nsamp
	portions<-paste(portion.starts, portion.ends, sep="-")

	cplots<-lapply(1:length(portions), function(portion.id){
				rnb.plot.negative.boxplot(object, sample.subset=portion.starts[portion.id]:portion.ends[portion.id],
						name.prefix=portions[portion.id], writeToFile=TRUE, report=report, width=10, height=6, low.png=75, high.png=100)
	})

	sn<-list("Samples #: " = portions)

	names(sn[[1]])<-portions

	report<-rnb.add.figure(report, description=descr, report.plots=cplots, sn)
	report

}

#######################################################################################################################

rnb.add.snp.boxplot<-function(report, object){

  txt <- "Box plot of the SNP probes."
  rplot <- rnb.plot.snp.boxplot(object, writeToFile=TRUE, report=report, width=9, height=6, low.png=100, high.png=300)
  rnb.add.figure(report, txt, rplot)
}

#######################################################################################################################

rnb.add.snp.barplot<-function(report, object){

	txt <- "Bar plot of all observed beta values for a SNP probe."
	rplots <- list()
	for (probe.id in rownames(object)) {
		rplot <- rnb.plot.snp.barplot(object, probe.id, TRUE, TRUE, report = report, width=9, height=6, high.png=300)
		rplots <- c(rplots, list(rplot))
	}
	setting.names <- list("Probe" = rownames(object))
	names(setting.names[[1]]) <- 1:length(setting.names[[1]])
	rnb.add.figure(report, txt, rplots, setting.names)
}

#######################################################################################################################

rnb.add.snp.heatmap<-function(report, object){
	txt <- paste0("Heatmap of the SNP probes. Euclidean distance and complete linkage are used for constructing the ",
		"dendrograms. Samples with the same genetic background are expected to cluster together.")
	rplot <- rnb.plot.snp.heatmap(object, writeToFile=TRUE, report=report, width=8, height=9, low.png=100, high.png=300)
	rnb.add.figure(report, txt, rplot)
}

#######################################################################################################################

#' rnb.add.snp.distances
#'
#' Adds a section about sample distances based on beta values of SNP probes.
#'
#' @param report Report to contain the section on SNP probe distances.
#' @param object Non-empty \code{matrix} containing the computed beta values on the SNP probes.
#' @return The (possibly modified) report.
#' @author Yassen Assenov
#' @noRd
rnb.add.snp.distances <- function(report, object) {

	## Extract the matrix of beta values on the SNP probes
	snp.betas <- t(object[!apply(is.na(object), 1, any), , drop = FALSE])
	if (ncol(snp.betas) <= 1) {
		snp.betas <- NULL
	}
	if (is.null(snp.betas)) {
		txt <- c("Distances based on SNP probes could not be calculated either because no such probes are found ",
			"in the dataset, or because almost all of them contain missing values.")
		rnb.add.paragraph(report, txt)
		return(report)
	}

	## Calculate Manhattan distances between samples based on the SNP probe intensities
	snp.distances <- as.matrix(stats::dist(snp.betas, method = "manhattan") / ncol(snp.betas))
	rnb.status("Calculated Manhattan distances between samples based on SNP probes")

	report.plots <- list()
	setting.names <- list()
	if (nrow(snp.betas) <= 24) {

		## Create a diagonal heatmap of distances
		i.width <- 4 + nrow(snp.betas) * 0.3
		i.height <- 2.2 + nrow(snp.betas) * 0.3
		txt <- paste("The figure below shows the relative distances between all pairs of samples based on the",
			"&beta; values of the considered SNP probes. The distance metric used is average absolute difference,",
			"which can be considered a scaled version of Manhattan distance.")

		tbl <- symmetric.melt(snp.distances)
		colnames(tbl)[3] <- "distance"
		colors.g <- rnb.getOption("colors.gradient")
		pp <- ggplot(tbl, aes_string(x = "x", y = "y", fill = "distance")) + labs(x = NULL, y = NULL) +
			coord_fixed(ratio = 1) + geom_tile(color = "white") +
			scale_x_discrete(expand = c(0, 0)) + scale_y_discrete(expand = c(0, 0)) +
			scale_fill_gradient(na.value = "white", low = colors.g[1], high = colors.g[2]) +
			theme(legend.justification = c(0, 1), legend.position = c(1, 1)) +
			theme(axis.ticks = element_blank(), axis.text.x = element_text(angle = 90, hjust = 1)) +
			theme(panel.grid.major = element_blank(), panel.background = element_blank()) +
			theme(panel.border = element_blank(), plot.margin = unit(c(0.1, 1.1, 0.1, 0.1), "in"))
		## Fix the areas for x and y axis labels
#		pp <- suppressWarnings(ggplot_gtable(ggplot_build(pp)))
#		pp$widths[[3]] <- unit(2, "in")
#		pp$heights[[length(pp$heights) - 2L]] <- unit(2, "in")
		rplot <- createReportPlot("snp_low_dimensional", report, width = i.width, height = i.height)
#		grid.newpage()
#		grid.draw(pp)
		print(pp)
		report.plots <- c(report.plots, off(rplot))
		txt <- c(txt, paste("Distances between pairs of samples based on", ncol(snp.betas), "SNP probes."))
		rm(colors.g)

	} else {

		## Create scatter plots
		i.width <- 6.2 + (nrow(snp.betas) >= 60)
		i.height <- i.width
		if (ncol(snp.betas) == 2) {
			tbl <- snp.betas
			alabels <- colnames(snp.betas)
			txt <- "The figure below shows the samples in the space defined by the two SNP probes."
		} else {
			tbl <- prcomp(snp.betas, center = TRUE, scale. = FALSE)$x[, 1:2, drop = FALSE]
			alabels <- paste("Principal component", 1:ncol(tbl))
			txt <- paste("The figure below shows the samples in the first two principal components of the space",
				"defined by the", ncol(snp.betas), "SNP probes.")
		}

		tbl <- data.frame(x = tbl[, 1], y = tbl[, 2], label = rownames(tbl))
		plot.types <- c("point" = "points", "label" = "identifiers")
		for (ptype in names(plot.types)) {
			pp <- ggplot(tbl, aes_string(x = "x", y = "y", label = "label")) + coord_fixed(ratio = 1)
			if (ncol(snp.betas) == 2) {
				pp <- pp + scale_x_continuous(limits = c(0, 1), breaks = seq(0, 1, length.out = 11), expand = c(0, 0)) +
					scale_y_continuous(limits = c(0, 1), breaks = seq(0, 1, length.out = 11), expand = c(0, 0))
			}
			if (ptype == "point") {
				pp <- pp + geom_point(size = 3)
			} else {
				pp <- pp + geom_text(size = 2, angle = 45)
			}
			pp <- pp + labs(x = alabels[1], y = alabels[2]) + theme(plot.margin = unit(0.1 + c(0, 0, 0, 0), "in"))
			fname <- paste0("snp_low_dimensional_", ptype)
			rplot <- createReportPlot(fname, report, width = i.width, height = i.height)
			print(pp)
			report.plots <- c(report.plots, off(rplot))
		}
		txt <- c(txt, "Scatter plot showing the samples in a space defined by the signal of their SNP probes.")
		setting.names <- list("Display samples as" = plot.types)
		rm(alabels, plot.types, ptype, fname)
	}
	rnb.add.paragraph(report, txt[1])
	report <- rnb.add.figure(report, txt[2], report.plots, setting.names)
	rm(snp.betas, report.plots, setting.names, txt, i.width, i.height, tbl, pp, rplot)

	## Export the distances to file
	fname <- "snp_distances.csv"
	write.csv(snp.distances, file = file.path(rnb.get.directory(report, "data", TRUE), fname))
	txt <- c('The full table of all pairwise distances is stored in a dedicated <a href="',
		rnb.get.directory(report, "data"), '/', fname, '">comma-separated file</a> accompanying this report.')
	rnb.add.paragraph(report, txt)

	report
}

#######################################################################################################################

#' rnb.add.snp.purity
#'
#' Adds a section about genetic purity estimated using the beta values of SNP probes.
#'
#' @param report       Report to contain the section on genetic purity.
#' @param mm.snp       Non-empty \code{matrix} containing the computed beta values on the SNP probes.
#' @param s.annotation Optionally, a sample annotation table to be scanned for information on Sentrix IDs.
#' @return The (possibly modified) report.
#' @author Yassen Assenov
#' @noRd
rnb.add.snp.purity <- function(report, mm.snp, s.annotation = data.frame()) {
	tbl <- pmin(mm.snp, abs(mm.snp - 0.5), abs(mm.snp - 1))
	tbl <- data.frame(
		"Genetic Noise" = unname(colMeans(tbl, na.rm = TRUE)),
		"ID" = factor(colnames(mm.snp), levels = colnames(mm.snp)),
		check.names = FALSE, stringsAsFactors = FALSE)
	ids <- s.annotation
	ids <- ids[, intersect(c("Sentrix ID", "Sentrix_ID"), colnames(ids)), drop = FALSE]
	if (ncol(ids) != 0) {
		ids <- as.character(ids[, 1])
		id.levels <- tryCatch(unique(sort(as.double(ids))), warning = function(wr) { unique(ids) })
		ids <- factor(ids, levels = id.levels)
		if (length(id.levels) > 1 && anyDuplicated(na.omit(ids)) != 0) {
			tbl[["Sentrix ID"]] <- ids
		}
		rm(ids, id.levels)
	}
	tbl <- tbl[, intersect(c("ID", "Sentrix ID", "Genetic Noise"), colnames(tbl)), drop = FALSE]
	tbl <- tbl[order(tbl[, "Genetic Noise"]), , drop = FALSE]

	## Export the table to a file
	fname <- "genetic_noise.csv"
	write.csv(tbl, file.path(rnb.get.directory(report, "data", TRUE), fname), row.names = FALSE)

	## Plot the data
	tbl[, "Genetic Noise"] <- tbl[, "Genetic Noise"] * 100
	xstep <- 2
	xmax <- ceiling(max(tbl[, "Genetic Noise"]) / xstep) * xstep
	i.width <- 6.2
	pp <- ggplot2::ggplot(tbl) + ggplot2::aes_string(x = '`ID`', y = '`Genetic Noise`') +
		ggplot2::labs(x = NULL, y = "Genetic noise (%)")
	if ("Sentrix ID" %in% colnames(tbl)) {
		i.width <- i.width + 1.4
		pp <- pp + ggplot2::aes_string(fill = '`Sentrix ID`') +
			ggplot2::theme(legend.position = c(1, 0.5), legend.justification = c(0, 0.5)) +
			ggplot2::theme(plot.margin = grid::unit(c(0, 1.4, 0, 0) + 0.1, "in"))
	}
	pp <- pp + ggplot2::geom_bar(stat = "identity") + ggplot2::coord_flip() +
		ggplot2::scale_y_continuous(breaks = seq(0, xmax, by = xstep), limits = c(0, xmax), expand = c(0, 0)) +
		ggplot2::theme(axis.ticks.y = ggplot2::element_blank())
	rplot <- createReportPlot("genetic_noise", report, width = i.width, height = nrow(tbl) * 0.12 + 1.2)
	print(pp)
	rplot <- off(rplot)
	
	## Add a section to the report
	txt <- c("The relative purity of a sample can be estimated by calculating the mean absolute distance from the ",
		"observed &beta; values of its SNP probes to an idealized profile, consisting of the values 0, 0.5 and 1 ",
		"only. This score is referred to as <em>genetic noise</em> in this report. Samples with large genetic noise ",
		"might contain subpopulations with genomic aberrations (deletions and amplifications), or be contaminated ",
		"with DNA from multiple individuals. It is important to note that the genetic noise could be strongly ",
		"influenced by batch effects and should therefore be considered only in the context of samples with similar ",
		"profiles of intensity values.")
	report <- rnb.add.section(report, "Genetic Purity", txt, 2)
	txt <- c('The figure below lists the genetic noise of all samples in the dataset. The full table with these ',
		'values is available in a dedicated <a href="', rnb.get.directory(report, 'data'), '/', fname,
		'">comma-separated value</a> file accompanying this report.')
	rnb.add.paragraph(report, txt)
	txt <- c("Bar plot showing the genetic noise of each sample. Color coding, if present, denotes slide number.")
	report <- rnb.add.figure(report, txt, rplot)
}

#######################################################################################################################

add.seq.coverage.plot<-function(report, object, covg.lists=NULL){

	txt <- "Effective read coverage of a single sample over all chromosomes of the genome."

	ids <- samples(object)
	names(ids) <- 1:length(ids)

	rplots<-lapply(ids, function(id) rnb.plot.biseq.coverage(rnbs.set=object, sample=id, report=report,
		writeToFile=TRUE, numeric.names=TRUE, create.pdf=FALSE, width=8, height=16, low.png=100, high.png=200,
		covg.lists=covg.lists[[id]]))

	rnb.add.figure(report, txt, rplots, list("Sample" = ids))
}


#######################################################################################################################

add.seq.coverage.histograms <- function(report, object){

	descr<-"Sequencing coverage histogram visualize the bulk distribution of read coverage for each sample."

	ids <- samples(object)

	cplots<-lapply(ids, rnb.plot.biseq.coverage.hist, rnbs.set=object, report=report, writeToFile=TRUE, numeric.names=TRUE, covg.max.percentile=0.99, width=8, height=7, low.png=100, high.png=300)

	names(cplots)<-1:length(ids)

	sn<-list("Sample labels" = ids)
	names(sn[[1]])<-1:length(ids)

	report<-rnb.add.figure(report, description=descr, report.plots=cplots, setting.names=sn)
	report

}

#######################################################################################################################

add.seq.coverage.num.sites.covg.tabs <- function(report, object){
	selCols <- c("sampleName", "sites_num", "sites_covgMean", "sites_covgMedian", paste0("sites_covgPerc",c("25","75")), paste0("sites_numCovg",c("5","10","30","60")))
	summary.tab <- rnb.sample.summary.table(object)[,selCols]

	fname <- paste("sample_covg_summary.csv",sep="")
	fname <- rnb.write.table(summary.tab, fname,fpath=rnb.get.directory(report, "data", absolute = TRUE), format="csv", gz=FALSE, row.names = FALSE, quote=FALSE)
	sectionText <- paste("This section contains summary metrics on the number of sites and coverage can be found in the table below.",
		" The summary table below is also available as ",
			"<a href=\"", rnb.get.directory(report, "data"), "/", fname,"\">csv file</a>.",sep="")
	column.description <- c(
		"<ul>",
			"<li>sampleName: Name of the sample</li>",
			"<li>sites_num: Number of sites covered in the sample</li>",
			"<li>sites_covgMean: Mean coverage of sites in the sample</li>",
			"<li>sites_covgMedian: Median coverage of sites in the sample</li>",
			"<li>sites_covgPerc25,75: 25 and 75 percentile of coverage of sites in the sample</li>",
			"<li>sites_numCovg5,10,30,60: Number of sites with coverage greater or equal to 5,10,30,60</li>",
		"</ul>"
	)
	sectionText <- paste(sectionText, paste(column.description, collapse="\n"))

	report <- rnb.add.section(report, "Sample coverage summary", sectionText, level=2)
	rnb.add.table(report, summary.tab)

	pp <- rnb.plot.num.sites.covg(object)
	fname <- paste("num_site_vs_coverage",sep="_")
	overviewPlot <- createReportGgPlot(pp, fname, report, width=12, height=7, create.pdf=TRUE, high.png=as.integer(300))
	overviewPlot <- off(overviewPlot,handle.errors=TRUE)

	desc <- "Covered sites and median coverages for each sample. Vertical bars depict the inter-quartile range."
	report <- rnb.add.figure(report, desc, overviewPlot)
	report
}

#######################################################################################################################

add.seq.coverage.violins<-function(report, object, sample.chunk.size=20){

	descr<-paste("Sequencing coverage histogram visualized as violin plots. Distributions are based on",
				 nsites(object),"methylation sites.")

	ids <- samples(object)
	n <- length(ids)
	num.intervals <- ceiling(n/sample.chunk.size)
	if (num.intervals>1){
		ssets <- cut(1:n,ceiling(n/sample.chunk.size),labels=FALSE)
	} else {
		ssets <- rep(1,n)
	}
	ssets.n <- length(unique(ssets))
	max.covg.2plot<-quantile(covg(object),probs=c(0.99), na.rm=TRUE)[1]

	cplots <- list()
	for (i in 1:ssets.n){
		ssamples <- ids[ssets==i]
		rep.plot <- rnb.plot.biseq.coverage.violin(rnbs.set=object,samples=ssamples,fname=paste("coverageViolin_chunk",i,sep=""),covg.range=c(0,max.covg.2plot),report=report, width=8, height=7, low.png=100, high.png=300)
		cplots<-c(cplots,list(rep.plot))
	}

	names(cplots)<-1:ssets.n

	sn<-list("Sample chunk" = 1:ssets.n)
	names(sn[[1]])<-paste("chunk",1:ssets.n,sep="")

	report<-rnb.add.figure(report, description=descr, report.plots=cplots, setting.names=sn)
	report
}

########################################################################################################################
## test_report_1.R
## created: 2012-04-16
## creator: Yassen Assenov
## ---------------------------------------------------------------------------------------------------------------------
## Tests on the Report and ReportPlot class definitions, in the form of an example report generation.
########################################################################################################################

test_report <- function() {
	
	## Initialize the reports locations and the log
	report.directory <- "RnBeads_report_testing"
	if (!rnb.initialize.reports(report.directory)) {
		stop(paste("Could not initialize reports in", report.directory))
	}
	logfile <- file.path(report.directory, "quality.log")
	logger.start("Testing Report Generation", fname = logfile)

	## Initialize the report
	report <- createReport(file.path(report.directory, "example.html"), "Example")
	logger.status("Initialized the example report")
	
	## Define references
	rf1 <- "Marina Bibikova et al. Genome-wide DNA methylation profiling using Infinium assay. Epigenomics 1 (2009) 177-200"
	rf2 <- c("The ENCODE Project Consortium. ", "An integrated encyclopedia of DNA elements in the human genome. ",
		"Nature 489 (2012) 57-74")
	report <- rnb.add.reference(report, rf1)
	report <- rnb.add.reference(report, rf2)

	## Add report sections
	txt <- c("This is example report ", rnb.get.reference(report, rf1), ". It is used in testing.")
	report <- rnb.add.section(report, "Introduction", txt)
	txt <- c("Some background knowledge is required ", rnb.get.reference(report, rf2), ".")
	report <- rnb.add.section(report, "Background", txt, level = 2)

	## Create plots
	doplot <- function(type = "p", ...) {
		plot(x = c(0.4, 0.6, 0.8, 1), y = c(2, 8, 3, 9), type = type, ..., main = NA, xlab = expression(beta),
			ylab = "Measure")
	}

	report.plot1 <- createReportPlot("example_scatterplot_data_zoom", report, high.png = 200)
	doplot(pch = 16, col = c("#000080", "#00FF00"))
	off(report.plot1)

	report.plot2 <- createReportPlot("example_lines_data_zoom", report, high.png = 200)
	doplot(type = "l", lwd = 2, col = "#00FF00")
	off(report.plot2)

	report.plot3 <- createReportPlot("example_both_data_zoom", report, high.png = 200)
	doplot(type = "b", lwd = 2, col = "#00FF00")
	off(report.plot3)

	report.plot4 <- createReportPlot("example_scatterplot_data_full", report, high.png = 200)
	doplot(pch = 16, col = c("#000080", "#00FF00"), xlim = c(0, 1))
	off(report.plot4)

	report.plot5 <- createReportPlot("example_lines_data_full", report, high.png = 200)
	doplot(type = "l", lwd = 2, col = "#00FF00", xlim = c(0, 1))
	off(report.plot5)

	report.plot6 <- createReportPlot("example_both_data_full", report, high.png = 200)
	doplot(type = "b", lwd = 2, col = "#00FF00", xlim = c(0, 1))
	off(report.plot6)

	logger.info("Generated plots")

	## Generate report figure
	report.plots <- list(report.plot1, report.plot2, report.plot3, report.plot4, report.plot5, report.plot6)
	setting.names <- list(
		"plot type" = c("scatterplot" = "scatter plot", "lines" = "line plot", "both" = "line-and-point plot"),
		"values to visualize" = c("data" = "random data", "letters" = "random letters"),
		"methylation value range" = c("full" = "full", "zoom" = "zoomed in"))
	description <- c("Example figure of four data points displayed in different plots.",
		"The horizontal axis depicts methylation &beta; value, and the vertical axis represents a measurement.",
		"All values and colors were selected randomly.")
	description <- paste(description, collapse = " ")
	report <- rnb.add.section(report, "The Figure", "Here comes <a href=\"#fig1image\">Figure 1</a>:")
	report <- rnb.add.figure(report, description, report.plots, setting.names)
	
	## Close the report
	report <- rnb.add.section(report, "Summary", "The generation of this report was successful.")
	off(report)
	logger.info("Closed the example report")

	logger.completed()

	## Remove the generated files
	unlink(report.directory, recursive = TRUE)
}

#' run_xeno
#' 
#' Run Xeno Mouse Fraction Analysis
#' 
#' @return a \code{data.frame} with the estimated mouse content
#' @param idat_path Path to the idat file for which the mouse fraction is to be computed
#' @param n_cores The number of cores to be used
#' @export
#' @author Nima Esmaeelpour
rnb.run_xeno <- function(idat_path, n_cores = 1) {
  rnb.require('sesame')
  rnb.require('BiocParallel')
  rnb.require('dplyr')
    if (!is.character(idat_path) || length(idat_path) != 1) {
    stop("`idat_path` must be a single character string.", call. = FALSE)
  }
  if (!dir.exists(idat_path)) {
    stop("Directory does not exist: ", idat_path, call. = FALSE)
  }
  if (!is.numeric(n_cores) || length(n_cores) != 1 || n_cores < 1) {
    stop("`n_cores` must be a positive integer.", call. = FALSE)
  }
  
  # ---- LOAD DATA ----
  
  idat_files <- list.files(idat_path, "\\.idat$")
  if (length(idat_files) == 0) {
    stop("No .idat files found in: ", idat_path, call. = FALSE)
  }

  # Read .idat files
  logger.info(sprintf("Started Reading %s directory...", idat_path))
  sdfs <- openSesame(
    idat_path,
    func = NULL,
    BPPARAM = MulticoreParam(n_cores)
  )
  logger.info(sprintf("Finished Reading %s directory...", idat_path))

  # Convert to list if only one sample (i.e. two .idat files)
  if (!is.list(sdfs) || is(sdfs, "SigDF")) {
    sdfs <- list(sample = sdfs)
  }

  # Add xeno content
  results <- vector("list", length(sdfs))
  i <- 1
  for (sample_name in names(sdfs)) {
    results[[i]] <- add_xeno(sdfs[[sample_name]], sample_name)
    i <- i + 1
  }

  # Return final df
  dplyr::bind_rows(results)
}

# ---- HELPER FUNCTIONS ----

standard_error <- function(x, na.rm = FALSE) {
  if (na.rm) {
    x <- x[!is.na(x)]
  }
  stats::sd(x) / sqrt(length(x))
}

filter_type_I_probes <- function(df, manifest) {
  dplyr::filter(df, Probe_ID %in% manifest$IlmnID)
}

calculate_wrong_color_fraction <- function(df, manifest) {
  channel_map <- dplyr::select(manifest, IlmnID, Color_Channel)

  df |>
    dplyr::left_join(channel_map, by = c("Probe_ID" = "IlmnID")) |>
    dplyr::mutate(
      total_signal = MR + UR + MG + UG,
      wrong_color_signal = dplyr::if_else(
        Color_Channel == "Grn",
        MR + UR,
        MG + UG
      ),
      wrong_color_fraction = wrong_color_signal / total_signal
    )
}

# ---- CORE LOGIC ----

MIN_TOTAL_SIGNAL <- 1000
LOW_SIGNAL_CUTOFF <- 0.4

#' Compute xenograft (interspecies) contamination metrics for one sample
#'
#' @param df          Raw probe-level data frame (columns: Probe_ID, MR, UR, MG, UG)
#' @param sample_name Character scalar identifying the sample
#' @return Single-row data.frame with contamination metrics
#' @noRd
add_xeno <- function(df, sample_name) {
  platform <- resolve_platform(df, sample_name)
  manifest <- platform$manifest
  interspecies_probes <- platform$interspecies_probes

  df <- df |>
    filter_type_I_probes(manifest) |>
    calculate_wrong_color_fraction(manifest) |>
    dplyr::filter(MR + UR + MG + UG > MIN_TOTAL_SIGNAL)

  background <- stats::median(df$wrong_color_fraction, na.rm = TRUE)
  background_se <- standard_error(df$wrong_color_fraction, na.rm = TRUE)
  background_mad <- stats::mad(df$wrong_color_fraction, na.rm = TRUE)

  xeno_signal_stats <- df |>
    dplyr::filter(Probe_ID %in% interspecies_probes) |>
    dplyr::summarize(
      n_interspecies_probes = dplyr::n(),
      median_signal = stats::median(wrong_color_fraction, na.rm = TRUE),
      se_signal = standard_error(wrong_color_fraction, na.rm = TRUE),
      mad_signal = stats::mad(wrong_color_fraction, na.rm = TRUE)
    )

  mouse_fraction <- compute_mouse_fraction(
    xeno_signal_stats$median_signal,
    background
  )

  data.frame(
    sample = sample_name,
    probes_used = xeno_signal_stats$n_interspecies_probes,
    mouse_signal = xeno_signal_stats$median_signal,
    sem_mouse_signal = xeno_signal_stats$se_signal,
    mad_mouse_signal = xeno_signal_stats$mad_signal,
    background_signal = background,
    sem_background = background_se,
    mad_background = background_mad,
    mouse_fraction = mouse_fraction,
    stringsAsFactors = FALSE
  )
}

#' Adjust raw mouse signal by background, clamped to `[0, 1]`
#' Returns 1 (worst case) when no interspecies probes are available.
#' @noRd
compute_mouse_fraction <- function(mouse_med, background) {
  dplyr::case_when(
    is.na(mouse_med) ~ 1,
    mouse_med < LOW_SIGNAL_CUTOFF ~ pmax(0, mouse_med - background),
    TRUE ~ mouse_med
  )
}

#' Resolve platform and return manifest and interspecies probes
#'
#' @param df Raw probe-level data frame
#' @param sample_name Character scalar identifying the sample
#' @return List with manifest and interspecies_probes
#' @noRd
resolve_platform <- function(df, sample_name) {
  load(system.file("data/xenomix.RData", package = "RnBeads"))
  n <- nrow(df)

  if (n == 937690) {
    list(
      manifest = epic_v2_manifest,
      interspecies_probes = interspecies_probes_v2
    )
  } else {
    list(
      manifest = epic_v1_manifest,
      interspecies_probes = interspecies_probes_v1
    )
  }
}
          

########################################################################################################################
## controlPlotsBiSeq.R
## created: 2012-04-04
## creator: Pavlo Lutsik
## ---------------------------------------------------------------------------------------------------------------------
## Quality control plots for the reduced-representation and the whole-genome bisulfite sequencing experiments.
########################################################################################################################

## F U N C T I O N S ###################################################################################################

#' rnb.plot.biseq.coverage
#'
#' Plots the sequencing coverage of the RnBiseqSet object across the genomic coordinate 
#'   
#' @param rnbs.set			RnBiseqSet object
#' @param type				\code{character} singleton. If \code{site} the coverage
#' 							information is plotted for each methylation site. Otherwise
#' 							should be one of the regions returned by \code{rnb.region.types}
#' @param sample			unique sample identifier. In case \code{rnb.getOption("identifiers.column")} 
#' 							is not \code{NULL}, \code{sample} should attain values from the corresponding column, 
#' 							or \code{colnames(meth(rnb.set))} otherwise
#' @param writeToFile 		flag specifying whether the output should be saved as \code{\linkS4class{ReportPlot}}
#' @param numeric.names if \code{TRUE} and \code{writeToFile} is \code{TRUE}substitute the plot options in the plot file name with digits
#' @param covg.lists		if available, the output of \code{\link{rnb.execute.quality}}
#' @param ... 				other arguments to \code{\link{createReportPlot}}
#' 
#' @return plot as an object of type \code{\linkS4class{ReportPlot}} if \code{writeToFile} is \code{TRUE} and of class 
#' 			\code{\link{ggplot}} otherwise.
#' @author Pavlo Lutsik
#' @export
rnb.plot.biseq.coverage<-function(
		rnbs.set, 
		sample, 
		type = "sites", 
		writeToFile = FALSE, 
		numeric.names = FALSE, 
		covg.lists=NULL, 
		...){
	
	if(!inherits(rnbs.set, "RnBiseqSet"))
		stop("Invalid value for rnbs.set: object of class RnBiseqSet expected")
	
	if(!(is.character(type) && type %in% c("sites", rnb.region.types(assembly(rnbs.set)))))
		stop("Invalid value for type: \"sites\" or one of the rnb.region.types() expected")
	
	if (!(is.character(sample) && sample %in% samples(rnbs.set))) {
		stop("Invalid value for sample: character expected")
	}
	assembly <- assembly(rnbs.set)	
	covg.rnbs<-covg(rnbs.set, type)
	covg.rnbs[is.na(covg.rnbs)]<-0
	
		
	if(type=="sites"){
		type<-"CpG"
		map<-sites(rnbs.set)
	}else{
		map<-regions(rnbs.set, type)
	}
	
	sample.id<-match(sample, samples(rnbs.set))
	
	if(max(covg.rnbs[,sample.id])==0){
		
		if(writeToFile) plot.file<-createReportPlot(paste("CoveragePlot", ifelse(numeric.names, sample.id, sample), sep="_"), ...)
			print(rnb.message.plot("All CpG have zero coverage in this sample"))
		if(writeToFile) {
			off(plot.file)
			return(plot.file)
		}else{
			return(invisible(NULL))
		}
	}
	
	if(is.null(covg.lists)){
				
		covg.lists<-lapply(names(rnb.get.chromosomes(assembly=assembly)), function(chr) {
					chr.id<-match(chr, names(rnb.get.chromosomes(assembly=assembly)))
					covg.weight<-rep(0, rnb.annotation.size(assembly=assembly)[chr.id])
					covg.weight[map[map[,2]==chr.id,3]]<-covg.rnbs[map[,2]==chr.id,sample.id]
					covg.weight
				})
	}else if(is.list(covg.lists)){
		
		covg.lists<-lapply(1:length(covg.lists), function(chr.id){
					covg.weight<-covg.lists[[chr.id]]
					covg.weight[map[map[,2]==chr.id,3]]<-covg.rnbs[map[,2]==chr.id,sample.id]
					covg.weight
				})
	}
	names(covg.lists)<-names(rnb.get.chromosomes(assembly=assembly))
	
	cv.templ<-coverage(rnb.get.annotation(type,assembly=assembly), weight=covg.lists)
	
	if(writeToFile) plot.file<-createReportPlot(paste("CoveragePlot", ifelse(numeric.names, sample.id, sample), sep="_"), ...)
		
	names(cv.templ)<-NULL
	plot<-ggbio::autoplot(cv.templ, type = "viewMaxs", stat = "slice", lower = 8, ylab="", ylim=2000, labeller=function(variable,value){names(rnb.get.chromosomes(assembly=assembly))[value]})+
					scale_y_continuous(limits=c(0,2000))
		
	if(writeToFile) {
		print(plot)
		off(plot.file)
		return(plot.file)
	}else{
		return(plot)
	}
}

########################################################################################################################

#' rnb.plot.biseq.coverage.hist
#'
#' Plots the histograms of the coverage 
#'   
#' @param rnbs.set			RnBiseqSet object
#' @param sample			unique sample identifier. In case \code{rnb.getOption("identifiers.column")} 
#' 							is not \code{NULL}, \code{sample} should attain values from the corresponding column, 
#' 							or \code{colnames(meth(rnb.set))} otherwise
#' @param type				\code{character} singleton. If \code{site} the coverage
#' 							information is plotted for each methylation site. Otherwise
#' 							should be one of the regions returned by \code{rnb.region.types}
#' @param writeToFile 		a flag specifying whether the output should be saved as \code{\linkS4class{ReportPlot}}
#' @param numeric.names if \code{TRUE} and \code{writeToFile} is \code{TRUE}substitute the plot options in the plot file name with digits
#' @param covg.max.percentile the maximum percentile of the coverage to be plotted
#' @param ... 				other arguments to \code{\link{createReportPlot}}
#' 
#' @return plot as an object of type \code{\linkS4class{ReportPlot}} if \code{writeToFile} is \code{TRUE} and of class 
#' 			\code{\link{ggplot}} otherwise.
#'
#' @author Pavlo Lutsik
#' @export
rnb.plot.biseq.coverage.hist<-function(
		rnbs.set, 
		sample, 
		type = "sites", 
		writeToFile = FALSE, 
		numeric.names = FALSE, 
		covg.max.percentile=1.0, 
		...){
	
	if(!inherits(rnbs.set, "RnBiseqSet"))
		stop("Invalid value for rnbs.set: object of class RnBiseqSet expected")
	
	if(!(is.character(type) && type %in% c("sites", rnb.region.types(assembly(rnbs.set)))))
		stop("Invalid value for type: \"sites\" or one of the rnb.region.types() expected")
	
	if (!(is.character(sample) && sample %in% samples(rnbs.set))) {
		stop("Invalid value for sample: character expected")
	}
	if (!is.double(covg.max.percentile) ||  covg.max.percentile > 1 || covg.max.percentile < 0){
		stop("Invalid value for covg.max.percentile: number between 0 and 1 expected")
	}
	
	sample.id <- match(sample, samples(rnbs.set))
	covg.rnbs <- covg(rnbs.set, type)
	max.covg  <- max(covg.rnbs, na.rm=TRUE)
	if (covg.max.percentile > 0 && covg.max.percentile < 1){
		max.covg <- quantile(covg.rnbs, probs=c(covg.max.percentile), na.rm=TRUE)[1]
	}
	covg.rnbs<-covg.rnbs[,sample.id]
	covg.rnbs[is.na(covg.rnbs)]<-0
	
	rnb.cleanMem()
	
	if(writeToFile) {
		plot.file<-createReportPlot(paste("CoverageHistogram", ifelse(numeric.names, sample.id, sample), sep="_"), ...)
	}
		
	dframe<-data.frame(Coverage=covg.rnbs)
	#TODO: cut after 95 percentile of coverage
	pp <- ggplot(dframe, aes_string(x = "Coverage")) + coord_cartesian(xlim=c(0,max.covg))+ 
		labs(x = "Coverage", y = "# CpGs") + geom_histogram(aes(y = after_stat(count)), binwidth = 10)
	
	if(writeToFile) {
		print(pp)
		off(plot.file)
		return(plot.file)
	}else{
		return(pp)
	}
	
}
########################################################################################################################

#' rnb.plot.biseq.coverage.violin
#'
#' Plots the violin plots of the coverage distribution
#'   
#' @param rnbs.set			RnBiseqSet object
#' @param samples			unique sample identifiers. In case \code{rnb.getOption("identifiers.column")} 
#' 							is not \code{NULL}, \code{samples} should attain values from the corresponding column, 
#' 							or \code{colnames(meth(rnb.set))} otherwise
#' @param fname				base filename for the files to be plotted. If NULL, the plot will not be written to file
#' @param type				\code{character} singleton. If \code{site} the coverage
#' 							information is plotted for each methylation site. Otherwise
#' 							should be one of the regions returned by \code{rnb.region.types}
#' @param covg.range	    Vector of length 2 specifying the range of coverage to be plotted. if \code{NULL} (default) the entire range will be plotted
#' @param ... 				other arguments to \code{\link{createReportPlot}}
#' 
#' @return plot as an object of type \code{\linkS4class{ReportPlot}} if \code{writeToFile} is \code{TRUE} and of class 
#' 			\code{\link{ggplot}} otherwise.
#'
#' @author Fabian Mueller
#' @export
rnb.plot.biseq.coverage.violin <- function(rnbs.set, samples, fname=NULL, type = "sites", covg.range=NULL, ...){
	
	if(!inherits(rnbs.set, "RnBiseqSet"))
		stop("Invalid value for rnbs.set: object of class RnBiseqSet expected")
	
	if(!(is.character(type) && type %in% c("sites", rnb.region.types(assembly(rnbs.set)))))
		stop("Invalid value for type: \"sites\" or one of the rnb.region.types() expected")
	
	if (!(is.character(samples) && all(samples %in% samples(rnbs.set)))) {
		stop("Invalid value for samples: character expected")
	}
	if (!is.null(covg.range) && length(covg.range)!=2){
		stop("Invalid value for covg.range. vector of length 2 or NULL expected")
	}
	
	sample.id <- match(samples, samples(rnbs.set))
	covg.rnbs <- covg(rnbs.set, type)
	max.covg  <- max(covg.rnbs, na.rm=TRUE)
	covg.rnbs <- covg.rnbs[,sample.id]
	covg.rnbs[is.na(covg.rnbs)]<-0
	
	dframe <- data.frame(Coverage=as.vector(covg.rnbs),
			Sample=rep(samples,times=rep(nrow(covg.rnbs),length(samples))))
	
	if(!is.null(fname)) plot.file <- createReportPlot(fname, ...)
	
	pp <- ggplot(dframe, aes_string(x = "Sample", y = "Coverage")) 
	if (!is.null(covg.range)) {
		pp <- pp + ylim(covg.range[1],covg.range[2])
	}
	pp <- pp + labs(x = "Sample", y = "Coverage") +
		geom_violin(aes_string(fill="Sample"),scale="count") +
		scale_fill_manual(values=rep(rnb.getOption("colors.category"),length.out=length(samples))) +
		coord_flip() + theme(legend.position="none")
	
	if(!is.null(fname)) {
		print(pp)
		off(plot.file)
		return(plot.file)
	}else{
		return(pp)
	}
}

########################################################################################################################

#' rnb.plot.coverage.thresholds
#'
#' Plots the number of remaining CpGs after applying different thresholds for coverage and support.
#'
#' @param rnb.set       Methylation dataset as an object of type \code{\linkS4class{RnBiseqSet}}.
#' @param min.coverages Non-empty \code{integer} vector storing the unique positive cutoff values to be applied for
#'                      minimal coverage. Names, if present, are interpreted as colors that must be used to denote the
#'                      corresponding values.
#' @param fname         File name to save the generated plot to. See the \emph{Details} section for restrictions.
#' @param ...           Additional named parameters related to saving the plot to files. These can include:
#'                      \code{report}, \code{width}, \code{height}, \code{create.pdf}, \code{low.png} and
#'                      \code{high.png}. These parameters are ignored when \code{fname} is \code{NULL} or \code{NA}.
#' @return If \code{fname} is \code{NULL} or \code{NA} (default), the generated plot as an object of type
#'         \code{ggplot2}; otherwise, the initialized and closed \code{\linkS4class{ReportPlot}} object, invisibly.
#' 
#' @details
#' If \code{fname} is specified, this function calls \code{\link{createReportPlot}} to save the plot to PDF and/or PNG
#' files. See \link[=createReportPlot]{its documentation} for information on acceptable file names. Additional
#' parameters - \code{report}, \code{width}, \code{height}, etc. - can also be given. If image width is not specified,
#' it is set to a value between 4.7 and 9.2 (inches), depending on the number of samples in the dataset. The default
#' image height is fixed to 7.2.
#' 
#' @author Yassen Assenov
#' @export
rnb.plot.coverage.thresholds <- function(rnb.set, min.coverages, fname = NA, ...) {

	## Validate parameter values
	if (!inherits(rnb.set, "RnBiseqSet")) {
		stop("invalid value for rnb.set")
	}
	if (is.double(min.coverages) && isTRUE(all(min.coverages == as.integer(min.coverages)))) {
		cnames <- names(min.coverages)
		min.coverages <- as.integer(min.coverages)
		names(min.coverages) <- cnames
		rm(cnames)
	}
	if (!(is.integer(min.coverages) && length(min.coverages) != 0 && all(!is.na(min.coverages))
		  && all(min.coverages > 0))) {
		stop("invalid value for min.coverages")
	}
	if (anyDuplicated(min.coverages)) {
		stop("invalid value for min.coverage; duplicates found")
	}
	min.coverages <- sort(min.coverages)
	if (is.null(fname)) {
		fname <- NA
	} else if (!is.na(fname)) {
		if (!is.character(fname)) {
			stop("invalid value for fname")
		}
	}

	## Define colors to denote min coverage thresholds
	if (is.null(names(min.coverages))) {
		n <- length(min.coverages)
		colors.covered <- rgb(seq(0, 0.5, length.out = n), seq(0, 0.5, length.out = n), seq(0, 1, length.out = n))
		names(colors.covered) <- min.coverages
	} else {
		colors.covered <- names(min.coverages)
		colors.covered[is.na(colors.covered)] <- "#808080"
		names(colors.covered) <- min.coverages
	}

	## Construct data.frame storing the plot information
	matrix.coverages <- covg(rnb.set)
	n <- ncol(matrix.coverages)
	if (n <= 50) {
		s.values <- n:1
		s.indices <- 1:n
		xlabel <- "Support (number of samples)"
	} else {
		s.values <- seq(0, 1, length.out = 51)[-1]
		s.indices <- as.integer(ceiling(quantile(0:n, probs = s.values)))
		s.indices <- n + 1L - s.indices
		xlabel <- "Support (percentile)"
	}
	dframe <- do.call(rbind, lapply(as.integer(min.coverages), function(mc) {
			interrogated <- as.integer(rowSums(matrix.coverages >= mc))
			cpgs <- cumsum(rev(tabulate(interrogated, ncol(matrix.coverages)))) / 10^6
			data.frame("mc" = mc, "s" = s.values, cpgs = cpgs[s.indices])
		}
	))
	dframe$mc <- factor(as.character(dframe$mc), levels = as.character(min.coverages))

	## Generate the plot
	if (n <= 20) {
		img.width <- 4.7
		xbreaks <- 1:n
		xlabels <- as.character(xbreaks)
		xlimits <- 0.5 + c(n, 0)
	} else if (n <= 50) {
		img.width <- 1.7 + 0.15 * n
		xbreaks <- seq(2L, n, by = 2L)
		xlabels <- as.character(xbreaks)
		xlimits <- 0.5 + c(n, 0)
	} else {
		img.width <- 1.7 + 0.15 * 50
		xbreaks <- seq(0, 1, length.out = 26)
		xlabels <- as.character(xbreaks * 100)
		xlimits <- c(1.01, 0.01)
	}
	pp <- ggplot2::ggplot(dframe, aes_string(x = "s", y = "cpgs", color = "mc", group = "mc")) + ggplot2::geom_line() +
		ggplot2::geom_point() + ggplot2::scale_color_manual(values = colors.covered) +
		ggplot2::scale_x_reverse(breaks = xbreaks, limits = xlimits, labels = xlabels, expand = c(0, 0)) +
		ggplot2::labs(x = xlabel, y = "CpGs (million)", color = "Coverage\nthreshold") +
		ggplot2::theme(legend.position = c(1, 0.5), legend.justification = c(0, 0.5)) +
		ggplot2::theme(plot.margin = unit(0.1 + c(0, 1, 0, 0), "in"))
	if (is.na(fname)) {
		return(pp)
	}

	## Save the plot to file(s)
	additional.params <- list(...)
	if (is.null(additional.params$width)) {
		additional.params$width <- img.width
	}
	if (is.null(additional.params$height)) {
		additional.params$height <- 7.2
	}
	additional.params$fname <- fname
	rplot <- do.call(createReportPlot, additional.params)
	print(pp)
	rplot <- off(rplot)

	## Attach the data as an attribute to the plot
	dframe$mc <- as.integer(as.character(dframe$mc))
	colnames(dframe) <- c("Minimal coverage", xlabel, "CpGs (million)")
	attr(rplot, "data") <- dframe
	invisible(rplot)
}


########################################################################################################################

#' rnb.plot.num.sites.covg
#'
#' plot the number of sites vs median and other percentiles of coverage
#'   
#' @param rnbs			RnBiseqSet object
#' @param addSampleNames	should the sample names be added to the plot
#' @param bar.percentiles	the percentiles to be used for the error bars. Must be a vector of length 2 of which the
#' 							first two elements will be used
#' 
#' @return plot as an object of type \code{\link{ggplot}}
#'
#' @author Fabian Mueller
#' @export
rnb.plot.num.sites.covg <- function(rnbs, addSampleNames=(length(samples(rnbs))<100), bar.percentiles=c(0.25,0.75)){
	rnb.require("scales")
	perc.vec <- c(bar.percentiles[1],0.5,bar.percentiles[2])
	df2p <- do.call("rbind",lapply(1:length(samples(rnbs)),FUN=function(j){
		sn <- samples(rnbs)[j]
		cc <- as.vector(covg(rnbs,j=j))
		cc[cc<1] <- NA
		ns <- sum(!is.na(cc))
		quants <- quantile(cc, probs=perc.vec, na.rm=TRUE)
		res <- data.frame(
			sample=sn,
			numSites=ns,
			covgPercLow=quants[1],
			covgMedian=quants[2],
			covgPercUp=quants[3],
			stringsAsFactors=FALSE
		)
		return(res)
	}))
	rownames(df2p) <- samples(rnbs)

	pp <- ggplot(df2p) + aes(x=numSites, y=covgMedian, ymin=covgPercLow, ymax=covgPercUp) + geom_pointrange() +
		  scale_x_continuous(labels = comma) +  ylab(paste("coverage (median,",bar.percentiles[1],"and",bar.percentiles[2],"percentiles)"))
	if (addSampleNames){
		pp <- pp + geom_text(aes(label=sample), hjust=0.5, vjust=-0.5, angle=90, size=2, color="grey50")
	}
	pp
}

library(argparse)
suppressPackageStartupMessages(library(RnBeads))

ap <- ArgumentParser()
ap$add_argument("-x", "--xml", action="store", help="Configuration xml file")
ap$add_argument("-s", "--rnbSet", action="store", dest="rnbSetLoc", help="Location of stored RnBSet")
ap$add_argument("-i", "--id", action="store", help="An analysis id that will be used to name the output")
ap$add_argument("-o", "--output", action="store", help="Output directory")
ap$add_argument("-c", "--cores", action="store", type="integer", default=1, help="Number of cores used for the analysis")
ap$add_argument("-r", "--region", action="append", dest="regionTypes", help="region type")
ap$add_argument("-p", "--pheno", action="append", dest="comparisons", help="Phenotype/comparison column")
cmdArgs <- ap$parse_args()
module.name <- "differential_chunk"

logger.start(fname=NA)
logger.status(c("...Started module:",module.name))

logger.start("Configuring Analysis")
	rnb.settings <- rnb.xml2options(cmdArgs$xml,return.full.structure=TRUE)

	report.dir <- rnb.settings$analysis.params[["dir.reports"]]
	analysis.options <- rnb.settings$options

	cmp.cols <- cmdArgs$comparisons
	reg.types <- cmdArgs$regionTypes
	chunk.id <- cmdArgs$id

	if ("preanalysis.script" %in% names(rnb.settings)){
		source(rnb.settings$preanalysis.script)
	} 
	## Set options
	if (length(analysis.options) != 0) {
		do.call(rnb.options, analysis.options)
	}

	if(rnb.getOption("region.subsegments")>1L){
		reg.types.subseg <- grep("\\.subsegments$",reg.types,value=TRUE)
		if (length(reg.types.subseg)>0){
			subseg.annot.dir <- file.path(report.dir,"preprocessing_data","subsegments")
			logger.start("Loading region sub-segmentation annotation")
			for (rt in reg.types.subseg){
				logger.status(c("Loading annotation",rt))
				fn <- file.path(subseg.annot.dir,paste0("regions_",rt,".RData"))
				rnb.load.annotation(fn, rt)
			}
			logger.completed()
		}
	}

	logger.machine.name()

	if (cmdArgs$cores > 1) {
		parallel.setup(cmdArgs$cores)
	}

	aname <- rnb.getOption("analysis.name")
	if (!(is.null(aname) || is.na(aname) || nchar(aname) == 0)) {
		logger.info(c("Analysis Title:", aname))
	}
	ncores <- parallel.getNumWorkers()
	if (ncores == -1) {
		ncores <- 1L
	}
	logger.info(c("Number of cores:", ncores))
	rm(aname, ncores)

	lolaDbPaths <- NULL
	savedLolaDbPath <- file.path(cmdArgs$output, "lolaDbPaths.rds")
	if(rnb.getOption("differential.enrichment.lola")){
		if (file.exists(savedLolaDbPath)){
			lolaDbPaths <- readRDS(savedLolaDbPath)
			logger.info(c("Successfully loaded LOLA DB paths"))
		} else{
			logger.info(c("No LOLA DB file found"))
		}
		
	}
logger.completed()

logger.start("Loading RnBSet")
	rnb.set <- load.rnb.set(cmdArgs$rnbSetLoc)
logger.completed()

logger.start(fname=c(file.path(report.dir,paste0("analysis_",module.name,"_",chunk.id,".log")),NA))

################################################################################
# main script
################################################################################

permutations <- rnb.getOption("differential.permutations")

logger.start("Differential Methylation")
	logger.info(c("Using",permutations,"permutation tests"))

	if (is.null(cmp.cols)){
		cmp.cols <- colnames(pheno(rnb.set))
	}
	logger.info(c("Using columns:",paste(cmp.cols,collapse=",")))

	if (length(reg.types)>0){
		logger.info(c("Using region types:",paste(reg.types,collapse=",")))
	} else {
		logger.info(c("Skipping region level analysis"))
	}
	disk.dump <- rnb.getOption("disk.dump.big.matrices")
	diffmeth <- rnb.execute.computeDiffMeth(rnb.set,cmp.cols,region.types=reg.types,n.perm=permutations,covg.thres=rnb.getOption("filtering.coverage.threshold"),
			pheno.cols.all.pairwise=rnb.getOption("differential.comparison.columns.all.pairwise"),columns.pairs=rnb.getOption("columns.pairing"),
			skip.sites=!rnb.getOption("analyze.sites"),
			# disk.dump=disk.dump,disk.dump.dir=paste0(cmdArgs$output,"_diffMethTableDir"))
			disk.dump=disk.dump,disk.dump.dir=paste0(tempfile(pattern=""),"_diffMethTableDir"))
	if (rnb.getOption("differential.enrichment.go")){
		dm.go.enrich <- performGoEnrichment.diffMeth(rnb.set,diffmeth,verbose=FALSE)
		if(rnb.getOption("differential.variability")){
		  dm.go.enrich <- performGOEnrichment.diffVar(rnb.set,diffmeth,dm.go.enrich,verbose=FALSE)
		}
	} else {
		dm.go.enrich <- NULL
		logger.info(c("Skipping GO enrichment analysis of differentially methylated regions"))
	}
	if (rnb.getOption("differential.enrichment.lola") && !is.null(lolaDbPaths)){
		dm.lola.enrich <- performLolaEnrichment.diffMeth(rnb.set,diffmeth,lolaDbPaths,verbose=FALSE)
		if(rnb.getOption("differential.variability")){
		  dm.lola.enrich <- performLolaEnrichment.diffVar(rnb.set,diffmeth,enrich.diffMeth = dm.lola.enrich, lolaDbPaths,verbose=FALSE)
		}
	} else {
		dm.lola.enrich <- NULL
		logger.info(c("Skipping LOLA enrichment analysis of differentially methylated regions"))
	}


	logger.start("Saving")
		diffmeth.path <- file.path(cmdArgs$output,paste0(module.name,"_",chunk.id,"_rnbDiffMeth"))
		save.rnb.diffmeth(diffmeth, diffmeth.path)
		diffmeth.go.enrichment <- dm.go.enrich
		if (!is.null(diffmeth.go.enrichment)){
			save(diffmeth.go.enrichment, file=file.path(diffmeth.path, "enrichment_go.RData"))
		}
		diffmeth.lola.enrichment <- dm.lola.enrich
		if (!is.null(diffmeth.lola.enrichment)){
			save(diffmeth.lola.enrichment, file=file.path(diffmeth.path, "enrichment_lola.RData"))
		}
	logger.completed()
logger.completed()

logger.start("Check result")
#check whether the output regions match the annotation regions
comps <- get.comparisons(diffmeth)
rts <- reg.types
if (rnb.getOption("analyze.sites")) rts <- c("sites",rts)
for (rr in rts){
	aa <- annotation(rnb.set,rr)
	n.regs <- nrow(aa)
	for (cc in comps){
		dmt <- get.table(diffmeth,cc,rr,return.data.frame=TRUE)
		if (n.regs != nrow(dmt)){
			msg <- paste0("Number of regions in the DMR table does not match the number in the annotation table [",cc,"--",rr,"]. This could be a memory issue. Try assigning more main memory to the task or reducing parallelity.")
			logger.error(msg)
		}
	}
}
logger.completed()

logger.status(c("...Completed module:",module.name))
quit(save='no')

########################################################################################################################
## Report-methods.R
## created: 2012-05-08
## creator: Yassen Assenov
## ---------------------------------------------------------------------------------------------------------------------
## Report class method definition.
########################################################################################################################

## F U N C T I O N S ###################################################################################################

#' Checks if the given strings are valid to be used as file names.
#'
#' @param fname \code{character} vector of candidate strings for names.
#' @return \code{logical} vector showing, for each element of \code{fname}, if it meets the criteria for a vaild file
#'         name.
#' @author Fabian Mueller
#' @noRd
is.valid.fname <- function(fname) {
	grepl("^[A-Za-z0-9._-]+$", fname)
}

########################################################################################################################

#' Checks if the given path is valid to be used as a relative link (file or directory) in an HTML report.
#'
#' @param dname Name of the file or directory.
#' @return \code{TRUE} if \code{dname} meets the criteria for a file or directory to be easily referenced in an HTML
#'         link, \code{FALSE} otherwise.
#' @author Yassen Assenov
#' @noRd
is.valid.relative <- function(dname) {
	if (!grepl("^[A-Za-z0-9/\\._-]+$", dname)) {
		return(FALSE)
	}
	if (grepl("^/", dname)) {
		return(FALSE)
	}
	if (grepl("/$", dname)) {
		return(FALSE)
	}
	return(TRUE)
}

########################################################################################################################

#' Creates the given directory. This function attempts to create all non-existant elements of the given path.
#'
#' @param dname           Directory to be created.
#' @param accept.existing Flag indicating if an existing empty directory should be accepted. When this is \code{TRUE}
#'                        and \code{dname} points to an existing empty directory, this function returns \code{TRUE}.
#'                        Hidden files (file names starting with \code{"."} on Unix platforms) are ignored. Thus, if
#'                        \code{dname} already exists and contains hidden files only, it is treated as empty.
#' @param showWarnings    Flag indicating if warnings should be shown.
#' @return                \code{logical} indicating if the creation was successful.
#'
#' @seealso \code{\link{dir.create}}
#' @author Yassen Assenov
#' @noRd
create.path <- function(dname, accept.existing = TRUE, showWarnings = TRUE) {
	if (file.exists(dname)) {
		if (accept.existing) {
			if (file.info(dname)[1, "isdir"]) {
				## TODO: Should invisible files also be considered?
				return(length(dir(dname)) == 0)
			}
		}
		return(FALSE)
	}
	return(dir.create(dname, showWarnings = showWarnings, recursive = TRUE))
}

########################################################################################################################

#' Completes the given HTML report by adding a footer notice and closing the \code{<body>} and \code{<html>} tags.
#'
#' @param report Report instance to close.
#' @param rtype  Contents of the HTML file. It should be \code{"report"} or \code{"index"}.
#' @author Yassen Assenov
#' @noRd
complete.report <- function(report, report.type = "report") {

	## Add list of references to the report
	if (length(report@references) != 0) {
		report <- rnb.add.section(report, "References", NULL)
		reftexts <- report@references
		reftexts <- paste0("<a id=\"ref", 1:length(reftexts), "\">", reftexts, "</a>")
		rnb.add.list(report, as.list(reftexts), type = "o")
	}

	## Close any open sub-sections and sections
	for (i in length(report@opensections):1) {
		if (report@opensections[i] != 0) {
			write.line('</div>', report@fname)
			report@opensections[i] <- 0L
		}
	}

	## Add footer to the report
	write.line("\n<div id=\"copyright\">", report@fname)
	write.line("\t<div id=\"rnbeads\">", report@fname)
	write.line(c("\tThis ", report.type, " was generated on ", format(Sys.time(), "%Y-%m-%d"), " by ",
			"<a href=\"http://rnbeads.org/\">RnBeads</a> version ",
			paste(as.character(utils::packageVersion("RnBeads")), collapse = "."), "."), report@fname)
	write.line("\t</div>", report@fname)
	write.line("\t<div id=\"validlogo\">", report@fname)
	write.line(c("\t\t<a href=\"http://validator.w3.org/check?uri=referer\">",
			"<img alt=\"Valid XHTML 1.1\" height=\"31\" src=\"http://www.w3.org/Icons/valid-xhtml11-blue\"",
			" width=\"88\" /></a>"), report@fname)
	write.line(c("\t\t<a href=\"http://jigsaw.w3.org/css-validator/check/referer\">",
			"<img alt=\"Valid CSS\" height=\"31\" src=\"http://jigsaw.w3.org/css-validator/images/vcss-blue\"",
			" width=\"88\" /></a>"), report@fname)
	write.line("\t</div>", report@fname)
	write.line("</div>", report@fname)

	## Close all tags
	write.line("</body>", report@fname)
	write.line("</html>", report@fname)
	return(report)
}

########################################################################################################################

setMethod("show", "Report",
	function(object) {
		cat("Object of class Report - an XHTML Report in RnBeads\n")
		infotext <- c("Report's file" = object@fname,
			"Directory for configuration files" = object@dir.conf,
			"Directory for data files" = object@dir.data,
			"Directory for figure image files" = object@dir.pngs,
			"Directory for figure PDF files" = object@dir.pdfs,
			"Directory for high-resolution image files" = object@dir.high,
			"Number of references" = length(object@references))
		n <- max(nchar(names(infotext)))
		for (i in 1:length(infotext)) {
			cat(sprintf(paste0("%", n, "s: %s\n"), names(infotext)[i], infotext[i]))
		}
	}
)

########################################################################################################################

#' rnb.initialize.reports
#'
#' Creates a new directory to host HTML reports and copies the shared configuration files.
#'
#' @param dir.reports       Directory to host report files. This must be a \code{character} of length one that specifies
#'                          a non-existent path, as this methods attempts to create it.
#' @param dir.configuration Subdirectory to host configuration files shared by the reports. This must be a
#'                          \code{character} of length one that gives location as a path relative to \code{dir.reports}.
#'                          Also, strong restrictions apply to the path name. See the description of the
#'                          \code{\link{createReport}} function for more details. This method creates the directory and
#'                          copies configuration files that define cascading style sheet (CSS) definitions and
#'                          Javascript functions used by the HTML reports.
#' @return \code{TRUE} if the report directory was successfully created and the configuration files were copied to the
#'         specified location; \code{FALSE} otherwise.
#'
#' @examples
#' \donttest{
#' dir.reports <- "~/infinium_studies/cancer_study/reports"
#' if (!rnb.initialize.reports(dir.reports)) {
#'     cat("ERROR: Could not initialize configuration in ", dir.reports, "\n", sep = "")
#' }
#' }
#'
#' @seealso \code{\link{createReport}} for initializing an HTML report
#' @author Yassen Assenov
#' @export
rnb.initialize.reports <- function(dir.reports, dir.configuration = "configuration") {
	if (!(is.character(dir.reports) && length(dir.reports) == 1 && (!is.na(dir.reports[1])))) {
		stop("invalid value for dir.reports")
	}
	if (!(is.character(dir.configuration) && length(dir.configuration) == 1 && (!is.na(dir.configuration[1])))) {
		stop("invalid value for dir.configuration")
	}
	if (!is.valid.relative(dir.configuration)) {
		stop("invalid value for dir.configuration")
	}
	## Remove trailing slash or backslash in dir.reports (if it exists)
	dir.reports <- sub("^(.+)[/\\\\]$", "\\1", dir.reports[1])
	## Attempt to create dir.reports
	if (!create.path(dir.reports, FALSE, showWarnings = FALSE)) {
		return(FALSE)
	}
	## Attempt to create dir.configuration
	dname <- normalizePath(file.path(dir.reports, dir.configuration), mustWork = FALSE)
	if (!create.path(dname, showWarnings = FALSE)) {
		return(FALSE)
	}
	## Copy configuration files
	cfiles <- c("arrow_down.png", "arrow_right.png", "RnBeads.png", "pdf_active.png", "pdf_inactive.png", "report.css",
		"report.js")
	cfiles <- system.file(file.path("extdata", cfiles), package = "RnBeads", mustWork = TRUE)
	return(all(file.copy(cfiles, dname)))
}

## M E T H O D S #######################################################################################################

setValidity("Report",
	function(object) {
		res <- validate.single(object@fname, "fname")
		if (res != "ok") { return(res) }
		res <- validate.single(object@dir.conf, "dir.conf")
		if (res != "ok") { return(res) }
		res <- validate.single(object@dir.data, "dir.data")
		if (res != "ok") { return(res) }
		res <- validate.single(object@dir.pngs, "dir.pngs")
		if (res != "ok") { return(res) }
		res <- validate.single(object@dir.pdfs, "dir.pdfs")
		if (res != "ok") { return(res) }
		res <- validate.single(object@dir.high, "dir.high")
		if (res != "ok") { return(res) }
		res <- validate.single(object@figures, "figures")
		if (res != "ok") { return(res) }
		res <- validate.single(object@tables, "tables")
		if (res != "ok") { return(res) }
		regex.fname <- "^[A-Za-z0-9/\\._-]+$"
		if (!grepl(regex.fname, basename(object@fname))) {
			return("fname is invalid")
		}
		if (!grepl(".+\\.(htm|html|xhtml|xml)$", tolower(basename(object@fname)))) {
			return("fname is invalid")
		}
		if (!is.valid.relative(object@dir.data)) {
			return("dir.data is invalid")
		}
		if (!is.valid.relative(object@dir.pngs)) {
			return("dir.pngs is invalid")
		}
		if (!is.valid.relative(object@dir.pdfs)) {
			return("dir.pdfs is invalid")
		}
		if (!is.valid.relative(object@dir.high)) {
			return("dir.high is invalid")
		}
		if (object@sections < 0) {
			return("sections must be a non-negative value")
		}
		if (object@figures < 0) {
			return("figures must be a non-negative value")
		}
		if (object@tables < 0) {
			return("tables must be a non-negative value")
		}
		TRUE
	}
)

########################################################################################################################

setMethod("initialize", "Report",
	function(.Object, fname, title, page.title, authors, dirs, init.configuration) {
		.Object@fname <- normalizePath(fname, mustWork = FALSE)
		## Read or generate report-specific directory names
		dir.configuration <- dirs["configuration"]
		if (is.na(dir.configuration)) {
			dir.configuration <- "configuration"
		}
		if (!is.valid.relative(dir.configuration)) {
			stop("dir.configuration is invalid")
		}
		.Object@dir.conf <- dir.configuration
		dprefix <- sub("^(.*[\\\\/])*([^\\\\/]+)\\.[^\\.]+$", "\\2", fname)
		.Object@dir.data <- dirs["data"]
		if (is.na(.Object@dir.data)) {
			.Object@dir.data <- paste(dprefix, "data", sep = "_")
		}
		.Object@dir.pngs <- dirs["pngs"]
		if (is.na(.Object@dir.pngs)) {
			.Object@dir.pngs <- paste(dprefix, "images", sep = "_")
		}
		.Object@dir.pdfs <- dirs["pdfs"]
		if (is.na(.Object@dir.pdfs)) {
			.Object@dir.pdfs <- paste(dprefix, "pdfs", sep = "_")
		}
		.Object@dir.high <- dirs["high"]
		if (is.na(.Object@dir.high)) {
			.Object@dir.high <- .Object@dir.pngs
		}
		validObject(.Object)
		if (!is.null(authors) && length(authors) != 0) {
			if (!all(grepl("^[A-Za-z ,.-]+$", authors))) {
				stop("invalid value for authors")
			}
			authors <- paste(authors, collapse = ", ")
		} else {
			authors <- NULL
		}

		## Create directories
		dnames <- file.path(dirname(.Object@fname), c(.Object@dir.data, .Object@dir.pngs, .Object@dir.pdfs, .Object@dir.high))
		dnames <- setdiff(unique(normalizePath(dnames, mustWork = FALSE)), dirname(.Object@fname))
		for (dname in dnames) {
			if (!create.path(dname)) {
				stop(paste("directory", dname, "cannot be created"))
			}
		}
		rm(dnames, dname)

		## Validate configuration directory; create if it's missing
		if (init.configuration) {
			dname <- normalizePath(file.path(dirname(.Object@fname), dir.configuration), mustWork = FALSE)
			if (exists(dname)) {
				if (file.info(dname)[1, "isdir"] == FALSE) {
					stop(paste(dname, "is not a directory"))
				}
			} else if (!create.path(dname)) {
				stop(paste("directory", dname, "cannot be created"))
			}
			## Copy configuration files
			cfiles <- c("arrow_down.png", "arrow_right.png", "RnBeads.png", "pdf_active.png", "pdf_inactive.png",
				"report.css", "report.js")
			cfiles <- system.file(file.path("extdata", cfiles), package = "RnBeads", mustWork = TRUE)
			if (!all(file.copy(cfiles, dname))) {
				stop(paste("configuration could not be initialized in", dname))
			}
			rm(dname, cfiles)
		}

		## Create the HTML file and print its header
		dir.configuration <- ifelse(dir.configuration != ".", paste0(dir.configuration, "/"), "")
		wline <- function(txt, indent = 0) {
			write.line(txt, fname, indent = indent)
		}
		write.line("<?xml version=\"1.0\" encoding=\"UTF-8\"?>", fname, append = FALSE)
		wline("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.1//EN\" \"http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd\">")
		wline("<html xmlns=\"http://www.w3.org/1999/xhtml\">")
		wline("<head>")
		wline("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />", 1)
		if (!is.null(authors)) {
			wline(c("<meta name=\"author\" content=\"", authors, "\" />"), 1)
		}
		wline(c("<link rel=\"stylesheet\" href=\"", dir.configuration, "report.css\" type=\"text/css\" />"), 1)
		if (!is.null(page.title)) {
			wline(c("<title>", page.title[1], "</title>"), 1)
		}
		wline(c("<script src=\"", dir.configuration, "report.js\" type=\"text/javascript\"></script>"), 1)
		wline("</head>\n")
		wline(c("<body style=\"background:url(", dir.configuration, "RnBeads.png) no-repeat 5px 25px;\">\n"))
		wline(c("<h1>", title, "</h1>\n"))
		.Object
	}
)

########################################################################################################################

#' rnb.get.directory
#'
#' Gets the location of the given report-specific directory.
#'
#' @param report   Report of interest.
#' @param dir      Type of directory to get. Must be one of \code{"data"}, \code{"images"}, \code{"images-high"} or
#'                 \code{"pdfs"}.
#' @param absolute Flag indicating if the absolute path of the directory is to be returned. If this is \code{FALSE},
#'                 the directory name is returned relative to the report's HTML file location.
#' @return Path of the requested directory as a single-element \code{character} vector.
#'
#' @examples
#' \donttest{
#' report <- createReport("example.html", "Example", init.configuration = TRUE)
#' rnb.get.directory(report, "data")
#' }
#' @seealso \code{\linkS4class{Report}} for functions adding contents to an HTML report
#' @author Yassen Assenov
#' @export
rnb.get.directory <- function(report, dir = c("data", "images", "images-high", "pdfs"), absolute = FALSE) {
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	if (is.null(dir) || length(dir) == 0 || (!(dir[1] %in% c("data", "images", "images-high", "pdfs")))) {
		stop("invalid value for dir")
	}
	if (!parameter.is.flag(absolute)) {
		stop("invalid value for absolute; expected TRUE or FALSE")
	}
	dir <- dir[1]
	if (dir == "data") {
		result <- report@dir.data
	} else if (dir == "pdfs") {
		result <- report@dir.pdfs
	} else if (dir == "images") {
		result <- report@dir.pngs
	} else { # dir == "images-high"
		result <- report@dir.high
	}
	if (absolute) {
		result <- normalizePath(file.path(dirname(report@fname), result), mustWork = FALSE)
	}
	return(result)
}

########################################################################################################################

#' rnb.add.section
#'
#' Generates HTML code for a new section in the specified report.
#'
#' @param report      Report to write the text to.
#' @param title       Section header. This must be a single-element \code{character} vector.
#' @param description Human-readable paragraph text of the section in the form of a \code{character} vector. Elements
#'                    of this vector are concatenated without a separator to form the full description. Set this to
#'                    \code{NULL} if the section does not (yet) contain text.
#' @param level       Section level as a single \code{integer}. It must be one of \code{1}, \code{2} or \code{3},
#'                    denoting section, subsection and sub-subsection, respectively.
#' @param collapsed   Flag indicating if the contents of this section is to be initially collapsed. Possible values are
#'                    \code{TRUE} (the section is not visible), \code{FALSE} (default, the section is expanded) and
#'                    \code{"never"} (the section cannot be collapsed or expanded).
#' @return The modified report.
#'
#' @examples
#' \donttest{
#' report <- createReport("example.html", "Example", init.configuration = TRUE)
#' report <- rnb.add.section(report, "Introduction", "This is how it's done.")
#' }
#' @seealso \code{\linkS4class{Report}} for other functions adding contents to an HTML report
#' @author Yassen Assenov
#' @export
rnb.add.section <- function(report, title, description, level = 1L, collapsed = FALSE) {
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	if (!is.character(title) || validate.single(title) != "ok") {
		stop("invalid value for title")
	}
	if (!(is.null(description) || is.character(description))) {
		stop("invalid value for description")
	}
	if (is.numeric(level) && all(level == as.integer(level))) {
		level <- as.integer(level)
	}
	if (!(is.integer(level) && length(level) == 1 && 1 <= level[1] && level[1] <= 3)) {
		stop("invalid value for level")
	}
	if (is.character(collapsed)) {
		if (!(length(collapsed) == 1 && isTRUE(collapsed == "never"))) {
			stop('invalid value for collapsed; expected TRUE, FALSE or "never"')
		}
		hclass <- 'fixed'
	} else {
		if (!parameter.is.flag(collapsed)) {
			stop('invalid value for collapsed; expected TRUE, FALSE or "never"')
		}
		hclass <- ifelse(collapsed, 'collapsed', 'expanded')
	}
	## Close previous sub-sections and sections if necessary.
	for (i in length(report@opensections):level) {
		if (report@opensections[i] != 0) {
			write.line('</div>', report@fname)
			report@opensections[i] <- 0L
		}
	}

	i <- report@sections <- report@sections + 1L
	attr.text <- ifelse(is.logical(collapsed), paste0(' onclick="toggleSection(\'', i, '\')"'), '')
	attr.text <- paste0(' class="', hclass, '" id="hsection', i, '"', attr.text)
	write.line(c("\n<h", (level + 1), attr.text, ">", title, "</h", (level + 1), ">"), report@fname)

	if (is.logical(collapsed)) {
		txt <- c('<div id="section', i, '" style="display:', ifelse(collapsed, 'none', 'block'), '">')
		write.line(txt, report@fname)
		
		## Mark the newly open section
		report@opensections[level] <- i
	}
	if (!is.null(description)) {
		write.line(c("<p>", description, "</p>"), report@fname)
	}
	return(report)
}

########################################################################################################################

#' rnb.add.paragraph
#'
#' Generates HTML code for a new paragraph in the specified report.
#'
#' @param report          Report to write the text to.
#' @param txt             \code{character} vector (or array) storing the text to be written. The elements of this vector
#'                        are concatenated without a separator.
#' @param paragraph.class CSS class definition of the paragraph. This must be either \code{NULL} (default) or one of:
#'                        \describe{
#'                          \item{\code{"centered"}}{This paragraph gives a formula or a short statement. Text is
#'                               horizontally centered.}
#'                          \item{\code{"note"}}{This paragraph describes a note. Text is italic.}
#'                          \item{\code{"task"}}{This paragraph describes a task. Text is bold and bright red.}
#'                        }
#' @return The modified report, invisibly.
#'
#' @examples
#' \donttest{
#' report <- createReport("example.html", "Example", init.configuration = TRUE)
#' txt <- "A pessimist is a person who has had to listen to too many optimists."
#' txt <- c(txt, " <i>Don Marquis</i>")
#' rnb.add.paragraph(report, txt)
#' }
#' @seealso \code{\linkS4class{Report}} for other functions adding contents to an HTML report
#' @author Yassen Assenov
#' @export
rnb.add.paragraph <- function(report, txt, paragraph.class = NULL) {
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	if (!is.character(txt)) {
		stop("invalid value for txt; expected character vector")
	}
	if (is.null(paragraph.class)) {
		paragraph.class <- ""
	} else {
		if (!(is.character(paragraph.class) && length(paragraph.class) == 1 && (!is.na(paragraph.class)))) {
			stop("invalid value for paragraph.class; expected single character")
		}
		paragraph.class <- paragraph.class[1]
		if (!(paragraph.class %in% c("centered", "note", "task"))) {
			stop("invalid value for paragraph.class; expected one of centered, note, task")
		}
		paragraph.class <- paste(" class=", paragraph.class, "", sep = "\"")
	}
	write.line(c("<p", paragraph.class, ">", txt, "</p>"), report@fname)
	return(invisible(report))
}

########################################################################################################################

#' rnb.add.list
#'
#' Generates HTML code for a list in the specified report.
#'
#' @param report Report to write the text to.
#' @param txt    Non-empty list of items to be written. An attribute named \code{type}, if it exists, specifies the type
#'               of the list. See the \emph{Details} section for more information. Every item must be either a nested
#'               \code{list}, denoting a sublist, or a \code{character} vector (or array), storing the text to be
#'               written. Any other objects are coerced to a character type. Elements are concatenated without a
#'               separator to form the text for a list item.
#' @param type   List type to be used for the list and/or its sublists in case the attribute \code{type} is not
#'               specified.
#' @return The modified report, invisibly.
#'
#' @details There are two ways to specify a list type: (1) setting a value for the attribute \code{type} of the list, or
#'          (2) using the function's parameter \code{type}. The value of the function's parameter is used only for lists
#'          and sublists that do not contain an attribute named \code{type}. The following types are supported:
#'          \describe{
#'            \item{\code{"o"}}{Ordered list using arabic numbers - \code{1}, \code{2}, \code{3}, etc.}
#'            \item{\code{"u"}}{Unordered list using bullet points.}
#'          }
#'          Note that every list type must be a one-element \code{character} vector containing one of the codes listed
#'          above. Specifying any other value for list type results in an error.
#'
#' @examples
#' \donttest{
#' report <- createReport("example.html", "Example", init.configuration = TRUE)
#' recipe <- list("Sift flour in a bowl", "Add sugar and mix", "Add milk and mix")
#' rnb.add.list(report, recipe, type="o")
#' }
#' @seealso \code{\linkS4class{Report}} for other functions adding contents to an HTML report
#' @author Yassen Assenov
#' @export
rnb.add.list <- function(report, txt, type = "u") {
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	if (!(is.list(txt) && length(txt) != 0)) {
		stop("invalid value for txt; expected non-empty list")
	}
	if (!(is.character(type) && length(type) == 1)) {
		stop("invalid value for type; character expected")
	}
	type <- tolower(type)
	## TODO: Add support for more list types
	if (!(type %in% c("o", "u"))) {
		stop("invalid value for type; expected one of: o, u")
	}
	get.html <- function(txt, indent) {
		ltype <- attr(txt, "type")
		if (is.null(ltype)) { ltype <- type }
		if (ltype == "u") {
			tagtext <- "ul"
		} else if (ltype == "o") {
			tagtext = "ol"
		} else {
			stop("invalid list type; expected one of: o, u")
		}
		indenttext <- paste(rep("\t", times = indent), collapse = "")
		contents <- sapply(txt, function(litem) {
			if (is.list(litem)) {
				paste0("\t<li>\n", paste(get.html(litem, indent + 2), collapse = ""), "\n\t</li>\n")
			} else {
				paste0("\t<li>", paste(litem, collapse = ""), "</li>\n")
			}
		})
		paste0(indenttext, c(paste0("<", tagtext, ">\n"), contents, paste0("</", tagtext, ">")))
	}
	write.line(get.html(txt, 0L), report@fname)
	return(invisible(report))
}

########################################################################################################################

#' rnb.add.table
#'
#' Generates HTML code for a table in the specified report.
#'
#' @param report           Report to write the text to.
#' @param tdata            Matrix or data frame to be presented in HTML form. Column names, if present, are used to
#'                         define table columns. If this table contains 0 (zero) rows or 0 columns, calling this
#'                         function has no effect.
#' @param row.names        Flag indicating if row names should also be printed. If this parameter is \code{TRUE} and
#'                         \code{tdata} defines row names, these are printed in the left-most column and are displayed
#'                         as header cells. Keep in mind that \code{data.frame}s always define row names.
#' @param first.col.header Flag indicating if all cells in the first column must be displayed as header cells. Note
#'                         that, if both this parameter and \code{row.names} are \code{TRUE}, and \code{tdata} contains
#'                         row names, the constructed HTML table will have 2 columns of header cells.
#' @param indent           Default indentation, in number of tabulation characters, to apply to HTML tags. This
#'                         indentation is also applied to \code{thead}.
#' @param tag.attrs        Named \code{character} vector specifying the list of attributes to be set to the
#'                         \code{<table>} element. Setting this to \code{NULL} or an empty \code{character} vector
#'                         disables attributes.
#' @param thead            \code{character} vector storing a table header to include. This can, for example, be a
#'                         \code{character} that defines column widths. Every element in this vector is written on a
#'                         separate line, applying the indentation given by \code{indent}.
#' @param tcaption         Text to include as a caption below the table, or \code{NULL} if the table does not contain
#'                         caption.
#' @param na               \code{character} to be used for printing \code{NA} values in the table. This parameter is
#'                         not considered when printing \code{thead} or the table's column names.
#' @return The modified report, invisibly.
#'
#' @seealso \code{\link{rnb.add.tables}} for adding a listing of tables; \code{\linkS4class{Report}} for other functions
#'   adding contents to an HTML report
#' @author Yassen Assenov
#' @export
rnb.add.table <- function(report, tdata, row.names = TRUE, first.col.header = FALSE, indent = 0,
	tag.attrs = c("class" = "tabdata"), thead = NULL, tcaption = NULL, na = "<span class=\"disabled\">n/a</span>") {

	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	if (!(is.matrix(tdata) || is.data.frame(tdata))) {
		stop("invalid value for tdata; expected matrix or data.frame")
	}
	if (!parameter.is.flag(first.col.header)) {
		stop("invalid value for first.col.header; expected TRUE or FALSE")
	}
	if (is.numeric(indent) && all(indent == as.integer(indent))) {
		indent <- as.integer(indent)
	}
	if (!(is.integer(indent) && length(indent) == 1 && (!is.na(indent)) && 0 <= indent)) {
		stop("invalid value for indent; expected single non-negative integer")
	}
	if (!is.null(tag.attrs)) {
		if (!(is.character(tag.attrs) && length(tag.attrs) == length(names(tag.attrs)))) {
			stop("invalid value for tag.attrs; expected named character vector")
		}
		if (length(tag.attrs) != 0) {
			tag.attrs <- paste("", paste(names(tag.attrs), "=\"", tag.attrs, "\"", sep = "", collapse = " "))
		} else {
			tag.attrs <- ""
		}
	} else {
		tag.attrs <- ""
	}
	if (!is.null(thead)) {
		if (!(is.character(thead))) {
			stop("invalid value for thead; expected character")
		}
		if (length(thead) == 0) {
			thead <- NULL
		}
	}
	if (!is.null(tcaption)) {
		if (!is.character(tcaption)) {
			stop("invalid value for tcaption; expected character")
		}
	}
	if (!(is.character(na) && length(na) == 1)) {
		stop("invalid value for na; expected single character")
	}
	if (nrow(tdata) * ncol(tdata) == 0) {
		return(invisible(report))
	}

	## Convert factors to characters
	if (is.data.frame(tdata)) {
		for (j in 1:ncol(tdata)) {
			if (is.factor(tdata[[j]])) {
				tdata[[j]] <- as.character(tdata[[j]])
			}
		}
	}

	wline <- function(txt, indentation = indent) {
		write.line(txt, report@fname, indent = indentation)
	}
	wline(c("<div class=\"restricted\"><table", tag.attrs, ">"))
	if (!is.null(thead)) {
		mapply(wline, thead)
	}
	startcolumn <- as.integer(ifelse(row.names & (!is.null(rownames(tdata))), 0, 1))
	if (!is.null(colnames(tdata))) {
		wline("<tr>")
		if (startcolumn == 0) {
			cnames <- c("", colnames(tdata))
		} else {
			cnames <- colnames(tdata)
		}
		for (cname in cnames) {
			wline(c("<td class=\"header\">", cname, "</td>"), indent + 1)
		}
		wline("</tr>")
		rm(cnames)
	}
	if (nrow(tdata) != 0) {
		for (i in 1:nrow(tdata)) {
			wline("<tr>")
			for (j in startcolumn:ncol(tdata)) {
				value <- base::ifelse(j == 0, rownames(tdata)[i], tdata[[i, j]])
				if (is.na(value)) { value <- na }
				cclass <- ifelse(j == 0 || (j == 1 && first.col.header), "header", "centered")
				wline(c("<td class=\"", cclass, "\">", value, "</td>"), indent + 1)
			}
			wline("</tr>")
		}
	}
	wline("</table></div>")
	if (!is.null(tcaption)) {
		wline(c("<p class=\"centered\"><span class=\"note\">", tcaption, "</span></p>"))
	}
	wline("", indentation = 0)
	return(invisible(report))
}

########################################################################################################################

#' rnbreport.get.element.values
#'
#' @param elements       Observed values of element names, in the form of a \code{list} of \code{character} vectors of
#'                       equal length. The <i>i</i>-th element of this list should be extracted from the <i>i</i>-th
#'                       table identifier or plot file name.
#' @param settings.count Number of variable elements for which descriptions are provided. The is the length of
#'                       \code{setting.names}.
#' @return List of observed values per element; an empty list if no variation is observed (\code{elements} if of length
#'         one). In case the number of variable elements does not match \code{settings.count}, the return value is
#'         \code{character} containing an error message.
#' @author Yassen Assenov
#' @noRd
rnbreport.get.element.values <- function(elements, settings.count) {
	element.count <- range(sapply(elements, length))
	if (element.count[1] != element.count[2]) {
		return(NULL)
	}
	element.count <- element.count[1]
	if (element.count < settings.count) {
		return("too many entries in setting.names")
	}
	element.values <- lapply(1:element.count, function(i) { sort(unique(sapply(elements, "[", i))) })
	names(element.values) <- 1:length(element.values)
	i <- which(sapply(element.values, length) != 1)[1]
	if ((!is.na(i)) && settings.count < length(element.values) - i + 1) {
		return("too few entries in setting.names")
	}
	return(element.values)
}

########################################################################################################################

#' rnbreport.create.settings.table
#'
#' Creates the HTML code for a table of selecting settings. This is used in figures and in listings of tables.
#'
#' @param type              One of \code{"fig"} or \code{"tab"}.
#' @param n                 Index of figure or table listing in the report.
#' @param element.values    List of all observed values for each element in the identifiers. This is extracted by
#'                          \code{\link{rnbreport.get.element.values}}.
#' @param setting.names     List of element descriptors, as provided to the calling function.
#' @param selected.elements Element values that should be initially selected.
#' @param indent            Indentation, in number of tabulation characters, of the <table> tag.
#' @return One-element \code{character} containing the generated HTML code for the settings table; \code{NULL} if
#'         \code{setting.names} is incomplete, that is, not all provided \code{element.values} have descriptions.
#' @author Yassen Assenov
#' @noRd
rnbreport.create.settings.table <- function(type, n, element.values, setting.names, selected.elements, indent = 1L) {
	updatefun <- ifelse(type == "fig", "updateFigure", "updateTable")
	result <- character()
	wline <- function(txt, ind = 0L) {
		result <<- c(result, paste(c(paste(rep("\t", indent + ind), collapse = ""), txt), collapse = ""))
	}
	i <- length(element.values) - length(setting.names) + 1
	element.values <- element.values[i:length(element.values)]
	## Validate setting.names
	missing.names <- sapply(1:length(element.values), function(i) {
		length(setdiff(element.values[[i]], names(setting.names[[i]]))) != 0
	})
	if (any(missing.names)) {
		return(NULL)
	}

	## Create a table of settings
	wline("<table class=\"tabsettings\">")
	for (i in 1:length(element.values)) {
		si <- as.integer(names(element.values)[i])
		wline("<tr>")
		wline(c("<td>", names(setting.names)[i], "</td>"), 1)
		wline("<td>", 1)
		txt <- ifelse(length(element.values[[i]]) == 1, " disabled=\"disabled\"", "")
		txt <- c("<select", txt, " id=\"", type, n, "setting", si)
		wline(c(txt, "\" onchange=\"", updatefun, "(\'", type, n, "\')\">"), 2)
		s.options <- setting.names[[i]]
		s.options <- s.options[intersect(names(s.options), element.values[[i]])]
		for (j in 1:length(s.options)) {
			sv <- names(s.options)[j]
			txt <- ifelse(selected.elements[si] == sv, " selected=\"selected\"", "")
			wline(c("<option", txt, " value=\"", sv, "\">", s.options[j], "</option>"), 3)
		}
		wline("</select>", 2)
		wline("</td>", 1)
		wline("</tr>")
	}
	wline("</table>")
	wline("<br />")

	return(paste(result, collapse = "\n"))
}

########################################################################################################################

#' rnb.add.tables
#'
#' Generates HTML code for a listing of tables (of which only one is visible at any moment) in the specified report.
#'
#' @param report         Report to write the text to.
#' @param tables         Non-empty \code{list} of tables, each one represented by a \code{\link{data.frame}} or
#'                       \code{\link{matrix}}. The names of this list are used as table identifiers; each one consists
#'                       of elements separated by underscore character (\code{_}).
#' @param setting.names  List of table name element descriptors. Every variable elements in the table names must be
#'                       included in this list.
#' @param selected.table Index of the table to be initially selected in this listing.
#' @param indent         Default indentation, in number of tabulation characters, to apply to every table.
#' @param ...            Other parameters passed to \code{\link{rnb.add.table}}.
#' @return The modified report.
#'
#' @seealso \code{\link{rnb.add.table}} for adding a single table to a report; \code{\linkS4class{Report}} for other
#'   functions adding contents to an HTML report
#' @author Yassen Assenov
#' @export
rnb.add.tables <- function(report, tables, setting.names, selected.table = 1L, indent = 2L, ...) {
	
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	if (!(is.list(tables) && length(tables) != 0 &&
			all(sapply(tables, function(x) { is.matrix(x) || is.data.frame(x) })))) {
		stop("invalid value for tables")
	}
	if (!(is.list(setting.names) && all(sapply(setting.names, is.vector)))) {
		stop("invalid value for setting.names")
	}
	if (is.numeric(selected.table) && all(as.integer(selected.table) == selected.table)) {
		selected.table <- as.integer(selected.table)
	}
	if (!is.integer(selected.table) && length(selected.table) == 1 && (!is.na(selected.table))) {
		stop("invalid value for selected.table")
	}
	if (selected.table < 1 || length(tables) < selected.table) {
		stop("invalid value for selected.table")
	}

	## Identify name elements
	elements <- strsplit(names(tables), "_", fixed = TRUE)
	element.values <- rnbreport.get.element.values(elements, length(setting.names))
	if (is.null(element.values)) {
		stop("inconsistent table names")
	}
	if (is.character(element.values)) {
		stop(element.values)
	}

	n <- report@tables + 1L
	wline <- function(txt, ind = 0L) {
		write.line(txt, report@fname, indent = ind)
	}

	## Create a table of settings
	wline(c("<div class=\"figure\" id=\"tab", n, "figure\">"), 0)
	if (length(setting.names) != 0) {
		txt <- rnbreport.create.settings.table("tab", n, element.values, setting.names, elements[[selected.table]])
		if (is.null(txt)) {
			stop("missing plot file element descriptors")
		}
		wline(txt, 0)
	}

	## Create tables
	for (i in 1:length(tables)) {
		txt <- c(" style=\"display:", ifelse(i == selected.table, "block", "none"), "\"")
		wline(c("<div id=\"tab", n, "_", names(tables)[i], "\"", txt, ">"), 1)
		rnb.add.table(report, tables[[i]], indent = indent, ...)
		wline("</div>\n", 1)
	}

	wline("</div>", 0)
	report@tables <- n
	return(report)
}

########################################################################################################################

#' rnb.add.figure
#'
#' Generates HTML code for a figure in the specified report. A figure is a collection of images (plots), of which only
#' one is visible at any given moment.
#'
#' @param report         Report to write the text to.
#' @param description    Human-readable description of the figure. This must be a non-empty \code{character} vector. The
#'                       elements of this vector are concatenated without a separator to form the full description.
#' @param report.plots   Object of type \code{\linkS4class{ReportPlot}}, or a list of such objects.
#' @param setting.names  List of plot file element descriptors. Every variable elements in the plot file names must be
#'                       included in this list. Set this to empty list if no variable elements are present, that is, if
#'                       the figure should present a single report plot.
#' @param selected.image Index of plot to be initially selected in the figure.
#' @return The modified report.
#'
#' @seealso \code{\link{rnb.add.tables}} for adding a listing of tables; \code{\linkS4class{Report}} for other functions
#'   adding contents to an HTML report
#' @author Yassen Assenov
#' @export
rnb.add.figure <- function(report, description, report.plots, setting.names = list(), selected.image = as.integer(1)) {

	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	if (!(is.character(description) && length(description) != 0)) {
		stop("invalid value for description")
	}
	if (inherits(report.plots, "ReportPlot")) {
		report.plots <- list(report.plots)
	}
	if (!(is.list(report.plots) && length(report.plots) != 0 && all(sapply(report.plots, inherits, "ReportPlot")))) {
		stop("invalid value for report.plots")
	}
	if (!(is.list(setting.names) && all(sapply(setting.names, is.vector)))) {
		stop("invalid value for setting.names")
	}
	if (is.numeric(selected.image) && all(as.integer(selected.image) == selected.image)) {
		selected.image <- as.integer(selected.image)
	}
	if (!is.integer(selected.image) && length(selected.image) == 1 && (!is.na(selected.image))) {
		stop("invalid value for selected.image")
	}
	if (selected.image < 1 || length(report.plots) < selected.image) {
		stop("invalid value for selected.image")
	}

	## The following must be synchronizes with the values in report.css
	PLOT.IMAGE.WIDTH.MAX <- 850  ## maximum visible width, in pixels, of a figure plot image
	PLOT.IMAGE.HEIGHT.MAX <- 850 ## maximum visible height, in pixels, of a figure plot image

	iheight <- range(sapply(report.plots, slot, "height"))
	iwidth <- range(sapply(report.plots, slot, "width"))
	ilow <- range(sapply(report.plots, slot, "low.png"))
	if (iheight[1] != iheight[2] || iwidth[1] != iwidth[2] || ilow[1] != ilow[2]) {
		stop("inconsistent image sizes")
	}
	iheight <- iheight[1] * ilow[1]
	iwidth <- iwidth[1] * ilow[1]
	pdfs <- NULL
	if (all(sapply(report.plots, slot, "create.pdf"))) {
		pdfs <- unique(sapply(report.plots, slot, "dir.pdf"))
		if (length(pdfs) != 1) {
			pdfs <- NULL # inconsistent directory to store the PDF files; do not create a link
		}
	}
	ihigh <- range(sapply(report.plots, slot, "high.png"))
	ihigh <- ifelse(ihigh[1] == ihigh[2] && ihigh[1] > 0, ihigh[1], 0L)

	## Get the file name elements
	fnames <- sapply(report.plots, slot, "fname")
	felements <- strsplit(fnames, "_", fixed = TRUE)
	element.values <- rnbreport.get.element.values(felements, length(setting.names))
	if (is.null(element.values)) {
		stop("inconsistent figure file names")
	}
	if (is.character(element.values)) {
		stop(element.values)
	}

	fn <- report@figures + 1L
	wline <- function(txt, indent = 1) {
		write.line(txt, report@fname, indent = indent)
	}

	## Create a table of settings if necessary
	if (length(setting.names) != 0) {
		txt <- rnbreport.create.settings.table("fig", fn, element.values, setting.names, felements[[selected.image]])
		if (is.null(txt)) {
			stop("missing plot file element descriptors")
		}
	} else {
		txt <- NULL
	}
	wline("<div class=\"figure\">", 0)
	if (!is.null(txt)) {
		wline(txt, 0)
	}

	## Create image placeholder
	txt <- paste0(report@dir.pngs, "/", fnames[selected.image], ".png")
	txt <- paste0("<img alt=\"Figure ", fn, "\" id=\"fig", fn, "image\" height=\"", iheight, "\" src=\"", txt,
		"\" width=\"", iwidth, "\" />")
	if (ihigh != 0) {
		## Add link to a high-resolution image
		fimage.high <- ifelse(report@dir.high == report@dir.pngs, "_high_resolution", "")
		fimage.high <- paste0(report@dir.high, "/", fnames[selected.image], fimage.high, ".png")
		txt <- paste0("<a href=\"", fimage.high, "\" id=\"fig", fn, "imagehigh\">", txt, "</a>")
	}
	if (iwidth > PLOT.IMAGE.WIDTH.MAX) {
		if (iheight > PLOT.IMAGE.HEIGHT.MAX) {
			txt <- paste0("<div class=\"restricted\">", txt, "</div>")
		} else {
			txt <- paste0("<div class=\"restrictedx\">", txt, "</div>")
		}
	} else if (iheight > PLOT.IMAGE.HEIGHT.MAX) {
		txt <- paste0("<div class=\"restrictedy\">", txt, "</div>")
	}
	wline(txt)

	## Create figure title and description
	wline("<p class=\"caption\">")
	if (!is.null(pdfs)) {
		## Create a link to a PDF file if applicable
		txt <- c("<span title=\"Open PDF\"><a class=\"figicon\" id=\"fig", fn, "pdf\" href=\"", report@dir.pdfs, "/",
			fnames[selected.image], ".pdf\"><img alt=\"Open PDF\" height=\"20\" onmouseout=\"pdficon(this, false)\" ",
			"onmouseover=\"pdficon(this)\" src=\"", report@dir.conf, "/pdf_inactive.png\" width=\"20\" /></a></span>")
		wline(txt, 2)
	}
	txt <- c("<a id=\"fig", fn, "caption\" onmousedown=\"showHide(\'fig", fn,
		"description\')\" onmouseout=\"doCursor()\" onmouseover=\"doCursor()\">Figure ", fn, "</a>")
	wline(txt, 2)
	wline("</p>")
	wline(c("<p class=\"figdescription\" id=\"fig", fn, "description\">", description, "</p>"))

	wline("</div>\n", 0)
	report@figures <- fn
	return(report)
}

########################################################################################################################

#' rnb.add.reference
#'
#' Adds a reference item to the given report.
#'
#' @param report Report to add a reference item to.
#' @param txt    Text of the reference in the form of a non-empty \code{character} vector. The elements of this vector
#'               are concatenated without a separator.
#' @return The modified report.
#'
#' @examples
#' \donttest{
#' report <- createReport("example.html", "Example", init.configuration = TRUE)
#' txt.reference <- c("Bird A. ", "<i>Nucleic Acids Res.</i> <b>8</b> (1980)")
#' report <- rnb.add.reference(report, txt.reference)
#' txt <- c("This was shown in ", rnb.get.reference(report, txt.reference), ".")
#' rnb.add.paragraph(report, txt)
#' }
#' @seealso \code{\link{rnb.get.reference}} for adding citations in the report's text; \code{\linkS4class{Report}} for
#'   other functions adding contents to an HTML report
#' @author Yassen Assenov
#' @export
rnb.add.reference <- function(report, txt) {
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	if (!(is.character(txt) && length(txt) != 0)) {
		stop("invalid value for txt; expected character vector")
	}
	report@references <- union(report@references, paste(txt, collapse = ""))
	return(report)
}
	
########################################################################################################################

#' rnb.get.reference
#'
#' Creates a string that points to the given reference item in the specified report.
#'
#' @param report Report that contains the reference to be cited.
#' @param txt    Text of the reference in the form of a non-empty \code{character} vector. This reference must already
#'               added to the report.
#' @return Citation of the reference item (including a link) in the form of a one-element \code{character} vector. If
#'         the specified reference item is not found in the report, this method returns an empty string.
#'
#' @examples
#' \donttest{
#' report <- createReport("example.html", "Example", init.configuration = TRUE)
#' txt.reference <- c("Bird A. ", "<i>Nucleic Acids Res.</i> <b>8</b> (1980)")
#' report <- rnb.add.reference(report, txt.reference)
#' txt <- c("This was shown in ", rnb.get.reference(report, txt.reference), ".")
#' rnb.add.paragraph(report, txt)
#' }
#' @seealso \code{\link{rnb.add.reference}} for adding a reference item to a report; \code{\linkS4class{Report}} for
#'   other functions adding contents to an HTML report
#' @author Yassen Assenov
#' @export
rnb.get.reference <- function(report, txt) {
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	if (!(is.character(txt) && length(txt) != 0)) {
		stop("invalid value for txt; expected character vector")
	}
	result <- which(report@references == paste(txt, collapse = ""))
	if (length(result) != 0) {
		result <- paste0("[<a href=\"#ref", result, "\">", result, "</a>]")
	} else {
		result <- ""
	}
	return(result)
}

#' @import methods
#' @export
setGeneric("off",
           function(object, ...) standardGeneric("off"))

#' off-methods
#'
#' Performs cleanup and/or other finishing activities and closes the specified device, connection, or document.
#'
#' @param object       Object to be closed.
#' @param handle.errors Flag indicating if the method should attempt to catch and process errors (e.g. I/O errors)
#'                      internally. Setting this to \code{TRUE} does not guarantee that the method never stops with an
#'                      error.
#' @param ...         additional arguments
#'
#' @return The closed object, invisibly.
#'
#' @export
#' @docType methods
#' @rdname off
#' @aliases off
setMethod("off", signature(object="Report"),
  function(object, ...) {
    return(invisible(complete.report(object)))
  }
)

########################################################################################################################
########################################################################################################################

#' createReport
#'
#' Creates a new report object.
#'
#' @param fname              Single-element \code{character} vector denoting the name of the file to contain the HTML
#'                           report. If this file already exists, it will be overwritten.
#' @param title              Title of the report in the form of a single-element \code{character} vector.
#' @param page.title         Web page title. This usually appears in the web browser's window title when the report is
#'                           open. If specified, this must be a vector. Note that only the first element is used.
#' @param authors            Optional list of authors in the form of a \code{character} vector. This list is included in
#'                           the header of the generated HTML file. Note that author names can contain only Latin leters,
#'                           space, dash (\code{-}), comma (\code{,}) or dot (\code{.}).
#' @param dirs               Location of the supporting directories, that is, paths that are expected to contain
#'                           additional files linked to from the HTML report. See the \emph{Details} section for a list
#'                           of these directories.
#' @param init.configuration Flag indicating if the report configuration data should be initialized. If this parameter
#'                           is \code{TRUE}, the method creates the respective directory and copies configuration files
#'                           that define cascading style sheet (CSS) definitions and Javascript functions used by the
#'                           HTML report. If such configuration files already exist, they will be overwritten. Since the
#'                           aforementioned files can be shared by multiple reports, it is recommended that the
#'                           configuration is initialized using the method \code{\link{rnb.initialize.reports}}, instead
#'                           of setting this flag to \code{TRUE}.
#' @return Newly created \code{\linkS4class{Report}} object.
#'
#' @details
#' If specified, the parameter \code{dirs} must be a \code{character} vector. The following names are read:
#' \itemize{
#'  \item{\code{"configuration" }}{Directory that contains the auxilliary configuration files, such as style sheets and
#'        Javascript files. If missing or \code{NA}, the default value used is \code{"configuration"}.}
#'  \item{\code{"data" }}{Directory to contain the tables, lists and other generated data files that are linked to in
#'        the HTML report. If missing or \code{NA}, the value used is formed from the file name \code{fname} (without
#'        the extension) and the suffix \code{"_data"}.}
#'  \item{\code{"pngs" }}{Directory to contain the low resolution PNG images shown in the HTML report. If missing or
#'        \code{NA}, the value used is formed from the file name \code{fname} (without the extension) and the suffix
#'        \code{"_images"}.}
#'  \item{\code{"pdfs" }}{Directory to contain the PDF images (if such are created). If not missing or \code{NA}, the
#'        value used is formed from the file name \code{fname} (without the extension) and the suffix \code{"_pdf"}.}
#'  \item{\code{"high" }}{Directory to contain the high resolution PNG images (if such are created). If missing or
#'        \code{NA}, the value used is the same as the \code{pngs} directory.}
#' }
#' Any other elements, if present, are ignored. Note that these directories are not required to point to different
#' locations. In particular, if the directories for low and for high resolution images are identical, the
#' high-resolution image files are assumed to be the ones with suffix \code{"_high_resolution.png"}. See
#' \code{\link{createReportPlot}} for creating image files.
#' In order to ensure independence of the operating system, there are strong restrictions on the names of the file and
#' directories. The name of the report's HTML file can consist of the following symbols only: Latin letters, digits, dot
#' (\code{.}), dash (\code{-}) and underline (\code{_}). The extension of the report's HTML file must be one of
#' \code{htm}, \code{html}, \code{xhtml} or \code{xml}. The supporting directories must be given as relative paths;
#' the restrictions on the path names are identical to the ones for file name. Forward slash (\code{/}) is to be used as
#' path separator. Path names cannot start or end with a slash. None of the directory names can be
#' an empty string, use \code{"."} instead. A value in the form \code{"mypath/.html"} for \code{fname} is invalid.
#' Upon initialization, the report attempts to create or overwrite the specified \code{fname}. If the path to it does
#' not exist, or if the current process does not have permissions to write to the file, report initialization will fail.
#' The report object visits each supporting directory (except \code{configuration}) and attempts to create it, unless it
#' is an existing empty directory. Report initialization will fail if any of the visited directories does not meet the
#' criteria and could not be created. Hidden files (file names starting with \code{"."} on Unix platforms) are ignored.
#' Thus, all supporting directories that already exist and contain hidden files only are considered valid.
#'
#' @examples
#' \donttest{
#' report <- createReport("example.html", "Example", init.configuration = TRUE)
#' }
#' @seealso \code{\linkS4class{Report}} for functions adding contents to an HTML report
#' @author Yassen Assenov
#' @export
createReport <- function(fname, title, page.title = "RnBeads report", authors = NULL, dirs = NULL, init.configuration = FALSE) {
	if (!(is.character(fname) && length(fname) == 1 && (!is.na(fname[1])))) {
		stop("invalid value for fname")
	}
	if (!(is.character(title) && length(title) == 1 && (!is.na(title[1])))) {
		stop("invalid value for title")
	}
	if (!(is.null(page.title) || is.vector(page.title))) {
		stop("invalid value for page.title")
	}
	if (!(is.null(authors) || is.character(authors))) {
		stop("invalid value for authors")
	}
	if (is.null(dirs)) {
		dirs <- character(0)
	} else {
		if (!is.character(dirs)) {
			stop("invalid value for dirs")
		}
		dnames <- names(dirs)
		if (is.null(dnames)) { dnames <- character(0) }
		dnames <- intersect(dnames, c("configuration", "data", "pngs", "pdfs", "high"))
		if (length(dnames) != 0 && any(is.na(dirs[dnames]))) {
			stop("missing values for dirs")
		}
	}
	if (!parameter.is.flag(init.configuration)) {
		stop("invalid value for init.configuration; expected TRUE or FALSE")
	}
	new("Report", fname, title, page.title, authors, dirs, init.configuration[1])
}

## get.components.sva
##
## Computes Surrogate Variables for a given dataset and target
##
## @param df.data an MxN data.frame containing the values for M sites on N samples
## @param df.pheno an NxK annotation table containing sample annotations
## @param pheno.col the column name in df.pheno containing the target variable for SVA
## @param conf.cols a vector of confounding variable column names
## @param numSVmethod Method for estimating the number of surrogate variables. passed directly to \code{sva}
## @return A matrix containing the computed Surrogate Variables. Rows correspond to samples, columns correspond to the SVs
##
## @author Fabian Mueller
get.components.sva <- function(df.data,df.pheno,pheno.col,conf.cols=NULL,numSVmethod="leek"){
	rnb.require("sva")
	if (length(conf.cols)>0){
		logger.info(paste0("Adjusting for known covariates: ",paste(conf.cols,collapse=",")))
	}
	n.samples <- nrow(df.pheno)
	sel.samples <- !is.na(df.pheno[,pheno.col]) #remove sample with no primary variable from dataset
	df.data <- df.data[,sel.samples] 
	df.pheno <- df.pheno[sel.samples,]
	fff.txt <- paste0("~1+",paste(pheno.col,collapse="+"))
	if (length(conf.cols)>0){
		fff.txt <- paste0(fff.txt,"+",paste(conf.cols,collapse="+"))
	}
	mod <- model.matrix(as.formula(fff.txt),data=df.pheno)
	fff0.txt <- "~1"
	if (length(conf.cols)>0){
		fff0.txt <- paste0("~1+",paste(conf.cols,collapse="+"))
	}
	mod0 <- model.matrix(as.formula(fff0.txt),data=df.pheno)
	svobj <- NULL
	tryCatch(
		svobj <- sva(df.data,mod,mod0,numSVmethod=numSVmethod),
		error = function(ee) {
			logger.warning(c("Could not compute SVA components:",ee$message))
	})
	valid.res <- !is.null(svobj)
	if (valid.res && is.null(ncol(svobj$sv))){
		logger.info("No surragate variables found")
		valid.res <- FALSE
	}
	if (valid.res){
		res <- matrix(NA,n.samples,ncol(svobj$sv))
		res[sel.samples,] <- svobj$sv
	} else {
		res <- matrix(NA,n.samples,0)
	}
	return(res)
}

## get.components.isva
##
## Computes Independent Surrogate Variables for a given dataset and target
##
## @param df.data an MxN data.frame containing the values for M sites on N samples
## @param df.pheno an NxK annotation table containing sample annotations
## @param pheno.col the column name in df.pheno containing the target variable for SVA
## @param conf.cols a vector of confounding variable column names
## @return A matrix containing the computed Surrogate Variables. Rows correspond to samples, columns correspond to the SVs
##
## @author Fabian Mueller
get.components.isva <- function(df.data,df.pheno,pheno.cols,conf.cols=NULL){
	rnb.require("isva")
	if (length(conf.cols)>0){
		conf.is.factor <- apply(df.pheno[,conf.cols],2,FUN=function(x){
			is.factor(x) || is.logical(x)
		})
		isvobj <- DoISVA(df.data,df.pheno[,pheno.cols],df.pheno[,conf.cols],conf.is.factor)
	} else {
		# logger.start("Computing components: ISVA")
		isvobj <- DoISVA(df.data,df.pheno[,pheno.cols])
		# logger.completed()
	}
	return(isvobj$isv)
}

## get.pheno.tab
##
## helper function thart retrieves a list of columns from the sample annotation table
## that will be used in the batch effects and SVA steps of RnBeads
##
## @param rnb.set an \code{RnBSet} object
## @return A list of inferred phenotype data
##
## @author Yassen Assenov, Fabian Mueller
get.pheno.tab <- function(rnb.set){
	pheno.table <- pheno(rnb.set)
	if (is.null(pheno.table)) {
		stop("invalid value for rnb.set; missing phenotype data")
	}
	
	## Get a list of comparable traits
	predefined.columns <- rnb.getOption("exploratory.columns")
	if (!is.null(predefined.columns)) {
		if (is.character(predefined.columns)) {
			predefined.columns <- intersect(predefined.columns, colnames(pheno.table))
		} else { # is.integer(columns)
			predefined.columns <- intersect(predefined.columns, 1:ncol(pheno.table))
		}
		pheno.table <- pheno.table[, predefined.columns]
	}
	traits <- lapply(pheno.table, function(x) {
				if (length(unique(na.omit(x))) < 2) {
					return(NULL)
				}
				if (is.character(x)) {
					x <- as.factor(x)
				}
				if (is.logical(x)) {
					x <- as.factor(x)
				}
				if (is.factor(x)) {
					y <- na.omit(x)
					if (anyDuplicated(y) == 0) {
						return(NULL)
					}
				}
				return(x)
			}
	)
	traits <- traits[sapply(traits, is.null) == FALSE]
	return(traits)
}

## compute.sva.assoc
##
## Helper function to compute the associations of Surrogate Variables (SVs)
## with principal components and sample annotations
## implemented in analogy to the pca analysis by Yassen
##
## @param meth.vals an MxN data.frame containing the values for M sites on N samples. Should not contain missing values.
## @param pheno.list a list of sample annotations as returned by \code{get.pheno.tab}
## @param sva.tables a named list of computed SVA tables as returned by \code{get.components.sva}. It should contain one element
##					 per target variable
## @return An object of class \code{SvaAssoc}: basically a list whose only element is named \code{sva}. This element is
## 		   again a list containing containing the association information for each target variable SVA.
##         The association information in turn consists of association with the Principal components (element \code{pca}) and
##         with the sample annation (element \code{traits}). These elements contain matrices with correlation coefficients,
##         the name of the association test, the resulting p-value from an association test and names of errors that occurred.
##
## @author Fabian Mueller
compute.sva.assoc <- function(meth.vals,pheno.list,sva.tables){
	result <- list()
	logger.start("Computing components: PCA")
	pcaobj <- prcomp(t(meth.vals), center = TRUE, scale. = FALSE)
	pca.tab <- pcaobj$x
	logger.completed()
	n.pc <- 3
	n.ph <- length(pheno.list)
	result[["sva"]] <- list()
	for (cc in names(sva.tables)){
		result[["sva"]][[cc]] <- list()
		logger.start(c("Target variable:",cc))
		n.sv <- ncol(sva.tables[[cc]])
		if (n.sv > 0){
			sv.tab <- sva.tables[[cc]]
			names.svs <- paste0("sv",1:n.sv)
			# names.svs <- 1:n.sv
			names.pcs <- paste0("pc",1:n.pc)
			# names.pcs <- 1:n.pc
			names.traits <- names(pheno.list)
			cor.tab.pca  <- matrix(nrow = n.pc, ncol = n.sv, dimnames = list("Principal component" = names.pcs, "Surrogate Variable" = names.svs))
			cor.tab.ph   <- matrix(nrow = n.ph, ncol = n.sv, dimnames = list("Sample Trait" = names.traits, "Surrogate Variable" = names.svs))
			test.tab.ph  <- matrix(nrow = n.ph, ncol = n.sv, dimnames = list("Sample Trait" = names.traits, "Surrogate Variable" = names.svs))
			pval.tab.ph  <- matrix(nrow = n.ph, ncol = n.sv, dimnames = list("Sample Trait" = names.traits, "Surrogate Variable" = names.svs))
			err.tab.ph   <- matrix(nrow = n.ph, ncol = n.sv, dimnames = list("Sample Trait" = names.traits, "Surrogate Variable" = names.svs))
			for (j in 1:n.sv){
				for (i in 1:n.pc){
					t.result <- test.traits(sv.tab[,j],pca.tab[,i])
					cor.tab.pca[i,j] <- t.result[["correlation"]]
				}
				for (i in 1:n.ph){
					t.result <- test.traits(sv.tab[,j],pheno.list[[i]])
					cor.tab.ph[i,j]  <- t.result[["correlation"]]
					test.tab.ph[i,j] <- t.result[["test"]]
					pval.tab.ph[i,j] <- t.result[["pvalue"]]
					err.tab.ph[i,j]  <- t.result[["error"]]
				}
			}
		} else {
			cor.tab.pca <- NA
			cor.tab.ph  <- NA
			test.tab.ph <- NA
			pval.tab.ph <- NA
			err.tab.ph  <- NA
		}
		result[["sva"]][[cc]][["pca"]] <- list(
					"correlations" = cor.tab.pca)
		result[["sva"]][[cc]][["traits"]] <- list(
					"failures" = err.tab.ph,
					"tests" = test.tab.ph,
					"correlations" = cor.tab.ph,
					"pvalues" = pval.tab.ph)

		logger.completed()
	}
	class(result) <- "SvaAssoc"
	return(result)
}

#' rnb.execute.sva
#'
#' Conduct Surrogate Variable Analysis (SVA) on the beta values of an RnBSet for given target variables
#'
#' @param rnb.set The \code{RnBSet} object on which the SVA should be conducted
#' @param cmp.cols a vector of sample annotation column names which will be the targets of the SVA.
#' @param columns.adj Column names in the table of phenotypic information to be used for confounder adjustment.
#' @param assoc a flag indicating whether association information with principal components and other sample annotation should
#'				be returned
#' @param numSVmethod method to estimate the number of surrogate variables. Passed to \code{sva}.
#' @return An object of class \code{SvaResult}: basically a list containing the following elements:
#' \describe{
#'   \item{\code{num.components}}{a vector storing the number of detected SVs for each target variable}
#'   \item{\code{sva.performed}}{a vector storing whether SVA was performed on a target variable and whether more than 0 SVs were found}
#'   \item{\code{targets}}{a vector storing the names of the target variables}
#'   \item{\code{components}}{a list storing for each target variable a matrox containing the sample-wise SVs as rows}
#'   \item{\code{assoc}}{a special object containing association information of SVs with principal components and sample annotations
#'	 typically only used \code{rnb.section.sva}.}
#' }
#' @export
#' @author Fabian Mueller
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' sva.obj <- rnb.execute.sva(rnb.set.example,c("Sample_Group","Treatment"),numSVmethod="be")
#' sva.obj$sva.performed
#' sva.obj$num.components
#' rnb.set.mod <- set.covariates.sva(rnb.set.example, sva.obj)
#' has.covariates.sva(rnb.set.example,"Sample_Group")
#' has.covariates.sva(rnb.set.mod,"Sample_Group")
#' has.covariates.sva(rnb.set.mod,"Treatment")
#' }
rnb.execute.sva <- function(rnb.set, cmp.cols=rnb.getOption("inference.targets.sva"),
		columns.adj=rnb.getOption("covariate.adjustment.columns"), assoc=TRUE,
		numSVmethod=rnb.getOption("inference.sva.num.method")){
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	if (!(is.character(cmp.cols) || is.null(cmp.cols))){
		stop("invalid value for cmp.cols")
	}
	result <- list()
	logger.start("Conducting SVA analysis")

	if (is.null(cmp.cols)){
		cmp.cols <- rnb.getOption("inference.targets.sva")
	}

	ph <- pheno(rnb.set)
	ph.cols <- colnames(ph)
	
	if (is.character(cmp.cols)){
		cmp.cols.ph <- intersect(ph.cols,cmp.cols)
		if (length(cmp.cols.ph)<length(cmp.cols)){
			logger.warning(c("The following columns are not specified in the sample annotation table:",
							 paste(setdiff(cmp.cols,cmp.cols.ph),collapse=","),
							 "--> ignoring them in the SVA adjustment"))
		}
		cmp.cols <- cmp.cols.ph
	}
	logger.info(c("Considering the following target variables:",paste(cmp.cols,collapse=",")))
	logger.start("Preparing data")
		mm <- meth(rnb.set)
		nonzero.var.rows <- rowVars(mm,na.rm=TRUE) > 0
		mm.vao <- mm[nonzero.var.rows,]
		logger.info(c("Removed",nrow(mm)-sum(nonzero.var.rows),"sites without variation"))
		mm.nao <- na.omit(mm.vao)
		logger.info(c("Removed",nrow(mm.vao)-nrow(mm.nao),"sites containing missing values"))
	logger.completed()

	logger.start("Computing components: SVA")
		sva.tables <- list()
		for (cc in cmp.cols){
			logger.start(c("Target variable:",cc))
			adj.cols <- setdiff(columns.adj,cc)
			sv <- get.components.sva(mm.nao,ph,cc, conf.cols=adj.cols, numSVmethod=numSVmethod)
			sva.tables[[cc]] <- sv
			logger.completed()
		}
		num.components <- sapply(sva.tables,ncol)
		names(num.components) <- names(sva.tables)
		computed.sva <- num.components > 0
		result[["num.components"]] <- num.components
		result[["sva.performed"]] <- computed.sva
		logger.info(c("Found SVs for ",sum(computed.sva),"comparisons"))

	logger.completed()
	sva.assoc <- list()
	if (assoc && any(computed.sva)){
		logger.start("Computing association of SVA with PCA and sample annotations")
			pheno.list <- get.pheno.tab(rnb.set)
			sva.assoc <- compute.sva.assoc(mm.nao,pheno.list,sva.tables)
		logger.completed()
	}

	result[["targets"]] <- cmp.cols
	result[["components"]] <- sva.tables
	result[["assoc"]] <- sva.assoc
	class(result) <- "SvaResult"
	logger.completed()
	return(result)
}

#' rnb.section.sva
#'
#' Adds a section on Surrogate Variable Analysis (SVA) to an RnBeads Report
#' @param report An object of class \code{\linkS4class{Report}}.
#' @param sva.obj An object of class \code{SvaResult} as returned by \code{rnb.execute.sva}.
#' @return The modified report
#' @author Fabian Mueller
#' @noRd
rnb.section.sva <- function(report, sva.obj) {
	if (!inherits(sva.obj,"SvaResult")){
		stop("invalid value for sva.obj. Expected SvaResult!")
	}
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	do.it <- any(sva.obj$sva.performed)

	txt <- c("Surrogate variables were tested for association ",
			"with principal components and with sample traits using the specified target variables. The table below lists the ",
			"target variables along with the number of computed Surrogate Variables (SVs)")
	if (!do.it) {
		txt <- "Surrogate variable analysis did not result in any surrogate variables."
	}
	report <- rnb.add.section(report, "Surrogate Variable Analysis", txt, level = 1)
	
	if (do.it) {
		no.sva.inds <- which(sva.obj$num.components<1)
		targets <- sva.obj$targets
		names(targets) <- paste0("t",1:length(targets))
		tbl <- data.frame(targets,sva.obj$num.components[targets])
		colnames(tbl) <- c("Target","#SV")
		rnb.add.table(report, tbl)

		targets <- targets[sva.obj$sva.performed]
		setting.names <- list("Target" = targets)

		data.dir.abs <- rnb.get.directory(report, "data", TRUE)
		data.dir.rel <- rnb.get.directory(report, "data")

		txt <- c("Surrogate variables were tested for association ",
				 "with sample traits using the tests outlined in the table below.")
		report <- rnb.add.section(report, "Association of surrogate variables with sample traits", txt, level = 2)
		## Display summary table of tests performed
		i <- which(sva.obj$sva.performed)[1]
		assoc.tables <- sva.obj$assoc$sva[[i]][["traits"]]
		tbl <- matrix(c(rownames(assoc.tables$tests), assoc.tables$tests[,1]), ncol = 2)
		colnames(tbl) <- c("Trait", "Test")
		rnb.add.table(report, tbl)

		append.table <- function(tbl, fname.plot, fname.tab, heatmap.fun, header.r, max.col.count = NULL) {
			mytbl <- cbind(rownames(tbl), as.data.frame(tbl, check.names = FALSE))
			colnames(mytbl)[1] <- paste0(header.r, " \\ SV") 
			fname.tab.abs <- file.path(data.dir.abs,fname.tab)
			fname.tab.rel <- paste(data.dir.rel, fname.tab, sep = "/")
			utils::write.csv(mytbl, file = fname.tab.abs, na = "", row.names = FALSE)
			sprintf("<a href=\"%s\">csv</a>", fname.tab.rel)
			if(!is.null(max.col.count)){
			  width <- 0.25 + max.col.count * 0.28 + 1.6
			}
			hmap <- heatmap.fun(report, tbl, fname.plot,width = width)
			list(link = sprintf("<a href=\"%s\">csv</a>", fname.tab.rel), plot = hmap$plot, description = hmap$description)
		}

		tab.header.r <- "Trait"
		#Traits: correlations
		file.tbl <- matrix(character(), nrow = 0, ncol = 2, dimnames = list(NULL, c("Target", "File Name")))
		rplots <- list()
		max.col.count <- max(unlist(lapply(sva.obj$assoc$sva,function(x){
		  x[["traits"]]$correlations
		})),na.rm = T)
		for (i in 1:length(targets)){
			tt <- targets[i]
			tn <- names(targets)[i]
			tbl <- sva.obj$assoc$sva[[tt]][["traits"]]$correlations
			if (!all(is.na(tbl))) {
				#reassign names to make plot layout better
				names.traits <- rownames(tbl)
				tbl <- unname(tbl)
				dimnames(tbl) = list(names.traits,"Surrogate Variable"=1:ncol(tbl))
				fname.plot <- paste0("heatmap_cor_sva_trait_",tn)
				fname.tab  <- paste0("cor_sva_trait_",tn,".csv")
				result <- append.table(tbl, fname.plot, fname.tab, plot.heatmap.pc.correlations, tab.header.r, max.col.count)
				file.tbl <- rbind(file.tbl, c(targets[tn], result$link))
				rplots[[targets[tn]]] <- result$plot
				description <- result$description
			}
		}
		tbl.correlations.present <- (length(rplots) != 0)
		if (tbl.correlations.present) {
			txt <- c("The following table shows the corralation between surrogate variables and sample annotations:")
			rnb.add.paragraph(report, txt)
			report <- rnb.add.figure(report, description, rplots, setting.names)
			colnames(file.tbl) <- c("Target", "Table file")
			rnb.add.exported.tables(report, file.tbl)
		}
		#Traits: p-values
		file.tbl <- matrix(character(), nrow = 0, ncol = 2, dimnames = list(NULL, c("Target", "File Name")))
		rplots <- list()
		for (i in 1:length(targets)){
			tt <- targets[i]
			tn <- names(targets)[i]
			tbl <- sva.obj$assoc$sva[[tt]][["traits"]]$pvalues
			if (!all(is.na(tbl))) {
				#reassign names to make plot layout better
				names.traits <- rownames(tbl)
				tbl <- unname(tbl)
				dimnames(tbl) = list(names.traits,"Surrogate Variable"=1:ncol(tbl))
				fname.plot <- paste0("heatmap_pval_sva_trait_",tn)
				fname.tab  <- paste0("pval_sva_trait_",tn,".csv")
				result <- append.table(tbl, fname.plot, fname.tab, plot.heatmap.pc.pvalues, tab.header.r, max.col.count)
				file.tbl <- rbind(file.tbl, c(targets[tn], result$link))
				rplots[[targets[tn]]] <- result$plot
				description <- result$description
			}
		}
		tbl.pvalues.present <- (length(rplots) != 0)
		if (tbl.pvalues.present) {
			txt <- c("The following table shows p-values resulting from the statistical tests for association of ",
				"surrogate variables and sample annotations:")
			rnb.add.paragraph(report, txt)
			report <- rnb.add.figure(report, description, rplots, setting.names)
			colnames(file.tbl) <- c("Target", "Table file")
			rnb.add.exported.tables(report, file.tbl)
		}

		txt <- c("Surrogate variables were tested for association ",
				 "with principal components using correlation coefficients.")
		report <- rnb.add.section(report, "Association of surrogate variables with principal components", txt, level=2)
		tab.header.r <- "PC"
		#PCA: correlations
		file.tbl <- matrix(character(), nrow = 0, ncol = 2, dimnames = list(NULL, c("Target", "File Name")))
		rplots <- list()
		for (i in 1:length(targets)){
			tt <- targets[i]
			tn <- names(targets)[i]
			tbl <- sva.obj$assoc$sva[[tt]][["pca"]]$correlations
			if (!all(is.na(tbl))) {
				#reassign names to make plot layout better
				tbl <- unname(tbl)
				dimnames(tbl) = list("Principal Component"=1:nrow(tbl),"Surrogate Variable"=1:ncol(tbl))
				fname.plot <- paste0("heatmap_cor_sva_pca_",tn)
				fname.tab  <- paste0("cor_sva_pca_",tn,".csv")
				result <- append.table(tbl, fname.plot, fname.tab, plot.heatmap.pc.correlations, tab.header.r, max.col.count)
				file.tbl <- rbind(file.tbl, c(targets[tn], result$link))
				rplots[[targets[tn]]] <- result$plot
				description <- result$description
			}
		}
		tbl.correlations.present <- (length(rplots) != 0)
		if (tbl.correlations.present) {
			txt <- c("The following table shows the corralation between surrogate variables and principal components:")
			rnb.add.paragraph(report, txt)
			report <- rnb.add.figure(report, description, rplots, setting.names)
			colnames(file.tbl) <- c("Target", "Table file")
			rnb.add.exported.tables(report, file.tbl)
		}
	}
	return(report)
}

#' set.covariates.sva
#'
#' Adds the results of Surrogate Variable Analysis (SVA) to an RnBSet
#' @param rnb.set The \code{RnBSet} object to which the results should be added
#' @param sva.obj An object of class \code{SvaResult} as returned by \code{rnb.execute.sva}.
#' @return The modified \code{RnBSet}. Note that the association information will not be stored.
#' @export
#' @author Fabian Mueller
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' sva.obj <- rnb.execute.sva(rnb.set.example,c("Sample_Group","Treatment"),numSVmethod="be")
#' sva.obj$sva.performed
#' sva.obj$num.components
#' rnb.set.mod <- set.covariates.sva(rnb.set.example, sva.obj)
#' has.covariates.sva(rnb.set.example,"Sample_Group")
#' has.covariates.sva(rnb.set.mod,"Sample_Group")
#' }
set.covariates.sva <- function(rnb.set, sva.obj) {
	sva.obj[["assoc"]] <- NULL # the association tables are not stored in the rnb.set
	rnb.set@inferred.covariates[["sva"]] <- sva.obj
	return(rnb.set)
}

#' get.covariates.sva
#'
#' Retrieves an NxK table of Surrogate variables stored in an RnBSet for a given target variable
#' @param rnb.set \code{RnBSet} object
#' @param target target variable. Must be in \code{pheno(rnb.set)} and belong to target variables for which the
#'		  SVs have already been computed and stored in the RnBSet.
#' @return an NxK table of K Surrogate variables stored for N samples of the \code{rnb.set}. \code{NULL}
#'		   if the components have not been computed or added to \code{rnb.set}.
#' @export
#' @author Fabian Mueller
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' sva.obj <- rnb.execute.sva(rnb.set.example,c("Sample_Group","Treatment"),numSVmethod="be")
#' sva.obj$sva.performed
#' sva.obj$num.components
#' rnb.set.mod <- set.covariates.sva(rnb.set.example, sva.obj)
#' get.covariates.sva(rnb.set.mod,"Sample_Group")
#' }
get.covariates.sva <- function(rnb.set, target) {
	if (!.hasSlot(rnb.set,"inferred.covariates")) { #.hasSlot ensure backwards compatibility
		return(NULL)
	}
	if (!is.element(target,names(rnb.set@inferred.covariates[["sva"]]$components))) {
		return(NULL)
	}
	res <- rnb.set@inferred.covariates[["sva"]]$components[[target]]
	return(res)
}

#' has.covariates.sva
#'
#' Returns whether Surrogate Variables have been computed and added to the \code{rnb.set} for a given target variable
#' @param rnb.set \code{RnBSet} object
#' @param target target variable. Must be in \code{pheno(rnb.set)} and belong to target variables for which the
#'		  SVs have already been computed and stored in the RnBSet.
#' @return \code{logical(1)}
#' @export
#' @author Fabian Mueller
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' sva.obj <- rnb.execute.sva(rnb.set.example,c("Sample_Group","Treatment"),numSVmethod="be")
#' sva.obj$sva.performed
#' sva.obj$num.components
#' rnb.set.mod <- set.covariates.sva(rnb.set.example, sva.obj)
#' has.covariates.sva(rnb.set.example,"Sample_Group")
#' has.covariates.sva(rnb.set.mod,"Sample_Group")
#' has.covariates.sva(rnb.set.mod,"Treatment")
#' }
has.covariates.sva <- function(rnb.set, target) {
	if (!.hasSlot(rnb.set,"inferred.covariates")) { #.hasSlot ensure backwards compatibility
		return(FALSE)
	}
	res <- is.element(target,names(rnb.set@inferred.covariates[["sva"]]$components))
	if (res) res <- ncol(rnb.set@inferred.covariates[["sva"]]$components[[target]]) > 0
	return(res)
}

#' rnb.step.sva
#'
#' Performs Surrogate Variable Analysis (SVA) and adds a corresponding section to an RnBeads Report
#' @param rnb.set The \code{RnBSet} object on which the SVA should be conducted
#' @param report An object of class \code{\linkS4class{Report}}.
#' @param ... arguments passed to \code{rnb.execute.sva}
#' @return A list containing the modified \code{RnBSet} and the modified report
#' @author Fabian Mueller
#' @noRd
rnb.step.sva <- function(rnb.set, report, ...){
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	logger.start("Computing Surrogate Variables and Associations")
	sva.obj <- rnb.execute.sva(rnb.set,...)
	logger.completed()
	logger.start("Adding Report Section")
	report <- rnb.section.sva(report, sva.obj)
	logger.completed()

	#modify the rnb.set
	rnb.set <- set.covariates.sva(rnb.set, sva.obj)
	if (any(sva.obj[["sva.performed"]])){
		logger.info("Added SVA results to the RnBSet")
	}

	#result: modified rnb.set and modified report
	res <- list(rnb.set=rnb.set,report=report)
	return(res)
}

#' Analysis of genome-scale DNA methylation data with RnBeads
#'
#' RnBeads facilitates comprehensive analysis of various types of DNA methylation data at the genome scale. It extends
#' previous approaches for such analysis by high throughput capabilities, as well as presenting results in a
#' comprehensive, highly interpretable fashion. 
#'
#' The complete analysis can be performed by calling the function \code{\link{rnb.run.analysis}}. 
#' 
#' @references Yassen Assenov*, Fabian Mueller*, Pavlo Lutsik*, Joern Walter, Thomas Lengauer and Christoph Bock (2014) Compehensive Analysis of DNA Methylation Data with RnBeads, Nature Methods, 11(11):1138-1140.
#' @import methods MASS cluster fields ggplot2 S4Vectors IRanges GenomicRanges methylumi ff limma
#' @importFrom BiocGenerics annotation
#' @importFrom BiocGenerics annotation<-
#' @importFrom grDevices colorRampPalette densCols dev.control dev.off dev2bitmap pdf png rainbow rgb xy.coords
#' @importFrom graphics abline close.screen layout legend lines mtext par plot plot.new polygon screen split.screen
#' @importFrom gridExtra arrangeGrob
#' @importFrom gplots colorpanel heatmap.2
#' @importFrom illuminaio readIDAT
#' @importFrom matrixStats colMaxs colMedians colMins colQuantiles colSds colVars
#' @importFrom matrixStats rowMaxs rowMedians rowMins rowQuantiles rowSds rowVars
#' @importFrom plyr rbind.fill
#' @importFrom stats as.dendrogram as.dist as.formula bartlett.test coef cutree dbeta density dexp dist dnorm ecdf
#' @importFrom stats fisher.test hclust knots kruskal.test lm model.matrix optim p.adjust pbeta pchisq pf pnorm
#' @importFrom stats prcomp predict pt qbeta residuals rnorm rt t.test vcov wilcox.test
#' @importFrom utils browseURL capture.output combn data download.file installed.packages memory.size
#' @importFrom utils read.csv read.delim read.table untar unzip write.table zip
#' @docType package
#' @name RnBeads
NULL

#' RnBeads option values and restrictions
#'
#' The values of options in RnBeads are stored in dedicated R objects accompanying the package. These objects are named
#' \code{infos}, \code{accepted}, \code{current} and \code{previous}. They should not be loaded or otherwise operated on
#' by users. Please refer to the documentation of \code{\link{rnb.options}} for accessing and modifying option values in
#' \pkg{RnBeads}.
#'
#' @docType data
#' @keywords datasets
#' @name accepted
#' @aliases current infos previous
#' @format \code{infos} is a \code{data.frame} containing information about all options in \pkg{RnBeads}. Row names in
#'         this table are the option names; the column names are \code{"Type"}, \code{"Named"}, \code{"Null"},
#'         \code{"Max"}, \code{"Min"}, \code{"MaxInclusive"} and \code{"MinInclusive"}.
#'         \code{accepted} is a \code{list} containing the sets of accepted values for some of the options.
#'         \code{current} is a \code{list} with current values for all options.
#'         \code{previous} is a \code{list} with previous values for the affected options; this list is only temporarily
#'         used while setting option values through \code{\link{rnb.options}} or \code{\link{rnb.xml2options}}.
#' @author Yassen Assenov
NULL

#' LUMP Support
#'
#' The sites used by the LUMP algorithm for estimating immune cell content are stored in an object named
#' \code{lump.hg19}. This object should not be loaded or otherwise operated on by users. Please refer to the
#' documentation of \code{\link{rnb.execute.lump}} for information on the algorithm and its implementation in
#' \pkg{RnBeads}.
#'
#' @docType data
#' @keywords datasets
#' @name lump.hg19
#' @format \code{lump.*} is a \code{list} of non-empty \code{integer} matrices, one per supported platform. Every
#'         \code{matrix} contains exactly two columns, denoting chromosome index and chromosome-based index,
#'         respectively. These indices refer to positions within the probe/site annotation table employed by
#'         \pkg{RnBeads} for the corresponding platform.
#' @author Yassen Assenov
NULL

#' LUMP Support (hg38)
#'
#' Those are the same sites as reported in \code{lump.hg19}, but lifted to `hg38` with UCSC's liftOver functionality. This only
#' applies for the CpG-wise sites; i.e. those used for sequencing data sets, since `hg38` is not supported for array-based data
#' sets.
#'
#' @docType data
#' @keywords datasets
#' @name lump.hg38
#' @format \code{lump.*} is a \code{list} of non-empty \code{integer} matrices, one per supported platform. Here, only `CpG` is 
#'          available for BS datasets.
#' @author Michael Scherer
NULL

#' XenoMix EPICv1 (hg38)
#'
#' This is the manifest file for EPICv1, which is required for detecting inter-species probes for estimating mouse contamination
#' with XenoMix
#'
#' @docType data
#' @keywords datasets
#' @name epic_v1_manifest
#' @format \code{data.frame} provided by Illumina capturing information about the probe design.
#' @author Nima Esmaeelpour
NULL

#' XenoMix EPICv2 (hg38)
#'
#' This is the manifest file for EPICv2, which is required for detecting inter-species probes for estimating mouse contamination
#' with XenoMix
#'
#' @docType data
#' @keywords datasets
#' @name epic_v2_manifest
#' @format \code{data.frame} provided by Illumina capturing information about the probe design.
#' @author Nima Esmaeelpour
NULL

#' XenoMix interspeciesprobes (v1)
#'
#' These are the inter-species probes for EPICv1
#'
#' @docType data
#' @keywords datasets
#' @name interspecies_probes_v1
#' @format \code{list} of CGIs for the inter-species probes.
#' @author Nima Esmaeelpour
NULL

#' XenoMix interspeciesprobes (v2)
#'
#' These are the inter-species probes for EPICv2
#'
#' @docType data
#' @keywords datasets
#' @name interspecies_probes_v2
#' @format \code{list} of CGIs for the inter-species probes.
#' @author Nima Esmaeelpour
NULL

## rnb.section.region.description
##
## Adds a section summarizing the requested regions.
##
## @param report  Report on loading to contain the newly constructed section.
## @param rnb.set Methylation dataset to be analyzed.
## @param regions Non-empty \code{character} vector of region names.
## @return The modified report.
##
## @author Yassen Assenov, Fabian Mueller
rnb.section.region.description <- function(report, rnb.set, regions) {

	#Descriptive table
	msg <- function(txt, e.class = "disabled") {
		paste("<span class=\"", e.class, "\">", txt, "</span>", sep = "")
	}
	reg.counts <- sapply(regions, function(region) {
			tryCatch(as.character(nrow(meth(rnb.set, region))), error = function(e) { msg("not supported") })
		}
	)
	get.r.description <- function(reg) {
		result <- attr(rnb.get.annotation(reg, assembly = assembly(rnb.set)), "description")
		if (is.null(result) || identical("", result)) {
			result <- msg("n.a.")
		} else {
			result <- paste("<p style=\"font-weight:normal;text-align:left\">", result, "</p>", sep = "")
		}
		result
	}
	reg.descriptions <- sapply(regions, function(region) {
			tryCatch(get.r.description(region), error = function(e) { msg("not imported", "outdated") })
		}
	)
	table.statistics <- data.frame(
		"Annotation" = regions, "Description" = reg.descriptions, "Regions in the Dataset" = reg.counts,
		check.names = FALSE, stringsAsFactors = FALSE)
	txt <- c(ifelse(rnb.getOption("analyze.sites"), "In addition to CpG sites, there", "There"),
		ifelse(length(regions) == 1, " is one set", paste(" are", length(regions), "sets")),
		" of genomic regions to be covered in the analysis. The table below gives a summary of these annotations.")
	report <- rnb.add.section(report, "Region Annotations", txt)
	table.header <- c("<colgroup>", paste("\t<col width=\"", c(210, 420, 150), "px\" />", sep = ""), "</colgroup>")
	rnb.add.table(report, table.statistics, row.names = FALSE, thead = table.header)

	names(regions) <- ifelse(is.valid.fname(regions),regions,paste("reg",1:length(regions),sep=""))
	plots.sizes <- list()
	plots.n.sites <- list()
	plots.site.distr <- list()
	for (i in 1:length(regions)){
		reg <- regions[i]
		reg.file.name <- names(regions)[i]
		aa <- annotation(rnb.set,type=reg)
		regions2sites <- regionMapping(rnb.set,reg)
		reg.sizes <- as.integer(aa$End - aa$Start + 1L) #+1 as we use 1-based, right-inclusive regions
		reg.n.sites <- vapply(regions2sites,length,0L)
		df2p <- data.frame(size=reg.sizes,n.sites=reg.n.sites)

		if (max(reg.sizes)==min(reg.sizes)){
			pp <- rnb.message.plot(paste0("Constant region size: ",min(reg.sizes)))
		} else {
			pp <- ggplot(df2p) + aes(x=size) + geom_density(color=rnb.getOption('colors.category')[1],
				fill=rnb.getOption('colors.category')[1], alpha=0.5) + 
				scale_x_continuous(limits = quantile(df2p$size, c(0.05, 0.95)))
		}
		figName <- paste("regionSizes",reg.file.name,sep="_")
		report.plot <- createReportGgPlot(pp,figName, report,create.pdf=TRUE)
		report.plot <- off(report.plot,handle.errors=TRUE)
		plots.sizes <- c(plots.sizes,list(report.plot))

		pp <- ggplot(df2p) + aes(x=n.sites) + geom_density(color=rnb.getOption('colors.category')[1],
			fill=rnb.getOption('colors.category')[1], alpha=0.5)
		figName <- paste("regionNsites",reg.file.name,sep="_")
		report.plot <- createReportGgPlot(pp,figName, report,create.pdf=TRUE)
		report.plot <- off(report.plot,handle.errors=TRUE)
		plots.n.sites <- c(plots.n.sites,list(report.plot))

		pp <- rnb.plot.region.site.density(rnb.set,reg)
		figName <- paste("regionSiteDistr",reg.file.name,sep="_")
		report.plot <- createReportGgPlot(pp,figName, report,create.pdf=TRUE)
		report.plot <- off(report.plot,handle.errors=TRUE)
		plots.site.distr <- c(plots.site.distr,list(report.plot))
	}

	setting.names <- list('Region type' = regions)

	#length distribution and number of sites per region
	txt <- c("The plots below show region size distributions for the region types above.")
	report <- rnb.add.section(report, "Region length distributions", txt, level = 2)
	
	description <- "Distribution of region lengths"
	report <- rnb.add.figure(report, description, plots.sizes, setting.names)

	txt <- c("The plots below show the distributions of the number of sites per region type.")
	report <- rnb.add.section(report, "Number of sites per region", txt, level = 2)

	description <- "Distribution of the number of sites per region"
	report <- rnb.add.figure(report, description, plots.n.sites, setting.names)

	txt <- c("The plots below show distributions of sites across the different region types.")
	report <- rnb.add.section(report, "Region site distributions", txt, level = 2)
	
	description <- "Distribution of sites across regions. relative coordinates of 0 and 1 corresponds to the start
					and end coordinates of that region respectively. Coordinates smaller than 0 and greater than 1 denote flanking regions
					normalized by region length."
	report <- rnb.add.figure(report, description, plots.site.distr, setting.names)
	
	return(report)
}

########################################################################################################################
## lolaUtils.R
## created: 2017-07-06
## creator: Fabian Mueller
## ---------------------------------------------------------------------------------------------------------------------
## Methods for LOLA enrichment analysis
########################################################################################################################

#' getRoadmapMdFromId
#'
#' Helper function to retrieve metadata from Roadmap sample IDs
#'
#' @param sampleIds	vector of sample ids in Roadmap fort (e.g. "E014")
#' @return a atble containing metadata for the supplied sample ids
#'
#' @author Fabian Mueller
#' @noRd
getRoadmapMdFromId <- function(sampleIds=NULL){
	remcAnnotFn <- system.file(file.path("extdata", "remc_metadata_2013.tsv"), package = "RnBeads")
	eidColname <- make.names("Epigenome ID (EID)")
	nameColname <- make.names("Epigenome Mnemonic")
	annot <- read.table(remcAnnotFn, header=TRUE, sep="\t", quote="", skip=0, comment.char="", stringsAsFactors=FALSE)
	annot <- annot[!is.na(annot[,eidColname]) & nchar(annot[,eidColname])>0, ]
	rownames(annot) <- annot[,eidColname]
	if (length(sampleIds)<1) sampleIds <- annot[,eidColname]

	snames <- annot[sampleIds,nameColname]
	snames[is.na(snames)] <- sampleIds[is.na(snames)]
	res <- data.frame(sampleId=sampleIds, sampleName=snames, stringsAsFactors=FALSE)
	return(res)
}

#' getCellTypesFromLolaDb
#'
#' retrieve or guess cell types from a LOLA DB object
#'
#' @param lolaDb   LOLA DB object as returned by \code{LOLA::loadRegionDB} or \code{\link{loadLolaDbs}}
#' @return character vector with cell types
#'
#' @author Fabian Mueller
#' @export
#' @examples
#' \donttest{
#' # download LOLA DB
#' lolaDest <- tempfile()
#' dir.create(lolaDest)
#' lolaDirs <- downloadLolaDbs(lolaDest, dbs="LOLACore")
#' lolaDb <- loadLolaDbs(lolaDirs[["hg19"]])
#' getCellTypesFromLolaDb(lolaDb)
#' }
getCellTypesFromLolaDb <- function(lolaDb){
	tt <- data.frame(lolaDb$regionAnno)
	res <- tt$cellType

	# special treatment for "sheffield_dnase" from LOLACore
	isInCollection <- tt$collection=="sheffield_dnase"
	if (sum(isInCollection) > 0) res[isInCollection] <- tt$description[isInCollection]

	# special treatment for "roadmap_epigenomics" from LOLAExt
	isInCollection <- tt$collection=="roadmap_epigenomics"
	if (sum(isInCollection) > 0){
		sampleIds <- gsub("^(.+)-(.+)$", "\\1", tt$filename[isInCollection])
		res[isInCollection] <- paste0(getRoadmapMdFromId(sampleIds)[,"sampleName"], " (", sampleIds,")")
	}

	return(res)
}

#' getTargetFromLolaDb
#'
#' retrieve or guess the target from a LOLA DB object. Here, target typically
#' refers to antibodies for ChIP-seq experiments, but could also refer to other annotations
#' (e.g. motifs in TF motif databases, annotation according to UCSC features etc.)
#'
#' @param lolaDb   LOLA DB object as returned by \code{LOLA::loadRegionDB} or \code{\link{loadLolaDbs}}
#' @return character vector with targets
#'
#' @author Fabian Mueller
#' @export
#' @examples
#' \donttest{
#' # download LOLA DB
#' lolaDest <- tempfile()
#' dir.create(lolaDest)
#' lolaDirs <- downloadLolaDbs(lolaDest, dbs="LOLACore")
#' lolaDb <- loadLolaDbs(lolaDirs[["hg19"]])
#' getTargetFromLolaDb(lolaDb)
#' }
getTargetFromLolaDb <- function(lolaDb){
	tt <- data.frame(lolaDb$regionAnno)
	res <- tt$antibody

	# special treatment for "ucsc_features" from LOLACore
	isInCollection <- tt$collection=="ucsc_features"
	if (sum(isInCollection) > 0) res[isInCollection] <- gsub("^UCSC ", "", tt$description[isInCollection])

	# special treatment for "sheffield_dnase" from LOLACore
	isInCollection <- tt$collection=="sheffield_dnase"
	if (sum(isInCollection) > 0) res[isInCollection] <- "DNase"

	# special treatment for "encode_segmentation" from LOLACore
	isInCollection <- tt$collection=="encode_segmentation"
	if (sum(isInCollection) > 0){
		res[isInCollection] <- gsub(" Segments$", "", tt$description[isInCollection])
	}

	# special treatment for "jaspar_motifs" from LOLAExt
	isInCollection <- tt$collection=="jaspar_motifs"
	if (sum(isInCollection) > 0) res[isInCollection] <- gsub("\\.bed$", "", tt$filename[isInCollection])

	# special treatment for "roadmap_epigenomics" from LOLAExt
	isInCollection <- tt$collection=="roadmap_epigenomics"
	if (sum(isInCollection) > 0){
		res[isInCollection] <- gsub("(\\.(hotspot\\.(fdr0\\.01|all)|macs2))?\\.(peaks\\.bed|narrowPeak)$", "", 
			gsub("^(.+)-(.+)$", "\\2", tt$filename[isInCollection])
		)
	}

	return(res)
}

#' getNamesFromLolaDb
#'
#' get human readable names from a LOLA DB object
#'
#' @param lolaDb   LOLA DB object as returned by \code{LOLA::loadRegionDB} or \code{\link{loadLolaDbs}}
#' @param addCollectionNames  attach the name of the collection to the name
#' @param addDbId  attach the index of the item in the LOLA DB object to the name
#' @return character vector with human readable names
#'
#' @author Fabian Mueller
#' @export
#' @examples
#' \donttest{
#' # download LOLA DB
#' lolaDest <- tempfile()
#' dir.create(lolaDest)
#' lolaDirs <- downloadLolaDbs(lolaDest, dbs="LOLACore")
#' lolaDb <- loadLolaDbs(lolaDirs[["hg19"]])
#' getNamesFromLolaDb(lolaDb)
#' }
getNamesFromLolaDb <- function(lolaDb, addCollectionNames=FALSE, addDbId=TRUE){
	tt <- data.frame(lolaDb$regionAnno)
	res <- paste0(tt$description)

	cellTypes <- getCellTypesFromLolaDb(lolaDb)
	targets <- getTargetFromLolaDb(lolaDb)

	# if there is a valid "target" (see getTargetFromLolaDb) inferred from the annotation, construct a name from it
	hasDesc <- !is.na(targets)
	if (sum(hasDesc) > 0) res[hasDesc] <- targets[hasDesc]

	# if both cellType and target annotation were inferred, construct a name from them
	hasDesc <- !is.na(cellTypes) & !is.na(targets)
	if (sum(hasDesc) > 0) res[hasDesc] <- paste0(targets[hasDesc], " - ", cellTypes[hasDesc])

	# add suffixes if requested
	if (addCollectionNames){
		res <- paste0(res, " [", tt$collection, "]")
	}
	if (addDbId){
		res <- paste0(res, " [db", 1:nrow(tt), "]")
	}
	return(res)
}

#' lolaPrepareDataFrameForPlot
#'
#' Helper function for preparing a data.frame for multiple LOLA plots
#'
#' @param lolaDb   LOLA DB object as returned by \code{LOLA::loadRegionDB} or \code{\link{loadLolaDbs}}
#' @param lolaRes  LOLA enrichment result as returned by the \code{runLOLA} function from the \code{LOLA} package
#' @param scoreCol column name in \code{lolaRes} to be plotted
#' @param orderCol column name in \code{lolaRes} which is used for sorting the results
#' @param signifCol column name of the significance score in \code{lolaRes}. Should be one of \code{c("pValueLog", "qValue")}.
#' @param includedCollections vector of collection names to be included in the plot. If empty (default), all collections are used
#' @param pvalCut p-value cutoff to be employed for filtering the results
#' @param maxTerms maximum number of items to be included in the plot
#' @param colorpanel colors to be used for coloring the bars according to "target" (see \code{\link{getTargetFromLolaDb}}). An empty
#'                 vector indicates that black will be used for all bars.
#' @param groupByCollection facet the plot by collection
#' @param orderDecreasing flag indicating whether the value in \code{orderCol} should be considered as decreasing (as opposed
#'                 to increasing). \code{NULL} (default) for automatic determination.
#' @return ggplot object containing the plot
#'
#' @author Fabian Mueller
#' @noRd
lolaPrepareDataFrameForPlot <- function(lolaDb, lolaRes, scoreCol="pValueLog", orderCol=scoreCol, signifCol="qValue", includedCollections=c(), pvalCut=0.01, maxTerms=50, perUserSet=FALSE, groupByCollection=TRUE, orderDecreasing=NULL){
	#dedect by column name whether decreasing order needs to be used
	if (is.null(orderDecreasing)){
		oset.dec <- c("pValueLog", "qValueLog", "logOddsRatio", "oddsRatio")
		oset.inc <- c("maxRnk", "meanRnk", "qValue")
		if (!is.element(orderCol, c(oset.dec, oset.inc))){
			logger.error(c("Could not determine whether to use increasing or decreasing order for column:", orderCol))
		}
		orderDecreasing <- is.element(orderCol, oset.dec)
	}

	if (!is.element(signifCol, c("pValueLog", "qValue"))){
		logger.error(c("Invalid significance column name:", signifCol))
	}

	lolaRes <- as.data.frame(lolaRes)

	if (signifCol == "qValue"){
		lolaRes[["qValueLog"]] <- -log10(lolaRes[["qValue"]])
		signifCol <- "qValueLog"
	}

	#check if column name is in the LOLA results
	# take older versions of LOLA into account (oddsRatio was named logOddsRatio before)
	if (is.element("logOddsRatio", colnames(lolaRes))){
		logger.warning("Detected old version of LOLA. Renaming 'logOddsRatio' to 'oddsRatio'")
		colnames(lolaRes)[colnames(lolaRes)=="logOddsRatio"] <- "oddsRatio"
	}
	if (scoreCol=="logOddsRatio") {
		logger.warning("In newer versions of LOLA the odds ratio column is called 'oddsRatio' (no longer 'logOddsRatio')")
		scoreCol <- "oddsRatio"
	}
	if (orderCol=="logOddsRatio") {
		logger.warning("In newer versions of LOLA the odds ratio column is called 'oddsRatio' (no longer 'logOddsRatio')")
		orderCol <- "oddsRatio"
	}

	lolaRes$name <- getNamesFromLolaDb(lolaDb, addCollectionNames=!groupByCollection)[lolaRes$dbSet]
	lolaRes$name <- trimChar(lolaRes$name, len.out=50, trim.str="...", len.pref=30)
	lolaRes$target <- getTargetFromLolaDb(lolaDb)[lolaRes$dbSet]

	if (!is.element(scoreCol, colnames(lolaRes))){
		logger.error(c("Missing score column from LOLA result:", scoreCol))
	}
	if (!is.element(orderCol, colnames(lolaRes))){
		logger.error(c("Missing order column from LOLA result:", orderCol))
	}

	if (length(includedCollections) > 0){
		lolaRes <- lolaRes[lolaRes[["collection"]] %in% includedCollections,]
	}

	calledSignif <- lolaRes[[signifCol]] > -log10(pvalCut)

	if (sum(calledSignif) < 1){
		# lolaRes.signif <- lolaRes
		logger.warning("No significant association found")
		return(NULL)
	} else {
		lolaRes.signif <- lolaRes[calledSignif,]
	}
	lolaRes.signif <- lolaRes.signif[order(lolaRes.signif[,orderCol], decreasing=orderDecreasing),]

	# data frame for plotting sorted according to significance/odds ratio
	df2p <- lolaRes.signif
	dbSets2keep <- unique(df2p[["dbSet"]])
	if (length(dbSets2keep) > maxTerms){
		if (perUserSet){
			logger.info(c("Reduced the number of region sets in the plot to", maxTerms, "per userSet"))
			uSetF <- lolaRes[["userSet"]]
			if (!is.factor(uSetF)) uSetF <- factor(uSetF)
			dbSets2keep <- c()
			for (ss in levels(uSetF)){
				dbSets.cur <- unique(df2p[uSetF==ss,"dbSet"])
				dbSets2keep <- union(dbSets2keep, dbSets.cur[1:min(maxTerms, length(dbSets.cur))])
			}
		} else {
			logger.info(c("Reduced the number of region sets in the plot to", maxTerms))
			dbSets2keep <- dbSets2keep[1:maxTerms]
		}
		df2p <- df2p[df2p[["dbSet"]] %in% dbSets2keep, ]
	}

	df2p[["term"]] <- factor(df2p[["name"]], levels=unique(df2p[["name"]])) #sort factor by occurence (data.frame is ordered)
	df2p[["target"]] <- factor(df2p[["target"]])

	#bound score at maximum (excluding +/-Inf)
	scoresFinite <- df2p[[scoreCol]][is.finite(df2p[[scoreCol]])]
	cutVal <- max(scoresFinite, na.rm=TRUE)
	df2p[[scoreCol]][df2p[[scoreCol]]>cutVal] <- cutVal
	cutVal <- min(scoresFinite, na.rm=TRUE)
	df2p[[scoreCol]][df2p[[scoreCol]]<cutVal] <- cutVal

	return(df2p)
}


#' lolaVolcanoPlot
#'
#' plot a volcano plot showing LOLA enrichment results: LOLA p-value against the log-odds score. Colored by rank
#'
#' @param lolaDb   LOLA DB object as returned by \code{LOLA::loadRegionDB} or \code{\link{loadLolaDbs}}
#' @param lolaRes  LOLA enrichment result as returned by the \code{runLOLA} function from the \code{LOLA} package
#' @param includedCollections vector of collection names to be included in the plot. If empty (default), all collections are used
#' @param signifCol column name of the significance score in \code{lolaRes}. Should be one of \code{c("pValueLog", "qValue")}.
#' @param colorBy  annotation/column in the the LOLA DB that should be used for point coloring
#' @param colorpanel colors to be used for coloring the points
#' @return ggplot object containing the plot
#'
#' @author Fabian Mueller
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' # compute differential methylation
#' dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
#' # download LOLA DB
#' lolaDest <- tempfile()
#' dir.create(lolaDest)
#' lolaDirs <- downloadLolaDbs(lolaDest, dbs="LOLACore")
#' # perform enrichment analysis
#' res <- performLolaEnrichment.diffMeth(rnb.set.example,dm,lolaDirs[["hg19"]])
#' # select the 500 most hypermethylated tiling regions in ESCs compared to iPSCs
#' # in the example dataset
#' lolaRes <- res$region[["hESC vs. hiPSC (based on Sample_Group)"]][["tiling"]]
#' lolaRes <- lolaRes[lolaRes$userSet=="rankCut_500_hyper",]
#' # plot
#' lolaVolcanoPlot(res$lolaDb, lolaRes, signifCol="qValue")
#' }
lolaVolcanoPlot <- function(lolaDb, lolaRes, includedCollections=c(), signifCol="qValue", colorBy="maxRnk", colorpanel=c()){
	if (length(unique(lolaRes[["userSet"]])) > 1){
		logger.warning("Multiple userSets contained in LOLA result object")
	}

	#prepare data.frame for plotting
	# adjust maxTerms, pvalCut to not filter anything
	df2p <- lolaPrepareDataFrameForPlot(lolaDb, lolaRes, scoreCol="pValueLog", signifCol=signifCol, orderCol="maxRnk", includedCollections=includedCollections, pvalCut=1.1, maxTerms=Inf, perUserSet=FALSE, groupByCollection=TRUE, orderDecreasing=NULL)
	#reverse the order of points s.t. the plot looks nice
	df2p <- df2p[nrow(df2p):1,]

	if (signifCol == "qValue"){
		signifCol <- "qValueLog"
	}

	is.color.gradient <- FALSE
	is.color.discrete <- FALSE
	if (!is.null(colorBy) && nchar(colorBy) > 0 && !is.element(colorBy, colnames(df2p))){
		logger.warning(c("Invalid colorBy argument:", colorBy, "--> using black as color"))
	} else {
		is.color.gradient <- is.numeric(df2p[[colorBy]])
		is.color.discrete <- !is.color.gradient
	}

	oddsRatioCol <- "oddsRatio"
	if (!is.element(oddsRatioCol, colnames(df2p))){
		logger.error("Invalid LOLA result. Could not find a valid column containing odds ratios.")
	}
	pp <- ggplot(df2p) + aes_string(oddsRatioCol, signifCol, color=colorBy) + geom_point()
	cpanel <- colorpanel
	if (length(cpanel) < 1){
		if (is.color.gradient){
			cpanel <- rev(rnb.getOption("colors.gradient"))
		}
		if (is.color.discrete){
			if (is.factor(df2p[[colorBy]])){
				cpanel <- rep(rnb.getOption("colors.category"), length.out=nlevels(df2p[[colorBy]]))
				names(cpanel) <- levels(df2p[[colorBy]])
			} else if (is.character(df2p[[colorBy]])){
				cpanel.names <- unique(df2p[[colorBy]])
				cpanel <- rep(rnb.getOption("colors.category"), length.out=length(cpanel.names))
				names(cpanel) <- cpanel.names
			} else {
				logger.error("invalid discrete coloring column type")
			}
		}
	}
	if (is.color.gradient){
		pp <- pp + scale_color_gradientn(colors=cpanel)
	}
	if (is.color.discrete){
		pp <- pp + scale_color_manual(na.value="#C0C0C0", values=cpanel)
	}
		
	return(pp)
}

#' lolaBarPlot
#'
#' plot a barplot of LOLA enrichment results
#'
#' @param lolaDb   LOLA DB object as returned by \code{LOLA::loadRegionDB} or \code{\link{loadLolaDbs}}
#' @param lolaRes  LOLA enrichment result as returned by the \code{runLOLA} function from the \code{LOLA} package
#' @param scoreCol column name in \code{lolaRes} to be plotted
#' @param orderCol column name in \code{lolaRes} which is used for sorting the results
#' @param signifCol column name of the significance score in \code{lolaRes}. Should be one of \code{c("pValueLog", "qValue")}
#' @param includedCollections vector of collection names to be included in the plot. If empty (default), all collections are used
#' @param pvalCut p-value cutoff to be employed for filtering the results
#' @param maxTerms maximum number of items to be included in the plot
#' @param colorpanel colors to be used for coloring the bars according to "target" (see \code{\link{getTargetFromLolaDb}}). An empty
#'                 vector indicates that black will be used for all bars.
#' @param groupByCollection facet the plot by collection
#' @param orderDecreasing flag indicating whether the value in \code{orderCol} should be considered as decreasing (as opposed
#'                 to increasing). \code{NULL} (default) for automatic determination.
#' @return ggplot object containing the plot
#'
#' @author Fabian Mueller
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' # compute differential methylation
#' dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
#' # download LOLA DB
#' lolaDest <- tempfile()
#' dir.create(lolaDest)
#' lolaDirs <- downloadLolaDbs(lolaDest, dbs="LOLACore")
#' # perform enrichment analysis
#' res <- performLolaEnrichment.diffMeth(rnb.set.example,dm,lolaDirs[["hg19"]])
#' # select the 500 most hypermethylated tiling regions in ESCs compared to iPSCs
#' # in the example dataset
#' lolaRes <- res$region[["hESC vs. hiPSC (based on Sample_Group)"]][["tiling"]]
#' lolaRes <- lolaRes[lolaRes$userSet=="rankCut_500_hyper",]
#' # plot
#' lolaBarPlot(res$lolaDb, lolaRes, scoreCol="oddsRatio", orderCol="maxRnk", pvalCut=0.05)
#' }
lolaBarPlot <- function(lolaDb, lolaRes, scoreCol="pValueLog", orderCol=scoreCol, signifCol="qValue", includedCollections=c(), pvalCut=0.01, maxTerms=50, colorpanel=sample(rainbow(maxTerms,v=0.5)), groupByCollection=TRUE, orderDecreasing=NULL){
	if (length(unique(lolaRes[["userSet"]])) > 1){
		logger.warning("Multiple userSets contained in LOLA result object")
	}
	if (scoreCol=="logOddsRatio") {
		logger.warning("In newer versions of LOLA the odds ratio column is called 'oddsRatio' (no longer 'logOddsRatio')")
		scoreCol <- "oddsRatio"
	}
	#prepare data.frame for plotting
	df2p <- lolaPrepareDataFrameForPlot(lolaDb, lolaRes, scoreCol=scoreCol, orderCol=orderCol, signifCol=signifCol, includedCollections=includedCollections, pvalCut=pvalCut, maxTerms=maxTerms, perUserSet=FALSE, groupByCollection=groupByCollection, orderDecreasing=orderDecreasing)

	if (is.null(df2p)){
		return(rnb.message.plot("No significant association found"))
	}

	aesObj <- aes_string("term", scoreCol)
	if (length(colorpanel) > 0) aesObj <- aes_string("term", scoreCol, fill="target")

	pp <- ggplot(df2p) + aesObj + geom_col() + 
		  scale_x_discrete(name="") + 
		  theme(axis.text.x=element_text(angle=90, hjust=1, vjust=0.5))

	if (length(colorpanel) > 0){
		# check if the defined color panel already contains a mapping of targets to colors
		# otherwise create an arbitrary mapping
		if (!is.null(names(colorpanel)) && all(levels(df2p[["target"]]) %in% names(colorpanel))){
			cpanel <- colorpanel
		} else {
			cpanel <- rep(colorpanel, length.out=nlevels(df2p[["target"]]))
			names(cpanel) <- levels(df2p[["target"]])
		}
		pp <- pp + scale_fill_manual(na.value="#C0C0C0", values=cpanel, guide=FALSE)
	}

	if (groupByCollection){
		pp <- pp + facet_grid(. ~ collection, scales = "free", space = "free")
	}
	return(pp)
}

#' lolaBarPlot.hyp
#'
#' Adaptation of  \code{\link{lolaBarPlot}} to plot hypo and hpermethylated regions side by side
#' Hyper-/hypomethylation is assumed to be annotated in the userSet column of lolaRes
#'
#' @param lolaDb   LOLA DB object as returned by \code{LOLA::loadRegionDB} or \code{\link{loadLolaDbs}}
#' @param lolaRes  LOLA enrichment result as returned by the \code{runLOLA} function from the \code{LOLA} package
#' @param scoreCol column name in \code{lolaRes} to be plotted
#' @param orderCol column name in \code{lolaRes} which is used for sorting the results
#' @param signifCol column name of the significance score in \code{lolaRes}. Should be one of \code{c("pValueLog", "qValue")}
#' @param includedCollections vector of collection names to be included in the plot. If empty (default), all collections are used
#' @param pvalCut p-value cutoff (negative log10) to be employed for filtering the results
#' @param maxTerms maximum number of items to be included in the plot
#' @param colorpanel colors to be used for coloring the bars according to "target" (see \code{\link{getTargetFromLolaDb}}). An empty
#'                 vector indicates that black will be used for all bars.
#' @param groupByCollection facet the plot by collection
#' @param orderDecreasing flag indicating whether the value in \code{orderCol} should be considered as decreasing (as opposed
#'                 to increasing). \code{NULL} (default) for automatic determination.
#' @return ggplot object containing the plot
#'
#' @author Fabian Mueller
#' @noRd
lolaBarPlot.hyp <- function(lolaDb, lolaRes, scoreCol="pValueLog", orderCol=scoreCol, signifCol="qValue", includedCollections=c(), pvalCut=0.01, maxTerms=50, colorpanel=c("#a6611a", "#018571"), groupByCollection=TRUE, orderDecreasing=NULL){
	isHypo  <- grepl("hypo", lolaRes[["userSet"]], ignore.case=TRUE)
	isHyper <- grepl("hyper", lolaRes[["userSet"]], ignore.case=TRUE)
	if (sum(isHypo) < 1) logger.error("LOLA Result does not contain hypOmethylated userSet")
	if (sum(isHyper) < 1) logger.error("LOLA Result does not contain hypERmethylated userSet")
	lolaRes$userSet[isHypo]  <- "hypomethylated"
	lolaRes$userSet[isHyper] <- "hypermethylated"
	if (sum(isHypo) + sum(isHyper) != nrow(lolaRes)){
		lolaRes <- lolaRes[isHypo | isHyper,]
		logger.warning("LOLA Result contains userSets not annotated as hyper or hypo")
	}
	if (scoreCol=="logOddsRatio") {
		logger.warning("In newer versions of LOLA the odds ratio column is called 'oddsRatio' (no longer 'logOddsRatio')")
		scoreCol <- "oddsRatio"
	}
	#prepare data.frame for plotting
	df2p <- lolaPrepareDataFrameForPlot(lolaDb, lolaRes, scoreCol=scoreCol, orderCol=orderCol, signifCol=signifCol, includedCollections=includedCollections, pvalCut=pvalCut, maxTerms=maxTerms, perUserSet=TRUE, groupByCollection=groupByCollection, orderDecreasing=orderDecreasing)

	if (is.null(df2p)){
		return(rnb.message.plot("No significant association found"))
	}

	cpanel <- c(hypomethylated=colorpanel[1], hypermethylated=colorpanel[2])
	aesObj <- aes_string("term", scoreCol, fill="userSet")

	pp <- ggplot(df2p) + aesObj + geom_col(position="dodge") + 
		  scale_x_discrete(name="") + 
		  scale_fill_manual(values=cpanel) +
		  theme(axis.text.x=element_text(angle=90, hjust=1, vjust=0.5))

	if (groupByCollection){
		pp <- pp + facet_grid(. ~ collection, scales = "free", space = "free")
	}
	return(pp)
}

#' lolaBoxPlotPerTarget
#'
#' plot a boxplot showing LOLA enrichment results per "target" group (see \code{\link{getTargetFromLolaDb}} for an explanation of
#' "target").
#'
#' @param lolaDb   LOLA DB object as returned by \code{LOLA::loadRegionDB} or \code{\link{loadLolaDbs}}
#' @param lolaRes  LOLA enrichment result as returned by the \code{runLOLA} function from the \code{LOLA} package
#' @param scoreCol column name in \code{lolaRes} to be plotted
#' @param orderCol column name in \code{lolaRes} which is used for sorting the results
#' @param signifCol column name of the significance score in \code{lolaRes}. Should be one of \code{c("pValueLog", "qValue")}
#' @param includedCollections vector of collection names to be included in the plot. If empty (default), all collections are used
#' @param pvalCut  p-value cutoff to be employed for filtering the results
#' @param maxTerms maximum number of items to be included in the plot
#' @param colorpanel colors to be used for coloring the bars according to "target" (see \code{\link{getTargetFromLolaDb}}). An empty
#'                 vector indicates that black will be used for all bars.
#' @param groupByCollection facet the plot by collection
#' @param orderDecreasing flag indicating whether the value in \code{orderCol} should be considered as decreasing (as opposed
#'                 to increasing). \code{NULL} (default) for automatic determination.
#' @param scoreDecreasing flag indicating whether the value in \code{scoreCol} should be considered as decreasing (as opposed
#'                 to increasing). \code{NULL} (default) for automatic determination.
#' @return ggplot object containing the plot
#'
#' @author Fabian Mueller
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' # compute differential methylation
#' dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
#' # download LOLA DB
#' lolaDest <- tempfile()
#' dir.create(lolaDest)
#' lolaDirs <- downloadLolaDbs(lolaDest, dbs="LOLACore")
#' # perform enrichment analysis
#' res <- performLolaEnrichment.diffMeth(rnb.set.example,dm,lolaDirs[["hg19"]])
#' # select the 500 most hypermethylated tiling regions in ESCs compared to iPSCs
#' # in the example dataset
#' lolaRes <- res$region[["hESC vs. hiPSC (based on Sample_Group)"]][["tiling"]]
#' lolaRes <- lolaRes[lolaRes$userSet=="rankCut_500_hyper",]
#' # plot
#' lolaBoxPlotPerTarget(res$lolaDb, lolaRes, scoreCol="oddsRatio", orderCol="maxRnk", pvalCut=0.05)
#' }
lolaBoxPlotPerTarget <- function(lolaDb, lolaRes, scoreCol="pValueLog", orderCol=scoreCol, signifCol="qValue", includedCollections=c(), pvalCut=0.01, maxTerms=50, colorpanel=c(), groupByCollection=TRUE, orderDecreasing=NULL, scoreDecreasing=NULL){
	if (length(unique(lolaRes[["userSet"]])) > 1){
		logger.warning("Multiple userSets contained in LOLA result object")
	}
	# detect by column name whether decreasing order needs to be used for the score column
	if (is.null(scoreDecreasing)){
		oset.dec <- c("pValueLog", "logOddsRatio", "oddsRatio")
		oset.inc <- c("maxRnk", "meanRnk")
		if (!is.element(scoreCol, c(oset.dec, oset.inc))){
			logger.error(c("Could not determine whether to use increasing or decreasing order for score column:", scoreCol))
		}
		scoreDecreasing <- is.element(scoreCol, oset.dec)
	}
	if (scoreCol=="logOddsRatio") {
		logger.warning("In newer versions of LOLA the odds ratio column is called 'oddsRatio' (no longer 'logOddsRatio')")
		scoreCol <- "oddsRatio"
	}
	#prepare data.frame for plotting
	df2p <- lolaPrepareDataFrameForPlot(lolaDb, lolaRes, scoreCol=scoreCol, orderCol=orderCol, signifCol=signifCol, includedCollections=includedCollections, pvalCut=pvalCut, maxTerms=Inf, perUserSet=FALSE, groupByCollection=groupByCollection, orderDecreasing=orderDecreasing)

	if (is.null(df2p)){
		return(rnb.message.plot("No significant association found"))
	}

	# maxTerms now applies to the targets, not to LOLA DB items
	if (nlevels(df2p$target) > maxTerms){
		# if there are too many targets, select the targets from the top of the ordered table
		targets2keep <- unique(as.character(df2p$target))[1:maxTerms]
		df2p <- df2p[df2p$target %in% targets2keep,]
		df2p[["target"]] <- factor(as.character(df2p[["target"]]))
		logger.info(c("Reduced the number of targets in the plot to", maxTerms))
	}

	# sort targets by median score
	targetScore <- tapply(df2p[[scoreCol]], df2p[["target"]], FUN=function(x){median(x, na.rm=TRUE)})
	df2p[["target"]] <- factor(as.character(df2p[["target"]]),
		levels=names(targetScore)[order(targetScore, decreasing=scoreDecreasing)]
	)

	aesObj <- aes_string("target", scoreCol)
	if (length(colorpanel) > 0) aesObj <- aes_string("target", scoreCol, fill="target")

	pp <- ggplot(df2p) + aesObj + geom_boxplot(outlier.shape=NA) +
		  geom_dotplot(aes(fill=NULL), binaxis="y", stackdir="center") +
		  scale_x_discrete(name="") + 
		  theme(axis.text.x=element_text(angle=90, hjust=1, vjust=0.5))

	if (length(colorpanel) > 0){
		# check if the defined color panel already contains a mapping of targets to colors
		# otherwise create an arbitrary mapping
		if (!is.null(names(colorpanel)) && all(levels(df2p[["target"]]) %in% names(colorpanel))){
			cpanel <- colorpanel
		} else {
			cpanel <- rep(colorpanel, length.out=nlevels(df2p[["target"]]))
			names(cpanel) <- levels(df2p[["target"]])
		}
		pp <- pp + scale_fill_manual(na.value="#C0C0C0", values=cpanel, guide=FALSE)
	} 
	if (groupByCollection){
		pp <- pp + facet_grid(. ~ collection, scales = "free", space = "free")
	}
	return(pp)
}

################################################################################
# Cluster Architecture Descriptions
################################################################################
# General Virtual class template
################################################################################
#' ClusterArchitecture Class
#'
#' A virtual class for storing specifications of architectures for different compute clusters. It is designed to
#' let other classes inherit from it
#' 
#' @details
#' For a concrete child class for a sun grid architecture specification see \code{\linkS4class{ClusterArchitectureSGE}}
#' If you want to implement your own child class be sure to at least implement the following functions:
#' \code{\link{getSubCmdTokens,ClusterArchitecture-method}}.
#'
#' @section Slots:
#' \describe{
#'   \item{\code{name}}{A name or identifier}
#'   \item{\code{executables}}{A NAMED character vector of executables that can be used by the cluster. For instance, the \code{R} executable is important} 
#'   \item{\code{getSubCmdTokens.optional.args}}{character vector containing the valid optional arguments to the \code{\link{getSubCmdTokens,ClusterArchitecture-method}} function.}
#' }
#'
#' @section Methods:
#' \describe{
#'   \item{\code{\link{getSubCmdTokens,ClusterArchitecture-method}}}{Returns a vector of command line tokens corresponding to submitting
#'   a job with the given command to the cluster}
#'   \item{\code{\link{getSubCmdStr,ClusterArchitecture-method}}}{Returns a string for the of command line corresponding to submitting
#'   a job with the given command to the cluster}
#'   \item{\code{\link{setExecutable,ClusterArchitecture,character,character-method}}}{Tells the cluster architecture about an executable that can be submitted as job}
#'   \item{\code{\link{getExecutable,ClusterArchitecture,character-method}}}{Gets the location of an executable associated with a name}
#' }
#'
#' @name ClusterArchitecture-class
#' @rdname ClusterArchitecture-class
#' @author Fabian Mueller
#' @exportClass ClusterArchitecture
setClass("ClusterArchitecture",
	slots = list(
		name = "character",
		executables = "character",
		getSubCmdTokens.optional.args = "character"
	),
	contains = "VIRTUAL"
)

#' initialize.ClusterArchitecture
#'
#' Initialize an ClusterArchitecture object
#' 
#' @param .Object New instance of \code{ClusterArchitecture}.
#' @param name A name or identifier
#'
#' @author Fabian Mueller
#' @docType methods
setMethod("initialize","ClusterArchitecture",
	function(
		.Object,
		name="ClusterArchitecture"
	) {
		.Object@name           <- name
		.Object@executables    <- character(0)
		.Object@getSubCmdTokens.optional.args <- character(0)
		.Object
	}
)

#implement for each inheriting class
if (!isGeneric("getSubCmdTokens")) {
	setGeneric(
		"getSubCmdTokens",
		function(object, cmd.tokens, log, job.name="", res.req=character(0), depend.jobs=character(0), ...) standardGeneric("getSubCmdTokens"),
		signature=c("object","cmd.tokens","log","job.name","res.req","depend.jobs")
	)
}
#' getSubCmdTokens-methods
#'
#' Returns a string for the of command line corresponding to submitting
#' a job with the given command to the cluster.
#' @details
#' For a concrete child class implementation for a sun grid architecture specification see \code{\link{getSubCmdTokens,ClusterArchitectureSGE-method}}
#'
#' @param object \code{\linkS4class{ClusterArchitecture}} object
#' @param cmd.tokens a character vector specifying the executable command that should be wrapped in the cluster submission command
#' @param log file name and path of the log file that the submitted job writes to
#' @param job.name name of the submitted job
#' @param res.req character vector specifying required resources. The resource requirements should be the values of the vector,
#'                the names should specify the resource name
#' @param depend.jobs character vector containg names or ids of jobs the submitted job will depend on.
#' @return A character vector containing the submission command tokens
#'
#' @rdname getSubCmdTokens-ClusterArchitecture-methods
#' @docType methods
#' @aliases getSubCmdTokens
#' @aliases getSubCmdTokens,ClusterArchitecture-method
#' @author Fabian Mueller
#' @export
setMethod("getSubCmdTokens",
	signature(
		object="ClusterArchitecture"
	),
	function(
		object,
		cmd.tokens,
		log,
		job.name = "",
		res.req = character(0),
		depend.jobs = character(0)
	) {
		return(character(0))
	}
)

if (!isGeneric("getSubCmdStr")) setGeneric("getSubCmdStr", function(object,...) standardGeneric("getSubCmdStr"))
#' getSubCmdStr-methods
#'
#' Returns a string for the of command line corresponding to submitting
#' a job with the given command to the cluster.
#'
#' @param object \code{\linkS4class{ClusterArchitecture}} object
#' @param ... arguments passed on to \code{\link{getSubCmdTokens,ClusterArchitecture-method}}
#' @return A string containing the submission command
#'
#' @rdname getSubCmdStr-ClusterArchitecture-methods
#' @docType methods
#' @aliases getSubCmdStr
#' @aliases getSubCmdStr,ClusterArchitecture-method
#' @author Fabian Mueller
#' @export
setMethod("getSubCmdStr","ClusterArchitecture",
	function(
		object,
		...
	) {
		res <- getSubCmdTokens(object,...)
		res <- paste(res, collapse=" ")
		return(res)
	}
)

if (!isGeneric("setExecutable")) setGeneric("setExecutable", function(object,exec.name,exec.loc) standardGeneric("setExecutable"))
#' setExecutable-methods
#'
#' Tells the cluster architecture about an executable that can be submitted as job
#'
#' @param object \code{\linkS4class{ClusterArchitecture}} object
#' @param exec.name A name/identifier that will be associated with the given executable
#' @param exec.loc The executable's location
#' @return The modified object
#'
#' @rdname setExecutable-ClusterArchitecture-methods
#' @docType methods
#' @aliases setExecutable
#' @aliases setExecutable,ClusterArchitecture-method
#' @author Fabian Mueller
#' @export
setMethod("setExecutable",
	signature(
		object="ClusterArchitecture",
		exec.name="character",
		exec.loc="character"
	),
	function(
		object,
		exec.name,
		exec.loc
	) {
		object@executables[exec.name] <- exec.loc
		return(object)
	}
)

if (!isGeneric("getExecutable")) setGeneric("getExecutable", function(object,exec.name) standardGeneric("getExecutable"))
#' getExecutable-methods
#'
#' Retrieves the executable associated with a name/identifier
#'
#' @param object \code{\linkS4class{ClusterArchitecture}} object
#' @param exec.name The executable's name/identifier
#' @return The executable. If the name is not associated with any executable, the names will be returned and a warning will be raised
#'
#' @rdname getExecutable-ClusterArchitecture-methods
#' @docType methods
#' @aliases getExecutable
#' @aliases getExecutable,ClusterArchitecture-method
#' @author Fabian Mueller
#' @export
setMethod("getExecutable",
	signature(
		object="ClusterArchitecture",
		exec.name="character"
	),
	function(
		object,
		exec.name
	) {
		res <- object@executables[exec.name]
		if (is.na(res)){
			warning(paste0("Executable '",exec.name,"' not found. Returning the name as value"))
			res <- exec.name
		}
		return(res)
	}
)

########################################################################################################################
## exportGEO.R
## created: 2017-09-25
## creator: Pavlo Lutsik
## ---------------------------------------------------------------------------------------------------------------------
## Uploading 450k/EPIC data to GEO.
########################################################################################################################



# prepareGEOSampleInfoTemplate
# 
# Prepare a GEO sample information template for further use in \code{prepareSOFTfileForGEO}
# @param rnb.set                   Object inheriting from class \code{RnBeadSet}
#                                  with \code{"GSE"}.
# @param sample.source.col         \code{integer} specifying a column in the \code{pheno} slot of \code{rnb.set} 
#                                  containing information which will be written into the field \code{Sample_source_name_ch1}
#                                  of each sample record
# @param sample.description        \code{integer} specifying a column in the \code{pheno} slot of \code{rnb.set} 
#                                  containing information which will be written into the field \code{Sample_desription}
#                                  of each sample record
# @param sample.title.col         \code{integer} specifying a column in the \code{pheno} slot of \code{rnb.set} 
#                                  containing information which will be written into the field \code{Sample_title}
#                                  of each sample record
#
# @return matrix of type \code{character} with columns (named) corresponding to SOFT labels, and rows to samples in \code{rnb.set}
#
prepareGEOSampleInfoTemplate<-function(rnb.set, pheno.cols=1:ncol(pheno(rnb.set)), sample.title.col=NULL, sample.source.col=NULL, sample.description.col=NULL, sample.extra.info=NULL)
{
	pheno.data<-pheno(rnb.set)
	
	sample.names<-samples(rnb.set)
	if(!is.null(sample.title.col)){
		sample.titles<-pheno(rnb.set)[[sample.title.col]]
	}else{
		sample.titles<-sample.names
	}
	if(!is.null(sample.source.col)){
		sample.sources<-pheno(rnb.set)[[sample.source.col]]
	}else{
		sample.sources<-NULL
	}
	if(!is.null(sample.description.col)){
		sample.descr<-pheno(rnb.set)[[sample.description.col]]
	}else{
		sample.descr<-NULL
	}
	
	platform <- c("HumanMethylation27", "HumanMethylation450", "HumanMethylationEPIC", "HumanMethylationEPICv2")[match(rnb.set@target, c('probes27','probes450','probesEPIC', 'probesEPICv2'))]
	chipVersion<-"unknown"
	organism <- c("Homo sapiens", "Mus muscullus", "Rattus norvegicus")[which(!is.na(pmatch( c('hg','mm','rn'), assembly(rnb.set))))]
	platformID<-c(
			"HumanMethylation27"="GPL8490",
			"HumanMethylation450"="GPL13534",
			"HumanMethylationEPIC"="GPL23976", ## Alternative: GPL21145
			"HumanMethylationEPICv2"="GPL33022")[platform]
	
	templateTitle <- c(
			"Sample_title", 
			"Sample_channel_count",
			"Sample_source_name_ch1", 
			"Sample_organism_ch1", 
		rep("Sample_characteristics_ch1", length(pheno.cols)),
			"Sample_molecule_ch1", 
			"Sample_extract_protocol_ch1",
			"Sample_label_ch1", 
			"Sample_label_protocol_ch1", 
			"Sample_hyb_protocol",
			"Sample_scan_protocol", 
			"Sample_description", 
			"Sample_data_processing",
			"Sample_platform_id", 
			"Sample_supplementary_file",
			"Sample_contact_name",
			"Sample_contact_email",
			"Sample_contact_laboratory",
			"Sample_contact_department",
			"Sample_contact_institute",
			"Sample_contact_address",
			"Sample_contact_city",
			"Sample_contact_state",
			"Sample_contact_zip/postal_code",
			"Sample_contact_country")
	
	templateContent <- c(
			"unknown sample", 
			"2", 
			"unknown", 
			organism, 
			colnames(pheno.data)[pheno.cols],
			"genomic DNA",
			"standard as recommended by Illumina", 
			"Cy3,Cy5", 
			"standard as recommended by Illumina",
			"standard as recommended by Illumina", 
			"standard as recommended by Illumina",
			"unknown sample", 
			"unknown", 
			platformID, 
			"none",
			"unknown",
			"unknown",
			"unknown",
			"unknown",
			"unknown",
			"unknown",
			"unknown",
			"unknown",
			"unknown",
			"unknown"
			)
	
	
	if(!is.null(sample.extra.info)){
		for(key in names(sample.extra.info)){
			if(key %in% templateTitle){
				templateContent[templateTitle==key]<-sample.extra.info[[key]]
			}
		}
	}
	
	if(templateContent[templateTitle=="Sample_data_processing"]==""){
		preprocessMethod <- paste("normalization: ", norm_method, "; ", "background: ", bgcorr_method, sep="")
		templateContent[templateTitle == "Sample_data_processing"] <- preprocessMethod
	}
	
	norm_method<-rnb.set@status$normalized
	bgcorr_method<-rnb.set@status$background

	template <- templateTitle
	sample.characteristics.cols<-grep("Sample_characteristics_ch1", template)
	for (i in seq(sample.names)) {
		templateContentCopy<-templateContent
		
		templateContentCopy[templateTitle == "Sample_title"] <- sample.titles[i]
		if(!is.null(sample.sources)){
			templateContentCopy[templateTitle == "Sample_source_name_ch1"] <- sample.sources[i]
		}
		if(!is.null(sample.descr)){
			templateContentCopy[templateTitle == "Sample_description"] <- sample.descr[i]
		}
		
		templateContentCopy[sample.characteristics.cols] <- paste(
				templateContentCopy[sample.characteristics.cols], 
				as.character(pheno.data[i,pheno.cols]), sep=": ")
		template <- rbind(template, templateContentCopy)
	}
	template <- cbind(c("sampleID", sample.names), template)
	
	colnames(template)<-template[1,]
	template<-template[-1,]
	
	return(template)
}

#'
#' prepareSOFTfileForGEO
#'
#' Starting from an \code{RnBeadSet} object generates a batch submission file for Gene Expression Omnibus series in SOFT format 
#'
#' @param rnb.set                   Object inheriting from class \code{RnBeadSet}
#'                                  with \code{"GSE"}.
#' @param filename                  Absolute path or a name of a SOFT file to be generated
#' 
#' @param sample.source.col         \code{integer} singleton specifying a column in the \code{pheno} slot of \code{rnb.set} 
#'                                  containing information which will be written into the field \code{Sample_source_name_ch1}
#'                                  of each sample record
#' @param sample.description.col    \code{integer} singleton specifying a column in the \code{pheno} slot of \code{rnb.set} 
#'                                  containing information which will be written into the field \code{Sample_desription}
#'                                  of each sample record
#' @param sample.title.col         \code{integer} singleton specifying a column in the \code{pheno} slot of \code{rnb.set} 
#'                                  containing information which will be written into the field \code{Sample_title}
#'                                  of each sample record. If \code{NULL}, the result of \code{samples(rnb.set)} will be used
#' @param export.cols               \code{integer} vector specifying columns in the \code{pheno} slot of \code{rnb.set} 
#'                                  containing information which will be written into the fields \code{Sample_characteristics_ch1}
#'                                  of each sample record
#' @param rnb.set.raw               Object inheriting from class \code{RnBeadSet}
#' @param sample.extra.info         Optionally, a \code{list} with elements to be written to all series record. Elements should be character
#'                                  singletons named with valid SOFT labels of a SAMPLE section, e.g.\code{Sample_extract_protocol}, 
#'                                  \code{Sample_hyb_protocol}, \code{Sample_label_protocol_ch1}, \code{Sample_data_processing}, 
#'                                  \code{Sample_contact_name}, \code{Sample_contact_email} etc.
#' @param series.info               A \code{list} with elements to be written to the series record. Elements should be character
#'                                  singletons named \code{SERIES} (contains a valid GSE identifier for updating an existing series)
#'                                  \code{Series_title}, \code{Series_summary}, \code{Series_type}, \code{Series_overall_design},
#'                                  \code{Series_contributor}, \code{Series_sample_id}
#' 
#' @return \code{TRUE} on success.
#'
#' @details The code was largely adapted from a similar function in package \code{lumi} which is due to Pan Du.
#' 
#' @author Pavlo Lutsik
#' @export
#'
prepareSOFTfileForGEO<-function(rnb.set, 
		filename, 
		sample.source.col=NULL, 
		sample.description.col=NULL,
		sample.title.col = NULL,
		export.cols=seq(ncol(pheno(rnb.set))),
		rnb.set.raw=NULL, 
		sample.extra.info=NULL,
		series.info=NULL){
	
	if(!inherits(rnb.set, "RnBeadSet")){
		rnb.error("Wrong value for rnb.set supplied: not an RnBeadSet object")
	}
	
	if(!is.null(rnb.set.raw) && !inherits(rnb.set.raw, "RnBeadSet")){
		rnb.error("Wrong value for rnb.set.raw supplied: not an RnBeadSet object")
	}
	
	if(!(is.character(filename) && length(filename)==1)){
		rnb.error("Wrong value for rnb.set.raw supplied: not an RnBeadSet object")
	}
	
	expr.norm <- signif(meth(rnb.set),5)
	
	rnb.set.intens<-NULL
	if (is.null(rnb.set.raw)) {
		meth.raw <- NULL
		rnb.set.intens<-rnb.set
	}else{
		meth.raw <- signif(meth(rnb.set.raw),5)
		rnb.set.intens<-rnb.set.raw
	}
	
	if(!is.null(dpval(rnb.set.intens))){
		detect <- signif(dpval(rnb.set.intens), 5)
	}else{
		detect <- NULL
	}
	
	if(inherits(rnb.set.intens, "RnBeadRawSet") && !is.null(M(rnb.set.intens)) && !is.null(U(rnb.set.intens))){
		methyData <- signif(M(rnb.set.intens), 5)
		unmethyData <- signif(U(rnb.set.intens), 5)
	}else{
		methyData<-NULL
		unmethyData<-NULL
	}
	
	sampleInfo <- prepareGEOSampleInfoTemplate(
			rnb.set, 
			pheno.cols=export.cols,
			sample.title.col=sample.title.col,
			sample.source.col=sample.source.col, 
			sample.description.col=sample.description.col, 
			sample.extra.info=sample.extra.info)
	
	sampleInfoTitle <- colnames(sampleInfo)
	if (any(sapply(sampleInfo[, -1, drop = F], nchar) == 0))
		stop("No blank fields are allowed in the sampleInfo table!\nYou can check some example submissions, like GSM296418, at the GEO website.\n")
	
	nuID <- rownames(annotation(rnb.set))
	probeId <- nuID
	
#	if (length(which(is.nuID(sample(nuID, 100)))) < 20) {
#		nuID <- NULL
#	}
	
	sampleID <- sampleInfo[, "sampleID"]
	sampleTitle <- sampleInfo[, "Sample_title"]
	outputFile <- filename
	for (i in seq(sampleID)) {
		cat("Processing sample", i, "\n")
		if (i == 1) {
			cat("^SAMPLE = ", sampleTitle[i], "\n", sep = "",
					file = outputFile, append = FALSE)
		}
		else {
			cat("^SAMPLE = ", sampleTitle[i], "\n", sep = "",
					file = outputFile, append = TRUE)
		}
		sampleInfo.i <- paste("!", sampleInfoTitle[-1], " = ",
				sampleInfo[i, -1], "\n", sep = "", collapse = "")
		sampleInfo.i <- gsub("'", "\\'", sampleInfo.i)
		cat(sampleInfo.i, file = outputFile, sep = "", append = TRUE)
		tableHead <- "ID_REF"
		cat("#ID_REF = Illumina ID\n", file = outputFile, append = TRUE)
#		if (!is.null(nuID)) {
#			cat("#nuID = nucleotide universal IDentifier (nuID), convertible to and from probe sequence. See Bioconductor lumi package for more details.\n",
#					file = outputFile, append = TRUE)
#			tableHead <- c(tableHead, "nuID")
#		}
		cat("#VALUE = Beta-value\n", file = outputFile, append = TRUE)
		if (!is.null(meth.raw))
			cat("#RAW_VALUE = raw Beta-value\n", file = outputFile,
					append = TRUE)
		tableHead <- c(tableHead, "VALUE")
		if (!is.null(meth.raw))
			tableHead <- c(tableHead, "RAW_VALUE")
		if (!is.null(methyData)) {
			cat("#METHYLATED = the intensities measured by methylated probes\n",
					file = outputFile, append = TRUE)
			tableHead <- c(tableHead, "METHYLATED")
		}
		if (!is.null(unmethyData)) {
			cat("#UNMETHYLATED = the intensities measured by unmethylated probes\n",
					file = outputFile, append = TRUE)
			tableHead <- c(tableHead, "UNMETHYLATED")
		}
		if (!is.null(detect)) {
			cat("#Detection_Pval = the detection p-value of the probe\n",
					file = outputFile, append = TRUE)
			tableHead <- c(tableHead, "Detection_Pval")
		}
		sampleTable.i <- probeId
#		if (!is.null(nuID))
#			sampleTable.i <- cbind(sampleTable.i, nuID)
		sampleTable.i <- cbind(sampleTable.i, expr.norm[, sampleID[i],
						drop = FALSE])
		if (!is.null(meth.raw))
			sampleTable.i <- cbind(sampleTable.i, meth.raw[, sampleID[i],
							drop = FALSE])
		if (!is.null(methyData))
			sampleTable.i <- cbind(sampleTable.i, methyData[,
							sampleID[i], drop = FALSE])
		if (!is.null(unmethyData))
			sampleTable.i <- cbind(sampleTable.i, unmethyData[,
							sampleID[i], drop = FALSE])
		if (!is.null(detect))
			sampleTable.i <- cbind(sampleTable.i, detect[, sampleID[i],
							drop = FALSE])
		sampleTable.i <- rbind(tableHead, sampleTable.i)
		cat("!sample_table_begin\n", file = outputFile, append = TRUE)
		write.table(sampleTable.i, sep = "\t", quote = FALSE,
				file = outputFile, append = TRUE, col.names = FALSE,
				row.names = FALSE)
		cat("!sample_table_end\n", file = outputFile, append = TRUE)
	}
	
	if(!is.null(series.info)){
		label_series<-c(
				'Series_title',
				'Series_summary',
				'Series_overall_design')
		
		cat(paste('^SERIES', series.info[["SERIES"]], sep=" = "), file = outputFile, append = TRUE)
		
		labelvalue_series<-paste('\n!',sapply(label_series,function(x){paste(x,series.info[[x]], sep=" = ")}), sep='')
		cat(labelvalue_series, file = outputFile, append = TRUE)
		
		cat(paste('\n!Series_summary', series.info$Series_summary, sep=" = "), file = outputFile, append = TRUE)
		
		seriescontributor<-strsplit(series.info$Series_contributor,';')[[1]]
		cat(paste('\n!Series_contributor',seriescontributor, sep=" = "), file = outputFile, append = TRUE)
		
		cat(paste('\n!Series_sample_id', paste(sampleID, sep=";") ,sep=" = "), file = outputFile, append = TRUE)
	
	}
	return(TRUE)
}

########################################################################################################################
## combining.R
## created: 2017-04-13
## creator: Yassen Assenov
## ---------------------------------------------------------------------------------------------------------------------
## Functions related to combining datasets of the same or different platforms, differential methylation results, etc.
########################################################################################################################

## TODO: Move the "combine" method here; use the function rnb.combine.arrays in it

## F U N C T I O N S ###################################################################################################

#' Combine dataset slots
#' 
#' Combines the data matrices of two datasets into a single one.
#' 
#' @param m1    Matrix of the first dataset. Set this to \code{NULL} if the first dataset does not contain this slot.
#' @param m2    Matrix of the second dataset. Set this to \code{NULL} if the second dataset does not contain this slot.
#' @param ii    Two-column \code{integer} \code{matrix} of indices in \code{m1} and \code{m2} to be considered for the
#'              combined matrix.
#' @param nn    Two-element \code{integer} \code{vector} listing the number of samples in the first and second dataset,
#'              respectively. If \code{m1} is provided, it should contain \code{nn[1]} columns.
#' @param useff Flag indicating if disk dumping for the output matrix must be enabled.
#' @return When \code{m1} and \code{m2} are both \code{NULL}, the return value is also \code{NULL}. Otherwise, the
#'         constructed combined matrix as an \code{ff} object (if \code{useff} is \code{TRUE}), or as an instance of
#'         \code{matrix} (if \code{useff} is \code{FALSE}). The dimensions of the matrix are \code{nrow(ii)} x
#'         \code{sum(nn)}.
#' 
#' @author Yassen Assenov
#' @noRd
rnb.combine.matrices <- function(m1, m2, ii, nn, useff = rnb.getOption("disk.dump.big.matrices")) {
	if (is.null(m1)) {
		if (is.null(m2)) {
			return(NULL)
		}
		if ("ff" %in% class(m2)) {
			d.type <- vmode(m2)
		} else {
			d.type <- typeof(m2)
		}
	} else if ("ff" %in% class(m1)) {
		d.type <- vmode(m1)
	} else {
		d.type <- typeof(m1)
	}
	na.value <- do.call(paste0("as.", d.type), list(x = NA))
	if (useff) {
		mm <- ff(na.value, dim = c(nrow(ii), sum(nn)), dimnames = list(NULL, NULL), vmode = d.type)
	} else {
		mm <- matrix(na.value, nrow = nrow(ii), ncol = sum(nn))
	}
	if (!is.null(m1)) {
        present<-which(!is.na(ii[,1]))
		for (i in 1:(nn[1])) {
			mm[present, i] <- m1[ii[present, 1], i]
		}
	}
	if (!is.null(m2)) {
        present<-which(!is.na(ii[,2]))
		for (i in 1:(nn[2])) {
			mm[present, nn[1] + i] <- m2[ii[present, 2], i]
		}
	}
	mm
}

########################################################################################################################

#' Combine pheno tables
#' 
#' Combines two sample annotation tables.
#' 
#' @param dataset1 First input dataset as an object of type inheriting \code{\linkS4class{RnBeadSet}}.
#' @param dataset2 Second input dataset as an object of type inheriting \code{\linkS4class{RnBeadSet}}.
#' @return Combined sample annotation table as a \code{data.frame}.
#' 
#' @author Yassen Assenov
#' @noRd
rnb.combine.pheno <- function(dataset1, dataset2) {
	tbl1 <- dataset1@pheno
	tbl2 <- dataset2@pheno
	if (!identical(sapply(tbl1, class), sapply(tbl2, class))) {
		stop("Mismatch in sample annotation tables")
	}
	for (i in which(sapply(tbl1, class) == "factor")) {
		if (!identical(levels(tbl1[, i]), levels(tbl2[, i]))) {
			stop("Mismatch in sample annotation tables")
		}
	}
	rbind(tbl1, tbl2)
}

########################################################################################################################

#' Combine array-based datasets
#'
#' Concatenates two array-based datasets focusing on the common probes.
#'
#' @param dataset1 First input dataset as an object of type inheriting \code{\linkS4class{RnBeadSet}}.
#' @param dataset2 Second input dataset as an object of type inheriting \code{\linkS4class{RnBeadSet}}.
#' @param type Type of the combine operation as a character singleton, one of "common", "all.x", "all.y" and "all".
#' @return Combined dataset as an object of type inheriting \code{\linkS4class{RnBeadSet}}.
#'
#' @details \describe{
#'   \item{Sample annotation tables}{This method expects that the sample annotation tables of the two datasets have
#'      identical structures.}
#'   \item{Genome assembly}{This method expects that the two datasets target the same genome assembly.}
#'   \item{Platform}{The platform of the combined dataset is the most recent among the platforms of the input datasets.}
#'   \item{Intensity values}{The combined dataset is of type \code{\linkS4class{RnBeadRawSet}} only when both input
#'      datasets are of this type. Otherwise, any intensity value data is ignored.}
#'   \item{Probes}{Only the common probes are included in the resulting dataset.}
#'   \item{Regions}{Regions summarized in any of the input datasets are ignored. In the resulting dataset, regions are
#'      summarized as specified in the analysis option \code{"region.types"}.}
#'   \item{Quality control data}{QC data in the input datasets is ignored. The combined dataset includes no data on QC
#'      probe intensities.}
#'   \item{Infered covariates}{Inferred covariates in the input datasets are ignored. The combined dataset includes no
#'      data on inferred covariates.}
#'   \item{Disk dumping}{The combined dataset stores big tables on disk when the analysis option
#'      \code{"disk.dump.big.matrices"} is enabled.}
#' }
#' 
#' @author Yassen Assenov
#' @export
rnb.combine.arrays <- function(dataset1, dataset2, type="common") {
	if (!inherits(dataset1, "RnBeadSet")) {
		stop("Invalid value for dataset1")
	}
	if (!inherits(dataset2, "RnBeadSet")) {
		stop("Invalid value for dataset2")
	}
	if (dataset1@assembly != dataset2@assembly) {
		stop("Incompatible assemblies")
	}
	i <- c(dataset1@target, dataset2@target)
	common.platform <- c("probesEPICv2" = "EPICv2", "probesEPIC" = "EPIC", "probes450" = "450k", "probes27" = "27k") ## TODO: Validate for EPICv2 (on EPICv1 ?)
	if (!(i[1] %in% names(common.platform))) {
		stop("Unsupported platform for dataset1")
	}
	if (!(i[2] %in% names(common.platform))) {
		stop("Unsupported platform for dataset2")
	}
    if(i[1] == "probesEPICv2") {
        rownames(dataset1@sites) <- sapply(strsplit(rownames(dataset1@sites),"_"), `[`, 1)
    }
    if(i[2] == "probesEPICv2") {
        rownames(dataset2@sites) <- sapply(strsplit(rownames(dataset2@sites),"_"), `[`, 1)
    }

	common.platform <- common.platform[intersect(names(common.platform), unique(i))]
	common.platform <- unname(common.platform[1])
	is.raw <- (inherits(dataset1, "RnBeadRawSet") && inherits(dataset2, "RnBeadRawSet"))

	## Combine sample annotation tables
	#tbl <- rnb.combine.pheno(dataset1, dataset2)
    tbl<-plyr::rbind.fill(pheno(dataset1),pheno(dataset2))
	nn <- c(nrow(dataset1@pheno), nrow(dataset2@pheno))

	## Identify common probes
    if(type == "common"){
	    common.sites <- intersect(rownames(dataset1@sites), rownames(dataset2@sites))
    }else if(type == "all.x"){
        common.sites <- rownames(dataset1@sites)
    }else if(type == "all.y"){
        common.sites <- rownames(dataset2@sites)
    }else if(type == "all"){
        common.sites <- unique(c(rownames(dataset1@sites), rownames(dataset2@sites)))
    }else{
        rnb.error("Unsupported value for type")
    }
	if (length(common.sites) == 0) {
		stop("No common sites identified")
	}
#	ii <- cbind( # probe names must be sorted in both datasets!
#		which(rownames(dataset1@sites) %in% common.sites),
#		which(rownames(dataset2@sites) %in% common.sites))
    ii <- cbind(
        match(common.sites, rownames(dataset1@sites)),
        match(common.sites, rownames(dataset2@sites))
            )
	useff <- rnb.getOption("disk.dump.big.matrices")
	
	## Combine the data matrices
	result <- list(
		"pheno" = tbl,
		"probes" = common.sites,
		"platform" = common.platform,
		"region.types" = rnb.region.types.for.analysis(dataset1@assembly),
		"qc" = NULL,
		"useff" = useff,
		"ffcleanup" = rnb.getOption("enforce.destroy.disk.dumps"))
	slot.names <- c("p.values")
	if (is.raw) {
		slot.names <- c(slot.names, "M", "U", "M0", "U0", "bead.counts.M", "bead.counts.U")
	} else {
		result[["ffcleanup"]] <- NULL
		slot.names <- c(slot.names, "betas")
	}
	for (slot.name in slot.names) {
		if (slot.name == "p.values") {
			s.name <- "pval.sites"
		} else if (slot.name == "betas") {
			s.name <- "meth.sites"
		} else {
			s.name <- slot.name
		}
		result[[slot.name]] <- rnb.combine.matrices(slot(dataset1, s.name), slot(dataset2, s.name), ii, nn, useff)
	}
	
	## Construct the resulting object
	result <- do.call(ifelse(is.raw, "RnBeadRawSet", "RnBeadSet"), result)
	rm(tbl, i, common.platform, is.raw, nn, common.sites, ii, useff, slot.names, slot.name, s.name)

	## Set normalization and background subtraction methods
	if (dataset1@status$normalized == dataset2@status$normalized) {
		result@status$normalized <- dataset1@status$normalized
	} else {
		result@status$normalized <- "none"
		warning("Inconsistent normalization methods; setting to none")
	}
	if (dataset1@status$background == dataset2@status$background) {
		result@status$background <- dataset1@status$background
	} else {
		result@status$background <- "none"
		warning("Inconsistent background subtraction methods; setting to none")
	}
	result
}
###############################################################################
#' rnb.combine.seq
#'
#' Initial implementation of the combine method for sequencing datasets.
#' 
#' @param x An object of type \code{\link{RnBiseqSet-class}} used for concatenation
#' @param y Another object of type \code{\link{RnBiseqSet-class}} used for concatenation
#' @param type A character representing the type of combination. Needs to be one of \code{"common"},
#'          \code{all.x}, \code{all.y} or \code{all}.
#' @return An \code{\link{RnBiseqSet-class}} object with combined information
#' @details The type parameters determines the mode of combination:
#'           \itemize{
#'             \item{\code{"common"}}{ The intersection between the sites present in the two 
#'                             datasets is used for the new dataset.}
#'             \item{\code{"all.x"}}{ All sites present in x are used.}
#'             \item{\code{"all.y"}}{ All sites present in y are used.}
#'             \item{\code{"all"}}{ The union between the sites of both datasets is used.}
#'           }
#'
#' @export
rnb.combine.seq<-function(x,y,type="common"){
#    if (class(x) != class(y)){
#        stop("Could not combine RnBiseqSet objects: incompatible classes")
#    }
    if (assembly(x) != assembly(y)){
        stop("Could not combine RnBiseqSet objects: incompatible assemblies")
    }
#    if (x@target != y@target){
#        stop("Could not combine RnBiseqSet objects: incompatible platforms")
#    }
    common.samples <- intersect(samples(x),samples(y))
    if (length(common.samples)>0){
        stop(paste0("Could not combine RnBSet objects: the following samples overlap in both objects: ", paste(common.samples,collapse=",")))
    }
    useff <- x@status$disk.dump
    if (x@status$disk.dump != y@status$disk.dump){
        warning(paste0("disk dump status of the two objects to combine disagree. Using disk dump: ", useff))
    }
    usebigff <- useff
    if (usebigff) usebigff <- !is.null(x@status$disk.dump.bigff)
    if (usebigff) usebigff <- x@status$disk.dump.bigff
    if(usebigff){
        bff.finalizer <- rnb.getOption("disk.dump.bigff.finalizer")
    }
    # prepare a new object
    new.set<-x
    # if(nrow(pheno(x))>=nrow(pheno(y))){
    # 	new.set<-y
    # }else{
    # 	new.set<-x
    # }
    
    new.set@pheno <- plyr::rbind.fill(pheno(x),pheno(y))
    
    # combine sites
    sites1<-x@sites
    sites2<-y@sites
    
    ## Identify common probes
    if(type == "common"){
        common.chr<-intersect(unique(sites1[,2]), unique(sites2[,2]))
    }else if(type == "all.x"){
        common.chr<-unique(sites1[,2])
    }else if(type == "all.y"){
        common.chr<-unique(sites2[,2])
    }else if(type == "all"){
        common.chr<-union(unique(sites1[,2]), unique(sites2[,2]))
    }else{
        rnb.error("Unsupported value for type")
    }
    
    subs1<-list()
    subs2<-list()
    map1<-list()
    map2<-list()
    common.sites<-list()
    
    for(chr in common.chr){
        if(type == "common"){
            sts<-sort(intersect(sites1[sites1[,2]==chr,3],sites2[sites2[,2]==chr,3]))
        }else if(type == "all.x"){
            sts<-sites1[sites1[,2]==chr,3]
        }else if(type == "all.y"){
            sts<-sites2[sites2[,2]==chr,3]
        }else if(type == "all"){
            sts<-sort(union(sites1[sites1[,2]==chr,3],sites2[sites2[,2]==chr,3]))
        }
        mtch1<-match(sites1[sites1[,2]==chr,3], sts)
        valid1<-which(!is.na(mtch1))
        map1[[chr]]<-valid1
        subs1[[chr]]<-mtch1[valid1]
        mtch2<-match(sites2[sites2[,2]==chr,3], sts)
        valid2<-which(!is.na(mtch2))
        map2[[chr]]<-valid2
        subs2[[chr]]<-mtch2[valid2]
        common.sites[[chr]]<-cbind(rep(1,length(sts)), rep(chr,length(sts)), sts)
    }
    
    total.sites<-sum(sapply(common.sites, nrow))
    
    if("ff_matrix" %in% c(class(sites1), class(sites2))){
        new.sites <- ff(vmode="integer", dim=c(total.sites,3))
        ixx<-1
        for(sts in common.sites){
            new.sites[ixx:(ixx+nrow(sts)),]<-sts
            ixx<-ixx+nrow(sts)+1
        }
    }else{
        new.sites<-do.call("rbind", common.sites)
    }
    
    colnames(new.sites)<-NULL
    new.set@sites<-new.sites
    
    slot.names<-RNBSET.SLOTNAMES
    
    if(inherits(x, "RnBeadSet")){
        slot.names<-c(slot.names, RNBEADSET.SLOTNAMES)
    }
    if(inherits(x, "RnBeadRawSet")){
        slot.names<-c(slot.names, RNBRAWSET.SLOTNAMES)
    }
    
    for(sl in slot.names){
        if(all(!is.null(slot(x,sl)),!is.null(slot(y,sl)))){
            if(useff){
                #new.matrix<-ff(vmode=vmode(slot(x,sl)), dim=c(total.sites,nrow(pheno(new.set))))
                if (usebigff){
                    new.matrix <- BigFfMat(row.n=total.sites, col.n=nrow(pheno(new.set)), vmode=vmode(slot(x,sl)), finalizer=bff.finalizer)
                } else {
                    new.matrix <- create.empty.ff.matrix.tmp(vm=vmode(slot(x,sl)), dim=c(total.sites,nrow(pheno(new.set))))
                }
            }else{
                new.matrix<-matrix(NA, nrow=total.sites, ncol=nrow(pheno(new.set)))
            }
            for(chr in common.chr){
                #new.matrix[new.sites[,2]==chr,1:nrow(pheno(x))]<-slot(x,sl)[sites1[map1[[chr]],2]==chr,][subs1[[chr]],]
                #new.matrix[new.sites[,2]==chr,(nrow(pheno(x))+1):nrow(pheno(new.set))]<-slot(y,sl)[sites2[,2]==chr,][subs2[[chr]],]
                ix<-which(new.sites[,2]==chr)
                new.matrix[ix[subs1[[chr]]],1:nrow(pheno(x))]<-slot(x,sl)[which(sites1[,2]==chr)[map1[[chr]]],,drop=FALSE]
                new.matrix[ix[subs2[[chr]]],(nrow(pheno(x))+1):nrow(pheno(new.set))]<-slot(y,sl)[which(sites2[,2]==chr)[map2[[chr]]],,drop=FALSE]
            }
            #colnames(new.matrix)<-c(colnames(slot(x,sl)), colnames(slot(y,sl)))
            slot(new.set, sl)<-new.matrix
            rm(new.matrix)
            rnb.cleanMem()
        }else{
            slot(new.set, sl)<-NULL
        }
        
        if(x@status$disk.dump && isTRUE(x@status$discard.ff.matrices)){
            delete(slot(x, sl))
        }
        if(y@status$disk.dump && isTRUE(y@status$discard.ff.matrices)){
            delete(slot(y, sl))
        }
    }
    
    if(inherits(x,"RnBeadSet")){
        if(all(!is.null(qc(x)),!is.null(qc(y)))){
            cpn<-intersect(rownames(qc(x)$Cy3), rownames(qc(x)$Cy3))
            cy3.new<-cbind(qc(x)$Cy3[cpn,], qc(y)$Cy3[cpn,])
            cy5.new<-cbind(qc(x)$Cy5[cpn,], qc(y)$Cy5[cpn,])
            colnames(cy3.new)<-NULL
            colnames(cy5.new)<-NULL
            new.set@qc<-list(Cy3=cy3.new, Cy5=cy5.new)
        }else{
            new.set@qc<-NULL
        }
    }
    
    new.set@status<-list()
    if(inherits(new.set, "RnBeadSet")){
        new.set@status$normalized<-"none"
        new.set@status$background<-"none"
    }
    new.set@status$disk.dump<-useff
    new.set@status$disk.dump.bigff<-usebigff
    
    for (region.type in union(summarized.regions(x), summarized.regions(y))) {
        if (region.type %in% rnb.region.types(assembly(new.set))) {
            new.set <- summarize.regions(new.set, region.type)
        }
    }
    new.set@inferred.covariates<-list()
    
    rm(common.sites, sites1, sites2, subs1, subs2, x, y)
    rnb.cleanMem()
    new.set
}

###############################################################################
# basic_combine
#
# Initial implementation of the combine method
#
#
basic_combine<-function(x,y){
    if (class(x) != class(y)){
        stop("Could not combine RnBSet objects: incompatible classes")
    }
    if (assembly(x) != assembly(y)){
        stop("Could not combine RnBSet objects: incompatible assemblies")
    }
    if (x@target != y@target){
        stop("Could not combine RnBSet objects: incompatible platforms")
    }
    common.samples <- intersect(samples(x),samples(y))
    if (length(common.samples)>0){
        stop(paste0("Could not combine RnBSet objects: the following samples overlap in both objects: ", paste(common.samples,collapse=",")))
    }
    useff <- x@status$disk.dump
    if (x@status$disk.dump != y@status$disk.dump){
        warning(paste0("disk dump status of the two objects to combine disagree. Using disk dump: ", useff))
    }
    usebigff <- useff
    if (usebigff) usebigff <- !is.null(x@status$disk.dump.bigff)
    if (usebigff) usebigff <- x@status$disk.dump.bigff
    if(usebigff){
        bff.finalizer <- rnb.getOption("disk.dump.bigff.finalizer")
    }
    # prepare a new object
    new.set<-x
    # if(nrow(pheno(x))>=nrow(pheno(y))){
    # 	new.set<-y
    # }else{
    # 	new.set<-x
    # }
    
    new.set@pheno <- plyr::rbind.fill(pheno(x),pheno(y))
    
    # combine sites
    sites1<-x@sites
    sites2<-y@sites
    
    common.chr<-union(unique(sites1[,2]), unique(sites2[,2]))
    
    subs1<-list()
    subs2<-list()
    common.sites<-list()
    
    for(chr in common.chr){
        sts<-sort(union(sites1[sites1[,2]==chr,3],sites2[sites2[,2]==chr,3]))
        subs1[[chr]]<-match(sites1[sites1[,2]==chr,3], sts)
        subs2[[chr]]<-match(sites2[sites2[,2]==chr,3], sts)
        common.sites[[chr]]<-cbind(rep(1,length(sts)), rep(chr,length(sts)), sts)
    }
    
    total.sites<-sum(sapply(common.sites, nrow))
    
    if("ff_matrix" %in% c(class(sites1), class(sites2))){
        new.sites <- ff(vmode="integer", dim=c(total.sites,3))
        ixx<-1
        for(sts in common.sites){
            new.sites[ixx:(ixx+nrow(sts)),]<-sts
            ixx<-ixx+nrow(sts)+1
        }
    }else{
        new.sites<-do.call("rbind", common.sites)
    }
    
    colnames(new.sites)<-NULL
    new.set@sites<-new.sites
    
    slot.names<-RNBSET.SLOTNAMES
    
    if(inherits(x, "RnBeadSet")){
        slot.names<-c(slot.names, RNBEADSET.SLOTNAMES)
    }
    if(inherits(x, "RnBeadRawSet")){
        slot.names<-c(slot.names, RNBRAWSET.SLOTNAMES)
    }
    
    for(sl in slot.names){
        if(all(!is.null(slot(x,sl)),!is.null(slot(y,sl)))){
            if(useff){
                #new.matrix<-ff(vmode=vmode(slot(x,sl)), dim=c(total.sites,nrow(pheno(new.set))))
                if (usebigff){
                    new.matrix <- BigFfMat(row.n=total.sites, col.n=nrow(pheno(new.set)), vmode=vmode(slot(x,sl)), finalizer=bff.finalizer)
                } else {
                    new.matrix <- create.empty.ff.matrix.tmp(vm=vmode(slot(x,sl)), dim=c(total.sites,nrow(pheno(new.set))))
                }
            }else{
                new.matrix<-matrix(NA, nrow=total.sites, ncol=nrow(pheno(new.set)))
            }
            for(chr in common.chr){
                #new.matrix[new.sites[,2]==chr,1:nrow(pheno(x))]<-slot(x,sl)[sites1[,2]==chr,][subs1[[chr]],]
                #new.matrix[new.sites[,2]==chr,(nrow(pheno(x))+1):nrow(pheno(new.set))]<-slot(y,sl)[sites2[,2]==chr,][subs2[[chr]],]
                ix<-which(new.sites[,2]==chr)
                new.matrix[ix[subs1[[chr]]],1:nrow(pheno(x))]<-slot(x,sl)[sites1[,2]==chr,,drop=FALSE]
                new.matrix[ix[subs2[[chr]]],(nrow(pheno(x))+1):nrow(pheno(new.set))]<-slot(y,sl)[sites2[,2]==chr,,drop=FALSE]
            }
            #colnames(new.matrix)<-c(colnames(slot(x,sl)), colnames(slot(y,sl)))
            slot(new.set, sl)<-new.matrix
            rm(new.matrix)
            rnb.cleanMem()
        }else{
            slot(new.set, sl)<-NULL
        }
        
        if(x@status$disk.dump && isTRUE(x@status$discard.ff.matrices)){
            delete(slot(x, sl))
        }
        if(y@status$disk.dump && isTRUE(y@status$discard.ff.matrices)){
            delete(slot(y, sl))
        }
    }
    
    if(inherits(x,"RnBeadSet")){
        if(all(!is.null(qc(x)),!is.null(qc(y)))){
            cpn<-intersect(rownames(qc(x)$Cy3), rownames(qc(x)$Cy3))
            cy3.new<-cbind(qc(x)$Cy3[cpn,], qc(y)$Cy3[cpn,])
            cy5.new<-cbind(qc(x)$Cy5[cpn,], qc(y)$Cy5[cpn,])
            colnames(cy3.new)<-NULL
            colnames(cy5.new)<-NULL
            new.set@qc<-list(Cy3=cy3.new, Cy5=cy5.new)
        }else{
            new.set@qc<-NULL
        }
    }
    
    new.set@status<-list()
    if(inherits(new.set, "RnBeadSet")){
        new.set@status$normalized<-"none"
        new.set@status$background<-"none"
    }
    new.set@status$disk.dump<-useff
    new.set@status$disk.dump.bigff<-usebigff
    
    for (region.type in union(summarized.regions(x), summarized.regions(y))) {
        if (region.type %in% rnb.region.types(assembly(new.set))) {
            new.set <- summarize.regions(new.set, region.type)
        }
    }
    new.set@inferred.covariates<-list()
    
    rm(common.sites, sites1, sites2, subs1, subs2, x, y)
    rnb.cleanMem()
    new.set
}

############################################################################
#' test_utils.R
#'
#' Tests for the imputation, CNV estimation and age prediction methods in RnBeads
############################################################################

#test_cnv <- function(){
#  require(RnBeads.hg19)
#  dir.report <- file.path(tempdir(),"report43/")
#  unlink(dir.report,recursive = TRUE)
#  report <- createReport(file.path(dir.report,'index.html'),'MyReport',init.configuration=TRUE)
#  data(small.example.object)
#  report <- rnb.step.cnv(rnb.set.example,report)
#  passed <- inherits(report,"Report")
#  unlink(dir.report,recursive = TRUE)
#  checkTrue(passed)
#}

test_imputation <- function(){
  require(RnBeads.hg19)
  data(small.example.object)
  methData <- meth(rnb.set.example)
  methData <- apply(methData,2,function(x){
    x[sample(1:length(x),42)] <- NA
    x
  })
  rnb.set.example@meth.sites <- methData
  rnb.set.knn <- rnb.execute.imputation(rnb.set.example,method = "knn")
  rnb.set.samples <- rnb.execute.imputation(rnb.set.example,method = "mean.samples")
  rnb.set.cpgs <- rnb.execute.imputation(rnb.set.example,method = "mean.cpgs")
  rnb.set.random <- rnb.execute.imputation(rnb.set.example,method = "random")
  passed <- !(any(is.na(meth(rnb.set.knn)))) && !(any(is.na(meth(rnb.set.samples)))) && !(any(is.na(meth(rnb.set.cpgs)))) && !(any(is.na(meth(rnb.set.random))))
  checkTrue(passed)
}

test_age.prediction <- function(){
  require(RnBeads.hg19)
  data(small.example.object)
  rnb.set.example <- rnb.execute.age.prediction(rnb.set.example)
  ph <- pheno(rnb.set.example)
  passed <- is.element("predicted_ages",colnames(ph)) && all(is.numeric(ph$predicted_ages))
  checkTrue(passed)
}

test_utils <- function(){
  logger.start("Testing imputation")
  test_imputation()
  logger.completed()
  logger.start("Testing age prediction")
  test_age.prediction()
  logger.completed()
#  logger.start("Testing CNV computation")
#  test_cnv()
#  logger.completed()
}

test_utils()

########################################################################################################################
## loading.R
## created: 30-05-2012
## creator: Pavlo Lutsik
## ---------------------------------------------------------------------------------------------------------------------
## High-level functions handling the loading step of the data import module.
########################################################################################################################

## F U N C T I O N S ###################################################################################################

#' rnb.execute.import
#'
#' Loads the data from the specified type and encapsulates it in either an \code{\linkS4class{RnBSet}}-inheriting object
#'
#' @param data.source 		non-empty \code{character} vector or \code{list} specifying the location of the data items. The
#'                    		expected format depends on the \code{data.type} that is given. See the \emph{Details} section.
#' @param data.type   		type of the input data; must be one of \code{"idat.dir"}, \code{"data.dir"}, \code{"data.files"},
#'                    		\code{"GS.report"}, \code{"GEO"} or \code{"rnb.set"}.
#' @param dry.run			if \code{TRUE} and \code{data.type} is \code{"bs.bed.dir"}, only a test data import is performed and first 10,000 lines are read from each BED file
#' @param verbose			flag specifying whether diagnostic output should be written to the console or to the RnBeads logger
#' 							in case the latter is initialized
#'
#' @return Loaded data as an object of type \code{\linkS4class{RnBSet}} (when the input data type is
#'         \code{"data.dir"}, \code{"data.files"} or \code{"GEO"}) or of type \code{\linkS4class{MethyLumiSet}} (when
#'         the data type is \code{"idat.dir"} or \code{"GS.report"}).
#'
#' @details The interpretation of \code{data.source} depends on the value of \code{data.type} and is summarized in the
#' following table:
#' \tabular{lccl}{
#'   \bold{\code{data.type}} \tab \bold{Type of \code{data.source}} \tab \bold{Maximal length of \code{data.source}} \tab
#'     \bold{Interpretation}\cr
#'   \code{"infinium.idat.dir"}  \tab \code{list} or \code{character} \tab \code{2}       \tab
#'     (1) Directory containing IDAT files; (2) a sample annotation table as a \code{data.frame} or the name of the corresponding file\cr
#'   \code{"infinium.data.dir"}  \tab \code{character} \tab \code{1} \tab
#'    Directory containing data tables in plain text format. The directory should contain one file with \code{Sample|sample} token in the filename
#' for the table of sample annotations, and one file with a token \code{beta} in the filename, with beta-values. It may also contain tables with p-values
#' (token \code{pval}) and bead counts (\code{bead}). In the latter case beta-value, p-value and bead count tables should have matching columns and rows.
#' The beta-value, p-value and bead-count tables should contain row names, i.e. the first column should contain the Infinium CG identifiers and not have
#' a column header (for that the first row should have one entry less than all other rows). Sample annotation table should contain as many rows as there are
#' columns in other tables. The character used as value separator in the text tables can be set using the \code{import.table.separator} option
#' (see \code{\link{rnb.options}} for details). \cr
#' 	 \code{"infinium.data.files"} \tab \code{character} \tab \code{2..4} \tab
#' 	The character vector should contain at least full paths to the sample annotation file and beta-value table. Detection p-values and bead counts table
#'  can be added as the third and the fourth elements. The table format requirements are the same as for \code{"data.dir"} above.\cr
#'   \code{"infinium.GS.report"} \tab \code{character}                \tab \code{1}       \tab
#'     Genome Studio report file\cr
#'   \code{"infinium.GEO"}       \tab \code{character}                \tab \code{1}       \tab
#'     \href{http://www.ncbi.nlm.nih.gov/geo/}{GEO} identifier or downloaded series matrix file\cr
#'   \code{"bs.bed.dir"}       \tab \code{list} or \code{character}                \tab \code{1..3}       \tab
#'     (1) Directory with BED files each giving a DNA methylation profile of a sample; (2) a sample annotation table as a \code{data.frame} or the name of the corresponding file;
#' 		(3) number of the sample annotation sheet column containing the file names. One of the first two elements have to be present. In case only the directory is specified,
#' 		it should contain a sample annotation file with a token "sample" in the file name. In case only the sample sheet is specified, one column should be giving full absolute paths
#' 		of the BED-like files with sequencing information. If both elements (1) and (2) are specified, the files should reside in the directory, specified as element (1).
#' 		If the third element is absent, an attempt will be made to find the file name containing column automatically. For this reason the file names in the sample annotation sheet
#' 		should be given with extensions (".bed", ".cov" etc).\cr
#'  \code{"rnb.set"}       \tab \code{\linkS4class{RnBSet}}                \tab \code{1}       \tab
#'     object of class inheriting from \code{\linkS4class{RnBSet}}\cr
#' }
#'
#' @examples
#' \donttest{
#' # Directory where your data is located
#' data.dir <- "~/RnBeads/data/Ziller2011_PLoSGen_450K"
#' idat.dir <- file.path(data.dir, "idat")
#' sample.annotation <- file.path(data.dir, "sample_annotation.csv")
#' data.source <- c(idat.dir, sample.annotation)
#' rnb.set <- rnb.execute.import(data.source = data.source, data.type = "idat.dir")
#' }
#' @seealso \code{\link{read.data.dir}}, \code{\link{read.idat.files}}, \code{\link{read.GS.report}},
#'          \code{\link{rnb.read.geo}}, \code{\link{read.bed.files}}
#' #'
#' @author Pavlo Lutsik
#' @export
rnb.execute.import<-function(data.source, data.type=rnb.getOption("import.default.data.type"),
		dry.run=FALSE, verbose=TRUE) {

	if (!((is.character(data.source) || is.list(data.source) || inherits(data.source, "RnBSet")) && length(data.source) != 0)) {
		stop("invalid value for data.source; expected list or character")
	}
	if (!(is.character(data.type) && length(data.type) == 1 && (!is.na(data.type)))) {
		stop("invalid value for data.type; expected one-element character")
	}
	if (!(data.type %in% c("data.dir", "data.files", "idat.dir", "GS.report", "GEO", "bed.dir", "rnb.set", "rnb.set.dir",
						"infinium.data.dir", "infinium.data.files", "infinium.idat.dir", "infinium.GS.report", "infinium.GEO",
						"bs.bed.dir","bs.data.dir","bs.data.files","rnb.set"))) {
		stop("invalid value for data.type; expected one of data.dir, idat.dir, GS.report, GEO, bed.dir or rnb.set")
	}

#	if(data.type %in% c("GS.report", "infinium.GS.report")){
#		if(!suppressPackageStartupMessages(require("IlluminaHumanMethylation450k.db")))
#			stop("Missing required package IlluminaHumanMethylation450k.db")
#	}

	if(data.type %in% c("data.dir", "infinium.data.dir", "bs.data.dir")) {

		if (!(is.character(data.source) && length(data.source) == 1 && (!any(is.na(data.source))))) {
			stop("invalid value for data.source; expected character of length 1")
		}

		if(!file.exists(data.source) || !file.info(data.source)[1,"isdir"]){
			rnb.error("invalid data.source parameter, data directory not found or is a file")
		}

		result <- read.data.dir(dir=data.source, sep=rnb.getOption("import.table.separator"))

    }else if(data.type %in% c("data.files", "infinium.data.files", "bs.data.files")){

		if (!(is.character(data.source) && length(data.source) %in% 2L:4L && (!any(is.na(data.source))))) {
			stop("invalid value for data.source; expected character of length from 2 to 4")
		}

		sapply(data.source, function(dsf){
			if(!file.exists(dsf) || file.info(dsf)[1,"isdir"]){
				msg<-sprintf("invalid data.source parameter, file %s not found, or is a directory", dsf)
				rnb.error(msg)
			}
		})
		names(data.source) <- c("pheno", "betas", "p.values", "bead.counts")[1:length(data.source)]
		result <- do.call(read.data.dir, c(as.list(data.source), list(verbose=verbose)))

	} else if(data.type %in% c("idat.dir", "infinium.idat.dir")){

		## TODO: Fix this once read.idat.files can also work with one parameter only
	    if(!(is.character(data.source) || is.list(data.source)) && !length(data.source) %in% 1L:2L) {
			stop("invalid value for data.source; expected list or character of length 1 to 2")
		}
		data.source <- as.list(data.source)

		if(!file.exists(data.source[[1]]) || !file.info(data.source[[1]])[1,"isdir"]){
			rnb.error("invalid data.source parameter, idat.dir is not found, or is not directory")
		}

		if(length(data.source)==1L){
			result <- read.idat.files(base.dir=data.source[[1]], useff=rnb.getOption("disk.dump.big.matrices"), dpval.method=rnb.getOption("import.dpval.method"), verbose=verbose )
		}else{
			if (!is.data.frame(data.source[[2]])) {
				if(!file.exists(data.source[[2]]) || file.info(data.source[[2]])[1,"isdir"]){
					rnb.error("invalid data.source parameter, sample annotation file not found, or is a directory")
				}
			}
			result <- read.idat.files(base.dir=data.source[[1]], sample.sheet=data.source[[2]],
					useff=rnb.getOption("disk.dump.big.matrices"),  dpval.method=rnb.getOption("import.dpval.method"), verbose=verbose)
		}
		gc()
		rnb.cleanMem()

	} else if(data.type %in% c("GS.report", "infinium.GS.report")) {

		if (!((is.character(data.source) || is.list(data.source)) && length(data.source) %in% c(1L,2L) && (!is.na(data.source[1])))) {
			stop("invalid value for data.source; expected one-element character")
		}
		if(!file.exists(data.source[[1]]) || file.info(data.source)[1,"isdir"]){
			rnb.error("invalid data.source parameter, GS report file not found, or is a directory")
		}
		if(is.character(data.source) && length(data.source)==2){
			pd<-data.source[2]
		}else if(is.list(data.source) && length(data.source)==2){
			pd<-data.source[[2]]
		}else{
			pd<-NULL
		}

		if(rnb.getOption("import.table.separator")!="\t"){
			rnb.warning("The table separator is not tab: this is unusual for Genome Studio reports ")
		}

		result <- read.GS.report(data.source[1], pd=pd, verbose=verbose)

	} else if(data.type %in% c("GEO", "infinium.GEO")) {

		if (!(is.character(data.source) && length(data.source) == 1 && isTRUE(data.source != ""))) {
			stop("invalid value for data.source; expected one-element character")
		}
		result <- rnb.read.geo(data.source)

	} else if(data.type %in% c("bed.dir", "bs.bed.dir")){

		## TODO: Fix this once read.idat.files can also work with one parameter only
		if(!(is.character(data.source) || is.list(data.source)) && !length(data.source) %in% 1L:3L) {
			stop("invalid value for data.source; expected list or character of length 1 to 3")
		}
		data.source <- as.list(data.source)

		if(length(data.source)==1L && is.null(data.source[[1]])){
			msg<-"invalid data.source parameter, data directory should be provided"
			rnb.error(msg)
		}

		if(!is.null(data.source[[1L]]) && (!file.exists(data.source[[1]]) || !file.info(data.source[[1]])[1,"isdir"])){
			rnb.error("invalid data.source parameter, bed.dir is not found, or is not directory")
		}

		if(length(data.source)==1L){
			data.source<-c(data.source, list(NULL))
		}else{
			if(is.character(data.source[[2]])){
				if(!file.exists(data.source[[2]]) || file.info(data.source[[2]])[1,"isdir"]){
					rnb.error("invalid data.source parameter, sample annotation file is not found, or is a directory")
				}
			}
		}

		msg<-"No column with file names specified: will try to find one"
		if(length(data.source)<3L){
			filename.column=NA
			if(verbose){
				rnb.info(msg)
			}
		}else{
			# if(is.integer(data.source[[3L]])){
			filename.column=data.source[[3L]]
			# }else{
			# 	filename.column=NA
			# 	if(verbose){
			# 		rnb.info(msg)
			# 	}
			# }
		}

		if(dry.run){
			nrows=10000L
		}else{
			nrows=-1L
		}

		if (rnb.getOption("import.bed.style")=="EPP"){
			result <- read.bed.files(base.dir=data.source[[1]], sample.sheet=data.source[[2]], file.names.col=filename.column,
					verbose=verbose,
					skip.lines=0,
					pos.coord.shift=0,
					is.epp.style=TRUE,
                    mean.meth.col=NA,
                    coverage.col=NA,
					nrows=nrows)
		} else if (rnb.getOption("import.bed.style")=="Encode"){
			result <- read.bed.files(base.dir=data.source[[1]], sample.sheet=data.source[[2]], file.names.col=filename.column,
					verbose=verbose,
					chr.col=rnb.getOption("import.bed.columns")["chr"],
					start.col=rnb.getOption("import.bed.columns")["start"],
					end.col=rnb.getOption("import.bed.columns")["end"],
					strand.col=rnb.getOption("import.bed.columns")["strand"],
					mean.meth.col=11L,
					coverage.col=10L,
					c.col=rnb.getOption("import.bed.columns")["c"],
					t.col=rnb.getOption("import.bed.columns")["t"],
					nrows=nrows)
		} else if (rnb.getOption("import.bed.style")=="bismarkCytosine"){
			result <- read.bed.files(base.dir=data.source[[1]], sample.sheet=data.source[[2]], file.names.col=filename.column,
					verbose=verbose,
					pos.coord.shift=1L,
					skip.lines=0,
					chr.col=1L,
					start.col=2L,
					end.col=NA,
					c.col=4L,
					t.col=5L,
					strand.col=3L,
					mean.meth.col=NA,
					coverage.col=NA,
					coord.shift = 0L,
					nrows=nrows)
		} else if (rnb.getOption("import.bed.style")=="bismarkCov"){
			result <- read.bed.files(base.dir=data.source[[1]], sample.sheet=data.source[[2]], file.names.col=filename.column,
					verbose=verbose,
					skip.lines=0,
					chr.col=1L,
					start.col=2L,
					end.col=NA,
					c.col=5L,
					t.col=6L,
					strand.col=NA,
					mean.meth.col=NA,
					coverage.col=NA,
					coord.shift = 0L,
					nrows=nrows)
		}else if(rnb.getOption("import.bed.style")=="BisSNP"){
			result<-read.bed.files(base.dir=data.source[[1]], sample.sheet=data.source[[2]], file.names.col=filename.column,
					verbose=verbose,
					pos.coord.shift=1L,
					skip.lines=1L,
					chr.col=1L,
					start.col=2L,
					end.col=NA,
					c.col=NA,
					t.col=NA,
					strand.col=6L,
					mean.meth.col=4L,
					coverage.col=5L,
					coord.shift=0L,
					nrows=nrows)
		}else {
			skip.lines <- 1
			pos.coord.shift <- 1
			result <- read.bed.files(base.dir=data.source[[1]], sample.sheet=data.source[[2]], file.names.col=filename.column,
					verbose=verbose,
					pos.coord.shift=pos.coord.shift,
					skip.lines=skip.lines,
					chr.col=rnb.getOption("import.bed.columns")["chr"],
					start.col=rnb.getOption("import.bed.columns")["start"],
					end.col=rnb.getOption("import.bed.columns")["end"],
					strand.col=rnb.getOption("import.bed.columns")["strand"],
					mean.meth.col=rnb.getOption("import.bed.columns")["meth"],
					coverage.col=rnb.getOption("import.bed.columns")["coverage"],
					c.col=rnb.getOption("import.bed.columns")["c"],
					t.col=rnb.getOption("import.bed.columns")["t"],
					coord.shift=rnb.getOption("import.bed.frame.shift"),
					nrows=nrows)
		}

	} else if (data.type == "rnb.set") {

		if (!inherits(data.source, "RnBSet")) {
			stop("invalid value for data.source: expected object of type RnBSet")
		}
		result <- data.source

	} else if (data.type == "rnb.set.dir") {

		data.source <- as.list(data.source)
		if(length(data.source) != 1 || !file.exists(data.source[[1]])){
			rnb.error("invalid data.source parameter, rnb.set.dir not found")
		}
		result <- load.rnb.set(data.source[[1L]])

	} else {
		stop("invalid value for data.type; expected one of data.dir, idat.dir, GS.report, GEO or rnb.set")
	}

	if(inherits(result, "MethyLumiSet")){
		result<-as(result, "RnBeadRawSet")
	}

	## TODO: Also extract the useful information from data.type and data.source, and package it in a suitable way to be
	##       passed to rnb.section.import
	return(result)
}

#######################################################################################################################

#' rnb.section.import
#'
#' A data loading section of the analysis report
#'
#' @param report    Report to contain the section. This must be an object of type \code{\linkS4class{Report}}.
#' @param object ...
#' @param data.type a characted vector of length one specifying the type of the input data: one of the \code{"data.dir"}, \code{"idat.dir"},\code{"GS.report"}
#' @param data.source a character vector specifying the location of the data items on disk. The expected length of the vector differs for different values of \code{data.type}:
#'         \describe{
#'           \item{\code{"data.dir"}}{\code{1<=length(data.source)<=4}}
#'           \item{\code{"infinium.idat.dir"}}{\code{length(data.source)=2}}
#'           \item{\code{"infinium.GS.report"}}{\code{length(data.source)=1}}
#'           \item{\code{"GEO"}}{\code{length(data.source)=1}}
#'         }
#' @return The modified report.
#'
#' @author Pavlo Lutsik
#' @noRd
rnb.section.import<-function(report, object, data.source, data.type=rnb.getOption("import.default.data.type")){
	## TODO: Update the method and/or its documentation
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	if (!(inherits(object, "MethyLumiSet") || inherits(object, "RnBSet"))) {
		stop("invalid value for object; expected MethyLumiSet or RnBSet")
	}
	if (!(is.character(data.source) || is.list(data.source) || inherits(data.source, "RnBSet"))) {
		stop("invalid value for data.source; expected list or character")
	}
	if (!(is.character(data.type) && length(data.type) == 1 && (!is.na(data.type)))) {
		stop("invalid value for data.type; expected one-element character")
	}
	if (!(data.type %in% c("data.dir", "data.files", "idat.dir", "GS.report", "GEO", "bed.dir", "rnb.set", "rnb.set.dir",
						"infinium.data.dir", "infinium.data.files", "infinium.idat.dir", "infinium.GS.report", "infinium.GEO",
						"bs.bed.dir","bs.data.dir","bs.data.files","rnb.set"))) {
		stop("invalid value for data.type; expected one of data.dir, idat.dir, GS.report, GEO, bed.dir or rnb.set")
	}

	pvals.present <- FALSE
	bead.counts.present <- FALSE
	pheno.table <- NULL

	if (inherits(object,"RnBSet")){
		num.samples <- length(samples(object))
		num.probes <- nsites(object)
		if(inherits(object,"RnBeadSet"))
			pvals.present <- !is.null(dpval(object)) else pvals.present<-FALSE
		bead.counts.present <- hasCovg(object)
		pheno.table <- pheno(object)
	} else { # class(object) == "MethyLumiSet"
		num.samples<-dim(phenoData(object))[1]
		num.probes<-dim(betas(object))[1]
		pvals.present<-!is.null(pvals(object))
#		bead.counts.present<-!is.null(assayData(object))
		pheno.table <- as(phenoData(object), "data.frame")
	}

	report <- rnb.add.section(report, "Loading", "This section presents a summary of the data import step.")

	if(data.type %in% c("data.dir","infinium.data.dir","bs.data.dir")) dtype="data directory"
	if(data.type %in% c("idat.dir", "infinium.idat.dir")) dtype=".idat files"
	if(data.type %in% c("GS.report", "infinium.GS.report")) dtype="GenomeStudio report"
	if(data.type %in% c("GEO", "infinium.GEO")) dtype="GEO data set"
	if(data.type %in% c("bed.dir", "bs.bed.dir")) dtype="Bisulfite sequencing data set"
	if(data.type %in% c("rnb.set", "rnb.set.dir")) dtype="RnBead Set object"
	descr<-c(
		   "Data type",
		   if (dtype == ".idat files"){
			"Infinium platform"
		   },
		   "Annotation package",
	       "Data source",
		   if(class(object)=="RnBiseqSet") "Genome assembly used" else NULL,
           "Number of samples",
           sprintf("Number of %s",rnb.get.row.token(object, plural=TRUE)),
           "Detection p-values",
		   sprintf("%s information is", rnb.get.covg.token(object)),
           if(inherits(object, "RnBeadSet")) "Quality control information" else NULL)

	vals<-c(dtype,
			if (dtype == ".idat files"){
				platform <- rnb.detect.infinium.platform(list.files(path = data.source[[1]], full.names = TRUE, pattern = 'idat'))
				txt <- c("probes27"="HumanMethylation27",
				"probes450"="HumanMethylation450",
				"probesEPIC"="MethylationEPIC",
				"probesEPICv2"="MethylationEPICv2",
				"probesMMBC"="MouseMethylationBeadChip")
				txt[platform]
		   },
		   paste0("RnBeads.", rnb.getOption("assembly")),
		if(is.list(data.source)){ 
					if(is.null(data.source[[1]])){
						NA_character_
					}else{
						as.character(data.source[[1]])
					} 
				} else if(is.character(data.source)){ 
					data.source[1]
				} else if(is.null(data.source)) {
					NA_character_
				} else {
					deparse(substitute(data.source))
				},
		if(class(object)=="RnBiseqSet") assembly(object) else NULL,
		num.samples,
		num.probes,
		if(pvals.present) "present" else "absent",
		if(bead.counts.present) "present" else "absent",
		if(inherits(object, "RnBeadSet")){if(class(object)=="MethyLumiSet") "present" else "absent"} else NULL)
	table.df<-data.frame(Descr=as.character(descr), Vals=as.character(vals))
	colnames(table.df)<-NULL
	rnb.add.table(report, table.df, row.names=FALSE, first.col.header=TRUE)

	## Study the traits in the sample annotation table
	get.trait.type <- function(x, any.missing = any(is.na(x))) {
		if (length(unique(x)) == 1) {
			if (all(is.na(x))) {
				return("all missing")
			}
			return("all identical")
		}
		if (is.logical(x)) {
			return("yes/no")
		}
		ids <- (any.missing == FALSE) && (anyDuplicated(x) == 0)
		if (is.numeric(x) || is.integer(x)) {
			return(paste0("numbers", ifelse(ids, " or identifiers", "")))
		}
		# is.factor(x) || is.character(x)
		return(paste(ifelse(ids, "identifiers", "categories"), "or text"))
	}
	report <- rnb.add.section(report, "Sample Annotation", NULL, level = 2)
	if (is.null(pheno.table) || (nrow(pheno.table) * ncol(pheno.table) == 0)) {
		rnb.add.paragraph(report, "The loaded dataset does not include sample annotation information.")
	} else {
		txt <- "The loaded dataset includes a table of sample annotations describing the following traits:"
		rnb.add.paragraph(report, txt)
		traits.summary <- data.frame(
			"No." = 1:ncol(pheno.table),
			"Trait" = colnames(pheno.table),
			"Missing Values" = as.character(sapply(pheno.table, function(x) { sum(is.na(x)) })),
			"Type" = as.character(NA), check.names = FALSE, stringsAsFactors = FALSE)
		traits.summary[traits.summary[["Missing Values"]] == "0", "Missing Values"] <- ""
		traits.summary[, "Type"] <- sapply(pheno.table, get.trait.type)
		colnames(traits.summary)[3] <- paste("<span title=\"",
			"number of samples that are not assigned a value for the respective trait",
			"\">Missing Values</span>", sep = "")
		rnb.add.table(report, traits.summary, row.names = FALSE)

		## Export the annotation table
		fname <- "annotation.csv"
		fname <- rnb.write.table(pheno.table, fname, fpath = rnb.get.directory(report, "data", absolute = TRUE),
			format = "csv", gz = FALSE, row.names = FALSE)
		txt <- c("The full sample annotation table is available as a <a href=\"", rnb.get.directory(report, "data"), "/",
			fname, "\">", "comma-separated value file</a> accompanying this report.")
		rnb.add.paragraph(report, txt)
	}

	return(report)
}
########################################################################################################################
#' rnb.step.import
#'
#' Loads the data of the specified type and creates the corresponding report section.
#'
#' @param data.source Non-empty \code{character} vector or \code{list} specifying the location of the data items. The
#'                    expected format depends on the \code{data.type} that is given. See the documentation of
#'                    \code{\link{rnb.execute.import}} for more details.
#' @param data.type   Type of the input data; must be one of \code{"data.dir"}, \code{"data.files"}, \code{"infinium.idat.dir"}, \code{"infinium.GS.report"},
#'                    \code{"infinium.GEO"} or \code{"rnb.set"}.
#' @param report      Report on to contain the Loading section. This must be an object of type
#'                    \code{\linkS4class{Report}}.
#'
#' @return a list with two elements: \describe{
#'
#'                        \item{\code{object}}{Loaded data as \code{\linkS4class{RnBSet}} (in case \code{data.type="data.dir"}) or \code{\linkS4class{MethyLumiSet}} (in case \code{data.type=c("idat.dir", "GS.report")}}
#'                        \item{\code{report}}{the modified report}
#'
#' }
#' @author Pavlo Lutsik
#' @noRd
rnb.step.import <- function(data.source, data.type = rnb.getOption("import.default.data.type"), report) {
	if (!(is.character(data.source) || is.list(data.source) || inherits(data.source, "RnBSet"))) {
		stop("invalid value for data.source; expected list or character")
	}
	if (!(is.character(data.type) && length(data.type) == 1 && (!is.na(data.type)))) {
		stop("invalid value for data.type; expected one-element character")
	}
	if (!(data.type %in% c("data.dir", "idat.dir", "GS.report", "GEO", "bed.dir", "rnb.set", "rnb.set.dir",
						"infinium.data.dir", "infinium.data.files","infinium.idat.dir", "infinium.GS.report", "infinium.GEO",
						"bs.bed.dir","bs.data.dir","bs.data.files","rnb.set"))) {

		stop("invalid value for data.type; expected one of data.dir, idat.dir, GS.report, GEO, bed.dir or rnb.set")
	}
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}

	if (rnb.getOption("logging") && logger.isinitialized() == FALSE) {
		logger.start(fname = NA) # initialize console logger
	}

	logger.info(sprintf("Loading data of type \"%s\"",data.type))

	## Load the data into an object
	if(data.type %in% c("bed.dir", "bs.bed.dir") && rnb.getOption("import.bed.test")){
		logger.start("Performing loading test")
		logger.info("The first 10000 rows will be read from each data file")
		object <- rnb.execute.import(data.source, data.type, dry.run=TRUE)
		logger.start("Checking the loaded object")
		valid<-check.rnb.biseq.set(object, verbose=TRUE)
		if(valid){
			logger.info("The object loaded during the loading test is valid")
		}else{
			logger.warning(c("The object loaded during the loading test contains invalid information (see details above).",
							"Please check the whether the data source arguments as well as the data import options, like table separator, BED style or BED column assignment, are set correctly"))
		}
		logger.completed()
		rnb.cleanMem()
		logger.completed()
	}

	if(!data.type %in% c("bed.dir", "bs.bed.dir") || !rnb.getOption("import.bed.test.only")){
		object <- rnb.execute.import(data.source, data.type)
		rnb.cleanMem()
		if(data.type=="bed.dir" && !rnb.getOption("import.skip.object.check")){
			logger.start("Checking the loaded object")
			valid <- check.rnb.biseq.set(object, verbose=TRUE)
			if(valid){
				logger.info("The loaded object is valid")
			}else{
				logger.warning(c("The loaded object contains invalid information (see details above).",
								"Please check the whether the data source arguments as well as the data import options, like table separator, BED style or BED column assignment, are set correctly"))
			}
			rnb.cleanMem()
			logger.completed()
		}
	}

	if (inherits(data.source, "RnBSet")) {
		d.source <- paste("object of type", class(data.source))
	} else { # inherits(object, "")
		d.source <- as.character(data.source[1])
	}
	logger.status(c("Loaded data from", d.source))

	## Perform sex prediction
	if (rnb.getOption("import.sex.prediction") && inherits(object, c("RnBeadRawSet","RnBiseqSet")) && object@target %in% c("probes450","probesEPIC","probesEPICv2","CpG")) {
		object <- rnb.execute.sex.prediction(object)
		if (is.null(object@inferred.covariates$sex)) {
			object@inferred.covariates$sex <- FALSE
		} else {
			logger.status("Predicted sex for the loaded samples")
		}
	}

	## Create a section in the report
	report <- rnb.section.import(report, object, data.source, data.type)
	logger.status("Added data loading section to the report")
	if (inherits(object, "RnBSet")) {
		d.source <- paste("object of type", class(data.source))
		nsamples <- length(samples(object))
		nsites <- nsites(object)
	} else { # inherits(object, "MethyLumiSet")
		nsamples <- dim(phenoData(object))[1]
		nprobes <- dim(betas(object))[1]
	}
	logger.status(c("Loaded", nsamples, "samples and", nsites, "sites"))
	logger.info(c("Output object is of type", class(object)))

	return(list(rnb.set=object, report=report))
}

#######################################################################################
##		agePrediction.R							     ##
##	This file contains all the relevant functions for the		 	     ##
##	age prediction section of RnBeads.					     ##
#######################################################################################

#######################################################################################################################
#' rnb.section.ageprediction
#'
#' Adds a section for the age prediction part to the report. In this function, we check the options
#' \bold{\code{inference.age.prediction.training}} and \bold{\code{inference.age.prediction.predictor}}
#' and accordingly either the predefined predictor (in extdata), a newly trained predictor (in extdata) or a
#' formerly trained predictor that is stored in the path given by \bold{\code{inference.age.prediction.predictor}} is applied
#' to execute epigenetic age prediction.
#'
#' @param object           methylation dataset as an object of type \code{\linkS4class{RnBSet}}.
#' @param report           report in which the age prediction section should be included. This must be an object of type \code{\linkS4class{Report}}.
#' @return	(possibly modified) report.
#'
#' @author Michael Scherer
#' @noRd

rnb.section.ageprediction <- function(object,report){
	title <- "Age Prediction"
	descr <- "Plots for the visualization of predicted ages based on the DNA methylation pattern."
	report <- rnb.add.section(report,title,descr)
	predictedAges <- pheno(object)$predicted_ages
	if(is.null(predictedAges)){
		object <- rnb.execute.age.prediction(object)
		predictedAges <- pheno(object)$predicted_ages
		if(!is.null(predictedAges)){
			logger.start("Adding Age Prediction Section to Report")
			report <- rnb.section.ageprediction(object,report)
			logger.completed()
		}else{
			logger.info("Age prediction was not successful")
			prediction_path <- rnb.getOption("inference.age.prediction.predictor")
			if(!file.exists(prediction_path)){
					text <- paste("The specified predictor does not exist at",prediction_path,", please check the option <em>inference.age.prediction.predictor</em> again.")
					report <- rnb.add.paragraph(report,text)
			}
			return(report)
		}
	}
	if(!is.null(predictedAges) && sum(is.na(predictedAges))==length(samples(object))){
		txt <- "The Age prediction was not successful, therefore no age prediction section was added to the report"
		logger.info("No Age Prediction Section Added")
		report <- rnb.add.paragraph(report,txt)
		return(report)
	}

	prediction_path <- rnb.getOption("inference.age.prediction.predictor")
	if(inherits(object,"RnBeadSet")){
		header <- "A new predictor was trained for the specified data set"
		if(is.null(prediction_path) || prediction_path==""){
			if(rnb.getOption("inference.age.prediction.training")){
					txt <- "No age information was available for the column age in the annotation, therefore the predefined predictor is used in the further calculation."
					report <- rnb.add.paragraph(report,txt)
			}
			header <- "A predefined predictor was used"
			if(assembly(object)=="hg38"){
				text <- "Currently there is no predictor available for Genome Build <em>hg38</em>. Prediction will be performed on a predictor trained on a hg19-data set."
				logger.warning(text)
				report <- rnb.add.paragraph(report,text)
			}
			if(dim(annotation(object))[1] < 30000){
			  prediction_path <- system.file(file.path("extdata", "predefined_predictor_27K.csv"), package="RnBeads")
	  		}else{
	  		  prediction_path <- system.file(file.path("extdata", "predefined_predictor_450K.csv"), package="RnBeads")
			}
		}else{
			if(!rnb.getOption("inference.age.prediction.training")){
				header <- "A user-specified predictor was used"
			}
		}
	}else if(inherits(object,"RnBiseqSet")){
		header <- "A new predictor was trained for the specified data set"
		if(is.null(prediction_path) || prediction_path==""){
			if(rnb.getOption("inference.age.prediction.training")){
					txt <- "No age information was available for the column age in the annotation, therefore the predefined predictor is used in the further calculation."
					report <- rnb.add.paragraph(report,txt)
			}
		  header <- "A predefined predictor was used"
		  if(assembly(object)=="hg19"){
		    prediction_path <- system.file(file.path("extdata", "predefined_predictor_Biseq_hg19.csv"), package="RnBeads")
		  }else{
		    prediction_path <- system.file(file.path("extdata", "predefined_predictor_Biseq_hg38.csv"), package="RnBeads")
		  }
		}else{
			if(!rnb.getOption("inference.age.prediction.training")){
				header <- "A user-specified predictor was used"
			}
		}
	}
	txt <- c(header, " and is available as a <a href=\"",prediction_path,"\">","comma-separated file</a>.")
	rnb.add.paragraph(report, txt)

	add.info <- function(stitle, ffunction, txt, actualAges, predictedAges,...) {
		result <- rnb.add.section(report, stitle, txt, level = 2)
		result <- ffunction(report, object,actualAges,predictedAges,...)
		rnb.status(c("Added", stitle))
		result
	}

	ph <- pheno(object)
	age <- rnb.getOption("inference.age.column")
	if(age %in% colnames(ph)){
		actualAges <- ph[,age]
		if(!is.null(actualAges)){
			options(warn=-1)
			if(is.factor(actualAges)){
					actualAges <- as.character(actualAges)
			}
			if(is.character(actualAges)){
					actualAges <- lapply(actualAges,convert.string.ages)
					if(length(actualAges)==0 || is.null(actualAges)){
						predictedAges <- ph$predicted_ages
						report <- add.age.histogram(report,predictedAges)
						logger.warning("Could not read age column")
						text <- "The age column has a format that can not be read by RnBeads. Please check the option <em>inference.age.column</em>."
						report <- rnb.add.paragraph(report,text)
						return(report)
					}
					set.na <- function(z){
						if(length(z)==0){
							NA
						}else{
							z
						}
					}
					actualAges <- unlist(lapply(actualAges,set.na))
			}
			actualAgesNumeric <- as.numeric(actualAges)
			if(sum(is.na(actualAgesNumeric)) != length(actualAges)){
				predictedAges <- ph$predicted_ages
				txt <- "Plotting annotated ages versus predicted ages and indicating different traits with different colors and different shapes."
				report <- add.info("Comparison Plot",add.agecomparison.plot,txt,actualAgesNumeric,predictedAges)
				txt <- "Plotting differences between predicted and annotated age for each sample."
				report <- add.info("Error Plot",add.combination.plot,txt,actualAgesNumeric,predictedAges)
				#txt <- "Plotting quantiles for the difference between predicted and annotated ages."
				#report <- add.info("Quantile Plot",add.quantile.plot,txt,actualAgesNumeric,predictedAges)
				# stratification in sample groups
				txt <- "Plotting predicted ages in different sample groups."
				report <- rnb.add.section(report,"Stratification Plot",txt,level=2)
				report <- add.stratification.plot(report,actualAgesNumeric,predictedAges,sample.groups=rnb.sample.groups(object))
				rnb.status(c("Added","Stratification Plot"))
			}else{
				if(is.factor(actualAges)){
					actualAges <- as.character(actualAges)
				}
				if(is.character(actualAges)){
					actualAges <- lapply(actualAges,convert.string.ages)
					if(length(actualAges)==0 || is.null(actualAges)){
						predictedAges <- ph$predicted_ages
						report <- add.age.histogram(report,predictedAges)
						logger.warning("Could not read age column")
						text <- "The age column has a format that can not be read by RnBeads. Please check the option <em>inference.age.column</em>."
						report <- rnb.add.paragraph(report,text)
						return(report)
					}
					set.na <- function(z){
						if(length(z)==0){
							NA
						}else{
							z
						}
					}
					actualAges <- unlist(lapply(actualAges,set.na))
					actualAgesNumeric <- as.numeric(actualAges)
					predictedAges <- ph$predicted_ages
					txt <- "Plotting annotated ages versus predicted ages and indicating different traits with different colors."
					report <- add.info("Comparison Plot",add.agecomparison.plot,txt,actualAgesNumeric,predictedAges)
					#txt <- "Plotting differences between predicted ages for each sample."
					report <- add.info("Error Plot",add.combination.plot,txt,actualAgesNumeric,predictedAges)
					#txt <- "Plotting quantiles for the difference between predicted and annotated ages."
					#report <- add.info("Quantile Plot",add.quantile.plot,txt,actualAgesNumeric,predictedAges)
				}else{
					txt <- "The age annotation column of the dataset has a format that can not be read."
					predictedAges <- ph$predicted_ages
					report <- add.age.histogram(report,predictedAges)
					report <- rnb.add.paragraph(report,txt)
				}
			}
		}
	}else{
		txt <- "There was no age annotation available for the data set, therefore no inference based on age prediction could be done."
		report <- rnb.add.paragraph(report,txt)
		report <- add.age.histogram(report,predictedAges)
	}
	options(warn=1)
	return(report)
}

#######################################################################################################################
#' rnb.step.ageprediction
#'
#' This function is the starting point for the report generation of the epigenetic age prediction. It call the functions
#' \code{rnb.section.ageprediction} and, if activated, \code{rnb.run.cross.validation}.
#'
#' @param object	a \code{\linkS4class{RnBSet}} object
#' @param report	report in which the age prediction section should be created. This must be an object of
#'              	type \code{\linkS4class{Report}}.
#'
#' @return	modified report object
#'
#' @author Michael Scherer
#' @noRd

rnb.step.ageprediction <- function(object,report){

	if(!inherits(object,"RnBSet")){
		stop("Supplied object is not of the class inheriting from RnBSet")
	}
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}

	logger.start("Adding Age Prediction Section to Report")
	report <- rnb.section.ageprediction(object,report)
	logger.completed()

	if(rnb.getOption("inference.age.prediction.cv")){
		if(rnb.getOption("inference.age.prediction.training")){
			report <- run.cross.validation(object,report)
		}else{
			logger.info("Could not run Cross-Validation without Training. Please set inference.age.prediction.training.")
			txt <- "Cross-Validation cannot be performed without training of a new predictor. Please set the option <em>inference.age.prediction.training</em> and run inference again."
			report <- rnb.add.paragraph(report,txt)
		}
	}

	return(report)
}

#######################################################################################################################
#' rnb.execute.age.prediction
#'
#' Performs age prediction by either the specified predictor in the option \code{inference.age.prediction.predictor}
#' or by the corresponding predefined predictor.
#'
#' @param object  a \code{\linkS4class{RnBSet}} object for which age prediction should
#' 			be performed
#'
#' @return	modified  \code{\linkS4class{RnBSet}} object
#'
#' @author Michael Scherer
#' @export

rnb.execute.age.prediction <- function(object){
	if(!inherits(object,"RnBSet")){
		stop("Supplied object is not of the class inheriting from RnBSet")
	}
	prediction_path <- rnb.getOption("inference.age.prediction.predictor")
	if(inherits(object,"RnBSet")){
			logger.start("Performing Age Prediction")
			rnbSet <- agePredictor(object,prediction_path)
			logger.completed()
	}
	return(rnbSet)
}

#######################################################################################################################
#' rnb.execute.training
#'
#' Trains a new age predictor on the specified data set and writes it to the given path. Elastic net regression is
#' to fit the input ages to the methylation values .
#'
#' @param object	a \code{\linkS4class{RnBSet}} object on which a new predictor should be created
#' @param path		path to which the predictor should be written out
#' @param alpha     alpha parameter used in the elastic net regression
#'
#' @author Michael Scherer
#' @export

rnb.execute.training <- function(object,path="",alpha=0.8){
	if(path==""){
		stop("A path to store the predictor has to be specified.")
	}else{
		logger.start("Training new age predictor")
		age <- rnb.getOption("inference.age.column")
		if(!(age %in% colnames(pheno(object)))){
			logger.warning("No Age information available, training not successful")
		}else{
			prediction_path <- trainPredictor(object,path,alpha=0.8)
		}
		logger.completed()
	}
}

#######################################################################################################################
#' add.agecomparison.plot
#'
#' Adds a plot that compares predicted ages by the age predictor with the annotated ages of the samples. The individual
#' points are marked in different colors and shapes depending on different samples characteristics as defined by
#' \code{rnb.sample.groups}.
#'
#' @param report	report in which the corresponding plot should be integrated.
#' @param object	methylation dataset as an object of type \code{\linkS4class{RnBSet}}.
#' @param actualAges	annotated ages as found in the sample annotation sheet.
#' @param predictedAges	ages as predicted by the age prediction algorithm.
#'
#' @return	modified report with the age plots integrated.
#'
#' @author	Michael Scherer
#' @noRd

add.agecomparison.plot <- function(report, object, actualAges, predictedAges){

	descr <- "Comparing predicted ages by age predictor with annotated ages. Points that are labeled with their identifiers have a larger difference than 15 years between predicted and annotated age."

	ph <- pheno(object)
	Default <- rep(NA,dim(ph)[1])
	ph <- cbind(ph,Default=Default)
	naAges <- is.na(actualAges)
	actualAges <- actualAges[!naAges]
	predictedAges <- predictedAges[!naAges]
	notPredicted <- is.na(predictedAges)
	actualAges <- actualAges[!notPredicted]
	predictedAges <- predictedAges[!notPredicted]
	ph <- ph[!naAges,]
	ph <- ph[!notPredicted,]
	sample_names <- samples(object)
	sample_names <- sample_names[!naAges]
	sample_names <- sample_names[!notPredicted]

	traits <- names(rnb.sample.groups(object))
	# Remove non-ASCII characters
	traits <- traits[!grepl("[^ -~]",traits)]
	traits <- c("Default",traits)
	remove.age <- traits %in% rnb.getOption("inference.age.column")
	traits <- traits[!remove.age]

	report.plots <- list()
	if(length(traits)<=1){
		actualAges <- as.numeric(actualAges)
		report.plot <- createReportPlot("age_comparison_",report)
		data <- data.frame(actualAges,predictedAges,Sample=sample_names)
		diff <- abs(predictedAges-actualAges)
		plot <- ggplot(data,aes(x=actualAges,y=predictedAges),environment=environment())+geom_point()+geom_text(aes(label=ifelse(diff > 15, as.character(Sample),"")),hjust=0,vjust=0,size=3)+xlim(-1,100)+ylim(-1,100)+geom_abline(intercept=0,slope=1)+xlab("Annotated Ages")+ylab("Predicted Age")+theme(legend.title=element_blank())
		print(plot)
		report.plot <- off(report.plot)
		report.plots <- c(report.plots,report.plot)
	}else{
		report.plots.add <- foreach(i=1:length(traits),.combine=c) %dopar%{
	    trait <- traits[i]
		  report.plots.inner <- list()
			if(trait %in% colnames(ph)){
				placeholder <- ph[,trait]
				if(!is.null(placeholder)){
					placeholder <- as.factor(placeholder)
					for(secondTrait in traits){
						if(secondTrait %in% colnames(ph)){
							placeholder2 <- ph[,secondTrait]
							if(!is.null(placeholder2)){
								if(!all(is.na(placeholder)) && !all(is.na(placeholder2))){
									if((mean(nchar(as.character(placeholder)),na.rm=TRUE)<42) && (mean(nchar(as.character(placeholder2)),na.rm=TRUE)<42)){
										placeholder2 <- as.factor(placeholder2)
										trait <- gsub(" ","",trait)
										trait <- gsub("[[:punct:]]","",trait)
										trait <- gsub("[^ -~]","",trait)
										secondTrait <- gsub(" ","",secondTrait)
										secondTrait <- gsub("[[:punct:]]","",secondTrait)
										secondTrait <- gsub("[^ -~]","",secondTrait)
										report.plot <- createReportPlot(paste0("age_comparison_",trait,"_",secondTrait), report)
										if(length(placeholder)==0){
											placeholder <- rep(NA,length(actualAges))
										}
										if(length(placeholder2)==0){
											placeholder2 <- rep(NA,length(actualAges))
										}
										data <- data.frame(actualAges,predictedAges,placeholder,Sample=sample_names)
										diff <- abs(predictedAges-actualAges)
										cvalues <- rep(rnb.getOption("colors.category"),length.out=length(placeholder))
										ptvalues <- rep(rnb.getOption("points.category"),length.out=length(placeholder2))
										plot <- ggplot(data,aes(x=actualAges,y=predictedAges,group=placeholder),environment=environment())+geom_point(aes(colour=placeholder,shape=placeholder2))+geom_text(aes(label=ifelse(diff > 15, as.character(Sample),"")),hjust=0,vjust=0,size=3)+xlim(-1,100)+ylim(-1,100)+geom_abline(intercept=0,slope=1)+xlab("Annotated Ages")+ylab("Predicted Age")+scale_colour_manual(name=trait,na.value = ifelse(trait=="Default","black","#C0C0C0"), values = cvalues)+scale_shape_manual(name=secondTrait,na.value=ifelse(trait=="Default",20,1L),values=ptvalues)
										print(plot)
										report.plot <- off(report.plot)
										report.plots.inner <- c(report.plots.inner,report.plot)
									}
								}else{
placeholder2 <- as.factor(placeholder2)
										trait <- gsub(" ","",trait)
										trait <- gsub("[[:punct:]]","",trait)
										trait <- gsub("[^ -~]","",trait)
										secondTrait <- gsub(" ","",secondTrait)
										secondTrait <- gsub("[[:punct:]]","",secondTrait)
										secondTrait <- gsub("[^ -~]","",secondTrait)
										report.plot <- createReportPlot(paste0("age_comparison_",trait,"_",secondTrait), report)
										if(length(placeholder)==0){
											placeholder <- rep(NA,length(actualAges))
										}
										if(length(placeholder2)==0){
											placeholder2 <- rep(NA,length(actualAges))
										}
										data <- data.frame(actualAges,predictedAges,placeholder,Sample=sample_names)
										diff <- abs(predictedAges-actualAges)
										cvalues <- rep(rnb.getOption("colors.category"),length.out=length(placeholder))
										ptvalues <- rep(rnb.getOption("points.category"),length.out=length(placeholder2))
										plot <- ggplot(data,aes(x=actualAges,y=predictedAges,group=placeholder),environment=environment())+geom_point(aes(colour=placeholder,shape=placeholder2))+geom_text(aes(label=ifelse(diff > 15, as.character(Sample),"")),hjust=0,vjust=0,size=3)+xlim(-1,100)+ylim(-1,100)+geom_abline(intercept=0,slope=1)+xlab("Annotated Ages")+ylab("Predicted Age")+scale_colour_manual(name=trait,na.value = ifelse(trait=="Default","black","#C0C0C0"), values = cvalues)+scale_shape_manual(name=secondTrait,na.value=ifelse(trait=="Default",20,1L),values=ptvalues)
										print(plot)
										report.plot <- off(report.plot)
										report.plots.inner <- c(report.plots.inner,report.plot)
								}
							}
						}
					}
					return(report.plots.inner)
				}
			}
		}
		report.plots <- c(report.plots,report.plots.add)
	}
	traits <- gsub(" ","",traits)
	traits <- gsub("[^ -~]","",traits)
	traits <- gsub("[[:punct:]]","",traits)
	setting_names <- list("Compare first trait"=traits,"Compare second trait"=traits[])
	names(setting_names[[1]]) <- traits
	names(setting_names[[2]]) <- traits
	if(length(traits)==1){
		setting_names = list()
	}
	report <- rnb.add.figure(report, descr, report.plots,setting_names)

	return(report)
}


#######################################################################################################################
#' add.error.plot
#'
#' Adds a plot to the given report that describes the differences between predicted ages by the age prediction
#' algorithm and the ages as annotated in the sample annotation sheet(if available). Possible outliers are marked by
#' their sample names.
#'
#' @param report	report in which the corresponding plot should be integrated.
#' @param object	methylation dataset as an object of type \code{\linkS4class{RnBSet}}.
#' @param actualAges	annotated ages as found in the sample annotation sheet.
#' @param predictedAges	ages as predicted by the age prediction algorithm.
#'
#' @return	modified report with the error plot integrated.
#'
#' @author	Michael Scherer
#' @noRd


add.error.plot <- function(report, object ,actualAges ,predictedAges){

	descr <- "Differences between predicted ages and annotated ages. The mean of the difference is shown in blue, +/- two times the Standard Deviation in red. Points that are labeled with their identifiers have a higher deviation from the mean difference than four times the standard deviation."

	ph <- pheno(object)

	diff <- predictedAges-actualAges
	mean <- mean(diff,na.rm=TRUE)
	sd <- sd(diff,na.rm=TRUE)
	deviance <- mean-diff
	deviance <- abs(deviance)
	data <- data.frame(row.names(ph),diff,deviance)
	colnames(data) <- c("Sample","Value","Deviance")
	count <- length(predictedAges)
	hline_mean <- data.frame(yint=mean,Measure="Mean")
	hline_sd <- data.frame(yint=c(mean+4*sd,mean-4*sd),Measure="4*Standard Deviation")
	x_width <- count + 0.1*count
	x_width <- round(x_width,0)

	plot <- ggplot(data,aes(x=order(sort(row.names(data))),y=Value,label=Sample),environment=environment())+geom_text(aes(label=ifelse(Deviance > 4*sd, as.character(Sample),"")),hjust=0,vjust=0,size=3)+geom_point()+geom_hline(data=hline_mean,aes(yintercept=yint,linetype=Measure),color="blue",show_guide=TRUE)+geom_hline(data=hline_sd,aes(yintercept=yint,linetype=Measure),color="red",show_guide=TRUE)+xlab("Sample Number")+ylab("Difference between predicted age and annotated age")+xlim(0,x_width)
	report.plot <- createReportPlot("age_prediction_error", report)
	print(plot)
	report.plot <- off(report.plot)

	report <- rnb.add.figure(report,descr,report.plot)

	return(report)
}

#######################################################################################################################
#' add.quantile.plot
#'
#' Adds a plot to the given report object, that shows the distribution of the differences between
#' predicted and annotated ages. Point inside the 1- and 99-quantile, respectively are marked
#' by their sample identifiers.
#'
#' @param report	report in which the corresponding plot should be integrated.
#' @param object	methylation dataset as an object of type \code{\linkS4class{RnBSet}}.
#' @param actualAges	annotated ages as found in the sample annotation sheet.
#' @param predictedAges	ages as predicted by the age prediction algorithm.
#'
#' @return	modified report with the quantile plot integrated.
#'
#' @author	Michael Scherer
#' @noRd

add.quantile.plot <- function(report, object, actualAges, predictedAges){

	descr <- "Shown in red are the 1- and 99- percentiles for the difference between predicted ages and annotated ages. The black line indicates the median of this difference. The point labels are the sample identifiers given by the phenotypic table."

	ph <- pheno(object)

	diff <- predictedAges - actualAges
	data <- data.frame(Difference=diff,Sample=row.names(ph))
	q1 <- quantile(diff,0.01,na.rm=TRUE)
	q99 <- quantile(diff,0.99,na.rm=TRUE)
	density <- density(diff)
	density.frame <- data.frame(Difference=density$x,Density=density$y)
	median <- median(diff)

	plot <- ggplot(data,aes(x=Difference,y=0.0))+geom_density(aes(y=after_stat(density)))+geom_point(aes(label=Sample))+geom_text(aes(label=Sample),hjust=.5,vjust=1.5,size=3,colour='#BE1E2D')+geom_area(data=subset(density.frame,Difference>=q1 & Difference<=q99),aes(x=Difference,y=Density),fill='#75BC1C')+geom_area(data=subset(density.frame,Difference<=q1),aes(x=Difference,y=Density),fill='#BE1E2D')+geom_area(data=subset(density.frame,Difference>=q99),aes(x=Difference,y=Density),fill='#BE1E2D')+geom_vline(xintercept=median)+ylab("Density")
	report.plot <- createReportPlot("age_prediction_quantile", report)
	print(plot)
	report.plot <- off(report.plot)

	report <- rnb.add.figure(report,descr,report.plot)

	return(report)
}

#######################################################################################################################
#' add.combination.plot
#'
#' This function adds a plot that is the combination of two plots in one plot.The upper
#' panel shows the distribution and the quantiles for the differences between predicted and annotated
#' ages and the lower panel shows the individual errors for each sample.
#'
#' @param report	report in which the corresponding plot should be integrated.
#' @param object	methylation dataset as an object of type \code{\linkS4class{RnBSet}}.
#' @param actualAges	annotated ages as found in the sample annotation sheet.
#' @param predictedAges	ages as predicted by the age prediction algorithm.
#'
#' @return	modified report with the combination plot integrated.
#'
#' @author	Michael Scherer
#' @noRd

add.combination.plot <- function(report, object, actualAges,predictedAges){

	descr <- "Upper Panel: Shown is the distribution of the differences between predicted ages and annotated ages. The black line indicates the median of these differences. \n \\ Lower panel: Differences between predicted ages and annotated ages. The mean of the difference is shown as a dashed line, the 1- and 99-perecntile, respectively, as solid lines. Points that are labeled with their identifiers lay outside of the quantiles."

	ph <- pheno(object)

	actualAges <- as.numeric(actualAges)
	naAges <- is.na(actualAges)
	actualAges <- actualAges[!naAges]
	predictedAges <- predictedAges[!naAges]
	notPredicted <- is.na(predictedAges)
	actualAges <- actualAges[!notPredicted]
	ph <- ph[!naAges,]
	ph <- ph[!notPredicted,]
	sample_names <- samples(object)
	sample_names <- sample_names[!naAges]
	sample_names <- sample_names[!notPredicted]

	diff <- predictedAges - actualAges
	q1 <- quantile(diff,0.01,na.rm=TRUE)
	q99 <- quantile(diff,0.99,na.rm=TRUE)
	na <- is.na(diff)
	diff <- diff[!na]

	density <- density(diff)
	Sample <- rep(NA,length(density$x))
	Set <- rep("Density",length(density$y))
	density.frame <- data.frame(Sample,Difference=as.numeric(density$x),Density=as.numeric(density$y),Set)
	median <- median(diff)

	mean <- mean(diff,na.rm=TRUE)
	data <- data.frame(sample_names,as.numeric(diff))
	data <- cbind(data,as.numeric(row.names(data)))
	Set <- rep("Difference",length(diff))
	data <- cbind(data,Set)
	colnames(data) <- c("Sample","Difference","Density","Set")
	hline_mean <- data.frame(yint=mean,Measure="Mean")
	hline_quantiles <- data.frame(yint=c(q1,q99),Measure="1- and 99-quantiles")

	complete_data <- rbind(density.frame,data)
	cvalues <- rep(rnb.getOption("colors.category"))

	report.plots <- list()

	report.plot <- createReportPlot("combination_plot_Points",report)
	plot <- ggplot(complete_data,aes(x=Difference,y=Density,colour=Set))+geom_point()+facet_grid(Set~.,scales="free",drop=TRUE)+geom_text(aes(label=ifelse(Difference <= q1, as.character(Sample),""),hjust=.5),vjust=0,size=3,colour="black")+geom_text(aes(x=Difference,y=Density,label=ifelse(Difference >= q99, as.character(Sample),"")),hjust=.5,vjust=0,size=3,colour="black")+geom_vline(data=hline_mean,aes(xintercept=yint,linetype=Measure),show_guide=TRUE)+geom_vline(data=hline_quantiles,aes(xintercept=yint,linetype=Measure),show_guide=TRUE)+ylab("Sample Number / Density")+xlab("Difference between predicted age and annotated age")+scale_colour_manual(na.value = "#C0C0C0", values = cvalues, guide=FALSE)
	print(plot)
	report.plot <- off(report.plot)
	report.plots <- c(report.plots,report.plot)

	report.plot <- createReportPlot("combination_plot_Identifiers",report)
	plot <- ggplot(complete_data,aes(x=Difference,y=Density,colour=Set))+geom_point()+facet_grid(Set~.,scales="free",drop=TRUE)+geom_text(aes(label=as.character(Sample)),colour="black",size=3)+geom_vline(data=hline_mean,aes(xintercept=yint,linetype=Measure),show_guide=TRUE)+geom_vline(data=hline_quantiles,aes(xintercept=yint,linetype=Measure),show_guide=TRUE)+ylab("Sample Number / Density")+xlab("Difference between predicted age and annotated age")+scale_colour_manual(na.value = "#C0C0C0", values = cvalues, guide=FALSE)
	print(plot)
	report.plot <- off(report.plot)
	report.plots <- c(report.plots,report.plot)

	setting_names <- list("Sample representation"=c("Points","Identifiers"))
	names(setting_names[[1]]) <- c("Points","Identifiers")

	report <- rnb.add.figure(report,descr,report.plots,setting_names)

	return(report)
}

#######################################################################################
#' add.age.histogram
#'
#' This function creates an age histogram for the predicted ages in the case
#' where there is no age annotation available for comparison with predicted ages.
#'
#' @param report	report object to be modified.
#' @param ages		ages for which the histogram should be created.
#'
#' @return		modified report object with the age histogram.
#'
#' @author	Michael Scherer
#' @noRd

add.age.histogram <- function(report,ages){
	data <- data.frame(Age=ages)
	colors <- rnb.getOption("colors.category")
	gradient <- rnb.getOption("colors.gradient")
	plot <- ggplot(data,aes(x=Age))+geom_histogram(aes(fill=after_stat(count),y=after_stat(density)),binwidth=5) +geom_density(color=colors[2])+scale_fill_gradient(low=gradient[1],high=gradient[2],name="Count")+ylab("Density")
	report.plot <- createReportPlot("predicted_ages_histogram",report)
	print(plot)
	report.plot <- off(report.plot)

	descr <- "Age distributions of the predicted ages by the age prediction algorithm."
	report <- rnb.add.figure(report,descr,report.plot)

	return(report)
}

#######################################################################################
#' add.stratification.plot
#'
#' This function creates a plot comparing predicted ages and the difference between annotated and 
#' predicted ages in the different sample groups defined by \code{\link{rnb.sample.groups}}
#'
#' @param report	report object to be modified.
#' @param annotated.ages		annotated ages for the individuals.
#' @param predicted.ages  ages as predicted by the age prediction algorithm
#' @param sample.groups sample groups as defined by \code{\link{rnb.sample.groups}}
#'
#' @return		modified report object with the age stratification plot
#'
#' @author	Michael Scherer
#' @noRd

add.stratification.plot <- function(report,annotated.ages,predicted.ages,sample.groups){
  report.plots <- list()
  nsamples <- length(annotated.ages)
  report.plots <- foreach(c=1:length(sample.groups),.combine = c) %dopar%{
    sample.group <- sample.groups[[c]]
    cvalues <- rep(rnb.getOption("colors.category"),length.out=length(sample.group))
    cat <- rep(NA,nsamples)
    for(i in 1:length(sample.group)){
      cat[sample.group[[i]]] <- names(sample.group)[i]
    }
    trait <- gsub("[[:punct:]]","",names(sample.groups)[c])
    trait <- gsub("[^ -~]","",trait)
    trait <- gsub(" ","",trait)
    to.plot <- data.frame(Increase=predicted.ages-annotated.ages,Group=cat)
    plot <- ggplot(to.plot,aes(x=Group,y=Increase,fill=Group))+geom_boxplot()+scale_fill_manual(values=cvalues)+ylab("Predicted Age - Annotated Age")+
      theme(axis.text.x=element_text(angle=90,hjust=1))
    report.plot.1 <- createReportPlot(paste("age_stratification",trait,"increase",sep="_"),report)
    print(plot)
    report.plot.1 <- off(report.plot.1)
    to.plot <- data.frame(Predicted=predicted.ages,Group=cat)
    plot <- ggplot(to.plot,aes(x=Group,y=Predicted,fill=Group))+geom_boxplot()+scale_fill_manual(values=cvalues)+ylab("Predicted Age")+
      theme(axis.text.x=element_text(angle=90,hjust=1))
    report.plot.2 <- createReportPlot(paste("age_stratification",trait,"predicted",sep="_"),report)
    print(plot)
    report.plot.2 <- off(report.plot.2)
    to.plot <- data.frame(Annotated=annotated.ages,Group=cat)
    plot <- ggplot(to.plot,aes(x=Group,y=Annotated,fill=Group))+geom_boxplot()+scale_fill_manual(values=cvalues)+ylab("Annotated Age")+
      theme(axis.text.x=element_text(angle=90,hjust=1))
    report.plot.3 <- createReportPlot(paste("age_stratification",trait,"annotated",sep="_"),report)
    print(plot)
    report.plot.3 <- off(report.plot.3)
    return(c(report.plots,report.plot.1,report.plot.2,report.plot.3))
  }
  s.groups <- gsub("[[:punct:]]","",names(sample.groups))
  s.groups <- gsub("[^ -~]","",s.groups)
  s.groups <- gsub(" ","",s.groups)
  names(s.groups) <- s.groups
  p.types <- c("Predicted Ages - Annotated Ages","Predicted Ages","Annotates Ages")
  names(p.types) <- c("increase","predicted","annotated")
  s.names <- list(Group=s.groups,Type=p.types)
  descr <- "Predicted Ages are stratified over different sample groups."
  report <- rnb.add.figure(report,descr,unlist(report.plots),s.names)
  return(report)
}

#######################################################################################
#' age.transformation
#'
#' This function is used to transform the input ages, to account for the fact that
#' the human DNA methylation pattern changes more in the first phase of life than in the later.
#'
#' The function is directly taken from Steve Horvath's prediction algorithm
#' DNA methylation age of human tissues and cell types, Steve Horvath, Genome Biology, 2013
#'
#' @param x		input age about to be transformed
#' @param adult.age	threshold for deciding between 'fast' and 'slow' changes in DNA methylation
#'
#' @return	transformed age
#'
#' @author 	Steve Horvath
#' @noRd

age.transformation <- function(x,adult.age=20)
{
	x=(x+1)/(1+adult.age);
	y=ifelse(x<=1, log( x),x-1);
	y
}

#######################################################################################
#' age.anti.transformation
#'
#' This function the ages that were transformed by \code{age.transformation} back to actual ages.
#'
#' The function is directly taken from Steve Horvath's prediction algorithm
#' DNA methylation age of human tissues and cell types, Steve Horvath, Genome Biology, 2013
#'
#' @param x		input age about to be transformed back
#' @param adult.age	threshold for deciding between 'fast' and 'slow' changes in DNA methylation
#'
#' @return	age transformed back
#'
#' @author	Steve Horvath
#' @noRd

age.anti.transformation <- function(x,adult.age=20)
{
	ifelse(x<0, (1+adult.age)*exp(x)-1, (1+adult.age)*x+adult.age)
}

#######################################################################################
#' agePredictor
#'
#' This function is the core of the age prediction algorithm. It takes a \code{RnBSet} object
#' and loads a given predictor from a csv file to directly predict the age of the samples
#' by this predictor.
#'
#' @param rnbSet	a \code{RnBSet} object containing the relevant methylation infos.
#' @param path		path to a csv file in which a trained predictor should lay.
#'			DEFAULT:	No predictor is specified. In this case the corresponding
#'					predefined predictor is used.
#'
#' @return	modified \code{RnBSet} with two new columns in the phenotypic table (predicted_ages
#'		for the predicted epigenetic ages and age_increase for the difference between predicted
#'		and annnotated ages)
#'
#' @author	Michael Scherer
#' @noRd

agePredictor <- function(rnbSet, path=""){
	if(!is.null(path) && path != ""){
		if(!file.exists(path)){
			logger.info(c("The specified predictor does not exists at ",path))
	  		return(rnbSet)
	  	}
	}else{
		if(inherits(rnbSet,"RnBeadSet")){
			anno <- annotation(rnbSet)
			if(dim(anno)[1] > 30000){
				path <- system.file(file.path("extdata", "predefined_predictor_450K.csv"), package="RnBeads")
			}else{
				path <- system.file(file.path("extdata", "predefined_predictor_27K.csv"), package="RnBeads")
			}
		}else if(inherits(rnbSet,"RnBiseqSet")){
			if(assembly(rnbSet)=="hg19"){
				path <- system.file(file.path("extdata", "predefined_predictor_Biseq_hg19.csv"), package="RnBeads")
			}else{
				path <- system.file(file.path("extdata", "predefined_predictor_Biseq_hg38.csv"), package="RnBeads")
			}
		}
	}
	if(inherits(rnbSet,"RnBeadSet")){
		rnbSet <- agePredictorChip(rnbSet,path)
	}else if(inherits(rnbSet,"RnBiseqSet")){
		rnbSet <- agePredictorBiseq(rnbSet,path)
	}
	return(rnbSet)
}

#######################################################################################
#' agePredictorChip
#'
#' This function is the corresponding instance of the general age prediction algorithm for data
#' created by Illumina Infinium Bead Chips
#'
#' @param rnbSet	a \code{RnBeadSet} object containing the relevant methylation infos.
#' @param path		path to a csv file in which a trained predictor should exist.
#'
#' @return	modified \code{RnBeadSet} with two new columns in the phenotypic table (predicted_ages
#'		for the predicted epigenetic ages and age_increase for the difference between predicted
#'		and annnotated ages)
#'
#' @author	Michael Scherer
#' @noRd
agePredictorChip <- function(rnbSet, path){
	rnb.require("impute")
	options(warn=-1)
	coeffs <- read.csv(path)
	methData <- meth(rnbSet)
	anno <- annotation(rnbSet)
	ph <- pheno(rnbSet)
	row.names(methData) <- row.names(anno)
	intercept <- coeffs[1,2]
	coeffs <- coeffs[-1,]
	usedCpGs <- coeffs[,1]
	match <- match(sort(usedCpGs),usedCpGs)
	usedCpGs <- sort(usedCpGs)
	usedCoeffs <- coeffs[,2]
	usedCoeffs <- usedCoeffs[match]
	selectCpGs <- row.names(methData) %in% usedCpGs
	existingCpGs <- usedCpGs  %in% row.names(methData)
	selected <- methData[selectCpGs,]
	selected <- selected[sort(row.names(selected)),]
	options(warn=-1)
	dummy <- capture.output(selected <- (impute.knn(selected,colmax=1))$data)
	options(warn=1)
	selected <- t(selected)
	if(length(usedCoeffs) > dim(selected)[2]){
		usedCoeffs <- usedCoeffs[existingCpGs]
	}
	predictedAges <- intercept + selected%*%usedCoeffs
	predictedAges <- age.anti.transformation(predictedAges)
	predictedAges <- as.numeric(predictedAges)
	ages <- ph$predicted_ages
	increase <- ph$age_increase
	if(is.null(ages)){
		rnbSet <- addPheno(rnbSet,predictedAges,"predicted_ages")
	}
	if(is.null(increase) || any(is.na(increase))){
		age <- rnb.getOption("inference.age.column")
		if(age %in% colnames(ph)){
			actualAges <- ph[,age]
			nas <- is.na(actualAges)
			difference <- rep(NA,length(actualAges))
			actualAges <- actualAges[!nas]
			predictedAges <- predictedAges[!nas]
			if(!is.null(actualAges)){
				if(is.factor(actualAges)){
					actualAges <- as.character(actualAges)
				}
				if(is.character(actualAges)){
					actualAges <- unlist(lapply(actualAges,convert.string.ages))
				}
				if(length(actualAges)==0 || is.null(actualAges)){
					logger.warning("Could not read age column")
					return(rnbSet)
				}
				actualAges <- as.numeric(actualAges)
				difference[!nas] <- predictedAges-actualAges
				rnbSet <- addPheno(rnbSet,difference,"age_increase")
			}
		}
	}
	options(warn=1)
	return(rnbSet)
}

#######################################################################################
#' agePredictorBiseq
#'
#' This function is the corresponding instance of the general age prediction algorithm for data
#' created by a sequencing-based approach.
#'
#' @param rnbSet	a \code{RnBiseqSet} object containing the relevant methylation infos.
#' @param path		path to a csv file in which a trained predictor should exist.
#'
#' @return	modified \code{RnBiseqSet} with two new columns in the phenotypic table (predicted_ages
#'		for the predicted epigenetic ages and age_increase for the difference between predicted
#'		and annnotated ages)
#'
#' @author	Michael Scherer
#' @noRd

agePredictorBiseq <- function(rnbSet, path){
	rnb.require("impute")
	options(warn=-1)
	coeffs <- read.csv(path)
	methData <- meth(rnbSet)
	anno <- annotation(rnbSet)
	ph <- pheno(rnbSet)
	start <- anno$Start
	end <- anno$End
	chromosome <- anno$Chromosome
	names <- paste0(chromosome,"_",start,"_",end)
	row.names(methData) <- names
	intercept <- coeffs[1,2]
	coeffs <- coeffs[-1,]
	usedCpGs <- coeffs[,1]
	match <- match(sort(usedCpGs),usedCpGs)
	usedCpGs <- sort(usedCpGs)
	usedCoeffs <- coeffs[,2]
	usedCoeffs <- usedCoeffs[match]
	selectCpGs <- row.names(methData) %in% usedCpGs
	existingCpGs <- usedCpGs  %in% row.names(methData)
	selected <- methData[selectCpGs,]
	selected <- selected[sort(row.names(selected)),]
	options(warn=-1)
	dummy <- capture.output(selected <- (impute.knn(selected,colmax=1))$data)
	options(warn=1)
	selected <- t(selected)
	if(dim(selected)[1]==0){
		logger.info("Age prediction could not be performed; NAs introduced")
		predictedAges <- rep(NA,length(samples(rnbSet)))
	}else{
		if(length(usedCoeffs) > dim(selected)[2]){
			usedCoeffs <- usedCoeffs[existingCpGs]
		}
		na_coeffs <- is.na(usedCoeffs)
		usedCoeffs <- usedCoeffs[!na_coeffs]
		selected <- selected[,!na_coeffs]
		predictedAges <- intercept + selected%*%usedCoeffs
		predictedAges <- age.anti.transformation(predictedAges)
		predictedAges <- as.numeric(predictedAges)
	}
	ages <- ph$predicted_ages
	increase <- ph$age_increase
	if(is.null(ages)){
		rnbSet <- addPheno(rnbSet,predictedAges,"predicted_ages")
	}
	if(is.null(increase) || any(is.na(increase))){
		age <- rnb.getOption("inference.age.column")
		if(age %in% colnames(ph)){
			actualAges <- ph[,age]
			nas <- is.na(actualAges)
			difference <- rep(NA,length(actualAges))
			actualAges <- actualAges[!nas]
			predictedAges <- predictedAges[!nas]
			if(!is.null(actualAges)){
				if(is.factor(actualAges)){
					actualAges <- as.character(actualAges)
				}
				if(is.character(actualAges)){
					actualAges <- unlist(lapply(actualAges,convert.string.ages))
				}
				if(length(actualAges)==0 || is.null(actualAges)){
					logger.warning("Could not read age column")
					return(rnbSet)
				}
				actualAges <- as.numeric(actualAges)
				difference[!nas] <- predictedAges-actualAges
				rnbSet <- addPheno(rnbSet,difference,"age_increase")
			}
		}
	}
	options(warn=1)
	return(rnbSet)
}

#######################################################################################
#' trainPredictor
#'
#' This function is the starting point for training a new predictor based on the the input dataset.
#'
#' @param rnbSet	An \code{RnBSet} object containing the methylation info on which
#'			the new predictor should be trained
#' @param data.dir	directory to which the resulting predictor should be written out
#' @param alpha     alpha parameter used in the elastic net regression
#'
#' @return	absolute path to the corresponding predictor. The path is also set as the
#'		option \code{inference.age.prediction.predictor}
#'
#' @author	Michael Scherer
#' @noRd

trainPredictor <- function(rnbSet,data.dir,alpha=0.8){
	if(!file.exists(data.dir)){
		stop("The specified directory does not exist!")
	}
	age <- rnb.getOption("inference.age.column")
	if(inherits(rnbSet,"RnBeadSet")){
		if(!(age %in% colnames(pheno(rnbSet)))){
			logger.warning("No Age information available, training not successful.")
			return("")
		}
		path <- simpleGlmnet(rnbSet,file.path(data.dir,"trained_predictor.csv"),alpha=alpha)
		if(!is.null(path)&&path!=""){
			rnb.options(inference.age.prediction.predictor=path)
		}else{
			logger.warning("No new predictor created, deault predictor remains.")
		}
	}else if(inherits(rnbSet,"RnBiseqSet")){
		if(!(age %in% colnames(pheno(rnbSet)))){
			logger.warning("No Age information available, training not successful.")
			return("")
		}
		path <- simpleGlmnetBiseq(rnbSet,file.path(data.dir,"trained_predictor_Biseq.csv"),alpha=alpha)
		if(!is.null(path)&&path!=""){
			rnb.options(inference.age.prediction.predictor=path)
		}else{
			logger.warning("No new predictor created, deault predictor remains.")
		}
	}
	return(path)
}

#######################################################################################
#' simpleGlmnet
#'
#' This function fits the metylation data to the available ages by fitting
#' a generalized linear model (alpha parameter = 0.8). The lambda parameter is chosen by
#' 10-fold cross-validation. This function was created for array-based data.
#'
#' @param trainRnBSet	a \code{RnBeadSet} object containing the methylation info on which
#'			the new predictor should be trained
#' @param filePath	path to which the new predictor should be written out
#' @param alpha     alpha parameter used in the elastic net regression
#'
#' @return	absolute path to the corresponding predictor. \code{NULL} if the function was
#'		unable to create such an predictor.
#'
#' @author	Michael Scherer
#' @noRd

simpleGlmnet <- function(trainRnBSet,filePath="",alpha=0.8){
	rnb.require("glmnet")
	rnb.require("impute")
	methData <- meth(trainRnBSet)
	anno <- annotation(trainRnBSet)
	ph <- pheno(trainRnBSet)
	age <- rnb.getOption("inference.age.column")
	ages <- ph[,age]
	naAges <- is.na(ages)
	if(!is.null(ages)){
		if(is.factor(ages)){
			ages <- as.character(ages)
		}
		if(is.character(ages)){
			ages <- unlist(lapply(ages,convert.string.ages))
		}
		if(length(ages)==0){
			return(NULL)
		}
		ages <- as.numeric(ages)
		ages <- age.transformation(ages)
		methData <- methData[,!naAges]
		ages <- ages[!naAges]
		row.names(methData) <- row.names(anno)
		Xchrom <- is.element(anno$Chromosome,"chrX")
		methData <- methData[!Xchrom,]
		anno <- anno[!Xchrom,]
		Ychrom <- is.element(anno$Chromosome,"chrY")
		methData <- methData[!Ychrom,]
		anno <- anno[!Ychrom,]
		#Imputing of missing values is performed by a k-nearest-neighbors approach
		options(warn=-1)
		dummy <- capture.output(methData <- (impute.knn(methData,colmax=1))$data)
		options(warn=1)
		methData <- t(methData)
		missingAges <- is.na(ages)
		methData <- methData[!missingAges,]
		ages <- ages[!missingAges]
		cv <- cv.glmnet(methData,ages,parallel=TRUE)
		model <- glmnet(methData,ages,alpha=alpha,lambda=cv$lambda.min)
		coeffs <- as.matrix(coef(model))
		names <- row.names(coeffs)
		non_zero <- which(coeffs!=0)
		coeffs <- coeffs[non_zero]
		names <- names[non_zero]
		names(coeffs) <- names
		if(writePredictorToCsv(coeffs,filePath)){
			return(filePath)
		}
	}
	return(NULL)
}

#######################################################################################
#' simpleGlmnetBiseq
#'
#' This function fits the metylation data to the available ages by fitting
#' a generalized linear model (alpha parameter = 0.8). The lambda parameter is chosen by
#' 10-fold cross-validation. This function was created for sequencing-based data.
#'
#' @param trainRnBSet	a \code{RnBiseqSet} object containing the methylation info on which
#'			the new predictor should be trained
#' @param filePath	path to which the new predictor should be written out
#' @param alpha     alpha parameter used in the elastic net regression
#'
#' @return	absolute path to the corresponding predictor. \code{NULL} if the function was
#'		unable to create such an predictor.
#'
#' @author	Michael Scherer
#' @noRd

simpleGlmnetBiseq <- function(trainRnBSet,filePath="",alpha=0.8){
	rnb.require("glmnet")
	methData <- meth(trainRnBSet)
	anno <- annotation(trainRnBSet)
	ph <- pheno(trainRnBSet)
	age <- rnb.getOption("inference.age.column")
	ages <- ph[,age]
	naAges <- is.na(ages)
	if(!is.null(ages)){
		if(is.factor(ages)){
			ages <- as.character(ages)
		}
		if(is.character(ages)){
			ages <- unlist(lapply(ages,convert.string.ages))
		}
		if(length(ages)==0){
			return(NULL)
		}
		ages <- as.numeric(ages)
		ages <- age.transformation(ages)
		methData <- methData[,!naAges]
		ages <- ages[!naAges]
		Xchrom <- is.element(anno$Chromosome,"chrX")
		methData <- methData[!Xchrom,]
		anno <- anno[!Xchrom,]
		Ychrom <- is.element(anno$Chromosome,"chrY")
		methData <- methData[!Ychrom,]
		anno <- anno[!Ychrom,]
		start <- anno$Start
		end <- anno$End
		chromosome <- anno$Chromosome
		names <- paste0(chromosome,"_",start,"_",end)
		row.names(methData) <- names
		options(warn=-1)
		methData <- imputeBiseq(methData)
		options(warn=1)
		methData <- t(methData)
		missingAges <- is.na(ages)
		methData <- methData[!missingAges,]
		ages <- ages[!missingAges]
		cv <- cv.glmnet(methData,ages)
		model <- glmnet(methData,ages,alpha=alpha,lambda=cv$lambda.min)
		coeffs <- as.matrix(coef(model))
		names <- row.names(coeffs)
		non_zero <- which(coeffs!=0)
		coeffs <- coeffs[non_zero]
		names <- names[non_zero]
		names(coeffs) <- names
		if(writePredictorToCsv(coeffs,filePath)){
			return(filePath)
		}
	}
	return(NULL)
}

#######################################################################################
#' run.cross.validation
#'
#' This function performs 10-fold cross validation to estimate the performance of a
#' newly trained predictor. If \code{parallel.isEnabled()}, the function perfoms cross
#' validation in parallel. The function adds a table to the specified \code{report} containing
#' the result of the 10-fold cross validation.
#'
#' @param rnbSet	a \code{RnBSet} object containing the methylation info and ages on which
#'			the new predictor should be trained
#' @param report	report to which the table should be added
#' @param alpha alpha parameter used in the elastic net regression
#'
#' @return	modified report object
#'
#' @author	Michael Scherer
#' @export

run.cross.validation <- function(rnbSet,report,alpha=0.8){
	logger.start("10-fold Cross Validation")
	if(length(samples(rnbSet))<30){
		txt <- "ATTENTION: Cross-validated correlation result might be misleading, since there are less than 3 samples per fold."
		logger.warning(txt)
		report <- rnb.add.paragraph(report,txt)
	}
	txt <- "Result of the 10-fold cross validation on the given dataset. The corresponding error measures are: Correlation between predicted and annotated ages, Mean absolute deviation and Median absolute deviation"
	cvalues <- rep(rnb.getOption("colors.category"))
	descr <- "Boxplot for the two error measures Mean and Median absolute Error. Each Boxplot consists of 10 different points for each cross-validation fold, respectively."
	if(inherits(rnbSet,"RnBeadSet")){
		result <- cv.array(rnbSet,alpha = alpha)
		if(is.null(result)){
			text <- "Problem when performing age prediction."
			report <- rnb.add.paragraph(report,text)
			return(report)
		}
		report <- rnb.add.table(report,result,tcaption=txt)
		means <- result["Mean",]
		means <- t(means)
		medians <- result["Median",]
		medians <- t(medians)
		toPlot <- data.frame(Mean=means,Median=medians)
		toPlot <- reshape2::melt(toPlot,id=c())
		colnames(toPlot) <- c("Measure","Error")
		plot <- ggplot(toPlot,aes(x=Measure,y=Error,fill=Measure))+geom_boxplot()+scale_fill_manual(values=cvalues)+ylab("Error [years]")
		report.plot <- createReportPlot("cv_error_boxplot",report)
		print(plot)
		report.plot <- off(report.plot)
		report <- rnb.add.figure(report,descr,report.plot)
	}else if(inherits(rnbSet,"RnBiseqSet")){
		result <- cv.biseq(rnbSet,alpha=alpha)
		if(is.null(result)){
			text <- "Problem when performing age prediction."
			report <- rnb.add.paragraph(report,text)
			return(report)
		}
		report <- rnb.add.table(report,result,tcaption=txt)
		means <- result["Mean",]
		means <- t(means)
		medians <- result["Median",]
		medians <- t(medians)
		toPlot <- data.frame(Mean=means,Median=medians)
		toPlot <- reshape2::melt(toPlot,id=c())
		colnames(toPlot) <- c("Measure","Error")
		plot <- ggplot(toPlot,aes(x=Measure,y=Error,fill=Measure))+geom_boxplot()+scale_fill_manual(values=cvalues)+ylab("Error [years]")
		report.plot <- createReportPlot("cv_error_boxplot",report)
		print(plot)
		report.plot <- off(report.plot)
		report <- rnb.add.figure(report,descr,report.plot)
	}
	logger.completed()
	return(report)
}

######################################################################################
#' general.cv
#'
#' This functions performs k-fold-cross-validation on the predictor with methylation
#' data as input and ages to be trained on
#'
#' @param	fitFunction	a function that fits a predictor from training data to
#'				predict age from methylation data (e.g \code{simpeGlmnetEvaluate})
#' @param	ages		the ages to be trained on
#' @param	methData	input methylation matrix
#' @param	k		the fold parameter
#' @param alpha     alpha parameter used in the elastic net regression
#'
#' @return	a matrix that contains the summarized quality measures for
#'		the predictor which are:
#'			$cor[k+1]:	the mean correlation between predicted
#'					ages and actual ages
#'			$cor[1:k]:	individual correlations between predicted
#'					ages and actual ages for each fold
#'			$mean[k+1]:	the mean of the mean absolute deviation
#'					between predicted ages and actual ages
#'			$mean[1:k]:	the individual mean absolute deviations for
#'					each fold
#'			$median[k+1]:	the mean of the median absolute deviation
#'			$median[1:k]:	the individual median absolute devations
#'					for each fold
#'
#' @author	Michael Scherer
#' @noRd

general.cv <- function(fitFunction,ages,methData,k=10,alpha=0.8){
	nSamples <- length(ages)
	size <- nSamples%/%k
	count <- 1
	ret <- foreach(i=seq(1,k*size,by = size),.combine='cbind',.packages=c("RnBeads","impute","glmnet"),.export=c("age.transformation","age.anti.transformation","simpleGlmnetEvaluate")) %dopar% {
		logger.info(as.character(count))
		choose <- rep(FALSE,nSamples)
		choose[i:(i+size-1)] <- TRUE
		testSet <- methData[,choose]
		testAges <- ages[choose]
		notChosen <- !choose
		trainSet <- methData[,notChosen]
		trainAges <- ages[notChosen]
		predictor <- fitFunction(trainSet,trainAges,alpha)
		predictedAges <- predictor(testSet)
		cor <- cor(predictedAges,testAges)
		cor <- round(cor,2)
		testAges <- age.anti.transformation(testAges)
		mean <- mean(abs(predictedAges-testAges))
		mean <- round(mean,2)
		median <- median(abs(predictedAges-testAges))
		median <- round(median,2)
		column <- c(cor,mean,median)
		count <- count+1
		column
	}
	lastCol <- c(mean(ret[1,1:10]),mean(ret[2,1:10]),mean(ret[3,1:10]))
	ret <- cbind(ret,lastCol)
	colnames(ret) <- c(paste("Fold",seq(1:10)),"Mean")
	row.names(ret) <- c("Correlation","Mean","Median")
	return(ret)
}

###################################################################################
#' cv.array
#'
#' This function randomly changes the order of the samples in the data set and then
#' calls the \code{general.cv} to perform cross-validation for a RnBSet derived from
#' BeadChip arrays
#'
#' @param	rnbSet	\code{RnBeadSet} object on which the cross validation should be perfomed
#' @param alpha     alpha parameter used in the elastic net regression
#'
#' @return	the result of the cross validation in a data.frame format
#'
#' @author	Michael Scherer
#' @noRd

cv.array <- function(rnbSet,alpha=0.8){
	rnb.require("impute")
	ph <- pheno(rnbSet)
	age <- rnb.getOption("inference.age.column")
	if(age %in% colnames(ph)){
		ages <- ph[,age]
		if(is.factor(ages)){
			ages <- as.character(ages)
		}
		if(is.character(ages)){
			ages[!is.na(ages)] <- unlist(lapply(ages[!is.na(ages)],convert.string.ages))
		}
		ages <- as.numeric(ages)
		if(length(ages)==0 || is.null(ages) || sum(is.na(ages)) == length(ages)){
			logger.warning("Could not read age column")
			return(NULL)
		}
		ages <- age.transformation(ages)
		missingAges <- is.na(ages)
		ages <- ages[!missingAges]
		rnbSet <- remove.samples(rnbSet,missingAges)
		set.seed(Sys.time())
		samples <- samples(rnbSet)
		sampled <- sample(samples)
		methData <- meth(rnbSet)
		match <- match(sampled,samples)
		methData <- methData[,match]
		ages <- ages[match]
		anno <- annotation(rnbSet)
		row.names(methData) <- row.names(anno)
		Xchrom <- is.element(anno$Chromosome,"chrX")
		methData <- methData[!Xchrom,]
		anno <- anno[!Xchrom,]
		Ychrom <- is.element(anno$Chromosome,"chrY")
		methData <- methData[!Ychrom,]
		anno <- anno[!Ychrom,]
		options(warn=-1)
		dummy <- capture.output(methData <- (impute.knn(methData,colmax=1))$data)
		options(warn=1)
		rm(dummy)
		result <- general.cv(simpleGlmnetEvaluate,ages,methData,alpha=alpha)
		result <- as.data.frame(result)
		return(result)
	}
	return(NULL)
}

###################################################################################
#' cv.biseq
#'
#' This function randomly changes the order of the samples in the data set and then
#' calls the \code{general.cv} to perform cross-validation for a RnBSet derived from
#' bisulfite sequencing
#'
#' @param	rnbSet	\code{RnBiseqSet} object on which the cross validation should be perfomed
#' @param alpha     alpha parameter used in the elastic net regression
#'
#' @return	the result of the cross validation in a data.frame format
#'
#' @author	Michael Scherer
#' @noRd

cv.biseq <- function(rnbSet,alpha=0.8){
	ph <- pheno(rnbSet)
	age <- rnb.getOption("inference.age.column")
	if(age %in% colnames(ph)){
		ages <- ph[,age]
		if(is.factor(ages)){
			ages <- as.character(ages)
		}
		if(is.character(ages)){
			ages[!is.na(ages)] <- unlist(lapply(ages[!is.na(ages)],convert.string.ages))
		}
		ages <- as.numeric(ages)
		if(length(ages)==0 || is.null(ages) || sum(is.na(ages)) == length(ages)){
			logger.warning("Could not read age column")
			return(NULL)
		}
		missingAges <- is.na(ages)
		ages <- ages[!missingAges]
		rnbSet <- remove.samples(rnbSet,missingAges)
		ages <- age.transformation(ages)
		set.seed(Sys.time())
		samples <- samples(rnbSet)
		sampled <- sample(samples)
		methData <- meth(rnbSet)
		match <- match(sampled,samples)
		methData <- methData[,match]
		ages <- ages[match]
		anno <- annotation(rnbSet)
		start <- anno$Start
		end <- anno$End
		chromosome <- anno$Chromosome
		names <- paste0(chromosome,"_",start,"_",end)
		row.names(methData) <- names
		Xchrom <- is.element(anno$Chromosome,"chrX")
		methData <- methData[!Xchrom,]
		anno <- anno[!Xchrom,]
		Ychrom <- is.element(anno$Chromosome,"chrY")
		methData <- methData[!Ychrom,]
		anno <- anno[!Ychrom,]
		methData <- imputeBiseq(methData)
		result <- general.cv(simpleGlmnetEvaluate,ages,methData,alpha=alpha)
		result <- as.data.frame(result)
		return(result)
	}
	return(NULL)
}

###################################################################################
#' simpleGlmnetEvaluate
#'
#' This function is needed to perform cross-validation. In contrast to simpleGlmnet
#' it does not write the predictor to a csv-file but returns a prediction
#' function that is to be applied in each fold.
#'
#' @param	methData input methylation data as a data.frame for age prediction
#' @param	ages	  training ages
#' @param alpha   alpha parameter used in the elastic net regression
#'
#' @return	the age prediction function to be applied in each fold
#'
#' @author	Michael Scherer
#' @noRd

simpleGlmnetEvaluate <- function(methData,ages,alpha=0.8){
	methData <- t(methData)
	missingAges <- is.na(ages)
	methData <- methData[!missingAges,]
	ages <- ages[!missingAges]
	cv <- cv.glmnet(methData,ages,parallel=TRUE)
	model <- glmnet(methData,ages,alpha=alpha,lambda=cv$lambda.min)
	finalModel <- createPredictor(model,lambda=cv$lambda.min)
	return(finalModel)
}

###################################################################################
#' createPredictor
#'
#' This function is needed to perform cross-validation. It creates a prediction
#' function in contrast to writing the predictor to a csv-file.
#'
#' @param	linearModel	an output of glmnet from which the predictor should
#'			     be created
#'
#' @return	the age prediction function
#'
#' @author	Michael Scherer
#' @noRd

createPredictor <- function(linearModel,lambda=NULL){
	ret <- function(rnbSet){
		if(inherits(rnbSet,"RnBSet")){
			methData <- meth(rnbSet)
			coeffs <- as.matrix(coef(linearModel))
			intercept <- coeffs[1]
			anno <- annotation(rnbSet)
			coeffs <- coeffs[-1]
			usedCpGs <- row.names(coeffs)
			match <- match(sort(usedCpGs),usedCpGs)
			usedCpGs <- sort(usedCpGs)
			usedCoeffs <- coeffs
			usedCoeffs <- usedCoeffs[match]
			selectCpGs <- row.names(methData) %in% usedCpGs
			existingCpGs <- usedCpGs  %in% row.names(methData)
			selected <- methData[selectCpGs,]
			selected <- selected[sort(row.names(selected)),]
			options(warn=-1)
			dummy <- capture.output(selected <- (impute.knn(selected,colmax=1))$data)
			options(warn=1)
			selected <- t(selected)
			if(length(usedCoeffs) > dim(selected)[2]){
				usedCoeffs <- usedCoeffs[existingCpGs]
			}
			predictedAges <- intercept + selected%*%usedCoeffs
			predictedAges <- age.anti.transformation(predictedAges)
			return(predictedAges)
		}else{
			methData <- rnbSet
			methData <- t(methData)
			methData <- as.matrix(methData)
			predictedAges <- predict(linearModel,methData,s=lambda)
			predictedAges <- as.vector(predictedAges)
			predictedAges <- age.anti.transformation(predictedAges)
			return(predictedAges)
		}
	}
	return(ret)
}


#######################################################################################
#' writePredictorToCsv
#'
#' This function writes the coefficients of a linear model into a csv file.
#'
#' @param linearModel	linear Model which should be stored as a csv file
#' @param path		path in which the new predictor should be written
#'
#' @return	TRUE if the writing was successful
#'
#' @author	Michael Scherer
#' @noRd

writePredictorToCsv <- function(linearModel,path){
	coeffs <- as.matrix(linearModel)
	write.csv(coeffs,path)
	return(TRUE)
}

#######################################################################################
#' imputeBiseq
#'
#' Since the k-nearest-neighboring approach does not converge for all CpGs present in
#' sequencing data, a simple mean imputation is performed for this kind of data.
#'
#' @param methData	methylation data as a data frame or matix to be imputed
#'
#' @return	the changed methylation data frame
#'
#' @author	Michael Scherer
#' @noRd

imputeBiseq <- function(methData){
	samples <- dim(methData)[2]
	nas <- is.na(methData)
	newMean <- function(x){
		mean(x,na.rm=TRUE)
	}
	means <- apply(methData,1,newMean)
	for(i in (1:(dim(nas)[1]))){
		methData[i,nas[i,]] <- means[i]
	}
	return(methData)
}

#######################################################################################
#' convert.string.ages
#'
#' This function converts different formats of age information into numeric format to
#' be used in the following functions.
#'
#' @param age_information	input ages
#'
#' @return	the input ages in numeric format
#'
#' @author	Michael Scherer
#' @noRd

convert.string.ages <- function(age_information){
	temp <- strsplit(age_information,"[.]")
	temp <- temp[[1]][1]
	temp <- unique(na.omit(as.numeric(unlist(strsplit(temp,"[^0-9]+")))))
	if(length(temp)>1){
		temp <- temp[1]
	}
	temp
}

########################################################################################################################
## readGEO.R
## created: 2012-03-27
## creator: Yassen Assenov
## ---------------------------------------------------------------------------------------------------------------------
## Loading array-based methylation datasets from GEO.
########################################################################################################################

## G L O B A L S #######################################################################################################

GEO.PLATFORMS <- c(
	"GPL8490" = "probes27",
	"GPL13534" = "probes450",
	"GPL16304" = "probes450",
	"GPL21145" = "probesEPIC",
	"GPL33022" = "probesEPICv2") ## https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GPL33022

## F U N C T I O N S ###################################################################################################

#' Preprocesse sample characteristics
#'
#' Preprocesses sample characteristics lines from a series matrix definition file.
#'
#' @param txt               Lines in the file to be parsed.
#' @param regex.sample.info Regular expression capturing a definition of sample characteristics.
#' @return \code{character} vector storing sequence of preprocessed lines defining sample characteristics; an empty
#'         \code{vector} if none of the lines in \code{txt} matches the given regular expression.
#'
#' @author Yassen Assenov
#' @noRd
rnb.geo.parse.sample.info <- function(txt, regex.sample.info = "^!Sample_([^\t]+)\\t(.+)$") {
	s.info.lines <- grep(regex.sample.info, txt, value = TRUE)
	if (length(s.info.lines) == 0) {
		return(character())
	}
	result <- gsub(regex.sample.info, "\\2", s.info.lines)
	names(result) <- gsub(regex.sample.info, "\\1", s.info.lines)
	result
}

########################################################################################################################

#' Parses sample annotation
#'
#' Parses the sample annotation table from the corresponding lines in a series matrix definition file.
#'
#' @param txt Lines of the series matrix definition file that define sample characteristics. These should be
#'            preprocessed and not contain the initial sample specification (the token starting with \code{"!Sample_"}).
#' @return Sample annotation table as a \code{data.frame}.
#'
#' @author Yassen Assenov
#' @noRd
rnb.geo.parse.sample.anno <- function(txt) {

	## Validate that the sample annotation forms a table
	regex.sample.value <- "^\"([^\"]*)\"$"
	s.infos <- strsplit(txt, "\t", fixed = TRUE)
	s.valid <- mapply(grepl, x = s.infos, MoreArgs = list(pattern = regex.sample.value), SIMPLIFY = FALSE)
	if (!all(unlist(s.valid))) {
		stop("Cannot parse sample information; quotation marks and/or TAB characters are probably escaped")
	}
	s.infos <- lapply(s.infos, function(x) { gsub(regex.sample.value, "\\1", x) })
	sample.count <- range(sapply(s.infos, length))
	if (sample.count[1] != sample.count[2]) {
		stop("Incosistent number of samples in the annotation")
	}
	sample.count <- sample.count[1]

	## Parse characteristics_ch1
	for (i in which(names(s.infos) == "characteristics_ch1")) {
		ch1.values <- strsplit(s.infos[[i]], ": ", fixed = TRUE)
		if (all(sapply(ch1.values, length) %in% c(0, 2))) {
			j <- which(sapply(ch1.values, length) == 2)
			if (length(j) == 0) { next }
			column.title <- unique(sapply(ch1.values[j], '[', 1L))
			if (length(column.title) == 1) {
				s.infos[[i]] <- sapply(ch1.values, '[', 2L)
				names(s.infos)[i] <- column.title
			}
		}
	}
	suppressWarnings(rm(i, ch1.values, j, column.title))

	## Detect column types
	s.infos <- as.data.frame(s.infos, check.names = FALSE, stringsAsFactors = FALSE)
	buffer <- character()
	b.connection <- textConnection('buffer', open = "w", local = TRUE)
	write.table(s.infos, file = b.connection, quote = FALSE, sep = "\t", row.names = FALSE)
	close(b.connection)
	b.connection <- textConnection(buffer)
	s.infos <- read.delim(b.connection, quote = "", check.names = FALSE)
	close(b.connection)
	for (i in 1:ncol(s.infos)) {
		if (is.factor(s.infos[, i]) && nlevels(s.infos[, i]) == sample.count) {
			s.infos[, i] <- as.character(s.infos[, i])
		}
	}
	## Auto-rename columns in case there are duplicates
	if (ncol(s.infos) > 1) {
		s.infos <- s.infos[, rep(TRUE, ncol(s.infos))]
	}
	s.infos
}

########################################################################################################################

#' Parse sample identifiers
#'
#' Parses column names from the first line of series matrix table definition.
#' @param txt Line in the series matrix file that defines table columns; this is the line following the matrix
#'            definition \code{"series_matrix_table_begin"}.
#' @return All column names in the form of a \code{character} vector.
#'
#' @author Yassen Assenov
#' @noRd
rnb.geo.parse.ids <- function(txt) {
	txt <- strsplit(txt, "\t", fixed = TRUE)[[1]]
	if (length(txt) < 2 || txt[1] != '"ID_REF"') {
		stop("Unexpected header of series matrix table")
	}
	regex.id <- "^\\\"(GSM\\d+)\\\"$"
	if (!all(grepl(regex.id, txt[-1]))) {
		stop("Unexpected header of series matrix table")
	}
	gsub(regex.id, "\\1", txt[-1])
}

########################################################################################################################

#' Initialize methylation matrix
#'
#' Initializes the beta value matrix from a series matrix file.
#' @param txt        Line in the series matrix file that defines table columns; this is the line following the matrix
#'                   definition \code{"series_matrix_table_begin"}.
#' @param N.expected Number of columns expected, based on the sample definition lines above.
#' @param assay.type Expected assay. This must be one of \code{"probes27"}, \code{"probes450"} or \code{"probesEPIC"}.
#' @return Newly initialized \code{matrix} of type \code{double}.
#'
#' @author Yassen Assenov
#' @noRd
rnb.geo.init.matrix <- function(txt, N.expected, assay.type) {
	## TODO: Does it need implementation for EPICv2?
	sample.ids <- rnb.geo.parse.ids(txt)
	if (length(sample.ids) != N.expected) {
		stop("Inconsistent sample characteristics and series matrix table")
	}
	genome.assembly <- rnb.getOption("assembly")
	probe.ids <- rnb.get.annotation(assay.type, assembly = genome.assembly) ## TODO: Improve genome build selection
	probe.ids <- unlist(lapply(probe.ids, names), use.names = FALSE)

	## TODO: Consider rnb.getOption("disk.dump.big.matrices") and initialize different things
	matrix(NA_real_, length(probe.ids), length(sample.ids), dimnames = list(probe.ids, sample.ids))
}

########################################################################################################################

#' Parse a series matrix file
#'
#' Parses the sample annotation data and methylation values from a series matrix file.
#'
#' @param fname   Name of the file that contains the series matrix definition. This can also be gzipped.
#' @param verbose Flag indicating if messages are to be sent to the logger.
#' @return \code{list} with three elements: sample annotation table (\code{data.frame}), methylation beta value matrix,
#'         and a platform, specified as one of \code{"probes27"}, \code{"probes450"} or \code{"probesEPIC"}.
#'
#' @author Yassen Assenov
#' @noRd
rnb.geo.parse.series.matrix <- function(fname, verbose) {

	if (grepl("\\.gz$", fname)) {
		con <- gzfile(fname, "rt")
	} else {
		con <- file(fname, "rt")
	}
	if (verbose) {
		logger.start("Parsing Series Matrix File")
	}

	N.buffer <- 100000L # size of buffer, in number of lines
	regex.platform <- "^\\!Series_platform_id\t\\\"(GPL\\d+)\\\"$"
	data.platform <- NULL
	s.infos <- character()
	state.data.matrix <- 0L # 0: not started, 1: parsing header, 2: parsing data, 3: ready
	data.matrix <- NULL
	regex.data <- "^\\\"([^\"]+)\\\"\t(.*)$"
	skipped.records <- 0L

	repeat {
		txt <- NULL; invisible(gc())
		txt <- scan(con, "", n = N.buffer, sep = "\n", quiet = TRUE)
		if (length(txt) == 0) {
			break
		}

		## Parse platform
		i <- grep(regex.platform, txt)
		if (length(i) == 1) {
			if (!is.null(data.platform)) {
				stop("Multiple platforms defined")
			}
			data.platform <- gsub(regex.platform, "\\1", txt[i])
			if (!(data.platform %in% names(GEO.PLATFORMS))) {
				stop(paste("Platform", data.platform, "is not supported"))
			}
			if (verbose) {
				i <- paste0("Identified platform ", data.platform, " (", GEO.PLATFORMS[data.platform], ")")
				logger.status(i)
			}
		} else if (length(i) > 1) {
			stop("Multiple platforms defined; RnBeads supports a single platform only")
		}

		i <- which(txt == "!series_matrix_table_begin")
		if (length(i) == 0) {
			if (state.data.matrix == 0L) {
				s.infos <- c(s.infos, rnb.geo.parse.sample.info(txt))
				next
			}
			if (state.data.matrix == 1L) {
				data.matrix <- rnb.geo.init.matrix(txt[1], nrow(s.infos), GEO.PLATFORMS[data.platform])
				i.first <- 2L
				state.data.matrix <- 2L
			} else if (state.data.matrix == 2L) {
				i.first <- 1L
			}
		} else if (length(i) == 1 && state.data.matrix == 0L) {
			## Parse sample annotation table from the identified lines
			if (i > 1) {
				s.infos <- c(s.infos, rnb.geo.parse.sample.info(head(txt, i - 1)))
			}
			s.infos <- rnb.geo.parse.sample.anno(s.infos)
			## Initialize data matrix
			if (i + 1L <= length(txt)) {
				data.matrix <- rnb.geo.init.matrix(txt[i + 1], nrow(s.infos), GEO.PLATFORMS[data.platform])
				state.data.matrix <- 2L
				if (i + 1L == length(txt)) {
					next
				}
				i.first <- i + 2L
			} else {
				state.data.matrix <- 1L
				next
			}
		} else {
			stop("Multiple series matrix defined; RnBeads supports a single series matrix only")
		}

		i <- which(txt == "!series_matrix_table_end")
		if (length(i) == 0) {
			if (state.data.matrix != 2L) {
				next
			}
			i.last <- length(txt)
		} else if (length(i) == 1 && state.data.matrix == 2L) {
			i.last <- i - 1L
			if (i.last < i.first) {
				stop("Empty series matrix definition")
			}
		} else {
			stop("Multiple or misplaced end definition(s) of a series matrix")
		}

		## Parse lines to fill in the beta value matrix
		if (state.data.matrix == 2L) {
			txt <- txt[i.first:i.last]
			i.probes <- grep(regex.data, txt, invert = TRUE)
			if (length(i.probes) != 0) {
				stop("Invalid format of data table")
			}
			i.probes <- match(gsub(regex.data, "\\1", txt), rownames(data.matrix))
			i.found <- which(i.probes != 0L)
			skipped.records <- skipped.records + length(i.probes) - length(i.found)
			if (length(i.found) != 0) {
				txt <- gsub(regex.data, "\\2", txt[i.found])
				i.probes <- i.probes[i.found]
				txt <- gsub("\t$", "\t\t", txt)
				mm <- suppressWarnings(sapply(strsplit(txt, "\t", fixed = TRUE), as.double))
				if (!(is.matrix(mm) && nrow(mm) == ncol(data.matrix))) {
					stop("Invalid format of data table")
				}
				data.matrix[i.probes, ] <- t(mm)
			}
			suppressWarnings(rm(i.probes, i.found, mm))

			if (length(i) == 1) {
				state.data.matrix <- 3L
			}
		}
	}
	close(con)

	if (!(is.data.frame(s.infos) && state.data.matrix == 3L)) {
		stop("Missing or incomplete data table")
	}
	if (skipped.records != 0 && verbose) {
		i <- ifelse(skipped.records == 1, "record", "records")
		logger.warning(paste("Skipped", skipped.records, i, "referring to unsupported probes"))
	}
	data.matrix <- data.matrix[!apply(is.na(data.matrix), 1, all), , drop = FALSE]
	colnames(data.matrix) <- NULL
	if (verbose) {
		logger.status(paste("Loaded non-missing data for", nrow(data.matrix), "probes"))
		logger.completed()
	}
	list("annotation" = s.infos, "betas" = data.matrix, "platform" = GEO.PLATFORMS[data.platform])
}

########################################################################################################################

#' Import methylation data from GEO
#'
#' Imports Infinium 450K, MethylationEPIC or MethylationEPIC2 data series from the Gene Expression Omnibus. This function uses the
#' series matrix file.
#'
#' @param accession  Character string, starting with \code{"GSE"}, representing the GEO series for download and parsing.
#'                   Alternatively, this parameter can specify the file name of a previously downloaded GEO series
#'                   matrix file or its gzipped representation (in which case the filename must end in \code{".gz"}).
#'                   Other file formats, such as SOFT files, are not supported.
#' @param verbose    Flag indicating if messages should be created informing about the progress. If the logger is
#'                   initialized prior to calling this function, the informative messages are sent to the logger.
#'                   Warnings and errors are not affected by this parameters, the function always outputs them.
#' @param destdir    The destination directory for any downloads. Defaults to the (architecture-dependent) temporary
#'                   directory. Keep in mind that GEO series can be demanding in terms of storage space.
#' @return \code{\linkS4class{RnBeadSet}} object with phenotypic and beta value information.
#'
#' @author Yassen Assenov, modified by Baris Kalem
#' @export
rnb.read.geo <- function(accession = NULL, verbose = logger.isinitialized(), destdir = tempdir()) {
	## TODO: Make it compatible with EPIC v2
	if (verbose) {
		rnb.logger.start("Loading GEO Data Series")
	}
	if (!(is.character(accession) && length(accession) == 1 && isTRUE(accession != ""))) {
		stop("Invalid value for accession")
	}
	if (isTRUE(file.info(accession)[1, "isdir"] == FALSE)) {
		## Specified existing file
		fname <- accession
	} else if (!grepl("^GSE\\d\\d+$", accession)) {
		stop("Invalid value for accession; expected GEO data series identifier or an existing file")
	} else {
		## Specified GSE data series, download from GEO
		fname <- tempfile("gse", destdir, ".txt.gz")
		gse.url <- gsub("^(GSE\\d\\d).*$", "\\1", accession)
		gse.url <- sprintf("%snnn/%s/matrix/%s", gse.url, accession, accession)
		gse.url <- paste0("ftp://ftp.ncbi.nlm.nih.gov/geo/series/", gse.url, "_series_matrix.txt.gz")
		if (verbose) {
			rnb.info(paste("Using URL", gse.url))
			rnb.info(paste("Using download method", getOption("download.file.method")))
		}
		result <- tryCatch( # quiet = TRUE would be cleaner, but it seems to fail on MacOS
			download.file(gse.url, fname, quiet = FALSE, mode = "wb"),
			error = function(err) { return(-99L) })
		if (result != 0) {
			invisible(file.remove(fname))
			stop(paste("Could not download GEO Data Series, error code", result))
		}
		if (verbose) {
			rnb.status(paste("Downloaded GSE matrix to", fname))
		}
		rm(gse.url, result)
	}

	## Parse the series matrix file
	result <- rnb.geo.parse.series.matrix(fname, verbose)
	x <- c("probes27" = "27k", "probes450" = "450k", "probesEPIC" = "EPIC", "probesEPICv2" = "EPICv2")[result[[3]]]
	RnBeadSet(pheno = result[[1]], betas = result[[2]], platform = x)
}

########################################################################################################################
## differentialMethylation.R
## created: 2012-12-06
## creator: Fabian Mueller
## ---------------------------------------------------------------------------------------------------------------------
## Methods for determining differential methylation between groups.
########################################################################################################################

P.VAL.CUT <- 0.05
#parameters for density scatterplots on the site level
DENS.SCATTER.SPARSE.POINTS.PERC <- 0.01 #percentage of points to plot in the sparsely populated regions
DENS.SCATTER.SPARSE.POINTS.MAX <- 1e4 #maximum number of points to plot in the sparsely populated regions
DENS.SCATTER.SUBSAMPLE.THRES <- 2e6 #threshold to induce subsampling

### tTestP
###
### wrapper for t.test that catches errors
### @author Fabian Mueller
### @param ... parameters to be passed on to t.test
### @return vector of p-values resulting from t.test. Returns NA if error occurred
### @seealso \code{\link{t.test}}
tTestP <- function(...) {
	tryCatch(suppressWarnings(t.test(...)$p.value), error = function(e) { NA })
}

#' rowWelchP
#'
#' performs a two-sided Welch's t-test (unequal variances, unequal sample sizes) on each row of a matrix X with the indices inds.1 vs indices inds.g2 as group assignments.
#' @author Fabian Mueller
#' @param X Matrix on which the test is performed for every row
#' @param inds.g1 column indices of group 1 members
#' @param inds.g2 column indices of group 2 members
#' @param na.rm Should NAs be removed (logical)
#' @param alternative Testing alternative. Must be one of "two.sided" (default),"less","greater" or "all".
#' 		  in case of "all" a data frome with corresping alternative variables is returned. 
#' 		  Otherwise the result is a vector.
#' @return vector (or data.frame if alternative=="all") of p-values resulting from the Welch's t-test
#' @export
#' @note Requires \code{matrixStats} package
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' meth.mat <- meth(rnb.set.example)
#' sample.groups <- rnb.sample.groups(rnb.set.example)[[1]]
#' p.vals <- rowWelchP(meth.mat,sample.groups[[1]],sample.groups[[2]])
#' }
rowWelchP <- function(X,inds.g1,inds.g2=-inds.g1,na.rm=FALSE,alternative="two.sided"){
	if (!(alternative %in% c("two.sided","less","greater","all"))) {
		stop("invalid value for testing alternative")
	}
	X.1 <- X[,inds.g1]
	X.2 <- X[,inds.g2]
	if (na.rm){
		n.1 <- rowSums(!is.na(X.1), FALSE)
		n.2 <- rowSums(!is.na(X.2), FALSE)
	} else {
		n.1 <- length(inds.g1)
		n.2 <- length(inds.g2)
	}
	rm.1 <- rowMeans(X.1, na.rm = na.rm)
	rm.2 <- rowMeans(X.2, na.rm = na.rm)
	rv.1 <- rowVars(X.1, na.rm = na.rm)
	rv.2 <- rowVars(X.2, na.rm = na.rm)
	rq.1 <- rv.1/n.1
	rq.2 <- rv.2/n.2
	t.stat <- (rm.1 - rm.2)/sqrt(rq.1 + rq.2)
	rdf <- (rq.1 + rq.2)^2/(rq.1^2/(n.1-1) + rq.2^2/(n.2-1)) #degrees of freedom
	rp <- rep(NA,nrow(X))
	if (alternative == "two.sided" || alternative == "all") {
		rp.2s <- 2*pt(-abs(t.stat),rdf)
	}
	if (alternative == "less" || alternative == "all") {
		rp.l <- pt(t.stat,rdf)
	}
	if (alternative == "greater" || alternative == "all") {
		rp.g <- pt(t.stat,rdf,lower.tail=FALSE)
	}
	if (alternative == "two.sided") rp <- rp.2s
	if (alternative == "greater")   rp <- rp.g
	if (alternative == "less")      rp <- rp.l
	if (alternative == "all")		rp <- data.frame(less=rp.l,greater=rp.g,two.sided=rp.2s)
	return(rp)
}

#' rowPairedTP
#'
#' performs a two-sided t-test for paired samples on each row of a matrix X with the indices inds.1 vs indices inds.g2 as group assignments.
#' @author Fabian Mueller
#' @param X Matrix on which the test is performed for every row
#' @param inds.g1 column indices of group 1 members. \code{length(inds.g1)==length(inds.g2)} has to hold true.
#' @param inds.g2 column indices of group 2 members. \code{length(inds.g1)==length(inds.g2)} has to hold true.
#' @param alternative Testing alternative. Must be one of "two.sided" (default),"less","greater" or "all".
#' 		  in case of "all" a data frome with corresping alternative variables is returned. 
#' 		  Otherwise the result is a vector.
#' @return vector (or data.frame if alternative=="all") of p-values from a paired t-test
#' @export
#' @note Requires \code{matrixStats} package
rowPairedTP <- function(X,inds.g1,inds.g2=-inds.g1,alternative="two.sided"){
	if (!(alternative %in% c("two.sided","less","greater","all"))) {
		stop("invalid value for testing alternative")
	}
	X.1 <- X[,inds.g1]
	X.2 <- X[,inds.g2]
	if (ncol(X.1)!=ncol(X.2)){
		stop("unequal number of indices for the two groups")
	}
	X.d <-  X.1 - X.2
	n <- rowSums(!is.na(X.d), FALSE)
	d.bar <- rowMeans(X.d, na.rm = TRUE)
	s.d <- sqrt(rowVars(X.d, na.rm = TRUE))

	t.stat <- sqrt(n)*(d.bar)/s.d
	
	rp <- rep(NA,nrow(X))
	if (alternative == "two.sided" || alternative == "all") {
		rp.2s <- 2*pt(-abs(t.stat),n-1)
	}
	if (alternative == "less" || alternative == "all") {
		rp.l <- pt(t.stat,n-1)
	}
	if (alternative == "greater" || alternative == "all") {
		rp.g <- pt(t.stat,n-1,lower.tail=FALSE)
	}
	if (alternative == "two.sided") rp <- rp.2s
	if (alternative == "greater")   rp <- rp.g
	if (alternative == "less")      rp <- rp.l
	if (alternative == "all")		rp <- data.frame(less=rp.l,greater=rp.g,two.sided=rp.2s)
	return(rp)
}

#' rowOneSampleTP
#'
#' performs a two-sided t-test for paired samples on each row of a matrix X with the indices inds.1 vs indices inds.g2 as group assignments.
#' @author Fabian Mueller
#' @param X Matrix on which the test is performed for every row
#' @param mu The mean that is tested against
#' @param alternative Testing alternative. Must be one of "two.sided" (default),"less","greater" or "all".
#' 		  in case of "all" a data frome with corresping alternative variables is returned. 
#' 		  Otherwise the result is a vector.
#' @return vector (or data.frame if alternative=="all") of p-values from a paired t-test
#' @export
#' @note Requires \code{matrixStats} package
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' meth.mat <- meth(rnb.set.example)
#' p.vals <- rowOneSampleTP(meth.mat,mu=0,alternative="greater")
#' }
rowOneSampleTP <- function(X,mu=0,alternative="two.sided"){
	if (!(alternative %in% c("two.sided","less","greater","all"))) {
		stop("invalid value for testing alternative")
	}

	n <- rowSums(!is.na(X), FALSE)
	x.bar <- rowMeans(X, na.rm = TRUE)
	s.d <- sqrt(rowVars(X, na.rm = TRUE))
	
	t.stat <- sqrt(n)*(x.bar - mu)/s.d
	
	rp <- rep(NA,nrow(X))
	if (alternative == "two.sided" || alternative == "all") {
		rp.2s <- 2*pt(-abs(t.stat),n-1)
	}
	if (alternative == "less" || alternative == "all") {
		rp.l <- pt(t.stat,n-1)
	}
	if (alternative == "greater" || alternative == "all") {
		rp.g <- pt(t.stat,n-1,lower.tail=FALSE)
	}
	if (alternative == "two.sided") rp <- rp.2s
	if (alternative == "greater")   rp <- rp.g
	if (alternative == "less")      rp <- rp.l
	if (alternative == "all")		rp <- data.frame(less=rp.l,greater=rp.g,two.sided=rp.2s)
	return(rp)
}

#' limmaP
#'
#' applies hierarchical modeling anlalogous to differential expression employed in the \code{limma} package and returns
#' p-values (and optional residuals) for differential methylation
#' @author Fabian Mueller
#' @param X Matrix on which the test is performed for every row
#' @param inds.g1 column indices of group 1 members
#' @param inds.g2 column indices of group 2 members
#' @param adjustment.table a \code{data.frame} containing variables to adjust for in the testing
#' @param fun.conversion conversion function to transform the beta values into M values. By default, it is the logit function with adjustment
#' 						 for infinity values. See \code{\link{rnb.beta2mval}} for details.
#' @param paired should a paired analysis model be used. If so, the first index in \code{inds.g1} must correspond to the first
#' 				 index in  \code{inds.g2} and so on.
#' @param return.residuals Logical, if \code{TRUE}, the (M-value) residuals of the fitted model with
#'        				   covariate contributions removed are returned in addition to p-values. Default: \code{FALSE}.
#' 						   It can be set to \code{TRUE} by setting the option \code{differential.residuals.output} to "csv" or "ff" or "both".
#' @return If \code{return.residuals==FALSE} a numeric vector of p-values resulting from limma's differential analysis.
#'         If \code{TRUE}, a list with elements:
#' 		   \itemize{
#' 		   \item{\code{diffmeth.p.val}}{ Numeric vector of p-values.}
#' 		   \item{\code{residuals}}{ Numeric matrix (features x samples) of M-values with covariate contributions removed.}
#' 		   }
#' @note Requires \code{limma} package
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' meth.mat <- meth(rnb.set.example)
#' sample.groups <- rnb.sample.groups(rnb.set.example)[[1]]
#' p.vals <- limmaP(meth.mat,sample.groups[[1]],sample.groups[[2]])
#' res <- limmaP(meth.mat,sample.groups[[1]],sample.groups[[2]], return.residuals=TRUE)
#' resid_mat <- res$residuals
#' }
limmaP <- function(X,inds.g1,inds.g2=-inds.g1,adjustment.table=NULL,fun.conversion=rnb.beta2mval,paired=FALSE,return.residuals=FALSE){
	# suppressPackageStartupMessages(require(limma))
	if (is.logical(inds.g1)) inds.g1 <- which(inds.g1)
	if (is.logical(inds.g2)) inds.g2 <- which(inds.g2)
	n.g1 <- length(inds.g1)
	n.g2 <- length(inds.g2)
	n <- n.g1 + n.g2
	if (!is.null(adjustment.table)){
		if (!(is.data.frame(adjustment.table) && nrow(adjustment.table)==n && (!any(is.na(adjustment.table))))) {
			stop("invalid value for adjustment.table")
		}
		m <- ncol(adjustment.table)
		if (m == 0) {
			adjustment.table <- NULL
		} else {
			colnames(adjustment.table) <- paste0("x",1:m,"x")
		}
	}

	ind.vec <- c(inds.g1,inds.g2)
	if (length(ind.vec) < 2) stop("need at least two samples indices to compare")
	X.m <- fun.conversion(X[,ind.vec,drop=FALSE])

	## Set a covariate defining the two groups
	df <- data.frame(xg = factor(rep(c("group1","group2"), c(n.g1,n.g2)), levels=c("group1","group2")))
	if (!is.null(adjustment.table)){
		## Add covariates to adjust for
		df <- cbind(df,adjustment.table)
	}
	if (paired){
		## Add a covariate for pairing
		if (n.g1 != n.g2) {
			stop("Could not conduct paired limma analysis: unequal groupsizes")
		}
		df$xp <- as.factor(rep(1:n.g1,2))
	}

	formula.text <- paste0(c("~0",colnames(df)),collapse="+")
	design.m <- model.matrix(as.formula(formula.text),data=df)
	colnames(design.m) <- make.names(colnames(design.m),unique=TRUE)
	colnames(design.m)[1:2] <- c("group1","group2")
	fit <- limma::lmFit(X.m,design.m)
	# keep pre-contrast fit to compute fitted covariate contributions later
	fit0 <- fit
	contrasts.m <- makeContrasts(group1vs2=group1-group2,levels=design.m)
	fit <- limma::contrasts.fit(fit,contrasts.m)
	fit <- limma::eBayes(fit)
	pvals <- fit$p.value[,"group1vs2"]

	if (return.residuals) {
		# compute residuals by removing covariate contributions from the pre-contrast fit
		if (!is.null(adjustment.table)){
			cov_cols <- paste0("x",1:m,"x")
			cov_cols <- cov_cols[cov_cols %in% colnames(design.m)]
		} else {
			cov_cols <- character(0)
		}
		if (length(cov_cols) > 0 && !is.null(fit0$coefficients)){
			coef_cov <- fit0$coefficients[, cov_cols, drop = FALSE]
			design_cov <- design.m[, cov_cols, drop = FALSE]
			fitted_cov <- coef_cov %*% t(design_cov)
			resid_mat <- X.m - fitted_cov
			# logger.info("LimmaP: residuals computed from covariate coefficients")
		} else {
			resid_mat <- X.m
		}
		return(list(diffmeth.p.val = pvals, residuals = resid_mat))
	}
	return(pvals)
}

#' computeDiffTab.site
#'
#' computes a difference table containing multiple difference measures,
#' In the simple version the difference in means,
#' quotients in means and a p-value for the comparison of two groups in a table are computed.
#' This is computed for each row of the input table. The extended version contains additional columns
#' @rdname computeDiffTab.site
#' @author Fabian Mueller
#' @aliases computeDiffTab.site
#' @aliases computeDiffTab.default.site
#' @aliases computeDiffTab.extended.site
#' @param X Matrix on which the difference measures are calculated for every row
#' @param inds.g1 column indices of group 1 members
#' @param inds.g2 column indices of group 2 members
#' @param diff.method Method to determine p-values for differential methylation. Currently supported are 
#' 				"ttest" for a two-sided Welch t-test, "refFreeEWAS" for adjusting for cell mixtures,
#' 				and "limma" for p-values resulting from linear modeling of the transformed beta values (M-values)
#'				and using techniques from expression microarray analysis employed in the \code{limma} package.
#' @param variability.method Method to determine p-values for differential variability. Currently supported are 
#' 				"diffVar" for the diffVar method implemented in the missMethyl bioconductor package,
#' 				and "iEVORA".
#' @param paired should a paired a analysis be performed. If \code{TRUE} then inds.g1 and inds.g2 should have exactly the same length and should be
#' 			     order, such that the first element of inds.g1 corresponds to the first element of inds.g2 and so on.
#' @param adjustment.table a table of variables to be adjusted for in the differential methylation test. Currently this is only supported for
#'        \code{diff.method=="limma"}
#' @param eps Epsilon for computing quotients (avoid division by 0 by adding this value to denominator and enumerator before calculating the quotient)
#' @param covg coverage information (should be NULL for disabled or of equal dimensions as X)
#' @param covg.thres a coverage threshold
#' @param imputed flag indicating if methylation matrix was already imputed
#' @param return.residuals Logical, if \code{TRUE}, the (M-value) residuals of the fitted model with
#'        				   covariate contributions removed are returned in addition to p-values. Default: \code{FALSE}.
#' 						   It can be set to \code{TRUE} by setting the option \code{differential.residuals.output} to "csv" or "object".
#' @return If \code{return.residuals==FALSE} a numeric vector of p-values resulting from limma's differential analysis.
#' @return a dataframe containing the following variables:
#' \item{mean.g1}{Mean of group 1}
#' \item{mean.g2}{Mean of group 2}
#' \item{mean.diff}{Difference in means}
#' \item{mean.quot.log2}{log2 of the quotient of means}
#' \item{diffmeth.p.val}{P-value (as determined by \code{diff.method})}
#' \item{max.g1/max.g2}{[extended version only] Group maxima}
#' \item{min.g1/min.g2}{[extended version only] Group minima}
#' \item{sd.g1/sd.g2}{[extended version only] Group standard deviations}
#' \item{min.diff}{[extended version only] Minimum of 0 and single linkage difference between the groups}
#' \item{diffmeth.p.adj.fdr}{[extended version only] FDR adjusted p-values}
#' \item{num.na.g1/num.na.g2}{[extended version only] number of NA methylation values for groups 1 and 2 respectively}
#' \item{mean.covg.g1/mean.covg.g2}{[extended version with coverage information only] mean coverage of groups 1 and 2 respectively}
#' \item{min.covg.g1/min.covg.g2}{[extended version with coverage information only] minimum coverage of groups 1 and 2 respectively}
#' \item{max.covg.g1/max.covg.g2}{[extended version with coverage information only] maximum coverage of groups 1 and 2 respectively}
#' \item{covg.thresh.nsamples.g1/2}{[extended version with coverage information only] number of samples in group 1 and 2 respectively exceeding the
#' 									coverage threshold for this site.}
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' meth.mat <- meth(rnb.set.example)
#' sample.groups <- rnb.sample.groups(rnb.set.example)[[1]]
#' dm <- computeDiffTab.extended.site(meth.mat,sample.groups[[1]],sample.groups[[2]])
#' summary(dm)
#' }
computeDiffTab.default.site <- function(
  X,
  inds.g1,
  inds.g2,
  diff.method = rnb.getOption("differential.site.test.method"),
  variability.method = rnb.getOption("differential.variability.method"),
  paired = FALSE,
  adjustment.table = NULL,
  eps = 0.01,
  imputed = FALSE,
  return.residuals = FALSE
  ) {
	if (!(diff.method %in% c("ttest","limma","refFreeEWAS"))) {
		stop("Invalid method for differential site methylation test method")
	}
	# require(matrixStats)
	tab.g1 <- X[,inds.g1,drop=FALSE]
	tab.g2 <- X[,inds.g2,drop=FALSE]
	if(length(inds.g1)<2) {
		logger.info("Group 1 has less than 2 members")
		tab.g1 <- as.matrix(tab.g1)
	}
	if(length(inds.g2)<2) {
		logger.info("Group 2 has less than 2 members")
		tab.g2 <- as.matrix(tab.g2)
	}

	if (!is.null(adjustment.table)){
		if (!is.element(diff.method,c("limma","refFreeEWAS"))){
			logger.warning("Adjust for covariates is currently not supported for the selected p-value method --> skipping covariate adjustment")
		} else {
			logger.info(paste0("Adjusting for covariates: ",paste(colnames(adjustment.table),collapse=",")))
		}
	}

	mean.g1 <- rowMeans(tab.g1, na.rm = TRUE)
	mean.g2 <- rowMeans(tab.g2, na.rm = TRUE)

	if (paired){
		mean.diff <- rowMeans(tab.g1 - tab.g2, na.rm = TRUE)
		mean.quot <- rowMeans((tab.g1+eps)/(tab.g2+eps), na.rm = TRUE)
	} else {
		mean.diff <- mean.g1 - mean.g2
		mean.quot <- (mean.g1+eps)/(mean.g2+eps)
	}
	mean.quot.log2 <- log2(mean.quot)

	p.vals <- rep(as.double(NA),nrow(X))
	do.p.vals <- ncol(tab.g1) > 1 || ncol(tab.g2) > 1
	if (do.p.vals) {
		if (diff.method == "limma"){
			logger.info("Conducting differential analysis using limma")
			res_limma <- tryCatch(
				limmaP(X,inds.g1,inds.g2,adjustment.table=adjustment.table,paired=paired,return.residuals=return.residuals),
				error = function(ee) {
					logger.warning(c("Could not compute p-values using limma:",ee$message))
					NULL
				}
			)
			if (!is.null(res_limma)){
				if (return.residuals && is.list(res_limma)){
					p.vals <- res_limma$diffmeth.p.val
					resid_mat <- res_limma$residuals
					# logger.info("residuals calculated and returned")
				} else {
					p.vals <- res_limma
				}
			}
		} else if (diff.method == "refFreeEWAS"){
			p.vals <- refFreeEWASP(X, inds.g1, inds.g2, adjustment.table=adjustment.table, paired=paired)
		} else if (paired){
			logger.info("Conducting differential analysis using paired Student t-test")
			p.vals <- rowPairedTP(X,inds.g1,inds.g2)
		} else if (length(inds.g1)>1 && length(inds.g2)>1) {
			logger.info("Conducting differential analysis using two-sided Welch t-test")
			p.vals <- rowWelchP(X,inds.g1,inds.g2,na.rm=TRUE)
		} else if(length(inds.g1)>1) {
			logger.info("Conducting differential analysis using two-sided Welch t-test")
			p.vals <- rowOneSampleTP(tab.g1,mu=tab.g2[,1])
		} else if(length(inds.g2)>1) {
			logger.info("Conducting differential analysis using two-sided Welch t-test")
			p.vals <- rowOneSampleTP(tab.g2,mu=tab.g1[,1])
		}
	} else {
		logger.warning("Skipping p-value computation due to insufficient sample numbers")
	}
	# if residuals requested, compute per-group residual means and include them in the table
	if (exists("resid_mat")){
		resid.mean.g1 <- rowMeans(resid_mat[,inds.g1,drop=FALSE], na.rm=TRUE)
		resid.mean.g2 <- rowMeans(resid_mat[,inds.g2,drop=FALSE], na.rm=TRUE)
		resid.mean.diff <- resid.mean.g1 - resid.mean.g2
		tt <- data.frame(mean.g1=mean.g1,mean.g2=mean.g2,mean.diff=mean.diff,mean.quot.log2=mean.quot.log2,diffmeth.p.val=p.vals,
						 resid.mean.g1=resid.mean.g1,resid.mean.g2=resid.mean.g2,resid.mean.diff=resid.mean.diff)
		# attach the full residual matrix as attribute for later saving by addDiffMethTable
		attr(tt, "resid_mat") <- resid_mat
		# logger.info("computeDiffTab.default.site: residuals computed and included in the returned table as an attribute")
	} else {
		tt <- data.frame(mean.g1=mean.g1,mean.g2=mean.g2,mean.diff=mean.diff,mean.quot.log2=mean.quot.log2,diffmeth.p.val=p.vals)
	}

	if(rnb.getOption("differential.variability")){
	  if(!imputed){
	    X <- rnb.execute.imputation(X)
	    tab.g1 <- X[,inds.g1,drop=FALSE]
	    tab.g2 <- X[,inds.g2,drop=FALSE]
	  }
	  p.vals.var <- rep(as.double(NA),nrow(X))
	  do.p.vals <- ncol(tab.g1) > 1 || ncol(tab.g2) > 1
	  if (do.p.vals) {
	    if (variability.method == "diffVar"){
	      logger.info("Conducting differential variability using diffVar")
	      tryCatch(
	        p.vals.var <- diffVar(X,inds.g1,inds.g2,adjustment.table=adjustment.table,paired=paired),
	        error = function(ee) {
	          logger.warning(c("Could not compute p-values using diffVar:",ee$message))
	          logger.completed()
	        }
	      )
	    } else if (variability.method == "iEVORA"){
	      logger.info("Conducting differential variability using iEVORA")
	      if(paired){
	        logger.warning("Cannot conduct paired variability analysis with iEVORA, changing to diffVar.")
	        rnb.options("differential.variability.method"="diffVar")
	        tryCatch(
	          p.vals.var <- diffVar(X,inds.g1,inds.g2,adjustment.table=adjustment.table,paired=paired),
	          error = function(ee) {
	            logger.warning(c("Could not compute p-values using diffVar:",ee$message))
	            logger.completed()
	          }
	        )
	      }else{
  	      tryCatch(
  	        p.vals.var <- apply.iEVORA(X,inds.g1,inds.g2),
  	        error = function(ee) {
  	          logger.warning(c("Could not compute p-values using iEVORA:",ee$message))
  	          logger.completed()
  	        }
  	      )
	      }
	    }
	  } else {
	    logger.warning("Skipping p-value computation due to insufficient sample numbers")
	  }
	  p.vals.is.na <- is.na(p.vals.var)
	  if (!all(p.vals.is.na)){
	    if (any(p.vals.is.na)){
	      logger.info(c(sum(p.vals.is.na),"p-values are NA. They are treated as 1 in FDR adjustment"))
	      p.vals.var[is.na(p.vals.t.na.adj)] <- 1
	    }
	    p.vals.var.adj <- p.adjust(p.vals.var, method = "fdr")
	  } else {
	    p.vals.var.adj <- rep(NA,length(p.vals.var))
	  }
	  
	  var.g1 <- apply(tab.g1,1,var)
	  var.g2 <- apply(tab.g2,1,var)
	  
	  if(paired){
	    var.diff <- apply(tab.g1 - tab.g2,1,var)
	    var.log.ratio <- apply(X,1,function(x){
	      log2(var(x[inds.g1]+eps)/var(x[inds.g2]+eps))
	    })
	  }else{
	    var.diff <- var.g1-var.g2
	    var.log.ratio <- ifelse(var.g1==0|var.g2==0,1,log2(var.g1/var.g2))
	  }
	  neg.log10.p <- -log10(p.vals.var)
	  neg.log10.fdr <- -log10(p.vals.var.adj)
	  tt <- data.frame(tt,var.g1=var.g1,var.g2=var.g2,var.diff=var.diff,var.log.ratio=var.log.ratio,diffVar.p.val=p.vals.var,diffVar.p.adj.fdr=p.vals.var.adj,log10P=neg.log10.p,
	                   log10FDR=neg.log10.fdr)
	}
	return(tt)
}
#' @rdname computeDiffTab.site
#' @export
computeDiffTab.extended.site <- function(
  X,
  inds.g1,
  inds.g2,
  diff.method = rnb.getOption("differential.site.test.method"),
  variability.method = rnb.getOption("differential.variability.method"),
  paired = FALSE,
  adjustment.table = NULL,
  eps = 0.01,
  covg = NULL,
  covg.thres = rnb.getOption("filtering.coverage.threshold"),
  imputed = FALSE,
  return.residuals = FALSE
) {
	# require(matrixStats)
	tt.basic <- computeDiffTab.default.site(
	  X,
	  inds.g1 = inds.g1,
	  inds.g2 = inds.g2,
	  diff.method = diff.method,
	  variability.method = variability.method,
	  paired = paired,
	  adjustment.table = adjustment.table,
	  eps = eps,
	  imputed = imputed,
	  return.residuals = return.residuals
	)
	tab.g1 <- X[,inds.g1]
	tab.g2 <- X[,inds.g2]
	if (length(inds.g1)>1){
		max.g1 <- rowMaxs(tab.g1,na.rm=TRUE)
		min.g1 <- rowMins(tab.g1,na.rm=TRUE)
		sd.g1  <- rowSds(tab.g1,na.rm=TRUE)
		num.na.g1 <- rowSums(is.na(tab.g1))
	} else {
		max.g1  <- tab.g1
		min.g1  <- tab.g1
		sd.g1   <- NA
		num.na.g1 <- rep(0,length(tab.g1))
		num.na.g1[is.na(tab.g1)] <- 1
	}
	if (length(inds.g2)>1){
		max.g2 <- rowMaxs(tab.g2,na.rm=TRUE)
		min.g2 <- rowMins(tab.g2,na.rm=TRUE)
		sd.g2  <- rowSds(tab.g2,na.rm=TRUE)
		num.na.g2 <- rowSums(is.na(tab.g2))
	} else {
		max.g2  <- tab.g2
		min.g2  <- tab.g2
		sd.g2   <- NA
		num.na.g2 <- rep(0,length(tab.g2))
		num.na.g2[is.na(tab.g2)] <- 1
	}

	min.diff <- ifelse(max.g1 < min.g2, max.g1 - min.g2,ifelse(max.g2 < min.g1, min.g1 - max.g2, 0))
	p.vals.t.na.adj <- tt.basic$diffmeth.p.val
	p.vals.is.na <- is.na(tt.basic$diffmeth.p.val)
	if (!all(p.vals.is.na)){
		if (any(p.vals.is.na)){
			logger.info(c(sum(p.vals.is.na),"p-values are NA. They are treated as 1 in FDR adjustment"))
			p.vals.t.na.adj[is.na(p.vals.t.na.adj)] <- 1
		}
		p.vals.adj.t <- p.adjust(p.vals.t.na.adj, method = "fdr")
	} else {
		p.vals.adj.t <- rep(NA,nrow(tt.basic))
	}
	

	tt.ext <- data.frame(max.g1=max.g1,min.g1=min.g1,sd.g1=sd.g1,max.g2=max.g2,min.g2=min.g2,sd.g2=sd.g2,min.diff=min.diff,diffmeth.p.adj.fdr=p.vals.adj.t,
						 num.na.g1=num.na.g1,num.na.g2=num.na.g2)
	#coverage information
	if (!is.null(covg) & all(dim(covg)==dim(X))){
		covg[is.na(covg)] <- 0 #set NA to 0 coverage
		
		tab.covg.g1 <- covg[,inds.g1]
		tab.covg.g2 <- covg[,inds.g2]
		if(length(inds.g1)<2) {
			logger.info("Group 1 has less than 2 members")
			tab.covg.g1 <- as.matrix(tab.covg.g1)
		}
		if(length(inds.g2)<2) {
			logger.info("Group 2 has less than 2 members")
			tab.covg.g2 <- as.matrix(tab.covg.g2)
		}
		mean.covg.g1 <- rowMeans(tab.covg.g1)
		mean.covg.g2 <- rowMeans(tab.covg.g2)
		min.covg.g1 <- rowMins(tab.covg.g1)
		min.covg.g2 <- rowMins(tab.covg.g2)
		max.covg.g1 <- rowMaxs(tab.covg.g1)
		max.covg.g2 <- rowMaxs(tab.covg.g2)
		covg.thresh.nsamples.g1 <- rowSums(tab.covg.g1>=covg.thres)
		covg.thresh.nsamples.g2 <- rowSums(tab.covg.g2>=covg.thres)

		tt.ext.covg	<- data.frame(mean.covg.g1=mean.covg.g1,mean.covg.g2=mean.covg.g2,
								  min.covg.g1=min.covg.g1,min.covg.g2=min.covg.g2,
								  max.covg.g1=max.covg.g1,max.covg.g2=max.covg.g2,
								  covg.thresh.nsamples.g1=covg.thresh.nsamples.g1,covg.thresh.nsamples.g2=covg.thresh.nsamples.g2)
		tt.ext <- cbind(tt.ext,tt.ext.covg)	
	}
	tt <- cbind(tt.basic,tt.ext)
	# preserve residual matrix attribute from basic table if present (cbind drop attributes)
	if (!is.null(attr(tt.basic, "resid_mat"))) {
		attr(tt, "resid_mat") <- attr(tt.basic, "resid_mat")
		# logger.info("extended.site: residuals preserved in the returned table as an attribute")
	}
	return(tt)
}

#' combineTestPvalsMeth
#'
#' combine p-values of multiple tests using (a generalization of) Fisher's method. The parameter setting here is taylored to DNA methylation, but can be adapted. 
#' Reference: Makambi, K. (2003). Weighted inverse chi-square method for correlated significance tests. Journal of Applied Statistics, 30(2), 225-234.
#' @author Fabian Mueller, Christoph Bock
#' @aliases combineTestPvalsMeth
#' @param pvalues p-values to combine
#' @param testWeights weights for the individual tests
#' @param correlated are the individual tests correlated
#' @param methExpectedTestCorrelation expected correlation. Empirically approximated to the default value of 0.8 for DNA-methylation
#' @return the combined p-value
#' @export
#' @examples
#' \donttest{
#' p.vals <- 10^-c(0,1,5)
#' combineTestPvalsMeth(p.vals)
#' }
combineTestPvalsMeth <- function(pvalues,testWeights=NULL,correlated=FALSE,methExpectedTestCorrelation = 0.8) {
	if (is.null(pvalues)){
		return(NA)	
	}
	if (!is.numeric(pvalues)){
		logger.warning(c("Non numeric value for pvalues in combination:",pvalues))
		return(NA)
	} 
	
	if (!is.null(testWeights)) {
		# check if weights are valid
		if (length(pvalues) != length(testWeights)) stop("Number of items in <pvalues> and in <testWeights> must be identical if weights are to be used")
		if (sum(is.na(testWeights))>0) stop("NA values are not permitted for the test weights")
		if (sum(testWeights<0)>0) stop("Weights must be positive")
		# standardize weights
		testWeights = testWeights/sum(testWeights)
	} else {
		# use equal weighting
		testWeights = rep(1/length(pvalues),length(pvalues))
	}
	pvalues[is.na(pvalues)] = 1
	
	if (length(pvalues) < 1){
		return(NA)	
	} else if (length(pvalues) == 1) {
		return(pvalues[1])
	} else if (length(pvalues) > sqrt(.Machine$integer.max)){
		logger.info(c("Too many p-values to combine --> using subsampling"))
		nn <- trunc(sqrt(.Machine$integer.max))
		ss <- sample(length(pvalues),nn)
		pvalues <- pvalues[ss]
		testWeights <- testWeights[ss]
		testWeights <- testWeights/sum(testWeights)
	}
	
	##estimating the correlation
	#s <- -2*log(pvalues)
	#q.t <- var(s)
	#to.root <- 10.028 - 4*q.t/3
	#rho <- -2.167 + sqrt(to.root)
	#rho <- max(rho,0)
	
	if (correlated==FALSE & is.null(testWeights)) {
		# use Fisher's classical method for combining p-values in the case of independence
		tcombined = sum(-2*log(pvalues))
		return(pchisq(tcombined,2*length(pvalues),lower.tail=FALSE))
	} else {
		# use the method proposed in Makambi (2003) Journal of Applied Statistics
		r = ifelse(correlated,methExpectedTestCorrelation,0)
		m = length(pvalues)
		M.Fm = sum(-2*log(pvalues)*testWeights)
		var.M.Fm = 4*sum(testWeights^2)

		ij.pairs <- expand.grid(1:m,1:m) #create all combinations of 2 indices
		ij.pairs <- ij.pairs[ij.pairs[,1]!=ij.pairs[,2],] #remove those pairs where i==j
		tw.i <- testWeights[ij.pairs[,1]]
		tw.j <- testWeights[ij.pairs[,2]]
		vv <- tw.i * tw.j * (3.25*r+0.75*r^2)
		var.M.Fm <- var.M.Fm + sum(vv)
		
		nu = 8/var.M.Fm
		tcombined = nu*M.Fm/2 # chi square test generalization of Makambi	   	   
		return(pchisq(tcombined,nu,lower.tail=FALSE))
	}
}

#' computeDiffTab.region
#'
#' computes a difference table containing multiple difference measures,
#' In the simple version the mean of the difference in means,
#' the mean quotient in means and a combination of p-values on the site level are computed.
#' This is computed for each row of the input table. The extended version contains additional columns
#' @rdname computeDiffTab.region
#' @author Fabian Mueller
#' @aliases computeDiffTab.region
#' @aliases computeDiffTab.default.region
#' @param dmtp differential methylation table on the site level (as obtained from \code{\link{computeDiffTab.default.site}})
#' @param regions2sites a list containing for each region the indices of the corresponding sites in the site differential methylation table
#' @param includeCovg flag indicating whether to include coverage information
#' @return a dataframe containing the following variables for a given genomic region:
#' \item{mean.mean.g1,mean.mean.g2}{mean of mean methylation levels for group 1 and 2 across all sites in a region}
#' \item{mean.mean.diff}{Mean difference in means across all sites in a region}
#' \item{mean.mean.quot.log2}{Mean quotient in means across all sites in a region}
#' \item{comb.p.val}{Combined p-value using a generalization of Fisher's method. See \code{\link{combineTestPvalsMeth}} for details.}
#' \item{comb.p.adj.fdr}{FDR adjusted combined p-value}
#' \item{num.sites}{number of sites that were considered for a region}
#' \item{mean.num.na.g1/2}{mean number (accross all considered sites) of samples that contained an NA for group 1 and 2 respectively}
#' \item{mean.mean.covg.g1/2}{Mean value of mean coverage values (across all samples in a group) across all sites in a region}
#' \item{mean.nsamples.covg.thresh.g1/2}{mean number (accross all considered sites) of samples that have a coverage larger than the specified threshold
#' 		(see \code{\link{computeDiffTab.default.site}} for details) for group 1 and 2 respectively}
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' meth.mat <- meth(rnb.set.example)
#' sample.groups <- rnb.sample.groups(rnb.set.example)[[1]]
#' dm.sites <- computeDiffTab.extended.site(meth.mat,sample.groups[[1]],sample.groups[[2]])
#' map.regions.to.sites <- regionMapping(rnb.set.example,"promoters")
#' dm.promoters <- computeDiffTab.default.region(dm.sites,map.regions.to.sites)
#' }
computeDiffTab.default.region <- function(dmtp,regions2sites,includeCovg=FALSE){
	if (length(regions2sites)<1){
		stop("regions2sites argument must have length > 0")
	}
	col.id.g1 <- "mean.g1"
	col.id.g2 <- "mean.g2"
	col.id.diff <- "mean.diff"
	col.id.quot <- "mean.quot.log2"
	col.id.p  <- "diffmeth.p.val"
	col.id.num.na.g1 <- "num.na.g1"
	col.id.mean.covg.g1 <- "mean.covg.g1"
	col.id.covg.thresh.ns.g1 <- "covg.thresh.nsamples.g1"
	col.id.num.na.g2 <- "num.na.g2"
	col.id.mean.covg.g2 <- "mean.covg.g2"
	col.id.covg.thresh.ns.g2 <- "covg.thresh.nsamples.g2"
	n.regs.with.sites <- length(regions2sites)
	mean.g1 <- rep(NA,n.regs.with.sites)
	mean.g2 <- rep(NA,n.regs.with.sites)
	diff <- rep(NA,n.regs.with.sites)
	quot <- rep(NA,n.regs.with.sites)
	p.vals <- rep(NA,n.regs.with.sites)
	num.sites <- rep(NA,n.regs.with.sites)
	mean.num.na.g1 <- rep(NA,n.regs.with.sites)
	mean.num.na.g2 <- rep(NA,n.regs.with.sites)
	mean.mean.covg.g1 <- rep(NA,n.regs.with.sites)
	mean.mean.covg.g2 <- rep(NA,n.regs.with.sites)
	mean.nsamples.covg.thresh.g1 <- rep(NA,n.regs.with.sites)
	mean.nsamples.covg.thresh.g2 <- rep(NA,n.regs.with.sites)
	col.vec <- c(col.id.g1, col.id.g2, col.id.diff, col.id.quot, col.id.p, col.id.num.na.g1, col.id.num.na.g2)
	# include residual mean columns if residual output requested
	res.opt <- rnb.getOption("differential.residuals.output")
	if (!is.null(res.opt)){
		col.id.resid.g1 <- "resid.mean.g1"
		col.id.resid.g2 <- "resid.mean.g2"
		col.id.resid.diff <- "resid.mean.diff"
		col.vec <- c(col.vec, col.id.resid.g1, col.id.resid.g2, col.id.resid.diff)
	}
	if (includeCovg) {
		col.vec <- c(col.vec,c(col.id.mean.covg.g1, col.id.mean.covg.g2, col.id.covg.thresh.ns.g1, col.id.covg.thresh.ns.g2))
	}
	if(rnb.getOption("differential.variability")){
	  col.id.var.g1 <- "var.g1"
	  col.id.var.g2 <- "var.g2"
	  col.id.diff.var <- "var.diff"
	  col.id.quot.var <- "var.log.ratio"
	  col.id.p.var <- "diffVar.p.val"
	  mean.var.g1 <- rep(NA,n.regs.with.sites)
	  mean.var.g2 <- rep(NA,n.regs.with.sites)
	  diff.var <- rep(NA,n.regs.with.sites)
	  quot.var <- rep(NA,n.regs.with.sites)
	  p.vals.var <- rep(NA,n.regs.with.sites)
	  col.vec <- c(col.vec, col.id.var.g1, col.id.var.g2, col.id.diff.var, col.id.quot.var, col.id.p.var)
	}
	# ensure missing expected columns are present (pad with NA) to avoid subsetting errors
	missing.cols <- setdiff(col.vec, colnames(dmtp))
	if (length(missing.cols) > 0){
		for (mc in missing.cols) dmtp[[mc]] <- NA
	}
	dmt4fastProc <- dmtp[,col.vec] # now safe to subset, not looking up the columns by name, but by index reduces runtime
	
	if(parallel.isEnabled()){
		dm <- foreach(i=1:n.regs.with.sites, .combine='rbind',.multicombine=TRUE,.maxcombine=200) %dopar% {
			pids <- regions2sites[[i]]
			subtab <- dmt4fastProc[pids,]#these lookups take up most of the time
			
			# compute aggregates by column names (safe regardless of column order)
			mean.g1   <- mean(subtab[,"mean.g1"],na.rm=TRUE)
			mean.g2   <- mean(subtab[,"mean.g2"],na.rm=TRUE)
			diff      <- mean(subtab[,"mean.diff"],na.rm=TRUE)
			quot      <- mean(subtab[,"mean.quot.log2"],na.rm=TRUE)
			num.sites <- length(pids)
			mean.num.na.g1 <- mean(subtab[,"num.na.g1"], na.rm=TRUE)
			mean.num.na.g2 <- mean(subtab[,"num.na.g2"], na.rm=TRUE)
			if (includeCovg){
				mean.mean.covg.g1 <- mean(subtab[,"mean.covg.g1"], na.rm=TRUE)
				mean.mean.covg.g2 <- mean(subtab[,"mean.covg.g2"], na.rm=TRUE)
				mean.nsamples.covg.thres.g1 <- mean(subtab[,"covg.thresh.nsamples.g1"], na.rm=TRUE)
				mean.nsamples.covg.thres.g2 <- mean(subtab[,"covg.thresh.nsamples.g2"], na.rm=TRUE)
			} else {
				mean.mean.covg.g1 <- NA
				mean.mean.covg.g2 <- NA
				mean.nsamples.covg.thres.g1 <- NA
				mean.nsamples.covg.thres.g2 <- NA
			}
			res <- combineTestPvalsMeth(na.omit(subtab[,"diffmeth.p.val"]),correlated=TRUE)
			p.vals <- NA
			if (length(res)>0) p.vals <- res
			if(rnb.getOption("differential.variability")){
			  mean.var.g1   <- mean(subtab[,"var.g1"],na.rm=TRUE)
			  mean.var.g2   <- mean(subtab[,"var.g2"],na.rm=TRUE)
			  diff.var      <- mean(subtab[,"var.diff"],na.rm=TRUE)
			  quot.var      <- mean(subtab[,"var.log.ratio"],na.rm=TRUE)
			  res <- combineTestPvalsMeth(na.omit(subtab[,"diffVar.p.val"]),correlated=TRUE)
			  p.vals.var <- NA
			  if (length(res)>0) p.vals.var <- res
			}else{
			  mean.var.g1 <- NA
			  mean.var.g2 <- NA
			  diff.var <- NA
			  quot.var <- NA
			  p.vals.var <- NA
			}
			# residual means (if present)
			resid.mean.g1 <- NA
			resid.mean.g2 <- NA
			resid.mean.diff <- NA
			if ("resid.mean.g1" %in% colnames(subtab)){
				resid.mean.g1 <- mean(subtab[,"resid.mean.g1"], na.rm=TRUE)
			}
			if ("resid.mean.g2" %in% colnames(subtab)){
				resid.mean.g2 <- mean(subtab[,"resid.mean.g2"], na.rm=TRUE)
			}
			if (!is.na(resid.mean.g1) && !is.na(resid.mean.g2)){
				resid.mean.diff <- resid.mean.g1 - resid.mean.g2
			}
			c(mean.g1, mean.g2, diff, quot, num.sites, mean.num.na.g1, mean.num.na.g2,
			  mean.mean.covg.g1, mean.mean.covg.g2, mean.nsamples.covg.thres.g1, mean.nsamples.covg.thres.g2,
			  p.vals,mean.var.g1,mean.var.g2,diff.var,quot.var,p.vals.var,resid.mean.g1,resid.mean.g2,resid.mean.diff)
		}
		mean.g1                      <- dm[, 1]
		mean.g2                      <- dm[, 2]
		diff                         <- dm[, 3]
		quot                         <- dm[, 4]
		num.sites                    <- dm[, 5]
		mean.num.na.g1               <- dm[, 6]
		mean.num.na.g2               <- dm[, 7]
		mean.mean.covg.g1            <- dm[, 8]
		mean.mean.covg.g2            <- dm[, 9]
		mean.nsamples.covg.thresh.g1 <- dm[,10]
		mean.nsamples.covg.thresh.g2 <- dm[,11]
		p.vals                       <- dm[,12]
		mean.var.g1                  <- dm[,13]
		mean.var.g2                  <- dm[,14]
		diff.var                     <- dm[,15]
		quot.var                     <- dm[,16]
		p.vals.var                  <- dm[,17]

	} else {
		dummy <- sapply(1:n.regs.with.sites,FUN=function(i){
			pids <- regions2sites[[i]]
			subtab <- dmt4fastProc[pids,]#these lookups take up most of the time
			
			mean.g1[i] <<- mean(subtab[,1],na.rm=TRUE)
			mean.g2[i] <<- mean(subtab[,2],na.rm=TRUE)
			diff[i]    <<- mean(subtab[,3],na.rm=TRUE)
			quot[i]    <<- mean(subtab[,4],na.rm=TRUE)
			
			num.sites[i] <<- length(pids)
			mean.num.na.g1[i] <<- mean(subtab[,6])
			mean.num.na.g2[i] <<- mean(subtab[,7])
			var.start <- 8
			if (includeCovg){
				mean.mean.covg.g1[i] <<- mean(subtab[,8])
				mean.mean.covg.g2[i] <<- mean(subtab[,9])
				mean.nsamples.covg.thresh.g1[i] <<- mean(subtab[,10])
				mean.nsamples.covg.thresh.g2[i] <<- mean(subtab[,11])
				var.start <- 12
			}
			
			res <- combineTestPvalsMeth(na.omit(subtab[,5]),correlated=TRUE)
			if (length(res)>0) p.vals[i]  <<- res
			if(rnb.getOption("differential.variability")){
			  mean.var.g1[i] <<- mean(subtab[,var.start],na.rm=TRUE)
			  mean.var.g2[i] <<- mean(subtab[,var.start+1],na.rm=TRUE)
			  diff.var[i] <<- mean(subtab[,var.start+2],na.rm=TRUE)
			  quot.var[i] <<- mean(subtab[,var.start+3],na.rm=TRUE)
			  res <- combineTestPvalsMeth(na.omit(subtab[,var.start+4]),correlated=TRUE)
			  if (length(res)>0) p.vals.var[i]  <<- res
			}
			
			return(TRUE)
		})
	}
	
	p.vals.na.adj <- p.vals
	p.vals.is.na <- is.na(p.vals)
	if (any(p.vals.is.na)){
		logger.info(c(sum(p.vals.is.na),"p-values are NA. They are treated as 1 in FDR adjustment"))
		p.vals.na.adj[is.na(p.vals.na.adj)] <- 1
	}
	p.vals.adj <- p.adjust(p.vals.na.adj, method = "fdr")
	
	tt <- data.frame(mean.mean.g1=mean.g1,mean.mean.g2=mean.g2,mean.mean.diff=diff,mean.mean.quot.log2=quot,comb.p.val=p.vals,comb.p.adj.fdr=p.vals.adj,
					 num.sites=num.sites,
					 mean.num.na.g1=mean.num.na.g1,mean.num.na.g2=mean.num.na.g2)
	if (includeCovg){
		tt <- cbind(tt,data.frame(mean.mean.covg.g1=mean.mean.covg.g1,mean.mean.covg.g2=mean.mean.covg.g2,
								  mean.nsamples.covg.thresh.g1=mean.nsamples.covg.thresh.g1,mean.nsamples.covg.thresh.g2=mean.nsamples.covg.thresh.g2))
	}
	if(rnb.getOption("differential.variability")){
	  p.vals.var.na.adj <- p.vals.var
	  p.vals.var.is.na <- is.na(p.vals.var)
	  if (any(p.vals.var.is.na)){
	    logger.info(c(sum(p.vals.var.is.na),"p-values are NA. They are treated as 1 in FDR adjustment"))
	    p.vals.var.na.adj[is.na(p.vals.var.na.adj)] <- 1
	  }
	  p.vals.var.adj <- p.adjust(p.vals.var.na.adj, method = "fdr")
	  tt <- cbind(tt,data.frame(mean.var.g1=mean.var.g1,mean.var.g2=mean.var.g2,mean.var.diff=diff.var,
	                            mean.var.log.ratio=quot.var,comb.p.val.var=p.vals.var,comb.p.adj.var.fdr=p.vals.var.adj))
	}
	# If site residual matrix is present, compute region residuals by averaging site residuals per region
	resid_site_mat <- attr(dmtp, "resid_mat")
	if (!is.null(resid_site_mat)){
		# prepare region x sample matrix
		n.samples <- ncol(resid_site_mat)
		resid_region_mat <- matrix(NA, nrow=n.regs.with.sites, ncol=n.samples)
		rownames(resid_region_mat) <- names(regions2sites)
		# preserve column names if available
		if (!is.null(colnames(resid_site_mat))) colnames(resid_region_mat) <- colnames(resid_site_mat)
		for (i in seq_len(n.regs.with.sites)){
			pids <- regions2sites[[i]]
			if (length(pids) > 0 && !all(is.na(pids))){
				# read slice; if ff-backed, this reads only the needed rows
				slice <- resid_site_mat[pids, , drop=FALSE]
				resid_region_mat[i, ] <- colMeans(slice, na.rm=TRUE)
			}
		}
		attr(tt, "resid_mat") <- resid_region_mat
	}
	rownames(tt) <- names(regions2sites)
	return(tt)
}

### combinedRanking.tab
###
### computes the combined ranking for each row as the maximum rank among all columns
### @author Fabian Mueller
### @aliases combinedRanking.tab
### @param tt differential methylation table
### @param rerank if \code{TRUE} then an additional ranking will be performed on the combined rank in order to obtain values in [1,nrow(tt)]
### @return a vector containing the combined ranking
combinedRanking.tab <- function(tt,rerank=FALSE){
	rank.mat <- c()
	for (i in 1:ncol(tt)){
		rrs <- rank(tt[,i],na.last="keep",ties.method="min")
		if (!all(is.na(rrs))) {
			rank.mat <- cbind(rank.mat,rrs)
		}
	}
	if (is.null(rank.mat)){
		logger.warning("Could not compute combined ranking: To few non-NA columns specified")
		return(rep(NA,nrow(tt)))
	}
	res <- rowMaxs(rank.mat,na.rm=FALSE)
	res[res==-Inf] <- NA
	if (rerank) res <- rank(res,na.last="keep",ties.method="min")
	return(res)
}

### extractRankingCols
###
### extracts and transforms the relevant columns from a differential methylation table for ranking on the site and region level
### respectively
### @author Fabian Mueller
### @rdname extractRankingCols
### @aliases extractRankingCols
### @aliases extractRankingCols.site
### @aliases extractRankingCols.region
### @param tt differential methylation table
### @return a matrix containing the transformed and extracted values:
### \item{}{difference in mean methylation (negative absolute value) (mean of differences in means on the region level)}
### \item{}{(mean) quotient in mean methylation (negative absolute value of the logarithm) (mean of quotients in means on the region level)}
### \item{}{p-value from t-test. (combination of p-values using an extension of Fisher's method on the region level)}
extractRankingCols.site <- function(tt){
	return(cbind(-abs(tt$mean.diff),  -abs(tt$mean.quot.log2),  tt$diffmeth.p.val))
}
### @rdname extractRankingCols
extractRankingCols.region <- function(tt){
	return(cbind(-abs(tt$mean.mean.diff),  -abs(tt$mean.mean.quot.log2),  tt$comb.p.val))
}

### doPerm
###
### perform ONE permutation test
### @author Fabian Mueller
### @aliases doPerm
### @param b beta value matrix
### @param all.inds column indices in \code{b} to be used for permutation test
### @param n.inds.g1 number of columns in group 1
### @return vector of ranks resulting from the permutation test
doPerm <- function(b,all.inds,n.inds.g1,...){
	perm.inds.inds.g1 <- sample(1:length(all.inds),n.inds.g1)
	perm.inds.g1 <- all.inds[perm.inds.inds.g1]
	perm.inds.inds.g2 <- setdiff(1:length(all.inds),perm.inds.inds.g1)
	perm.inds.g2 <- all.inds[perm.inds.inds.g2]
	dm <- computeDiffTab.default.site(b,inds.g1=perm.inds.g1,inds.g2=perm.inds.g2,...)
	dm4ranking <- extractRankingCols.site(dm)
	perm.ranks.cur <- combinedRanking.tab(dm4ranking,rerank=FALSE)
	return(perm.ranks.cur)
}


### groupPermutationP.site
###
### computes a p-value for permuting the two sample groups by calculating combined ranks for each permutaion and scoring how many of them yield a better rank for each site.
### @author Fabian Mueller
### @aliases groupPermutationP.site
### @param b beta value matrix
### @param inds.1 column indices in \code{b} of group 1 members
### @param inds.g2 column indices in \code{b} of group 2 members
### @return a vector of p-values
groupPermutationP.site <- function(b,inds.g1,inds.g2,n.perm=500,...){
	report.interval <- 10
	all.inds <- c(inds.g1,inds.g2)
	n.inds.g1 <- length(inds.g1)
	dm <- computeDiffTab.default.site(b,inds.g1=inds.g1,inds.g2=inds.g2,...)
	dm4ranking <- extractRankingCols.site(dm)
	ranking.org <- combinedRanking.tab(dm4ranking,rerank=FALSE)
	perm.ranks <- matrix(NA,ncol=n.perm,nrow=nrow(b))
	for (i in 1:n.perm){
		if (i %% report.interval == 0) logger.status(c("Reached permutation",i))
		perm.ranks[,i] <- doPerm(b,all.inds,n.inds.g1)
	}
	p.perm <- rowSums(perm.ranks <  ranking.org)/n.perm #the m < v operator (m is a matrix, v is a vector) in R works in an equivalent way to the following procedure: m is regareded as the concatetened column vector. v is repeated. elementwise comparison. m is reassemled into a matrix.
	return(p.perm)
}

### @rdname groupPermutationP.site
groupPermutationP.site.parallel <- function(b,inds.g1,inds.g2,n.perm=500,...){
	all.inds <- c(inds.g1,inds.g2)
	n.inds.g1 <- length(inds.g1)
	dm <- computeDiffTab.default.site(b,inds.g1=inds.g1,inds.g2=inds.g2,...)
	dm4ranking <- extractRankingCols.site(dm)
	ranking.org <- combinedRanking.tab(dm4ranking,rerank=FALSE)
	perm.ranks <- foreach(i=1:n.perm,.combine='cbind') %dopar% doPerm(b,all.inds,n.inds.g1,...)
	p.perm <- rowSums(perm.ranks <  ranking.org)/n.perm #the m < v operator (m is a matrix, v is a vector) in R works in an equivalent way to the following procedure: m is regareded as the concatetened column vector. v is repeated. elementwise comparison. m is reassemled into a matrix.
	return(p.perm)
}

### computeDiffMeth.bin.site
###
### computes a differential methylation in the binary case (2 groups) on the site level.
### @author Fabian Mueller
### @aliases computeDiffMeth.bin.site
### @param b beta value matrix
### @param inds.1 column indices in \code{b} of group 1 members
### @param inds.g2 column indices in \code{b} of group 2 members
### @param n.perm number of permutations to be performed for the ranking permutaion tests. Set to values < 1 to disable permutation tests
### @return A data.frame containing differential methylation information with the variables from \code{\link{computeDiffTab.extended.site}} and additionally 
### \item{combinedRank}{the combined rank obtained from the the differential methylation information. As the the worst rank among all columns selected for the ranking.
### 					   \code{\link{extractRankingCols.site}} determines which these are.}
### \item{rankPermP}{[optional] p-value obtained from permuation tests of sample group assignments}
computeDiffMeth.bin.site <- function(b,inds.g1,inds.g2,n.perm=0,...){
	#sanity checks
	if (length(union(inds.g1,inds.g2)) != (length(inds.g1)+length(inds.g2))){
		logger.error("Overlapping sample sets in differential methylation analysis")
	}
	logger.start("Computing Differential Methylation Table")
	diffmeth.tab <- computeDiffTab.extended.site(b,inds.g1=inds.g1,inds.g2=inds.g2,...)
	diffmethTab4ranks <- extractRankingCols.site(diffmeth.tab)
	combRank <- combinedRanking.tab(diffmethTab4ranks,rerank=FALSE)
	diffmeth.tab$combinedRank <- combRank
	if(rnb.getOption("differential.variability")){
	  cols.rank.var <- cols.to.rank.site(diffmeth.tab)
	  comb.rank.var <- combinedRanking.tab(cols.rank.var,rerank=FALSE)
	  diffmeth.tab$combinedRank.var <- comb.rank.var
	}
	logger.completed()
	if (n.perm > 0){
		logger.start("Conducting Permutation Tests")
		if (n.perm > 2000) {
			logger.warning("The number of permutation tests conducted exceeds 2000. 
							Depending on the system's resources this could lead to errors")
		}
		if(parallel.isEnabled()) {
			logger.info("Using multicore")
			diffmeth.tab$rankPermP <- groupPermutationP.site.parallel(b,inds.g1,inds.g2,n.perm=n.perm)
		} else {
			logger.info("Using single core")
			diffmeth.tab$rankPermP <- groupPermutationP.site(b,inds.g1,inds.g2,n.perm=n.perm)
		}
		logger.completed()
	}
	# check if the residual matrix is preserved as an attribute
	
	# if (!is.null(attr(diffmeth.tab, "resid_mat"))) {
	# 	logger.info("bin.site: Residual matrix preserved in the returned table as an attribute")
	# } else {
	# 	logger.info("bin.site: Residual matrix NOT preserved in the returned table as an attribute")
	# }
	return(diffmeth.tab)
}

### computeDiffMeth.bin.region
###
### computes a differential methylation in the binary case (2 groups) on the region level.
### @author Fabian Mueller
### @aliases computeDiffMeth.bin.region
### @param dmtp differential methylation table on the site level (as obtained from \code{\link{computeDiffMeth.bin.site}})
### @param inds.1 column indices in \code{b} of group 1 members
### @param inds.g2 column indices in \code{b} of group 2 members
### @param n.perm number of permutations to be performed for the ranking permutaion tests. Set to values < 1 to disable permutation tests
### @return blubb
computeDiffMeth.bin.region <- function(rnbSet,dmtp,inds.g1,inds.g2,region.types=rnb.region.types.for.analysis(rnbSet), ...){
	#sanity checks
	if (length(union(inds.g1,inds.g2)) != (length(inds.g1)+length(inds.g2))){
		logger.error("Overlapping sample sets in differential methylation analysis")
	}
	logger.start('Computing Differential Methylation Tables (Region Level)')
	skip.sites <- FALSE
	if (is.null(dmtp)){
		logger.info("Computing differential methylation for regions directly (NOT using site-specific differential methylation)")
		skip.sites <- TRUE
	}
	diffmeth.tabs <- list()
	for (rt in region.types){
		if (skip.sites){
			covMat <- covg(rnbSet, rt)
			dmtr <- computeDiffMeth.bin.site(meth(rnbSet,rt), inds.g1, inds.g2, covg=covMat, ...)
		} else {
			inclCov <- !is.null(covg(rnbSet))
			regions2sites <- regionMapping(rnbSet,rt)
			regions2sites.is.all.na <- sapply(regions2sites,FUN=function(x){all(is.na(x))})
			if (any(regions2sites.is.all.na)) {
				stop(paste("Region mapping of RnBSet from sites to regions is inconsistent (",rt,")"))
			}
			# regions2sites <- regions2sites[!regions2sites.is.all.na]
			# regions2sites <- lapply(regions2sites,FUN=function(x){na.omit(x)})
			# attr(regions2sites, "omitted.regions") <- which(regions2sites.is.all.na)
			dmtr <- computeDiffTab.default.region(dmtp,regions2sites,includeCovg=inclCov)
			dmtr4ranks <- extractRankingCols.region(dmtr)
			combRank <- combinedRanking.tab(dmtr4ranks,rerank=FALSE)
			dmtr$combinedRank <- combRank
			if(rnb.getOption("differential.variability")){
			  diff.var.ranks <- cols.to.rank.region(dmtr)
			  comb.rank.var <- combinedRanking.tab(diff.var.ranks,rerank=FALSE)
			  dmtr$combinedRank.var <- comb.rank.var
			}
		}
		diffmeth.tabs <- c(diffmeth.tabs,list(dmtr))
		logger.status(c("Computed table for", rt))
	}
	names(diffmeth.tabs) <- region.types
	logger.completed()
	return(diffmeth.tabs)
}

#' exportDMRs2regionFile
#'
#' export differentially methylated regions to region file (standard bed). The output is in BED6 format where the score corresponds to 
#' to the combined rank (rank==1 would receive a score of 1000 and a combined rank equal to the number of regions a score of 0)
#' @author Fabian Mueller
#' @aliases exportDMRs2regionFile
#' @param rnbSet the RnBSet object for which the DMRs were computed.
#' @param diffmeth DiffMeth object. See \code{\link{rnb.execute.computeDiffMeth}} for details.
#' @param dest destination file name
#' @param comp.name name of the comparison
#' @param region.type region type.
#' @param rank.cut rank cutoff. If \code{NULL} (default), all regions are processed.
#' @param rerank flag indicating whether the ranks should be reranked or whether \code{rank.cut} refers to the absolute rank
#' @return \code{NULL}
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
#' exportDMRs2regionFile(rnb.set.example,dm,tempfile(),get.comparisons(dm)[1],"promoters")
#' }
exportDMRs2regionFile <- function(rnbSet,diffmeth,dest,comp.name,region.type,rank.cut=NULL,rerank=FALSE){
	annot <- annotation(rnbSet, type=region.type, add.names=FALSE)
#	dmt <- diffmeth$region[[comp]][[region.type]]
	dmt <- get.table(diffmeth,comp.name,region.type)
	is.dmr <- rep(TRUE,dim(dmt)[1])
	if (!is.null(rank.cut)){
		rrs <- dmt[,"combinedRank"]
		if (rerank){
			rrs <- rank(rrs,na.last="keep",ties.method="min")
		}
		is.dmr <- rrs<=rank.cut
		dmt <- dmt[is.dmr,]
		annot <- annot[is.dmr,]
	}
	n.regs <- nrow(dmt)
	score <- round((n.regs-dmt[,"combinedRank"]-1)/n.regs * 1000,0)
	reg.names <- paste0("reg",1:n.regs)
	if (!is.null(rownames(annot))){
		reg.names <- rownames(annot)
	}
	outtab <- data.frame(chr=annot$Chromosome,chromStart=annot$Start,chromEnd=annot$End,name=reg.names,score=score,strand=annot$Strand)
	write.table(outtab,sep="\t",quote=FALSE,col.names=FALSE,row.names=FALSE,file=dest)
	return(NULL)
}

#' auto.select.rank.cut
#'
#' automatically select a rank cutoff for given ranks and p-values
#' current implementation: sort the p-values according to rank. select as rank cutoff
#' the rank for which the worst (i.e. max) p-value in the top list is still smaller than
#' the best (i.e. min) p-value of the group of worst-ranking p-values of equal size as the top-list
#' @author Fabian Mueller
#' @param p vector of p-values
#' @param r vector of ranks
#' @param alpha the percentile to select the top and bottom part of the list
#' @return the maximum rank fulfilling the criterion
auto.select.rank.cut <- function(p,r,alpha=0.1){
	res <- 0
	lp <- length(p)
	j <- 1L:lp
	oa <- order(r)
	od <- rev(oa)
	p.oa <- p[oa]
	cmin.d <- cummin(p[od])
	cquant <- cummax(p.oa)
	inds.better.than.tail <- which(cquant<cmin.d)
	if (length(inds.better.than.tail) > 0){
		L <- max(inds.better.than.tail)
		res <- r[oa][L]
	}
	return(res)
}

### rnb.diffmeth.heatmap.select.rows
###
### selects row indices for differential methylation heatmaps
rnb.diffmeth.heatmap.select.rows <- function(
  dmt,
  selected.mask,
  max.features
) {
	if (length(selected.mask) != nrow(dmt)){
		stop("invalid value for selected.mask")
	}
	selected.inds <- which(selected.mask)
	if (length(selected.inds) < 1){
		return(integer())
	}
	ord <- order(dmt[selected.inds,"combinedRank"], na.last = TRUE)
	selected.inds <- selected.inds[ord]
	max.features <- max(1L, as.integer(max.features))
	if (length(selected.inds) > max.features){
		selected.inds <- selected.inds[1:max.features]
	}
	return(selected.inds)
}

### rnb.diffmeth.get.residual.matrix
###
### retrieves residual matrix for a given comparison and region type from RnBDiffMeth object
rnb.diffmeth.get.residual.matrix <- function(
  diffmeth,
  comparison,
  region.type,
  row.inds,
  sample.inds
) {
	if (is.null(diffmeth) || !inherits(diffmeth, "RnBDiffMeth")){
		return(NULL)
	}
	res.obj <- NULL
	if (region.type == "sites"){
		res.obj <- diffmeth@residSites[[comparison]]
	} else {
		if (!is.null(diffmeth@residRegions[[region.type]])){
			res.obj <- diffmeth@residRegions[[region.type]][[comparison]]
		}
	}
	if (is.null(res.obj)){
		return(NULL)
	}
	res.mat <- tryCatch({
		if (inherits(res.obj, "ff")) {
			as.matrix(res.obj[])
		} else {
			as.matrix(res.obj)
		}
	}, error = function(e) {
		logger.warning(c("Could not retrieve residual matrix for heatmap:", e$message))
		NULL
	})
	if (is.null(res.mat) || nrow(res.mat) < max(row.inds) || ncol(res.mat) < max(sample.inds)){
		return(NULL)
	}
	return(res.mat[row.inds, sample.inds, drop = FALSE])
}

### rnb.diffmeth.get.heatmap.signal.matrix
###
### retrieves matrix used for heatmap according to selected signal type
rnb.diffmeth.get.heatmap.signal.matrix <- function(
  rnbSet,
  diffmeth,
  comparison,
  region.type,
  row.inds,
  sample.inds,
  signal.type="beta"
) {
	if (signal.type == "beta"){
		return(meth(rnbSet, type = region.type, i = row.inds, j = sample.inds))
	}
	if (signal.type == "mvalue"){
		return(rnb.beta2mval(meth(rnbSet, type = region.type, i = row.inds, j = sample.inds)))
	}
	# residual signal
	return(
		rnb.diffmeth.get.residual.matrix(diffmeth, comparison, region.type, row.inds, sample.inds)
	)
}

### rnb.diffmeth.heatmap.row.zscore
###
### computes row-wise z-scores while preserving NAs
rnb.diffmeth.heatmap.row.zscore <- function(mm) {
	if (!(is.matrix(mm) && nrow(mm) > 0 && ncol(mm) > 0)){
		return(mm)
	}
	if (requireNamespace("matrixStats", quitely = TRUE)) {
		mx <- matrixStats::rowMeans2(mm, na.rm = TRUE)
		sx <- matrixStats::rowSds(mm, na.rm = TRUE)
		sx[is.na(sx) | sx == 0] <- Inf
		res <- (mm - mx) /sx
		res[is.infinite(sx), ] <- 0
	} else {
		res <- t(apply(mm, 1, function(x){
			sx <- stats::sd(x, na.rm = TRUE)
			mx <- mean(x, na.rm = TRUE)
			if (is.na(sx) || sx == 0){
				return(rep(0, length(x)))
			}
			(x - mx) / sx
		}))
		colnames(res) <- colnames(mm)
		rownames(res) <- rownames(mm)
	}
	res
}

### rnb.diffmeth.prepare.heatmap.signal
###
### applies optional transformation to selected heatmap signal matrix
rnb.diffmeth.prepare.heatmap.signal <- function(mm, do.zscore = FALSE) {
	if (isTRUE(do.zscore)){
		mm <- rnb.diffmeth.heatmap.row.zscore(mm)
	}
	return(mm)
}

### rnb.diffmeth.create.heatmap.plot
###
### helper for creating a differential methylation heatmap report plot
rnb.diffmeth.create.heatmap.plot <- function(
  report,
  figName,
  mm,
  grp1.size,
  grp2.size,
  grp1.name,
  grp2.name,
  cluster.rows = TRUE,
  cluster.columns = TRUE,
  show.row.names = FALSE,
  show.column.names = TRUE,
  legend.title = "signal"
) {
	report.plot <- createReportPlot(figName, report, width = 8, height = 8, create.pdf = FALSE, high.png = 200)
	if (!(is.matrix(mm) && nrow(mm) > 0 && ncol(mm) > 1)){
		print(rnb.message.plot("Heatmap could not be created"))
		return(off(report.plot))
	}
	if (all(is.na(mm))){
		print(rnb.message.plot("Heatmap not available (all values are NA)"))
		return(off(report.plot))
	}
	tryCatch({
		col_split <- factor(
			c(rep(grp1.name, grp1.size), rep(grp2.name, grp2.size)),
			levels = c(grp1.name, grp2.name)
		)
		max.val <- max(mm, na.rm = TRUE)
		min.val <- min(mm, na.rm = TRUE)
		mid.val <- ifelse(min.val < 0, 0, ave(c(min.val, max.val)))
		hm.cols <- circlize::colorRamp2(
			breaks = c(min.val, mid.val, max.val),
			colors = rnb.getOption("colors.meth")
		)
		grid::grid.newpage()
		hm <- ComplexHeatmap::Heatmap(
			mm,
			name = legend.title,
			col = hm.cols,
			cluster_rows = cluster.rows,
			cluster_columns = cluster.columns,
			show_row_names = show.row.names,
			row_names_side = "left",
			row_names_gp = grid::gpar(fontsize = 8),
			show_column_names = show.column.names,
			column_names_gp = grid::gpar(fontsize = 8),
			column_split = col_split,
			na_col = "grey90"
		)
		ComplexHeatmap::draw(hm, heatmap_legend_side = "right")
	}, error = function(e){
		logger.warning(c("Could not create differential methylation heatmap:", e$message))
		print(rnb.message.plot("Heatmap could not be created"))
	})
	report.plot <- off(report.plot)
	return(report.plot)
}

### addReportPlots.diffMeth.bin.site.heatmap
###
### adds differential methylation heatmaps for the site level binary case to a report for available signal types ((beta, m-value, residuals).
addReportPlots.diffMeth.bin.site.heatmap <- function(
  report,
  rnbSet, 
  diffmeth,
  comparison,
  dmt,
  cmpName,
  sample.inds,
  diffSiteRankCut, 
  autoRankCut = NULL,
  grp1.name = "Group1",
  grp2.name = "Group2",
  rerank = TRUE,
  thres.p.val = 0.05,
  max.features = rnb.getOption("differential.heatmap.top.features"),
  cluster.rows = rnb.getOption("differential.heatmap.cluster.rows"),
  cluster.columns = rnb.getOption("differential.heatmap.cluster.columns"),
  show.row.names = rnb.getOption("differential.heatmap.show.row.names"),
  show.column.names = rnb.getOption("differential.heatmap.show.column.names")
) {
	figPlots <- list()
	if (!(is.list(sample.inds) && all(c("group1","group2") %in% names(sample.inds)))){
		return(figPlots)
	}
	grp1.size <- length(sample.inds$group1)
	grp2.size <- length(sample.inds$group2)
	if (grp1.size < 1 || grp2.size < 1){
		return(figPlots)
	}
	all.sample.inds <- c(sample.inds$group1, sample.inds$group2)

	
	signals <- c("beta", "mvalue", "residuals")

	create.plot <- function(selected.mask, measure.id, signal.type, heatmap.zscore) {
		sel.inds <- rnb.diffmeth.heatmap.select.rows(dmt, selected.mask, max.features)
		zScore <- ifelse(isTRUE(heatmap.zscore), "zscore", "raw")
		figName <- paste("diffMeth_site_heatmap", cmpName, measure.id, signal.type, zScore, sep = "_")
		if (length(sel.inds) < 1){
			report.plot <- createReportPlot(figName, report, width = 8, height = 8, create.pdf = FALSE, high.png = 200)
			print(rnb.message.plot("No loci matched this criterion"))
			report.plot <- off(report.plot)
			return(report.plot)
		}
		mm <- rnb.diffmeth.get.heatmap.signal.matrix(
			rnbSet = rnbSet, 
			diffmeth = diffmeth,
			comparison = comparison,
			region.type = "sites",
			row.inds = sel.inds,
			sample.inds = all.sample.inds,
			signal.type = signal.type
		)
		if (is.null(mm)){
			report.plot <- createReportPlot(figName, report, width = 8, height = 8, create.pdf = FALSE, high.png = 200)
			print(rnb.message.plot(paste0("Signal type ", signal.type, " not available for this comparison")))
			report.plot <- off(report.plot)
			return(report.plot)
		}
		mm <- rnb.diffmeth.prepare.heatmap.signal(mm, do.zscore = heatmap.zscore)
		legend.title <- ifelse(isTRUE(heatmap.zscore), paste0(signal.type," (z-score)"), signal.type)
		return(
			rnb.diffmeth.create.heatmap.plot(
			  report,
			  figName,
			  mm,
			  grp1.size,
			  grp2.size,
			  grp1.name,
			  grp2.name,
			  cluster.rows = cluster.rows,
			  cluster.columns = cluster.columns,
			  show.row.names = show.row.names,
			  show.column.names = show.column.names,
			  legend.title = legend.title
			)
		)
	}

	create.plot.all.signals <- function(selected.mask, measure.id){
		plots.for.measure <- list()
		for (heatmap.zscore in c(FALSE, TRUE)) {
			for (sig in signals){
				plots.for.measure <- c(plots.for.measure, list(create.plot(selected.mask, measure.id, sig, heatmap.zscore)))
			}
		}
		return(plots.for.measure)
	}

	if (is.element("diffmeth.p.adj.fdr", colnames(dmt))){
		figPlots <- c(figPlots, create.plot.all.signals(dmt[,"diffmeth.p.adj.fdr"] < P.VAL.CUT, "fdrAdjPval"))
	}

	rrs <- dmt[,"combinedRank"]
	if (rerank) rrs <- rank(rrs, na.last = "keep", ties.method = "min")
	for (i in seq_along(diffSiteRankCut)){
		rc <- diffSiteRankCut[i]
		figPlots <- c(figPlots, create.plot.all.signals(rrs < rc, paste("rc", i, sep="")))
		if (rc >= max.features) {
			break
		}
	}
	if (is.integer(autoRankCut)){
		figPlots <- c(figPlots, create.plot.all.signals(dmt[,"combinedRank"] <= autoRankCut, "rcAuto"))
	}
	if (is.element("rankPermP", colnames(dmt))){
		figPlots <- c(figPlots, create.plot.all.signals(dmt[,"rankPermP"] < thres.p.val, "permutationP"))
	}

	return(figPlots)
}

### addReportPlots.diffMeth.bin.region.heatmap
###
### adds differential methylation heatmaps for the region level binary case to a report for available signal types ((beta, m-value, residuals).
addReportPlots.diffMeth.bin.region.heatmap <- function(
  report, 
  rnbSet, 
  diffmeth,
  comparison,
  dmt,
  cmpName,
  regName,
  region.type,
  sample.inds,
  diffRegionRankCut,
  autoRankCut = NULL,
  grp1.name = "Group1",
  grp2.name = "Group2",rerank = TRUE,
  max.features = rnb.getOption("differential.heatmap.top.features"),
  cluster.rows = rnb.getOption("differential.heatmap.cluster.rows"),
  cluster.columns = rnb.getOption("differential.heatmap.cluster.columns"),
  show.row.names = rnb.getOption("differential.heatmap.show.row.names"),
  show.column.names = rnb.getOption("differential.heatmap.show.column.names")
) {
	figPlots <- list()
	if (!(is.list(sample.inds) && all(c("group1","group2") %in% names(sample.inds)))){
		return(figPlots)
	}
	grp1.size <- length(sample.inds$group1)
	grp2.size <- length(sample.inds$group2)
	if (grp1.size < 1 || grp2.size < 1){
		return(figPlots)
	}
	all.sample.inds <- c(sample.inds$group1, sample.inds$group2)

	
	signals <- c("beta", "mvalue", "residuals")

	create.plot <- function(selected.mask, measure.id, signal.type, heatmap.zscore) {
		sel.inds <- rnb.diffmeth.heatmap.select.rows(dmt, selected.mask, max.features)
		zScore <- ifelse(isTRUE(heatmap.zscore), "zscore", "raw")
		figName <- paste("diffMeth_region_heatmap", cmpName, regName, measure.id, signal.type, zScore, sep = "_")
		if (length(sel.inds) < 1){
			report.plot <- createReportPlot(figName, report, width = 8, height = 8, create.pdf = FALSE, high.png = 200)
			print(rnb.message.plot("No loci matched this criterion"))
			report.plot <- off(report.plot)
			return(report.plot)
		}
		mm <- rnb.diffmeth.get.heatmap.signal.matrix(
			rnbSet = rnbSet,
			diffmeth = diffmeth,
			comparison = comparison,
			region.type = region.type,
			row.inds = sel.inds,
			sample.inds = all.sample.inds,
			signal.type = signal.type
		)
		if (is.null(mm)){
			report.plot <- createReportPlot(figName, report, width = 8, height = 8, create.pdf = FALSE, high.png = 200)
			print(rnb.message.plot(paste0("Signal type ", signal.type, " not available for this comparison")))
			report.plot <- off(report.plot)
			return(report.plot)
		}
		mm <- rnb.diffmeth.prepare.heatmap.signal(mm, do.zscore = heatmap.zscore)
		legend.title <- ifelse(isTRUE(heatmap.zscore), paste0(signal.type, " (z-score))"), signal.type)
		return(
			rnb.diffmeth.create.heatmap.plot(
			  report,
			  figName,
			  mm,
			  grp1.size,
			  grp2.size,
			  grp1.name,
			  grp2.name,
			  cluster.rows = cluster.rows,
			  cluster.columns = cluster.columns,
			  show.row.names = show.row.names,
			  show.column.names = show.column.names,
			  legend.title = legend.title
			)
		)
	}

	create.plot.all.signals <- function(selected.mask, measure.id){
		plots.for.measure <- list()
		for (heatmap.zscore in c(FALSE, TRUE)) {
			for (sig in signals){
				plots.for.measure <- c(plots.for.measure, list(create.plot(selected.mask, measure.id, sig, heatmap.zscore)))
			}
		}
		return(plots.for.measure)
	}

	if (is.element("comb.p.adj.fdr", colnames(dmt))){
		figPlots <- c(figPlots, create.plot.all.signals(dmt[,"comb.p.adj.fdr"] < P.VAL.CUT, "fdrAdjPval"))
	}

	rrs <- dmt[,"combinedRank"]
	if (rerank) rrs <- rank(rrs, na.last = "keep", ties.method = "min")
	for (i in seq_along(diffRegionRankCut)){
		rc <- diffRegionRankCut[i]
		figPlots <- c(figPlots, create.plot.all.signals(rrs < rc, paste("rc", i, sep="")))
		if (rc >= max.features) {
			break
		}
	}
	if (is.integer(autoRankCut)){
		figPlots <- c(figPlots, create.plot.all.signals(dmt[,"combinedRank"] <= autoRankCut, "rcAuto"))
	}

	return(figPlots)
}

### addReportPlots.diffMeth.bin.site.scatter
###
### adds report scatterplots for differential methylation for the site level binary case to a report.
### @author Fabian Mueller
### @aliases addReportPlots.diffMeth.bin.site.scatter
### @param report the report to be modified
### @param dmt differential methylation table as created by \code{computeDiffMeth.bin.site}
### @param cmpName Comparison name as it will appear in the filename and figure selection box
### @param diffSiteRankCut vector of combined ranking cutoffs for classifying a site as differentially methylated
### @param grp1.name name of group 1 in the compoarison (for labelling in the plots)
### @param grp2.name name of group 2 in the compoarison (for labelling in the plots)
### @return list of report plot objects added
addReportPlots.diffMeth.bin.site.scatter <- function(report,dmt,cmpName,diffSiteRankCut,autoRankCut=NULL,grp1.name="Group1",grp2.name="Group2",
		rerank=TRUE,thres.p.val=0.05){
	df2p <- dmt #data frame to plot
	figPlots <- list()
	
	sparse.points <- DENS.SCATTER.SPARSE.POINTS.PERC
	if (DENS.SCATTER.SPARSE.POINTS.MAX < sparse.points*nrow(df2p)){
		sparse.points <- DENS.SCATTER.SPARSE.POINTS.MAX
	}
	dens.subsample <- FALSE
	if (nrow(df2p) > DENS.SCATTER.SUBSAMPLE.THRES){
		dens.subsample <- DENS.SCATTER.SUBSAMPLE.THRES
	}

	#scatterplot based on adjusted p-value significance
	if (is.element("diffmeth.p.adj.fdr",colnames(dmt))){
		df2p$isDMP <- df2p[,"diffmeth.p.adj.fdr"] < P.VAL.CUT
		pp <- create.densityScatter(df2p[,c("mean.g1","mean.g2")],is.special=df2p$isDMP,
					dens.subsample=dens.subsample,sparse.points=sparse.points,add.text.cor=TRUE) +
				labs(x=paste("mean.beta",grp1.name,sep="."),y=paste("mean.beta",grp2.name,sep=".")) + coord_fixed()
		cur.cut.name <- "fdrAdjPval"
		figName <- paste("diffMeth_site",cmpName,cur.cut.name,sep="_")
		report.plot <- createReportGgPlot(pp,figName, report,create.pdf=FALSE,high.png=200)
		report.plot <- off(report.plot,handle.errors=TRUE)
		figPlots <- c(figPlots,list(report.plot))
	}
	
	#scatterplot based on rank cutoff significance
	rrs <- dmt[,"combinedRank"]
	if (rerank)	rrs <- rank(rrs,na.last="keep",ties.method="min")
	for (i in 1:length(diffSiteRankCut)){
		rc <- diffSiteRankCut[i]
		cur.cut.name <- paste("rc",i,sep="")
		df2p$isDMP <- rrs < rc
		
		pp <- create.densityScatter(df2p[,c("mean.g1","mean.g2")],is.special=df2p$isDMP,
					dens.subsample=dens.subsample,sparse.points=sparse.points,add.text.cor=TRUE) +
				labs(x=paste("mean.beta",grp1.name,sep="."),y=paste("mean.beta",grp2.name,sep=".")) + coord_fixed()
		figName <- paste("diffMeth_site",cmpName,cur.cut.name,sep="_")
		report.plot <- createReportGgPlot(pp,figName, report,create.pdf=FALSE,high.png=200)
		report.plot <- off(report.plot,handle.errors=TRUE)
		figPlots <- c(figPlots,list(report.plot))
	}
	if (is.integer(autoRankCut)){
		df2p$isDMP <- dmt[,"combinedRank"] <= autoRankCut
		pp <- create.densityScatter(df2p[,c("mean.g1","mean.g2")],is.special=df2p$isDMP,
					dens.subsample=dens.subsample,sparse.points=sparse.points,add.text.cor=TRUE) +
				labs(x=paste("mean.beta",grp1.name,sep="."),y=paste("mean.beta",grp2.name,sep=".")) + coord_fixed()
		figName <- paste("diffMeth_site",cmpName,"rcAuto",sep="_")
		report.plot <- createReportGgPlot(pp,figName, report,create.pdf=FALSE,high.png=200)
		report.plot <- off(report.plot,handle.errors=TRUE)
		figPlots <- c(figPlots,list(report.plot))
	}
	pp <- create.hex.summary.plot(df2p,q="combinedRank") + coord_fixed() +
			labs(x=paste("mean.beta",grp1.name,sep="."),y=paste("mean.beta",grp2.name,sep="."),fill = "median combined rank")
	figName <- paste("diffMeth_site",cmpName,"rankGradient",sep="_")
	report.plot <- createReportGgPlot(pp,figName, report,create.pdf=FALSE,high.png=200)
	report.plot <- off(report.plot,handle.errors=TRUE)
	figPlots <- c(figPlots,list(report.plot))

	if (is.element("rankPermP",colnames(dmt))){
		df2p$isDMP <- dmt[,"rankPermP"] < thres.p.val
		pp <- create.densityScatter(df2p[,c("mean.g1","mean.g2")],is.special=df2p$isDMP,
					dens.subsample=dens.subsample,sparse.points=sparse.points,add.text.cor=TRUE) +
				labs(x=paste("mean.beta",grp1.name,sep="."),y=paste("mean.beta",grp2.name,sep=".")) + coord_fixed()

		figName <- paste("diffMeth_site",cmpName,"permutationP",sep="_")
		report.plot <- createReportGgPlot(pp,figName, report,create.pdf=FALSE,high.png=200)
		report.plot <- off(report.plot,handle.errors=TRUE)
		figPlots <- c(figPlots,list(report.plot))
	}

	return(figPlots)
}

### addReportPlots.diffMeth.bin.site.volcano
###
### adds report volcano plots for differential methylation for the site level binary case to a report.
### @author Fabian Mueller
### @aliases addReportPlots.diffMeth.bin.site.volcano
### @param report the report to be modified
### @param dmt differential methylation table as created by \code{computeDiffMeth.bin.site}
### @param cmpName Comparison name as it will appear in the filename and figure selection box
### @param grp1.name name of group 1 in the compoarison (for labelling in the plots)
### @param grp2.name name of group 2 in the compoarison (for labelling in the plots)
### @return list of report plot objects added
addReportPlots.diffMeth.bin.site.volcano <- function(report,dmt,cmpName,grp1.name="Group1",grp2.name="Group2"){
	df2p <- dmt #data frame to plot
	figPlots <- list()
	dont.plot.p.val <- all(is.na(df2p[,"diffmeth.p.val"]))
	
	figName <- paste("diffMeth_site_volcano",cmpName,"diff","pVal",sep="_")
	if (!dont.plot.p.val){
		pp <- ggplot(df2p) + aes(mean.diff,-log10(diffmeth.p.val),color=log10(combinedRank)) + scale_color_gradientn(colours=rev(rnb.getOption("colors.gradient"))) +
		  				 geom_point()#(alpha=0.3)
	 } else {
		pp <- rnb.message.plot("No p-value available")
	 }
	report.plot <- createReportGgPlot(pp,figName, report,create.pdf=FALSE,high.png=200)
	report.plot <- off(report.plot,handle.errors=TRUE)
	figPlots <- c(figPlots,list(report.plot))
	
	#order the plotting data frame according to rank (descending order to plot best/lowest-ranking last)
	df2p <- df2p[order(df2p[,"combinedRank"], na.last=FALSE, decreasing=TRUE),]

	figName <- paste("diffMeth_site_volcano",cmpName,"quot","pVal",sep="_")
	if (!dont.plot.p.val){
		pp <- ggplot(df2p) + aes(mean.quot.log2,-log10(diffmeth.p.val),color=log10(combinedRank)) + scale_color_gradientn(colours=rev(rnb.getOption("colors.gradient"))) +
				geom_point()
	} else {
		pp <- rnb.message.plot("No p-value available")
	}
	report.plot <- createReportGgPlot(pp,figName, report,create.pdf=FALSE,high.png=200)
	report.plot <- off(report.plot,handle.errors=TRUE)
	figPlots <- c(figPlots,list(report.plot))
	
	figName <- paste("diffMeth_site_volcano",cmpName,"diff","pValAdj",sep="_")
	pp <- ggplot(df2p) + aes(mean.diff,-log10(diffmeth.p.adj.fdr),color=log10(combinedRank)) + 
		  scale_color_gradientn(colours=rev(rnb.getOption("colors.gradient"))) +
		  geom_point()
	report.plot <- createReportGgPlot(pp,figName, report,create.pdf=FALSE,high.png=200)
	report.plot <- off(report.plot,handle.errors=TRUE)
	figPlots <- c(figPlots,list(report.plot))
	
	figName <- paste("diffMeth_site_volcano",cmpName,"quot","pValAdj",sep="_")
	pp <- ggplot(df2p) + aes(mean.quot.log2,-log10(diffmeth.p.adj.fdr),color=log10(combinedRank)) + 
		  scale_color_gradientn(colours=rev(rnb.getOption("colors.gradient"))) +
		  geom_point()
	report.plot <- createReportGgPlot(pp,figName, report,create.pdf=FALSE,high.png=200)
	report.plot <- off(report.plot,handle.errors=TRUE)
	figPlots <- c(figPlots,list(report.plot))
	
	#mean vs quotient plot
	figName <- paste("diffMeth_site_volcano",cmpName,"diff","quotSig",sep="_")
	pp <- ggplot(df2p) + aes(mean.diff,mean.quot.log2,color=log10(combinedRank)) + scale_color_gradientn(colours=rev(rnb.getOption("colors.gradient"))) +
			geom_point()
	report.plot <- createReportGgPlot(pp,figName, report,create.pdf=FALSE,high.png=200)
	report.plot <- off(report.plot,handle.errors=TRUE)
	figPlots <- c(figPlots,list(report.plot))
	
	figName <- paste("diffMeth_site_volcano",cmpName,"quot","quotSig",sep="_")
	pp <- rnb.message.plot("Quotient--Quotient scatterplot not available")
	report.plot <- createReportGgPlot(pp,figName, report,create.pdf=FALSE,high.png=200)
	report.plot <- off(report.plot,handle.errors=TRUE)
	figPlots <- c(figPlots,list(report.plot))
	
	return(figPlots)
}

### addReportPlots.diffMeth.bin.region.scatter
###
### adds report plots for differential methylation for the region level binary case to a report.
### @author Fabian Mueller
### @aliases addReportPlots.diffMeth.bin.region.scatter
### @param report the report to be modified
### @param dmt differential methylation table as created by \code{computeDiffMeth.bin.region}
### @param cmpName Comparison name as it will appear in the filename and figure selection box
### @param regName Region type name as it will appear in the filename and figure selection box
### @param diffSiteRankCut vector of combined ranking cutoffs for classifying a site as differentially methylated
### @param grp1.name name of group 1 in the compoarison (for labelling in the plots)
### @param grp2.name name of group 2 in the compoarison (for labelling in the plots)
### @param useSiteCols use site specific column names instead of region ones
### @return list of report plot objects added
addReportPlots.diffMeth.bin.region.scatter <- function(report,dmt,cmpName,regName,diffRegionRankCut,autoRankCut=NULL,grp1.name="Group1",grp2.name="Group2",rerank=TRUE,useSiteCols=FALSE){
	df2p <- dmt #data frame to plot
	figPlots <- list()
	
	cn.x <- "mean.mean.g1"
	cn.y <- "mean.mean.g2"
	al.x <- paste("mean.mean.beta",grp1.name,sep=".")
	al.y <- paste("mean.mean.beta",grp2.name,sep=".")
	if (useSiteCols){
		cn.x <- "mean.g1"
		cn.y <- "mean.g2"
		al.x <- paste("mean.beta",grp1.name,sep=".")
		al.y <- paste("mean.beta",grp2.name,sep=".")
	}

	#subsampling for the densitity estimation when there are too many regions
	dens.subsample <- FALSE
	if (nrow(df2p) > DENS.SCATTER.SUBSAMPLE.THRES){
		dens.subsample <- DENS.SCATTER.SUBSAMPLE.THRES
	}

	#scatterplot based on adjusted p-value significance
	if (is.element("comb.p.adj.fdr",colnames(dmt))){
		df2p$isDMR <- df2p[,"comb.p.adj.fdr"] < P.VAL.CUT

		pp <- create.densityScatter(df2p[,c(cn.x, cn.y)],is.special=df2p$isDMR,dens.subsample=dens.subsample,add.text.cor=TRUE) +
				labs(x=al.x, y=al.y) + coord_fixed()
		cur.cut.name <- "fdrAdjPval"
		figName <- paste("diffMeth_region",cmpName,regName,cur.cut.name,sep="_")
		report.plot <- createReportGgPlot(pp,figName, report,create.pdf=FALSE,high.png=200)
		report.plot <- off(report.plot,handle.errors=TRUE)
		figPlots <- c(figPlots,list(report.plot))
	}
	
	rrs <- dmt[,"combinedRank"]
	if (rerank)	rrs <- rank(rrs,na.last="keep",ties.method="min")
	for (i in 1:length(diffRegionRankCut)){
		rc <- diffRegionRankCut[i]
		cur.cut.name <- paste("rc",i,sep="")
		df2p$isDMR <- rrs < rc
		
		pp <- create.densityScatter(df2p[,c(cn.x, cn.y)],is.special=df2p$isDMR,dens.subsample=dens.subsample,add.text.cor=TRUE) +
				labs(x=al.x, y=al.y) + coord_fixed()
		
		figName <- paste("diffMeth_region",cmpName,regName,cur.cut.name,sep="_")
		report.plot <- createReportGgPlot(pp,figName, report,create.pdf=FALSE,high.png=200)
		report.plot <- off(report.plot,handle.errors=TRUE)
		figPlots <- c(figPlots,list(report.plot))
	}
	
	if (is.integer(autoRankCut)){
		df2p$isDMR <- dmt[,"combinedRank"] <= autoRankCut
		pp <- create.densityScatter(df2p[,c(cn.x, cn.y)],is.special=df2p$isDMR,dens.subsample=dens.subsample,add.text.cor=TRUE) +
				labs(x=al.x, y=al.y) + coord_fixed()
		figName <- paste("diffMeth_region",cmpName,regName,"rcAuto",sep="_")
		report.plot <- createReportGgPlot(pp,figName, report,create.pdf=FALSE,high.png=200)
		report.plot <- off(report.plot,handle.errors=TRUE)
		figPlots <- c(figPlots,list(report.plot))
	}

	figName <- paste("diffMeth_region",cmpName,regName,"rankGradient",sep="_")
	pp <- create.hex.summary.plot(df2p,q="combinedRank") + coord_fixed() +
		labs(x=al.x, y=al.y, fill="median combined rank")
	report.plot <- createReportGgPlot(pp,figName, report,create.pdf=FALSE,high.png=200)
	report.plot <- off(report.plot,handle.errors=TRUE)
	figPlots <- c(figPlots,list(report.plot))
	
	return(figPlots)
}

### addReportPlots.diffMeth.bin.region.volcano
###
### adds report volcano plots for differential methylation for the region level binary case to a report.
### @author Fabian Mueller
### @aliases addReportPlots.diffMeth.bin.region.volcano
### @param report the report to be modified
### @param dmt differential methylation table as created by \code{computeDiffMeth.bin.region}
### @param cmpName Comparison name as it will appear in the filename and figure selection box
### @param regName Region type name as it will appear in the filename and figure selection box
### @param grp1.name name of group 1 in the compoarison (for labelling in the plots)
### @param grp2.name name of group 2 in the compoarison (for labelling in the plots)
### @param useSiteCols use site specific column names instead of region ones
### @return list of report plot objects added
addReportPlots.diffMeth.bin.region.volcano <- function(report,dmt,cmpName,regName,grp1.name="Group1",grp2.name="Group2",useSiteCols=FALSE){
	cn.d <- "mean.mean.diff"
	cn.q <- "mean.mean.quot.log2"
	cn.p <- "comb.p.val"
	cn.pa <- "comb.p.adj.fdr"
	if (useSiteCols){
		cn.d <- "mean.diff"
		cn.q <- "mean.quot.log2"
		cn.p <- "diffmeth.p.val"
		cn.pa <- "diffmeth.p.adj.fdr"
	}

	df2p <- dmt #data frame to plot
	figPlots <- list()
	dont.plot.p.val <- all(is.na(df2p[,cn.p]))

	#order the plotting data frame according to rank (descending order to plot best/lowest-ranking last)
	df2p <- df2p[order(df2p[,"combinedRank"], na.last=FALSE, decreasing=TRUE),]
	
	figName <- paste("diffMeth_region_volcano",cmpName,regName,"diff","pVal",sep="_")
	if (!dont.plot.p.val){
		pp <- ggplot(df2p) + aes_string(cn.d, paste0("-log10(",cn.p,")"), color="log10(combinedRank)") +
			scale_color_gradientn(colours=rev(rnb.getOption("colors.gradient"))) +
			geom_point()#(alpha=0.3)
	} else {
		pp <- rnb.message.plot("No p-value available")
	}
	report.plot <- createReportGgPlot(pp,figName, report,create.pdf=FALSE,high.png=200)
	report.plot <- off(report.plot,handle.errors=TRUE)
	figPlots <- c(figPlots,list(report.plot))
	
	figName <- paste("diffMeth_region_volcano",cmpName,regName,"diff","pValAdj",sep="_")
	pp <- ggplot(df2p) + aes_string(cn.d, paste0("-log10(",cn.pa,")"), color="log10(combinedRank)") +
		scale_color_gradientn(colours=rev(rnb.getOption("colors.gradient"))) +
		geom_point()#(alpha=0.3)
	report.plot <- createReportGgPlot(pp,figName, report,create.pdf=FALSE,high.png=200)
	report.plot <- off(report.plot,handle.errors=TRUE)
	figPlots <- c(figPlots,list(report.plot))
	
	figName <- paste("diffMeth_region_volcano",cmpName,regName,"quot","pVal",sep="_")
	if (!dont.plot.p.val){
		pp <- ggplot(df2p) + aes_string(cn.q, paste0("-log10(",cn.p,")"), color="log10(combinedRank)") +
			scale_color_gradientn(colours=rev(rnb.getOption("colors.gradient"))) +
			geom_point()#(alpha=0.3)
	} else {
		pp <- rnb.message.plot("No p-value available")
	}
	report.plot <- createReportGgPlot(pp,figName, report,create.pdf=FALSE,high.png=200)
	report.plot <- off(report.plot,handle.errors=TRUE)
	figPlots <- c(figPlots,list(report.plot))
	
	figName <- paste("diffMeth_region_volcano",cmpName,regName,"quot","pValAdj",sep="_")
	pp <- ggplot(df2p) + aes_string(cn.q, paste0("-log10(",cn.pa,")"), color="log10(combinedRank)") +
		scale_color_gradientn(colours=rev(rnb.getOption("colors.gradient"))) +
		geom_point()#(alpha=0.3)
	report.plot <- createReportGgPlot(pp,figName, report,create.pdf=FALSE,high.png=200)
	report.plot <- off(report.plot,handle.errors=TRUE)
	figPlots <- c(figPlots,list(report.plot))
	
	figName <- paste("diffMeth_region_volcano",cmpName,regName,"diff","quotSig",sep="_")
	pp <- ggplot(df2p) + aes_string(cn.d, cn.q, color="log10(combinedRank)") +
		scale_color_gradientn(colours=rev(rnb.getOption("colors.gradient"))) +
		geom_point()#(alpha=0.3)
	report.plot <- createReportGgPlot(pp, figName, report,create.pdf=FALSE,high.png=200)
	report.plot <- off(report.plot,handle.errors=TRUE)
	figPlots <- c(figPlots,list(report.plot))
	
	figName <- paste("diffMeth_region_volcano",cmpName,regName,"quot","quotSig",sep="_")
	pp <- rnb.message.plot("Quotient--Quotient scatterplot not available")
	report.plot <- createReportGgPlot(pp,figName, report,create.pdf=FALSE,high.png=200)
	report.plot <- off(report.plot,handle.errors=TRUE)
	figPlots <- c(figPlots,list(report.plot))
	
	return(figPlots)
}

### a more robust version of summary.GOHyperGResult (from GOstats)
robustHyperGResultSummary <- function(hgr,maxPval=0.01,htmlLinks=FALSE){
	rnb.require("Category")
	AMIGO_URL <- "http://amigo.geneontology.org/amigo/term/%s"
	GOenv <- function(what) {
		annotate::getAnnMap(what, "GO", load=TRUE, type=c("db", "env"))
	}
	goIds <- sigCategories(hgr, maxPval)
	pvals <- pvalues(hgr)[goIds]
	odds <- oddsRatios(hgr)[goIds]
	ec <- expectedCounts(hgr)[goIds]
	cc <- geneCounts(hgr)[goIds]
	ss <- universeCounts(hgr)[goIds]
	go.info <- BiocGenerics::mget(goIds, GOenv("TERM"), ifnotfound=NA)
	if(any(is.na(go.info))){
	  logger.info(paste("Invalid GOID found, removing",names(go.info[is.na(go.info)]),collapse = " "))
	}    
	goTerms <- rep(NA,length(cc))
	goTerms[!is.na(go.info)] <- sapply(go.info[!is.na(go.info)],Term)
	if (htmlLinks) {
		goTerms <- paste0('<a href="', sprintf(AMIGO_URL, goIds), '">', goTerms, '</a>')
	}
	tt <- data.frame(GOMFID=goIds,Pvalue=pvals,OddsRatio=odds,ExpCount=ec,Count=cc,Size=ss, Term=goTerms)
	return(tt)
}

### addReportPlots.diffMeth.enrich.GO.wordcloud
###
### adds a wordcloud for differential methylation GO annotation to the report
### @author Fabian Mueller
### @param report the report to be modified
### @param hgr GOHyperGREsult object
### @param figName name for the figure as it will appear in the filename
### @return report plot object
addReportPlots.diffMeth.enrich.GO.wordcloud <- function(report,hgr,figName){
	report.plot <- createReportPlot(figName, report,create.pdf=FALSE,high.png=200)
	if (!is.null(hgr)) {	
		tt <- robustHyperGResultSummary(hgr,maxPval=1+1e-10,htmlLinks=FALSE)
		color.scale <- gplots::colorpanel(n=10,low=rnb.getOption("colors.gradient")[2],high=rnb.getOption("colors.gradient")[1])
		suppressWarnings(wordcloud::wordcloud(tt$Term,-log(tt$Pvalue)+1e-10, scale=c(3,.04),min.freq=0,max.words=Inf,
				random.order=TRUE, rot.per=0, colors=color.scale, vfont=c("sans serif","plain")))
	} else {
		print(rnb.message.plot("Enrichment not available"))
	}
	report.plot <- off(report.plot)
	return(report.plot)
}

### rnb.section.diffMeth.introduction
###
### add information to the report for site level analysis
### @author Fabian Mueller
### @aliases rnb.section.diffMeth.introduction
### @param diffmeth RnBDiffMeth object. See \code{\link{RnBDiffMeth-class}} for details.
### @param report report object to which the content is added
### @return the updated report object
rnb.section.diffMeth.introduction <- function(diffmeth,report){
	sectionText <- c("Differential methylation analysis was conducted on site and region level according to the ",
		"sample groups specified in the analysis.")
	report <- rnb.add.section(report, "Introduction: Differential Methylation of Sample Groups", sectionText)
	report <- rnb.add.section(report, "Comparisons", "The following comparisons were made:", level = 2)
	group.sizes <- get.comparison.groupsizes(diffmeth)
	group.size.string <- paste0("[",group.sizes[,1]," vs. ",group.sizes[,2],"]")
	rnb.add.list(report, as.list(paste(get.comparisons(diffmeth), group.size.string)))

	if (.hasSlot(diffmeth,"comparison.info")) { #.hasSlot ensure backwards compatibility
		#construct comparison summary table
		cmp.info <- diffmeth@comparison.info
		is.paired <- vapply(cmp.info,FUN=function(x){x$paired},logical(1))
		is.adj.sva <- vapply(cmp.info,FUN=function(x){x$adj.sva},logical(1))
		is.adj.celltype <- vapply(cmp.info,FUN=function(x){x$adj.celltype},logical(1))
		adj.vars <- lapply(cmp.info,FUN=function(x){
			if (is.null(x$adjustment.table)){
				return(c())
			} else {
				return(colnames(x$adjustment.table))
			}
		})
		has.adj.var <- vapply(adj.vars,FUN=function(x){length(x)>0},logical(1))
		summary.tab <- data.frame(comparison=names(cmp.info))
		if (any(is.paired)){
			summary.tab <- data.frame(summary.tab,paired=is.paired)
		}
		if (any(is.adj.sva)){
			summary.tab <- data.frame(summary.tab,SVAdjust=is.adj.sva)
		}
		if (any(has.adj.var)){
			adj.vars.text <- sapply(adj.vars,FUN=function(x){
				paste(x,collapse=",")
			})
			#write the covariate table to the report directories
			covar.tab.handler <- function(i){
				ccn <- names(cmp.info)[i]
				x <- cmp.info[[i]]
				adj.tab <- x$adjustment.table
				if (is.null(adj.tab)){
					return("")
				}
				if (!is.null(rownames(adj.tab))){
					adj.tab <- data.frame(sample=rownames(adj.tab),adj.tab)
				}
				fname <- paste("adjTable_",i,".csv",sep="")
				fname.rel <- rnb.write.table(
					adj.tab, fname, fpath=rnb.get.directory(report, "data", absolute = TRUE),
					format="csv", gz=FALSE, row.names = FALSE, quote=FALSE
				)
				txt <- paste(c("<a href=\"", rnb.get.directory(report, "data"), "/", fname.rel,"\">","csv","</a>"),collapse="")
				return(txt)
			}
			covar.tab.links <- sapply(1:length(cmp.info),covar.tab.handler)
			summary.tab <- data.frame(summary.tab, adjustment=adj.vars.text, covariateTable=covar.tab.links)
		}
		if (ncol(summary.tab) > 1){
			rownames(summary.tab) <- 1:nrow(summary.tab)
			txt <- c(
				"The table below summarizes information on the comparisons.")
			rnb.add.paragraph(report, txt)
			rnb.add.table(report, summary.tab, first.col.header=TRUE)
		}
	}

	#include information on the p-value method
	site.test.method <- get.site.test.method(diffmeth)
	txt <- c(
		"In the following anlyses, p-values on the site level were computed using the <code>",site.test.method,"</code> method. "
	)
	if (site.test.method == "limma"){
		txt <- c(txt,
			"I.e. hierarchical linear models from the <a href=http://bioinf.wehi.edu.au/limma/>limma</a> package were employed ",
			"and fitted using an empirical Bayes approach on derived M-values."
		)
	} else if (site.test.method == "ttest"){
		txt <- c(txt,
			"I.e. a two-sided Welch t-test was employed."
		)
	} else if (site.test.method == "refFreeEWAS"){
		refText <- c(
			"Houseman, E. A., Molitor, J., and Marsit, C. J. (2014). Reference-Free Cell Mixture Adjustments in Analysis of DNA Methylation Data. ",
			"<i>Bioinformatics</i>, <a href=http://dx.doi.org/doi:10.1093/bioinformatics/btu029>doi:10.1093/bioinformatics/btu029</a>"
		)
		report <- rnb.add.reference(report, refText)

		txt <- c(txt,
			"I.e. the model from the <a href=http://cran.r-project.org/web/packages/RefFreeEWAS/index.html/>RefFreeEWAS</a> package was employed ",
			"which accounts for tissue heterogeneity in the profiled samples.", "See ",
			rnb.get.reference(report, refText), " for the detailed description of this method.")
	}
	report <- rnb.add.section(report, "P-values", txt, level = 2)

	logger.status("Added introductory section")
	return(report)
}


### rnb.section.replicate.concordance
###
### add information to the report for sample replicat analysis, such as scatterplots
### @author Fabian Mueller
### @param replicateList a list containing replicates as returned by \link{rnb.sample.replicates}
### @param types the vector of site and region types on which the analysis should be conducted
### @param report report object to which the content is added
### @return the updated report object
rnb.section.replicate.concordance <- function(rnbSet,replicateList,types,report){
	if (length(replicateList)<1){
		stop("no valid replicates")
	}
	logger.start("Adding Replicate Level Information")
	sectionText <- c("Sample replicates were compared. This section shows pairwise scatterplots for each sample ",
		"replicate group on both site and region level.")
	report <- rnb.add.section(report, "Analysis of Sample Replicates", sectionText)
	#scatterplots
	logger.start("Adding Scatterplots")
	rep.n <- c()
	rep.v <- c()
	rep.cor <- c()
	addedPlots <- list()
	for (k in 1:length(types)){
		rep.cor.t <- c()
		tt <- types[k]
		cur.type.fname <- paste("type",k,sep="")
		for (i in 1:length(replicateList)){
			rr <- names(replicateList)[i]
			rep.inds <- replicateList[[i]]
			if (length(rep.inds)>1){
				comps <- combn(rep.inds,2) #generate pairwise comparisons --> matrix with 2 rows and (length(rep.inds) choose 2) columns
				for (j in 1:ncol(comps)){
					dd <- data.frame(meth(rnbSet,type=tt)[,comps[,j]])
					cur.cor <- cor(dd[,1],dd[,2],use="pairwise.complete.obs")
					rep.cor.t <- c(rep.cor.t,cur.cor)
					s1 <- colnames(dd)[1]
					s2 <- colnames(dd)[2]
					cur.rep.cmp.fname <- paste("rep",paste(i,j,sep="c"),sep="")
					rep.n <- c(rep.n,cur.rep.cmp.fname)
					rep.v <- c(rep.v,paste(s1," vs. ",s2," (",rr,")",sep=""))
					figName <- paste("replicateScatter",cur.rep.cmp.fname,cur.type.fname,sep="_")
					
					pp <- create.densityScatter(dd[,c(s1,s2)],is.special=NULL,add.text.cor=TRUE) + coord_fixed()
					
					report.plot <- createReportGgPlot(pp,figName, report,create.pdf=FALSE,high.png=200)
					report.plot <- off(report.plot,handle.errors=TRUE)
					addedPlots <- c(addedPlots,list(report.plot))
				}
			}
		}
		rep.cor <- cbind(rep.cor,rep.cor.t)
		logger.status(c("Processed", tt))
	}
	names(types) <- paste("type",1:length(types),sep="")
	rep.n <- unique(rep.n)
	rep.v <- unique(rep.v)
	names(rep.v) <- rep.n
	setting.names <- list(
			'replicate' = rep.v,
			'site/region' = types)
	description <- 'Scatterplot for replicate methylation comparison.
					The transparency corresponds to point density.
					The 1% of the points in the sparsest populated plot regions are drawn explicitly.'
	report <- rnb.add.figure(report, description, addedPlots, setting.names)
	
	tt <- data.frame(rep.cor)
	colnames(tt) <- types
	rownames(tt) <- rep.v
	tt <- round(tt,4)
	
	txt <- c("The following table contains pearson correlation coefficients:")
	rnb.add.paragraph(report, txt)
	rnb.add.table(report,tt)
	
	logger.completed()
	logger.completed()
	return(report)
}

get.diffmeth.tab.col.desc.list.txt <- function(target, includeCovg, hasVariability, covgThres=-1L, comb.p.ref.txt="", skipSites=FALSE){
	targ <- "site"
	if (target=="regions") targ <- "region"

	res <- list(
		paste0("id: ", targ, " id"),
		paste0("Chromosome: chromosome of the ", targ),
		paste0("Start: start coordinate of the ", targ)
	)
	if (target=="regions") {
		res <- c(res, list(
			paste0("End: end coordinate of the ", targ)
		))
	} else if (target=="sites") {
		res <- c(res, list(
			paste0("Strand: strand of the ", targ)
		))
	}
	if (target=="sites" || skipSites) {
		res <- c(res, list(
			paste0("mean.g1,mean.g2: (where g1 and g2 is replaced by the respective group names in the table) mean methylation in each of the two groups"),
			paste0("mean.diff: difference in methylation means between the two groups: mean.g1-mean.g2. In case of paired analysis, it is the mean of the pairwise differences."),
			paste0("mean.quot.log2: log2 of the quotient in methylation: log2((mean.g1+epsilon)/(mean.g2+epsilon)), where epsilon:=0.01. In case of paired analysis, it is the mean of the pairwise quotients."),
			paste0("diffmeth.p.val: p-value obtained from linear models employed in the limma package (or alternatively from a two-sided Welch t-test; which type of p-value is computed is specified in the differential.site.test.method option)."),
			paste0("max.g1,max.g2: maximum methylation level in group 1 and 2 respectively"),
			paste0("min.g1,min.g2: minimum methylation level in group 1 and 2 respectively"),
			paste0("sd.g1,sd.g2: standard deviation of methylation levels"),
			paste0("min.diff: Minimum of 0 and the smallest pairwise difference between samples of the two groups"),
			paste0("diffmeth.p.adj.fdr: FDR adjusted p-value of all sites"),
			paste0("combinedRank: mean.diff, mean.quot.log2 and diffmeth.p.val are ranked for all ",target,". This aggregates them using the maximum, i.e. worst rank of a site among the three measures"),
			paste0("num.na.g1,num.na.g2: number of NA methylation values for groups 1 and 2 respectively"),
			paste0("resid.mean.g1,resid.mean.g2,resid.mean.diff: mean residual (M-value) for groups 1 and 2 respectively. (residuals: M-values with estimated covariate contributions subtracted)."),
			paste0("resid.mean.diff: difference of per-group residual means (resid.mean.g1 - resid.mean.g2)")
		))
		if (includeCovg){
			res <- c(res, list(
				paste0("mean.covg.g1,mean.covg.g2: mean coverage of groups 1 and 2 respectively (In case of Infinium array methylation data, coverage is defined as combined beadcount.)"),
				paste0("min.covg.g1,min.covg.g2: minimum coverage of groups 1 and 2 respectively"),
				paste0("max.covg.g1,max.covg.g2: maximum coverage of groups 1 and 2 respectively"),
				paste0("covg.thresh.nsamples.g1,covg.thresh.nsamples.g2: number of samples in group 1 and 2 respectively exceeding the coverage threshold (", covgThres, ") for this ", targ,".")
			))
		}
		if(hasVariability){
		  res <- c(res, list(
		    "Strand: strand of the site",
		    c("var.g1, var.g2: (g1 and g2 are replaced by the corrspondinhg group used in the differentiality analysis) ",
		      "the variances found in the groups"),
		    "var.diff: difference in variance values between the two groups g1 and g2 (=var.g1-var.g2). In case of paired analysis, it is the variance of the pairwise differences.",
		    "var.log.ratio: Log2 of the ratio between the variances of the two groups g1 and g2 (=log2(var.g1+eps/var.g2+eps), default eps=0.01). In case of paired analysis, it is the variance of the pairwise quotients.",
		    "diffVar.p.val: p-value resulting from applying the selected differentially variability method (diffVar or iEVORA)",
		    "diffVar.p.adj.fdr: FDR-adjusted p-value for differential variability",
		    "log10P: negative decadic logarithm of the p-value",
		    "log10FDR: negative decadic logarithmn of the FDR-adjusted p-value",
		    "combinedRank.var: var.diff, var.log.ratio and comb.p.val.var are ranked for all sites and the higher (=worst) rank is selected. "
		  ))
		}
	} else {
		res <- c(res, list(
			"[symbol]: associated gene symbol to the given region [only valid for gene associated regions]",
			"[entrezID]: Entrez ID of the gene associated with the region [only valid for gene associated regions]",
			"mean.mean.g1,mean.mean.g2: (where g1 and g2 is replaced by the respective group names in the table) mean of mean methylation levels for group 1 and 2 across all sites in a region",
			"mean.mean.diff: Mean difference in means across all sites in a region",
			"mean.mean.quot.log2: log2 of the mean quotient in means across all sites in a region",
			c("comb.p.val: Combined p-value aggregating p-values of all sites in the region using a generalization of Fisher's method ", comb.p.ref.txt),
			"comb.p.adj.fdr: FDR adjusted combined p-value",
			c("combinedRank: mean.mean.diff, mean.mean.quot.log2 and comb.p.val are ranked for all regions. ",
					"This column aggregates them using the maximum, i.e. worst rank of a site among the three measures"),
			"num.sites: number of sites associated with the region",
			"mean.num.na.g1,mean.num.na.g2: Mean number of NA methylation values accross all sites in group 1 and group 2 respectively"
		))
		if (includeCovg){
			res <- c(res,list(
			"mean.mean.covg.g1,mean.mean.covg.g2: Mean value of mean coverage values (across all samples in a group) across all sites in a region",
			c("mean.nsamples.covg.thresh.g1,mean.nsamples.covg.thresh.g2: mean number of samples (accross all considered sites) that have a coverage larger than ", covgThres," for the site in group 1 and group 2 respectively")
			))
		}
		if(hasVariability){
		  res <- c(res, list(
		    paste0("End: end coordinate of the ", target),
		    "[symbol]: associated gene symbol to the given region [only valid for gene associated regions]",
		    "[entrezID]: Entrez ID of the gene associated with the region [only valid for gene associated regions]",
		    "mean.var.g1, mean.var.g2: Average variances found in the regions for the twp groups. g1 and g2 is replaced by the group name",
		    "mean.var.diff: Average difference between the variances found in the two groups over the regions",
		    "mean.var.log.ratio: Region-wise average for the log2 ratio between the variances found in the two groups",
		    "comb.p.val.var: Combined p-value aggregating p-values of all sites in the region using a generalization of Fisher's method ",
		    "comb.p.adj.var.fdr: FDR adjusted combined p-value",
		    "combinedRank.var: mean.var.diff, mean.var.log.ratio and comb.p.val.var are ranked for all regions and the higher (=worst) rank is selected. "
		  ))
		}
	}
	return(res)

	##region
	sectionText <- list(
		"id: region id",
		"Chromosome: chromosome of the region",
		"Start: Start coordinate of the region",
		"End: End coordinate of the region",
		"[symbol]: associated gene symbol to the given region [only valid for gene associated regions]",
		"[entrezID]: Entrez ID of the gene associated with the region [only valid for gene associated regions]",
		"mean.mean.g1,mean.mean.g2: (where g1 and g2 is replaced by the respective group names in the table) mean of mean methylation levels for group 1 and 2 across all sites in a region",
		"mean.mean.diff: Mean difference in means across all sites in a region",
		"mean.mean.quot.log2: log2 of the mean quotient in means across all sites in a region",
		c("comb.p.val: Combined p-value aggregating p-values of all sites in the region using a generalization of Fisher's method ", rnb.get.reference(report, refText)),
		"comb.p.adj.fdr: FDR adjusted combined p-value",
		c("combinedRank: mean.mean.diff, mean.mean.quot.log2 and comb.p.val are ranked for all regions. ",
				"This column aggregates them using the maximum, i.e. worst rank of a site among the three measures"),
		"num.sites: number of sites associated with the region",
		"mean.num.na.g1,mean.num.na.g2: Mean number of NA methylation values accross all sites in group 1 and group 2 respectively"
		)
	if (includeCovg){
		sectionText <- c(sectionText,list(
		"mean.mean.covg.g1,mean.mean.covg.g2: Mean value of mean coverage values (across all samples in a group) across all sites in a region",
		c("mean.nsamples.covg.thresh.g1,mean.nsamples.covg.thresh.g2: mean number of samples (accross all considered sites) that have a coverage larger than ",
		  get.covg.thres(diffmeth)," for the site in group 1 and group 2 respectively")
		))
	}
}
get.diffmeth.tab.annot.cols <- function(target, includeCovg, hasVariability, covgThres=-1L, skipSites=FALSE){
	res <- c()
	if (target=="sites" || skipSites) {
		res <- c("mean.g1","mean.g2","mean.diff","mean.quot.log2",
				"diffmeth.p.val","max.g1","min.g1","sd.g1",
				"max.g2","min.g2","sd.g2",
				"min.diff","diffmeth.p.adj.fdr","combinedRank",
				"num.na.g1","num.na.g2")
		# include residual mean columns if residual output requested
		res.opt <- rnb.getOption("differential.residuals.output")
		if (!is.null(res.opt)){
			res <- c(res, "resid.mean.g1", "resid.mean.g2", "resid.mean.diff")
		}
		if (includeCovg){
			res <- c(res,c("mean.covg.g1","mean.covg.g2",
							"min.covg.g1","min.covg.g2","max.covg.g1","max.covg.g2",
							"covg.thresh.nsamples.g1","covg.thresh.nsamples.g2"))
		}
		if(hasVariability){
		  res <- c(res,"var.g1","var.g2","var.diff",
		           "var.log.ratio","diffVar.p.val",
		           "diffVar.p.adj.fdr","log10P",
		           "log10FDR","combinedRank.var")
		}
	} else {
		res <- c("mean.mean.g1","mean.mean.g2",
					"mean.mean.diff","mean.mean.quot.log2",
					"comb.p.val","comb.p.adj.fdr","combinedRank",
					"num.sites","mean.num.na.g1","mean.num.na.g2")
		if (includeCovg){
			res <- c(res,c("mean.mean.covg.g1","mean.mean.covg.g2",
									   "mean.nsamples.covg.thresh.g1","mean.nsamples.covg.thresh.g2"))
		}
		if(hasVariability){
		  res <- c(res,"mean.var.g1","mean.var.g2","mean.var.diff",
		           "mean.var.log.ratio","comb.p.val.var","comb.p.adj.var.fdr","combinedRank.var")
		}
	}
	return(res)
}
get.diffmeth.tab.annot.colnames.pretty <- function(target, name.g1, name.g2, includeCovg, hasVariability, covgThres=-1L, skipSites=FALSE){
	if (target=="sites" || skipSites) {
		res <- c(paste("mean",name.g1,sep="."),paste("mean",name.g2,sep="."),"mean.diff","mean.quot.log2",
				"diffmeth.p.val",paste("max",name.g1,sep="."),paste("min",name.g1,sep="."),paste("sd",name.g1,sep="."),
				paste("max",name.g2,sep="."),paste("min",name.g2,sep="."),paste("sd",name.g2,sep="."),
				"min.diff","diffmeth.p.adj.fdr","combinedRank",
				paste("num.na",name.g1,sep="."),paste("num.na",name.g2,sep="."))
		# include pretty names for residual means when residual output is requested
		res.opt <- rnb.getOption("differential.residuals.output")
		if (!is.null(res.opt) && !identical(res.opt, "none")){
			res <- c(res, paste("resid.mean",name.g1,sep="."), paste("resid.mean",name.g2,sep="."), "resid.mean.diff")
		}
		if (includeCovg){
			res <- c(res,c(paste("mean.covg",name.g1,sep="."),paste("mean.covg",name.g2,sep="."),
										   paste("min.covg",name.g1,sep="."),paste("min.covg",name.g2,sep="."),
										   paste("max.covg",name.g1,sep="."),paste("max.covg",name.g2,sep="."),
										   paste("nsamples.covg",paste("thres",covgThres,sep=""),name.g1,sep="."),
										   paste("nsamples.covg",paste("thres",covgThres,sep=""),name.g2,sep=".")))
		}
		if(hasVariability){
		  res <- c(res,
		           paste("var",name.g1,sep="."),paste("var",name.g2,sep="."),"var.diff",
		           "var.log.ratio","diffVar.p.val","diffVar.p.adj.fdr","log10P","log10FDR","combinedRank.var")
		}
	} else {
		res <- c(paste("mean.mean",name.g1,sep="."),paste("mean.mean",name.g2,sep="."),
				"mean.mean.diff","mean.mean.quot.log2",
				"comb.p.val","comb.p.adj.fdr","combinedRank",
				"num.sites",paste("mean.num.na",name.g1,sep="."),paste("mean.num.na",name.g2,sep="."))
		if (includeCovg){
			res <- c(res,c(paste("mean.mean.covg",name.g1,sep="."),paste("mean.mean.covg",name.g2,sep="."),
										   paste("mean.nsamples.covg",paste("thres",covgThres,sep=""),name.g1,sep="."),
										   paste("mean.nsamples.covg",paste("thres",covgThres,sep=""),name.g2,sep=".")))
		}
		if(hasVariability){
		  res <- c(res,
		           paste("mean.var",name.g1,sep="."),paste("mean.var",name.g2,sep="."),"mean.var.diff",
		           "mean.var.log.ratio","comb.p.val.var","comb.p.adj.var.fdr","combinedRank.var")
		}
	}
	return(res)
}

### rnb.section.diffMeth.site
###
### add information to the report for site level analysis
### @author Fabian Mueller
### @aliases rnb.section.diffMeth.site
### @param diffmeth RnBDiffMeth  object. See \code{\link{RnBDiffMeth-class}} for details.
### @param report report object to which the content is added
### @return the updated report object
rnb.section.diffMeth.site <- function(rnbSet,diffmeth,report,gzTable=FALSE){
	if (length(get.comparisons(diffmeth))<1){
		stop("no valid comparisons")
	}
	diffSiteRankCut <- c(100,1000,10000,100000) #the cutoffs for determining a site as differentially methylated according to combined rank
	logger.start("Adding Site Level Information")
	sectionText <- paste("Differential methylation on the site level was computed based on a variety of metrics. 
						  Of particular interest for the following plots and analyses are the following quantities for each site:
						  a) the difference in mean methylation levels of the two groups being compared, b) the quotient in mean methylation and
						  c) a statistical test (limma or t-test depending on the settings) assessing whether the methylation values in the two groups originate from distinct distributions.
						  Additionally each site was assigned a rank based on each of these three criteria. A combined rank is computed as the maximum (i.e. worst)
						  rank among the three ranks. The smaller the combined rank for a site, the more evidence for differential methylation it exhibits.
						  This section includes scatterplots of the site group means as well as volcano plots
						  of each pairwise comparison colored according to the combined ranks or p-values of a given site.")
	report <- rnb.add.section(report, 'Site Level', sectionText)
	heatmap.txt <- c("In addition, pairwise comparison heatmaps of the top differentially methylated sites are shown.")
	.add.heatmap <-rnb.getOption("differential.report.heatmaps")
	if (.add.heatmap){
		rnb.add.paragraph(report, heatmap.txt)
	}

	logger.start("Selection of rank cutoffs")
	rank.cuts.auto <- lapply(1:length(get.comparisons(diffmeth)),FUN=function(i){
		cc <- names(get.comparisons(diffmeth))[i]
		ccc <- get.comparisons(diffmeth)[cc]
		dmt <- get.table(diffmeth,ccc,"sites",return.data.frame=TRUE)
		res <- auto.select.rank.cut(dmt$diffmeth.p.adj.fdr,dmt$combinedRank,alpha=0.1)
		return(as.integer(res))
	})
	txt <- paste("The following rank cutfoffs have been automatically selected for the analysis of differentially",
						 "methylated sites:")
	rnb.add.paragraph(report, txt)
	tt <- data.frame(unlist(rank.cuts.auto))
	colnames(tt) <- c("Rank Cutoff")
	rownames(tt) <- get.comparisons(diffmeth)
	rnb.add.table(report,tt)
	logger.completed()
	
	#scatterplots
	logger.start("Adding scatterplots")
	rnb.cleanMem()
	addedPlots <- list()
	if(parallel.isEnabled()){
		addedPlots <- foreach(i=1:length(get.comparisons(diffmeth)),.combine="c") %dopar% {
			cc <- names(get.comparisons(diffmeth))[i]
			ccc <- get.comparisons(diffmeth)[cc]
			dmt <- get.table(diffmeth,ccc,"sites",return.data.frame=TRUE)
			ccn <- ifelse(is.valid.fname(cc),cc,paste("cmp",i,sep=""))
			grp.names <- get.comparison.grouplabels(diffmeth)[ccc,]
			auto.rank.cut <- rank.cuts.auto[[i]]
			#DEBUG MESSAGE
#			logger.info(paste("processing...",cc,"[",i,"] {",ccc,"}{",ccn,"}"))
			res <- addReportPlots.diffMeth.bin.site.scatter(report,dmt,ccn,diffSiteRankCut=diffSiteRankCut,
					autoRankCut=auto.rank.cut,grp1.name=grp.names[1],grp2.name=grp.names[2])
			rnb.cleanMem()
			#DEBUG MESSAGE
#			logger.info(paste("done...",cc,"[",i,"] {",ccc,"}{",ccn,"}"))
			res
		}
	} else {
		for (i in 1:length(get.comparisons(diffmeth))){
			cc <- names(get.comparisons(diffmeth))[i]
			ccc <- get.comparisons(diffmeth)[cc]
			dmt <- get.table(diffmeth,ccc,"sites",return.data.frame=TRUE)
			ccn <- ifelse(is.valid.fname(cc),cc,paste("cmp",i,sep=""))
			grp.names <- get.comparison.grouplabels(diffmeth)[ccc,]
			auto.rank.cut <- rank.cuts.auto[[i]]
			addedPlots <- c(addedPlots,addReportPlots.diffMeth.bin.site.scatter(report,dmt,ccn,diffSiteRankCut=diffSiteRankCut,
							autoRankCut=auto.rank.cut,grp1.name=grp.names[1],grp2.name=grp.names[2]))
			rnb.cleanMem()
		}
	}


	comps <- get.comparisons(diffmeth)
	diffMethType = c(paste("FDR adjusted p-value &lt;",P.VAL.CUT),
					 paste("combined rank among the ",diffSiteRankCut," best ranking sites",sep=""),
					 "automatically selected rank cutoff",
					 "combined rank based gradient","rank permutation test p-value")
	names(diffMethType) = c("fdrAdjPval",paste("rc",1:length(diffSiteRankCut),sep=""),"rcAuto","rankGradient","permutationP")
	setting.names <- list(
		'comparison' = comps,
		'differential methylation measure' = diffMethType)
	description <- c('Scatterplot for differential methylation (sites). If the selected criterion is not <code>rankGradient</code>:
		The transparency corresponds to point density. If the number of points exceeds ',DENS.SCATTER.SUBSAMPLE.THRES,
		' then the number of points for density estimation is reduced to that number by random sampling.',
		'The',round(DENS.SCATTER.SPARSE.POINTS.PERC*100),
		'% of the points in the sparsest populated plot regions are drawn explicitly (up to a maximum of ',DENS.SCATTER.SPARSE.POINTS.MAX,
		" points).",
		'Additionally, the colored points represent differentially methylated sites (according to the selected criterion). 
		If the selected criterion is <code>rankGradient</code>: median combined ranks accross hexagonal bins are shown
		as a gradient according to the color legend.')
	report <- rnb.add.figure(report, description, addedPlots, setting.names)
	logger.completed()
	
	#volcano plots
	logger.start("Adding volcano plots")
	rnb.cleanMem()
	addedPlots <- list()
	if(parallel.isEnabled()){
		addedPlots <- foreach(i=1:length(get.comparisons(diffmeth)),.combine="c") %dopar% {
			cc <- names(get.comparisons(diffmeth))[i]
			ccc <- get.comparisons(diffmeth)[cc]
			dmt <- get.table(diffmeth,ccc,"sites",return.data.frame=TRUE)
			ccn <- ifelse(is.valid.fname(cc),cc,paste("cmp",i,sep=""))
			grp.names <- get.comparison.grouplabels(diffmeth)[ccc,]
			res <- addReportPlots.diffMeth.bin.site.volcano(report,dmt,ccn,
					grp1.name=grp.names[1],grp2.name=grp.names[2])
			rnb.cleanMem()
			res
		}
	} else {
		for (i in 1:length(get.comparisons(diffmeth))){
			cc <- names(get.comparisons(diffmeth))[i]
			ccc <- get.comparisons(diffmeth)[cc]
			dmt <- get.table(diffmeth,ccc,"sites",return.data.frame=TRUE)
			ccn <- ifelse(is.valid.fname(cc),cc,paste("cmp",i,sep=""))
			grp.names <- get.comparison.grouplabels(diffmeth)[ccc,]
			addedPlots <- c(addedPlots,addReportPlots.diffMeth.bin.site.volcano(report,dmt,ccn,
							grp1.name=grp.names[1],grp2.name=grp.names[2]))
			rnb.cleanMem()
		}
	}

	diff.measure <- c("diff"="Difference","quot"="Quotient")
	signif.measure <- c("pVal"="p-value","pValAdj"="adjusted p-value","quotSig"="Quotient (only meaningful if 'Difference' is selected above)")
	setting.names <- list(
		'comparison' = comps,
		'difference metric' = diff.measure,
		'significance metric' = signif.measure)
	description <- 'Volcano plot for differential methylation quantified by various metrics. Color scale according to
					combined ranking.'
	report <- rnb.add.figure(report, description, addedPlots, setting.names)
	logger.completed()

	#heatmaps
	if (rnb.getOption("differential.report.heatmaps")){
		logger.start("Adding heatmaps")
		if (!(.hasSlot(diffmeth,"comparison.info") && length(diffmeth@comparison.info) > 0)){
			logger.warning("Could not add differential methylation heatmaps: missing comparison sample indices")
		} else {
			rnb.require("ComplexHeatmap")
			rnb.cleanMem()
			addedPlots <- list()
			if(parallel.isEnabled()){
				addedPlots <- foreach(i=seq_along(get.comparisons(diffmeth)),.combine="c") %dopar% {
					cc <- names(get.comparisons(diffmeth))[i]
					ccc <- get.comparisons(diffmeth)[cc]
					dmt <- get.table(diffmeth,ccc,"sites",return.data.frame=TRUE)
					ccn <- ifelse(is.valid.fname(cc),cc,paste("cmp",i,sep=""))
					grp.names <- get.comparison.grouplabels(diffmeth)[ccc,]
					auto.rank.cut <- rank.cuts.auto[[i]]
					cmp.info <- diffmeth@comparison.info[[ccc]]
					res <- addReportPlots.diffMeth.bin.site.heatmap(report, rnbSet, diffmeth, ccc, dmt, ccn,
						sample.inds = cmp.info$group.inds,
						diffSiteRankCut = diffSiteRankCut, autoRankCut = auto.rank.cut,
						grp1.name = grp.names[1], grp2.name = grp.names[2])
					rnb.cleanMem()
					res
				}
			} else {
				for (i in seq_along(get.comparisons(diffmeth))){
					cc <- names(get.comparisons(diffmeth))[i]
					ccc <- get.comparisons(diffmeth)[cc]
					dmt <- get.table(diffmeth,ccc,"sites",return.data.frame=TRUE)
					ccn <- ifelse(is.valid.fname(cc),cc,paste("cmp",i,sep=""))
					grp.names <- get.comparison.grouplabels(diffmeth)[ccc,]
					auto.rank.cut <- rank.cuts.auto[[i]]
					cmp.info <- diffmeth@comparison.info[[ccc]]
					addedPlots <- c(addedPlots,addReportPlots.diffMeth.bin.site.heatmap(report, rnbSet, diffmeth, ccc, dmt, ccn,
						sample.inds = cmp.info$group.inds,
						diffSiteRankCut = diffSiteRankCut, autoRankCut = auto.rank.cut,
						grp1.name = grp.names[1], grp2.name = grp.names[2]))
					rnb.cleanMem()
				}
			}

			if (length(addedPlots) > 0){
				# Extract measure and signal type from filenames
				fname.parts <- sapply(addedPlots, FUN=function(rp){
					parts <- strsplit(slot(rp, "fname"), "_", fixed = TRUE)[[1]]
					# parts structure: diffMeth_site_heatmap_cmpName_measure_signal_zscore
					c(measure=parts[length(parts)-2], signal=parts[length(parts)-1], zscore=parts[length(parts)])
				})
				measure.ids <- unique(fname.parts["measure",])
				signal.ids <- unique(fname.parts["signal",])
				zscore.ids <- unique(fname.parts["zscore",])
				# DEBUG MESSAGE
				logger.info(paste("measure.ids:", paste(measure.ids, collapse = ", ")))
				logger.info(paste("signal.ids:", paste(signal.ids, collapse = ", ")))
				logger.info(paste("zscore.ids:", paste(zscore.ids, collapse = ", ")))
				
				diffMethType <- c(
					"fdrAdjPval" = paste("FDR adjusted p-value <", P.VAL.CUT),
					setNames(paste("combined rank among the ",diffSiteRankCut," best ranking sites",sep=""), paste("rc",seq_along(diffSiteRankCut),sep="")),
					"rcAuto" = "automatically selected rank cutoff",
					"permutationP" = "rank permutation test p-value"
				)
				diffMethType <- diffMethType[names(diffMethType) %in% measure.ids]
				
				signal.types <- c(
					"beta" = "Beta methylation",
					"mvalue" = "M-values",
					"residuals" = "Residuals"
				)
				signal.types <- signal.types[names(signal.types) %in% signal.ids]
				
				zscore.types <- c(
					"zscore" = "Yes",
					"raw" = "No"
				)

				zscore.types <- zscore.types[names(zscore.types) %in% zscore.ids]

				setting.names <- list(
					'comparison' = comps,
					'differential methylation measure' = diffMethType,
					'signal type' = signal.types,
					'z-score normalization' = zscore.types
				)
				description <- c('Heatmap for differential methylation (sites). Rows correspond to selected sites and columns to samples. ',
					'The number of displayed rows is controlled by option <code>differential.heatmap.top.features</code>. ',
					'The criterion for selecting the sites is controlled by the selected differential methylation measure from <code>diffMeth</code> tables. ',
					'Samples are split by comparison groups. Select the desired signal type and the row-wise z-score normalization option using the dropdown menus. ',
					'If residuals are not available, only beta methylation and M-values will be shown.')
				report <- rnb.add.figure(report, description, addedPlots, setting.names)
			} else {
				logger.warning("No differential methylation site heatmaps were generated")
			}
		}
		logger.completed()
	}

	logger.start("Adding tables")
	includeCovg <- hasCovg(rnbSet)
	hasVariability <- rnb.getOption("differential.variability")
	
	sectionText <- c("A tabular overview of measures for differential methylation on the site level for the individual comparisons are provided in this section.
					  Below, a brief explanation of the different columns can be found:")
	report <- rnb.add.section(report, "Differential Methylation Tables", sectionText, level = 2)

	sectionColDescList <- get.diffmeth.tab.col.desc.list.txt("sites", includeCovg, hasVariability,covgThres=get.covg.thres(diffmeth), skipSites=FALSE)
	rnb.add.list(report, sectionColDescList)
	

	sectionText <- "The tables for the individual comparisons can be found here:\n<ul>\n"
	annot.cols <- c("Chromosome","Start","Strand")
	sites.info <- annotation(rnbSet,type="sites",add.names=FALSE)[, annot.cols]
	#add cg identifier for infinium datasets
	if (!is.element("RnBiseqSet",class(rnbSet)) && !is.null(rownames(sites.info))){
		sites.info <- data.frame(cgid=rownames(sites.info),sites.info,stringsAsFactors=FALSE)
	}
	grp.names <- get.comparison.grouplabels(diffmeth)
	for (i in 1:length(comps)){
		cc <- comps[i]
		
		annot.vec <- get.diffmeth.tab.annot.cols("sites", includeCovg, hasVariability)
		colname.vec <- get.diffmeth.tab.annot.colnames.pretty("sites", grp.names[i,1], grp.names[i,2], includeCovg, hasVariability, covgThres=get.covg.thres(diffmeth), skipSites=FALSE)
		dmt <- get.table(diffmeth,cc,"sites",return.data.frame=TRUE)[,annot.vec]		
		colnames(dmt) <- colname.vec
		dmt <- cbind(rownames(dmt),sites.info,dmt)
		colnames(dmt)[1] <- "id"
		rownames(dmt) <- NULL
		ccn <- ifelse(is.valid.fname(cc),cc,paste("cmp",i,sep=""))
		fname <- paste("diffMethTable_site_",ccn,".csv",sep="")
		fname <- rnb.write.table(dmt,fname,fpath=rnb.get.directory(report, "data", absolute = TRUE),format="csv",gz=gzTable,row.names = FALSE,quote=FALSE)
		res.opt <- rnb.getOption("differential.residuals.output")
		if ("csv" %in% res.opt){
			resid_mat <- NULL
			cmp_idx <- which(diffmeth@comparisons == cc)
			if (length(cmp_idx) == 1) {
				res_obj <- diffmeth@residSites[[cmp_idx]]
				if (is.null(res_obj)){
					logger.info(c("No residuals found in diffmeth@residSites for comparison:", cc))
				} else {
					if (inherits(res_obj, "ff")){
						resid_mat <- tryCatch({
							as.matrix(res_obj[])
						}, error = function(e){
							logger.warning(c("Could not coerce ff residuals to RAM for comparison:", cc, e$message))
							NULL
						})
					} else {
						resid_mat <- res_obj
					}
				}
			}
			if (!is.null(resid_mat)){
				res_fname <- paste0("residuals_site_", ccn, ".csv")
				tryCatch({
					rnb.write.table(resid_mat, res_fname, fpath=rnb.get.directory(report, "data", absolute = TRUE), format="csv", gz=gzTable, row.names = FALSE, quote = FALSE)
				}, error = function(e){
					logger.warning(c("Could not write residuals to report data:", e$message))
				})
			}
		}
		txt <- paste(c("<a href=\"", rnb.get.directory(report, "data"), "/", fname,"\">",cc,"</a>"),collapse="")
		sectionText <- paste(sectionText,"<li>",txt,"</li>\n",sep="")
	}
	## Save table of only the nv-probes in EPICv2
	if (rnbSet@target == "probesEPICv2" & rnb.getOption("nv.probe.tables") & any(grepl("^nv", dmt$cgid))) { 
		logger.info("Saving table(s) containing only nv-probes")
		sectionText <- paste(sectionText,"</ul>",sep="")
		rnb.add.paragraph(report, sectionText)

		# nv_diff <- dmt ## New nv dmt df
		# saveRDS(nv_diff, "/Users/baris.kalem/Code/RnBeads_Project/nv_probes_EPICv2/debug/nv_diff.RDS")

		logger.info("Enriching nv-probes comparison table with HGNC symbols. see: GeneSymbol column in the finished table.")
		rnb.require("biomaRt")
		mart <- useMart("ENSEMBL_MART_ENSEMBL", dataset = "hsapiens_gene_ensembl",host="https://sep2025.archive.ensembl.org")

		refText <- c("Kaur, D., Lee, S. M., Goldberg, D., Spix, N. J., Hinoue, T., Li, H.-T., Dwaraka, V. B., Smith, R., ",
					 "Shen, H., Liang, G., Renke, N., Laird, P. W., & Zhou, W. (2023). Comprehensive evaluation of the Infinium human ",
					 "MethylationEPIC v2 BeadChip. <i>Epigenetics Communications</i>, <b>3</b>(1), 6.")
		report <- rnb.add.reference(report, refText)
		sectionText <- c(paste0("For Methylation EPICv2 platform, nv-probes are available. These probes are designed to detect somatic mutations in common cancers.
						  These probes measure DNA sequence variations rather than DNA cytosine methylation ", rnb.get.reference(report, refText), " .
						  Tables contain additional columns <code>ReferenceBase</code> and <code>AlternateBase</code> to indicate the point mutation that each nv-probe is investigating.
						  <code>GeneSymbol</code> column indicates the associated gene for this site.\n
						  The tables for the individual comparisons of <em>only nv-probes</em> can be found here:\n<ul>\n"))
		for (i in 1:length(comps)){
			cc <- comps[i]

			annot.vec <- get.diffmeth.tab.annot.cols("sites", includeCovg, hasVariability)
			colname.vec <- get.diffmeth.tab.annot.colnames.pretty("sites", grp.names[i,1], grp.names[i,2], includeCovg, hasVariability, covgThres=get.covg.thres(diffmeth), skipSites=FALSE)
			nv_diff <- get.table(diffmeth,cc,"sites",return.data.frame=TRUE)[,annot.vec]
			colnames(nv_diff) <- colname.vec
			nv_diff <- cbind(rownames(nv_diff),sites.info,nv_diff)
			colnames(nv_diff)[colnames(nv_diff) == "cgid"] <- "nv_probe_id" ## New probe id column name
			colnames(nv_diff)[colnames(nv_diff) == "diffmeth.p.val"] <- "diff.p.val" ## change diffmeth to diff, since nv-probes don't indicate methylation
			colnames(nv_diff)[colnames(nv_diff) == "diffmeth.p.adj.fdr"] <- "diff.p.adj.fdr"	
			
			nv_diff <- nv_diff[grepl("^nv", nv_diff$nv_probe_id), ] ## Filter to only have nv-probes in the table
			## Create a new column for Reference and Alternate bases
			probe_id_split <- strsplit(as.character(nv_diff$nv_probe_id), "-|_") ## e.g. nv-GRCh38-chr7-6387219-6387219-G-A_TC21
			nv_diff$ReferenceBase <- sapply(probe_id_split, "[[", 6)
			nv_diff$AlternateBase <- sapply(probe_id_split, "[[", 7)
			colnames(nv_diff)[1] <- "id"
			rownames(nv_diff) <- NULL
			## Add gene symbol of every nv-probe
			for (j in 1:nrow(nv_diff)) {
				nv_diff[j, "GeneSymbol"] <- get.genesymbol.of.coordinate(nv_diff[j, "Start"], nv_diff[j, "Start"], nv_diff[j, "Chromosome"], mart)
			}
			ccn <- ifelse(is.valid.fname(cc),cc,paste("cmp",i,sep=""))
			fname <- paste("diffTable_nv_site_",ccn,".csv",sep="")
			fname <- rnb.write.table(nv_diff,fname,fpath=rnb.get.directory(report, "data", absolute = TRUE),format="csv",gz=gzTable,row.names = FALSE,quote=FALSE)
			txt <- paste(c("<a href=\"", rnb.get.directory(report, "data"), "/", fname,"\">",cc,"</a>"),collapse="")
			sectionText <- paste(sectionText,"<li>",txt,"</li>\n",sep="")
		}
	}

	sectionText <- paste(sectionText,"</ul>",sep="")
	rnb.add.paragraph(report, sectionText)
	
	logger.completed()
	
	logger.completed()
	if(rnb.getOption("differential.variability")){
	  report <- rnb.section.diffVar(rnbSet,diffmeth,report,gzTable=gzTable,level=2)
	}
	return(report)
}

### rnb.section.diffMeth.region
###
### add information to the report for region level analysis
### @author Fabian Mueller
### @aliases rnb.section.diffMeth.region
### @param diffmeth RnBDiffMeth object. See \code{\link{RnBDiffMeth-class}} for details.
### @param report report object to which the content is added
### @param dm.go.enrich If GO enrichment analysis reports are desired this argument should not be \code{NULL} (which is the default value
###                  it should be an object of type \code{DiffMeth.go.enrich} (see \code{performGoEnrichment.diffMeth()} for details)
### @param dm.lola.enrich If LOLA enrichment analysis reports are desired this argument should not be \code{NULL} (which is the default value
###                  it should be an object of type \code{DiffMeth.lola.enrich} (see \code{performLolaEnrichment.diffMeth()} for details)
### @return the updated report object
rnb.section.diffMeth.region <- function(rnbSet,diffmeth,report,dm.go.enrich=NULL,dm.lola.enrich=NULL,gzTable=FALSE){
	if (length(get.comparisons(diffmeth))<1){
		stop("no valid comparisons")
	}
	if (length(get.region.types(diffmeth))<1){
		stop("no valid region types")
	}
	skipSites <- !includes.sites(diffmeth)

	diffRegionRankCut <- c(100,500,1000) #the cutoffs for determining a site as differentially methylated according to combined rank
	logger.start("Adding Region Level Information")
	refText <- c("Makambi, K. (2003) Weighted inverse chi-square method for correlated significance tests. ",
		"<i>Journal of Applied Statistics</i>, <b>30</b>(2), 225234")
	report <- rnb.add.reference(report, refText)
	
	sectionText <- c("Differential methylation on the region level was computed based on a variety of metrics. ", 
		"Of particular interest for the following plots and analyses are the following quantities for each region: ")

	if (skipSites){
		sectionText <- c(sectionText, c("the mean methylation difference in a region of the two groups being and of quotient of mean methylation levels",
			"as well as a p-value obtained from statistical testing (limma or t-test; depending on parameter settings).")
		)
	} else {
		sectionText <- c(sectionText, c("the mean difference in means across all sites in a region of the two groups being compared and the mean of quotients ",
			"in mean methylation as well as a combined p-value calculated from all site p-values in the region ",
			rnb.get.reference(report, refText), ". ")
		)
	}
	sectionText <- c(sectionText,c(
		" Additionally each region was assigned a rank based on each of these three criteria. ",
		"A combined rank is computed as the maximum (i.e. worst) value among the three ranks. The smaller the combined rank for a region, the more evidence for differential methylation it exhibits. ",
		"Regions were defined based on the region types specified in the analysis. ",
		"This section includes scatterplots of the region group means as well as volcano plots of each pairwise comparison ",
		"colored according to the combined rank of a given region.")
	)
	report <- rnb.add.section(report, "Region Level", sectionText)
	heatmap.txt <- c("In addition, pairwise comparison heatmaps of the top differentially methylated regions are shown.")
	.add.heatmap <-rnb.getOption("differential.report.heatmaps")
	if (.add.heatmap){
		rnb.add.paragraph(report, heatmap.txt)
	}

	comps <- get.comparisons(diffmeth)
	reg.types <- get.region.types(diffmeth)

	logger.start("Selection of rank cutoffs")
	rank.cuts.auto <- lapply(1:length(comps),FUN=function(i){
		lapply(1:length(reg.types),FUN=function(j){
			dmt <- get.table(diffmeth,comps[i],reg.types[j],return.data.frame=TRUE)
			res <- auto.select.rank.cut(dmt$comb.p.adj.fdr,dmt$combinedRank,alpha=0.1)
			return(as.integer(res))
		})
	})
	txt <- paste("The following rank cutfoffs have been automatically selected for the analysis of differentially",
						 "methylated regions:")
	rnb.add.paragraph(report, txt)

	tt <- data.frame(matrix(unlist(rank.cuts.auto),ncol=length(reg.types),nrow=length(comps),byrow=TRUE))
	colnames(tt) <- reg.types
	rownames(tt) <- comps
	rnb.add.table(report,tt)
	logger.completed()

	#scatterplots
	logger.start("Adding scatterplots")
	rnb.cleanMem()
	addedPlots <- list()
	grp.labels <- get.comparison.grouplabels(diffmeth)
	if(parallel.isEnabled()){
		#generate pairs of comparison, region combinations with indices
		iis <- 1:length(comps)
		jjs <- 1:length(reg.types)
		pps <- expand.grid(iis,jjs)
		
		addedPlots <- foreach(k=1:nrow(pps),.combine="c") %dopar% {
			i <- pps[k,1]
			j <- pps[k,2]
			cc <- names(comps)[i]
			ccc <- comps[cc]
			ccn <- ifelse(is.valid.fname(cc),cc,paste("cmp",i,sep=""))
			rr <- reg.types[j]
			rrn <- ifelse(is.valid.fname(rr),rr,paste("reg",j,sep=""))
			auto.rank.cut <- rank.cuts.auto[[i]][[j]]
			dmt <- get.table(diffmeth,ccc,rr,return.data.frame=TRUE)
			res <- addReportPlots.diffMeth.bin.region.scatter(report,dmt,ccn,rrn,diffRegionRankCut=diffRegionRankCut,
					autoRankCut=auto.rank.cut,grp1.name=grp.labels[ccc,1],grp2.name=grp.labels[ccc,2], useSiteCols=skipSites)
			rnb.cleanMem()
			res
		}
		
	} else {
		for (i in 1:length(comps)){
			cc <- names(comps)[i]
			ccc <- comps[cc]
			ccn <- ifelse(is.valid.fname(cc),cc,paste("cmp",i,sep=""))
			for (j in 1:length(reg.types)){
				rr <- reg.types[j]
				rrn <- ifelse(is.valid.fname(rr),rr,paste("reg",j,sep=""))
				auto.rank.cut <- rank.cuts.auto[[i]][[j]]
				dmt <- get.table(diffmeth,ccc,rr,return.data.frame=TRUE)
				addedPlots <- c(addedPlots,addReportPlots.diffMeth.bin.region.scatter(report,dmt,ccn,rrn,diffRegionRankCut=diffRegionRankCut,
								autoRankCut=auto.rank.cut,grp1.name=grp.labels[ccc,1],grp2.name=grp.labels[ccc,2], useSiteCols=skipSites))
				rnb.cleanMem()
			}
		}
	}
	
	names(reg.types) <- ifelse(is.valid.fname(reg.types),reg.types,paste("reg",1:length(reg.types),sep=""))
	diffMethType = c(paste("FDR adjusted p-value <",P.VAL.CUT),
					 paste("combined rank among the ",diffRegionRankCut," best ranking regions",sep=""),
					 "automatically selected rank cutoff","combined rank based gradient")
	names(diffMethType) = c("fdrAdjPval",paste("rc",1:length(diffRegionRankCut),sep=""),"rcAuto","rankGradient")
	setting.names <- list(
		'comparison' = comps,
		'regions' = reg.types ,
		'differential methylation measure' = diffMethType)
	description <- 'Scatterplot for differential methylation (regions). If the selected criterion is not <code>rankGradient</code>:
		The transparency corresponds to point density. The 1% of the points in the sparsest populated plot regions are drawn explicitly.
		Additionally, the colored points represent differentially methylated regions (according to the selected criterion). 
		If the selected criterion is <code>rankGradient</code>: median combined ranks accross hexagonal bins are shown
		as a gradient according to the color legend.'
	report <- rnb.add.figure(report, description, addedPlots, setting.names)
	logger.completed()
	
	#volcano plots
	logger.start("Adding volcano plots")
	rnb.cleanMem()
	addedPlots <- list()
	if(parallel.isEnabled()){
		#generate pairs of comparison, region combinations with indices
		iis <- 1:length(comps)
		jjs <- 1:length(reg.types)
		pps <- expand.grid(iis,jjs)
		
		addedPlots <- foreach(k=1:nrow(pps),.combine="c") %dopar% {
			i <- pps[k,1]
			j <- pps[k,2]
			cc <- names(comps)[i]
			ccc <- comps[cc]
			ccn <- ifelse(is.valid.fname(cc),cc,paste("cmp",i,sep=""))
			rr <- reg.types[j]
			rrn <- ifelse(is.valid.fname(rr),rr,paste("reg",j,sep=""))
			dmt <- get.table(diffmeth,ccc,rr,return.data.frame=TRUE)
			res <- addReportPlots.diffMeth.bin.region.volcano(report,dmt,ccn,rrn,
					grp1.name=grp.labels[ccc,1],grp2.name=grp.labels[ccc,2], useSiteCols=skipSites)
			rnb.cleanMem()
			res
		}
	} else {
		for (i in 1:length(comps)){
			cc <- names(comps)[i]
			ccc <- comps[cc]
			ccn <- ifelse(is.valid.fname(cc),cc,paste("cmp",i,sep=""))
			for (j in 1:length(reg.types)){
				rr <- reg.types[j]
				rrn <- ifelse(is.valid.fname(rr),rr,paste("reg",j,sep=""))
				dmt <- get.table(diffmeth,ccc,rr,return.data.frame=TRUE)
				addedPlots <- c(addedPlots,addReportPlots.diffMeth.bin.region.volcano(report,dmt,ccn,rrn,
								grp1.name=grp.labels[ccc,1],grp2.name=grp.labels[ccc,2], useSiteCols=skipSites))
				rnb.cleanMem()
			}
		}
	}
	diff.measure <- c("diff"="Difference","quot"="Quotient")
	signif.measure <- c("pVal"="combined p-value","pValAdj"="adjusted combined p-value","quotSig"="Quotient (only meaningful if 'Difference' is selected above)")
	setting.names <- list(
		'comparison' = comps,
		'regions' = reg.types ,
		'difference metric' = diff.measure,
		'significance metric' = signif.measure)
	description <- 'Volcano plot for differential methylation quantified by various metrics. Color scale according to
		combined ranking.'
	report <- rnb.add.figure(report, description, addedPlots, setting.names)
	logger.completed()

	#heatmaps
	if (rnb.getOption("differential.report.heatmaps")){
		logger.start("Adding heatmaps")
		if (!(.hasSlot(diffmeth,"comparison.info") && length(diffmeth@comparison.info) > 0)){
			logger.warning("Could not add differential methylation heatmaps: missing comparison sample indices")
		} else {
			rnb.require("ComplexHeatmap")
			rnb.cleanMem()
			addedPlots <- list()
			if(parallel.isEnabled()){
				#generate pairs of comparison, region combinations with indices
				iis <- seq_along(comps)
				jjs <- seq_along(reg.types)
				pps <- expand.grid(iis,jjs)

				addedPlots <- foreach(k=seq_len(nrow(pps)),.combine="c") %dopar% {
					i <- pps[k,1]
					j <- pps[k,2]
					cc <- names(comps)[i]
					ccc <- comps[cc]
					ccn <- ifelse(is.valid.fname(cc),cc,paste("cmp",i,sep=""))
					rr <- reg.types[j]
					rrn <- ifelse(is.valid.fname(rr),rr,paste("reg",j,sep=""))
					auto.rank.cut <- rank.cuts.auto[[i]][[j]]
					dmt <- get.table(diffmeth,ccc,rr,return.data.frame=TRUE)
					cmp.info <- diffmeth@comparison.info[[ccc]]
					res <- addReportPlots.diffMeth.bin.region.heatmap(report, rnbSet, diffmeth, ccc, dmt, ccn, rrn, rr,
						sample.inds = cmp.info$group.inds,
						diffRegionRankCut = diffRegionRankCut, autoRankCut = auto.rank.cut,
						grp1.name = grp.labels[ccc,1], grp2.name = grp.labels[ccc,2])
					rnb.cleanMem()
					res
				}
			} else {
				for (i in seq_along(comps)){
					cc <- names(comps)[i]
					ccc <- comps[cc]
					ccn <- ifelse(is.valid.fname(cc),cc,paste("cmp",i,sep=""))
					for (j in seq_along(reg.types)){
						rr <- reg.types[j]
						rrn <- ifelse(is.valid.fname(rr),rr,paste("reg",j,sep=""))
						auto.rank.cut <- rank.cuts.auto[[i]][[j]]
						dmt <- get.table(diffmeth,ccc,rr,return.data.frame=TRUE)
						cmp.info <- diffmeth@comparison.info[[ccc]]
						addedPlots <- c(addedPlots,addReportPlots.diffMeth.bin.region.heatmap(report, rnbSet, diffmeth, ccc, dmt, ccn, rrn, rr,
							sample.inds = cmp.info$group.inds,
							diffRegionRankCut = diffRegionRankCut, autoRankCut = auto.rank.cut,
							grp1.name = grp.labels[ccc,1], grp2.name = grp.labels[ccc,2]))
						rnb.cleanMem()
					}
				}
			}

			if (length(addedPlots) > 0){
				# Extract measure, signal type and zscore transformation from filenames
				fname.parts <- sapply(addedPlots, FUN=function(rp){
					parts <- strsplit(slot(rp, "fname"), "_", fixed = TRUE)[[1]]
					# parts structure: diffMeth_region_heatmap_cmpName_regName_measure_signal_zscore
					c(measure=parts[length(parts)-2], signal=parts[length(parts)-1], zscore=parts[length(parts)])
				})
				measure.ids <- unique(fname.parts["measure",])
				signal.ids <- unique(fname.parts["signal",])
				zscore.ids <- unique(fname.parts["zscore",])
				# DEBUG MESSAGE
				logger.info(paste("zscore.ids:", paste(zscore.ids, collapse = ", ")))
				
				diffMethType <- c(
					"fdrAdjPval" = paste("FDR adjusted p-value <",P.VAL.CUT),
					setNames(paste("combined rank among the ",diffRegionRankCut," best ranking regions",sep=""), paste("rc",seq_along(diffRegionRankCut),sep="")),
					"rcAuto" = "automatically selected rank cutoff"
				)
				diffMethType <- diffMethType[names(diffMethType) %in% measure.ids]
				
				signal.types <- c(
					"beta" = "Beta methylation",
					"mvalue" = "M-values",
					"residuals" = "Residuals"
				)
				signal.types <- signal.types[names(signal.types) %in% signal.ids]
				
				zscore.types <- c(
					"zscore" = "Yes",
					"raw" = "No"
				)
				zscore.types <- zscore.types[names(zscore.types) %in% zscore.ids]

				setting.names <- list(
					'comparison' = comps,
					'regions' = reg.types,
					'differential methylation measure' = diffMethType,
					'signal type' = signal.types,
					'z-score normalization' = zscore.types
				)
				description <- c('Heatmap for differential methylation (regions). Rows correspond to selected regions and columns to samples. ',
					'The number of displayed rows is controlled by option <code>differential.heatmap.top.features</code>. ',
					'The criterion for selecting the sites is controlled by the selected differential methylation measure from <code>diffMeth</code> tables. ',
					'Samples are split by comparison groups. Select the desired signal type and the row-wise z-score normalization option using the dropdown menus. ',
					'If residuals are not available, only beta methylation and M-values will be shown.')
				report <- rnb.add.figure(report, description, addedPlots, setting.names)
			} else {
				logger.warning("No differential methylation region heatmaps were generated")
			}
		}
		logger.completed()
	}

	logger.start("Adding tables")
	includeCovg <- hasCovg(rnbSet)
	hasVariability <- rnb.getOption("differential.variability")
	sectionText <- c("A tabular overview of measures for differential methylation on the region level for the ",
		"individual comparisons are provided in this section.")
	report <- rnb.add.section(report, "Differential Methylation Tables", sectionText, level = 2)
	#TODO: change section text when sites are skipped
	sectionColDescList <- get.diffmeth.tab.col.desc.list.txt("regions", includeCovg, hasVariability, covgThres=get.covg.thres(diffmeth), comb.p.ref.txt=rnb.get.reference(report, refText), skipSites=skipSites)
	rnb.add.list(report, sectionColDescList)

	sectionText <- "The tables for the individual comparisons can be found here:"
	rnb.add.paragraph(report, sectionText)
	#create data tables
	region.info.cols <- c("Chromosome","Start","End","symbol","entrezID")#additional information to be included in the output table
	reg.type.infos <- lapply(reg.types,FUN=function(rn){
		aa <- annotation(rnbSet,type=rn,add.names=FALSE)
		return(aa)
	})
	names(reg.type.infos) <- reg.types
	file.tab <- do.call("rbind",lapply(1:length(comps),FUN=function(ic){
		cc <- comps[ic]
		sapply(1:length(reg.types),FUN=function(ir){
			rr <- reg.types[ir]
			reg.info <- reg.type.infos[[rr]]
			region.info.cols.cur <- intersect(region.info.cols,colnames(reg.info))
			reg.info <- reg.info[,region.info.cols.cur]
			
			annot.vec <- get.diffmeth.tab.annot.cols("regions", includeCovg, hasVariability, skipSites=skipSites)
			colname.vec <- get.diffmeth.tab.annot.colnames.pretty("regions", grp.labels[ic,1], grp.labels[ic,2], includeCovg, hasVariability, covgThres=get.covg.thres(diffmeth), skipSites=skipSites)
			
			dmt <- get.table(diffmeth,cc,rr,return.data.frame=TRUE)[,annot.vec]
			colnames(dmt) <- colname.vec
			dmt <- cbind("id"=rownames(reg.info),reg.info,dmt)
			
			ccn <- ifelse(is.valid.fname(cc),cc,paste("cmp",ic,sep=""))
			rrn <- ifelse(is.valid.fname(rr),rr,paste("reg",ir,sep=""))
			fname <- paste("diffMethTable_region_",ccn,"_",rrn,".csv",sep="")
			fname <- rnb.write.table(dmt,fname,fpath=rnb.get.directory(report, "data", absolute = TRUE),format="csv",gz=gzTable,row.names=FALSE,quote=FALSE)
			res.opt <- rnb.getOption("differential.residuals.output")
			if ("csv" %in% res.opt){
				resid_mat <- NULL
				cmp_idx <- which(diffmeth@comparisons == cc)
				if (length(cmp_idx) == 1) {
					if (!is.null(diffmeth@residRegions[[rr]])){
						res_obj <- diffmeth@residRegions[[rr]][[cmp_idx]]
						if (is.null(res_obj)){
							logger.info(c("No residuals found in diffmeth@residRegions for comparison:", cc, "region:", rr))
						} else {
							if (inherits(res_obj, "ff")){
								resid_mat <- tryCatch({
									as.matrix(res_obj[])
								}, error = function(e){
									logger.warning(c("Could not coerce ff residuals to RAM for comparison:", cc, "region:", rr, e$message))
									NULL
								})
							} else {
								resid_mat <- res_obj
							}
						}
					}
				}
				if (!is.null(resid_mat)){
					res_fname <- paste0("residuals_region_", ccn, "_", rrn, ".csv")
					tryCatch({
						rnb.write.table(resid_mat, res_fname, fpath=rnb.get.directory(report, "data", absolute = TRUE), format="csv", gz=gzTable, row.names = FALSE, quote = FALSE)
					}, error = function(e){
						logger.warning(c("Could not write residuals to report data:", e$message))
					})
				}
			}
			txt <- paste(c("<a href=\"", rnb.get.directory(report, "data"), "/", fname,"\">","csv","</a>"),collapse="")
			return(txt)
		})
	}))
	rownames(file.tab) <- comps
	colnames(file.tab) <- reg.types
	rnb.add.table(report,file.tab)
	logger.completed()
	
	if(rnb.getOption("differential.variability")){
	  report <- rnb.section.diffVar.region(rnbSet,diffmeth,report,gzTable=gzTable,level=2)
	}
	
	sectionText <- "No GO Enrichment Analysis was conducted"
	if (class(dm.go.enrich)=="DiffMeth.go.enrich" & length(dm.go.enrich$region)>0){
		refText.gostats <- c("Falcon, S., & Gentleman, R. (2007). Using GOstats to test gene lists for GO term association. <i>Bioinformatics</i>, <b>23</b>(2), 257-258")
		report <- rnb.add.reference(report, refText.gostats)
		sectionText <- c(
			"GO Enrichment Analysis was conducted using a hypergeometric test that addresses the hierarchical structure of the ontology ",
			"(see ", rnb.get.reference(report, refText.gostats), " for details). ",
			"The wordclouds and tables below contain significant GO terms according to these tests."
		)
	}
	report <- rnb.add.section(report, 'GO Enrichment Analysis', sectionText, level = 2)
	if (class(dm.go.enrich)=="DiffMeth.go.enrich" && length(dm.go.enrich$region)>0){
		logger.start("Adding GO enrichment analysis results")
		rnb.require("annotate")
		comps <- names(dm.go.enrich$region)
		names(comps) <- paste("comp",1:length(comps),sep="")
		ontol <- names(dm.go.enrich$region[[1]])
		names(ontol) <- paste("ontol",1:length(ontol),sep="")
		names(reg.types) <- paste("reg",1:length(reg.types),sep="")
		hyper.hypo <- c("hypermethylated","hypomethylated")
		names(hyper.hypo) <- c("hyper","hypo")
		#reorder to make genes the default selection if present
		genes.ind <- which(reg.types=="genes")
		if (length(genes.ind)>0){
			reg.types <- reg.types[c(genes.ind,setdiff(1:length(reg.types),genes.ind))]
		}
		rank.cuts <- paste("combined rank among the ",diffRegionRankCut," best ranking regions",sep="")
		rank.cuts <- c(rank.cuts,paste("automatically selected rank cutoff"))
		rank.cuts.names.dm.go.enrich <-  paste("rankCut_",c(diffRegionRankCut,"autoSelect"),sep="")
		names(rank.cuts) <- c(paste("rc",1:length(diffRegionRankCut),sep=""),"rcAuto")
		names(rank.cuts.names.dm.go.enrich) <- c(paste("rc",1:length(diffRegionRankCut),sep=""),"rcAuto")
		setting.names <- list(
				'comparison' = comps,
				'Hypermethylation/hypomethylation' = hyper.hypo,
				'ontology' = ontol,
				'regions' = reg.types,
				'differential methylation measure' = rank.cuts)
		
		colnames2round <- c("Pvalue","OddsRatio","ExpCount")
		do.enrichment.table <- function(ccn,hhn,oon,rrn,rcn){
			rnb.require("Category")
			cc <- comps[ccn]
			hh <- hyper.hypo[hhn]
			oo <- ontol[oon]
			rr <- reg.types[rrn]
			rc <- rank.cuts.names.dm.go.enrich[rcn]

			ee <- dm.go.enrich$region[[cc]][[oo]][[rr]][[rc]][[hhn]]
			kk <- paste(c(ccn,hhn,oon,rrn,rcn),collapse="_")
			if (!is.null(ee)){
				if (length(sigCategories(ee))>0){
					tt <- robustHyperGResultSummary(ee,htmlLinks=TRUE)
					tt[,colnames2round] <- round(tt[,colnames2round],4)
				}
				else {
					tt <- data.frame("NA"=NA)
				}
			} else {
				tt <- data.frame("NA"=NA)
			}
			return(tt)
		}
		do.enrichment.table.var <- function(ccn,hhn,oon,rrn,rcn){
		  rnb.require("Category")
		  cc <- comps[ccn]
		  hh <- hyper.hypo[hhn]
		  oo <- ontol[oon]
		  rr <- reg.types[rrn]
		  rc <- rank.cuts.names.dm.go.enrich[rcn]
		  
		  ee <- dm.go.enrich$region_var[[cc]][[oo]][[rr]][[rc]][[hhn]]
		  kk <- paste(c("var",ccn,hhn,oon,rrn,rcn),collapse="_")
		  if (!is.null(ee)){
		    if (length(sigCategories(ee))>0){
		      tt <- robustHyperGResultSummary(ee,htmlLinks=TRUE)
		      tt[,colnames2round] <- round(tt[,colnames2round],4)
		    }
		    else {
		      tt <- data.frame("NA"=NA)
		    }
		  } else {
		    tt <- data.frame("NA"=NA)
		  }
		  return(tt)
		}
		do.enrichment.wordcloud <- function(ccn,hhn,oon,rrn,rcn){
			cc <- comps[ccn]
			hh <- hyper.hypo[hhn]
			oo <- ontol[oon]
			rr <- reg.types[rrn]
			rc <- rank.cuts.names.dm.go.enrich[rcn]

			ee <- dm.go.enrich$region[[cc]][[oo]][[rr]][[rc]][[hhn]]
							
			kk <- paste(c(ccn,hhn,oon,rrn,rcn),collapse="_")
			figName <- paste("enrichGOwordcloud_",kk,sep="")
			report.plot <- addReportPlots.diffMeth.enrich.GO.wordcloud(report,ee,figName)
			return(report.plot)
		}
		do.enrichment.wordcloud.var <- function(ccn,hhn,oon,rrn,rcn){
		  cc <- comps[ccn]
		  hh <- hyper.hypo[hhn]
		  oo <- ontol[oon]
		  rr <- reg.types[rrn]
		  rc <- rank.cuts.names.dm.go.enrich[rcn]
		  
		  ee <- dm.go.enrich$region_var[[cc]][[oo]][[rr]][[rc]][[hhn]]
		  
		  kk <- paste(c(ccn,hhn,oon,rrn,rcn),collapse="_")
		  figName <- paste("enrichGOwordcloudVar_",kk,sep="")
		  report.plot <- addReportPlots.diffMeth.enrich.GO.wordcloud(report,ee,figName)
		  return(report.plot)
		}

		#generate tuples of parameter combinations
		pps <- expand.grid(names(comps),names(hyper.hypo),names(ontol),names(reg.types),names(rank.cuts),stringsAsFactors=FALSE)
		kks <- paste(pps[,1],pps[,2],pps[,3],pps[,4],pps[,5],sep="_")

		tabs2write <- lapply(1:nrow(pps),FUN=function(k){
			do.enrichment.table(pps[k,1],pps[k,2],pps[k,3],pps[k,4],pps[k,5])

		})
		names(tabs2write) <- kks
		addedPlots <- lapply(1:nrow(pps),FUN=function(k){
			do.enrichment.wordcloud(pps[k,1],pps[k,2],pps[k,3],pps[k,4],pps[k,5])

		})
		names(addedPlots) <- kks
		
		description <- "Wordclouds for GO enrichment terms."
		report <- rnb.add.figure(report, description, addedPlots, setting.names)
		report <- rnb.add.tables(report, tabs2write, setting.names, row.names = FALSE)
		if(is.element("region_var",names(dm.go.enrich))){
		  tabs2write.var <- lapply(1:nrow(pps),FUN=function(k){
		    do.enrichment.table.var(pps[k,1],pps[k,2],pps[k,3],pps[k,4],pps[k,5])
		    
		  })
		  kks <- paste(kks,sep = "_")
		  names(tabs2write.var) <- kks
		  addedPlots.var <- lapply(1:nrow(pps),FUN=function(k){
		    do.enrichment.wordcloud.var(pps[k,1],pps[k,2],pps[k,3],pps[k,4],pps[k,5])
		    
		  })
		  names(addedPlots) <- kks
		  
		  hyper.hypo <- c("hypervariable","hypovariable")
		  names(hyper.hypo) <- c("hyper","hypo")
		  setting.names$'Hypermethylation/hypomethylation' <- hyper.hypo
		  sec.text <- "GO enrichment analysis was also performed for differentially variable regions (see differential methylation section for details on the methods)."
		  report <- rnb.add.section(report, title = "Differential Variability",description = sec.text,level = 3)
		  description <- "Workclouds for GO enrichment terms (Differential Variability)"
		  report <- rnb.add.figure(report,description,addedPlots.var,setting.names)
		  report <- rnb.add.tables(report,tabs2write.var,setting.names,row.names = FALSE)
		}
		logger.completed()
	}

	sectionText <- "No LOLA Enrichment Analysis was conducted"
	lolaDone <- class(dm.lola.enrich)=="DiffMeth.lola.enrich" && length(dm.lola.enrich$region)>0
	if (lolaDone){
		refText.lola <- c("Sheffield, N. C., & Bock, C. (2016). LOLA: enrichment analysis for genomic region sets and regulatory elements in R and Bioconductor. ",
			"<i>Bioinformatics</i>, <b>32</b>(4), 587-589")
		report <- rnb.add.reference(report, refText.lola)
		sectionText <- c("LOLA Enrichment Analysis ", rnb.get.reference(report, refText.lola),
		 " was conducted. The plots and tables below show enrichments across annotations in the supplied LOLA ",
		 "reference databases for the following collections:"
		)
	}
	report <- rnb.add.section(report, 'LOLA Enrichment Analysis', sectionText, level = 2)
	if (lolaDone){
		logger.start("Adding LOLA enrichment analysis results")
		lolaDb <- dm.lola.enrich$lolaDb
		rnb.add.list(report, as.list(lolaDb$collectionAnno[["collectionname"]]))

		comps <- names(dm.lola.enrich$region)
		names(comps) <- paste("comp",1:length(comps),sep="")
		reg.types <- get.region.types(diffmeth)
		names(reg.types) <- paste("reg",1:length(reg.types),sep="")
		hyper.hypo <- c("hypermethylated","hypomethylated")
		names(hyper.hypo) <- c("hyper","hypo")

		rank.cuts <- paste("combined rank among the ",diffRegionRankCut," best ranking regions",sep="")
		rank.cuts <- c(rank.cuts,paste("automatically selected rank cutoff"))
		rank.cuts.names.dm.lola.enrich <-  paste("rankCut_",c(diffRegionRankCut,"autoSelect"),sep="")
		names(rank.cuts) <- c(paste("rc",1:length(diffRegionRankCut),sep=""),"rcAuto")
		names(rank.cuts.names.dm.lola.enrich) <- c(paste("rc",1:length(diffRegionRankCut),sep=""),"rcAuto")
		setting.names <- list(
				'comparison' = comps,
				'Hypermethylation/hypomethylation' = hyper.hypo,
				'regions' = reg.types,
				'differential methylation measure' = rank.cuts)
		
		lolaTargets <- sort(unique(getTargetFromLolaDb(lolaDb)))
		targetColors <- sample(rainbow(length(lolaTargets), v=0.5))
		names(targetColors) <- lolaTargets

		collectionNames <- unique(lolaDb$collectionAnno[["collectionname"]])
		collectionColors <- rep(rnb.getOption("colors.category"), length.out=length(collectionNames))
		names(collectionColors) <- collectionNames

		volcano.colorBy <- c("maxRnk"="combined LOLA rank", "target"="target", "collection"="LOLA DB collection")
		setting.names.volcano <- c(setting.names, list('color'=volcano.colorBy))

		lolaVolcanoPlots <- list()
		lolaBarPlots <- list()
		lolaBoxPlots <- list()
		lolaVolcanoPlots.var <- list()
		lolaBarPlots.var <- list()
		lolaBoxPlots.var <- list()
		for (ccn in names(comps)){
			cc <- comps[ccn]
			for (rrn in names(reg.types)){
				rr <- reg.types[rrn]
				dmRes <- dm.lola.enrich$region[[cc]][[rr]]
				for (rcn in names(rank.cuts.names.dm.lola.enrich)){
					rc <- rank.cuts.names.dm.lola.enrich[rcn]
					# dmTab.hh <- dmRes[grepl(paste0("^", rc), dmRes[["userSet"]]),]
					for (hhn in names(hyper.hypo)){
						hh <- hyper.hypo[hhn]
						kk <- paste(c(ccn,hhn,rrn,rcn),collapse="_")
						dmTab <- dmRes[dmRes[["userSet"]]==paste(rc, hhn, sep="_"),]
						# print(dmTab)

						for (vcbn in names(volcano.colorBy)){
							kkk <- paste(kk, vcbn, sep="_")
							figName <- paste("lolaVolcano_", kkk, sep="")
							cpanel <- c()
							if (vcbn == "target")     cpanel <- targetColors
							if (vcbn == "collection") cpanel <- collectionColors
							pp <- tryCatch(
								lolaVolcanoPlot(lolaDb, dmTab, signifCol="qValue", colorBy=vcbn, colorpanel=cpanel),
								error = function(e) {
									logger.warning(c("Plotting error:", e$message))
									rnb.message.plot("Plotting error")
								}
							)
							if (vcbn == "target") pp <- pp + guides(color=FALSE)
							rPlot <- createReportGgPlot(pp, figName, report, create.pdf=FALSE, high.png=200)
							lolaVolcanoPlots[[kkk]] <- off(rPlot, handle.errors=TRUE)
						}
						

						figName <- paste("lolaBox_", kk, sep="")
						pp <- tryCatch(
							lolaBoxPlotPerTarget(lolaDb, dmTab, scoreCol="oddsRatio", orderCol="maxRnk", pvalCut=0.01, colorpanel=targetColors, maxTerms=100),
							error = function(e) {
								logger.warning(c("Plotting error:", e$message))
								rnb.message.plot("Plotting error")
							}
						)
						rPlot <- createReportGgPlot(pp, figName, report, create.pdf=TRUE, width=20, height=5)
						# lolaBoxPlots[[kk]] <- off(rPlot, handle.errors=TRUE)
						lolaBoxPlots[[kk]] <- suppressMessages(off(rPlot, handle.errors=TRUE))

						figName <- paste("lolaBar_", kk, sep="")
						pp <- tryCatch(
							lolaBarPlot(lolaDb, dmTab, scoreCol="oddsRatio", orderCol="maxRnk", pvalCut=0.01, colorpanel=targetColors, maxTerms=100),
							error = function(e) {
								logger.warning(c("Plotting error:", e$message))
								rnb.message.plot("Plotting error")
							}
						)
						rPlot <- createReportGgPlot(pp, figName, report, create.pdf=TRUE, width=20, height=5)
						lolaBarPlots[[kk]] <- off(rPlot, handle.errors=TRUE)
					}
				}
				if(is.element("region_var",names(dm.lola.enrich))){
				  dmRes <- dm.lola.enrich$region_var[[cc]][[rr]]
  				for (rcn in names(rank.cuts.names.dm.lola.enrich)){
  				  rc <- rank.cuts.names.dm.lola.enrich[rcn]
  				  for (hhn in names(hyper.hypo)){
  				    hh <- hyper.hypo[hhn]
  				    kk <- paste(c(ccn,hhn,rrn,rcn),collapse="_")
  				    dmTab <- dmRes[dmRes[["userSet"]]==paste(rc, hhn, sep="_"),]

  				    for (vcbn in names(volcano.colorBy)){
  				      kkk <- paste(kk, vcbn, sep="_")
  				      figName <- paste("lolaVolcanoVar_", kkk, sep="")
  				      cpanel <- c()
  				      if (vcbn == "target")     cpanel <- targetColors
  				      if (vcbn == "collection") cpanel <- collectionColors
  				      pp <- tryCatch(
  				      	lolaVolcanoPlot(lolaDb, dmTab, signifCol="qValue", colorBy=vcbn, colorpanel=cpanel),
  				      	error = function(e) {
  				      		logger.warning(c("Plotting error:", e$message))
  				      		rnb.message.plot("Plotting error")
  				      	}
  				      )
  				      if (vcbn == "target") pp <- pp + guides(color=FALSE)
  				      rPlot <- createReportGgPlot(pp, figName, report, create.pdf=FALSE, high.png=200)
  				      lolaVolcanoPlots.var[[kkk]] <- off(rPlot, handle.errors=TRUE)
  				    }
  				    
  				    
  				    figName <- paste("lolaBoxVar_", kk, sep="")
  				    pp <- tryCatch(
  				    	lolaBoxPlotPerTarget(lolaDb, dmTab, scoreCol="oddsRatio", orderCol="maxRnk", pvalCut=0.01, colorpanel=targetColors, maxTerms=100),
  				    	error = function(e) {
  				    		logger.warning(c("Plotting error:", e$message))
  				    		rnb.message.plot("Plotting error")
  				    	}
  				    )
  				    rPlot <- createReportGgPlot(pp, figName, report, create.pdf=TRUE, width=20, height=5)
  				    lolaBoxPlots.var[[kk]] <- suppressMessages(off(rPlot, handle.errors=TRUE))
  				    
  				    figName <- paste("lolaBarVar_", kk, sep="")
  				    pp <- tryCatch(
  				    	lolaBarPlot(lolaDb, dmTab, scoreCol="oddsRatio", orderCol="maxRnk", pvalCut=0.01, colorpanel=targetColors, maxTerms=100),
  				    	error = function(e) {
  				    		logger.warning(c("Plotting error:", e$message))
  				    		rnb.message.plot("Plotting error")
  				    	}
  				    )
  				    rPlot <- createReportGgPlot(pp, figName, report, create.pdf=TRUE, width=20, height=5)
  				    lolaBarPlots.var[[kk]] <- off(rPlot, handle.errors=TRUE)
  				  }
  				}
				}
			}
		}

		desc <- c(
			"Scatter plot showing the effect size (log-odds ratio) vs. the significance (-log10(q-value)), similar to a 'volcano plot' ",
			"as it is called in other contexts."
		)
		report <- rnb.add.figure(report, desc, lolaVolcanoPlots, setting.names.volcano)
		
		desc <- c(
			"Boxplots showing log-odds ratios from LOLA enrichment analysis. Shown are those groups of terms  per category ",
			"that share the same putative target. Only terms that exhibit statistical significance (p-value < 0.01) are included. ",
			"If more than 100 terms are enriched, the 100 terms receiving the highest joined ",
			"LOLA ranks are shown. Coloring of the bars reflects the putative targets of the terms."
		)
		report <- rnb.add.figure(report, desc, lolaBoxPlots, setting.names)

		desc <- c(
			"Barplots showing log-odds ratios from LOLA enrichment analysis. Shown are those terms that exhibit statistical ",
			"significance (p-value < 0.01). If more than 100 terms are enriched, the 100 terms receiving the highest joined ",
			"LOLA ranks are shown. Coloring of the bars reflects the putative targets of the terms."
		)
		report <- rnb.add.figure(report, desc, lolaBarPlots, setting.names)

		if(rnb.getOption("differential.variability") && is.element("region_var",names(dm.lola.enrich))){
		  hyper.hypo <- c("hypervariable","hypovariable")
		  names(hyper.hypo) <- c("hyper","hypo")
		  setting.names.volcano$'Hypermethylation/hypomethylation' <- hyper.hypo
		  setting.names$'Hypermethylation/hypomethylation' <- hyper.hypo
		  sec.text <- "LOLA enrichment analysis was also conducted for differentially variable regions."
		  report <- rnb.add.section(report = report, title = "Differential Variability",description = sec.text,level = 3)
		  report <- rnb.add.figure(report,desc,lolaVolcanoPlots.var,setting.names.volcano)
		  report <- rnb.add.figure(report,desc,lolaBoxPlots.var,setting.names)
		  report <- rnb.add.figure(report,desc,lolaBarPlots.var,setting.names)
		}
		
		logger.completed()
	}
	
	logger.completed()
	return(report)
}

#' get.adjustment.variables
#'
#' Given indices for two groups of samples for comparison, this function
#' retrieves \code{data.frame} containing the variables to be adjusted for
#' @author Fabian Mueller
#' @param rnbSet RnBSet object
#' @param inds.g1 sample indices in \code{rnbSet} of group 1 members
#' @param inds.g2 sample indices in \code{rnbSet} of group 2 members
#' @param colnames.adj column names in \code{pheno(rnbSet)} to retrieve
#' @param colname.target column names in \code{pheno(rnbSet)} of the target variable. Only important if \code{adjust.sva==TRUE}
#' @param adjust.sva flag indicating whether the resulting table should also contain surrogate variables (SVs) for the given target variable.
#' @param adjust.celltype flag indicating whether the resulting table should also contain estimated celltype contributions.
#' 				See \code{\link{rnb.execute.ct.estimation}} for details.
#' @return a \code{data.frame} containing one column for each selected variable from the phenotypic data
#'         each row corresponds to a sample in the union of samples of the wto groups with the first
#'         \code{length(inds.g1)} rows corresponding to group 1 and the remaining rows corresponding to group 2
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' sample.groups <- rnb.sample.groups(rnb.set.example)[[1]]
#' get.adjustment.variables(rnb.set.example,sample.groups[[1]],sample.groups[[2]],"Cell_Line")
#' }
get.adjustment.variables <- function(rnbSet, inds.g1, inds.g2=-inds.g1, colnames.adj=c(), colname.target="",
		adjust.sva=FALSE, adjust.celltype=FALSE){
	if (!all(colnames.adj %in% colnames(pheno(rnbSet)))){
		stop("invalid adjustment columns specified. please make sure they are a subset of the sample annotaiton table")
	}
	if (length(inds.g1)>=nrow(pheno(rnbSet))){
		stop("invalid group indexing. You must specify 2 groups")
	}
	if (is.element(colname.target,colnames.adj)){
		logger.info(c(colname.target, "is declared as part of the adjustment covariates. --> removed it from the list"))
		colnames.adj <- setdiff(colnames.adj,colname.target)
	}
	sample.names <- c(samples(rnbSet)[inds.g1],samples(rnbSet)[inds.g2])
	res <- data.frame(matrix(nrow=length(sample.names),ncol=0))
	rownames(res) <- sample.names
	if (length(colnames.adj)>0){
		res <- rbind(pheno(rnbSet)[inds.g1,colnames.adj,drop=FALSE],pheno(rnbSet)[inds.g2,colnames.adj,drop=FALSE])
	}
	inds.g1.after <- 1:length(inds.g1)
	inds.g2.after <- (length(inds.g1)+1):nrow(res)
	for (cc in colnames.adj){
		#remove variables which contain NAs
		contains.na <- any(is.na(res[,cc]))
		#remove constant variables
		is.const <- length(unique(res[,cc])) == 1
		#remove unique variables
		is.uniq <- !is.numeric(res[,cc]) && length(unique(res[,cc])) == nrow(res)
		#remove variables in which the values coincide completely with the grouping
		coinc.group <- length(unique(res[inds.g1.after,cc]))==1 && length(unique(res[inds.g2.after,cc]))==1
		#convert character to factor
		if (contains.na || is.const || is.uniq || coinc.group) {
			res <- res[,setdiff(colnames(res),cc),drop=FALSE]
		} else {
			if (is.character(res[,cc])){
				res[,cc] <- factor(res[,cc])
			}
		}
	}
	#SVs
	if (adjust.sva){
		if (has.covariates.sva(rnbSet, colname.target)){
			sv.tab <- get.covariates.sva(rnbSet, colname.target)
			if (is.null(colnames(sv.tab))) colnames(sv.tab) <- paste0("sv",1:ncol(sv.tab))
			res.sv.tab <- rbind(sv.tab[inds.g1,,drop=FALSE],sv.tab[inds.g2,,drop=FALSE])
			res <- cbind(res,res.sv.tab)
		} else {
			logger.warning(c("Could not retrieve surrogate variables for target '",colname.target,"'"))
		}
	}
	# Celltypes
	if (adjust.celltype){
		if (has.covariates.ct(rnbSet)){
			if (!isTRUE(colname.target == attr(get.covariates.ct(rnbSet), "column"))){
				ct.tab <- get.covariates.ct(rnbSet)
				if (is.null(colnames(ct.tab))) colnames(ct.tab) <- paste0("celltype",1:ncol(sv.tab))
				res.ct.tab <- rbind(ct.tab[inds.g1,,drop=FALSE],ct.tab[inds.g2,,drop=FALSE])
				res <- cbind(res,res.ct.tab)
			}
		} else {
			logger.warning(c("Could not retrieve celltype contributions for target '",colname.target,"'"))
		}
	}
	return(res)
}

#' get.comparison.info
#'
#' retrieve the comparison information for an RnBSet object
#' @author Fabian Mueller
#' @param x \code{RnBSet} object
#' @param pheno.cols column names of the pheno slot in \code{x} on which the dataset should be partitioned. Those columns are required to be factors or logical.
#' 				     In case of factors, each group in turn will be compared to all other groups
#' @param region.types which region types should be processed for differential methylation
#' @param pheno.cols.all.pairwise integer or character vector specifying the colomns of \code{pheno(x)} on which all pairwise comparisons should be conducted.
#' 		 A value of \code{NULL} indicates no columns.
#' @param columns.pairs argument passed on to \code{rnb.sample.groups}. See its documentation for details.
#' @param columns.adj Column names or indices in the table of phenotypic information to be used for confounder adjustment in the
#'        differential methylation analysis.
#' @param adjust.sva flag indicating whether the adjustment table should also contain surrogate variables (SVs) for the given target variable.
#' @param adjust.celltype flag indicating whether the resulting table should also contain estimated celltype contributions.
#' 				See \code{\link{rnb.execute.ct.estimation}} for details.
#' @param pheno.cols.adjust.sva Target variables for SVA adjustment. Only important if \code{adjust.sva==TRUE}. Only the intersection of
#'			\code{pheno.cols} and \code{pheno.cols.adjust.sva} is considered for SVA adjustment.
#' @param adjust.na.rm Flag indicating whether NAs in the adjustment table should be removed.
#' @return a list containing one element for each comparison to be conducted. Each element is again a list containing:
#' \describe{
#'   \item{\code{comparison}}{the name of the comparison}
#'   \item{\code{pheno.colname}}{the column name of the sample annotation table the comparison is derived from}
#'   \item{\code{group.names}}{the names of the two groups being compared}
#'   \item{\code{group.inds}}{the sample indices of the samples belonging to the two groups}
#'   \item{\code{paired}}{flag indicating whether paired analysis is conducted}
#'   \item{\code{adj.sva}}{flag indicating whether adjustment for SVA is conducted}
#'   \item{\code{adj.celltype}}{flag indicating whether adjustment for cell type is conducted}
#'   \item{\code{adjustment.table}}{the covariate adjustment table. \code{NULL} if the comparison is not adjusted}
#'   \item{\code{region.types}}{the region types applicable to the analysis}
#' }
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' cmp.info <- get.comparison.info(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
#' cmp.info[[1]]
#' }
get.comparison.info <- function(x, pheno.cols=rnb.getOption("differential.comparison.columns"),
		region.types=rnb.region.types.for.analysis(x),
		pheno.cols.all.pairwise=rnb.getOption("differential.comparison.columns.all.pairwise"), 
		columns.pairs=rnb.getOption("columns.pairing"), columns.adj=rnb.getOption("covariate.adjustment.columns"),
		adjust.sva=rnb.getOption("differential.adjustment.sva"), pheno.cols.adjust.sva=rnb.getOption("inference.targets.sva"),
		adjust.celltype=rnb.getOption("differential.adjustment.celltype"),
		adjust.na.rm=TRUE){
	#example
	#rr <- get.comparison.info(rnb.set, pheno.cols=rnb.getOption("differential.comparison.columns"))
	if (is.null(pheno.cols)){
		pheno.cols <- colnames(pheno(x))
	}
	group.info <- rnb.sample.groups(x, columns = pheno.cols, columns.pairs = columns.pairs)
	if (length(group.info) < 1){
		logger.warning("No valid grouping information found. NULL returned")
		return(NULL)
	}
	group.info.all.pairwise <- rep(FALSE,length(group.info))
	names(group.info.all.pairwise) <- names(group.info)
	if (!is.null(pheno.cols.all.pairwise)){
		if (is.integer(pheno.cols.all.pairwise)){
			ccns <- colnames(pheno(x))
			if (any(pheno.cols.all.pairwise<1) | any(pheno.cols.all.pairwise>length(ccns))){
				logger.warning("Invalid value for pheno.cols.all.pairwise. NULL returned")
				return(NULL)
			}
			pheno.cols.all.pairwise <- ccns[pheno.cols.all.pairwise]
		}

		pheno.cols.all.pairwise <- intersect(pheno.cols.all.pairwise,names(group.info.all.pairwise))
		if (length(pheno.cols.all.pairwise)<1) {
			logger.warning("No pairwise comparison specifier could be matched to the comparisons conducted")
		} else {
			logger.info(c("Conducting all pairwise comparisons for columns:",paste(pheno.cols.all.pairwise,collapse=",")))
			logger.info("All pairwise comparisons are performed on a subset of sample groupings. Caution: this could increase the runtime significantly
						 due to combinatorial explosion.")
			group.info.all.pairwise[pheno.cols.all.pairwise] <- TRUE
		}
	}
	sva.targets <- intersect(pheno.cols,pheno.cols.adjust.sva)

	#helper function for excluding samples for which the adjustment table contains NAs
	rm.na.from.adj.tab <- function(rnb.set,adj.var.tab,inds1,inds2,cmp.name){
		res <- list(
			adj.var.tab=adj.var.tab,
			inds1 = inds1,
			inds2 = inds2,
			samples.excluded = c()
		)
		if (any(is.na(adj.var.tab))){
			sample.name2ind <- 1:length(samples(rnb.set))
			names(sample.name2ind) <- samples(rnb.set)
			samples.na <- apply(adj.var.tab,1,FUN=function(x){any(is.na(x))})
			names(samples.na) <- rownames(adj.var.tab)
			sample.names.na <- rownames(adj.var.tab)[samples.na]
			sample.inds.na <- sample.name2ind[sample.names.na]

			res$inds1 <- setdiff(inds1,sample.inds.na)
			res$inds2 <- setdiff(inds2,sample.inds.na)
			res$adj.var.tab <- na.omit(adj.var.tab)
			res$samples.excluded <- sample.names.na
			warn.msg <- paste0("The following samples were excluded from comparison '",cmp.name,
				"' because they contain missing values in the adjustment table:", paste(sample.names.na,collapse=",")
			)
			logger.warning(warn.msg)
		}
		return(res)
	}

	res <- list()
	for (i in 1:length(group.info)){
		groups <- group.info[[i]]
		is.paired <- attr(group.info,"paired")[i]
		cc <- names(group.info)[i]
		
		adj.cols <- setdiff(columns.adj,cc)
		adj.sva <- cc %in% sva.targets && adjust.sva && has.covariates.sva(x, cc)
		## FIXME: Cell Type adjustment is disabled when running diff. methylation on the same column
		## It must be disabled in all traits for which adjustment cannot be made + we need to announce this to the user
		adj.celltype <- adjust.celltype && has.covariates.ct(x) && !isTRUE(cc == attr(get.covariates.ct(x), "column"))
		get.adj.tab <- length(adj.cols)>0 || adj.sva || adj.celltype

		if (length(groups)>2){
			if (group.info.all.pairwise[i]) {
				#perform all pairwise comparisons
				pps <- combn(1:length(groups),2)
				for (j in 1:ncol(pps)){
					pp1 <- pps[1,j]
					pp2 <- pps[2,j]
					ll1 <- names(groups)[pp1]
					ll2 <- names(groups)[pp2]
					grps <- c(ll1,ll2)
					comparison <- paste(paste(grps,collapse=" vs. ")," (based on ",cc,")",sep="")

					inds1 <- groups[[pp1]]
					inds2 <- groups[[pp2]]
					adj.var.tab <- NULL
					if (get.adj.tab) {
						adj.var.tab <- get.adjustment.variables(x,inds1,inds2,adj.cols, colname.target=cc, adjust.sva=adj.sva, adjust.celltype=adj.celltype)
					}
					if (adjust.na.rm & !is.null(adj.var.tab)) {
						adj.clean.helper.object <- rm.na.from.adj.tab(x,adj.var.tab,inds1,inds2,comparison)
						inds1 <- adj.clean.helper.object$inds1
						inds2 <- adj.clean.helper.object$inds2
						adj.var.tab <- adj.clean.helper.object$adj.var.tab
					}
					res.cur <- list(
						comparison=comparison,
						pheno.colname=cc,
						group.names=grps,
						group.inds=list(group1=inds1,group2=inds2),
						paired=is.paired,
						adj.sva=adj.sva,
						adj.celltype=adj.celltype,
						adjustment.table=adj.var.tab,
						region.types=region.types
					)
					res.append <- list(res.cur)
					names(res.append) <- res.cur$comparison
					res <- c(res,res.append)
				}
			} else {
				for (j in 1:length(groups)){
					ll <- names(groups)[j]
					grps <- c(ll,paste("non.",ll,sep=""))
					if (is.paired){
						logger.warning(c("Paired analysis is not supported annotations with more than 2 categories and comparing one group vs. all others. ",
									"--> Using unpaired analysis.",
									"Consider enabling the differential.comparison.columns.all.pairwise option or reducing the number of groups ",
									"in this column to 2."))
						is.paired <- FALSE
					}
					comparison <- paste(paste(grps,collapse=" vs. ")," (based on ",cc,")",sep="")
					
					inds1 <- groups[[j]]
					inds2 <- setdiff(unlist(groups),inds1)
					adj.var.tab <- NULL
					if (get.adj.tab) {
						adj.var.tab <- get.adjustment.variables(x,inds1,inds2,adj.cols, colname.target=cc, adjust.sva=adj.sva, adjust.celltype=adj.celltype)
					}
					if (adjust.na.rm & !is.null(adj.var.tab)) {
						adj.clean.helper.object <- rm.na.from.adj.tab(x,adj.var.tab,inds1,inds2,comparison)
						inds1 <- adj.clean.helper.object$inds1
						inds2 <- adj.clean.helper.object$inds2
						adj.var.tab <- adj.clean.helper.object$adj.var.tab
					}
					res.cur <- list(
						comparison=comparison,
						pheno.colname=cc,
						group.names=grps,
						group.inds=list(group1=inds1,group2=inds2),
						paired=is.paired,
						adj.sva=adj.sva,
						adj.celltype=adj.celltype,
						adjustment.table=adj.var.tab,
						region.types=region.types
					)
					res.append <- list(res.cur)
					names(res.append) <- res.cur$comparison
					res <- c(res,res.append)
				}
			}
		} else { # length(groups) == 2
			ll1 <- names(groups)[1]
			ll2 <- names(groups)[2]
			grps <- c(ll1,ll2)
			comparison <- paste0(paste(grps,collapse=" vs. ")," (based on ",cc,")")
			inds1 <- groups[[1]]
			inds2 <- groups[[2]]
			adj.var.tab <- NULL
			if (get.adj.tab) {
				adj.var.tab <- get.adjustment.variables(x,inds1,inds2,adj.cols, colname.target=cc, adjust.sva=adj.sva, adjust.celltype=adj.celltype)
			}
			if (adjust.na.rm & !is.null(adj.var.tab)) {
				adj.clean.helper.object <- rm.na.from.adj.tab(x,adj.var.tab,inds1,inds2,comparison)
				inds1 <- adj.clean.helper.object$inds1
				inds2 <- adj.clean.helper.object$inds2
				adj.var.tab <- adj.clean.helper.object$adj.var.tab
			}
			res.cur <- list(
				comparison=comparison,
				pheno.colname=cc,
				group.names=grps,
				group.inds=list(group1=inds1,group2=inds2),
				paired=is.paired,
				adj.sva=adj.sva,
				adj.celltype=adj.celltype,
				adjustment.table=adj.var.tab,
				region.types=region.types
			)
			res.append <- list(res.cur)
			names(res.append) <- res.cur$comparison
			res <- c(res,res.append)
		}
	}
	return(res)
}

#' rnb.execute.computeDiffMeth
#'
#' computes differential methylation
#' @author Fabian Mueller
#' @aliases rnb.execute.computeDiffMeth
#' @param x RnBSet object
#' @param pheno.cols column names of the pheno slot in \code{x} on which the dataset should be partitioned. Those columns are required to be factors or logical.
#' 				     In case of factors, each group in turn will be compared to all other groups
#' @param region.types which region types should be processed for differential methylation
#' @param covg.thres coverage threshold for computing the summary statistics. See \code{\link{computeDiffTab.extended.site}} for details.
#' @param pheno.cols.all.pairwise integer or character vector specifying the colomns of \code{pheno(x)} on which all pairwise comparisons should be conducted.
#' 		 A value of \code{NULL} (default) indicates no columns.
#' @param columns.pairs argument passed on to \code{rnb.sample.groups}. See its documentation for details.
#' @param columns.adj Column names or indices in the table of phenotypic information to be used for confounder adjustment in the
#'        differential methylation analysis.
#' @param adjust.sva flag indicating whether the adjustment table should also contain surrogate variables (SVs) for the given target variable.
#' @param adjust.celltype flag indicating whether the resulting table should also contain estimated celltype contributions.
#' 				See \code{\link{rnb.execute.ct.estimation}} for details.
#' @param skip.sites flag indicating whether differential methylation in regions should be computed directly and not from sites. This leads to skipping of site-specific differential methylation
#' @param pheno.cols.adjust.sva Column names or indices in the table of phenotypic information to be used for SVA adjustment in the
#'        differential methylation analysis.
#' @param disk.dump Flag indicating whether the resulting differential methylation object should be file backed, ie.e the matrices dumped to disk
#' @param disk.dump.dir disk location for file backing of the resulting differential methylation object. Only meaningful if \code{disk.dump=TRUE}.
#' 						must be a character specifying an NON-EXISTING valid directory.
#' @param ... arguments passed on to binary differential methylation calling. See \code{\link{computeDiffTab.extended.site}} for details.
#' @return an \code{\linkS4class{RnBDiffMeth}} object. See class description for details.
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
#' get.comparisons(dm)
#' }
rnb.execute.computeDiffMeth <- function(
  x,
  pheno.cols,
  region.types = rnb.region.types.for.analysis(x),
  covg.thres = rnb.getOption("filtering.coverage.threshold"),
  pheno.cols.all.pairwise = rnb.getOption("differential.comparison.columns.all.pairwise"),
  columns.pairs=rnb.getOption("columns.pairing"),
  columns.adj=rnb.getOption("covariate.adjustment.columns"),
  adjust.sva=rnb.getOption("differential.adjustment.sva"), 
  pheno.cols.adjust.sva=rnb.getOption("inference.targets.sva"),
  adjust.celltype=rnb.getOption("differential.adjustment.celltype"),
  skip.sites=!rnb.getOption("analyze.sites"),
  disk.dump=rnb.getOption("disk.dump.big.matrices"),
  disk.dump.dir=tempfile(pattern="diffMethTables_"),
  ...
) {
	logger.start("Retrieving comparison info")
	cmp.info <- get.comparison.info(x, pheno.cols=pheno.cols, region.types=region.types, 
		pheno.cols.all.pairwise=pheno.cols.all.pairwise, columns.pairs=columns.pairs, columns.adj=columns.adj,
		adjust.sva=adjust.sva, pheno.cols.adjust.sva=pheno.cols.adjust.sva, adjust.celltype=adjust.celltype)
	logger.completed()
	if (is.null(cmp.info)) {
		return(NULL)
	}

	diff.method <- rnb.getOption("differential.site.test.method")
	variability.method <- rnb.getOption("differential.variability.method")
	dot.args <- list(...)

	# determine whether residuals should be computed based on rnb option
	.resid.opt <- rnb.getOption("differential.residuals.output")
	.request.resid <- (!is.null(.resid.opt))

	if (is.element("diff.method",names(dot.args))){
		diff.method <- dot.args[["diff.method"]]
	}
	if(is.element("variability.method",names(dot.args))){
	  variability.method <- dot.args[["variability.method"]]
	}
	logger.start("Computing differential methylation tables")

	diffmeth <- new("RnBDiffMeth",site.test.method=diff.method,disk.dump=disk.dump,disk.path=disk.dump.dir)
	
	for (i in 1:length(cmp.info)){
		cmp.info.cur <- cmp.info[[i]]
		logger.start(c("Comparing:",cmp.info.cur$comparison))
		if (cmp.info.cur$paired){
			logger.status("Conducting PAIRED analysis")
		}

		if (skip.sites){
			logger.info("Skipping site-specific differential methylation calling")
			dm <- NULL
		} else {
			dm <- computeDiffMeth.bin.site(
				meth(x),
				inds.g1 = cmp.info.cur$group.inds$group1,
				inds.g2 = cmp.info.cur$group.inds$group2,
				covg = covg(x),
				covg.thres = covg.thres,
				paired = cmp.info.cur$paired,
				adjustment.table = cmp.info.cur$adjustment.table,
				imputed=isImputed(x),
				return.residuals = .request.resid,
				...
			)
			diffmeth <- addDiffMethTable(diffmeth,dm,cmp.info.cur$comparison,"sites",cmp.info.cur$group.names)
		}
		rnb.cleanMem()
		if (length(cmp.info.cur$region.types)>0){
			if (skip.sites){
				dmr <- computeDiffMeth.bin.region(
					x,
					NULL,
					cmp.info.cur$group.inds$group1,
					cmp.info.cur$group.inds$group2,
					region.types = cmp.info.cur$region.types,
					covg.thres = covg.thres,
					paired = cmp.info.cur$paired,
					adjustment.table = cmp.info.cur$adjustment.table,
					return.residuals = .request.resid,
					...
				)
			} else {
				dmr <- computeDiffMeth.bin.region(
					x,
					dm,
					cmp.info.cur$group.inds$group1,
					cmp.info.cur$group.inds$group2,
					region.types = cmp.info.cur$region.types,
					return.residuals = .request.resid
				)	
			}		
			for (rt in cmp.info.cur$region.types){
				diffmeth <- addDiffMethTable(diffmeth,dmr[[rt]],cmp.info.cur$comparison, 
					rt, cmp.info.cur$group.names
				)
			}
		}
		logger.completed()
	}

	diffmeth <- addComparisonInfo(diffmeth,cmp.info)
	logger.completed()
	return(diffmeth)
}

########################################################################################################################
## test_differential.R
## creator: Fabian Müller
## ---------------------------------------------------------------------------------------------------------------------
## Unit tests for differential methylation functions
########################################################################################################################

test_rowWelchP <- function(){
	require(RnBeads.hg19)
	data(small.example.object)
	meth.mat <- meth(rnb.set.example)
	sample.groups <- rnb.sample.groups(rnb.set.example)[[1]]
	p.vals <- rowWelchP(meth.mat,sample.groups[[1]],sample.groups[[2]])
	passes <- is.numeric(p.vals) && all(p.vals>=0) && all(p.vals<=1)
	checkTrue(passes)
}

test_limmaP_1 <- function(){
	require(RnBeads.hg19)
	data(small.example.object)
	meth.mat <- meth(rnb.set.example)
	sample.groups <- rnb.sample.groups(rnb.set.example)[[1]]
	p.vals <- limmaP(meth.mat,sample.groups[[1]],sample.groups[[2]])
	passes <- is.numeric(p.vals) && all(p.vals>=0) && all(p.vals<=1)
	checkTrue(passes)
}

test_limmaP_2 <- function() {
	require(RnBeads.hg19)
	data(small.example.object)
	X <- meth(rnb.set.example)
	tbl <- data.frame(
		stype = c("a", "b", "a", "b", "a", "b", "b", "b", "b", "a", "b", "a"),
		sex = c("f", "m", "f", "f", "f", "f", "m", "m", "f", "f", "m", "m"),
		group = c("1", "2", "1", "1", "1", "1", "2", "2", "1", "1", "2", "2"))

	p.vals <- limmaP(X, tbl$stype == "a", tbl$stype == "b", tbl[, -1])
	checkTrue(is.numeric(p.vals) && all(p.vals>=0) && all(p.vals<=1))
}

test_computeDiffTab.extended.site <- function(){
	require(RnBeads.hg19)
	data(small.example.object)
	meth.mat <- meth(rnb.set.example)
	sample.groups <- rnb.sample.groups(rnb.set.example)[[1]]
	dm <- computeDiffTab.extended.site(meth.mat,sample.groups[[1]],sample.groups[[2]])
	checkTrue(dim(dm)[1]>1)
	checkEquals(dim(dm)[2],15)
}

test_computeDiffTab.default.region <- function(){
	require(RnBeads.hg19)
	data(small.example.object)
	meth.mat <- meth(rnb.set.example)
	sample.groups <- rnb.sample.groups(rnb.set.example)[[1]]
	dm.sites <- computeDiffTab.extended.site(meth.mat,sample.groups[[1]],sample.groups[[2]])
	map.regions.to.sites <- regionMapping(rnb.set.example,"promoters")
	dm.promoters <- computeDiffTab.default.region(dm.sites,map.regions.to.sites)
	checkTrue(dim(dm.promoters)[1]>1)
	checkEquals(dim(dm.promoters)[2],9)
}

test_combineTestPvalsMeth <- function(){
	p.vals <- 10^-c(0,1,5)
	res <- combineTestPvalsMeth(p.vals)
	checkEquals(res,0.00011, tolerance=1e-5,scale=1)
}

test_get.adjustment.variables <- function(){
	require(RnBeads.hg19)
	data(small.example.object)
	sample.groups <- rnb.sample.groups(rnb.set.example)[[1]]
	res <- get.adjustment.variables(rnb.set.example,sample.groups[[1]],sample.groups[[2]],"Cell_Line")
	checkTrue(is.data.frame(res))
	checkEquals(colnames(res),"Cell_Line")
}

test_get.comparison.info <- function(){
	require(RnBeads.hg19)
	data(small.example.object)
	cmp.info <- get.comparison.info(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
	checkTrue(is.list(cmp.info))
	checkTrue(length(cmp.info)>0)
	req.names <- c("comparison","pheno.colname","group.names","group.inds","paired","adj.sva","adj.celltype","adjustment.table","region.types")
	checkTrue(all(req.names %in% names(cmp.info[[1]])))
}

test_rnb.execute.computeDiffMeth <- function(){
	require(RnBeads.hg19)
	data(small.example.object)
	dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
	checkEquals(as.character(class(dm)),"RnBDiffMeth",checkNames=FALSE)
}

test_diffVar <- function(){
  require(RnBeads.hg19)
  data(small.example.object)
  methData <- meth(rnb.set.example)
  groups <- rnb.sample.groups(rnb.set.example)
  ind1 <- groups$Sample_Group[[1]]
  ind2 <- groups$Sample_Group[[2]]
  vals <- diffVar(methData,ind1,ind2)
  if(all(is.na(vals))){
    logger.info("diffVar from missMethyl package not properly running")
    passed <- TRUE
  }else{
    passed <- is.numeric(vals) && all(vals>=0) && all(vals<=1)
  }  
  checkTrue(passed)
}

test_diffVar_covariate <- function() {
  require(RnBeads.hg19)
  data(small.example.object)
  X <- meth(rnb.set.example)
  tbl <- data.frame(
    stype = c("a", "b", "a", "b", "a", "b", "b", "b", "b", "a", "b", "a"),
    sex = c("f", "m", "f", "f", "f", "f", "m", "m", "f", "f", "m", "m"),
    group = c("1", "2", "1", "1", "1", "1", "2", "2", "1", "1", "2", "2"))
  
  p.vals <- diffVar(X, tbl$stype == "a", tbl$stype == "b", tbl[, -1])
  if(all(is.na(p.vals))){
    logger.info("diffVar from missMethyl package not properly running")
    passed <- TRUE
  }else{
    passed <- is.numeric(p.vals) && all(p.vals>=0) && all(p.vals<=1)
  }
  checkTrue(passed)
}

test_diffVar_paired <- function() {
  require(RnBeads.hg19)
  data(small.example.object)
  X <- meth(rnb.set.example)
  tbl <- data.frame(
    stype = c("a", "b", "a", "b", "a", "b", "a", "b", "a", "b", "a", "b"),
    donor = c("1", "1", "2", "2", "3", "3", "4", "4", "5", "5", "6", "6"))
  
  p.vals <- diffVar(X, tbl$stype == "a", tbl$stype == "b", paired=T)
  checkTrue(length(p.vals) == nrow(X))
}

test_iEVORA <- function(){
  require(RnBeads.hg19)
  data(small.example.object)
  methData <- meth(rnb.set.example)
  groups <- rnb.sample.groups(rnb.set.example)
  ind1 <- groups$Sample_Group[[1]]
  ind2 <- groups$Sample_Group[[2]]
  vals <- apply.iEVORA(methData,ind1,ind2)
  passed <- is.numeric(vals) && all(vals>=0) && all(vals<=1)
  checkTrue(passed)
}

test_rnb.execute.diffVar <- function(){
  require(RnBeads.hg19)
  data(small.example.object)
  dm <- rnb.execute.diffVar(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
  checkEquals(as.character(class(dm)),"RnBDiffMeth",checkNames=FALSE)
}



test_rnb.execute.diffMethANDVar <- function(){
  require(RnBeads.hg19)
  data(small.example.object)
  temp.option <- rnb.getOption("differential.variability")
  rnb.options(differential.variability=TRUE)
  dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
  rnb.options(differential.variability=temp.option)
  comps <- get.comparisons(dm)
  reg <- rnb.region.types()[1]
  passed <- is.element("combinedRank.var",colnames(get.table(dm,comps[1],reg)))
  checkTrue(passed)
}

test_class_RnBDiffMeth <- function(){
	require(RnBeads.hg19)
	data(small.example.object)
	#DiffMeth example objects
	dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
	dm1 <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group"),region.types=c("genes","tiling"))
	dm1b <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group"),region.types=c("genes","tiling"))
	dm2 <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"),region.types=c("promoters"))
	dm3 <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Treatment"),region.types=c("genes","tiling"))
	checkEquals(as.character(class(dm)), "RnBDiffMeth")
	checkEquals(as.character(class(dm1)),"RnBDiffMeth")
	checkEquals(as.character(class(dm2)),"RnBDiffMeth")
	checkEquals(as.character(class(dm3)),"RnBDiffMeth")
	logger.start(c("Testing function:","get.region.types"))
		res <- get.region.types(dm)
		checkEquals(res,rnb.region.types.for.analysis(rnb.set.example),checkNames=FALSE)
	logger.completed()
	logger.start(c("Testing function:","get.comparisons"))
		res <- get.comparisons(dm)
		checkTrue(length(res)>1)
	logger.completed()
	logger.start(c("Testing function:","get.comparison.grouplabels"))
		res <- get.comparison.grouplabels(dm)
		checkTrue(nrow(res)>1)
		checkEquals(ncol(res),2)
	logger.completed()
	logger.start(c("Testing function:","get.site.test.method"))
		res <- get.site.test.method(dm)
		checkEquals(res,rnb.getOption("differential.site.test.method"))
	logger.completed()
	logger.start(c("Testing function:","get.table"))
		res <- get.table(dm,get.comparisons(dm)[1],"promoters",return.data.frame=TRUE)
		checkTrue(nrow(res)>1)
		checkEquals(ncol(res),14)
	logger.completed()
	logger.start(c("Testing function:","addDiffMethTable"))
		sample.groups <- rnb.sample.groups(rnb.set.example,"Sample_Group")[[1]]
		dmt.sites <- computeDiffTab.extended.site(meth(rnb.set.example),sample.groups[[1]],sample.groups[[2]])
		map.regions.to.sites <- regionMapping(rnb.set.example,"promoters")
		dmt.promoters <- computeDiffTab.default.region(dmt.sites,map.regions.to.sites)
		cmp.name <- get.comparisons(dm1)[1]
		grp.labs <- get.comparison.grouplabels(dm1)[1,]
		#add the promoter level differential methylation table
		dm.add <- addDiffMethTable(dm1,dmt.promoters,cmp.name,"promoters",grp.labs)
		checkEquals(get.region.types(dm.add),c("genes","tiling","promoters"),checkNames=FALSE)
	logger.completed()
	logger.start(c("Testing functions:","join.diffMeth, is.valid"))
		dm.join1 <- join.diffMeth(dm1b,dm2)
		#the following joint object is invalid, because some region type - comparison combinations are missing
		checkTrue(!is.valid(dm.join1))
		dm.join2 <- join.diffMeth(dm.join1,dm3)
		#after joining the missing information, the new object is valid
		checkTrue(is.valid(dm.join2))
	logger.completed()
	logger.start("Destructor")
		destroy(dm.join2)
		destroy(dm.add)
		# destroy(dm1) #already included in the destructor of dm.add
		# destroy(dm1b) #already included in the destructor of dm.join2
		# destroy(dm.join2) #already included in the destructor of dm.join2
		destroy(dm2)
		destroy(dm3)
	logger.completed()
}

test_differential <- function(){
	require(RUnit)
	logger.start(fname=NA)
	logger.start(c("Unit testing:","differential"))
		logger.start(c("Testing function:","rowWelchP"))
			test_rowWelchP()
		logger.completed()
		logger.start(c("Testing function:","limmaP"))
			test_limmaP_1()
		logger.completed()
		logger.start(c("Testing function:","computeDiffTab.extended.site"))
			test_computeDiffTab.extended.site()
		logger.completed()
		logger.start(c("Testing function:","computeDiffTab.default.region"))
			test_computeDiffTab.default.region()
		logger.completed()
		logger.start(c("Testing function:","combineTestPvalsMeth"))
			test_combineTestPvalsMeth()
		logger.completed()
		logger.start(c("Testing function:","get.adjustment.variables"))
			test_get.adjustment.variables()
		logger.completed()
		logger.start(c("Testing function:","get.comparison.info"))
			test_get.comparison.info()
		logger.completed()
		logger.start(c("Testing function:","rnb.execute.computeDiffMeth"))
			test_rnb.execute.computeDiffMeth()
		logger.completed()
		logger.start("Testing function: diffVar")
		  test_diffVar()
		  test_diffVar_covariate()
		logger.completed()
		logger.start("Testing function: apply.iEVORA")
		  test_iEVORA()
		logger.completed()
		logger.start("Testing function: rnb.execute.diffVar")
		  test_rnb.execute.diffVar()
		logger.completed()
		logger.start("Testing function: rnb.execute.computeDiffMeth and Variability")
		  test_rnb.execute.diffMethANDVar()
		logger.completed()
		logger.start(c("Testing class:","RnBDiffMeth"))
			test_class_RnBDiffMeth()
		logger.completed()
	logger.completed()
}

test_differential()

#methods for subdividing regions into smaller regions of clustered CpG sites

########################################################################################################################
### get.subsegments
###
### For a given region. Subdivide this region into subsegments by
### hierarchical clustering on the site distances in a particular region and then splitting the into subregions consisting
### of these site clusters. The number of clusters is determined in such way that the mean number of sites per cluster
### is given by the \code{ns} parameter.
### @param reg.annot a \code{data.frame} of exactly 1 row as output by \code{annotation(rnb.set,type=...)} specifying the region to be subdivided
### @param site.coords the genomic start coordinates of the sites to be considered for subdivision
### @param ns the mean number of sites per cluster.
### @param site.length length of the site motif. Typically 2 for CpGs.
### @return a data.frame of the same format as \code{reg.annot} with each row corresponding
###				to a subsegment of the original region spanning first site to last site. All columns except
###				for genomic coordinates will be identical to \code{reg.annot}
### @author Fabian Mueller
get.subsegments <- function(reg.annot,site.coords,ns=10,site.length=2){
	res <- reg.annot
	n.sites <- length(site.coords)
	if (n.sites > ns){
		#do the clustering
		site.dists <- dist(site.coords,method="manhattan") #method doesn't matter
		site.clust <- hclust(site.dists,method="single")
		n.clust <- ceiling(n.sites/ns)
		clusts <- cutree(site.clust,k=n.clust)
		#TODO: see if the dynamicTreeCut package can improve the method in resonable time

		#repeat the region annotation for each cluster and just adjust the coordinates
		res <- reg.annot[rep(1,n.clust),]
		clust.coords.min <- tapply(site.coords,clusts,min)
		clust.coords.max <- tapply(site.coords,clusts,max)+(site.length-1)
		res$Start <- clust.coords.min
		res$End <- clust.coords.max
		rownames(res) <- paste0(rownames(reg.annot),".sub",1:n.clust)
	}
	return(res)
}

########################################################################################################################
### add.subsegment.annotation
###
### For a given region annotation data.frame. Subdivide each region into subsegments by
### hierarchical clustering on the site distances in a particular region and then splitting the into subregions consisting
### of these site clusters. The number of clusters is determined in such way that the mean number of sites per cluster
### is given by the \code{ns} parameter.
### @param reg.name the name/region.type of the newly generated subregions
### @param regions a \code{data.frame} as output by \code{annotation(rnb.set,type=...)} specifying the regions to be subdivided
### @param sites a \code{data.frame} as output by \code{annotation(rnb.set,type="sites")} specifying all sites to be considered
###				for subdivision.
### @param add.region.types.to.options Flag indicating whether to add the newly created subregions to the package's
### 				  \code{region.types} option
### @param regions2sites a mapping of regions to sites. A list as output by \code{regionMapping()}
### @param ns the mean number of sites per cluster.
### @param save.dir a directory to save the annotation to for later reloading. (binary \code{RData} format.)
### @param assembly the genome assembly to be used for the annotation
### @return nothing of particular interest
### @author Fabian Mueller
add.subsegment.annotation <- function(reg.name,regions,sites,regions2sites,ns=10,save.dir=NULL,assembly="hg19"){
	if (nrow(regions) != length(regions2sites)){
		stop("incompatible regions and regions2sites")
	}
	n.regions <- nrow(regions)

	logger.start("Subsegmentation")
	#parallel computing here taks up too much memory and serial computing is not great, but ok timewise
	# if (parallel.isEnabled()) {
	# 	subregions <- foreach(i=1:n.regions, .combine='rbind',.multicombine=TRUE,.maxcombine=100) %dopar% {
	# 		get.subsegments(regions[i,],sites[regions2sites[[i]],]$Start,ns=ns)
	# 	}
	# } else {
		subregions <- do.call(rbind,lapply(1:n.regions,FUN=function(i){
			get.subsegments(regions[i,],sites[regions2sites[[i]],]$Start,ns=ns)
		}))
	# }
	logger.completed()
	#renaming and reformating such that the annotation can be set
	names(subregions$Start) <- NULL
	names(subregions$End) <- NULL
	subregions$CpG <- NULL
	subregions$GC <- NULL
	subregions$Start <- as.integer(subregions$Start)
	subregions$End <- as.integer(subregions$End)
	# #TEMPRORARY for testing
	# fn <- file.path(save.dir,paste0("regions_",reg.name,"_tmp.RData"))
	# save(list=ls(),file=fn)
	rnb.cleanMem()
	logger.start("Adding CpG statistics")
	rnb.set.annotation.and.cpg.stats(reg.name,subregions,description=NULL,assembly=assembly)
	logger.completed()
	if (!is.null(save.dir)) {
		logger.start("Saving annotation")
		fn <- file.path(save.dir,paste0("regions_",reg.name,".RData"))
		rnb.save.annotation(fn,reg.name,assembly=assembly)
		logger.completed()
	}
}

########################################################################################################################
#' addRegionSubsegments
#'
#' For the region annotation of a given \code{RnBSet} object. Subdivide each region into subsegments by
#' hierarchical clustering on the site distances in a particular region and then splitting the region into subregions consisting
#' of these site clusters. The number of clusters is determined in such way that the mean number of sites per cluster
#' is given by the \code{ns} parameter.
#' @param rnb.set an \code{RnBSet object}
#' @param annotation.dir a directory to save the annotation to for later reloading. (binary \code{RData} format.)
#' @param region.types the region types to which subsegmentation should be applied. Must be a non-empty
#' 					   subset of \code{summarized.regions(rnb.set)}. Defaults (\code{NULL}) to all region types in
#' 					   \code{rnb.set}
#' @param add.region.types.to.options Flag indicating whether to add the newly created subregions to the package's
#' 				  \code{region.types} option
#' @param ns the mean number of sites per cluster.
#' @return the modified \code{RnBSet} object
#' @author Fabian Mueller
#' @export addRegionSubsegments
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' rnb.set.mod <- addRegionSubsegments(rnb.set.example,tempdir(),region.types=c("tiling","genes"))
#' summary(meth(rnb.set.mod,type="tiling.subsegments"))
#' }
addRegionSubsegments <- function(rnb.set,annotation.dir,region.types=NULL,add.region.types.to.options=FALSE,ns=10){
	logger.start("Adding region sub-segmentation to RnBSet")
	if (is.null(region.types)){
		region.types <- summarized.regions(rnb.set)
	} else {
		region.types <- intersect(region.types,summarized.regions(rnb.set))
	}
	if (length(region.types)<1){
		stop("Invalid region types specified for subsegmentation")
	}
	assembly <- assembly(rnb.set)
	#check if all regions are annotated
	if (!all(region.types %in% rnb.region.types(assembly=assembly))){
		stop(paste("Region annotation not found."))
	}

	aa.s <- annotation(rnb.set,"sites")
	for (rt in region.types){
		logger.status(c("Subsegmenting region:",rt))
		aa.r <- annotation(rnb.set,rt)
		regions2sites <- regionMapping(rnb.set,rt)
		subreg.name <- paste(rt,"subsegments",sep=".")
		logger.start("Annotating")
		add.subsegment.annotation(subreg.name,aa.r,aa.s,regions2sites,ns=ns,save.dir=annotation.dir,assembly=assembly)
		logger.completed()
		logger.start("Summarizing regions in RnBSet")
		rnb.set <- summarize.regions(rnb.set,region.type=subreg.name)
		logger.completed()
		if (add.region.types.to.options){
			rnb.options(region.types=c(rnb.getOption("region.types"),subreg.name))
		}
	}
	logger.completed()
	return(rnb.set)
}

########################################################################################################################
#' load.region.subsegment.annotation
#'
#' For the region annotation of a given \code{RnBSet} object. Subdivide each region into subsegments by
#' hierarchical clustering on the site distances in a particular region and then splitting the region into subregions consisting
#' of these site clusters. The number of clusters is determined in such way that the mean number of sites per cluster
#' is given by the \code{ns} parameter.
#' @param rnb.set The \code{RnBSet object} with subsegments specified in the regions
#' @param annotation.dir a directory to load the annotation from. (binary \code{RData} format.)
#' @return invisible \code{TRUE}
#' @author Fabian Mueller
#' @export
load.region.subsegment.annotation <- function(rnb.set,annotation.dir){
	logger.start("Loading region sub-segmentation annotation")
	region.types <- grep("\\.subsegments$",summarized.regions(rnb.set),value=TRUE)

	for (rt in region.types){
		logger.status(c("Loading annotation",rt))
		fn <- file.path(annotation.dir,paste0("regions_",rt,".RData"))
		rnb.load.annotation(fn, rt)
	}
	logger.completed()
	invisible(TRUE)
}

########################################################################################################################
#' rnb.section.region.subsegmentation
#'
#' Given a \code{RnBSet} with subsegmentation information, add a correspong section to the provided RnBeads report.
#' @param rnb.set The \code{RnBSet object} with subsegments specified in the regions
#' @param report Report to summarize the outcome of the region subsegmentation. This must be an object of type
#'               \code{\linkS4class{Report}}.
#' @return The modified \code{report}, invisibly.
#' @author Fabian Mueller
#' @noRd
rnb.section.region.subsegmentation <- function(report,rnb.set){
	logger.start("Adding region sub-segmentation section")
	txt <- c("Region subsegmentation was performed in order to obtained a more fine-grained view of local methylation ",
			 "levels. Each region was subdivided into subregions containing ",rnb.getOption("region.subsegments")," sites ",
			 "on average. These subregions were obtained by hierarchical clustering using ",
			 "single linkage agglomeration based on genomic coordinates. Each cluster then ",
			 "corresponds to a subsegment. This guarantees that neighboring ",
			 "sites end up in the same subregion. The following table provides some key statistics on the ",
			 "subsegmentation.")
	rnb.add.section(report, "Region sub-segmentation", txt)

	reg.types.subseg <- grep("\\.subsegments$",summarized.regions(rnb.set),value=TRUE)
	reg.types.base    <- sub("\\.subsegments$","",reg.types.subseg)
	reg.types.all <- c(reg.types.base,reg.types.subseg)

	reg.types.base.files <- paste0("reg",1:length(reg.types.base))
	names(reg.types.base) <- reg.types.base.files

	get.summary.df.from.list <- function(ll,value.name="value"){
		res <- reshape2::melt(ll)
		colnames(res) <- c(value.name,"region")
		res$is.subsegmentation <- grepl("\\.subsegments$",res$region)
		res$region <- sub("\\.subsegments$","",res$region)
		res$region <- factor(res$region,levels=reg.types.base)
		return(res)
	}

	list.allreg.size <- lapply(reg.types.all,FUN=function(rn){
		aa <- annotation(rnb.set,type=rn)
		return(aa[,"End"]-aa[,"Start"])
	})
	names(list.allreg.size) <- reg.types.all
	df.allreg.size <- get.summary.df.from.list(list.allreg.size,"region.size")

	list.allreg.num.sites <- lapply(reg.types.all,FUN=function(rn){
		reg.map <- regionMapping(rnb.set,rn)
		return(vapply(reg.map,length,integer(1)))
	})
	names(list.allreg.num.sites) <- reg.types.all
	df.allreg.num.sites <- get.summary.df.from.list(list.allreg.num.sites,"num.sites")
	#add a table containing key statistics on subsegmentation
	num.regs  <- vapply(list.allreg.size,length,integer(1))
	mean.size <- vapply(list.allreg.size,mean,numeric(1),na.rm=TRUE)
	median.size <- vapply(list.allreg.size,median,numeric(1),na.rm=TRUE)
	mean.num.sites <- vapply(list.allreg.num.sites,mean,numeric(1),na.rm=TRUE)
	median.num.sites <- vapply(list.allreg.num.sites,median,numeric(1),na.rm=TRUE)

	tt <- data.frame(num.regs[reg.types.base],
					 num.regs[reg.types.subseg],
					 mean.num.sites[reg.types.base],
					 mean.num.sites[reg.types.subseg],
					 median.num.sites[reg.types.base],
					 median.num.sites[reg.types.subseg],
					 mean.size[reg.types.base],
					 mean.size[reg.types.subseg],
					 median.size[reg.types.base],
					 median.size[reg.types.subseg])
	colnames(tt) <-c("Number of Regions (base)",
					 "Number of Regions (subseg)",
					 "Mean number of sites (base)",
					 "Mean number of sites (subseg)",
					 "Median number of sites (base)",
					 "Median number of sites (subseg)",
					 "Mean region size (base)",
					 "Mean region size (subseg)",
					 "Median region size (base)",
					 "Median region size (subseg)")
	tt <- round(tt,4)
	
	rnb.add.table(report,tt)

	#create some nice plots
	cvalues <- rep(rnb.getOption("colors.category"), length.out = 2)
	figPlots.size <- list()
	figPlots.num.sites <- list()
	for (i in 1:length(reg.types.base)){
		rn  <- reg.types.base[i]
		rnf <- reg.types.base.files[i]
		#plot distribution of region sizes
		figName <- paste("subseg_distribution","size",rnf,sep="_")
		df2p <- df.allreg.size[df.allreg.size$region==rn,]
		pp <- ggplot(df2p, aes(x = log2(region.size), color = is.subsegmentation)) +
			labs(y = "Density", color = "Region subsegmentation") +
			geom_density() + scale_color_manual(values = cvalues) +
			theme(legend.position="bottom")
		report.plot <- createReportGgPlot(pp,figName, report,create.pdf=TRUE,high.png=200)
		report.plot <- off(report.plot,handle.errors=TRUE)
		figPlots.size <- c(figPlots.size,list(report.plot))

		#plot distribution of number of sites per region
		figName <- paste("subseg_distribution","numSites",rnf,sep="_")
		df2p <- df.allreg.num.sites[df.allreg.num.sites$region==rn,]
		pp <- ggplot(df2p, aes(x = log2(num.sites), color = is.subsegmentation)) +
			labs(y = "Density", color = "Region subsegmentation") +
			geom_density() + scale_color_manual(values = cvalues) +
			theme(legend.position="bottom")
		report.plot <- createReportGgPlot(pp,figName, report,create.pdf=TRUE,high.png=200)
		report.plot <- off(report.plot,handle.errors=TRUE)
		figPlots.num.sites <- c(figPlots.num.sites,list(report.plot))
	}
	setting.names <- list(
			'region type' = reg.types.base)
	description <- c('Distribution of the log2 of the region sizes of the base regions ',
					 'and of the regions resulting from sub-segmentation.')
	report <- rnb.add.figure(report, description, figPlots.size, setting.names)
	
	description <- c('Distribution of the log2 of the number of sites in the base regions ',
					 'and in the regions resulting from sub-segmentation.')
	report <- rnb.add.figure(report, description, figPlots.num.sites, setting.names)

	logger.completed()
	invisible(report)
}

########################################################################################################################

#' rnb.step.region.subsegmentation
#'
#' Perform region subsegmentation (see \code{\link{addRegionSubsegments}}) for details and add a corresponding
#' section to the provided report
#' @param rnb.set an \code{RnBSet object}
#' @param report Report to summarize the outcome of this procedure. This must be an object of type
#'               \code{\linkS4class{Report}}. The subsegments will be stored in the 'data/subsegmentation' directory
#' 				 of the report
#' @param region.types the region types to which subsegmentation should be applied. Must be a non-empty
#' 					   subset of \code{summarized.regions(rnb.set)}. Defaults (\code{NULL}) to all region types in
#' 					   \code{rnb.set}
#' @param add.region.types.to.options,ns arguments passed on to \code{\link{addRegionSubsegments}}
#' @return a list with two elements: \code{rnb.set} the modified \code{RnBSet} object with subregions included
#'         and \code{report} the modified report with a section concerning region subsegmentation added
#' @author Fabian Mueller
#' @noRd
rnb.step.region.subsegmentation <- function(rnb.set,report,region.types=NULL,
		add.region.types.to.options=TRUE,ns=rnb.getOption("region.subsegments")){
	logger.start("Region sub-segmentation")
	annot.dir <- file.path(rnb.get.directory(report, dir="data",absolute=TRUE),"subsegmentation")
	create.path(annot.dir, accept.existing=FALSE)
	rnb.set <- addRegionSubsegments(rnb.set,annot.dir,region.types=region.types,add.region.types.to.options=add.region.types.to.options,ns=ns)
	report <- rnb.section.region.subsegmentation(report,rnb.set)
	res <- list(rnb.set=rnb.set,report=report)
	logger.completed()
	invisible(res)
}

########################################################################################################################
## regionProfiles.R
## created: 2012-12-18
## creator: Fabian Mueller
## ---------------------------------------------------------------------------------------------------------------------
## Methods for computing region methylation profiles
########################################################################################################################

#' rnb.find.relative.site.coord
#'
#' given a region types, assigns sites to regions and determines relative positions
#' of sites in the assigned region
#'
#' @param rnb.set RnBSet object
#' @param region.type Region type for which the coordinates are computed
#' @param extend.by A number between 0 and 1 specifying the percentage by which a region is
#'        extended in order to capture methylation information before region start and after region end
#' @return a data frame containing the site index, the assigned region index and the relative coordinate
#'         The relative coordinate is 0 if the site's coordinate is identical to the region start coordinate
#'         and 1 if identical to the regions end coordinate and scaled inbetween. Coordinates can be
#'         less than 0 or larger than 1 if a site is in the upstream or downstream flanking region respectively
#'
#' @author Fabian Mueller
#' @export
rnb.find.relative.site.coord <- function(rnb.set,region.type,extend.by=0.33) {
	annot.s <- annotation(rnb.set,"sites")
	gr.s <- data.frame2GRanges(annot.s, chrom.column="Chromosome",start.column="Start",end.column="End",
							   strand.column="Strand",assembly=assembly(rnb.set),sort.result=FALSE)
	annot.r <- annotation(rnb.set,region.type)
	gr.r <- data.frame2GRanges(annot.r, chrom.column="Chromosome",start.column="Start",end.column="End",
							   strand.column="Strand",assembly=assembly(rnb.set),sort.result=FALSE)
	r.lengths <- end(gr.r)-start(gr.r)+1
	r.strands <- as.character(strand(gr.r))
	#extend the regions to contain the flanking regions: first extend at the end by twice extend.by and then shift to
	# also have the flanking region at the start
	gr.re <- shift(resize(gr.r,width=ceiling(r.lengths*(1+2*extend.by))),ifelse(r.strands=="-",1,-1) * ceiling(r.lengths*extend.by))

	strand.spec <- rnb.getOption("strand.specific")
	over.re <- findOverlaps(gr.s,gr.re,select="all",ignore.strand=!strand.spec)
	
	#get the relative coordinates
	si <- queryHits(over.re)
	ri <- subjectHits(over.re)
	rel.coords <- ifelse(
		r.strands[ri]=="-",
		end(gr.r)[ri]   - end(gr.s)[si],
		start(gr.s)[si] - start(gr.r)[ri]
	)/r.lengths[ri]
	res <- data.frame(site.idx=si,gene.idx=ri,relative.coord=rel.coords)
	return(res)
}

#' rnb.plot.region.site.density
#'
#' Plots the density of sites accross the specified region type
#'
#' @param rnb.set RnBSet object
#' @param region.type Region type for which the plot should be generated
#' @param extend.by A number between 0 and 1 specifying the percentage by which a region is
#'        extended in order to capture methylation information before region start and after region end
#' @return a ggplot2 object for plotting
#'         the plot shows the density of sites accross the specified region type for all regions of that
#'         type from 0 (region start) to 1 (region end). Sites in the flanking areas are
#'         also shown (coordinates <0 and >1).
#'
#' @author Fabian Mueller
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' rnb.plot.region.site.density(rnb.set.example,"genes")
#' }
rnb.plot.region.site.density <- function(rnb.set,region.type,extend.by=0.33){
	dframe <- rnb.find.relative.site.coord(rnb.set,region.type,extend.by=extend.by)
	pp <- ggplot(dframe) + aes(x=relative.coord) + geom_density(color=rnb.getOption('colors.category')[1],
		  fill=rnb.getOption('colors.category')[1], alpha=0.5) +
		  xlab("relative position") + ylab("site density") + geom_vline(xintercept=c(0,1))
	return(pp)
}

#' rnb.plot.region.profiles
#'
#' Creates a composite plot showing the sample and groupwise smoothed estimates
#' of methylation values accross all regions of the specified type
#'
#' @param rnb.set RnBSet object
#' @param group.index.list a list (preferably named) containing sample indices for each group
#'		a list of such lists is for instance generated by the \code{rnb.sample.groups} function.
#' @param region.type Region type for which the plot should be generated
#' @param region.profile Alternative to specifying \code{region.type}, the function can accept a region profile
#'                       generated by the \code{rnb.find.relative.site.coord} function
#' @param extend.by A number between 0 and 1 specifying the percentage by which a region is
#'        extended in order to capture methylation information before region start and after region end
#' @param cvalues Color values that will be assigned to sample groups
#' @return a ggplot2 object for plotting
#'         the plot shows the smoothed methylation levels of sites accross the specified region type for 
#'         all regions of that type from 0 (region start) to 1 (region end). Sites in the flanking areas are
#'         also shown (coordinates <0 and >1).
#'         Smoothing is stratified by sample (dashed lines) and sample group (thick solid lines).
#'         Cubic splines are used for smoothing
#'
#' @author Fabian Mueller
#' @export
#' @examples
#' \donttest{
#' #Careful: this might take a while
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' rnb.plot.region.profiles(rnb.set.example,rnb.sample.groups(rnb.set.example)[[1]],"genes")
#' }
rnb.plot.region.profiles <- function(rnb.set, group.index.list, region.type="", region.profile=NULL, extend.by=0.33,
		cvalues=rnb.getOption("colors.category")){
	if (is.null(region.profile)){
		if (!is.element(region.type,summarized.regions(rnb.set))){
			stop("Invalid region type. You must specify either region.type or region.profile")
		}
		region.profile <- rnb.find.relative.site.coord(rnb.set,region.type,extend.by=extend.by)
	}
	grp.names <- names(group.index.list)
	if (is.null(grp.names)) grp.names <- 1:length(group.index.list)

	df2p <- do.call("rbind",lapply(1:length(group.index.list),FUN=function(gi){
		grp.name <- grp.names[gi]
		sample.idx <- group.index.list[[gi]]
		sample.names <- samples(rnb.set)[sample.idx]
		n.samples <- length(sample.idx)
		res <- data.frame()
		if (n.samples > 0){
			mm <- meth(rnb.set)[region.profile$site.idx, sample.idx, drop=FALSE]
			res <- data.frame(
				site.idx=rep(region.profile$site.idx,n.samples),
				relative.coord=rep(region.profile$relative.coord,n.samples),
				meth=as.vector(mm),
				sample=rep(sample.names,rep(nrow(mm),n.samples)),
				group=grp.name
			)
		}
		return(res)
	}))
	
	col.vals <- rep(cvalues,length.out=length(unique(df2p$group)))
	pp <- ggplot(df2p,aes(x=relative.coord,y=meth,group=sample,color=group,fill=group)) + 
		  ylim(0, 1) + ylab("methylation") + xlab("relative position") +
		  scale_color_manual(values = col.vals) + scale_fill_manual(values = col.vals) +
		  geom_smooth(alpha=0.15,linetype="dotted",method="gam",formula=y ~ s(x, bs = "cs")) +
		  geom_smooth(aes(group=group),size=2,alpha=0.15,method="gam",formula=y ~ s(x, bs = "cs")) +
		  # geom_smooth(alpha=0.15,linetype="dotted") +
		  # geom_smooth(aes(group=group),size=2,alpha=0.15) +
		  geom_vline(xintercept = c(0,1), linetype = "solid",size=1.5)
	return(pp)
}

#' rnb.plot.region.profiles
#'
#' Plots the density of methylation levels accross all regions of the specified type
#'
#' @param rnb.set RnBSet object
#' @param sample Index or name of the sample for which the plot should be generated
#' @param region.type Region type for which the plot should be generated
#' @param region.profile Alternative to specifying \code{region.type}, the function can accept a region profile
#'                       generated by the \code{rnb.find.relative.site.coord} function
#' @param extend.by A number between 0 and 1 specifying the percentage by which a region is
#'        extended in order to capture methylation information before region start and after region end
#' @return a ggplot2 object for plotting
#'         the plot shows the density of methylation levels of sites accross the specified region type for 
#'         all regions of that type from 0 (region start) to 1 (region end). Sites in the flanking areas are
#'         also shown (coordinates <0 and >1).
#'
#' @author Fabian Mueller
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' logger.start(fname=NA)
#' rnb.plot.region.profile.density(rnb.set.example,1,"genes")
#' }
rnb.plot.region.profile.density <- function(rnb.set, sample, region.type="", region.profile=NULL, extend.by=0.33){
	if (is.null(region.profile)){
		if (!is.element(region.type,summarized.regions(rnb.set))){
			stop("Invalid region type. You must specify either region.type or region.profile")
		}
		region.profile <- rnb.find.relative.site.coord(rnb.set,region.type,extend.by=extend.by)
	}

	df2p <- data.frame(
		site.idx=region.profile$site.idx,
		relative.coord=region.profile$relative.coord,
		meth=meth(rnb.set)[region.profile$site.idx,sample]
	)
	df2p <- na.omit(df2p)
	#the standard bandwith function of MASS::kde2d is unstable when looking at
	#distributions with very low variance. Here's a more stable version
	stable.bandwidth.fun <- function(x,eps=1e-4){
	    r <- quantile(x, c(0.25, 0.75))
	    h <- (r[2] - r[1])/1.34
	    if (h==0) h <- eps
	    4 * 1.06 * min(sqrt(var(x)), h) * length(x)^(-1/5)
	}
	stable.h <- c(stable.bandwidth.fun(df2p$relative.coord),stable.bandwidth.fun(df2p$meth))

	pp <- ggplot(df2p) + aes(x=relative.coord,y=meth) + 
		  stat_density2d(geom="tile",fill="#1F78B4", aes(alpha=after_stat(density)^0.25), contour = FALSE, h=stable.h) +
		  ylim(0, 1) + ylab("methylation") + xlab("relative position") + theme(legend.position="none") +
		  geom_vline(xintercept = c(0,1), linetype = "solid",size=1.5)
	return(pp)
}


## rnb.section.region.profiles
##
## Computes methylation distributions for various region types and sample groups
##
## @param report  Report to contain the methylation deviation section. This must be an object of type
##                \code{\linkS4class{Report}}.
## @param rnb.set RnBSet
## @param reg.types region types that should be reported on
## @param columns Optional; predefined column names (in the form of a \code{character} vector) or indices (an
##                \code{integer} vector) in the sample annotation table. Only these columns are considered for grouping
##                samples and defining profiles. All other columns in the phenotype table are ignored.
## @param subsample restrict the analysis to this many randomly chosen site occurrences per region type.
##				  This can help reduce the runtime and memory requirements
## @return The modified report.
## @author Fabian Mueller
rnb.section.region.profiles <- function(report, rnb.set, reg.types, columns=rnb.getOption("exploratory.columns"),
		subsample=rnb.getOption("distribution.subsample")) {
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	regTypes_notInSet <- setdiff(reg.types, summarized.regions(rnb.set))
	if (length(regTypes_notInSet)>0){
		reg.types <- intersect(reg.types, summarized.regions(rnb.set))
		logger.warning(c("The following regions are not annotated in the RnBSet object:", paste(regTypes_notInSet,collapse=","), "--> just using",paste(reg.types,collapse=",")))
	}

	txt <- c("Methylation profiles were computed for the specified region types. ",
			 "Composite plots are shown")
	report <- rnb.add.section(report, "Regional Methylation Profiles", txt)
	
	sample.inds <- rnb.sample.groups(rnb.set, columns = columns)
	sample.inds <- c(list("all samples"=list(all=1:length(samples(rnb.set)))),sample.inds)
	rplots <- list()
	for (i in  1:length(reg.types)){
		rt <- reg.types[i]
		reg.rel.coords <- rnb.find.relative.site.coord(rnb.set,rt)
		n.sites <- nrow(reg.rel.coords)
		if (subsample > 0 && subsample < n.sites){
			logger.info(c("Subsampling",subsample,"of",n.sites,"site occurences","(region type:",rt,")"))
			reg.rel.coords <- reg.rel.coords[sort(sample.int(n.sites,subsample)),]
		}
		for (j in 1:length(sample.inds)){
			cc <- names(sample.inds)[j]
			fname <- paste("regionProfile_","reg",i,"_col",j,sep="")
			logger.status(c("Plotting region profiles for region", rt, "and column",cc,"..."))
			pp <- rnb.plot.region.profiles(rnb.set,sample.inds[[j]],region.profile=reg.rel.coords)
			report.plot <- createReportGgPlot(pp,fname, report, width = 10, height = 5, create.pdf = TRUE)
			report.plot <- off(report.plot,handle.errors=TRUE)
			rplots <- c(rplots,report.plot)
		}
	}
	
	traits <- names(sample.inds)
	names(traits) <- paste("col",1:length(sample.inds),sep="")
	region.types <- reg.types
	names(region.types) <- paste("reg",1:length(region.types),sep="")
	setting.names <- list(
			'Region type'  = region.types,
			'Sample trait' = traits)
	description <- 'Regional methylation profiles (composite plots) according to sample groups. 
					For each region in the corresponding region type, relative coordinates of 0 and 1 corresponds to the start
					and end coordinates of that region respectively. Coordinates smaller than 0 and greater than 1 denote flanking regions
					normalized by region length. Scatterplot smoothers for each sample and sample group were fit. Horizontal lines indicate
					region boundaries. For smoothing, generalized additive models with cubic spine smoothing were used.
					Deviation bands indicate 95% confidence intervals'
	report <- rnb.add.figure(report, description, rplots, setting.names)
	
	return(report)
}

#' rnb.step.region.profiles
#'
#' Computes methylation distributions for various region types and sample groups
#'
#' @param rnb.set RnBSet
#' @param report  Report to contain the methylation deviation section. This must be an object of type
#'                \code{\linkS4class{Report}}.
#' @param region.types for which region types should distributions be computed
#' @param columns Optional; predefined column names (in the form of a \code{character} vector) or indices (an
#'                \code{integer} vector) in the sample annotation table. Only these columns are considered for grouping
#'                samples and defining profiles. All other columns in the phenotype table are ignored.
#' @param subsample restrict the analysis to this many randomly chosen site occurrences per region type.
#'				  This can help reduce the runtime and memory requirements
#' @return The modified report.
#'
#' @author Fabian Mueller
#' @noRd
rnb.step.region.profiles <- function(rnb.set, report, region.types, columns=rnb.getOption("exploratory.columns"),
		subsample=rnb.getOption("distribution.subsample")) {
	if (!inherits(rnb.set, "RnBSet")) {
		stop("invalid value for rnb.set")
	}
	if (!inherits(report, "Report")) {
		stop("invalid value for report")
	}
	if (!(is.character(region.types) && length(region.types) != 0 && all(!is.na(region.types)))) {
		stop("invalid value for region.types")
	}
	logger.start("Drawing regional methylation profiles")
	logger.start("Creating a Corresponding Section in the Report")
	report <- rnb.section.region.profiles(report,rnb.set,region.types,columns=columns,subsample=subsample)
	logger.completed()
	logger.completed()
	return(report)
}


## locus.profile.get.base.tracks
##
## For a genomic location, this function retrieves a list of basic tracks for later plotting
## They include a chromosome ideogram, genome coordinates, ENSEMBL gene annotationm, 
## CpG Island annotation.
## 
## @param chrom chromosome of window to plot
## @param start start coordinate of window to plot
## @param end end coordinate of window to plot
## @param assembly genome assembly. currently only \code{'hg19'}, \code{'mm10'} and \code{'mm9'} are supported.
## @return a list of \code{Gviz} tracks to be plotted
##
locus.profile.get.base.tracks <- function(chrom,start,end,assembly="hg19"){
	if (start < 0 || end < 0 || start >= end){
		stop("invalid values for start and end coordinates")
	}
	if (!chrom %in% names(rnb.get.chromosomes(assembly=assembly))) {
		stop("invalid value for chrom")
	}
	
	rnb.require("biomaRt")
	rnb.require("Gviz")	
	mart <- NULL
	featMap <- Gviz:::.getBMFeatureMap()
	if (assembly == "hg19"){
		# mart <- tryCatch(
		# 	useMart("ensembl", dataset = "hsapiens_gene_ensembl"),
		# 	error = function(ee) {
		# 		logger.warning(c("could not retrieve Ensembl genes from biomart: ",ee$message))
		# 		NULL
		# 	}
		# )
		mart <- tryCatch(
			useMart("ENSEMBL_MART_ENSEMBL", dataset = "hsapiens_gene_ensembl",host="https://feb2014.archive.ensembl.org"), ## Ensembl 75
			error = function(ee) {
				logger.warning(c("could not retrieve Ensembl genes from biomart: ",ee$message))
				NULL
			}
		)
		featMap["symbol"] <- "external_gene_id"
	} else if (assembly == "hg38"){
		mart <- tryCatch(
			useMart("ENSEMBL_MART_ENSEMBL", dataset = "hsapiens_gene_ensembl",host="https://sep2025.archive.ensembl.org"), ## Ensembl 115
			error = function(ee) {
				logger.warning(c("could not retrieve Ensembl genes from biomart: ",ee$message))
				NULL
			}
		)
		featMap["symbol"] <- "external_gene_name"
	} else if (assembly == "mm9"){
		mart <- tryCatch(useMart("ENSEMBL_MART_ENSEMBL", dataset = "mmusculus_gene_ensembl",host="may2012.archive.ensembl.org"),
			error = function(ee) {
				logger.warning(c("could not retrieve Ensembl genes from biomart: ",ee$message))
				NULL
			}
		)
		featMap["symbol"] <- "external_gene_id"
	} 
	else if (assembly == "mm10"){
		mart <- tryCatch(useMart("ENSEMBL_MART_ENSEMBL", dataset = "mmusculus_gene_ensembl",host="aug2014.archive.ensembl.org"),
			error = function(ee) {
				logger.warning(c("could not retrieve Ensembl genes from biomart: ",ee$message))
				NULL
			}
		)
		featMap["symbol"] <- "external_gene_name"
	} else {
		stop("invalid assembly")
	}
	itrack <- IdeogramTrack(genome=assembly, chromosome=chrom)
	gtrack <- GenomeAxisTrack(labelPos="above",add53=TRUE,add35=TRUE)
	
	if (!is.null(mart)){
		grtrack.ensembl <- tryCatch(
				BiomartGeneRegionTrack(genome=assembly, biomart=mart, featureMap=featMap, chromosome=chrom,start=start,end=end,name="ENSEMBL genes",
									   showId=TRUE,size=0.5,
									   col.title="white",background.title="lightsteelblue4"),
				error = function(ee) {
					logger.warning(c("could not create ENSEMBL gene track: ",ee$message))
					NULL
				}
		)
	} else {
		grtrack.ensembl <- NULL
	}
		
	cgitrack <- tryCatch(
			UcscTrack(genome=assembly,chromosome=chrom,from=start,to=end,track="cpgIslandExt",trackType="AnnotationTrack",
					  start="chromStart",end="chromEnd",id="name",shape="box",fill="dark green",size=0.5,name="CpG Islands",
					  col.title="white",background.title="lightsteelblue4"),
			error = function(ee) {
				logger.warning(c("could not create CpG Island track: ",ee$message))
				NULL
			}
	)
	
	tracklist <- list(itrack,gtrack,grtrack.ensembl,cgitrack)
	is.error <- sapply(tracklist,is.null)
	return(tracklist[!is.error])
}

## locus.profile.get.methylation.track.param.processing
##
## Performs parameter processing for methylation track generation
## 
## @param chrom chromosome of window to plot
## @param start start coordinate of window to plot
## @param end end coordinate of window to plot
## @param assembly [required when \code{rnbSet} is \code{NULL}]. Genome assembly. currently only \code{'hg19'} and \code{'mm9'} are supported.
## @param rnbSet [required when any of \code{assembly,annot,meth} is \code{NULL}] RnBSet object
## @param annot [required when \code{rnbSet} is \code{NULL}]. Annotation table for methylation values. As obtained by \code{annotation(rnbSet)}
## @param meth [required when \code{rnbSet} is \code{NULL}]. Methylation value table. As obtained by \code{meth(rnbSet)}
## @param region.type On what level should methylation values be processed. Must be one of \code{"sites"} or region types specified in \code{rnb.region.types()}
##        when \code{rnbSet} is not \code{NULL}, it must be compatible with it
## @return a \code{GRanges} object for the specified genomic interval with the methylation values stored in \code{elementMetadata}
##
locus.profile.get.methylation.track.param.processing <- function(chrom,start,end,assembly=NULL,rnbSet=NULL,annot=NULL,meth=NULL,region.type="sites"){
	if (!inherits(rnbSet, "RnBSet") && !is.null(rnbSet)) {
		stop("invalid value for rnbSet")
	}
	if (start < 0 || end < 0 || start >= end){
		stop("invalid values for start and end coordinates")
	}
	if (!chrom %in% names(rnb.get.chromosomes(assembly=assembly))) {
		stop("invalid value for chrom")
	}
	if (is.null(rnbSet)){
		if (is.null(assembly) || is.null(annot) || is.null(meth) || (nrow(meth)!=nrow(annot))){
			stop("Missing or invalid annot, meth or assembly")
		}
	} else {
		meth <- meth(rnbSet,type=region.type)
		annot <- annotation(rnbSet,type=region.type)
		assembly <- assembly(rnbSet)
	}
	
	if (region.type == "sites"){
		is.in.locus <- annot$Chromosome == chrom & annot$Start >= start & annot$End <= end
	} else {
		is.in.locus <- annot$Chromosome == chrom & annot$End >= start & annot$Start <= end
	}
	if (sum(is.in.locus) < 1){
		warning("No matching sites found. returning NULL")
		return(NULL)
	}
	aa <- annot[is.in.locus,]
	mm <- meth[is.in.locus,]
	if (sum(is.in.locus) == 1){
		mm <- matrix(mm,nrow=1)
		colnames(mm) <- colnames(meth)
	}
	mm <- data.frame(mm,Chromosome=aa$Chromosome,Start=aa$Start,End=aa$End,Strand=aa$Strand,stringsAsFactors=FALSE,check.names=FALSE)
	#make strand unique for plotting
	mm$Strand <- rep("*",nrow(mm))
	mm.gr <- data.frame2GRanges(mm, ids = NULL, chrom.column = "Chromosome", start.column = "Start",
			end.column = "End", strand.column = "Strand", assembly = assembly)
	
	return(mm.gr)
}

## locus.profile.get.methylation.track.heatmap
##
## For a genomic location, this function retrieves a list of methylation heatmap tracks based on the provided
## methylation information
## 
## @param chrom chromosome of window to plot
## @param start start coordinate of window to plot
## @param end end coordinate of window to plot
## @param assembly [required when \code{rnbSet} is \code{NULL}]. Genome assembly. currently only \code{'hg19'} and \code{'mm9'} are supported.
## @param rnbSet [required when any of \code{assembly,annot,meth} is \code{NULL}] RnBSet object
## @param annot [required when \code{rnbSet} is \code{NULL}]. Annotation table for methylation values. As obtained by \code{annotation(rnbSet)}
## @param meth [required when \code{rnbSet} is \code{NULL}]. Methylation value table. As obtained by \code{meth(rnbSet)}
## @param grps a list of indices for each group to be compared or NULL if no sample grouping information should be displayed
## @param region.type On what level should methylation values be processed. Must be one of \code{"sites"} or region types specified in \code{rnb.region.types()}
##        when \code{rnbSet} is not \code{NULL}, it must be compatible with it
## @param cvals.grps colors to be used for the different groups
## @param cvals.meth colors to be used for methylation values and heatmaps
## @return a list of \code{Gviz} tracks to be plotted
##
locus.profile.get.methylation.track.heatmap <- function(chrom,start,end,
		assembly=NULL,rnbSet=NULL,annot=NULL,meth=NULL,grps=NULL,region.type="sites",
		cvals.grps=rnb.getOption("colors.category"), cvals.meth=rnb.getOption("colors.meth")){
	
	if (!(class(grps) %in% c("list","array","NULL"))) {
		stop("invalid value for grps")
	}
	mm.gr <- locus.profile.get.methylation.track.param.processing(chrom,start,end,assembly,rnbSet,annot,meth,region.type)
	if (is.null(mm.gr)){
		return(list())
	}
	mtracks <- list()
	rt.str4title <- region.type
	if (region.type=="sites"){
		rt.str4title <- ""
	}
	cvals.grps <- rep(cvals.grps, length.out = length(grps)) #adjust the length of the color vector. Recycle colors if necassary
	if (is.null(grps)){
		mtracks <- list(DataTrack(range=mm.gr,name=paste("meth:",rt.str4title),type=c("heatmap"),ncolor=100,gradient=cvals.meth,
				col.title="white",background.title=cvals.grps[1]))
	} else {
		for (i in 1:length(grps)){
			gg <- names(grps)[i]
			cols.grp <- colnames(elementMetadata(mm.gr))[grps[[gg]]]
			dt <- DataTrack(range=mm.gr,data=cols.grp,name=paste("meth:",rt.str4title," ",gg),type=c("heatmap"),ncolor=100,gradient=cvals.meth,
					col.title="white",background.title=cvals.grps[i],showSampleNames=TRUE)
			mtracks <- c(mtracks,list(dt))
		}
	}
	
	return(mtracks)
}

## locus.profile.get.methylation.track.smooth
##
## For a genomic location, this function retrieves a list of methylation scatterplot and smoothing tracks based on the provided
## methylation information
## 
## @param chrom chromosome of window to plot
## @param start start coordinate of window to plot
## @param end end coordinate of window to plot
## @param assembly [required when \code{rnbSet} is \code{NULL}]. Genome assembly. currently only \code{'hg19'} and \code{'mm9'} are supported.
## @param rnbSet [required when any of \code{assembly,annot,meth} is \code{NULL}] RnBSet object
## @param annot [required when \code{rnbSet} is \code{NULL}]. Annotation table for methylation values. As obtained by \code{annotation(rnbSet)}
## @param meth [required when \code{rnbSet} is \code{NULL}]. Methylation value table. As obtained by \code{meth(rnbSet)}
## @param grps a list of indices for each group to be compared or NULL if no sample grouping information should be displayed
## @param region.type On what level should methylation values be processed. Must be one of \code{"sites"} or region types specified in \code{rnb.region.types()}
##        when \code{rnbSet} is not \code{NULL}, it must be compatible with it
## @param cvals.grps colors to be used for the different groups
## @return a list of \code{Gviz} tracks to be plotted
##
locus.profile.get.methylation.track.smooth <- function(chrom,start,end,
		assembly=NULL,rnbSet=NULL,annot=NULL,meth=NULL,grps=NULL,region.type="sites",cvals.grps=rnb.getOption("colors.category"),smooth.profile="wide"){
	
	if (!(class(grps) %in% c("list","array","NULL"))) {
		stop("invalid value for grps")
	}
	mm.gr <- locus.profile.get.methylation.track.param.processing(chrom,start,end,assembly,rnbSet,annot,meth,region.type)
	
	if (is.null(mm.gr)){
		return(list())
	}
	
	grouping <- NULL
	if (!is.null(grps)){
		grouping <- rep(NA,ncol(elementMetadata(mm.gr)))
		for (gg in names(grps)){
			grouping[grps[[gg]]] <- gg
		}
		grouping <- factor(grouping, levels=names(grps))
	}
	smooth.span <- 0.9
	smooth.degree <- 1
	smooth.family <- "gaussian"
	if (smooth.profile=="wide"){
		smooth.span <- 0.9
	} else if (smooth.profile=="narrow"){
		smooth.span <- 0.2
	} else {
		stop("invalid value for smooth.profile")
	}
	cvals.grps <- rep(cvals.grps, length.out = length(grps)) #adjust the length of the color vector. Recycle colors if necassary
	mtrack.smooth <- DataTrack(range=mm.gr,groups=grouping,name=paste("methylation"),type=c("smooth","p"),span=smooth.span,degree=smooth.degree,family=smooth.family,jitter.x=FALSE,col=cvals.grps, legend=FALSE,
			col.title="white",col.axis="white",background.title="lightsteelblue4")
	
	return(list(mtrack.smooth))
}


#' rnb.plot.locus.profile 
#'
#' Computes methylation distributions for various region types and sample groups
#'
#' @param rnbSet RnBSet object
#' @param chrom chromosome of window to plot
#' @param start start coordinate of window to plot
#' @param end end coordinate of window to plot
#' @param grps a list of indices for each group to be compared or NULL if no sample grouping information should be displayed
#' @param plot.m.regions character vector of region types whose methylation values should be displayed If \code{grps} is not \code{NULL}
#' 		  the methylation values will be separated by sample groups.
#' @param plot.m.heatmap flag indicating whether sites methylation values should be displayed in a heatmap. If \code{grps} is not \code{NULL}
#' 		  the heatmaps will be separated by sample groups.
#' @param plot.m.smooth flag indicating whether a scatterplot with smoothing curves should be displayed. If \code{grps} is not \code{NULL}
#' 		  the colors will be used to separate sample groups.
#' @param cvals.grps colors to be used for the different groups
#' @param cvals.meth colors to be used for methylation values and heatmaps
#' @param smooth.profile profile to be used for the smoothing curves. Allowed values include 
#'        \code{wide} (default) which yields smoother curvers and \code{narrow} which yields more "wiggly" curves
#' @return a \code{ggplot2} plot object containing the plot
#'
#' @author Fabian Mueller
#' @export
#' @examples
#' \donttest{
#' #see RnBeads vignette (section: 'Generating Locus Profile Plots') for examples
#' }
rnb.plot.locus.profile <- function(rnbSet,chrom,start,end,grps=NULL,
		plot.m.regions=NULL,plot.m.heatmap=TRUE,plot.m.smooth=TRUE,
		cvals.grps=rnb.getOption("colors.category"),cvals.meth=rnb.getOption("colors.meth"), smooth.profile="wide"){	
	if (!inherits(rnbSet, "RnBSet")) {
		stop("invalid value for rnbSet")
	}
	if (!(class(grps) %in% c("list","array","NULL"))) {
		stop("invalid value for grps")
	}
	if (length(grps)<1 && !is.null(grps)) {
		stop("invalid value for grps")
	}
	if (!chrom %in% names(rnb.get.chromosomes(assembly=assembly(rnbSet)))) {
		stop("invalid value for chrom")
	}
	if (start < 0 || end < 0 || start > end){
		stop("invalid values for start and end coordinates")
	}
	if (!is.element(smooth.profile, c("wide", "narrow"))){
		stop("invalid value for smooth.profile")
	}
	rnb.require("Gviz")	
	
	#get methylation info
	mm <- meth(rnbSet,type="sites")
	site.annot <- annotation(rnbSet,type="sites")
	
	tracklist <- locus.profile.get.base.tracks(chrom,start,end,assembly=assembly(rnbSet))

	#region information
	for (rr in plot.m.regions){
		mmr <- meth(rnbSet,type=rr)
		reg.annot <- annotation(rnbSet,type=rr)
		
		tts <- locus.profile.get.methylation.track.heatmap(chrom,start,end,
				assembly=assembly(rnbSet),annot=reg.annot,meth=mmr,grps=grps,region.type=rr,cvals.meth=cvals.meth)
		tracklist <- c(tracklist,tts)
	}
	
	if (plot.m.heatmap){
		tts <- locus.profile.get.methylation.track.heatmap(chrom,start,end,
				assembly=assembly(rnbSet),annot=site.annot,meth=mm,grps=grps,region.type="sites",cvals.meth=cvals.meth,cvals.grps=cvals.grps)
		tracklist <- c(tracklist,tts)
	}
	if (plot.m.smooth){
		tts <- locus.profile.get.methylation.track.smooth(chrom,start,end,
				assembly=assembly(rnbSet),annot=site.annot,meth=mm,grps=grps,region.type="sites",cvals.grps=cvals.grps, smooth.profile=smooth.profile)
		tracklist <- c(tracklist,tts)
	}
	
	plotTracks(tracklist,from=start,to=end)
	#plotTracks(c(tracklist,tts),from=start,to=end)
}

#' rnb.section.locus.profiles 
#'
#' given a list of interesting loci, generates a section in an RnBeads report
#' containing locus specific information and methylation profiles
#'
#' @param rnb.set Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.
#' @param locus.tab a table with columns named \code{chrom,start,end,id} specificying the genomic chromosome, start location end location and a name for
#' 	      the locus respectively
#' @param type you can specify a predefined section type like 'genes' to get custom section text. Otherwise, the generic 'custom regions' will be used
#' @param grp.cols a list of lists of indices for each group to be compared or NULL if no sample grouping information should be displayed
#' @param cvals.grps colors to be used for the different groups
#' @param cvals.meth colors to be used for methylation values and heatmaps
#' @param include.all.sample.group include a grouping  containing all samples
#' @return the modified report
#'
#' @author Fabian Mueller
#' @noRd
rnb.section.locus.profiles <- function(rnb.set, locus.tab, report, type="", grp.cols = rnb.getOption("exploratory.columns"),
		cvals.grps = rnb.getOption("colors.category"), cvals.meth = rnb.getOption("colors.meth"), include.all.sample.group=TRUE) {
	
	if (!all(c("chrom","start","end","id") %in% colnames(locus.tab))){
		stop("Invalid value for locus.tab: it must be a table containing the columns chrom,start,end,id")
	}
	if (any(duplicated(locus.tab[,"id"]))){
		stop("Invalid value for locus.tab: non-unique ids")
	}
	rnb.require("Gviz")
	rnb.require("biomaRt")
	if (rnb.getOption("logging") && logger.isinitialized() == FALSE) {
		logger.start(fname = NA) # initialize console logger
	}
	
	grps.all <- rnb.sample.groups(rnb.set, columns = grp.cols)
	if (include.all.sample.group){
		grps.all <- c(list("All Samples"=list("all"=1:length(samples(rnb.set)))),grps.all)
	}
	logger.info(c("Grouping by:",paste(names(grps.all),collapse=",")))

	logger.start("Retrieving Site Annotation and Methylation Values")
	mm <- meth(rnb.set,type="sites")
	site.annot <- annotation(rnb.set,type="sites")
	assembly <- assembly(rnb.set)
	logger.completed()
	
	if (type=="genes"){
		fname <- "gene_loci.csv"
		fname <- rnb.write.table(locus.tab,fname,fpath=rnb.get.directory(report, "data", absolute = TRUE),format="csv",gz=FALSE,row.names = FALSE,quote=FALSE)	
		txt <- c("Locus profiles were generated for selected genes of interest. Their genomic locations can be found in ",
				 "<a href=\"", rnb.get.directory(report, "data"), "/", fname, "\">this table</a> (coordinates are based on assembly ",assembly,").")
		rnb.add.section(report, "Locus Profiles (Genes)", txt)
	} else {
		fname <- "custom_loci.csv"
		fname <- rnb.write.table(locus.tab,fname,fpath=rnb.get.directory(report, "data", absolute = TRUE),format="csv",gz=FALSE,row.names = FALSE,quote=FALSE)	
		txt <- c("Locus profiles were generated for selected regions of interest. Their genomic locations can be found in ",
				 "<a href=\"", rnb.get.directory(report, "data"), "/", fname, "\">this table</a> (coordinates are based on assembly ",assembly,").")
		rnb.add.section(report, "Locus Profiles (Custom Regions)", txt)
	}
	
	
	logger.start("Generating Locus Views")
	do.plots <- function(i){
		plot.list <- list()
		logger.status(c("Processing Locus",locus.tab[i,"id"]))
		chrom <- locus.tab[i,"chrom"]
		start <- locus.tab[i,"start"]
		end   <- locus.tab[i,"end"]
		span <- end - start
		extend.by <- round(span*0.05)

		tl.base <- locus.profile.get.base.tracks(chrom,start,end,assembly=assembly)
		for (j in 1:length(grps.all)){
			# logger.status(c("Sample grouping by",names(grps.all)[j],"[",locus.tab[i,"id"],"]"))
			ggs <- grps.all[[j]]
			tts.heat <- locus.profile.get.methylation.track.heatmap(chrom,start,end,
					assembly=assembly,annot=site.annot,meth=mm,grps=ggs,region.type="sites",cvals.grps=cvals.grps,cvals.meth=cvals.meth)
			tts.smooth <- locus.profile.get.methylation.track.smooth(chrom,start,end,
					assembly=assembly,annot=site.annot,meth=mm,grps=ggs,region.type="sites",cvals.grps=cvals.grps)
			tl.all <- c(tl.base,tts.heat,tts.smooth)
			
			figName <- paste("locusMeth",paste("loc",i,sep=""),paste("grp",j,sep=""),sep="_")
			report.plot <- createReportPlot(figName, report, create.pdf=TRUE, high.png=300, width = 12, height = 7)
				tryCatch(
					plotTracks(tl.all,from=start,to=end,extend.right=extend.by,extend.left=extend.by),
					error = function(ee) {
						logger.warning(c("A plotting error occurred ","[",locus.tab[i,"id"],"]:",ee$message))
						print(rnb.message.plot("Plot could not be created"))
					}
				)
			off(report.plot)
			plot.list <- c(plot.list,list(report.plot))
		}
		return(plot.list)
	}
	if(parallel.isEnabled()){
		figPlots <- foreach(i=1:nrow(locus.tab),.combine="c") %dopar% {
			do.plots(i)
		}
	} else {
		figPlots <- list()
		for (i in 1:nrow(locus.tab)){
			figPlots <- c(figPlots,do.plots(i))
		}
	}
	groups <- names(grps.all)
	names(groups) <- paste("grp",1:length(grps.all),sep="")
	loci <- locus.tab[,"id"]
	names(loci) <- paste("loc",1:nrow(locus.tab),sep="")
	setting.names <- list("Locus"=loci,"Sample group" = groups)
	description <- c('Locus profiles. Annotation elements include ENSEMBL gene annotations, CpG Islands according to UCSC. ',
					 '(Groupwise) methylation profiles are shown as heatmap and scatterplot smoothers (Loess Gaussian smoothing with degree 1).')
	report <- rnb.add.figure(report, description, figPlots, setting.names)
	logger.completed()
	
	return(report)
}

#' sortAndExtendRegions
#'
#' helper function to sort a data.frame of genomic regions according to ID and extend
#' the boundaries by a certain percentage.
#'
#' @param interesting.loci region table
#' @param extend.by percentage by which the regions are extended
#' @param do.sort sort according to region.id
#' @return The modified report.
#'
#' @author Fabian Mueller
#' @noRd
sortAndExtendRegions <- function(interesting.loci, extend.by=0.1, do.sort=TRUE){
	if (do.sort){
		#sort according to id
		interesting.loci <- interesting.loci[order(interesting.loci$id),]
	}
	#extend the window by extend.by*100% on each side
	spans <- interesting.loci$end - interesting.loci$start
	interesting.loci$start <- interesting.loci$start - round(spans*extend.by)
	interesting.loci$end <- interesting.loci$end + round(spans*extend.by)
	return(interesting.loci)
}

#' rnb.step.locus.profiles
#'
#' Computes methylation distributions for various region types and sample groups
#'
#' @param rnb.set RnBSet
#' @param report  Report to contain the methylation deviation section. This must be an object of type
#'                \code{\linkS4class{Report}}.
#' @param locus.bed a bed file containing regions of interest. If not supplied the corresponding section will be skipped
#' @param gene.list a list of gene symbols that locus profiles will be generated for. If not supplied the corresponding section will be skipped
#' @param gene.annotation.name Name of the gene annotation to be used. Must be one of the region types in \code{rnb.set} and must contain a column called \code{symbol}
#' @param ... arguments passed to \code{rnb.section.locus.profiles}
#' @return The (modified) report.
#'
#' @author Fabian Mueller
#' @noRd
rnb.step.locus.profiles <- function(rnb.set, report, locus.bed="", gene.list=c(), gene.annotation.name="genes", ...) {
	logger.start("Creating locus profiles")
	if (file.exists(locus.bed)){
		logger.start("Creating locus profiles for custom regions")
		logger.info(c("loading region data from file:",locus.bed))
		region.tab <- rnb.load.bed(locus.bed)
		colnames(region.tab)[1:3] <- c("chrom","start","end")
		if (ncol(region.tab)<4){
			if (!is.null(rownames(region.tab))){
				region.tab <- data.frame(region.tab,id=rownames(region.tab),stringsAsFactors=FALSE)
			} else {
				region.tab <- data.frame(region.tab,id=paste0("locus",1:nrow(region.tab)),stringsAsFactors=FALSE)
			}
		}
		colnames(region.tab)[4] <- "id"
		# region.tab <- sortAndExtendRegions(region.tab)
		report <- rnb.section.locus.profiles(rnb.set, region.tab, report, ...)
		logger.completed()
	}
	logger.completed()
	if (length(gene.list) > 0 && is.character(gene.list)){
		logger.start("Creating locus profiles for custom regions")
		if (!is.element("genes",summarized.regions(rnb.set))){
			logger.warning("RnBSet does not contain annotation for genes --> skipping gene locus profiling")
			return(report)
		}
		gene.list <- tolower(gene.list)
		annot.genes <- annotation(rnb.set,type=gene.annotation.name)
		annot.genes <- annot.genes[,c("Chromosome","Start","End","symbol")]
		colnames(annot.genes) <- c("chrom","start","end","id")
		annot.genes$chrom <- as.character(annot.genes$chrom)
		annot.genes$id <- as.character(annot.genes$id)
		annot.gene.symbols.l <- tolower(annot.genes[,"id"])
		annot.inds <- na.omit(match(gene.list,annot.gene.symbols.l))
		genes.not.covered <- setdiff(gene.list,annot.gene.symbols.l)
		if (length(genes.not.covered)>0){
			logger.info(c("Genes not covered in annotation:",paste(genes.not.covered,collapse=",")))
		}
		if (length(annot.inds) < 1){
			logger.warning("No genes from the list are covered in the RnBSet --> skipping gene locus profiling")
			return(report)
		}
		interesting.loci <- sortAndExtendRegions(annot.genes[annot.inds,])
		report <- rnb.section.locus.profiles(rnb.set, interesting.loci, report, type="genes", ...)
		logger.completed()
	}
	return(report)
}

############################################
# WBC INFERENCE
############################################

# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose other than its incorporation into a
# commercial product is hereby granted without fee, provided that the
# above copyright notice appear in all copies and that both that
# copyright notice and this permission notice appear in supporting
# documentation, and that the name of Brown University not be used in
# advertising or publicity pertaining to distribution of the software
# without specific, written prior permission.

# BROWN UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY
# PARTICULAR PURPOSE.  IN NO EVENT SHALL BROWN UNIVERSITY BE LIABLE FOR
# ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

projectWBC = function(Y, coefWBC, contrastWBC=NULL, nonnegative=TRUE){ 

  if(is.null(contrastWBC)) Xmat = coefWBC
  else Xmat = coefWBC %*% t(contrastWBC) 

  nCol = dim(Xmat)[2]
  nSubj = dim(Y)[2]

  mixCoef = matrix(0, nSubj, nCol)
  rownames(mixCoef) = colnames(Y)
  colnames(mixCoef) = colnames(Xmat)

  if(nonnegative){
    rnb.require('quadprog')

    Amat = diag(nCol)
    b0vec = rep(0,nCol)

    for(i in 1:nSubj){
      obs = which(!is.na(Y[,i])) 
      Dmat = t(Xmat[obs,])%*%Xmat[obs,]
      mixCoef[i,] = solve.QP(Dmat, t(Xmat[obs,])%*%Y[obs,i], Amat, b0vec)$sol
    }
  }
  else{
    for(i in 1:nSubj){
      obs = which(!is.na(Y[,i])) 
      Dmat = t(Xmat[obs,])%*%Xmat[obs,]
      mixCoef[i,] = solve(Dmat, t(Xmat[obs,]) %*% Y[obs,i])
    }
  }

  return(mixCoef)
}

############################################

inferWBCbyLme = function(Y, pheno, modelFix, modelRand, coefWBC, 
   contrastWBC=NULL, detailLevel=1, silentErrors=TRUE){

  M = dim(coefWBC)[1]
  n = dim(Y)[2]
  if(dim(Y)[1] != M) stop("Y does not match coefWBC in dimension\n")

  if(detailLevel == -1){
    lmeVcomp = list(
      mu = matrix(NA,M,n),
      e = matrix(NA,M,n),
      a = list()
     )
  }

  sigmaResid = sigmaIcept = nObserved = nClusters = rep(NA, M)
  coefEsts = list()
  for(j in 1:M){
    ii = !is.na(Y[j,])
    nObserved[j] = sum(ii)
    pheno$y = Y[j,]

    try({
      fit = try(lme(modelFix, random=modelRand, data=pheno[ii,]), 
        silent=silentErrors)

      if(inherits(fit,"try-error")){
        fit = lm(modelFix, data=pheno[ii,])
        fitCoef = fit$coef
        sigmaResid[j] = summary(fit)$sigma
        sigmaIcept[j] = 0
        nClusters[j] = 0
        if(detailLevel == -1){
           lmeVcomp$mu[j,ii] = predict(fit)
           lmeVcomp$e[j,ii] = residuals(fit)
           lmeVcomp$a[[j]] = list()
        }
      }
      else{
        fitCoef = fit$coef$fixed
        sigmaResid[j] = fit$sigma
        sigmaIcept[j] = sqrt(getVarCov(fit)[1])
        nClusters[j] = length(fit$coef$random[[1]])
        if(detailLevel == -1){
           lmeVcomp$mu[j,ii] = predict(fit,level=0)
           lmeVcomp$e[j,ii] = residuals(fit,level=1)
           lmeVcomp$a[[j]] = fit$coef$random
        }
      }
      coefEsts[[j]] = fitCoef
    })
  }
  coefEsts = t(matrix(unlist(coefEsts), length(fitCoef), M))
  rownames(coefEsts) = rownames(coefWBC)
  colnames(coefEsts) = names(fitCoef)

  if(is.null(contrastWBC)) Z = cbind(1,coefWBC)
  else Z = cbind(1, coefWBC %*% t(contrastWBC))
  colnames(Z)[1] = "<Intercept>"

  ZtZ = t(Z) %*% Z
  ZtZqr = qr(ZtZ)
  G = solve(ZtZqr, t(Z) %*% coefEsts)

  out = list(method="lme", GammaMatrix=G, Beta1Matrix=coefEsts, 
    sigma=cbind(resid=sigmaResid, intercept=sigmaIcept),
    N=cbind(obs=nObserved,clusters=nClusters))

  if(detailLevel>=1){
    out$var.unscaled=solve(ZtZqr)

    d = dim(Z)[2]
    out$predicted = Z %*% G
    residual2 = coefEsts - out$predicted
    out$Sigma = (1/(M-d)) * t(residual2)%*%residual2

    X = model.matrix(modelFix, pheno)
    Xbar = apply(X,2,mean)
    Xctr = (X - matrix(1,n,1) %*% Xbar)
    residual2z = Xctr %*% t(residual2)
    out$ssu = apply(residual2z*residual2z, 2, sum)

    residual1 = Y - coefEsts %*% t(X) 
    out$sse = apply(residual1*residual1, 1, sum, na.rm=TRUE)

    residual0 = Y - apply(Y,1,mean,na.rm=TRUE) %*% matrix(1,1,n)
    out$ss0 = apply(residual0*residual0, 1, sum, na.rm=TRUE)

    if(detailLevel>=1){
       out$residuals = list(stage1=residual1, stage2=residual2, inner=t(residual2z))
    }
  }
  if(detailLevel==-1){
    trueLme = which(sapply(lmeVcomp$a, length)>0)
    ch = sort(unique(unlist(lapply(lmeVcomp$a[trueLme], function(u)rownames(u[[1]])))))
    nch = length(ch)
    a = matrix(0, M, nch)
    colnames(a) = ch
    for(j in trueLme){
      aa = lmeVcomp$a[[j]][[1]]
      a[j,rownames(aa)] = aa
    }
    lmeVcomp$ch = names(lmeVcomp$a[[trueLme[1]]])
    lmeVcomp$a = a
    out$lmeVcomp = lmeVcomp
  }

  class(out) = "inferWBC"
  out
}

summary.inferWBC = function(x, xboot=NULL){

  out = list(Coef=x$Gamma)
  class(out) = "inferWBCsummary"

  if(is.null(x$var.unscaled)){
     return(out)
  }

  sUnscaled = sqrt(diag(x$var.unscaled))
  sigma = sqrt(diag(x$Sigma))
  se = outer(sUnscaled,sigma,"*")
  rownames(se) = rownames(x$Gamma)
  out$StdErrNaive=se

  class(out) = "inferWBCsummary"

  if(!is.null(x$ss0)){
    out$RsquareTotal = 1-sum(x$ssu+x$sse)/sum(x$ss0)
    out$RsquareStage1 = 1-sum(x$ssu)/sum(x$ss0-x$sse)
  }

  if(is.null(xboot)) return(out)

  sboot = summary(xboot)

  out$Bias.Single = x$Gamma - sboot$Mean.Single
  out$Bias.Double = x$Gamma - sboot$Mean.Double 
  out$SD.Single = sboot$SD.Single
  out$SD.Double = sboot$SD.Double
  out$numBoots = sboot$numBoots

  out
}

print.inferWBC = function(x, digits=4){
  cat("WBC inference object (method = ",x$method,")\n\n",sep="")
  print(summary(x), digits=digits)
}

print.inferWBCsummary = function(x, digits=4, displayFactor=100){
  vars = setdiff(colnames(x$Coef),"(Intercept)")
  if(length(x)==1) {
    cat("(no inference data)\n")
    print(x$Coef[,vars])
    return()
  }

  tag = flag = "?"
  tab = NULL
  for(v in vars){
    cat(v,"\n")
    tab = cbind(Est=x$Coef[,v], StdErr0=x$StdErrNaive[,v])
    if(is.null(x$SD.Single)){
       flag = "StdErr0"
       tag = "naive"
    }
    else{
       tab = cbind(tab, StdErr1=x$SD.Single[,v])
       if(is.null(x$SD.Double)){
          flag = "StdErr1"
          tag = "single bootstrap"
       }
       else{
         tab = cbind(tab, StdErr2=x$SD.Double[,v])
          flag = "StdErr2"
          tag = "double bootstrap"
       }
    }
    tab = tab*displayFactor
    tab = cbind(tab, Zscore = as.vector(x$Coef[,v]/tab[,flag])*displayFactor)
    tab = cbind(tab, Pvalue = 2*pnorm(-abs(tab[,"Zscore"])))
    print(tab, digits=digits)
    cat("\n")
  }

  cat("Inference based on ", tag, " standard errors (", flag, ")\n",sep="")
  if(dim(tab)[2]>4) cat("\tfrom",x$numBoots,"bootstrap iterations\n\n")
  else cat("\n")

  cat("Proportion of total variation explained by WBC:", 
       format(x$RsquareTotal,digits=digits),"\n")
  cat("Proportion of stage 1 model explained by WBC:", 
       format(x$RsquareStage1,digits=digits),"\n")

}

bootInferWBCbyLme = function(Y, pheno, modelFix, modelRand, 
  coefWBC, contrastWBC=NULL, strata=NULL, R=10,
  vcovWBC=NULL, # Supply this as a list to compute double-bootstrap
  degFree=NULL,  # Supply this as a vector to use t-distribution for 2-boot
  saveBetaCoefficients = FALSE,
  silentErrors = TRUE
){

  M = dim(coefWBC)[1]
  d = dim(coefWBC)[2]

  if(is.null(contrastWBC)) {
     dBeta0 = d
     contrastWBC = diag(d)
     rownames(contrastWBC) = colnames(coefWBC)
  }
  else {
     dBeta0 = dim(contrastWBC)[1]
  }

  n = dim(pheno)[1]
  if(is.null(strata)) stratList = list(1:n)
  else{
    stratList = split(1:n, strata)
  }
  nStrata = length(stratList)
  stratListN = sapply(stratList, length)

  doDoubleBoot = FALSE
  if(!is.null(vcovWBC)){
     doDoubleBoot = TRUE
     oneR = matrix(1,R,1)
     Beta0 = array(NA, dim=c(M, dBeta0, R))
     for(j in 1:M){
       vchol = chol(vcovWBC[[j]])
       if(is.null(degFree)) noise = rnorm(R*d)
       else noise = rt(R*d, degFree[j])

       bootWBC = t(oneR %*% coefWBC[j,] + matrix(noise, R, d) %*% vchol)
       Beta0[j,,] = contrastWBC %*% bootWBC
     }
  }

  fit0 = inferWBCbyLme(Y, pheno, modelFix, modelRand, 
   coefWBC, contrastWBC, detailLevel=-1, silentErrors=silentErrors)
  nchip = dim(fit0$lmeVcomp$a)[2]
  chips = as.character(pheno[[fit0$lmeVcomp$ch]])

  GammaList = Beta1List = list()
  for(r in 1:R){
    bootsL = list()
    for(s in 1:nStrata){
       bootsL[[s]] = sample(stratList[[s]], stratListN[s], replace=TRUE)
    }
    boots = unlist(bootsL)

    Ystar = fit0$lmeVcomp$mu + fit0$lmeVcomp$e[,boots] 
    astar = fit0$lmeVcomp$a[,sample(1:nchip, nchip, replace=TRUE)]

    colnames(astar) = colnames(fit0$lmeVcomp$a)

    Ystar = Ystar + astar[,chips]

    GammaObject = inferWBCbyLme(Ystar, pheno, modelFix, modelRand, 
       coefWBC, contrastWBC, detailLevel=0, silentErrors=silentErrors)

    if(r %% 10==0) cat(r,"\n")
    GammaList[[r]] = GammaObject$GammaMatrix
    Beta1List[[r]] = GammaObject$Beta1Matrix
  }

  Z1 = cbind(1, coefWBC %*% t(contrastWBC))

  out = list()
  #out = list(numBoots=R, tDistribution=!is.null(degFree))

  out$Gamma1 = array(NA, dim=c(dBeta0+1, dim(Beta1List[[1]])[2], R))
  for(r in 1:R) {
    out$Gamma1[,,r] = solve(t(Z1)%*%Z1, t(Z1) %*% Beta1List[[r]])
  }

  dimnames(out$Gamma1) = c(dimnames(GammaList[[1]]), list(1:R))

  if(doDoubleBoot){
    out$Gamma2 = array(NA, dim=c(dBeta0+1, dim(Beta1List[[1]])[2], R))
    for(r in 1:R) {
      Z2 = cbind(1, Beta0[,,r])
      out$Gamma2[,,r] = solve(t(Z2)%*%Z2, t(Z2) %*% Beta1List[[r]])
    }
    dimnames(out$Gamma2) = c(dimnames(GammaList[[1]]), list(1:R))
  }

  if(saveBetaCoefficients) {
    if(doDoubleBoot) out$Beta0 = Beta0
    out$Beta1 = array(NA, dim=c(dim(Beta1List[[1]]),R))
    for(r in 1:R) {
      out$Beta1[,,r] = Beta1List[[r]]
    }
    dimnames(out$Beta1) = c(dimnames(Beta1List[[1]]), list(1:R))
  }

  class(out) = "inferWBCBoot"
  out
}

summary.inferWBCBoot = function(x){
  out = list(numBoots = dim(x$Gamma1)[3])
  out$Mean.Single = apply(x$Gamma1,1:2,mean)
  out$SD.Single = apply(x$Gamma1,1:2,sd)
  if(!is.null(x$Gamma2)){
    out$Mean.Double = apply(x$Gamma2,1:2,mean)
    out$SD.Double = apply(x$Gamma2,1:2,sd)
  }
  out
}

print.inferWBCBoot = function(x, digits=NULL){
  print(summary(x), digits=digits)
}

inferWBCbyLm = function(Y, pheno, modelFix, coefWBC, 
   contrastWBC=NULL, detailLevel=1, silentErrors=TRUE){

  M = dim(coefWBC)[1]
  n = dim(Y)[2]
  if(dim(Y)[1] != M) stop("Y does not match coefWBC in dimension\n")

  if(detailLevel == -1){
    lmVcomp = list(
      mu = matrix(NA,M,n),
      e = matrix(NA,M,n)
     )
  }

  sigmaResid = sigmaIcept = nObserved = nClusters = rep(NA, M)
  coefEsts = list()
  for(j in 1:M){
    ii = !is.na(Y[j,])
    nObserved[j] = sum(ii)
    pheno$y = Y[j,]

     fit = lm(modelFix, data=pheno[ii,])
     fitCoef = fit$coef
     sigmaResid[j] = summary(fit)$sigma
     sigmaIcept[j] = 0
     nClusters[j] = 0
     if(detailLevel == -1){
        lmVcomp$mu[j,ii] = predict(fit)
        lmVcomp$e[j,ii] = residuals(fit)
     }
     coefEsts[[j]] = fitCoef
  }
  coefEsts = t(matrix(unlist(coefEsts), length(fitCoef), M))
  rownames(coefEsts) = rownames(coefWBC)
  colnames(coefEsts) = names(fitCoef)

  if(is.null(contrastWBC)) Z = cbind(1,coefWBC)
  else Z = cbind(1, coefWBC %*% t(contrastWBC))
  colnames(Z)[1] = "<Intercept>"

  ZtZ = t(Z) %*% Z
  ZtZqr = qr(ZtZ)
  G = solve(ZtZqr, t(Z) %*% coefEsts)

  out = list(method="lm", GammaMatrix=G, Beta1Matrix=coefEsts, 
    sigma=sigmaResid,N=nObserved)

  if(detailLevel>=1){
    out$var.unscaled=solve(ZtZqr)

    d = dim(Z)[2]
    out$predicted = Z %*% G
    residual2 = coefEsts - out$predicted
    out$Sigma = (1/(M-d)) * t(residual2)%*%residual2

    X = model.matrix(modelFix, pheno)
    Xbar = apply(X,2,mean)
    Xctr = (X - matrix(1,n,1) %*% Xbar)
    residual2z = Xctr %*% t(residual2)
    out$ssu = apply(residual2z*residual2z, 2, sum)

    residual1 = Y - coefEsts %*% t(X) 
    out$sse = apply(residual1*residual1, 1, sum, na.rm=TRUE)

    residual0 = Y - apply(Y,1,mean,na.rm=TRUE) %*% matrix(1,1,n)
    out$ss0 = apply(residual0*residual0, 1, sum, na.rm=TRUE)

    if(detailLevel>=1){
       out$residuals = list(stage1=residual1, stage2=residual2, inner=t(residual2z))
    }
  }
  if(detailLevel==-1){
    out$lmVcomp = lmVcomp
  }

  class(out) = "inferWBC"
  out
}

bootInferWBCbyLm = function(Y, pheno, modelFix, 
  coefWBC, contrastWBC=NULL, strata=NULL, R=10,
  vcovWBC=NULL, # Supply this as a list to compute double-bootstrap
  degFree=NULL,  # Supply this as a vector to use t-distribution for 2-boot
  saveBetaCoefficients = FALSE,
  silentErrors = TRUE
){

  M = dim(coefWBC)[1]
  d = dim(coefWBC)[2]

  if(is.null(contrastWBC)) {
     dBeta0 = d
     contrastWBC = diag(d)
     rownames(contrastWBC) = colnames(coefWBC)
  }
  else {
     dBeta0 = dim(contrastWBC)[1]
  }

  n = dim(pheno)[1]
  if(is.null(strata)) stratList = list(1:n)
  else{
    stratList = split(1:n, strata)
  }
  nStrata = length(stratList)
  stratListN = sapply(stratList, length)

  doDoubleBoot = FALSE
  if(!is.null(vcovWBC)){
     doDoubleBoot = TRUE
     oneR = matrix(1,R,1)
     Beta0 = array(NA, dim=c(M, dBeta0, R))
     for(j in 1:M){
       vchol = chol(vcovWBC[[j]])
       if(is.null(degFree)) noise = rnorm(R*d)
       else noise = rt(R*d, degFree[j])

       bootWBC = t(oneR %*% coefWBC[j,] + matrix(noise, R, d) %*% vchol)
       Beta0[j,,] = contrastWBC %*% bootWBC
     }
  }

  fit0 = inferWBCbyLm(Y, pheno, modelFix,
   coefWBC, contrastWBC, detailLevel=-1, silentErrors=silentErrors)

  GammaList = Beta1List = list()
  for(r in 1:R){
    bootsL = list()
    for(s in 1:nStrata){
       bootsL[[s]] = sample(stratList[[s]], stratListN[s], replace=TRUE)
    }
    boots = unlist(bootsL)
    Ystar = fit0$lmVcomp$mu + fit0$lmVcomp$e[,boots] 

    GammaObject = inferWBCbyLm(Ystar, pheno, modelFix,
       coefWBC, contrastWBC, detailLevel=0, silentErrors=silentErrors)

    if(r %% 10==0) cat(r,"\n")
    GammaList[[r]] = GammaObject$GammaMatrix
    Beta1List[[r]] = GammaObject$Beta1Matrix
  }

  Z1 = cbind(1, coefWBC %*% t(contrastWBC))

  out = list()

  out$Gamma1 = array(NA, dim=c(dBeta0+1, dim(Beta1List[[1]])[2], R))
  for(r in 1:R) {
    out$Gamma1[,,r] = solve(t(Z1)%*%Z1, t(Z1) %*% Beta1List[[r]])
  }

  dimnames(out$Gamma1) = c(dimnames(GammaList[[1]]), list(1:R))

  if(doDoubleBoot){
    out$Gamma2 = array(NA, dim=c(dBeta0+1, dim(Beta1List[[1]])[2], R))
    for(r in 1:R) {
      Z2 = cbind(1, Beta0[,,r])
      out$Gamma2[,,r] = solve(t(Z2)%*%Z2, t(Z2) %*% Beta1List[[r]])
    }
    dimnames(out$Gamma2) = c(dimnames(GammaList[[1]]), list(1:R))
  }

  if(saveBetaCoefficients) {
    if(doDoubleBoot) out$Beta0 = Beta0
    out$Beta1 = array(NA, dim=c(dim(Beta1List[[1]]),R))
    for(r in 1:R) {
      out$Beta1[,,r] = Beta1List[[r]]
    }
    dimnames(out$Beta1) = c(dimnames(Beta1List[[1]]), list(1:R))
  }

  class(out) = "inferWBCBoot"
  out
}
############################################

########################################################################################################################
## parallelProcessing.R
## created: 2012-05-04
## creator: Fabian Mueller
## ---------------------------------------------------------------------------------------------------------------------
## Methods for parallel processing.
########################################################################################################################

## G L O B A L S #######################################################################################################

.parallel <- new.env()
.parallel[["do.par"]] <- FALSE
.parallel[["num.cores"]] <- -1L

## F U N C T I O N S ###################################################################################################

#' parallel.setup
#'
#' Sets up parallel processing. Requires the \pkg{foreach} and \pkg{doParallel} packages
#' @param ... Parameters for \code{registerDoParallel} from the \pkg{doParallel} package.
#' 			  This allows, for instance, for specificying the number of workers.
#' @return \code{TRUE} (invisible) to indicate that parallelization is set up.
#' @note Requires the packages \pkg{foreach} and \pkg{doParallel}.
#'
#' @author Fabian Mueller
#' @export parallel.setup
#' @examples
#' \donttest{
#' parallel.setup(2)
#' parallel.teardown()
#' }
parallel.setup <- function(...){
	logger.start("Setting up Multicore")
	rnb.require("foreach")
	rnb.require("doParallel")
	# .parallel[["cl"]] <- makeCluster(...)
	# registerDoParallel(.parallel[["cl"]])
	if(!any(sapply(list(...),is.numeric))&&!any(sapply(list(...),function(x)grepl("cluster",class(x))))){
	  stop("Invalid input to 'parallel.setup', needs to be numberic value or a cluster object (see registerDoParallel)")  
	}
	registerDoParallel(...)
	.parallel[["num.cores"]] <- getDoParWorkers()
	.parallel[["do.par"]] <- TRUE
	logger.info(c("Using",.parallel[["num.cores"]],"cores"))
	logger.completed()
	invisible(.parallel[["do.par"]])
}

########################################################################################################################

#' parallel.teardown
#'
#' Disables parallel processing.
#'
#' @return \code{TRUE}, invisibly.
#'
#' @author Fabian Mueller
#' @export parallel.teardown
#' @examples
#' \donttest{
#' parallel.getNumWorkers()
#' parallel.setup(2)
#' parallel.getNumWorkers()
#' parallel.teardown()
#' parallel.getNumWorkers()
#' }
parallel.teardown <- function(){
	.parallel[["do.par"]] <- FALSE
	.parallel[["num.cores"]] <- -1
	# stopCluster(.parallel[["cl"]])
	.stopImplicitCluster()
	invisible(TRUE)
}

########################################################################################################################

#' parallel.getNumWorkers
#'
#' Gets the number of workers used for parallel processing.
#' 
#' @return Number of workers used for parallel processing; \code{-1} if parallel processing is not enabled.
#'
#' @author Fabian Mueller
#' @export parallel.getNumWorkers
#' @examples
#' \donttest{
#' parallel.getNumWorkers()
#' parallel.setup(2)
#' parallel.getNumWorkers()
#' parallel.teardown()
#' parallel.getNumWorkers()
#' }
parallel.getNumWorkers <- function(){
	return(.parallel[["num.cores"]])
}

########################################################################################################################

#' parallel.isEnabled
#'
#' Checks if whether parallel processing is enabled.
#'
#' @return \code{TRUE} if multicore processing is enabled, \code{FALSE} otherwise.
#'
#' @author Fabian Mueller
#' @export parallel.isEnabled
#' @examples
#' \donttest{
#' parallel.isEnabled()
#' parallel.setup(2)
#' parallel.isEnabled()
#' parallel.teardown()
#' parallel.isEnabled()
#' }
parallel.isEnabled <- function(){
	return(.parallel[["do.par"]])
}

########################################################################################################################
## controlPlots.R
## created: 2012-04-04
## creator: Pavlo Lutsik
## ---------------------------------------------------------------------------------------------------------------------
## Quality control plots for HumanMethylation450 experiments.
########################################################################################################################

## G L O B A L S #######################################################################################################

.control.plot.theme.samples<-ggplot2::theme(
					plot.margin=grid::unit(c(0.1,1,0.1,0.1), "cm"),
					axis.title.x = ggplot2::element_blank(),
					axis.text.x=ggplot2::element_text(size=7, angle=-45, hjust=0))

HM450.SAMPLE.INDEPENDENT<-c("STAINING", "EXTENSION", "HYBRIDIZATION", "TARGET REMOVAL")
HM450.SAMPLE.DEPENDENT<-c("BISULFITE CONVERSION I","BISULFITE CONVERSION II",
		"SPECIFICITY I","SPECIFICITY II", "NON-POLYMORPHIC" )

## F U N C T I O N S ###################################################################################################

#' rnb.plot.control.boxplot
#'
#' Box plots of various control probes
#'
#' @param rnb.set \code{\linkS4class{RnBeadRawSet}} or \code{\linkS4class{RnBeadSet}} object with valid quality
#' 				  control information.
#' @param type    type of the control probe; must be one of the \code{"BISULFITE CONVERSION I"},
#'                \code{"BISULFITE CONVERSION II"}, \code{"EXTENSION"}, \code{"HYBRIDIZATION"}, \code{"NEGATIVE"},
#'                \code{"NON-POLYMORPHIC"}, \code{"NORM_A"}, \code{"NORM_C"}, \code{"NORM_G"}, \code{"NORM_T"},
#'                \code{"SPECIFICITY I"}, \code{"SPECIFICITY II"}, \code{"STAINING"}, \code{"TARGET REMOVAL"}.
#' @param writeToFile flag specifying whether the output should be saved as \code{\linkS4class{ReportPlot}}
#' @param numeric.names if \code{TRUE} and \code{writeToFile} is \code{TRUE}substitute the plot options in the plot file name with digits
#' @param ... other arguments to \code{\link{createReportPlot}}
#'
#' @return plot as an object of type \code{\linkS4class{ReportPlot}} if \code{writeToFile} is \code{TRUE} and of class
#' 			\code{\link{ggplot}} otherwise.
#'
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' rnb.plot.control.boxplot(rnb.set.example)
#' }
#'
#' @author Pavlo Lutsik
#' @export
rnb.plot.control.boxplot <- function(
		rnb.set,
		type = rnb.infinium.control.targets(rnb.set@target)[1],
		writeToFile = FALSE,
		numeric.names = FALSE,
		...) {

	if(!type %in% rnb.infinium.control.targets(rnb.set@target)) {
		stop(paste("Unrecognized control type:", type))
	}

	## Extract intensities of the control probes
	if(rnb.set@target=="probesEPIC"){
		meta <- rnb.get.annotation("controlsEPIC", assembly = "hg19")
	}else if(rnb.set@target=="probesEPICv2"){
		meta <- rnb.get.annotation("controlsEPICv2", assembly = "hg38")
	}else if(rnb.set@target=="probes450"){
		meta <- rnb.get.annotation("controls450", assembly = "hg19")
	}else if(rnb.set@target=="probes27"){
		meta <- rnb.get.annotation("controls27")
	}else if(rnb.set@target=="probesMMBC"){
        meta <- rnb.get.annotation("controlsMMBC", assembly="mm10")
    }

	if(rnb.set@target=="probesEPIC"){
		types<-rnb.infinium.control.targets(rnb.set@target)[c(14,4,3,15,1:2,12:13,6,11)]
	}else if(rnb.set@target=="probesEPICv2"){ 
		types<-rnb.infinium.control.targets(rnb.set@target)[c(14,4,3,15,1:2,12:13,6,11)]
	}else if(rnb.set@target=="probes450"){
		types<-rnb.infinium.control.targets(rnb.set@target)[c(13,4,3,14,1:2,11:12,6)]
	}else if(rnb.set@target=="probes27"){
		types<-rnb.infinium.control.targets(rnb.set@target)[c(10,3,2,11,1,9,6)]
	}else if(rnb.set@target=="probesMMBC"){
        types<-rnb.infinium.control.targets(rnb.set@target)[c(14,4,3,15,1:2,12:13,6,11)]
    }

	if(!type %in% types){
		warning("Unoptimized probe type, plotting performance may be decreased")
	}

	if(rnb.set@target=="probes450" || rnb.set@target=="probesEPIC" || rnb.set@target=="probesMMBC" || rnb.set@target=="probesEPICv2"){
		rownames(meta)<-meta[["ID"]]
		### TODO: Remove the following passage
		### for testing purposes only!
		if(rnb.set@target=="probesEPIC" || rnb.set@target=="probesEPICv2"){ 
			meta<-rnb.update.controlsEPIC.enrich(meta) ## TODO: Not validated for EPICv2
		}
		meta <- meta[type == meta[["Target"]], ]
		ids<-as.character(meta[["ID"]])
		ids<-intersect(rownames(qc(rnb.set)[[1]]), ids)
		meta<-meta[ids,]
	}else if(rnb.set@target=="probes27"){
		meta <- meta[type == meta[["Type"]], ]
		ids<-as.character(meta[["Address"]])
		ids<-intersect(rownames(qc(rnb.set)[[1]]), ids)
		meta<-meta[meta$Address %in% ids,]
	}

	if(nrow(meta)==0){
		plot.obj<-rnb.message.plot(sprintf("Box plot for control type %s not available", type))
		if(writeToFile){
			plot.file<-createReportGgPlot(plot.obj, paste("ControlBoxPlot", ifelse(numeric.names, match(type, types), gsub(" ", ".", type)) , sep="_"), ...)
			
			print(plot.obj)
			off(plot.file)
			return(plot.file)
		}
		return(plot.obj)
	}

	intensities <- lapply(c("green" = "Cy3", "red" = "Cy5"), function(channel) {
		result <- qc(rnb.set)[[channel]][ids, ]
		if (is.vector(result)) {
			return(as.matrix(result))
		}
		return(t(result))
	})

	scales<-lapply(qc(rnb.set), get.unified.scale)

	if(rnb.set@target=="probes450" || rnb.set@target=="probesEPIC" || rnb.set@target=="probesMMBC" || rnb.set@target=="probesEPICv2"){
		## Shorten the words describing probe's expected intensity
		INTENSITIES <- c("Background" = "Bgnd", "High" = "High", "Low" = "Low", "Medium" = "Med")
		levels(meta[, "Expected Intensity"]) <- INTENSITIES[levels(meta[, "Expected Intensity"])]
		meta[, "Expected Intensity"] <- as.character(meta[, "Expected Intensity"])

		## Define bar labels in the plot
		plot.names<-sapply(1:length(ids), sprintf, fmt="Probe%2g")

		exp.intensity <- lapply(c("green" = "Green", "red" = "Red"), function(channel) {
			ifelse(meta[[paste("Evaluate", channel)]] == "+", meta[["Expected Intensity"]], "Bgnd")
		})
		plot.names <- lapply(exp.intensity, function(exps) { paste(plot.names, exps, sep = ":\n") })

		plot.data <- lapply(names(intensities), function(channel) {
			channel.data <- intensities[[channel]]
			data.frame(
				"Intensity" = as.numeric(channel.data),
				"Probe" = as.factor(sapply(plot.names[[channel]], rep, times=nrow(channel.data))))
		})
	}else if(rnb.set@target=="probes27"){
		plot.names<-list()
		plot.names$green <- gsub("conversion |Extension |mismatch ", "", meta$Name)
		plot.names$red <- gsub("conversion |Extension |mismatch ", "", meta$Name)
		plot.data <- lapply(names(intensities), function(channel) {
					channel.data <- intensities[[channel]]
					data.frame(
							"Intensity" = as.numeric(channel.data),
							"Probe" = as.factor(sapply(plot.names[[channel]], rep, times=nrow(channel.data))))
				})
	}

	if(writeToFile){
		plot.file<-createReportPlot(paste("ControlBoxPlot", ifelse(numeric.names, match(type, types), gsub(" ", ".", type)) , sep="_"), ...)
		grid.newpage()
	}

	## Define viewports and assign it to grid layout
	#grid.newpage()
	#pushViewport(viewport(layout = grid.layout(2,1)))

	plots<-list()

	for (i in 1:length(intensities)) {
		channel <- names(intensities)[i]
#		print(qplot(Probe, Intensity, data=na.omit(plot.data[[i]]), geom = "boxplot",
		plots[[i]]<-qplot(Probe, Intensity, data=na.omit(plot.data[[i]]), geom = "boxplot",
				main=paste(type, paste(channel, "channel"), sep=": "),
				ylab="Intensity",fill=I(channel)) + scale_y_continuous(limits=scales[[i]])
		#,vp=viewport(layout.pos.row=i, layout.pos.col=1))
	}

	## FIXME: Rewrite the plotting function without using the function arrangeGrob in gridExtra
	grb<-do.call(arrangeGrob, plots)

	grid.draw(grb)
	if(writeToFile) {
		off(plot.file)
		return(plot.file)
	}else{
		return(invisible(grb))
	}
}

#######################################################################################################################

#' rnb.plot.negative.boxplot
#'
#' Box plots of negative control probes
#'
#' @param rnb.set 		\code{\linkS4class{RnBeadSet}} object with valid quality control information
#' @param sample.subset an integer vector specifying the subset of samples for which the plotting should be performed
#' @param writeToFile 	flag specifying whether the output should be saved as \code{\linkS4class{ReportPlot}}
#' @param name.prefix	in case \code{writeToFile} is \code{TRUE}, a \code{character} singleton specifying a prefix to the variable part of the image file names
#' @param ... 			other arguments to \code{\link{createReportPlot}}
#'
#' @return plot as an object of type \code{\linkS4class{ReportPlot}} if \code{writeToFile} is \code{TRUE} and of class
#' 			\code{\link{ggplot}} otherwise.
#'
#' @author Pavlo Lutsik
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' rnb.plot.negative.boxplot(rnb.set.example)
#' }
rnb.plot.negative.boxplot<- function(
		rnb.set,
		sample.subset=1:length(samples(rnb.set)),
		writeToFile = FALSE,
		name.prefix=NULL,
		...) {

	
	if(rnb.set@target=="probesEPIC" || rnb.set@target=="probesEPICv2" || rnb.set@target=="probesMMBC" ){
		genome.assembly<-rnb.getOption("assembly")
		meta <- rnb.get.annotation(gsub("probes", "controls", rnb.set@target), assembly=c("probesEPIC"=genome.assembly, "probesMMBC"="mm10", "probesEPICv2"="hg38")[rnb.set@target])
		## Extract intensities of the control probes
		### TODO: Remove the following passage
		### for testing purposes only!
		if(rnb.set@target=="probesEPIC" || rnb.set@target=="probesEPICv2"){
			meta<-rnb.update.controlsEPIC.enrich(meta) ## TODO: Not validated for EPICv2 (duplicate)
		}
		meta <- meta["NEGATIVE" == meta[["Target"]], ]
		ids<-as.character(meta[["ID"]])
		ids<-intersect(rownames(qc(rnb.set)[[1]]), ids)
		meta<-meta[ids,]
	}else if(rnb.set@target=="probes450"){
		meta <- rnb.get.annotation("controls450")
		meta <- meta["NEGATIVE" == meta[["Target"]], ]
		ids<-as.character(meta[["ID"]])
		ids<-intersect(rownames(qc(rnb.set)[[1]]), ids)
		meta<-meta[ids,]
	}else if(rnb.set@target=="probes27"){
		meta <- rnb.get.annotation("controls27")
		meta <- meta["Negative" == meta[["Type"]], ]
		ids<-as.character(meta[["Address"]])
		ids<-intersect(rownames(qc(rnb.set)[[1]]), ids)
		meta<-meta[meta$Address %in% ids,]
	}
	if(nrow(meta)==0){
		if(writeToFile) plot.file<-createReportPlot(paste("ControlBoxPlot", ifelse(numeric.names, match(type, types), gsub(" ", ".", type)) , sep="_"), ...)
		print(rnb.message.plot("Box plot for negative control probes is not available"))
		if(writeToFile){
			off(plot.file)
			return(plot.file)
		}
		return(invisible())
	}

	intensities <- lapply(c("green" = "Cy3", "red" = "Cy5"), function(channel) {
				result <- qc(rnb.set)[[channel]][ids,sample.subset]
				if (is.vector(result)) {
					return(as.matrix(result))
				}
				return(result)
			})

	## Shorten the words describing probe's expected intensity
#	INTENSITIES <- c("Background" = "Bgnd", "High" = "High", "Low" = "Low", "Medium" = "Med")
#	levels(meta[, "Expected Intensity"]) <- INTENSITIES[levels(meta[, "Expected Intensity"])]
#	meta[, "Expected Intensity"] <- as.character(meta[, "Expected Intensity"])

	## Define bar labels in the plot
#	plot.names<-sapply(1:length(ids), sprintf, fmt="Probe%2g")
	sample.ids<-samples(rnb.set)[sample.subset]

	## FIXME: This might leads to duplicated identifiers and/or reordering of samples
	sample.labels<-abbreviate.names(sample.ids)

#	exp.intensity <- lapply(c("green" = "Green", "red" = "Red"), function(channel) {
#				ifelse(meta[[paste("Evaluate", channel)]] == "+", meta[["Expected Intensity"]], "Bgnd")
#			})

#	plot.names <- lapply(exp.intensity, function(exps) { paste(plot.names, exps, sep = ":\n") })

	plot.data <- lapply(names(intensities), function(channel) {
				channel.data <- intensities[[channel]]
				data.frame(
						"Intensity" = as.numeric(channel.data),
						"Sample" = factor(sapply(sample.ids, rep, times=nrow(channel.data)), as.character(sample.ids)))
			})
	if(is.null(name.prefix)){
		plot.name<-"NegativeControlBoxPlot"
	}else{
		plot.name<-paste("NegativeControlBoxPlot", name.prefix, sep="_")
	}
	if(writeToFile) {
		plot.file<-createReportPlot(plot.name, ...)
		grid.newpage()
	}

	## Define viewports and assign it to grid layout
	#grid.newpage()
	#pushViewport(viewport(layout = grid.layout(2,1)))

	plots<-list()

	for (i in 1:length(intensities)) {
		channel <- names(intensities)[i]
		plots[[i]]<-qplot(Sample, Intensity, data=na.omit(plot.data[[i]]), geom = "boxplot",
						main=paste("Negative control probes ", paste(channel, "channel"), sep=": "),
						ylab="Intensity",fill=I(channel))+
						scale_x_discrete(labels=sample.labels)+
						.control.plot.theme.samples
						#,vp=viewport(layout.pos.row=i, layout.pos.col=1))
	}

	## FIXME: Rewrite the plotting function without using the function arrangeGrob in gridExtra
	grb<-do.call(arrangeGrob, plots)

	grid.draw(grb)
	if(writeToFile) {
		off(plot.file)
		return(plot.file)
	}else{
		return(invisible(grb))
	}
}
#######################################################################################################################
#' rnb.plot.control.barplot
#'
#' Per-sample bar plots of Illumina HumanMethylation control probes
#'
#' @param rnb.set 		\code{\linkS4class{RnBeadRawSet}} or \code{\linkS4class{RnBeadSet}} object with valid
#' 						quality control information
#' @param probe 		exact id of the control probe consisting of the control probe type (see \code{\link{rnb.plot.control.boxplot}})
#' @param sample.subset an integer vector specifying the subset of samples for which the plotting should be performed
#' @param writeToFile 	flag specifying whether the output should be saved as \code{\linkS4class{ReportPlot}}
#' @param numeric.names if \code{TRUE} and \code{writeToFile} is \code{TRUE}substitute the plot options in the plot file name with digits
#' @param name.prefix	in case \code{writeToFile} is \code{TRUE}, a \code{character} singleton specifying a prefix to the variable part of the image file names
#' @param verbose		if \code{TRUE} additional diagnostic output is generated
#' @param ... 			other arguments to \code{\link{createReportPlot}}
#'
#' @return plot as an object of type \code{\linkS4class{ReportPlot}} if \code{writeToFile} is \code{TRUE} and of class
#' 			\code{\link{ggplot}} otherwise.
#'
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' control.meta.data <- rnb.get.annotation("controls450")
#' ctrl.probe<-paste0(unique(control.meta.data[["Target"]])[4], ".3")
#' print(ctrl.probe) # EXTENSION.3
#' rnb.plot.control.barplot(rnb.set.example, ctrl.probe)
#' }
#'
#' @author Pavlo Lutsik
#' @export
rnb.plot.control.barplot<-function(
		rnb.set,
		probe,
		sample.subset=1:length(samples(rnb.set)),
		writeToFile=FALSE,
		numeric.names=FALSE,
		name.prefix=NULL,
		verbose=FALSE,
		...)
{
	
	
	if(rnb.set@target=="probesEPIC" || rnb.set@target=="probesEPICv2" || rnb.set@target=="probesMMBC"){
		genome.assembly<-rnb.getOption("assembly") ## TODO: Improve genome build selection
		control.meta.data <- rnb.get.annotation(gsub("probes", "controls", rnb.set@target), assembly=c("probesEPIC"=genome.assembly, "probesEPICv2"="hg38", "probesMMBC"="mm10")[rnb.set@target])
		### TODO: Remove the following passage
		### for testing purposes only!
		#if(rnb.set@target=="probesEPIC"){
		#	control.meta.data<-rnb.update.controlsEPIC.enrich(control.meta.data)
		#}
		type=strsplit(probe,".", fixed=TRUE)[[1]][1]
		index=as.integer(strsplit(probe,".", fixed=TRUE)[[1]][2])
		if(!(type %in% rnb.infinium.control.targets(rnb.set@target))){
			rnb.error(c("Unrecognized control probe:", probe))
		}
		id<-subset(control.meta.data, Target==type & Index==index)$ID
		id.col<-"ID"
		target.col<-"Target"
		targets<-c(14,4,3,15,1:2,12:13,6,11)
	}else if(rnb.set@target=="probes450"){
		control.meta.data <- rnb.get.annotation("controls450")
		type=strsplit(probe,".", fixed=TRUE)[[1]][1]
		index=as.integer(strsplit(probe,".", fixed=TRUE)[[1]][2])
		if(!(type %in% rnb.infinium.control.targets(rnb.set@target))){
			rnb.error(c("Unrecognized control probe:", probe))
		}
		id<-subset(control.meta.data, Target==type & Index==index)$ID
		id.col<-"ID"
		target.col<-"Target"
		targets<-c(13,4,3,14,1:2,11:12,6)
	}else if(rnb.set@target=="probes27"){
		control.meta.data <- rnb.get.annotation("controls27")
		if(!probe %in% control.meta.data$Name){
			rnb.error(c("Unrecognized control probe:", probe))
		}
		id<-control.meta.data[probe==control.meta.data[["Name"]],"Address"]
		id.col<-"Address"
		target.col<-"Type"
		targets<-c(10,3,2,11,1,9,6)
	}

	#get intensities
	green<-qc(rnb.set)$Cy3[,sample.subset, drop=FALSE]
	red<-qc(rnb.set)$Cy5[,sample.subset, drop=FALSE]

	#get full set of probes
	full.probe.set<-control.meta.data[[id.col]][control.meta.data[[target.col]] %in% rnb.infinium.control.targets(rnb.set@target)[targets]]
	
	probe.name<-gsub(" ", ".", probe)

	if(! id %in% rownames(green)){
		
		plot.obj<-rnb.message.plot(sprintf("Box plot for control type %s not available", type))
		if(writeToFile){
			plot.name<-paste('ControlBarPlot', name.prefix, ifelse(numeric.names, 
							match(id, full.probe.set), 
							probe.name) , sep="_")
			plot.file<-createReportGgPlot(plot.obj, plot.name, ...)
			print(plot.obj)
			off(plot.file)
			return(plot.file)
		}
		return(plot.obj)
	}

	green<-green[as.character(id),]
	red<-red[as.character(id),]

#   if(is.null(dim(green))) green<-t(as.matrix(green))
#   if(is.null(dim(red))) red<-t(as.matrix(red))

	## get meta information

	if(rnb.set@target=="probes450" || rnb.set@target=="probesEPIC" || rnb.set@target=="probesMMBC" || rnb.set@target=="probesEPICv2"){
		meta<-subset(control.meta.data, ID==id)
	}else if(rnb.set@target=="probes27"){
		meta<-subset(control.meta.data, Address==id)
	}


	plot.name<-paste('ControlBarPlot', name.prefix, ifelse(numeric.names, 
					match(id, full.probe.set), 
					probe.name) , sep="_")
	if(writeToFile){
		plot.file<-createReportPlot(plot.name, ...)
		grid.newpage()
	}

	Samples<-factor(samples(rnb.set)[sample.subset], as.character(samples(rnb.set)[sample.subset]))
	## shorten sample names

	## FIXME: This might leads to duplicated identifiers and/or reordering of samples
	sample.labels<-abbreviate.names(samples(rnb.set)[sample.subset])

	#grid.newpage()
	# define viewports and assign it to grid layout
	#pushViewport(viewport(layout = grid.layout(2,1)))

	### plot green channel

	if(rnb.set@target=="probes450" || rnb.set@target=="probesEPIC" || rnb.set@target=="probesMMBC" || rnb.set@target=="probesEPICv2"){
		main_txt_grn<-paste(probe, meta[,"Description"], "green channel", if(meta[, "Evaluate Green"]=="+") meta[, "Expected Intensity"] else "Background", sep=": ")
		main_txt_red<-paste(probe, meta[,"Description"], "red channel",if(meta[, "Evaluate Red"]=="+") meta[, "Expected Intensity"] else "Background", sep=": ")
	}else{
		main_txt_grn<-paste(probe, sep="")
		main_txt_red<-paste(probe, sep="")
	}

	empty.probe<-sum(sapply(green, is.na))==length(green)

    if(empty.probe) {
		green<-rep(0, length(green))
	}
	
	green.plot<-ggplot(data = data.frame(Samples=factor(Samples, levels=Samples), Intensity=green), aes(x = Samples, y = Intensity)) +
			geom_bar(stat = "identity", position = "stack",fill = "green") + ggtitle(main_txt_grn) +
					scale_x_discrete(labels=sample.labels)+ylab("Intensity")+
					.control.plot.theme.samples

	if(empty.probe) {
		green.plot<-green.plot+annotate("text", label="No intensities found for this quality control probe", x=length(green)%/%2, y=0.5, size=5)
	}
	#print(green.plot, vp=viewport(layout.pos.row=1, layout.pos.col=1))

	### plot red channel
	if(sum(sapply(red, is.na))==length(red)) empty.probe<-TRUE else empty.probe<-FALSE
	if(empty.probe) {
		red<-rep(0, length(red));
	}
	red.plot<-ggplot(data = data.frame(Samples=factor(Samples, levels=Samples), Intensity=red), aes(x = Samples, y = Intensity)) +
			geom_bar(stat = "identity", position = "stack",fill = "red") + ggtitle(main_txt_red) +
			scale_x_discrete(labels=sample.labels)+ylab("Intensity")+
			.control.plot.theme.samples

	if(empty.probe){
		red.plot<-red.plot+annotate("text", label="No intensities found for this quality control probe", x=length(red)%/%2, y=0.5, size=5)
	}
	#print(red.plot, vp=viewport(layout.pos.row=2, layout.pos.col=1))

	## FIXME: Rewrite the plotting function without using the function arrangeGrob in gridExtra
	grb<-arrangeGrob(green.plot, red.plot)
  grid.draw(grb)
  
  if(writeToFile) {
	  off(plot.file)
	  return(plot.file)
  }else{
	  return(invisible(grb))
  }

}

#######################################################################################################################

#' rnb.get.snp.matrix
#' 
#' Gets the matrix with "beta" values of SNP probes in the given dataset.
#' 
#' @param dataset       Microarray-based methylation dataset as an object of type inheriting \code{RnBeadSet}.
#' @param threshold.nas Threshold for removing probes (rows) and samples (columns). If more than this fraction of
#'                      \code{NA}s are present in the SNP matrix, the corresponding row or column is removed.
#' @return Two-dimensional \code{matrix} with the values at the SNP probes.
#' 
#' @author Yassen Assenov
#' @noRd
rnb.get.snp.matrix <- function(dataset, threshold.nas = 1) {
	if (dataset@target %in% c("probes450", "probesEPIC", "probesMMBC", "probesEPICv2")) {
		result <- meth(dataset, row.names=TRUE)
		result <- result[grep("^rs", rownames(result)), , drop = FALSE]
	} else if (dataset@target == "probes27") {
		result <- HM27.snp.betas(qc(dataset))
	} else {
		stop("invalid value for dataset")
	}
	if (length(result) == 0) {
		stop("invalid value for dataset; no SNP probes found")
	}
	
	i <- which(apply(is.na(result), 1, mean) > threshold.nas)
	if (length(i) != 0) {
		if (length(i) == nrow(result)) {
			rnb.error("Not enough SNP data available (too many missing values per probe)")
		}
		result <- result[-i, , drop = FALSE]
		rnb.warning(paste(length(i), "probes ignored because their they contain too many NAs"))
	}
	i <- which(apply(is.na(result), 2, mean) > threshold.nas)
	if (length(i) != 0) {
		if (length(i) == ncol(result)) {
			rnb.error("Not enough SNP data available (too many missing values per sample)")
		}
		result <- result[, -i, drop = FALSE]
		rnb.warning(paste(length(i), "samples ignored because their SNP probes contain too many NAs"))
	}
	return(result)
}

#######################################################################################################################

#' rnb.plot.snp.boxplot
#'
#' Box plots of beta-values from the genotyping probes
#'
#' @param dataset     Dataset as an object of type inheriting \code{\linkS4class{RnBeadSet}}, or a matrix of
#'                    methylation beta values.
#' @param writeToFile Flag specifying whether the output should be saved as \code{\linkS4class{ReportPlot}}.
#' @param ...         Additional named arguments passed to \code{\link{createReportPlot}}.
#' @return If \code{writeToFile} is \code{TRUE}: plot as an object of type \code{\linkS4class{ReportPlot}}. Otherwise:
#'         plot as an object of type \code{\link{ggplot}}.
#'
#' @export
#' @author Pavlo Lutsik
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' rnb.plot.snp.boxplot(rnb.set.example)
#' }
rnb.plot.snp.boxplot <- function(dataset, writeToFile=FALSE, ...) {

	if (inherits(dataset, "RnBeadSet")) {
		dataset <- rnb.get.snp.matrix(dataset)
	}
	if (!(is.matrix(dataset) && is.numeric(dataset) && length(dataset) != 0)) {
		stop("invalid value for dataset")
	}
	if (!parameter.is.flag(writeToFile)) {
		stop("invalid value for writeToFile; expected TRUE or FALSE")
	}

	dframe <- data.frame(
		bv = as.numeric(dataset),
		SNP = factor(rep(rownames(dataset), ncol(dataset)), levels = rownames(dataset)))
	rplot <- qplot(SNP, bv, data = dframe, geom = "boxplot", main = "", ylab = expression(beta)) +
		theme(axis.text.x = element_text(angle = 90, vjust = 0))

	if (writeToFile) {
		plot.file <- createReportPlot("SNPBoxplot", ...)
		print(rplot)
		off(plot.file)
		return(plot.file)
	}
	return(invisible(rplot))
}

#######################################################################################################################

#' rnb.plot.snp.barplot
#'
#' Bar plots of beta-values from the genotyping probes
#'
#' @param dataset       Dataset as an instance of \code{\linkS4class{RnBeadRawSet}} or \code{\linkS4class{RnBeadSet}}.
#'                      Alternatively, the dataset can be specified as a non-empty \code{matrix} containing the computed
#'                      beta values on the SNP probes.
#' @param probeID       Probe identifier. This must be one of \code{rownames(meth(dataset))}.
#' @param writeToFile   Flag specifying whether the output should be saved as \code{\linkS4class{ReportPlot}}.
#' @param numeric.names if \code{TRUE} and \code{writeToFile} is \code{TRUE}substitute the plot options in the plot file
#'                      name with digits.
#' @param ...           Additional named arguments passed to \code{\link{createReportPlot}}.
#'
#' @return plot as an object of type \code{\linkS4class{ReportPlot}} if \code{writeToFile} is \code{TRUE} and of class
#' 			\code{\link{ggplot}} otherwise.
#'
#' @export
#' @author Pavlo Lutsik
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' samp<-samples(rnb.set.example)[1]
#' rnb.plot.snp.barplot(rnb.set.example, samp)
#' }
rnb.plot.snp.barplot <- function(dataset, probeID, writeToFile=FALSE, numeric.names=FALSE, ...) {

	if (inherits(dataset, "RnBeadSet")) {
		dataset <- rnb.get.snp.matrix(dataset)
	}
	if (!(is.matrix(dataset) && is.numeric(dataset) && length(dataset) != 0)) {
		stop("invalid value for dataset")
	}
	ids <- rownames(dataset)
	if (is.null(ids)) {
		stop("invalid value for dataset; missing row names")
	}
	if (is.null(colnames(dataset))) {
		stop("invalid value for dataset; missing column names")
	}
	if (!(is.character(probeID) && length(probeID) == 1 && isTRUE(probeID != ""))) {
		stop("invalid value for probeID")
	}
	if (!(probeID %in% ids)) {
		stop("invalid value for probeID; missing in dataset")
	}
	if (!parameter.is.flag(writeToFile)) {
		stop("invalid value for writeToFile; expected TRUE or FALSE")
	}
	if (!parameter.is.flag(numeric.names)) {
		stop("invalid value for numeric.names; expected TRUE or FALSE")
	}

	dframe <- data.frame(
		bv = dataset[probeID, ],
		sm = factor(colnames(dataset), levels = colnames(dataset)))
	rplot <- ggplot(dframe, aes_string(x = 'sm', y = 'bv')) + geom_bar(stat = "identity", position = "stack") + 
		scale_y_continuous(limits = c(0, 1)) + labs(title = probeID, x = NULL, y = expression(beta)) +
		.control.plot.theme.samples

	if(writeToFile){
		pfile <- ifelse(numeric.names, match(probeID, ids), gsub("[ |_]", ".", probeID))
		pfile <- createReportPlot(paste0('SNPBarPlot_', pfile), ...)
		print(rplot)
		off(pfile)
		return(pfile)
	}
	return(invisible(rplot))
}

#######################################################################################################################

#' rnb.plot.snp.heatmap
#'
#' Heatmap of beta values from genotyping probes.
#'
#' @param dataset     Dataset as an object of type inheriting \code{\linkS4class{RnBeadSet}}, or a matrix of
#'                    methylation beta values.
#' @param writeToFile Flag specifying whether the output should be saved as \code{\linkS4class{ReportPlot}}.
#' @param ...         Additional named arguments passed to \code{\link{createReportPlot}}. These are used only if
#'                    \code{writeToFile} is \code{TRUE}.
#' @return If \code{writeToFile} is \code{TRUE}, plot as an object of type \code{\linkS4class{ReportPlot}}. Otherwise,
#'         there is no value returned (invisible \code{NULL}).
#'
#' @export
#' @author Pavlo Lutsik
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' rnb.plot.snp.heatmap(rnb.set.example)
#' }
rnb.plot.snp.heatmap <- function(dataset, writeToFile = FALSE, ...) {

	if (inherits(dataset, "RnBeadSet")) {
		dataset <- rnb.get.snp.matrix(dataset)
	}
	if (!(is.matrix(dataset) && is.numeric(dataset) && length(dataset) != 0)) {
		stop("invalid value for dataset")
	}
	if (!parameter.is.flag(writeToFile)) {
		stop("invalid value for writeToFile; expected TRUE or FALSE")
	}

	if (writeToFile) {
		plot.file <- createReportPlot('SNPHeatmap', ...)
	}
	if (nrow(dataset) < 2 || ncol(dataset) < 2) {
		rnb.message.plot("Heatmap is not available due to insufficient data.")
	} else {
		sample.ids <- abbreviate.names(colnames(dataset))
		meth.colors <- get.methylation.color.panel()
		meth.breaks <- seq(0, 1, length.out = length(meth.colors) + 1L)
		suppressWarnings(heatmap.2(dataset, scale = "none", na.rm = FALSE,
				breaks = meth.breaks, col = meth.colors, trace = "none",
				margins = c(8, 8), labRow = rownames(dataset), labCol = sample.ids,
				density.info = "density", key.title = NA, key.xlab = expression(beta), key.ylab = "Density"))
	}
	if (writeToFile) {
		off(plot.file)
		return(plot.file)
	}
	return(invisible(NULL))
}

#######################################################################################################################

## point.whisker.ggplot
##
## Creates a ggplot object containing a point-and-whisker plot.
##
## @param dframe Table of values in the form of a \code{data.frame} containig at least the following columns:
##               \code{"Position"}, \code{"Average"}, \code{"Deviation"}.
## @param yrange Requested value range of the y axis.
## @param xlab Title of the x axis.
## @param ylab Title of the y axis.
##
## @author Yassen Assenov
point.whisker.ggplot <- function(dframe, yrange, xlab = "Position on the slide", ylab = "Methylation") {
	ggplot(dframe, aes_string(x = "Position", y = "Average")) +
		coord_cartesian(ylim = yrange) + scale_x_discrete(drop = FALSE) + ggplot2::geom_point(size = 3) +
		geom_errorbar(aes_string(ymin = "Average - Deviation", ymax = "Average + Deviation"), width = 0.4) +
		labs(x = xlab, y = ylab) + theme(axis.text.x = element_text(angle = 90, vjust = 0.5))
}

#######################################################################################################################

#' rnb.plot.sentrix.distribution
#'
#' Creates a point-and-whisker plots showing beta value distributions at Sentrix positions for the given slide.
#'
#' @param rnb.set    HumanMethylation450K dataset as an object of type \code{\linkS4class{RnBeadSet}}.
#' @param sentrix.id Slide number (Sentrix ID) as an \code{integer} or \code{character} singleton.
#' @return Generated point-and-whisker plot (an instance of \code{\link{ggplot}}) of mean methylations for the samples
#'         on the specified slide, or \code{FALSE} if the dataset is non-empty but does not contain samples on the
#'         given slide. If the provided dataset does not contain valid Sentrix ID and position information (or is an
#'         empty dataset), this method returns \code{NULL}.
#'
#'
#' @author Yassen Assenov
#' @export
#' @examples
#' \donttest{
#' library(RnBeads.hg19)
#' data(small.example.object)
#' sid<-as.character(pheno(rnb.set.example)[["Sentrix_ID"]][1])
#' rnb.plot.sentrix.distribution(rnb.set.example,sid)
#' }
rnb.plot.sentrix.distribution <- function(rnb.set, sentrix.id) {
	if (!inherits(rnb.set, "RnBeadSet")) {
		stop("invalid value for rnb.set")
	}
	if (!((is.character(sentrix.id) || is.integer(sentrix.id)) && length(sentrix.id) == 1 && (!is.na(sentrix.id)))) {
		stop("invalid value for sentrix.id")
	}
	dframe <- rnb.get.sentrix.data(rnb.set)
	if (is.null(dframe) || nrow(dframe) == 0) {
		return(NULL)
	}

	## Find the samples in the dataset on the specified slide
	s.indices <- which(dframe[, "Slide"] == sentrix.id) # this works both for integer and character sentrix.ids
	if (length(s.indices) == 0) {
		return(FALSE)
	}

	## Compute mean methylation and standard deviation
	m.methylations <- apply(as.matrix(meth(rnb.set)), 2, function(x) { c(mean(x, na.rm = TRUE), sd(x, na.rm = TRUE)) })
	if(!is.matrix(m.methylations)){
		m.methylations<-matrix(m.methylations, ncol=1)
	}
	yrange <- range(m.methylations[1, ] - m.methylations[2, ], m.methylations[1, ] + m.methylations[2, ])
	m.methylations <- cbind(dframe[s.indices, ], data.frame(
			"Identifier" = colnames(meth(rnb.set))[s.indices],
			"Average" = m.methylations[1, ],
			"Deviation" = m.methylations[2, ]))
	m.methylations <- m.methylations[order(m.methylations[, "Position"]), ]

	## Create the plot
	point.whisker.ggplot(m.methylations, yrange)
}

#######################################################################################################################

#' rnb.plot.sentrix.distributions
#'
#' Creates one or more point-and-whisker plots showing beta value distributions at Sentrix positions.
#'
#' @param rnb.set HumanMethylation450K dataset as an object of type \code{\linkS4class{RnBeadSet}}.
#' @param fprefix File name prefix to be used in the generated plots. In order to ensure independence of the operating
#'                system, there are strong restrictions on the name of the file. See the documentation of
#'                \code{\link{createReportPlot}} for more information.
#' @param ...     Other arguments passed to \code{\link{createReportPlot}}. These can include the named parameters
#'                \code{report}, \code{width}, \code{height}, and others.
#' @return Point-and-whisker plot (an instance of \code{\linkS4class{ReportPlot}}), or a list of such plots - one per
#'         slide. If the provided dataset does not contain valid Sentrix ID and position information (or is an empty
#'         dataset), this method returns \code{NULL}.
#'
#' @details
#' If no additional parameters are specified, this function creates one PDF and one low-resolution PNG file for every
#' generated plot.
#'
#' @seealso \code{\link{rnb.plot.sentrix.distribution}} for creating a single plot for a specified slide number
#'
#' @author Yassen Assenov
#' @export
rnb.plot.sentrix.distributions <- function(rnb.set, fprefix = "sentrix_whisker", ...) {
	if (!inherits(rnb.set, "RnBeadSet")) {
		stop("invalid value for rnb.set")
	}
	if (!(is.character(fprefix) && length(fprefix) == 1 && (!is.na(fprefix)))) {
		stop("invalid value for fprefix")
	}
	if (!grepl("^[A-Za-z0-9._-]+$", fprefix)) {
		stop("invalid value for fprefix")
	}
	dframe <- rnb.get.sentrix.data(rnb.set)
	if (is.null(dframe) || nrow(dframe) == 0) {
		return(NULL)
	}

	m.methylations <- apply(meth(rnb.set), 2, function(x) { c(mean(x, na.rm = TRUE), sd(x, na.rm = TRUE)) })
	if(!is.matrix(m.methylations)){
		m.methylations<-matrix(m.methylations, ncol=1)
	}
	i.valid <- which(!is.na(m.methylations[2, ]))
	if (length(i.valid) == 0) {
		return(NULL)
	}
	m.methylations <- m.methylations[, i.valid, drop=FALSE]
	yrange <- range(m.methylations[1, ] - m.methylations[2, ], m.methylations[1, ] + m.methylations[2, ])
	m.methylations <- cbind(dframe[i.valid, ], data.frame(
			"Identifier" = samples(rnb.set)[i.valid],
			"Average" = m.methylations[1, ],
			"Deviation" = m.methylations[2, ]))
	m.methylations <- m.methylations[with(m.methylations, order(Slide, Position)), ]
	slides <- tapply(1:nrow(m.methylations), m.methylations[, "Slide"], identity)

	report.plots <- list()
	for (si in 1:length(slides)) {
		fname <- ifelse(length(slides) == 1, fprefix, paste(fprefix, si, sep = "_"))
		rplot <- createReportPlot(fname = fname, ...)
		print(point.whisker.ggplot(m.methylations[slides[[si]], ], yrange))
		report.plots[[names(slides)[si]]] <- off(rplot)
	}
	if (length(report.plots) == 1) {
		return(report.plots[[1]])
	}
	return(report.plots)
}

#######################################################################################################################

get.unified.scale<-function(intensities){

	ir<-range(as.numeric(intensities), na.rm=TRUE)

	return(c(0,ir[2]+1000))

}
#######################################################################################################################
HM27.snp.betas<-function(qc.int){
	cmd<-rnb.get.annotation("controls27")
	cmd.snp<-cmd[grep("Genotyping", cmd$Type),]
	snp.ids<-cmd.snp[["Address"]]
	cy3.snp<-qc.int[["Cy3"]][rownames(qc.int[["Cy3"]]) %in% snp.ids,]
	cy5.snp<-qc.int[["Cy5"]][rownames(qc.int[["Cy5"]]) %in% snp.ids,]

	rownames(cy3.snp)<-cmd.snp$Name
	rownames(cy5.snp)<-cmd.snp$Name

	meth.cy3<-cy3.snp[paste(HM27.CY3.SNP.PROBES, "B", sep="_"),]
	umeth.cy3<-cy3.snp[paste(HM27.CY3.SNP.PROBES, "A", sep="_"),]

	meth.cy5<-cy5.snp[paste(HM27.CY5.SNP.PROBES, "B", sep="_"),]
	umeth.cy5<-cy5.snp[paste(HM27.CY5.SNP.PROBES, "A", sep="_"),]

	snp.betas.cy3<-meth.cy3/(meth.cy3+umeth.cy3+1)
	snp.betas.cy5<-meth.cy5/(meth.cy5+umeth.cy5+1)

	snp.betas<-rbind(snp.betas.cy3, snp.betas.cy5)
	rownames(snp.betas)<-gsub("_B", "", rownames(snp.betas))
	return(snp.betas)
}
#######################################################################################################################
rnb.update.controlsEPIC.enrich <- function(control.probe.infos) {
	
	## Control probe colors associated with the evaluation of the Red channel
	CONTROL.COLORS.GREEN <- c("Black", "Blue", "Cyan", "Green", "Lime", "Limegreen", "Skyblue")
	
	## Control probe colors associated with the evaluation of the Red channel
	CONTROL.COLORS.RED <- c("Gold", "Orange", "Purple", "Red", "Tomato", "Yellow")
	
	## Add columns Evaluate Green and Evaluate Red
	control.probe.infos[["Evaluate Green"]] <- "-"
	control.probe.infos[["Evaluate Red"]] <- "-"
	i <- grep("^DNP", control.probe.infos[, "Description"])
	control.probe.infos[i, "Evaluate Green"] <- "-"
	control.probe.infos[i, "Evaluate Red"] <- "+"
	i <- grep("^Biotin", control.probe.infos[, "Description"])
	control.probe.infos[i, "Evaluate Green"] <- "+"
	control.probe.infos[i, "Evaluate Red"] <- "-"
	i <- (control.probe.infos[["Color"]] %in% CONTROL.COLORS.GREEN)
	control.probe.infos[i, "Evaluate Green"] <- "+"
	control.probe.infos[i, "Evaluate Red"] <- "-"
	i <- (control.probe.infos[["Color"]] %in% CONTROL.COLORS.RED)
	control.probe.infos[i, "Evaluate Green"] <- "-"
	control.probe.infos[i, "Evaluate Red"] <- "+"
	i <- grep("^NEGATIVE", control.probe.infos[, "Target"])
	control.probe.infos[i, "Evaluate Green"] <- "+"
	control.probe.infos[i, "Evaluate Red"] <- "+"
	control.probe.infos[["Evaluate Green"]] <- factor(control.probe.infos[["Evaluate Green"]], levels = c("-", "+"))
	control.probe.infos[["Evaluate Red"]] <- factor(control.probe.infos[["Evaluate Red"]], levels = c("-", "+"))
	
	## Add column Expected Intensity
	control.probe.infos[["Expected Intensity"]] <- as.character(NA)
	i <- control.probe.infos[, "Target"] %in% c("NEGATIVE", "TARGET REMOVAL", "RESTORATION")
	control.probe.infos[i, "Expected Intensity"] <- "Background"
	i <- c("BISULFITE CONVERSION II", "SPECIFICITY II", "EXTENSION", "NON-POLYMORPHIC")
	i <- control.probe.infos[, "Target"] %in% i
	control.probe.infos[i, "Expected Intensity"] <- "High"
	i <- control.probe.infos[, "Target"] %in% paste("NORM", c("A", "C", "G", "T"), sep = "_")
	control.probe.infos[i, "Expected Intensity"] <- "High"
	i <- grep("\\((High)|(20K)\\)$", control.probe.infos[, "Description"])
	control.probe.infos[i, "Expected Intensity"] <- "High"
	i <- grep("\\((Medium)|(5K)\\)$", control.probe.infos[, "Description"])
	control.probe.infos[i, "Expected Intensity"] <- "Medium"
	i <- grep("\\(Low\\)$", control.probe.infos[, "Description"])
	control.probe.infos[i, "Expected Intensity"] <- "Low"

	#i <- grep("\\((Bkg)|(5K)\\)$", control.probe.infos[, "Description"])
	i <- grep("\\(Bkg\\)$", control.probe.infos[, "Description"])
	control.probe.infos[i, "Expected Intensity"] <- "Background"
	i <- grep("^BS Conversion I[- ]C", control.probe.infos[, "Description"])
	control.probe.infos[i, "Expected Intensity"] <- "High"
	i <- grep("^BS Conversion I[- ]U", control.probe.infos[, "Description"])
	control.probe.infos[i, "Expected Intensity"] <- "Background"
	i <- grep("^GT Mismatch.+\\(PM\\)$", control.probe.infos[, "Description"])
	control.probe.infos[i, "Expected Intensity"] <- "High"
	i <- grep("^GT Mismatch.+\\(MM\\)$", control.probe.infos[, "Description"])
	control.probe.infos[i, "Expected Intensity"] <- "Background"
	control.probe.infos[["Expected Intensity"]] <- factor(control.probe.infos[["Expected Intensity"]])
	
	## Add column Sample-dependent
	control.probe.infos[["Sample-dependent"]] <-
			!(control.probe.infos[["Target"]] %in% CONTROL.TARGETS.SAMPLE.INDEPENDENT)
	
	control.probe.infos[["Index"]][order(control.probe.infos$Target)]<-unlist(sapply(sort(unique(control.probe.infos$Target)), function(target) 1:length(which(control.probe.infos$Target==target))))
	
	return(control.probe.infos)
}

%\VignetteIndexEntry{Comprehensive DNA Methylation Analysis with RnBeads}
%\VignetteKeywords{analysis, methylation, RnBeads}

\documentclass[a4paper]{scrartcl}
\usepackage[OT1]{fontenc}
\usepackage{Sweave}
\usepackage{hyperref}
\usepackage{color}
\usepackage{vmargin}
\usepackage[american]{babel}
\usepackage{fancyhdr}
\usepackage{listings}
\usepackage{amsmath}

\selectlanguage{american}
\definecolor{darkblue}{rgb}{0.0,0.0,0.3}
\hypersetup{colorlinks,breaklinks,
            linkcolor=darkblue,urlcolor=darkblue,
            anchorcolor=darkblue,citecolor=darkblue}
\setpapersize{A4}
\setmargins{2.5cm}{2.0cm}% % left margin and upper margin
           {16cm}{23cm}%   % text width and height
           {60pt}{22pt}%   % Header height and margin
           {0pt}{30pt}%    % \footheight (egal) und Fusszeilenabstand

%%%%%%%%%%%%%%%%% Some style sttings %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\newcommand{\insertemptypage}[0]{\newpage \thispagestyle{empty} \vspace*{\fill}}

% Layout
\newcommand{\emphr}[1]{\textcolor{red}{\sffamily{\textbf{#1}}}}
\newcommand{\itemr}[1]{\item \emphr{#1} \\}
\newcommand{\ns}[1]{{\sffamily #1}}  % non-serif font
\newcommand{\nsb}[1]{\ns{\textbf{#1}}}
\newcommand{\nsbc}[1]{\nsb{\textcolor{blue}{#1}}}
\newcommand{\nsbcr}[1]{\nsb{\textcolor{red}{#1}}}
\newcommand{\cs}[1]{\texttt{#1}}    % code style

\newcommand{\vocab}[1]{\textit{\ns{#1}}}
\definecolor{grey}{rgb}{0.6,0.6,0.6}
% Ueberschriften
\renewcommand*{\sectfont }{\color{blue} \sffamily \bfseries}
%\renewcommand{\thesection}{\Alph{section}}

%neuer absatz
\newcommand{\newpara}[0]{\vskip 10pt}

%simple hyperrefs without having to type the link double
\newcommand{\hrefs}[1]{\href{#1}{#1}}

% Mathematik-Modus
% Mengen
\newcommand{\setN}[0]{\ensuremath{\mathds{N}}}
\newcommand{\setR}[0]{\ensuremath{\mathds{R}}}
\newcommand{\setZ}[0]{\ensuremath{\mathds{Z}}}
\newcommand{\set}[1]{\left\{#1\right\}}
%\newcommand{\tupt}[1]{\ensuremath{\langle}#1\ensuremath{\rangle}}

% Statistics
\newcommand{\pr}[0]{\ensuremath{\text{Pr}}}
\newcommand{\E}[0]{\ensuremath{\text{E}}}
\newcommand{\var}[0]{\ensuremath{\text{Var}}}
\newcommand{\cov}[0]{\ensuremath{\text{Cov}}}
\newcommand{\sd}[0]{\ensuremath{\text{sd}}}
% Statistical Independence
\newcommand\independent{\protect\mathpalette{\protect\independenT}{\perp}}
\def\independenT#1#2{\mathrel{\setbox0\hbox{$#1#2$}%
\copy0\kern-\wd0\mkern4mu\box0}}

% Unsicher
\newcommand{\chk}[0]{\textcolor{red}{(\textbf{???)}}}
\newcommand{\chktxth}[1]{\textcolor{red}{(\textbf{???} #1 \textbf{???})}}
\newcommand{\chkside}[0]{\marginpar{\textcolor{red}{(\textbf{???)}}}}
\newcommand{\chktxt}[1]{\marginpar{\textcolor{red}{(\textbf{???} #1 \textbf{???})}}}

\newcommand{\todo}[1]{\marginpar{\textcolor{red}{\textbf{TODO:}\newline #1}}}
\newcommand{\todoh}[1]{\textcolor{red}{\textbf{TODO:} #1}}
\newcommand{\todomark}[0]{\marginpar{\textcolor{red}{\textbf{TODO}}}}
\newcommand{\chkm}[0]{\textcolor{red}{(\textbf{???)}} \marginpar{\textcolor{red}{\textbf{TODO}}}}
\newcommand{\ph}[1]{\paragraph{} \textcolor{red}{[ #1 ]}} %placeholder

\newcommand{\ra}[0]{$\rightarrow$}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\lhead{\cs{RnBeads}}
\rhead{\thepage}
\cfoot{}

\pagestyle{fancy}

\begin{document}
\SweaveOpts{concordance=TRUE}

\title{\cs{RnBeads} -- Comprehensive Analysis of DNA Methylation Data}

\author{Fabian M\"uller, Yassen Assenov, Pavlo Lutsik, Michael Scherer \\
	\small{Contact: \cs{team@rnbeads.org}} \\
	\small{Package version: \cs{2.10.0}}}

\thispagestyle{empty}
\maketitle

\cs{RnBeads} is an R package for the comprehensive analysis of genome-wide DNA methylation data with single basepair resolution. Supported assays include the Infinium 450k microarray, whole genome bisulfite sequencing (WGBS), reduced representation bisulfite sequencing (RRBS), other forms of enrichment bisulfite sequencing, and any other large-scale method that can provide DNA methylation data at single basepair resolution (e.g. MeDIP-seq after suitable preprocessing). It builds upon a significant and ongoing community effort to devise effective algorithms for dealing with large-scale DNA methylation data and implements these methods in a user-friendly, high-throughput workflow, presenting the analysis results in a highly interpretable fashion.

<<echo=FALSE,warning=FALSE>>=
suppressPackageStartupMessages(library(RnBeads))
#for knitr: avoid tidy because it messes up line breaks
# opts_chunk$set(tidy=FALSE)
@

\tableofcontents

\section{Getting Started}

\subsection{Installation}
	Automatic installation of \cs{RnBeads} and one or more of its companion data packages is performed just like any other Bioconductor package. As an example, the following commands install RnBeads and its annotation package for the human genome annotation hg38:
<<eval=FALSE>>=
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
BiocManager::install(c("RnBeads", "RnBeads.hg38"))
@
	
	Alternatively, we also provide a an easy-to-use installation script that ensures all dependencies are installed. This way, you can install RnBeads by running a single line of code in your R environment:
<<eval=FALSE>>=
source("http://rnbeads.org/data/install.R")
@

	Note that you might have to install \textit{Ghostscript} which \cs{RnBeads} uses for generating plot files. \textit{Ghostscript} is included in most Unix/Linux distributions by default, but may require installation on other operating systems. You can obtain the version corresponding to your operating system freely from the program's website\footnote{\href{http://www.ghostscript.com/download/gsdnld.html}{http://www.ghostscript.com/download/gsdnld.html}}. Follow their installation instructions. You might have to set path variables as well. Additional hints on installation of Ghostscript can be found on the FAQ section of the \href{http://rnbeads.mpi-inf.mpg.de/}{RnBeads website}\footnote{\href{https://rnbeads.org/}{https://rnbeads.org/}}. The website also describes the installation process in detail.

\subsection{Overview of \cs{RnBeads} Modules}
	\cs{RnBeads} implements a comprehensive analysis pipeline from data import via
	filtering, normalization and exploratory analyses to characterizing
	differential methylation. Its modularized design allows for conducting
	simple vanilla types of analysis (with the need to specify only a few
	parameters; \cs{RnBeads} automatically takes care of the rest) as well as
	targeted, highly customizable analysis. Multiple input formats are supported.
	Moreover, advanced code logging capabilities are integrated into the package.
	Finally, the output can be presented as comprehensive, interpretable analysis reports in \cs{html} format. Figure~\ref{fig:modules} visualizes the standard workflow through \cs{RnBeads} modules. Analysis modules include:
	\begin{figure}
		\noindent \begin{centering}
			\includegraphics[width=0.99\textwidth]{"figures/rnbeads_workflow"}
		\par \end{centering}
		\caption{Workflow and modules in RnBeads}
		\label{fig:modules}
	\end{figure}

	\begin{description}
		\item[Data Import] Various input formats are supported both for Infinium and bisulfite sequencing data.
        \item[Quality Control] If quality control data is available from the input
		(e.g. in \cs{IDAT} format files for Infinium data), QC based on the various
		types of QC probes on the Infinium methylation chip is performed. For
		bisulfite sequencing experiments, QC analysis is based on coverage
		information.
		\item[Preprocessing] Includes probe and sample filtering. This happens in an automated fashion after the user specifies filtering criteria in which he is interested. Furthermore, for Infinium 450k data a number of microarray
		normalization methods are implemented. The normalization report helps to assess the effects of the normalization procedure.
		\item[Tracks and Tables] Methylation profiles can be exported to bed, bigBed and bigWig file formats and visualized in various genome browsers via the export to track hubs.
		\item[Covariate Inference] In this optional module, sample covariates potentially confounding the analysis can be identified and quantified. In consecutive modules, e.g. they can be taken into account and corrected for during differential methylation analysis.
		\item[Exploratory Analysis] Dimension reduction, statistical association tests and visualization techniques are applied in order to discover associations of the methylation data with various sample characteristics specified in the sample annotation sheet provided by the user. Methylation fingerprints of samples and sample groups are identified and presented. Unsupervised learning techniques (clustering) are applied.
		\item[Differential Methylation Analysis] Provided with categorical sample, differentially methylated sites and genomic
		regions are identified, their degree of differential methylation is quantified
		and visualized. Furthermore, differentially methylated regions (DMRs) can be annotated, e.g. with enrichment analysis.
	\end{description}

\subsection{Datasets}
	In this vignette, we first describe how to analyze data generated by the Infinium 450k methylation array. Section~\ref{sec:bisulfite} describes how to handle bisulfite sequencing data. The general concepts of \cs{RnBeads} are shared for array and sequencing data. In fact, for the most part, the same methods and functions are used. Therefore, the 450k examples will also be very instructive for the analysis of bisulfite sequencing data.
	We will introduce basic features of \cs{RnBeads} using a dataset of Infinium 450k methylation data from multiple human embryonic stem cells (hESCs) and induced pluripotent stem cells (hiPSCs) lines. This dataset has been published as a supplement to a study on non-CpG methylation \cite{Ziller2011} and can be downloaded as a zip file from the \cs{RnBeads} examples website\footnote{\href{https://rnbeads.org/examples.html}{https://rnbeads.org/examples.html}}. Note that while the article~\cite{Ziller2011} focuses on non-CpG methylation, we will only investigate methylation in the CpG context here. Before working with the data, unzip the archive to a directory of your choice.

\subsection{Setting Up the Analysis Environment}
	Before we get started, we need to carry out some preparations, such as loading the package and specifying input and output directories:

<<>>=
library(RnBeads)
# Directory where your data is located
data.dir <- "~/RnBeads/data/Ziller2011_PLoSGen_450K"
idat.dir <- file.path(data.dir, "idat")
sample.annotation <- file.path(data.dir, "sample_annotation.csv")
# Directory where the output should be written to
analysis.dir <- "~/RnBeads/analysis"
# Directory where the report files should be written to
report.dir <- file.path(analysis.dir, "reports")
@
We will use these variables throughout this vignette whenever we mean to specify input and output files. Of course, you have to adapt the directories corresponding to your own file system and operating system. The zip file you downloaded contains a directory that entails a sample annotation sheet (we store its location in the \cs{sample.annotation} variable in the code above) and a directory containing the IDAT files that hold the methylation data (\cs{idat.dir}).


\section{Vanilla Analysis}
\label{sec:vanilla}

	So, let's get our hands dirty and perform an analysis. First we may want to specify a few analysis parameters:
<<>>=
rnb.options(filtering.sex.chromosomes.removal=TRUE,
    identifiers.column="Sample_ID")
@
	This tells \cs{RnBeads} to remove all probes on sex chromosomes for the analysis in the filtering step. Furthermore, \cs{RnBeads} will look for a column called ``Sample\_Name'' in the sample annotation sheet and use its contents as identifiers in the analysis. Feel free to set other options as well (you can inspect available analysis options using \cs{?rnb.options}). More details on analysis options can be found in Section~\ref{sec:anaParams} of this vignette.

    The main function in RnBeads is \cs{rnb.run.analysis()}. It executes all
    analysis modules, as specified in the package options and generates a
    dedicated report for each module. The method has multiple
    arguments, the most important of which are those specifying the type of the
    data to be analyzed, the location of the methylation data and sample
    annotation, and the location of the output directory. In this example we are
    dealing with data from the Infinium 450k microarray and thus, for a simple
    analysis run we execute \cs{rnb.run.analysis} using four arguments.
    Specifically, \cs{data.dir} defines the location of the input IDAT
    intensity files. A sample annotation sheet is essential for every analysis.
    It is a tabular file containing sample information such as sample
    identifiers, file locations, phenotypic information, batch processing
    information and potential confounding factors. Feel free to inspect the
    sample annotation sheet provided in the dataset you just downloaded to get
    an impression on how such annotation can be structured.
    \cs{rnb.run.analysis()} finds the location of the annotation table using
    the \cs{sample.sheet} parameter. \cs{report.dir} specifies the location of
    the output directory. Make sure that this directory is non-existing, as
    otherwise \cs{RnBeads} will not be able to create reports there. Finally,
    we have to specify the type of the input data which should be
    \cs{"idat.dir"} for processing IDAT files. Now we can run our analysis with
    our defined options:

<<eval=FALSE>>=
rnb.run.analysis(dir.reports=report.dir, sample.sheet=sample.annotation,
    data.dir=idat.dir, data.type="infinium.idat.dir")
@
	Note that in the above code we specified the arguments explicitly, i.e. we used the syntax \cs{argument.name=argument.value}. It is good practice to stick to this explicit argument statement.

	Now, as running through the entire analysis pipeline takes a while, get a cup of coffee. You might also want to browse through some of the examples on the \cs{RnBeads} website to get an idea of what kind of results you can expect once the analysis is done. Anyhow, it might take a while (on our testing machine, 	roughly four hours for the complete analysis).

	After the analysis has finished, you can find a variety of files in the report
	directory you specified. Of particular interest are the \cs{html} files containing the
	analysis results and links to tables and figures which are stored in the output directory structure.
	Furthermore, \cs{analysis.log} is a log file containing status
	information on your analysis. Here, you can check for potential errors and warnings that occurred
	in the execution of the pipeline.
%	You can also specify the log destination \underline{before} you run the analysis. The following command redirects the logging to the console only:
% <<eval=FALSE>>=
% logger.start(fname = NA)
% @

	Now, feel free to inspect the actual results of our analysis. All of them are
	stored as \cs{html} report files. A good starting point is the \cs{index.html}
	in your reports directory which can be opened in your favorite web browser.
	This overview page provides the links to the analysis reports of the
	individual modules. You can also access the individual reports directly by
	opening the corresponding \cs{html} files in the reports directory. These
	reports contain method descriptions of the conducted analyses as well as the
	results. The reports are designed to be self-contained and self-explanatory,
	therefore, we do not discuss them in detail here. Take your time browsing
	through the results of the analysis and getting an idea of what \cs{RnBeads}
	can do for you.

	The reports provide a convenient way of sharing results and
	re-inspecting them later. It is particularly easy to share results over the
	Internet if you have webspace available somewhere. Simply upload the entire
	report directory to a server and send a corresponding link to your
	collaborators. If you compress the the analysis report directories you can also share them via
	cloud services such as Google Drive and Dropbox.
	You can find more example reports on the \cs{RnBeads} website.

	In your subsequent analysis runs you might not want to execute the
	full-scale \cs{RnBeads} pipeline. You can deactivate individual modules by
	setting the corresponding global \cs{RnBeads} options before running the
	analysis. For instance, you could deactivate the differential methylation module:
<<>>=
rnb.options(differential=FALSE)
@

\section{Working with \cs{RnBSet} Objects}

	Every analysis workflow in \cs{RnBeads} is centered around a dataset object, which is implemented as an R object of type \cs{RnBSet} (an \cs{S4} class). This object contains the sample annotation table, methylation values for individual sites and regions, as well as additional, platform-specific data. The function \cs{rnb.run.analysis}, introduced in the previous section, returns such an object (invisibly) which contains the processed dataset. \cs{RnBeads} provides a multitude of functions operating on a dataset; some of these functions retrieve information from the dataset, others create plots summarizing some of the data contained, or modify the methylation values or sample annotation. In this section, we introduce the \cs{RnBSet} class and show a few examples on how to work with these objects in an interactive R session. The following sections give more details by describing each of the \cs{RnBeads} modules. With the exception of the import module, which prepares such objects from input data files, all of these modules operate directly on \cs{RnBSet} objects.

	\cs{RnBSet} objects come in different flavors (subclasses), depending on the data type that was used to create them: \cs{RnBeadSet} and \cs{RnBeadRawSet} objects are derived from processed and unprocessed array data respectively, while \cs{RnBiseqSet} objects result from processing bisulfite-sequencing data.

	Here, we use a small example dataset which is contained in the \cs{RnBeads.hg19} package and comprises a subset of probes of the Infnium 450K dataset introduced earlier in this vignette.
	Readers are encouraged to try the functions presented here on the full dataset or on their own datasets. As a first example, by printing a dataset you can see a summary of its type and size.

<<>>=
library(RnBeads.hg19)
data(small.example.object)
rnb.set.example
@

	As shown in the code snippet above, the example dataset consists of 12 samples $\times$ 1,736 probes. In addition to measurements at individual CpGs, it has summarized the methylation state for 470 genomic tiling regions, 184 gene bodies, 184 gene promoters and 137 CpG islands. Notice the first line of printed for this dataset, it states that it is an object of type \cs{RnBeadRawSet}. These objects denote array-based datasets containing array-specific measurements, such as probe intensity values, detection p-values, and others.

	You can use the following functions in order to obtain the number of CpGs covered and the sample identifiers of the dataset:

<<>>=
nsites(rnb.set.example)
samples(rnb.set.example)
@
	The following command shows the the first 4 columns of the sample annotation table:

<<>>=
pheno(rnb.set.example)[, 1:4]
@

Methylation values for individual CpGs can be obtained using the \cs{meth} command:

<<>>=
mm <- meth(rnb.set.example)
@

Let's take look at the distribution of methylation values in sample number 5 of our dataset:

<<fig.height=3>>=
hist(mm[,5], col="steelblue", breaks=50)
@

\cs{RnBSet} objects also contain summarized methylation levels for various region types. To inspect which region types are represented in an object use \cs{summarized.regions}:

<<>>=
summarized.regions(rnb.set.example)
@

The next commands instruct \cs{RnBeads} that the first column in the annotation table denotes sample identifiers, and extract the methylation beta values at the first five gene promoters for the first three samples:

<<>>=
rnb.options(identifiers.column="Sample_ID")
meth(rnb.set.example, type="promoters", row.names=TRUE, i=1:5, j=1:3)
@

Similarly, the method \cs{mval} is used to extract M values:

<<>>=
mval(rnb.set.example, type="promoters", row.names=TRUE)[1:5, 1:3]
@

To get the coordinates and additional annotation for sites or regions contained in the dataset, use the \cs{annotation} function:

<<>>=
annot.sites <- annotation(rnb.set.example)
head(annot.sites)
annot.promoters <- annotation(rnb.set.example, type="promoters")
head(annot.promoters)
@

Furthermore, indices of overlapping regions can be added to each annotated
methylation site.

<<>>=
annot.sites.with.rgns <- annotation(rnb.set.example, include.regions=TRUE)
head(annot.sites.with.rgns)
@

You can manipulate \cs{RnBSet} objects using several methods. Here, we show you
how to remove samples and sites from the dataset and how to add columns to the sample annotation table:

<<>>=
# Remove samples
rnb.set2 <- remove.samples(rnb.set.example, samples(rnb.set.example)[c(2, 6, 10)])
setdiff(samples(rnb.set.example), samples(rnb.set2))

# Remove probes which are not in CpG context
notCpG <- annot.sites[,"Context"]!="CG"
rnb.set.example <- remove.sites(rnb.set.example, notCpG)
nsites(rnb.set.example)

# Add a sample annotation column indicating whether the given
# sample represents an iPS cell line
is.hiPSC <- pheno(rnb.set.example)[, "Sample_Group"]=="hiPSC"
rnb.set.example <- addPheno(rnb.set.example, is.hiPSC, "is_hiPSC")
pheno(rnb.set.example)[, c("Sample_ID", "Cell_Line", "is_hiPSC")]
@

Since the example object was derived from an Infinium 450k experiment it also
stores low-level information about the corresponding experiment, such as 
measured intensities in each of the color channels, out-of-band intensities,
number of beads, detection \textit{p}-values etc:

<<>>=
# Methylated ...
Mint <- M(rnb.set.example, row.names=TRUE)
Mint[1:5,1:3]
# ...and unmethylated probe intensities
Uint <- U(rnb.set.example, row.names=TRUE)
Uint[1:5,1:3]
# Infinium bead counts
nbead <- covg(rnb.set.example, row.names=TRUE)
nbead[1:5,1:3]
# detection P-values
pvals <- dpval(rnb.set.example, row.names=TRUE)
pvals[1:5,1:3]
@

Moreover, the object also contains quality control information. RnBeads offers a
number of diagnostic plots visualizing this information:

<<fig.height=5>>=
# boxplot of control probes
rnb.plot.control.boxplot(rnb.set.example)
# barplot of a selected control probe
control.meta.data <- rnb.get.annotation("controls450")
ctrl.probe<-paste0(unique(control.meta.data[["Target"]])[4], ".1")
rnb.plot.control.barplot(rnb.set.example, ctrl.probe)
@

Finally, the full control probe intensities can be retrieved using method
\cs{qc}:

<<>>=
qc_data<-qc(rnb.set.example)
@

\cs{qc\_data} is a list with elements \cs{Cy3} and \cs{Cy5} containing
all Infinium control probe intensities for the green and red color channels
respectively. It can be used for more advanced QC procedures.


\section{Tailored Analysis: \cs{RnBeads} Modules}

	In this section, we show how the package's modules can be invoked individually given
 	suitable options and input data. Most modules operate on the \cs{RnBSet} objects introduced in the previous section. Some modules depend on data generated by other modules (c.f. Figure~\ref{fig:modules}). We also introduce useful
	parameter settings and report-independent stand-alone methods for methylation
	analysis.

	Note that the examples in this section operate on the complete dataset and therefore potentially take longer to execute than the operations that were executed on the reduced dataset introduced in the previous section.

	To get started, let us initialize a new report directory prior to our
	analysis. Keep in mind that \cs{RnBeads} does not overwrite existing reports
	and thus the analysis fails when a report's HTML file or subdirectory already exists.
<<eval=FALSE>>=
report.dir <- file.path(analysis.dir, "reports_details")
rnb.initialize.reports(report.dir)
@
	Additionally, as we do not need to store the logging messages in a file and just want
	them printed, we restrict logging to the console:
<<eval=FALSE>>=
logger.start(fname=NA)
@

\subsection{Data Import}
\label{sec:input}

%\subsubsection{Infinium 450k}
	RnBeads supports multiple input formats for microarray-based and bisulfite
	sequencing data. The following paragraphs focus on Infinium 450k methylation
	data sets. Section~\ref{sec:bisulfite} describes in details how bisulfite
	sequencing data can be loaded.

	Here, we use a more standardized way to specify the input location compared to
	the one in Section~\ref{sec:vanilla}: the \cs{data.source} argument. Its
 	value is dependent on the nature of the input data.
 	Table~\ref{tab:data-formats} lists all supported data types together with the
 	recommended format for the \cs{data.source} argument. %TODO:@Pavlo: is this table up to date?
 	Since the input data in
 	our example consists of IDAT files, \cs{data.source} should be a vector of
 	two elements, containing (1) the directory the IDAT files are stored in and (2) the file containing the sample annotation table.

<<eval=FALSE>>=
data.source <- c(idat.dir, sample.annotation)
@

	We can load the dataset into an \cs{RnBeadSet} object:

<<eval=FALSE>>=
result <- rnb.run.import(data.source=data.source,
    data.type="infinium.idat.dir", dir.reports=report.dir)
rnb.set <- result$rnb.set
@
	The new variable \cs{result} is a list with two elements: the dataset
	(\cs{rnb.set}) and a report object (see Section~\ref{sec:htmlReports}). Every
	\cs{rnb.run.*()} method creates a report. The generated import report
	(\cs{import.html}) contains information on the loaded dataset including data
	type, number of loaded samples and sites, description of the parsed annotation
	table etc. If you wish to parse a methylation dataset into an \cs{RnBSet} object without producing an analysis report, you can use \cs{rnb.execute.import}:
<<eval=FALSE>>=
rnb.set <- rnb.execute.import(data.source=data.source,
	data.type="infinium.idat.dir")
@

	\cs{rnb.set} is an object of class \cs{RnBeadRawSet}. This S4 class inherits
	from class \cs{RnBSet} which is the main data container class in \cs{RnBeads}
	and serves as input for many methods of the analysis pipeline.
	Just like other \cs{RnBSet} objects, the \cs{RnBeadRawSet} class contains slots for sample annotation (\cs{pheno}) and methylation
	values of sites and predefined genomic regions (accessible by the \cs{meth} function). In addition, it contains raw microarray intensities (\cs{M} and \cs{U} functions), detection p-values (\cs{dpval}) and quality control information (\cs{qc}):
<<eval=FALSE>>=
rnb.set
summary(pheno(rnb.set))
dim(M(rnb.set))
summary(M(rnb.set))
dim(meth(rnb.set))
summary(meth(rnb.set))
dim(meth(rnb.set, type="promoters"))
summary(meth(rnb.set, type="promoters"))
summary(dpval(rnb.set))
str(qc(rnb.set))
@


	\begin{table}[h!]
	\label{tab:data-formats}
	\caption{\cs{RnBeads} input data types.}
	\vspace{10px}
	\begin{minipage}[h!]{\textwidth}
	\begin{tabular}{ | l | p{11cm} |}
		\hline
		\textbf{\cs{data.type}} & \textbf{\cs{data.source} recommended
		format\footnote{Alternative formats of the \cs{data.source} argument are possible; please refer to the R documentation of the \cs{rnb.execute.import} function}} \\
		\hline
		\hline
		\multicolumn{2}{|c|}{\textit{Infinium 450k data}} \\
		\hline
		\cs{infinium.idat.dir} & A character vector of length 1 or 2 containing the
		directory in which the IDAT files are stored and (optionally) the filename of the sample annotation table (if the latter is located in a different directory) \\
		\hline
		\cs{infinium.GS.report} & A character vector of length 1 containing the
		filename of a Genome Studio report \\
		\hline
		\cs{infinium.GEO} & A character vector of length 1 containing the GEO
		accession number
		\\
		\hline
		\cs{infinium.data.dir} & A character vector of length 1 giving the name of the
		target directory. The filenames in the directory should contain key words: \cs{sample} for the sample information table, \cs{beta} for the table with beta-values, \cs{pval} for p-values table, and \cs{beads} for bead counts \\
		\hline
		\cs{infinium.data.files}& A character vector of length 2 to 4. At minimum
		paths to the sample annotation table and to a table containing beta-values should be given. Additionally, paths to a table with detection p-values, and bead counts can be specified \\
		\hline
		\multicolumn{2}{|c|}{\textit{Bisulfite Sequencing data}} \\
		\hline
		\cs{bs.bed.dir} & A character vector of length 3. It contains [1] the
		directory where the methylation bed files can be found, [2] the filename of the sample annotation table, and optionally [3] the index of the sample annotation file column, which contains the file names\\
		\hline
	\end{tabular}
	\end{minipage}
	\end{table}
	\vspace{10px}

	Notably, \cs{RnBeads} accepts GEO accession numbers as input. Here is an example of a somewhat larger dataset, from another study on pluripotent and differentiated cells \cite{Nazor2012}:
<<eval=FALSE>>=
rnb.set.geo <- rnb.execute.import(data.source="GSE31848",
	data.type="infinium.GEO")
@

	As the dataset is fairly large, downloading and import might take a while.
	Note that data sets loaded from GEO typically do not include detection
	p-values and QC probe information. Therefore, most of the normalization
	methods and some of the subsequent reports and analyses cannot be carried out.
	Examples include parts of the \emph{Quality Control} module or probe filtering based on detection p-values as performed by the \emph{Preprocessing} module.



\subsection{Quality Control}

%\subsubsection{Infinium 450k}

	\cs{RnBeads} generates quality control plots both for Infinium 450k and bisulfite sequencing data. Experimental quality control for the Infinium 450k data is performed using the microarray control probe information which should be present in the input data. We recommend starting out from the IDAT files which contain this information. Table~\ref{tab:hm450-controls} gives short description of each control probe type represented on the array. The following paragraphs describe some QC plots targeting Infinium 450k datasets only. For the quality control of bisulfite sequencing data, sequencing coverage is taken into account and its distribution is visualized in a series of plots. For more details see Section~\ref{sec:bisulfite}.

	\begin{table}[h!]
	\label{tab:hm450-controls}
	\caption{Types of Infinium 450k control probes.}
	\vspace{10px}
	\begin{tabular}{| l | p{10cm} |}
		\hline
		\textbf{Control type}& \textbf{Purpose} \\
		\hline
		\hline
		\multicolumn{2}{|c|}{\textit{Sample-independent controls}}\\
		\hline
		\cs{STAINING} & Monitor the efficiency of the staining step. These controls are independent of the hybridization and extension step. \\
		\hline
		\cs{HYBRIDIZATION} 	& Monitor overall hybridization performance using synthetic targets as a reference. The targets are present in the hybridization buffer at three concentrations: low, medium and high, which should result in three well separable intensity intervals.\\
		\hline
		\cs{EXTENSION} & Monitor the extension efficiency of A, T, C, and G nucleotides from a hairpin probe \\
		\hline
		\cs{TARGET REMOVAL} & Monitor efficiency of the stripping step after the extension reaction \\
		\hline
		\multicolumn{2}{|c|}{\textit{Sample-dependent controls}}\\
		\hline
		\cs{BISULFITE CONVERSION I} & Monitor bisulfite conversion efficiency as reported by Infinium I design probes. Converted (C) and unconverted (U) probes are present \\
		\hline
		\cs{BISULFITE CONVERSION II} & Monitor bisulfite conversion efficiency as reported by the Infinium II design probes \\
		\hline
		\cs{SPECIFICITY I} & Monitor allele-specific extension for Infinium I probes. \cs{PM} probes should give high signal, while the \cs{MM} probes should give background signal levels \\
		\hline
		\cs{SPECIFICITY II} & Monitor allele-specific extension for Infinium II probes \\
		\hline
		\cs{NON-POLYMORPHIC} & Monitor the efficiency of all steps of the procedure by querying a non-polymorphic base in the genome -- one probe for each nucleotide \\
		\hline
		\cs{NEGATIVE} & Monitor signal at bisulfite-converted sequences that do not contain CpGs. Used to estimate the overall background of the signal in both channels. Moreover, the detection p-value for each probe are estimated based on the intensities of the negative control probes \\
		\hline
		\cs{NORM} & Normalization controls are used to calculate the scaling factor for the Illumina default normalization \\
		\hline
	\end{tabular}
	\end{table}


<<eval=FALSE>>=
rnb.run.qc(rnb.set, report.dir)
@
	is the command which generates the QC report (\cs{qc.html}). For Infinium data,
	this report contains three major quality control plots: \emph{control boxplot}, \emph{control barplot} and \emph{negative control boxplot} that are briefly explained below. %TODO:@Pavlo: the SNP plots are also standard, aren't they? Please add them here

	\emph{Control Boxplot} is a diagnostic plot showing the distribution of signal intensity for each of the quality control probes in both of the channels (Figure~\ref{fig:control-boxplot-bad-conv}). The expected intensity level (high, medium, low or background) is given in the plot labels. The boxplots provide a useful tool for detecting experimentally failed assays. Low quality samples, if any, will appear as outliers. When examining the boxplot pay attention to the intensity scale.

	\emph{Control Barplot} focuses on individual samples. In case problematic behavior has been spotted in the control boxplots, the low quality samples can be identified by inspecting the barplots. Here the intensity for each sample measured by each of the control probe types is shown (Figure~\ref{fig:control-barplot-bad-conv}). The problematic samples should be excluded from the analysis by modifying the supplied sample annotation sheet or \cs{RnBeadSet} object (see Section~\ref{sec:input}).

	\emph{Negative Control Boxplots} show distributions of intensities for the approximately 600 negative control probes which are present on the Infinium 450k array. The negative control probes are specifically designed for estimating the background signal level in both channels. In both channels the negative control probe intensities are expected to be normally distributed around a relatively low mean (as a simple rule, the 0.9 quantile should be below 1000). Any strong deviations from such a picture in one or more samples may indicate quality issues; discarding such samples could be beneficial for downstream analyses.

	For a loaded dataset stored in variable \cs{rnb.set}, one can generate quality control plots directly. For instance you can generate a boxplot specifying the type of quality control probe as the second argument:
<<eval=FALSE>>=
rnb.plot.control.boxplot(rnb.set, "BISULFITE CONVERSION I")
@

	To get a quality barplot, the second argument should be the unique identifier of the quality control probe:
<<eval=FALSE>>=
rnb.plot.control.barplot(rnb.set, "BISULFITE CONVERSION I.2")
@
	Negative control boxplots are generated with the following command:
<<eval=FALSE>>=
rnb.plot.negative.boxplot(rnb.set)
@

	The quality control report of the example dataset indicates high experimental
	quality. The quality boxplots for most of the controls show very compact
	intensity distributions. Let us, however, consider an example of quality
	control output where the experimental quality of the assay is suboptimal. In Figure~\ref{fig:control-boxplot-bad-conv} a boxplot of intensity measured by bisulfite conversion control probes is given for a data set of 17 samples. It is easy to spot the outliers which show quite high intensity in the green channel at probes for which background level intensities are expected (probes 4, 5 and 6). In order to identify problematic samples, let us examine the control barplot for one of the problematic probes \cs{BISULFITE COVERSION I.6} (Figure~\ref{fig:control-barplot-bad-conv}). Note that sample \cs{CellLineA\_1} has an intensity which is at least four times higher than that of any other sample. This might indicate problems with bisulfite conversion and the problematic sample should be discarded. Since the quality information-based filtering of the Infinium 450k samples is as a rule performed via the inspection by the wet-lab researcher, low-quality samples could be removed, for example, by manual editing of the sample annotation file and restarting the analysis on the high-quality samples only.

	\begin{figure}
		\noindent \begin{centering}
			\includegraphics[width=1.00\textwidth]{"figures/control_boxplot_bad_conversion"}
		\par \end{centering}
		\caption{An example quality control boxplot.}
		\label{fig:control-boxplot-bad-conv}
	\end{figure}
	\begin{figure}
		\noindent \begin{centering}
			\includegraphics[width=1.00\textwidth]{"figures/control_barplot_bad_conversion"}
		\par \end{centering}
		\caption{An example quality control barplot.}
		\label{fig:control-barplot-bad-conv}
	\end{figure}

	\paragraph{Sample mixups}

	The Infinium 450k BeadChip also contains a small number (65) of genotyping probes. Their dbSNP identifiers and additional information can be obtained using the following commands:

<<eval=FALSE>>=
annot450 <- rnb.annotation2data.frame(rnb.get.annotation("probes450"))
annot450[grep("rs", rownames(annot450)), ]
@

	Despite a certain level of technical variation, each SNP takes one of the three
	possible $\beta$-value levels: low (homozygous state, first allele), high
	(homozygous state, second allele) or intermediate (heterozygous). Examining
	these values can be very useful for the identification of sample mixups,
	especially in case of studies with genetically matched design. In this case,
	samples with the same genetic background should have very similar values for
	each of the 65 SNPs. Clustered heatmaps for the SNP-values are useful to
	obtain a global overview of genotype-related sample grouping and similarities and thus
	provide an efficient way of detecting sample-mixups.
	Figure~\ref{fig:snp-heatmap} gives an example of such a heatmap. Such heatmaps are included into the quality control report by default, but can also be individually generated with the following command:

<<eval=FALSE>>=
rnb.plot.snp.heatmap(rnb.set.unfiltered)
@

	\emph{SNP distance heatmap} is an alternative way to get an overview of the
	SNP probes. The heatmap is a diagonal matrix visualizing coherence between each of the samples with respect to their SNP probe patterns.

	\emph{SNP barplots} visualize the SNP-probe beta-values directly:
<<eval=FALSE>>=
rnb.plot.snp.barplot(rnb.set.unfiltered, samples(rnb.set)[1])
@
	Note that when executing \cs{rnb.run.preprocessing} in the earlier example, we removed the SNP probe information from the \cs{RnBSet} object.
	Hence, these plots have to be done based on the unfiltered object.

	\begin{figure}
		\label{fig:snp-heatmap}
		\noindent \begin{centering}
			\includegraphics[width=1.00\textwidth]{"figures/snp_heatmap"}
		\par \end{centering}

		\caption{SNP heatmap}
	\end{figure}

Finally, an optional \emph{SNP boxplot} gives a global picture of the SNP probes and are helpful for the cases in which all the analyzed samples have the same genetic background (e.g. samples of of the same cell line). To include the SNP boxplot into the quality control report use the corresponding option:
<<eval=FALSE>>=
rnb.options(qc.snp.boxplot=TRUE)
@

	\subsubsection{Sex Prediction}

	\begin{figure}
		\label{fig:increase-gen-infinium}
		\noindent \begin{centering}
			\includegraphics[width=0.6\textwidth]{"figures/increases_gender"}
		\par \end{centering}
		\caption{Mean signal increase in the X and Y chromosomes of 4587 female and 4633 male samples, represented by red and blue points, respectively. The presented dataset includes tumor and normal samples from 30 tumor types, downloaded from The Cancer Genome Atlas (TCGA). The solid black line shows the decision boundary in the classifier applied by RnBeads. Training a logistic regression model on the presented data produces an almost identical decision boundary.}
	\end{figure}

	\cs{RnBeads} includes a method for inferring the sex of methylation samples. It is accessible using the function \cs{rnb.execute.sex.prediction}. Currently, the classifier is applicable for Infinium 450k, EPIC and bisulfite sequencing datasets with either signal intensity or coverage information. Sex prediction is automatically executed when loading those datasets from IDAT/BED files; it can be disabled using the corresponding options, as shown below:

<<eval=FALSE>>=
rnb.options(import.sex.prediction = FALSE)
@

	The methods for inferring sexes from Infinium datsets are different from those for bisulfite sequencing data sets. For Infinium datasets, the assumption made in RnBeads is that the quantity (number) of X and Y chromosomes in a sample is reflected in the signal intensities on the array. The measurements used in the prediction are $MI_X$: \emph{mean increase in X} and $MI_Y$: \emph{mean increaase in Y}. These metrics are defined as $\overline{I}_{X} - \overline{I}_{a}$ and $\overline{I}_{Y} - \overline{I}_{a}$, where $\overline{I}_X$, $\overline{I}_Y$ and $\overline{I}_a$ denote the average probe signal intensity on the X chromosome, the Y chromosome and all autosomes, respectively. Probes known to be cross-reactive, as well as those overlapping with known SNPs, are ignored in the calculations presented here. \cs{RnBeads} uses these metrics in a logistic regression classifier to predict sex of a sample, $MI_Y > 3 MI_X + 3$ indicates male sex. Figure~\ref{fig:increase-gen-infinium} shows the \emph{mean increase} values for over 9,200 samples with known sex, as well as the decision boundary of the classifier applied in \cs{RnBeads}.\\
	For bisulfite sequencing (RRBS or WGBS) datasets, \cs{RnBeads} compares the sequencing coverages for the sex chromosomes to those of the autosomes. From these values, a logistic regression classifier is employed to predict class probabilities of each sample for the male and female category. Both the predicted probability and the predicted class are added to the \cs{RnBSet's} sample annotation. Coefficients for the logistic regression classifier were trained on a large dataset (c.f. Figure ~\ref{tab:gender-prediction-bs}) containing sex annotated samples. 
	
	\begin{table}[h!]
			\caption{Dataset used to train coefficients of the sex prediction logistic regression model for bisulfite sequencing datasets.}
			\vspace{10px}
		\begin{tabular}{|p{2.95cm}|p{2.7cm}|c|c|c|c|c|}
		\hline
		\textbf{Dataset} & \textbf{Source} & \textbf{Organism} & \textbf{Method} & \textbf{\#total} & \textbf{\#female} & \textbf{\#male}\\
		\hline
		\hline
BLUEPRINT (08/2016 release) & \href{http://www.blueprint-epigenome.eu/}{BLUEPRINT} & human & WGBS & 188 & 99 & 89\\
DEEP data (version 11/2016) & \href{http://www.deutsches-epigenom-programm.de/}{DEEP} & human & WGBS & 31 & 14 & 17\\
Kiel Cohort & IKMB Kiel & human & RRBS & 239 & 106 & 133\\
Ewing sarcoma tumor samples & GEO \href{https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE88826}{GSE88826} & human & RRBS & 96 & 53 & 42\\
Reizel GEO & GEO \href{https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE60012}{GSE60012} & mouse & RRBS & 152 & 94 & 58\\
\hline
 \textbf{Complete} &  &  &  & \textbf{706} & \textbf{366} & \textbf{339}\\
 \hline
		\end{tabular}
		\label{tab:gender-prediction-bs}
	\end{table}

	\subsubsection{CNV estimation}
	Copy number variation (CNV) can be estimated from signal intensity values
	of BeadArray data sets. RnBeads' CNV estimation module comes with two flavors:
	a reference-based and a reference-free approach. We recommend to use the 
	reference-free approach, since validation of the reference data set is still to be
	conducted. CNV estimation is only to be executed within the QC module and not 
	individually. It is activated by:
	
<<eval=FALSE>>=
rnb.options(qc.cnv=TRUE)
result <- run.run.qc(rnb.set.unfiltered,dir.reports=report.dir)
@

	The estimation is based on the \href{http://bioconductor.org/packages/release/bioc/html/GLAD.html}{GLAD} package.
	We obtained the reference data set from two published twin studies 
	(\href{https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE85647}{GSE85647} and 
	\href{https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE100940}{GSE100940}) as the median
	signal intensity for the probes. To activate the reference-based mode, you need to set:

<<eval=FALSE>>=
rnb.options(qc.cnv.refbased=TRUE)
@

	In the default mode, RnBeads computes median signal intensities for all samples in
	the data set and uses this as the reference to detect potential outliers with respect
	to signal intensity. 

\subsection{Preprocessing}
	Several steps might be required to prepare the data for downstream analysis.
	Importantly, probe and sample filtering is carried out according to default or
	user specified criteria. Furthermore, for Infinium data, it is usually recommended
	to normalize the data. The \emph{Preprocessing} module takes care of this. In a
	first filtering step, probes and samples that could bias the normalization
	procedure are removed. Subsequently, normalization is conducted for microarray
	datasets. Finally, additional filtering steps are potentially executed. The
	following commands run these steps and return a preprocessed \cs{RnBSet} object:

<<eval=FALSE>>=
rnb.set.unfiltered <- rnb.set
result <- rnb.run.preprocessing(rnb.set.unfiltered, dir.reports=report.dir)
rnb.set <- result$rnb.set
@

	The following two subsections contain details on the filtering and normalization steps, respectively.

	\subsubsection{Filtering}
		\cs{RnBeads} facilitates filtering of sites and samples according to a variety of criteria. As noted above, the filtering steps are executed in two stages: one before and the other after data normalization\footnote{Note: for bisulfite sequencing date, no normalization is performed and hence the
		two filtering stages are executed consecutively.}.
		In the default pipeline, these steps exclude Infinium probes overlapping with
		too many SNPs and sites with too many missing values. Additionally, if
		detection p-values are available, both sites and samples are filtered using a
		greedy approach (see the generated report for details).

		You can also invoke individual filtering steps explicitly using the
		corresponding \cs{rnb.execute.*} functions:
<<eval=FALSE>>=
nrow(meth(rnb.set.unfiltered)) # the number of sites in the unfiltered object
# Remove probes outside of CpG context
rnb.set.filtered <- rnb.execute.context.removal(rnb.set.unfiltered)$dataset
nrow(meth(rnb.set.filtered)) # the number of CpG sites in the unfiltered object
# SNP filtering allowing no SNPs in the probe sequence
rnb.set.filtered <-
    rnb.execute.snp.removal(rnb.set.filtered, snp="any")$dataset
# the number of CpG sites in the unfiltered object
# that do not contain a SNP
nrow(meth(rnb.set.filtered))

# Remove CpGs on sex chromosomes
rnb.set.filtered <- rnb.execute.sex.removal(rnb.set.filtered)$dataset
nrow(meth(rnb.set.filtered))
# Remove probes and samples based on a greedy approach
rnb.set.filtered <- rnb.execute.greedycut(rnb.set.filtered)$dataset
nrow(meth(rnb.set.filtered))
# Remove probes containing NA for beta values
rnb.set.filtered <- rnb.execute.na.removal(rnb.set.filtered)$dataset
nrow(meth(rnb.set.filtered))
# Remove probes for which the beta values have low standard deviation
rnb.set.filtered <-
    rnb.execute.variability.removal(rnb.set.filtered, 0.005)$dataset
nrow(meth(rnb.set.filtered))
@
		Some of these filtering criteria do not apply to bisulfite sequencing data while others are added. See Section~\ref{sec:bisulfite} for more details.


	\subsubsection{Normalization}

		For Infinium 450k data \cs{RnBeads} includes possibilities for normalization of the microarray signals in order to decrease the level of technical noise and eliminate the systematic biases of the microarray procedure.

		Currently, \cs{RnBeads} supports several normalization methods: %TODO:@Pavlo: please update this list
		\begin{itemize}
			\item manufacturer-recommended scaling to the internal controls, implemented in the \cs{methylumi} R-package \cite{methylumi},
			\item SWAN-normalization~\cite{Maksimovic2012}, as implemented in the \cs{minfi} package~\cite{minfi},
			\item data-driven normalization methods~\cite{Pidsley2013} implemented in the \cs{wateRmelon} package,
			\item the BMIQ procedure~\cite{Teschendorff2013beta}.
		\end{itemize}
		In addition, background subtraction methods implemented in the \cs{methylumi} package~-- e.g. NOOB and Lumi -- can be applied in combination with any of the above normalization methods~\cite{Triche2013}. Table~\ref{tab:hm450-norm-methods} summarizes supported normalization and background correction methods.

		\begin{table}[h!]
		\label{tab:hm450-norm-methods}
		\caption{Infinium 450k normalization and background subtraction methods.}
		\vspace{10px}
		\begin{minipage}[h!]{\textwidth}
		\begin{tabular}{ | l | c | c | c |}
			\hline
			\hline
			\textbf{Method} & \textbf{Backend package} & \textbf{Option} & \textbf{Reference} \\
			\hline
			\multicolumn{4}{|c|}{\textit{Normalization} (\cs{normalization.method=})} \\
			\hline
			Normalization to internal controls & \cs{methylumi} & \cs{"illumina"} & \cite{Triche2013, methylumi}\\
			SWAN & \cs{minfi} & \cs{"swan"} & \cite{Maksimovic2012, minfi} \\
			"Data-driven" normalization & \cs{wateRmelon} & \cs{"wm.*"} \footnote{ * is a placeholder for the \cs{wateRmelon} normalization methods, e.g. "dasen", "nanet" etc.} & \cite{Pidsley2013} \\
            Functional normalization & \cs{minfi} & \cs{minfi.funnorm} &
            \cite{Fortin2014, minfi}\\
			BMIQ & \cs{BMIQ} & \cs{"bmiq"} \footnote{ unlike other normalization and
			background subtraction methods BMIQ is applied to the summarized methylation calls, so it can be used even if raw intensity data were not loaded, like in case of \cs{GEO} or \cs{data.dir} data types} & \cite{Teschendorff2013beta} \\
			\hline
			\multicolumn{4}{|c|}{\textit{Background correction} (\cs{normalization.background.method=})}\\
			\hline
			NOOB & \cs{methylumi} & \cs{"methylumi.noob"} & \cite{Triche2013, methylumi}\\
			Lumi & \cs{methylumi} & \cs{"methylumi.lumi"} & \cite{Triche2013, methylumi}\\
			\hline
		\end{tabular}
		\end{minipage}
		\end{table}

		The preprocessing report includes a number of diagnostic plots that visualize the effects of the normalization procedure: density plots of the methylation values prior and subsequent to the normalization procedure, histograms of the per-data point beta-value differences, and others.

		Note that the data contained within a loaded \cs{RnBeadRawSet} object can be normalized without generating a report:

<<eval=FALSE>>=
rnb.set.norm <- rnb.execute.normalization(rnb.set.unfiltered, method="illumina",
    bgcorr.method="methylumi.noob")
@

%\subsubsection{Bisulfite sequencing}

\subsection{Covariate Inference}

	This optional module facilitates the inference of additional dataset covariates. For instance, surrogate variables can be estimated to detect and adjust for possible hidden confounders in the dataset. Additionally, the effects of cell type contributions can be assessed.

	Note that this module is disabled by default. You can enable it for the \cs{rnb.run.analysis} command using the following option setting:
<<eval=FALSE>>=
rnb.options(inference=TRUE)
@
	The annotation inference module can be executed individually using its dedicated function:

<<eval=FALSE>>=
rnb.set <- rnb.run.inference(rnb.set, report.dir)$rnb.set
@

	\subsubsection{Surrogate Variable Analysis}
	\label{sec:sva}
	Surrogate Variable Analysis (SVA)~\cite{Leek2007sva} has become a popular tool
	to adjust for hidden confounders in expression microarray analysis and can
	also be employed in the DNA methylation setting. \cs{RnBeads} uses the \cs{sva} package~\cite{Leek2012sva} to infer surrogate variables which can be related to other sample annotation and can be used to adjust differential methylation analysis using the \cs{limma} method. In order to compute SVs for given target variables, to add the covariates to an \cs{RnBSet} and to generate a corresponding report, consider the following example:

<<eval=FALSE>>=
rnb.options(
	inference.targets.sva=c("Sample_Group"),
	inference.sva.num.method="be"
)
rnb.set.new <- rnb.run.inference(rnb.set, report.dir)$rnb.set
@

	If you just want to conduct the analysis without generating a report, you can compute the SVs and add the information to an \cs{RnBSet} the following way:
<<eval=FALSE>>=
sva.obj <- rnb.execute.sva(rnb.set, cmp.cols="Sample_Group", numSVmethod="be")
rnb.set <- set.covariates.sva(rnb.set, sva.obj)
@

	Section~\ref{recipe:SVAdjust} contains a recipe on how these covariates can be used for adjustment during differential methylation detection.

	\subsubsection{Inference of Cell Type Contributions}
	\label{sec:ct-inference}
	\cs{RnBeads} is capable of tackling cell type heterogeneity of the analyzed samples. In case methylomes of reference cell types are available, the annotation inference module will try to estimate the relative contributions of underlying cell types in target data. RnBeads applies the method by Houseman et al.~\cite{Houseman2012}. The following paragraphs give details on how this algorithm is incorporated in \cs{RnBeads}. Section~\ref{sec:celltypeheterogeneity} describes the usage of an alternative reference-free approach.

	Methylation profiles of reference cell types should be processed together with the target data. Assignment of the samples to cell types is done via a special column of the sample annotation file set by the \cs{inference.reference.methylome.column}. The column should map each reference sample to one of the cell types, while the target samples should be assigned with missing (\cs{NA}) values. As an example, let the reference methylome column be called "CellType". After setting the option:

<<eval=FALSE>>=
rnb.options(inference.reference.methylome.column="CellType")
@
    \cs{RnBeads} will automatically activate the cell type section of the annotation
    inference module. The estimation procedure from~\cite{Houseman2012}
    consists of two major steps. First, the marker model is fit to the reference
    methylome data, to identify CpG sites, methylation level of which is
    associated with the cell type. To limit computational load at this step,
    RnBeads by default uses only 50,000 CpGs with highest variance across all
    samples. Alternative number can be set via the
    \cs{inference.max.cell.type.markers} option. The final number of
    selected cell type markers is controlled by
    \cs{inference.top.cell.type.markers} which is 500 by default. The report
    visualizes the selection by plotting the F-statitistics of the marker
    model fit. In the second step, the marker model coefficients for selected
    markers CpGs are used for estimation of cell type contributions by
    solving a (constrained) quadratic optimization problem for corresponding
    vectors of each of the target methylomes a.k.a. the projection. The
    inferred contributions will be plotted in the annotation inference report
    and saved in the \cs{RnBSet} object and later used as covariates in the
    association analysis to adjust for the variation in cell type composition.
    The latter option is only available in case \cs{"limma"} method is used for
    the testing differential methylation.

    Alternatively, the reference-based analysis of cell type contributions can
    be performed without report generation:
<<eval=FALSE>>=
ct.obj<-rnb.execute.ct.estimation(rnb.set, cell.type.column="CellType",
    test.max.markers=10000, top.markers=500)
@
    The returned \cs{S3} object of class \cs{CellTypeInferenceResult} contains
    all information about the estimation procedure and can be supplied as an
    argument to functions \cs{rnb.plot.marker.fstat} and
    \cs{rnb.plot.ct.heatmap}. Cell type contributions are available by direct
    slot access:
<<eval=FALSE>>=
ct.obj$contributions
@
for raw coefficients of the projection model,
<<eval=FALSE>>=
ct.obj$contributions.constr
@
to get constrained (non-negative) coefficients which resemble cell
type counts and sum up to a small value $<1$ .

\subsubsection{Age Prediction}

In addition to the inference possibilities mentioned before, \cs{RnBeads} is capable of predicting human age from DNA methylation data. The prediction was initially performed by building an elastic net regression model on the methylation values of the CpG sites. Performance was evaluated by testing the prediction on several independent test data sets.

	Basically, the age prediction section of \cs{RnBeads} is activated by setting:
<<eval=FALSE>>=
rnb.options(inference.age.prediction=TRUE)
@
	Or by performing the command
<<eval=FALSE>>=
rnb.execute.age.prediction(rnb.set)
@
	The difference between the first and the second option is that the first one creates a report, whereas the second modifies the \cs{rnb.set} object by adding additional columns \cs{predicted\_ages} and \cs{age\_increase}. The first one corresponds to the predicted age by the prediction algorithm and the second to the difference between predicted and annotated age.

	Then, the age prediction section is able to run in two different modes. The default mode runs the prediction by loading a predefined predictor in the form of a CSV-file, which then is used to predict human age from DNA methylation data. More specifically, the predefined predictor consists of coefficients of an elastic net regression run for 761 CpG sites used in the final age prediction. As a second possibility, the user can set the option \cs{inference.age.prediction.predictor} to a path to a predictor that was previously trained in a \cs{RnBeads} run. For instance, this predictor could be trained by running the second mode of the age prediction, which is the training mode. Training is performed by either one of these commands:
<<eval=FALSE>>=
rnb.execute.training(rnb.set,predictor.path)
rnb.options(inference.age.prediction.training=TRUE)
@
	In the first case, a new predictor is trained based on the methylation information in the \cs{rnb.set} object and written as a CSV-file at the given \cs{predictor.path}, which has to be a full path. The second option requires running \cs{rnb.run.inference} to create a new predictor that then is written into the \cs{RnBeads} data folder and sets the option \cs{inference.age.prediction.predictor} to the corresponding one.

	Additionally, by setting the option \cs{inference.age.prediction.cv}, a 10-fold cross-validation is performed to serve as an accuracy measure for the training functionality. Setting the option \cs{inference.age.column} enables \cs{RnBeads} to read the corresponding age annotation column of the data set, which enables the creation of evaluation plots. These include scatterplots of the deviation between annotated and predicted age as well as density plots for the differences between predicted and annotated ages. On default, \cs{inference.age.column} is set to \cs{age}.

\subsubsection{Immune Cell Content Estimation}

	The LUMP algorithm for estimating immune cell content was developed by Dvir Aran, Marina Sirota and Atul J. Buttea for Infinium 450k data~\cite{Aran2015}. It is implemented in \cs{RnBeads} for all supported platforms in the genome assembly hg19: Infinium 27k, Infinium 450k, MethylationEPIC and sequencing-based data.

	By default, immune cell content estimation is attempted when the Covariate Inference module is run. Upon successful completion of the LUMP algorithm, the sample annotation table receives a column named \cs{Immune Cell Content (LUMP)} that lists the estimated immune cell fractions for the samples. If such a column already exists, its contents will be overwritten. LUMP can be disabled using the following option:

<<eval=FALSE>>=
rnb.options(inference.immune.cells=FALSE)
@
	The dedicated function to obtain LUMP estimates expects a methylation dataset as a parameter:
<<eval=FALSE>>=
immune.content <- rnb.execute.lump(rnb.set)
@

	If none of the CpGs used in LUMP is measured in the dataset of interest, the return value is \cs{NULL}. Otherwise, the function returns a numeric vector with immune cell fractions for every sample, that is, values between 0 and 1. The vector also has an integer attribute named \cs{"sites"} that stores the number of CpGs or probes used in estimating immume cell proportions for the specific dataset.

\subsection{Exploratory Analysis}
\label{sec:exploratory}

	You guessed it: the command for running the exploratory analysis module is
<<eval=FALSE>>=
rnb.run.exploratory(rnb.set, report.dir)
@
	Note that depending on the options settings and types of enabled plots, this might take a while. It creates a report that lets you inspect the association of various sample traits with each other, with quality control probes (if available), as well as with sparse representations of the methylation profiles. This module also contains sections dedicated to inspecting methylation profiles of sample groups, as well as unsupervised learning techniques to cluster samples. The corresponding report file is \cs{exploratory.html}.

	\subsubsection{Low-dimensional Representation and Batch Effects}

	The following series of commands calculates the sparse representations, i.e. by Principal Component Analysis (PCA) and Multidimensional Scaling (MDS) for single CpG sites and promoters and plots them:
<<eval=FALSE>>=
dred.sites <- rnb.execute.dreduction(rnb.set)
dred.promoters <- rnb.execute.dreduction(rnb.set, target="promoters")
dred <- list(sites=dred.sites, promoters=dred.promoters)
sample.colors <- ifelse(pheno(rnb.set)$Sample_Group=="hESC", "red", "blue")
plot(dred[[1]]$mds$euclidean, col=sample.colors,
    xlab="Dimension 1", ylab="Dimension 2")
@
		\begin{figure}
			\noindent \begin{centering}
				\includegraphics[width=0.50\textwidth]{"figures/mds_sampleGroup"}
			\par \end{centering}

			\caption{\label{fig:mds} Multidimensional scaling applied to the example dataset. Points are colored according to cell type: Blue: iPSCs, Red: ESCs}
		\end{figure}

	The resulting plot can be seen in Figure~\ref{fig:mds}. Subsequently, let us investigate associations of sample annotations:
<<eval=FALSE>>=
assoc <- rnb.execute.batcheffects(rnb.set, pcoordinates=dred)
str(assoc)
@
	The initiated object contains slots for the association of sample annotations with each other (\cs{assoc\$traits}) and with the principal components calculated in the previous step (\cs{assoc\$pc}: a list containing elements for each region type; here: \cs{sites} and \cs{promoters}). Each of these slots summarizes whether a test could be conducted (\cs{failures}), which kind of test was applied (\cs{tests}), the resulting p-value (\cs{pvalues}) and, if calculated, a table of \cs{correlations}.
	The \cs{batch.dreduction.columns} option determines which sample annotations are processed (see Section~\ref{sec:annotation}). By default, these are all columns in the table \cs{pheno(rnb.set)} which have a sufficient number of samples per group and not too many resulting groups (controlled by the \cs{min.group.size} and \cs{max.group.count} parameters). However, you can restrict the analysis to certain columns via commands like:
<<eval=FALSE>>=
rnb.options(exploratory.columns=c("Sample_Group", "Passage_No"))
@
	Next, one can compute associations of the principal components with the Infinium control probes (if that information is available):
<<eval=FALSE>>=
assoc.qc <- rnb.execute.batch.qc(rnb.set, pcoordinates=dred)
@

	A good example for useful visualizations is the so-called deviation plots. These plots show the median $\beta$ values and their percentiles in a ``snake-like'' curve. The plot can be annotated with coloring according to site or region features. The code snippet below creates Figure~\ref{fig:devPlot} which illustrates the beta value distributions according to probe design type:
<<eval=FALSE>>=
probe.types <- as.character(annotation(rnb.set)[, "Design"])
pdf(file="deviationPlot_probeType.pdf", width=11)
deviation.plot.beta(meth(rnb.set), probe.types,
    c.legend=c("I"="blue", "II"="red"))
dev.off()
@
		\begin{figure}
			\noindent \begin{centering}
				\includegraphics[width=0.75\textwidth]{"figures/deviationPlot_probeType"}
			\par \end{centering}

			\caption{Deviation plot of the example dataset. The blue line corresponds to the median methylation value among all samples, the yellow bands to the 5-th and 95-th percentiles. Coloring below the horizontal axis indicates probe types (Blue: Infinium Design Type I, Red: Infinium Design Type II).}
			\label{fig:devPlot}
		\end{figure}
		Note that creating such a plot requires a few minutes.

	\subsubsection{Clustering}
	Use the following command to apply all combinations of clustering parameter settings and conduct the corresponding clusterings on the DNA methylation levels of individual CpGs as well as promoter regions:
<<eval=FALSE>>=
clusterings.sites <- rnb.execute.clustering(rnb.set, region.type="sites")
clusterings.promoters <-
    rnb.execute.clustering(rnb.set, region.type="promoters")
@
	The values returned by the commands above are lists of clustering results. Each element in the list is an object of type \cs{RnBeadClustering}. Currently, \cs{RnBeads} performs agglomerative hierarchical clustering in conjunction with three distance metrics and three linkage methods. The distance metrics are correlation-based ($1 - \rho(x, y)$), Manhattan and Euclidean distances. The agglomeration methods are average, complete and median-based linkages.
	Here's a little recipe which lets you create a clustered heatmap of the
	methylation values according to the first clustering of the first 100 promoters (Figure~\ref{fig:heatmapPromoters}) using the following commands:
<<eval=FALSE>>=
# Get the methylation values
X <- meth(rnb.set, type="promoters")[1:100, ]
# Convert the clustering result to a dendrogram
# Index 7 holds euclidean distance, average linkage clustering
cresult <- clusterings.promoters[[7]]@result
attr(cresult, "class") <- "hclust"
cresult <- as.dendrogram(cresult)
# Save the heatmap as pdf file
pdf(file="promoter_heatmap.pdf")
heatmap.2(X, Rowv=TRUE, Colv=cresult, dendrogram="both",
    scale="none", trace="none")
dev.off()
@
	\begin{figure}
		\noindent \begin{centering}
			\includegraphics[width=0.75\textwidth]{"figures/promoter_heatmap"}
		\par \end{centering}

		\caption{Clustered heatmap of 100 promoters.}
		\label{fig:heatmapPromoters}
	\end{figure}

	\subsection{Differential Methylation Analysis}
		\label{sec:diffMeth}
		Differential methylation analysis in \cs{RnBeads} can be conducted on the site level as well as on the genomic region level. Genomic regions are inferred from the annotation data (see Section~\ref{sec:annotation}). As seen before, a comprehensive analysis can be obtained by the corresponding \cs{rnb.run.*.} command:
<<eval=FALSE>>=
rnb.run.differential(rnb.set, report.dir)
@
        Note that this analysis module requires more runtime than other modules. Specifically computing differential methylation can be achieved using the following steps:
<<eval=FALSE>>=
# Specify the sample annotation table columns for which
# differential methylation is to be computed
cmp.cols <- "Sample_Group"
# Specify the region types
reg.types <- c("genes", "promoters")
# Conduct the analysis
diffmeth <- rnb.execute.computeDiffMeth(rnb.set, cmp.cols,
    region.types=reg.types)
@
		Differential methylation for each selected annotation column is performed in a one-vs-all manner, i.e. samples in each group are compared to all samples not in the group. This is done for each group and annotation column. Depending on the number of comparisons, this can take a while. The \cs{differential.comparison.columns.all.pairwise} option facilitates conducting all pairwise group comparisons on the specified columns. Handle this option with care, as the number of comparisons can explode very quickly if the number of groups specified in an annotation column is large. In order to define custom comparisons between groups you can edit the sample annotation sheet before data loading. For instance, if you want to carry out a specific comparison in addition to the already defined comparisons, consider adding a column containing the labels \cs{"Group1"} and \cs{"Group2"} for specific samples and \cs{"NA"} for the others. Alternatively, you can use \cs{addPheno()} to add sample annotation to the \cs{RnBSet} object as seen before.

        Let us inspect the resulting \cs{RnBDiffMeth} object (an S4 class) more closely.

<<eval=FALSE>>=
str(diffmeth)
@
		It contains tables storing the differential methylation information for each comparison for both site and region level. You can see the comparisons that were conducted using \\ \cs{get.comparisons(diffmeth)}. The regions that were compared can be seen with \\ \cs{get.region.types(diffmeth)}. Information on which groups have been compared in each comparison can be obtained by \cs{get.comparison.grouplabels(diffmeth)}. For a given comparison and region type you can get get a table containing information on differential methylation using \cs{get.table()}:
<<eval=FALSE>>=
comparison <- get.comparisons(diffmeth)[1]
tab.sites <- get.table(diffmeth, comparison, "sites", return.data.frame=TRUE)
str(tab.sites)
tab.promoters <- get.table(diffmeth, comparison, "promoters",
	return.data.frame=TRUE)
str(tab.promoters)
@
		Below you can find the most important columns of the differential methylation tables on the site level \\ (\cs{tab.sites} from the code above):

		\begin{description}
			\item[\cs{mean.g1, mean.g2:}] mean methylation in each of the two groups
			\item[\cs{mean.diff:}] difference in methylation means between the two groups: \cs{mean.g1-mean.g2}. In case of paired analysis, it is the mean of the pairwise differences.
			\item[\cs{mean.quot.log2:}] log2 of the quotient in methylation: $\log_2\frac{\text{\cs{mean.g1}}+\varepsilon}{\text{\cs{mean.g2}}+\varepsilon}$, where $\varepsilon=0.01$. In case of paired analysis, it is the mean of the pairwise quotients
			\item[\cs{diffmeth.p.val:}] p-value obtained from a two-sided Welch t-test or alternatively from linear models employed in the limma package
						(which type of p-value is computed is specified in the differential.site.test.method option). In case of paired analysis, the paired Student's t-test is applied.
			\item[\cs{max.g1, max.g2:}] maximum methylation level in group 1 and 2 respectively
			\item[\cs{min.g1, min.g2:}] minimum methylation level in group 1 and 2 respectively
			\item[\cs{sd.g1, sd.g2:}] standard deviation of methylation levels
			\item[\cs{min.diff:}] minimum of 0 and the smallest pairwise difference between samples of the two groups
			\item[\cs{diffmeth.p.adj.fdr:}] FDR adjusted p-values of all the sites
			\item[\cs{combinedRank:}] \cs{mean.diff}, \cs{mean.quot} and \cs{t.test.p.val} are ranked for all sites. This aggregates them using the maximum, i.e. worst rank of a probe among the three measures
		\end{description}

		and the region level (e.g. \cs{tab.promoters}):
		\begin{description}
			\item[\cs{mean.mean.g1, mean.mean.g2:}] mean of mean methylation levels for group 1 and 2 across all probes in a region
			\item[\cs{mean.mean.diff:}] mean difference in means across all sites in a region
			\item[\cs{mean.mean.quot.log2:}] log2 of the mean quotient in means across all sites in a region
			\item[\cs{comb.p.val:}] combined p-value using a generalization of Fisher's method \cite{Makambi2003}
			\item[\cs{comb.p.adj.fdr}] FDR adjusted combined p-value
			\item[\cs{num.sites:}] number of sites associated with the region
			\item[\cs{combinedRank:}] \cs{mean.mean.diff}, \cs{mean.mean.quot.log2} and \cs{comb.p.val} are ranked for all regions. This column aggregates them using the maximum, i.e. worst rank of a probe among the three measures
		\end{description}

		For explanations on additional columns and more details be referred to the corresponding report (\cs{differential.html}) and the help pages (\cs{?computeDiffTab.site,?computeDiffTab.region}). In an \cs{RnBeads} analysis, we recommend to conduct differential methylation analysis based on the combined ranking score. It combines absolute and relative effect sizes as well as p-values from statistical modeling into a single score. The lower the combined rank, the higher is the degree of differential methylation. I.e. when you sort the list according to their combined rank, the best ranking sites will be at the top of the list, and when you work your way towards the bottom of the list, false positives become more likely.

<<eval=FALSE>>=
dmps <- tab.sites[order(tab.sites[, "combinedRank"]), ]
summary(dmps[1:100, ])
summary(dmps[1:1000, ])
@

	    However, traditional, cutoff-based approaches are also possible:
<<eval=FALSE>>=
which.diffmeth <- abs(dmps[, "mean.diff"])>0.2 & dmps$diffmeth.p.adj.fdr<0.05
dmps.cutoff <- dmps[which.diffmeth, ]
@
        Note that you can specify how p-values are computed by using \cs{RnBeads}' \\ \cs{'differential.site.test.method'} option. Currently supported are linear modeling using the \cs{limma} package and regular t-tests.

        The following code creates a volcano plot comparing effect size and significance for all promoter regions (see Figure~\ref{fig:volcanoDMRs}):
<<eval=FALSE>>=
dmrs <- get.table(diffmeth, comparison, "promoters")
plot(dmrs[, "mean.mean.diff"], -log10(dmrs[, "comb.p.val"]),
    xlab="mean difference", ylab="-log10(combined p-value)", col="blue")
@
        \begin{figure}
        	\noindent \begin{centering}
        		\includegraphics[width=0.50\textwidth]{"figures/volcanoDMRs_promoter"}
        	\par \end{centering}

        	\caption{\label{fig:volcanoDMRs} Volcano plots for promoter differential methylation comparing hESCs against hiPSCs.}
        \end{figure}
        
        \subsubsection{Differential Variability Analysis}
        
        In addition to comparing mean methylation levels inside groups with each other, the variances inside each group can be used to detect differences among them. For this purpose, \cs{RnBeads} employs a similar approach to the one used for differential methylation. To conduct differential variability (together with differential methylation) analysis, execute the following lines of code:
<<eval=FALSE>>=
rnb.options("differential.variability"=TRUE)
rnb.run.differential(rnb.set,report.dir)
@
        
        To conduct solely differential variability analysis (without report generation), one can call:
<<eval=FALSE>>=
cmp.cols <- "Sample_Group"
reg.types <- c("genes","promoters")
diff.var <- rnb.execute.diffVar(rnb.set,cmp.cols,region.types=reg.types)
comparison <- get.comparisons(diff.var)[1]
tab.sites <- get.table(diff.var,comparison,"sites",return.data.frame=TRUE)
tab.genes <- get.table(diff.var,comparison,"genes",return.data.frame=TRUE)
@
        
        After this operation, the tables contained in the \cs{diff.var} object (of class \cs{RnBDiffMeth}) possess different columns in comparison to differential methylation. Those columns are (from \cs{tab.sites} above):
        \begin{description}
        	\item[\cs{var.g1, var.g2:}] variances in each of the two groups
        	\item[\cs{var.diff:}] differences between the variances found is the two groups: \cs{var.g1-var.g2}.
        	\item[\cs{var.log.ratio:}] log2 of the quotient of variances: $\log_2 \frac{\cs{var.g1}+\epsilon}{\cs{var.g2}+\epsilon}$, with $\epsilon=0.01$.
        	\item[\cs{diffVar.p.val:}] p-value resulting from employing the differential variability test specified in the option differential.variability.method. Those are either `diffVar' from the `missMethyl' package or the iEVORA algorithm.
        	\item[\cs{diffVar.p.adj.fdr:}] FDR adjustment of the above p-value.
        	\item[\cs{combinedRank.var:}] \cs{var.diff}, \cs{var.log.ratio} and \cs{diffVar.p.val} for all sites. The rank is computed as the maximum, i.e. worst rank among these values.
        \end{description}
        
        For the table \cs{tab.genes} (i.e. the region level), the columns are:
        
        \begin{description}
        	\item[\cs{mean.var.g1, mean.var.g2:}] mean of variances for both groups across all sites in the region.
        	\item[\cs{mean.var.diff:}] mean difference between the probe variances in the two groups.
        	\item[\cs{mean.var.log.ratio:}] mean difference between the probe quotients in the two groups.
        	\item[\cs{comb.p.val.var:}] combined p-value for the region, combined as for differential methylation.
        	\item[\cs{comb.p.adj.var.fdr:}] FDR adjustment of the above p-value.
        	\item[\cs{combinedRank.var:}] \cs{mean.var.diff, mean.var.log.ratio} and \cs{comb.p.val.var} for all regions. The rank is computed as the worst rank among those. 
        \end{description}
        
        Similar to differential methylation, the differential variability module adds diagnostic plots to the \cs{RnBeads} report in \cs{differential.html}. Enrichment analysis is performed analogously to differential methylation (see below).
        \subsubsection{Paired Analysis}

        Upon specifying the corresponding options, \cs{RnBeads} can also take into account sample pairing information when computing p-values.
        Suppose you have a sample annotation table like this one\footnote{Note: while we use the first names of super-heroes in this table for
        instructive purposes, you should never reveal patient names that have not been pseudonymized in your analysis}: \\

        \begin{tabular}{| l | l | l |}
        \hline
		sample & individual	& diseaseState \\
		\hline
		sample\_1 & Bruce & normal \\
		sample\_2 & Bruce & tumor \\
		sample\_3 & Clark & normal \\
		sample\_4 & Clark & tumor \\
		sample\_5 & Peter & normal \\
		sample\_6 & Peter & tumor \\
		\hline
		\end{tabular} \\

		Further suppose, you want to compare tumor vs normal but with the pairing information by the patient/individual. Then you would apply the following option setting \underline{before} running the analysis:
<<eval=FALSE>>=
rnb.options("differential.comparison.columns"=c("diseaseState"),
    "columns.pairing"=c("diseaseState"="individual"))
@
		Note, that since the dataset used in this vignette has no column called \cs{diseaseState}, the option settings above will cause some the examples in the next sections to fail. So, revert the option setting before continuing:

<<eval=FALSE>>=
rnb.options("differential.comparison.columns"=NULL,
    "columns.pairing"=NULL)
@

		\subsubsection{Adjusting for Covariates in the Differential Analysis}

		When the \cs{'differential.site.test.method'} option is set to \cs{'limma'} (default), you can specify columns of potential confounding factors that should be taken into account when computing the site specific p-values. Again, setting the corresponding option prior to analysis is the key here:
<<eval=FALSE>>=
rnb.options("covariate.adjustment.columns"=c("Passage_No"))
diffmeth.adj <- rnb.execute.computeDiffMeth(rnb.set, cmp.cols,
    region.types=reg.types)
@

		\subsubsection{Enrichment Analysis of Differentially Methylated Regions}

		\cs{RnBeads} allows you to characterize differentially methylated regions by computing enrichments for Gene Ontology (GO)~\cite{Ashburner2000} terms and for custom genomic annotations using the \cs{LOLA} tool~\cite{Sheffield2016}. You can active these analyses for your \cs{RnBeads} runs using the corresponding options:
<<eval=FALSE>>=
rnb.options("differential.enrichment.go"=TRUE)
rnb.options("differential.enrichment.lola"=TRUE)
@

		If you want to manually extract the enrichment results, you can use \cs{RnBeads}' build-in functions. To compute GO enrichments from the computed differential methylation results you can use the following commands (takes a while because an online GO database is queried for each comparison, region type and for different differential methylation rank cutoffs):

<<eval=FALSE>>=
enrich.go <- performGoEnrichment.diffMeth(rnb.set, diffmeth, verbose=TRUE)
@
		The result is a nested list storing enrichment tables for each comparison, ontology, differential methylation rank cutoff and direction of differential methylation. Each element in this nested list corresponds to a result computed using the \cs{GOStats} package~\cite{Falcon2007}. It contains enrichment odds ratios and p-values. For instance, let us inspect the enrichment of the 500 highest ranking hypomethylated promoters (in ESCs compared to iPSCs) for biological process (BP) ontology terms:
<<eval=FALSE>>=
enrich.table.go <- 
    enrich.go[["region"]][[comparison]][["BP"]][["promoters"]][["rankCut_500"]][["hypo"]]
class(enrich.table.go)
summary(enrich.table.go)
@

		In addition, \cs{RnBeads} supports enrichment analyses of differential methylation results using the \cs{LOLA} tool~\cite{Sheffield2016}. This tool facilitates enrichment analyses of arbitrary genomic and epigenomic annotations using Fisher's exact test. To run a \cs{LOLA} analysis manually, you first need to obtain a corresponding database. For instance, to download the core database from the tool website to a temporary destination, execute the following code:
<<eval=FALSE>>=
lolaDest <- tempfile()
dir.create(lolaDest)
lolaDirs <- downloadLolaDbs(lolaDest, dbs="LOLACore")
@
		
		This database currently contains annotations for transcription factor binding sites and motifs, DNase accessible regions and genomic features. For more detailed information on the \cs{LOLA} tool, available precomputed databases and instructions on how to assemble your own database, please be referred to the \cs{LOLA} documentation\footnote{\href{http://databio.org/lola/}{http://databio.org/lola/}}. You can now compute the enrichment of differentially methylated regions to reference region sets in the database:
<<eval=FALSE>>=
enrich.lola <-
    performLolaEnrichment.diffMeth(rnb.set, diffmeth, lolaDirs[["hg19"]])
@
		The result is a structured object containing enrichment tables for different comparisons, region types and rank cutoffs as well as information on the reference database. These objects are best explored using \cs{RnBeads}' utility functions. Here we plot the enrichment results of the 500 highest ranking hypermethylated promoter regions in ESCs compared to iPSCs.
<<eval=FALSE>>=
enrich.table.lola <- enrich.lola$region[[comparison]][["promoters"]]
enrich.table.lola <-
    enrich.table.lola[enrich.table.lola$userSet=="rankCut_500_hyper",]
lolaBarPlot(enrich.lola$lolaDb, enrich.table.lola, scoreCol="oddsRatio",
    orderCol="maxRnk", pvalCut=0.05)
lolaBoxPlotPerTarget(enrich.lola$lolaDb, enrich.table.lola, scoreCol="pValueLog",
    orderCol="maxRnk", pvalCut=0.05)
@
		Figures~\ref{fig:lolaBar} and~\ref{fig:lolaBox} show the results as bar and boxplots. Interpreting the results, we find patterns associated with pluripotency hypermethylated in ESCs compared to iPSCs: enriched region sets are associated with reprogramming factors (POU5F1, SOX2, NANOG) and accessible chromatin (DNase) in stem cells. Additionally, signatures of gene repression (repressed state in the ENCODE segmentation and the associated H3K27me3 histone mark) are present in more differentiated cells.
		\begin{figure}
			\noindent \begin{centering}
				\includegraphics[width=0.95\textwidth]{"figures/lola_bar"}
			\par \end{centering}

			\caption{\label{fig:lolaBar} Bar plot of odds ratios for \cs{LOLA} region sets significantly enriched in promoters hypermethylated in ESCs compared to iPSCs. The bars are grouped by category in the \cs{LOLA} core database. The coloring indicates different features (chromatin marks, states, transcription factors, etc.).}
		\end{figure}
		\begin{figure}
			\noindent \begin{centering}
				\includegraphics[width=0.95\textwidth]{"figures/lola_box"}
			\par \end{centering}

			\caption{\label{fig:lolaBox} Box plot of log p-values for \cs{LOLA} region sets significantly enriched in promoters hypermethylated in ESCs compared to iPSCs. Each bar summarizes a given feature across experiments in the \cs{LOLA} core database.}
		\end{figure}

\subsection{Tracks and Tables}

	By default, methylation tracks are exported in \cs{bed} format. They contain genomic locations and beta values for each site in the filtered \cs{RnBSet} for each sample in the dataset. Additionally, track hubs\footnote{\href{http://genome.ucsc.edu/goldenPath/help/hgTrackHubHelp.html}{http://genome.ucsc.edu/goldenPath/help/hgTrackHubHelp.html}} for the UCSC or Ensembl Genome browsers can be generated. For using the track hubs in conjunction with a genome browser you will need a server that can be reached via a URL to provide the data. The command to convert the \cs{RnBSet} to the desired output and to generate the corresponding report is
<<eval=FALSE>>=
rnb.options(export.to.csv=TRUE)
rnb.run.tnt(rnb.set, report.dir)
@
	The report contains specific instructions on how to visualize the data in the genome browser. Additionally, by default the Tracks and Tables report will also contain a sample summary table. In this particular example, prior to starting the module, we enable the option \cs{export.to.csv}. Hence, the generated report contains also single-site methylation value matrices, exported to comma-separated value files.

\section{Analyzing Bisulfite Sequencing Data}
	\label{sec:bisulfite}
    \cs{RnBeads} also facilitates the analysis of bisulfite data obtained from experiments such as whole genome, reduced representation or locus specific bisulfite sequencing. The general structure for analyzing bisulfite sequencing data with \cs{RnBeads} is the same as for Infinium data. However, the devil is in the details. Here are the key differences:
    \begin{itemize}
        \item In addition to human methylation data, \cs{RnBeads} also supports mouse and rat methylome analysis through its companion packages (\cs{RnBeads.mm9,RnBeads.mm10,RnBeads.rn5}).
        \item Loading is performed with \cs{bed} file input.
        \item No normalization will be conducted during preprocessing.
        \item There are no control probes in bisulfite sequencing that are incorporated in the QC and batch effect reports. Instead, coverage information is used to assess experimental quality.
        \item Certain types of analysis would take significantly longer when
        using bisulfite sequencing data and are therefore disabled in standard analysis. Among them are the greedycut algorithm for filtering unreliable samples and sites and deviation plots as obtained in the profiles module for Infinium datasets.
    \end{itemize}

\subsection{Data Loading}

	\cs{RnBeads} expects sequencing data to be \cs{bed}-formatted or have a similar coordinate file format.
	%\cs{loading.bed.columns} option controls the assignment of various information to the column of the BED file. The methylation information can be present either as preprocessed methylation calls in one column, or as C and T counts in two columns (see \cs{loading.bed.columns} Documentation for futher details).
	The \cs{data.source} argument of the \cs{rnb.run.import()} and \cs{rnb.run.analysis()} functions for analyzing bisulfite sequencing data requires a vector or list of length 1, 2 or 3 containing:
	\begin{enumerate}
		\item the directory where the methylation \cs{bed} files are located;
		\item the sample annotation sheet
		\item the index of the column of the sample annotation sheet that contains the names or full paths to the \cs{bed} files
	\end{enumerate}

	In case only the first element of \cs{data.source} is given, \cs{RnBeads} expects the \cs{bed} files and sample annotation to be located in the same directory. The sample annotation file should contain token "\cs{sample}" in the file name. The sample annotation should contain the names of the \cs{bed} files relative to the specified directory. There should be exactly one \cs{bed} file for each sample in the directory. One of the first two elements have to be present. In case only the sample sheet is specified, one column should be giving full absolute paths of the BED-like files with sequencing information. If both elements (1) and (2) are specified, the files should reside in the directory, specified as element (1). If the third element is missing, \cs{RnBeads} attempts to find the file name-containing column automatically.

	The \cs{bed} files are expected to contain methylation and localization information for each individual site (CpG), where information includes the chromosome, coordinates (start and end coordinates) and genomic strand. Methylation information should contain any two numbers that indicate how many methylation events (\#M -- number of cytosines sequenced for a particular position), unmethylation events (\#U -- number of thymines sequenced) and total events (\#T = \#U + \#M) were measured for a particular location. This can be specified in any of the following forms:
	\begin{itemize}
		\item any two measurements among \#M, \#U and \#T;
		\item \#T and the methylation percentage: $\frac{\text{\#M}}{\text{\#U}+\text{\#M}} \times 100$
	\end{itemize}

	The package option \cs{'import.bed.columns'} controls how \cs{RnBeads} interprets the \cs{bed} files. Since there is no clear standard available for this type of methylation data, specification can be tricky. Examples for possible uncertainties are: Are the coordinates 0-based or 1-based? What chromosome and strand notation is used? Are the CpGs listed for both strands or just one? Are the coordinates on the reverse strand shifted? Is the methylation percentage specified in the interval [0,100] or [0,1]? Many more questions may arise. Hence, there is no guarantee that \cs{RnBeads} correctly loads each and every format. Probably the most convenient and safe way to load bisulfite sequencing data is to use the format presets which are tailored to particular tools and pipelines. These presets are specified in the package option \cs{'import.bed.style'}. Here are examples of the presets currently available:
	\begin{description}
		\item[\cs{'BisSNP'}] The \cs{bed} files are assumed to have been generated by the methylation calling tool \cs{BisSNP}~\cite{Liu2012}. A tab-separated file contains the chromosome name, start coordinate, end coordinate, methylation value in percent, the coverage, the strand, as well as additional information not taken into account by \cs{RnBeads}. The file should contain a header line. Coordinates are 0-based, spanning the first and the last coordinate in a site (i.e. end-start = 1 for a CpG). Sites on the negative strand are shifted by +1. Here are some example lines (genome assembly \cs{hg19}):
\begin{tiny}
\begin{lstlisting}
track name=file_sorted.realign.recal.cpg.filtered.sort.CG.bed type=bedDetail description="CG methylation
chr1    10496   10497   79.69   64      +       10496   10497   180,60,0        0       0
chr1    10524   10525   90.62   64      +       10524   10525   210,0,0 0       0
chr1    864802  864803  58.70   46      +       864802  864803  120,120,0       0       5
chr1    864803  864804  50.00   4       -       864803  864804  90,150,0        1       45
...
\end{lstlisting}
\end{tiny}

		\item[\cs{'EPP'}] The \cs{bed} files are assumed to have the format as output files from the Epigenome Processing Pipeline developed by Fabian M\"uller and Christoph Bock. A tab-separated file contains: the chromosome name, start coordinate, end coordinate, methylation value and coverage as a string ('\#M/\#T'), methylation level scaled to the interval 0 to 1000, strand, and additional information not taken into account by \cs{RnBeads}. The file should not contain a header line. Coordinates are 0-based, spanning the first coordinate in a site and the first coordinate outside the site (i.e. end-start = 2 for a CpG). Here are some example lines (genome assembly \cs{mm9}):
\begin{tiny}
\begin{lstlisting}
chr1    3010957 3010959 '27/27' 1000    +
chr1    3010971 3010973 '10/20' 500     +
chr1    3011025 3011027 '57/70' 814     -
...
\end{lstlisting}
\end{tiny}

		\item[\cs{'bismarkCov'}] The \cs{cov} files are assumed to have the format as defined by Bismark's coverage file output converted from its bedGraph output (Bismark's \cs{bismark2bedGraph} module; see the section ``Optional bedGraph output'' in the Bismark User Guide). A tab-separated file contains: the chromosome name, cytosine coordinate, cytosine coordinate (again), methylation value in percent, number of methylated reads (\#M) and the number of unmethylated reads (\#U). The file should not contain a header line. Coordinates are 1-based. Strand information does not need to be provided, but is inferred from the coordinates: Coordinates on the negative strand specify the cytosine base position (G on the positive strand). Coordinates referring to cytosines not in CpG content are automatically discarded. Here are some example lines (genome assembly \cs{hg19}):
\begin{tiny}
\begin{lstlisting}
...
chr9    73252   73252   100     1       0
chr9    73253   73253   0       0       1
chr9    73256   73256   100     1       0
chr9    73260   73260   0       0       1
chr9    73262   73262   100     1       0
chr9    73269   73269   100     1       0
...
\end{lstlisting}
\end{tiny}

		\item[\cs{'bismarkCytosine'}] The \cs{bed} files are assumed to have the format as defined by Bismark's cytosine report output (Bismark's \cs{coverage2cytosine} module; see the section ``Optional genome-wide cytosine report output'' in the Bismark User Guide). A tab-separated file contains: the chromosome name, cytosine coordinate, strand, number of methylated reads (\#M), number of unmethylated reads (\#U), and additional information not taken into account by \cs{RnBeads}. The file should not contain a header line. Coordinates are 1-based. Coordinates on the negative strand specify the cytosine position (G on the positive strand). CpG without coverage are allowed, but not required. Here are some example lines (genome assembly \cs{hg19}):
\begin{tiny}
\begin{lstlisting}
...
chr22   16050097        +       0       0       CG      CGG
chr22   16050098        -       0       0       CG      CGA
chr22   16050114        +       0       0       CG      CGG
chr22   16050115        -       0       0       CG      CGT
...
chr22   16115591        +       1       1       CG      CGC
chr22   16117938        -       0       2       CG      CGT
chr22   16122790        +       0       1       CG      CGC
...
\end{lstlisting}
\end{tiny}

		\item[\cs{'Encode'}] The \cs{bed} files are assumed to have the format as the ones that can be downloaded from UCSC's ENCODE data portal. A tab-separated file contains: the chromosome name, start coordinate, end coordinate, some identifier, read coverage (\#T), strand, start and end coordinates again (\cs{RnBeads} discards this duplicated information), some color value, read coverage (\#T) and the methylation percentage. The file should contain a header line. Coordinates are 0-based. Note that this file format is very similar but not identical to the \cs{'BisSNP'} one. Here are some example lines (genome assembly \cs{hg19}):
\begin{tiny}
\begin{lstlisting}
track name="SL1815 MspIRRBS" description="HepG2_B1__GC_" visibility=2 itemRgb="On"
chr1    1000170 1000171 HepG2_B1__GC_   62      +       1000170 1000171 55,255,0   62      6
chr1    1000190 1000191 HepG2_B1__GC_   62      +       1000190 1000191 0,255,0    62      3
chr1    1000191 1000192 HepG2_B1__GC_   31      -       1000191 1000192 0,255,0    31      0
chr1    1000198 1000199 HepG2_B1__GC_   62      +       1000198 1000199 55,255,0   62      10
chr1    1000199 1000200 HepG2_B1__GC_   31      -       1000199 1000200 0,255,0    31      0
chr1    1000206 1000207 HepG2_B1__GC_   31      -       1000206 1000207 55,255,0   31      10
...
\end{lstlisting}
\end{tiny}
	\end{description}

	Note, that in all of the formats above, CpGs may be listed on both strands individually to indicate strand-specific methylation. However, by default, \cs{RnBeads} will merge methylation information for both strands for each CpG.

	If you have a format that you feel should be supported by a preset in \cs{RnBeads} or you have trouble loading a certain format, do not hesitate to contact the \cs{RnBeads} developers.

	Assuming input file locations and annotation stored in the variable \cs{data.source}, and \cs{report.dir} specifies a valid report directory, the entire analysis pipeline can be conducted analogously to Infinium data:
<<eval=FALSE>>=
rnb.run.analysis(dir.reports=report.dir, data.source=data.source,
    data.type="bs.bed.dir")
@

	Modules can also be started individually. After executing \cs{rnb.run.import} with \cs{"bed.dir"} as data type, the resulting object is of class \cs{RnBSet}. This object is very similar to objects of its child class (\cs{RnBeadSet}) but does not contain Infinium-specific information.

	% By default, the loading module performs a small test by reading 10,000 lines from each \cs{bed} file before loading the complete dataset. The loaded test object is checked for validity and the results are written to the analysis log. In case the tests indicate serious inconsistencies, we recommend canceling the analysis and double-checking all the input parameters and options. The behavior of the loading test is controlled by the options \cs{loading.bed.test} and \cs{loading.bed.test.only}. To disable the test, set the former option to \cs{FALSE}:
% <<eval=FALSE>>=
% rnb.options(import.bed.test=FALSE)
% @
	%
	% In order to disble the actual loading and perform the test set the latter option to \cs{TRUE}:
% <<eval=FALSE>>=
% rnb.options(loading.bed.test.only=TRUE)
% @

\subsection{Quality Control}

	Sequencing depth/coverage is currently the only metric used by \cs{RnBeads}' quality control reports for bisulfite sequencing. Other quality control steps such as estimating the bisulfite conversion rate using spike-in controls, assessing the alignment rate, etc. are typically done as part of the preprocessing prior to an \cs{RnBeads} analysis.
	Several control plots are available. For example, the \emph{histogram of coverage}, \emph{genomic coverage plot} (see example in Figure~\ref{fig:genomic-covg-plot}) and \emph{violin plot}, each visualizes coverage information for individual samples. The \emph{coverage thresholds plot} gives an overview of how many sites meet the criteria for a deep coverage in a minimal fraction of the samples in the whole dataset.

	\begin{figure}
		\noindent \begin{centering}
			\includegraphics[width=0.60\textwidth]{"figures/genomic_covg_plot"}
		\par \end{centering}
		\caption{\label{fig:genomic-covg-plot} Sequencing coverage along the genome position}
	\end{figure}

\subsection{Filtering}

	The code snippet below shows an example sequence of filtering steps on a bisulfite sequencing dataset.
<<eval=FALSE>>=
# Remove CpGs on sex chromosomes
rnb.set.filtered <- rnb.execute.sex.removal(rnb.set.unfiltered)$dataset
# Remove sites that have an exceptionally high coverage
rnb.set.filtered <-
    rnb.execute.highCoverage.removal(rnb.set.filtered)$dataset
# Remove sites containing NA for beta values
rnb.set.filtered <- rnb.execute.na.removal(rnb.set.filtered)$dataset
# Remove sites for which the beta values have low standard deviation
rnb.set.filtered <-
    rnb.execute.variability.removal(rnb.set.filtered, 0.005)$dataset
@

	Executing Greedycut on large bisulfite sequencing datasets is not recommended because of its long running time.


\section{Advanced Usage}

	\cs{RnBeads} contains a variety of other useful methods and annotations that are used throughout the standard pipeline run, but that can be executed individually as well. We also present methods and functionality beyond the standard analysis pipeline. This section provides instructive code examples for \cs{RnBeads} functions that you might consider helpful. If you are curious concerning the functions used, don't hesitate to use R's help functionality (\cs{?functionName}). In addition, \cs{RnBeads}'s annotation data is briefly described here. For more information refer to the \emph{RnBeads -- Annotations} document.

	\subsection{Analysis Parameter Overview}
    	\label{sec:anaParams}
    \cs{RnBeads} offers a multitude of options that let you configure your analysis. They are introduced in this section. You can obtain an overview on the option values by the following command:
<<eval=FALSE>>=
rnb.options()
@
    If you want to inquire about a specific option, you can also use:
<<eval=FALSE>>=
rnb.getOption("analyze.sites")
@
    To set options to specific values use:
<<eval=FALSE>>=
rnb.options(filtering.sex.chromosomes.removal=TRUE,
    colors.category=rainbow(8))
@

	A complete list of available options is contained in the help pages for \cs{rnb.options}:

<<eval=FALSE>>=
?rnb.options
@

\subsection{Annotation Data}
\label{sec:annotation}

	There are four companion data packages of \cs{RnBeads}, named \cs{RnBeads.hg19}, \cs{RnBeads.mm10}, \cs{RnBeads.mm9} and \cs{RnBeads.rn5}. They contain annotations for CpG sites, array probes and predefined genomic elements. These annotation tables are used throughout the analysis. User-defined annotations can be added to focus the analysis on specific genomic regions.

	In this section, we show how to access the available annotation tables. Note that the first time annotations are requested (e.g., by calling one of the functions presented here), a set of tables is loaded. This inevitably leads to a slight delay. Subsequent calls to these functions are faster.

	Annotations used by \cs{RnBeads} (provided by separate packages listed above) can be accessed using the \cs{rnb.get.annotation()} command:
<<eval=FALSE>>=
library(RnBeads)
rnb.get.annotation(type="CpG") # all CpGs in the human genome
rnb.get.annotation(type="probes450") # all Infinium 450k probes
@

	By default they are provided as \cs{GRangesList} objects (see the \cs{GenomicRanges} package for more details). However, you can convert them to regular data frames:
<<eval=FALSE>>=
probes.450k <- rnb.annotation2data.frame(rnb.get.annotation(type="probes450"))
head(probes.450k)
@

	Annotation is also available on the genomic region level. Available region types are revealed by:
<<>>=
rnb.region.types()
@

	The default gene-related regions are of Ensembl genes (defined as the whole locus from transcription start site to transcription end site) and promoters (defined as the regions 1.5 kb upstream and 0.5 kb downstream of transcription start sites). Furthermore, CpG island and tiling regions (5kb windows) are supported. Region annotation tables can be accessed similarly to the probe annotation data:
<<eval=FALSE>>=
rnb.get.annotation("promoters",assembly="mm9")
@

	Annotation of sites and regions represented by an \cs{rnb.set} object can be accessed via the \cs{annotation()} function:
<<eval=FALSE>>=
head(annotation(rnb.set, type="sites"))
head(annotation(rnb.set, type="genes"))
@

	Here we use our example \cs{RnBeadSet} from the previous sections. Note that the rows in the resulting dataframe are in the same order as the rows returned by the \cs{meth()} and \\ \cs{rnb.execute.computeDiffMeth()} functions. Hence you can use the \cs{annotation} command in order to annotate obtained methylation or differential methylation values:
<<eval=FALSE>>=
aa <- annotation(rnb.set, type="promoters")
annotated.dmrs <- data.frame(aa, dmrs)
head(annotated.dmrs)
@

	The code above uses the \cs{rnb.set} object and the differential methylation table (\cs{dmrs}) defined in Section~\ref{sec:diffMeth}.

\subsubsection{Custom Annotation}

	Custom annotations can be included using the function \cs{rnb.set.annotation}.
	For instance, the code below retrieves annotation data for genomic enhancers as specified by a chromatin state segmentation approach employed in the ENCODE project. We retrieve the state segmentation annotations for the H1 embryonic stem cell line from the UCSC Table Browser using the \cs{rtracklayer} package. Then we restrict the resulting table to enhancer states and set the annotation for \cs{RnBeads}:

<<eval=FALSE>>=
# Retrieve the chromHMM state segmentation from UCSC
library(rtracklayer)
mySession <- browserSession()
genome(mySession) <- "hg19"
tab.chromHMM.h1 <- getTable(ucscTableQuery(mySession,
    track="wgEncodeBroadHmm", table="wgEncodeBroadHmmH1hescHMM"))

# Filter for enhancer states
tab.enhancers <- tab.chromHMM.h1[grep("Enhancer", tab.chromHMM.h1$name), ]

# Select the interesting parts of the table and rename columns
tab.enhancers <- tab.enhancers[, c("chrom", "chromStart", "chromEnd", "name")]
colnames(tab.enhancers) <- c("Chromosome", "Start", "End", "name")

# Create RnBeads annotation by providing a data.frame
rnb.set.annotation("enhancersH1EscChromHMM", tab.enhancers, assembly="hg19")

# Set the options to include the enhancer annotation
rnb.options(region.types=c(rnb.getOption("region.types"),
    "enhancersH1EscChromHMM"))
# Parse the input again, this time with the enhancer annotation added
rnb.set.enh <-
    rnb.execute.import(data.source=data.source, data.type="idat.dir")
rnb.set.enh

# Annotation and methylation levels of enhancer regions in this object
annot.enh <- annotation(rnb.set.enh, "enhancersH1EscChromHMM")
head(annot.enh)
meth.enh <- meth(rnb.set.enh, "enhancersH1EscChromHMM")
head(meth.enh)
@

	Note that the included genomic regions remain available to \cs{RnBeads} in the current R session only. If you later want to reuse custom annotation data, use the \cs{rnb.save.annotation()} and \cs{rnb.load.annotation()} functions:
<<eval=FALSE>>=
annotation.filename <- file.path(analysis.dir,
    "annotation_hg19_enhancersH1EscChromHMM.RData")
# Save the enhancer annotation to disk
rnb.save.annotation(annotation.filename, "enhancersH1EscChromHMM",
    assembly="hg19")
# Load the enhancer annotation as a duplicate
rnb.load.annotation(annotation.filename, "enhancersH1EscChromHMM_duplicate")
# Check that the annotation has been successfully loaded
rnb.region.types()
@

	On the \cs{RnBeads} website\footnote{\href{http://rnbeads.mpi-inf.mpg.de/regions.html}{http://rnbeads.mpi-inf.mpg.de/regions.html}}, we also provide a resource of region sets that are of general use. These include tiling regions of various window sizes, alternative gene models, variably methylated regions and putative regulatory regions derived from epigenome segmentation data. The code below shows how tiling regions of size 1kb can be loaded.
<<eval=FALSE>>=
rnb.load.annotation.from.db("tiling1kb", assembly="hg19")
rnb.region.types()
@

\subsection{Parallel Processing}

	If you run \cs{RnBeads} on a computational environment that supports parallel processing (e.g. a computer using multiple cores), enabling parallel computation can speed up some of the analysis modules quite significantly. Particularly the exploratory analysis and differential methylation modules profit from this as they operate on large tables and generate a large number of plots. Prior to any analysis, parallel processing can be enabled by the following command:
<<warning=FALSE>>=
logger.start(fname=NA)
parallel.isEnabled()
num.cores <- 2
parallel.setup(num.cores)
parallel.isEnabled()
@

	where \cs{num.cores} specifies the number of compute cores you would like to use. If successful, \cs{parallel.getNumWorkers()} returns the number of cores used:

<<warning=FALSE>>=
if (parallel.isEnabled()) parallel.getNumWorkers()
@

	You can disable parallel processing again using
<<warning=FALSE>>=
parallel.teardown()
@
	Note, that it is good programming practice to always disable parallel processing again, after the computation is done.

\subsection{Extra-Large Matrices, RAM and Disk Space Management}

	\cs{RnBeads} can perform analysis on much larger data sets than those presented
	in the examples above. Starting from several hundred of Infinium450k samples or
	already a few dozen WGBS profiles the data matrices in the \cs{RnBeads} objects
	become too large to keep them in main memory (RAM). To handle large amounts of
	data \cs{RnBeads} makes use of the disk-backed objects provided by the
	\cs{ff} package \cite{ffpackage}. In other words, rather than carrying around
	the huge matrices in main memory, they are stored on the hard drive. This
	enables large analysis also on machines with an intermediate amount of RAM,
	but comes at the cost of slightly increased runtimes (accessing matrices
	on the hard drive is not as fast as accessing them from the main memory). In
	order to enable this feature use:
<<eval=FALSE>>=
rnb.options(disk.dump.big.matrices=TRUE, disk.dump.bigff=TRUE)
@

	The file system location which will be used by the \cs{ff} objects is controlled by a global R option (notice that it is not an \cs{RnBeads} option!):
<<eval=FALSE>>=
options(fftempdir="/path/to/a/very/large/directory/")
@

	By default, the \cs{ff} files will be saved to the current R temporary
	directory, which is usually a subdirectory of the systems global temporary directory and can be found by executing:
<<eval=FALSE>>=
tempdir()
@

	In case of Infinium 450k data sets the package will need roughly 10 GB of hard
	drive space per 100 samples. So make sure that you have enough space for your
	analysis. You can monitor the amount of hard drive space occupied by the \cs{RnBeads} objects in the logger statements by setting:
<<eval=FALSE>>=
rnb.options(logging.disk=TRUE)
@
In order to ensure that the disk-backed objects are effectively cleaned up during the pipeline execution use the following option:
<<eval=FALSE>>=
rnb.options(enforce.memory.management=TRUE)
@

	This will force \cs{RnBeads} to use garbage collection so that no obsolete
	\cs{ff} files remain on the hard drive.
% Finally, in extreme cases, when no more than one copy of the data can be stored on the disk, (with great caution!) use the following option:
% <<eval=FALSE>>=
% rnb.options(enforce.destroy.disk.dumps=TRUE)
% @
% This option will mostly affect the disk usage during the execution complete analysis pathway.

	After setting these memory tweaking options, you can load data into \cs{RnBSet} objects in the same ways that you already know:
<<eval=FALSE>>=
rnb.set.disk <- rnb.execute.import(data.source=data.source, data.type="idat.dir")
@

	Data handling is exactly the same way as before. You can execute all functions that also work non-disk-backed \cs{RnBSet} objects.

	The \cs{RnBSet} objects created while the \cs{disk.dump.big.matrices} option
	is enabled cannot be saved using standard \cs{save} and \cs{load} methods in R. Saving should be performed as follows:
<<eval=FALSE>>=
save.dir <- file.path(report.dir, "analysis")
save.rnb.set(rnb.set.disk, path=save.dir, archive=TRUE)
@

After that you should be able to find a file \cs{analysis15.zip} in the
\cs{report.dir} directory. This object can be reloaded into another R session, possibly on a different machine:
<<eval=FALSE>>=
rns <- load.rnb.set(path=paste0(save.dir, ".zip"))
@

The \cs{ff} files behind an \cs{RnBeads} object can be deleted completely from the hard disk by executing the destructor method:
<<eval=FALSE>>=
destroy(rnb.set.disk)
@

Overall, when applied to data sets with a large number of samples or to
whole-genome bisulfite sequencing data, \cs{RnBeads} can become computationally
demanding, so one should consider running the package in an high-performance environment, e.g. on dedicated compute servers or clusters.

\subsection{Some Sugar and Recipes}

	This section introduces a few examples on how to work with \cs{RnBSet} objects
	once they have been computed. \cs{RnBeads} contains various functions that cater to downstream analysis.

\subsubsection{Plotting Methylation Value Distributions}

	\begin{figure}
		\noindent \begin{centering}
			\includegraphics[width=0.50\textwidth]{"figures/beta_density"}
		\par \end{centering}
		\caption{\label{fig:betaDensity} Density plots of probe beta values comparing hESCs to hiPSCs.}
	\end{figure}

	This example produces beta density distributions for groups of samples as in Figure~\ref{fig:betaDensity}
<<eval=FALSE>>=
rnb.plot.betadistribution.sampleGroups(meth(rnb.set),
    rnb.sample.groups(rnb.set)[["Sample_Group"]], "ESC or iPSC")
@

	\begin{figure}
		\noindent \begin{centering}
			\includegraphics[width=0.50\textwidth]{"figures/dreduction_custom"}
		\par \end{centering}
		\caption{\label{fig:dreductionCustom} The second and fifth principal components of the (unnormalized) dataset. Point colors denote sex information.}
	\end{figure}

	\subsubsection{Plotting Low Dimensional representations}

	As mentioned in Section~\ref{sec:exploratory}, RnBeads creates figures that show the high-dimensional methylation data transformed to two dimensions only, by applying the techniques MDS and PCA. Such plots can be created using the function \cs{rnb.plot.dreduction}. This function provides even greater flexibility in the visualization than what can be seen in the reports. For example, the code snippet below modifies the default colors denoting different categories and then displays the second and fifth principal components of the example dataset. Sex information is mapped to colors. The resulting plot is shown in Figure~\ref{fig:dreductionCustom}.
<<eval=FALSE>>=
theme_set(theme_bw())
rnb.options(colors.category = c("red", "blue", "grey"))
print(rnb.plot.dreduction(rnb.set, dimensions = c(2, 5),
    point.colors="Predicted Sex"))
@

	\subsubsection{Generating Locus Profile Plots (aka Genome-Browser-Like Views)}
	\begin{figure}
		\noindent \begin{centering}
			\includegraphics[width=0.99\textwidth]{"figures/locusProfile_hoxd3"}
		\par \end{centering}
		\caption{\label{fig:locusPlot} Methylation profile for the HOXB3 locus}
	\end{figure}
	\cs{RnBeads} provides functionality for generating plots of methylation levels in custom genomic intervals similarly as in current genome browsers. It makes use of the popular \cs{ggbio} package for this. The following code produces the plot depicted in Figure~\ref{fig:locusPlot}:
<<eval=FALSE>>=
# Coordinates around the HOXD3 locus
chrom <- "chr2"
start <- 177010000
end <- 177040000
sample.grouping <- rnb.sample.groups(rnb.set)[[1]]
rnb.plot.locus.profile(rnb.set, chrom, start, end, grps=sample.grouping)
@

	The \cs{rnb.section.locus.profiles()} function is very useful if you are interested in a whole set of genomic regions and want to create a corresponding \cs{RnBeads} report for easy browsing of multiple genomic locations.

	\subsubsection{Adjusting for Surrogate Variables in Differential Methylation Analysis}
	\label{recipe:SVAdjust}
	\begin{figure}
		\noindent \begin{centering}
			\includegraphics[width=0.6\textwidth]{"figures/sva_adj_pval_qqplot"}
		\par \end{centering}
		\caption{\label{fig:qqplotSVAdjust} QQ-plot comparing the negative $\log_{10}$ p-values for the unadjusted and the SV-adjusted analysis of differential methylation.}
	\end{figure}

	In section~\ref{sec:sva} you learned how you can use \cs{RnBeads} to infer Surrogate Variables (SVs). Here, we see an example on how these inferred SVs can be adjusted for in differential methylation analysis and how the results can differ compared to an unadjusted analysis. In the code below, we first infer the SVs. Then we compute tables containing metrics of differential methylation for the unadjusted and the SVA-adjusted case. Finally, we compare the p-values obtained from the \cs{limma} method using a qq-plot (see Figure~\ref{fig:qqplotSVAdjust}).

<<eval=FALSE>>=
# set the target comparison column and region types for
# differential methylation analysis
cmp.cols <- "Sample_Group"
cmp.name <- "hESC vs. hiPSC (based on Sample_Group)"
reg.types <- c("cpgislands", "promoters")
# if you have not done so yet, compute the SVs and add them to the RnBSet object
sva.obj <- rnb.execute.sva(rnb.set,cmp.cols=cmp.cols,numSVmethod="be")
rnb.set.sva <- set.covariates.sva(rnb.set, sva.obj)

# compute differential methylation tables: unadjusted and SVA adjusted
diffmeth.base <- rnb.execute.computeDiffMeth(
	rnb.set.sva, cmp.cols, region.types=reg.types,
	adjust.sva=FALSE
)
diffmeth.sva <- rnb.execute.computeDiffMeth(
	rnb.set.sva, cmp.cols, region.types=reg.types,
	adjust.sva=TRUE, pheno.cols.adjust.sva=cmp.cols
)
dm.tab.base <- get.table(diffmeth.base, cmp.name, "sites",
	return.data.frame=TRUE)
dm.tab.sva  <- get.table(diffmeth.sva,  cmp.name, "sites",
	return.data.frame=TRUE)
# compute quantiles of -log10 p-values and prepare a data.frame for plotting
p.val.perc.base <- quantile(-log10(dm.tab.base$diffmeth.p.val),
	probs = seq(0, 1, 0.01))
p.val.perc.sva  <- quantile(-log10(dm.tab.sva$diffmeth.p.val),
	probs = seq(0, 1, 0.01))
df <- data.frame(
	neg.log10.p.val.base=p.val.perc.base,
	neg.log10.p.val.sva=p.val.perc.sva,
	quantile=seq(0, 1, 0.01)
)
# plot using ggplot2
library(ggplot2)
ggplot(df,aes(x=neg.log10.p.val.base,y=neg.log10.p.val.sva,color=quantile)) +
geom_point() + geom_abline(intercept=0, slope=1) + xlim(0, 5) + ylim(0, 5)
@

	The resulting Figure~\ref{fig:qqplotSVAdjust} shows a slight inflation of the unadjusted p-values when compared to the SVA-adjusted case.

		\subsubsection{Generating Density-Scatterplots}

		\begin{figure}
			\noindent \begin{centering}
				\includegraphics[width=0.6\textwidth]{"figures/diffmeth_density_scatter"}
			\par \end{centering}
			\caption{\label{fig:diffmethDensityScatter} Density-scatterplot showing probes differentially methylated between ESCs and iPSCs.}
		\end{figure}

		The Differential Methylation reports generated by RnBeads contain scatterplots indicating point density and highlight the most likely candidates for differential methylation. You can easily produce these plots on your own using \cs{RnBeads}' \cs{create.densityScatter} function. The following example visualizes the differences between ESCs and iPSCs you have seen in section~\ref{sec:diffMeth} assuming you have already computed the \cs{diffmeth} object (see Figure~\ref{fig:diffmethDensityScatter}).

<<eval=FALSE>>=
comparison <- "hESC vs. hiPSC (based on Sample_Group)"
dframe <- get.table(diffmeth, comparison, "sites", return.data.frame=TRUE)
# define the probes with an FDR corrected p-value below 0.05
# as differentially methylated
isDMP <- dframe[,"diffmeth.p.adj.fdr"] < 0.05

create.densityScatter(dframe, is.special=isDMP, sparse.points=0.001,
	add.text.cor=TRUE) + labs(x="mean beta: hESC", y="mean beta: hiPSC") +
	coord_fixed()
@

	\subsection{Correcting for Cell Type Heterogeneity Effects}
	\label{sec:celltypeheterogeneity}

	Cell type heterogeneity of the profiled samples is a widely recognized source of confounding in DNA methylation profiling studies. Several methods have been proposed to tackle it in large data sets, some of which rely upon availability of reference methylomes~\cite{Houseman2012, Guintivano2013},   while the other can be applied directly~\cite{Houseman2014, Zou2014}. \cs{RnBeads} supports several approaches for cell type heterogeneity inference and adjustment. In case the reference methylomes are available, they can be used to estimate the relative contributions of each cell type (see Section~\ref{sec:ct-inference}). Next, the contribution estimates can be used as covariates in the limma-based differential methylation analysis.

	For the case when reference methylomes are absent, differential methylation analysis in \cs{RnBeads} can be corrected using the reference-free method by Houseman \textit{et al.}~\cite{Houseman2014}. \cs{RefFreeEWAS} analysis is fully integrated with differential methylation module of \cs{RnBeads}, and, similarly to limma-based analysis, can adjust for covariates. It can be activated using a single option:
<<eval=FALSE>>=
rnb.options(differential.site.test.method="refFreeEWAS")
@
    Keep in mind that RefFreeEWAS uses bootstrapping to estimate the
    variance of the model coefficients, and thus may take a lot of
    computational time already on moderate-sized data sets.

	Finally, \cs{RnBeads} also provides an option to export the data ready to be
	processed by the \cs{EWASHER} tool~\cite{Zou2014}. The input files as well as
	a brief guidance on how to start the \cs{EWASHER}, are prepared in a
	specialized section of the inference module report. The \cs{EWASHER} export can
	is controlled via a dedicated option:
<<eval=FALSE>>=
rnb.options(export.to.ewasher=TRUE)
@

	\subsection{Deploying RnBeads on a Scientific Compute Cluster}
	\label{sec:computeCluster}

	\nsbcr{This section is intended for advanced users only}

	\cs{RnBeads} can be deployed on a scientific compute cluster. We have implemented the necessary configurations for a Sun Grid Engine (SGE) environment.
	However, the required classes can be easily extended to fit any cluster environment. To run an RnBeads analysis on the cluster, it is necessary that the analysis options and data source are specified in an \cs{XML} file. You can find examples for such files in the ``Methylome Resource'' section of the RnBeads website. The following example illustrates how you can submit the RnBeads jobs for an analysis to the cluster. Log on to a machine which runs R and which can submit jobs to the cluster. Run R and then submit an analysis using the following commands:

<<eval=FALSE>>=
library(RnBeads)
# specify the xml file for your analysis
xml.file <- "MY_ANALYSIS_SETTINGS.XML"
# set the cluster architecture specific to your environment. We use SGE here
arch <- new("ClusterArchitectureSGE")
# initialize the object containing the RnBeads specific cluster configuration
rnb.cr <- new("RnBClusterRun",arch)
# set up the cluster so that 32GB of memory are required
# (SGE resource is called "mem_free")
rnb.cr <- setModuleResourceRequirements(rnb.cr,c(mem_free="32G"),"all")
# set up the cluster to use 4 cores on each node for all modules
rnb.cr <- setModuleNumCores(rnb.cr,4L,"all")
# set up the cluster to use 2 cores for the exploratory analysis module
rnb.cr <- setModuleNumCores(rnb.cr,2L,"exploratory")
# run the actual analysis
run(rnb.cr, "rnbeads_analysis", xml.file)
@

	If you want to configure your own cluster environment, all you have to do is to extend \cs{RnBeads}' \cs{ClusterArchitecture} S4 class. Here, all that you really need to define is the \cs{getSubCmdTokens} method, which returns a vector of command line tokens required for submitting a cluster job in your environment.  The following example illustrates the specification of a new class for a hypothetical environment which submits jobs to the cluster using the submission syntax

% \begin{small}
	\cs{iSubmit --myParameter 5 --waitForJobs [job1,job2] --jobName [MY\_JOB\_NAME]}
	\cs{--logFile [LOGFILE] '[COMMAND\_TO\_RUN]'}
% \end{small}

<<eval=FALSE>>=
# Define the new class, extending RnBeads ClusterArchitecture class
setClass("ClusterArchitectureMyEnv",
	contains = "ClusterArchitecture"
)
#define the getSubCmdTokens method
setMethod("getSubCmdTokens",
  signature(
    object="ClusterArchitectureMyEnv"
  ),
  function(
    object,
    cmd.tokens,
    log,
    job.name = "",
    res.req = character(0),
    depend.jobs = character(0)
  ) {
    dependency.token <- NULL
    if (length(depend.jobs)>0){
      dependency.token <- c( "--waitForJobs", paste(depend.jobs,collapse=","))
    }
    job.name.token <- NULL
    if (nchar(job.name)>0) {
      job.name.token <- c("--jobName",job.name)
    }
    result <- c(
      "iSubmit", "--myParamter", "5",
      dependency.token, job.name.token,
      "--logFile",log,
      paste0("'",paste(cmd.tokens,collapse=" "),"'")
    )
    return(result)
  }
)
@

	If you now want to submit an RnBeads analysis using your brand new environment all you have to do is to to replace \cs{"ClusterArchitectureSGE"} by \cs{"ClusterArchitectureMyEnv"} in the previous example.

	For the SGE example, see

<<eval=FALSE>>=
?`getSubCmdTokens,ClusterArchitectureSGE-method`
@
	It might also be helpful to have a look at the source code for the SGE class. You can find it in the source files of RnBeads (location: \cs{R/clusterArchitectureSGE.R}).
	
\subsection{Genome-wide segmentation of the methylome}

Global DNA methylation has been shown to be organized into larger domains using whole-genome bisulfite sequencing data \cite{Salhab2018}. The strategy is based on a hidden-markov model that first classifies the regions into either partially methylated domains (PMDs) and into the remaining. The latter is then further classified into fully methylated domains (FMDs), lowly methylated regions (LMRs), and unmethylated regions (UMRs) according to overall methylation state and CpG content \cite{Burger2013}. Each of these categories is annotated to a distinct biological function. \cs{RnBeads} support segmentation according to this proposed \cs{MethylSeekR} approach.

MethylSeekR is only applicable to larger WGBS datasets and produces a segmentation for each sample individually. Such a dataset is too large to be shiped together with \cs{RnBeads}, thus we assume we loaded a larger, preprocessed \cs{RnBiseqSet} object obtained from WGBS data called \cs{rnb.set}. 

<<eval=FALSE>>=
?rnb.execute.segmentation
rnb.set <- rnb.execute.segmentation(rnb.set=rnb.set,
	sample.name=samples(rnb.set)[1])
rnb.set
@

The dataset now contains further regions types representing the segmentation described above. This segmentation can now be exported to BED files or the methylation values of the CpGs in the different segments can be plotted.

<<eval=FALSE>>=
segmentation.results <- "~/segmentation"
rnb.bed.from.segmentation(rnb.set = rnb.set, 
	sample.name = samples(rnb.set)[1], store.path = segmentation.results)
plot <- rnb.boxplot.from.segmentation(rnb.set = rnb.set,
	 sample.name = samples(rnb.set)[1])+theme_bw()
@

The resulting BED file and plots will look as shown in Figure \ref{fig:segmentation}, but might vary from sample to sample.

	\begin{figure}
		\noindent \begin{centering}
			\includegraphics[width=1.00\textwidth]{"figures/segmentation"}
		\par \end{centering}
		\caption{An example BED files (left) and methylation distribution (right) according to the different segments defined by the MethylSeekR approach.}
		\label{fig:segmentation}
	\end{figure}

\section{Beyond DNA Methylation Analysis}

	\cs{RnBeads}' extensive logging features and functions which let you generate flexible hypertext reports extend the package's usefulness beyond DNA methylation analysis.

	\subsection{HTML Reports}
		\label{sec:htmlReports}

		You can easily create entire report websites containing analysis results from within R using \cs{RnBeads}. The reports' format is \cs{html} and they thus provide a useful way of keeping track of your research results and
		share them with collaborators.

		Reports need to be initialized, i.e. the directory for the report needs to be specified and stylesheets and other configuration settings need to be copied to the target directory.
<<eval=FALSE>>=
report.dir <- "RnBeads_report"
rnb.initialize.reports(report.dir)
report <- createReport(file.path(report.dir, "myreport.html"), "An Eye Opener",
    page.title="Fascinating Report", authors=c("Me", "You"))
@
    	The last command in the above example creates an object of class \cs{Report}:
<<eval=FALSE>>=
str(report)
@
		The command \cs{rnb.initialize.reports} creates the \cs{configuration} directory (usually shared among several reports), containing common icons, as well as files that define the behaviour, visual style and layout of the report pages. Also, notice that the \cs{RnBeads\_report} directory now contains other subdirectories according to the following slots of the \cs{report} object:
	    \begin{description}
	        \item[\cs{dir.data}] Directory in which data is placed that is linked to in the report. This can be any data.
	        \item[\cs{dir.pngs}] Directory in which image files in \cs{png} format are placed. These are files that are generated by methods like \cs{createReportPlot()}. They should be relatively small in file size such that they can easily displayed on the web.
	        \item[\cs{dir.pdf}] Directory in which pdf versions of plots will be stored.
	        \item[\cs{dir.high}] Directory in which high resolution versions of plots will be stored. They can be larger in terms of file sizes.
	    \end{description}

	    If you view \cs{myreport.html} in your favorite browser you will notice that it contains titles, but no real content as of yet. So, let us change that by adding a section with some paragraphs:

<<eval=FALSE>>=
stext <- "Here is some text for our awesome report"
report <- rnb.add.section(report, "Adding a text section", stext)
lorem1 <- c("Lorem ipsum dolor sit amet, consectetur adipiscing elit.
            Maecenas vestibulum placerat lobortis. Ut viverra fringilla
            urna at rutrum. In hac habitasse platea dictumst. Vestibulum
            adipiscing rutrum libero et interdum. Etiam sed odio ac nibh
            ullamcorper congue. Proin ac ipsum elit. Ut porta lorem sed
            lorem molestie pharetra.",
            "Vestibulum ante ipsum primis in faucibus orci luctus et
            ultrices posuere cubilia Curae; Cras ac augue eu turpis
            dignissim aliquam. Vivamus at arcu ligula, vel scelerisque nisi.
            Vivamus ac lorem libero, quis venenatis metus. Fusce et lectus
            at lectus vestibulum faucibus ac id sem.")
report <- rnb.add.section(report, "A subsection", lorem1, level=2)
lorem2 <- "Nunc congue molestie fringilla. Aliquam erat volutpat.
           Integer consequat turpis nec dolor pulvinar et vulputate magna
           adipiscing. Curabitur purus dolor, porttitor vel dapibus quis,
           eleifend at lacus. Cras at mauris est, quis aliquam libero.
           Nulla facilisi. Nam facilisis placerat aliquam. Morbi in odio
           non ligula mollis rhoncus et et erat. Maecenas ut dui nisl.
           Mauris consequat cursus augue quis euismod."
rnb.add.paragraph(report, lorem2)
rnb.add.paragraph(report, "TODO: Add content here", paragraph.class="task")
rnb.add.paragraph(report, "To be or not to be, that is the question",
    paragraph.class="note")
@
    	Again, view \cs{myreport.html} and notice that it has been filled with content. Other objects, like lists and tables can be added as well:

<<eval=FALSE>>=
report <- rnb.add.section(report, "Lists and Tables", "")
ll <- lapply(LETTERS[1:10], function(x) { paste(rep(x, 3), collapse="") })
rnb.add.list(report, ll, type="o")
tt <- matrix(sapply(LETTERS[1:24],
    function(x) { paste(rep(x, 3), collapse="") }), ncol=4)
colnames(tt) <- paste("Col", 1:4)
rownames(tt) <- paste("Row", 1:6)
rnb.add.table(report, tt, row.names=TRUE, first.col.header=TRUE,
    tcaption="A table")
@
    	Note that text will be placed in the report as \cs{html} text. Hence you can format text and add it to the report:

<<eval=FALSE>>=
stext <- c('<p>Some German umlauts:
                <ul>
                    <li>&Auml;</li>
                    <li>&Ouml;</li>
                    <li>&Uuml;</li>
                </ul>
            </p>',
            '<p> A link: <a href="https://rnbeads.org/">
            RnBeads website</a></p>')
report <- rnb.add.section(report, "HTML code", stext)
@

		Now, only text is boring. So, let us add some eye candy, i.e. some plots:

<<eval=FALSE>>=
report <- rnb.add.section(report, "Plots", "")
report.plot <- createReportPlot("hist_normal", report, create.pdf=TRUE,
    high.png=200)
hist(rnorm(1000), breaks=50, col="blue")
off(report.plot)
desc <- 'A histogramm of samples drawn from the normal distribution
         Click <a href="myreport_pdfs/hist_normal.pdf">
         here</a> for the pdf version.'
report <- rnb.add.figure(report, desc, report.plot)
@
		Now, \cs{myreport.html} will contain a figure. If created as in the case above, the high resolution version of it is accessible by clicking on the figure or in the \cs{myreport\_images} directory. A \cs{pdf} version will be available in the \cs{myreport\_pdfs} directory.

		In bioinformatics analysis, inspecting plots with multiple parameter settings is a common task. \cs{RnBeads}' reports can handle these using dropdown menus for possible values of different parameters and showing the corresponding plot file. Here's an example:

<<eval=FALSE>>=
# Plot a sine curve
plotSine <- function(a, b, fname) {
    report.plot <- createReportPlot(fname, report, create.pdf=FALSE,
        high.png=200)
    curve(sin(a*x)+b, -2*pi, 2*pi, ylim=c(-2, 2), col="blue", lwd=2)
    return(off(report.plot))
}

# Set parameters for different sine curves
period.params <- c(a05=0.5, a1=1, a2=2)
intercept.params <- c(im1=-1, i0=0, i1=1)
plot.list <- list()
for (aa in names(period.params)){
    for (bb in names(intercept.params)){
        fname <- paste("sinePlot", aa, bb, sep="_")
        current.plot <- plotSine(period.params[aa], intercept.params[bb],
            fname)
        plot.list <- c(plot.list, current.plot)
    }
}
setting.names <- list('period parameter'=period.params,
    'intercept parameter'=intercept.params)
description <- 'Sine for various parameters'
report <- rnb.add.figure(report, description, plot.list, setting.names,
     selected.image=5)
@

		Finally, close the report, which adds the footer to the page:

<<eval=FALSE>>=
off(report)
@



	\subsection{Logging}
	In order to keep track of what your program is doing, logging is often essential. \cs{RnBeads}' logging functionality is implemented via a few easy-to-handle methods that can be integrated in any code (not just bioinformatics). In this section, we introduce a few examples on how to use the logging capabilities.

	First, a logger needs to be initialized:
<<warning=FALSE>>=
logger.start("Logging tutorial", fname=NA)
@

	will initialize a logger with its standard output being the console. You can also specify a file name for \cs{fname} to redirect the logging output to a file.
<<warning=FALSE>>=
logger.isinitialized()
@

	will tell you whether a logger has been initialized. Whenever you start a particular task, you can tell the logger to initialize a new section:
<<warning=FALSE>>=
logger.start("Some tedious task")
@

	By default, the logging message is automatically appended to the initialized logger. When you complete your task, just tell the logger:
<<warning=FALSE>>=
logger.completed()
@
    Notice the indentation of ``Some tedious task''. You can nest as many subtasks as you like:
<<warning=FALSE>>=
f <- function(){
    logger.start("Another tedious task")
    Sys.sleep(2)
        logger.start("Some tedious subtask")
        Sys.sleep(2)
        logger.completed()
        logger.start("Another, more tedious subtask")
        Sys.sleep(2)
            logger.start("Some tedious subsubtask")
            Sys.sleep(2)
            logger.completed()
            logger.start("Some even more tedious subsubtask")
            Sys.sleep(3)
            logger.completed()
        logger.completed()
    logger.completed()
}
f()
@

	The logging capabilities also include status messages in various flavors:
	\begin{description}
		\item[info] informs the user of a particular program setting, e.g. the current value of a particular variable. The information in the message is typically dependent on the program input.
		\item[status] informs the user of which part of a program has been reached. These messages are typically fixed strings.
		\item[warning] warns the user that something did not go as expected.
		\item[error] a more severe form of a warning: something went terribly wrong. Causes an error to be thrown which terminates the program if not caught.
	\end{description}
	Here is an example function that uses the logging functionality
<<warning=FALSE>>=
draw.lotto <- function(count=6) {
	urn <- 1:49
	primes <- c(2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47)
	logger.start(c("Drawing", count, "numbers from", length(urn)))
	for (i in 1:count) {
		if (i > 9) {
			msg <- "Too many numbers drawn"
			logger.error(msg, terminate=FALSE)
			stop(msg)
		}
		drawn <- sample(urn, 1)
		urn <- setdiff(urn, drawn)
		logger.info(c("The next number is", drawn))
		if (drawn %in% primes) {
			logger.warning("A prime number was drawn")
		}
	}
	logger.completed()
}
draw.lotto()
#draw.lotto(15)
@
    Note that setting the parameter \cs{terminate} in \cs{logger.error()} to \cs{TRUE} will cause R to terminate. The logger does not automatically generate R warnings and errors.


% <<echo=FALSE>>=
% unlink(report.dir, recursive=TRUE)
% @

    \begin{thebibliography}{10}
      \bibitem{Ziller2011} Ziller, M. J., M\"uller, F., Liao, J., Zhang, Y., Gu, H., Bock, C., Boyle, P., \emph{et al.} (2011). \newblock \emph{Genomic Distribution and Inter-Sample Variation of Non-CpG Methylation across Human Cell Types.} \newblock PLoS Genetics, 7(12)
      \bibitem{Nazor2012} Nazor, K.L., Altun, G., Lynch, C., Tran, H., Harness, J.V., Slavin, I., Garitaonandia, I. \emph{et al.} (2012), \newblock \emph{Recurrent Variations in DNA Methylation in Human Pluripotent Stem Cells and Their Differentiated Derivatives.} \newblock Cell Stem Cell, 10(5),620-634
    %\bibitem{genomeStudio} \newblock \emph{GenomeStudio Methylation Module v1.8. User Guide.} \newblock Illumina, 13(7), R44
	  \bibitem{Makambi2003} Makambi, K. (2003). \newblock \emph{Weighted inverse chi-square method for correlated significance tests.} \newblock Journal of Applied Statistics, 30(2), 225-234
	  \bibitem{Maksimovic2012} Maksimovic, J., Gordon, L., Oshlack, A. (2012). \newblock \emph{SWAN: Subset quantile Within-Array Normalization for Illumina Infinium Infinium 450k BeadChips.} \newblock Genome Biology, 13(6), R44
	  \bibitem{Liu2012} Liu, J., Siegmund, K., Laird, P., Berman, B. (2012). \newblock \emph{Bis-SNP: combined DNA methylation and SNP calling for Bisulfite-seq data} \newblock Genome Biology, 13(7), R44
	  \bibitem{Pidsley2013} Pidsley, R., Wong, C.,  Volta, M., Lunnon, K., Mill,
	    J., and Schalkwyk, L. (2013)  \newblock \emph{A data-driven approach to preprocessing Illumina 450K methylation array data. BMC Genomics},14(1), 293
      \bibitem{ffpackage} Adler, D., Gl{\"a}ser, C., Nenadic, O., Oehlschl{\"a}gel, J.
	    and Zucchini, W. (2012) \newblock \emph{ff: memory-efficient storage of
	    large data on disk and fast access functions. R package version 2.2-12.} http://ff.r-forge.r-project.org
      \bibitem{methylumi}Davis, S., Du, P., Bilke, S., Triche, T., Bootwalla, M.
	    (2013) \newblock \emph{methylumi: Handle Illumina methylation data. R
	    package version 2.6.1}.
	  \bibitem{minfi}  Hansen, K.D., Aryee, M \newblock \emph{minfi: Analyze
	    Illumina's 450k methylation arrays. R package version 1.6.0.}
  	  \bibitem{Teschendorff2013beta} Teschendorff, A.,Marabita, F., Lechner, M., Bartlett, T., Tegner, J., Gomez-Cabrero, D., and Beck, S. (2013).\newblock \emph{A beta-mixture quantile normalization method for correcting probe design bias in Illumina Infinium 450 k DNA methylation data.} \newblock Bioinformatics, 29(2),189-196
  	  \bibitem{Triche2013} Triche, T.J., Jr., Weisenberger, D.J., Van Den Berg,
  	  D., Laird, P.W., and Siegmund, K.D. (2013). \newblock \emph{Low-level processing of Illumina Infinium DNA Methylation BeadArrays}. \newblock Nucleic Acids Res, 41(7), e90
  	  \bibitem{Leek2007sva} Leek, J. T., Storey, J. D. (2007). \newblock \emph{Capturing Heterogeneity in Gene Expression Studies by Surrogate Variable Analysis}. \newblock PLoS Genetics, 3(9), 1724-1735.
  	  \bibitem{Leek2012sva} Leek, J. T., Johnson, W. E., Parker, H. S., Jaffe, A. E., Storey, J. D. (2012). \newblock \emph{The sva package for removing batch effects and other unwanted variation in high-throughput experiments}. \newblock Bioinformatics, 28(6), 882-883.
      \bibitem{Houseman2012} Houseman, E.A., Accomando W.P., Koestler D.C.,Christensen B.C., Marsit C.J., Nelson H.H., Wiencke J.K., Kelsey K.T.
      (2012). \newblock \emph{DNA methylation arrays as surrogate measures of
      cell mixture distribution}. \newblock BMC Bioinformatics 13:86.
      \bibitem{Guintivano2013} Guintivano J., Aryee M.J., Kaminsky Z.A.
      (2013). \newblock \emph{A cell epigenotype specific model for the
      correction of brain cellular heterogeneity bias and its application to age, brain region and major depression}. \newblock Epigenetics, 8(3), 290-302.
      \bibitem{Ashburner2000} Ashburner, M., Ball, C. A., Blake, J. A., Botstein, D., Butler, H., Cherry, J. M., \emph{et al.} (2000). \newblock \emph{Gene ontology: tool for the unification of biology.}  \newblock Nature Genetics, 25(1), 25-29
      \bibitem{Sheffield2016} Sheffield, N. C., and Bock, C. (2016). \newblock \emph{LOLA: enrichment analysis for genomic region sets and regulatory elements in R and Bioconductor}. \newblock Bioinformatics, 32(4), 587-589
      \bibitem{Falcon2007} Falcon, S., and Gentleman, R. (2007). \newblock \emph{Using GOstats to test gene lists for GO term association}. \newblock Bioinformatics, 23(2), 257-258
      \bibitem{Houseman2014} Houseman, E. A., Molitor, J., and Marsit, C. J.
      (2014). \newblock \emph{Reference-Free Cell Mixture Adjustments in Analysis of DNA Methylation Data}. \newblock Bioinformatics, btu029.
      \bibitem{Zou2014}Zou, J., Lippert, C., Heckerman, D., Aryee, M., and
      Listgarten, J. (2014). \newblock \emph{Epigenome-wide association studies without the need for cell-type composition}. \newblock Nature Methods. 11, 309-311.
      \bibitem{Fortin2014} Fortin, J.-P., Labbe, A., Lemire, M., Zanke, B. W., Hudson, T. J., Fertig, E. J., \emph{et al.} (2014). \newblock \emph{Functional normalization of 450k methylation array data improves replication in large cancer studies}. \newblock Genome Biology, 15(12), 503
      \bibitem{Aran2015} Aran, D., Sirota, M., Butte, A.J. (2014). \newblock \emph{Systematic pan-cancer analysis of tumour purity}. \newblock Nature Communications, 6, 8971
      \bibitem{Salhab2018} Salhab, A., Nordstr\"om, K., Gasparoni, G., Kattler, K., Ebert, P., et.al. (2018) \newblock \emph{A comprehensive analysis of 195 DNA methylomes reveals shared and cell-specific features of partially methylated domains.} \newblock Genome Biology, 19(1), 9-11.
      \bibitem{Burger2013} Burger, L., Gaidatzis, D., Sch\"ubeler, D., and Stadler, M. B. (2013). \newblock\emph{Identification of active regulatory regions from DNA methylation data}. \newblock Nucleic Acids Research, 41(16), e155. https://doi.org/10.1093/nar/gkt599
	\end{thebibliography}

\end{document}

%\VignetteIndexEntry{RnBeads Annotation}
%\VignetteKeywords{analysis, methylation, RnBeads}

\documentclass[a4paper]{scrartcl}
\usepackage[OT1]{fontenc}
\usepackage{hyperref}
\usepackage{color}
\usepackage[table]{xcolor}
\usepackage{vmargin}
\usepackage[english,american]{babel}
\usepackage{fancyhdr}

\selectlanguage{american}
\definecolor{darkblue}{rgb}{0.0,0.0,0.3}
\hypersetup{colorlinks,breaklinks,
            linkcolor=darkblue,urlcolor=darkblue,
            anchorcolor=darkblue,citecolor=darkblue}
\setpapersize{A4}
\setmargins{2.5cm}{2.0cm}% % left margin and upper margin
           {16cm}{23cm}%   % text width and height
           {60pt}{22pt}%   % Header height and margin
           {0pt}{30pt}%    % \footheight (egal) and footer margin

%%%%%%%%%%%%%%%%% Some style sttings %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\newcommand{\insertemptypage}[0]{\newpage \thispagestyle{empty} \vspace*{\fill}}

% Layout
\newcommand{\emphr}[1]{\textcolor{red}{\sffamily{\textbf{#1}}}}
\newcommand{\itemr}[1]{\item \emphr{#1} \\}
\newcommand{\ns}[1]{{\sffamily #1}}  % non-serif font
\newcommand{\nsb}[1]{\ns{\textbf{#1}}}
\newcommand{\nsbc}[1]{\nsb{\textcolor{blue}{#1}}}
\newcommand{\nsbcr}[1]{\nsb{\textcolor{red}{#1}}}
\newcommand{\cs}[1]{\texttt{#1}}    % code style

\newcommand{\vocab}[1]{\textit{\ns{#1}}}
\definecolor{grey}{rgb}{0.6,0.6,0.6}
\definecolor{backgroundtabheader}{rgb}{0.7,0.7,0.7}
\definecolor{backgroundtabrow1}{rgb}{1,1,1}
\definecolor{backgroundtabrow2}{rgb}{0.9,0.9,0.9}
% Ueberschriften
\renewcommand*{\sectfont }{\color{blue} \sffamily \bfseries}
%\renewcommand{\thesection}{\Alph{section}}

%neuer absatz
\newcommand{\newpara}[0]{\vskip 10pt}

%simple hyperrefs without having to type the link double
\newcommand{\hrefs}[1]{\href{#1}{#1}}

% Mathematik-Modus
% Mengen
\newcommand{\setN}[0]{\ensuremath{\mathds{N}}}
\newcommand{\setR}[0]{\ensuremath{\mathds{R}}}
\newcommand{\setZ}[0]{\ensuremath{\mathds{Z}}}
\newcommand{\set}[1]{\left\{#1\right\}}
\newcommand{\tupt}[1]{\ensuremath{\langle}#1\ensuremath{\rangle}}

% Statistics
\newcommand{\pr}[0]{\ensuremath{\text{Pr}}}
\newcommand{\E}[0]{\ensuremath{\text{E}}}
\newcommand{\var}[0]{\ensuremath{\text{Var}}}
\newcommand{\cov}[0]{\ensuremath{\text{Cov}}}
\newcommand{\sd}[0]{\ensuremath{\text{sd}}}
% Statistical Independence
\newcommand\independent{\protect\mathpalette{\protect\independenT}{\perp}}
\def\independenT#1#2{\mathrel{\setbox0\hbox{$#1#2$}%
\copy0\kern-\wd0\mkern4mu\box0}}

% Unsicher
\newcommand{\chk}[0]{\textcolor{red}{(\textbf{???)}}}
\newcommand{\chktxth}[1]{\textcolor{red}{(\textbf{???} #1 \textbf{???})}}
\newcommand{\chkside}[0]{\marginpar{\textcolor{red}{(\textbf{???)}}}}
\newcommand{\chktxt}[1]{\marginpar{\textcolor{red}{(\textbf{???} #1 \textbf{???})}}}

\newcommand{\todo}[1]{\marginpar{\textcolor{red}{\textbf{TODO:}\newline #1}}}
\newcommand{\todomark}[0]{\marginpar{\textcolor{red}{\textbf{TODO}}}}
\newcommand{\chkm}[0]{\textcolor{red}{(\textbf{???)}} \marginpar{\textcolor{red}{\textbf{TODO}}}}
\newcommand{\ph}[1]{\paragraph{} \textcolor{red}{[ #1 ]}} %placeholder

\newcommand{\ra}[0]{$\rightarrow$}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\lhead{\cs{RnBeads Annotations}}
\rhead{\thepage}
\cfoot{}

\pagestyle{fancy}

\begin{document}

\title{\cs{RnBeads} -- Annotations}

\author{Yassen Assenov, Fabian M\"uller, Pavlo Lutsik, Michael Scherer}

\thispagestyle{empty}
\maketitle

\section{Introduction}

\cs{RnBeads} is accompanied by four annotation packages: \cs{RnBeads.hg19}, \cs{RnBeads.mm10}, \cs{RnBeads.mm9}, \cs{RnBeads.rn5}. They contain genomic locus and Infinium probe annotation tables, as well as definition tables for tiling regions, genes, promoters and CpG islands. The \cs{RnBeads} package contains routines for handling and updating these annotations. The vignette named \textbf{RnBeads -- Comprehensive DNA Methylation Analysis} also contains examples that use functions presented here.\\

The data packages described above are hosted on the \cs{RnBeads} website\footnote{\href{https://rnbeads.org/}{rnbeads.org}}. Note that at least one of the data packages should be installed prior to installing \cs{RnBeads}.

\section{Data Extraction and Processing}

\subsection{Sites}

Currently, every data package examines cytosines in the context of CpG and contains an annnotation table of all CpGs in the respective genome. CpG density and GC content are also computed for the neighborhood of length 100 base pairs centered on each locus. The total number of dinucleotides annotated in HG19 is 28,217,009 represented both on the forward and reverse DNA strands.

\subsection{HumanMethylation450 Probes}

The Infinium HumanMethylation450 BeadChip interrogates over 470,000 loci in the human genome. Control and methylation probe annotation is obtained from online sources, validated for consistency and enriched with locations for the SNP-associated loci obtained from dbSNP. In addition, the annotation includes information about CpG density and GC content of the sequence neighborhood around every probe target as described in the section above.

\subsection{dbSNP}

Single-Nucleotide Polymorphisms (SNPs) in the human genome are downloaded from the respective FTP directory of dbSNP\footnote{\href{http://www.ncbi.nlm.nih.gov/snp/organisms}{www.ncbi.nlm.nih.gov/snp/organisms/}}. \cs{RnBeads} uses the provided VCF files; for mouse and rat these are one file per chromosome. The following table shows the references in dbSNP used for the supported assemblies. \\

\begin{tabular}{|c|l|}
\hline
\multicolumn{1}{|c}{\textbf{Package}} & \multicolumn{1}{c|}{\textbf{Reference}} \\
\hline
RnBeads.hg19 & GRCh37.p10 \\
RnBeads.mm10 & GCF\_000001635.21 \\
RnBeads.rn5  & GCF\_000001895.4 \\
\hline
\end{tabular} \\

The data package \cs{RnBeads.mm9} does not include SNP-related information. Only SNPs that meet all of the following criteria are considered:

\begin{enumerate}
	\item \textbf{assembly} is \texttt{GRCh37.p5,reference};
	\item \textbf{chrom} is the expected chromosome;
	\item \textbf{allele origin} is not \texttt{somatic};
	\item (for HG19) major allele frequency is at most 0.95.
\end{enumerate}

These SNP positions are used to enrich the annotations for CpG dinucleoties and Infinium probes.

\subsection{Regions}

Every data package defines the following sets of regions for the dedicated assembly:
\begin{description}
	\item[Tiling regions]{Tiling regions with a window size of 5 kilobases are defined over the whole genome.}
	\item[Genes and promoters]{Ensembl\footnote{\href{http://www.ensembl.org/}{www.ensembl.org/}} gene definitions are downloaded using the \cs{biomaRt} package. A promoter is defined as the region spanning 1,500 bases upstream and 500 bases downstream of the transcription start site of the corresponding gene.}
	\item[GpG islands]{The CpG island track is downloaded from the dedicated FTP directory of the UCSC Genome Browser\footnote{\href{http://genome.ucsc.edu/}{genome.ucsc.edu/}}.}
\end{description}
CpG density and GC content are computed for all region types listed above.

\section{Annotations}

<<echo=FALSE,warning=FALSE>>=
suppressPackageStartupMessages(library(RnBeads))
#for knitr: avoid tidy because it messes up line breaks
# opts_chunk$set(tidy=FALSE)
@

\subsection{Genome Assemblies}

Every annotation table and other data structure is specific to a genome assembly. The list of supported assemblies can be obtained by the calling the function \cs{rnb.get.assemblies}, as shown in the example below.

<<warning=FALSE>>=
rnb.get.assemblies()
@

\subsection{Built-in Annotation Tables}

Control probe information for the Illumina 450K array is stored (and can be extracted) as a \cs{data.frame} in which every row corresponds to a control probe. Every other annotation table is represented by a dedicated \cs{GRangesList} object, that is, a list of consistent \cs{GRanges} objects, one per chromosome. Every instance of \cs{GRanges} defines the genomic positions of the corresponding sites, probes or regions. Identifiers, if present, can be obtained using the \cs{names} method. Strand information is also included when applicable. Any additional annotation is stored as metadata. Please refer to the documentations of the \cs{GenomicRanges} package and the \cs{GRanges} class for more details.\\

\begin{table}
\centering
\rowcolors{2}{backgroundtabrow2}{backgroundtabrow1}
\begin{tabular}{|c|p{5.1cm}|p{7cm}|}
\hline
\multicolumn{1}{|c}{\textbf{No.}} & \multicolumn{1}{c}{\textbf{Column Name}} & \multicolumn{1}{c|}{\textbf{Description}} \\
\hline
1 & ID                 & Probe identifier. \\
2 & Target             & Probe category. \\
3 & Color              & Probe color. \\
4 & Description        & Probe description, abbreviated. \\
5 & AVG                & ... \\
6 & Evaluate Green     & If probe is used in the evaluation of the green channel. \\
7 & Evaluate Red       & If probe is used in the evaluation of the red channel. \\
8 & Expected Intensity & Expected intensity of the probe. \\
9 & Sample-dependent   & Flag indicating if the intensity of the probe is sample-dependent. \\
10 & Index             & Index of the probe in its category. \\
\hline
\end{tabular}
\caption{Columns in the HumanMethylation450 control probe annotation table.}
\label{tab:controls}
\end{table}

In an R session, every annotation object can be accessed using the function \cs{rnb.get.annotation}. The code snippet below shows how the control probe annotations can be extracted. Table~\ref{tab:controls} briefly describes the characteristics of these probes.

<<eval=FALSE>>=
control.annotation <- rnb.get.annotation("controls450")
head(control.annotation)
@

\begin{table}
\centering
\rowcolors{2}{backgroundtabrow2}{backgroundtabrow1}
\begin{tabular}{|c|p{5.1cm}|p{7cm}|}
\hline
\multicolumn{1}{|c}{\textbf{No.}} & \multicolumn{1}{c}{\textbf{Column Name}} & \multicolumn{1}{c|}{\textbf{Description}} \\
\hline
1 & Design             & Probe design type. \tabularnewline
2 & Color              & Color channel. \tabularnewline
3 & Context            & Probe context. \tabularnewline
4 & Random             & Flag indicating if the probe\'s location was randomly chosen. \tabularnewline
5 & HumanMethylation27 & Flag indicating if the probe is also covered by HumanMethylation27k assay. \tabularnewline
6 & Mismatches A       & Number of base mismatches between the provided and expected probe sequence. \tabularnewline
7 & Mismatches B       & Number of base mismatches between the provided and expected probe sequence. \tabularnewline
8 & CGI Relation       & Relation to a CpG island. \tabularnewline
9 & CpG                & Number of CpG dinucleotides in the sequence neighborhood of the target. \tabularnewline
10 & GC                & Percentage of C and G bases in the sequence neighborhood of the target. \tabularnewline
11 & SNPs 3            & Number of SNPs that overlap with the last 3 bases of a probe's target sequence. \tabularnewline
12 & SNPs 5            & Number of SNPs that overlap with the last 5 bases of a probe's target sequence. \tabularnewline
13 & SNPs Full         & Number of SNPs that overlap with the probe's sequence. \\
\hline
\end{tabular}
\caption{Metadata accompanying the HumanMethylation450 probe definitions.}
\label{tab:probes}
\end{table}

In a similar manner, the command below extracts the annotation of HumanMethylation450 probes. The metadata structure for the methylation and SNP-associated probes is presented in Table~\ref{tab:probes}.

<<eval=FALSE>>=
probe.annotation <- rnb.get.annotation("probes450")
probe.annotation
@

\begin{table}
\centering
\rowcolors{2}{backgroundtabrow2}{backgroundtabrow1}
\begin{tabular}{|c|p{5.1cm}|p{7cm}|}
\hline
\multicolumn{1}{|c}{\textbf{No.}} & \multicolumn{1}{c}{\textbf{Column Name}} & \multicolumn{1}{c|}{\textbf{Description}} \\
\hline
1 & symbol   & Gene symbols associated with this region. \\
2 & entrezID & Entrez gene identifiers associated with this region. \\
3 & CpG      & Number of CpG dinucleotides in the region. \\
4 & GC       & Total number of C and G bases in the region. \\
\hline
\end{tabular}
\caption{Metadata accompanying the Ensembl gene and promoter definitions.}
\label{tab:genes}
\end{table}

Every data package integrates various region types with the CpG sites (and Infinium probes for HG19) presented in the previous section. Obtaining a region annotation information is very similar to accessing site or probe annotation. The code snippet below extracts the available gene definitions. Table~\ref{tab:genes} provides information about the metadata of these regions.

<<eval=FALSE>>=
gene.annotation <- rnb.get.annotation("genes")
attr(gene.annotation, "version")
gene.annotation
@

The function \cs{rnb.region.types} returns all supported region definitions for a given genome assembly. These currently include \cs{"cpgislands"}, \cs{"genes"}, \cs{"promoters"}, \cs{"tiling"}.

The \cs{GRangesList} objects are well optimized for storage and fast access of genomic elements. However, exporting such objects to tables in human-readable form is not trivial. The function \cs{rnb.annotation2data.frame} provides an easy way to convert a probe or region annotation to a single table. The code snippet below exemplifies this function by saving the gene promoter definitions to a comma-separated value (CSV) file named \cs{promoters.csv}.

<<eval=FALSE>>=
promoters <- rnb.annotation2data.frame(rnb.get.annotation("promoters"))
write.csv(promoters, file = "promoters.csv", row.names = FALSE)
@

\subsection{Custom Annotations}

Custom region annotations can be included using the function \cs{rnb.set.annotation}. The information is either loaded from a BED file, or it must be presented as a \cs{data.frame} containing at least the following three columns - \textbf{chromosome}, \textbf{start} and \textbf{end}. Note that the included genomic regions remain available in the current R session only. \\

The vignette named \textbf{RnBeads -- Comprehensive DNA Methylation Analysis} contains a detailed example for including custom annotations to an analysis.

\section{Data Sources}

The data packages accompanying \cs{RnBeads} obtained data for the annotation tables from the following sources:
\begin{description}
	\item[Other Bioconductor packages] Methylation and control probe annotation is obtained from the package \href{http://www.bioconductor.org/packages/release/data/annotation/html/FDb.InfiniumMethylation.hg19.html}{FDb.InfiniumMethylation.hg19}. In addition, \cs{RnBeads} uses the packages:
	\begin{description}
		\item \href{http://www.bioconductor.org/packages/release/data/annotation/html/BSgenome.Hsapiens.UCSC.hg19.html}{BSgenome.Hsapiens.UCSC.hg19}
		\item \href{http://www.bioconductor.org/packages/release/data/annotation/html/BSgenome.Mmusculus.UCSC.mm9.html}{BSgenome.Mmusculus.UCSC.mm9}
		\item \href{http://www.bioconductor.org/packages/release/data/annotation/html/BSgenome.Mmusculus.UCSC.mm10.html}{BSgenome.Mmusculus.UCSC.mm10}
		\item \href{http://www.bioconductor.org/packages/release/data/annotation/html/BSgenome.Rnorvegicus.UCSC.rn5.html}{BSgenome.Rnorvegicus.UCSC.rn5}
	\end{description}
	to validate the probe annotations and to enrich all annotation tables with sequence-based properties.
	\item[CRAN packages] Additional control probe information is obtained from the package \href{http://cran.r-project.org/web/packages/HumMeth27QCReport/index.html}{HumMeth27QCReport}.
	\item[dbSNP] Information on SNPs in the human genome is downloaded as BED files from the \href{ftp://ftp.ncbi.nih.gov/snp/organisms/}{FTP site of dbSNP}.
	\item[Gene Expression Omnibus (GEO)] Methylation and control probe annotation is obtained from GEO record \href{http://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GPL13534}{GPL13534} and validated for consistency with the other sources.
	\item[Biomart] \href{http://www.ensembl.org/}{Ensembl} gene definitions (Ensembl versions 73 (for hg19, mm10 and rn5), and 67 (for mm9) are downloaded using the \href{http://www.bioconductor.org/packages/release/bioc/html/biomaRt.html}{biomaRt} package.
	\item[UCSC Genome Browser] The CpG island tracks are downloaded from the \href{ftp://hgdownload.cse.ucsc.edu/goldenPath/}{FTP site of the UCSC Genome Browser}.
\end{description}

\end{document}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plottingUtils.R
\name{create.hex.summary.plot}
\alias{create.hex.summary.plot}
\title{create.hex.summary.plot}
\usage{
create.hex.summary.plot(
  df2p,
  x = colnames(df2p)[1],
  y = colnames(df2p)[2],
  q = colnames(df2p)[3],
  bins = 128,
  fun = median,
  ...
)
}
\arguments{
\item{df2p}{\code{data.frame} to be plotted.}

\item{x}{name of the variable in \code{df2p} considered as x-axis}

\item{y}{name of the variable in \code{df2p} considered as y-axis}

\item{q}{name of the variable in \code{df2p} considered as quantity to be summarized over bins}

\item{bins, fun, ...}{arguments to be passed on to \code{stat_summary_hex}}
}
\value{
\code{ggplot} object
}
\description{
Creates a summary plot binning the data given by a certain quantity in heagonal bins
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/controlPlotsBiSeq.R
\name{rnb.plot.num.sites.covg}
\alias{rnb.plot.num.sites.covg}
\title{rnb.plot.num.sites.covg}
\usage{
rnb.plot.num.sites.covg(
  rnbs,
  addSampleNames = (length(samples(rnbs)) < 100),
  bar.percentiles = c(0.25, 0.75)
)
}
\arguments{
\item{rnbs}{RnBiseqSet object}

\item{addSampleNames}{should the sample names be added to the plot}

\item{bar.percentiles}{the percentiles to be used for the error bars. Must be a vector of length 2 of which the
first two elements will be used}
}
\value{
plot as an object of type \code{\link{ggplot}}
}
\description{
plot the number of sites vs median and other percentiles of coverage
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBeadSet-class.R
\docType{methods}
\name{qc,RnBeadSet-method}
\alias{qc,RnBeadSet-method}
\alias{qc}
\title{qc-methods}
\usage{
\S4method{qc}{RnBeadSet}(object)
}
\arguments{
\item{object}{Dataset of interest.}
}
\value{
Quality control information available for the dataset in the form of a \code{list} with two elements:
\code{Cy3} and \code{Cy5}.
}
\description{
Extracts HumanMethylation quality control information
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
qcinf<-dpval(rnb.set.example, row.names=TRUE)
head(qcinf$Cy3)
head(qcinf$Cy5)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/controlPlots.R
\name{rnb.plot.sentrix.distribution}
\alias{rnb.plot.sentrix.distribution}
\title{rnb.plot.sentrix.distribution}
\usage{
rnb.plot.sentrix.distribution(rnb.set, sentrix.id)
}
\arguments{
\item{rnb.set}{HumanMethylation450K dataset as an object of type \code{\linkS4class{RnBeadSet}}.}

\item{sentrix.id}{Slide number (Sentrix ID) as an \code{integer} or \code{character} singleton.}
}
\value{
Generated point-and-whisker plot (an instance of \code{\link{ggplot}}) of mean methylations for the samples
        on the specified slide, or \code{FALSE} if the dataset is non-empty but does not contain samples on the
        given slide. If the provided dataset does not contain valid Sentrix ID and position information (or is an
        empty dataset), this method returns \code{NULL}.
}
\description{
Creates a point-and-whisker plots showing beta value distributions at Sentrix positions for the given slide.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
sid<-as.character(pheno(rnb.set.example)[["Sentrix_ID"]][1])
rnb.plot.sentrix.distribution(rnb.set.example,sid)
}
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computeCluster.R
\name{logger.machine.name}
\alias{logger.machine.name}
\title{logger.machine.name}
\usage{
logger.machine.name()
}
\value{
None (invisible \code{NULL}).
}
\description{
Log the machine name the analysis is run on
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBeadRawSet-class.R
\docType{class}
\name{RnBeadRawSet-class}
\alias{RnBeadRawSet-class}
\alias{RnBeadRawSet}
\alias{initialize,RnBeadRawSet-method}
\title{RnBeadRawSet-class}
\usage{
RnBeadRawSet(
  pheno,
  probes,
  M,
  U,
  M0 = NULL,
  U0 = NULL,
  bead.counts.M = NULL,
  bead.counts.U = NULL,
  p.values = NULL,
  qc = NULL,
  platform = "450k",
  beta.offset = 100,
  summarize.bead.counts = TRUE,
  summarize.regions = TRUE,
  region.types = rnb.region.types.for.analysis(ifelse(platform == "MMBC", "mm10",
    ifelse(target == "probesEPICv2", "hg38", rnb.getOption("assembly")))),
  useff = rnb.getOption("disk.dump.big.matrices"),
  ffcleanup = FALSE
)
}
\arguments{
\item{pheno}{Phenotypic data.}

\item{probes}{\code{character} vector of Infinium(R) probe identifiers}

\item{M}{Matrix of intensities for the probes measuring the abundance of methylated molecules}

\item{U}{Matrix of intensities for the probes measuring the abundance of unmethylated molecules}

\item{M0}{Matrix of "out-of-band" intensities for the probes measuring the abundance of methylated molecules}

\item{U0}{Matrix of "out-of-band" intensities for the probes measuring the abundance of unmethylated molecules}

\item{bead.counts.M}{Matrix of bead counts per probe.}

\item{bead.counts.U}{Matrix of bead counts per probe.}

\item{p.values}{Matrix of detection p-values.}

\item{qc}{...}

\item{platform}{\code{character} singleton specifying the microarray platform: \code{"450k"} corresponds to HumanMethylation450 microarray, and \code{"27k"} stands for HumanMethylation27.}

\item{beta.offset}{A regularization constant which is added to the denominator at beta-value calculation}

\item{summarize.bead.counts}{If \code{TRUE} the coverage slot is filled by summarizing the \code{bead.counts.M} and \code{bead.counts.U} matrices. For type I probes the summarization is done using \code{min} operation, while for type II probes the bead counts should be identical in both supplied matrices}

\item{summarize.regions}{...}

\item{region.types}{A \code{character} vector specifying the region types, for which the methylation infromation will be summarized.}

\item{useff}{If \code{TRUE} the data matrices will be stored as \code{ff} objects}

\item{ffcleanup}{If \code{TRUE} and disk dumping has been enabled the data of the input \code{ff} objects will be deleted}
}
\value{
an object of class RnBeadRawSet
}
\description{
Main class for storing HumanMethylation micorarray data which includes intensity information
}
\section{Slots}{

\describe{
\item{\code{pheno}}{Phenotypic data.}
\item{\code{M}}{\code{matrix} of intensities for the probes measuring the abundance of methylated molecules.} 
\item{\code{U}}{\code{matrix} of intensities for the probes measuring the abundance of unmethylated molecules.} 
	\item{\code{M0}}{\code{matrix} of "out-of-band" intensities for the probes measuring the abundance of methylated molecules.} 
	\item{\code{U0}}{\code{matrix} of "out-of-band" intensities for the probes measuring the abundance of unmethylated molecules.}
	\item{\code{bead.counts.M}}{\code{matrix} of bead counts per probe.}
	\item{\code{bead.counts.U}}{\code{matrix} of bead counts per probe.}
}
}

\section{Methods and Functions}{

\describe{
  \item{\code{samples}}{Gets the identifiers of all samples in the dataset.}
  \item{\code{\link[=M,RnBeadRawSet-method]{M}}}{Get the matrix of intensities for the probes measuring the abundance of methylated molecules.}
  \item{\code{\link[=U,RnBeadRawSet-method]{U}}}{Get the matrix of intensities for the probes measuring the abundance of unmethylated molecules.}
  \item{\code{\link{intensities.by.color}}}{Get probe intensities in each color channel.}
}
}

\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataExport.R
\name{rnb.RnBSet.to.bed}
\alias{rnb.RnBSet.to.bed}
\title{Export to BED files}
\usage{
rnb.RnBSet.to.bed(
  rnb.set,
  out.dir,
  reg.type = "sites",
  names.quant.meth = TRUE,
  add.track.line = TRUE,
  lexicographic = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{rnb.set}{Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.}

\item{out.dir}{Output directory. If not existing, it will be created. otherwise files in that directory are overwritten.}

\item{reg.type}{Region type to be extracted.}

\item{names.quant.meth}{should the names of the bed regions contain information on the methylation level.
If TRUE the following format is applied: meth_percent%[coverage]. Coverage is available only when
 \code{covg(rnb.set)} is not NULL}

\item{add.track.line}{Add a track line to the bed file to enable browsers like IGV to display the data better}

\item{lexicographic}{Should lexicographic ordering be used for chromosome names}

\item{verbose}{More detailed logger output}
}
\value{
(invisibly) a summary list containing information on the conversion step.
        elements are \code{filenames} (a table containing information on which sample has been written to what filename)
        and \code{assembly} (a string indicating the assembly used by \code{rnb.set}).
}
\description{
Exports the beta values from a methylation dataset to BED files.
}
\details{
Details on the BED file format can be found in the \href{http://genome.ucsc.edu/FAQ/FAQformat.html}{UCSC Genome Browser
         documentation}.  Each methylation site is an entry in the resulting bed file. The Score column corresponds
         to a site's methylation value in the interval \code{[0,1]}.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
rnb.RnBSet.to.bed(rnb.set.example,tempdir())
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataImport.R
\name{read.bed.files}
\alias{read.bed.files}
\title{read.bed.files}
\usage{
read.bed.files(
  base.dir = NULL,
  file.names = NULL,
  sample.sheet = NULL,
  file.names.col = 0,
  assembly = rnb.getOption("assembly"),
  region.types = rnb.region.types.for.analysis(assembly),
  pos.coord.shift = 1L,
  skip.lines = 1,
  sep.samples = rnb.getOption("import.table.separator"),
  merge.bed.files = TRUE,
  useff = rnb.getOption("disk.dump.big.matrices"),
  usebigff = rnb.getOption("disk.dump.bigff"),
  verbose = TRUE,
  ...
)
}
\arguments{
\item{base.dir}{Directory with BED files contatining processed methylation data}

\item{file.names}{Optional non-empty \code{character} vector listing the names of the files that should be
loaded relative to \code{base.dir}. If supplied, this vector must not contain \code{NA}
   among its elements.}

\item{sample.sheet}{Optional file name containing a table of sample annotation data, or the table itself in the form
of a \code{\link{data.frame}} or \code{matrix}. Only (and all) samples defined in this table
will be loaded. The table is expected to contain a column named \code{"barcode"} that lists the
samples' Sentrix  barcodes. If such a column is not present, this function searches for columns
\code{"Sentrix_ID"} and \code{"Sentrix_Position"} (or similar) that build a barcode.}

\item{file.names.col}{Column of the sample sheet which contains the file names (integer singleton). If \code{NA}
an attempt will be made to find a suiting column automatically.}

\item{assembly}{Genome assembly. Defaults to human (\code{"hg19"})}

\item{region.types}{\code{character} vector storing the types of regions for which the methylation information is to
be summarized. The function \code{\link{rnb.region.types}} provides the list of all supported
regions. Setting this to \code{NULL} or an empty vector restricts the dataset to site
methylation only.}

\item{pos.coord.shift}{The frame shift between the the CpG annotation (1-based) and the coordinates in the loaded BEDs.
If BEDs have 0-based coordinates, \code{pos.coord.shift=1} (default).}

\item{skip.lines}{The number of top lines to skip while reading the BED files}

\item{sep.samples}{\code{character} singleton used as field separator in the sample sheet file.
Default value is taken by the call to \code{rnb.getOption("import.table.separator")}}

\item{merge.bed.files}{In case multiple BED files are specified for each sample, the flag indicates whether the
methylation calls should be merged after reading}

\item{useff}{If \code{TRUE}, functionality provided by the \code{ff} package will be used to read the data efficiently.}

\item{usebigff}{flag specifying whether the extended ff functionality should be used (large matrix support for ff)}

\item{verbose}{Flag indicating ifthe messages to the logger should be sent. Note that the logger
must be initialized prior to calling this function. Logging is useful for keeping a
record of the downloaded and processed samples. Also, informative messages are
stored in case of an error.}

\item{...}{Further arguments which are passed to the internal function \code{read.single.bed} and to \code{read.table}}
}
\value{
an object of class \code{\linkS4class{RnBiseqSet}}
}
\description{
Reads a reduced-representation/whole-genome bisulfite sequencing data set from a set of BED files
}
\details{
To control the BED column assignment, one should also supply arguments to \code{read.single.bed}.
}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\docType{methods}
\name{regionMapping,RnBSet-method}
\alias{regionMapping,RnBSet-method}
\alias{regionMapping}
\title{regionMapping-methods}
\usage{
\S4method{regionMapping}{RnBSet}(object, region.type)
}
\arguments{
\item{object}{Dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.}

\item{region.type}{region type. see \code{\link{rnb.region.types}} for possible values}
}
\value{
A list containing for each region the indices (as integers) of sites that belong to that region
}
\description{
get the mapping of regions in the RnBSet object to methylation site indices in the RnBSet object
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
promoter.probe.list <- regionMapping(rnb.set.example,"promoters")
#get the number of CpGs per promoter in the dataset:
sapply(promoter.probe.list,length)
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differentialMethylation.R
\name{rowOneSampleTP}
\alias{rowOneSampleTP}
\title{rowOneSampleTP}
\usage{
rowOneSampleTP(X, mu = 0, alternative = "two.sided")
}
\arguments{
\item{X}{Matrix on which the test is performed for every row}

\item{mu}{The mean that is tested against}

\item{alternative}{Testing alternative. Must be one of "two.sided" (default),"less","greater" or "all".
in case of "all" a data frome with corresping alternative variables is returned. 
Otherwise the result is a vector.}
}
\value{
vector (or data.frame if alternative=="all") of p-values from a paired t-test
}
\description{
performs a two-sided t-test for paired samples on each row of a matrix X with the indices inds.1 vs indices inds.g2 as group assignments.
}
\note{
Requires \code{matrixStats} package
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
meth.mat <- meth(rnb.set.example)
p.vals <- rowOneSampleTP(meth.mat,mu=0,alternative="greater")
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterArchitecture.R
\docType{methods}
\name{getSubCmdTokens,ClusterArchitecture-method}
\alias{getSubCmdTokens,ClusterArchitecture-method}
\alias{getSubCmdTokens}
\title{getSubCmdTokens-methods}
\usage{
\S4method{getSubCmdTokens}{ClusterArchitecture}(
  object,
  cmd.tokens,
  log,
  job.name = "",
  res.req = character(0),
  depend.jobs = character(0)
)
}
\arguments{
\item{object}{\code{\linkS4class{ClusterArchitecture}} object}

\item{cmd.tokens}{a character vector specifying the executable command that should be wrapped in the cluster submission command}

\item{log}{file name and path of the log file that the submitted job writes to}

\item{job.name}{name of the submitted job}

\item{res.req}{character vector specifying required resources. The resource requirements should be the values of the vector,
the names should specify the resource name}

\item{depend.jobs}{character vector containg names or ids of jobs the submitted job will depend on.}
}
\value{
A character vector containing the submission command tokens
}
\description{
Returns a string for the of command line corresponding to submitting
a job with the given command to the cluster.
}
\details{
For a concrete child class implementation for a sun grid architecture specification see \code{\link{getSubCmdTokens,ClusterArchitectureSGE-method}}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{rnb.sample.replicates}
\alias{rnb.sample.replicates}
\title{rnb.sample.replicates}
\usage{
rnb.sample.replicates(rnb.set, replicate.id.col)
}
\arguments{
\item{rnb.set}{Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.}

\item{replicate.id.col}{Trait (column name in the sample annotation table) that indicates sample replicates.
   Replicates should have the same value for this trait, while samples without replicates are
expected to have unique values or missing values.}
}
\value{
List of length of the number of replicates in the dataset. Each element is an \code{integer} vector storing
        the corresponding sample indices.
}
\description{
Identifies sample replicates defined in the given sample annotation table.
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/segmentation.R
\name{rnb.execute.segmentation}
\alias{rnb.execute.segmentation}
\title{rnb.execute.segmentation}
\usage{
rnb.execute.segmentation(
  rnb.set,
  sample.name,
  meth.level = 0.5,
  fdr = 5,
  min.cover = 5,
  n.cores = 1,
  chr.sel = "chr2",
  plot.path = getwd(),
  temp.dir = tempdir()
)
}
\arguments{
\item{rnb.set}{An object of type \code{\link{RnBiseqSet-class}} containing methylation and coverage information.}

\item{sample.name}{The sample for which segmentation is to be executed. Segemntation can only be exectued for each sample
individually.}

\item{meth.level}{Methylation cutoff to be used in UMR/LMR computation}

\item{fdr}{False discovery rate cutoff to be used in percent}

\item{min.cover}{The coverage threshold}

\item{n.cores}{The number of cores available for analysis}

\item{chr.sel}{Selected chromosome for model training in "chrAB" format. Defaults to "chr2".}

\item{plot.path}{Location on disk on which diagnostic plots are to be stored. Defaults to the working directory.}

\item{temp.dir}{The temporary directory. Defaults to the R temporary directory.}
}
\value{
The input RnBSet object with segementation added as an additional region type. Furthermore, three new annotations
         are set globally containing segmentation into PMDs, UMRs/LMRs, and HMDs for the sample that was specified.
}
\description{
This function computes methylation segmentation by MethylSeekR into PMDs, UMRs/LMRs, and HMDs. It is recommened to only
execute this function on WGBS data  (with coverage >=10 according to the developer's recommendation), but could also be
used with RRBS_HaeIII without guarantee and the results should be interpreted carefully.
}
\details{
For further descriptions on the methods, see \code{MethylSeekR}-documentation. The new annotations can be accessed
         via \code{rnb.get.annotation("[PMDs,UMRs,LMRs,HMDs]_[sample.name]")}.
}
\references{
1. Burger, Lukas, Gaidatzis, Dimos, Schuebeler, Dirk, and Stadler, Michael B. (2013)
            Identification of active regulatory regions from DNA methylation data.
            Nucleic Acids Research 41(16):e155.
}
\author{
Michael Scherer, based on a script by Abdulrahman Salhab
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBeads-package.R
\docType{data}
\name{lump.hg19}
\alias{lump.hg19}
\title{LUMP Support}
\format{
\code{lump.*} is a \code{list} of non-empty \code{integer} matrices, one per supported platform. Every
        \code{matrix} contains exactly two columns, denoting chromosome index and chromosome-based index,
        respectively. These indices refer to positions within the probe/site annotation table employed by
        \pkg{RnBeads} for the corresponding platform.
}
\description{
The sites used by the LUMP algorithm for estimating immune cell content are stored in an object named
\code{lump.hg19}. This object should not be loaded or otherwise operated on by users. Please refer to the
documentation of \code{\link{rnb.execute.lump}} for information on the algorithm and its implementation in
\pkg{RnBeads}.
}
\author{
Yassen Assenov
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\docType{methods}
\name{regions,RnBSet-method}
\alias{regions,RnBSet-method}
\alias{regions}
\title{regions-methods}
\usage{
\S4method{regions}{RnBSet}(object, type = NULL)
}
\arguments{
\item{object}{Dataset of interest.}

\item{type}{Region type(s) of interest as a \code{character} vector. If this is set to \code{NULL}, all region
types summarized in the object are returned.}
}
\value{
Methylation site and region assignment. If \code{type} is singleton, a \code{matrix} is returned. The first
		   column corresponds to the methylation context index. The second column is the index of the chromosome in
        the genome, and the third is the index of the region in the \code{GRanges} object of the region type
        annotation. When \code{length(type)>1}, a list of such matrices is returned for each element of \code{type}.
		   If \code{type} is \code{NULL}, matrices for all summarized region types are returned.
}
\description{
Methylation regions, information for which is present in the \code{RnBSet} object.
}
\note{
Methylation context index is an integer number denoting the sequence context of the cytosine of interest. Index
\code{1} corresponds to \code{CpG}, the only supported index in bisulfite sequencing datasets.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
head(regions(rnb.set.example))
}
}
\seealso{
\code{\link[=summarized.regions,RnBSet-method]{summarized.regions}} for all summarized region types in a dataset;
  \code{\link{rnb.get.chromosomes}} listing all supported chromosomes for a given genome assembly
}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lolaUtils.R
\name{getNamesFromLolaDb}
\alias{getNamesFromLolaDb}
\title{getNamesFromLolaDb}
\usage{
getNamesFromLolaDb(lolaDb, addCollectionNames = FALSE, addDbId = TRUE)
}
\arguments{
\item{lolaDb}{LOLA DB object as returned by \code{LOLA::loadRegionDB} or \code{\link{loadLolaDbs}}}

\item{addCollectionNames}{attach the name of the collection to the name}

\item{addDbId}{attach the index of the item in the LOLA DB object to the name}
}
\value{
character vector with human readable names
}
\description{
get human readable names from a LOLA DB object
}
\examples{
\donttest{
# download LOLA DB
lolaDest <- tempfile()
dir.create(lolaDest)
lolaDirs <- downloadLolaDbs(lolaDest, dbs="LOLACore")
lolaDb <- loadLolaDbs(lolaDirs[["hg19"]])
getNamesFromLolaDb(lolaDb)
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.R
\name{rnb.run.import}
\alias{rnb.run.import}
\alias{rnb.run.qc}
\alias{rnb.run.preprocessing}
\alias{rnb.run.inference}
\alias{rnb.run.tnt}
\alias{rnb.run.exploratory}
\alias{rnb.run.differential}
\title{RnBeads Modules in the Analysis Pipeline}
\usage{
rnb.run.import(
  data.source,
  data.type = rnb.getOption("import.default.data.type"),
  dir.reports,
  init.configuration = !file.exists(file.path(dir.reports, "configuration")),
  close.report = TRUE,
  show.report = FALSE
)

rnb.run.qc(
  rnb.set,
  dir.reports,
  init.configuration = !file.exists(file.path(dir.reports, "configuration")),
  close.report = TRUE,
  show.report = FALSE
)

rnb.run.preprocessing(
  rnb.set,
  dir.reports,
  init.configuration = !file.exists(file.path(dir.reports, "configuration")),
  close.report = TRUE,
  show.report = FALSE
)

rnb.run.inference(
  rnb.set,
  dir.reports,
  init.configuration = !file.exists(file.path(dir.reports, "configuration")),
  close.report = TRUE,
  show.report = FALSE
)

rnb.run.tnt(
  rnb.set,
  dir.reports,
  init.configuration = !file.exists(file.path(dir.reports, "configuration")),
  close.report = TRUE,
  show.report = FALSE
)

rnb.run.exploratory(
  rnb.set,
  dir.reports,
  init.configuration = !file.exists(file.path(dir.reports, "configuration")),
  close.report = TRUE,
  show.report = FALSE
)

rnb.run.differential(
  rnb.set,
  dir.reports,
  init.configuration = !file.exists(file.path(dir.reports, "configuration")),
  close.report = TRUE,
  show.report = FALSE
)
}
\arguments{
\item{data.source}{\code{character} vector specifying the location of the data items on disk. The expected
length of the vector differs for different values of \code{data.type}; see
\code{\link{rnb.execute.import}} for a more detailed description.}

\item{data.type}{\code{character} vector of length one specifying the type of the input data. The value of
this parameter must be one of \code{"idat.dir"}, \code{"data.dir"}, \code{"data.files"},
\code{"GS.report"}, \code{"GEO"} or \code{"rnb.set"}. See \code{\link{rnb.execute.import}}
for a more detailed description.}

\item{dir.reports}{Directory to host the generated report file. Note that if this directory contains files,
they may be overwritten.}

\item{init.configuration}{Flag indicating if the configuration directory (usually shared among reports) should also
be created.}

\item{close.report}{Flag indicating if the created report is to be closed using the
\code{\link[=off,Report-method]{off}} method.}

\item{show.report}{Flag indicating if the report is to be displayed after it is created. If this is,
\code{TRUE} \code{\link{rnb.show.report}} is called to open the generated HTML file.}

\item{rnb.set}{Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.}
}
\value{
For \code{rnb.run.import}, \code{rnb.run.preprocessing} and \code{rnb.run.inference}, the returned value is
        a list of two elements - the initialized or modified dataset and the created report. All other functions
        return the created report, invisibly.
}
\description{
Functions that start the predefined modules in the \pkg{RnBeads} analysis pipeline.
}
\details{
The functions start the import, quality control, preprocessing, covariate inference, tracks and tables,
         exploratory analysis and differential methylation modules, respectively.
}
\examples{
\donttest{
### Running the modules step by step

# Directory where your data is located
data.dir <- "~/RnBeads/data/Ziller2011_PLoSGen_450K"
idat.dir <- file.path(data.dir, "idat")
sample.annotation <- file.path(data.dir, "sample_annotation.csv")

# Directory where the output should be written to
analysis.dir <- "~/RnBeads/analysis"
# Directory where the report files should be written to
report.dir <- file.path(analysis.dir, "reports_details")
rnb.initialize.reports(report.dir)
# Set some analysis options
rnb.options(filtering.sex.chromosomes.removal = TRUE, identifiers.column = "Sample_ID")
## Restrict logging to the console only
logger.start(fname = NA)

## Data import
data.source <- c(idat.dir, sample.annotation)
result <- rnb.run.import(data.source=data.source, data.type="idat.dir", dir.reports=report.dir)
rnb.set <- result$rnb.set

## Quality Control
rnb.run.qc(rnb.set, report.dir)

## Preprocessing
rnb.set <- rnb.run.preprocessing(rnb.set, dir.reports=report.dir)$rnb.set

## Data export
rnb.options(export.to.csv = TRUE)
rnb.run.tnt(rnb.set, report.dir)

## Exploratory analysis
rnb.run.exploratory(rnb.set, report.dir)

## Differential methylation
rnb.run.differential(rnb.set, report.dir)
}

}
\seealso{
\code{\link{rnb.run.analysis}} which executes these modules in the order given above
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sva.R
\name{rnb.execute.sva}
\alias{rnb.execute.sva}
\title{rnb.execute.sva}
\usage{
rnb.execute.sva(
  rnb.set,
  cmp.cols = rnb.getOption("inference.targets.sva"),
  columns.adj = rnb.getOption("covariate.adjustment.columns"),
  assoc = TRUE,
  numSVmethod = rnb.getOption("inference.sva.num.method")
)
}
\arguments{
\item{rnb.set}{The \code{RnBSet} object on which the SVA should be conducted}

\item{cmp.cols}{a vector of sample annotation column names which will be the targets of the SVA.}

\item{columns.adj}{Column names in the table of phenotypic information to be used for confounder adjustment.}

\item{assoc}{a flag indicating whether association information with principal components and other sample annotation should
be returned}

\item{numSVmethod}{method to estimate the number of surrogate variables. Passed to \code{sva}.}
}
\value{
An object of class \code{SvaResult}: basically a list containing the following elements:
\describe{
  \item{\code{num.components}}{a vector storing the number of detected SVs for each target variable}
  \item{\code{sva.performed}}{a vector storing whether SVA was performed on a target variable and whether more than 0 SVs were found}
  \item{\code{targets}}{a vector storing the names of the target variables}
  \item{\code{components}}{a list storing for each target variable a matrox containing the sample-wise SVs as rows}
  \item{\code{assoc}}{a special object containing association information of SVs with principal components and sample annotations
 typically only used \code{rnb.section.sva}.}
}
}
\description{
Conduct Surrogate Variable Analysis (SVA) on the beta values of an RnBSet for given target variables
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
sva.obj <- rnb.execute.sva(rnb.set.example,c("Sample_Group","Treatment"),numSVmethod="be")
sva.obj$sva.performed
sva.obj$num.components
rnb.set.mod <- set.covariates.sva(rnb.set.example, sva.obj)
has.covariates.sva(rnb.set.example,"Sample_Group")
has.covariates.sva(rnb.set.mod,"Sample_Group")
has.covariates.sva(rnb.set.mod,"Treatment")
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\docType{methods}
\name{mask.sites.meth,RnBSet-method}
\alias{mask.sites.meth,RnBSet-method}
\alias{mask.sites.meth}
\title{mask.sites.meth-methods}
\usage{
\S4method{mask.sites.meth}{RnBSet}(object, mask, verbose = FALSE)
}
\arguments{
\item{object}{Dataset of interest.}

\item{mask}{logical matrix indicating which sites should be masked}

\item{verbose}{if \code{TRUE} additional diagnostic output is generated}
}
\value{
The modified dataset.
}
\description{
Given a logical matrix, sets corresponding entries in the methylation table to NA (masking).
Low memory footprint
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBeads-package.R
\docType{data}
\name{accepted}
\alias{accepted}
\alias{current}
\alias{infos}
\alias{previous}
\title{RnBeads option values and restrictions}
\format{
\code{infos} is a \code{data.frame} containing information about all options in \pkg{RnBeads}. Row names in
        this table are the option names; the column names are \code{"Type"}, \code{"Named"}, \code{"Null"},
        \code{"Max"}, \code{"Min"}, \code{"MaxInclusive"} and \code{"MinInclusive"}.
        \code{accepted} is a \code{list} containing the sets of accepted values for some of the options.
        \code{current} is a \code{list} with current values for all options.
        \code{previous} is a \code{list} with previous values for the affected options; this list is only temporarily
        used while setting option values through \code{\link{rnb.options}} or \code{\link{rnb.xml2options}}.
}
\description{
The values of options in RnBeads are stored in dedicated R objects accompanying the package. These objects are named
\code{infos}, \code{accepted}, \code{current} and \code{previous}. They should not be loaded or otherwise operated on
by users. Please refer to the documentation of \code{\link{rnb.options}} for accessing and modifying option values in
\pkg{RnBeads}.
}
\author{
Yassen Assenov
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normalization.R
\name{rnb.execute.imputation}
\alias{rnb.execute.imputation}
\title{rnb.execute.imputation}
\usage{
rnb.execute.imputation(
  rnb.set,
  method = rnb.getOption("imputation.method"),
  update.ff = TRUE,
  ...
)
}
\arguments{
\item{rnb.set}{Dataset object inheriting from \code{\linkS4class{RnBSet}}.}

\item{method}{Imputation method to be used, must be one of \code{"mean.cpgs"}, \code{"mean.samples"},
\code{"random"}, \code{"knn"}, \code{"median.cpgs"}, \code{"median.samples"}, or \code{"none"}.}

\item{update.ff}{flag indicating if the disk based matrices should be updated. Should be set to FALSE, if methylation
matrix should only temporarly be changed. If this value is FALSE, the region level methylation values
are not updated and only the site-wise matrix is changed temporarly.}

\item{...}{Optional arguments passed to knn.imputation}
}
\value{
The modified rnb.set object without missing methylation values.
}
\description{
Removes missing methylation values in the methylation matrix of the given object
}
\details{
Imputes missing values by applying on the following methods:
         \describe{
           \item{mean.cpgs:}{missing values are inferred as the average methylation value from all other 
           (non-mising) CpGs in this sample}
           \item{mean.samples:}{missing values are inferred as the average methylation value from all other 
           (non-mising) values at this CpG sites in all other samples}
           \item{random:}{missing values are inferred by randomly selecting a (non-missing) methylation value
           from any other sample at this CpG site}
           \item{knn:}{missing values are inferred by k-nearest neighbors imputation (see \pkg{impute})}
           \item{median.cpgs:}{missing values are inferred as the median methylation value from all other 
           (non-mising) CpGs in this sample}
           \item{median.samples:}{missing values are inferred as the median methylation value from all other 
           (non-mising) values at this CpG sites in all other samples}
           \item{none:}{imputation should not be performed}
         }
}
\author{
Michael Scherer
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computeCluster.R
\docType{methods}
\name{initialize,RnBClusterRun-method}
\alias{initialize,RnBClusterRun-method}
\title{initialize.RnBClusterRun}
\usage{
\S4method{initialize}{RnBClusterRun}(.Object, architecture)
}
\arguments{
\item{.Object}{New instance of \code{RnBClusterRun}.}

\item{architecture}{A \code{\linkS4class{ClusterArchitecture}} object managing the settings for a scientific compute cluster.}
}
\description{
Initialize an RnBClusterRun object
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{rnb.beta2mval}
\alias{rnb.beta2mval}
\title{rnb.beta2mval}
\usage{
rnb.beta2mval(betas, epsilon = 1e-05)
}
\arguments{
\item{betas}{\code{numeric} vector or matrix of beta values to be transformed.}

\item{epsilon}{Single \code{numeric} in the range [0, 0.5], giving the threshold of beta values to use when
adjusting for potential M values close to +infinity or -infinity. Setting this parameter to 0 (zero)
disables stabilization; in which case M values of -infinity or +infinity could be returned.}
}
\value{
The calculated and adjusted M values.
}
\description{
Transforms beta values to M values, adjusting for +infinity and -infinity.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
mvals <- rnb.beta2mval(meth(rnb.set.example))
summary(mvals)
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\name{isImputed,RnBSet-method}
\alias{isImputed,RnBSet-method}
\alias{isImputed}
\title{isImputed}
\usage{
\S4method{isImputed}{RnBSet}(object)
}
\arguments{
\item{object}{Object for which the information should be returned}
}
\value{
TRUE, if the object has been imputed and FALSE otherwise.
}
\description{
Getter for the imputation field. Return TRUE, if the object has been imputed and FALSE otherwise.
}
\author{
Michael Scherer
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sex.R
\name{rnb.execute.sex.prediction}
\alias{rnb.execute.sex.prediction}
\title{rnb.execute.sex.prediction}
\usage{
rnb.execute.sex.prediction(rnb.set)
}
\arguments{
\item{rnb.set}{Methylation dataset as an object of type \code{\linkS4class{RnBeadRawSet}}.}
}
\value{
The possibly modified dataset. If sex could be predicted, the sample annotation table is enriched with
two more columns - \code{"Predicted Male Probability"} and \code{"Predicted Sex"}.
}
\description{
Infers the sex of every sample in the given dataset, based on average signal intensity values on
the autosomes and the sex chromosomes.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
rnb.set.example <- rnb.execute.sex.prediction(rnb.set.example)
table(rnb.set.example[, "Predicted Sex"])
}
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\docType{methods}
\name{remove.regions,RnBSet-method}
\alias{remove.regions,RnBSet-method}
\alias{remove.regions}
\title{remove.regions-methods}
\usage{
\S4method{remove.regions}{RnBSet}(object, region.type)
}
\arguments{
\item{object}{Dataset of interest.}

\item{region.type}{Type of the region annotation for which the summarization should be removed}
}
\value{
object of the same class as the supplied one without the summarized methylation information for the specified region type
}
\description{
Remove the summarized methylation information for a given region type from an \code{RnBSet} object.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
summarized.regions(rnb.set.example)
rnb.set.reduced<-remove.regions(rnb.set.example, "genes")
summarized.regions(rnb.set.reduced)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/segmentation.R
\name{rnb.boxplot.from.segmentation}
\alias{rnb.boxplot.from.segmentation}
\title{rnb.boxplot.from.segmentation}
\usage{
rnb.boxplot.from.segmentation(rnb.set, sample.name, type = "final")
}
\arguments{
\item{rnb.set}{An \code{\link{RnBSet-class}} object obtained by executing \code{rnb.execute.segmentation}.}

\item{sample.name}{The sample name for which segmentation was computed.}

\item{type}{The type of segmentation (\code{PMDs}, \code{UMRs}, \code{LMRs}, \code{HMDs} or \code{final}).}
}
\value{
An object of type \code{ggplot} visualizing the methylation values in the segments.
}
\description{
This function creates a boxplot from the segmentation result of \code{rnb.execute.segmentation}.
}
\author{
Michael Scherer
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotations.R
\name{rnb.infinium.control.targets}
\alias{rnb.infinium.control.targets}
\title{rnb.infinium.control.targets}
\usage{
rnb.infinium.control.targets(target = "probes450")
}
\arguments{
\item{target}{A singleton of type \code{character}, specifying the microarray platform.
\code{"probesEPICv2"},\code{"probesEPIC"},\code{"probes450"} and \code{"probes27"} correspond to MethylationEPICv2, 
Methylation EPIC, HumanMethylation450, and HumanMethylation27 microarrays respectively.}
}
\value{
\code{character} vector of control targets.
}
\description{
Extracts all control probe types in the HumanMethylation450 assay.
}
\examples{
\donttest{
"NEGATIVE" \%in\% rnb.infinium.control.targets()
}
}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\docType{methods}
\name{annotation,RnBSet-method}
\alias{annotation,RnBSet-method}
\alias{annotation}
\title{annotation-methods}
\usage{
\S4method{annotation}{RnBSet}(object, type = "sites", add.names = FALSE, include.regions = FALSE)
}
\arguments{
\item{object}{dataset as an object of type inheriting \code{RnBSet}.}

\item{type}{loci or regions for which the annotation should be obtained. If the value of this parameter is
\code{"sites"} (default), individual methylation sites are annotated. Otherwise, this must be one of
the available region types, as returned by \code{\link{rnb.region.types}}.}

\item{add.names}{flag specifying whether the unique site identifiers should be used as row names of the
resulting data frame}

\item{include.regions}{if \code{TRUE} one additional column is added to the returned annotation dat frame
for each of the available region types, giving the indices of the}
}
\value{
Annotation table in the form of a \code{data.frame}.
}
\description{
Genomic annotation of the methylation sites or regions covered in the supplied dataset.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
## show present sites
head(annotation(rnb.set.example, add.names=TRUE))
## show promoters
ann.prom<-annotation(rnb.set.example, type="promoters", add.names=TRUE)
head(ann.prom)
}
}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBeads-package.R
\docType{data}
\name{epic_v2_manifest}
\alias{epic_v2_manifest}
\title{XenoMix EPICv2 (hg38)}
\format{
\code{data.frame} provided by Illumina capturing information about the probe design.
}
\description{
This is the manifest file for EPICv2, which is required for detecting inter-species probes for estimating mouse contamination
with XenoMix
}
\author{
Nima Esmaeelpour
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ReportPlot-class.R
\docType{class}
\name{ReportGgPlot-class}
\alias{ReportGgPlot-class}
\alias{initialize,ReportGgPlot-method}
\title{ReportGgPlot Class}
\description{
Information about the files created to store one generated plot in a report. Report plots are initialized using the
function \code{\link{createReportGgPlot}}. It inherits from the \code{\linkS4class{ReportPlot}} class and handling is
analogous, except that it contains an additional slot to store a \code{ggplot} object.
}
\section{Slots}{

\describe{
  \item{\code{ggp}}{\code{ggplot} object to be printed}
}
}

\section{Notes}{

No device is being opened until \code{off(reportGgPlot)} is called.
}

\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differentialMethylation.R
\name{limmaP}
\alias{limmaP}
\title{limmaP}
\usage{
limmaP(
  X,
  inds.g1,
  inds.g2 = -inds.g1,
  adjustment.table = NULL,
  fun.conversion = rnb.beta2mval,
  paired = FALSE
)
}
\arguments{
\item{X}{Matrix on which the test is performed for every row}

\item{inds.g1}{column indices of group 1 members}

\item{inds.g2}{column indices of group 2 members}

\item{adjustment.table}{a \code{data.frame} containing variables to adjust for in the testing}

\item{fun.conversion}{conversion function to transform the beta values into M values. By default, it is the logit function with adjustment
for infinity values. See \code{\link{rnb.beta2mval}} for details.}

\item{paired}{should a paired analysis model be used. If so, the first index in \code{inds.g1} must correspond to the first
index in  \code{inds.g2} and so on.}
}
\value{
vector of p-values resulting from limma's differential analysis
}
\description{
applies hierarchical modeling anlalogous to differential expression employed in the \code{limma} package and returns
p-values for differential methylation
}
\note{
Requires \code{limma} package
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
meth.mat <- meth(rnb.set.example)
sample.groups <- rnb.sample.groups(rnb.set.example)[[1]]
p.vals <- limmaP(meth.mat,sample.groups[[1]],sample.groups[[2]])
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Report-methods.R
\name{rnb.add.reference}
\alias{rnb.add.reference}
\title{rnb.add.reference}
\usage{
rnb.add.reference(report, txt)
}
\arguments{
\item{report}{Report to add a reference item to.}

\item{txt}{Text of the reference in the form of a non-empty \code{character} vector. The elements of this vector
are concatenated without a separator.}
}
\value{
The modified report.
}
\description{
Adds a reference item to the given report.
}
\examples{
\donttest{
report <- createReport("example.html", "Example", init.configuration = TRUE)
txt.reference <- c("Bird A. ", "<i>Nucleic Acids Res.</i> <b>8</b> (1980)")
report <- rnb.add.reference(report, txt.reference)
txt <- c("This was shown in ", rnb.get.reference(report, txt.reference), ".")
rnb.add.paragraph(report, txt)
}
}
\seealso{
\code{\link{rnb.get.reference}} for adding citations in the report's text; \code{\linkS4class{Report}} for
  other functions adding contents to an HTML report
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Report-methods.R
\name{rnb.add.table}
\alias{rnb.add.table}
\title{rnb.add.table}
\usage{
rnb.add.table(
  report,
  tdata,
  row.names = TRUE,
  first.col.header = FALSE,
  indent = 0,
  tag.attrs = c(class = "tabdata"),
  thead = NULL,
  tcaption = NULL,
  na = "<span class=\\"disabled\\">n/a</span>"
)
}
\arguments{
\item{report}{Report to write the text to.}

\item{tdata}{Matrix or data frame to be presented in HTML form. Column names, if present, are used to
define table columns. If this table contains 0 (zero) rows or 0 columns, calling this
function has no effect.}

\item{row.names}{Flag indicating if row names should also be printed. If this parameter is \code{TRUE} and
\code{tdata} defines row names, these are printed in the left-most column and are displayed
as header cells. Keep in mind that \code{data.frame}s always define row names.}

\item{first.col.header}{Flag indicating if all cells in the first column must be displayed as header cells. Note
that, if both this parameter and \code{row.names} are \code{TRUE}, and \code{tdata} contains
row names, the constructed HTML table will have 2 columns of header cells.}

\item{indent}{Default indentation, in number of tabulation characters, to apply to HTML tags. This
indentation is also applied to \code{thead}.}

\item{tag.attrs}{Named \code{character} vector specifying the list of attributes to be set to the
\code{<table>} element. Setting this to \code{NULL} or an empty \code{character} vector
disables attributes.}

\item{thead}{\code{character} vector storing a table header to include. This can, for example, be a
\code{character} that defines column widths. Every element in this vector is written on a
separate line, applying the indentation given by \code{indent}.}

\item{tcaption}{Text to include as a caption below the table, or \code{NULL} if the table does not contain
caption.}

\item{na}{\code{character} to be used for printing \code{NA} values in the table. This parameter is
not considered when printing \code{thead} or the table's column names.}
}
\value{
The modified report, invisibly.
}
\description{
Generates HTML code for a table in the specified report.
}
\seealso{
\code{\link{rnb.add.tables}} for adding a listing of tables; \code{\linkS4class{Report}} for other functions
  adding contents to an HTML report
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotations.R
\name{rnb.set.annotation}
\alias{rnb.set.annotation}
\title{rnb.set.annotation}
\usage{
rnb.set.annotation(type, regions, description = NULL, assembly = "hg19")
}
\arguments{
\item{type}{One-element \code{character} vector giving the name of the annotation. If this region type is
already available, it will be overwritten for the current session. The type cannot be one of
\code{"CpG"}, \code{"probes450"} or \code{"controls450"}, because these names are reserved for the
annotation tables of CpG dinucleotides, and Infinium methylation and control probes, respectively.}

\item{regions}{BED file defining regions (see \emph{Details}). Alternatively, the value of this parameter can be
a table of genomic regions in the form of a \code{\link{data.frame}}, containing at least the
following three columns - \code{"Chromosome"}, \code{"Start"} and \code{"End"} (notice the upper
case). The \code{"chromosome"} column must be a \code{character} or \code{factor} vector that
lists chromosome names. The \code{"start"} and \code{"end"} columns are expected to contain
genomic positions as \code{integer}s. The row names of this \code{data.frame} are used as region
identifiers.}

\item{description}{Optional; short description in the form of a non-empty \code{character} vector. The elements in
this vector are concatenated without a separator to form the description of the annotation.}

\item{assembly}{Genome assembly of interest. See \code{\link{rnb.get.assemblies}} for the list of supported
genomes.}
}
\value{
Invisibly, \code{TRUE} if an existing annotation was replaced and \code{FALSE} otherwise.
}
\description{
Adds or replaces a region annotation table.
}
\details{
In case the parameter \code{regions} specifies an existing BED file, regions are loaded from this file. The number of
columns defined must be at least 3. Columns after the sixth one, if present, are dropped. The columns are given the
following names: \code{"chromosome"}, \code{"start"}, \code{"end"}, \code{"id"}, \code{"score"} and \code{"strand"}.

The annotation tables in \pkg{RnBeads} focus on chromosomes \code{"chr1"}, \code{"chr2"}, ..., \code{"chr22"},
\code{"chrX"} and \code{"chrY"}. Regions on other chromosomes are ignored. This function also recognizes the
convention of chromosome names such as \code{"1"}, adopted, for example, by \href{http://www.ensembl.org/}{Ensembl}.
Apart from this, the region definition table is not examined in details by this function; therefore, regions located
on unsupported chromosomes or having invalid (e.g. negative) genomic coordinates are simply not mapped to any sites
or probes.
}
\examples{
\donttest{
my.regions <- data.frame(
    chromosome = c("chr1", "chr1"),
    start = c(49242278L, 49242372L),
    end = c(49242590L, 49242810L),
    rownames = c("BEND5E1", "CpG:38"))
txt <- "First exon of the BEND5 gene and an overlapping CpG island."
rnb.set.annotation("my regions", my.regions, txt)
}
}
\seealso{
\code{\link{rnb.get.annotation}} for extracting annotation;
  \code{\link{rnb.region.types}} for all loaded region types in a genome assembly
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/profiles.R
\name{rnb.plot.betadistribution.sampleGroups}
\alias{rnb.plot.betadistribution.sampleGroups}
\title{rnb.plot.betadistribution.sampleGroups}
\usage{
rnb.plot.betadistribution.sampleGroups(
  beta.matrix,
  sample.group.inds,
  annotation = "Group",
  log.str = NULL,
  points.per.group = rnb.getOption("distribution.subsample")
)
}
\arguments{
\item{beta.matrix}{Beta values in the form of a non-empty \code{matrix} of type \code{double}. Rows in this
matrix must correspond to Infinium probes, and columns - to samples.}

\item{sample.group.inds}{Named \code{list} that contains indices for the samples contained in the groups in
   \code{beta.matrix}. The number of groups is determined by the length of the list, and its
names are used as group names.}

\item{annotation}{Name of the annotation being visualized, in the form of a \code{character} vector of length 1.}

\item{log.str}{string specifying more details for the log file}

\item{points.per.group}{the targeted number of points per group. Set this to a value < 1 to disable subsampling. More
information in the Details section of \code{\link{rnb.step.betadistribution}}}
}
\value{
the plot as a \code{ggplot2} object
}
\description{
Plots beta value distrubions given a sample grouping.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
meth.mat <- meth(rnb.set.example)
sample.groups <- rnb.sample.groups(rnb.set.example)[[1]]
rnb.plot.betadistribution.sampleGroups(meth.mat,sample.groups)
}
}
\seealso{
rnb.plot.betadistribution.probeCategories
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/profiles.R
\name{rnb.plot.nv.heatmap}
\alias{rnb.plot.nv.heatmap}
\title{rnb.plot.nv.heatmap}
\usage{
rnb.plot.nv.heatmap(dataset, writeToFile = FALSE, ...)
}
\arguments{
\item{dataset}{Dataset as an object of type inheriting \code{\linkS4class{RnBeadSet}}, or a matrix of
methylation beta values.}

\item{writeToFile}{Flag specifying whether the output should be saved as \code{\linkS4class{ReportPlot}}.}

\item{...}{Additional named arguments passed to \code{\link{createReportPlot}}. These are used only if
\code{writeToFile} is \code{TRUE}.}
}
\value{
If \code{writeToFile} is \code{TRUE}, plot as an object of type \code{\linkS4class{ReportPlot}}. Otherwise,
        there is no value returned (invisible \code{NULL}).
}
\description{
Heatmap of beta values from nv probes.
}
\author{
Pavlo Lutsik, Baris Kalem
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clustering.R
\name{rnb.execute.clustering.all}
\alias{rnb.execute.clustering.all}
\title{rnb.execute.clustering.all}
\usage{
rnb.execute.clustering.all(rnb.set)
}
\arguments{
\item{rnb.set}{Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.}
}
\value{
List of list of clustering results; each element corresponds to one region type and is a list of objects
        of type \code{\linkS4class{RnBeadClustering}}.
}
\description{
Performs hierarchical clustering on the samples of the given dataset using multiple distance metrics and
agglomeration methods for all suggested site and region types.
}
\seealso{
\code{\link{rnb.execute.clustering}} for performing clustering using a single site or region type.
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterArchitectureSGE.R
\docType{methods}
\name{getSubCmdTokens,ClusterArchitectureSGE-method}
\alias{getSubCmdTokens,ClusterArchitectureSGE-method}
\title{getSubCmdTokens-methods}
\usage{
\S4method{getSubCmdTokens}{ClusterArchitectureSGE}(
  object,
  cmd.tokens,
  log,
  job.name = "",
  res.req = character(0),
  depend.jobs = character(0),
  sub.binary = TRUE,
  quote.cmd = TRUE,
  queue = NULL
)
}
\arguments{
\item{object}{\code{\linkS4class{ClusterArchitectureSGE}} object}

\item{cmd.tokens}{a character vector specifying the executable command that should be wrapped in the cluster submission command}

\item{log}{file name and path of the log file that the submitted job writes to}

\item{job.name}{name of the submitted job}

\item{res.req}{character vector specifying required resources. The resource requirements should be the values of the vector,
the names should specify the resource name}

\item{depend.jobs}{character vector containg names or ids of jobs the submitted job will depend on.}

\item{sub.binary}{treat the command as binary (see \code{-b} flag of \code{qsub} of the SGE documentation)}

\item{quote.cmd}{Flag indicating whether the submitted cammed should also be wrapped in quotes}

\item{queue}{The name of the queue to submit jobs to}
}
\value{
A character vector containing the submission command tokens
}
\description{
Returns a string for the of command line corresponding to submitting
a job with the given command to the cluster.
}
\details{
For a concrete child class implementation for a sun grid architecture specification see \code{\linkS4class{ClusterArchitectureSGE}}
}
\examples{
\donttest{
arch <- new("ClusterArchitectureSGE",
	name="my_sge_architecture"
)
getSubCmdTokens(arch,c("Rscript","my_great_script.R"),"my_logfile.log")
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{rnb.mval2beta}
\alias{rnb.mval2beta}
\title{rnb.mval2beta}
\usage{
rnb.mval2beta(mvals)
}
\arguments{
\item{mvals}{\code{numeric} vector or matrix of M values to be transformed.}
}
\value{
The calculated beta values.
}
\description{
Transforms M values to beta values.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
mvals <- rnb.beta2mval(meth(rnb.set.example))
bvals <- rnb.mval2beta(mvals)
all((bvals-meth(rnb.set.example))<1e-10)
}
}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regionProfiles.R
\name{rnb.plot.region.profiles}
\alias{rnb.plot.region.profiles}
\title{rnb.plot.region.profiles}
\usage{
rnb.plot.region.profiles(
  rnb.set,
  group.index.list,
  region.type = "",
  region.profile = NULL,
  extend.by = 0.33,
  cvalues = rnb.getOption("colors.category")
)
}
\arguments{
\item{rnb.set}{RnBSet object}

\item{group.index.list}{a list (preferably named) containing sample indices for each group
a list of such lists is for instance generated by the \code{rnb.sample.groups} function.}

\item{region.type}{Region type for which the plot should be generated}

\item{region.profile}{Alternative to specifying \code{region.type}, the function can accept a region profile
generated by the \code{rnb.find.relative.site.coord} function}

\item{extend.by}{A number between 0 and 1 specifying the percentage by which a region is
extended in order to capture methylation information before region start and after region end}

\item{cvalues}{Color values that will be assigned to sample groups}
}
\value{
a ggplot2 object for plotting
        the plot shows the smoothed methylation levels of sites accross the specified region type for 
        all regions of that type from 0 (region start) to 1 (region end). Sites in the flanking areas are
        also shown (coordinates <0 and >1).
        Smoothing is stratified by sample (dashed lines) and sample group (thick solid lines).
        Cubic splines are used for smoothing
}
\description{
Creates a composite plot showing the sample and groupwise smoothed estimates
of methylation values accross all regions of the specified type
}
\examples{
\donttest{
#Careful: this might take a while
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
rnb.plot.region.profiles(rnb.set.example,rnb.sample.groups(rnb.set.example)[[1]],"genes")
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filtering.R
\name{rnb.execute.na.removal}
\alias{rnb.execute.na.removal}
\title{rnb.execute.na.removal}
\usage{
rnb.execute.na.removal(
  rnb.set,
  threshold = rnb.getOption("filtering.missing.value.quantile")
)
}
\arguments{
\item{rnb.set}{Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.}

\item{threshold}{Maximum quantile of \code{NA}s allowed per site. This must be a value between 0 and 1.}
}
\value{
List of four or five elements:
        \describe{
          \item{\code{"dataset.before"}}{Copy of \code{rnb.set}.}
          \item{\code{"dataset"}}{The (possibly modified) dataset after performing the missing value removal.}
          \item{\code{"filtered"}}{\code{integer} vector storing the indices (in beta matrix of the unfiltered
               dataset) of all removed sites.}
			 \item{\code{"threshold"}}{Copy of \code{threshold}.}
			 \item{\code{"naCounts"}}{Vector storing the number of NAs per site}
        }
}
\description{
Removes all probes with missing value (if such exists) from the given dataset.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
rnb.set.filtered <- rnb.execute.na.removal(rnb.set.example, 0)$dataset
identical(meth(rnb.set.example), meth(rnb.set.filtered)) # TRUE
}
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/greedycut.R
\name{rnb.execute.greedycut}
\alias{rnb.execute.greedycut}
\title{rnb.execute.greedycut}
\usage{
rnb.execute.greedycut(
  rnb.set,
  pval.threshold = rnb.getOption("filtering.greedycut.pvalue.threshold"),
  min.coverage = rnb.getOption("filtering.coverage.threshold"),
  rc.ties = rnb.getOption("filtering.greedycut.rc.ties")
)
}
\arguments{
\item{rnb.set}{HumanMethylation450K dataset as an object of type \code{\linkS4class{RnBeadSet}}.}

\item{pval.threshold}{The P-value threshold. For further information, see the option \code{"filtering.greedycut.pvalue.threshold"}
in \code{\link{rnb.options}}.}

\item{min.coverage}{The coverage threshold. For further information, see the option \code{"filtering.coverage.threshold"}
in \code{\link{rnb.options}}.}

\item{rc.ties}{Flag indicating what the behaviour of the algorithm should be in case of ties between values of rows
(probes) and columns (samples). See the corresponding parameter in
\code{\link{greedycut.filter.matrix}} for more details.}
}
\value{
\code{NULL} if \code{rnb.set} does not contain a matrix of detection p-values, or if all p-values denote
        reliable measurements. Otherwise, a list of the following elements:
        \describe{
          \item{"infos"}{Table summarizing the iterations of the algorithm, as returned by
               \code{\link{greedycut.filter.matrix}}.}
          \item{"statistics"}{Additional statistics on all iterations, as returned by
               \code{\link{greedycut.get.statistics}}.}
          \item{"iteration"}{Number of Greedycut iterations + \code{1} applied to the dataset, that is,
               a value of 1 indicates that the dataset was not modified.}
          \item{"sites"}{Indices of all sites to be removed.}
          \item{"samples"}{Indices of all samples to be removed.}
        }
}
\description{
Executes the Greedycut procedure for probe and sample filtering based on the detection p-values, and calculates
statistics on its iterations.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
greedy.result <- rnb.execute.greedycut(rnb.set.example)
# Number of applied iterations
greedy.result$iteration
}
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plottingUtils.R
\name{rnb.plot.beta.comparison}
\alias{rnb.plot.beta.comparison}
\title{rnb.plot.beta.comparison}
\usage{
rnb.plot.beta.comparison(
  beta.values,
  fprefix,
  report = NULL,
  qq.length = 501L,
  points.per.group = rnb.getOption("distribution.subsample")
)
}
\arguments{
\item{beta.values}{Two beta value sequences in the form of a named \code{list} of two non-empty vectors of type
\code{double}. If any of the vectors contains \code{NA}s, this method may exit with an error.}

\item{fprefix}{File name prefix for the plots. This function appends the suffixes \code{"_density"},
\code{"_histogram"} and \code{"_qq"} to this prefix.}

\item{report}{Report to which the plots are to be added.}

\item{qq.length}{Positive \code{integer} value showing the number of quantiles to be calculated and presented
in the generated Q-Q plot.}

\item{points.per.group}{Maximum number of values to use in plotting a group's distribution. Groups that contain more
observations than this threshold are subsampled. Setting this parameter to a value less than
\code{2} disables subsampling.}
}
\value{
List of all generated plots, each being an object ot type \code{\linkS4class{ReportPlot}}.
}
\description{
Draws plots that compare two distributions of beta values.
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/options.R
\name{rnb.options2xml}
\alias{rnb.options2xml}
\title{rnb.options2xml}
\usage{
rnb.options2xml(pretty = TRUE)
}
\arguments{
\item{pretty}{Flag indicating if the document should be formatted to be easily readable. For example, if this is set
to \code{TRUE} (default), every element is located on separate line. Formatting does not affect the
validity of the generated XML tree.}
}
\value{
XML document in the form of a \code{character} that encodes all options and their current values.
}
\description{
Exports all option values to an XML document.
}
\examples{
\donttest{
cat(rnb.options2xml(), file = "rnbeads_options.xml")
}
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/combining.R
\name{rnb.combine.seq}
\alias{rnb.combine.seq}
\title{rnb.combine.seq}
\usage{
rnb.combine.seq(x, y, type = "common")
}
\arguments{
\item{x}{An object of type \code{\link{RnBiseqSet-class}} used for concatenation}

\item{y}{Another object of type \code{\link{RnBiseqSet-class}} used for concatenation}

\item{type}{A character representing the type of combination. Needs to be one of \code{"common"},
\code{all.x}, \code{all.y} or \code{all}.}
}
\value{
An \code{\link{RnBiseqSet-class}} object with combined information
}
\description{
Initial implementation of the combine method for sequencing datasets.
}
\details{
The type parameters determines the mode of combination:
          \itemize{
            \item{\code{"common"}}{ The intersection between the sites present in the two 
                            datasets is used for the new dataset.}
            \item{\code{"all.x"}}{ All sites present in x are used.}
            \item{\code{"all.y"}}{ All sites present in y are used.}
            \item{\code{"all"}}{ The union between the sites of both datasets is used.}
          }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filtering.R
\name{rnb.execute.low.coverage.masking}
\alias{rnb.execute.low.coverage.masking}
\title{rnb.execute.low.coverage.masking}
\usage{
rnb.execute.low.coverage.masking(
  rnb.set,
  covg.threshold = rnb.getOption("filtering.coverage.threshold")
)
}
\arguments{
\item{rnb.set}{Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.}

\item{covg.threshold}{Threshold for minimal acceptable coverage, given as a non-negative \code{integer} value. All
methylation measurements with lower coverage than this threshold are set to \code{NA}. If this
parameter is \code{0}, calling this method has no effect.}
}
\value{
List of three elements:
        \describe{
          \item{\code{"dataset.before"}}{Copy of \code{rnb.set}.}
          \item{\code{"dataset"}}{The (possibly) modified dataset after retaining sites on autosomes only.}
          \item{\code{"mask"}}{A logical matrix of dimension \code{meth(rnb.set,type="sites")} indicating which
				   methylation values have been masked}
        }
}
\description{
Replaces all low coverage sites by \code{NA}.
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotations.R
\name{rnb.load.annotation.from.db}
\alias{rnb.load.annotation.from.db}
\title{rnb.load.annotation.from.db}
\usage{
rnb.load.annotation.from.db(types, assembly = "hg19")
}
\arguments{
\item{types}{One-element \code{character} vector giving the name of the region annotation. If this annotation
is already available, it will be overwritten for the current session.}

\item{assembly}{Genome assembly of interest. See \code{\link{rnb.get.assemblies}} for the list of supported genomes.}
}
\value{
Invisibly, \code{TRUE} if the annotation was loaded successfully; an error message if the objects in the
        given file do not encode an annotation.
}
\description{
Loads a previously region annotation from the RnBeads resource database
}
\details{
This function checks whether a region annotation is present in the RnBeads resources,
downloads the corresponding annotation file(s) from the and then runs \code{\link{rnb.load.annotation}}
to import the annotation.
}
\examples{
\donttest{
rnb.region.types() 
rnb.load.annotation.from.db(c("tiling1kb", "dynamicMethZiller2013"))
rnb.region.types()
}

}
\seealso{
\code{\link{rnb.load.annotation}} for loading annotation from a binary file
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrichment.R
\name{performGOenrichment.diffMeth.entrez}
\alias{performGOenrichment.diffMeth.entrez}
\title{performGOenrichment.diffMeth.entrez}
\usage{
performGOenrichment.diffMeth.entrez(
  gids,
  uids,
  ontology,
  assembly = "hg19",
  ...
)
}
\arguments{
\item{gids}{gene ids to test (entrez IDs)}

\item{uids}{ids to test against (universe)}

\item{ontology}{which ontology should be used (see \code{GOHyperGParams} from the \code{GOstats} package for details)}

\item{assembly}{Genome to be used. One of the following: hg19, mm9, mm10 or rn5}

\item{...}{arguments passed on to the parameters of \code{GOHyperGParams} from the \code{GOstats} package}
}
\value{
a \code{GOHyperGresult} object (see the \code{GOstats} package for further details)
}
\description{
performs Gene Ontology (GO) enrichment analysis for a list of Entrez identifiers
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
dmt <- get.table(dm,get.comparisons(dm)[1],"promoters")
annot <- annotation(rnb.set.example,"promoters")
all.promoters <- annot$entrezID
#get the hypermethylated promoters
hyper.promoters <- annot$entrezID[dmt[,"mean.mean.diff"]>0]
result <- performGOenrichment.diffMeth.entrez(hyper.promoters,all.promoters,"BP",assembly="hg19")
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBDiffMeth-class.R
\docType{methods}
\name{get.comparison.groupsizes,RnBDiffMeth-method}
\alias{get.comparison.groupsizes,RnBDiffMeth-method}
\alias{get.comparison.groupsizes}
\title{get.comparison.groupsizes-methods}
\usage{
\S4method{get.comparison.groupsizes}{RnBDiffMeth}(object)
}
\arguments{
\item{object}{\code{\linkS4class{RnBDiffMeth}} object}
}
\value{
character matrix containing comparison group sizes
}
\description{
Gets all comparison group sizes represented in the object as character matrix of dimension n.comparisons x 2
where the columns specify sizes of groups 1 and 2 respectively
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
get.comparison.groupsizes(dm)
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computeCluster.R
\docType{methods}
\name{setModuleNumCores,RnBClusterRun,integer,character-method}
\alias{setModuleNumCores,RnBClusterRun,integer,character-method}
\alias{setModuleNumCores}
\alias{setModuleNumCores,RnBClusterRun-method}
\title{setModuleNumCores-methods}
\usage{
\S4method{setModuleNumCores}{RnBClusterRun,integer,character}(object, num.cores, modules = "all")
}
\arguments{
\item{object}{\code{\linkS4class{RnBClusterRun}} object}

\item{num.cores}{an integer specifying the number of cores to be used}

\item{modules}{vector of applicable pipeline modules. Can be \code{"all"} to specify all modules}
}
\value{
The modified object
}
\description{
Specifies the number of cores used by the different pipeline modules
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cellTypeAdjustment.R
\name{get.covariates.ct}
\alias{get.covariates.ct}
\title{get.covariates.ct}
\usage{
get.covariates.ct(rnb.set)
}
\arguments{
\item{rnb.set}{\code{RnBSet} object}
}
\value{
an NxK matrix of K cell types contributions for N samples of the \code{rnb.set}. \code{NULL}
	   if the components have not been computed or added to \code{rnb.set}.
}
\description{
Retrieves an NxK matrix of cell type contributions stored in an RnBSet for a given target variable
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normalization.R
\name{rnb.execute.normalization}
\alias{rnb.execute.normalization}
\title{rnb.execute.normalization}
\usage{
rnb.execute.normalization(
  object,
  method = rnb.getOption("normalization.method"),
  bgcorr.method = rnb.getOption("normalization.background.method"),
  verbose = TRUE
)
}
\arguments{
\item{object}{Methylation dataset as an object of type \code{\linkS4class{MethyLumiSet}} or
\code{\linkS4class{RnBSet}}.}

\item{method}{Normalization method, must be one of \code{"none"}, \code{"illumina"}, \code{"swan"},
\code{"minfi.funnorm"}, \code{"bmiq"}, or \code{wm.*} where \code{*} stands for one of the methods
implemented in \pkg{wateRmelon} package.
Note that the execution of methods SWAN and minfi.funnorm requires packages \pkg{minfi} and
\pkg{IlluminaHumanMethylation450kmanifest}. The BMIQ method requires the package \pkg{RPMM}. The
\code{wm.*} methods naturally require \pkg{wateRmelon}.}

\item{bgcorr.method}{Character singleton specifying which background subtraction should be used. Only methods impemented
in the \pkg{methylumi} package are supported at the moment, namely \code{methylumi.noob}, \code{methylumi.goob}
and \code{methylumi.doob}. See Triche et al. for detailed description of the methods.}

\item{verbose}{flag specifying whether diagnostic output should be written to the console or to the RnBeads logger
in case the latter is initialized}
}
\value{
Normalized dataset as an object of type \code{\linkS4class{RnBeadSet}}.
}
\description{
Performs normalization of the provided HumanMethylation450 data set.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
rnb.set.norm<-rnb.execute.normalization(rnb.set.example, method="illumina", bgcorr.method="none")
}
}
\references{
1. Triche, Timothy J., Jr., Weisenberger, Daniel J., Van Den Berg, David, Laird, Peter W. and Siegmund, Kimberly D. (2013)
            Low-level processing of Illumina Infinium DNA Methylation BeadArrays.
            Nucleic Acids Research 41(7):e90-e90.
}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBeadRawSet-class.R
\docType{methods}
\name{M,RnBeadRawSet-method}
\alias{M,RnBeadRawSet-method}
\alias{M}
\title{M-methods}
\usage{
\S4method{M}{RnBeadRawSet}(object, row.names = FALSE)
}
\arguments{
\item{object}{Dataset of interest.}

\item{row.names}{Flag indicating whether the resulting matrix will be assigned row names}
}
\value{
\code{matrix} of the methylated probe intensities
}
\description{
Extract raw methylated probe intensity from an object of \code{RnBeadRawSet} class.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
M.intensity<-M(rnb.set.example)
head(M.intensity)
} 

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotations.R
\name{rnb.region.types}
\alias{rnb.region.types}
\title{rnb.region.types}
\usage{
rnb.region.types(assembly = "hg19")
}
\arguments{
\item{assembly}{Genome assembly of interest. See \code{\link{rnb.get.assemblies}} for the list of supported genomes.}
}
\value{
Region types supported by \pkg{RnBeads} in the form of a \code{character} vector. The built-in ones are
        \code{"cpgislands"}, \code{"genes"}, \code{"promoters"} and \code{"tiling"}. The names of all custom region
        definitions are also included in the returned vector.
}
\description{
Gets the supported region annotations for a given genome assembly.
}
\examples{
\donttest{
"promoters" \%in\% rnb.region.types() # TRUE
}

}
\seealso{
\code{\link{rnb.get.annotation}}, \code{\link{rnb.set.annotation}}
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotations.R
\name{rnb.get.chromosomes}
\alias{rnb.get.chromosomes}
\title{rnb.get.chromosomes}
\usage{
rnb.get.chromosomes(assembly = "hg19")
}
\arguments{
\item{assembly}{Genome assembly of interest. See \code{\link{rnb.get.assemblies}} for the list of supported genomes.}
}
\value{
\code{character} vector of supported chromosomes for the specified genome assembly. The elements of the
        vector follow the \href{http://www.ensembl.org/}{Ensembl} convention (\code{"1"}, \code{"2"}, ...), and the
        names of this vector - the convention of the \href{http://genome.ucsc.edu/}{UCSC Genome Browser}
        (\code{"chr1"}, \code{"chr2"}, ...).
}
\description{
Gets the chromosome names supported for the specified assembly.
}
\examples{
\donttest{
"chrX" \%in\% names(rnb.get.chromosomes())
}
}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBDiffMeth-class.R
\name{load.rnb.diffmeth}
\alias{load.rnb.diffmeth}
\title{load.rnb.diffmeth}
\usage{
load.rnb.diffmeth(path)
}
\arguments{
\item{path}{path of the saved object (a directory containing a corresponding \code{rnbDiffMeth.RData} file and possibly \code{rnbDiffMeth_tables} files)}
}
\value{
the loaded \code{\linkS4class{RnBDiffMeth}} object
}
\description{
load a saved \code{\linkS4class{RnBDiffMeth}} object from disk
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differentialMethylation.R
\name{auto.select.rank.cut}
\alias{auto.select.rank.cut}
\title{auto.select.rank.cut}
\usage{
auto.select.rank.cut(p, r, alpha = 0.1)
}
\arguments{
\item{p}{vector of p-values}

\item{r}{vector of ranks}

\item{alpha}{the percentile to select the top and bottom part of the list}
}
\value{
the maximum rank fulfilling the criterion
}
\description{
automatically select a rank cutoff for given ranks and p-values
current implementation: sort the p-values according to rank. select as rank cutoff
the rank for which the worst (i.e. max) p-value in the top list is still smaller than
the best (i.e. min) p-value of the group of worst-ranking p-values of equal size as the top-list
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\docType{methods}
\name{summarize.regions,RnBSet-method}
\alias{summarize.regions,RnBSet-method}
\alias{summarize.regions}
\title{summarize.regions-methods}
\usage{
\S4method{summarize.regions}{RnBSet}(
  object,
  region.type,
  aggregation = rnb.getOption("region.aggregation"),
  overwrite = TRUE
)
}
\arguments{
\item{object}{Dataset of interest.}

\item{region.type}{Type of the region annotation for which the summarization will be performed or \code{"strands"} for summarizing the methylation values from both strands}

\item{aggregation}{Operation to summarize the methylation values. Currently supported values are \code{"mean"}, \code{"median"}, \code{"min"}, \code{"max"} and \code{"coverage.weighted"}}

\item{overwrite}{If \code{TRUE} the existing region-level information for \code{region.type} is discarded}
}
\value{
object of the same class as the supplied one containing the summarized methylation information for the specified region types
}
\description{
Summarize DNA methylation information for which is present in the \code{RnBSet} object.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
rnb.set.summarized<-summarize.regions(rnb.set.example, "genes", overwrite=TRUE)
head(meth(rnb.set.summarized, type="genes", row.names=TRUE))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\name{load.rnb.set}
\alias{load.rnb.set}
\title{load.rnb.set}
\usage{
load.rnb.set(path, temp.dir = tempdir())
}
\arguments{
\item{path}{full path of the file or directory. If \code{archive} is \code{FALSE})
without an extension.}

\item{temp.dir}{\code{character} singleton which specifies temporary directory, used while loading}
}
\value{
Loaded object
}
\description{
Loading of the \code{RnBSet} objects with large matrices of type \pkg{ff}.
}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\docType{methods}
\name{getNumNaMeth,RnBSet-method}
\alias{getNumNaMeth,RnBSet-method}
\alias{getNumNaMeth}
\title{getNumNaMeth-methods}
\usage{
\S4method{getNumNaMeth}{RnBSet}(object, type = "sites", chunkSize = 1e+05, mask = NULL)
}
\arguments{
\item{object}{object inheriting from \code{\linkS4class{RnBSet}}}

\item{type}{"sites" or region type}

\item{chunkSize}{size of each submatrix (performance tuning parameter)}

\item{mask}{logical matrix. its entries will also be considered NAs in counting}
}
\value{
vector containing the number of NAs per site/region
}
\description{
for each site/region, the getNumNaMeth retrieves the number of NA values accross all samples.
Does this efficiently by breaking down the methylation matrix into submatrices
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/controlPlots.R
\name{rnb.plot.snp.boxplot}
\alias{rnb.plot.snp.boxplot}
\title{rnb.plot.snp.boxplot}
\usage{
rnb.plot.snp.boxplot(dataset, writeToFile = FALSE, ...)
}
\arguments{
\item{dataset}{Dataset as an object of type inheriting \code{\linkS4class{RnBeadSet}}, or a matrix of
methylation beta values.}

\item{writeToFile}{Flag specifying whether the output should be saved as \code{\linkS4class{ReportPlot}}.}

\item{...}{Additional named arguments passed to \code{\link{createReportPlot}}.}
}
\value{
If \code{writeToFile} is \code{TRUE}: plot as an object of type \code{\linkS4class{ReportPlot}}. Otherwise:
        plot as an object of type \code{\link{ggplot}}.
}
\description{
Box plots of beta-values from the genotyping probes
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
rnb.plot.snp.boxplot(rnb.set.example)
}
}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBeadRawSet-class.R
\name{intensities.by.color}
\alias{intensities.by.color}
\title{intensities.by.color}
\usage{
intensities.by.color(
  raw.set,
  address.rownames = TRUE,
  add.oob = all(!is.null(M0(raw.set)), !is.null(U0(raw.set))),
  add.controls = !is.null(qc(raw.set)),
  add.missing = TRUE,
  re.separate = FALSE
)
}
\arguments{
\item{raw.set}{Methylation dataset as an instance of \code{RnBeadRawSet} object.}

\item{address.rownames}{if \code{TRUE} the rows of the returned matrices are named with the with the corresponding Illumina probe addresses}

\item{add.oob}{if \code{TRUE} the "out-of-band" intensities are included}

\item{add.controls}{if \code{TRUE} the control probe intensities are included}

\item{add.missing}{if \code{TRUE} the rows for the probes missing in \code{raw.set} is imputed with \code{NA} values}

\item{re.separate}{if \code{TRUE} the type I and type II intensities, as well as the out-of-band and control probe intensities
(if set to \code{TRUE}), will be returned as separate elements per channel and not as concatenated rows.}
}
\value{
A \code{list} with elements \code{Cy3} and \code{Cy5} containing average bead intensities measured for each
        each probe in the green and red channels, respectively. Exception, if \code{re.separate} is \code{TRUE} a \code{list}
        with elements \code{Cy3.I}, \code{Cy5.I}, and \code{II} will be returned. The elements \code{Cy3.I.oob}, 
        \code{Cy5.I.oob} and also \code{Cy3.ctl}, \code{Cy5.ctl} will be returned 
        if the respective parameters (\code{add.oob} and \code{add.ctl}) are set to true.
}
\description{
Rearranges information from "M" and "U" slots of a RnBeadsRawSet object by color channel.
}
\author{
Pavlo Lutsik, Nathan Steenbuck
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differentialMethylation.R
\name{rnb.execute.computeDiffMeth}
\alias{rnb.execute.computeDiffMeth}
\title{rnb.execute.computeDiffMeth}
\usage{
rnb.execute.computeDiffMeth(
  x,
  pheno.cols,
  region.types = rnb.region.types.for.analysis(x),
  covg.thres = rnb.getOption("filtering.coverage.threshold"),
  pheno.cols.all.pairwise = rnb.getOption("differential.comparison.columns.all.pairwise"),
  columns.pairs = rnb.getOption("columns.pairing"),
  columns.adj = rnb.getOption("covariate.adjustment.columns"),
  adjust.sva = rnb.getOption("differential.adjustment.sva"),
  pheno.cols.adjust.sva = rnb.getOption("inference.targets.sva"),
  adjust.celltype = rnb.getOption("differential.adjustment.celltype"),
  skip.sites = !rnb.getOption("analyze.sites"),
  disk.dump = rnb.getOption("disk.dump.big.matrices"),
  disk.dump.dir = tempfile(pattern = "diffMethTables_"),
  ...
)
}
\arguments{
\item{x}{RnBSet object}

\item{pheno.cols}{column names of the pheno slot in \code{x} on which the dataset should be partitioned. Those columns are required to be factors or logical.
In case of factors, each group in turn will be compared to all other groups}

\item{region.types}{which region types should be processed for differential methylation}

\item{covg.thres}{coverage threshold for computing the summary statistics. See \code{\link{computeDiffTab.extended.site}} for details.}

\item{pheno.cols.all.pairwise}{integer or character vector specifying the colomns of \code{pheno(x)} on which all pairwise comparisons should be conducted.
A value of \code{NULL} (default) indicates no columns.}

\item{columns.pairs}{argument passed on to \code{rnb.sample.groups}. See its documentation for details.}

\item{columns.adj}{Column names or indices in the table of phenotypic information to be used for confounder adjustment in the
differential methylation analysis.}

\item{adjust.sva}{flag indicating whether the adjustment table should also contain surrogate variables (SVs) for the given target variable.}

\item{pheno.cols.adjust.sva}{Column names or indices in the table of phenotypic information to be used for SVA adjustment in the
differential methylation analysis.}

\item{adjust.celltype}{flag indicating whether the resulting table should also contain estimated celltype contributions.
See \code{\link{rnb.execute.ct.estimation}} for details.}

\item{skip.sites}{flag indicating whether differential methylation in regions should be computed directly and not from sites. This leads to skipping of site-specific differential methylation}

\item{disk.dump}{Flag indicating whether the resulting differential methylation object should be file backed, ie.e the matrices dumped to disk}

\item{disk.dump.dir}{disk location for file backing of the resulting differential methylation object. Only meaningful if \code{disk.dump=TRUE}.
must be a character specifying an NON-EXISTING valid directory.}

\item{...}{arguments passed on to binary differential methylation calling. See \code{\link{computeDiffTab.extended.site}} for details.}
}
\value{
an \code{\linkS4class{RnBDiffMeth}} object. See class description for details.
}
\description{
computes differential methylation
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
get.comparisons(dm)
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differentialVariability.R
\name{rnb.section.diffVar.region}
\alias{rnb.section.diffVar.region}
\title{rnb.section.diffVar.region}
\usage{
rnb.section.diffVar.region(
  rnb.set,
  diff.meth,
  report,
  gzTable = FALSE,
  level = 1
)
}
\arguments{
\item{rnb.set}{Object of type \code{\linkS4class{RnBSet}} containing methylation information}

\item{diff.meth}{RnBDiffMeth object. See \code{\link{RnBDiffMeth-class}} for details.}

\item{report}{Report object to which the content is added}

\item{gzTable}{Flag indicating if tables should be gzipped}

\item{level}{Which level of section should be created. See \code{rnb.add.section}.}
}
\value{
The modified report object
}
\description{
Adds information for differentially variable regions to the report.
}
\author{
Michael Scherer
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differentialVariability.R
\name{diffVar}
\alias{diffVar}
\title{diffVar}
\usage{
diffVar(meth.matrix, inds.g1, inds.g2, adjustment.table = NULL, paired = FALSE)
}
\arguments{
\item{meth.matrix}{Matrix containing the methylation information used to calculate differentially variable sites 
between the two groups}

\item{inds.g1}{Indices in the phenotypic table corresponding to the first group.}

\item{inds.g2}{Indices in the phenotypic table corresponding to the second group.}

\item{adjustment.table}{A \code{data.frame} containing variables to adjust for in the testing}

\item{paired}{Should the analysis be performed in a paired fashion. If yes, the first index in \code{inds.g1} must 
correspond to the first in \code{inds.g2} and so on.}
}
\value{
P-values as the result of the diffVar method not adjusted for multiple hypothesis testing.
}
\description{
This routine applies the diffVar method from the \code{missMethyl} package that determines sites exhibiting
differential variability between two sample groups
}
\references{
Phipson, Belinda, Oshlack, Alicia (2014)
            DiffVar: a new method for detecting differential variability with application to methylation in cancer and aging
            Genome Biology 15(9):465.
}
\author{
Michael Scherer
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cellTypeAdjustment.R
\name{rnb.plot.ct.heatmap}
\alias{rnb.plot.ct.heatmap}
\title{rnb.plot.ct.heatmap}
\usage{
rnb.plot.ct.heatmap(ct.obj, type = "nonnegative", writeToFile = FALSE, ...)
}
\arguments{
\item{ct.obj}{Object of class \code{CellTypeInferenceResult} as returned by \link{rnb.execute.ct.estimation}.}

\item{type}{Type of cell type contributions to plot.}

\item{writeToFile}{If \code{TRUE}, the plot will be written to a file.}

\item{...}{Other arguments passed to \code{\link{createReportPlot}}.}
}
\value{
if \code{writeToFile=TRUE} an object of class \code{\linkS4class{ReportPlot}}, 
						or the protted matrix otherwise
}
\description{
Plot contributions of the cell types
}
\details{
The cell type contributions are visualized as a heatmap
}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBeadSet-class.R
\docType{class}
\name{RnBeadSet-class}
\alias{RnBeadSet-class}
\alias{RnBeadSet}
\alias{initialize,RnBeadSet-method}
\title{RnBeadSet Class}
\usage{
RnBeadSet(
  pheno,
  probes,
  betas,
  p.values = NULL,
  bead.counts = NULL,
  qc = NULL,
  platform = "450k",
  summarize.regions = TRUE,
  region.types = rnb.region.types.for.analysis(ifelse(platform == "MMBC", "mm10",
    rnb.getOption("assembly"))),
  useff = rnb.getOption("disk.dump.big.matrices")
)
}
\arguments{
\item{pheno}{Phenotypic data.}

\item{probes}{\code{character} vector of Infinium(R) probe identifiers}

\item{betas}{\code{matrix} or \code{ff_matrix} of beta values. If \code{probes} are missing should contain Infinium probe identifiers as row names.}

\item{p.values}{\code{matrix} or \code{ff_matrix} of detection p-values.}

\item{bead.counts}{...}

\item{qc}{...}

\item{platform}{\code{character} singleton specifying the microarray platform: \code{"450k"} corresponds to HumanMethylation450 microarray, and \code{"27k"} stands for HumanMethylation27.}

\item{summarize.regions}{...}

\item{region.types}{A \code{character} vector specifying the region types, for which the methylation infromation will be summarized.}

\item{useff}{If \code{TRUE} the data matrices will be stored as \code{ff} objects}
}
\value{
an object of class RnBeadSet
}
\description{
Stores the preprocessed information from HumanMethylation experiments
}
\details{
There are multiple ways to create an object of type \code{RnBeadSet}:
\describe{
 \item{Loading from files}{Dataset can be loaded from text or binary files. See the function
      \code{\link{rnb.execute.import}} for more details.}
 \item{Downloading from GEO}{See the function \code{\link{rnb.read.geo}} for details.}
 \item{Converting from \code{MethyLumiSet}}{...}
}
}
\section{Slots}{

\describe{
  \item{\code{pval.sites}}{\code{matrix} of detection p-values with the same dimensions as \code{betas}, or
       \code{NULL} if the detection p-values are not available.}
  \item{\code{pval.regions}}{\code{list} of methylation \code{matrix} objects, one per available region type. Every row in a 
		matrix corresponds to a methylation site, and every column - to a sample.}
  \item{\code{covg.sites}}{\code{matrix} of bead counts per probe with the same dimensions as \code{betas}, or
       \code{NULL} if this data are not available.}
  \item{\code{qc}}{Quality control probe information in the form of a \code{list} of two elements - \code{"Cy3"} and
       \code{"Cy5"}, storing intensities of probes on the green and red channels, respectively. This slot's value is
       \code{NULL} if no control probe information is available.}
}
}

\section{Methods and Functions}{

\describe{
  \item{\code{\link[=samples,RnBSet-method]{samples}}}{Gets the identifiers of all samples in the dataset.}
  \item{\code{\link[=pheno,RnBSet-method]{pheno}}}{Gets the phenotypic and processing data of the dataset.}
  \item{\code{\link[=meth,RnBSet-method]{meth}}}{Gets the \code{matrix} of methylation beta-values of the dataset.}
  \item{\code{\link[=dpval,RnBeadSet-method]{dpval}}}{Gets the \code{matrix} of detection p-values of the dataset.}
  \item{\code{\link[=covg,RnBSet-method]{covg}}}{Gets the \code{matrix} of bead counts of the dataset.}
  \item{\code{\link[=qc,RnBeadSet-method]{qc}}}{Gets the intensities of the quality control probes.}
  \item{\code{\link[=remove.sites,RnBSet-method]{remove.sites}}}{Removes probes from the dataset.}
  \item{\code{\link[=remove.samples,RnBeadSet-method]{remove.samples}}}{Removes samples from the dataset.}
  \item{\code{\link[BiocGenerics]{combine}}}{Combines two datasets.}
}
}

\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/immune.R
\name{rnb.execute.lump}
\alias{rnb.execute.lump}
\title{Leukocytes unmethylation for purity}
\usage{
rnb.execute.lump(dataset)
}
\arguments{
\item{dataset}{Methylation dataset to study, provided as an object of type inheriting \code{RnBSet}.}
}
\value{
Purity esimates provided as a \code{vector} of values in the range \code{[0, 1]}. The attribute
        \code{"sites"} contains the number of sites used in estimating the immune cell proportions. In case the
        dataset does not contain measurements for any of the sites on which LUMP focuses, the return values is
        \code{NULL}.
}
\description{
Implementation of the LUMP (Leukocytes UnMethylation for Purity) algorithm for purity estimation on methylation
datasets.
}
\details{
The LUMP algorithm is developed by Dvir Aran, Marina Sirota and Atul J. Buttea.
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/controlPlots.R
\name{rnb.plot.control.barplot}
\alias{rnb.plot.control.barplot}
\title{rnb.plot.control.barplot}
\usage{
rnb.plot.control.barplot(
  rnb.set,
  probe,
  sample.subset = 1:length(samples(rnb.set)),
  writeToFile = FALSE,
  numeric.names = FALSE,
  name.prefix = NULL,
  verbose = FALSE,
  ...
)
}
\arguments{
\item{rnb.set}{\code{\linkS4class{RnBeadRawSet}} or \code{\linkS4class{RnBeadSet}} object with valid
quality control information}

\item{probe}{exact id of the control probe consisting of the control probe type (see \code{\link{rnb.plot.control.boxplot}})}

\item{sample.subset}{an integer vector specifying the subset of samples for which the plotting should be performed}

\item{writeToFile}{flag specifying whether the output should be saved as \code{\linkS4class{ReportPlot}}}

\item{numeric.names}{if \code{TRUE} and \code{writeToFile} is \code{TRUE}substitute the plot options in the plot file name with digits}

\item{name.prefix}{in case \code{writeToFile} is \code{TRUE}, a \code{character} singleton specifying a prefix to the variable part of the image file names}

\item{verbose}{if \code{TRUE} additional diagnostic output is generated}

\item{...}{other arguments to \code{\link{createReportPlot}}}
}
\value{
plot as an object of type \code{\linkS4class{ReportPlot}} if \code{writeToFile} is \code{TRUE} and of class
			\code{\link{ggplot}} otherwise.
}
\description{
Per-sample bar plots of Illumina HumanMethylation control probes
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
control.meta.data <- rnb.get.annotation("controls450")
ctrl.probe<-paste0(unique(control.meta.data[["Target"]])[4], ".3")
print(ctrl.probe) # EXTENSION.3
rnb.plot.control.barplot(rnb.set.example, ctrl.probe)
}

}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\docType{methods}
\name{addPheno,RnBSet-method}
\alias{addPheno,RnBSet-method}
\alias{addPheno}
\title{addPheno}
\usage{
\S4method{addPheno}{RnBSet}(object, trait, header)
}
\arguments{
\item{object}{\code{\linkS4class{RnBSet}} of interest.}

\item{trait}{Trait as a non-empty \code{vector} or \code{factor}. The length of this vector must be equal to the
number of samples in \code{object}, the i-th element storing the value for the i-th sample. Note that
names, if present, are ignored.}

\item{header}{Trait name given as a one-element \code{character}. This is the heading to be used for the sample
annotation table. This method fails if such a trait already exists; in other words, if
\code{header \%in\% names(pheno(object))}.}
}
\value{
The modified dataset as an object of type \code{\linkS4class{RnBSet}}.
}
\description{
Adds phenotypic or processing information to the sample annotation table of the given \code{RnBSet} object.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
is.hiPSC <- pheno(rnb.set.example)[, "Sample_Group"]=="hiPSC"
rnb.set.mod <- addPheno(rnb.set.example, is.hiPSC, "is_hiPSC")
pheno(rnb.set.mod)
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\docType{methods}
\name{summarized.regions,RnBSet-method}
\alias{summarized.regions,RnBSet-method}
\alias{summarized.regions}
\title{summarized.regions-methods}
\usage{
\S4method{summarized.regions}{RnBSet}(object)
}
\arguments{
\item{object}{Methylation dataset of interest.}
}
\value{
\code{character} vector listing all genomic annotations summarized in the given dataset. If the dataset
        contains methylation in sites only, an empty vector is returned.
}
\description{
Gets the genomic annotations for which methylation data is present in the \code{RnBSet} object.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
summarized.regions(rnb.set.example)
}
}
\seealso{
\code{\link[=summarize.regions,RnBSet-method]{summarize.regions}} for calculating region-wise methylation in a dataset;
         \code{\link{rnb.set.annotation}} for adding or replacing a region annotation table
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/segmentation.R
\name{rnb.bed.from.segmentation}
\alias{rnb.bed.from.segmentation}
\title{rnb.bed.from.segmentation}
\usage{
rnb.bed.from.segmentation(
  rnb.set,
  sample.name,
  type = "final",
  store.path = getwd()
)
}
\arguments{
\item{rnb.set}{An \code{\link{RnBSet-class}} object obtained by executing \code{rnb.execute.segmentation}.}

\item{sample.name}{The sample name for which segmentation was computed.}

\item{type}{The type of segmentation (\code{PMDs}, \code{UMRs}, \code{LMRs}, \code{HMDs} or \code{final}).}

\item{store.path}{Path to which the BED file is to be stored.}
}
\description{
This function creates a BED file from the segmentation result of \code{rnb.execute.segmentation} and stores it on disk.
}
\author{
Michael Scherer
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logger.R
\name{logger.getfiles}
\alias{logger.getfiles}
\title{logger.getfiles}
\usage{
logger.getfiles()
}
\value{
Vector storing the full names of the files that are being used by the logger. This vector contains \code{NA}
        as an element if the logger is (also) using the console for its output. If logging functionality is disabled
        (see \code{\link{rnb.options}}) or the logger is not initialized, this function returns \code{NULL}.
}
\description{
Gets the files currently used by the logger.
}
\examples{
\donttest{
if (NA \%in\% logger.getfiles())
  cat("Console logger is enabled\n")
}
}
\seealso{
\code{\link{logger.isinitialized}} to check if logging is activated;
  \code{\link{logger.start}} for initializing a logger or starting a section
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/agePrediction.R
\name{rnb.execute.training}
\alias{rnb.execute.training}
\title{rnb.execute.training}
\usage{
rnb.execute.training(object, path = "", alpha = 0.8)
}
\arguments{
\item{object}{a \code{\linkS4class{RnBSet}} object on which a new predictor should be created}

\item{path}{path to which the predictor should be written out}

\item{alpha}{alpha parameter used in the elastic net regression}
}
\description{
Trains a new age predictor on the specified data set and writes it to the given path. Elastic net regression is
to fit the input ages to the methylation values .
}
\author{
Michael Scherer
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataExport.R
\name{rnb.RnBSet.to.bedGraph}
\alias{rnb.RnBSet.to.bedGraph}
\title{rnb.RnBSet.to.bedGraph}
\usage{
rnb.RnBSet.to.bedGraph(
  rnb.set,
  out.dir = ".",
  reg.type = "sites",
  parameters = character(),
  digits = NULL
)
}
\arguments{
\item{rnb.set}{Dataset as an instance of class \code{\linkS4class{RnBSet}}.}

\item{out.dir}{One-element \code{character} vector signifying the output directory in which to create
\code{bedGraph} files. Setting this to \code{"."} (default) uses the current working directory. If
the output directory does not exist, this function attempts to create it. Any existing files in
this directory could be overwritten.}

\item{reg.type}{Site or region type to be exported.}

\item{parameters}{Named \code{character} vector storing parameters (other than \code{"type"} and \code{"name"}) to
include in the track definition line. The names of this vector must be the parameter names, and its
elements - the corresponding values; missing values (\code{NA}s) are allowed neither for names, nor
for values. This function does not test if all provided parameter names and values conform to the
BedGraph track specification.}

\item{digits}{Optionally, number of significant digits after the decimal point to round methylation values to. If
specified, this parameter must be an \code{integer} between \code{0} and \code{10}.}
}
\value{
(invisibly) a summary list containing information on the conversion step.
        elements are \code{filenames} (a table containing information on which sample has been written to what filename)
        and \code{assembly} (a string indicating the assembly used by \code{rnb.set}).
}
\description{
Exports the methylation data of an \code{\linkS4class{RnBSet}} object to \code{*.bedGraph} files.
}
\details{
The description of the BedGraph track format can be found \href{http://genome.ucsc.edu/goldenPath/help/bedgraph.html}{here}.
         Each methylation site is an entry in the resulting bedGraph file. The Score column corresponds to a site's
         methylation value in the interval \code{[0,1]}.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
rnb.RnBSet.to.bedGraph(rnb.set.example,tempdir())
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sva.R
\name{has.covariates.sva}
\alias{has.covariates.sva}
\title{has.covariates.sva}
\usage{
has.covariates.sva(rnb.set, target)
}
\arguments{
\item{rnb.set}{\code{RnBSet} object}

\item{target}{target variable. Must be in \code{pheno(rnb.set)} and belong to target variables for which the
SVs have already been computed and stored in the RnBSet.}
}
\value{
\code{logical(1)}
}
\description{
Returns whether Surrogate Variables have been computed and added to the \code{rnb.set} for a given target variable
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
sva.obj <- rnb.execute.sva(rnb.set.example,c("Sample_Group","Treatment"),numSVmethod="be")
sva.obj$sva.performed
sva.obj$num.components
rnb.set.mod <- set.covariates.sva(rnb.set.example, sva.obj)
has.covariates.sva(rnb.set.example,"Sample_Group")
has.covariates.sva(rnb.set.mod,"Sample_Group")
has.covariates.sva(rnb.set.mod,"Treatment")
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lolaUtils.R
\name{getCellTypesFromLolaDb}
\alias{getCellTypesFromLolaDb}
\title{getCellTypesFromLolaDb}
\usage{
getCellTypesFromLolaDb(lolaDb)
}
\arguments{
\item{lolaDb}{LOLA DB object as returned by \code{LOLA::loadRegionDB} or \code{\link{loadLolaDbs}}}
}
\value{
character vector with cell types
}
\description{
retrieve or guess cell types from a LOLA DB object
}
\examples{
\donttest{
# download LOLA DB
lolaDest <- tempfile()
dir.create(lolaDest)
lolaDirs <- downloadLolaDbs(lolaDest, dbs="LOLACore")
lolaDb <- loadLolaDbs(lolaDirs[["hg19"]])
getCellTypesFromLolaDb(lolaDb)
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R, R/clustering.R
\docType{methods}
\name{samples,RnBSet-method}
\alias{samples,RnBSet-method}
\alias{samples}
\alias{samples,RnBeadClustering-method}
\title{samples-methods}
\usage{
\S4method{samples}{RnBSet}(object)

\S4method{samples}{RnBeadClustering}(object)
}
\arguments{
\item{object}{Dataset of interest.}
}
\value{
\code{character} vector of sample identifiers.
}
\description{
Extracts sample identifiers
}
\details{
The column of the sample annotation table which contains identifiers is globally controlled via the
 \code{"identifiers.column"} option. In case the latter is \code{NULL} column names of the matrix returned
by the \code{meth} method are treated as sample identifiers. In case the latter are also missing, a \code{character}
vector with sample numbers is returned.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
samples(rnb.set.example)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBeads-package.R
\docType{data}
\name{interspecies_probes_v2}
\alias{interspecies_probes_v2}
\title{XenoMix interspeciesprobes (v2)}
\format{
\code{list} of CGIs for the inter-species probes.
}
\description{
These are the inter-species probes for EPICv2
}
\author{
Nima Esmaeelpour
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterArchitectureLSF.R
\docType{methods}
\name{getSubCmdTokens,ClusterArchitectureLSF-method}
\alias{getSubCmdTokens,ClusterArchitectureLSF-method}
\title{getSubCmdTokens-methods}
\usage{
\S4method{getSubCmdTokens}{ClusterArchitectureLSF}(
  object,
  cmd.tokens,
  log,
  job.name = "",
  res.req = character(0),
  depend.jobs = character(0)
)
}
\arguments{
\item{object}{\code{\linkS4class{ClusterArchitectureLSF}} object}

\item{cmd.tokens}{a character vector specifying the executable command that should be wrapped in the cluster submission command}

\item{log}{file name and path of the log file that the submitted job writes to}

\item{job.name}{name of the submitted job}

\item{res.req}{named vector of requested resources. Two options are available: \code{"clock.limit"} and \code{"memory.size"}}

\item{depend.jobs}{character vector containg names or ids of jobs the submitted job will depend on.}
}
\value{
A character vector containing the submission command tokens
}
\description{
Returns a string for the of command line corresponding to submitting
a job with the given command to the cluster.
}
\details{
For a concrete child class implementation for a LSF architecture specification see \code{\linkS4class{ClusterArchitectureLSF}}
}
\examples{
\donttest{
arch <- new("ClusterArchitectureLSF",
	name="my_lsf_architecture"
)
getSubCmdTokens(arch,c("Rscript","my_great_script.R"),"my_logfile.log")
}
}
\author{
Michael Scherer
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cellTypeAdjustment.R
\name{rnb.execute.ct.estimation}
\alias{rnb.execute.ct.estimation}
\title{rnb.execute.ct.estimation}
\usage{
rnb.execute.ct.estimation(
  rnb.set,
  cell.type.column = NA,
  test.max.markers = NA,
  top.markers = 500,
  method = "houseman1",
  verbose = TRUE
)
}
\arguments{
\item{rnb.set}{object of class \code{\linkS4class{RnBSet}}}

\item{cell.type.column}{integer index or character identifier of a column in sample annotation table of \code{rnb.set}
which gives the mapping of samples to reference cell types}

\item{test.max.markers}{Maximal amount of CpG positions to use for marker selection. If this option is set to
   \code{NA} or \code{NULL}, all sites are considered. Please take into account the extended
computation time in such a case.}

\item{top.markers}{the number of markers to select}

\item{method}{algorithm used for estmation of the cell type contributions}

\item{verbose}{flag specifying whether diagnostic output should be written to the console 
or to the RnBeads logger in case the latter is initialized}
}
\value{
object of class \code{CellTypeInferenceResult}
}
\description{
Perform the estimation of the cell type contributions in each analyzed sample.
}
\details{
The only supported method is the one from Houseman et al BMC Bioinformatics 2012
}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotations.R
\name{rnb.export.all.annotation}
\alias{rnb.export.all.annotation}
\title{rnb.export.all.annotation}
\usage{
rnb.export.all.annotation(
  out.dir,
  types = c("CpG", rnb.region.types(assembly)),
  assembly = "hg19",
  format = "bed"
)
}
\arguments{
\item{out.dir}{The directory to write the files to}

\item{types}{One-element \code{character} vector giving the name of the region annotation.}

\item{assembly}{Genome assembly of interest. See \code{\link{rnb.get.assemblies}} for the list of supported genomes.}

\item{format}{output format. currently only \code{"bed"} is supported.}
}
\value{
\code{TRUE}, invisibly.
}
\description{
Wrapper for exporting all annotation sets
}
\examples{
\donttest{
logger.start(fname=NA)
rnb.export.all.annotation(tempdir(),c("genes","promoters"))
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loading.R
\name{rnb.execute.import}
\alias{rnb.execute.import}
\title{rnb.execute.import}
\usage{
rnb.execute.import(
  data.source,
  data.type = rnb.getOption("import.default.data.type"),
  dry.run = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{data.source}{non-empty \code{character} vector or \code{list} specifying the location of the data items. The
expected format depends on the \code{data.type} that is given. See the \emph{Details} section.}

\item{data.type}{type of the input data; must be one of \code{"idat.dir"}, \code{"data.dir"}, \code{"data.files"},
\code{"GS.report"}, \code{"GEO"} or \code{"rnb.set"}.}

\item{dry.run}{if \code{TRUE} and \code{data.type} is \code{"bs.bed.dir"}, only a test data import is performed and first 10,000 lines are read from each BED file}

\item{verbose}{flag specifying whether diagnostic output should be written to the console or to the RnBeads logger
in case the latter is initialized}
}
\value{
Loaded data as an object of type \code{\linkS4class{RnBSet}} (when the input data type is
        \code{"data.dir"}, \code{"data.files"} or \code{"GEO"}) or of type \code{\linkS4class{MethyLumiSet}} (when
        the data type is \code{"idat.dir"} or \code{"GS.report"}).
}
\description{
Loads the data from the specified type and encapsulates it in either an \code{\linkS4class{RnBSet}}-inheriting object
}
\details{
The interpretation of \code{data.source} depends on the value of \code{data.type} and is summarized in the
following table:
\tabular{lccl}{
  \bold{\code{data.type}} \tab \bold{Type of \code{data.source}} \tab \bold{Maximal length of \code{data.source}} \tab
    \bold{Interpretation}\cr
  \code{"infinium.idat.dir"}  \tab \code{list} or \code{character} \tab \code{2}       \tab
    (1) Directory containing IDAT files; (2) a sample annotation table as a \code{data.frame} or the name of the corresponding file\cr
  \code{"infinium.data.dir"}  \tab \code{character} \tab \code{1} \tab
   Directory containing data tables in plain text format. The directory should contain one file with \code{Sample|sample} token in the filename
for the table of sample annotations, and one file with a token \code{beta} in the filename, with beta-values. It may also contain tables with p-values
(token \code{pval}) and bead counts (\code{bead}). In the latter case beta-value, p-value and bead count tables should have matching columns and rows.
The beta-value, p-value and bead-count tables should contain row names, i.e. the first column should contain the Infinium CG identifiers and not have
a column header (for that the first row should have one entry less than all other rows). Sample annotation table should contain as many rows as there are
columns in other tables. The character used as value separator in the text tables can be set using the \code{import.table.separator} option
(see \code{\link{rnb.options}} for details). \cr
	 \code{"infinium.data.files"} \tab \code{character} \tab \code{2..4} \tab
	The character vector should contain at least full paths to the sample annotation file and beta-value table. Detection p-values and bead counts table
 can be added as the third and the fourth elements. The table format requirements are the same as for \code{"data.dir"} above.\cr
  \code{"infinium.GS.report"} \tab \code{character}                \tab \code{1}       \tab
    Genome Studio report file\cr
  \code{"infinium.GEO"}       \tab \code{character}                \tab \code{1}       \tab
    \href{http://www.ncbi.nlm.nih.gov/geo/}{GEO} identifier or downloaded series matrix file\cr
  \code{"bs.bed.dir"}       \tab \code{list} or \code{character}                \tab \code{1..3}       \tab
    (1) Directory with BED files each giving a DNA methylation profile of a sample; (2) a sample annotation table as a \code{data.frame} or the name of the corresponding file;
		(3) number of the sample annotation sheet column containing the file names. One of the first two elements have to be present. In case only the directory is specified,
		it should contain a sample annotation file with a token "sample" in the file name. In case only the sample sheet is specified, one column should be giving full absolute paths
		of the BED-like files with sequencing information. If both elements (1) and (2) are specified, the files should reside in the directory, specified as element (1).
		If the third element is absent, an attempt will be made to find the file name containing column automatically. For this reason the file names in the sample annotation sheet
		should be given with extensions (".bed", ".cov" etc).\cr
 \code{"rnb.set"}       \tab \code{\linkS4class{RnBSet}}                \tab \code{1}       \tab
    object of class inheriting from \code{\linkS4class{RnBSet}}\cr
}
}
\examples{
\donttest{
# Directory where your data is located
data.dir <- "~/RnBeads/data/Ziller2011_PLoSGen_450K"
idat.dir <- file.path(data.dir, "idat")
sample.annotation <- file.path(data.dir, "sample_annotation.csv")
data.source <- c(idat.dir, sample.annotation)
rnb.set <- rnb.execute.import(data.source = data.source, data.type = "idat.dir")
}
}
\seealso{
\code{\link{read.data.dir}}, \code{\link{read.idat.files}}, \code{\link{read.GS.report}},
         \code{\link{rnb.read.geo}}, \code{\link{read.bed.files}}
#'
}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bigFf.R
\name{dim,BigFfMat-method}
\alias{dim,BigFfMat-method}
\title{Dimensions of BigFfMat}
\usage{
\S4method{dim}{BigFfMat}(x)
}
\arguments{
\item{x}{BigFfMat object}
}
\description{
Dimensions of BigFfMat
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\docType{methods}
\name{nsites,RnBSet-method}
\alias{nsites,RnBSet-method}
\alias{nsites}
\title{nsites-methods}
\usage{
\S4method{nsites}{RnBSet}(object, type = "sites")
}
\arguments{
\item{object}{\code{RnBSet} of interest.}

\item{type}{\code{character} singleton. If this is set to \code{"sites"} (default), the number of sites is returned.
Otherwise, this should be one of region types for for which the number of regions is returned.}
}
\value{
\code{integer} stating the number of sites/regions. \code{NA} if the regions have not been summarized yet.
}
\description{
Returns the number of sites/regions for a given \code{RnBSet} object
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
nsites(rnb.set.example)
}
}
\seealso{
\code{\link[=meth,RnBSet-method]{meth}} Retrieving the matrix of methylation values
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parallelProcessing.R
\name{parallel.isEnabled}
\alias{parallel.isEnabled}
\title{parallel.isEnabled}
\usage{
parallel.isEnabled()
}
\value{
\code{TRUE} if multicore processing is enabled, \code{FALSE} otherwise.
}
\description{
Checks if whether parallel processing is enabled.
}
\examples{
\donttest{
parallel.isEnabled()
parallel.setup(2)
parallel.isEnabled()
parallel.teardown()
parallel.isEnabled()
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cellTypeAdjustment.R
\name{estimateProportionsCP}
\alias{estimateProportionsCP}
\title{estimateProportionsCP}
\usage{
estimateProportionsCP(
  rnb.set,
  cell.type.column,
  n.most.variable = NA,
  n.markers = 500L,
  constrained = TRUE,
  full.output = FALSE
)
}
\arguments{
\item{rnb.set}{RnBSet object}

\item{cell.type.column}{integer index or character identifier of a column
in the RnBSet object sample annotation table 
which gives the mapping to reference cell type
samples}

\item{n.most.variable}{Singleton integer specifying how many top variable CpGs should be used for marker selection.
   If this option is set to \code{NA} or \code{NULL}, all sites are considered. Please take
into account the extended computation time in such a case.}

\item{n.markers}{singleton integer specifying how many CpGs should
be used as markers for fitting the projection model}

\item{constrained}{if \code{TRUE} the returned cell type proportion estimates
are non-negative}

\item{full.output}{if \code{TRUE} not only the estimated proportions 
but also the intermediate analysis results are returned}
}
\value{
a matrix of estimated cell type contributions (samples times cell types) or a list with results of the intermetidate steps (see details).
}
\description{
Estimates cell type proportions using the constrained projection method from Houseman et al. [1]
}
\details{
This is a minimally customized implementation of the method by Houseman et al. [1] based on the orginial code kindly provided by 
Andres Houseman. Note that RnBeads does not provide any reference data sets, and the methylomes of purified cell types should be provided 
by the user as a part of the object supplied via \code{rnb.set}. The column specified by \code{cell.type.column} should 
give assignment of each reference methylome replicate to a cell type and missing values for all the target samples. 
First the marker selection model is fit to estimate association of each CpG with the given reference cell types (first expression in eq. (1) of [1]). 
The strength of association is expressed as an F-statistic.
Since fitting the marker selection model to all CpGs can take a lot of time, one can limit the marker search only to variable CpG positions
by setting \code{n.most.variable} to non-\code{NA} positive integer. The CpGs will be ranked using across-sample variance in the 
reference data set and \code{n.most.variable} will be taken to fit the marker selection model.
Coefficients of the fit, together with the F-statistic value for each CpG, are returned in case \code{full.output} is \code{TRUE}. 
Thereafter, \code{n.markers} are selected as true quantitative markers and the projection model (eq. [2]) is fit to estimate contributions of each cell type.
Depending on the value of \code{constrained} the returned coefficients can be either raw or enforced to attain values between 0 and 1 with within-sample sum 
less or equal to 1.
}
\note{
Requires the package \pkg{nlme}.
}
\references{
1. Houseman, Eugene and Accomando, William and Koestler, Devin and Christensen, Brock and Marsit, Carmen and Nelson, 
	Heather and Wiencke, John and Kelsey, Karl. DNA methylation arrays as surrogate measures of cell mixture distribution. 
 BMC Bioinformatics 2012, 13:86
}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\docType{methods}
\name{geno,RnBSet-method}
\alias{geno,RnBSet-method}
\alias{geno}
\title{geno-methods}
\usage{
\S4method{geno}{RnBSet}(object, type = NULL)
}
\arguments{
\item{object}{dataset of interest.}

\item{type}{\code{character} singleton. Available options are \code{NULL} (all genotyping probes), \code{'nv'} for the
genotyping probes of the arrays, or \code{'rs'} for the SNP probes.}
}
\value{
\code{matrix} with genotyping information in the form of beta values. \code{0} stands for homozygous allele 1, \code{0.5}
     for heterozygous and \code{1} for homozygous allele 2.
}
\description{
Extracts genotyping information from either the SNP (rs) or mutation (nv) probes
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differentialMethylation.R
\name{computeDiffTab.default.site}
\alias{computeDiffTab.default.site}
\alias{computeDiffTab.site}
\alias{computeDiffTab.extended.site}
\title{computeDiffTab.site}
\usage{
computeDiffTab.default.site(
  X,
  inds.g1,
  inds.g2,
  diff.method = rnb.getOption("differential.site.test.method"),
  variability.method = rnb.getOption("differential.variability.method"),
  paired = FALSE,
  adjustment.table = NULL,
  eps = 0.01,
  imputed = FALSE
)

computeDiffTab.extended.site(
  X,
  inds.g1,
  inds.g2,
  diff.method = rnb.getOption("differential.site.test.method"),
  variability.method = rnb.getOption("differential.variability.method"),
  paired = FALSE,
  adjustment.table = NULL,
  eps = 0.01,
  covg = NULL,
  covg.thres = rnb.getOption("filtering.coverage.threshold"),
  imputed = FALSE
)
}
\arguments{
\item{X}{Matrix on which the difference measures are calculated for every row}

\item{inds.g1}{column indices of group 1 members}

\item{inds.g2}{column indices of group 2 members}

\item{diff.method}{Method to determine p-values for differential methylation. Currently supported are 
    "ttest" for a two-sided Welch t-test, "refFreeEWAS" for adjusting for cell mixtures,
    and "limma" for p-values resulting from linear modeling of the transformed beta values (M-values)
and using techniques from expression microarray analysis employed in the \code{limma} package.}

\item{variability.method}{Method to determine p-values for differential variability. Currently supported are 
"diffVar" for the diffVar method implemented in the missMethyl bioconductor package,
and "iEVORA".}

\item{paired}{should a paired a analysis be performed. If \code{TRUE} then inds.g1 and inds.g2 should have exactly the same length and should be
order, such that the first element of inds.g1 corresponds to the first element of inds.g2 and so on.}

\item{adjustment.table}{a table of variables to be adjusted for in the differential methylation test. Currently this is only supported for
\code{diff.method=="limma"}}

\item{eps}{Epsilon for computing quotients (avoid division by 0 by adding this value to denominator and enumerator before calculating the quotient)}

\item{imputed}{flag indicating if methylation matrix was already imputed}

\item{covg}{coverage information (should be NULL for disabled or of equal dimensions as X)}

\item{covg.thres}{a coverage threshold}
}
\value{
a dataframe containing the following variables:
\item{mean.g1}{Mean of group 1}
\item{mean.g2}{Mean of group 2}
\item{mean.diff}{Difference in means}
\item{mean.quot.log2}{log2 of the quotient of means}
\item{diffmeth.p.val}{P-value (as determined by \code{diff.method})}
\item{max.g1/max.g2}{[extended version only] Group maxima}
\item{min.g1/min.g2}{[extended version only] Group minima}
\item{sd.g1/sd.g2}{[extended version only] Group standard deviations}
\item{min.diff}{[extended version only] Minimum of 0 and single linkage difference between the groups}
\item{diffmeth.p.adj.fdr}{[extended version only] FDR adjusted p-values}
\item{num.na.g1/num.na.g2}{[extended version only] number of NA methylation values for groups 1 and 2 respectively}
\item{mean.covg.g1/mean.covg.g2}{[extended version with coverage information only] mean coverage of groups 1 and 2 respectively}
\item{min.covg.g1/min.covg.g2}{[extended version with coverage information only] minimum coverage of groups 1 and 2 respectively}
\item{max.covg.g1/max.covg.g2}{[extended version with coverage information only] maximum coverage of groups 1 and 2 respectively}
\item{covg.thresh.nsamples.g1/2}{[extended version with coverage information only] number of samples in group 1 and 2 respectively exceeding the
									coverage threshold for this site.}
}
\description{
computes a difference table containing multiple difference measures,
In the simple version the difference in means,
quotients in means and a p-value for the comparison of two groups in a table are computed.
This is computed for each row of the input table. The extended version contains additional columns
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
meth.mat <- meth(rnb.set.example)
sample.groups <- rnb.sample.groups(rnb.set.example)[[1]]
dm <- computeDiffTab.extended.site(meth.mat,sample.groups[[1]],sample.groups[[2]])
summary(dm)
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filtering.R
\name{rnb.execute.cross.reactive.removal}
\alias{rnb.execute.cross.reactive.removal}
\title{rnb.execute.cross.reactive.removal}
\usage{
rnb.execute.cross.reactive.removal(rnb.set)
}
\arguments{
\item{rnb.set}{Methylation dataset as an object of type inheriting \code{\linkS4class{RnBeadSet}}.}
}
\value{
\code{list} of four elements:
        \describe{
          \item{\code{"dataset.before"}}{Copy of \code{rnb.set}.}
          \item{\code{"dataset"}}{The (possibly) modified dataset object after removing probes that have a high
               likelihood of cross-hybridization.}
          \item{\code{"filtered"}}{\code{integer} vector storing the indices (in beta matrix of the unfiltered
               dataset) of all removed probes.}
        }
}
\description{
Removes all probes defined as cross-reactive from the given dataset.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
rnb.set.filtered <- rnb.execute.cross.reactive.removal(rnb.set.example)$dataset
identical(meth(rnb.set.example), meth(rnb.set.filtered)) # FALSE
}
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lolaUtils.R
\name{lolaBoxPlotPerTarget}
\alias{lolaBoxPlotPerTarget}
\title{lolaBoxPlotPerTarget}
\usage{
lolaBoxPlotPerTarget(
  lolaDb,
  lolaRes,
  scoreCol = "pValueLog",
  orderCol = scoreCol,
  signifCol = "qValue",
  includedCollections = c(),
  pvalCut = 0.01,
  maxTerms = 50,
  colorpanel = c(),
  groupByCollection = TRUE,
  orderDecreasing = NULL,
  scoreDecreasing = NULL
)
}
\arguments{
\item{lolaDb}{LOLA DB object as returned by \code{LOLA::loadRegionDB} or \code{\link{loadLolaDbs}}}

\item{lolaRes}{LOLA enrichment result as returned by the \code{runLOLA} function from the \code{LOLA} package}

\item{scoreCol}{column name in \code{lolaRes} to be plotted}

\item{orderCol}{column name in \code{lolaRes} which is used for sorting the results}

\item{signifCol}{column name of the significance score in \code{lolaRes}. Should be one of \code{c("pValueLog", "qValue")}}

\item{includedCollections}{vector of collection names to be included in the plot. If empty (default), all collections are used}

\item{pvalCut}{p-value cutoff to be employed for filtering the results}

\item{maxTerms}{maximum number of items to be included in the plot}

\item{colorpanel}{colors to be used for coloring the bars according to "target" (see \code{\link{getTargetFromLolaDb}}). An empty
vector indicates that black will be used for all bars.}

\item{groupByCollection}{facet the plot by collection}

\item{orderDecreasing}{flag indicating whether the value in \code{orderCol} should be considered as decreasing (as opposed
to increasing). \code{NULL} (default) for automatic determination.}

\item{scoreDecreasing}{flag indicating whether the value in \code{scoreCol} should be considered as decreasing (as opposed
to increasing). \code{NULL} (default) for automatic determination.}
}
\value{
ggplot object containing the plot
}
\description{
plot a boxplot showing LOLA enrichment results per "target" group (see \code{\link{getTargetFromLolaDb}} for an explanation of
"target").
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
# compute differential methylation
dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
# download LOLA DB
lolaDest <- tempfile()
dir.create(lolaDest)
lolaDirs <- downloadLolaDbs(lolaDest, dbs="LOLACore")
# perform enrichment analysis
res <- performLolaEnrichment.diffMeth(rnb.set.example,dm,lolaDirs[["hg19"]])
# select the 500 most hypermethylated tiling regions in ESCs compared to iPSCs
# in the example dataset
lolaRes <- res$region[["hESC vs. hiPSC (based on Sample_Group)"]][["tiling"]]
lolaRes <- lolaRes[lolaRes$userSet=="rankCut_500_hyper",]
# plot
lolaBoxPlotPerTarget(res$lolaDb, lolaRes, scoreCol="oddsRatio", orderCol="maxRnk", pvalCut=0.05)
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBDiffMeth-class.R
\docType{methods}
\name{get.covg.thres,RnBDiffMeth-method}
\alias{get.covg.thres,RnBDiffMeth-method}
\alias{get.covg.thres}
\title{get.covg.thres-methods}
\usage{
\S4method{get.covg.thres}{RnBDiffMeth}(object)
}
\arguments{
\item{object}{RnBDiffMeth object}
}
\value{
integer coverage threshold
}
\description{
Gets the coverage threshold employed for obtaining statistics in the differential methylation tables
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
get.covg.thres(dm)
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plottingUtils.R
\name{densRanks}
\alias{densRanks}
\title{densRanks}
\usage{
densRanks(x, y = NULL, nbin = 128, bandwidth)
}
\arguments{
\item{x}{x-coordinate}

\item{y}{y-coordinate}

\item{nbin}{number of bins}

\item{bandwidth}{bandwidth}
}
\description{
Rank the points accordind to density of the region they fall in. Densities are computed
as Kernel Density estimates. The method and parameters are implemented in analogy to
\code{grDevices::densCols}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Report-methods.R
\name{rnb.add.tables}
\alias{rnb.add.tables}
\title{rnb.add.tables}
\usage{
rnb.add.tables(
  report,
  tables,
  setting.names,
  selected.table = 1L,
  indent = 2L,
  ...
)
}
\arguments{
\item{report}{Report to write the text to.}

\item{tables}{Non-empty \code{list} of tables, each one represented by a \code{\link{data.frame}} or
\code{\link{matrix}}. The names of this list are used as table identifiers; each one consists
of elements separated by underscore character (\code{_}).}

\item{setting.names}{List of table name element descriptors. Every variable elements in the table names must be
included in this list.}

\item{selected.table}{Index of the table to be initially selected in this listing.}

\item{indent}{Default indentation, in number of tabulation characters, to apply to every table.}

\item{...}{Other parameters passed to \code{\link{rnb.add.table}}.}
}
\value{
The modified report.
}
\description{
Generates HTML code for a listing of tables (of which only one is visible at any moment) in the specified report.
}
\seealso{
\code{\link{rnb.add.table}} for adding a single table to a report; \code{\linkS4class{Report}} for other
  functions adding contents to an HTML report
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterArchitecture.R
\docType{methods}
\name{initialize,ClusterArchitecture-method}
\alias{initialize,ClusterArchitecture-method}
\title{initialize.ClusterArchitecture}
\usage{
\S4method{initialize}{ClusterArchitecture}(.Object, name = "ClusterArchitecture")
}
\arguments{
\item{.Object}{New instance of \code{ClusterArchitecture}.}

\item{name}{A name or identifier}
}
\description{
Initialize an ClusterArchitecture object
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/combining.R
\name{rnb.combine.arrays}
\alias{rnb.combine.arrays}
\title{Combine array-based datasets}
\usage{
rnb.combine.arrays(dataset1, dataset2, type = "common")
}
\arguments{
\item{dataset1}{First input dataset as an object of type inheriting \code{\linkS4class{RnBeadSet}}.}

\item{dataset2}{Second input dataset as an object of type inheriting \code{\linkS4class{RnBeadSet}}.}

\item{type}{Type of the combine operation as a character singleton, one of "common", "all.x", "all.y" and "all".}
}
\value{
Combined dataset as an object of type inheriting \code{\linkS4class{RnBeadSet}}.
}
\description{
Concatenates two array-based datasets focusing on the common probes.
}
\details{
\describe{
  \item{Sample annotation tables}{This method expects that the sample annotation tables of the two datasets have
     identical structures.}
  \item{Genome assembly}{This method expects that the two datasets target the same genome assembly.}
  \item{Platform}{The platform of the combined dataset is the most recent among the platforms of the input datasets.}
  \item{Intensity values}{The combined dataset is of type \code{\linkS4class{RnBeadRawSet}} only when both input
     datasets are of this type. Otherwise, any intensity value data is ignored.}
  \item{Probes}{Only the common probes are included in the resulting dataset.}
  \item{Regions}{Regions summarized in any of the input datasets are ignored. In the resulting dataset, regions are
     summarized as specified in the analysis option \code{"region.types"}.}
  \item{Quality control data}{QC data in the input datasets is ignored. The combined dataset includes no data on QC
     probe intensities.}
  \item{Infered covariates}{Inferred covariates in the input datasets are ignored. The combined dataset includes no
     data on inferred covariates.}
  \item{Disk dumping}{The combined dataset stores big tables on disk when the analysis option
     \code{"disk.dump.big.matrices"} is enabled.}
}
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBDiffMeth-class.R
\docType{methods}
\name{get.comparison.grouplabels,RnBDiffMeth-method}
\alias{get.comparison.grouplabels,RnBDiffMeth-method}
\alias{get.comparison.grouplabels}
\title{get.comparison.grouplabels-methods}
\usage{
\S4method{get.comparison.grouplabels}{RnBDiffMeth}(object)
}
\arguments{
\item{object}{\code{\linkS4class{RnBDiffMeth}} object}
}
\value{
character matrix containing comparison group names
}
\description{
Gets all comparison grouplabels represented in the object as character matrix of dimension n.comparisons x 2
where the columns specify group names 1 and 2 respectively
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
get.comparison.grouplabels(dm)
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logger.R
\name{logger.start}
\alias{logger.start}
\alias{logger.completed}
\alias{logger.close}
\title{Log File Management}
\usage{
logger.start(txt = character(0), fname = NULL)

logger.completed()

logger.close()
}
\arguments{
\item{txt}{Description to add to the log file. The words \code{STARTED} and \code{COMPLETED} are prepended to the
message upon initialization and completion of the section, respectively.}

\item{fname}{Name of the log file and/or console. Note that at most one file name can be specified. The function
\code{logger.start} normalizes the given name, that is, it converts it to an absolute name. If this
parameter is \code{NA}, logger messages are printed to the console. If it is a two-element vector
containing one file name and \code{NA}, the logger is (re)initialized to print messages both to the
given file name and the console. A value of \code{NULL} (default) indicates the logger should continue
using the previously specified file.}
}
\value{
None (invisible \code{NULL}).
}
\description{
Functions for logger management.
}
\section{Details}{

\code{logger.start} initializes the logger and/or starts a new section. \code{logger.completed} completes the last
(innermost) open section in the log. \code{logger.close} deinitializes the logger. Note that after reinitialization
or deinitialization, the information about the current output file, as well as any open sections, is deleted.
}

\examples{
\donttest{
if (!logger.isinitialized())
  logger.start(fname = NA)
logger.start("Tests for Significance")
logger.completed()
logger.close()
}
}
\seealso{
logger.isinitialized
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parallelProcessing.R
\name{parallel.setup}
\alias{parallel.setup}
\title{parallel.setup}
\usage{
parallel.setup(...)
}
\arguments{
\item{...}{Parameters for \code{registerDoParallel} from the \pkg{doParallel} package.
This allows, for instance, for specificying the number of workers.}
}
\value{
\code{TRUE} (invisible) to indicate that parallelization is set up.
}
\description{
Sets up parallel processing. Requires the \pkg{foreach} and \pkg{doParallel} packages
}
\note{
Requires the packages \pkg{foreach} and \pkg{doParallel}.
}
\examples{
\donttest{
parallel.setup(2)
parallel.teardown()
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clustering.R
\docType{class}
\name{RnBeadClustering-class}
\alias{RnBeadClustering-class}
\alias{initialize,RnBeadClustering-method}
\title{RnBeadClustering Class}
\description{
Storage class for the results of a clustering algorithm applied on an \code{\linkS4class{RnBSet}} dataset.
}
\section{Slots}{

\describe{
  \item{dissimilarity}{Dissimilarity metric used in the form of a one-element \code{character} vector.}
  \item{dimensionality}{Dimensionality of the clustered points in the form of a one-element \code{integer} vector.}
  \item{algorithm}{Clustering algorithm (and optionally, type) as a \code{character} vector of length 1 or 2.}
  \item{result}{Resulting object after applying the clustering algorithm on a dataset.}
  \item{assignments}{Cluster assignments for the samples in the dataset as a matrix. Row names in this matrix are
       sample identifiers, and each column is dedicated to partitioning into \emph{k} clusters for a fixed \emph{k}.}
  \item{silhouettes}{\code{numeric} vector of mean silhouette values for each tested value of \emph{k}.}
}
}

\section{Methods and Functions}{

\describe{
  \item{\code{samples}}{Gets the identifiers of all samples used in the clustering.}
}
}

\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/options.R
\name{rnb.performance.profile}
\alias{rnb.performance.profile}
\title{rnb.performance.profile}
\usage{
rnb.performance.profile(data.type = "450k", profile)
}
\arguments{
\item{data.type}{Type of dataset targeted; this must be one of \code{"450k"} (default) or \code{"bs"}.}

\item{profile}{Option profile; this must be one of \code{"minimal"}, \code{"moderate"} or \code{"full"}.}
}
\value{
Invisibly, a \code{list} containing the previous values of all modified options.
}
\description{
Enables one of the pre-installed anlaysis option profiles.
}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filtering.R
\name{rnb.execute.high.dpval.masking}
\alias{rnb.execute.high.dpval.masking}
\title{rnb.execute.high.dpval.masking}
\usage{
rnb.execute.high.dpval.masking(rnb.set, dpval.threshold = 0.05)
}
\arguments{
\item{rnb.set}{Methylation dataset as an object of type inheriting \code{\linkS4class{RnBeadSet}}.}

\item{dpval.threshold}{Threshold for maximal acceptable detection p-value, given as a non-negative \code{numeric} value between 0 and 1. All
methylation measurements with detection p-value than this threshold are set to \code{NA}. If this
parameter is \code{0}, calling this method has no effect.}
}
\value{
List of three elements:
        \describe{
          \item{\code{"dataset.before"}}{Copy of \code{rnb.set}.}
          \item{\code{"dataset"}}{The (possibly) modified dataset after retaining sites on autosomes only.}
          \item{\code{"mask"}}{A logical matrix of dimension \code{meth(rnb.set,type="sites")} indicating which
				   methylation values have been masked}
        }
}
\description{
Replaces all low coverage sites by \code{NA}.
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qualityControl.R
\name{rnb.execute.quality}
\alias{rnb.execute.quality}
\title{rnb.execute.quality}
\usage{
rnb.execute.quality(
  object,
  type = "sites",
  qc.coverage.plots = rnb.getOption("qc.coverage.plots"),
  verbose = TRUE
)
}
\arguments{
\item{object}{Methylation dataset as an object of class \code{\linkS4class{RnBeadSet}},
\code{\linkS4class{RnBeadRawSet}} or \code{\linkS4class{RnBiseqSet}}.}

\item{type}{\code{character} vector of length \code{1} giving the type of genomic regions for which the
quality control information is summarized.}

\item{qc.coverage.plots}{Flag indicating if sequencing coverage information is summarized and returned. This
parameter is considered only when \code{object} is of type \code{\linkS4class{RnBiseqSet}}.}

\item{verbose}{Flag specifying whether diagnostic output should be written to the console or to the
RnBeads logger in case the latter is initialized.}
}
\value{
\code{\linkS4class{RnBeadSet}} object with imputed quality control information
}
\description{
Performs quality control calculations on the loaded DNA methylation data set.
}
\details{
Currently, summarizing coverage for \code{\linkS4class{RnBiseqSet}} object is the only available function.
}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sex.R
\name{rnb.execute.gender.prediction}
\alias{rnb.execute.gender.prediction}
\title{rnb.execute.gender.prediction}
\usage{
rnb.execute.gender.prediction(rnb.set)
}
\arguments{
\item{rnb.set}{Methylation dataset after running the sex prediction step, as an object of type
\code{\linkS4class{RnBSet}}.}
}
\value{
The possibly modified dataset. If sex could be predicted, the sample annotation table is enriched with
}
\description{
Deprecated function name, now called \code{\link{rnb.execute.sex.prediction}}.
}
\seealso{
rnb.execute.sex.prediction
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\docType{methods}
\name{hasCovg,RnBSet-method}
\alias{hasCovg,RnBSet-method}
\alias{hasCovg}
\title{hasCovg-methods}
\usage{
\S4method{hasCovg}{RnBSet}(object, type = "sites")
}
\arguments{
\item{object}{\code{RnBSet} of interest.}

\item{type}{\code{character} singleton. If \code{sites} or a region type summarized in the object}
}
\value{
\code{TRUE} if the \code{RnBSet} object contains coverage information for sites or the specified region type. \code{FALSE} otherwise
}
\description{
Returns \code{TRUE} if the \code{RnBSet} object contains coverage information for sites or the specified region type.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
## per-site beta-value matrix
hasCovg(rnb.set.example)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R, R/RnBeadSet-class.R,
%   R/RnBeadRawSet-class.R
\docType{methods}
\name{remove.sites,RnBSet-method}
\alias{remove.sites,RnBSet-method}
\alias{remove.sites}
\alias{remove.sites,RnBeadSet-method}
\alias{remove.sites,RnBeadRawSet-method}
\title{remove.sites-methods}
\usage{
\S4method{remove.sites}{RnBSet}(object, probelist, verbose = FALSE)

\S4method{remove.sites}{RnBeadSet}(object, probelist, verbose = TRUE)

\S4method{remove.sites}{RnBeadRawSet}(object, probelist, verbose = TRUE)
}
\arguments{
\item{object}{Dataset of interest.}

\item{probelist}{List of probes to be removed in the form of a \code{logical}, \code{integer} or \code{character}
vector. If this parameter is \code{logical}, it is not recycled; its length must be equal to the
number of probes in \code{object}. If it is \code{integer} or \code{character}, it must list only
probes that exist in the dataset. Specifying probe indices larger than the number of probes, or
non-existent probe identifiers results in an error.}

\item{verbose}{if \code{TRUE} additional diagnostic output is generated}
}
\value{
The modified dataset.
}
\description{
Removes the specified probes from the dataset.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
print(rnb.set.example)
## remove 100 random sites
s2r<-sample.int(nrow(sites(rnb.set.example)), 100)
rnb.set.f<-remove.sites(rnb.set.example, s2r)
print(rnb.set.f)
}
}
\seealso{
\code{\link[=remove.samples,RnBSet-method]{remove.samples}} for removing samples from a methylation dataset
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parallelProcessing.R
\name{parallel.teardown}
\alias{parallel.teardown}
\title{parallel.teardown}
\usage{
parallel.teardown()
}
\value{
\code{TRUE}, invisibly.
}
\description{
Disables parallel processing.
}
\examples{
\donttest{
parallel.getNumWorkers()
parallel.setup(2)
parallel.getNumWorkers()
parallel.teardown()
parallel.getNumWorkers()
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logger.R
\name{logger.isinitialized}
\alias{logger.isinitialized}
\title{logger.isinitialized}
\usage{
logger.isinitialized()
}
\value{
\code{TRUE} if the logger was initialized and is in use; \code{FALSE} otherwise.
}
\description{
Checks if the logger is initialized.
}
\examples{
\donttest{
if (!logger.isinitialized())
  logger.start(fname = NA)
}
}
\seealso{
\code{\link{logger.start}} for initializing a logger or starting a section
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differentialMethylation.R
\name{computeDiffTab.default.region}
\alias{computeDiffTab.default.region}
\alias{computeDiffTab.region}
\title{computeDiffTab.region}
\usage{
computeDiffTab.default.region(dmtp, regions2sites, includeCovg = FALSE)
}
\arguments{
\item{dmtp}{differential methylation table on the site level (as obtained from \code{\link{computeDiffTab.default.site}})}

\item{regions2sites}{a list containing for each region the indices of the corresponding sites in the site differential methylation table}

\item{includeCovg}{flag indicating whether to include coverage information}
}
\value{
a dataframe containing the following variables for a given genomic region:
\item{mean.mean.g1,mean.mean.g2}{mean of mean methylation levels for group 1 and 2 across all sites in a region}
\item{mean.mean.diff}{Mean difference in means across all sites in a region}
\item{mean.mean.quot.log2}{Mean quotient in means across all sites in a region}
\item{comb.p.val}{Combined p-value using a generalization of Fisher's method. See \code{\link{combineTestPvalsMeth}} for details.}
\item{comb.p.adj.fdr}{FDR adjusted combined p-value}
\item{num.sites}{number of sites that were considered for a region}
\item{mean.num.na.g1/2}{mean number (accross all considered sites) of samples that contained an NA for group 1 and 2 respectively}
\item{mean.mean.covg.g1/2}{Mean value of mean coverage values (across all samples in a group) across all sites in a region}
\item{mean.nsamples.covg.thresh.g1/2}{mean number (accross all considered sites) of samples that have a coverage larger than the specified threshold
		(see \code{\link{computeDiffTab.default.site}} for details) for group 1 and 2 respectively}
}
\description{
computes a difference table containing multiple difference measures,
In the simple version the mean of the difference in means,
the mean quotient in means and a combination of p-values on the site level are computed.
This is computed for each row of the input table. The extended version contains additional columns
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
meth.mat <- meth(rnb.set.example)
sample.groups <- rnb.sample.groups(rnb.set.example)[[1]]
dm.sites <- computeDiffTab.extended.site(meth.mat,sample.groups[[1]],sample.groups[[2]])
map.regions.to.sites <- regionMapping(rnb.set.example,"promoters")
dm.promoters <- computeDiffTab.default.region(dm.sites,map.regions.to.sites)
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\docType{methods}
\name{updateRegionSummaries,RnBSet-method}
\alias{updateRegionSummaries,RnBSet-method}
\alias{updateRegionSummaries}
\title{updateRegionSummaries}
\usage{
\S4method{updateRegionSummaries}{RnBSet}(object)
}
\arguments{
\item{object}{Dataset of interest.}
}
\value{
Sample annotation information available for the dataset in the form of a \code{data.frame}.
}
\description{
Updates the region information present in an RnBSet by invoking summarize.regions on all region types
present in the object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cellTypeAdjustment.R
\name{set.covariates.ct}
\alias{set.covariates.ct}
\title{set.covariates.ct}
\usage{
set.covariates.ct(rnb.set, ct.obj)
}
\arguments{
\item{rnb.set}{The \code{RnBSet} object to which the results should be added}

\item{ct.obj}{An object of class \code{CellTypeInferenceResult} returned by \code{rnb.execute.ct.estimation}.}
}
\value{
The modified \code{RnBSet}.
}
\description{
Adds the results of cell type estimation to an RnBSet
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differentialMethylation.R
\name{rowPairedTP}
\alias{rowPairedTP}
\title{rowPairedTP}
\usage{
rowPairedTP(X, inds.g1, inds.g2 = -inds.g1, alternative = "two.sided")
}
\arguments{
\item{X}{Matrix on which the test is performed for every row}

\item{inds.g1}{column indices of group 1 members. \code{length(inds.g1)==length(inds.g2)} has to hold true.}

\item{inds.g2}{column indices of group 2 members. \code{length(inds.g1)==length(inds.g2)} has to hold true.}

\item{alternative}{Testing alternative. Must be one of "two.sided" (default),"less","greater" or "all".
in case of "all" a data frome with corresping alternative variables is returned. 
Otherwise the result is a vector.}
}
\value{
vector (or data.frame if alternative=="all") of p-values from a paired t-test
}
\description{
performs a two-sided t-test for paired samples on each row of a matrix X with the indices inds.1 vs indices inds.g2 as group assignments.
}
\note{
Requires \code{matrixStats} package
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotations.R
\name{rnb.load.annotation}
\alias{rnb.load.annotation}
\title{rnb.load.annotation}
\usage{
rnb.load.annotation(fname, type)
}
\arguments{
\item{fname}{One-element \code{character} vector giving the name of the file that contains the annotation data.}

\item{type}{One-element \code{character} vector giving the name of the region annotation. If this annotation
is already available, it will be overwritten for the current session.}
}
\value{
Invisibly, \code{TRUE} if the annotation was loaded successfully; an error message if the objects in the
        given file do not encode an annotation.
}
\description{
Loads a previously saved custom region annotation from a binary (RData) file.
}
\details{
If the region annotation cannot be loaded from the specified location, this function exits with an error message in
the form \code{"unable to load object from ..."}. This could happen, for example, when \code{fname} does not refer to
a valid RData file, or the file cannot be accessed due to security restrictions.

If the file is loaded in the current session, but no annotation was added, the function returns invisibly one of the
following short failure messages:
\describe{
  \item{\code{"invalid format"}}{The RData file does not store exactly the following three objects - \code{assembly},
       \code{regions}, and \code{mapping}, or they are not of the expected type.}
  \item{\code{"unsupported assembly"}}{The specified assembly is unknown.}
  \item{\code{"invalid format of regions"}}{The specified region annotation table is invalid.}
  \item{\code{"invalid format of mappings"}}{The specified region mapping tables are invalid.}
}
}
\seealso{
\code{\link{rnb.save.annotation}} for saving annotation to a binary file; \code{\link{rnb.set.annotation}}
         for loading an annotation from a BED file.
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regionProfiles.R
\name{rnb.plot.region.site.density}
\alias{rnb.plot.region.site.density}
\title{rnb.plot.region.site.density}
\usage{
rnb.plot.region.site.density(rnb.set, region.type, extend.by = 0.33)
}
\arguments{
\item{rnb.set}{RnBSet object}

\item{region.type}{Region type for which the plot should be generated}

\item{extend.by}{A number between 0 and 1 specifying the percentage by which a region is
extended in order to capture methylation information before region start and after region end}
}
\value{
a ggplot2 object for plotting
        the plot shows the density of sites accross the specified region type for all regions of that
        type from 0 (region start) to 1 (region end). Sites in the flanking areas are
        also shown (coordinates <0 and >1).
}
\description{
Plots the density of sites accross the specified region type
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
rnb.plot.region.site.density(rnb.set.example,"genes")
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cellTypeAdjustment.R
\name{refFreeEWASP}
\alias{refFreeEWASP}
\title{refFreeEWASP}
\usage{
refFreeEWASP(
  X,
  inds.g1,
  inds.g2 = -inds.g1,
  adjustment.table = NULL,
  paired = FALSE,
  nboot = 100,
  ignore.na = TRUE,
  rescale.residual = TRUE
)
}
\arguments{
\item{X}{Matrix on which the test is performed for every row}

\item{inds.g1}{column indices of group 1 members}

\item{inds.g2}{column indices of group 2 members}

\item{adjustment.table}{a \code{data.frame} containing variables to adjust for in the testing}

\item{paired}{should a paired analysis model be used. If so, the first index in \code{inds.g1} must correspond to the first
index in  \code{inds.g2} and so on.}

\item{nboot}{The number of bootstrapping resamples}

\item{ignore.na}{in this case all \code{NA} containing rows are removed}

\item{rescale.residual}{rescale the residual matrix as z-scores}
}
\value{
vector of p-values for the "adjusted" regression coefficients from the Reference-free EWAS model
}
\description{
NOTE: This function is deprecated, since the RefFreeEWAS package is not supported and available anymore
Applies the reference-free cell-type heterogeneity adjustment model from [1] and 
returns corrected p-values
}
\note{
Requires the package \pkg{RefFreeEWAS}.
}
\references{
1. Houseman, E. Andres, John Molitor, and Carmen J. Marsit. "Reference-Free Cell Mixture Adjustments in Analysis of DNA Methylation Data." 
Bioinformatics (2014): btu029.
}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\docType{methods}
\name{pheno,RnBSet-method}
\alias{pheno,RnBSet-method}
\alias{pheno}
\title{pheno-methods}
\usage{
\S4method{pheno}{RnBSet}(object)
}
\arguments{
\item{object}{Dataset of interest.}
}
\value{
Sample annotation information available for the dataset in the form of a \code{data.frame}.
}
\description{
Extracts sample phenotype and/or processing information.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
pheno(rnb.set.example)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Report-methods.R
\name{rnb.add.figure}
\alias{rnb.add.figure}
\title{rnb.add.figure}
\usage{
rnb.add.figure(
  report,
  description,
  report.plots,
  setting.names = list(),
  selected.image = as.integer(1)
)
}
\arguments{
\item{report}{Report to write the text to.}

\item{description}{Human-readable description of the figure. This must be a non-empty \code{character} vector. The
elements of this vector are concatenated without a separator to form the full description.}

\item{report.plots}{Object of type \code{\linkS4class{ReportPlot}}, or a list of such objects.}

\item{setting.names}{List of plot file element descriptors. Every variable elements in the plot file names must be
included in this list. Set this to empty list if no variable elements are present, that is, if
the figure should present a single report plot.}

\item{selected.image}{Index of plot to be initially selected in the figure.}
}
\value{
The modified report.
}
\description{
Generates HTML code for a figure in the specified report. A figure is a collection of images (plots), of which only
one is visible at any given moment.
}
\seealso{
\code{\link{rnb.add.tables}} for adding a listing of tables; \code{\linkS4class{Report}} for other functions
  adding contents to an HTML report
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotations.R
\name{rnb.annotation2data.frame}
\alias{rnb.annotation2data.frame}
\title{rnb.annotation2data.frame}
\usage{
rnb.annotation2data.frame(annotation.table, add.names = TRUE)
}
\arguments{
\item{annotation.table}{Annotation in the form of non-empty \code{GRangesList} object, as returned by
\code{\link{rnb.get.annotation}}.}

\item{add.names}{Flag indicating if element names should be extracted and returned also as a column named
\code{"ID"} in the resulting \code{data.frame}. Note that element names, if present, are
set to be the row names of the table.}
}
\value{
Annotation in the form of a single \code{data.frame}. The columns in this table include, among other,
        \code{"Chromosome"}, \code{"Start"} and \code{"End"}.
}
\description{
Transform the specified site, probe or region annotation to \code{data.frame}.
}
\examples{
\donttest{
library(RnBeads.hg19)
head(rnb.annotation2data.frame(rnb.get.annotation("probes450")))
}
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/agePrediction.R
\name{run.cross.validation}
\alias{run.cross.validation}
\title{run.cross.validation}
\usage{
run.cross.validation(rnbSet, report, alpha = 0.8)
}
\arguments{
\item{rnbSet}{a \code{RnBSet} object containing the methylation info and ages on which
the new predictor should be trained}

\item{report}{report to which the table should be added}

\item{alpha}{alpha parameter used in the elastic net regression}
}
\value{
modified report object
}
\description{
This function performs 10-fold cross validation to estimate the performance of a
newly trained predictor. If \code{parallel.isEnabled()}, the function perfoms cross
validation in parallel. The function adds a table to the specified \code{report} containing
the result of the 10-fold cross validation.
}
\author{
Michael Scherer
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterArchitectureLSF.R
\docType{methods}
\name{initialize,ClusterArchitectureLSF-method}
\alias{initialize,ClusterArchitectureLSF-method}
\title{initialize.ClusterArchitectureLSF}
\usage{
\S4method{initialize}{ClusterArchitectureLSF}(.Object, name = "ClusterArchitectureLSF", ...)
}
\arguments{
\item{.Object}{New instance of \code{ClusterArchitectureLSF}.}

\item{name}{A name or identifier}

\item{...}{arguments passed on to the constructor of \code{\linkS4class{ClusterArchitecture}} (the parent class)}
}
\description{
Initialize an ClusterArchitecture object for a LSF
}
\author{
Michael Scherer
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBeadRawSet-class.R
\name{as.RnBeadRawSet}
\alias{as.RnBeadRawSet}
\title{Conversion to/from RnBeadRawSet}
\description{
The \code{"as"} method can be used for the following conversions:
\itemize{
  \item{}{\code{MethyLumiSet} (in package \pkg{methylumi}) to \code{\linkS4class{RnBeadRawSet}}}
  \item{}{\code{RnBeadRawSet} to \code{MethyLumiSet}}
  \item{}{\code{RGChannelSet} (in package \pkg{minfi}) to \code{\linkS4class{RnBeadRawSet}}}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataImport.R
\name{read.sample.annotation}
\alias{read.sample.annotation}
\title{read.sample.annotation}
\usage{
read.sample.annotation(fname, sep = rnb.getOption("import.table.separator"))
}
\arguments{
\item{fname}{Name of text file that contains a sample annotation table with a header. This method handles a variety
of file formats, including comma-separated values file exported from Genome Studio.}

\item{sep}{One-element \code{character} used as field separator in the tables file.}
}
\value{
Sample annotation table in the form of a \code{data.frame}, in which every row corresponds to a sample, and
        every column - to a trait.
}
\description{
Reads Illumina Infinium sample annotation.
}
\examples{
\donttest{

  annotation.file<-system.file("")
  sa<-read.sample.annotation(annotation.file)
  sa
}
}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataImport.R
\name{read.idat.files2}
\alias{read.idat.files2}
\title{read.idat.files2}
\usage{
read.idat.files2(
  base.dir,
  barcodes = NULL,
  sample.sheet = NULL,
  sep.samples = rnb.getOption("import.table.separator"),
  load.chunk = NULL,
  keep.methylumi = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{base.dir}{Directory that contains the \code{.idat} files to be read; or a character vector of such
directories.}

\item{barcodes}{Optional non-empty \code{character} vector listing the barcodes of the samples that should be
loaded. If supplied, this vector must not contain \code{NA} among its elements.}

\item{sample.sheet}{Optional file name containing a table of sample annotation data, or the table itself in the
form of a \code{\link{data.frame}} or \code{matrix}. Only (and all) samples defined in this
table will be loaded. The table is expected to contain a column named \code{"barcode"} that
lists the samples' Sentrix  barcodes. If such a column is not present, this function searches
for columns \code{"Sentrix_ID"} and \code{"Sentrix_Position"} (or similar) that build a
barcode.}

\item{sep.samples}{character used as field separator in the sample sheet file.
Default value is taken by the call to \code{rnb.getOption("import.table.separator")}}

\item{load.chunk}{\code{integer} of size one, giving the number of IDAT files which should be loaded in
one loading cycle or \code{NULL}, in which case an attempt will be made to load all files
in one go. Should be assigned in case the number of IDATs is more than one thousand.}

\item{keep.methylumi}{a flag indicating whether the a \code{MethyLumiSet}
object should be returned instead of a \code{RnBeadRawSet}.}

\item{verbose}{Flag indicating ifthe messages to the logger should be sent. Note that the logger
must be initialized prior to calling this function. Logging is useful for keeping a
record of the downloaded and processed samples. Also, informative messages are
stored in case of an error.}
}
\value{
Loaded dataset of HumanMethylation450K samples, encapsulated in an object of type \code{MethyLumiSet}.
}
\description{
Reads a directory of \code{.idat} files and initializes an object of type \code{\linkS4class{MethyLumiSet}}.
}
\details{
If neither \code{barcodes}, nor \code{sample.sheet} are specified, the function attempts to locate a file in
\code{base.dir} containing sample annotation information. It fails ifsuch a file cannot be (unambiguously)
identified. If both \code{barcodes} and \code{sample.sheet} are supplied, only \code{sample.sheet} is used in loading
methylation data. The value of \code{barcodes} is tested for validity but it is not used as a filter.
}
\seealso{
\code{\link{methylumIDAT}} in package \pkg{methylumi}
}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genomewide.R
\name{rnb.execute.genomewide}
\alias{rnb.execute.genomewide}
\title{Genome-wide methylation level}
\usage{
rnb.execute.genomewide(dataset)
}
\arguments{
\item{dataset}{Methylation dataset to study, provided as an object of type inheriting \code{RnBSet}.}
}
\value{
\code{vector} of values in the range \code{[0, 1]}, storing the average beta values per sample.
}
\description{
Computes genome-wide methylation levels per sample.
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differentialVariability.R
\name{apply.iEVORA}
\alias{apply.iEVORA}
\title{apply.iEVORA}
\usage{
apply.iEVORA(meth.matrix, inds.g1, inds.g2)
}
\arguments{
\item{meth.matrix}{Matrix containing the methylation information used to calculate differentially variable sites 
between the two groups}

\item{inds.g1}{Indices in the phenotypic table corresponding to the first group.}

\item{inds.g2}{Indices in the phenotypic table corresponding to the second group.}
}
\value{
Q-values as the result of applying the iEVORA method and then correct for multiple testing.
}
\description{
This routine applies the iEVORA method created by Teschendorff et.al. to the supplied methylation matrix in a similar way
as the diffVar method.
}
\author{
Michael Scherer
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterArchitectureSLURM.R
\docType{methods}
\name{initialize,ClusterArchitectureSLURM-method}
\alias{initialize,ClusterArchitectureSLURM-method}
\title{initialize.ClusterArchitectureSLURM}
\usage{
\S4method{initialize}{ClusterArchitectureSLURM}(.Object, name = "ClusterArchitectureSLURM", ...)
}
\arguments{
\item{.Object}{New instance of \code{ClusterArchitectureSLURM}.}

\item{name}{A name or identifier}

\item{...}{arguments passed on to the constructor of \code{\linkS4class{ClusterArchitecture}} (the parent class)}
}
\description{
Initialize an ClusterArchitecture object for a SLURM
}
\author{
Michael Scherer
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differentialMethylation.R
\name{combineTestPvalsMeth}
\alias{combineTestPvalsMeth}
\title{combineTestPvalsMeth}
\usage{
combineTestPvalsMeth(
  pvalues,
  testWeights = NULL,
  correlated = FALSE,
  methExpectedTestCorrelation = 0.8
)
}
\arguments{
\item{pvalues}{p-values to combine}

\item{testWeights}{weights for the individual tests}

\item{correlated}{are the individual tests correlated}

\item{methExpectedTestCorrelation}{expected correlation. Empirically approximated to the default value of 0.8 for DNA-methylation}
}
\value{
the combined p-value
}
\description{
combine p-values of multiple tests using (a generalization of) Fisher's method. The parameter setting here is taylored to DNA methylation, but can be adapted. 
Reference: Makambi, K. (2003). Weighted inverse chi-square method for correlated significance tests. Journal of Applied Statistics, 30(2), 225-234.
}
\examples{
\donttest{
p.vals <- 10^-c(0,1,5)
combineTestPvalsMeth(p.vals)
}
}
\author{
Fabian Mueller, Christoph Bock
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/greedycut.R
\name{greedycut.filter.matrix}
\alias{greedycut.filter.matrix}
\title{greedycut.filter.matrix}
\usage{
greedycut.filter.matrix(mm, rows2ignore = integer(), rc.ties = "row")
}
\arguments{
\item{mm}{Numeric matrix to filter.}

\item{rows2ignore}{\code{integer} vector containing indices of rows in \code{mm} to be ignored by this  function.}

\item{rc.ties}{Flag indicating what the behaviour of the algorithm should be in case of ties between values of
rows and columns. The value of this parameter must be one of \code{"row"}, \code{"column"} or
\code{"any"} (the last one indicating random choice).}
}
\value{
Table summarizing the iterations of the algorithm in the form of a \code{data.frame} with the following
        columns : Index, Type, Score, Normalized score, Rows, Columns.
}
\description{
Performs all iterations of the Greedycut algorithm for removing rows and columns from the given matrix.
}
\seealso{
\code{\link{greedycut.get.submatrix}} for extracting the resulting matrix after filtering
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataImport.R
\name{read.single.bed}
\alias{read.single.bed}
\title{read.single.bed}
\usage{
read.single.bed(
  file,
  chr.col = 1L,
  start.col = 2L,
  end.col = 3L,
  strand.col = 6L,
  mean.meth.col = 7L,
  coverage.col = 8L,
  c.col = NA,
  t.col = NA,
  is.epp.style = FALSE,
  coord.shift = 0L,
  ffread = FALSE,
  context = "cg",
  ...
)
}
\arguments{
\item{file}{the input BED file}

\item{chr.col}{chromosome column index}

\item{start.col}{start column index}

\item{end.col}{end column index}

\item{strand.col}{strand column index}

\item{mean.meth.col}{mean methylation column index}

\item{coverage.col}{column with coverage information}

\item{c.col}{converted C counts column index}

\item{t.col}{unconverted C counts column index}

\item{is.epp.style}{Flag for custom Broad Epigenome Pipeline (EPP) bed style (columns \code{"chrom"}, \code{"start"},
\code{"end"}, \code{"methylated_count/total_count"}, \code{"meth_score_scaled_0_1000"} and
\code{"strand"} in this order). Setting this to \code{TRUE} overwrites all other parameters except
\code{file}, and also neglects \code{...}.}

\item{coord.shift}{An integer specifying the coordinate adjustment applied to the start and end coordinates.}

\item{ffread}{Use \code{ff} package functionality}

\item{context}{prefix for the output rownames}

\item{...}{further arguments to \code{read.table} or \code{read.table.ffdf}}
}
\value{
a \code{data.frame} or \code{ff.data.frame} object with DNA methylation and coverage information. The row names are formed by the following convension:
 		\code{context\\.read.delim(file,...)[,chr.col]\\.read.delim(file,...)[,start.col]\\.read.delim(file,...)[,strand.col]}.
}
\description{
reads a BED file with methylation information
}
\details{
Missing columns should be assigned with \code{NA}. In case \code{mean.meth.col} is absent at least \code{coverage.col}
			and one of \code{c.col} or \code{t.col} should be specified.
}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotations.R
\name{rnb.export.annotation}
\alias{rnb.export.annotation}
\title{rnb.export.annotation}
\usage{
rnb.export.annotation(fname, type, assembly = "hg19", format = "bed")
}
\arguments{
\item{fname}{One-element \code{character} vector giving the name of the file to contain the annotation data. If
this file already exists, it will be overwritten.}

\item{type}{One-element \code{character} vector giving the name of the region annotation.}

\item{assembly}{Genome assembly of interest. See \code{\link{rnb.get.assemblies}} for the list of supported genomes.}

\item{format}{Output format. currently only \code{"bed"} is supported.}
}
\value{
\code{TRUE}, invisibly.
}
\description{
Export the annotation to a defined format (currently only bed is supported
}
\examples{
\donttest{
rnb.export.annotation(tempfile(pattern="promoters",fileext=".bed"),"promoters")
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cellTypeAdjustment.R
\name{rnb.plot.marker.fstat}
\alias{rnb.plot.marker.fstat}
\title{rnb.plot.marker.fstat}
\usage{
rnb.plot.marker.fstat(ct.object, writeToFile = FALSE, ...)
}
\arguments{
\item{ct.object}{Object of class \code{CellTypeInferenceResult} as returned by \link{rnb.execute.ct.estimation}.}

\item{writeToFile}{If \code{TRUE}, the plot will be written to a file.}

\item{...}{Other arguments to \code{\link{createReportPlot}}.}
}
\value{
if \code{writeToFile=TRUE} an object of class \code{\linkS4class{ReportPlot}}, 
						and the plotted reordered F-statistics vector otherwise
}
\description{
Plot the the cell type marker selection based on the reference methylome data
}
\details{
The F-statistic values from the cell type association model (first part of eqn. (1) in [1]) are plotted in decreasing order 
			for all tested CpG positions. A vertical line gives a cut-off for the number of selected cell type markers.
}
\references{
1. Houseman, Eugene and Accomando, William and Koestler, Devin and Christensen, Brock and Marsit, Carmen and Nelson, 
	Heather and Wiencke, John and Kelsey, Karl. DNA methylation arrays as surrogate measures of cell mixture distribution. BMC Bioinformatics 2012, 13:86
}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filtering.R
\name{rnb.execute.variability.removal}
\alias{rnb.execute.variability.removal}
\title{rnb.execute.variability.removal}
\usage{
rnb.execute.variability.removal(
  rnb.set,
  min.deviation = rnb.getOption("filtering.deviation.threshold")
)
}
\arguments{
\item{rnb.set}{Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.}

\item{min.deviation}{Threshold for standard deviation per site. This must be a scalar between 0 and 1. All sites, for
which the standard deviation of methylation values (for all samples in \code{rnb.set}) is lower
than this threshold, will be filtered out.}
}
\value{
List of four elements:
        \describe{
          \item{\code{"dataset.before"}}{Copy of \code{rnb.set}.}
          \item{\code{"dataset"}}{The (possibly modified) dataset after removing sites with low variability.}
          \item{\code{"filtered"}}{\code{integer} vector storing the indices (in beta matrix of the unfiltered
               dataset) of all removed sites.}
			 \item{\code{"threshold"}}{The value of the given parameter \code{min.deviation}.}
        }
}
\description{
Removes all sites or probes with low variability from the given dataset.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
rnb.set.filtered <- rnb.execute.variability.removal(rnb.set.example, 0.01)
}
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Report-methods.R
\name{rnb.add.section}
\alias{rnb.add.section}
\title{rnb.add.section}
\usage{
rnb.add.section(report, title, description, level = 1L, collapsed = FALSE)
}
\arguments{
\item{report}{Report to write the text to.}

\item{title}{Section header. This must be a single-element \code{character} vector.}

\item{description}{Human-readable paragraph text of the section in the form of a \code{character} vector. Elements
of this vector are concatenated without a separator to form the full description. Set this to
\code{NULL} if the section does not (yet) contain text.}

\item{level}{Section level as a single \code{integer}. It must be one of \code{1}, \code{2} or \code{3},
denoting section, subsection and sub-subsection, respectively.}

\item{collapsed}{Flag indicating if the contents of this section is to be initially collapsed. Possible values are
\code{TRUE} (the section is not visible), \code{FALSE} (default, the section is expanded) and
\code{"never"} (the section cannot be collapsed or expanded).}
}
\value{
The modified report.
}
\description{
Generates HTML code for a new section in the specified report.
}
\examples{
\donttest{
report <- createReport("example.html", "Example", init.configuration = TRUE)
report <- rnb.add.section(report, "Introduction", "This is how it's done.")
}
}
\seealso{
\code{\linkS4class{Report}} for other functions adding contents to an HTML report
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Report-methods.R
\name{rnb.get.reference}
\alias{rnb.get.reference}
\title{rnb.get.reference}
\usage{
rnb.get.reference(report, txt)
}
\arguments{
\item{report}{Report that contains the reference to be cited.}

\item{txt}{Text of the reference in the form of a non-empty \code{character} vector. This reference must already
added to the report.}
}
\value{
Citation of the reference item (including a link) in the form of a one-element \code{character} vector. If
        the specified reference item is not found in the report, this method returns an empty string.
}
\description{
Creates a string that points to the given reference item in the specified report.
}
\examples{
\donttest{
report <- createReport("example.html", "Example", init.configuration = TRUE)
txt.reference <- c("Bird A. ", "<i>Nucleic Acids Res.</i> <b>8</b> (1980)")
report <- rnb.add.reference(report, txt.reference)
txt <- c("This was shown in ", rnb.get.reference(report, txt.reference), ".")
rnb.add.paragraph(report, txt)
}
}
\seealso{
\code{\link{rnb.add.reference}} for adding a reference item to a report; \code{\linkS4class{Report}} for
  other functions adding contents to an HTML report
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{rnb.region.types.for.analysis}
\alias{rnb.region.types.for.analysis}
\title{rnb.region.types.for.analysis}
\usage{
rnb.region.types.for.analysis(rnb.set)
}
\arguments{
\item{rnb.set}{Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.}
}
\value{
List of all region types to be analyzed in the current dataset in the form of a \code{character} vector.
}
\description{
Identifies the region types that are summarized by the given dataset and pointed to for analysis.
}
\details{
This function intersects the value of the analysis option \code{"region.types"} with the region types that are
summarized in the provided dataset. In case the option's value is \code{NULL}, this function returns all summarized
region types in \code{rnb.set}.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
"promoters" \%in\% rnb.region.types.for.analysis(rnb.set.example)
}
}
\seealso{
\code{\link{rnb.getOption}} for checking the value of the \code{"region.types"} option;
  \code{\link[=summarized.regions,RnBSet-method]{summarized.regions}} for obtaining the region types summarized in a
  dataset
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cellTypeAdjustment.R
\name{has.covariates.ct}
\alias{has.covariates.ct}
\title{has.covariates.ct}
\usage{
has.covariates.ct(rnb.set)
}
\arguments{
\item{rnb.set}{\code{RnBSet} object}
}
\value{
\code{TRUE} if the supplied object contains the cell type covariates information and \code{FALSE} otherwise
}
\description{
Checks whether the given \code{RnBSet} object contains cell type contribution estimates
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.R
\name{rnb.run.dj}
\alias{rnb.run.dj}
\title{rnb.run.dj}
\usage{
rnb.run.dj()
}
\value{
Nothing of particular interest
}
\description{
Starts the RnBeads Data Juggler (RnBeadsDJ) for configuring and running RnBeads analyses from the web browser
}
\details{
A Shiny app is launched in the web browser
}
\seealso{
\code{\link{rnb.run.analysis}} for starting an analysis pipeline
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBeadRawSet-class.R
\docType{methods}
\name{U,RnBeadRawSet-method}
\alias{U,RnBeadRawSet-method}
\alias{U}
\title{U-methods}
\usage{
\S4method{U}{RnBeadRawSet}(object, row.names = FALSE)
}
\arguments{
\item{object}{Dataset of interest.}

\item{row.names}{Flag indicating whether the resulting matrix will be assigned row names}
}
\value{
\code{matrix} of the unmethylated probe intensities
}
\description{
Extract raw unmethylated probe intensity from an object of \code{RnBeadRawSet} class.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
U.intensity<-U(rnb.set.example)
head(U.intensity)
} 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\docType{methods}
\name{sites,RnBSet-method}
\alias{sites,RnBSet-method}
\alias{sites}
\title{sites-methods}
\usage{
\S4method{sites}{RnBSet}(object)
}
\arguments{
\item{object}{Dataset of interest.}
}
\value{
A matrix of type \code{integer} describing the sites, information for which is
present in the \code{object}
}
\description{
Methylation sites object information for which is present in the \code{RnBSet} object.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
sites(rnb.set.example)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filtering.R
\name{rnb.execute.context.removal}
\alias{rnb.execute.context.removal}
\title{rnb.execute.context.removal}
\usage{
rnb.execute.context.removal(
  rnb.set,
  contexts = rnb.getOption("filtering.context.removal")
)
}
\arguments{
\item{rnb.set}{Methylation dataset as an object of type \code{\linkS4class{RnBeadSet}}.}

\item{contexts}{Probe contexts to be filtered out.}
}
\value{
List of three or four elements:
        \describe{
          \item{\code{"dataset.before"}}{Copy of \code{rnb.set}.}
          \item{\code{"dataset"}}{The (possibly modified) \code{RnBeadSet} object after performing the missing
               value removal.}
          \item{\code{"filtered"}}{\code{integer} vector storing the indices of all removed probes in
               \code{dataset.before}.}
          \item{\code{"contexts"}}{The value of the parameter \code{contexts}.}
        }
}
\description{
Removes all probes that belong to specific context from the given dataset.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
contexts.to.ignore <- c("CC", "CAG", "CAH")
rnb.set.filtered <- rnb.execute.context.removal(rnb.set.example, contexts.to.ignore)$dataset
identical(rnb.set.example, rnb.set.filtered) # FALSE
}
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataImport.R
\name{read.GS.report}
\alias{read.GS.report}
\title{read.GS.report}
\usage{
read.GS.report(
  gsReportFile,
  pd = NULL,
  sep = rnb.getOption("import.table.separator"),
  keep.methylumi = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{gsReportFile}{location of the GS report file}

\item{pd}{alternative sample annotation, if the \code{gsReporFile} is missing the sample section as
\code{data.frame} of \code{character} singleton with the file name}

\item{sep}{character used as field separator in the sample sheet file and in the GS report file
 (should be identical).
Default value is taken by the call to \code{rnb.getOption("import.table.separator")}}

\item{keep.methylumi}{a flag indicating whether the a \code{MethyLumiSet}
object should be returned instead of a \code{RnBeadRawSet}.}

\item{verbose}{Flag indicating ifthe messages to the logger should be sent. Note that the logger must be initialized prior to calling this function.
Logging is useful for keeping a record of the downloaded and processed samples. Also, informative messages are stored in case of an error.}
}
\value{
MethylumiSet object with the data from the report
}
\description{
Reads in a Genome Studio report, exported as a single file.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/options.R
\name{rnb.options}
\alias{rnb.options}
\alias{rnb.getOption}
\title{RnBeads Options}
\usage{
rnb.options(...)

rnb.getOption(x)
}
\arguments{
\item{...}{Option names as \code{character}s, or new option values given in the form \code{name = value}.}

\item{x}{Option name in the form of a \code{character} vector of length 1.}
}
\value{
For \code{rnb.getOption}, the current value for \code{x}. For \code{rnb.options()}, a list of all
        \pkg{RnBeads} options and their current values. If option names are given, a list of all requested options
        and their values. If option values are set, \code{rnb.options} returns the previous values of the modified
        options, invisibly.
}
\description{
Allows the user to set and examine a variety of \pkg{RnBeads} global options. They affect the way in which the
package computes and displays its results.
}
\details{
Invoking \code{rnb.options()} with no arguments returns a list with the current values of the options. To access the
value of a single option, one should use, e.g., \code{rnb.getOption("filtering.greedycut")}, rather than
\code{rnb.options("filtering.greedycut")} which is a \emph{list} of length one. Also, only a limited set of options
is available (see below). Attempting to get or set the value of a non-existing option results in an error.
}
\section{Options used in RnBeads}{

\describe{
  \item{\bold{\code{analysis.name}}\code{ = NULL}}{
       One-element \code{character} vector storing a short title of the analysis. If specified, this name appears at
       the page title of every report.}
  \item{\bold{\code{logging}}\code{ = TRUE}}{
       Flag indicating if logging functionality is enabled in the automatic runs of the pipeline.}
  \item{\bold{\code{email}}\code{ = NULL}}{
       Email address associated with the analyses.}
  \item{\bold{\code{assembly}}\code{ = "hg19"}}{
       Genome assembly to be used. Currently only important for bisulfite mode. The supported genomes returned by the
       function \code{\link{rnb.get.assemblies}}.}
  \item{\bold{\code{analyze.sites}}\code{ = TRUE}}{
       Flag indicating if analysis on site or probe level is to be conducted. Note that the preprocessing module
       always operates on the site level (only), regardless of the value of this option.}
  \item{\bold{\code{preanalysis.script}}\code{no default}}{
       Path to an R script that is executed before every module. This options is useful
       if e.g., if you specify custom regions that have to be present in a cluster enviroment.}
  \item{\bold{\code{region.types}}\code{ = NULL}}{
       Region types to carry out analysis on, in the form of a \code{character} vector. \code{NULL} (default value)
       signifies that all available region annotations (as returned by \code{\link{rnb.region.types}}) are summarized
       upon loading and normalization, and the other modules analyze all regions summarized in the dataset. If this
       option is set to an empty vector, analysis on the region level is skipped.}
  \item{\bold{\code{region.aggregation}}\code{ = "mean"}}{
       Aggregation function to apply when calculating the methylation value for a region based on the values of the
       CpGs associated with that region. Accepted values for this function are \code{"min"}, \code{"max"},
       \code{"mean"} (default), \code{"median"}, \code{"sum"}, \code{"coverage.weighted"}. The last method is
       applicable only for sequencing-based methylation datasets. It computes the weighted average of the values of
       the associated CpGs, whereby weights are calculated based on the coverages of the respective sites.}
  \item{\bold{\code{region.subsegments}}\code{ = 0}}{
       If a number larger than 1 is specified, \pkg{RnBeads} will subdivide each region specified in the
       \code{region.types} option into subsegments containing on average \code{region.subsegments} sites per
       subsegment. This is done by clustering the sites within each regions according to their genomic coordinates.
       These subsegments are then used for subsequent analysis.
       Use cautiously as this will significantly increase the runtime of the pipeline.}
  \item{\bold{\code{region.subsegments.types}}\code{ = NULL}}{
       The region types to which subsegmentation will be applied. Defaults to \code{region.types} when set to
       \code{NULL}.}
  \item{\bold{\code{identifiers.column}}\code{ = NULL}}{
       Column name or index in the table of phenotypic information to be used when plotting sample identifiers. If
       this option is \code{NULL}, it points to a non-existing column or a column that does not list IDs, the default
       identifiers are used. These are the row names of the sample phenotype table (and the column names of the beta
       value matrix).}
  \item{\bold{\code{colors.category}}\code{ = c("#1B9E77","#D95F02",...)}}{
       \code{character} vector of length 2 or more giving the color scheme for displaying categorical trait values in
       plots. RnBeads denotes missing values (\code{NA}) by grey, therefore, it is not recommended to include shades
       of grey in this vector. The default value of this option is the result of the \code{"Dark2"} palette of
       \emph{RColorBrewer} with 8 values.}
  \item{\bold{\code{colors.gradient}}\code{ = c("#132B43","#56B1F7")}}{
       \code{character} vector of length 2 or more giving the color scheme for displaying continuous (gradient) trait
       values in plots. \pkg{RnBeads} interpolates between the color values.}
  \item{\bold{\code{min.group.size}}\code{ = 2}}{
       Minimum number of samples each subgroup defined by a trait, in order for this trait to be considered in the
       methylation profiles and in the differential methylation modules. This must be a positive \code{integer}.}
  \item{\bold{\code{max.group.count}}\code{ = NULL}}{
       Maximum number of subgroups defined by a trait, in order for this trait to be considered in the methylation
       profiles and in the differential methylation modules. This must be an \code{integer} of value \code{2} or
       more. As a special case, a value of \code{NULL} (default) indicates that the maximum number of subgroups is
	  the number of samples in an analysis minus \code{1}, i.e. traits with all unique values will be ignored.}
  \item{\bold{\code{replicate.id.column}}\code{ = NULL}}{
       Column name in the sample annotation table that indicates sample replicates. Replicates are expected to
       contain the same value. Samples without replicates should contain unique or missing values. If this option is
       \code{NULL} (default), replicate handling is disabled.}
  \item{\bold{\code{gz.large.files}}\code{ = FALSE}}{
       Flag indicating whether large output files should be compressed (in \code{.gz} format).}
  \item{\bold{\code{import}}\code{ = TRUE}}{
       Flag controlling whether data import report should be generated. This option be set to \code{FALSE} only when
       the provided data source is an object of type \linkS4class{RnBSet}, i.e. the data has been previously loaded
       by \pkg{RnBeads}.}
  \item{\bold{\code{import.default.data.type}}\code{ = "infinium.idat.dir"}}{
       Type of data assumed to be supplied by default (Infinium 450k microarray).
		  For sequencing data set this to \code{bs.bed.dir} and save the options.
		  See \code{\link{rnb.execute.import}} for further details.}
  \item{\bold{\code{import.table.separator}}\code{ = ","}}{
       Separator used in the plain text data tables. See \code{\link{rnb.execute.import}} for details.}
  \item{\bold{\code{import.bed.style}}\code{ = "bismarkCov"}}{
		  Preset for bed-like formats. \code{"BisSNP", "Encode","EPP", "bismarkCytosine", "bismarkCov"} are currently
		  supported. See the \pkg{RnBeads} vignette and the FAQ section on the website for more details.}
  \item{\bold{\code{import.bed.columns}}}{Column indices in the supplied BED file with DNA methylation information.
       These are represented by a named \code{integer} vector, in which the names are: \code{"chr"}, \code{"start"},
       \code{"end"}, \code{"strand"}, \code{"meth"}, \code{"coverage"}, \code{"c"} and \code{"t"}. These names
       correspond the columns for chromosome, start position, end position, strand, methylation degree, read
       coverage, number of reads with C and number of reads with T, respectively. Methylation degree and/or read
       coverage, if not specified, are inferred from the values in the columns \code{"c"} and \code{"t"}.
		  Further details and examples of BED files can be found in Section 4.1 of the RnBeads vignette.}
  \item{\bold{\code{import.bed.frame.shift}}\code{ = 1}}{Singleton of type \code{integer} specifying the frame shift between
       the coordinates in the input BED file and the corresponding genomic reference. This (\code{integer}) value
       is added to the coordinates from the BED file before matching the methylation sites to the annotated ones.}
  \item{\bold{\code{import.bed.test}}\code{ = TRUE}}{
		  Perform a small loading test, by reading 1000 rows from each BED file, after which normal loading is performed.
       See \pkg{RnBeads} vignette and the FAQ section on the website for more details.}
  \item{\bold{\code{import.bed.test.only}}\code{ = FALSE}}{
		  Perform only the small loading test, and skip loading all the data.}
  \item{\bold{\code{import.skip.object.check}}\code{ = FALSE}}{
	  Skip the check of the loaded RnBSet object after loading. Helps with keeping the memory profile down}
  \item{\bold{\code{import.idat.platform}}\code{ = NULL}}{
	  Character specifying the Infinium platform that is uses. Has to be one of \code{'probes27'},
	  \code{'probes450'}, \code{'probesEPIC'} or \code{'probesEPICv2'}. If \code{'auto'}, the platform is automatically detected
	  from the IDAT file names.}
  \item{\bold{\code{import.sex.prediction}}\code{ = TRUE}}{
       Flag indicating if sex prediction is to be performed. Sex prediction is supported for Infinium 450k, EPIC, EPICv2
       and bisulfite sequencing datasets with signal intensity or coverage information.
       The value of this option is ignored for 27k datasets.}
  \item{\bold{\code{qc}}\code{ = TRUE}}{
       Flag indicating if the quality control module is to be executed.}
  \item{\bold{\code{qc.boxplots}}\code{ = TRUE}}{
       [Microarrays] Add boxplots for all types of quality control probes to the quality control report. The boxplots
       give signal distribution across samples.}
  \item{\bold{\code{qc.barplots}}\code{ = TRUE}}{
       [Microarrays] Add barplots for each quality control probes to the quality control report.}
  \item{\bold{\code{qc.negative.boxplot}}\code{ = TRUE}}{
       [Microarrays] Add boxplot of negative control probe intensities for all samples.}
  \item{\bold{\code{qc.snp.heatmap}}\code{ = TRUE}}{
       [Microarrays] Flag indicating if a heatmap of the beta values for all SNP probes is to be generated.}
  \item{\bold{\code{qc.snp.barplot}}\code{ = FALSE}}{
       [Microarrays] Add bar plots of the beta-values observed for each SNP-calling probe.}
  \item{\bold{\code{qc.snp.boxplot}}\code{ = FALSE}}{
       [Microarrays] Add boxplot of beta-values for the SNP-calling probes.}
  \item{\bold{\code{qc.snp.distances}}\code{ = TRUE}}{
       [Microarrays] Flag indicating if intersample distances based on the beta values of SNP probes are to be
        displayed. This can help identify genetically similar or identical samples.}
  \item{\bold{\code{qc.snp.purity}}\code{ = FALSE}}{
       [Microarrays] Flag indicating if genetic purity should be estimated based on the beta values of SNP probes.}
  \item{\bold{\code{qc.sample.batch.size}}\code{ = 50}}{
       [Microarrays] Maximal number of samples included in a single quality control barplot and negative control boxplot.}
  \item{\bold{\code{qc.coverage.plots}}\code{ = FALSE}}{
       [Bisulfite sequencing] Add genome-wide sequencing coverage plot for each sample.}
  \item{\bold{\code{qc.coverage.threshold.plot}}\code{ = 1:10}}{
       [Bisulfite sequencing] Values for coverage cutoffs to be shown in a coverage thresholds plot. This must be an \code{integer}
       vector of positive values. Setting this to an empty vector disables the coverage thresholds plot.}
  \item{\bold{\code{qc.coverage.histograms}}\code{ = FALSE}}{
       [Bisulfite sequencing] Add sequencing coverage histogram for each sample.}
  \item{\bold{\code{qc.coverage.violins}}\code{ = FALSE}}{
       [Bisulfite sequencing] Add sequencing coverage violin plot for each sample.}
  \item{\bold{\code{qc.cnv}}\code{ = FALSE}}{
       [Microarrays] Add CNV estimation for each position in each sample.}
  \item{\bold{\code{qc.cnv.refbased}}\code{ = TRUE}}{
       [Microarrays] Should CNV estimation be performed with a reference (twin study) or with the mean over the samples.}
  \item{\bold{\code{preprocessing}}\code{ = TRUE}}{Flag controlling whether the data should be preprocessed
		  (whether quality filtering and in case of Infinium microarray data normalization should be applied).}
  \item{\bold{\code{normalization}}\code{ = NULL}}{
       Flag controlling whether the data should be normalized and normalization report generated. Setting this to
       \code{NULL} (default) enables this step for analysis on Infinium datasets, but disables it in case of
       sequencing-based datasets. Note that normalization is never applied in sequencing datasets; if this flag is
       enabled, it will lead to a warning message.}
  \item{\bold{\code{normalization.method}}\code{ = "wm.dasen"}}{
       Normalization method to be applied, or \code{"none"}. Multiple normalization methods are supported:
       \code{"illumina"} -
       \href{http://www.bioconductor.org/packages/devel/bioc/html/methylumi.html}{methylumi}-implemented
       Illumina scaling normalization; \code{"swan"} - SWAN-normalization by Gordon et al., as implemented
       in \href{http://www.bioconductor.org/packages/release/bioc/html/minfi.html}{minfi}; \code{"bmiq"} -
       beta-mixture quantile normalization method by Teschendorff et al; as well as \code{"wm.dasen"} (default),
       \code{"wm.nasen"}, \code{"wm.betaqn"}, \code{"wm.naten"}, \code{"wm.nanet"}, \code{"wm.nanes"},
       \code{"wm.danes"}, \code{"wm.danet"}, \code{"wm.danen"}, \code{"wm.daten1"}, \code{"wm.daten2"},
       \code{"wm.tost"}, \code{"wm.fuks"} and \code{"wm.swan"} - all normalization methods implemented in the
       \href{http://www.bioconductor.org/packages/release/bioc/html/wateRmelon.html}{wateRmelon} package. When
       setting this option to a specific algorithm, make sure its dedicated package is installed.
       \code{"scaling.internal"}, \code{"scalings"} - different scaling methods as implemented in the
       \href{https://www.bioconductor.org/packages/release/bioc/html/sesame.html}{sesame} R package.
       }
  \item{\bold{\code{normalization.background.method}}\code{ = "none"}}{
       A character singleton specifying which background subtraction is to be performed during normalization.
       The following values are accepted: \code{"none"} (default), \code{"methylumi.noob"}, \code{"methylumi.goob"},
       \code{"methylumi.lumi"}, \code{"enmix.oob"},  \code{"sesame.noob"} and \code{"sesame.noobsb"}.}
  \item{\bold{\code{normalization.plot.shifts}}\code{ = TRUE}}{
       Flag indicating if the report on normalization should include plots of shifts (degrees of beta value
       correction).}
  \item{\bold{\code{filtering.whitelist}}\code{ = NULL}}{Name of a file specifying site or probe identifiers to be
       whitelisted. Every line in this file must contain exactly one identifier. The whitelisted sites are always
       retained in the analysed datasets, even if filtering criteria or blacklisting requires their removal.
       For Infinium studies, the file must contain Infinium probe identifiers. For bisulfite sequencing studies,
       the file must contain CpG positions in the form "chromosome:coordinate" (1-based coordinate of the cytosine),
       e.g. \code{chr2:48607772}. Unknown identifiers are silently ignored.}
  \item{\bold{\code{filtering.blacklist}}\code{ = NULL}}{Name of a file specifying site or probe identifiers to be
       blacklisted. Every line in this file must contain exactly one identifier. The blacklisted sites are removed
       from the analysed datasets as a first step in the preprocessing module. For Infinium studies, the file must
       contain Infinium probe identifiers. For bisulfite sequencing studies, the file must contain CpG positions in
       the form "chromosome:coordinate" (1-based coordinate of the cytosine), e.g. \code{chr2:48607772}.
       Unknown identifiers are silently ignored.}
  \item{\bold{\code{filtering.context.removal}}\code{ = c("CC","CAG",...)}}{
       \code{character} vector giving the list of probe context types to be removed as a filtering step. Possible
       context values are \code{"CC"}, \code{"CG"}, \code{"CAG"}, \code{"CAH"}, \code{"CTG"}, \code{"CTH"} and
       \code{"Other"}. Probes in the second context measure CpG methylation; the last context denotes probes
       dedicated to SNP detection. Setting this option to \code{NULL} or an empty vector effectively disables the
       step of context-specific probe removal.}
  \item{\bold{\code{filtering.snp}}\code{ = "any"}}{
       Removal of sites or probes based on overlap with SNPs. The accepted values for this option are:
       \describe{
          \item{\code{"no"}}{no SNP-based filtering;}
          \item{\code{"3"}}{filter out a probe when the last 3 bases in its target sequence overlap with SNP;}
          \item{\code{"5"}}{filter out a probe when the last 5 bases in its target sequence overlap with SNP;}
          \item{\code{"any"} or \code{"yes"}}{filter out a CpG site or probe when any base in its target sequence
               overlaps with SNP.}}
       Bisulfite sequencing datasets operate on sites instead of probes, therefore, the values \code{"3"} and
       \code{"5"} are treated as \code{"yes"}.}
  \item{\bold{\code{filtering.cross.reactive}}\code{ = TRUE}}{
       Flag indicating if the removal of potentially cross-reactive probes should be performed as a filtering step
       in the preprocessing module. A probes whose sequence maps to multiple genomic locations (allowing up to 3
       mismatches) is cross-reactive.}
  \item{\bold{\code{filtering.greedycut}}\code{ = NULL}}{
       Flag indicating if the Greedycut procedure should be run as a filtering step in the preprocessing module.
       \code{NULL} (default) indicates that Greedycut will be run for array-based datasets, but not for
       sequencing-based datasets.}
  \item{\bold{\code{filtering.greedycut.pvalue.threshold}}\code{ = 0.05}}{
       Threshold for the detection p-value to be used in Greedycut. This is a value between 0 and 1. This option has
       effect only when \code{filtering.greedycut} is \code{TRUE}.}
  \item{\bold{\code{filtering.greedycut.rc.ties}}\code{ = "row"}}{
       Indicator of what the behaviour of Greedycut should be in case of ties between the scores of rows (probes) and
       columns (samples). The value of this option must be one of \code{"row"}, \code{"column"} or \code{"any"}; the
       last one indicating random choice. This option has effect only when \code{filtering.greedycut} is
       \code{TRUE}.}
  \item{\bold{\code{filtering.sex.chromosomes.removal}}\code{ = TRUE}}{
       Flag indicating if the removal of probes located on sex chromosomes should be performed as a filtering step.}
  \item{\bold{\code{filtering.missing.value.quantile}}\code{ = 0.5}}{
       Number between 0 and 1, indicating the fraction of allowed missing values per site. A site is filtered out
       when its methylation beta values are \code{NA}s in a larger fraction of samples than this threshold. Setting
       this option to 1 (default) retains all sites, and thus effectively disables the missing value filtering step
       in the preprocessing module. If this is set to 0, all sites that contain missing values are filtered out.}
  \item{\bold{\code{filtering.coverage.threshold}}\code{ = 5}}{
       Threshold for minimal acceptable coverage. This must be a non-negative value. Setting this option to 0 (zero)
       effectively considers any known or unknown read coverage for sufficiently deep.}
  \item{\bold{\code{filtering.low.coverage.masking}}\code{ = FALSE}}{
       Flag indicating whether methylation values for low coverage sites should be set to missing. In combination
       with \code{filtering.missing.value.quantile} this can lead to the removal of sites.}
  \item{\bold{\code{filtering.high.coverage.outliers}}\code{ = FALSE}}{
       (Bisulfite sequencing mode) Flag indicating whether methylation sites with a coverage of more than 10 times
       the 95-percentile of coverage should be removed.}
  \item{\bold{\code{filtering.deviation.threshold}}\code{ = 0}}{
       Threshold used to filter probes based on the variability of their assigned beta values. This must be a real
       value between 0 and 1, denoting minimum standard deviation of the beta values in one site across all samples.
       Any sites that have standard deviation lower than this threshold are filtered out. Note that sites with
       undetermined varibility, that is, sites for which there are no measurements (all beta values are \code{NA}s),
       are retained. Setting this option to 0 (default) disables filtering based on methylation variability.}
  \item{\bold{\code{imputation.method}}\code{ = "none"}}{
       Character indicating which imputation method should be used to replace missing values. This option has to be
       one of the following values \code{"none"}, \code{"mean.cpgs"}, \code{"mean.samples"}, \code{"random"},
       \code{"median.cpgs"}, \code{"median.samples"} or \code{"knn"}. Setting this option to \code{"none"} inactivates
       imputation (default).}
  \item{\bold{\code{inference}}\code{ = FALSE}}{
       Flag indicating if the covariate inference analysis module is to be executed.}
  \item{\bold{\code{inference.genome.methylation}}\code{ = "Genome-wide methylation"}}{
       Name of the column to add to the sample annotation, storing the genome-wide methylation level. If such a
       column already exists, its values will be overwritten. Setting this option to \code{NULL} or an empty
       \code{character} disables computing and adding genome-wide methylation levels.}
  \item{\bold{\code{inference.targets.sva}}\code{ = character()}}{
       Column names in the sample annotation table for which surrogate variable analysis (SVA) should be conducted.
	  An empty vector (default) means that SVA is skipped.}
  \item{\bold{\code{inference.reference.methylome.column}}\code{ = character()}}{
       Column name in the sample annotation table giving the assignment of samples to reference methylomes.
		  The target samples should have \code{NA} values in this column.}
  \item{\bold{\code{inference.max.cell.type.markers}}\code{ = 50000}}{
       Number of most variable CpGs which are tested for association with the reference cell types. Setting this
       option to \code{NULL} forces the algorithm to use all available sites in the dataset, and may greatly
       increase the running time for cell type comoposition estimation.}
  \item{\bold{\code{inference.top.cell.type.markers}}\code{ = 500}}{
       Number of top cell type markers used for determining cell type contributions to the target DNA methylation
		  profiles using the projection method of Houseman et al.}
  \item{\bold{\code{inference.sva.num.method}}\code{ = "leek"}}{
       Name of the method to be used for estimating the number of surrogate variables.
	  must be either 'leek' or 'be', See \code{sva} function for details.}
  \item{\bold{\code{inference.age.column}}\code{ = "age"}}{
       Name of the column in which the ages of the donors are annotated. This function can be of numeric, string or
	  factor format.}
  \item{\bold{\code{inference.age.prediction}}\code{ = TRUE}}{
       Flag indicating if the epigenetic age prediction within the inference module is to be executed.}
  \item{\bold{\code{inference.age.prediction.training}}\code{ = FALSE}}{
       Flag indicating if a new predictor should be created based on the provided data set.}
  \item{\bold{\code{inference.age.prediction.cv}}\code{ = FALSE}}{
       Flag indicating if predictive power of a predictor that was trained in that run of the age prediction should
	be assessed by cross-validation. This option only has an influence if
	\code{inference.age.prediction.training}\code{ = TRUE}.}
  \item{\bold{\code{inference.immune.cells}}\code{ = TRUE}}{
       Flag indicating if immune cell content estimation is to be performed. Immune cell content prediction is based
       on the LUMP algorithm and is currently supported for the hg19 assembly only.}
  \item{\bold{\code{inference.mouse.fraction}}\code{ = FALSE}}{
       Flag indicating if mouse contamination is to be estimated. This is only meaningful, if PDX models or artificial
       human/mouse mixtures were used. If this option is \code{TRUE}, the \code{sesame} R package has to be 
       installed.}
  \item{\bold{\code{exploratory}}\code{ = TRUE}}{
       Flag indicating if the exploratory analysis module is to be executed.}
  \item{\bold{\code{exploratory.columns}}\code{ = NULL}}{
       Traits, given as column names or indices in the sample annotation table, to be used in the exploratory
       analysis. These traits are used in multiple steps in the module: they are visualized using point types and
       colors in the dimension reduction plots; tested for strong correlations and associations with principal
       components in a methylation space; used to define groups when plotting beta distributions and/or inter-sample
       methylation variability. The default value of this parameter - \code{NULL} - indicates that columns should be
       automatically selected; see \code{\link{rnb.sample.groups}} for how this is done.}
  \item{\bold{\code{exploratory.top.dimensions}}\code{ = 0}}{
       Number of most variable probes, sites or regions to select prior to performing dimension reduction techniques
       and tests for associations. Preselection can significantly reduce the running time and memory usage in the
       exploratory analysis module. Setting this number to zero (default) disables preselection.}
  \item{\bold{\code{exploratory.principal.components}}\code{ = 8}}{
       Maximum number of principal components to be tested for associations with other factors, such as control probe
       states and sample traits. This must be an \code{integer} value between \code{0} and \code{10}. Setting this
       option to \code{0} disables such tests.}
  \item{\bold{\code{exploratory.correlation.pvalue.threshold}}\code{ = 0.01}}{
       Significance threshold for a p-value resulting from applying a test for association. This is a value between
       0 and 1.}
  \item{\bold{\code{exploratory.correlation.permutations}}\code{ = 10000}}{
       Number of permutations in tests performed to check for associations between traits, and between control probe
       intensities and coordinates in the prinicipal component space. This must be a non-negative \code{integer}.
       Setting this option to \code{0} disables permutation tests.}
  \item{\bold{\code{exploratory.correlation.qc}}\code{ = TRUE}}{
       [Infinium 450k] Flag indicating if quality-associated batch effects should be studied. This amounts to testing for
       associations between intensities of quality control probes and principal components. This option has effect
       only when \code{exploratory.principal.components} is non-zero.}
  \item{\bold{\code{exploratory.beta.distribution}}\code{ = TRUE}}{
       Flag indicating whether beta value distributions for sample groups and probe or site categories should be
       computed.}
  \item{\bold{\code{exploratory.intersample}}\code{ = FALSE}}{
       Flag indicating if methylation variability in sample groups should be computed as part of the exploratory
       analysis module. If NULL (default), the plots are created for Bead Array data sets and deactivated for sequencing
       data sets.}
  \item{\bold{\code{exploratory.deviation.plots}}\code{ = FALSE}}{
       Flag indicating if the inter-sample methylation variability step in the exploratory analysis module should
       include deviation plots. Deviation plots show intra-group methylation variability at the covered sites and
       regions. Setting this option to \code{NULL} (default) enables deviation plots on Infinium datasets, but
       disables them in case of sequencing-based datasets, because their generation can be very computationally
       intensive. This option has effect only when \code{exploratory.intersample} is \code{TRUE}.}
  \item{\bold{\code{exploratory.clustering}}\code{ = "all"}}{
       Which sites should be used by clustering algorithms in the exploraroty analysis module.
       \pkg{RnBeads} performs several algorithms that cluster the samples in the dataset. If this option is set to
       \code{"all"} (default), clustering is performed using all sites; a value of \code{"top"} indicates that only
       the most variable sites are used (see the option \code{exploratory.clustering.top.sites}); and \code{"none"}
       disables clustering.}
  \item{\bold{\code{exploratory.clustering.top.sites}}\code{ = 1000}}{
       Number of most variable sites to use when visualizing heatmaps. This must be a non-empty \code{integer} vector
       containing positive values. This option is ignored when \code{exploratory.clustering} is \code{"none"}.}
  \item{\bold{\code{exploratory.clustering.heatmaps.pdf}}\code{ = FALSE}}{
       Flag indicating if the generated methylation value heatmaps in the clustering section of the exploratory
       analysis module should be saved as PDF files. Enabling this option is not recommended for large values of
       \code{exploratory.clustering.top.sites} (more than 200), because heatmaps might generate very large PDF files.}
  \item{\bold{\code{exploratory.region.profiles}}\code{ = ""}}{
       Region types for generating regional methylation profiles. If \code{NULL} (default), regional methylation
       profiles are created only for the region types that are available for the targeted assembly and summarized in
       the dataset of interest. Setting this option to an empty vector disables the region profiles step in the
       exploratory analysis module.}
  \item{\bold{\code{exploratory.gene.symbols}}\code{ = NULL}}{
       A list of gene symbols to be used for custom locus profiling. Locus views will be generated for these genes.
       }
  \item{\bold{\code{exploratory.custom.loci.bed}}\code{ = NULL}}{
       Path to a bed file containing custom genomic regions. Locus views will be generated for these regions.
       }
  \item{\bold{\code{nv.heatmap}}\code{ = TRUE}}{
       [Microarrays] Flag indicating if a heatmap of the beta values for all nv probes is to be generated.}
  \item{\bold{\code{nv.beta.distribution}}\code{ = TRUE}}{
       [Microarrays] Flag indicating if a density plot of the beta values for all nv probes is to be generated.}
  \item{\bold{\code{differential}}\code{ = TRUE}}{
       Flag indicating if the differential methylation module is to be executed.}
  \item{\bold{\code{differential.site.test.method}}\code{ = "limma"}}{
       Method to be used for calculating p-values on the site level. Currently supported options are "ttest" for a (paired)
       t-test and "limma" for a linear modeling approach implemented in the \code{limma} package for differential expression
       in microarrays.}
  \item{\bold{\code{differential.variability}}\code{ = FALSE}}{
       Flag indicating if differential variability analysis is to be conducted. If TRUE, the method specified in
       \code{differential.variability.method} is applied to detect sites that show differential variability between the groups
       that are specified.
       }
  \item{\bold{\code{differential.variability.method}}\code{ = "diffVar"}}{
       Method to be used for calculating p-values on the differential variable sites. Currently supported options are "diffVar"
       implemented in the \code{missMethyl} package and "iEVORA".}
  \item{\bold{\code{differential.permutations}}\code{ = 0}}{
       Number of permutation tests performed to compute the p-value of rank permutation tests in the differential
       methylation analysis. This must be a non-negative \code{integer}. Setting this option to \code{0} (default)
       disables permutation tests for rank permutations. Note that p-values for differential methylation are
       computed and also considered for the ranking in any case.}
  \item{\bold{\code{differential.comparison.columns}}\code{ = NULL}}{
       Column names or indices in the table of the sample annotation table to be used for group definition in the
       differential methylation analysis. The default value - \code{NULL} - indicates that columns should be
       automatically selected. See \code{\link{rnb.sample.groups}} for how this is done. By default,
	  the comparisons are done in a one vs. all manner if there are multiple
	  groups defined in a column. }
  \item{\bold{\code{differential.comparison.columns.all.pairwise}}\code{ = NULL}}{
       Column names or indices in the table of sample annotation table to be used for group definition in the
       differential methylation analysis in which all pairwise comparisons between groups should be conducted (the default
	  is one vs all if multiple groups are specified in a column).
       Caution: for large numbers of sample groups this can lead to combinatorial explosion and thus to huge runtimes.
       A value of \code{NULL} (default) indicates that no column is selected for all pairwise comparisons explicitely.
       If specified, the selected columns must be a subset of the columns that will be selected according to the
       \code{differential.comparison.columns} option.}
  \item{\bold{\code{covariate.adjustment.columns}}\code{ = NULL}}{
       Column names or indices in the table of phenotypic information to be used for confounder adjustment in the
       differential methylation analysis. Currently this is only supported for \code{differential.site.test.method=="limma"}.
       }
 \item{\bold{\code{columns.pairing}}\code{ = NULL}}{
		  A NAMED vector containing for each column name for which paired analysis
		  should be performed (say columnA) the name or index of another column (say columnB) in which same values indicate
		  the same pairing. columnA should be the name of the value columnB in this vector.
		  For more details see \code{\link{rnb.sample.groups}}}
  \item{\bold{\code{differential.adjustment.sva}}\code{ = FALSE}}{
       Flag indicating if the differential methylation analysis should account for Surrogate Variables. If
       \code{TRUE}, \pkg{RnBeads} looks for overlaps between the \code{differential.comparison.columns} and
       \code{inference.targets.sva} options and include the surrogate variables as confounding factors only for these
       columns. In other words, it will only have an effect if the corresponding inference option
	  (see \code{inference.targets.sva} option for details) is enabled.
	  Currently this is only supported for \code{differential.site.test.method=="limma"}.}
  \item{\bold{\code{differential.adjustment.celltype}}\code{ = FALSE}}{
       Should the differential methylation analysis account for celltype using the reference based Houseman method.
       It will only have an effect if the corresponding inference option is enabled (see \code{inference.reference.methylome.column}
	  option for details). Currently this is only supported for \code{differential.site.test.method=="limma"}.
       }
  \item{\bold{\code{differential.enrichment.go}}\code{ = FALSE}}{
       Flag indicating whether \href{http://www.geneontology.org/}{Gene Ontology} (GO)-enrichment analysis is to be
       conducted on the identified differentially methylated regions.}
  \item{\bold{\code{differential.enrichment.lola}}\code{ = FALSE}}{
       Flag indicating whether \code{LOLA}-enrichment analysis is to be
       conducted on the identified differentially methylated regions.}
  \item{\bold{\code{differential.enrichment.lola.dbs}}\code{ = c("${LOLACore}")}}{
       Vector of directories containing LOLA databases. The following placeholders are allowed which will
       automatically download corresponding databases from the internet: \code{"${LOLACore}"} and \code{"${LOLAExt}"}
       for the Core and Extended LOLA Databases respectively.}
  \item{\bold{\code{differential.report.sites}}\code{ = TRUE}}{
       Flag indicating whether a section corresponding to differential site methylation should be added to the report.
       Has no effect on the actual analysis, just the report. To disable differential site methylation analysis entirely
       use the \code{analyze.sites} option.}
  \item{\bold{\code{export.to.bed}}\code{ = FALSE}}{
       Flag indicating whether the data should be exported to bed files.}
  \item{\bold{\code{export.to.trackhub}}\code{ = NULL}}{
       \code{character} vector specifying which data types should be exported to
       \href{http://genome.ucsc.edu/goldenPath/help/hgTrackHubHelp.html}{Track hub directories}. Possible values
       in the vector are \code{"bigBed"} and \code{"bigWig"}. When this options is set to \code{NULL}, track hub
       export is disabled. Note that if \code{"bigBed"} is contained in this option, bed files are created
       automatically.}
  \item{\bold{\code{export.to.csv}}\code{ = FALSE}}{
       Flag indicating whether methylation value matrices are to be exported to comma-separated value (CSV) files.}
  \item{\bold{\code{export.to.ewasher}}\code{ = FALSE}}{
       Flag indicating whether methylation values and differential methylation analysis settings should be exported to
	  a format compatible with FaST-LMM-EWASher, a tool for adjusting for cell-type compositions.
	  See \href{http://www.nature.com/nmeth/journal/v11/n3/full/nmeth.2815.html}{Zou, J., et al., Nature Methods, 2014} for further details on the tool.}
  \item{\bold{\code{export.types}}\code{ = "sites"}}{
       \code{character} vector of sites and region names to be exported. If \code{NULL}, no region methylation values
       are exported.}
  \item{\bold{\code{disk.dump.big.matrices}}\code{ = TRUE}}{
       Flag indicating whether big tables should be stored on disk rather than in main memory in order to keep memory
       requirements down. May slow down analysis!}
  \item{\bold{\code{logging.exit.on.error}}\code{ = FALSE}}{
       Flag indicating if the active R session should be terminated when an error is encountered during execution.}
  \item{\bold{\code{distribution.subsample}}\code{ = 1000000}}{
       When plotting methylation value distributions, this threshold specifies the number of observations drawn per
       group. Distributions are estimated and plotted based on these random subsamples. This approach can
       significantly reduce the memory requirements of the preprocessing and exploratory analysis modules, where
       methylation value distributions are plotted. Setting this to \code{0} disables subsampling. More information
       is presented the Details section of \code{\link{rnb.step.betadistribution}}}.
  \item{\bold{\code{enforce.memory.management}}\code{ = FALSE}}{
       Flag indicating whether in some places of the code memory management should actively being enforced in order to
       achieve a better memory profile. I.e. garbage collection, variable removal is conducted actively.
       May slow down analysis.}
  \item{\bold{\code{enforce.destroy.disk.dumps}}\code{ = FALSE}}{
       Flag indicating whether disked dumped big matrices (see \code{disk.dump.big.matrices} option) should actively
       be deleted when RnBSets are modified. You should switch it to \code{TRUE} when \code{disk.dump.big.matrices}
		  is \code{TRUE} and the amount of hard drive space is also limited.}
}
}

\examples{
\donttest{
str(rnb.options())
rnb.getOption("filtering.greedycut")
}

}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataExport.R
\name{rnb.export.to.trackhub}
\alias{rnb.export.to.trackhub}
\title{rnb.export.to.trackhub}
\usage{
rnb.export.to.trackhub(
  rnb.set,
  out.dir,
  reg.type = "sites",
  data.type = "bigBed",
  ...
)
}
\arguments{
\item{rnb.set}{Object of class \code{\linkS4class{RnBSet}}}

\item{out.dir}{output directory. If not existing, it will be created. otherwise files in that directory are overwritten.}

\item{reg.type}{region type to be converted}

\item{data.type}{either "bigBed" or "bigWig"}

\item{...}{parameters passed on to the track hub generating procedure}
}
\value{
a list containing information on the export
}
\description{
convert an \code{\linkS4class{RnBSet}}  object to a UCSC-style track hub.
}
\details{
During execution the RnBSet is converted to bed files. If the operating system is supported (currently Unix and MacOS only)
         these are automatically converted to bigBed files. If your operating system is not supported, you need to create them manually
         (see the \href{http://genome.ucsc.edu/FAQ/FAQformat.html}{UCSC Genome Browser documentation} for details).
         For details on UCSC track hubs see the
         \href{http://genome.ucsc.edu/goldenPath/help/hgTrackHubHelp.html}{UCSC tracks help page}.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
rnb.export.to.trackhub(rnb.set.example,tempdir())
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBDiffMeth-class.R
\docType{methods}
\name{destroy,RnBDiffMeth-method}
\alias{destroy,RnBDiffMeth-method}
\title{destroy-methods}
\usage{
\S4method{destroy}{RnBDiffMeth}(object)
}
\arguments{
\item{object}{\code{\linkS4class{RnBDiffMeth}} object}
}
\value{
Nothing of particular interest
}
\description{
remove tables stored to disk from the file system. Useful for cleaning up disk dumped objects.
CAUTION: currently only works with reloaded objects
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBDiffMeth-class.R
\docType{methods}
\name{includes.sites,RnBDiffMeth-method}
\alias{includes.sites,RnBDiffMeth-method}
\alias{includes.sites}
\title{includes.sites-methods}
\usage{
\S4method{includes.sites}{RnBDiffMeth}(object)
}
\arguments{
\item{object}{RnBDiffMeth object}
}
\value{
\code{TRUE} if the differential methylation object contains site-level information. \code{FALSE} otherwise
}
\description{
Returns \code{TRUE} if the differential methylation object contains site-level information
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
includes.sites(dm)
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/controlPlotsBiSeq.R
\name{rnb.plot.biseq.coverage}
\alias{rnb.plot.biseq.coverage}
\title{rnb.plot.biseq.coverage}
\usage{
rnb.plot.biseq.coverage(
  rnbs.set,
  sample,
  type = "sites",
  writeToFile = FALSE,
  numeric.names = FALSE,
  covg.lists = NULL,
  ...
)
}
\arguments{
\item{rnbs.set}{RnBiseqSet object}

\item{sample}{unique sample identifier. In case \code{rnb.getOption("identifiers.column")} 
is not \code{NULL}, \code{sample} should attain values from the corresponding column, 
or \code{colnames(meth(rnb.set))} otherwise}

\item{type}{\code{character} singleton. If \code{site} the coverage
information is plotted for each methylation site. Otherwise
should be one of the regions returned by \code{rnb.region.types}}

\item{writeToFile}{flag specifying whether the output should be saved as \code{\linkS4class{ReportPlot}}}

\item{numeric.names}{if \code{TRUE} and \code{writeToFile} is \code{TRUE}substitute the plot options in the plot file name with digits}

\item{covg.lists}{if available, the output of \code{\link{rnb.execute.quality}}}

\item{...}{other arguments to \code{\link{createReportPlot}}}
}
\value{
plot as an object of type \code{\linkS4class{ReportPlot}} if \code{writeToFile} is \code{TRUE} and of class 
			\code{\link{ggplot}} otherwise.
}
\description{
Plots the sequencing coverage of the RnBiseqSet object across the genomic coordinate
}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Report-methods.R
\name{rnb.add.paragraph}
\alias{rnb.add.paragraph}
\title{rnb.add.paragraph}
\usage{
rnb.add.paragraph(report, txt, paragraph.class = NULL)
}
\arguments{
\item{report}{Report to write the text to.}

\item{txt}{\code{character} vector (or array) storing the text to be written. The elements of this vector
are concatenated without a separator.}

\item{paragraph.class}{CSS class definition of the paragraph. This must be either \code{NULL} (default) or one of:
\describe{
  \item{\code{"centered"}}{This paragraph gives a formula or a short statement. Text is
       horizontally centered.}
  \item{\code{"note"}}{This paragraph describes a note. Text is italic.}
  \item{\code{"task"}}{This paragraph describes a task. Text is bold and bright red.}
}}
}
\value{
The modified report, invisibly.
}
\description{
Generates HTML code for a new paragraph in the specified report.
}
\examples{
\donttest{
report <- createReport("example.html", "Example", init.configuration = TRUE)
txt <- "A pessimist is a person who has had to listen to too many optimists."
txt <- c(txt, " <i>Don Marquis</i>")
rnb.add.paragraph(report, txt)
}
}
\seealso{
\code{\linkS4class{Report}} for other functions adding contents to an HTML report
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ReportPlot-methods.R
\name{get.files}
\alias{get.files}
\title{get.files}
\usage{
get.files(report.plot)
}
\arguments{
\item{report.plot}{Report plot of interest. This must be an object of type \code{\linkS4class{ReportPlot}}.}
}
\value{
Non-empty \code{character} vector of absolute file names.
}
\description{
Gets the list of all files that are planned to be generated, or were already generated by the given report plot.
}
\examples{
\donttest{
plot.image <- createReportPlot('scatterplot', high.png = 200)
get.files(plot.image)
}
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batch.quality.R
\name{rnb.execute.batch.qc}
\alias{rnb.execute.batch.qc}
\title{rnb.execute.batch.qc}
\usage{
rnb.execute.batch.qc(rnb.set, pcoordinates, permutations = NULL)
}
\arguments{
\item{rnb.set}{HumanMethylation450K dataset as an object of type \code{\linkS4class{RnBeadSet}}.}

\item{pcoordinates}{Coordinates of the samples of \code{rnb.set} in the principal components space, as returned by
\code{\link{rnb.execute.dreduction}}.}

\item{permutations}{Matrix of sample index permutations, as returned by \code{\link{rnb.execute.batcheffects}}. If
this parameter is \code{NULL}, permutation-based p-values are not calculated.}
}
\value{
\code{NULL} if no principal components for batch analysis are specified (
        \code{rnb.getOption("exploratory.principal.components") == 0}); otherwise, a hierarchical structure of
        matrices in the form of a nested list. The root branches are represented by the elements
        \code{"correlations"} and \code{"pvalues"}. Every element is a list of control probe types; each type is in
        turn a list of up to two matrices of correlations between probe values and principal components - one for the
        probes on the green channel and one for the red channel. Note that the \code{"pvalues"} branch is not
        returned when \code{permutations} is \code{NULL}.
}
\description{
Computation of correlations and permutation-based p-values for detecting quality-associated batch effects.
}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrichment.R
\name{loadLolaDbs}
\alias{loadLolaDbs}
\title{loadLolaDbs}
\usage{
loadLolaDbs(lolaDbPaths)
}
\arguments{
\item{lolaDbPaths}{vector of names of LOLA DB paths to be loaded}
}
\value{
LOLA DB list as returned by \code{LOLA::loadRegionDB}
}
\description{
Load LOLA databases from disk and merge them
}
\examples{
\donttest{
# download LOLA DB
lolaDest <- tempfile()
dir.create(lolaDest)
lolaDirs <- downloadLolaDbs(lolaDest, dbs="LOLACore")
lolaDb <- loadLolaDbs(lolaDirs[["hg19"]])
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBDiffMeth-class.R
\docType{methods}
\name{reload,RnBDiffMeth-method}
\alias{reload,RnBDiffMeth-method}
\alias{reload}
\title{reload-methods}
\usage{
\S4method{reload}{RnBDiffMeth}(
  object,
  save.file,
  disk.path = tempfile(pattern = "diffmeth_", tmpdir = getOption("fftempdir"))
)
}
\arguments{
\item{object}{\code{\linkS4class{RnBDiffMeth}} object}

\item{save.file}{location of the ff data saved to disk (i.e. save in save.RData and save.ffData)}

\item{disk.path}{path on the disk for DMTs. can be new or be the same as in the original object}
}
\value{
the updated RnBDiffMeth object
}
\description{
reload disk dumped tables. Useful if the table files are manually copied or if the object is loaded again.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
#compute differential methylation
pcols <- c("Sample_Group","Treatment")
tdir <- tempfile(pattern="working")
dm <- rnb.execute.computeDiffMeth(rnb.set.example,pcols,disk.dump=TRUE,disk.dump.dir=tdir)
#get temporary file names
fn.save.tabs <- tempfile(pattern="saveTables")
fn.save.obj  <- tempfile(pattern="saveObject")
#save the object and the tables to disk
save(dm,file=fn.save.obj)
save.tables(dm,fn.save.tabs)
#delete the object from the workspace
destroy(dm)
rm(dm)
#reload the object and tables
load(fn.save.obj)
dm.new <- reload(dm,fn.save.tabs)
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterArchitectureSGE.R
\docType{class}
\name{ClusterArchitectureSGE-class}
\alias{ClusterArchitectureSGE-class}
\title{ClusterArchitectureSGE Class}
\description{
A child class of \code{\linkS4class{ClusterArchitecture}} implementing specifications of Sun Grid Engine (SGE) architectures.
}
\details{
Follow this template if you want to create your own ClusterArchitecture class.
}
\section{Slots}{

see \code{\linkS4class{ClusterArchitecture}}
}

\section{Methods}{

\describe{
  \item{\code{\link{getSubCmdTokens,ClusterArchitectureSGE-method}}}{Returns a vector of command line tokens corresponding to submitting
  a job with the given command to the cluster}
}
}

\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Report-methods.R
\name{createReport}
\alias{createReport}
\title{createReport}
\usage{
createReport(
  fname,
  title,
  page.title = "RnBeads report",
  authors = NULL,
  dirs = NULL,
  init.configuration = FALSE
)
}
\arguments{
\item{fname}{Single-element \code{character} vector denoting the name of the file to contain the HTML
report. If this file already exists, it will be overwritten.}

\item{title}{Title of the report in the form of a single-element \code{character} vector.}

\item{page.title}{Web page title. This usually appears in the web browser's window title when the report is
open. If specified, this must be a vector. Note that only the first element is used.}

\item{authors}{Optional list of authors in the form of a \code{character} vector. This list is included in
the header of the generated HTML file. Note that author names can contain only Latin leters,
space, dash (\code{-}), comma (\code{,}) or dot (\code{.}).}

\item{dirs}{Location of the supporting directories, that is, paths that are expected to contain
additional files linked to from the HTML report. See the \emph{Details} section for a list
of these directories.}

\item{init.configuration}{Flag indicating if the report configuration data should be initialized. If this parameter
is \code{TRUE}, the method creates the respective directory and copies configuration files
that define cascading style sheet (CSS) definitions and Javascript functions used by the
HTML report. If such configuration files already exist, they will be overwritten. Since the
aforementioned files can be shared by multiple reports, it is recommended that the
configuration is initialized using the method \code{\link{rnb.initialize.reports}}, instead
of setting this flag to \code{TRUE}.}
}
\value{
Newly created \code{\linkS4class{Report}} object.
}
\description{
Creates a new report object.
}
\details{
If specified, the parameter \code{dirs} must be a \code{character} vector. The following names are read:
\itemize{
 \item{\code{"configuration" }}{Directory that contains the auxilliary configuration files, such as style sheets and
       Javascript files. If missing or \code{NA}, the default value used is \code{"configuration"}.}
 \item{\code{"data" }}{Directory to contain the tables, lists and other generated data files that are linked to in
       the HTML report. If missing or \code{NA}, the value used is formed from the file name \code{fname} (without
       the extension) and the suffix \code{"_data"}.}
 \item{\code{"pngs" }}{Directory to contain the low resolution PNG images shown in the HTML report. If missing or
       \code{NA}, the value used is formed from the file name \code{fname} (without the extension) and the suffix
       \code{"_images"}.}
 \item{\code{"pdfs" }}{Directory to contain the PDF images (if such are created). If not missing or \code{NA}, the
       value used is formed from the file name \code{fname} (without the extension) and the suffix \code{"_pdf"}.}
 \item{\code{"high" }}{Directory to contain the high resolution PNG images (if such are created). If missing or
       \code{NA}, the value used is the same as the \code{pngs} directory.}
}
Any other elements, if present, are ignored. Note that these directories are not required to point to different
locations. In particular, if the directories for low and for high resolution images are identical, the
high-resolution image files are assumed to be the ones with suffix \code{"_high_resolution.png"}. See
\code{\link{createReportPlot}} for creating image files.
In order to ensure independence of the operating system, there are strong restrictions on the names of the file and
directories. The name of the report's HTML file can consist of the following symbols only: Latin letters, digits, dot
(\code{.}), dash (\code{-}) and underline (\code{_}). The extension of the report's HTML file must be one of
\code{htm}, \code{html}, \code{xhtml} or \code{xml}. The supporting directories must be given as relative paths;
the restrictions on the path names are identical to the ones for file name. Forward slash (\code{/}) is to be used as
path separator. Path names cannot start or end with a slash. None of the directory names can be
an empty string, use \code{"."} instead. A value in the form \code{"mypath/.html"} for \code{fname} is invalid.
Upon initialization, the report attempts to create or overwrite the specified \code{fname}. If the path to it does
not exist, or if the current process does not have permissions to write to the file, report initialization will fail.
The report object visits each supporting directory (except \code{configuration}) and attempts to create it, unless it
is an existing empty directory. Report initialization will fail if any of the visited directories does not meet the
criteria and could not be created. Hidden files (file names starting with \code{"."} on Unix platforms) are ignored.
Thus, all supporting directories that already exist and contain hidden files only are considered valid.
}
\examples{
\donttest{
report <- createReport("example.html", "Example", init.configuration = TRUE)
}
}
\seealso{
\code{\linkS4class{Report}} for functions adding contents to an HTML report
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subSegments.R
\name{load.region.subsegment.annotation}
\alias{load.region.subsegment.annotation}
\title{load.region.subsegment.annotation}
\usage{
load.region.subsegment.annotation(rnb.set, annotation.dir)
}
\arguments{
\item{rnb.set}{The \code{RnBSet object} with subsegments specified in the regions}

\item{annotation.dir}{a directory to load the annotation from. (binary \code{RData} format.)}
}
\value{
invisible \code{TRUE}
}
\description{
For the region annotation of a given \code{RnBSet} object. Subdivide each region into subsegments by
hierarchical clustering on the site distances in a particular region and then splitting the region into subregions consisting
of these site clusters. The number of clusters is determined in such way that the mean number of sites per cluster
is given by the \code{ns} parameter.
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.R
\name{rnb.run.xml}
\alias{rnb.run.xml}
\title{rnb.run.xml}
\usage{
rnb.run.xml(fname, create.r.command = FALSE)
}
\arguments{
\item{fname}{XML configuration file to read.}

\item{create.r.command}{Flag indicating if the R command(s) that correspond to the given XML configuration should be
generated. If this is set to \code{TRUE}, a file named \code{"analysis.R"} is created in the
reports directory.}
}
\value{
Invisibly, the loaded, normalized and/or possibly filtered dataset as an object of type inheriting
        \code{\linkS4class{RnBSet}}.
}
\description{
Starts the analysis pipeline from an XML configuration file. This function uses the \pkg{XML} package to parse the
configuration file.
}
\details{
Two values are required to be specified (as tags) in the configuration file - \code{data.source} and
\code{dir.reports}. They define the input and output directory, respectively. In addition, the file may define
analysis option values. The vignette \emph{Comprehensive DNA Methylation Analysis with RnBeads} describes in details
the syntax of the XML configuration file.

The sample annotation table must be stored as a file in \code{data.source}. For more information about the required
parameters, see the documentation of \code{\link{rnb.run.analysis}}, which is called by this function.
}
\seealso{
\code{\link{rnb.run.analysis}} for starting an analysis pipeline
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xenomix.R
\name{rnb.run_xeno}
\alias{rnb.run_xeno}
\title{run_xeno}
\usage{
rnb.run_xeno(idat_path, n_cores = 1)
}
\arguments{
\item{idat_path}{Path to the idat file for which the mouse fraction is to be computed}

\item{n_cores}{The number of cores to be used}
}
\value{
a \code{data.frame} with the estimated mouse content
}
\description{
Run Xeno Mouse Fraction Analysis
}
\author{
Nima Esmaeelpour
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataExport.R
\name{rnb.execute.export.csv}
\alias{rnb.execute.export.csv}
\title{rnb.execute.export.csv}
\usage{
rnb.execute.export.csv(
  rnb.set,
  output.location,
  region.types = rnb.getOption("export.types")
)
}
\arguments{
\item{rnb.set}{Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.}

\item{output.location}{\code{character} or \code{\linkS4class{Report}} specifying the output directory. If this is a
report, the output directory is set to be a subdirectory named \code{csv} of the report's data
directory. Set this parameter to the empty string (\code{""}) or \code{NA} to use the current
working directory. If the given path does not exist, this function attempts to create it.}

\item{region.types}{\code{character} vector indicating region types to be exported.}
}
\value{
\code{character} vector containing the names of the files to which data were exported; prepended by
        \code{output.location}. In case a certain region type could not be exported (see the \emph{Details} section),
        the corresponding element of this vector is \code{NA}.
}
\description{
Exports (selected) methylation tables of the given dataset to comma-separated value files.
}
\details{
The names of the generated output files are formed by the prefix \code{"betas_"}, followed by a number between
1 and \code{length(region.types)}. The extension is \code{.csv} or \code{.csv.gz}, depending on the value of the
\pkg{RnBeads} option \code{"gz.large.files"}. Any such files that already exist in the output directory, are
overwritten.

There are several reasons why a certain output file cannot be (fully) generated. Examples for failures are listed
below:
\itemize{
  \item{}{The corresponding region type is invalid.}
  \item{}{The corresponding region type is not supported by the dataset. If the type is loaded in \pkg{RnBeads},
          use the \code{\link[=summarize.regions,RnBSet-method]{summarize.regions}} method prior to calling this function,
          in order to include the support of this region type in the dataset.}
  \item{}{Due to security restrictions, the creation of files in the output directory is not allowed.}
  \item{}{A file or directory with the same name exists and cannot be overwritten.}
  \item{}{The disk is full or the user quota is exceeded.}
}
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
rnb.execute.export.csv(rnb.set.example, "", summarized.regions(rnb.set.example))
}
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/exportGEO.R
\name{prepareSOFTfileForGEO}
\alias{prepareSOFTfileForGEO}
\title{prepareSOFTfileForGEO}
\usage{
prepareSOFTfileForGEO(
  rnb.set,
  filename,
  sample.source.col = NULL,
  sample.description.col = NULL,
  sample.title.col = NULL,
  export.cols = seq(ncol(pheno(rnb.set))),
  rnb.set.raw = NULL,
  sample.extra.info = NULL,
  series.info = NULL
)
}
\arguments{
\item{rnb.set}{Object inheriting from class \code{RnBeadSet}
with \code{"GSE"}.}

\item{filename}{Absolute path or a name of a SOFT file to be generated}

\item{sample.source.col}{\code{integer} singleton specifying a column in the \code{pheno} slot of \code{rnb.set} 
containing information which will be written into the field \code{Sample_source_name_ch1}
of each sample record}

\item{sample.description.col}{\code{integer} singleton specifying a column in the \code{pheno} slot of \code{rnb.set} 
containing information which will be written into the field \code{Sample_desription}
of each sample record}

\item{sample.title.col}{\code{integer} singleton specifying a column in the \code{pheno} slot of \code{rnb.set} 
containing information which will be written into the field \code{Sample_title}
of each sample record. If \code{NULL}, the result of \code{samples(rnb.set)} will be used}

\item{export.cols}{\code{integer} vector specifying columns in the \code{pheno} slot of \code{rnb.set} 
containing information which will be written into the fields \code{Sample_characteristics_ch1}
of each sample record}

\item{rnb.set.raw}{Object inheriting from class \code{RnBeadSet}}

\item{sample.extra.info}{Optionally, a \code{list} with elements to be written to all series record. Elements should be character
singletons named with valid SOFT labels of a SAMPLE section, e.g.\code{Sample_extract_protocol}, 
\code{Sample_hyb_protocol}, \code{Sample_label_protocol_ch1}, \code{Sample_data_processing}, 
\code{Sample_contact_name}, \code{Sample_contact_email} etc.}

\item{series.info}{A \code{list} with elements to be written to the series record. Elements should be character
singletons named \code{SERIES} (contains a valid GSE identifier for updating an existing series)
\code{Series_title}, \code{Series_summary}, \code{Series_type}, \code{Series_overall_design},
\code{Series_contributor}, \code{Series_sample_id}}
}
\value{
\code{TRUE} on success.
}
\description{
Starting from an \code{RnBeadSet} object generates a batch submission file for Gene Expression Omnibus series in SOFT format
}
\details{
The code was largely adapted from a similar function in package \code{lumi} which is due to Pan Du.
}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logger.R
\name{logger.validate.file}
\alias{logger.validate.file}
\title{logger.validate.file}
\usage{
logger.validate.file(file, is.file = TRUE, terminate = TRUE)
}
\arguments{
\item{file}{Name of file or directory to validate.}

\item{is.file}{Flag indicating if the given name must denote an existing file. If this is \code{FALSE}, the given
name must denote a directory. Set this to \code{NA} if both types are an acceptable scenario.}

\item{terminate}{Flag indicating if the execution is to be terminated in case the validation fails. This parameter
determines if an error message (\code{terminate} is \code{TRUE}) or a warning message
(\code{terminate} is \code{FALSE}) is to be sent to the log when the specified file or directory
does not exist, is not of the accepted type or is not accessible.}
}
\value{
Whether the validation succeeded or not, invisibly. Note that when \code{terminate} is \code{TRUE} and the
        validation fails, the R session is closed and thus no value is returned.
}
\description{
Validates the specified file or directory exists. Prints an error or a warning message to the log if it does not
exist, it is not of the accepted type or is not accessible.
}
\examples{
\donttest{
if (!logger.isinitialized())
  logger.start(fname = NA)
# Validate the current working directory exists
logger.validate.file(getwd(), FALSE)
}
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computeCluster.R
\name{combine.diffMeth.objs}
\alias{combine.diffMeth.objs}
\title{combine.diffMeth.objs}
\usage{
combine.diffMeth.objs(obj.list)
}
\arguments{
\item{obj.list}{a list containing outputs from \code{rnb.run.differential}}
}
\description{
combine differential methylation objects (output from \code{rnb.run.differential}).
To be more precise, the \code{diffmeth} and \code{dm.go.enrich} are merged.
individual objects that are merged are assumed to belong to the same analysis
and vary only in their indexing of region types and comparisons
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plottingUtils.R
\name{create.densityScatter}
\alias{create.densityScatter}
\title{create.densityScatter}
\usage{
create.densityScatter(
  df2p,
  is.special = NULL,
  dens.subsample = FALSE,
  dens.special = TRUE,
  sparse.points = 0.01,
  dens.n = 100,
  add.text.cor = FALSE
)
}
\arguments{
\item{df2p}{\code{data.frame} to be plotted. Only the fist two columns are taken into account as
x and y coordinates respectively}

\item{is.special}{boolean vector of length equal to the number of rows in \code{df2p}. Specifies
which points should be highlighed seperately in a different color}

\item{dens.subsample}{if the number of points exceeds this number, subsample the number of points for the
density estimation to that number. Any non-numeric value disables subsampling.}

\item{dens.special}{Flag indicating whether the points of the special population should be colored
according to their density}

\item{sparse.points}{Either percentage (\code{<=1,>=0}) or the absolute number 
 of points in the sparsely populated area that should be drawn seperately. A value of 0 means that these points
will not be drawn.}

\item{dens.n}{passed on to \code{ggplot2::stat_density2d}: argument: \code{n}}

\item{add.text.cor}{flag indicating whether a text token with the correlation coefficient should be included in the lower
right corner of the plot}
}
\value{
\code{ggplot} object
}
\description{
Creates a density scatterplot highlighting points in sparsely populated plot regions
as well as points marked as special in a seperate color
}
\examples{
\donttest{
d <- data.frame(x=rnorm(1000),y=rnorm(1000))
s <- rep(FALSE,1000)
s[sample(1:length(s),100)] <- TRUE
create.densityScatter(d,s)
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/profiles.R
\name{rnb.plot.betadistribution.probeCategories}
\alias{rnb.plot.betadistribution.probeCategories}
\title{rnb.plot.betadistribution.probeCategories}
\usage{
rnb.plot.betadistribution.probeCategories(
  beta.matrix,
  probe.cat,
  annotation = "Group",
  color.legend = NULL,
  log.str = NULL,
  points.per.group = rnb.getOption("distribution.subsample")
)
}
\arguments{
\item{beta.matrix}{Beta values in the form of a non-empty \code{matrix} of type \code{double}. Rows in this matrix
must correspond to Infinium probes, and columns - to samples.}

\item{probe.cat}{\code{factor} vector of length \code{nrow(beta.matrix)} corresponding to the
probe categories.}

\item{annotation}{Name of the annotation being visualized, in the form of a \code{character} vector of length 1.}

\item{color.legend}{Color legend to use in the form of a \code{character} vector with element names. The values in
this vector should encode colors. All values in \code{probe.cat} must be present in the names of
this color legend. If this parameter is \code{NULL}, a default color legend is be constructed.}

\item{log.str}{string specifying more details for the log file}

\item{points.per.group}{the targeted number of points per group. Set this to a value < 1 to disable subsampling. More
information in the Details section of \code{\link{rnb.step.betadistribution}}}
}
\value{
The plot as a \code{ggplot2} object.
}
\description{
plot beta value distrubions given probe categories
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
meth.mat <- meth(rnb.set.example)
probe.types <- annotation(rnb.set.example)[, "Design"]
rnb.plot.betadistribution.probeCategories(meth.mat,probe.types,annotation="Infinium probe type")
}
}
\seealso{
rnb.plot.betadistribution.sampleGroups
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{rnb.call.destructor}
\alias{rnb.call.destructor}
\title{rnb.call.destructor}
\usage{
rnb.call.destructor(object, ...)
}
\arguments{
\item{object}{object to be destroyed}

\item{...}{further arguments to the method \code{\link[=destroy,RnBSet-method]{destroy}}}
}
\value{
invisible \code{TRUE}
}
\description{
calls the destructor of an RnBSet, RnBeadSet or RnBeadRawSet object
conditionally on whether the \code{enforce.destroy.disk.dumps} option is enabled.
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.R
\name{rnb.run.analysis}
\alias{rnb.run.analysis}
\title{RnBeads Analysis Pipeline}
\usage{
rnb.run.analysis(
  dir.reports,
  data.source = NULL,
  sample.sheet = NULL,
  data.dir = NULL,
  GS.report = NULL,
  GEO.acc = NULL,
  data.type = rnb.getOption("import.default.data.type"),
  initialize.reports = TRUE,
  build.index = TRUE,
  save.rdata = TRUE
)
}
\arguments{
\item{dir.reports}{Directory to host the generated report files. This must be a \code{character} of length one
that specifies either a non-existent path (when \code{initialize.reports} is \code{TRUE}),
or an existing directory (when \code{initialize.reports} is \code{FALSE}). In the latter
case, a call to \code{\link{rnb.initialize.reports}} might be required before viewing the
reports.}

\item{data.source}{Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}, or a
\code{character} vector specifying the location of the data items on disk. The expected
length of the vector differs for different values of \code{data.type};
see \code{\link{rnb.execute.import}} for a more detailed description. If set, the parameters
   \code{sample.sheet}, \code{data.dir}, \code{GS.report}, \code{GEO.acc} will be ignored.}

\item{sample.sheet}{A spreadsheet-like text file with sample annotations. The required columns are different
for different values of \code{data.type}.}

\item{data.dir}{For \code{data.type \%in\% c("data.dir", "idat.dir", "bed.dir")} a character singleton
specifying the location of the directory with data files. The directory should have zero
depth, i.e. should contain no subdirectories.}

\item{GS.report}{GenomeStudio report file. \code{data.type} will be automatically set to \code{"GS.report"}.}

\item{GEO.acc}{Gene Expression Omnibus accession of the data series with HumanMethylation450 data.
\code{data.type} will be automatically set to \code{"GEO"}.}

\item{data.type}{\code{character} vector of length one specifying the type of the input data.
The value must be one of \code{"data.dir"}, \code{"idat.dir"}, \code{"GS.report"},
\code{"GEO"} or \code{"rnb.set"}. See \code{\link{rnb.execute.import}} for a more detailed
description.}

\item{initialize.reports}{Flag indicating if the report's directory must be initialized. If this parameter is set to
\code{TRUE}, this function attempts to create the path specified by \code{dir.reports}.
Otherwise, \code{dir.reports} is expected to signify an existing directory.}

\item{build.index}{Flag indicating if a report index file (named \code{"index.html"}) should be created after
all modules in the pipeline complete their analyses. If this is \code{TRUE}, the index file
is also displayed using the function \code{\link{rnb.show.report}}.}

\item{save.rdata}{Flag indicating whether important data objects (the filtered and unfiltered RnBSets,
differential methylation) should be saved to an RData file in the reports folder.}
}
\value{
Invisibly, the loaded, normalized and/or possibly filtered dataset as an object of type inheriting
        \code{\linkS4class{RnBSet}}.
}
\description{
Starts the \pkg{RnBeads} analysis pipeline on the given dataset. It loads the dataset if it is specified as a
location.
}
\seealso{
\link[=rnb.run.import]{RnBeads modules}
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBDiffMeth-class.R
\docType{methods}
\name{initialize,RnBDiffMeth-method}
\alias{initialize,RnBDiffMeth-method}
\title{initialize.RnBDiffMeth}
\usage{
\S4method{initialize}{RnBDiffMeth}(
  .Object,
  site.test.method = rnb.getOption("differential.site.test.method"),
  variability.method = rnb.getOption("differential.variability.method"),
  covg.thres = rnb.getOption("filtering.coverage.threshold"),
  disk.dump = FALSE,
  disk.path = NULL
)
}
\arguments{
\item{.Object}{New instance of \code{RnBDiffMeth}.}

\item{site.test.method}{method which was applied to obtain the site-level p-values.}

\item{variability.method}{method to be used to calculate differentially variable sites. Has to be one of: `diffVar' or 
`iEVORA'.}

\item{covg.thres}{coverage threshold. Important for certain columns of the differential methylation tables.
See \code{computeDiffMeth.bin.site} and \code{computeDiffMeth.bin.region} for details.}

\item{disk.dump}{Flag indicating whether the tables should be stored on disk rather than in the main memory}

\item{disk.path}{Path on the disk for DMTs.Only meaningful if \code{disk.dump} is \code{TRUE}}
}
\description{
Initialize an RnBDiffMeth object
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\docType{methods}
\name{sampleCovgApply,RnBSet-method}
\alias{sampleCovgApply,RnBSet-method}
\alias{sampleCovgApply}
\title{sampleCovgApply-methods}
\usage{
\S4method{sampleCovgApply}{RnBSet}(object, fn, type = "sites", ...)
}
\arguments{
\item{object}{object inheriting from \code{\linkS4class{RnBSet}}}

\item{fn}{function to be applied}

\item{type}{\code{character} singleton. Specify "sites" (default) or a region type over which the function is applied}

\item{...}{arguments passed on to the function}
}
\value{
Result analogous to \code{apply(covg(rnbSet, type), 2, FUN=FUN)}
}
\description{
Applies a function over the coverage values for all samples in an \code{RnBSet} using a low memory footprint.
}
\seealso{
\code{\link[=meth,RnBSet-method]{covg}} Retrieving the matrix of coverage values
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBDiffMeth-class.R
\name{save.rnb.diffmeth}
\alias{save.rnb.diffmeth}
\title{save.rnb.diffmeth}
\usage{
save.rnb.diffmeth(object, path)
}
\arguments{
\item{object}{\code{\linkS4class{RnBDiffMeth}} object}

\item{path}{path on the disk to save to.}
}
\description{
save an \code{\linkS4class{RnBDiffMeth}} object to disk
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/controlPlotsBiSeq.R
\name{rnb.plot.coverage.thresholds}
\alias{rnb.plot.coverage.thresholds}
\title{rnb.plot.coverage.thresholds}
\usage{
rnb.plot.coverage.thresholds(rnb.set, min.coverages, fname = NA, ...)
}
\arguments{
\item{rnb.set}{Methylation dataset as an object of type \code{\linkS4class{RnBiseqSet}}.}

\item{min.coverages}{Non-empty \code{integer} vector storing the unique positive cutoff values to be applied for
minimal coverage. Names, if present, are interpreted as colors that must be used to denote the
corresponding values.}

\item{fname}{File name to save the generated plot to. See the \emph{Details} section for restrictions.}

\item{...}{Additional named parameters related to saving the plot to files. These can include:
\code{report}, \code{width}, \code{height}, \code{create.pdf}, \code{low.png} and
\code{high.png}. These parameters are ignored when \code{fname} is \code{NULL} or \code{NA}.}
}
\value{
If \code{fname} is \code{NULL} or \code{NA} (default), the generated plot as an object of type
        \code{ggplot2}; otherwise, the initialized and closed \code{\linkS4class{ReportPlot}} object, invisibly.
}
\description{
Plots the number of remaining CpGs after applying different thresholds for coverage and support.
}
\details{
If \code{fname} is specified, this function calls \code{\link{createReportPlot}} to save the plot to PDF and/or PNG
files. See \link[=createReportPlot]{its documentation} for information on acceptable file names. Additional
parameters - \code{report}, \code{width}, \code{height}, etc. - can also be given. If image width is not specified,
it is set to a value between 4.7 and 9.2 (inches), depending on the number of samples in the dataset. The default
image height is fixed to 7.2.
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataExport.R
\name{rnb.execute.tnt}
\alias{rnb.execute.tnt}
\title{rnb.execute.tnt}
\usage{
rnb.execute.tnt(
  rnb.set,
  out.dir,
  exp.bed = rnb.getOption("export.to.bed"),
  exp.trackhub = rnb.getOption("export.to.trackhub"),
  region.types = rnb.getOption("export.types"),
  ...
)
}
\arguments{
\item{rnb.set}{\code{\linkS4class{RnBSet}} object}

\item{out.dir}{output directory.}

\item{exp.bed}{A character vector indicating which data types should be exported to UCSC. Possible values in the vector are \code{bigBed} and \code{bigWig}.
If \code{NULL}, UCSC export is disabled}

\item{exp.trackhub}{file types which should be exported to a trackhub structure.}

\item{region.types}{a character vector indicating region types to be exported}

\item{...}{Arguments passed to \code{\link{rnb.export.to.trackhub}}}
}
\value{
a list containing information on the export
}
\description{
export RnBSet to various output data formats
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
rnb.execute.tnt(rnb.set.example,tempdir())
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterArchitectureLSF.R
\docType{class}
\name{ClusterArchitectureLSF-class}
\alias{ClusterArchitectureLSF-class}
\title{ClusterArchitectureLSF Class}
\description{
A child class of \code{\linkS4class{ClusterArchitecture}} implementing specifications of IBM LSF architectures.
}
\details{
Follow this template if you want to create your own ClusterArchitecture class.
}
\section{Slots}{

see \code{\linkS4class{ClusterArchitecture}}
}

\section{Methods}{

\describe{
  \item{\code{\link{getSubCmdTokens,ClusterArchitectureLSF-method}}}{Returns a vector of command line tokens corresponding to submitting
  a job with the given command to the cluster}
}
}

\author{
Michael Scherer
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logger.R
\name{logger.argument}
\alias{logger.argument}
\title{logger.argument}
\usage{
logger.argument(
  arg.names,
  full.name,
  arg.type = "character",
  accepted.values = NULL,
  default = NULL,
  arg.list = commandArgs()
)
}
\arguments{
\item{arg.names}{\code{character} vector of acceptable argument names. This function scans the provided
arguments and performs a case insensitive match.}

\item{full.name}{One-element \code{character} vector giving the argument's full name or description. This is
used in a log message in case of an error.}

\item{arg.type}{Variable type of the argument. Must be one of \code{"character"}, \code{"logical"},
\code{"integer"}, \code{"double"}, \code{"numeric"} or \code{"real"}. The last three types are
all synonyms.}

\item{accepted.values}{Vector of accepted values for the argument. This must be of the type given in \code{arg.type}.
Set this to \code{NULL} if there are no restrictions on the argument values.}

\item{default}{Default value for the argument in case it is not specified. Setting this to \code{NULL} makes
the argument required, that is, an error is generated if the argument is not specified. Set
this to \code{NA} if is not a required argument and it shouldn't default to a specific value.
Otherwise, if \code{accepted.values} is provided, this must be one of its elements.}

\item{arg.list}{Vector of arguments provided at the execution of the script. The arguments should be provided
as \emph{name=value} pairs.}
}
\value{
Argument's value, or \code{NULL} if such is not provided.
}
\description{
Reads a command-line argument supplied to a script.
}
\details{
This is convenience function for reading parameters supplied to the script in the form \emph{name = value}.
         It expects that logging is enabled (see \code{\link{rnb.options}}). The function fails if this condition is
         not met.
}
\examples{
\donttest{
n.iterations <- logger.argument("iterations", "number of iterations", "integer",
  accepted.values = 1:100, default = 1L)
logger.close()
}
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ReportPlot-class.R
\docType{class}
\name{ReportPlot-class}
\alias{ReportPlot-class}
\alias{initialize,ReportPlot-method}
\title{ReportPlot Class}
\description{
Information about the files created to store one generated plot in a report. Report plots are initialized using the
function \code{\link{createReportPlot}}.
}
\section{Slots}{

\describe{
  \item{\code{fname}}{Relative file name. It does not include path or extension.}
  \item{\code{width}}{Width of the image in inches.}
  \item{\code{height}}{Height of the image in inches.}
  \item{\code{create.pdf}}{Flag indicating if a PDF image is created.}
  \item{\code{low.png}}{Resolution, in dots per inch, used for the figure image.}
  \item{\code{high.png}}{Resolution, in dots per inch, used for the high-resolution image.}
  \item{\code{dir.pdf}}{Directory that contains the generated PDF file.}
  \item{\code{dir.png.low}}{Directory that contains the generated figure image file.}
  \item{\code{dir.png.high}}{Directory that contains the generated high-resolution image file.}
}
}

\section{Methods and Functions}{

\describe{
  \item{\code{\link{get.files}}}{Gets the list of all files that are planned to be generated,
       or were already generated by the report plot.}
  \item{\code{\link[=off,ReportPlot-method]{off}}}{Copies the figure to a PNG file (if needed) and closes the device
       associated with the report plot.}
}
}

\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bigFf.R
\name{[,BigFfMat,ANY,ANY,ANY-method}
\alias{[,BigFfMat,ANY,ANY,ANY-method}
\title{Extract parts of BigFfMat}
\usage{
\S4method{[}{BigFfMat,ANY,ANY,ANY}(x, i, j, drop = TRUE)
}
\arguments{
\item{x}{BigFfMat object}

\item{i}{row indices (integer, logical, character are allowed)}

\item{j}{column indices (integer, logical, character are allowed)}

\item{drop}{analogous to generic drop}
}
\description{
Extract parts of BigFfMat
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batch.R
\name{rnb.execute.batcheffects}
\alias{rnb.execute.batcheffects}
\title{rnb.execute.batcheffects}
\usage{
rnb.execute.batcheffects(rnb.set, pcoordinates = NULL)
}
\arguments{
\item{rnb.set}{Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.}

\item{pcoordinates}{Coordinates of the samples of \code{rnb.set} in the principal components space, as returned by
\code{\link{rnb.execute.dreduction}}.}
}
\value{
Results of attempted tests for associations in the form of a list with up to three elements:
        \describe{
          \item{\code{"permutations"}}{\code{integer} matrix of index permutations. The number of rows in the matrix
               is \emph{N} - the number of samples in \code{rnb.set}. Every column in this matrix denotes a sample
               permutation; the first column is the sequence 1 to \emph{N}. This element is included only when
               \code{rnb.getOption("exploratory.correlation.permutations")} is non-zero and there are numeric traits
               to be tested.}
          \item{\code{"pc"}}{List of four matrices named \code{"failures"}, \code{"tests"}, \code{"correlations"}
               and \code{"pvalues"}. The rows in each of these matrices correspond to the first several principal
               components, and the columns - to selected traits. This element is not included in the returned list
               when \code{pcoordinates} is \code{NULL}.}
          \item{\code{"traits"}}{List of four square symmetric matrices named \code{"failures"}, \code{"tests"},
               \code{"correlations"} and \code{"pvalues"}, containing information about the performed tests for
               pairwise trait association. This element is included only if two or more traits were tested.}
        }
}
\description{
Performs tests for association between traits and principal components.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
regs <- c("sites", summarized.regions(rnb.set.example))
dreduction <- function(x) rnb.execute.dreduction(rnb.set.example, x)
pcoordinates <- lapply(regs, dreduction)
names(pcoordinates) <- regs
result <- rnb.execute.batcheffects(rnb.set.example, pcoordinates)
}
}
\seealso{
\code{\link{rnb.run.exploratory}} for running the whole exploratory analysis module
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotations.R
\docType{data}
\name{RnBeads.data}
\alias{RnBeads.data}
\title{RnBeads Annotation Tables}
\format{
\code{list} of four elements - \code{"regions"}, \code{"sites"}, \code{"controls"} and \code{"mappings"}.
        These elements are described below.
        \describe{
          \item{\code{"regions"}}{\code{list} of \code{NULL}s; the names of the elements correspond to the built-in
               region annotation tables. Once the default annotations are loaded, the attribute \code{"builtin"} is
               a \code{logical} vector storing, for each region annotation, whether it is the default (built-in) or
               custom.}
          \item{\code{"sites"}}{\code{list} of \code{NULL}s; the names of the elements correspond to the site and
               probe annotation tables.}
          \item{\code{"controls"}}{\code{list} of \code{NULL}s; the names of the elements correspond to the control
               probe annotation tables. The attribute \code{"sites"} is a \code{character} vector pointing to the
               site annotation that encompasses the respective control probes.}
          \item{\code{"mappings"}}{\code{list} of \code{NULL}s; the names of the elements correspond to the built-in
               region annotation tables.}
        }
}
\description{
RnBeads uses sets of annotation tables and mappings (from regions to sites) for each of the supported genomes. The
structures for one assembly are stored in a separate dedicated annotation package. The following annotation packages
are available in Bioconductor:
\describe{
	 \item{\pkg{RnBeads.hg38}}{for \code{"hg38"}}
  \item{\pkg{RnBeads.hg19}}{for \code{"hg19"}}
  \item{\pkg{RnBeads.mm10}}{for \code{"mm10"}}
  \item{\pkg{RnBeads.mm9}}{for \code{"mm9"}}
  \item{\pkg{RnBeads.rn5}}{for \code{"rn5"}}
}
}
\details{
An assembly-specific scaffold is automatically loaded upon initialization of its annotation, that is, by the first
valid call to any of the following functions: \code{\link{rnb.get.chromosomes}},
\code{\link{rnb.get.annotation}}, \code{\link{rnb.set.annotation}}, \code{\link{rnb.get.mapping}},
\code{\link{rnb.annotation.size}}. Adding an annotation amounts to attaching its table(s) and mapping structures to
the scaffold.
}
\author{
Yassen Assenov
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\docType{methods}
\name{mergeSamples,RnBSet-method}
\alias{mergeSamples,RnBSet-method}
\alias{mergeSamples}
\title{mergeSamples}
\usage{
\S4method{mergeSamples}{RnBSet}(object, grp.col)
}
\arguments{
\item{object}{input RnBSet object}

\item{grp.col}{a column name (string) of \code{pheno(rnb.set)} that contains unique identifiers for sample groups/replicates
to be combined}
}
\value{
the modified RnBSet object
}
\description{
Take an RnBSet object and merge methylation and phenotype information given a grouping column in the pheno table
coverage is combined by taking the sum of coverages
pheno is combined by concatenating entries from all samples
}
\details{
combines phenotype information, coverage information and methylation information
methylation is combined by taking the average. Detection p-values are combined using Fisher's method.
For methylation arrays, bead counts are currently not taken into account.
objects of class \code{RnBeadRawSet} are automatically converted to \code{RnBeadSet}.
}
\note{
Requires the packages \pkg{foreach} and \pkg{doParallel}.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
rnb.set.example
rnb.set.merged <- mergeSamples(rnb.set.example,"Cell_Line")
rnb.set.merged
pheno(rnb.set.merged)
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\name{rnb.sample.summary.table}
\alias{rnb.sample.summary.table}
\alias{rnb.sample.summary.table,RnBSet-method}
\title{rnb.sample.summary.table}
\usage{
rnb.sample.summary.table(rnbSet)
}
\arguments{
\item{rnbSet}{\code{\linkS4class{RnBSet}} of interest.}
}
\value{
a summary table (as data.frame) with the following variables for each sample (rows):
\item{sampleName}{Name of the sample}
\item{*_num (* can be 'sites' or a region type)}{Number of sites or regions with coverage in the sample}
\item{*_covgMean (\code{RnBiseqSet} only)}{Mean coverage of sites or regions in the sample}
\item{*_covgMedian (\code{RnBiseqSet} only)}{Median coverage of sites or regions in the sample}
\item{*_covgPerc25 (\code{RnBiseqSet} only)}{25 percentile of coverage of sites or regions in the sample}
\item{*_covgPerc75 (\code{RnBiseqSet} only)}{75 percentile of coverage of sites or regions in the sample}
\item{*_numCovg5,10,30,60 (\code{RnBiseqSet} only)}{Number of sites or regions with coverage greater or equal to 5,10,30,60}
\item{sites_numDPval5em2,1em2,1em3 (\code{RnBeadSet} only)}{Number of sites with a detection p-value smaller than 0.05,0.01,0.001}
\item{**_numSitesMean (** is any region type)}{Mean number of sites in a region}
\item{**_numSitesMedian}{Median number of sites in a region}
\item{**_numSites2,5,10,20}{Number of regions with at least 2,5,10,20 sites with valid methylation measurements}
}
\description{
Creates a sample summary table from an RnBSet object
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
rnb.sample.summary.table(rnb.set.example)
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R, R/RnBeadSet-class.R,
%   R/RnBeadRawSet-class.R
\docType{methods}
\name{destroy,RnBSet-method}
\alias{destroy,RnBSet-method}
\alias{destroy}
\alias{destroy,RnBeadSet-method}
\alias{destroy,RnBeadRawSet-method}
\title{destroy-methods}
\usage{
\S4method{destroy}{RnBSet}(object)

\S4method{destroy}{RnBeadSet}(object)

\S4method{destroy}{RnBeadRawSet}(object)
}
\arguments{
\item{object}{object inheriting from \code{\linkS4class{RnBSet}}}
}
\value{
Nothing of particular interest
}
\description{
Remove tables stored to disk from the file system. Useful for cleaning up disk dumped objects.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterArchitecture.R
\docType{methods}
\name{getSubCmdStr,ClusterArchitecture-method}
\alias{getSubCmdStr,ClusterArchitecture-method}
\alias{getSubCmdStr}
\title{getSubCmdStr-methods}
\usage{
\S4method{getSubCmdStr}{ClusterArchitecture}(object, ...)
}
\arguments{
\item{object}{\code{\linkS4class{ClusterArchitecture}} object}

\item{...}{arguments passed on to \code{\link{getSubCmdTokens,ClusterArchitecture-method}}}
}
\value{
A string containing the submission command
}
\description{
Returns a string for the of command line corresponding to submitting
a job with the given command to the cluster.
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterArchitectureSGE.R
\docType{methods}
\name{initialize,ClusterArchitectureSGE-method}
\alias{initialize,ClusterArchitectureSGE-method}
\title{initialize.ClusterArchitectureSGE}
\usage{
\S4method{initialize}{ClusterArchitectureSGE}(.Object, name = "ClusterArchitectureSGE", ...)
}
\arguments{
\item{.Object}{New instance of \code{ClusterArchitectureSGE}.}

\item{name}{A name or identifier}

\item{...}{arguments passed on to the constructor of \code{\linkS4class{ClusterArchitecture}} (the parent class)}
}
\description{
Initialize an ClusterArchitecture object for a Sun Grid Engine (SGE)
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBDiffMeth-class.R
\docType{methods}
\name{addDiffMethTable,RnBDiffMeth-method}
\alias{addDiffMethTable,RnBDiffMeth-method}
\alias{addDiffMethTable}
\title{addDiffMethTable-methods}
\usage{
\S4method{addDiffMethTable}{RnBDiffMeth}(
  object,
  dmt,
  comparison,
  region.type,
  grp.labs = c("group1", "group2")
)
}
\arguments{
\item{object}{\code{\linkS4class{RnBDiffMeth}} object}

\item{dmt}{Differential methylation table to add}

\item{comparison}{character or index of the comparison of the table to retrieve}

\item{region.type}{character or index of the region type of the table to retrieve}

\item{grp.labs}{character vector of length 2 specifying the names of the groups being compared}
}
\value{
the updated RnBDiffMeth object
}
\description{
Adds a differential methylation table
}
\note{
Caveat: if disk dumping is enabled the resulting object tables will be stored in the initial location of the object.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
dm <- rnb.execute.computeDiffMeth(rnb.set.example,"Sample_Group",c("genes","tiling"))
s.groups <- rnb.sample.groups(rnb.set.example,"Sample_Group")[[1]]
dmt.sites <- computeDiffTab.extended.site(meth(rnb.set.example),s.groups[[1]],s.groups[[2]])
map.regions.to.sites <- regionMapping(rnb.set.example,"promoters")
dmt.promoters <- computeDiffTab.default.region(dmt.sites,map.regions.to.sites)
cmp.name <- get.comparisons(dm)[1]
grp.labs <- get.comparison.grouplabels(dm)[1,]
#add the promoter level differential methylation table
dm.add <- addDiffMethTable(dm,dmt.promoters,cmp.name,"promoters",grp.labs)
get.region.types(dm.add)
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBDiffMeth-class.R
\docType{methods}
\name{get.site.test.method,RnBDiffMeth-method}
\alias{get.site.test.method,RnBDiffMeth-method}
\alias{get.site.test.method}
\title{get.site.test.method-methods}
\usage{
\S4method{get.site.test.method}{RnBDiffMeth}(object)
}
\arguments{
\item{object}{RnBDiffMeth object}
}
\value{
character describing the site test method
}
\description{
Gets the site testing method used to obtain the p-values in the differential methylation tables
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
get.site.test.method(dm)
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBDiffMeth-class.R
\docType{methods}
\name{is.valid,RnBDiffMeth-method}
\alias{is.valid,RnBDiffMeth-method}
\alias{is.valid}
\title{is.valid-methods}
\usage{
\S4method{is.valid}{RnBDiffMeth}(object, verbose = FALSE)
}
\arguments{
\item{object}{\code{\linkS4class{RnBDiffMeth}} object}

\item{verbose}{print more info to the logger}
}
\value{
TRUE iff all differential methylation tables are present and accounted for
}
\description{
Validate an RnBDiffMeth object, ie. verify that all differential methylation tables are specified
and accounted for
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
dm1 <- rnb.execute.computeDiffMeth(rnb.set.example,"Sample_Group",c("genes","tiling"))
dm2 <- rnb.execute.computeDiffMeth(rnb.set.example,c("Sample_Group","Treatment"),"promoters")
dm.join1 <- join.diffMeth(dm1,dm2)
#The following joint object is invalid due to missing region type - comparison combinations
is.valid(dm.join1)
dm3 <- rnb.execute.computeDiffMeth(rnb.set.example,c("Treatment"),c("genes","tiling"))
dm.join2 <- join.diffMeth(dm.join1,dm3)
#After joining the missing information, the new object is valid
is.valid(dm.join2)
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{rnb.show.report}
\alias{rnb.show.report}
\title{rnb.show.report}
\usage{
rnb.show.report(report)
}
\arguments{
\item{report}{\code{\linkS4class{Report}} object to open.}
}
\value{
None (invisible \code{NULL}).
}
\description{
Opens the given HMTL report file in the browser.
}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\docType{methods}
\name{updateMethylationSites,RnBSet-method}
\alias{updateMethylationSites,RnBSet-method}
\alias{updateMethylationSites}
\title{updateMethylationSites-methods}
\usage{
\S4method{updateMethylationSites}{RnBSet}(object, meth.data, verbose = FALSE)
}
\arguments{
\item{object}{Dataset of interest.}

\item{meth.data}{This object has to be a \code{data.frame} of equal dimension than the one already contained in 
\code{object}, containing the methylation info that should be associated with the object.}

\item{verbose}{if \code{TRUE} additional diagnostic output is generated}
}
\value{
The modified dataset.
#'
}
\description{
Replaces the methylation info with the specified data frame.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotations.R
\name{get.cpg.stats}
\alias{get.cpg.stats}
\title{get.cpg.stats}
\usage{
get.cpg.stats(chrom.sequence, starts, ends)
}
\arguments{
\item{chrom.sequence}{Chromosome sequence, usually obtained from the assembly's genome definition. This must be an
object of type \code{MaskedDNAString}.}

\item{starts}{\code{integer} vector of start positions for the regions of interest.}

\item{ends}{\code{integer} vector of end positions for the regions of interest.}
}
\value{
Table of statistics for the regions in the form of a \code{matrix} with the following columns:
        \code{"CpG"} and \code{"GC"}. The columns contain the number of CpG dinucleoties and the number of C and G
        bases in each region.
}
\description{
Computes CpG-related statistics for the specified regions.
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotations.R
\name{rnb.remove.annotation}
\alias{rnb.remove.annotation}
\title{rnb.remove.annotation}
\usage{
rnb.remove.annotation(type, assembly = "hg19")
}
\arguments{
\item{type}{One-element \code{character} vector giving the name of the region annotation.}

\item{assembly}{Genome assembly of interest. See \code{\link{rnb.get.assemblies}} for the list of supported genomes.}
}
\value{
Invisibly, \code{TRUE} if the annotation has been successfully deleted, or \code{FALSE} if the specified
        region type is not supported.
}
\description{
Deletes a region annotation table. Use this function with caution; its operation cannot be undone.
}
\examples{
\donttest{
t.regions <- rnb.get.annotation("tiling")
rnb.remove.annotation("tiling")
}

}
\seealso{
\code{\link{rnb.get.annotation}}, \code{\link{rnb.region.types}}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBDiffMeth-class.R
\docType{methods}
\name{get.region.types,RnBDiffMeth-method}
\alias{get.region.types,RnBDiffMeth-method}
\alias{get.region.types}
\title{get.region.types-methods}
\usage{
\S4method{get.region.types}{RnBDiffMeth}(object)
}
\arguments{
\item{object}{\code{\linkS4class{RnBDiffMeth}} object}
}
\value{
character vector containing region types
}
\description{
Gets all region types represented in the object as character vector
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
get.region.types(dm)
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computeCluster.R
\docType{class}
\name{RnBClusterRun-class}
\alias{RnBClusterRun-class}
\title{RnBClusterRun Class}
\description{
A class for configuring and running RnBeads on a scientific compute cluster.
}
\section{Slots}{

\describe{
  \item{\code{architecture}}{A \code{\linkS4class{ClusterArchitecture}} object managing the settings for a scientific compute cluster}
  \item{\code{modules}}{A vector of pipeline modules} 
  \item{\code{module.res.req}}{Stores the resource requirements for each module. A list containing named vectors for the resources} 
  \item{\code{module.num.cores}}{Stores the number of cores for each module}
}
}

\section{Methods}{

\describe{
  \item{\code{\link{setModuleResourceRequirements,RnBClusterRun,character,character-method}}}{Sets the resource requirements for the different pipeline modules}
  \item{\code{\link{setModuleNumCores,RnBClusterRun,integer,character-method}}}{Sets the number of cores used by the different pipeline modules}
  \item{\code{\link{getModuleNumCores,RnBClusterRun-method}}}{Gets the number of cores used by the different pipeline modules}
  \item{\code{\link{run,RnBClusterRun-method}}}{Submit the pipeline modules to the cluster}
}
}

\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBiseqSet-class.R
\docType{class}
\name{RnBiseqSet-class}
\alias{RnBiseqSet-class}
\alias{RnBiseqSet}
\alias{initialize,RnBiseqSet-method}
\title{RnBiseqSet Class}
\usage{
RnBiseqSet(
  pheno,
  sites,
  meth,
  covg = NULL,
  assembly = "hg19",
  target = "CpG",
  summarize.regions = TRUE,
  region.types = rnb.region.types.for.analysis(assembly),
  useff = rnb.getOption("disk.dump.big.matrices"),
  usebigff = rnb.getOption("disk.dump.bigff"),
  verbose = FALSE
)
}
\arguments{
\item{pheno}{phenotypic data.}

\item{sites}{CpG site definition, as a \code{data.frame}
with 3 variables: chromosome (of type
\code{character}), position (\code{integer}) and
strand (\code{character}, one of \code{"+"}, \code{"-"} or \code{"*"}}

\item{meth}{summarized methylation calls as a \code{matrix} or \code{ff_matrix}}

\item{covg}{read coverage information as a \code{matrix} or \code{ff_matrix}}

\item{assembly}{the genome assembly}

\item{target}{target DNA methylation features (CpG sites)}

\item{summarize.regions}{...}

\item{region.types}{region annotations for which the methylation data should be summarized}

\item{useff}{flag specifying whether the ff functionality should be used}

\item{usebigff}{flag specifying whether the extended ff functionality should be used (large matrix support for ff)}

\item{verbose}{flag specifying whether the diagnostic messages should be written to the 
console or to the RnBeads logger, if the latter is initialized}
}
\value{
an object of class RnBiseqSet
}
\description{
A class for storing the DNA methylation and quality information from bisulfite sequencing experiments
}
\details{
TBA
}
\section{Slots}{

\describe{
  \item{\code{status}}{Normalization status.}
}
}

\section{Methods and Functions}{

\describe{
  \item{\code{\link[BiocGenerics]{combine}}}{Combines two datasets.}
}
}

\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotations.R
\name{rnb.save.annotation}
\alias{rnb.save.annotation}
\title{rnb.save.annotation}
\usage{
rnb.save.annotation(fname, type, assembly = "hg19")
}
\arguments{
\item{fname}{One-element \code{character} vector giving the name of the file to contain the annotation data. If
this file already exists, it will be overwritten.}

\item{type}{One-element \code{character} vector giving the name of the region annotation.}

\item{assembly}{Genome assembly of interest. See \code{\link{rnb.get.assemblies}} for the list of supported genomes.}
}
\value{
\code{TRUE}, invisibly.
}
\description{
Saves the specified region annotation table and its accompanying data structures to a binary file.
}
\details{
This function is used in combination with \code{\link{rnb.load.annotation}} to enable fast reloading of custom region
annotations. If can also be used to save a build-in region annotation (e.g. before overwriting it) but not site or
control probe annotations.
}
\seealso{
\code{\link{rnb.load.annotation}} for loading a saved annotation
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterArchitectureSLURM.R
\docType{methods}
\name{getSubCmdTokens,ClusterArchitectureSLURM-method}
\alias{getSubCmdTokens,ClusterArchitectureSLURM-method}
\title{getSubCmdTokens-methods}
\usage{
\S4method{getSubCmdTokens}{ClusterArchitectureSLURM}(
  object,
  cmd.tokens,
  log,
  job.name = "",
  res.req = character(0),
  depend.jobs = character(0),
  sub.binary = TRUE,
  quote.cmd = TRUE
)
}
\arguments{
\item{object}{\code{\linkS4class{ClusterArchitectureSLURM}} object}

\item{cmd.tokens}{a character vector specifying the executable command that should be wrapped in the cluster submission command}

\item{log}{file name and path of the log file that the submitted job writes to}

\item{job.name}{name of the submitted job}

\item{res.req}{named vector of requested resources. Two options are available: \code{"clock.limit"} and \code{"memory.size"}}

\item{depend.jobs}{character vector containg names or ids of jobs the submitted job will depend on.}

\item{sub.binary}{flag indicating if the command is to be submitted using the \code{"wrap"} option of SLURM}

\item{quote.cmd}{Flag indicating whether the submitted cammed should also be wrapped in quotes}
}
\value{
A character vector containing the submission command tokens
}
\description{
Returns a string for the of command line corresponding to submitting
a job with the given command to the cluster.
}
\details{
For a concrete child class implementation for a SLURM architecture specification see \code{\linkS4class{ClusterArchitectureSLURM}}
}
\examples{
\donttest{
arch <- new("ClusterArchitectureSLURM",
	name="my_slurm_architecture"
)
getSubCmdTokens(arch,c("Rscript","my_great_script.R"),"my_logfile.log")
}
}
\author{
Michael Scherer
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Report-methods.R, R/ReportPlot-methods.R
\docType{methods}
\name{off,Report-method}
\alias{off,Report-method}
\alias{off}
\alias{off,ReportPlot-method}
\alias{off,ReportGgPlot-method}
\title{off-methods}
\usage{
\S4method{off}{Report}(object, ...)

\S4method{off}{ReportPlot}(object)

\S4method{off}{ReportGgPlot}(object, handle.errors = FALSE)
}
\arguments{
\item{object}{Object to be closed.}

\item{...}{additional arguments}

\item{handle.errors}{Flag indicating if the method should attempt to catch and process errors (e.g. I/O errors)
internally. Setting this to \code{TRUE} does not guarantee that the method never stops with an
error.}
}
\value{
The closed object, invisibly.
}
\description{
Performs cleanup and/or other finishing activities and closes the specified device, connection, or document.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computeCluster.R
\docType{methods}
\name{run,RnBClusterRun-method}
\alias{run,RnBClusterRun-method}
\alias{run}
\title{run-methods}
\usage{
\S4method{run}{RnBClusterRun}(
  object,
  analysis.id,
  config.xml,
  split.differential = TRUE,
  dry.run = FALSE,
  long.cmd.thres = 1024L,
  queue = NULL
)
}
\arguments{
\item{object}{\code{\linkS4class{RnBClusterRun}} object}

\item{analysis.id}{analysis id. used for naming submitted jobs and log files}

\item{config.xml}{XML file specifying the analysis options and parameter settings}

\item{split.differential}{flag indicating whether to split the differnetial methylation module
into seperate jobs according to sample annotation column and region type.}

\item{dry.run}{Prevent the actual job submission. Rather only write to a shell script file}

\item{long.cmd.thres}{commands that are longer than this number will be encapsulated in shell scripts
rather than being submitted as direct command}

\item{queue}{The name of the queue the jobs are going to be submitted to}
}
\value{
Nothing of importance
}
\description{
Runs the analysis by submitting jobs for each module to the compute cluster
}
\examples{
\donttest{
#specify the xml file for your analysis
xml.file <- "MY_ANALYSIS_SETTINGS.XML"
#set the cluster architecture specific to your environment
arch <- new("ClusterArchitectureSGE")
rnb.cr <- new("RnBClusterRun",arch)
#set up the cluster so that 32GB of memory are required (SGE resource is called "mem_free")
rnb.cr <- setModuleResourceRequirements(rnb.cr,c(mem_free="32G"),"all")
#set up the cluster to use 4 cores on each node for all modules
rnb.cr <- setModuleNumCores(rnb.cr,4L,"all")
#set up the cluster to use 2 cores for the exploratory analysis module
rnb.cr <- setModuleNumCores(rnb.cr,2L,"exploratory")
#run the actual analysis (remove dry.run=TRUE, to really submit the jobs)
run(rnb.cr, "rnbeads_analysis", xml.file, dry.run=TRUE)
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plottingUtils.R
\name{rnb.plot.dreduction}
\alias{rnb.plot.dreduction}
\title{rnb.plot.dreduction}
\usage{
rnb.plot.dreduction(
  rnb.set,
  plot.type = "pca",
  dimensions = 1:2,
  distance.metric = "euclidean",
  target = "sites",
  point.types = 0L,
  point.colors = 0L,
  legend.space = 2
)
}
\arguments{
\item{rnb.set}{Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}. This dataset
must contain at least four samples.}

\item{plot.type}{Type of plot to be created. This must be one of \code{"pca"} (projection to two principal
components), \code{"mds"} (multidimensional scaling to two dimensions) or \code{"tsne"}
(t-distributed stochastic neighbor embedding to two dimensions). The section
\emph{Details} provides more details on how the dimension reduction techniques are applied.}

\item{dimensions}{Vector of two positive \code{integer} values giving the principle components to be shown in
the horizontal and vertical axis of the plot. This parameter is considered only when
\code{plot.type} is \code{"pca"}.}

\item{distance.metric}{Distance metric to be applied when reducing the dimensionality of the methylation data. This
must be one of \code{"eucledian"} or \code{"manhattan"}. The second metric is not supported by
principal component analysis.}

\item{target}{Site or region type to be used in the dimension reduction technique. This must be either
\code{"sites"} (individual CpGs) or one of the region types summarized in \code{rnb.set}.}

\item{point.types}{Trait, specified as column name or index in the sample annnotation table of \code{rnb.set}, to
be used to define point types in the plot. Setting this parameter to zero (default) or to a
trait that does not define categories results in all samples being displayed as filled
circles. If this parameter specifies a column that can be used as sample identifiers, the plot
displays the samples as identifiers instead of points.}

\item{point.colors}{Trait, specified as column name or index in the sample annnotation table of \code{rnb.set}, to
be used to define sample colors in the plot. Setting this parameter to zero (default) or to a
trait that does not define numerical values or categories results in all samples being
displayed in black.}

\item{legend.space}{Width, in inches, of the space dedicated for legends that will be assigned on the right side
of the plot. This parameter is considered only if legends are actually included, that is, if
sample traits are mapped to point types and/or colors.}
}
\value{
The generated plot as an object of type \code{\link[ggplot2:ggplot]{ggplot}}. The object also contains an
        attribute \code{"info"}, which is a list with the following elements:
        \describe{
          \item{\code{"Target" }}{Targeted sites or regions; the value of the parameter \code{target}.}
          \item{\code{"Technique" }}{Dimension reduction technique applied; one of \code{"PCA"} or \code{"MDS"}.}
          \item{\code{"All" }}{Total number of sites or regions defining the high dimensional methylation space.}
          \item{\code{"Missing" }}{Number of dimensions ignored because they contain (only) missing values.}
          \item{\code{"Selected" }}{Number of dimensions used when applying a dimension reduction technique.}
          \item{\code{"Explained" }}{Value between \code{0} and \code{1} showing the variance explained by the
            selected dimensions, as a fraction of the total variance of all dimensions.}
        }
}
\description{
Creates a dimension reduction plot based on the methylation values of the given dataset.
}
\details{
The analysis option \code{"exploratory.top.dimensions"} controls whether dimension reduction is applied on all
probes, sites or regions available in the given dataset, or only on the most variable ones. In case a trait is mapped
to point types, the shapes to use are taken from the option \code{"points.category"}. Similary, the option
\code{"colors.category"} determines which colors are used when mapping sample categories to color. In cases when
numerical values are mapped to color, the option \code{"colors.3.gradient"} is used. If the set of value contains
both positive and negative numbers, the middle point in the color legend is set to zero. See
\emph{\link[=rnb.options]{RnBeads Options}} for more information on the options mentioned above.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
pdf("PCA.pdf", width = 7.2, height = 5.2)
print(rnb.plot.dreduction(rnb.set.example, point.colors="Sample_Group"))
dev.off()
}
}
\seealso{
\code{\link[=summarized.regions,RnBSet-method]{summarized.regions}} for listing all region types summarized
         in a dataset
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBDiffMeth-class.R
\docType{methods}
\name{save.tables,RnBDiffMeth-method}
\alias{save.tables,RnBDiffMeth-method}
\alias{save.tables}
\title{save.tables-methods}
\usage{
\S4method{save.tables}{RnBDiffMeth}(object, file)
}
\arguments{
\item{object}{\code{\linkS4class{RnBDiffMeth}} object}

\item{file}{path on the disk to save to.}
}
\value{
success
}
\description{
save the disk dumped tables to an ff archive for later reloading
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
pcols <- c("Sample_Group","Treatment")
tdir <- tempfile()
dm <- rnb.execute.computeDiffMeth(rnb.set.example,pcols,disk.dump=TRUE,disk.dump.dir=tdir)
save.tables(dm,tempfile())
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\name{save.rnb.set}
\alias{save.rnb.set}
\title{save.rnb.set}
\usage{
save.rnb.set(object, path, archive = TRUE)
}
\arguments{
\item{object}{\code{RnBSet}-inheriting object.}

\item{path}{the name of the output file (or directory if \code{archive} is \code{FALSE})
without an extension. If only the file name is given the object will be saved
in the current working directory.}

\item{archive}{if \code{TRUE} (default value) the output is a ZIP-file.}
}
\value{
invisibly, the full path to the ZIP file (if \code{archive} is \code{TRUE}),
					  or to the output directory (otherwise)
}
\description{
Consistent saving of an \code{RnBSet} objects with large matrices of type \link{ff}.
}
\details{
The saved object can be reloaded with the \link{load.rnb.set} function.
}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computeCluster.R
\docType{methods}
\name{getModuleNumCores,RnBClusterRun-method}
\alias{getModuleNumCores,RnBClusterRun-method}
\alias{getModuleNumCores}
\title{getModuleNumCores-methods}
\usage{
\S4method{getModuleNumCores}{RnBClusterRun}(object)
}
\arguments{
\item{object}{\code{\linkS4class{RnBClusterRun}} object}
}
\value{
A named vector containing the number of cores for each module
}
\description{
Retrieves the number of cores used by each module
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBDiffMeth-class.R
\docType{methods}
\name{join.diffMeth,RnBDiffMeth,RnBDiffMeth-method}
\alias{join.diffMeth,RnBDiffMeth,RnBDiffMeth-method}
\alias{join.diffMeth}
\title{join.diffMeth-methods}
\usage{
\S4method{join.diffMeth}{RnBDiffMeth,RnBDiffMeth}(obj1, obj2)
}
\arguments{
\item{obj1}{\code{\linkS4class{RnBDiffMeth}} object. Its base properties will be used to create the joint object
this is particularly imported for disk dumped objects as its path will be used and tables from the second
object will be copied there}

\item{obj2}{\code{\linkS4class{RnBDiffMeth}} object}
}
\value{
the merged \code{\linkS4class{RnBDiffMeth}} object
}
\description{
Merges two disjoint RnBDiffMeth objects into one. Disjoint here means, that no differential methylation table is specified in both
objects.
}
\note{
Caveat: if disk dumping is enabled the resulting object tables will be stored in the initial location of the first object to be joined
I.e. deleting the first object will lead to a broken joined object and deleting the joined object will lead to an broken first object.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
dm1 <- rnb.execute.computeDiffMeth(rnb.set.example,"Sample_Group",c("genes","tiling"))
dm2 <- rnb.execute.computeDiffMeth(rnb.set.example,c("Sample_Group","Treatment"),"promoters")
dm.join1 <- join.diffMeth(dm1,dm2)
#The following joint object is invalid due to missing region type - comparison combinations
is.valid(dm.join1)
dm3 <- rnb.execute.computeDiffMeth(rnb.set.example,"Treatment",c("genes","tiling"))
dm.join2 <- join.diffMeth(dm.join1,dm3)
#After joining the missing information, the new object is valid
is.valid(dm.join2)
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plottingUtils.R
\name{deviation.plot.beta}
\alias{deviation.plot.beta}
\title{deviation.plot.beta}
\usage{
deviation.plot.beta(betas, c.values = NULL, c.legend = NULL)
}
\arguments{
\item{betas}{Non-empty numeric \code{matrix} of methylation beta values. Rows in this matrix must denote sites or
regions, and columns - samples. If a locus (row in the matrix) contains missing values only, it is
not included in the plot.}

\item{c.values}{Vector (usually a \code{factor}) storing category or quantitative values for each site or region. The
length of this vector must be equal to \code{nrow(betas)}, the \emph{i}-th element storing the
property values for the \emph{i}-th locus in \code{betas}. Note that this vector's names, if present,
are ignored.}

\item{c.legend}{If \code{c.values} stores categories, this parameter specifies the mapping from property values to
colors. The mapping is in the form of a named \code{character} vector. All values that appear in
\code{c.values} must be present among the names of this vector. The order of the values in this
mapping determines in which order the colors are stacked (when the number of loci is large). If
\code{c.values} denotes a quantitative measure, this parameter is a singleton \code{integer},
specifying the color scheme for visualizing the values. Currently, the only supported values are
\code{2} and \code{3}. See \code{\link{rnb.options}} for more details.}
}
\value{
Methylation variability as a number between \code{0} and \code{1}, invisibly. This number denotes the relative
        area of variation in the generated plot.
}
\description{
Creates a deviation plot based on the methylation beta values of a population.
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{rnb.get.reliability.matrix}
\alias{rnb.get.reliability.matrix}
\title{rnb.get.reliability.matrix}
\usage{
rnb.get.reliability.matrix(rnb.set, row.names = FALSE)
}
\arguments{
\item{rnb.set}{Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.}

\item{row.names}{Flag indicating of row names are to be generated in the result.}
}
\value{
\code{logical} matrix in which every row corresponds to a CpG site or probe and every column - to a patient.
        If the dataset does not contain coverage or detection p-value information, the returned value is \code{NULL}.
}
\description{
Gets a matrix of reliability indications for every measurement in the given dataset.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
rnb.options(identifiers.column = "Sample_ID")
str(rnb.get.reliability.matrix(rnb.set.example))
}
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\docType{methods}
\name{covg,RnBSet-method}
\alias{covg,RnBSet-method}
\alias{covg}
\title{covg-methods}
\usage{
\S4method{covg}{RnBSet}(object, type = "sites", row.names = FALSE, i = NULL, j = NULL)
}
\arguments{
\item{object}{Dataset of interest.}

\item{type}{\code{character} singleton. If \code{sites} DNA methylation information per each available
site is returned. Otherwise should be one of region types for for which the summarized
coverage information is available}

\item{row.names}{Flag indicating of row names are to be generated in the result.}

\item{i}{indices of sites/regions to be retrieved. By default (\code{NULL}), all will be retrieved.}

\item{j}{indices of samples to be retrieved. By default (\code{NULL}), all will be retrieved.}
}
\value{
coverage information available for the dataset in the form of a \code{matrix}.
}
\description{
Extract coverage information from an object of \code{RnBSet} class.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
## per-site beta-value matrix
cvg<-covg(rnb.set.example, row.names=TRUE)
head(cvg)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sesame.R
\name{rnb.execute.pOOBAH}
\alias{rnb.execute.pOOBAH}
\title{rnb.execute.pOOBAH}
\usage{
rnb.execute.pOOBAH(
  raw.set,
  anno.table = NULL,
  pval.thresh = 0.05,
  verbose = FALSE
)
}
\arguments{
\item{raw.set}{Methylation dataset as an instance of \code{RnBeadRawSet}.}

\item{anno.table}{Annotation for \code{raw.set}.}

\item{pval.thresh}{Computed detection p-values above this threshold are masked. Default value is 0.05.}

\item{verbose}{If set to true, a short information is printed on how many probes are masked by the method.}
}
\value{
Returns a modified \code{RnBeadRawSet}, in which signal intensities are masked, if their computed p-value  
        was greater than \code{pval.thresh}. Note, in datasets with several samples, signal intensities of a specific probe 
        might be masked in sample A, but not in sample B, as \emph{pOOBAH} is applied separately to each sample.
        For example: the signal intensities of probe cg24488772 might be masked in sample 1, but not in sample 12.
}
\description{
Probe signal intensities are masked based on their out-of-band signal intensities to counter hybridization failure.
}
\details{
rnb.execute.pOOBAH is used to apply the method \emph{pOOBAH} (P-value with OOB probes for Array Hybridization), 
which was conceived by Zhou, Triche, Laird and Shen to mask probes associated with hybridization failures. 
pOOBAH has been implemented in the R-package \emph{"sesame"}, a dependency needed for this function (see Zhou et al, 2018 and the respective Bioconductor/github pages). 
\emph{pOOBAH} computes the detection p-values by constructing 2 empirical cumulative density functions (eCDFs) based 
on the out-of-band signal intensities of the red and the green channel, respectively, to detect hybridization failures. 
The (in-band) green and red channel signal intensities of the probes are passed to the eCDFs and the probes with a 
p-value higher than the given threshold (\code{pval.thresh}) are masked, as they are considered background. 
\emph{pOOBAH} is applied \emph{separately to each sample}. 
Hybridization failures might occur due to somatic or germline deletions. 
In addition, unreliable low-intensity probes might also be masked.
}
\examples{
library(RnBeads.hg19) 
data(small.example.object)


}
\author{
\emph{pOOBAH} method: Wanding Zhou. Adapted by Nathan Steenbuck.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CNV.R
\name{rnb.step.cnv}
\alias{rnb.step.cnv}
\title{rnb.step.cnv}
\usage{
rnb.step.cnv(rnb.set, report)
}
\arguments{
\item{rnb.set}{An object of type \code{\linkS4class{RnBeadRawSet}}}

\item{report}{Report on quality control to contain the generated sections. This must be an object of type
\code{\linkS4class{Report}}.}
}
\value{
The modified report.
}
\description{
Performs copy number calling from the Infinium intenstity data and adds the results to the report
}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differentialVariability.R
\name{get.table.ids}
\alias{get.table.ids}
\title{Returns the colum names of the differential variability table.}
\usage{
get.table.ids(includeCovg = FALSE)
}
\arguments{
\item{includeCovg}{Flag indicating if dataset contains coverage information}
}
\value{
Column names of the differential variability table
}
\description{
Returns the colum names of the differential variability table.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotations.R
\name{rnb.annotation.size}
\alias{rnb.annotation.size}
\title{rnb.annotation.size}
\usage{
rnb.annotation.size(type = "CpG", assembly = "hg19")
}
\arguments{
\item{type}{Name of annotation. Control probe annotations are not accepted.}

\item{assembly}{Genome assembly of interest. See \code{\link{rnb.get.assemblies}} for the list of supported genomes.}
}
\value{
\code{integer} vector showing the number of elements the specified annotation contains per chromosome. The
        names of the vector are the names of \code{\link{rnb.get.chromosomes}} for the given genome assembly.
        Chromosomes that are not covered by the annotation have their respective value set to \code{0} (zero).
}
\description{
Gets the size, in number of genomic elements, of the specified annotation.
}
\examples{
\donttest{
library(RnBeads.hg19)
rnb.annotation.size("probes450")
}
}
\seealso{
\code{\link{rnb.region.types}} for a list of supported region annotations
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subSegments.R
\name{addRegionSubsegments}
\alias{addRegionSubsegments}
\title{addRegionSubsegments}
\usage{
addRegionSubsegments(
  rnb.set,
  annotation.dir,
  region.types = NULL,
  add.region.types.to.options = FALSE,
  ns = 10
)
}
\arguments{
\item{rnb.set}{an \code{RnBSet object}}

\item{annotation.dir}{a directory to save the annotation to for later reloading. (binary \code{RData} format.)}

\item{region.types}{the region types to which subsegmentation should be applied. Must be a non-empty
subset of \code{summarized.regions(rnb.set)}. Defaults (\code{NULL}) to all region types in
\code{rnb.set}}

\item{add.region.types.to.options}{Flag indicating whether to add the newly created subregions to the package's
\code{region.types} option}

\item{ns}{the mean number of sites per cluster.}
}
\value{
the modified \code{RnBSet} object
}
\description{
For the region annotation of a given \code{RnBSet} object. Subdivide each region into subsegments by
hierarchical clustering on the site distances in a particular region and then splitting the region into subregions consisting
of these site clusters. The number of clusters is determined in such way that the mean number of sites per cluster
is given by the \code{ns} parameter.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
rnb.set.mod <- addRegionSubsegments(rnb.set.example,tempdir(),region.types=c("tiling","genes"))
summary(meth(rnb.set.mod,type="tiling.subsegments"))
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plottingUtils.R
\name{rnb.color.legends}
\alias{rnb.color.legends}
\title{rnb.color.legends}
\usage{
rnb.color.legends(
  report,
  legends,
  fprefix = ifelse(is.character(legends), "legend", "legend_"),
  description = "Color legend.",
  setting.names = NULL,
  size.factor = 3
)
}
\arguments{
\item{report}{Report to contain the legend figure. This must be an object of type \code{\linkS4class{Report}}.}

\item{legends}{Color legend in the form of a non-empty \code{character} vector. Element names denote legend
labels, and the elements themselves specify colors. This parameter can also be a \code{list} of
color legends. Special restrictions apply to the names of the list elements, see \emph{Details}.}

\item{fprefix}{File name or prefix for the plot files.}

\item{description}{Text of the figure description. See the correponding parameter in
\code{\link{rnb.add.figure}} for more details.}

\item{setting.names}{One-element list containing a plot file descriptor, when \code{legends} is a list. See the
corresponding parameter in \code{\link{rnb.add.figure}} for more details. If this is set to
\code{NULL} (default), the list is automatically created using \code{names(legends)} (when
\code{legends} is a list), or as an empty list (when \code{legends} is a vector).}

\item{size.factor}{Relative size, in inches of the plots. Legends are displayed in columns of up to 10 items; each
column is effectively a square with the specified size.}
}
\value{
The modified report.
}
\description{
Creates a figure in the given report that contains one or more color legends.
}
\details{
In case \code{legends} specifies multiple legends in the form of a list, \code{names(legends)} are appended
         to \code{fprefix} to generate file names. In order to ensure independence of the operating system, there are
         strong restrictions on these names. They can consist of the following symbols only: Latin letters, digits,
         dot (\code{.}), dash (\code{-}) and underline (\code{_}).
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plottingUtils.R
\name{rnb.message.plot}
\alias{rnb.message.plot}
\title{rnb.message.plot}
\usage{
rnb.message.plot(txt)
}
\arguments{
\item{txt}{Text to be plotted.}
}
\value{
The newly initialized \code{ggplot} instance.
}
\description{
Creates a plot, using \pkg{ggplot2}, with a single text message.
}
\examples{
\donttest{
x11(width = 5, height = 5)
rnb.message.plot("Missing data")
}
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clustering.R
\name{rnb.execute.clustering}
\alias{rnb.execute.clustering}
\title{rnb.execute.clustering}
\usage{
rnb.execute.clustering(rnb.set, region.type = "sites")
}
\arguments{
\item{rnb.set}{Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.}

\item{region.type}{the clustering is performed on methylation levels from regions of that type. 
see \code{\link{rnb.region.types}} for possible values.}
}
\value{
List of clustering results, whereby each element is an object of type \code{\linkS4class{RnBeadClustering}}.
        In case clustering cannot be performed, the return value is \code{NULL}. Reasons for a failure include, among
        others, the case when \code{rnb.set} contains less than 3 samples, or undefined distances between a pair of
        samples due to (too many) missing values in the respective methylation matrix.
}
\description{
Performs hierarchical clustering on the samples of the given dataset using multiple distance metrics and
agglomeration methods for a single given region type.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
results <- rnb.execute.clustering(rnb.set.example, "promoters")
# List applied dissimilarity metrics
sapply(results, slot, "dissimilarity")
# List applied clustering algorithms
str(lapply(results, slot, "algorithm"))
}
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotations.R
\name{rnb.get.annotation}
\alias{rnb.get.annotation}
\title{rnb.get.annotation}
\usage{
rnb.get.annotation(type = "CpG", assembly = "hg19")
}
\arguments{
\item{type}{Name of annotation.}

\item{assembly}{Genome assembly of interest. See \code{\link{rnb.get.assemblies}} for the list of supported genomes.}
}
\value{
Probe, site or region annotation table. If the specified type refers to control probes, the returned value is
        a \code{data.frame} listing all respective control probes. Otherwise, this function returns an object of type
        \code{\link{GRangesList}} - a list of consistent \code{\link{GRanges}} objects, one per chromosome.
}
\description{
Extracts the requested annotation for the given genome.
}
\details{
When the returned value is of type \code{GRangesList}, it defines the genomic positions of the requested sites,
probes or regions. Identifiers, if present, can be obtained using the \code{names} method. Strand information is also
included when applicable. Any additional annotation is stored as metadata in the respective \code{GRanges} objects.
}
\examples{
\donttest{
rnb.get.annotation("promoters")
}
}
\seealso{
\code{\link{rnb.set.annotation}} for adding annotation;
  \code{\link{rnb.region.types}} for all loaded region types in a genome assembly
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.R
\name{rnb.xml2options}
\alias{rnb.xml2options}
\title{rnb.xml2options}
\usage{
rnb.xml2options(fname, return.full.structure = FALSE)
}
\arguments{
\item{fname}{File name containing the XML analysis option values. The name of the root node in this document must be
\code{"rnb.xml"}.}

\item{return.full.structure}{if enabled, return the full structure instead of just the option list}
}
\value{
List of two sublists - \code{"analysis.params"} and \code{"options"}, storing the specified analysis
        parameters and previous values of the RnBeads options, respectively.
}
\description{
Parses and partially validates parameters and RnBeads options from an XML tree.
}
\examples{
\donttest{
fname <- paste0("extdata/optionProfiles/",profile,".xml")
rnb.xml2options(system.file(fname,package="RnBeads"))
}
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{rnb.write.table}
\alias{rnb.write.table}
\title{rnb.write.table}
\usage{
rnb.write.table(tt, fname, fpath = "", format = "csv", gz = FALSE, ...)
}
\arguments{
\item{tt}{Table to be written to file, usually in the form of a \code{matrix} or \code{data.frame}.}

\item{fname}{Target file name. If this file already exists, it will be overwritten.}

\item{fpath}{Target file path. If "" (default value), \code{fname} is assumed to contain the absolute path.}

\item{format}{Target format; one of \code{"csv"}, \code{"tab"} or \code{"txt"}, denoting comma-separated,
tab-separated and default text format, respectively. The last format allows for a user-specified
delimiter through an additional parameter \code{sep}. See the documentation of
\code{\link{write.table}} for more details.}

\item{gz}{Flag indicating whether the file should be zipped in \code{gz} format.}

\item{...}{Any additional arguments to be passed on to \code{write.table} or \code{utils::write.csv}.}
}
\value{
The (possibly updated) target file name, invisibly. If \code{gz} is \code{TRUE}, the string \code{".gz"} will
        be appended to \code{fname}.
}
\description{
Writes a table to a file. Different formats and compression options are available.
}
\examples{
\donttest{
data(mtcars)
rnb.write.table(mtcars,tempfile(pattern="cars",fileext=".csv"))
}
}
\seealso{
\code{\link{write.table}}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\docType{methods}
\name{mval,RnBSet-method}
\alias{mval,RnBSet-method}
\alias{mval}
\title{mval-methods}
\usage{
\S4method{mval}{RnBSet}(object, type = "sites", row.names = FALSE, epsilon = 0)
}
\arguments{
\item{object}{dataset of interest.}

\item{type}{\code{character} singleton. If this is set to \code{"sites"} (default), DNA methylation information
for each available site is returned. Otherwise, this should be one of region types for for which
summarized DNA methylation information is computed in the given dataset.}

\item{row.names}{Flag indicating of row names are to be generated in the result.}

\item{epsilon}{Threshold of beta values to use when adjusting for potential M values close to +infinity or
-infinity. See \code{\link{rnb.beta2mval}} for more details.}
}
\value{
\code{matrix} with methylation M values.
}
\description{
Extracts DNA methylation information (M values) for a specified set of genomic features.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
## per-site M-value matrix
mm<-mval(rnb.set.example, row.names=TRUE)
head(mm)
## M-values for each covered gene
gmm<-mval(rnb.set.example, type="gene", row.names=TRUE)
head(gmm)
}
}
\seealso{
\code{\link[=meth,RnBSet-method]{meth}} for extracting methylation beta values
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/controlPlots.R
\name{rnb.plot.sentrix.distributions}
\alias{rnb.plot.sentrix.distributions}
\title{rnb.plot.sentrix.distributions}
\usage{
rnb.plot.sentrix.distributions(rnb.set, fprefix = "sentrix_whisker", ...)
}
\arguments{
\item{rnb.set}{HumanMethylation450K dataset as an object of type \code{\linkS4class{RnBeadSet}}.}

\item{fprefix}{File name prefix to be used in the generated plots. In order to ensure independence of the operating
system, there are strong restrictions on the name of the file. See the documentation of
\code{\link{createReportPlot}} for more information.}

\item{...}{Other arguments passed to \code{\link{createReportPlot}}. These can include the named parameters
\code{report}, \code{width}, \code{height}, and others.}
}
\value{
Point-and-whisker plot (an instance of \code{\linkS4class{ReportPlot}}), or a list of such plots - one per
        slide. If the provided dataset does not contain valid Sentrix ID and position information (or is an empty
        dataset), this method returns \code{NULL}.
}
\description{
Creates one or more point-and-whisker plots showing beta value distributions at Sentrix positions.
}
\details{
If no additional parameters are specified, this function creates one PDF and one low-resolution PNG file for every
generated plot.
}
\seealso{
\code{\link{rnb.plot.sentrix.distribution}} for creating a single plot for a specified slide number
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBDiffMeth-class.R
\docType{class}
\name{RnBDiffMeth-class}
\alias{RnBDiffMeth-class}
\title{RnBDiffMeth Class}
\description{
A class for storing differential methylation data.
}
\details{
Contains differential methylation tables (DMT) for multiple comparisons and region types. DMTs can be stored in memory
as R objects or on disk
}
\section{Slots}{

\describe{
  \item{\code{sites}}{List of differential methylation tables on site level (see \code{computeDiffMeth.bin.site} for details).
					Indexed by comparison.}
  \item{\code{regions}}{List of lists of differential methylation tables on region levels (see \code{computeDiffMeth.bin.region} for details).
					Indexed by region type on the top level and comparison on the lower level.}
  \item{\code{comparisons}}{character vector of all comparisons stored in the objects. Vector indices correspond to indices in the \code{sites} and
					\code{regions} list slots.}
  \item{\code{region.types}}{character vector of all region types stored in the objects. Vector indices correspond to indices in
					the \code{regions} list slot.}
  \item{\code{comparison.grouplabels}}{A character matrix with 2 columns containing group labels of all comparisons in the object}
  \item{\code{comparison.info}}{A list containing comparison information for each comparison. See \code{\link{get.comparison.info}} for details.}
  \item{\code{includesSites}}{Logical indicating whether the object contains site-level differential methylation information.}
  \item{\code{site.test.method}}{method which was applied to obtain the site-level p-values.}
  \item{\code{variability.method}}{method to be used to detect differentially variable sites.}
  \item{\code{covg.thres}}{coverage threshold. Important for certain columns of the differential methylation tables.}
  \item{\code{disk.dump}}{Flag indicating whether the tables should be stored on disk rather than in the main memory}
  \item{\code{disk.path}}{path on the disk for DMTs.Only meaningful if \code{disk.dump} is \code{TRUE}}
}
}

\section{Methods}{

\describe{
  \item{\code{\link{destroy,RnBDiffMeth-method}}}{remove tables stored to disk from the file system}
  \item{\code{\link{get.region.types,RnBDiffMeth-method}}}{Gets all region types represented in the object as character vector}
	 \item{\code{\link{get.comparisons,RnBDiffMeth-method}}}{Gets all comparisons represented in the object as character vector}
  \item{\code{\link{get.comparison.grouplabels,RnBDiffMeth-method}}}{Gets all comparison group names as a matrix}
  \item{\code{\link{get.covg.thres,RnBDiffMeth-method}}}{Gets the coverage threshold employed for obtaining statistics in the differential methylation tables}
  \item{\code{\link{get.table,RnBDiffMeth-method}}}{Gets a differential methylation table}
  \item{\code{\link{addDiffMethTable,RnBDiffMeth-method}}}{Adds a differential methylation table}
	 \item{\code{\link{reload,RnBDiffMeth-method}}}{relink disk dumped tables. Useful if the files are manually copied or if the object is loaded again}
  \item{\code{\link{save.tables,RnBDiffMeth-method}}}{save disk dumped tables as binaries and zip them.
						Useful if the files are copied or shared.}
  \item{\code{\link{join.diffMeth}}}{Merges two disjoint RnBDiffMeth objects into one}
}
}

\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/controlPlots.R
\name{rnb.plot.snp.heatmap}
\alias{rnb.plot.snp.heatmap}
\title{rnb.plot.snp.heatmap}
\usage{
rnb.plot.snp.heatmap(dataset, writeToFile = FALSE, ...)
}
\arguments{
\item{dataset}{Dataset as an object of type inheriting \code{\linkS4class{RnBeadSet}}, or a matrix of
methylation beta values.}

\item{writeToFile}{Flag specifying whether the output should be saved as \code{\linkS4class{ReportPlot}}.}

\item{...}{Additional named arguments passed to \code{\link{createReportPlot}}. These are used only if
\code{writeToFile} is \code{TRUE}.}
}
\value{
If \code{writeToFile} is \code{TRUE}, plot as an object of type \code{\linkS4class{ReportPlot}}. Otherwise,
        there is no value returned (invisible \code{NULL}).
}
\description{
Heatmap of beta values from genotyping probes.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
rnb.plot.snp.heatmap(rnb.set.example)
}
}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrichment.R
\name{performLolaEnrichment.diffVar}
\alias{performLolaEnrichment.diffVar}
\title{performLolaEnrichment.diffVar}
\usage{
performLolaEnrichment.diffVar(
  rnbSet,
  diffmeth,
  enrich.diffMeth = NULL,
  lolaDbPaths,
  rank.cuts.region = c(100, 500, 1000),
  add.auto.rank.cut = TRUE,
  rerank = TRUE,
  verbose = TRUE
)
}
\arguments{
\item{rnbSet}{RnBSet object for which differential variability was computed}

\item{diffmeth}{RnBDiffMeth object. See \code{\link{RnBDiffMeth-class}} for details.}

\item{enrich.diffMeth}{Enrichment object as obtained from \code{\link{performLolaEnrichment.diffMeth}}. If it is not provided
a new object is created.}

\item{lolaDbPaths}{LOLA database paths}

\item{rank.cuts.region}{Cutoffs for combined ranking that are used to determine differentially variable regions}

\item{add.auto.rank.cut}{flag indicating whether an automatically computed cut-off should also be considered.}

\item{rerank}{For deterimining differential variability: should the ranks be ranked again or should the absolute ranks be used.}

\item{verbose}{Enable for detailed status report}
}
\value{
a DiffMeth.lola.enrich object (S3) containing the following attributes
\item{region}{Enrichment information for differential variability on the region level. A \code{data.table} object
as returned by the \code{runLOLA} function from the \code{LOLA} package for further details. Each element will contain different
user sets for different rank cutoffs and hyper/hypomethylation events(\code{userSet} column)}
\item{lolaDb}{The loaded \code{lolaDb} object containing the merged databases as returned by \code{\link{loadLolaDbs}}}
}
\description{
performs LOLA enrichment analysis for a given differential variability table.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
# compute differential methylation
dm <- rnb.execute.diffVar(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
# download LOLA DB
lolaDest <- tempfile()
dir.create(lolaDest)
lolaDirs <- downloadLolaDbs(lolaDest, dbs="LOLACore")
# perform enrichment analysis
res <- performLolaEnrichment.diffVar(rnb.set.example,dm,lolaDirs[["hg19"]])
}
}
\author{
Michael Scherer and Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plottingUtils.R
\name{create.scatter.dens.points}
\alias{create.scatter.dens.points}
\title{create.scatter.dens.points}
\usage{
create.scatter.dens.points(
  df2p,
  is.special = NULL,
  dens.special = TRUE,
  mock = FALSE
)
}
\arguments{
\item{df2p}{\code{data.frame} to be plotted. Only the fist two columns are taken into account as
x and y coordinates respectively}

\item{is.special}{boolean vector of length equal to the number of rows in \code{df2p}. Specifies
which points should be highlighed seperately in a different color}

\item{dens.special}{Flag indicating whether the points of the special population should be colored
according to their density}

\item{mock}{Should only the axis be plotted? useful when exporting scatterplots with lots of points
as immage and the corresponding axis as vector graphics.}
}
\value{
\code{ggplot} object
}
\description{
Creates a scatterplot containing all points in a given data.frame. Points are colored according to point
density. Optionally, a selection of points are shown in a different color
}
\examples{
\donttest{
d <- data.frame(x=rnorm(1000),y=rnorm(1000))
s <- rep(FALSE,1000)
s[sample(1:length(s),100)] <- TRUE
create.scatter.dens.points(d,s)
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrichment.R
\name{performGOEnrichment.diffVar}
\alias{performGOEnrichment.diffVar}
\title{performGOEnrichment.diffVar}
\usage{
performGOEnrichment.diffVar(
  rnbSet,
  diffmeth,
  enrich.diffMeth = NULL,
  ontologies = c("BP", "MF"),
  rank.cuts.region = c(100, 500, 1000),
  add.auto.rank.cut = TRUE,
  rerank = TRUE,
  verbose = TRUE,
  ...
)
}
\arguments{
\item{rnbSet}{RnBSet object for which dirrential variability was computed}

\item{diffmeth}{RnBDiffMeth object. See \code{\link{RnBDiffMeth-class}} for details.}

\item{enrich.diffMeth}{Result of \code{performGOEnrichment.diffMeth}. NULL, if enrichment should only be performed for differential variability.}

\item{ontologies}{GO ontologies to use for enrichment analysis}

\item{rank.cuts.region}{Cutoffs for combined ranking that are used to determine differentially variable regions}

\item{add.auto.rank.cut}{flag indicating whether an automatically computed cut-off should also be considered.}

\item{rerank}{For deterimining differential variability: should the ranks be ranked again or should the absolute ranks be used.}

\item{verbose}{Enable for detailed status report}

\item{...}{arguments passed on to the parameters of \code{GOHyperGParams} from the \code{GOstats} package}
}
\value{
a DiffMeth.enrich object (S3) containing the following attributes
\item{region}{Enrichment information for differential variability on the region level. See \code{GOHyperGresult} from the \code{GOstats} package for furthert details}
}
\description{
performs Geno Ontology (GO) enrichment analysis for a given differential variability table.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
dm <- rnb.execute.diffVar(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
res <- performEnrichment.diffVar(rnb.set.example,dm)
}
}
\author{
Fabian Mueller and Michael Scherer
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differentialVariability.R
\name{rnb.execute.diffVar}
\alias{rnb.execute.diffVar}
\title{rnb.execute.diffVar}
\usage{
rnb.execute.diffVar(
  rnb.set,
  pheno.cols = rnb.getOption("differential.comparison.columns"),
  region.types = rnb.region.types.for.analysis(rnb.set),
  columns.adj = rnb.getOption("covariate.adjustment.columns"),
  adjust.celltype = rnb.getOption("differential.adjustment.celltype"),
  disk.dump = rnb.getOption("disk.dump.big.matrices"),
  disk.dump.dir = tempfile(pattern = "diffMethTables_")
)
}
\arguments{
\item{rnb.set}{Object of type \code{\linkS4class{RnBSet}} on which differential variability analysis should be conducted}

\item{pheno.cols}{Column names used to define the classes, whose methylation variability should be compared with each other}

\item{region.types}{Regions types to be used for the analysis. Defaults to the results given by rnb.region.types.for.analysis of the given RnBSet.}

\item{columns.adj}{Column names or indices in the table of phenotypic information to be used for confounder adjustment in the differential variability analysis.}

\item{adjust.celltype}{Flag indicating whether the resulting table should also contain estimated celltype contributions. See \code{\link{rnb.execute.ct.estimation}} for details.}

\item{disk.dump}{Flag indicating whether the resulting differential methylation object should be file backed, ie.e the matrices dumped to disk}

\item{disk.dump.dir}{disk location for file backing of the resulting differential methylation object. Only meaningful if \code{disk.dump=TRUE}.}
}
\value{
Object of type \code{\linkS4class{RnBDiffMeth}} containing information about the differential variability analysis.
}
\description{
This routine computes sites that are differentially variable between two sample groups specified as the column name 
in the phenotypic table.
}
\author{
Michael Scherer
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batch.R
\name{rnb.execute.dreduction}
\alias{rnb.execute.dreduction}
\title{rnb.execute.dreduction}
\usage{
rnb.execute.dreduction(rnb.set, target = "sites")
}
\arguments{
\item{rnb.set}{Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}. This dataset
must contain at least four samples.}

\item{target}{\code{character} singleton specifying the level of DNA methylation infromation. If this is
   \code{"sites"}, the DNA methylation information for the individual sites or probes is analyzed.
   Otherwise, this should be one of the supported region types, as returned by
\code{\link{rnb.region.types}}.}
}
\value{
Results of the dimension reduction in the form of a list with the following elements:
        \describe{
          \item{\code{pca}}{Results of the PCA as returned by the function \code{\link{prcomp}}.}
          \item{\code{mds}}{List of two elements - \code{"manhattan"} and \code{"euclidean"}, each of which is a
               two-column \code{matrix} storing the coordinates of the samples in a two-dimensional space. The
               matrices are computed using the function \code{\link{isoMDS}}.}
        }
}
\description{
Performs principal component analysis (PCA) and multi-dimensional scaling (MDS) of the samples in the given
methylation dataset.
}
\details{
Row names in the returned matrices are sample identifiers, determined based on the package option
\code{"identifiers.column"}. See \emph{\link[=rnb.options]{RnBeads Options}} for more information on this option.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
regs <- c("sites", summarized.regions(rnb.set.example))
dreduction <- function(x) rnb.execute.dreduction(rnb.set.example, x)
pcoordinates <- lapply(regs, dreduction)
names(pcoordinates) <- regs
str(pcoordinates)
}
}
\seealso{
\code{\link{rnb.run.exploratory}} for running the whole exploratory analysis module
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataExport.R
\name{rnb.RnBSet.to.GRangesList}
\alias{rnb.RnBSet.to.GRangesList}
\title{rnb.RnBSet.to.GRangesList}
\usage{
rnb.RnBSet.to.GRangesList(
  rnb.set,
  reg.type = "sites",
  return.regular.list = FALSE
)
}
\arguments{
\item{rnb.set}{Object of class \code{\linkS4class{RnBSet}}}

\item{reg.type}{region type to be converted}

\item{return.regular.list}{flag indicating whether a regular \code{list} object should be returned instead
of a \code{GRangesList}. Might improve performance in some cases}
}
\value{
a \code{GRangesList} or \code{list} object with one list element (\code{GRanges}) for each sample in \code{rnb.set}
}
\description{
convert an \code{\linkS4class{RnBSet}} object to a \code{GRangesList} object
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
result <- rnb.RnBSet.to.GRangesList(rnb.set.example)
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\docType{class}
\name{RnBSet-class}
\alias{RnBSet-class}
\title{RnBSet Class}
\description{
Basic class for storing DNA methylation and experimental quality information
}
\details{
It is a virtual class and objects of type \code{RnBSet} should not be instantiated. Instead, the child classes are
used: \code{\linkS4class{RnBeadRawSet}} and \code{\linkS4class{RnBeadSet}} for Infinium HumanMethylation and
\code{\linkS4class{RnBiseqSet}} for bisulfite sequencing data
}
\section{Slots}{

\describe{
  \item{\code{pheno}}{Sample annotations (phenotypic and processing data) in the form of a \code{data.frame}.}
  \item{\code{sites}}{A \code{matrix} object storing the identifiers of the methylation sites for which the
		methylation information is present}
  \item{\code{meth.sites}}{\code{matrix} of methylation values. Every row corresponds to a methylation site,
		and every column - to a sample.}
  \item{\code{covg.sites}}{\code{matrix} of coverage values. Every row corresponds to a methylation site,
		and every column - to a sample.}
  \item{\code{regions}}{\code{list} of all identifiers of methylation sites for which methylation information
		is available.}
  \item{\code{meth.regions}}{\code{list} of methylation \code{matrix} objects, one per available region type. Every row in a
		matrix corresponds to a methylation site, and every column - to a sample.}
  \item{\code{covg.regions}}{\code{list} of coverage \code{matrix} objects, one per available region type.
		Every row corresponds to a region, and every column - to a sample.}
	 \item{\code{status}}{\code{list} with meta-information about the object.}
	 \item{\code{assembly}}{\code{character} vector of length one, specifying the genome assembly which the object is linked to, e.g. "hg19".}
  \item{\code{target}}{\code{character} vector of length one, specifying the feature class:
		\code{"CpG"} for sequencing data, \code{"probes450"} and \code{"probes27"} for
		HumanMethylation450 and HumanMethylation27 microarrays respectively.}
  \item{\code{inferred.covariates}}{\code{list} with covariate information.
		Can contain elements \code{"sva"} and \code{"cell.types"}.}
	 \item{\code{version}}{Package version in which the dataset was created.}
	 \item{\code{imputed}}{Flag indicating if methylation matrix has been imputed.}
}
}

\section{Methods and Functions}{

\describe{
  \item{\code{\link[=pheno,RnBSet-method]{pheno}}}{Gets the phenotypic and processing data of the dataset.}
  \item{\code{\link[=samples,RnBSet-method]{samples}}}{Gets the identifiers of all samples in the dataset.}
  \item{\code{\link[=summarized.regions,RnBSet-method]{summarized.regions}}}{Gets the genomic annotations for
  which methylation data is present.}
	 \item{\code{\link[=meth,RnBSet-method]{meth}}}{Gets a \code{matrix} of methylation values in the dataset.}
	 \item{\code{\link[=mval,RnBSet-method]{mval}}}{Gets a \code{matrix} of M values in the dataset.}
  \item{\code{\link[=covg,RnBSet-method]{covg}}}{Gets the \code{matrix} of coverage values of the dataset.}
  \item{\code{\link[=remove.sites,RnBSet-method]{remove.sites}}}{Removes sites from the dataset.}
  \item{\code{\link[=remove.samples,RnBSet-method]{remove.samples}}}{Removes samples from the dataset.}
  \item{\code{\link[=addPheno,RnBSet-method]{addPheno,RnBSet-method}}}{Add sample annotation to the dataset.}
  \item{\code{\link[BiocGenerics]{combine}}}{Combines two datasets.}
  \item{\code{\link{regionMapping,RnBSet-method}}}{Retrieve the sites mapping to a given region type}
  \item{\code{\link[=rnb.sample.summary.table,RnBSet-method]{rnb.sample.summary.table}}}{Creates a sample summary table from an RnBSet object.}
  \item{\code{\link{isImputed,RnBSet-method}}}{Getter for the imputation slot.}
}
}

\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readGEO.R
\name{rnb.read.geo}
\alias{rnb.read.geo}
\title{Import methylation data from GEO}
\usage{
rnb.read.geo(
  accession = NULL,
  verbose = logger.isinitialized(),
  destdir = tempdir()
)
}
\arguments{
\item{accession}{Character string, starting with \code{"GSE"}, representing the GEO series for download and parsing.
Alternatively, this parameter can specify the file name of a previously downloaded GEO series
matrix file or its gzipped representation (in which case the filename must end in \code{".gz"}).
Other file formats, such as SOFT files, are not supported.}

\item{verbose}{Flag indicating if messages should be created informing about the progress. If the logger is
initialized prior to calling this function, the informative messages are sent to the logger.
Warnings and errors are not affected by this parameters, the function always outputs them.}

\item{destdir}{The destination directory for any downloads. Defaults to the (architecture-dependent) temporary
directory. Keep in mind that GEO series can be demanding in terms of storage space.}
}
\value{
\code{\linkS4class{RnBeadSet}} object with phenotypic and beta value information.
}
\description{
Imports Infinium 450K, MethylationEPIC or MethylationEPIC2 data series from the Gene Expression Omnibus. This function uses the
series matrix file.
}
\author{
Yassen Assenov, modified by Baris Kalem
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bmiq.R
\name{BMIQ}
\alias{BMIQ}
\title{BMIQ}
\usage{
BMIQ(
  beta.v,
  design.v,
  doH = TRUE,
  nfit = 50000,
  th1.v = c(0.2, 0.75),
  th2.v = NULL,
  niter = 5,
  tol = 0.001
)
}
\arguments{
\item{beta.v}{\code{double} vector consisting of beta values. Missing values (\code{NA}s) cannot be handled, so
these must be removed or imputed prior to running BMIQ. Before normalization, beta values that are
exactly 0 and exactly 1 are replaced by the minimum positive and maximum value below 1, respectively.}

\item{design.v}{\code{integer} vector of length \code{length(beta.v)}, containing the values \code{1} and \code{2}
only. These values specify probe design type.}

\item{doH}{Flag indicating if normalization for hemimethylated type II probes is to be performed.}

\item{nfit}{Number of probes of a given design to use for the fitting. Smaller values will make BMIQ faster at
the expense of accuracy. Values between 10000 and 50000 seem to work well.}

\item{th1.v}{Thresholds "type 1" to use for the initialization of the EM algorithm. These values should represent
best guesses for calling type I probes hemi-methylated and methylated, and are refined in further
steps by the algorithm.}

\item{th2.v}{Thresholds "type 2" to used for the initialization of the EM algorithm. These values should represent
best guesses for calling type II probes hemi-methylated and methylated, and are refined in further
steps by the EM algorithm. If this is \code{NULL} (default), the thresholds are estimated based on
\code{th1.v} and a modified PBC correction method.}

\item{niter}{Maximum number of EM iterations to be performed.}

\item{tol}{Tolerance threshold for EM algorithm.}
}
\value{
List with the following elements:
        \describe{
          \item{\code{"all"}}{The normalised beta-profile for the sample.}
          \item{\code{"class1"}}{Methylation state assigned to the type I probes.}
          \item{\code{"class2"}}{Methylation state assigned to the type II probes.}
          \item{\code{"av1"}}{Mean beta values for the \code{nL} classes for type I probes.}
          \item{\code{"av2"}}{Mean beta values for the \code{nL} classes for type II probes.}
          \item{\code{"hf"}}{Hubble dilation factor.}
          \item{\code{"th1"}}{Estimated thresholds used for type I probes.}
          \item{\code{"th2"}}{Estimated thresholds used for type II probes.}
        }
}
\description{
This function makes 3 indipendent attempts to fit a 3-state beta mixture model on the provided type I
             probes. An attempt is successful if at least 4 probes are assigned to each level. In case all attempts
             fail, the return value is \code{NULL}.
}
\details{
Performs Beta-mixture quantile normalization, adjusting for type II bias in Infinium 450K data.
}
\author{
Andrew Teschendorff and Steve Horvath; with minor modifications by Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\docType{methods}
\name{combine.rnb.sets,RnBSet,RnBSet-method}
\alias{combine.rnb.sets,RnBSet,RnBSet-method}
\alias{combine.rnb.sets}
\alias{combine.rnb.sets,RnBSet-method}
\title{combine.rnb.sets-methods}
\usage{
\S4method{combine.rnb.sets}{RnBSet,RnBSet}(x, y, type = "all")
}
\arguments{
\item{x, y}{\code{\linkS4class{RnBeadSet}}, \code{\linkS4class{RnBeadRawSet}}
or \code{\linkS4class{RnBiseqSet}} object}

\item{type}{\code{character} singleton defining the set operation applied to the two site sets, 
one of "all", "all.x", "all.y" or "common"}
}
\value{
combined \code{\linkS4class{RnBeadSet}}, \code{\linkS4class{RnBeadRawSet}} or
\code{\linkS4class{RnBiseqSet}} object
}
\description{
Combine two objects inheriting from \code{\linkS4class{RnBSet}} class
}
\details{
Combine method supports a merge of any two RnBSet objects that contain data of the same specie.
In case a non-synonymous merge is performed, the class conversion will follow the following hierarchy: 
\code{\linkS4class{RnBeadSet}} < \code{\linkS4class{RnBeadRawSet}} < \code{\linkS4class{RnBiseqSet}}.
In case \code{x} and \code{y} are both array data containers (\code{RnBeadSet} or \code{RnBeadRawSet}), 
the resulting object will have an annotation that corresponds to the newer array version 
(\code{27k} < \code{450k} < \code{EPIC} < \code{EPICv2}).
The sample sets of \code{x} and \code{y} should be unique. Sample annotation information is merged only for columns 
which have identical names in both objects. CpG sites of the new object are a union of those present in both objects.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
r1 <- rnb.set.example
r1 <- remove.samples(r1,samples(rnb.set.example)[1:5])
i <- which(r1@sites[,2] == 15 | r1@sites[,2] == 21)
sites.rem.r1 <- union(sample(1:nrow(meth(rnb.set.example)),500),i)
r1 <- remove.sites(r1,sites.rem.r1)
r2 <- rnb.set.example
r2 <- remove.samples(r2,samples(rnb.set.example)[6:12])
sites.rem.r2 <- sample(1:nrow(meth(rnb.set.example)),800)
r2 <- remove.sites(r2,sites.rem.r2)
rc <- combine.rnb.sets(r1,r2)
#assertion: check the number of sites
sites.rem.c <- intersect(sites.rem.r1,sites.rem.r2)
(nrow(meth(rnb.set.example))-length(sites.rem.c)) == nrow(meth(rc))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{rnb.load.sitelist}
\alias{rnb.load.sitelist}
\title{rnb.load.sitelist}
\usage{
rnb.load.sitelist(fname, verbose = FALSE)
}
\arguments{
\item{fname}{File listing the identifiers, one per line.}

\item{verbose}{Flag indicating if messages are to be printed. If the values is \code{TRUE} and a logger is
initialized, this function adds a message to the log.}
}
\value{
The loaded list of identifiers, or \code{NULL} if \code{fname} could not be open.
}
\description{
Loads a list of probe or site identifiers. This function is used in the preprocessing module for loading a whitelist
and/or a blacklist of identifiers.
}
\seealso{
\code{\link{logger.start}} for initializing a logger
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/greedycut.R
\name{greedycut.get.statistics}
\alias{greedycut.get.statistics}
\title{greedycut.get.statistics}
\usage{
greedycut.get.statistics(filterinfo)
}
\arguments{
\item{filterinfo}{Information on the filtering iterations as a \code{data.frame} returned by
\code{\link{greedycut.filter.matrix}}.}
}
\value{
Additional statistics on the iterations in the form of a \code{data.frame} with the following columns:
        \code{"Elements retained"}, \code{"Elements removed"}, \code{"Mismatches retained"},
        \code{"Mismatches removed"}, \code{"False Positive Rate"}, \code{"Sensitivity"}, \code{"D"}. The last column
        signifies distance from the diagonal in a ROC curve.
}
\description{
Calculates various statistics on the iterations of Greedycut.
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotations.R
\name{data.frame2GRanges}
\alias{data.frame2GRanges}
\title{data.frame2GRanges}
\usage{
data.frame2GRanges(
  dframe,
  ids = rownames(dframe),
  chrom.column = "Chromosome",
  start.column = "Start",
  end.column = "End",
  strand.column = NULL,
  assembly = "hg19",
  sort.result = TRUE
)
}
\arguments{
\item{dframe}{Table defining genomic regions.}

\item{ids}{Region names (identifiers) as a \code{character} vector, or \code{NULL} if no names are present.}

\item{chrom.column}{Column name or index that lists the chromosome names.}

\item{start.column}{Column name or index that lists the start positions of the regions.}

\item{end.column}{Column name or index that lists the end positions of the regions.}

\item{strand.column}{Column name or index that lists the strands on which the regions are located. Set this to
\code{NULL} if this region set is not strand-specific.}

\item{assembly}{Genome assembly of interest. See \code{\link{rnb.get.assemblies}} for the list of supported
genomes.}

\item{sort.result}{Should the resulting table be sorted}
}
\value{
\code{GRanges} object encapsulating all well defined regions on supported chromosomes, contained in
        \code{dframe}. Columns other that the ones listed as parameters in this function are included as metadata.
}
\description{
Converts a \code{data.frame} that defines genomic regions to object of type \code{GRanges}.
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterArchitecture.R
\docType{class}
\name{ClusterArchitecture-class}
\alias{ClusterArchitecture-class}
\title{ClusterArchitecture Class}
\description{
A virtual class for storing specifications of architectures for different compute clusters. It is designed to
let other classes inherit from it
}
\details{
For a concrete child class for a sun grid architecture specification see \code{\linkS4class{ClusterArchitectureSGE}}
If you want to implement your own child class be sure to at least implement the following functions:
\code{\link{getSubCmdTokens,ClusterArchitecture-method}}.
}
\section{Slots}{

\describe{
  \item{\code{name}}{A name or identifier}
  \item{\code{executables}}{A NAMED character vector of executables that can be used by the cluster. For instance, the \code{R} executable is important} 
  \item{\code{getSubCmdTokens.optional.args}}{character vector containing the valid optional arguments to the \code{\link{getSubCmdTokens,ClusterArchitecture-method}} function.}
}
}

\section{Methods}{

\describe{
  \item{\code{\link{getSubCmdTokens,ClusterArchitecture-method}}}{Returns a vector of command line tokens corresponding to submitting
  a job with the given command to the cluster}
  \item{\code{\link{getSubCmdStr,ClusterArchitecture-method}}}{Returns a string for the of command line corresponding to submitting
  a job with the given command to the cluster}
  \item{\code{\link{setExecutable,ClusterArchitecture,character,character-method}}}{Tells the cluster architecture about an executable that can be submitted as job}
  \item{\code{\link{getExecutable,ClusterArchitecture,character-method}}}{Gets the location of an executable associated with a name}
}
}

\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBeadSet-class.R, R/RnBiseqSet-class.R
\name{coercion-methods}
\alias{coercion-methods}
\title{as("RnBeadSet", "MethyLumiSet")}
\description{
Convert a \code{\linkS4class{RnBeadSet}} object to \code{\linkS4class{MethyLumiSet}}

Convert a \code{\linkS4class{RnBeadSet}} object to a "mock" \code{\linkS4class{RnBiseqSet}} object
(used in the combine method)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computeCluster.R
\docType{methods}
\name{setModuleResourceRequirements,RnBClusterRun,character,character-method}
\alias{setModuleResourceRequirements,RnBClusterRun,character,character-method}
\alias{setModuleResourceRequirements}
\alias{setModuleResourceRequirements,RnBClusterRun-method}
\title{setModuleResourceRequirements-methods}
\usage{
\S4method{setModuleResourceRequirements}{RnBClusterRun,character,character}(object, resources, modules = "all")
}
\arguments{
\item{object}{\code{\linkS4class{RnBClusterRun}} object}

\item{resources}{A NAMED character vector containing the resource reuirements as value and the resource name as name}

\item{modules}{vector of applicable pipeline modules. Can be \code{"all"} to specify all modules}
}
\value{
The modified object
}
\description{
Specifies resource requirements for the different pipeline modules
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\docType{methods}
\name{assembly,RnBSet-method}
\alias{assembly,RnBSet-method}
\alias{assembly}
\title{assembly-methods}
\usage{
\S4method{assembly}{RnBSet}(object)
}
\arguments{
\item{object}{Dataset of interest.}
}
\value{
Sample annotation information available for the dataset in the form of a \code{data.frame}.
}
\description{
Extracts information about assembly
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
assembly(rnb.set.example) # "hg19"
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Report-methods.R
\name{rnb.get.directory}
\alias{rnb.get.directory}
\title{rnb.get.directory}
\usage{
rnb.get.directory(
  report,
  dir = c("data", "images", "images-high", "pdfs"),
  absolute = FALSE
)
}
\arguments{
\item{report}{Report of interest.}

\item{dir}{Type of directory to get. Must be one of \code{"data"}, \code{"images"}, \code{"images-high"} or
\code{"pdfs"}.}

\item{absolute}{Flag indicating if the absolute path of the directory is to be returned. If this is \code{FALSE},
the directory name is returned relative to the report's HTML file location.}
}
\value{
Path of the requested directory as a single-element \code{character} vector.
}
\description{
Gets the location of the given report-specific directory.
}
\examples{
\donttest{
report <- createReport("example.html", "Example", init.configuration = TRUE)
rnb.get.directory(report, "data")
}
}
\seealso{
\code{\linkS4class{Report}} for functions adding contents to an HTML report
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Report-class.R
\docType{class}
\name{Report-class}
\alias{Report-class}
\alias{initialize,Report-method}
\title{Report Class}
\description{
Handler of a generated HTML report. Reports are initialized using the function \code{\link{createReport}}.
}
\section{Slots}{

\describe{
  \item{\code{fname}}{Name of the file that contains the HTML report.}
  \item{\code{dir.conf}}{Directory that contains configuration files; usually shared between reports.}
  \item{\code{dir.data}}{Directory that contains the generated external lists and tables.}
  \item{\code{dir.pngs}}{Directory that contains the generated figure image files.}
  \item{\code{dir.pdfs}}{Directory that contains the generated figure PDF files.}
  \item{\code{dir.high}}{Directory that contains the generated high-resolution image file.}
  \item{\code{sections}}{Number of sections and subsections currently added to the report.}
  \item{\code{opensections}}{Indices of currently active section and subsections.}
  \item{\code{figures}}{Number of figures currently added to the report.}
  \item{\code{tables}}{Number of selectable tables added to the report.}
  \item{\code{references}}{List of references to be added at the end of the report.}
}
}

\section{Methods and Functions}{

\describe{
  \item{\code{\link{rnb.get.directory}}}{Gets the location of a given report-specific directory.}
  \item{\code{\link{rnb.add.section}}}{Generates HTML code for a new section in the report.}
  \item{\code{\link{rnb.add.paragraph}}}{Generates HTML code for a new paragraph in the report.}
  \item{\code{\link{rnb.add.list}}}{Generates HTML code for a list in the report.}
  \item{\code{\link{rnb.add.table}}}{Generates HTML code for a table in the report.}
  \item{\code{\link{rnb.add.tables}}}{Generates HTML code for a listing of tables in the report.}
  \item{\code{\link{rnb.add.figure}}}{Generates HTML code for a figure in the report.}
  \item{\code{\link{rnb.add.reference}}}{Adds a reference item to the report.}
  \item{\code{\link[=off,Report-method]{off}}}{Completes the HTML report by adding a reference section (if needed),
       a footer notice and closing the \code{<body>} and \code{<html>} tags.}
}
}

\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/profiles.R
\name{rnb.step.betadistribution}
\alias{rnb.step.betadistribution}
\title{rnb.step.betadistribution}
\usage{
rnb.step.betadistribution(
  rnb.set,
  report,
  columns = rnb.getOption("exploratory.columns"),
  points.per.group = rnb.getOption("distribution.subsample")
)
}
\arguments{
\item{rnb.set}{HumanMethylation450K dataset as an object of type \code{\linkS4class{RnBSet}}.}

\item{report}{Report to contain the methylation deviation section. This must be an object of type
\code{\linkS4class{Report}}.}

\item{columns}{Optional; predefined column names (in the form of a \code{character} vector) or indices (an
\code{integer} vector) in the sample annotation table. Only these columns are considered for grouping
samples and defining profiles. All other columns in the phenotype table are ignored.}

\item{points.per.group}{the targeted number of points (T) per group. Set this to a value < 1 to disable subsampling. More
information in the Details section}
}
\value{
The modified report.
}
\description{
Computes the distributions of beta values across various sample groups and adds a corresponding section to the
report.
}
\section{Details}{

If subsampling is enabled (i.e. \code{points.per.group}>0), 
observations per group are subsampled according to the following procedure:
Given K groups and numbers of observed beta values per group
N_1,...,N_K, and the target number of points per group T: the total number of points N = sum(N_1,...,N_K) is computed
Afterwards the proportions p_k = N_k/N is computed and from each group, S_k = p_k*(K*T) observations
are randomly selected from all observations belonging to group k.
}

\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBDiffMeth-class.R
\docType{methods}
\name{get.comparisons,RnBDiffMeth-method}
\alias{get.comparisons,RnBDiffMeth-method}
\alias{get.comparisons}
\title{get.comparisons-methods}
\usage{
\S4method{get.comparisons}{RnBDiffMeth}(object)
}
\arguments{
\item{object}{\code{\linkS4class{RnBDiffMeth}} object}
}
\value{
character vector containing comparisons
}
\description{
Gets all comparisons represented in the object as character vector
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
get.comparisons(dm)
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/controlPlots.R
\name{rnb.plot.control.boxplot}
\alias{rnb.plot.control.boxplot}
\title{rnb.plot.control.boxplot}
\usage{
rnb.plot.control.boxplot(
  rnb.set,
  type = rnb.infinium.control.targets(rnb.set@target)[1],
  writeToFile = FALSE,
  numeric.names = FALSE,
  ...
)
}
\arguments{
\item{rnb.set}{\code{\linkS4class{RnBeadRawSet}} or \code{\linkS4class{RnBeadSet}} object with valid quality
control information.}

\item{type}{type of the control probe; must be one of the \code{"BISULFITE CONVERSION I"},
\code{"BISULFITE CONVERSION II"}, \code{"EXTENSION"}, \code{"HYBRIDIZATION"}, \code{"NEGATIVE"},
\code{"NON-POLYMORPHIC"}, \code{"NORM_A"}, \code{"NORM_C"}, \code{"NORM_G"}, \code{"NORM_T"},
\code{"SPECIFICITY I"}, \code{"SPECIFICITY II"}, \code{"STAINING"}, \code{"TARGET REMOVAL"}.}

\item{writeToFile}{flag specifying whether the output should be saved as \code{\linkS4class{ReportPlot}}}

\item{numeric.names}{if \code{TRUE} and \code{writeToFile} is \code{TRUE}substitute the plot options in the plot file name with digits}

\item{...}{other arguments to \code{\link{createReportPlot}}}
}
\value{
plot as an object of type \code{\linkS4class{ReportPlot}} if \code{writeToFile} is \code{TRUE} and of class
			\code{\link{ggplot}} otherwise.
}
\description{
Box plots of various control probes
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
rnb.plot.control.boxplot(rnb.set.example)
}

}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotations.R
\name{rnb.set.annotation.and.cpg.stats}
\alias{rnb.set.annotation.and.cpg.stats}
\title{rnb.set.annotation.and.cpg.stats}
\usage{
rnb.set.annotation.and.cpg.stats(
  type,
  regions,
  description = NULL,
  assembly = "hg19"
)
}
\arguments{
\item{type, description, assembly}{Parameters handled exactly as in \code{\link{rnb.set.annotation}}}

\item{regions}{a data.frame handled similarly as by \code{\link{rnb.set.annotation}} with the exception that
the genomic location columns should be specified using upper case first letters}
}
\value{
Invisibly, \code{TRUE} if an existing annotation was replaced and \code{FALSE} otherwise.
}
\description{
wrapper for \code{\link{rnb.set.annotation}} to accept the region format as output by \code{annotation(rnb.set)}.
Additionally, CpG statistics are added to the annotation.
}
\seealso{
\code{\link{rnb.set.annotation}}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/controlPlots.R
\name{rnb.plot.negative.boxplot}
\alias{rnb.plot.negative.boxplot}
\title{rnb.plot.negative.boxplot}
\usage{
rnb.plot.negative.boxplot(
  rnb.set,
  sample.subset = 1:length(samples(rnb.set)),
  writeToFile = FALSE,
  name.prefix = NULL,
  ...
)
}
\arguments{
\item{rnb.set}{\code{\linkS4class{RnBeadSet}} object with valid quality control information}

\item{sample.subset}{an integer vector specifying the subset of samples for which the plotting should be performed}

\item{writeToFile}{flag specifying whether the output should be saved as \code{\linkS4class{ReportPlot}}}

\item{name.prefix}{in case \code{writeToFile} is \code{TRUE}, a \code{character} singleton specifying a prefix to the variable part of the image file names}

\item{...}{other arguments to \code{\link{createReportPlot}}}
}
\value{
plot as an object of type \code{\linkS4class{ReportPlot}} if \code{writeToFile} is \code{TRUE} and of class
			\code{\link{ggplot}} otherwise.
}
\description{
Box plots of negative control probes
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
rnb.plot.negative.boxplot(rnb.set.example)
}
}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/greedycut.R
\name{greedycut.get.submatrix}
\alias{greedycut.get.submatrix}
\title{greedycut.get.submatrix}
\usage{
greedycut.get.submatrix(
  mm,
  filter.info,
  it.num = nrow(filter.info) - as.integer(1)
)
}
\arguments{
\item{mm}{Data \code{matrix} to be filtered.}

\item{filter.info}{Information on the filtering iterations as a \code{data.frame} returned by
\code{\link{greedycut.filter.matrix}}.}

\item{it.num}{Number of iterations to execute. Defaults to all iterations.}
}
\value{
Data matrix containing subsets of the rows and columns of \code{mm}.
}
\description{
Filters a data matrix executing the given number of iterations of Greedycut.
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.R
\name{rnb.build.index}
\alias{rnb.build.index}
\title{rnb.build.index}
\usage{
rnb.build.index(
  dir.reports,
  fname = "index.html",
  dir.configuration = "configuration",
  open.index = TRUE
)
}
\arguments{
\item{dir.reports}{Directory that contains HTML reports generated by \pkg{RnBeads} modules. If this directory
does not exist, is a regular file, is inaccessible, or does not contain any recognizable
HTML report files, this function does not generate an HTML index file and produces an error
or a warning message.}

\item{fname}{One-element \code{character} vector specifying the name of the index file to be generated.
See the \emph{Details} section for restrictions on the name. The file will be created in
\code{dir.reports}. If such a file already exists, it will be overwritten.}

\item{dir.configuration}{Subdirectory that hosts configuration files shared by the reports. This must be a
\code{character} vector of length one that gives location as a path relative to
\code{dir.reports}. Strong restrictions apply to the path name. See the description of the
\code{\link{createReport}} function for more details.}

\item{open.index}{Flag indicating if the index should be displayed after it is created. If this is
\code{TRUE}, \code{\link{rnb.show.report}} is called to open the generated HTML file.}
}
\value{
Names of all HTML report files that were referenced in the newly generated index, invisibly. The order of the
        file names is the same as the one they are listed in the index. If no known reports are found in the given
        directory, the returned value is an empty \code{character} vector.
}
\description{
Creates an HTML index file that contains listing of all available \pkg{RnBeads} reports. If no known reports are
found in the specified directory, no index is created.
}
\details{
In order to ensure independence of the operating system, there are strong restrictions on the name of the index file.
It can consist of the following symbols only: Latin letters, digits, dot (\code{.}), dash (\code{-}) and underline
(\code{_}). The extension of the file must be one of \code{htm}, \code{html}, \code{xhtml} or \code{xml}. The name
must not include paths, that is, slash (\code{/}) or backslash (\code{\\}) cannot be used. In addition, it cannot be
any of the recognized \pkg{RnBeads} report file names.
}
\seealso{
\code{\link{rnb.run.analysis}}, \code{\link{rnb.initialize.reports}}
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differentialMethylation.R
\name{exportDMRs2regionFile}
\alias{exportDMRs2regionFile}
\title{exportDMRs2regionFile}
\usage{
exportDMRs2regionFile(
  rnbSet,
  diffmeth,
  dest,
  comp.name,
  region.type,
  rank.cut = NULL,
  rerank = FALSE
)
}
\arguments{
\item{rnbSet}{the RnBSet object for which the DMRs were computed.}

\item{diffmeth}{DiffMeth object. See \code{\link{rnb.execute.computeDiffMeth}} for details.}

\item{dest}{destination file name}

\item{comp.name}{name of the comparison}

\item{region.type}{region type.}

\item{rank.cut}{rank cutoff. If \code{NULL} (default), all regions are processed.}

\item{rerank}{flag indicating whether the ranks should be reranked or whether \code{rank.cut} refers to the absolute rank}
}
\value{
\code{NULL}
}
\description{
export differentially methylated regions to region file (standard bed). The output is in BED6 format where the score corresponds to 
to the combined rank (rank==1 would receive a score of 1000 and a combined rank equal to the number of regions a score of 0)
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
exportDMRs2regionFile(rnb.set.example,dm,tempfile(),get.comparisons(dm)[1],"promoters")
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ReportPlot-methods.R
\name{createReportPlot}
\alias{createReportPlot}
\title{createReportPlot}
\usage{
createReportPlot(
  fname,
  report = NULL,
  width = 7,
  height = 7,
  create.pdf = TRUE,
  low.png = 100L,
  high.png = 0L
)
}
\arguments{
\item{fname}{\code{character} vector with one element storing the name of the output file, without the
extension. The initialized object appends \code{.pdf} and/or \code{.png} to this name.}

\item{report}{Report (object of type \code{\linkS4class{Report}}) to which this plot is going to be added. This
is used to set the directories for PDF and/or PNG files generated for these plots. If this
parameter is \code{NULL}, the current working directory is used to host all generated images.}

\item{width}{\code{numeric} storing the width of the device in inches. The length of this vector must be
\code{1}.}

\item{height}{\code{numeric} storing the height of the device in inches. The length of this vector must be
\code{1}.}

\item{create.pdf}{Flag indicating if a PDF image is to be created. The length of this vector must be \code{1}.}

\item{low.png}{Resolution, in dots per inch, used for the figure image. Set this to \code{0} or a negative value
to disable the creation of a low resolution image. The length of this vector must be \code{1}.}

\item{high.png}{Resolution, in dots per inch, used for a dedicated image. Set this to \code{0} or a negative value
to disable the creation of a high resolution image. The length of this vector must be \code{1}.}
}
\value{
Newly created \code{ReportPlot} object.
}
\description{
Initializes a report plot and opens a device to create it. The type of the device created depends on the
parameters \code{create.pdf}, \code{low.png} and \code{high.png}. If \code{create.pdf} is \code{TRUE}, a PDF device
is opened and its contents are later copied to PNG device(s) if needed. Otherwise, a PNG device is opened. Note that
at least one of the following conditions must be met:
\itemize{
  \item{}{\code{create.pdf == TRUE}}
  \item{}{\code{low.png > 0}}
  \item{}{\code{high.png > 0}}
}
}
\details{
In order to ensure independence of the operating system, there are strong restrictions on the name of the file. It
can consist of the following symbols only: Latin letters, digits, dot (\code{.}), dash (\code{-}) and underline
(\code{_}). The name must not include paths, that is, slash (\code{/}) or backslash (\code{\\}) cannot be used.
}
\examples{
\donttest{
plot.image <- createReportPlot('scatterplot_tumors')
plot(x = c(0.4, 1), y = c(9, 3), type = 'p', main = NA, xlab = expression(beta), ylab = 'Measure')
off(plot.image)
}

}
\seealso{
\code{\link{pdf}} for manually initializing a graphics device; \code{\linkS4class{Report}} for other
  functions adding contents to an HTML report
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBDiffMeth-class.R
\docType{methods}
\name{get.table,RnBDiffMeth-method}
\alias{get.table,RnBDiffMeth-method}
\alias{get.table}
\title{get.table-methods}
\usage{
\S4method{get.table}{RnBDiffMeth}(
  object,
  comparison,
  region.type,
  undump = TRUE,
  return.data.frame = FALSE
)
}
\arguments{
\item{object}{\code{\linkS4class{RnBDiffMeth}} object}

\item{comparison}{character or index of the comparison of the table to retrieve}

\item{region.type}{character or index of the region type of the table to retrieve}

\item{undump}{Flag indicating whether to convert the table into a matrix instead of using the file descriptor.
Only meaningful if the if the objects's \code{disk.dump} slot is true.}

\item{return.data.frame}{should a data.frame be returned instead of a matrix?}
}
\value{
differential methylation table. See \code{computeDiffMeth.bin.site} and \code{computeDiffMeth.bin.region} for details.
}
\description{
Gets a differential methylation table
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
dm.promoters <- get.table(dm,get.comparisons(dm)[1],"promoters",return.data.frame=TRUE)
summary(dm.promoters)
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filtering.R
\name{rnb.execute.high.coverage.removal}
\alias{rnb.execute.high.coverage.removal}
\title{rnb.execute.high.coverage.removal}
\usage{
rnb.execute.high.coverage.removal(rnb.set)
}
\arguments{
\item{rnb.set}{Methylation dataset as an object of type inheriting \code{\linkS4class{RnBiseqSet}}.}
}
\value{
\code{list} of two elements:
        \describe{
          \item{\code{"dataset"}}{The (possibly) modified dataset after retaining sites on autosomes only.}
          \item{\code{"filtered"}}{\code{integer} vector storing the indices of all removed sites.}
        }
}
\description{
Removes methylation sites with a coverage larger than 100 times the 95-percentile of coverage in each sample.
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Report-methods.R
\name{rnb.add.list}
\alias{rnb.add.list}
\title{rnb.add.list}
\usage{
rnb.add.list(report, txt, type = "u")
}
\arguments{
\item{report}{Report to write the text to.}

\item{txt}{Non-empty list of items to be written. An attribute named \code{type}, if it exists, specifies the type
of the list. See the \emph{Details} section for more information. Every item must be either a nested
\code{list}, denoting a sublist, or a \code{character} vector (or array), storing the text to be
written. Any other objects are coerced to a character type. Elements are concatenated without a
separator to form the text for a list item.}

\item{type}{List type to be used for the list and/or its sublists in case the attribute \code{type} is not
specified.}
}
\value{
The modified report, invisibly.
}
\description{
Generates HTML code for a list in the specified report.
}
\details{
There are two ways to specify a list type: (1) setting a value for the attribute \code{type} of the list, or
         (2) using the function's parameter \code{type}. The value of the function's parameter is used only for lists
         and sublists that do not contain an attribute named \code{type}. The following types are supported:
         \describe{
           \item{\code{"o"}}{Ordered list using arabic numbers - \code{1}, \code{2}, \code{3}, etc.}
           \item{\code{"u"}}{Unordered list using bullet points.}
         }
         Note that every list type must be a one-element \code{character} vector containing one of the codes listed
         above. Specifying any other value for list type results in an error.
}
\examples{
\donttest{
report <- createReport("example.html", "Example", init.configuration = TRUE)
recipe <- list("Sift flour in a bowl", "Add sugar and mix", "Add milk and mix")
rnb.add.list(report, recipe, type="o")
}
}
\seealso{
\code{\linkS4class{Report}} for other functions adding contents to an HTML report
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plottingUtils.R
\name{rnb.plot.pheno.categories}
\alias{rnb.plot.pheno.categories}
\title{rnb.plot.pheno.categories}
\usage{
rnb.plot.pheno.categories(
  annotations,
  columns = NULL,
  fileprefix = "barchart_pheno",
  report = NULL,
  color.values = rnb.getOption("colors.category")
)
}
\arguments{
\item{annotations}{Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}, or its sample
annotations in the form of a \code{data.frame}. If this parameter is a dataset, the annotation
information is extracted using the method \code{\link[=pheno,RnBSet-method]{pheno}}.}

\item{columns}{Optional; predefined column names (in the form of a \code{character} vector) or indices (an
\code{integer} vector) to consider. All other columns in the annotation table will be ignored.}

\item{fileprefix}{\code{character} vector with one element storing the file name prefix of the output files,
without the extension. Only a limited set of symbols is allowed to be used in this prefix.}

\item{report}{Report to contain the generated plots. If specified, this must be an object of type
\code{\linkS4class{Report}}.}

\item{color.values}{Non-empty \code{character} vector containing the color scheme to be mapped to the categories
defined in the annotation table. Colors are recycled if necessary, that is, if the length of this
vector is smaller than the number of categories in a trait.}
}
\value{
List of report plots. The names in this list are the column names in the annotation table that were selected
        for visualization. In case no suitable categorical traits are found among the provided annotations, this
        function returns an empty list.
}
\description{
Generates bar charts summarizing the categorical traits in a sample annotation table.
}
\details{
This function identifies the traits that define sample subgroups and then generates one report plot per
trait. Every report plot consists of two files. File names are formed by appending an index and file extension to
\code{fileprefix}. Thus, the suffixes appended are \code{"_1.pdf"}, \code{"_1.png"}, \code{"_2.pdf"},
\code{"_2.png"}, ... Existing files with the generated filenames are overwritten.
}
\seealso{
\code{\link{rnb.sample.groups}} for identifying traits in the annotation table that define sample subgroups;
         \code{\link{createReportPlot}} for the allowed symbols to be used in \code{fileprefix}
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.R
\name{rnb.run.example}
\alias{rnb.run.example}
\title{rnb.run.example}
\usage{
rnb.run.example(index = 4L, dir.output = "example")
}
\arguments{
\item{index}{Example to start. This must be one of \code{1}, \code{2}, \code{3} or \code{4}.}

\item{dir.output}{One-element \code{character} vector specifying the directory to contain the downloaded data files
and generated reports. This must be a non-existent path, as this function attempts to create it.}
}
\value{
Invisibly, the loaded, normalized and/or possibly filtered dataset as an object of type inheriting
        \code{\linkS4class{RnBSet}}.
}
\description{
Executes the analysis pipeline for an example from the RnBeads web site.
}
\details{
For more information about the examples, please visit the dedicated
\href{https://rnbeads.org/examples.html}{page on the RnBeads web site}.
}
\examples{
\donttest{
rnb.run.example()
}

}
\seealso{
\code{\link{rnb.run.analysis}} for starting the analysis pipeline from a local data source
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\docType{methods}
\name{meth,RnBSet-method}
\alias{meth,RnBSet-method}
\alias{meth}
\title{meth-methods}
\usage{
\S4method{meth}{RnBSet}(object, type = "sites", row.names = FALSE, i = NULL, j = NULL)
}
\arguments{
\item{object}{dataset of interest.}

\item{type}{\code{character} singleton. If this is set to \code{"sites"} (default), DNA methylation information
for each available site is returned. Otherwise, this should be one of region types for for which
summarized DNA methylation information is computed in the given dataset.}

\item{row.names}{flag indicating if row names are to be generated in the result.}

\item{i}{indices of sites/regions to be retrieved. By default (\code{NULL}), all will be retrieved.}

\item{j}{indices of samples to be retrieved. By default (\code{NULL}), all will be retrieved.}
}
\value{
\code{matrix} with methylation beta values.
}
\description{
Extracts DNA methylation information (beta values) for a specified set of genomic features.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
## per-site beta-value matrix
mm<-meth(rnb.set.example, row.names=TRUE)
head(mm)
## beta-values for each covered gene
gmm<-meth(rnb.set.example, type="gene", row.names=TRUE)
head(gmm)
}
}
\seealso{
\code{\link[=mval,RnBSet-method]{mval}} for calculating M values
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logger.R
\name{logger.status}
\alias{logger.status}
\alias{logger.info}
\alias{logger.warning}
\alias{logger.error}
\title{Writing text messages to the log file.}
\usage{
logger.status(txt)

logger.info(txt)

logger.warning(txt)

logger.error(txt, terminate = rnb.getOption("logging.exit.on.error"))
}
\arguments{
\item{txt}{Text to add to the log file. This must be a \code{character} vector; its elements are concatenated
using a single space (\code{" "}) as a separator.}

\item{terminate}{Flag indicating if the execution is to be terminated after this error message is added to the log.}
}
\value{
None (invisible \code{NULL}).
}
\description{
Appends a single-line status message to the log text file. The message is prepended by its type, which is one of
\code{STATUS}, \code{INFO}, \code{WARNING} or \code{ERROR}.
}
\examples{
\donttest{
if (!logger.isinitialized())
  logger.start(fname = NA)
logger.status(c("Reached step", 2))
logger.info(c("Provided email:", rnb.getOption("email")))
}
}
\seealso{
\code{\link{logger.isinitialized}}to check if logging is activated;
  \code{\link{logger.start}} for initializing a logger or starting a section
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lolaUtils.R
\name{getTargetFromLolaDb}
\alias{getTargetFromLolaDb}
\title{getTargetFromLolaDb}
\usage{
getTargetFromLolaDb(lolaDb)
}
\arguments{
\item{lolaDb}{LOLA DB object as returned by \code{LOLA::loadRegionDB} or \code{\link{loadLolaDbs}}}
}
\value{
character vector with targets
}
\description{
retrieve or guess the target from a LOLA DB object. Here, target typically
refers to antibodies for ChIP-seq experiments, but could also refer to other annotations
(e.g. motifs in TF motif databases, annotation according to UCSC features etc.)
}
\examples{
\donttest{
# download LOLA DB
lolaDest <- tempfile()
dir.create(lolaDest)
lolaDirs <- downloadLolaDbs(lolaDest, dbs="LOLACore")
lolaDb <- loadLolaDbs(lolaDirs[["hg19"]])
getTargetFromLolaDb(lolaDb)
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regionProfiles.R
\name{rnb.find.relative.site.coord}
\alias{rnb.find.relative.site.coord}
\title{rnb.find.relative.site.coord}
\usage{
rnb.find.relative.site.coord(rnb.set, region.type, extend.by = 0.33)
}
\arguments{
\item{rnb.set}{RnBSet object}

\item{region.type}{Region type for which the coordinates are computed}

\item{extend.by}{A number between 0 and 1 specifying the percentage by which a region is
extended in order to capture methylation information before region start and after region end}
}
\value{
a data frame containing the site index, the assigned region index and the relative coordinate
        The relative coordinate is 0 if the site's coordinate is identical to the region start coordinate
        and 1 if identical to the regions end coordinate and scaled inbetween. Coordinates can be
        less than 0 or larger than 1 if a site is in the upstream or downstream flanking region respectively
}
\description{
given a region types, assigns sites to regions and determines relative positions
of sites in the assigned region
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBeads-package.R
\docType{data}
\name{interspecies_probes_v1}
\alias{interspecies_probes_v1}
\title{XenoMix interspeciesprobes (v1)}
\format{
\code{list} of CGIs for the inter-species probes.
}
\description{
These are the inter-species probes for EPICv1
}
\author{
Nima Esmaeelpour
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filtering.R
\name{rnb.execute.sex.removal}
\alias{rnb.execute.sex.removal}
\title{rnb.execute.sex.removal}
\usage{
rnb.execute.sex.removal(rnb.set)
}
\arguments{
\item{rnb.set}{Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.}
}
\value{
List of three elements:
        \describe{
          \item{\code{"dataset.before"}}{Copy of \code{rnb.set}.}
          \item{\code{"dataset"}}{The (possibly) modified dataset after retaining sites on autosomes only.}
          \item{\code{"filtered"}}{\code{integer} vector storing the indices (in beta matrix of the unfiltered
               dataset) of all removed probes.}
        }
}
\description{
Removes all sites in sex chromosomes from the given dataset.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
rnb.set.filtered <- rnb.execute.sex.removal(rnb.set.example)$dataset
identical(meth(rnb.set.example), meth(rnb.set.filtered)) # FALSE
}
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{rnb.sample.groups}
\alias{rnb.sample.groups}
\title{rnb.sample.groups}
\usage{
rnb.sample.groups(
  annotations,
  columns = NULL,
  columns.pairs = NULL,
  min.group.size = rnb.getOption("min.group.size"),
  max.group.count = rnb.getOption("max.group.count")
)
}
\arguments{
\item{annotations}{Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}},
or its sample annotations in the form of a \code{data.frame}. If this parameter is
a dataset, the annotation information is extracted using the method
\code{\link[=pheno,RnBSet-method]{pheno}}.}

\item{columns}{Optional; predefined column names (in the form of a \code{character} vector) or indices (an
\code{integer} vector) to consider. All other columns in the annotation table will be ignored.}

\item{columns.pairs}{Optional; a NAMED vector containing for each column name for which paired comparisons should
be performed (say columnA) the name or index of another column (say columnB) in which same
values indicate the same pairing. columnA should be the name of the value columnB in this
vector.}

\item{min.group.size}{Minimum number of samples in each subgroup. This must be a positive \code{integer}.}

\item{max.group.count}{Maxumum number of subgroups defined by a trait. This must be an \code{integer} greater than 1.}
}
\value{
List of traits that define subgroups in the dataset. For each trait, the defined subgroups are represented by
        a list of \code{integer} vectors storing the corresponding sample indices.
}
\description{
Identifies sample subgroups defined in the given annotation information.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
str(rnb.sample.groups(rnb.set.example))
}

}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataImport.R
\name{read.data.dir}
\alias{read.data.dir}
\title{read.data.dir}
\usage{
read.data.dir(
  dir,
  pheno,
  betas,
  p.values,
  bead.counts,
  sep = rnb.getOption("import.table.separator"),
  verbose = TRUE
)
}
\arguments{
\item{dir}{directory containing the table files}

\item{pheno}{a file containing data sample annotations and phenotypic information}

\item{betas}{a file containing the beta values. If not supplied, the routine will look in dir for a file containing "beta" token in the filename}

\item{p.values}{a file containing the detection p values. If not supplied, the routine will look in dir for a file containing "pval" token in the filename}

\item{bead.counts}{a file containing the bead counts (optional). If not supplied, the routine will look in dir for a file containing "bead" token in the filename}

\item{sep}{character used as field separator in the tables files. Default value is taken by the call to \code{rnb.getOption("import.table.separator")}}

\item{verbose}{Flag indicating ifthe messages to the logger should be sent. Note that the logger must be initialized prior to calling this function.
Logging is useful for keeping a record of the downloaded and processed samples. Also, informative messages are stored in case of an error.}
}
\value{
Object of type \code{\linkS4class{RnBeadSet}}.
}
\description{
Reads in a directory with Illumina Infinium HumanMethylation450 data. The files shoudl be stored as data
}
\details{
Colnames in all files should match. They will be returned as the samples element of the list.
}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sva.R
\name{get.covariates.sva}
\alias{get.covariates.sva}
\title{get.covariates.sva}
\usage{
get.covariates.sva(rnb.set, target)
}
\arguments{
\item{rnb.set}{\code{RnBSet} object}

\item{target}{target variable. Must be in \code{pheno(rnb.set)} and belong to target variables for which the
SVs have already been computed and stored in the RnBSet.}
}
\value{
an NxK table of K Surrogate variables stored for N samples of the \code{rnb.set}. \code{NULL}
	   if the components have not been computed or added to \code{rnb.set}.
}
\description{
Retrieves an NxK table of Surrogate variables stored in an RnBSet for a given target variable
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
sva.obj <- rnb.execute.sva(rnb.set.example,c("Sample_Group","Treatment"),numSVmethod="be")
sva.obj$sva.performed
sva.obj$num.components
rnb.set.mod <- set.covariates.sva(rnb.set.example, sva.obj)
get.covariates.sva(rnb.set.mod,"Sample_Group")
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBeads-package.R
\docType{data}
\name{lump.hg38}
\alias{lump.hg38}
\title{LUMP Support (hg38)}
\format{
\code{lump.*} is a \code{list} of non-empty \code{integer} matrices, one per supported platform. Here, only `CpG` is 
         available for BS datasets.
}
\description{
Those are the same sites as reported in \code{lump.hg19}, but lifted to `hg38` with UCSC's liftOver functionality. This only
applies for the CpG-wise sites; i.e. those used for sequencing data sets, since `hg38` is not supported for array-based data
sets.
}
\author{
Michael Scherer
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R, R/RnBeadSet-class.R,
%   R/RnBeadRawSet-class.R
\docType{methods}
\name{remove.samples,RnBSet-method}
\alias{remove.samples,RnBSet-method}
\alias{remove.samples}
\alias{remove.samples,RnBeadSet-method}
\alias{remove.samples,RnBeadRawSet-method}
\title{remove.samples-methods}
\usage{
\S4method{remove.samples}{RnBSet}(object, samplelist)

\S4method{remove.samples}{RnBeadSet}(object, samplelist)

\S4method{remove.samples}{RnBeadRawSet}(object, samplelist)
}
\arguments{
\item{object}{Dataset of interest.}

\item{samplelist}{List of samples to be removed in the form of a \code{logical}, \code{integer} or \code{character}
vector. If this parameter is \code{logical}, it is not recycled; its length must be equal to the
number of samples in \code{object}. If it is \code{integer} or \code{character}, it must list only
samples that exist in the dataset. Specifying sample indices larger than the number of samples, or
non-existent sample identifiers results in an error.}
}
\value{
The modified dataset.
}
\description{
Removes the specified samples from the dataset.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
samples(rnb.set.example)
## remove 3 random samples
s2r<-sample.int(length(samples(rnb.set.example)), 3)
rnb.set.f<-remove.samples(rnb.set.example, s2r)
samples(rnb.set.f)
}
}
\seealso{
\code{\link[=remove.sites,RnBSet-method]{remove.sites}} for removing sites or probes from a methylation dataset
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sva.R
\name{set.covariates.sva}
\alias{set.covariates.sva}
\title{set.covariates.sva}
\usage{
set.covariates.sva(rnb.set, sva.obj)
}
\arguments{
\item{rnb.set}{The \code{RnBSet} object to which the results should be added}

\item{sva.obj}{An object of class \code{SvaResult} as returned by \code{rnb.execute.sva}.}
}
\value{
The modified \code{RnBSet}. Note that the association information will not be stored.
}
\description{
Adds the results of Surrogate Variable Analysis (SVA) to an RnBSet
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
sva.obj <- rnb.execute.sva(rnb.set.example,c("Sample_Group","Treatment"),numSVmethod="be")
sva.obj$sva.performed
sva.obj$num.components
rnb.set.mod <- set.covariates.sva(rnb.set.example, sva.obj)
has.covariates.sva(rnb.set.example,"Sample_Group")
has.covariates.sva(rnb.set.mod,"Sample_Group")
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ReportPlot-methods.R
\name{createReportGgPlot}
\alias{createReportGgPlot}
\title{createReportGgPlot}
\usage{
createReportGgPlot(
  ggp,
  fname,
  report = NULL,
  width = 7,
  height = 7,
  create.pdf = TRUE,
  low.png = as.integer(100),
  high.png = as.integer(0)
)
}
\arguments{
\item{ggp}{\code{ggplot} object to be plotted}

\item{fname}{\code{character} vector with one element storing the name of the output file, without the
extension. The initialized object appends \code{.pdf} and/or \code{.png} to this name.}

\item{report}{Report (object of type \code{\linkS4class{Report}}) to which this plot is going to be added. This
is used to set the directories for PDF and/or PNG files generated for these plots. If this
parameter is \code{NULL}, the current working directory is used to host all generated images.}

\item{width}{\code{numeric} storing the width of the device in inches. The length of this vector must be
\code{1}.}

\item{height}{\code{numeric} storing the height of the device in inches. The length of this vector must be
\code{1}.}

\item{create.pdf}{Flag indicating if a PDF image is to be created. The length of this vector must be \code{1}.}

\item{low.png}{Resolution, in dots per inch, used for the figure image. Set this to \code{0} or a negative value
to disable the creation of a low resolution image. The length of this vector must be \code{1}.}

\item{high.png}{Resolution, in dots per inch, used for a dedicated image. Set this to \code{0} or a negative value
to disable the creation of a high resolution image. The length of this vector must be \code{1}.}
}
\value{
Newly created \code{ReportGgPlot} object.
}
\description{
creates a report plot containing a \code{ggplot object}. Except for the \code{ggp} parameter, the signature and
behavior is identical to \code{\link{createReportPlot}}.
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/options.R
\name{rnb.is.option}
\alias{rnb.is.option}
\title{rnb.is.option}
\usage{
rnb.is.option(txt)
}
\arguments{
\item{txt}{Potential option name. This should be a one-element \code{character} vector.}
}
\value{
\code{TRUE} if the specified parameter is a valid analysis option name; \code{FALSE} otherwise.
}
\description{
Checks if the specified text is an option name.
}
\examples{
\donttest{
rnb.is.option("logging") # TRUE
rnb.is.option("Logging") # FALSE
}
}
\seealso{
\code{\link{rnb.options}} for getting and setting option values
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lolaUtils.R
\name{lolaVolcanoPlot}
\alias{lolaVolcanoPlot}
\title{lolaVolcanoPlot}
\usage{
lolaVolcanoPlot(
  lolaDb,
  lolaRes,
  includedCollections = c(),
  signifCol = "qValue",
  colorBy = "maxRnk",
  colorpanel = c()
)
}
\arguments{
\item{lolaDb}{LOLA DB object as returned by \code{LOLA::loadRegionDB} or \code{\link{loadLolaDbs}}}

\item{lolaRes}{LOLA enrichment result as returned by the \code{runLOLA} function from the \code{LOLA} package}

\item{includedCollections}{vector of collection names to be included in the plot. If empty (default), all collections are used}

\item{signifCol}{column name of the significance score in \code{lolaRes}. Should be one of \code{c("pValueLog", "qValue")}.}

\item{colorBy}{annotation/column in the the LOLA DB that should be used for point coloring}

\item{colorpanel}{colors to be used for coloring the points}
}
\value{
ggplot object containing the plot
}
\description{
plot a volcano plot showing LOLA enrichment results: LOLA p-value against the log-odds score. Colored by rank
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
# compute differential methylation
dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
# download LOLA DB
lolaDest <- tempfile()
dir.create(lolaDest)
lolaDirs <- downloadLolaDbs(lolaDest, dbs="LOLACore")
# perform enrichment analysis
res <- performLolaEnrichment.diffMeth(rnb.set.example,dm,lolaDirs[["hg19"]])
# select the 500 most hypermethylated tiling regions in ESCs compared to iPSCs
# in the example dataset
lolaRes <- res$region[["hESC vs. hiPSC (based on Sample_Group)"]][["tiling"]]
lolaRes <- lolaRes[lolaRes$userSet=="rankCut_500_hyper",]
# plot
lolaVolcanoPlot(res$lolaDb, lolaRes, signifCol="qValue")
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filtering.R
\name{rnb.execute.snp.removal}
\alias{rnb.execute.snp.removal}
\title{rnb.execute.snp.removal}
\usage{
rnb.execute.snp.removal(rnb.set, snp = rnb.getOption("filtering.snp"))
}
\arguments{
\item{rnb.set}{Methylation dataset as an object of type inheriting \code{\linkS4class{RnBSet}}.}

\item{snp}{Criterion for the removal of sites or probes based on overlap with SNPs. Possible values are
\code{"no"}, \code{"3"}, \code{"5"}, \code{"any"} or \code{"yes"}. See the documentation of
\code{\link{rnb.options}} for a detailed explanation of the procedures these values encode.}
}
\value{
\code{list} of four elements:
        \describe{
          \item{\code{"dataset.before"}}{Copy of \code{rnb.set}.}
          \item{\code{"dataset"}}{The (possibly) modified dataset object after removing probes that overlap
               with SNPs.}
          \item{\code{"filtered"}}{\code{integer} vector storing the indices (in beta matrix of the unfiltered
               dataset) of all removed sites or probes.}
          \item{\code{"snp"}}{The value of the \code{snp} parameter.}
        }
}
\description{
Removes all probes overlapping with single nucleotide polymorphisms (SNPs) from the given dataset.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
rnb.set.filtered <- rnb.execute.snp.removal(rnb.set.example, "any")$dataset
identical(meth(rnb.set.example), meth(rnb.set.filtered)) # FALSE
}
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterArchitecture.R
\docType{methods}
\name{setExecutable,ClusterArchitecture,character,character-method}
\alias{setExecutable,ClusterArchitecture,character,character-method}
\alias{setExecutable}
\alias{setExecutable,ClusterArchitecture-method}
\title{setExecutable-methods}
\usage{
\S4method{setExecutable}{ClusterArchitecture,character,character}(object, exec.name, exec.loc)
}
\arguments{
\item{object}{\code{\linkS4class{ClusterArchitecture}} object}

\item{exec.name}{A name/identifier that will be associated with the given executable}

\item{exec.loc}{The executable's location}
}
\value{
The modified object
}
\description{
Tells the cluster architecture about an executable that can be submitted as job
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Report-methods.R
\name{rnb.initialize.reports}
\alias{rnb.initialize.reports}
\title{rnb.initialize.reports}
\usage{
rnb.initialize.reports(dir.reports, dir.configuration = "configuration")
}
\arguments{
\item{dir.reports}{Directory to host report files. This must be a \code{character} of length one that specifies
a non-existent path, as this methods attempts to create it.}

\item{dir.configuration}{Subdirectory to host configuration files shared by the reports. This must be a
\code{character} of length one that gives location as a path relative to \code{dir.reports}.
Also, strong restrictions apply to the path name. See the description of the
\code{\link{createReport}} function for more details. This method creates the directory and
copies configuration files that define cascading style sheet (CSS) definitions and
Javascript functions used by the HTML reports.}
}
\value{
\code{TRUE} if the report directory was successfully created and the configuration files were copied to the
        specified location; \code{FALSE} otherwise.
}
\description{
Creates a new directory to host HTML reports and copies the shared configuration files.
}
\examples{
\donttest{
dir.reports <- "~/infinium_studies/cancer_study/reports"
if (!rnb.initialize.reports(dir.reports)) {
    cat("ERROR: Could not initialize configuration in ", dir.reports, "\n", sep = "")
}
}

}
\seealso{
\code{\link{createReport}} for initializing an HTML report
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataExport.R
\name{rnb.export.to.ewasher}
\alias{rnb.export.to.ewasher}
\title{rnb.export.to.ewasher}
\usage{
rnb.export.to.ewasher(rnb.set, out.dir, reg.type = "sites", ...)
}
\arguments{
\item{rnb.set}{Object of class \code{\linkS4class{RnBSet}}}

\item{out.dir}{output directory. If not existing, it will be created and all exported files will be placed here.
If existing, this functions results in an error.}

\item{reg.type}{region type to be exported}

\item{...}{passed on to \code{get.comparison.info}}
}
\value{
a list containing information on the export
}
\description{
Data exported to a format compatible with the FaST-LMM-EWASher tool for cell-mixture adjustment.
see \href{http://www.nature.com/nmeth/journal/v11/n3/full/nmeth.2815.html}{Zou, J., et al., Nature Methods, 2014}
for further details on the tool.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
rnb.export.to.ewasher(rnb.set.example,tempfile(pattern="forEwasher"))
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotations.R
\name{rnb.get.mapping}
\alias{rnb.get.mapping}
\title{rnb.get.mapping}
\usage{
rnb.get.mapping(region.type, target.type, assembly = "hg19")
}
\arguments{
\item{region.type}{Region type. The built-in types are \code{"cpgislands"}, \code{"genes"}, \code{"promoters"} and
\code{"tiling"}.}

\item{target.type}{Target type for sites.}

\item{assembly}{Genome assembly of interest. See \code{\link{rnb.get.assemblies}} for the list of supported
genomes.}
}
\value{
\code{list} of mapping structures, one per chromosome. Every mapping structure is an object of type
        \code{\link{IRanges}} and stores the range of indices of all sites contained in the respective region.
        Regions that do not contain sites are left out of the mapping.
}
\description{
Gets the mapping information used for a region type. These are structures used to map regions to the genomic loci (or
Infinium probes) that target them.
}
\examples{
\donttest{
promoters2probes <- rnb.get.mapping("promoters", "probes450")
promoters2probes[["chr21"]]
}
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filteringSummary.R
\name{rnb.execute.filter.summary}
\alias{rnb.execute.filter.summary}
\title{rnb.execute.filter.summary}
\usage{
rnb.execute.filter.summary(old.set, new.set)
}
\arguments{
\item{old.set}{Methylation dataset before filtering as an object of type inheriting \code{\linkS4class{RnBSet}}.}

\item{new.set}{Methylation dataset after filtering as an object of type inheriting \code{\linkS4class{RnBSet}}.}
}
\value{
\code{matrix} summarizing the number of removed and retained sites, samples, and (optionally) reliable and
        unreliable measurements.
}
\description{
Calculates a table summarizing the effect of the applied filtering procedures.
}
\details{
This function expects that the sites and samples in \code{new.set} are subsets of the sites and samples in
\code{old.set}, respectively. If this is not the case, it exists with an error.
}
\seealso{
\code{\link{rnb.run.preprocessing}} for running the whole preprocessing module
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parallelProcessing.R
\name{parallel.getNumWorkers}
\alias{parallel.getNumWorkers}
\title{parallel.getNumWorkers}
\usage{
parallel.getNumWorkers()
}
\value{
Number of workers used for parallel processing; \code{-1} if parallel processing is not enabled.
}
\description{
Gets the number of workers used for parallel processing.
}
\examples{
\donttest{
parallel.getNumWorkers()
parallel.setup(2)
parallel.getNumWorkers()
parallel.teardown()
parallel.getNumWorkers()
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBDiffMeth-class.R
\docType{methods}
\name{get.variability.method,RnBDiffMeth-method}
\alias{get.variability.method,RnBDiffMeth-method}
\alias{get.variability.method}
\title{get.variability.method-methods}
\usage{
\S4method{get.variability.method}{RnBDiffMeth}(object)
}
\arguments{
\item{object}{RnBDiffMeth object}
}
\value{
character describing the variability method
}
\description{
Gets the variability testing method used to obtain the p-values in the differential varibiality tables
}
\author{
Michael Scherer
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/controlPlotsBiSeq.R
\name{rnb.plot.biseq.coverage.violin}
\alias{rnb.plot.biseq.coverage.violin}
\title{rnb.plot.biseq.coverage.violin}
\usage{
rnb.plot.biseq.coverage.violin(
  rnbs.set,
  samples,
  fname = NULL,
  type = "sites",
  covg.range = NULL,
  ...
)
}
\arguments{
\item{rnbs.set}{RnBiseqSet object}

\item{samples}{unique sample identifiers. In case \code{rnb.getOption("identifiers.column")} 
is not \code{NULL}, \code{samples} should attain values from the corresponding column, 
or \code{colnames(meth(rnb.set))} otherwise}

\item{fname}{base filename for the files to be plotted. If NULL, the plot will not be written to file}

\item{type}{\code{character} singleton. If \code{site} the coverage
information is plotted for each methylation site. Otherwise
should be one of the regions returned by \code{rnb.region.types}}

\item{covg.range}{Vector of length 2 specifying the range of coverage to be plotted. if \code{NULL} (default) the entire range will be plotted}

\item{...}{other arguments to \code{\link{createReportPlot}}}
}
\value{
plot as an object of type \code{\linkS4class{ReportPlot}} if \code{writeToFile} is \code{TRUE} and of class 
			\code{\link{ggplot}} otherwise.
}
\description{
Plots the violin plots of the coverage distribution
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differentialMethylation.R
\name{rowWelchP}
\alias{rowWelchP}
\title{rowWelchP}
\usage{
rowWelchP(
  X,
  inds.g1,
  inds.g2 = -inds.g1,
  na.rm = FALSE,
  alternative = "two.sided"
)
}
\arguments{
\item{X}{Matrix on which the test is performed for every row}

\item{inds.g1}{column indices of group 1 members}

\item{inds.g2}{column indices of group 2 members}

\item{na.rm}{Should NAs be removed (logical)}

\item{alternative}{Testing alternative. Must be one of "two.sided" (default),"less","greater" or "all".
in case of "all" a data frome with corresping alternative variables is returned. 
Otherwise the result is a vector.}
}
\value{
vector (or data.frame if alternative=="all") of p-values resulting from the Welch's t-test
}
\description{
performs a two-sided Welch's t-test (unequal variances, unequal sample sizes) on each row of a matrix X with the indices inds.1 vs indices inds.g2 as group assignments.
}
\note{
Requires \code{matrixStats} package
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
meth.mat <- meth(rnb.set.example)
sample.groups <- rnb.sample.groups(rnb.set.example)[[1]]
p.vals <- rowWelchP(meth.mat,sample.groups[[1]],sample.groups[[2]])
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBeads-package.R
\docType{package}
\name{RnBeads}
\alias{RnBeads-package}
\alias{RnBeads}
\title{Analysis of genome-scale DNA methylation data with RnBeads}
\description{
RnBeads facilitates comprehensive analysis of various types of DNA methylation data at the genome scale. It extends
previous approaches for such analysis by high throughput capabilities, as well as presenting results in a
comprehensive, highly interpretable fashion.
}
\details{
The complete analysis can be performed by calling the function \code{\link{rnb.run.analysis}}.
}
\references{
Yassen Assenov*, Fabian Mueller*, Pavlo Lutsik*, Joern Walter, Thomas Lengauer and Christoph Bock (2014) Compehensive Analysis of DNA Methylation Data with RnBeads, Nature Methods, 11(11):1138-1140.
}
\author{
\strong{Maintainer}: Fabian Mueller \email{team@rnbeads.org}

Authors:
\itemize{
  \item Yassen Assenov \email{assenov@gmail.com}
  \item Christoph Bock \email{cbock@cemm.at}
  \item Pavlo Lutsik \email{p.lutsik@dkfz.de}
  \item Michael Scherer \email{mscherer@mpi-inf.mpg.de}
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/agePrediction.R
\name{rnb.execute.age.prediction}
\alias{rnb.execute.age.prediction}
\title{rnb.execute.age.prediction}
\usage{
rnb.execute.age.prediction(object)
}
\arguments{
\item{object}{a \code{\linkS4class{RnBSet}} object for which age prediction should
be performed}
}
\value{
modified  \code{\linkS4class{RnBSet}} object
}
\description{
Performs age prediction by either the specified predictor in the option \code{inference.age.prediction.predictor}
or by the corresponding predefined predictor.
}
\author{
Michael Scherer
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regionProfiles.R
\name{rnb.plot.locus.profile}
\alias{rnb.plot.locus.profile}
\title{rnb.plot.locus.profile}
\usage{
rnb.plot.locus.profile(
  rnbSet,
  chrom,
  start,
  end,
  grps = NULL,
  plot.m.regions = NULL,
  plot.m.heatmap = TRUE,
  plot.m.smooth = TRUE,
  cvals.grps = rnb.getOption("colors.category"),
  cvals.meth = rnb.getOption("colors.meth"),
  smooth.profile = "wide"
)
}
\arguments{
\item{rnbSet}{RnBSet object}

\item{chrom}{chromosome of window to plot}

\item{start}{start coordinate of window to plot}

\item{end}{end coordinate of window to plot}

\item{grps}{a list of indices for each group to be compared or NULL if no sample grouping information should be displayed}

\item{plot.m.regions}{character vector of region types whose methylation values should be displayed If \code{grps} is not \code{NULL}
the methylation values will be separated by sample groups.}

\item{plot.m.heatmap}{flag indicating whether sites methylation values should be displayed in a heatmap. If \code{grps} is not \code{NULL}
the heatmaps will be separated by sample groups.}

\item{plot.m.smooth}{flag indicating whether a scatterplot with smoothing curves should be displayed. If \code{grps} is not \code{NULL}
the colors will be used to separate sample groups.}

\item{cvals.grps}{colors to be used for the different groups}

\item{cvals.meth}{colors to be used for methylation values and heatmaps}

\item{smooth.profile}{profile to be used for the smoothing curves. Allowed values include 
\code{wide} (default) which yields smoother curvers and \code{narrow} which yields more "wiggly" curves}
}
\value{
a \code{ggplot2} plot object containing the plot
}
\description{
Computes methylation distributions for various region types and sample groups
}
\examples{
\donttest{
#see RnBeads vignette (section: 'Generating Locus Profile Plots') for examples
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrichment.R
\name{downloadLolaDbs}
\alias{downloadLolaDbs}
\title{downloadLolaDbs}
\usage{
downloadLolaDbs(dest, dbs = c("LOLACore"))
}
\arguments{
\item{dest}{destination directory}

\item{dbs}{vector of names of LOLA DBs to be downloaded. Currently 'LOLACore' and 'LOLAExt'
are supported}
}
\value{
a list containing vectors of directory names for each available genome assembly
}
\description{
Downloading prepared LOLA DBs from server
}
\details{
Requires a stable internet connection. Could take a while depending on the size of the database and the internet connection
}
\examples{
\donttest{
lolaDest <- tempfile()
dir.create(lolaDest)
lolaDirs <- downloadLolaDbs(lolaDest, dbs="LOLACore")
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regionProfiles.R
\name{rnb.plot.region.profile.density}
\alias{rnb.plot.region.profile.density}
\title{rnb.plot.region.profiles}
\usage{
rnb.plot.region.profile.density(
  rnb.set,
  sample,
  region.type = "",
  region.profile = NULL,
  extend.by = 0.33
)
}
\arguments{
\item{rnb.set}{RnBSet object}

\item{sample}{Index or name of the sample for which the plot should be generated}

\item{region.type}{Region type for which the plot should be generated}

\item{region.profile}{Alternative to specifying \code{region.type}, the function can accept a region profile
generated by the \code{rnb.find.relative.site.coord} function}

\item{extend.by}{A number between 0 and 1 specifying the percentage by which a region is
extended in order to capture methylation information before region start and after region end}
}
\value{
a ggplot2 object for plotting
        the plot shows the density of methylation levels of sites accross the specified region type for 
        all regions of that type from 0 (region start) to 1 (region end). Sites in the flanking areas are
        also shown (coordinates <0 and >1).
}
\description{
Plots the density of methylation levels accross all regions of the specified type
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
rnb.plot.region.profile.density(rnb.set.example,1,"genes")
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBeadSet-class.R
\docType{methods}
\name{dpval,RnBeadSet-method}
\alias{dpval,RnBeadSet-method}
\alias{dpval}
\title{dpval-methods}
\usage{
\S4method{dpval}{RnBeadSet}(object, type = "sites", row.names = FALSE, i = NULL, j = NULL)
}
\arguments{
\item{object}{\code{\linkS4class{RnBeadSet}} or \code{\linkS4class{RnBeadRawSet}} object}

\item{type}{\code{character} singleton. If \code{sites} detection p-values per each available 
site is returned. Otherwise should be one of region types for for which the summarized 
p-values are available}

\item{row.names}{Flag indicating of row names are to be generated in the result.}

\item{i}{Indices of sites/regions to be retrieved. By default (\code{NULL}), all will be retrieved.}

\item{j}{Indices of samples to be retrieved. By default (\code{NULL}), all will be retrieved.}
}
\value{
detection p-values available for the dataset in the form of a \code{matrix}.
}
\description{
Extract detection p-values from an object of \code{\linkS4class{RnBeadSet}} class.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
dp<-dpval(rnb.set.example, row.names=TRUE)
head(dp)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differentialMethylation.R
\name{get.adjustment.variables}
\alias{get.adjustment.variables}
\title{get.adjustment.variables}
\usage{
get.adjustment.variables(
  rnbSet,
  inds.g1,
  inds.g2 = -inds.g1,
  colnames.adj = c(),
  colname.target = "",
  adjust.sva = FALSE,
  adjust.celltype = FALSE
)
}
\arguments{
\item{rnbSet}{RnBSet object}

\item{inds.g1}{sample indices in \code{rnbSet} of group 1 members}

\item{inds.g2}{sample indices in \code{rnbSet} of group 2 members}

\item{colnames.adj}{column names in \code{pheno(rnbSet)} to retrieve}

\item{colname.target}{column names in \code{pheno(rnbSet)} of the target variable. Only important if \code{adjust.sva==TRUE}}

\item{adjust.sva}{flag indicating whether the resulting table should also contain surrogate variables (SVs) for the given target variable.}

\item{adjust.celltype}{flag indicating whether the resulting table should also contain estimated celltype contributions.
See \code{\link{rnb.execute.ct.estimation}} for details.}
}
\value{
a \code{data.frame} containing one column for each selected variable from the phenotypic data
        each row corresponds to a sample in the union of samples of the wto groups with the first
        \code{length(inds.g1)} rows corresponding to group 1 and the remaining rows corresponding to group 2
}
\description{
Given indices for two groups of samples for comparison, this function
retrieves \code{data.frame} containing the variables to be adjusted for
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
sample.groups <- rnb.sample.groups(rnb.set.example)[[1]]
get.adjustment.variables(rnb.set.example,sample.groups[[1]],sample.groups[[2]],"Cell_Line")
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/controlPlots.R
\name{rnb.plot.snp.barplot}
\alias{rnb.plot.snp.barplot}
\title{rnb.plot.snp.barplot}
\usage{
rnb.plot.snp.barplot(
  dataset,
  probeID,
  writeToFile = FALSE,
  numeric.names = FALSE,
  ...
)
}
\arguments{
\item{dataset}{Dataset as an instance of \code{\linkS4class{RnBeadRawSet}} or \code{\linkS4class{RnBeadSet}}.
Alternatively, the dataset can be specified as a non-empty \code{matrix} containing the computed
beta values on the SNP probes.}

\item{probeID}{Probe identifier. This must be one of \code{rownames(meth(dataset))}.}

\item{writeToFile}{Flag specifying whether the output should be saved as \code{\linkS4class{ReportPlot}}.}

\item{numeric.names}{if \code{TRUE} and \code{writeToFile} is \code{TRUE}substitute the plot options in the plot file
name with digits.}

\item{...}{Additional named arguments passed to \code{\link{createReportPlot}}.}
}
\value{
plot as an object of type \code{\linkS4class{ReportPlot}} if \code{writeToFile} is \code{TRUE} and of class
			\code{\link{ggplot}} otherwise.
}
\description{
Bar plots of beta-values from the genotyping probes
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
samp<-samples(rnb.set.example)[1]
rnb.plot.snp.barplot(rnb.set.example, samp)
}
}
\author{
Pavlo Lutsik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterArchitecture.R
\docType{methods}
\name{getExecutable,ClusterArchitecture,character-method}
\alias{getExecutable,ClusterArchitecture,character-method}
\alias{getExecutable}
\alias{getExecutable,ClusterArchitecture-method}
\title{getExecutable-methods}
\usage{
\S4method{getExecutable}{ClusterArchitecture,character}(object, exec.name)
}
\arguments{
\item{object}{\code{\linkS4class{ClusterArchitecture}} object}

\item{exec.name}{The executable's name/identifier}
}
\value{
The executable. If the name is not associated with any executable, the names will be returned and a warning will be raised
}
\description{
Retrieves the executable associated with a name/identifier
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotations.R
\name{rnb.get.assemblies}
\alias{rnb.get.assemblies}
\title{rnb.get.assemblies}
\usage{
rnb.get.assemblies()
}
\value{
All supported genome assemblies in the form of a \code{character} vector. These are \code{"hg19"},
        \code{"mm10"}, \code{"mm9"} and \code{"rn5"}.
}
\description{
Gets the supported genome assemblies.
}
\examples{
\donttest{
"hg19" \%in\% rnb.get.assemblies()
}
}
\author{
Yassen Assenov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differentialVariability.R
\name{cols.to.rank.site}
\alias{cols.to.rank.site}
\alias{cols.to.rank.region}
\title{cols.to.rank.site}
\usage{
cols.to.rank.site(diff.var)

cols.to.rank.region(diff.var)
}
\arguments{
\item{diff.var}{A differential variability table.}
}
\value{
A matrix with the absolute values of the relevant columns
}
\description{
Return a matrix containing the negative absolute values of the information used to rank the sites. Those are currently:
the variance difference, the log ratio in variances and the p-value from the statistical test.
}
\author{
Michael Scherer
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrichment.R
\name{performGoEnrichment.diffMeth}
\alias{performGoEnrichment.diffMeth}
\title{performGoEnrichment.diffMeth}
\usage{
performGoEnrichment.diffMeth(
  rnbSet,
  diffmeth,
  ontologies = c("BP", "MF"),
  rank.cuts.region = c(100, 500, 1000),
  add.auto.rank.cut = TRUE,
  rerank = TRUE,
  verbose = TRUE,
  ...
)
}
\arguments{
\item{rnbSet}{RnBSet object for which dirrential methylation was computed}

\item{diffmeth}{RnBDiffMeth object. See \code{\link{RnBDiffMeth-class}} for details.}

\item{ontologies}{GO ontologies to use for enrichment analysis}

\item{rank.cuts.region}{Cutoffs for combined ranking that are used to determine differentially methylated regions}

\item{add.auto.rank.cut}{flag indicating whether an automatically computed cut-off should also be considered.}

\item{rerank}{For deterimining differential methylation: should the ranks be ranked again or should the absolute ranks be used.}

\item{verbose}{Enable for detailed status report}

\item{...}{arguments passed on to the parameters of \code{GOHyperGParams} from the \code{GOstats} package}
}
\value{
a DiffMeth.go.enrich object (S3) containing the following attributes
\item{region}{Enrichment information for differential methylation on the region level. See \code{GOHyperGresult} from the \code{GOstats} package for furthert details}
}
\description{
performs Geno Ontology (GO) enrichment analysis for a given differential methylation table.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
res <- performGoEnrichment.diffMeth(rnb.set.example,dm)
}
}
\author{
Fabian Mueller
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/differentialMethylation.R
\name{get.comparison.info}
\alias{get.comparison.info}
\title{get.comparison.info}
\usage{
get.comparison.info(
  x,
  pheno.cols = rnb.getOption("differential.comparison.columns"),
  region.types = rnb.region.types.for.analysis(x),
  pheno.cols.all.pairwise = rnb.getOption("differential.comparison.columns.all.pairwise"),
  columns.pairs = rnb.getOption("columns.pairing"),
  columns.adj = rnb.getOption("covariate.adjustment.columns"),
  adjust.sva = rnb.getOption("differential.adjustment.sva"),
  pheno.cols.adjust.sva = rnb.getOption("inference.targets.sva"),
  adjust.celltype = rnb.getOption("differential.adjustment.celltype"),
  adjust.na.rm = TRUE
)
}
\arguments{
\item{x}{\code{RnBSet} object}

\item{pheno.cols}{column names of the pheno slot in \code{x} on which the dataset should be partitioned. Those columns are required to be factors or logical.
In case of factors, each group in turn will be compared to all other groups}

\item{region.types}{which region types should be processed for differential methylation}

\item{pheno.cols.all.pairwise}{integer or character vector specifying the colomns of \code{pheno(x)} on which all pairwise comparisons should be conducted.
A value of \code{NULL} indicates no columns.}

\item{columns.pairs}{argument passed on to \code{rnb.sample.groups}. See its documentation for details.}

\item{columns.adj}{Column names or indices in the table of phenotypic information to be used for confounder adjustment in the
differential methylation analysis.}

\item{adjust.sva}{flag indicating whether the adjustment table should also contain surrogate variables (SVs) for the given target variable.}

\item{pheno.cols.adjust.sva}{Target variables for SVA adjustment. Only important if \code{adjust.sva==TRUE}. Only the intersection of
\code{pheno.cols} and \code{pheno.cols.adjust.sva} is considered for SVA adjustment.}

\item{adjust.celltype}{flag indicating whether the resulting table should also contain estimated celltype contributions.
See \code{\link{rnb.execute.ct.estimation}} for details.}

\item{adjust.na.rm}{Flag indicating whether NAs in the adjustment table should be removed.}
}
\value{
a list containing one element for each comparison to be conducted. Each element is again a list containing:
\describe{
  \item{\code{comparison}}{the name of the comparison}
  \item{\code{pheno.colname}}{the column name of the sample annotation table the comparison is derived from}
  \item{\code{group.names}}{the names of the two groups being compared}
  \item{\code{group.inds}}{the sample indices of the samples belonging to the two groups}
  \item{\code{paired}}{flag indicating whether paired analysis is conducted}
  \item{\code{adj.sva}}{flag indicating whether adjustment for SVA is conducted}
  \item{\code{adj.celltype}}{flag indicating whether adjustment for cell type is conducted}
  \item{\code{adjustment.table}}{the covariate adjustment table. \code{NULL} if the comparison is not adjusted}
  \item{\code{region.types}}{the region types applicable to the analysis}
}
}
\description{
retrieve the comparison information for an RnBSet object
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
cmp.info <- get.comparison.info(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
cmp.info[[1]]
}
}
\author{
Fabian Mueller
}
